###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	geometry file plotter
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time

#	numpy
import numpy as np

#	numpy-stl
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors
from matplotlib.patches import FancyArrowPatch

# from tools.functions.functions_transform import rot1, rot2, rot3, gen_vectors2, rot_mat

###################################################################################################
#	define plotable arrow class
###################################################################################################

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################

#############################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)

fig_name = 'RSW_reference_frame.eps'
out_path = os.path.join(fig_path, fig_name)

#	figure save flag
savefile_flag = False
savefile_flag = True

###################################################################################################
#	figure parameters
###################################################################################################

axis_font_size = 8
axis_line_width = 1

axis_mutation_scale = 10

circle_line_width = 0.75
circle_radius = 20

#	graphical output
inch2cm = 2.54

figure = plt.figure(num=None, figsize=(9.0/inch2cm, 9.0/inch2cm),
 	dpi=300, facecolor='w', edgecolor='k'
)

plt.rc('text', usetex=True)
# plt.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
ax = plt.axes(projection='3d')

ax.set_xlim([0, 25])
ax.set_ylim([0, 25])
ax.set_zlim([0, 25])

###################################################################################################
#	plot inertial axes
###################################################################################################

x_unit = np.array([1.0, 0.0, 0.0])
y_unit = np.array([0.0, 1.0, 0.0])
z_unit = np.array([0.0, 0.0, 1.0])

#	coordinate origin
origin = [0, 0, 0]

axis_len = 10

xaxis = [axis_len, 0, 0]
yaxis = [0, axis_len, 0]
zaxis = [0, 0, axis_len]

###################################################################################################
#	x axis
xaxis_plot = np.vstack([origin, xaxis])
ax.plot(xaxis_plot[:, 0] / 2, xaxis_plot[:, 1] / 2, xaxis_plot[:, 2] / 2,
	lw=axis_line_width,
	color="k"
)
c = Arrow3D(xaxis_plot[:, 0], xaxis_plot[:, 1], xaxis_plot[:, 2],
	mutation_scale=axis_mutation_scale,
	lw=axis_line_width,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(xaxis_plot[1, 0] * 1.2, xaxis_plot[1, 1], xaxis_plot[1, 2],
#	'$X_{axis} (Earth\ Zenith\ Vector)$',
	'$R$',
	fontsize=axis_font_size,
	fontweight='bold'
)

###################################################################################################
#	y axis
yaxis_plot = np.vstack([origin, yaxis])
ax.plot(yaxis_plot[:, 0] / 2, yaxis_plot[:, 1] / 2, yaxis_plot[:, 2] / 2,
	lw=axis_line_width,
	color="k"
)
c = Arrow3D(yaxis_plot[:, 0], yaxis_plot[:, 1], yaxis_plot[:, 2],
	mutation_scale=axis_mutation_scale,
	lw=axis_line_width,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(yaxis_plot[1, 0] +  axis_len * 0.1, yaxis_plot[1, 1], yaxis_plot[1, 2],
#	'$Y_{axis}\ (Velocity\ Vector)$',
	'$S$',
	fontsize=axis_font_size,
	fontweight='bold'
)

###################################################################################################
#	z axis
zaxis_plot = np.vstack([origin, zaxis])
ax.plot(zaxis_plot[:, 0] / 2, zaxis_plot[:, 1] / 2, zaxis_plot[:, 2] / 2,
	lw=axis_line_width,
	color="k"
)
c = Arrow3D(zaxis_plot[:, 0], zaxis_plot[:, 1], zaxis_plot[:, 2],
	mutation_scale=axis_mutation_scale,
	lw=axis_line_width,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(zaxis_plot[1, 0], zaxis_plot[1, 1], zaxis_plot[1, 2],
#	'$Z_{axis} (Orbit\ Plane\ Normal)$',
	'$W$',
	fontsize=axis_font_size,
	fontweight='bold'
)



###################################################################################################
#	plot Earth position

origin_plot = np.vstack([origin, origin])

ax.plot(
	origin_plot[:, 0] - circle_radius, origin_plot[:, 1], origin_plot[:, 2],
	marker='o',
	color='tab:green'
)

ax.text(
	origin_plot[1, 0] - circle_radius,
	origin_plot[1, 1] +  axis_len * 0.1,
	origin_plot[1, 2] +  axis_len * 0.1,
	'$Earth$',
	fontsize=axis_font_size,
	fontweight='bold'
)

###################################################################################################
#	plot orbit

theta = np.linspace(0, 2 * np.pi, 201)
x = circle_radius * np.cos(theta)
y = circle_radius * np.sin(theta)
z = np.zeros(y.shape)

ax.plot(
	x - circle_radius, y, z,
	color='tab:grey',
	linestyle='dashdot',
	lw=circle_line_width
)

ax.view_init(24, 15)
plt.axis('off')

if savefile_flag is True:
	plt.savefig(out_path, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
sys.exit()
