###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Light curve rendering in the orbital frame
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

import random

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	scipy
from scipy import interpolate

#	progress bar
import progressbar

#	transformation
from tools.functions.functions_transform import site_vec, rv2kep, rot1, rot2, rot3, rot_mat, \
	axis2mat, mat2angles

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	case file generation
from tools.file_generation.case_generation import gen_case

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d
from descartes import PolygonPatch

#	shapely
import shapely.geometry as sg
# from shapely.geometry import Point
# from shapely.geometry import Polygon as sh
from shapely.ops import cascaded_union

##############################################
#	video command
#	ffmpeg -framerate 10 -pattern_type glob -i '*.png' -c:v libx264 -pix_fmt yuv420p ../orb.mp4

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	Function: 		gen_vectors_render2
#	Description: 	generate required data for render
#
###################################################################################################

def gen_vectors_render2(
		path_data, render_time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb,
		body_mesh, render_geom_dir,
		validation_plot_flag, sim_range_sec
	):

	print('\nGenerating render vector data...')

	#	define unit vectors
	x_vec = np.array((1.0, 0.0, 0.0))
	y_vec = np.array((0.0, 1.0, 0.0))
	z_vec = np.array((0.0, 0.0, 1.0))

	###############################################################################################
	#	parse time inputs
	###############################################################################################
	
	#	extract jd time data
	jd_time_data = sun_orb[:, 0]
	
	#	set simulation start julian date
	start_jdate = jd_time_data[0]
	stop_jdate = jd_time_data[-1]
	
	#	duration
	duration_sec = render_time_step * (len_record - 1)

	###############################################################################################
	#	parse attit inputs
	###############################################################################################

	################################################
	#	initial attitude

	#	attitude matrix
	init_attit_mat = rot_mat(init_attit)

	#	transpose for inverse matrix
	init_attit_mat_inv = np.transpose(init_attit_mat)

	################################################
	#	attitude motion in orbital frame

	#	rotation_vector in orbital frame
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)

	#	generate attitude motion matrix in orbital frame
	attit_mot_mat_orb = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_orb_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_orb)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_orb)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_orb = angles1

	################################################
	#	attitude motion in body frame

	#	generate attitude motion matrix in body frame
	attit_mot_mat_body = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_body_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1

	print('\nAttit Mot Body = {}'.format(attit_mot_body))

	###############################################################################################
	#	initialise geometry axes
	###############################################################################################

	#	body

	#	define geometry axes in body frame
	x_geom_body = np.copy(x_vec)
	y_geom_body = np.copy(y_vec)
	z_geom_body = np.copy(z_vec)

	#	rotate geometry axes into orbital frame
	x_geom_orb = np.dot(x_geom_body, init_attit_mat)
	y_geom_orb = np.dot(y_geom_body, init_attit_mat)
	z_geom_orb = np.dot(z_geom_body, init_attit_mat)

	###############################################################################################
	#	initialise reference axes
	###############################################################################################

	#	extract orbital vector data
	earth_orb_vec = earth_orb[0, 1:4]

	#	velcocity vector in orbital frame
	sat_orb_vel = sat_orb[0, 4:7]

	#	set orbital plane normal in orbital frame
	orb_norm_orb_unit = np.array([0.0, 0.0, 1.0])

	#	orbital reference frame vectors
	x_ref_orb = np.divide(earth_orb_vec, np.linalg.norm(earth_orb_vec))
	y_ref_orb = np.divide(sat_orb_vel, np.linalg.norm(sat_orb_vel))
	z_ref_orb = np.divide(orb_norm_orb_unit, np.linalg.norm(orb_norm_orb_unit))

	#	orbital reference frame vectors in body frame
	x_ref_orb_body = np.dot(x_ref_orb, init_attit_mat_inv)
	y_ref_orb_body = np.dot(y_ref_orb, init_attit_mat_inv)
	z_ref_orb_body = np.dot(z_ref_orb, init_attit_mat_inv)

	###############################################################################################
	#	loop through ephemeris data
	###############################################################################################

	###############################################################################################
	#	initialise step data

	total_rot_vec_vel_angle = np.array([0.0, 0.0, 0.0])

	#	generate orbital mesh from the body mesh
	orb_mesh_step = mesh.Mesh(body_mesh.data.copy())

	#	rotate into orbital frame
	orb_mesh_step.rotate_using_matrix(init_attit_mat)

	#	rotate geometry axes into orbital frame
	x_geom_orb_step = np.copy(x_geom_orb)
	y_geom_orb_step = np.copy(y_geom_orb)
	z_geom_orb_step = np.copy(z_geom_orb)

	#	generate rotation matrices for initial attitude

	init_attit_step = np.copy(init_attit)

	#	attitude matrix
	init_attit_mat_step = rot_mat(init_attit_step)

	#	transpose for inverse matrix
	init_attit_mat_inv_step = np.transpose(init_attit_mat_step)

	###############################################################################################
	#	initialise output arrays

	#	vector output arrays
	attitude_out = np.zeros((len_record, 3))
	vector_data_body = np.zeros((len_record, 37))
	vector_data_orb = np.zeros((len_record, 37))

	#	angle output arrays
	obs_orb_plot_angles = []
	obs_body_plot_angles = []
	illum_orb_plot_angles = []
	illum_body_plot_angles = []
	rot_orb_plot_angles = []
	rot_body_plot_angles = []
	geomz_orb_plot_angles = []
	geomz_body_plot_angles = []

	for i, (jdate, sat_orb_line, sun_orb_line, earth_orb_line, site_orb_line) in enumerate(
			zip(jd_time_data, sat_orb, sun_orb, earth_orb, site_orb)
		):

#		print('\n#####################################################################################')

		###########################################################################################
		#	calcualte new attitude state

#		#	propergate attitude
#		if i != 0:
#
#			print('\nNew Attiude = {} + {}'.format(attit_step, np.multiply(attit_mot, render_time_step)))
#			attit_step += np.multiply(attit_mot, render_time_step)
#			print('Attit Step = ', attit_step)

		###########################################################################################
		#	parse input orbital data
		###########################################################################################

		#	extract orbital vectors of step
		sat_orb_pos_step = sat_orb_line[1:4]
		sat_orb_vel_step = sat_orb_line[4:7]

		#	extract orbital vector data
		sun_orb_vec_step = sun_orb_line[1:4]
		earth_orb_vec_step = earth_orb_line[1:4]
		site_orb_vec_step = site_orb_line[1:4]

		#	unit vectors
		sun_orb_unit_step = np.divide(sun_orb_vec_step, np.linalg.norm(sun_orb_vec_step))
		earth_orb_unit_step = np.divide(earth_orb_vec_step, np.linalg.norm(earth_orb_vec_step))
		site_orb_unit_step = np.divide(site_orb_vec_step, np.linalg.norm(site_orb_vec_step))

		###########################################################################################
		#	orbital frame data
		###########################################################################################

		###############################################################################################
		#	orbital reference frame vectors
		x_ref_orb_step = np.divide(earth_orb_vec_step, np.linalg.norm(earth_orb_vec_step))
		y_ref_orb_step = np.divide(sat_orb_vel_step, np.linalg.norm(sat_orb_vel_step))
		z_ref_orb_step = np.divide(orb_norm_orb_unit, np.linalg.norm(orb_norm_orb_unit))
	
		###############################################################################################
		#	apply orbital attitude motion if i != 0

		if i != 0:
			x_geom_orb_step = np.dot(x_geom_orb_step, attit_mot_mat_orb)
			y_geom_orb_step = np.dot(y_geom_orb_step, attit_mot_mat_orb)
			z_geom_orb_step = np.dot(z_geom_orb_step, attit_mot_mat_orb)

		###########################################################################################
		#	vector positions in orbital frame
	
		#	apply initial attitude to rotation vector in orbital frame
		rot_vec_orb_unit_step = np.dot(rot_vec_body_unit, init_attit_mat_step)

		#	set orbital plane normal in orbital frame
		orb_norm_orb_unit_step = np.copy(orb_norm_orb_unit)

		###############################################################################################
		#	apply orbital attitude motion to geometry if i != 0
		if i != 0:
			orb_mesh_step.rotate_using_matrix(attit_mot_mat_orb)

		###########################################################################################
		#	body frame vectors
		###########################################################################################

		#	total attitude motion in body frame since simulation start
		total_attit_mot_body_step = np.multiply(attit_mot_body, i)

		#	total attitude rotation matrix
		total_attit_mot_body_mat_step = rot_mat(total_attit_mot_body_step)

		#	inverse total attitude rotation matrix
		total_attit_mot_body_mat_inv_step = np.transpose(total_attit_mot_body_mat_step)

		###############################################################################################
		#	orbital ref vectors in body frame

		#	apply initial attitude
		x_ref_orb_body_step = np.dot(x_ref_orb_step, init_attit_mat_inv_step)
		y_ref_orb_body_step = np.dot(y_ref_orb_step, init_attit_mat_inv_step)
		z_ref_orb_body_step = np.dot(z_ref_orb_step, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		x_ref_orb_body_step = np.dot(x_ref_orb_body_step, total_attit_mot_body_mat_inv_step)
		y_ref_orb_body_step = np.dot(y_ref_orb_body_step, total_attit_mot_body_mat_inv_step)
		z_ref_orb_body_step = np.dot(z_ref_orb_body_step, total_attit_mot_body_mat_inv_step)

		###############################################################################################
		#	define geometry axes in body frame
		x_geom_body_step = np.copy(x_vec)
		y_geom_body_step = np.copy(y_vec)
		z_geom_body_step = np.copy(z_vec)

		###########################################################################################
		#	vector positions in body frame
	
		################################################
		#	rotation vector in orbital frame
		rot_vec_body_unit_step = np.copy(rot_vec_body_unit)

		################################################
		#	orbital plane normal vector

		#	apply attitude
		orb_norm_body_unit_step = np.dot(orb_norm_orb_unit, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		orb_norm_body_unit_step = np.dot(orb_norm_body_unit_step, total_attit_mot_body_mat_inv_step)

		###########################################################################################
		#	convert input orbital data to body frame
		###########################################################################################

		################################################
		#	velocity vector

		#	apply initial attitude
		sat_body_vel_step = np.dot(sat_orb_vel_step, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		sat_body_vel_step = np.dot(sat_body_vel_step, total_attit_mot_body_mat_inv_step)

		################################################
		#	sun/illumination vector

		#	apply initial attitude
		sun_body_vec_step = np.dot(sun_orb_vec_step, init_attit_mat_inv_step)
		earth_body_vec_step = np.dot(earth_orb_vec_step, init_attit_mat_inv_step)
		site_body_vec_step = np.dot(site_orb_vec_step, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		sun_body_vec_step = np.dot(sun_body_vec_step, total_attit_mot_body_mat_inv_step)
		earth_body_vec_step = np.dot(earth_body_vec_step, total_attit_mot_body_mat_inv_step)
		site_body_vec_step = np.dot(site_body_vec_step, total_attit_mot_body_mat_inv_step)

		###########################################################################################
		#	observation vector validation
		###########################################################################################
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(site_orb_vec_step, x_ref_orb_step),
				np.linalg.norm(site_orb_vec_step) * np.linalg.norm(x_ref_orb_step)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(site_orb_vec_step, y_ref_orb_step),
				np.linalg.norm(site_orb_vec_step) * np.linalg.norm(y_ref_orb_step)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(site_orb_vec_step, z_ref_orb_step),
				np.linalg.norm(site_orb_vec_step) * np.linalg.norm(z_ref_orb_step)
			)
		)
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(site_body_vec_step, x_ref_orb_body_step),
				np.linalg.norm(site_body_vec_step) * np.linalg.norm(x_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(site_body_vec_step, y_ref_orb_body_step),
				np.linalg.norm(site_body_vec_step) * np.linalg.norm(y_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(site_body_vec_step, z_ref_orb_body_step),
				np.linalg.norm(site_body_vec_step) * np.linalg.norm(z_ref_orb_body_step)
			)
		)
		
		#	append angles to output array
		obs_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		obs_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###############################################################################################
		#	illumination vector validation
		###############################################################################################
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(sun_orb_vec_step, x_ref_orb_step),
				np.linalg.norm(sun_orb_vec_step) * np.linalg.norm(x_ref_orb_step)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(sun_orb_vec_step, y_ref_orb_step),
				np.linalg.norm(sun_orb_vec_step) * np.linalg.norm(y_ref_orb_step)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(sun_orb_vec_step, z_ref_orb_step),
				np.linalg.norm(sun_orb_vec_step) * np.linalg.norm(z_ref_orb_step)
			)
		)
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(sun_body_vec_step, x_ref_orb_body_step),
				np.linalg.norm(sun_body_vec_step) * np.linalg.norm(x_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(sun_body_vec_step, y_ref_orb_body_step),
				np.linalg.norm(sun_body_vec_step) * np.linalg.norm(y_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(sun_body_vec_step, z_ref_orb_body_step),
				np.linalg.norm(sun_body_vec_step) * np.linalg.norm(z_ref_orb_body_step)
			)
		)
		
		#	append angles to output array
		illum_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		illum_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###############################################################################################
		#	rotation vector validation
		###############################################################################################
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(rot_vec_orb_unit_step, x_ref_orb_step),
				np.linalg.norm(rot_vec_orb_unit_step) * np.linalg.norm(x_ref_orb_step)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(rot_vec_orb_unit_step, y_ref_orb_step),
				np.linalg.norm(rot_vec_orb_unit_step) * np.linalg.norm(y_ref_orb_step)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(rot_vec_orb_unit_step, z_ref_orb_step),
				np.linalg.norm(rot_vec_orb_unit_step) * np.linalg.norm(z_ref_orb_step)
			)
		)
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(rot_vec_body_unit, x_ref_orb_body_step),
				np.linalg.norm(rot_vec_body_unit) * np.linalg.norm(x_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(rot_vec_body_unit, y_ref_orb_body_step),
				np.linalg.norm(rot_vec_body_unit) * np.linalg.norm(y_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(rot_vec_body_unit, z_ref_orb_body_step),
				np.linalg.norm(rot_vec_body_unit) * np.linalg.norm(z_ref_orb_body_step)
			)
		)
		
		#	append angles to output array
		rot_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		rot_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###############################################################################################
		#	geom z-axis validation
		###############################################################################################
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(z_geom_orb_step, x_ref_orb_step),
				np.linalg.norm(z_geom_orb_step) * np.linalg.norm(x_ref_orb_step)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(z_geom_orb_step, y_ref_orb_step),
				np.linalg.norm(z_geom_orb_step) * np.linalg.norm(y_ref_orb_step)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(z_geom_orb_step, z_ref_orb_step),
				np.linalg.norm(z_geom_orb_step) * np.linalg.norm(z_ref_orb_step)
			)
		)
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(z_geom_body_step, x_ref_orb_body_step),
				np.linalg.norm(z_geom_body_step) * np.linalg.norm(x_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(z_geom_body_step, y_ref_orb_body_step),
				np.linalg.norm(z_geom_body_step) * np.linalg.norm(y_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(z_geom_body_step, z_ref_orb_body_step),
				np.linalg.norm(z_geom_body_step) * np.linalg.norm(z_ref_orb_body_step)
			)
		)
		
		#	append angles to output array
		geomz_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		geomz_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###########################################################################################
		#	Data Output
		###########################################################################################
	
		#	set image index
		index = i

		###########################################################################################
		#	populate output attitude array
#		attitude_out[index, :] = attit_step
		attitude_out[index, :] = np.array([0.0, 0.0, 0.0]) 
	
		###########################################################################################
		#	orbital frame vector data

		#	julian date
		vector_data_orb[index, 0] = jdate

		#	reference vectors
		vector_data_orb[index, 1:4] = x_ref_orb_step
		vector_data_orb[index, 4:7] = y_ref_orb_step
		vector_data_orb[index, 7:10] = z_ref_orb_step

		#	geometry vectors
		vector_data_orb[index, 10:13] = x_geom_orb_step
		vector_data_orb[index, 13:16] = y_geom_orb_step
		vector_data_orb[index, 16:19] = z_geom_orb_step

		#	earth, velocity and plane normal vectors
		vector_data_orb[index, 19:22] = earth_orb_vec_step
		vector_data_orb[index, 22:25] = sat_orb_vel_step
		vector_data_orb[index, 25:28] = orb_norm_orb_unit_step

		#	sun, site and rotation vector
		vector_data_orb[index, 28:31] = sun_orb_vec_step
		vector_data_orb[index, 31:34] = site_orb_vec_step
		vector_data_orb[index, 34:37] = rot_vec_orb_unit_step

		###########################################################################################
		#	body frame vector data

		#	julian date
		vector_data_orb[index, 0] = jdate

		#	reference vectors
		vector_data_body[index, 1:4] = x_ref_orb_body_step
		vector_data_body[index, 4:7] = y_ref_orb_body_step
		vector_data_body[index, 7:10] = z_ref_orb_body_step

		#	geometry vectors
		vector_data_body[index, 10:13] = x_geom_body_step
		vector_data_body[index, 13:16] = y_geom_body_step
		vector_data_body[index, 16:19] = z_geom_body_step

		#	earth, velocity and plane normal vectors
		vector_data_body[index, 19:22] = earth_body_vec_step
		vector_data_body[index, 22:25] = sat_body_vel_step
		vector_data_body[index, 25:28] = orb_norm_body_unit_step

		#	sun, site and rotation vector
		vector_data_body[index, 28:31] = sun_body_vec_step
		vector_data_body[index, 31:34] = site_body_vec_step
		vector_data_body[index, 34:37] = rot_vec_body_unit_step

		###########################################################################################
		#	inertial frame vector data
	
		###########################################################################################
		#	orbital mesh files
		###########################################################################################

		#	save and end
		if i >= len_record - 1:
			break

	###############################################################################################
	#	convert angle outputs to numpy arrays

	obs_orb_plot_angles = np.array(obs_orb_plot_angles)
	obs_body_plot_angles = np.array(obs_body_plot_angles)

	illum_orb_plot_angles = np.array(illum_orb_plot_angles)
	illum_body_plot_angles = np.array(illum_body_plot_angles)

	rot_orb_plot_angles = np.array(rot_orb_plot_angles)
	rot_body_plot_angles = np.array(rot_body_plot_angles)

	geomz_orb_plot_angles = np.array(geomz_orb_plot_angles)
	geomz_body_plot_angles = np.array(geomz_body_plot_angles)

	if validation_plot_flag is True:

		#	set use of LaTeX font
		plt.rc('text', usetex=True)

		###########################################################################################
		#	initialise figure
		fig, ax = plt.subplots(1, 3, figsize=(12, 4), dpi=300)
	
		markevery1 = np.arange(0, len(sim_range_sec), 60)
		markevery2 = np.arange(30, len(sim_range_sec), 60)
		mark_size = 5

		###########################################################################################
		ax[0].title.set_text(r'$Angle\ to\ Observation\ Vector$')

		ax[0].plot(
			sim_range_sec, obs_orb_plot_angles[:, 0],
			label='$Earth\_Vec\ in\ Orb\ Frame$',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='r'
		)

		ax[0].plot(
			sim_range_sec, obs_orb_plot_angles[:, 1],
			label='$Vel\_Vec\ in\ Orb\ Frame$',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='g'
		)

		ax[0].plot(
			sim_range_sec, obs_orb_plot_angles[:, 2],
			label='$Plane\_Norm\ in\ Orb\ Frame$',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='b'
		)
	
		ax[0].plot(
			sim_range_sec, obs_body_plot_angles[:, 0],
			label='$Earth\_Vec\ in\ Body\ Frame$',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:red'
		)

		ax[0].plot(
			sim_range_sec, obs_body_plot_angles[:, 1],
			label='$Vel\_Vec\ in\ Body\ Frame$',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:green'
		)

		ax[0].plot(
			sim_range_sec, obs_body_plot_angles[:, 2],
			label='$Plane\_Vec\ in\ Body\ Frame$',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:blue'
		)

		# ax[0].legend()
		ax[0].set_xlabel(r'$Time / s$')
		ax[0].set_ylabel(r'$Angle / radians$')

		###########################################################################################
		ax[1].title.set_text(r'$Angle\ to\ Illumination\ Vector$')

		ax[1].plot(
			sim_range_sec, illum_orb_plot_angles[:, 0],
#			label='Angle from Earth_Vec in Orb Frame',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='r'
		)

		ax[1].plot(
			sim_range_sec, illum_orb_plot_angles[:, 1],
#			label='Angle from Vel_Vec in Orb Frame',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='g'
		)

		ax[1].plot(
			sim_range_sec, illum_orb_plot_angles[:, 2],
#			label='Angle from Plane_Norm in Orb Frame',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='b'
		)
	
		ax[1].plot(
			sim_range_sec, illum_body_plot_angles[:, 0],
#			label='Angle from Earth_Vec in Body Frame',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:red'
		)

		ax[1].plot(
			sim_range_sec, illum_body_plot_angles[:, 1],
#			label='Angle from Vel_Vec in Body Frame',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:green'
		)

		ax[1].plot(
			sim_range_sec, illum_body_plot_angles[:, 2],
#			label='Angle from Plane_Vec in Body Frame',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:blue'
		)

		# ax[1].legend()
		ax[1].set_xlabel(r'$Time / s$')
		ax[1].set_ylabel(r'$Angle / radians$')

		###########################################################################################
		ax[2].title.set_text(r'$Angle\ to\ Rotation\ Vector$')

		ax[2].plot(
			sim_range_sec, rot_orb_plot_angles[:, 0],
#			label='Angle from Earth_Vec in Orb Frame',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='r'
		)

		ax[2].plot(
			sim_range_sec, rot_orb_plot_angles[:, 1],
#			label='Angle from Vel_Vec in Orb Frame',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='g'
		)

		ax[2].plot(
			sim_range_sec, rot_orb_plot_angles[:, 2],
#			label='Angle from Plane_Norm in Orb Frame',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='b'
		)
	
		ax[2].plot(
			sim_range_sec, rot_body_plot_angles[:, 0],
#			label='Angle from Earth_Vec in Body Frame',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:red'
		)

		ax[2].plot(
			sim_range_sec, rot_body_plot_angles[:, 1],
#			label='Angle from Vel_Vec in Body Frame',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:green'
		)

		ax[2].plot(
			sim_range_sec, rot_body_plot_angles[:, 2],
#			label='Angle from Plane_Vec in Body Frame',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:blue'
		)

		ax[2].set_xlabel(r'$Time / s$')
		ax[2].set_ylabel(r'$Angle / radians$')

		
		fig.legend(loc='lower center', frameon=False, ncol=2)

		fig.tight_layout()
		fig.subplots_adjust(bottom=0.3)


		####################################################
		#	save figure

		#	output file path
		fig_path = os.path.join(
			os.environ['HOME'],
			'PhD',
			'Documents',
			'Thesis',
			'Thesis_Main_Directory',
			'Figures'
		)
		
		#	set flag
		savefile_flag = True
		savefile_flag = False
		

		fig_name = 'transform_validation' + '.eps'
		out_str = os.path.join(fig_path, fig_name)
		
		if savefile_flag is True:
			plt.savefig(out_str, bbox_inches="tight", dpi=300)
			print('\n#######################################')
			print('Figure saved: ', fig_name)
		
		###########################################################################################
		#	angle to z axis

		markevery1 = np.arange(0, len(sim_range_sec), 20)
		markevery2 = np.arange(10, len(sim_range_sec), 20)

		fig3, ax3 = plt.subplots(figsize=(4, 4), dpi=300)

		ax3.title.set_text('$Angle\ to\ Geometry\ Z\ Axis$')

		ax3.plot(
			sim_range_sec, geomz_orb_plot_angles[:, 0],
			label='$Earth\_Vec\ in\ Orb\ Frame$',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='r'
		)

		ax3.plot(
			sim_range_sec, geomz_orb_plot_angles[:, 1],
			label='$Vel\_Vec\ in\ Orb\ Frame$',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='g'
		)

		ax3.plot(
			sim_range_sec, geomz_body_plot_angles[:, 0],
			label='$Earth\_Vec\ in\ Body\ Frame$',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:red'
		)

		ax3.plot(
			sim_range_sec, geomz_body_plot_angles[:, 1],
			label='$Vel\_Vec\ in\ Body\ Frame$',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:green'
		)


		markevery1 = np.arange(0, len(sim_range_sec), 40)
		markevery2 = np.arange(20, len(sim_range_sec), 40)


		ax3.plot(
			sim_range_sec, geomz_orb_plot_angles[:, 2],
			label='$Plane\_Norm\ in\ Orb\ Frame$',
			linestyle = 'dashed',
			marker='^', markevery=markevery1, markersize=mark_size,
			color='b'
		)
	
		ax3.plot(
			sim_range_sec, geomz_body_plot_angles[:, 2],
			label='$Plane\_Vec\ in\ Body\ Frame$',
			linestyle = 'dotted',
			marker='s', markevery=markevery2, markersize=mark_size,
			color='tab:blue'
		)

		ax3.set_xlabel(r'$Time / s$')
		ax3.set_ylabel(r'$Angle / radians$')

		fig3.legend(loc='lower center', frameon=False, ncol=2, fontsize=5)

		fig3.tight_layout()
		fig3.subplots_adjust(bottom=0.2)

		####################################################
		#	save figure
		
#		#	set flag
#		savefile_flag = True
#		savefile_flag = False

		fig_name = 'attitude_validation' + '.eps'
		out_str = os.path.join(fig_path, fig_name)
		
		if savefile_flag is True:
			plt.savefig(out_str, bbox_inches="tight", dpi=300)
			print('\n#######################################')
			print('Figure saved: ', fig_name)
		
		print('\nShowing validation plots.')
		plt.show()
		sys.exit()

	return vector_data_body, vector_data_orb, attitude_out

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT Render Orb')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

# local_input_path = os.path.join('input_dir', 'mmt_files', 'rocket_body')
local_input_path = os.path.join('input_dir', 'mmt_files', 'starlink', 'batch1')

read_path = os.path.join(path_data, local_input_path)

###################################################################################################
#	script specific initialisation

geom_auto_del = True
render_auto_flag = True

###################################################################################################
#	light curve generation inputs

#	source mmt file
mmt_file = 'STARLINK-1035_44740_MMT_6936.txt'

#	selected track
track_index = 1

#	parse filename
name_str = mmt_file.split('_')[0]
norad_id = int(mmt_file.split('-')[1].split('_')[1])

#	simulation time step
time_step = 0.1

window_start_date = [2010, 1, 1, 0, 0, 0]
window_stop_date = [2029, 11, 1, 0, 0, 0]

###################################################################################################
#	load geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir' + '/starlink'

geom_str = 'starlink_full'

geom_file = geom_str + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 0.0

rot_vec_azm_deg = random.randint(0, 180)
rot_vec_inc_deg = random.randint(0, 180)

print('\nAttitude azm angle = ', rot_vec_azm_deg)

#	attitude motion
rot_period = 10.0

#	define rotation axis in body frame as unit vector
rot_vec_body_unit = np.array([0.0, 1.0, 0.0])

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])

###################################################################################################
#	set render data

#	directory data
graphics_dir = 'graphics_dir'
render_dir = 'render_dir'
render_geom_dir = 'geom_files'
render_img_dir = 'image_files'

#	orb image data
render_image_dir = os.path.join(path_data, graphics_dir, render_dir, render_img_dir)
render_img_dir_list = os.listdir(render_image_dir)
nimg_files = len(render_img_dir_list)

#	orb image data
render_geom_dir = os.path.join(path_data, graphics_dir, render_dir, render_geom_dir)
render_geom_dir_list = os.listdir(render_geom_dir)
ngeom_files = len(render_geom_dir_list)

#	render outputs
render_data_orb = 'render_data_orb.txt'
render_data_body = 'render_data_body.txt'
render_data_inertial = 'render_data_inertial.txt'

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###############################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

###################################################################################################
#	load MMT file
###################################################################################################

#	set time
mmt_load_start = time.time()

print('\n[{}] Loading MMT data...'.format(datetime.datetime.now()))

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file

#	strip header
header_text = mmt_data[7]
raw_data = mmt_data[8:]

print('\n	Parsing MMT data...')

#	initialise output
n_rows = 14
parsed_data = np.zeros([len(raw_data), n_rows])

for i, line in enumerate(raw_data):

	#	split line
	date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
		distance, phase, channel, track_id = line.split(' ')

	#	parse data and set data types
	mag_std = float(mag_std)
	mag_app = float(mag_app)
	distance = float(distance)
	phase = float(phase)
	track_id = int(track_id)

	#	parse date
	year, month, day = date_str.split('-')
	year = int(year)
	month = int(month)
	day = int(day)

	#	parse time
	hour, minute, second = time_str.split(':')
	hour = int(hour)
	minute = int(minute)
	second = np.round(float(second), 1)

	#	calculate julian date
	jday, jfrac = date2jd(year, month, day, hour, minute, second)

	#	generate output line
	output_line = [
		track_id, norad_id,
		jday, jfrac,
		year, month, day,
		hour, minute, second,
		mag_std, mag_app,
		distance, phase
	]

	#	add to output array
	parsed_data[i, :] = output_line

print('		Total data length = ', len(parsed_data))
print('		Total number of objects = ', len(np.unique(parsed_data[:, 1])))
print('		Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

###################################################################################################
#	extracting data between specified windows

print('\n	Seaching for tracks inside time window:')

#	start window
window_start_jd = np.add(*date2jd(*window_start_date))
print('\n		Window start 	= ', window_start_date[0:3])

#	stop window
window_stop_jd = np.add(*date2jd(*window_stop_date))
print('		Window stop 	= ', window_stop_date[0:3])

#	extract track ids
track_id_list = np.unique(parsed_data[:, 0])
extracted_track_id_list = []

#	loop through tracks
ntracks = 0
first_lc_flag = True
for track_id in track_id_list:

	#	find track ids
	track_indices = np.where(parsed_data[:, 0]==track_id)[0]

	#	extract full data
	track_data = parsed_data[track_indices, :]

	#	extract light curve data
	jd_plot = np.add(track_data[:, 2], track_data[:, 3])
	std_mag_plot = track_data[:, 10]
	app_mag_plot = track_data[:, 11]
	distance_plot = track_data[:, 12]
	phase_plot = track_data[:, 13]
	start_line = track_data[0]
	stop_line = track_data[-1]
	start_jdate = np.add(start_line[2], start_line[3])
	stop_jdate = np.add(stop_line[2], stop_line[3])

	#	ensure between window start/stop
	if start_jdate > window_start_jd and stop_jdate < window_stop_jd:

		#	light curve is within window
		ntracks += 1

		#	if first light curve, start array
		if first_lc_flag is True:
			windowed_data = track_data
			first_lc_flag = False

		#	else append to list
		elif first_lc_flag is False:
			windowed_data = np.vstack([windowed_data, track_data])

		#	add track id to output
		extracted_track_id_list.append(track_id)

print('\n		{} track(s) extracted.'.format(ntracks))
extracted_track_id_list = np.array(extracted_track_id_list)

###############################################################################################
#	extracting requested single track

print('\n		Extracting requested track index {}'.format(track_index))

#	set track array index
track_array_index = track_index - 1
track_id = int(extracted_track_id_list[track_array_index])

track_indices = np.where(windowed_data[:, 0]==track_id)[0]
track_data = windowed_data[track_indices]

###############################################################################################
#	extract track time data

mmt_line = track_data[0]

#	parse line elements
track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = track_data[0]

start_jd = [jday, jfrac]

#	parse line elements
track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = track_data[-1]

stop_jd = [jday, jfrac]

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###############################################################################################
#	display initialisation time
mmt_load_end = time.time()
mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)

print('\n	MMT Load Finished.')
print('\n		- Time taken = {} s.'.format(mmt_load_time))

print('\n#####################################################################################')
print('Light curve parameters: ')
print('	Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_start_date))
print('	Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_stop_date))
print('	Simulation time step:		{}'.format(time_step))
print('	Duration:			{}		[steps]'.format(lc_len_record))
print('					{}		[s]'.format(np.round(lc_duration_sec, 1)))
print('\n#####################################################################################')

###################################################################################################
#	acquire TLE
###################################################################################################

#	set time
tle_start = time.time()

print('\n[{}] Acquiring TLEs...'.format(datetime.datetime.now()))

#	TLE request dates, add day either side to increase tle find chance
tle_start_date = jd2date(*jday_jfrac(lc_start_jd - 1))
tle_stop_date = jd2date(*jday_jfrac(lc_stop_jd + 1)) 

#	generate TLE file name
tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)

#	assemble TLE path
tle_path = os.path.join(path_data, 'input_dir', 'tle_files')

#	if exists, load
if os.path.isfile(os.path.join(tle_path, tle_file)):

	print('\n	Reading existing TLE file.')

	#	load file
	with open(os.path.join(tle_path, tle_file), 'r') as file:
		tle_data = file.readlines()

else:
	print('\n	Requesting TLE file...')

	#	request TLEs
	tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)

	#	save file
	with open(os.path.join(tle_path, tle_file), 'w') as file:
		for line in tle_data:
			file.write(line)

#	grab first TLE pair
tle_line1 = tle_data[1]
tle_line2 = tle_data[2]

#	mean mot rad/sec
orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
orbital_mot = (2 * np.pi) / orbital_period

###############################################################################################
#	display TLE acquisition time
tle_end = time.time()
tle_time = np.round((tle_end - tle_start), 1)

print('\n	TLE Load Finished.')
print('\n		- Time taken = {} s.'.format(tle_time))

###################################################################################################
#	select time data to use
###################################################################################################

sim_range = lc_sim_range
len_record = lc_len_record
start_date = lc_start_date
stop_date = lc_stop_date

###################################################################################################
#	generate eop data arrays
###################################################################################################

#	set time
eop_start = time.time()

print('\n[{}] Generating EOP data array...'.format(datetime.datetime.now()))

#	EOP data
eop_dir = 'eop_files'
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	assemble eop path
eop_path = os.path.join(path_data, eop_dir)

#	generate interpolated eop_data
eop_log_level = 1
eop_data, dat_data = eop_interpolate(
	eop_path,
	eop_file,
	dat_file,
	sim_range,
	time_step,
	eop_log_level
)

###################################################################################################
#	display time
eop_end = time.time()
eop_time = np.round((eop_end - eop_start), 1)

print('\n	EOP data generated.')
print('\n		- Time taken = {} s.'.format(eop_time))

###################################################################################################
#	generate time objects
###################################################################################################

#	start dat value
dat = dat_data[0, -1]
	
#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate start time object
start_time_object = get_times(*start_date, ut_offset, dat)
	
#	end dat value
dat = dat_data[-1, -1]
	
#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate stop time object
stop_time_object = get_times(*stop_date, ut_offset, dat)

###################################################################################################
#	generate rotation matrices
###################################################################################################

#	set time
rot_mat_start = time.time()
print('\n[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))

#	tt time
jdaytt_start = start_time_object.jday_tt
jfractt_start = start_time_object.jfrac_tt
jdtt_start = jdaytt_start + jfractt_start

#	ut1 time
jdayut1_start = start_time_object.jday_ut1
jfracut1_start = start_time_object.jfrac_ut1
jdut1_start = jdayut1_start + jfracut1_start

#	generate rotation matrices
rot_mat_log_level = 0
pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
	jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
)

###################################################################################################
#	display time
rot_mat_end = time.time()
rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)

print('\n	Rotation Matrices Generated.')
print('\n		- Time taken = {} s.'.format(rot_mat_time))

###################################################################################################
#	MODULE:	Horizons Positions
###################################################################################################

#	set time
horz_start = time.time()
print('\n[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))

#	set horizons logging level
horz_logging_level = 0

sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
	path_data, horz_logging_level,
	start_time_object, stop_time_object,
	len_record, time_step,
	eop_path
)

#	find indices of light curve data
timecol = sun_icrf[:, 0]
lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]

###################################################################################################
#	display time
horz_end = time.time()
horz_time = np.round((horz_end - horz_start), 1)

print('\n	Horizons data generated.')
print('\n		- Time taken = {} s.'.format(horz_time))

###################################################################################################
#	MODULE:	Object Position
###################################################################################################

#	set time
sat_start = time.time()
print('\n[{}]	Generating object position data...'.format(datetime.datetime.now()))

satpos_logging_level = 0

sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
	path_data, satpos_logging_level, len_record, time_step,
	start_time_object, stop_time_object,
	norad_id,
	eop_data, dat_data,
	pm_array, pn_array, st_array, c2t_array
)

###################################################################################################
#	display time
sat_end = time.time()
sat_time = np.round((sat_end - sat_start), 1)

print('\n	Object position data generated.')
print('\n		- Time taken = {} s.'.format(sat_time))

###################################################################################################
#	MODULE:	Transformation
###################################################################################################

#	set time
trans_start = time.time()
print('\n[{}]	Performing transformations...'.format(datetime.datetime.now()))

identification_str = 'mmt-render'

trans_logging_level = 0
save_flag = False

sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
	= module_transformation(
		path_data, trans_logging_level, save_flag,
		start_time_object, stop_time_object,
		identification_str, site_str,
		len_record, time_step,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array,
		sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
	)

###################################################################################################
#	display time
trans_end = time.time()
trans_time = np.round((trans_end - trans_start), 1)

print('\n	Transformations complete.')
print('\n		- Time taken = {} s.'.format(trans_time))

###################################################################################################
#	define initial attitude state
###################################################################################################

##################################################
#	attitude

#	convert to radians
rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad = np.radians(rot_vec_inc_deg)

#	generate initial t = 0 attitude
init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])

##################################################
#	attitude motion in body frame

#	generate rotation angle in radians
if rot_period != 0.0:
	rot_ang_per_sec = (np.pi * 2) / rot_period
else:
	rot_ang_per_sec = 0.0

#	generate attitude matrix in body frame
attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)

#	extract euler angles
angles1 = np.array(mat2angles(attit_mat_body)[0:3])
angles2 = np.array(mat2angles(attit_mat_body)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

#	extract attitude motion in the body frame
attit_mot_body = angles1

##################################################
#	generate initial attitude rotation matrix
init_attit_mat = rot_mat(init_attit)

##################################################
#	calculate rotation vector in orbital frame as unit vector
rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)

##################################################
#	define velocity of rotation vector in orbital frame

#	when equal to orbital motion, rot vec is stationary in inertial frame
rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

# payload_pointing = np.dot(payload_pointing, init_attit_mat)

###################################################################################################
#	perform light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

validation_plot_flag = False
validation_plot_flag = True

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors_render2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb,
	body_mesh, render_geom_dir,
	validation_plot_flag, sim_range_sec
)

###################################################################################################
#	EoF
###################################################################################################
