###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing
from tools.functions.glim_slc import glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	matplotlib
import matplotlib.pyplot as plt

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
os.system('clear')
print('#####################################################################################')
print('			Globalstar File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set flag
savefile_flag = False
savefile_flag = True

#	output file name
fig_name = 'group_all_simulation_data' + '.eps'

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)

test_time1 = time.time()
total_time_start = time.time()

###################################################################################################
#	inputs
###################################################################################################

#	define read path
read_path = os.path.join(path_data, 'input_dir/mmt_files/globalstar/lightcurves')

specific_objects = [
	#	inactive batch
	26081,
	26082,
	26083,
	26084,
	#	active batch
	31571,
	31573,
	31574,
	31576,
	#	object of interest batch
	32263,
	32264,
	32265,
	32266
]

activity_flags = [
	#	inactive batch
	[False, 26081],
	[False, 26082],
	[False, 26083],
	[False, 26084],
	#	active batch
	[True, 31571],
	[True, 31573],
	[True, 31574],
	[True, 31576],
	#	object of interest batch
	[True, 32263],
	[True, 32264],
	[True, 32265],
	[False, 32266]
]


#	group 1
# specific_objects = [26081]	# uncertain
# specific_objects = [26082]	# inactive
# specific_objects = [26083]	# inactive
# specific_objects = [26084]	# inactive
# active_flag = False

# #	group 2
# specific_objects = [31571]	# active
# specific_objects = [31573]	# active
# specific_objects = [31574]	# active
# specific_objects = [31576]	# active
# active_flag = True

# #	group 3
# specific_objects = [32266]

specific_object_flag = False
specific_object_flag = True

active_rot_period = 0.0
active_rot_period = 10.0

###################################################################################################
#	inputs

#	set time windows
window_start_date = [2010, 8, 1, 0, 0, 0]
window_stop_date = [2020, 5, 1, 0, 0, 0]

#	simulation time step
time_step = 0.1

#	number of points for running average
n_avg_points = 40
	
###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	attitude state

# #	attitude
# rot_vec_azm_deg = 70.0
# rot_vec_inc_deg = 30.0
# 
# rot_vec_azm_deg = random.randint(0, 359)
# rot_vec_inc_deg = random.randint(0, 89)

# #	attitude motion
# rot_period = 60.0
# 
# #	attitude motion
# rot_period = 0.0

#	define rotation axis in body frame as unit vector
rot_vec_body_unit = np.array([0.0, 0.0, 1.0])

###################################################################################################
#	geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir/box_wing'

geom_str = 'globalstar_box_wing'
geom_file = geom_str + '.stl'

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([0.0, 0.0, 180.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

###################################################################################################
#	BRDF inputs

#	set initial BRDF values
d_brdf = 0.8
omega_brdf = 0.4
m_brdf = 0.2

###################################################################################################
#	plot light curve properties
###################################################################################################

print('\n#####################################################################################')
print('[{}] Plotting Starlink Light Curve Properties...'.format(datetime.datetime.now()))

###################################################################################################
#	load mmt_properties file

#	define output path
mmt_output_path = 'input_dir/mmt_files/globalstar'

#	define read file
mmt_output_file = 'globalstar_mmt_data_table.txt'

with open(os.path.join(path_data, mmt_output_path, mmt_output_file), 'r') as read_file:

	#	print data to output file
	globalstar_data = read_file.readlines()

###################################################################################################
#	figure parameters

#	set use of LaTeX font
plt.rc('text', usetex=True)

fig, ax = plt.subplots(2, 1, dpi=300, sharex=True)

# fig_avg, ax_avg = plt.subplots()
# fig_std, ax_std = plt.subplots()

# fig_std_full, ax_std_full = plt.subplots()

threshold_value = 0.27

###################################################################################################
#	load MMT file

globalstar_dir = os.listdir(read_path)

globalstar_file_list = []
activity_flag_list = []
for mmt_file in globalstar_dir:

	#	parse filename data
	name_str, norad_id, _, mmt_id = mmt_file[:-4].split('_')
	name_str = ''.join(name_str.split('-'))
	norad_id = int(norad_id)

	#	skip if not requested object
	if specific_object_flag is True:
		if norad_id not in specific_objects:
			continue

	globalstar_file_list.append(mmt_file)

	for line in activity_flags:
		temp_id = line[1]
		if temp_id == norad_id:
			activity_flag_list.append(line[0])

marker_range = [
	'p', '^', 's', 'h',
	'p', '^', 's', 'h',
	'p', '^', 's', 'h',
#	'p', '^', 's', 'h'
]

style_range = [
	'solid', 'solid', 'solid', 'solid',
	'dotted', 'dotted', 'dotted', 'dotted',
	'dashed', 'dashed', 'dashed', 'dashed',
#	'dashdot', 'dashdot', 'dashdot', 'dashdot'
]

# style_range = ['solid', 'dotted', 'dashed', 'dashdot']

legend_data1 = []
legend_data2 = []




for file_itt, (mmt_file, active_flag) in enumerate(zip(globalstar_file_list, activity_flag_list)):

	used_marker = marker_range[file_itt]
	style_used = style_range[file_itt]

	#	parse filename data
	name_str, norad_id, _, mmt_id = mmt_file[:-4].split('_')
	name_str = ''.join(name_str.split('-'))
	norad_id = int(norad_id)

	print('\n	#############################################################################')
	print('	Working on: ', mmt_file)
	print('	File {} of {}...'.format(int(file_itt), int(len(globalstar_file_list))))
	launches = []
	status_strings = []

	#	get MMT data
	for line in globalstar_data:
		temp_mmt_id = int(line.split(', ')[0])
		temp_name_string = line.split(', ')[1].strip()
		temp_norad_id = int(line.split(', ')[2])
	
		intl_desig = line.split(', ')[3].strip()
	
		status_string = line.split(', ')[4].strip()
		period_string = line.split(', ')[5].strip()

		launches.append([
			temp_norad_id,
			int(intl_desig.split('-')[0]),
			int(intl_desig.split('-')[1][0:3])
		])

		status_strings.append(status_string)
	
		if 'Period:' in period_string:
			period = float(period_string.split(': ')[1].split(' s')[0])
			period_flag = True
	
		if temp_norad_id == norad_id:
			break

	#	generate a label string
	label_str = '{} ({}) [Real]'.format(temp_name_string.split(' ')[1], norad_id)
	label_str_synth = '{} ({}) [Synthetic]'.format(temp_name_string.split(' ')[1], norad_id)
	print('\n	Label String: ', label_str)

	#	set file data
	output_path = os.path.join(path_data, mmt_output_path, 'parsed_lightcurves')
	output_file = mmt_file[:-4] + '_parsed.txt'

	if os.path.exists(os.path.join(output_path, output_file)):
		parsed_data = np.loadtxt(os.path.join(output_path, output_file))

	else:
	
		###############################################################################################
		#	object printout
		
		print('\n	Name string:	', temp_name_string)
		print('	Norad ID:	', temp_norad_id)
		print('	Status:		', status_string)
		print('	Periodicity:	', period_string)
		print('	Intl. Desig.:	', intl_desig)
	
		###############################################################################################
		#	load mmt file
		print('\n	Loading MMT file...')
	
		#	open and read file
		with open(os.path.join(read_path, mmt_file)) as read_file:
			mmt_data = read_file.readlines()
	
		#	set object ID to norad_id
#		object_id = temp_norad_id
		
		###########################################################################################
		#	parse MMT file

		#	find start of data
		for line_id, line in enumerate(mmt_data):
			if line[0] != '#':
				break

		#	strip header
		header_text = mmt_data[7]
		raw_data = mmt_data[8:]
		
		print('\n	Parsing MMT data...')
		
		#	initialise output
		n_rows = 14
		parsed_data = np.zeros([len(raw_data), n_rows])
		
		for i, line in enumerate(raw_data):
		
			#	split line
			date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
				distance, phase, channel, track_id = line.split(' ')
		
			#	parse data and set data types
			mag_std = float(mag_std)
			mag_app = float(mag_app)
			distance = float(distance)
			phase = float(phase)
			track_id = int(track_id)
		
			#	parse date
			year, month, day = date_str.split('-')
			year = int(year)
			month = int(month)
			day = int(day)
		
			#	parse time
			hour, minute, second = time_str.split(':')
			hour = int(hour)
			minute = int(minute)
			second = np.round(float(second), 1)
		
			#	calculate julian date
			jday, jfrac = date2jd(year, month, day, hour, minute, second)
		
			#	generate output line
			output_line = [
				track_id, norad_id,
				jday, jfrac,
				year, month, day,
				hour, minute, second,
				mag_std, mag_app,
				distance, phase
			]
		
			#	add to output array
			parsed_data[i, :] = output_line
		
		print('		Total data length = ', len(parsed_data))
		print('		Total number of objects = ', len(np.unique(parsed_data[:, 1])))
		print('		Total number of tracks = ', len(np.unique(parsed_data[:, 0])))
	
		#	save file
		np.savetxt(os.path.join(output_path, output_file), parsed_data)

	###############################################################################################
	#	extracting data between specified windows
	
	print('\n	Seaching for tracks inside time window:')
	
	#	start window
	window_start_jd = np.add(*date2jd(*window_start_date))
	print('\n		Window start 	= ', window_start_date[0:3])
	
	#	stop window
	window_stop_jd = np.add(*date2jd(*window_stop_date))
	print('		Window stop 	= ', window_stop_date[0:3])
	
	#	extract track ids
	track_id_list = np.unique(parsed_data[:, 0])
	extracted_track_id_list = []
	
	#	loop through tracks
	ntracks = 0
	first_lc_flag = True
	for track_id in track_id_list:
	
		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]
	
		#	extract light curve data
		jd_plot = np.add(track_data[:, 2], track_data[:, 3])
		std_mag_plot = track_data[:, 10]
		app_mag_plot = track_data[:, 11]
		distance_plot = track_data[:, 12]
		phase_plot = track_data[:, 13]
		start_line = track_data[0]
		stop_line = track_data[-1]
		start_jdate = np.add(start_line[2], start_line[3])
		stop_jdate = np.add(stop_line[2], stop_line[3])
	
		#	ensure between window start/stop
		if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
	
			#	light curve is within window
			ntracks += 1
	
			#	if first light curve, start array
			if first_lc_flag is True:
				windowed_data = track_data
				first_lc_flag = False
	
			#	else append to list
			elif first_lc_flag is False:
				windowed_data = np.vstack([windowed_data, track_data])
	
			#	add track id to output
			extracted_track_id_list.append(track_id)
	
	print('\n		{} track(s) extracted.'.format(ntracks))
	extracted_track_id_list = np.array(extracted_track_id_list)

	###############################################################################################
	#	extract required data
	
	print('\n	Looping through {} Track IDs...'.format(len(extracted_track_id_list)))

	lc_avg_real = []
	lc_std_real = []
	lc_avg_movavg_real = []
	lc_std_movavg_real = []

	lc_avg_synth = []
	lc_std_synth = []
	lc_avg_movavg_synth = []
	lc_std_movavg_synth = []

	full_datevec_array = []
	full_datevec_array_avg = []

	#	loop through track IDs
	for track_itt, track_id in enumerate(extracted_track_id_list):

		print('\n	#############################################################################')
		print('	Working on TrackID {}; {} of {}...'.format(
			int(track_id), int(track_itt) + 1, int(len(extracted_track_id_list))
		))
	
		#	find track ids
		track_indices = np.where(windowed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = windowed_data[track_indices, :]

		#	randomly generate an attiude state
		
		if active_flag == True:
			rot_vec_azm_deg = 0.0
			rot_vec_inc_deg = 0.0

			rot_period = 0.0
		else:
			random.seed(track_id)
			rot_vec_azm_deg = random.randint(0, 359)
			rot_vec_inc_deg = random.randint(0, 89)

			rot_period = active_rot_period
		
		###########################################################################################
		#	extract track time data
		
		mmt_line = track_data[0]
		
		#	parse line elements
		track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]
	
		###########################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
		sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

		###########################################################################################
		#	extract mmt brightness data
		mmt_signal_std_raw = track_data[:, 9]
		mmt_signal_app_raw = track_data[:, 10]

		###########################################################################################
		#	ensure time continuity
		###########################################################################################

		print('\n		Ensuring time continuity in extracted data...')
		
		track_data_temp = []
		sim_index = 0
		
		for itt, line in enumerate(track_data):
		
			#	parse line data
			track_id, object_id, \
				jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = line
			
			step_jd = jday + jfrac
		
			#	calculate current seconds
			current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
		
			output_line = [
				jday, jfrac, current_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]
		
			##################################################
			#	extracting data of time step
		
			#	data match found
			if current_sec == sim_range_sec[sim_index]:
				track_data_temp.append(output_line)
		
				#	itterate number of output data points
				sim_index += 1
		
				#	save current line as previous line
				prev_line = output_line
		
			#	data point is not required
			elif current_sec < sim_range_sec[sim_index]:
				pass
		
			#	interpolation required
			elif current_sec > sim_range_sec[sim_index]:
		
				#	parse previous line
				jday_prev, jfrac_prev, prev_sec, \
					year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
					mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
		
				#	calculate interpolation requirements
				gap = np.round(current_sec - prev_sec, 1)
				npts = int(gap / time_step)
		
				#	get current light curve data
				mag_std_last = mag_std
				mag_app_last = mag_app
				distance_last = distance
				phase_last = phase
		
				#	calculate linear interpolation step values
				mag_std_step = (mag_std_last - mag_std_prev) / npts
				mag_app_step = (mag_app_last - mag_app_prev) / npts
				distance_step = (distance_last - distance_prev) / npts
				phase_step = (phase_last - phase_prev) / npts
		
				#	calculate interpolation data
				for n in range(1, npts + 1):
		
					#	current simulation second
					interp_sec = np.round(prev_sec + (n * time_step), 1)

					#	time data
					jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
					jday, jfrac = jday_jfrac(jdate)
					year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
					#	calculate interpolated light curve data
					mag_std = mag_std_prev + (n * mag_std_step)
					mag_app = mag_app_prev + (n * mag_app_step)
					distance = distance_prev + (n * distance_step)
					phase = phase_prev + (n * phase_step)
		
					#	generate new output line
					interp_output_line = [
						jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
		
					if interp_sec == sim_range_sec[sim_index]:
						track_data_temp.append(interp_output_line)
						sim_index += 1
		
				if interp_sec != current_sec:
					#	append final line
					track_data_temp.append(output_line)
					sim_index += 1
		
			else:
				print('ERROR')
				sys.exit()
		
			#	save previous line
			if itt != 0:
				prev_line = output_line
		
			#	array is complete condition
			if len(track_data_temp) == lc_len_record:
				break
		
		#	ensure parsed MMT data matches simulation data
		track_data = np.array(track_data_temp)
		
		if len(track_data) == len(lc_sim_range):
			print('			Simulation and MMT data matches.')
		else:
			print('			Error: Simulation and MMT data does not match...Exiting.')
			sys.exit()

		###########################################################################################
		#	extract mmt brightness data

		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]
		
		print('\n		#####################################################################')
		print('		Extracted MMT track parameters: ')
		print(
			'			Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
			.format(*lc_start_date)
		)
		print(
			'			Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
			.format(*lc_stop_date)
		)
		print('			Simulation time step:		{}'.format(time_step))
		print('			Duration:			{}		[steps]'.format(lc_len_record))
		print('							{}		[s]'.format(np.round(lc_duration_sec, 1)))

#		###########################################################################################
#		#	display light curve properties
#		
#		print('\n		#####################################################################')
#		print('		Light curve parameters: ')
#		print('			Start Date:	{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_start_date))
#		print('			Stop Date:	{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_stop_date))
#		print('			Time step:	{}'.format(time_step))
#		print('			Duration:	{}		[steps]'.format(lc_len_record))
#		print('					{}		[s]'.format(np.round(lc_duration_sec, 1)))
#		print('\n		#####################################################################')
#
#		###########################################################################################
#		#	display light curve attitude state
#		
#		print('\n		#####################################################################')
#		print('		Light curve attitude state: ')
#		print('			Attitude Angles: 	{}, {}'.format(rot_vec_azm_deg, rot_vec_inc_deg))
#		print('			Rotation Vector:	{}, {}, {}'.format(*rot_vec_body_unit))
#		print('			Rotation Period:	{}'.format(rot_period))
#		print('\n		#####################################################################')

		###########################################################################################
		#	load / generate synthetic light curve
		###########################################################################################

		#	set file data
		lc_output_path = os.path.join(path_data, mmt_output_path, 'synthetic_lightcurves')

		lc_output_file = mmt_file[:-4]\
			+ '_track_{}'.format(int(track_id))\
			+ '_state_{}'.format(str(int(rot_vec_azm_deg)))\
			+ '_{}'.format(str(int(rot_vec_inc_deg)))\
			+ '_{}'.format(str(int(rot_period)))\
			+ '.txt'

#		print('\n		Looking in: ', os.path.join(mmt_output_path, 'synthetic_lightcurves'))
#		print('		For file: ', lc_output_file)

		if os.path.exists(os.path.join(lc_output_path, lc_output_file)):
			light_curve = np.loadtxt(os.path.join(lc_output_path, lc_output_file))
	
#			print('\n			Loaded file: ')
			light_curve_seconds = light_curve[:, 1]
			light_curve_data_app = light_curve[:, 2]
			light_curve_data_std = light_curve[:, 3]
	
		else:

			print('\n			Generating file: ')

			#######################################################################################
			#	acquire TLE
			#######################################################################################
			
			#	set time
			tle_start = time.time()
			
			print(
				'\n{}{}'.format(
					'###########################################',
					'##########################################'
			))
			print('Performing Light Curve Generation:')

			print('[{}] Acquiring TLEs...'.format(datetime.datetime.now()))
			
			#	TLE request dates, add day either side to increase tle find chance
			tle_start_date = jd2date(*jday_jfrac(lc_start_jd - 1))
			tle_stop_date = jd2date(*jday_jfrac(lc_stop_jd + 1)) 
			
			#	generate TLE file name
			tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
			tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
			tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)
			
			#	assemble TLE path
			tle_path = os.path.join(path_data, 'input_dir', 'tle_files')
			
			print('\n	Reading from TLE path: ', tle_path)
			print('	Looking for TLE file: ', tle_file)
	
			#	if exists, load
			if os.path.isfile(os.path.join(tle_path, tle_file)):
				print('\n		Loading existing TLE file...')
			
				#	load file
				with open(os.path.join(tle_path, tle_file), 'r') as file:
					tle_data = file.readlines()
			
			else:
				print('\n		Requesting TLE file...')
				continue_flag = True
				temp_start_date = tle_start_date
				temp_stop_date = tle_stop_date

				window_size = np.round((lc_stop_jd + 1) - (lc_start_jd - 1), 1)
				print('		Using window size of +/-: ', window_size / 2)

				while continue_flag is True:
			
					#	request TLEs
					output = req_tle(temp_start_date, temp_stop_date, norad_id)

					#	check for empty result
					if output[0] == 'NO RESULTS RETURNED':
						print('\n		{}: No resuts returned from Spacetrack'.format(norad_id))
						print('			Widening search window...')

						#	return to julian dates
						temp_start_jd = np.add(*date2jd(*temp_start_date))
						temp_stop_jd = np.add(*date2jd(*temp_stop_date))

						#	add a day either side then convert back to gregorian date
						temp_start_date = jd2date(*jday_jfrac(temp_start_jd - 1))
						temp_stop_date = jd2date(*jday_jfrac(temp_stop_jd + 1)) 

						window_size = np.round((temp_stop_jd + 1) - (temp_start_jd - 1), 1)
						print('\n			Using window size of +/-: ', window_size / 2)
						if window_size > 14:
							print('\n			Window size exceeds +/- 7 days...')
							print('				Exiting...')
							sys.exit()
					else:
						print('\n	TLE data acquired')
						tle_data = output
						continue_flag = False

		
				#	save file
				with open(os.path.join(tle_path, tle_file), 'w') as file:
					for line in tle_data:
						file.write(line)
			
			#	grab first TLE pair
			tle_line1 = tle_data[1]
			tle_line2 = tle_data[2]
			
			#	mean mot rad/sec
			orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
			orbital_mot = (2 * np.pi) / orbital_period
			
			#######################################################################################
			#	display TLE acquisition time
			tle_end = time.time()
			tle_time = np.round((tle_end - tle_start), 1)
			
			print('\n	TLE Load Finished.')
			print('\n		- Time taken = {} s.'.format(tle_time))
			
			#######################################################################################
			#	select time data to use
			#######################################################################################
			
			sim_range = lc_sim_range
			len_record = lc_len_record
			start_date = lc_start_date
			stop_date = lc_stop_date
			duration_sec = lc_duration_sec
			
			#######################################################################################
			#	generate eop data arrays
			#######################################################################################
			
			#	set time
			eop_start = time.time()
			
			print('\n[{}] Generating EOP data array...'.format(datetime.datetime.now()))
			
			#	EOP data
			eop_dir = 'eop_files'
			eop_file = 'eop_parsed.csv'
			dat_file = 'dat_parsed.csv'
			
			#	assemble eop path
			eop_path = os.path.join(path_data, eop_dir)
			
			#	generate interpolated eop_data
			eop_log_level = 0
			eop_data, dat_data = eop_interpolate(
				eop_path,
				eop_file,
				dat_file,
				sim_range,
				time_step,
				eop_log_level
			)
			
			#######################################################################################
			#	display time
			eop_end = time.time()
			eop_time = np.round((eop_end - eop_start), 1)
			
			print('\n	EOP data generated.')
			print('\n		- Time taken = {} s.'.format(eop_time))
			
			#######################################################################################
			#	generate time objects
			#######################################################################################
			
			#	start dat value
			dat = dat_data[0, -1]
				
			#	start eop values
			eop_line = eop_data[0, :]
			eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _,\
				lod, _, dX, _, dY, _ = eop_line
				
			#	generate start time object
			start_time_object = get_times(*start_date, ut_offset, dat)
				
			#	end dat value
			dat = dat_data[-1, -1]
				
			#	end eop values
			eop_line = eop_data[-1, :]
			eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _,\
				lod, _, dX, _, dY, _ = eop_line
				
			#	generate stop time object
			stop_time_object = get_times(*stop_date, ut_offset, dat)
			
			#######################################################################################
			#	generate rotation matrices
			#######################################################################################
			
			#	set time
			rot_mat_start = time.time()
			print('\n[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))
			
			#	tt time
			jdaytt_start = start_time_object.jday_tt
			jfractt_start = start_time_object.jfrac_tt
			jdtt_start = jdaytt_start + jfractt_start
			
			#	ut1 time
			jdayut1_start = start_time_object.jday_ut1
			jfracut1_start = start_time_object.jfrac_ut1
			jdut1_start = jdayut1_start + jfracut1_start
			
			#	generate rotation matrices
			rot_mat_log_level = 0
			pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
				jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
			)
			
			#######################################################################################
			#	display time
			rot_mat_end = time.time()
			rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)
			
			print('\n	Rotation Matrices Generated.')
			print('\n		- Time taken = {} s.'.format(rot_mat_time))
			
			#######################################################################################
			#	MODULE:	Horizons Positions
			#######################################################################################
			
			#	set time
			horz_start = time.time()
			print('\n[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))
			
			#	set horizons logging level
			horz_logging_level = 0
			
			sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
				path_data, horz_logging_level,
				start_time_object, stop_time_object,
				len_record, time_step,
				eop_path
			)
			
			#	find indices of light curve data
			timecol = sun_icrf[:, 0]
			lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]
			
			#######################################################################################
			#	display time
			horz_end = time.time()
			horz_time = np.round((horz_end - horz_start), 1)
			
			print('\n	Horizons data generated.')
			print('\n		- Time taken = {} s.'.format(horz_time))
			
			#######################################################################################
			#	MODULE:	Object Position
			#######################################################################################
			
			#	set time
			sat_start = time.time()
			print('\n[{}]	Generating object position data...'.format(datetime.datetime.now()))
			
			satpos_logging_level = 0
			
			sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
				path_data, satpos_logging_level, len_record, time_step,
				start_time_object, stop_time_object,
				norad_id,
				eop_data, dat_data,
				pm_array, pn_array, st_array, c2t_array
			)
			
			#######################################################################################
			#	display time
			sat_end = time.time()
			sat_time = np.round((sat_end - sat_start), 1)
			
			print('\n	Object position data generated.')
			print('\n		- Time taken = {} s.'.format(sat_time))
			
			#######################################################################################
			#	MODULE:	Transformation
			#######################################################################################
			
			#	set time
			trans_start = time.time()
			print('\n[{}]	Performing transformations...'.format(datetime.datetime.now()))
			
			identification_str = 'mmt-render'
			
			trans_logging_level = 0
			save_flag = False
			
			sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
				= module_transformation(
					path_data, trans_logging_level, save_flag,
					start_time_object, stop_time_object,
					identification_str, site_str,
					len_record, time_step,
					eop_data, dat_data,
					pm_array, pn_array, st_array, c2t_array,
					sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
				)
			
			#######################################################################################
			#	display time
			trans_end = time.time()
			trans_time = np.round((trans_end - trans_start), 1)
			
			print('\n	Transformations complete.')
			print('\n		- Time taken = {} s.'.format(trans_time))
		
			#######################################################################################
			#	define initial attitude state
			#######################################################################################
			
			##################################################
			#	attitude
			
			#	convert to radians
			rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
			rot_vec_inc_rad = np.radians(rot_vec_inc_deg)
			
			#	generate initial t = 0 attitude
			init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
			
			##################################################
			#	attitude motion in body frame
			
			#	generate rotation angle in radians
			if rot_period != 0.0:
				rot_ang_per_sec = (np.pi * 2) / rot_period
			else:
				rot_ang_per_sec = 0.0
			
			#	generate attitude matrix in body frame
			attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)
			
			#	extract euler angles
			angles1 = np.array(mat2angles(attit_mat_body)[0:3])
			angles2 = np.array(mat2angles(attit_mat_body)[3:6])
			
			#	convert negative zeros
			for i, ele in enumerate(angles1):
				if str(ele)[0] == '-' and ele == 0:
					angles1[i] = 0.0
			
			#	extract attitude motion in the body frame
			attit_mot_body = angles1
			
			##################################################
			#	generate initial attitude rotation matrix
			init_attit_mat = rot_mat(init_attit)
			
			##################################################
			#	calculate rotation vector in orbital frame as unit vector
			rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)
			
			##################################################
			#	define velocity of rotation vector in orbital frame
			
			#	when equal to orbital motion, rot vec is stationary in inertial frame
			rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
			rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])
			
			#######################################################################################
			#	generate brdf
			
			#	Refractive Index
			n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))
			
			#	Specular ratio
			s_brdf = 1.0 - d_brdf
			
			#	generate brdf
			brdf_data = np.zeros([nfaces, 5])
			
			for i in range(nfaces):
				brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
			
			#######################################################################################
			#	perform light curve vector generation
			#######################################################################################
			
			#	blank observation and illumination vectors in orbital frame
			obs_vec_orb = np.array([1.0, 0.0, 0.0])
			illum_vec_orb = np.array([1.0, 0.0, 0.0])
			
			#	generate orb2inertial velocity vector
			orb2inertial = np.array([0.0, 0.0, -orbital_mot])
	
			#	vector generation function
			vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
				path_data, time_step, len_record,
				sat_orb, sun_orb, earth_orb, site_orb,
				orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
				obs_vec_orb, illum_vec_orb
			)
			
			##################################################
			#	parse vector data
			
			#	extract orbital positions of custom input vectors
			illum_vec_orb_data = vector_data_orb[:, 28:31]
			obs_vec_orb_data = vector_data_body[:, 31:34]
			
			#	body data
			sun_body_data = vector_data_body[:, 28:31]
			site_body_data = vector_data_body[:, 31:34]
			earth_body_data = vector_data_body[:, 19:22]
			sat_body_vel = vector_data_body[:, 22:25]
			sat_body_pos = np.zeros([len(sat_body_vel), 3])
			sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
			illum_vec_body_data = vector_data_body[:, 28:31]
			obs_vec_body_data = vector_data_body[:, 31:34]
			
			#	orb data
			sun_orb_data = vector_data_orb[:, 28:31]
			site_orb_data = vector_data_orb[:, 31:34]
			earth_orb_data = vector_data_orb[:, 19:22]
			sat_orb_vel = vector_data_orb[:, 22:25]
			sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
			sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
			illum_vec_orb_data = vector_data_orb[:, 28:31]
			obs_vec_orb_data = vector_data_orb[:, 31:34]
		
			#######################################################################################
			#	generate light curve
			#######################################################################################
			
			print('\n[{}]	Generating Light Curve...'.format(datetime.datetime.now()))
		
			#	generate sim time
			sim_time = np.linspace(0, duration_sec, len_record)
			
			#	initilise lightcurve output
			final_light_curve = np.zeros((lc_len_record, 4))
			final_light_curve[:, 1] = sim_time
			
			jdate_start = start_jd[0] + start_jd[1]
		
		
			#	run light curve generation function
			light_curve = glim_slc_new(sun_body_data, site_body_data, final_light_curve, \
					jdate_start, body_vectors, body_areas, brdf_data)
		
			light_curve_seconds = light_curve[:, 1]
			light_curve_data_app = light_curve[:, 2]
			light_curve_data_std = light_curve[:, 3]
	
			#######################################################################################
			#	save synthetic light curve file
	
			print('\n	Saving file...')
	
			#	save file
			np.savetxt(os.path.join(lc_output_path, lc_output_file), light_curve)

			print(
				'\n{}{}'.format(
					'###########################################',
					'##########################################'
			))

		###########################################################################################
		#	generate date vectors
		###########################################################################################

		#	full plottable datevectors
		lc_datevec_array = []

		for jdate_line in lc_sim_range:

			#	convert jd to date
			year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(jdate_line))]

			#	turn into string
			datestr = str('-'.join(
				['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
			))
			
			timestr = ':'.join(
				['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
			)
			
			datevec  = ' '.join([datestr, timestr])
		
			#	convert to datetime object
			date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
		
			#	append  to output array
			lc_datevec_array.append(date_time_obj)

		#	append  to output array
		full_datevec_array.append(lc_datevec_array[0])
	
		###########################################################################################
		#	calculate moving averages
	
		lc_avg_real.append(np.average(mmt_signal_app_raw))
		lc_std_real.append(np.std(mmt_signal_app_raw))

		lc_avg_synth.append(np.average(light_curve_data_app))
		lc_std_synth.append(np.std(light_curve_data_app))
	
		if track_itt > (n_avg_points - 2):

			avg_avg_real = 0
			std_avg_real = 0

			avg_avg_synth = 0
			std_avg_synth = 0

			for avg_id in np.arange(n_avg_points):
				avg_avg_real += lc_avg_real[track_itt - avg_id]
				std_avg_real += lc_std_real[track_itt - avg_id]

				avg_avg_synth += lc_avg_synth[track_itt - avg_id]
				std_avg_synth += lc_std_synth[track_itt - avg_id]

			avg_avg_real = (1 / n_avg_points) * avg_avg_real
			std_avg_real = (1 / n_avg_points) * std_avg_real

			avg_avg_synth = (1 / n_avg_points) * avg_avg_synth
			std_avg_synth = (1 / n_avg_points) * std_avg_synth

			#	append to outputs
			lc_avg_movavg_real.append(avg_avg_real)
			lc_std_movavg_real.append(std_avg_real)

			lc_avg_movavg_synth.append(avg_avg_synth)
			lc_std_movavg_synth.append(std_avg_synth)

			full_datevec_array_avg.append(date_time_obj)

	test_time2 = time.time()
	print('run_time = ', (test_time2 - test_time1) / 60)

	###################################################################################################
	#	finalise data

	lc_avg_real = np.array(lc_avg_real)
	lc_std_real = np.array(lc_std_real)

	lc_avg_synth = np.array(lc_avg_synth)
	lc_std_synth = np.array(lc_std_synth)
	
	full_lc_avg_avg_real = np.average(lc_avg_real)
	full_lc_std_avg_real = np.average(lc_std_real)

	full_lc_avg_avg_synth = np.average(lc_avg_synth)
	full_lc_std_avg_synth = np.average(lc_std_synth)

	###################################################################################################
	#	final figure output

	print('\n	Plotting data...')

	plot_data = np.full([len(full_datevec_array_avg), 1], full_lc_std_avg_real)

	print('\n	Status String: ', status_string)
	if status_string == 'Inactive Satellite':
		color_str_real = 'r'
		color_str_synth = 'tab:red'
	elif status_string == 'Active Satellite':
		color_str_real = 'g'
		color_str_synth = 'tab:green'
	elif status_string == 'Uncertain Satellite':
		color_str_real = 'b'
		color_str_synth = 'tab:blue'
	else:
		print('FAILED MATCH')
		continue

	#	object of interest
	if norad_id == 32263:
		color_str_real = 'm'
		color_str_synth = 'tab:purple'
	elif norad_id == 32266:
		color_str_real = 'c'
		color_str_synth = 'tab:cyan'
	elif norad_id == 26081:
		color_str_real = 'b'
		color_str_synth = 'tab:blue'


	####################################################
	#	average magnitude data

	marker_size = 2

#	#	real data
#	markerline1, = ax[0].plot(
#		full_datevec_array, lc_avg_real,
#		linestyle='None', markeredgecolor='None',
#		marker=used_marker, color=color_str_real, alpha=0.1, ms=3,
#	)

	line1, = ax[0].plot(
		full_datevec_array_avg, lc_avg_movavg_real,
		color=color_str_real,
		linestyle=style_used,
		linewidth=1.5
	)

#	#	synthetic data
#	markerline2, = ax[0].plot(
#		full_datevec_array, lc_avg_synth,
#		linestyle='None', markeredgecolor='None',
#		marker=used_marker, color=color_str_synth, alpha=0.1, ms=3,
#	)

	line2, = ax[0].plot(
		full_datevec_array_avg, lc_avg_movavg_synth,
		color=color_str_synth,
		linestyle=style_used,
		linewidth=1.5
	)

#	legend_data1.append((markerline1, line1))
#	legend_data1.append((markerline2, line2))

	legend_data1.append(line1)
	legend_data1.append(line2)


	legend_data2.append(label_str)
	legend_data2.append(label_str_synth)

	####################################################
	#	standard deviation plot

	#	real data
#	ax[1].plot(
#		full_datevec_array, lc_std_real,
#		linestyle='None', markeredgecolor='None',
#		marker=used_marker, color=color_str_real, alpha=0.2, ms=3,
#	)

	ax[1].plot(
		full_datevec_array_avg, lc_std_movavg_real,
		color=color_str_real,
		linestyle=style_used,
		linewidth=1.5
	)

	#	synthetic data
#	ax[1].plot(
#		full_datevec_array, lc_std_synth,
#		linestyle='None', markeredgecolor='None',
#		marker=used_marker, color=color_str_synth, alpha=0.2, ms=3,
#	)

	ax[1].plot(
		full_datevec_array_avg, lc_std_movavg_synth,
		color=color_str_synth,
		linestyle=style_used,
		linewidth=1.5
	)

###################################################################################################
#	finalise output figures

#	finalise plot
# ax[0].set_ylim([12.0, 0.0])
#ax[0].set_title('Average Magnitude')

# ax[1].set_ylim([0.0, 2.0])
#ax[1].set_title('Standard Deviation')


#	start
year, mon, day, hour, minute, sec = [2014, 5, 1, 0, 0, 0]
#	turn into string
datestr = str('-'.join(
	['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
))
timestr = ':'.join(
	['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
)
datevec  = ' '.join([datestr, timestr])
#	convert to datetime object
date_time_obj1 = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')

#	end
year, mon, day, hour, minute, sec = [2020, 6, 1, 0, 0, 0]
#	turn into string
datestr = str('-'.join(
	['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
))
timestr = ':'.join(
	['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
)
datevec  = ' '.join([datestr, timestr])
#	convert to datetime object
date_time_obj2 = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')



ax[1].plot(
	[date_time_obj1, date_time_obj2], [threshold_value, threshold_value],
	color='k',
	linewidth=1.0
)

# ax[0].set_ylim([12.0, 0.0])
ax[0].set_ylabel(r'$Average\ Magnitude$')

# ax[1].set_ylim([0.0, 1.0])
ax[1].set_ylabel(r'$Standard\ Deviation$')
ax[1].set_xlabel(r'$Time\ /\ Years$')

# fig.legend(
# 	legend_data1, legend_data2,
# 	loc='center',
# 	fontsize=7,
# 	frameon=False,
# 	ncol=4,
# 	bbox_to_anchor=(0.54, 0.53),
# 	markerscale=2.0
# )

fig.tight_layout()
fig.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=None, hspace=0.5)
























# fig.legend(loc='center', frameon=False, ncol=4, bbox_to_anchor=(0.50, 0.53), fontsize=7)
# fig.tight_layout()
# fig.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=None, hspace=0.5)

#	show output
plt.pause(0.0001)

total_time_end = time.time()
print('Total Run Time = {} mins'.format((total_time_end - total_time_start) / 60))

####################################################
#	save figure
if savefile_flag is True:

	out_str = os.path.join(fig_path, fig_name)
	print('Output Path: ', out_str)

	if os.path.exists(out_str):
		check = input('\nFile already exists, overwrite? [y/n] ')
	else:
		check = 'y'

	if check == 'y':

		print('Saving: ', out_str)
	
		fig.savefig(out_str, bbox_inches="tight", dpi=300)
		print('\n#######################################')
		print('Figure saved: ', fig_name)
	
	else:

		print('Not overwriting existing file.')

print('\n	END.')
print('\n	Showing output...')

plt.show()
sys.exit()
