import os
import sys

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

import numpy as np
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

#	import from transformation functions
from tools.functions.functions_transform import site_vec, rot1, rot2, rot3, \
										rot_mat, \
										axis2mat, \
										mat2angles, \
										gen_vectors

###################################################################################################
#	define plotable arrow class
###################################################################################################

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

#############################################################################################
#	initialise
#############################################################################################

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output
geom_file_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)

geom_file_name = 'globalstar_box_wing.pdf'

savefile_flag = False
savefile_flag = True

#############################################################################################
#	define parts

#	spacecraft main bus
bus_depth = 1.5
bus_height = 3.1
bus_front_width = 2.4
bus_back_width = 1.2

# solar panel boom
boom_depth = 0.05
boom_height = 0.05
boom_width = 0.75

# solar panel
panel_height = 2.2
panel_width = 3.05
panel_depth = 0.05
panel_angle = 0.0

#############################################################################################
#	assemble mesh
#############################################################################################

#	find bus slope angle
angle = np.arctan(((bus_front_width / 2) - (bus_back_width / 2)) / bus_depth)

#	use slope angle to find y_0
y_0 = (bus_back_width / 2) + (bus_depth / 2.0) * np.tan(angle) 
y_delta = (boom_depth / 2) * np.tan(angle)

#############################################################################################
#	Spacecraft Bus
#############################################################################################

bus_verts_array = np.array([

	#	front face verts
	#	i = 0:7
	[bus_depth / 2, bus_front_width / 2, bus_height / 2],
	[bus_depth / 2, bus_front_width / 2, 0 + boom_height / 2],
	[bus_depth / 2, bus_front_width / 2, 0 - boom_height / 2],
	[bus_depth / 2, bus_front_width / 2, -bus_height / 2],
	[bus_depth / 2, -bus_front_width / 2, bus_height / 2],
	[bus_depth / 2, -bus_front_width / 2, 0 + boom_height / 2],
	[bus_depth / 2, -bus_front_width / 2, 0 - boom_height / 2],
	[bus_depth / 2, -bus_front_width / 2, -bus_height / 2],

	#	back face verts
	#	i = 8:15
	[-bus_depth / 2, bus_back_width / 2, bus_height / 2],
	[-bus_depth / 2, bus_back_width / 2, 0 + boom_height / 2],
	[-bus_depth / 2, bus_back_width / 2, 0 - boom_height / 2],
	[-bus_depth / 2, bus_back_width / 2, -bus_height / 2],
	[-bus_depth / 2, -bus_back_width / 2, bus_height / 2],
	[-bus_depth / 2, -bus_back_width / 2, 0 + boom_height / 2],
	[-bus_depth / 2, -bus_back_width / 2, 0 - boom_height / 2],
	[-bus_depth / 2, -bus_back_width / 2, -bus_height / 2],

	#	boom start verts
	#	i = 16:19
	[boom_depth / 2, y_0 + y_delta, boom_height / 2],
	[boom_depth / 2, y_0 + y_delta, -boom_height / 2],
	[-boom_depth / 2, y_0 - y_delta, -boom_height / 2],
	[-boom_depth / 2, y_0 - y_delta, boom_height / 2],

	#	top and bottom centers
	#	i = 20:21
	[0.0, y_0, bus_height / 2],
	[0.0, y_0, -bus_height / 2],

	#	boom start verts
	#	i = 22:25
	[boom_depth / 2, -y_0 - y_delta, boom_height / 2],
	[boom_depth / 2, -y_0 - y_delta, -boom_height / 2],
	[-boom_depth / 2, -y_0 + y_delta, -boom_height / 2],
	[-boom_depth / 2, -y_0 + y_delta, boom_height / 2],

	#	top and bottom centers
	#	i = 26:27
	[0.0, -y_0, bus_height / 2],
	[0.0, -y_0, -bus_height / 2],

])

bus_faces_array = np.array([

	#	front faces
	[4, 1, 0],
	[4, 5, 1],
	[6, 2, 1],
	[1, 5, 6],
	[7, 3, 2],
	[2, 6, 7],

	#	back faces
	[8, 9, 12],
	[13, 12, 9],
	[9, 10, 13],
	[10, 14, 13],
	[10, 11, 15],
	[15, 14, 10],

	#############################################################
	#	SIDE 1

	#	top side faces
 	[0, 1, 16],
 	[16, 20, 0],
 	[16, 19, 20],
 	[8, 20, 19],
 	[9, 8, 19],

	#	middle side faces
 	[1, 2, 16],
 	[2, 17, 16],
 	[9, 19, 18],
 	[10, 9, 18],

	#	top side faces
 	[2, 3, 17],
 	[21, 17, 3],
 	[21, 18, 17],
 	[11, 10, 21],
 	[21, 10, 18],

	#############################################################
	#	SIDE 2

	#	top side faces
	[22, 5, 4],
	[4, 26, 22],
 	[26, 25, 22],
	[25, 26, 12],
	[25, 12, 13],

	#	middle side faces
	[22, 6, 5],
	[22, 23, 6],
	[24, 25, 13],
	[24, 13, 14],

	#	bottom side faces
	[23, 7, 6],
	[7, 23, 27],
	[23, 24, 27],
	[27, 14, 15],
	[24, 14, 27],

	#############################################################
	#	top and bottom

	#	close top and bottom
	[8, 4, 0],
	[12, 4, 8],
	[3, 7, 11],
	[7, 15, 11],

])

#############################################################################################
#	generate output mesh

#	initialise output mesh
bus_mesh = mesh.Mesh(np.zeros(bus_faces_array.shape[0], dtype=mesh.Mesh.dtype))

#	generate output mesh
for i, f in enumerate(bus_faces_array):
    for j in range(3):
        bus_mesh.vectors[i][j] = bus_verts_array[f[j],:]

#	update mesh properties
bus_mesh.update_areas()
bus_mesh.update_normals()
bus_mesh.update_units()

#############################################################################################
#	Solar panel boom
#############################################################################################

boom_verts_array = np.array([

	#	boom start verts
	#	i = 16:19
	[boom_depth / 2, y_0 + y_delta, boom_height / 2],
	[boom_depth / 2, y_0 + y_delta, -boom_height / 2],
	[-boom_depth / 2, y_0 - y_delta, -boom_height / 2],
	[-boom_depth / 2, y_0 - y_delta, boom_height / 2],

	#	boom end verts
	#	i = 20:23
	[boom_depth / 2, y_0 + boom_width, boom_height / 2],
	[boom_depth / 2, y_0 + boom_width, -boom_height / 2],
	[-boom_depth / 2, y_0 + boom_width, -boom_height / 2],
	[-boom_depth / 2, y_0 + boom_width, boom_height / 2],

	#	boom start verts
	#	i = 26:29
	[boom_depth / 2, -y_0 - y_delta, boom_height / 2],
	[boom_depth / 2, -y_0 - y_delta, -boom_height / 2],
	[-boom_depth / 2, -y_0 + y_delta, -boom_height / 2],
	[-boom_depth / 2, -y_0 + y_delta, boom_height / 2],

	#	boom end verts
	#	i = 30:33
	[boom_depth / 2, -y_0 - boom_width, boom_height / 2],
	[boom_depth / 2, -y_0 - boom_width, -boom_height / 2],
	[-boom_depth / 2, -y_0 - boom_width, -boom_height / 2],
	[-boom_depth / 2, -y_0 - boom_width, boom_height / 2],

])

boom_faces_array = np.array([
	#	boom front faces
 	[0, 1, 4],
 	[1, 5, 4],

	#	boom back faces
 	[2, 3, 6],
 	[3, 7, 6],

	#	boom top faces
 	[7, 3, 0],
 	[7, 0, 4],

	#	boom bottom faces
 	[1, 2, 5],
 	[2, 6, 5],

	#	boom front faces
	[12, 9, 8],
	[12, 13, 9],
 
	#	boom back faces
	[14, 11, 10],
	[14, 15, 11],

	#	boom top faces
	[8, 11, 15],
	[12, 8, 15],

	#	boom bottom faces
	[13, 10, 9],
	[13, 14, 10],

])

#############################################################################################
#	generate output mesh

#	initialise output mesh
boom_mesh = mesh.Mesh(np.zeros(boom_faces_array.shape[0], dtype=mesh.Mesh.dtype))

#	generate output mesh
for i, f in enumerate(boom_faces_array):
    for j in range(3):
        boom_mesh.vectors[i][j] = boom_verts_array[f[j],:]

#	update mesh properties
boom_mesh.update_areas()
boom_mesh.update_normals()
boom_mesh.update_units()

#############################################################################################
#	Solar panels
#############################################################################################

solar_verts_array = np.array([

	#############################################################
	#	Side 1 Solar Panel

	#	i = 0:3
	#	solar panel verts
	[panel_depth / 2, y_0 + boom_width, panel_depth / 2],
	[panel_depth / 2, y_0 + boom_width, -panel_depth / 2],
	[-panel_depth / 2, y_0 + boom_width, -panel_depth / 2],
	[-panel_depth / 2, y_0 + boom_width, panel_depth / 2],

	#	i = 4:7
	#	solar panel inner verts
	[panel_depth / 2, y_0 + boom_width, panel_height / 2],
	[-panel_depth / 2, y_0 + boom_width, panel_height / 2],
	[panel_depth / 2, y_0 + boom_width, -panel_height / 2],
	[-panel_depth / 2, y_0 + boom_width, -panel_height / 2],

	# i = 8:11
	#	solar panel outer verts
	[panel_depth / 2, y_0 + boom_width + panel_width, panel_height / 2],
	[-panel_depth / 2, y_0 + boom_width + panel_width, panel_height / 2],
	[panel_depth / 2, y_0 + boom_width + panel_width, -panel_height / 2],
	[-panel_depth / 2, y_0 + boom_width + panel_width, -panel_height / 2],

	#############################################################
	#	Side 2 Solar Panel

	#	i = 12:15
	#	solar panel verts
	[panel_depth / 2, -y_0 - boom_width, panel_depth / 2],
	[panel_depth / 2, -y_0 - boom_width, -panel_depth / 2],
	[-panel_depth / 2, -y_0 - boom_width, -panel_depth / 2],
	[-panel_depth / 2, -y_0 - boom_width, panel_depth / 2],

	#	i = 16:19
	#	solar panel inner verts
	[panel_depth / 2, -y_0 - boom_width, panel_height / 2],
	[-panel_depth / 2, -y_0 - boom_width, panel_height / 2],
	[panel_depth / 2, -y_0 - boom_width, -panel_height / 2],
	[-panel_depth / 2, -y_0 - boom_width, -panel_height / 2],

	#	solar panel outer verts
	# i = 20:23
	[panel_depth / 2, -y_0 - boom_width - panel_width, panel_height / 2],
	[-panel_depth / 2, -y_0 - boom_width - panel_width, panel_height / 2],
	[panel_depth / 2, -y_0 - boom_width - panel_width, -panel_height / 2],
	[-panel_depth / 2, -y_0 - boom_width - panel_width, -panel_height / 2],

])

##################################################
#	rotate verts

#	rotation axis
axis = np.array([0.0, 1.0, 0.0])

#	convert to rotation matrix
rot_mat = axis2mat(panel_angle, axis)

#	rotate panel verts
for i, vert in enumerate(solar_verts_array):
	solar_verts_array[i] = np.dot(rot_mat, vert)

solar_faces_array = np.array([

#	#############################################################
#	#	solar panel 1
#
#	#	solar panel front
#	[0, 10, 8],
#	[0, 1, 10],
#
#	#	solar panel back
#	[3, 9, 11],
#	[2, 3, 11],
#
#	#	solar panel top/bottom
#	[8, 5, 4],
#	[8, 9, 5],
#	[6, 7, 10],
#	[7, 11, 10],
#
#	#	solar panel outsides
#	[11, 9, 8],
#	[8, 10, 11],
#
#	#############################################################
#	#	solar_panel 2
#	
#	#	solar panel insides
#
#	#	solar panel front
#	[15, 23, 21],
#	[15, 14, 23],
#	
#	#	solar panel back
#	[12, 20, 22],
#	[13, 12, 22],
#	
#	#	solar panel top/bottom
#	[21, 16, 17],
#	[21, 20, 16],
#	[19, 18, 23],
#	[18, 22, 23],
#
#	#	solar panel outsides
#	[22, 20, 21],
#	[21, 23, 22]

	#############################################################
	#	solar panel 1

	#	solar panel insides
	[0, 4, 3],
	[4, 5, 3],
	[1, 2, 7],
	[7, 6, 1],

	#	solar panel front
	[8, 4, 0],
	[0, 10, 8],
	[0, 1, 10],
	[1, 6, 10],

	#	solar panel back
	[3, 5, 9],
	[3, 9, 11],
	[2, 3, 11],
	[11, 7, 2],

	#	solar panel top/bottom
	[8, 5, 4],
	[8, 9, 5],
	[6, 7, 10],
	[7, 11, 10],

	#	solar panel outsides
	[11, 9, 8],
	[8, 10, 11],

	#############################################################
	#	solar_panel 2
	
	#	solar panel insides
	[15, 17, 12],
	[17, 16, 12],
	[14, 13, 18],
	[18, 19, 14],
	
	#	solar panel front
	[21, 17, 15],
	[15, 23, 21],
	[15, 14, 23],
	[14, 19, 23],
	
	#	solar panel back
	[12, 16, 20],
	[12, 20, 22],
	[13, 12, 22],
	[22, 18, 13],
	
	#	solar panel top/bottom
	[21, 16, 17],
	[21, 20, 16],
	[19, 18, 23],
	[18, 22, 23],

	#	solar panel outsides
	[22, 20, 21],
	[21, 23, 22]
])

#############################################################################################
#	generate output mesh

#	initialise output mesh
solar_mesh = mesh.Mesh(np.zeros(solar_faces_array.shape[0], dtype=mesh.Mesh.dtype))

#	generate output mesh
for i, f in enumerate(solar_faces_array):
    for j in range(3):
        solar_mesh.vectors[i][j] = solar_verts_array[f[j],:]

#	update mesh properties
solar_mesh.update_areas()
solar_mesh.update_normals()
solar_mesh.update_units()

y_axis = np.array([0.0, 1.0, 0.0])
solar_mesh.rotate(y_axis, panel_angle)

#	update mesh properties
solar_mesh.update_areas()
solar_mesh.update_normals()
solar_mesh.update_units()

solar_verts_array = solar_mesh.vectors

#############################################################################################
#	Solar panel connectors
#############################################################################################

connector_verts_array = np.array([

	#############################################################
	#	Side 1 Solar Panel

	#	boom end verts
	#	i = 0:3
	[boom_depth / 2, y_0 + boom_width, boom_height / 2],
	[boom_depth / 2, y_0 + boom_width, -boom_height / 2],
	[-boom_depth / 2, y_0 + boom_width, -boom_height / 2],
	[-boom_depth / 2, y_0 + boom_width, boom_height / 2],

	#	solar panel verts
	#	i = 4:7
	[panel_depth / 2, y_0 + boom_width, panel_depth / 2],
	[panel_depth / 2, y_0 + boom_width, -panel_depth / 2],
	[-panel_depth / 2, y_0 + boom_width, -panel_depth / 2],
	[-panel_depth / 2, y_0 + boom_width, panel_depth / 2],


	#############################################################
	#	Side 2 Solar Panel

	#	boom end verts
	#	i = 8:11
	[boom_depth / 2, -y_0 - boom_width, boom_height / 2],
	[boom_depth / 2, -y_0 - boom_width, -boom_height / 2],
	[-boom_depth / 2, -y_0 - boom_width, -boom_height / 2],
	[-boom_depth / 2, -y_0 - boom_width, boom_height / 2],

	# i = 12:15
	#	solar panel verts
	[panel_depth / 2, -y_0 - boom_width, panel_depth / 2],
	[panel_depth / 2, -y_0 - boom_width, -panel_depth / 2],
	[-panel_depth / 2, -y_0 - boom_width, -panel_depth / 2],
	[-panel_depth / 2, -y_0 - boom_width, panel_depth / 2],
])


connector_faces_array = np.array([

	#############################################################
	#	solar panel 1

	#	solar panel connector
	[0, 1, 4],
	[5, 4, 1],
	[1, 2, 5],
	[6, 5, 2],
	[2, 3, 7],
	[7, 6, 2],
	[7, 3, 0],
	[0, 4, 7],

	#############################################################
	#	solar_panel 2
	
	#	solar panel connector
	[12, 9, 8],
	[9, 12, 13],
	[13, 10, 9],
	[10, 13, 14],
	[15, 11, 10],
	[10, 14, 15],
	[8, 11, 15],
	[15, 12, 8],
])
	
#############################################################################################
#	generate output mesh

#	initialise output mesh
solar_connector_mesh = mesh.Mesh(np.zeros(connector_faces_array.shape[0], dtype=mesh.Mesh.dtype))

#	generate output mesh
for i, f in enumerate(connector_faces_array):
    for j in range(3):
        solar_connector_mesh.vectors[i][j] = connector_verts_array[f[j],:]

#	update mesh properties
solar_connector_mesh.update_areas()
solar_connector_mesh.update_normals()
solar_connector_mesh.update_units()

#############################################################################################
#	combine mesh objects
#############################################################################################

combined_mesh = mesh.Mesh(np.concatenate(
	[bus_mesh.data, boom_mesh.data, solar_mesh.data, solar_connector_mesh.data]
))

combined_mesh = mesh.Mesh(np.concatenate(
	[bus_mesh.data, boom_mesh.data, solar_mesh.data]
))

print('Mesh Nfaces = ', len(combined_mesh.normals))

#############################################################################################
#	Graphical Outputs
#############################################################################################

#	set use of LaTeX font
plt.rc('text', usetex=True)

###############################################
#	initilise figure
figure3d = plt.figure(figsize=(6, 8), dpi=300, facecolor='w', edgecolor='k')
ax3d = mplot3d.Axes3D(figure3d)

ax3d.set_xlim([-6, 6])
ax3d.set_ylim([-6, 6])
ax3d.set_zlim([-5, 5])

ax3d.set_xlabel('x - axis')
ax3d.set_ylabel('y - axis')
ax3d.set_zlabel('z - axis')

###############################################
#	set view direction
view_azimuth = -20
view_elevation = 25

ax3d.view_init(view_elevation, view_azimuth)

#############################################################################################
#	plot main bus

vectors = combined_mesh.vectors
normals = combined_mesh.normals

itt = 0

for vector, normal in zip(vectors, normals):

	print('Face is: ', itt)

	if itt == 42:
		itt += 1
		continue

	if itt <= 5:
		face_col = 'tab:red'
		alpha_val = 0.8
		zord = 10
	else:
		face_col = [0.5, 0.5, 0.5]
		alpha_val = 0.5
		zord = 1

	###############################################
	#	rending face

	#	Add final vert to vector array
	vector = np.vstack([vector, vector[0, :]])

	#	extract x, y and z values
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]

	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5, zorder=zord)
	pc.set_edgecolor([0.0, 0.0, 0.0])
	pc.set_facecolor(face_col)
	pc.set_alpha(alpha_val)
	ax3d.add_collection3d(pc)

	###############################################
	#	plotting normal

	#	generate normal unit vector
	normal_unit = normal / np.linalg.norm(normal)

	#	find facet origin
	origin_x = (vector[0, 0] + vector[1, 0] + vector[2, 0]) / 3
	origin_y = (vector[0, 1] + vector[1, 1] + vector[2, 1]) / 3
	origin_z = (vector[0, 2] + vector[1, 2] + vector[2, 2]) / 3

	#	generate normal plot
	normal_origin = np.array([origin_x, origin_y, origin_z])
	normal_plot = np.vstack([normal_origin, normal_unit + normal_origin])

#	#	plot normal
#	ax3d.plot(normal_plot[:, 0], normal_plot[:, 1], normal_plot[:, 2])

	itt += 1

#	plt.pause(0.01)
#	input('\nEnter...')

###############################################################################################
#	generate plottable body frame axis vectors

body_axes_scale = 5

origin = np.array([0.0, 0.0, 0.0])

x_axis = np.array([1.0, 0.0, 0.0])
y_axis = np.array([0.0, 1.0, 0.0])
z_axis = np.array([0.0, 0.0, 1.0])

font_size = 20

###############################################
#	add body x axis
bodyx_vec_plot = body_axes_scale * np.vstack((origin, x_axis))
a = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
	mutation_scale=10, lw=2, arrowstyle="-|>", color="k"
)
ax3d.add_artist(a)
ax3d.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1], bodyx_vec_plot[1, 2],
	r'$x_{b}$', fontsize=font_size, fontweight='bold'
)

ax3d.plot(
	bodyx_vec_plot[:, 0] / 2, bodyx_vec_plot[:, 1] / 2, bodyx_vec_plot[:, 2] / 2,
	lw=2, color='k'
)

###############################################
#	add body y axis
bodyy_vec_plot = 1.2 * body_axes_scale * np.vstack((origin, y_axis))
a = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
	mutation_scale=10, lw=2, arrowstyle="-|>", color="k"
)
ax3d.add_artist(a)
ax3d.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	r'$y_{b}$', fontsize=font_size, fontweight='bold'
)

ax3d.plot(
	bodyy_vec_plot[:, 0] / 2, bodyy_vec_plot[:, 1] / 2, bodyy_vec_plot[:, 2] / 2,
	lw=2, color='k'
)

###############################################
#	add body z axis
bodyz_vec_plot = body_axes_scale * np.vstack((origin, z_axis))
a = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
	mutation_scale=10, lw=2, arrowstyle="-|>", color="k"
)
ax3d.add_artist(a)
ax3d.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	r'$z_{b}$', fontsize=font_size, fontweight='bold'
)

ax3d.plot(
	bodyz_vec_plot[:, 0] / 2, bodyz_vec_plot[:, 1] / 2, bodyz_vec_plot[:, 2] / 2,
	lw=2, color='k'
)

plt.axis('off')
plt.pause(0.01)

###############################################################################################
#	save figure
if savefile_flag is True:

	out_str = os.path.join(geom_file_path, geom_file_name)

	if os.path.exists(out_str):
		print('\nPath: ', out_str, ' exists.')
		ans = input('\nReplace File? [y/n] ')

		if ans == 'y':
			saveflag = True
		else:
			saveflag = False

	else:
		saveflag = True

	if saveflag == True:
		print('Saving: ', out_str)
		figure3d.savefig(out_str, bbox_inches="tight", dpi=300)
		print('\n#######################################')
		print('Figure saved: ', geom_file_name)
	else:
		print('\nNot saving.')
	
print('\nEND')

plt.show()
sys.exit()

