###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

import numpy as np
import sys
import os

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors

from stl import mesh

#############################################################################################
#	inputs

#	define target radius
radius = 1.0

#	set division level
division_level = 2

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set flag
savefile_flag = True
savefile_flag = False

#	output file name
fig_name = 'geometry_model_figures' + '.pdf'

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)
#############################################################################################
#	initilisation

#	define golden ratio
GR = (1.0 + np.sqrt(5.0)) / 2.0

geom_id = division_level

#	calculate required edge length

#	vertices radius
edge_length = 0.5 * ((radius * 2.0) / (np.sqrt(GR * np.sqrt(5))))

print('\nRadius = ', radius)
print('\nRequired edge length = ', edge_length)

vert_coord = edge_length *  GR

#############################################################################################
#	define initial icosphere

vertices = np.array([
	[-edge_length,  vert_coord, 0],
	[ edge_length,  vert_coord, 0],
	[-edge_length, -vert_coord, 0],
	[ edge_length, -vert_coord, 0],

	[0, -edge_length,  vert_coord],
	[0,  edge_length,  vert_coord],
	[0, -edge_length, -vert_coord],
	[0,  edge_length, -vert_coord],

	[ vert_coord, 0, -edge_length],
	[ vert_coord, 0,  edge_length],
	[-vert_coord, 0, -edge_length],
	[-vert_coord, 0,  edge_length]
])

#	5 faces around point 0
faces = np.array([
	[0, 11, 5],
	[0, 5, 1],
	[0, 1, 7],
	[0, 7, 10],
	[0, 10, 11],
	[1, 5, 9],
	[5, 11, 4],
	[11, 10, 2],
	[10, 7, 6],
	[7, 1, 8],
	[3, 9, 4],
	[3, 4, 2],
	[3, 2, 6],
	[3, 6, 8],
	[3, 8, 9],
	[4, 9, 5],
	[2, 4, 11],
	[6, 2, 10],
	[8, 6, 7],
	[9, 8, 1]
])

print('\ninitial face array =\n', faces)
print('\ninitial vertex array =\n', vertices)

#############################################################################################
#	refinemnet

division_level_list = np.array([1, 2, 3, 4, 5])

#	initialise figure
figure3d = plt.figure(
	figsize=(10, 2.0),
	dpi=300,
	facecolor='w',
	edgecolor='k',
)

nfaces_array = []

for division_level in division_level_list:

	print('\nRefining with Division Level of: ', division_level)
	
	if division_level == 1:
	
		new_faces = np.copy(faces)
		new_verts = np.copy(vertices)
	
	elif division_level > 1:
	
		print('\nLooping through faces...')
		print('######################################')
	
		#	number of required triangles
		ntri = division_level**2
	
		for index, face in enumerate(faces):
	
			print('\n	Working on face: ', index)
		
			#	set current verts
			verts = vertices[face]
			vert1 = verts[0]
			vert2 = verts[1]
			vert3 = verts[2]
	
			#############################################################################################
			#	triangle generation
	
			#	full triangle vectors
			vector1 = vert2 - vert1
			vector2 = vert3 - vert2
			vector3 = vert1 - vert3
	
			#	building block vectors
			prime_vec1 = np.divide(-vector3, division_level)
			prime_vec2 = np.divide(vector1, division_level)
	
			#	initilise vertices of step array
			verts_step = []
	
			#	initilise new faces array
			faces_step = np.zeros((ntri, 3))
			
			#	generate new vertices
			print('\n	Generating new vertices.')
	
			row_num = 1
			face_num = 0
			vert_num = 0
	
			nrows = division_level + 1
			ncols = division_level + 1
	
			prev_vert_nums = []
	
			#	generate row 0 verts
			for col in range(0, ncols):
				temp_vert = vert1 + np.multiply(col, prime_vec2)
	
				#	limit number of decimal places and convert small values to 0
				if -1e-10 < temp_vert[0] < 1e-10:
					temp_vert[0] = 0.0
				else:
					temp_vert[0] = round(temp_vert[0], 8)
	
				if -1e-10 < temp_vert[1] < 1e-10:
					temp_vert[1] = 0.0
				else:
					temp_vert[1] = round(temp_vert[1], 8)
	
				if -1e-10 < temp_vert[2] < 1e-10:
					temp_vert[2] = 0.0
				else:
					temp_vert[2] = round(temp_vert[2], 8)
	
				#	append to verts of step
				verts_step.append(temp_vert)
				prev_vert_nums.append(vert_num)
	
				vert_num += 1
	
			#	generate numpy arrays
			verts_step = np.array(verts_step)
			prev_vert_nums = np.array(prev_vert_nums)
	
			print('\n	Row 0 verts generated.')
			print('\n	Looping through subsequent rows..')
			print('	######################################')
	
			#	loop through subsequent rows
			for row in range(1, nrows):
	
				row_verts = []
				row_vert_nums = []
	
				#	loop through columns of row
				for col in range(0, ncols - row):
	
					#	generate vert
					temp_vert = vert1 + np.multiply(col, prime_vec2) + np.multiply(row, prime_vec1)
	
					#	limit number of decimal places and convert small values to 0
					if -1e-10 < temp_vert[0] < 1e-10:
						temp_vert[0] = 0.0
					else:
						temp_vert[0] = round(temp_vert[0], 8)
	
					if -1e-10 < temp_vert[1] < 1e-10:
						temp_vert[1] = 0.0
					else:
						temp_vert[1] = round(temp_vert[1], 8)
	
					if -1e-10 < temp_vert[2] < 1e-10:
						temp_vert[2] = 0.0
					else:
						temp_vert[2] = round(temp_vert[2], 8)
	
					#	append to verts of row
					row_verts.append(temp_vert)
	
					#	append to vert number list
					row_vert_nums.append(vert_num)
	
					#	iterate vert number
					vert_num += 1
	
				#	set number of vert elements in row
				nele = len(row_verts)
				ele_factor = 0
	
				#	loop through elements in row, generate face indices
				for row_index, (row_vert, vert_id) in enumerate(zip(row_verts, row_vert_nums)):
	
	
					#	first tri of row
					if row_index == 0:
	
						#	generate 1 set of face indices
						face_index = [
							prev_vert_nums[row_index],
							prev_vert_nums[row_index + 1],
							vert_id
						]
	
						faces_step[face_num] = face_index
						face_num += 1
	
					#	subsequent tris of row
					elif row_index < nele - ele_factor:
	
						#	generate 2 sets of face indices
						face_index = [prev_vert_nums[0 + row_index], vert_id, vert_id - 1]
						faces_step[face_num] = face_index
						face_num += 1
	
						face_index = [
							prev_vert_nums[0 + row_index],
							prev_vert_nums[0 + row_index + 1],
							vert_id
						]
	
						faces_step[face_num] = face_index
						face_num += 1
	
					#	row finished, set prev_vert_nums
					if row_index == nele - 1:
						prev_vert_nums = row_vert_nums
	
				#	generate numpy array
				row_verts = np.array(row_verts)
	
				#	add verts of row to vert of step output array
				verts_step = np.vstack((verts_step, row_verts))
	
			#	finalise step outputs
			verts_step = np.array(verts_step)
			faces_step = np.array(faces_step).astype(np.int)
	
			###########################################################################################
			#	scale verts by desired radius
	
			#	initilise scaled vertices array
			verts_step_scaled = np.zeros((verts_step.shape))
	
			for i, vert in enumerate(verts_step):
	
				#	vert magnitude
				vert_mag = np.linalg.norm(vert)
	
				#	vert unit vector
				vert_unit = np.divide(vert, vert_mag)
		
				#	scale by radius
				verts_step_scaled[i, :] = np.multiply(vert_unit, radius)
	
			#############################################################################################
			#	add data to outputs
	
			print('\n	Reccording outputs.')
	
			#	for first face
			if index == 0:
	
				print('\n	Index = 0, Initilising outputs...\n')
	
				#	for first face, initilise outputs
				new_verts = np.copy(verts_step_scaled)
				new_faces = np.copy(faces_step)
				end_index = len(new_verts) - 1
	
			else:
	
				print('\n	Checking for vert of face existance in output...\n')
	
				for subface in faces_step:
	
					#	initilise new face index
					new_face_index = np.array([0, 0, 0])
	
					#	parse subverts of face
					subverts = verts_step_scaled[subface]
					subvert1 = subverts[0]
					subvert2 = subverts[1]
					subvert3 = subverts[2]
	
					#	set found flags
					flag1 = False
					flag2 = False
					flag3 = False
	
					#	check for subvert presence in output verts
					for subindex, existing_vert in enumerate(new_verts):
	
						#	found subvert1
						if np.array_equal(subvert1, existing_vert) is True:
							flag1 = True
							index1 = subindex
	
						#	found subvert2
						if np.array_equal(subvert2, existing_vert) is True:
							flag2 = True
							index2 = subindex
	
						#	found subvert3
						if np.array_equal(subvert3, existing_vert) is True:
							flag3 = True
							index3 = subindex
	
					#	add additional vertices as required
					if flag1 is False:
						new_verts = np.vstack((new_verts, subvert1))
						end_index += 1
						index1 = np.copy(end_index)
	
					if flag2 is False:
						new_verts = np.vstack((new_verts, subvert2))
						end_index += 1
						index2 = np.copy(end_index)
	
					if flag3 is False:
						new_verts = np.vstack((new_verts, subvert3))
						end_index += 1
						index3 = end_index
	
					#	assemble new face indices
					new_face_index = np.array([index1, index2, index3])
	
					#	add face to new_faces
					new_faces = np.vstack((new_faces, new_face_index))
	
			print('\n	Face completed.')
	
	
	print('\nFinished.\n')
	print('New nfaces = ', len(new_faces))
	print('New nverts = ', len(new_verts))
	print('New nfaces = ', new_faces.shape)
	print('New nverts = ', new_verts.shape)

	nfaces_array.append(len(new_faces))
	
	###################################################################################################
	#	graphical output
	###################################################################################################
	
	axes3d = figure3d.add_subplot(1, 5, division_level, projection='3d')
		
	for plot_face in new_faces:
	
		plot_verts = new_verts[plot_face]
	
		vert = np.vstack([plot_verts, plot_verts[0, :]])
		
		xs = vert[:, 0]
		ys = vert[:, 1]
		zs = vert[:, 2]
		
		verts = [list(zip(xs, ys, zs))]
		
		alpha = 0.3
		
		#	Graphics Output
		pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
		pc.set_alpha(alpha)
		pc.set_edgecolor([0.0, 0.0, 0.0])
		pc.set_facecolor('b')
		
		axes3d.add_collection3d(pc)
		
	axes3d.set_xlim([-0.7, 0.7])
	axes3d.set_ylim([-0.7, 0.7])
	axes3d.set_zlim([-0.6, 0.6])
#	axes3d.set_xlabel('x axis')
#	axes3d.set_ylabel('y axis')
#	axes3d.set_zlabel('z axis')

	plt.axis('off')

print('\nnfaces array: ', nfaces_array)

####################################################
#	save figure
plt.pause(0.001)
if savefile_flag is True:

	out_str = os.path.join(fig_path, fig_name)
	print('Output Path: ', out_str)

	if os.path.exists(out_str):
		check = input('\nFile already exists, overwrite? [y/n] ')
	else:
		check = 'y'

	if check == 'y':

		print('Saving: ', out_str)
	
		figure3d.savefig(out_str, bbox_inches="tight", dpi=300)
		print('\n#######################################')
		print('Figure saved: ', fig_name)
	
	else:

		print('Not overwriting existing file.')

sys.exit()
