###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat, rot3

#	matplotlib
import matplotlib.pyplot as plt

from scipy.optimize import curve_fit

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Light curve model sensitivity analysis')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

lightcurve_path = 'lightcurve_dir/sensitivity_study/lightcurves'

###################################################################################################
#	light curve generation inputs
###################################################################################################

#	distance to object (km)
obs_dist = 1000

#	list of geometry angles
geom_str_list = [
	'01_div1_sphere',
	'02_div2_sphere',
	'03_div3_sphere',
	'04_div4_sphere',
	'05_div5_sphere'
]

#	illumination vector position
illumvec_angle_multi = 0.5
illumvec_angle = (np.pi / 2) * illumvec_angle_multi

# #	define range for brdf parameter
# #	m
# brdf_range = np.linspace(0.05, 0.5, 10)
# #	omega
# brdf_range = np.linspace(0.1, 0.9, 9)
# #	case string
# case_string = 'omega'

#	case string
case_string = 'illumvec'

angle_multiplier = np.linspace(0.0, 1.0, 11)
angle_range = angle_multiplier * np.pi

###################################################################################################
#	attitude state
init_attit = np.array([0.0, 0.0, 0.0])

rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
rot_vec_vel_orb = np.array([1.0, 0.0, 0.0])
rot_ang_per_sec = 0.0

#	generate attitude motion vector
attit_mot_vec = np.array([1.0, 0.0, 0.0]) * rot_ang_per_sec

###################################################################################################
#	BRDF inputs

#	set initial BRDF values
d_brdf = 1.0
omega_brdf = 0.5
m_brdf = 0.2

#	geometry file
geom_dir = 'geometry_dir/geom_dir/sphere'

###################################################################################################
#	results generation loop
###################################################################################################

results_array = []

print('\n####################################################################################')
print('Looping through sensitivity study inputs...')

# for brdf_param in brdf_range:
# 
# 	print('\n	############################################################################')
# 	print('	BRDF Parameter = ', brdf_param)
# 	#	set brdf parameter
# 	m_brdf = brdf_param
# 	#	set brdf parameter
# 	omega_brdf = brdf_param

for (angle, angle_multi) in zip(angle_range, angle_multiplier):

	print('\n	############################################################################')
	print('	Angle Multiplier =	', angle_multi)
	print('	Angle =		', angle)

	diff_array = []

	###############################################################################################
	#	geometry file loop
	###############################################################################################

	for geom_str in geom_str_list:

		print('\n		####################################################################')
		print('		Working on geometry file: ', geom_str)
		
		#	set loop start time
		loop_start = time.time()
	
		###########################################################################################
		#	reinitialise geometry inputs
		
		geom_file = geom_str + '.stl'
		
		#	load static mesh
		body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
		
		#	set vectors and areas
		body_vectors = body_mesh.vectors
		body_normals = body_mesh.normals
		body_areas = body_mesh.areas
		
		nfaces = len(body_areas)
			
#		#	generate file name
#		lc_filename = '{}_{}_{}_{}_{}_{}_{}_{}_{}'.format(
#			geom_str,
#			np.round(nfaces, 2),
#			np.round(omega_brdf, 2),
#			np.round(m_brdf, 2),
#			np.round(d_brdf, 2),
#			np.round(init_attit, 2),
#			np.round(attit_mot_vec, 2),
#			case_string,
#			np.round(illumvec_angle_multi, 2),
#		)

		#	generate file name
		lc_filename = '{}_{}_{}_{}_{}_{}_{}_{}_{}'.format(
			geom_str,
			np.round(nfaces, 2),
			np.round(omega_brdf, 2),
			np.round(m_brdf, 2),
			np.round(d_brdf, 2),
			np.round(init_attit, 2),
			np.round(attit_mot_vec, 2),
			case_string,
			np.round(angle_multi, 2)
		)

		print('\n		Lightcurve file is:\n		', lc_filename)

		if os.path.exists(os.path.join(path_data, lightcurve_path, lc_filename)):
			print('\n	Rename file:')

			new_case_string = 'illumvec'

			new_lc_filename = '{}_{}_({}_{}_{})_[{},{},{}]_[{},{},{}]_{}_{}'.format(
				geom_str,
				np.round(nfaces, 2),
				np.round(omega_brdf, 2),
				np.round(m_brdf, 2),
				np.round(d_brdf, 2),
				*np.round(init_attit, 2),
				*np.round(attit_mot_vec, 2),
				np.round(illumvec_angle_multi, 2),
				new_case_string,
			)

			os.rename(
				os.path.join(path_data, lightcurve_path, lc_filename),
				os.path.join(path_data, lightcurve_path, new_lc_filename)
			)

			print('\n		New Lightcurve file is:\n		', new_lc_filename)
		
		else:
			print('\n	FILE NOT FOUND')

