###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat, rot3

#	matplotlib
import matplotlib.pyplot as plt

from scipy.optimize import curve_fit

##################################################################################################
#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Light curve model sensitivity analysis')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

lightcurve_path = 'lightcurve_dir/sensitivity_study/lightcurves'

###################################################################################################
#	light curve generation inputs
###################################################################################################

#	set show lightcurves flag
show_lc_flag = True
show_lc_flag = False

#	set file save data
savefile_flag = False
savefile_flag = True

fig_name = 'attit_mot_diff.eps'
fig_name = 'attit_mot_spec.eps'

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)

#	distance to object (km)
obs_dist = 1000

#	list of geometry angles
geom_str_list = [
	'01_div1_sphere',
	'02_div2_sphere',
	'03_div3_sphere',
	'04_div4_sphere',
	'05_div5_sphere'
]

#	illumination vector position
illumvec_angle_multi = 0.5
illumvec_angle = (np.pi / 2) * illumvec_angle_multi

##################################################
#	attit
#	case string
case_string = 'attit_mot'

angle_multiplier = np.linspace(0.0, 1.0, 11)

rpm = 2
angle_range = angle_multiplier * (rpm * (2 * np.pi) / 60)

baseline_rot_ang_per_sec = 0.0
#baseline_rot_ang_per_sec = (rpm * (2 * np.pi) / 60)

#	x
rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
baseline_rot_vec_body_unit = np.array([1.0, 0.0, 0.0])

#	y
rot_vec_body_unit = np.array([0.0, 1.0, 0.0])
baseline_rot_vec_body_unit = np.array([0.0, 1.0, 0.0])

#	z
rot_vec_body_unit = np.array([0.0, 0.0, 1.0])
baseline_rot_vec_body_unit = np.array([0.0, 0.0, 1.0])

###################################################################################################
#	BRDF inputs

#	set initial BRDF values
d_brdf = 1.0
omega_brdf = 0.9
m_brdf = 0.2

###################################################################################################
#	time data

#	time step
time_step = 0.1

#	total duration
duration_sec = 180

#	number of data points
len_record = int(np.round(duration_sec / time_step, 1)) + 1

start_date = [2020, 1, 1, 0, 0, 0]
start_jd = date2jd(*start_date)

stop_jd = np.add(*start_jd) + (duration_sec / (24 * 60 * 60))
stop_date = jd2date(*jday_jfrac(stop_jd))

###################################################################################################
#	generate fake inputs to gen_vectors

#	orbital motion
orbital_mot = 0.01

#	satellite position/velocity array
sat_orb = np.zeros([len_record, 7])
sat_orb[:, 1] = 1.0
sat_orb[:, 4] = 1.0

#	illumination vector position
illum_vec = np.array([1.0, 0.0, 0.0])
illum_vec = rot3(illum_vec, illumvec_angle)

#	sun vector array
sun_orb = np.zeros([len_record, 7])
sun_orb[:, 1:4] = illum_vec
sun_orb[:, 4] = 1.0

#	earth vector array
earth_orb = np.zeros([len_record, 7])
earth_orb[:, 1] = 1.0
earth_orb[:, 4] = 1.0

#	site vector array
site_orb = np.zeros([len_record, 7])
site_orb[:, 1] = obs_dist
site_orb[:, 4] = 1.0

###################################################################################################
#	attitude state
init_attit = np.array([0.0, 0.0, 0.0])

rot_vec_vel_orb = np.array([1.0, 0.0, 0.0])

#	generate attitude motion vector
baseline_attit_mot_vec = baseline_rot_vec_body_unit * baseline_rot_ang_per_sec

#	observation and illumination vector directions
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])
	
###################################################################################################
#	generate baseline lightcurve
###################################################################################################

print('\n####################################################################################')
print('Generating Baseline Light Curve...')

###################################################################################################
#	load geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir/sphere'

baseline_geom_str = '05_div5_sphere'
baseline_geom_file = baseline_geom_str + '.stl'

#	load static mesh
baseline_body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, baseline_geom_file))

#	set vectors and areas
baseline_body_vectors = baseline_body_mesh.vectors
baseline_body_normals = baseline_body_mesh.normals
baseline_body_areas = baseline_body_mesh.areas

baseline_nfaces = len(baseline_body_areas)

###################################################################################################
#	check for file existance

#	generate file name
lc_filename = '{}_{}_({}_{}_{})_[{},{},{}]_[{},{},{}]_{}_{}'.format(
	baseline_geom_str,
	np.round(baseline_nfaces, 2),
	np.round(omega_brdf, 2),
	np.round(m_brdf, 2),
	np.round(d_brdf, 2),
	*np.round(init_attit, 2),
	*np.round(baseline_attit_mot_vec, 2),
	np.round(illumvec_angle_multi, 2),
	case_string,
)

print('\nBaseline File:\n	', lc_filename)

gen_start = time.time()
	
if os.path.exists(os.path.join(path_data, lightcurve_path, lc_filename)):
	print('\n	Loading...')
	light_curve_array = np.loadtxt(os.path.join(path_data, lightcurve_path, lc_filename))

else:
	print('\n	Generating...')

	###############################################################################################
	#	generate brdf
	
	#	Refractive Index
	n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))
	
	#	Specular ratio
	s_brdf = 1.0 - d_brdf
	
	#	generate brdf
	baseline_brdf_data = np.zeros([baseline_nfaces, 5])
	
	for i in range(baseline_nfaces):
		baseline_brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
	
	###############################################################################################
	#	perform light curve vector generation
	
	#	vector generation function
	vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, baseline_rot_vec_body_unit, baseline_rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb
	)
	
	##################################################
	#	parse vector data
	
	#	extract orbital positions of custom input vectors
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_body[:, 31:34]
	
	#	body data
	sun_body_data = vector_data_body[:, 28:31]
	site_body_data = vector_data_body[:, 31:34]
	earth_body_data = vector_data_body[:, 19:22]
	sat_body_vel = vector_data_body[:, 22:25]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	
	illum_vec_body_data = vector_data_body[:, 28:31]
	obs_vec_body_data = vector_data_body[:, 31:34]
	
	#	orb data
	sun_orb_data = vector_data_orb[:, 28:31]
	site_orb_data = vector_data_orb[:, 31:34]
	earth_orb_data = vector_data_orb[:, 19:22]
	sat_orb_vel = vector_data_orb[:, 22:25]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_orb[:, 31:34]
	
	###############################################################################################
	#	generate light curve

	#	generate sim time
	sim_time = np.linspace(0, duration_sec, len_record)
	
	#	initilise lightcurve output
	light_curve_array = np.zeros((len_record, 4))
	light_curve_array[:, 1] = sim_time
	
	jdate_start = start_jd[0] + start_jd[1]

	#	run light curve generation function
	light_curve_array = glim_slc_new(
		illum_vec_body_data, obs_vec_body_data, light_curve_array,
		jdate_start, baseline_body_vectors, baseline_body_areas, baseline_brdf_data
	)

	#	save lightcurve file
	np.savetxt(os.path.join(path_data, lightcurve_path, lc_filename), light_curve_array)

#	extract light curve data
baseline_lc_seconds = light_curve_array[:, 1]
baseline_lc_app = light_curve_array[:, 2]
baseline_lc_std = light_curve_array[:, 3]

gen_end = time.time()
print('\n	Baseline generation time: {}'.format(np.round(gen_end - gen_start, 1)))




##################################################################################################
#	get fourier coefficients of baseline signal


x_base = baseline_lc_seconds
y_base = baseline_lc_app



##################################################
#	mimic signal in negative domain for sines only approximation
x_neg = -x_base[1:]
x_neg = np.flip(x_neg)
y_neg = y_base[1:]
y_neg = np.flip(y_neg)

x_base = np.append(x_neg, x_base)
y_base = np.append(y_neg, y_base)

##################################################
#	get starting coefficents

#	set P as domain length
P = len(x_base) - 1

#	find a0 and b0
a_0 = (2 / P) * np.trapz(y_base, x_base)
b_0 = 0.0

#	starting number of terms
N = 10000

#	set range of n
nvals = np.arange(1, N, 1)
	
#	initialise coefficients
base_coefs = np.zeros([len(nvals) + 1])
base_coefs[0] = a_0

#	find remaining coefficients
for i, n in enumerate(nvals):

	#	find a_n coefficient
	a_n = (2 / P) * np.trapz(y_base * np.cos(2 * np.pi * x_base * (n / P)), x_base)
	if -1e-10 < a_n < 1e-10:
		a_n = 0.0

	#	find b_n coefficient
	b_n = (2 / P) * np.trapz(y_base * np.sin(2 * np.pi * x_base * (n / P)), x_base)
	if -1e-10 < b_n < 1e-10:
		b_n = 0.0

	#	add to output array
	base_coefs[i + 1] = a_n

	#	check for zero b coefficent
	if b_n != 0.0:
		print('Non-Zero b coefficient!')
		sys.exit()

##################################################
#	generate fourier series
approx_base_signal = a_0 / 2

for i, n in enumerate(nvals):
	approx_base_signal += base_coefs[i + 1] * np.cos(2 * np.pi * n * x_base / P)\

#	calculate rms and norm
current_rms = rms_calc(y_base, approx_base_signal)
base_norm = np.linalg.norm(base_coefs)


plt.rc('text', usetex=True)

#fig_base, ax_base = plt.subplots(dpi=300)
#
#
#ax_base.plot(
#	baseline_lc_seconds[:-10], baseline_lc_app[:-10],
#	linestyle='solid',
#	linewidth=2.0,
#	color='k',
#	label=r'$\mathrm{Baseline\ Lightcurve}$'
#)
#
#ax_base.plot(
#	baseline_lc_seconds[:-10], approx_base_signal[len(baseline_lc_seconds) - 1:-10],
#	linestyle='dashed',
#	linewidth=1.0,
#	color='tab:red',
#	label=r'$\mathrm{Hilbert\ Approximation}$'
#)
#
#ax_base.set_xlabel(r'$\mathrm{Time\ /\ s}$')
#ax_base.set_ylabel(r'$\mathrm{Apparent\ Magnitude}$')
#
#ax_base.set_ylim(-1.2, -2.0)
#
#ax_base.legend(loc='upper left', frameon=False)
#
#
#current_rms = rms_calc(
#	baseline_lc_app[:-10],
#	approx_base_signal[len(baseline_lc_seconds) - 1:-10]
#)
#print('\n			RMS to Actual:		', current_rms)
#
#plt.pause(0.1)
#
#####################################################
##	save figure
#if savefile_flag is True:
#
#	fig_name = 'hilbert_space_baseline_approx_1.eps'
#	out_str = os.path.join(fig_path, fig_name)
#
#	print('\nAttempting to save to: ', out_str)
#
#	#	check for file existence
#	if os.path.exists(out_str):
#		ans = input('\nFile already exists, overwrite? [y/n]')
#
#		if ans == 'y':
#			write = True
#			print('\nOverwritng...')
#		else:
#			print('\nKeeping existing file...')
#			write = False
#
#	else:
#			write = True
#
#	if write is True:
#		print('\nSaving: ', out_str)
#		fig_base.savefig(out_str, bbox_inches="tight", dpi=300)
#		print('\n#######################################')
#		print('Figure saved: ', fig_name)

#plt.show()
#sys.exit()


##################################################################################################
#	generate number of faces array

nfaces_plot = []

for geom_str in geom_str_list:

	geom_file = geom_str + '.stl'
	
	#	load static mesh
	body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
	
	#	set vectors and areas
	body_vectors = body_mesh.vectors
	body_normals = body_mesh.normals
	body_areas = body_mesh.areas
	
	nfaces = len(body_areas)

	nfaces_plot.append(nfaces)

#	convert to numpy array
nfaces_plot = np.array(nfaces_plot)

###################################################################################################
#	results generation loop
###################################################################################################

results_array = []

#fig_case, ax_case = plt.subplots(dpi=300)


print('\n####################################################################################')
print('Looping through sensitivity study inputs...')

full_start = time.time()

for (angle, angle_multi) in zip(angle_range, angle_multiplier):

	print('\n	############################################################################')
	print('	Angle Multiplier =	', angle_multi)
	print('	Angle =		', angle)

	diff_array = []

	###############################################################################################
	#	reset light curve generation inputs

	rot_ang_per_sec = angle

	#	generate attitude motion vector
	attit_mot_vec = rot_vec_body_unit * rot_ang_per_sec

	###############################################################################################
	#	perform light curve vector generation
	
	#	vector generation function
	vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb
	)
	
	##################################################
	#	parse vector data
	
	#	extract orbital positions of custom input vectors
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_body[:, 31:34]
	
	#	body data
	sun_body_data = vector_data_body[:, 28:31]
	site_body_data = vector_data_body[:, 31:34]
	earth_body_data = vector_data_body[:, 19:22]
	sat_body_vel = vector_data_body[:, 22:25]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	
	illum_vec_body_data = vector_data_body[:, 28:31]
	obs_vec_body_data = vector_data_body[:, 31:34]
	
	#	orb data
	sun_orb_data = vector_data_orb[:, 28:31]
	site_orb_data = vector_data_orb[:, 31:34]
	earth_orb_data = vector_data_orb[:, 19:22]
	sat_orb_vel = vector_data_orb[:, 22:25]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_orb[:, 31:34]

	###############################################################################################
	#	geometry file loop
	###############################################################################################

	if show_lc_flag == True:
		temp_fig, temp_axes = plt.subplots()
		temp_axes.set_title('RPM = ' + str(np.round(angle, 2)))
		temp_axes.set_xlabel('Time/s')
		temp_axes.set_ylabel('Apparent Mag')
		temp_axes.invert_yaxis()

	for geom_str in geom_str_list:

		print('\n		####################################################################')
		print('		Working on geometry file: ', geom_str)
		
		#	set loop start time
		loop_start = time.time()
	
		###########################################################################################
		#	reinitialise geometry inputs
		
		geom_file = geom_str + '.stl'
		
		#	load static mesh
		body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
		
		#	set vectors and areas
		body_vectors = body_mesh.vectors
		body_normals = body_mesh.normals
		body_areas = body_mesh.areas
		
		nfaces = len(body_areas)
			
		#	generate file name
		lc_filename = '{}_{}_({}_{}_{})_[{},{},{}]_[{},{},{}]_{}_{}'.format(
			geom_str,
			np.round(nfaces, 2),
			np.round(omega_brdf, 2),
			np.round(m_brdf, 2),
			np.round(d_brdf, 2),
			*np.round(init_attit, 2),
			*np.round(attit_mot_vec, 2),
			np.round(illumvec_angle_multi, 2),
			case_string,
		)

		print('\n		Lightcurve file is:\n		', lc_filename)

		###########################################################################################
		#	check for file existance
		###########################################################################################

		if os.path.exists(os.path.join(path_data, lightcurve_path, lc_filename)):
			print('\n		Loading...')
			light_curve_array = np.loadtxt(os.path.join(path_data, lightcurve_path, lc_filename))
		
		else:
			print('\n		Generating...')
		
			#######################################################################################
			#	regenerate brdf
			brdf_data = np.zeros([nfaces, 5])
			
			#	Refractive Index
			n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))
			
			#	Specular ratio
			s_brdf = 1.0 - d_brdf
	
			for i in range(nfaces):
				brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
		
			#######################################################################################
			#	generate light curve
			#######################################################################################
			
			print('\n			Running light curve generation...')

			#######################################################################################
			#	initialise light curve generation
			
			#	generate sim time
			sim_time = np.linspace(0, duration_sec, len_record)
			
			#	initilise lightcurve output
			light_curve_array = np.zeros((len_record, 4))
			light_curve_array[:, 1] = sim_time
			
			jdate_start = start_jd[0] + start_jd[1]
			
			#######################################################################################
			#	generate payload no shadowing light curve
			
			#	run light curve generation function
			light_curve_array_pl_noshad = glim_slc_new(
				illum_vec_body_data, obs_vec_body_data, light_curve_array,
				jdate_start, body_vectors, body_areas, brdf_data
			)
		
			#	save
			np.savetxt(os.path.join(path_data, lightcurve_path, lc_filename), light_curve_array)
	
		#	extract light curve data
		lc_seconds = light_curve_array[:, 1]
		lc_app = light_curve_array[:, 2]
		lc_std = light_curve_array[:, 3]
	
		for i2, line2 in enumerate(lc_app):
			if np.isnan(line2):
				lc_app[i2] = 0.5 * (lc_app[i2 - 1] + lc_app[i2 + 1])

		#	calculate rms difference from baseline
		rms_diff = rms_calc(baseline_lc_app, lc_app)









		###########################################################################################
		#	get fourier coefficients of baseline signal
		
		x_case = lc_seconds
		y_case = lc_app
		
		##################################################
		#	mimic signal in negative domain for sines only approximation
		x_neg = -x_case[1:]
		x_neg = np.flip(x_neg)
		y_neg = y_case[1:]
		y_neg = np.flip(y_neg)
		
		x_case = np.append(x_neg, x_case)
		y_case = np.append(y_neg, y_case)
		
		##################################################
		#	get starting coefficents
		
		#	set P as domain length
		P = len(x_case) - 1
		
		#	find a0 and b0
		a_0 = (2 / P) * np.trapz(y_case, x_case)
		b_0 = 0.0
		
		#	starting number of terms
		N = 10000
		
		#	set range of n
		nvals = np.arange(1, N, 1)
			
		#	initialise coefficients
		case_coefs = np.zeros([len(nvals) + 1])
		case_coefs[0] = a_0
		
		#	find remaining coefficients
		for i, n in enumerate(nvals):
		
			#	find a_n coefficient
			a_n = (2 / P) * np.trapz(y_case * np.cos(2 * np.pi * x_case * (n / P)), x_case)
			if -1e-10 < a_n < 1e-10:
				a_n = 0.0
		
			#	find b_n coefficient
			b_n = (2 / P) * np.trapz(y_case * np.sin(2 * np.pi * x_case * (n / P)), x_case)
			if -1e-10 < b_n < 1e-10:
				b_n = 0.0
		
			#	add to output array
			case_coefs[i + 1] = a_n
		
			#	check for zero b coefficent
			if b_n != 0.0:
				print('Non-Zero b coefficient!')
				sys.exit()
		
		##################################################
		#	generate fourier series
		approx_case_signal = a_0 / 2
		
		for i, n in enumerate(nvals):
			approx_case_signal += case_coefs[i + 1] * np.cos(2 * np.pi * n * x_case / P)\
		
		#	calculate rms and norm
		current_rms = rms_calc(y_case, approx_case_signal)
		case_norm = np.linalg.norm(case_coefs)
		
		current_rms = rms_calc(lc_app[:-10], approx_case_signal[len(lc_seconds) - 1:-10])
		print('\n			RMS to Actual:		', current_rms)
		
		distance = np.linalg.norm(np.subtract(base_coefs, case_coefs))
		print('\n			Distance to Baseline:	', distance)
		print('			RMS to Baseline:	', rms_diff)



#		fit_ax.plot(lc_seconds, lc_app)
#		fit_ax.plot(lc_seconds, approx_case_signal[len(lc_seconds) - 1:])
#		
#		
#		fit_ax.cla()
		
		
		
#		ax_case.plot(
#			lc_seconds[:-10], lc_app[:-10],
#			linestyle='solid',
#			linewidth=2.0,
#			color='k',
#			label=r'$\mathrm{Case\ Lightcurve}$'
#		)
#		
#		ax_case.plot(
#			lc_seconds[:-10], approx_case_signal[len(lc_seconds) - 1:-10],
#			linestyle='dashed',
#			linewidth=1.0,
#			color='tab:red',
#			label=r'$\mathrm{Hilbert\ Approximation}$'
#		)
#		
#		ax_case.set_xlabel(r'$\mathrm{Time\ /\ s}$')
#		ax_case.set_ylabel(r'$\mathrm{Apparent\ Magnitude}$')
##		ax_case.set_ylim(-1.2, -2.0)
#		ax_case.legend(loc='upper left', frameon=False)
#		
#		
#		plt.pause(0.1)
#		ans = input('\nSave?...')











		#	calculate loop time
		loop_end = time.time()
	
#		diff_array.append(rms_diff)
		diff_array.append(distance)
		

		print('\n			Geometry Loop Time: {}'.format(np.round(loop_end - loop_start, 1)))
		print('\n			RMS: {}'.format(rms_diff))

		if show_lc_flag == True:
			temp_label = 'geom_str = ' + geom_str
			temp_axes.plot(lc_seconds, lc_app, label=temp_label)

	if show_lc_flag == True:
		temp_axes.legend()
		plt.pause(1)
		input('Enter to continue...')

	#	convert to numpy array
	diff_array = np.array(diff_array)

	#	append to results output
	results_array.append(diff_array)

print('\n	Full Loop Time: {}'.format(np.round(time.time() - full_start, 1)))

###################################################################################################
#	plot rms data

# define type of function to search
def model_func(x, a, k, b):
	return a * np.exp(-k*x) + b

results_array = np.array(results_array)
print('Results array shape: ', results_array.shape)

marker_list = [".", "^", "1", "s", "p", "P", "*", "H", "X", "d", "+", "x"]

x = nfaces_plot

fig_lc, ax_lc = plt.subplots(dpi=300)

#ax_lc.set_title('Attit_mot: {}    BRDF: {}'.format(rot_vec_body_unit, [d_brdf, omega_brdf, m_brdf]))

# for (results_line, omega_brdf) in zip(results_array, omega_range):

for index in range(len(results_array)):

	marker = marker_list[index]

	results_line = results_array[index]
	angle_multi = angle_multiplier[index]
	print('\nPlotting Angle Multiplier = ', angle_multi)

	y = results_line
	print('\nResults Line = ', results_line)


	#	generate best fitting line
	x2 = np.linspace(x[0], x[-1], 100)
	
	#	plot data points
	ax_lc.scatter(x, y, marker=marker, label=str(np.round(angle_multi, 1)) + '\ rpm')
	
	ax_lc.set_ylabel('Hilbert Distance')
	ax_lc.set_xlabel('Nfaces')
	ax_lc.legend(frameon=False)

	try:
		print('\n	Generating fit:')
		# curve fit
		a = 0.1
		k = 1.e-5
		b = y[-1]
		p0 = (a, k, b) # starting search koefs

		#	perform curve fitting
		opt, pcov = curve_fit(model_func, x, y, p0, maxfev = 2000)
		a, k, b = opt
		
		y2 = model_func(x2, a, k, b)
		ax_lc.plot(x2, y2, zorder=1)

	except:
		print('\nFailed to produce fit curve')

#	plt.pause(0.1)
#	input('\nEnter to Cont..')

####################################################
#	save figure
ans = input('\nSave?')
if ans == 'y':

	fig_name = 'sensitivity_attit_mot_z_diff_apx.eps'
	out_str = os.path.join(fig_path, fig_name)

	print('\nAttempting to save to: ', out_str)

	#	check for file existence
	if os.path.exists(out_str):
		ans = input('\nFile already exists, overwrite? [y/n]')

		if ans == 'y':
			write = True
			print('\nOverwritng...')
		else:
			print('\nKeeping existing file...')
			write = False

	else:
			write = True

	if write is True:
		print('\nSaving: ', out_str)
		fig_lc.savefig(out_str, bbox_inches="tight", dpi=300)
		print('\n#######################################')
		print('Figure saved: ', fig_name)


plt.show()
sys.exit()
