###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat, rot3

#	matplotlib
import matplotlib.pyplot as plt

from scipy.optimize import curve_fit

##################################################################################################
#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Light curve model sensitivity analysis')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

lightcurve_path = 'lightcurve_dir/sensitivity_study/lightcurves'

###################################################################################################
#	light curve generation inputs
###################################################################################################

#	distance to object (km)
obs_dist = 1000

###################################################################################################
#	time data

#	time step
time_step = 0.1

#	total duration
duration_sec = 180

#	number of data points
len_record = int(np.round(duration_sec / time_step, 1)) + 1

start_date = [2020, 1, 1, 0, 0, 0]
start_jd = date2jd(*start_date)

stop_jd = np.add(*start_jd) + (duration_sec / (24 * 60 * 60))
stop_date = jd2date(*jday_jfrac(stop_jd))

###################################################################################################
#	generate fake inputs to gen_vectors

#	orbital motion
orbital_mot = 0.01

#	position arrays
sat_orb = np.zeros([len_record, 7])
sat_orb[:, 1] = 1.0
sat_orb[:, 4] = 1.0

sun_orb = np.zeros([len_record, 7])
sun_orb[:, 1] = 1.0
sun_orb[:, 4] = 1.0

earth_orb = np.zeros([len_record, 7])
earth_orb[:, 1] = 1.0
earth_orb[:, 4] = 1.0

site_orb = np.zeros([len_record, 7])
site_orb[:, 1] = obs_dist
site_orb[:, 4] = 1.0

#	attitude state
init_attit = np.array([0.0, 0.0, 0.0])

rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
rot_ang_per_sec = 0.0
rot_vec_vel_orb = np.array([1.0, 0.0, 0.0])

#	observation and illumination vector directions
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

# #	apply movement to illumination vector
# angle_range = np.linspace(0.0, 89, len(sun_orb))
# for i, (line, angle) in enumerate(zip(sun_orb, angle_range)):
# 	sun_orb[i, 1:4] = rot3(line[1:4], np.radians(-angle))

###################################################################################################
#	BRDF inputs

#	set initial BRDF values
d_brdf = 1.0
omega_brdf = 0.5
m_brdf = 0.2

attit_range = np.linspace(0.0, np.pi, 10)

###################################################################################################
#	geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir/sphere'

baseline_geom_str = '05_div5_sphere'
baseline_geom_file = baseline_geom_str + '.stl'

#	load static mesh
baseline_body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, baseline_geom_file))

#	set vectors and areas
baseline_body_vectors = baseline_body_mesh.vectors
baseline_body_normals = baseline_body_mesh.normals
baseline_body_areas = baseline_body_mesh.areas

baseline_nfaces = len(baseline_body_areas)

###################################################################################################
#	generate brdf

#	Refractive Index
n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))

#	Specular ratio
s_brdf = 1.0 - d_brdf

#	generate brdf
baseline_brdf_data = np.zeros([baseline_nfaces, 5])

for i in range(baseline_nfaces):
	baseline_brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]

###################################################################################################
#	results generation inputs
###################################################################################################

geom_str_list = [
	'01_div1_sphere',
	'02_div2_sphere',
	'03_div3_sphere',
	'04_div4_sphere',
	'05_div5_sphere'
]

###################################################################################################
#	perform light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	extract orbital positions of custom input vectors
illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_body[:, 31:34]

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])

illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]

#	orb data
sun_orb_data = vector_data_orb[:, 28:31]
site_orb_data = vector_data_orb[:, 31:34]
earth_orb_data = vector_data_orb[:, 19:22]
sat_orb_vel = vector_data_orb[:, 22:25]
sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])

illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_orb[:, 31:34]

###################################################################################################
#	generate baseline lightcurve
###################################################################################################

print('\n####################################################################################')
print('Generating Baseline Light Curve...')

gen_start = time.time()
###################################################################################################
#	initialise light curve generation

#	generate sim time
sim_time = np.linspace(0, duration_sec, len_record)

#	initilise lightcurve output
light_curve_array = np.zeros((len_record, 4))
light_curve_array[:, 1] = sim_time

jdate_start = start_jd[0] + start_jd[1]

##################################################################################################
#	generate payload no shadowing light curve

lc_filename = '{}_{}_{}_{}_{}_{}_{}_stationary'.format(
	baseline_geom_str,
	baseline_nfaces,
	omega_brdf,
	m_brdf,
	d_brdf,
	init_attit,
	rot_vec_body_unit,
)

if os.path.exists(os.path.join(path_data, lightcurve_path, lc_filename)):
	print('\n	Loading...')
	light_curve_array = np.loadtxt(os.path.join(path_data, lightcurve_path, lc_filename))

else:
	print('\n	Generating...')
	#	run light curve generation function
	light_curve_array = glim_slc_new(
		illum_vec_body_data, obs_vec_body_data, light_curve_array,
		jdate_start, baseline_body_vectors, baseline_body_areas, baseline_brdf_data
	)
	np.savetxt(os.path.join(path_data, lightcurve_path, lc_filename), light_curve_array)

baseline_lc_seconds = light_curve_array[:, 1]
baseline_lc_app = light_curve_array[:, 2]
baseline_lc_std = light_curve_array[:, 3]

gen_end = time.time()
print('\n	Baseline generation time: {}'.format(np.round(gen_end - gen_start, 1)))

##################################################################################################
#	generate number of faces array

nfaces_plot = []

for geom_str in geom_str_list:

	geom_file = geom_str + '.stl'
	
	#	load static mesh
	body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
	
	#	set vectors and areas
	body_vectors = body_mesh.vectors
	body_normals = body_mesh.normals
	body_areas = body_mesh.areas
	
	nfaces = len(body_areas)

	nfaces_plot.append(nfaces)

#	convert to numpy array
nfaces_plot = np.array(nfaces_plot)

###################################################################################################
#	results generation loop
###################################################################################################

results_array = []

print('\n####################################################################################')
print('Looping through Attitude values...')

for attit_val in attit_range:

	print('\n	############################################################################')
	print('	Looping through geometry files...')
	print('\n	ATTIT_VAL = ', attit_val)

	#	attitude state
	init_attit = np.array([attit_val, 0.0, 0.0])

	diff_array = []
	
	###############################################################################################
	#	perform light curve vector generation
	
	#	vector generation function
	vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb
	)
	
	##################################################
	#	parse vector data
	
	#	extract orbital positions of custom input vectors
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_body[:, 31:34]
	
	#	body data
	sun_body_data = vector_data_body[:, 28:31]
	site_body_data = vector_data_body[:, 31:34]
	earth_body_data = vector_data_body[:, 19:22]
	sat_body_vel = vector_data_body[:, 22:25]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	
	illum_vec_body_data = vector_data_body[:, 28:31]
	obs_vec_body_data = vector_data_body[:, 31:34]
	
	#	orb data
	sun_orb_data = vector_data_orb[:, 28:31]
	site_orb_data = vector_data_orb[:, 31:34]
	earth_orb_data = vector_data_orb[:, 19:22]
	sat_orb_vel = vector_data_orb[:, 22:25]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_orb[:, 31:34]

	for geom_str in geom_str_list:

		print('\n		########################################################################')
		print('		Working on geometry file: ', geom_str)
		
		#	set loop start time
		loop_start = time.time()
	
		###############################################################################################
		#	reinitialise geometry inputs
		###############################################################################################
		
		geom_file = geom_str + '.stl'
		
		#	load static mesh
		body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
		
		#	set vectors and areas
		body_vectors = body_mesh.vectors
		body_normals = body_mesh.normals
		body_areas = body_mesh.areas
		
		nfaces = len(body_areas)
			
		#	regenerate filename
		lc_filename = '{}_{}_{}_{}_{}_{}_{}_stationary'.format(
			geom_str,
			np.round(nfaces, 2),
			np.round(omega_brdf, 2),
			np.round(m_brdf, 2),
			np.round(d_brdf, 2),
			np.round(init_attit, 2),
			np.round(rot_vec_body_unit, 2)
		)

		print('\n		Lightcurve file is: ', lc_filename)

		if os.path.exists(os.path.join(path_data, lightcurve_path, lc_filename)):
			print('\n		Loading...')
			light_curve_array = np.loadtxt(os.path.join(path_data, lightcurve_path, lc_filename))
		
		else:
			print('\n		Generating...')
		
			###############################################################################################
			#	regenerate brdf
			brdf_data = np.zeros([nfaces, 5])
			
			for i in range(nfaces):
				brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
		
			###############################################################################################
			#	generate light curve
			###############################################################################################
			
			print('\n			Running light curve generation...')

			###############################################################################################
			#	initialise light curve generation
			
			#	generate sim time
			sim_time = np.linspace(0, duration_sec, len_record)
			
			#	initilise lightcurve output
			light_curve_array = np.zeros((len_record, 4))
			light_curve_array[:, 1] = sim_time
			
			jdate_start = start_jd[0] + start_jd[1]
			
			###############################################################################################
			#	generate payload no shadowing light curve
			
			#	run light curve generation function
			light_curve_array_pl_noshad = glim_slc_new(
				illum_vec_body_data, obs_vec_body_data, light_curve_array,
				jdate_start, body_vectors, body_areas, brdf_data
			)
		
			#	save
			np.savetxt(os.path.join(path_data, lightcurve_path, lc_filename), light_curve_array)
	
		#	extract light curve data
		lc_seconds = light_curve_array[:, 1]
		lc_app = light_curve_array[:, 2]
		lc_std = light_curve_array[:, 3]
	
		#	calculate rms difference from baseline
		rms_diff = rms_calc(baseline_lc_app, lc_app)
	
		#	calculate loop time
		loop_end = time.time()
	
		diff_array.append(rms_diff)

		print('\n			Generation Loop Time: {}'.format(np.round(loop_end - loop_start, 1)))

	#	convert to numpy array
	diff_array = np.array(diff_array)

	#	append to results output
	results_array.append(diff_array)
	
print('\n	[{}]	Finished Generation Loop...'.format(datetime.datetime.now()))

###################################################################################################
#	plot rms data

# define type of function to search
def model_func(x, a, k, b):
	return a * np.exp(-k*x) + b

results_array = np.array(results_array)
print('Results array shape: ', results_array.shape)

x = nfaces_plot

fig_lc, ax_lc = plt.subplots()

# for (results_line, omega_brdf) in zip(results_array, omega_range):

for index in range(len(results_array)):


	results_line = results_array[index]
	attit_val = attit_range[index]

	print('\nPlotting Attit_Val = ', attit_val)

	y = results_line


	#	generate best fitting line
	x2 = np.linspace(x[0], x[-1], 100)
	
	#	plot data points
	ax_lc.scatter(x, y, label='attit_val = ' + str(attit_val))
	
	ax_lc.set_ylabel('RMS Error')
	ax_lc.set_xlabel('Nfaces')
	ax_lc.legend()

	# curve fit
	a = 0.1
	k = 1.e-5
	b = y[-1]
	p0 = (a, k, b) # starting search koefs
	print('\n	Initial Curve Fitting Parameters: ')
	print('		a = ', p0[0])
	print('		k = ', p0[1])
	print('		b = ', p0[2])

	#	perform curve fitting
	opt, pcov = curve_fit(model_func, x, y, p0)
	
	a, k, b = opt
	print('\n	Final Fitting Parameters: ')
	print('		a = ', a)
	print('		k = ', k)
	print('		b = ', b)
	
	y2 = model_func(x2, a, k, b)
	ax_lc.plot(x2, y2)

plt.show()
sys.exit()
