###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat, rot3

#	matplotlib
import matplotlib.pyplot as plt

##################################################################################################
#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Light curve model sensitivity analysis')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

###################################################################################################
#	light curve generation inputs
###################################################################################################

###################################################################################################
#	time data

#	time step
time_step = 0.1

#	total duration
duration_sec = 300

#	number of data points
len_record = int(np.round(duration_sec / time_step, 1)) + 1

start_date = [2020, 1, 1, 0, 0, 0]
start_jd = date2jd(*start_date)

stop_jd = np.add(*start_jd) + (duration_sec / (24 * 60 * 60))
stop_date = jd2date(*jday_jfrac(stop_jd))

###################################################################################################
#	generate fake inputs to gen_vectors

#	orbital motion
orbital_mot = 0.000001

#	position arrays
sat_orb = np.zeros([len_record, 7])
sat_orb[:, 1] = 1.0
sat_orb[:, 4] = 1.0

sun_orb = np.zeros([len_record, 7])
sun_orb[:, 1] = 1.0
sun_orb[:, 4] = 1.0

angle_range = np.linspace(35.0, 89, len(sun_orb))

for i, (line, angle) in enumerate(zip(sun_orb, angle_range)):
	sun_orb[i, 1:4] = rot3(line[1:4], np.radians(-angle))

earth_orb = np.zeros([len_record, 7])
earth_orb[:, 1] = 1.0
earth_orb[:, 4] = 1.0

site_orb = np.zeros([len_record, 7])
site_orb[:, 1] = 2000.0
site_orb[:, 1] = 500.0
site_orb[:, 4] = 1.0

#	attitude state
init_attit = np.array([0.0, 0.0, 0.0])

rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
rot_ang_per_sec = 0.0
rot_vec_vel_orb = np.array([1.0, 0.0, 0.0])

#	observation and illumination vector directions
obs_vec_orb = np.array([2000.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

###################################################################################################
#	geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir'

geom_str = '0.02m_cube'
geom_file = geom_str + '.stl'

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

###################################################################################################
#	BRDF inputs

#	set initial BRDF values
d_brdf = 0.0
omega_brdf = 0.5
m_brdf = 0.2

###################################################################################################
#	generate brdf

#	Refractive Index
n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))

#	Specular ratio
s_brdf = 1.0 - d_brdf

#	generate brdf
brdf_data = np.zeros([nfaces, 5])

for i in range(nfaces):
	brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]

###################################################################################################
#	perform light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	extract orbital positions of custom input vectors
illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_body[:, 31:34]

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])

illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]

#	orb data
sun_orb_data = vector_data_orb[:, 28:31]
site_orb_data = vector_data_orb[:, 31:34]
earth_orb_data = vector_data_orb[:, 19:22]
sat_orb_vel = vector_data_orb[:, 22:25]
sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])

illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_orb[:, 31:34]

###################################################################################################
#	generate baseline lightcurve
###################################################################################################

print('\n[{}]	Generating Light Curves...'.format(datetime.datetime.now()))

###############################################################################################
#	initialise light curve generation

#	generate sim time
sim_time = np.linspace(0, duration_sec, len_record)

#	initilise lightcurve output
light_curve_array = np.zeros((len_record, 4))
light_curve_array[:, 1] = sim_time

jdate_start = start_jd[0] + start_jd[1]

###############################################################################################
#	generate payload no shadowing light curve

#	run light curve generation function
light_curve_array = glim_slc_new(
	sun_body_data, site_body_data, light_curve_array,
	jdate_start, body_vectors, body_areas, brdf_data
)

baseline_lc_seconds = light_curve_array[:, 1]
baseline_lc_app = light_curve_array[:, 2]
baseline_lc_std = light_curve_array[:, 3]

###################################################################################################
#	plot light curves

fig_lc, ax_lc = plt.subplots(dpi=300, figsize=(8, 3))
ax_lc.plot(baseline_lc_seconds, baseline_lc_app, label='Synthetic')
plt.gca().invert_yaxis()

ax_lc.set_ylim([int(max(baseline_lc_app) + 1), int(min(baseline_lc_app) - 1)])


ax_lc.set_ylabel('apparent mag / mag')
ax_lc.set_xlabel('time / s')


distance = []
phase = []

for (line1, line2) in zip(sun_body_data, site_body_data):

	distance.append(np.linalg.norm(line2))

	phase.append(np.arccos(
		np.divide(
			np.dot(line1, line2),
			np.linalg.norm(line1) * np.linalg.norm(line2)
		)
	))


distance = np.array(distance)
phase = np.array(phase)

fig_dist, ax_dist = plt.subplots()
ax_dist.plot(baseline_lc_seconds, distance, label='Synthetic')
ax_dist.set_xlabel('time / s')
ax_dist.set_ylabel('distance / km')
ax_dist.set_xlabel('distance')

fig_phase, ax_phase = plt.subplots()
ax_phase.plot(baseline_lc_seconds, np.degrees(phase), label='Synthetic')
ax_phase.set_xlabel('time / s')
ax_phase.set_ylabel('phase / deg')
ax_phase.set_xlabel('phase')




plt.show()
sys.exit()
