#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('IOC Plotter.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

time_step = 0.1

#	output file path
paper_path = os.path.join(
	os.environ['HOME'],
	'PhD', 'Documents', 'Conferences', 'IOC_2019', 'technical_paper', 'acta_figures', 'Final'
)

poster_path = os.path.join(
	os.environ['HOME'],
	'PhD', 'Documents', 'Conferences', 'IOC_2019', 'Poster'
)

fig_name = 'real_lc_comp' + '.eps'
out_str = os.path.join(poster_path, fig_name)

###################################################################################################
#	load MMT file
###################################################################################################

mmt_file = 'cz-2d_41858_11-02-17_03-02-59.txt'

name_str, norad_id, date_str, time_str = mmt_file.split('_')
norad_id = int(norad_id)

title_str1 = '\ '.join(['a)', 'CZ-2D', str(norad_id)])

read_path = os.path.join(path_data, 'input_dir', 'mmt_files')

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file
###################################################################################################

#	header line
header_text = mmt_data[7]

#	strip header
mmt_data = mmt_data[8:]

###################################################################################################
#	extract start jd

mmt_line = mmt_data[0]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
start_jd = jd

###################################################################################################
#	extract stop jd

mmt_line = mmt_data[-1]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
stop_jd = jd

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(lc_duration_sec / time_step) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	extract MMT data

sim_index = 0
mmt_data_parsed = []

#	parse MMT data
for itt, mmt_line in enumerate(mmt_data):

	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)

	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')

	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)

	#	generate time array
	mmt_time = np.array([hour, minute, second])

	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])

	#	calculate julian date
	jd = date2jd(*datetime_array)
	step_jd = jd[0] + jd[1]

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	#	set light curve data datatypes
	std_mag = np.float(std_mag)
	app_mag = np.float(app_mag)
	distance = np.float(distance)
	phase = np.float(phase)

	#	generate light curve data array
	mmt_lc_data = [std_mag, app_mag, distance, phase]
		
	#	assemble line data
	line_data = np.hstack([jd, current_sec, mmt_lc_data])

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		mmt_data_parsed.append(line_data)
		sim_index += 1

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	set previous data values
		jd_prev = prev_line[0]
		jdfrac_prev = prev_line[1]
		prev_sec = prev_line[2]

		#	set previous light curve data
		std_mag_prev = prev_line[-4]
		app_mag_prev = prev_line[-3]
		distance_prev = prev_line[-2]
		phase_prev = prev_line[-1]

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		std_mag_last = std_mag
		app_mag_last = app_mag
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		std_mag_step = (std_mag_last - std_mag_prev) / npts
		app_mag_step = (app_mag_last - app_mag_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	fill first line
		interp_data = np.zeros([npts, len(line_data)])

		#	calculate interpolation data
		for n in range(1, len(interp_data) + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jd_prev + jdfrac_prev) + (n * (0.1 / 86400))
			jd = jday_jfrac(jdate)

			#	calculate interpolated light curve data
			std_mag = std_mag_prev + (n * std_mag_step)
			app_mag = app_mag_prev + (n * app_mag_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			mmt_lc_data = [std_mag, app_mag, distance, phase]
					
			#	assemble line data
			line_data = np.hstack([jd, interp_sec, mmt_lc_data])

			#	add to output
			interp_data[n - 1, :] = line_data

		#	find required lines in interpolated data
		for interp_line in interp_data:

			#	set current simulation second
			interp_line_sec = interp_line[2]

			if interp_line_sec == sim_range_sec[sim_index]:
				mmt_data_parsed.append(line_data)
				sim_index += 1

	#	save previous line
	if itt != 0:
		prev_line = line_data
		prev_time = mmt_time

#	ensure parsed MMT data matches simulation data
mmt_data_parsed = np.array(mmt_data_parsed)

if len(mmt_data_parsed) == len(lc_sim_range):
	print('\nSimulation and MMT data matches...')
else:
	print('\nError: Simulation and MMT data does not match...')
	sys.exit()

mmt_signal1_time = mmt_data_parsed[:, 2]
mmt_signal1_std = mmt_data_parsed[:, 3]
mmt_signal1_app = mmt_data_parsed[:, 4]

###################################################################################################
#	load MMT file 2
###################################################################################################

mmt_file = 'cz-2d_44548_29-10-2019_15-28-36.txt'

name_str, norad_id, date_str, time_str = mmt_file.split('_')
norad_id = int(norad_id)

title_str2 = '\ '.join(['b)', 'CZ-2D', str(norad_id)])

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'for_figure')

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file
###################################################################################################

#	header line
header_text = mmt_data[7]

#	strip header
mmt_data = mmt_data[8:]

###################################################################################################
#	extract start jd

mmt_line = mmt_data[0]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
start_jd = jd

###################################################################################################
#	extract stop jd

mmt_line = mmt_data[-1]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
stop_jd = jd

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(lc_duration_sec / time_step) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	extract MMT data

sim_index = 0
mmt_data_parsed = []

#	parse MMT data
for itt, mmt_line in enumerate(mmt_data):

	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)

	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')

	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)

	#	generate time array
	mmt_time = np.array([hour, minute, second])

	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])

	#	calculate julian date
	jd = date2jd(*datetime_array)
	step_jd = jd[0] + jd[1]

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	#	set light curve data datatypes
	std_mag = np.float(std_mag)
	app_mag = np.float(app_mag)
	distance = np.float(distance)
	phase = np.float(phase)

	#	generate light curve data array
	mmt_lc_data = [std_mag, app_mag, distance, phase]
		
	#	assemble line data
	line_data = np.hstack([jd, current_sec, mmt_lc_data])

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		mmt_data_parsed.append(line_data)
		sim_index += 1

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	set previous data values
		jd_prev = prev_line[0]
		jdfrac_prev = prev_line[1]
		prev_sec = prev_line[2]

		#	set previous light curve data
		std_mag_prev = prev_line[-4]
		app_mag_prev = prev_line[-3]
		distance_prev = prev_line[-2]
		phase_prev = prev_line[-1]

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		std_mag_last = std_mag
		app_mag_last = app_mag
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		std_mag_step = (std_mag_last - std_mag_prev) / npts
		app_mag_step = (app_mag_last - app_mag_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	fill first line
		interp_data = np.zeros([npts, len(line_data)])

		#	calculate interpolation data
		for n in range(1, len(interp_data) + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jd_prev + jdfrac_prev) + (n * (0.1 / 86400))
			jd = jday_jfrac(jdate)

			#	calculate interpolated light curve data
			std_mag = std_mag_prev + (n * std_mag_step)
			app_mag = app_mag_prev + (n * app_mag_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			mmt_lc_data = [std_mag, app_mag, distance, phase]
					
			#	assemble line data
			line_data = np.hstack([jd, interp_sec, mmt_lc_data])

			#	add to output
			interp_data[n - 1, :] = line_data

		#	find required lines in interpolated data
		for interp_line in interp_data:

			#	set current simulation second
			interp_line_sec = interp_line[2]

			if interp_line_sec == sim_range_sec[sim_index]:
				mmt_data_parsed.append(line_data)
				sim_index += 1

	#	save previous line
	if itt != 0:
		prev_line = line_data
		prev_time = mmt_time

#	ensure parsed MMT data matches simulation data
mmt_data_parsed = np.array(mmt_data_parsed)

if len(mmt_data_parsed) == len(lc_sim_range):
	print('\nSimulation and MMT data matches...')
else:
	print('\nError: Simulation and MMT data does not match...')
	sys.exit()

mmt_signal2_time = mmt_data_parsed[:, 2]
mmt_signal2_std = mmt_data_parsed[:, 3]
mmt_signal2_app = mmt_data_parsed[:, 4]

###################################################################################################
#	load MMT file 3
###################################################################################################

mmt_file = 'DELTA-2_28476_18-01-2018_03-09-42.txt'

name_str, norad_id, date_str, time_str = mmt_file.split('_')
norad_id = int(norad_id)

title_str3 = '\ '.join(['c)', name_str, str(norad_id)])

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'for_figure')

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file
###################################################################################################

#	header line
header_text = mmt_data[7]

#	strip header
mmt_data = mmt_data[8:]

###################################################################################################
#	extract start jd

mmt_line = mmt_data[0]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
start_jd = jd

###################################################################################################
#	extract stop jd

mmt_line = mmt_data[-1]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
stop_jd = jd

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(lc_duration_sec / time_step) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	extract MMT data

sim_index = 0
mmt_data_parsed = []

#	parse MMT data
for itt, mmt_line in enumerate(mmt_data):

	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)

	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')

	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)

	#	generate time array
	mmt_time = np.array([hour, minute, second])

	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])

	#	calculate julian date
	jd = date2jd(*datetime_array)
	step_jd = jd[0] + jd[1]

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	#	set light curve data datatypes
	std_mag = np.float(std_mag)
	app_mag = np.float(app_mag)
	distance = np.float(distance)
	phase = np.float(phase)

	#	generate light curve data array
	mmt_lc_data = [std_mag, app_mag, distance, phase]
		
	#	assemble line data
	line_data = np.hstack([jd, current_sec, mmt_lc_data])

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		mmt_data_parsed.append(line_data)
		sim_index += 1

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	set previous data values
		jd_prev = prev_line[0]
		jdfrac_prev = prev_line[1]
		prev_sec = prev_line[2]

		#	set previous light curve data
		std_mag_prev = prev_line[-4]
		app_mag_prev = prev_line[-3]
		distance_prev = prev_line[-2]
		phase_prev = prev_line[-1]

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		std_mag_last = std_mag
		app_mag_last = app_mag
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		std_mag_step = (std_mag_last - std_mag_prev) / npts
		app_mag_step = (app_mag_last - app_mag_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	fill first line
		interp_data = np.zeros([npts, len(line_data)])

		#	calculate interpolation data
		for n in range(1, len(interp_data) + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jd_prev + jdfrac_prev) + (n * (0.1 / 86400))
			jd = jday_jfrac(jdate)

			#	calculate interpolated light curve data
			std_mag = std_mag_prev + (n * std_mag_step)
			app_mag = app_mag_prev + (n * app_mag_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			mmt_lc_data = [std_mag, app_mag, distance, phase]
					
			#	assemble line data
			line_data = np.hstack([jd, interp_sec, mmt_lc_data])

			#	add to output
			interp_data[n - 1, :] = line_data

		#	find required lines in interpolated data
		for interp_line in interp_data:

			#	set current simulation second
			interp_line_sec = interp_line[2]

			if interp_line_sec == sim_range_sec[sim_index]:
				mmt_data_parsed.append(line_data)
				sim_index += 1

	#	save previous line
	if itt != 0:
		prev_line = line_data
		prev_time = mmt_time

#	ensure parsed MMT data matches simulation data
mmt_data_parsed = np.array(mmt_data_parsed)

if len(mmt_data_parsed) == len(lc_sim_range):
	print('\nSimulation and MMT data matches...')
else:
	print('\nError: Simulation and MMT data does not match...')
	sys.exit()

mmt_signal3_time = mmt_data_parsed[:, 2]
mmt_signal3_std = mmt_data_parsed[:, 3]
mmt_signal3_app = mmt_data_parsed[:, 4]

###################################################################################################
#	load MMT file 4
###################################################################################################

mmt_file = 'ELECTRON_44372_01-09-2019_00-54-05.txt'

name_str, norad_id, date_str, time_str = mmt_file.split('_')
norad_id = int(norad_id)

title_str4 = '\ '.join(['d)', name_str, str(norad_id)])

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'for_figure')

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file
###################################################################################################

#	header line
header_text = mmt_data[7]

#	strip header
mmt_data = mmt_data[8:]

###################################################################################################
#	extract start jd

mmt_line = mmt_data[0]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
start_jd = jd

###################################################################################################
#	extract stop jd

mmt_line = mmt_data[-1]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
stop_jd = jd

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(lc_duration_sec / time_step) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	extract MMT data

sim_index = 0
mmt_data_parsed = []

#	parse MMT data
for itt, mmt_line in enumerate(mmt_data):

	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)

	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')

	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)

	#	generate time array
	mmt_time = np.array([hour, minute, second])

	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])

	#	calculate julian date
	jd = date2jd(*datetime_array)
	step_jd = jd[0] + jd[1]

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	#	set light curve data datatypes
	std_mag = np.float(std_mag)
	app_mag = np.float(app_mag)
	distance = np.float(distance)
	phase = np.float(phase)

	#	generate light curve data array
	mmt_lc_data = [std_mag, app_mag, distance, phase]
		
	#	assemble line data
	line_data = np.hstack([jd, current_sec, mmt_lc_data])

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		mmt_data_parsed.append(line_data)
		sim_index += 1

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	set previous data values
		jd_prev = prev_line[0]
		jdfrac_prev = prev_line[1]
		prev_sec = prev_line[2]

		#	set previous light curve data
		std_mag_prev = prev_line[-4]
		app_mag_prev = prev_line[-3]
		distance_prev = prev_line[-2]
		phase_prev = prev_line[-1]

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		std_mag_last = std_mag
		app_mag_last = app_mag
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		std_mag_step = (std_mag_last - std_mag_prev) / npts
		app_mag_step = (app_mag_last - app_mag_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	fill first line
		interp_data = np.zeros([npts, len(line_data)])

		#	calculate interpolation data
		for n in range(1, len(interp_data) + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jd_prev + jdfrac_prev) + (n * (0.1 / 86400))
			jd = jday_jfrac(jdate)

			#	calculate interpolated light curve data
			std_mag = std_mag_prev + (n * std_mag_step)
			app_mag = app_mag_prev + (n * app_mag_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			mmt_lc_data = [std_mag, app_mag, distance, phase]
					
			#	assemble line data
			line_data = np.hstack([jd, interp_sec, mmt_lc_data])

			#	add to output
			interp_data[n - 1, :] = line_data

		#	find required lines in interpolated data
		for interp_line in interp_data:

			#	set current simulation second
			interp_line_sec = interp_line[2]

			if interp_line_sec == sim_range_sec[sim_index]:
				mmt_data_parsed.append(line_data)
				sim_index += 1

	#	save previous line
	if itt != 0:
		prev_line = line_data
		prev_time = mmt_time

#	ensure parsed MMT data matches simulation data
mmt_data_parsed = np.array(mmt_data_parsed)

if len(mmt_data_parsed) == len(lc_sim_range):
	print('\nSimulation and MMT data matches...')
else:
	print('\nError: Simulation and MMT data does not match...')
	sys.exit()

mmt_signal4_time = mmt_data_parsed[:, 2]
mmt_signal4_std = mmt_data_parsed[:, 3]
mmt_signal4_app = mmt_data_parsed[:, 4]

###################################################################################################
#	load MMT file 5
###################################################################################################

mmt_file = 'N-1_10675_04-07-2019_23-13-29.txt'

name_str, norad_id, date_str, time_str = mmt_file.split('_')
norad_id = int(norad_id)

title_str5 = '\ '.join(['e)', name_str, str(norad_id)])

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'for_figure')

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file
###################################################################################################

#	header line
header_text = mmt_data[7]

#	strip header
mmt_data = mmt_data[8:]

###################################################################################################
#	extract start jd

mmt_line = mmt_data[0]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
start_jd = jd

###################################################################################################
#	extract stop jd

mmt_line = mmt_data[-1]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
stop_jd = jd

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(lc_duration_sec / time_step) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	extract MMT data

sim_index = 0
mmt_data_parsed = []

#	parse MMT data
for itt, mmt_line in enumerate(mmt_data):

	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)

	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')

	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)

	#	generate time array
	mmt_time = np.array([hour, minute, second])

	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])

	#	calculate julian date
	jd = date2jd(*datetime_array)
	step_jd = jd[0] + jd[1]

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	#	set light curve data datatypes
	std_mag = np.float(std_mag)
	app_mag = np.float(app_mag)
	distance = np.float(distance)
	phase = np.float(phase)

	#	generate light curve data array
	mmt_lc_data = [std_mag, app_mag, distance, phase]
		
	#	assemble line data
	line_data = np.hstack([jd, current_sec, mmt_lc_data])

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		mmt_data_parsed.append(line_data)
		sim_index += 1

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	set previous data values
		jd_prev = prev_line[0]
		jdfrac_prev = prev_line[1]
		prev_sec = prev_line[2]

		#	set previous light curve data
		std_mag_prev = prev_line[-4]
		app_mag_prev = prev_line[-3]
		distance_prev = prev_line[-2]
		phase_prev = prev_line[-1]

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		std_mag_last = std_mag
		app_mag_last = app_mag
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		std_mag_step = (std_mag_last - std_mag_prev) / npts
		app_mag_step = (app_mag_last - app_mag_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	fill first line
		interp_data = np.zeros([npts, len(line_data)])

		#	calculate interpolation data
		for n in range(1, len(interp_data) + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jd_prev + jdfrac_prev) + (n * (0.1 / 86400))
			jd = jday_jfrac(jdate)

			#	calculate interpolated light curve data
			std_mag = std_mag_prev + (n * std_mag_step)
			app_mag = app_mag_prev + (n * app_mag_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			mmt_lc_data = [std_mag, app_mag, distance, phase]
					
			#	assemble line data
			line_data = np.hstack([jd, interp_sec, mmt_lc_data])

			#	add to output
			interp_data[n - 1, :] = line_data

		#	find required lines in interpolated data
		for interp_line in interp_data:

			#	set current simulation second
			interp_line_sec = interp_line[2]

			if interp_line_sec == sim_range_sec[sim_index]:
				mmt_data_parsed.append(line_data)
				sim_index += 1

	#	save previous line
	if itt != 0:
		prev_line = line_data
		prev_time = mmt_time

#	ensure parsed MMT data matches simulation data
mmt_data_parsed = np.array(mmt_data_parsed)

if len(mmt_data_parsed) == len(lc_sim_range):
	print('\nSimulation and MMT data matches...')
else:
	print('\nError: Simulation and MMT data does not match...')
	sys.exit()

mmt_signal5_time = mmt_data_parsed[:, 2]
mmt_signal5_std = mmt_data_parsed[:, 3]
mmt_signal5_app = mmt_data_parsed[:, 4]



###################################################################################################
#	load MMT file 6
###################################################################################################

mmt_file = 'EPSILON_43153_07-02-2018_03-29-49.txt'

name_str, norad_id, date_str, time_str = mmt_file.split('_')
norad_id = int(norad_id)

title_str6 = '\ '.join(['f)', name_str, str(norad_id)])

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'for_figure')

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file
###################################################################################################

#	header line
header_text = mmt_data[7]

#	strip header
mmt_data = mmt_data[8:]

###################################################################################################
#	extract start jd

mmt_line = mmt_data[0]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
start_jd = jd

###################################################################################################
#	extract stop jd

mmt_line = mmt_data[-1]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
stop_jd = jd

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(lc_duration_sec / time_step) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	extract MMT data

sim_index = 0
mmt_data_parsed = []

#	parse MMT data
for itt, mmt_line in enumerate(mmt_data):

	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)

	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')

	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)

	#	generate time array
	mmt_time = np.array([hour, minute, second])

	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])

	#	calculate julian date
	jd = date2jd(*datetime_array)
	step_jd = jd[0] + jd[1]

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	#	set light curve data datatypes
	std_mag = np.float(std_mag)
	app_mag = np.float(app_mag)
	distance = np.float(distance)
	phase = np.float(phase)

	#	generate light curve data array
	mmt_lc_data = [std_mag, app_mag, distance, phase]
		
	#	assemble line data
	line_data = np.hstack([jd, current_sec, mmt_lc_data])

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		mmt_data_parsed.append(line_data)
		sim_index += 1

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	set previous data values
		jd_prev = prev_line[0]
		jdfrac_prev = prev_line[1]
		prev_sec = prev_line[2]

		#	set previous light curve data
		std_mag_prev = prev_line[-4]
		app_mag_prev = prev_line[-3]
		distance_prev = prev_line[-2]
		phase_prev = prev_line[-1]

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		std_mag_last = std_mag
		app_mag_last = app_mag
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		std_mag_step = (std_mag_last - std_mag_prev) / npts
		app_mag_step = (app_mag_last - app_mag_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	fill first line
		interp_data = np.zeros([npts, len(line_data)])

		#	calculate interpolation data
		for n in range(1, len(interp_data) + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jd_prev + jdfrac_prev) + (n * (0.1 / 86400))
			jd = jday_jfrac(jdate)

			#	calculate interpolated light curve data
			std_mag = std_mag_prev + (n * std_mag_step)
			app_mag = app_mag_prev + (n * app_mag_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			mmt_lc_data = [std_mag, app_mag, distance, phase]
					
			#	assemble line data
			line_data = np.hstack([jd, interp_sec, mmt_lc_data])

			#	add to output
			interp_data[n - 1, :] = line_data

		#	find required lines in interpolated data
		for interp_line in interp_data:

			#	set current simulation second
			interp_line_sec = interp_line[2]

			if interp_line_sec == sim_range_sec[sim_index]:
				mmt_data_parsed.append(line_data)
				sim_index += 1

	#	save previous line
	if itt != 0:
		prev_line = line_data
		prev_time = mmt_time

#	ensure parsed MMT data matches simulation data
mmt_data_parsed = np.array(mmt_data_parsed)

if len(mmt_data_parsed) == len(lc_sim_range):
	print('\nSimulation and MMT data matches...')
else:
	print('\nError: Simulation and MMT data does not match...')
	sys.exit()

mmt_signal6_time = mmt_data_parsed[:, 2]
mmt_signal6_std = mmt_data_parsed[:, 3]
mmt_signal6_app = mmt_data_parsed[:, 4]



###################################################################################################
#	figure parameters
###################################################################################################

####################################################
#	set figure-wide parameters

dpi_val = 300
inch2cm = 2.54

font_sizex = 6
font_sizey = 6
font_pad = 0.5

label_sizex = 6
label_sizey = 6

marker_size = 4
marker_edge = 1

#	set plot index
max_index = int(9.2 * 20)

linewidth_val = 0.5

hspace_val = 0.6
wspace_val = 0.35

grid_width = 0.1
alt_line_width = 0.1

title_size = 8
title_pad = 2

figure = plt.figure(num=None, figsize=(18.0 / inch2cm, 12.0 / inch2cm),
 	dpi=dpi_val, facecolor='w', edgecolor='k'
)

plt.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
plt.rc('text', usetex=True)
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

font_size = 5

color_str = [0.0, 0.0, 0.0]
linewidth_val = 0.8

###################################################################################################
#	axis 1
ax1 = plt.subplot(321)
ax1.plot(mmt_signal1_time, mmt_signal1_app, linewidth=linewidth_val, color=color_str)

letter_col = [0.0, 0.5, 0.0]

#	finalise plot
#	reflection surface 1
ax1.text(0, 6.2, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax1.text(8, 6.2, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax1.text(18, 6.2, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax1.text(27, 6.2, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reflection surface 2
ax1.text(2, 2, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax1.text(5, 2, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax1.text(11, 2, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax1.text(15, 2, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reflection surface 3
ax1.text(4, 5.5, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax1.text(13, 5.5, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax1.text(22, 5.5, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax1.text(31, 5.5, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)



#	reverse y axis
ax1.invert_yaxis()

#	set grid
ax1.grid()

#	set title
ax1.set_title(r'$' + title_str1 + '$', pad=title_pad, fontsize=title_size)

ax1.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax1.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax1.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax1.tick_params(axis='y', labelsize=font_sizey, pad=0)

###################################################################################################
#	axis 2
ax2 = plt.subplot(322)
ax2.plot(mmt_signal2_time, mmt_signal2_app, linewidth=linewidth_val, color=color_str)

#	finalise plot

#	reflection surface 1
ax2.text(3, 5.0, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax2.text(16, 5.0, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax2.text(30, 5.0, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax2.text(43, 5.0, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reflection surface 2
ax2.text(7, 2.2, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax2.text(12, 2.2, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax2.text(21, 2.2, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax2.text(26, 2.2, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reflection surface 3
ax2.text(9, 4.6, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax2.text(23, 4.6, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax2.text(37, 4.6, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax2.text(51, 4.6, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reverse y axis
ax2.invert_yaxis()

#	set grid
ax2.grid()

#	set title
ax2.set_title(r'$' + title_str2 + '$', pad=title_pad, fontsize=title_size)

ax2.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax2.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax2.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax2.tick_params(axis='y', labelsize=font_sizey, pad=0)

###################################################################################################
#	axis 3
ax3 = plt.subplot(323)

time_plot = np.subtract(mmt_signal3_time[435:], mmt_signal3_time[435])

ax3.plot(time_plot, mmt_signal3_app[435:], linewidth=linewidth_val, color=color_str)

#	reflection surface 1
ax3.text(2, 8.7, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax3.text(9, 8.7, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax3.text(17, 8.7, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax3.text(25, 8.7, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reflection surface 2
ax3.text(4, 7.0, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax3.text(8, 7.0, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax3.text(11, 7.0, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax3.text(15, 7.0, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reflection surface 3
ax3.text(6, 8.2, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax3.text(13, 8.2, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax3.text(22, 8.2, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax3.text(29, 8.2, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reverse y axis
ax3.invert_yaxis()

#	set grid
ax3.grid()

#	set title
ax3.set_title(r'$' + title_str3 + '$', pad=title_pad, fontsize=title_size)

ax3.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax3.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax3.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax3.tick_params(axis='y', labelsize=font_sizey, pad=0)

###################################################################################################
#	axis 4
ax4 = plt.subplot(324)
ax4.plot(mmt_signal4_time, mmt_signal4_app, linewidth=linewidth_val, color=color_str)

#	finalise plot

#	reverse y axis
ax4.invert_yaxis()

#	set grid
ax4.grid()

#	set title
ax4.set_title(r'$' + title_str4 + '$', pad=title_pad, fontsize=title_size)

ax4.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax4.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax4.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax4.tick_params(axis='y', labelsize=font_sizey, pad=0)


###################################################################################################
#	axis 5
ax5 = plt.subplot(325)
ax5.plot(mmt_signal5_time, mmt_signal5_app, linewidth=linewidth_val, color=color_str)

#	finalise plot

#	finalise plot
#	reflection surface 1
ax5.text(2, 3.5, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax5.text(8, 3.5, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax5.text(14, 3.5, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax5.text(21, 3.5, r'$\bf{A}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reflection surface 2
ax5.text(3, 9.5, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax5.text(6, 9.5, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax5.text(10, 9.5, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax5.text(13, 9.5, r'$\bf{B}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reflection surface 3
ax5.text(5, 5.4, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax5.text(11, 5.4, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax5.text(18, 5.4, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)
ax5.text(24, 5.4, r'$\bf{C}$', fontsize=font_size, fontweight='bold', color=letter_col)

#	reverse y axis
ax5.invert_yaxis()

#	set grid
ax5.grid()

#	set title
ax5.set_title(r'$' + title_str5 + '$', pad=title_pad, fontsize=title_size)

ax5.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax5.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax5.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax5.tick_params(axis='y', labelsize=font_sizey, pad=0)


###################################################################################################
#	axis 6
ax6 = plt.subplot(326)
ax6.plot(mmt_signal6_time, mmt_signal6_app, linewidth=linewidth_val, color=color_str)

#	finalise plot

#	reverse y axis
ax6.invert_yaxis()

#	set grid
ax6.grid()

#	set title
ax6.set_title(r'$' + title_str6 + '$', pad=title_pad, fontsize=title_size)

ax6.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax6.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax6.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax6.tick_params(axis='y', labelsize=font_sizey, pad=0)

plt.show()

####################################################
#	saving figure

#	ask to save light curve file for testing
user_input = input(
	'\nSave Light Curve to:\n	{}? [y/n] '.format(out_str)
)

if user_input == 'y':
	
	print('\n	Saving Light Curve...')
	#	save light curve to file
	figure.savefig(out_str, bbox_inches="tight", dpi=300)

elif user_input == 'no':
	print('Not saved.')
sys.exit()
