#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('IOC Plotter.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

time_step = 0.1

#	output file path
paper_path = os.path.join(
	os.environ['HOME'],
	'PhD', 'Documents', 'Conferences', 'IOC_2019', 'technical_paper', 'acta_figures', 'Final'
)

poster_path = os.path.join(
	os.environ['HOME'],
	'PhD', 'Documents', 'Conferences', 'IOC_2019', 'Poster'
)

fig_name = 'lc_comp_single' + '.eps'
fig_name = 'lc_comp_full' + '.eps'

out_str = os.path.join(poster_path, fig_name)

###################################################################################################
#	load MMT file
###################################################################################################

mmt_file = 'cz-2d_41858_11-02-17_03-02-59.txt'

name_str, norad_id, date_str, time_str = mmt_file.split('_')
norad_id = int(norad_id)

read_path = os.path.join(path_data, 'input_dir', 'mmt_files')

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	load custom light curve data
###################################################################################################

print('\nLoading custom light curve data...')

output_path = os.path.join(path_data, 'paper_plots', 'best_fit')
best_fit_file = 'mmt_best_fit.txt'

#	load custom light curve
light_curve_custom = np.loadtxt(
	os.path.join(output_path, best_fit_file),
	delimiter=','
)

base_signal = light_curve_custom[:, 2]

###################################################################################################
#	parse MMT file
###################################################################################################

#	header line
header_text = mmt_data[7]

#	strip header
mmt_data = mmt_data[8:]

###################################################################################################
#	extract start jd

mmt_line = mmt_data[0]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
start_jd = jd

###################################################################################################
#	extract stop jd

mmt_line = mmt_data[-1]

#	parse line elements
date, time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
date = np.array(date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([date, time])

#	calculate julian date
jd = date2jd(*datetime_array)
stop_jd = jd

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(lc_duration_sec / time_step) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	extract MMT data

sim_index = 0
mmt_data_parsed = []

#	parse MMT data
for itt, mmt_line in enumerate(mmt_data):

	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)

	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')

	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)

	#	generate time array
	mmt_time = np.array([hour, minute, second])

	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])

	#	calculate julian date
	jd = date2jd(*datetime_array)
	step_jd = jd[0] + jd[1]

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	#	set light curve data datatypes
	std_mag = np.float(std_mag)
	app_mag = np.float(app_mag)
	distance = np.float(distance)
	phase = np.float(phase)

	#	generate light curve data array
	mmt_lc_data = [std_mag, app_mag, distance, phase]
		
	#	assemble line data
	line_data = np.hstack([jd, current_sec, mmt_lc_data])

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		mmt_data_parsed.append(line_data)
		sim_index += 1

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	set previous data values
		jd_prev = prev_line[0]
		jdfrac_prev = prev_line[1]
		prev_sec = prev_line[2]

		#	set previous light curve data
		std_mag_prev = prev_line[-4]
		app_mag_prev = prev_line[-3]
		distance_prev = prev_line[-2]
		phase_prev = prev_line[-1]

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		std_mag_last = std_mag
		app_mag_last = app_mag
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		std_mag_step = (std_mag_last - std_mag_prev) / npts
		app_mag_step = (app_mag_last - app_mag_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	fill first line
		interp_data = np.zeros([npts, len(line_data)])

		#	calculate interpolation data
		for n in range(1, len(interp_data) + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jd_prev + jdfrac_prev) + (n * (0.1 / 86400))
			jd = jday_jfrac(jdate)

			#	calculate interpolated light curve data
			std_mag = std_mag_prev + (n * std_mag_step)
			app_mag = app_mag_prev + (n * app_mag_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			mmt_lc_data = [std_mag, app_mag, distance, phase]
					
			#	assemble line data
			line_data = np.hstack([jd, interp_sec, mmt_lc_data])

			#	add to output
			interp_data[n - 1, :] = line_data

		#	find required lines in interpolated data
		for interp_line in interp_data:

			#	set current simulation second
			interp_line_sec = interp_line[2]

			if interp_line_sec == sim_range_sec[sim_index]:
				mmt_data_parsed.append(line_data)
				sim_index += 1

	#	save previous line
	if itt != 0:
		prev_line = line_data
		prev_time = mmt_time

#	ensure parsed MMT data matches simulation data
mmt_data_parsed = np.array(mmt_data_parsed)

if len(mmt_data_parsed) == len(lc_sim_range):
	print('\nSimulation and MMT data matches...')
else:
	print('\nError: Simulation and MMT data does not match...')
	sys.exit()

mmt_signal_time = mmt_data_parsed[:, 2]
mmt_signal_std = mmt_data_parsed[:, 3]
mmt_signal_app = mmt_data_parsed[:, 4]

###################################################################################################
#	generate datevec array
###################################################################################################

print('\nGenerating DateVec array...')

#	tick mark locators
years = mdates.YearLocator()		# every year
months = mdates.MonthLocator()		# every month
days = mdates.DayLocator()			# every day
hours = mdates.HourLocator()		# every hour
minutes = mdates.MinuteLocator()	# every minute
seconds = mdates.SecondLocator()	# every second

#	assemble jdate array
# mmt_jdate_col = np.add(mmt_data_parsed[:, 0], mmt_data_parsed[:, 1])
mmt_jdate_col = lc_sim_range

#	generate date vectors
datevec_array = []

for line in mmt_jdate_col:
	#	convert jd to date
	year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(line))]
	#	turn into string
	datestr = str('-'.join(['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]))
	timestr = ':'.join(['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)])
	datevec  = ' '.join([datestr, timestr])
	#	convert to datetime object
	date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
	#	append  to output array
	datevec_array.append(date_time_obj)

###################################################################################################
#	figure parameters
###################################################################################################

plot_max = 200
plot_max = len(mmt_signal_app)

####################################################
#	set figure-wide parameters

dpi_val = 300
inch2cm = 2.54

font_sizex = 6
font_sizey = 6
font_pad = 0.5

label_sizex = 10
label_sizey = 10

marker_size = 4
marker_edge = 1

#	set plot index
max_index = int(9.2 * 20)

linewidth_val = 0.5
linewidth_val = 1.0

hspace_val = 0.1
wspace_val = 0.0

grid_width = 0.1
alt_line_width = 0.1

title_size = 8
title_pad = 2

#	set use of latex
plt.rc('text', usetex=True)
plt.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]

# figure = plt.figure(num=None, figsize=(16.0 / inch2cm, 5.0 / inch2cm),
#  	dpi=dpi_val, facecolor='w', edgecolor='k'
# )

figure, ax = plt.subplots(2, 1, figsize=(16.0 / inch2cm, 10.0 / inch2cm),
 	dpi=dpi_val, facecolor='w', edgecolor='k', sharex=True
)

figure.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

# ax1 = plt.subplot()

ax1 = ax[0]
ax2 = ax[1]

color_str = [0.0, 0.5, 0.0]

###################################################################################################
#	axis 1: light curve

ax1.plot(mmt_signal_time[0:plot_max], mmt_signal_app[0:plot_max],
	linewidth=linewidth_val,
	color='k',
	label='Real'
)

ax1.plot(mmt_signal_time[0:plot_max], base_signal[0:plot_max],
	linewidth=linewidth_val,
	linestyle='dashed',
	color=color_str,
	label='Synthetic'
)

#	finalise plot

#	reverse y axis
ax1.invert_yaxis()

#	set grid
ax1.grid()
ax1.legend(loc='upper right')

# ax1.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax1.set_ylabel(r'$m_{std}$', fontsize=label_sizex)

ax1.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax1.tick_params(axis='y', labelsize=font_sizey, pad=0)

font_size = 10

###################################################################################################
#	axis 2: residuals

#	residuals
resids = abs(np.subtract(mmt_signal_app[0:plot_max], base_signal[0:plot_max]))

# ax2 = plt.subplot(212)

ax2.plot(mmt_signal_time[0:plot_max], resids,
	linewidth=linewidth_val,
	color='k',
)

#	trendline
b, m = np.polynomial.polynomial.polyfit(mmt_signal_time[0:plot_max], resids, 1)

ax2.plot(mmt_signal_time[0:plot_max], b + m * mmt_signal_time[0:plot_max],
	linewidth=linewidth_val,
	color=color_str,
)

ax2.set_xlabel(r'$time (s)$', fontsize=label_sizex)
ax2.set_ylabel(r'$Residuals$', fontsize=label_sizex)

ax2.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax2.tick_params(axis='y', labelsize=font_sizey, pad=0)

font_size = 10

ax2.grid()

plt.show()

####################################################
#	saving figure

#	ask to save light curve file for testing
user_input = input(
	'\nSave Light Curve to:\n	{}? [y/n] '.format(out_str)
)

if user_input == 'y':
	
	print('\n	Saving Light Curve...')
	#	save light curve to file
	figure.savefig(out_str, bbox_inches="tight", dpi=300)

elif user_input == 'no':
	print('Not saved.')
sys.exit()
