###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.colors as colors

from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('MMT testing script initilised.')

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

##################################################
#	figure output

#	set flag
savefile_flag = False
savefile_flag = True

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Conferences',
	'IOC_2019',
	'technical_paper',
	'acta_figures',
	'Final'
)
###################################################################################################
#	load polar data files
###################################################################################################

###################################################################################################
#	MMT Inputs

#	light curve example file used
polar_data_output_file = 'mmt_polar_data.txt'

##################################################
#	load polar data

print('\nLoading Polar Data file: ', polar_data_output_file)

#	set output file and path
polar_data_output_path = os.path.join(path_data, 'paper_plots/polar_data', polar_data_output_file)
mmt_polar_data = np.loadtxt(polar_data_output_path, delimiter=',')

#	normalise
mmt_polar_data = np.divide(mmt_polar_data, mmt_polar_data.max())

###################################################################################################
#	graphical outputs
###################################################################################################

####################################################
#	generate polar mesh grid

#	110 so that 90 gets plotted
inc_angle_range = np.arange(0, 110, 10)
azm_angle_range = np.arange(0, 370, 10)

r, th = np.meshgrid(inc_angle_range, azm_angle_range)

####################################################
#	set figure-wide parameters

dpi_val = 300
small = 1e-15
inch2cm = 2.54

####################################################
#	select colour map
cmap = plt.cm.get_cmap('cividis_r')
cmap = plt.cm.get_cmap('summer_r')
cmap.set_under('w')
 
####################################################
#	find azimuth offset to align with grid
nazm_vals = len(th) - 1
offset = (360 / nazm_vals) / 2

####################################################
#	figure parameters
font_sizex = 6
font_sizey = 6
font_pad = -3
marker_size = 8
marker_edge = 2

hspace_val = 0.35
wspace_val = 0.35

grid_width = 0.1
grid_color = [0.2, 0.2, 0.2]

title_size = 8
title_pad = 10

# ####################################################
# #	set figure wide colour bounds
# 
# #	set number of colours to use
# n_col = 100
# 
# #	get max vals
# e1_max = polar_data_e1.max()
# e2_max = polar_data_e2.max()
# e3_max = polar_data_e3.max()
# e4_max = polar_data_e4.max()
# 
# #	generate array of max vals
# max_vals = np.array([
# 	[e1_max, 0],
# 	[e2_max, 1],
# 	[e3_max, 2],
# 	[e4_max, 3]
# ])
# 
# #	sort decending
# max_vals = max_vals[max_vals[:, 0].argsort()][::-1]
# 
# #	take largest pixel
# max_arg = max_vals[0, 1]
# max_val = max_vals[0, 0]
# 
# #	extract data array containing largest pixel
# if max_arg == 0:
# 	maxdata = polar_data_e1
# elif max_arg == 1:
# 	maxdata = polar_data_e2
# elif max_arg == 2:
# 	maxdata = polar_data_e3
# elif max_arg == 3:
# 	maxdata = polar_data_e4
# 
# #	round max val and set step
# newmax = np.round(maxdata.max(), 1)
# step = 0.1
# 
# #	set bounds
# bounds = np.linspace(0, newmax + step, n_col)
# 
# #	generate colour bar bounds
# bounds[0] = small
# norm = colors.BoundaryNorm(boundaries=bounds, ncolors=256)
# 
# #	set step tickmarks
# tick_marks = np.arange(0.0, newmax + (step), step)

####################################################
#	initialise figure
plt.rc('text', usetex=True)

fig = plt.figure(figsize=(9.0/inch2cm, 9.0/inch2cm), dpi=dpi_val)

plt.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

###################################################################################################
#	example 1

####################################################
#	initialise axes
ax1 = plt.subplot(111, projection="polar")
ax1.set_theta_zero_location("N")
ax1.set_theta_direction(-1)

# ax1.set_title(r'$Example 1$', pad=title_pad, fontsize=title_size)

ax1.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax1.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax1.set_rticks(np.arange(0, 100, 20))
ax1.set_rlim(0, 100)

####################################################
#	colour the plot
# plt.pcolor(np.radians(th - offset), r, polar_data_e1, norm=norm, cmap=cmap)
plt.pcolor(np.radians(th - offset), r, mmt_polar_data, cmap=cmap)

####################################################
#	plot rotation vector

#	attitude
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 60.0

ax1.plot(np.radians(rot_vec_azm_deg), rot_vec_inc_deg,
	marker='+',
	color='g',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

####################################################
#	finalise figure
ax1.grid(which='major', axis='both', linewidth=grid_width, color=grid_color)

label_position=ax1.get_rlabel_position()
ax1.text(
	np.radians(70),
	ax1.get_rmax() / 2.,
	r'$\phi$',
	ha='center',
	va='center'
)

ax1.text(
	np.radians(160),
	ax1.get_rmax() + 15,
	r'$\psi$',
	ha='center',
	va='center'
)

####################################################
#	add colour bar
plt.subplots_adjust(bottom=0.1, right=0.8)
cax = plt.axes([0.87, 0.1, 0.03, 0.8])

# plt.colorbar(cax=cax, ticks=tick_marks)
cbar = plt.colorbar(cax=cax)
cbar.set_label(r'$RMS$')

cax.tick_params(axis='y', labelsize=font_sizey, length=3, direction='in')

####################################################
#	saving figure
fig_name = 'mmt_polar_comp' + '.eps'
out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	plt.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
sys.exit()
