###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Case File Generation
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call
import random

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

from tools.file_generation.case_generation import gen_case_obs

#	import from general function list
from tools.functions.functions_list import object_gen_obs, \
 									jd2date

#	import from transformation functions
from tools.functions.functions_transform import site_vec, rot1, rot2, axis2mat, mat2angles, rot_mat

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors

###################################################################################################
#	initilisation
###################################################################################################

#	clear terminal
call(['clear'])
print('Case file generation started...')

#############################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#############################################
#	case directories

#	case file save path
output_dir = os.path.join(path_data, 'input_dir', 'case_files')

#	geomtry input path
geom_path = os.path.join(path_data, 'geometry_dir', 'geom_dir', 'cylinder')

###################################################################################################
#	Simulation Parameters
###################################################################################################

##################################################
#	time data
start_time_str = 'start_time = 2016, 1, 1, 00, 00, 00.0'
stop_time_str = 'stop_time = 2016, 1, 1, 00, 00, 00.0'
append_str = '_491_25885_2016-12-22.txt'

##################################################
#	reflection case
reflection_case = 'diffuse'

##################################################
#	geometry case
geom_str = 'cylinder'

##################################################
#	illumination vector polar angles
illum_azm_deg = 229.08965701
illum_inc_deg = 77.55147339

illum_azm_rad = np.radians(illum_azm_deg)
illum_inc_rad = np.radians(illum_inc_deg)

##################################################
#	illumination vector polar angles
obs_azm_deg = 270.0
obs_inc_deg = 40.0

obs_azm_rad = np.radians(obs_azm_deg)
obs_inc_rad = np.radians(obs_inc_deg)

##################################################
#	attitude state
attit_x_deg = 140.0
attit_y_deg = 36.0
attit_z_deg = 0.0

attit_x_rad = np.radians(attit_x_deg)
attit_y_rad = np.radians(attit_y_deg)
attit_z_rad = np.radians(attit_z_deg)

attitude_deg = np.array([attit_x_deg, attit_y_deg, attit_z_deg])
attitude_rad = np.array([attit_x_rad, attit_y_rad, attit_z_rad])

##################################################
#	case indentifier
case_name = 'attit-x'

#	generate case vector
case_vector = [
	np.round(illum_azm_deg, 1),
	np.round(illum_inc_deg, 1),
	np.round(obs_azm_deg, 1),
	np.round(obs_inc_deg, 1)
]

#	generate ident string
ident_string = 'illum_{}-{}_obs_{}-{}'.format(*case_vector)

#	generate output file name
case_file_name = geom_str + '_' + ident_string + '_' + case_name + '.txt'

##################################################
#	BDRF parameters

#	Diffuse albedo
omega_bdrf = 0.5

#	Refractive Index
n_bdrf = (1 + np.sqrt(omega_bdrf)) / (1 - np.sqrt(omega_bdrf))

#	Root-Mean-Square slope
m_bdrf = 0.25

##################################################
#	attitude state

#	define a rotational period in seconds
rot_period = 10

#	x vector
rot_axis = np.array([1.0, 0.0, 0.0])

#	calculate rotation angle per second
rot_angle = 360.0 / rot_period

#	convert to rotation matrix
attitude_matrix = axis2mat(rot_angle, rot_axis)

#	extract euler angles
angles1 = np.array(mat2angles(attitude_matrix)[0:3])
angles2 = np.array(mat2angles(attitude_matrix)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

#	extract attitude motion
attitude_motion = angles1

#################################################
#	rotation vector

#	attitude rot mat
att_rot_mat = rot_mat(attitude_rad)

#	rotation vector
x_vec = np.array([1.0, 0.0, 0.0])
rot_vec = np.dot(x_vec, att_rot_mat)
rot_vec[0] = -rot_vec[0]

##################################################
#	define illumination vector

#	apply inclination
temp_illum_vec = rot2(x_vec, np.radians(illum_inc_rad))

#	apply azimuth
illum_vec = rot1(temp_illum_vec, np.radians(illum_azm_rad))
illum_vec[0] = -illum_vec[0]

#	correct small values
small = 1e-8
if -small < illum_vec[0] < small:
	illum_vec[0] = 0.0
if -small < illum_vec[1] < small:
	illum_vec[1] = 0.0
if -small < illum_vec[2] < small:
	illum_vec[2] = 0.0

###################################################################################################
#	case loop
###################################################################################################

#	range of attitude/azimuth angles
azm_ang_range = []
for angle in np.arange(0.0, 360.0, 10.0):
	azm_ang_range.append(angle)

#	range of attitude/inclination angles
inc_ang_range = []
for angle in np.arange(0.0, 90.0, 10.0):
	inc_ang_range.append(angle)

case_num = 1

with open(os.path.join(output_dir, case_file_name), mode='wt', encoding='utf-8') as case_file:

	for attit_x_deg in azm_ang_range:
		
		#	reset attitude
		attit_x_rad = np.radians(attit_x_deg)
		attitude_deg = np.array([attit_x_deg, attit_y_deg, attit_z_deg])
		attitude_rad = np.array([attit_x_rad, attit_y_rad, attit_z_rad])

		print('\nGenerating case:')
		print('\n	Illumination Azimuth / Inclination = 	[{}, {}]'.format(
			illum_azm_deg, illum_inc_deg
		))
		print('	Observation Azimuth / Inclination = 	[{}, {}]'.format(
			obs_azm_deg, obs_inc_deg
		))
		print('	Attitude / Rotation Vector = 		[{}, {}, {}]'.format(
			attit_x_deg, attit_y_deg, attit_z_deg
		))

		###########################################################################################
		#	set bdrf parameters
	
		if reflection_case == 'diffuse':
			#	Specular and Diffuse ratio
			d_bdrf = 1.0
			s_bdrf = 0.0
	
		elif reflection_case == 'specular':
			#	Specular and Diffuse ratio
			d_bdrf = 0.0
			s_bdrf = 1.0
	
		print('\n	Reflection Case = 	', reflection_case)

		###########################################################################################
		#	set geometry data
	
		if geom_str == 'div1':
			geom_file = '01_div1_sphere.stl'
			geom_cases = [[1, geom_file]]
	
		elif geom_str == 'div5':
			geom_file = '05_div5_sphere.stl'
			geom_cases = [[5, geom_file]]

		elif geom_str == 'cube':
			geom_file = '01_cube_cube.stl'
			geom_cases = [[1, geom_file]]

		elif geom_str == 'cylinder':
			geom_file = 'cylinder.stl'
			geom_cases = [[1, geom_file]]

		print('	Geometry File = 	', geom_file)
	
		###########################################################################################
		#	generate observation vector
		###########################################################################################

		#	apply elevation
		temp_obs_vec = rot2(x_vec, obs_inc_rad)
		
		#	apply azimuth
		obs_vec = rot1(temp_obs_vec, obs_azm_rad)
		obs_vec[0] = -obs_vec[0]

		#	correct small values
		if -small < obs_vec[0] < small:
			obs_vec[0] = 0.0
		if -small < obs_vec[1] < small:
			obs_vec[1] = 0.0
		if -small < obs_vec[2] < small:
			obs_vec[2] = 0.0

		###########################################################################################
		#	case generation
		###########################################################################################

		#	number of header data line in case file
		data_nlines = 14
		
		#	Generate file name
		if case_num < 10:
			case_num_str = '00000' + str(case_num)
		elif case_num < 100:
			case_num_str = '0000' + str(case_num)
		elif case_num < 1000:
			case_num_str = '000' + str(case_num)
		elif case_num < 10000:
			case_num_str = '00' + str(case_num)
		elif case_num < 100000:
			case_num_str = '0' + str(case_num)
		elif case_num > 100000:
			case_num_str = str(case_num)

		#	generate case indent string
		case_ident = '_'.join([case_num_str, ident_string, geom_str, reflection_case])

		#	generate file name
		case_name = case_ident + append_str

		###########################################################################################
		#	file generation function

		case_info = gen_case_obs(
			case_ident,
			start_time_str, stop_time_str,
			geom_path, geom_file,
			attit_x_deg, attit_y_deg, attit_z_deg,
			rot_axis, rot_angle,
			obs_azm_deg, obs_inc_deg,
			illum_azm_deg, illum_inc_deg,
			d_bdrf, s_bdrf, omega_bdrf, n_bdrf, m_bdrf
		)

		###########################################################################################
		#	case object generation function
		object_data, bdrf_data = object_gen_obs(case_name, case_info)

		###########################################################################################
		#	parse case object data
		###########################################################################################

		norad_id = object_data.norad_ident
	
		###########################################################################################
		#	brdf data

		#	convert bdrf array to numpy array
		bdrf_data = np.array(bdrf_data)
	
		#	number of bdrf elements
		bdrf_elements = len(bdrf_data[0, :])
	
		#	number of faces
		nfaces = len(bdrf_data)
	
		#	reshape length 
		reshape_len = nfaces * bdrf_elements 
		bdrf_reshape = np.reshape(bdrf_data, [1, reshape_len])
	
		###########################################################################################
		#	phase vectors
		observation_angles = object_data.observation_angles
		illumination_angles = object_data.illumination_angles

		###########################################################################################
		#	attitude
		initial_attitude = object_data.attitude
		rotation_axis = object_data.rotation_axis
		rotation_angle = object_data.rotation_angle

		###########################################################################################
		#	simulation time data
		jday_start = object_data.jday_utc_start
		jfrac_start = object_data.jfrac_utc_start
		jday_stop = object_data.jday_utc_stop
		jfrac_stop = object_data.jfrac_utc_stop
		
		# convert julian to utc date search strings
		year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)
		
		###########################################################################################
		#	observation site data
		site_latgd = np.radians(object_data.site_latitudegd)
		site_lon = np.radians(object_data.site_longitude)
		site_alt = float(object_data.site_altitude)
		site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

		###########################################################################################

		ident_list = [
			case_num, case_ident.split('_', maxsplit=1)[1],
			norad_id,
			site_latgd, site_lon, site_alt,
			jday_start, jfrac_start, jday_stop, jfrac_stop
		]

		#	generate case vector
		case_vector = np.hstack((
			ident_list,
			initial_attitude, rotation_axis, rotation_angle,
			observation_angles, illumination_angles,
			geom_file,
			nfaces
		))

		#	stack with brdf data
		case_vector = np.reshape(case_vector, [1, len(case_vector)])
		case_line = np.hstack((case_vector, bdrf_reshape))

		###########################################################################################
		#	write case line to file
		print_line = ', '.join(*case_line)
		case_file.write(print_line + '\n')
	
		#	itterate case number
		case_num += 1

	print('Generated case file.')

print('\nCases Generated: {:d}'.format(case_num - 1))
print('\nSaved to File Name: {:s}'.format(case_file_name))
