###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Case File Generation
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

from tools.file_generation.case_generation import gen_case

#	import from general function list
from tools.functions.functions_list import object_gen, \
 									jd2date

#	import from transformation functions
from tools.functions.functions_transform import site_vec, rot1, rot2

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors

###################################################################################################
#	initilisation
###################################################################################################

#	clear terminal
call(['clear'])
print('\nCase file generation started.')

#############################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)

#############################################
#	case directories

#	case file save path
output_dir = os.path.join(path_data, 'case_files')

#	define illumination vector azimuth and elevation
elevation = np.linspace(0, 90, 10)
azimuth = np.linspace(0, 360, 37)

#	geomtry input path
geom_path = os.path.join(path_data, 'geometry_dir')

#############################################
#	Simulation Inputs

#	true lightcurve times
start_time_str = 'start_time = 2016, 12, 22, 02, 05, 27.0'
stop_time_str = 'stop_time = 2016, 12, 22, 02, 09, 58.0'

append_str = '_491_25885_2016-12-22.txt'

reflection_ncases = 2
geom_ncases = 1
attitude_ncases = 4

geom_str = 'cylinder'

case_runs = [
	['0-attit', geom_str, 'diffuse'],
	['x-rot', 	geom_str, 'diffuse'],
	['y-rot', 	geom_str, 'diffuse'],
	['z-rot', 	geom_str, 'diffuse'],
	['0-attit', geom_str, 'specular'],
	['x-rot', 	geom_str, 'specular'],
	['y-rot', 	geom_str, 'specular'],
	['z-rot', 	geom_str, 'specular']
]

case_file_name = geom_str + '_total_cases.txt'

#	total number of cases
ncase_files = len(case_runs)

case_num = 1
case_file_num = 1

#############################################
#	Simulation Parameters

#	define constant observation vector
obs_vec = np.array([1.0, 0.0, 0.0])

#	BDRF parameters

#	Diffuse albedo
omega_bdrf = 0.5

#	Refractive Index
n_bdrf = (1 + np.sqrt(omega_bdrf)) / (1 - np.sqrt(omega_bdrf))

#	Root-Mean-Square slope
m_bdrf = 0.25

###################################################################################################
#	case loop
###################################################################################################

#	reset output dir and case file name
output_dir = os.path.join(path_data, 'case_files')

with open(os.path.join(output_dir, case_file_name), mode='wt', encoding='utf-8') as case_file:

	for case_line in case_runs:
	
		#	rotation case
		case_desig, geom_desig, reflection_case = case_line
	
		###########################################################################################
		#	set bdrf parameters
	
		if reflection_case == 'diffuse':
			#	Specular and Diffuse ratio
			d_bdrf = 1.0
			s_bdrf = 0.0
	
		elif reflection_case == 'specular':
			#	Specular and Diffuse ratio
			d_bdrf = 0.0
			s_bdrf = 1.0
	
		###########################################################################################
		#	set attitude motion
		onerpm = 0.1047
		attit = np.array([0.0, 0.0, 0.0])
		
		if case_desig == '0-attit':
			attit_mot = np.array([0.0, 0.0, 0.0])
		
		elif case_desig == 'x-rot':
			attit_mot = np.array([onerpm, 0.0, 0.0])
		
		elif case_desig == 'y-rot':
			attit_mot = np.array([0.0, onerpm, 0.0])
		
		elif case_desig == 'z-rot':
			attit_mot = np.array([0.0, 0.0, onerpm])
		
		###########################################################################################
		#	set geometry data
	
		if geom_desig == 'div1':
			geom_file = '01_div1_sphere.stl'
			geom_cases = [[1, geom_file]]
	
		elif geom_desig == 'div5':
			geom_file = '05_div5_sphere.stl'
			geom_cases = [[5, geom_file]]

		elif geom_desig == 'cube':
			geom_file = '01_cube_cube.stl'
			geom_cases = [[1, geom_file]]

		elif geom_desig == 'cylinder':
			geom_file = 'cylinder.stl'
			geom_cases = [[1, geom_file]]
	
		###########################################################################################
		
		#	number of header data line in case file
		data_nlines = 14
		
		###########################################################################################
		#	Inputs
		###########################################################################################
		
		#	generate phase vectors
		
		small = 1e-6
		
		phase_vectors = []
		case_names = []

		print('Generating phase vectors...')
		for elv in elevation:

			#	apply elevation
			temp_illum_vec = rot2(obs_vec, np.radians(elv))

			for azm in azimuth:

				#	apply azimuth
				output_illum_vec = rot1(temp_illum_vec, np.radians(azm))

				#	correct small values
				if -small < output_illum_vec[0] < small:
					output_illum_vec[0] = 0.0
				if -small < output_illum_vec[1] < small:
					output_illum_vec[1] = 0.0
				if -small < output_illum_vec[2] < small:
					output_illum_vec[2] = 0.0

				#	append vector to outputs
				phase_vectors.append(output_illum_vec)

				#	case name string
				name_str = 'azm' + str(int(azm)) + '_' + 'elv' + str(int(elv))
				case_names.append(name_str)

		phase_vectors = np.array(phase_vectors)
		
		###########################################################################################
		#	case generation loop
		###########################################################################################
	
		for name_str, case_data in zip(case_names, phase_vectors):
		
			#	set illumination vector from phase vectors
			illum_vec = case_data
		
			for geom_case in geom_cases:
	
				#	Parse case information
				geom_num = geom_case[0]
				geom_file = geom_case[1]

				#	Generate file name
				if case_num < 10:
					case_num_str = '00000' + str(case_num)
		
				elif case_num < 100:
					case_num_str = '0000' + str(case_num)
		
				elif case_num < 1000:
					case_num_str = '000' + str(case_num)
		
				elif case_num < 10000:
					case_num_str = '00' + str(case_num)
		
				elif case_num < 100000:
					case_num_str = '0' + str(case_num)
		
				elif case_num > 100000:
					case_num_str = str(case_num)
		
				case_ident = '_'.join([case_num_str, case_desig, name_str, geom_str, reflection_case])

				print(name_str)
				sys.exit()
		
				#	Generate file name
				case_name = case_ident + append_str
	
				#	file generation function
				case_info = gen_case(
					case_ident,
					start_time_str,
					stop_time_str,
					geom_path,
					geom_file,
					attit_mot,
					attit,
					obs_vec,
					illum_vec,
					d_bdrf,
					s_bdrf,
					omega_bdrf,
					n_bdrf,
					m_bdrf
				)


#				print('Generated case indent: {}, with attit mot: {}'.format(case_ident, attit_mot))
	
				object_data, bdrf_data = object_gen(case_name, case_info)

				#	convert bdrf array to numpy array
				bdrf_data = np.array(bdrf_data)
	
				#	number of bdrf elements
				bdrf_elements = len(bdrf_data[0, :])
	
				#	number of faces
				nfaces = len(bdrf_data)
	
				#	reshape length 
				reshape_len = nfaces * bdrf_elements 
				bdrf_reshape = np.reshape(bdrf_data, [1, reshape_len])
	
				#	phase vectors
				observation_vector = object_data.obs_vec
				illumination_vector = object_data.illum_vec
				
				#	attitude
				initial_attitude = object_data.attitude
				attitude_motion = object_data.attitude_motion
				
				#	simulation time data
				jday_start = object_data.jday_utc_start
				jfrac_start = object_data.jfrac_utc_start
				jday_stop = object_data.jday_utc_stop
				jfrac_stop = object_data.jfrac_utc_stop
				
				#	observation site data
				site_latgd = np.radians(object_data.site_latitudegd)
				site_lon = np.radians(object_data.site_longitude)
				site_alt = float(object_data.site_altitude)
				site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

				# convert julian to utc date search strings
				year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)
				
				#	create required subdirectories
				norad_id = object_data.norad_ident
	
				case_vector = np.hstack((
					[
						case_num,
						case_ident.split('_', maxsplit=1)[1],
						norad_id,
						site_latgd,
						site_lon,
						site_alt,
						jday_start,
						jfrac_start,
						jday_stop,
						jfrac_stop
					],
					initial_attitude,
					attitude_motion,
					observation_vector,
					illumination_vector,
					geom_file,
					nfaces
				))
	
				case_vector = np.reshape(case_vector, [1, len(case_vector)])
	
				case_line = np.hstack((case_vector, bdrf_reshape))

				#	write data to file
				print_line = ', '.join(*case_line)
				case_file.write(print_line + '\n')
	
				case_num += 1
		
	print('Generated case file: {}'.format(case_file_name))
	case_file_num += 1

print('\n{:d} Cases Generated'.format(case_num - 1))
print('\n{:d} Case Files Generated'.format(case_file_num - 1))
