###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front end
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	GLIMMA 
from GLIMMA_obs import glimma_obs

from tools.functions.glim_transform import rot1, rot2, rot3

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	parrellisation
from multiprocessing import Pool

#	progress bar
import progressbar
import tqdm

###################################################################################################
#	clear terminal
call(['clear'])
print('####################################################################################')
print('\nGLIMMA front end script initilised.\n')

#	set start time
sim_start = time.time()

###################################################################################################
#	inputs
###################################################################################################

##################################################
#	simulation parameters

#	parralellisation
para_flag = True
para_flag = False
ncores = 8

#	time step in seconds
time_step = 0.1

#	length of data record [number of points]
len_record = 1500

##################################################
#	case parameters

#	reflection case
reflection_case = 'diffuse'

#	geometry case
geom_str = 'cylinder'

#	illumination vector polar angles
illum_azm_deg = 229.1
illum_inc_deg = 77.6

illum_azm_rad = np.radians(illum_azm_deg)
illum_inc_rad = np.radians(illum_inc_deg)

#	illumination vector polar angles
obs_azm_deg = 270.0
obs_inc_deg = 40.0

obs_azm_rad = np.radians(obs_azm_deg)
obs_inc_rad = np.radians(obs_inc_deg)

#	attitude state
attit_x_deg = 140.0
attit_y_deg = 36.0
attit_z_deg = 0.0

attit_x_rad = np.radians(attit_x_deg)
attit_y_rad = np.radians(attit_y_deg)
attit_z_rad = np.radians(attit_z_deg)

attitude_rad = np.array([attit_x_rad, attit_y_rad, attit_z_rad])

##################################################
#	case indentifier
case_name = 'obsvec-y'
case_name = 'attit-y'
case_name = 'attit-x'

#	generate case vector
case_vector = [illum_azm_deg, illum_inc_deg, obs_azm_deg, obs_inc_deg]

#	generate ident string
ident_string = 'illum_{}-{}_obs_{}-{}'.format(*case_vector)

#	generate output file name
input_file = geom_str + '_' + ident_string + '_' + case_name + '.txt'

#	generate case designation
case_desig = input_file[0:-4]

#	output file
output_file = geom_str + '_' \
		+ ident_string + '_' \
		+ case_name + '_lightcurves_{}-{}.txt'.format(len_record + 1, time_step)

###################################################################################################
#	initilisation
###################################################################################################

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set input case path
case_path = os.path.join(path_data, 'input_dir', 'case_files')
case_dir = os.listdir(case_path)

#	set lightcurve output path
lightcurve_path = os.path.join(path_data, 'lightcurve_dir', case_desig)

#	generate lightcurve case designation directory
if not os.path.exists(lightcurve_path):
	os.mkdir(lightcurve_path)

#	set case output path
output_dir = os.path.join(lightcurve_path, 'total_lightcurves')

#	generate lightcurve output directory
if not os.path.exists(output_dir):
	os.mkdir(output_dir)

##################################################
#	Object 25885 TLE Data

#	end of 2016 tle
tle_line1 = '1 25885U 99043C   16347.16557405 -.00000088  00000-0 -43593-3 0  9992'
tle_line2 = '2 25885  52.0136 114.2293 0006645 165.6082 252.1930 11.39172050765052'

#	generate sim parameter list
sim_para = [root_path, archive_path, len_record, time_step, tle_line1, tle_line2]

##################################################
#	parse case files

#	read in case file
with open(os.path.join(case_path, input_file), mode='r', encoding='utf-8') as case_file:
	total_case_data = case_file.read()

#	split data and convert to numpy array
total_case_data = total_case_data.split('\n')
total_case_data = np.array(total_case_data)

#	remove final newline character
if total_case_data[-1] == '':
	total_case_data = total_case_data[0:-1]

#	reshape input array to required format
total_case_data = np.reshape(total_case_data, [len(total_case_data), 1])

#	initialise arrays of inputs
root_path_array = np.full([len(total_case_data), 1], root_path)
archive_path_array = np.full([len(total_case_data), 1], archive_path)
len_record_array = np.full([len(total_case_data), 1], len_record)
time_step_array = np.full([len(total_case_data), 1], time_step)

tle_line1_array = np.full([len(total_case_data), 1], tle_line1)
tle_line2_array = np.full([len(total_case_data), 1], tle_line2)

#	finalise total data array
total_data = np.hstack((
	root_path_array,
	archive_path_array,
	len_record_array,
	time_step_array,
	tle_line1_array,
	tle_line2_array,
	total_case_data
))

###################################################################################################
#	run GLIMMA
###################################################################################################

first_flag = True

#	parrellised
if para_flag is True:

	#	initialise multiproccessing pool of workers
	p = Pool(ncores)

	light_curve_data = []

	#	map the total data array across GLIMMA
	for single_lc in tqdm.tqdm(p.imap_unordered(glimma_obs, total_data), total=len(total_data)):
		light_curve_data.append(single_lc)

	#	generate output array
	for data_array in light_curve_data:

		#	current light curve data
		light_curve_save = np.array(data_array)

		#	if first result, initialise array
		if first_flag is True:
			output_data = light_curve_save
			first_flag = False

		#	else stack with ongoing array
		else:
			output_data = np.vstack((output_data, light_curve_save))

#	sequential
elif para_flag is False:

	for run_line in total_data:

		###########################################################################################
		#	parse data
		data = run_line[6].split(', ')[0:23]

		#	set case identifier
		run_id = data[0]; case_ident = data[1]; norad_id = data[2]

		#	site data
		site_lat, site_lon, site_alt = data[3:6]

		#	simulation start
		start_jd, start_jfrac, stop_jd, stop_jfrac = data[6:10]

		#	attitude
		attitude = np.array(data[10:13]).astype(float)

		rot_axis = np.array(data[13:16]).astype(float)
		rot_angle = np.array(data[16]).astype(float)

		#	angles data
		obs_angles = np.array(data[17:19]).astype(float)
		obs_azm, obs_inc = obs_angles

		#	generate observation vector
		x_vec = np.array([1.0, 0.0, 0.0])
		obs_vec_temp = rot2(x_vec, np.radians(obs_inc))
		obs_vec = rot1(obs_vec_temp, np.radians(obs_azm))
		obs_vec[0] = -obs_vec[0]

		illum_angles = np.array(data[19:21]).astype(float)
		illum_azm, illum_inc = illum_angles

		#	generate illumination vector
		illum_vec_temp = rot2(x_vec, np.radians(illum_inc))
		illum_vec = rot1(illum_vec_temp, np.radians(illum_azm))
		illum_vec[0] = -illum_vec[0]

		#	geometry
		geom_file = data[21]
		nfaces = data[22]

		###########################################################################################
		#	generate light curve

		#	ensure requested case
		print('\nRunning on:')
		print('\n	Geometry  = ', geom_file)
		print('\n	Attitude = ', attitude)
		print('	Rotation Axis = ', rot_axis)
		print('	Rotation Angle = ', rot_angle)
		print('\n	Obs Vec Azimuth = ', obs_azm)
		print('	Obs Vec Inclination = ', obs_inc)
		print('	Obs Vec = ', obs_vec)
		print('\n	Illum Vec Azimuth = ', illum_azm)
		print('	Illum Vec Inclination = ', illum_inc)
		print('	Illum Vec = ', illum_vec)

		#	generate lightcurve
		light_curve = glimma_obs(run_line)

		sys.exit()

###################################################################################################
#	save output file
###################################################################################################

print('Total light curve data array generated.')

print('\nSaving file to: ', os.path.join(output_dir, output_file))

#	open output file
with open(os.path.join(output_dir, output_file), mode='w', encoding='utf-8') as case_file:

	#	print data to output file
	for line in output_data:
		output_line = ', '.join([*line]) + '\n'
		case_file.write(output_line)

sim_end = time.time()
print('\n GLIMMA Complete.')
print('Time taken = ', (sim_end - sim_start) / 60)
