###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front end
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

import tqdm

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	GLIMMA 
from GLIMMA import glimma

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	parrellisation
from multiprocessing import Pool

#	progress bar
import progressbar

###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front end script initilised.\n')

#	set start time
sim_start = time.time()

###################################################################################################
#	inputs
###################################################################################################

##################################################
#	simulation parameters

#	parralellisation
para_flag = True
para_flag = False
ncores = 8

#	time step in seconds
time_step = 0.1

#	length of data reccord [number of points]
len_reccord = 60 * 4 * 10

##################################################
#	I/O files

#	input file
case_file = 'cylinder_rotvec_cases.txt'

#	output file
output_file = 'cylinder_rotvec_lightcurves_0.1s.txt'

#	generate case designation
case_desig = '_'.join(['lightcurve_data', case_file.split('_')[0], '{:1.1f}'.format(time_step)])

###################################################################################################
#	initilisation
###################################################################################################

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)

#	set input case path
case_path = os.path.join(path_data, 'case_files')
case_dir = os.listdir(case_path)

#	set lightcurve output path
lightcurve_path = os.path.join(path_data, 'lightcurve_dir', case_desig)

#	generate lightcurve case designation directory
if not os.path.exists(lightcurve_path):
	os.mkdir(lightcurve_path)

#	set case output path
output_dir = os.path.join(lightcurve_path, 'total_lightcurves')

#	generate lightcurve output directory
if not os.path.exists(output_dir):
	os.mkdir(output_dir)

##################################################
#	Object 25885 TLE Data

#	end of 2016 tle
tle_line1 = '1 25885U 99043C   16347.16557405 -.00000088  00000-0 -43593-3 0  9992'
tle_line2 = '2 25885  52.0136 114.2293 0006645 165.6082 252.1930 11.39172050765052'

#	generate sim parameter list
sim_para = [root_path, archive_path, len_reccord, time_step, tle_line1, tle_line2]

##################################################
#	parse case files

#	read in case file
with open(os.path.join(case_path, case_file), mode='r', encoding='utf-8') as case_file:
	total_case_data = case_file.read()

#	split data and convert to numpy array
total_case_data = total_case_data.split('\n')
total_case_data = np.array(total_case_data)

#	remove final newline character
if total_case_data[-1] == '':
	total_case_data = total_case_data[0:-1]

#	reshape input array to required format
total_case_data = np.reshape(total_case_data, [len(total_case_data), 1])

#	initialise arrays of inputs
root_path_array = np.full([len(total_case_data), 1], root_path)
archive_path_array = np.full([len(total_case_data), 1], archive_path)
len_reccord_array = np.full([len(total_case_data), 1], len_reccord)
time_step_array = np.full([len(total_case_data), 1], time_step)

tle_line1_array = np.full([len(total_case_data), 1], tle_line1)
tle_line2_array = np.full([len(total_case_data), 1], tle_line2)

#	finalise total data array
total_data = np.hstack((
	root_path_array,
	archive_path_array,
	len_reccord_array,
	time_step_array,
	tle_line1_array,
	tle_line2_array,
	total_case_data
))


# #	parse data
# data = total_data[0][6].split(', ')[0:24]
# 
# #	set case identifier
# run_id = data[0]
# case_ident = data[1]
# norad_id = data[2]
# 
# #	site data
# site_lat = data[3]
# site_lon = data[4]
# site_alt = data[5]
# 
# #	simulation start
# start_jd = data[6]
# start_jfrac = data[7]
# stop_jd = data[8]
# stop_jfrac = data[9]
# 
# #	attitude
# attit = data[10:13]
# attit_mot = data[13:16]
# 
# #	attitude
# obs_vec = data[16:19]
# ill_vec = data[19:22]
# 
# #	geometry
# geom_file = data[22]
# nfaces = data[23]
# 
# #	parse azimuth elevation values
# azm = np.int(case_ident.split('_')[1][3:])
# elv = np.int(case_ident.split('_')[2][3:])
# 
# #	parse attitude case
# attit_case = case_ident.split('_')[0]

###################################################################################################
#	run GLIMMA
###################################################################################################

first_flag = True

#	parrellised
if para_flag is True:

	#	initialise multiproccessing pool of workers
	p = Pool(ncores)

	light_curve_data = []

	#	map the total data array across GLIMMA
	for single_lc in tqdm.tqdm(p.imap_unordered(glimma, total_data), total=len(total_data)):
		light_curve_data.append(single_lc)

	#	generate output array
	for data_array in light_curve_data:

		#	current light curve data
		light_curve_save = np.array(data_array)

		#	if first result, initialise array
		if first_flag is True:
			output_data = light_curve_save
			first_flag = False

		#	else stack with ongoing array
		else:
			output_data = np.vstack((output_data, light_curve_save))

#	sequential
elif para_flag is False:

	for run_line in total_data:

		###########################################################################################
		#	parse data
		data = run_line[6].split(', ')[0:24]

		#	set case identifier
		run_id = data[0]
		case_ident = data[1]
		norad_id = data[2]

		#	site data
		site_lat = data[3]
		site_lon = data[4]
		site_alt = data[5]

		#	simulation start
		start_jd = data[6]
		start_jfrac = data[7]
		stop_jd = data[8]
		stop_jfrac = data[9]

		#	attitude
		attit = data[10:13]
		attit_mot = data[13:16]

		#	attitude
		obs_vec = data[16:19]
		ill_vec = data[19:22]

		#	geometry
		geom_file = data[22]
		nfaces = data[23]

		#	parse azimuth elevation values
		azm = np.int(case_ident.split('_')[1][3:])
		elv = np.int(case_ident.split('_')[2][3:])

		#	parse attitude case
		attit_case = case_ident.split('_')[0]

#		###########################################################################################
#		#	generate light curve
#		if elv == 90 and azm == 10:

		#	ensure requested case
		print('\nRunning on:')
		print('\n	Geom Case = ', geom_file)
		print('	Attit Case = ', attit_mot)
		print('	Elevation = ', elv)
		print('	Azimuth = ', azm)

		#	generate lightcurve
		light_curve = glimma(run_line)

		#	extract light curve data
		light_curve_data = np.array(light_curve[:, -1]).astype(np.float)

		#	generate time data
		time_data = np.zeros([len(light_curve)])

		for i, time_line in enumerate(light_curve[:, 2]):
			time_data[i] = np.round(
				86400 * (np.float(light_curve[i, 2]) - np.float(light_curve[0, 2])), 1
			)

		#	plot lightcurve
		plt.plot(time_data, light_curve_data)
		plt.grid()
		plt.show()

		sys.exit()

###################################################################################################
#	save output file
###################################################################################################

print('Total light curve data array generated.')

print('\nSaving file to: ', os.path.join(output_dir, output_file))

#	open output file
with open(os.path.join(output_dir, output_file), mode='w', encoding='utf-8') as case_file:

	#	print data to output file
	for line in output_data:
		output_line = ', '.join([*line]) + '\n'
		case_file.write(output_line)

sim_end = time.time()
print('\n GLIMMA Complete.')
print('Time taken = ', (sim_end - sim_start) / 60)
