###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	progress bar
import progressbar

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	import from transformation functions
from tools.functions.functions_transform import site_vec, \
										rot_mat, \
										gen_vectors
 
#	import from general function list
from tools.functions.functions_list import object_gen, \
 									date2jd, \
 									jd2date, \
 									jday_jfrac, \
 									eop_interpolate, \
 									get_times, \
 									gen_sgp4
 
#	horizons interface
from tools.functions.horizons_interface import req_horizons
 
#	import GLIMMA transformation module
from tools.functions.glim_transform import glim_trans

#	light curve generation
from tools.functions.glim_slc import glim_slc_new

###################################################################################################
#	GLIMMA Function
###################################################################################################

def glimma(input_str):

	###############################################################################################
	#	parse run string data
	###############################################################################################

	#	parse input string
	root_path, archive_path, len_reccord, time_step, tle_line1, tle_line2, line_data = input_str

	#	set data types
	len_reccord = int(len_reccord)
	time_step = float(time_step)

	#	split line data
	line_data = line_data.split(', ')
	
	#	case data
	case_num = int(line_data[0])
	case_ident = line_data[1]
	norad_id = int(line_data[2])

	#	site data
	site_latgd = float(line_data[3])
	site_lon = float(line_data[4])
	site_alt = float(line_data[5])
	site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

	#	time data
	jday_start = float(line_data[6])
	jfrac_start = float(line_data[7])
	jday_stop = float(line_data[8])
	jfrac_stop = float(line_data[9])

	# convert julian to utc date search strings
	year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)

	#	attitude state
	initial_attitude = np.array(line_data[10:13]).astype(np.float)
	attitude_motion = np.array(line_data[13:16]).astype(np.float)

	#	observation and illumination vectors
	observation_vector = np.array(line_data[16:19]).astype(np.float)
	illumination_vector = np.array(line_data[19:22]).astype(np.float)

	#	geometry file
	geom_file = line_data[22]
	nfaces = int(line_data[23])

	#	bdrf data
	bdrf_data = line_data[24:]

	#	reshape bdrf data
	bdrf_elements = 5
	bdrf_data = np.reshape(np.array(bdrf_data), [nfaces, bdrf_elements]).astype(np.float)

	#	parse case ident string
	attit_case, \
	lat_case, \
	lon_case, \
	geom_case, \
	reflection_case, \
		= case_ident.split('_')

	###############################################################################################
	#	initilise
	###############################################################################################

	#	path to data
	path_data = os.path.join(root_path, archive_path)

	#	flag for outputing geom files for render
	render_flag = False

	#	find prop step in days
	prop_step = time_step / (24 * 60 * 60)
	
	#	add zero to len reccord
	len_reccord = len_reccord + 1
	
#	print('\nSimulation with {} steps at a {} time step, for a total duation of {} seconds.'.format(
#		len_reccord, time_step, time_step * (len_reccord - 1))
#	)
	
	#	extract mean motion from TLE
	mean_mot = float(tle_line2[52:64])
	orbital_mot = mean_mot * ((2 * np.pi) / (24 * 60 * 60))

	print('\n[{}]	Simulation initialised with inputs: \
		\n	Attit Case = {} \
		\n	Lat Case = {} \
		\n	Lon Case = {} \
		\n	Geom Case = {} \
		\n	Reflection Case {}'.format(
			datetime.datetime.now(),
			attit_case,
			lat_case,
			lon_case,
			geom_case,
			reflection_case)
	)


	##################################################
	#	directories
	
	#	horizons data
	horz_dir = 'pos_dir/horz_dir'
	#	sgp4 data
	sgp4_dir = 'sgp4_dir'
	#	pos data
	pos_dir = 'pos_dir'
	#	EOP data
	eop_dir = 'eop_files'
	#	geometry
	geom_dir = 'geometry_dir'
	
	##################################################
	#	data files
	
	#	icrf
	sun_icrf_file = 'sun_icrf.txt'
	earth_icrf_file = 'earth_icrf.txt'
	#	sgp4
	sgp4_file = 'sgp4_data.txt'
	#	EOP
	eop_file = 'eop_parsed.csv'
	dat_file = 'dat_parsed.csv'
	#	itrf
	sat_itrf_file = 'sat_itrf.txt'
	sun_itrf_file = 'sun_itrf.txt'
	#	scrf
	site_scrf_file = 'site_scrf.txt'
	sun_scrf_file = 'sun_scrf.txt'
	#	azel
	sat_azel_file = 'sat_azel.txt'
	sun_azel_file = 'sun_azel.txt'
	#	gcrf
	sat_gcrf_file = 'sat_gcrf.txt'
	sun_gcrf_file = 'sun_gcrf.txt'
	site_gcrf_file = 'site_gcrf.txt'
	#	orb
	sat_orb_file = 'sat_orb.txt'
	sun_orb_file = 'sun_orb.txt'
	earth_orb_file = 'earth_orb.txt'
	site_orb_file = 'site_orb.txt'
	
	###############################################################################################
	#	initilise time data
	
	#	julian date
	jd_start = jday_start + jfrac_start
	mjd_start = jd_start - 2400000.5
	
	#	set end date as defined by len_reccord and prop_step
	jd_stop = jd_start + ((len_reccord - 1) * prop_step)
	mjd_stop = jd_stop - 2400000.5
	
	#	convert stop julian date to day and day fraction
	if jd_stop - np.floor(jd_stop) < 0.5:
		jday_stop = np.floor(jd_stop) - 0.5
		jfrac_stop = jd_stop - jday_stop
	
	elif jd_stop - np.floor(jd_stop) > 0.5:
		jday_stop = np.floor(jd_stop) + 0.5
		jfrac_stop = jd_stop - jday_stop
	
	#	convert julian date to date
	year2, mon2, day2, hour2, min2, sec2 = jd2date(jday_stop, jfrac_stop)
	
	#	set end date
	sim_end_date = [
		year2,
		mon2,
		day2,
		hour2,
		min2,
		round(sec2, 1)
	]
	
	#	simulation range in julian date
	sim_range = np.linspace(jd_start, jd_stop, len_reccord)
	
	#	simulation time in seconds
	duration = np.round((time_step * len_reccord) - time_step, 1)
	sim_time = np.linspace(0, duration, len_reccord)

	print_jd = sim_range
	
	###############################################################################################
	#	generate EOP data
	###############################################################################################

	print('\n[{}]	Generating EOP data. . .'.format(datetime.datetime.now()))

	#	assemble eop path
	eop_path = os.path.join(root_path, archive_path, eop_dir)

	#	generate interpolated eop_data
	eop_data, dat_data = eop_interpolate(eop_path, eop_file, dat_file, sim_range, time_step)
	
	###############################################################################################
	#	generate time objects
	###############################################################################################
	
	print('\n[{}]	Generating Time data. . .'.format(datetime.datetime.now()))

	#	start dat value
	dat = dat_data[0, -1]
	
	#	start eop values
	eop_line = eop_data[0, :]
	eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
	#	generate start time object
	start_time = get_times(
		year1,
		mon1,
		day1,
		hour1,
		min1,
		round(sec1, 1),
		ut_offset,
		dat
	)
	
	#	end dat value
	dat = dat_data[-1, -1]
	
	#	end eop values
	eop_line = eop_data[-1, :]
	eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
	stop_time = get_times(
		year2,
		mon2,
		day2,
		hour2,
		min2,
		sec2,
		ut_offset,
		dat
	)
	
	#	extract barycentric julian days and day fractions
	jday_tdb_start = start_time.jday_tdb
	jfrac_tdb_start = start_time.jfrac_tdb
	jday_tdb_stop = stop_time.jday_tdb
	jfrac_tdb_stop = stop_time.jfrac_tdb
	
	#	calculate juluan dates
	start_tdb_jdate = (jday_tdb_start + jfrac_tdb_start)
	stop_tdb_jdate = (jday_tdb_stop + jfrac_tdb_stop)
	
	#############################################################################################
	#	Horizons query
	#############################################################################################
	
	#	assemble path
	horz_path = os.path.join(path_data, horz_dir)
	
	print('\n[{}]	Checking for HORIZONS data. . .'.format(datetime.datetime.now()))
	if os.path.isfile(os.path.join(horz_path, sun_icrf_file)):
#		print('	Horizons Data exists, opening. . .')
	
		#	load sun icrf data
		sun_icrf_raw = np.loadtxt(
			os.path.join(horz_path, sun_icrf_file),
			delimiter=','
		)
	
		#	load earth itrf data
		earth_icrf_raw = np.loadtxt(
			os.path.join(horz_path, earth_icrf_file),
			delimiter=','
		)
	
	else:
	
#		print('	No Horizons file. Acquisition required.')
		sun_icrf_raw, earth_icrf_raw = req_horizons(
			start_time,
			stop_time,
			horz_path,
			eop_path,
			sim_range,
			time_step,
		)
	
		#	output file names
		output_files = [sun_icrf_file, earth_icrf_file]
	
		#	output data arrays
		output_data = [sun_icrf_raw, earth_icrf_raw]
	
		#	save files
		for (out_file, out_data) in zip(output_files, output_data):
			horz_output_path = os.path.join(horz_path, out_file)
		
			with open(horz_output_path, 'w') as output_file:
				for line in out_data:
					#	format time data
					time_str = '{:.10f}, {:d}, {:d}, {:d}, {:d}, {:d}, {:.1f}'.format(
						line[0],
						int(line[1]),
						int(line[2]),
						int(line[3]),
						int(line[4]),
						int(line[5]),
						line[6]
					)
		
					#	format position data
					data_str = '{:.10f}, {:.10f}, {:.10f}'.format(
						line[7],
						line[8],
						line[9],
					)
		
					#	assemble output string and write to file
					output_str = time_str + ', ' + data_str
					output_file.write(output_str + '\n')
		
			print('\nHorizons icrf file {} has been generated.'.format(out_file))
	
	#############################################################################################
	#	sgp4 generation
	#############################################################################################
	
	#	sgp4 path
	sgp4_path = os.path.join(path_data, sgp4_dir, sgp4_file)
	
	print('\n[{}]	Checking for SGP4 data'.format(datetime.datetime.now()))
	if os.path.isfile(os.path.join(path_data, sgp4_dir, sgp4_file)):
#		print('	SGP4 Data exists, opening. . .')
	
		#	load sgp4 data
		sat_teme_raw = np.loadtxt(
			os.path.join(path_data, sgp4_dir, sgp4_file),
			delimiter=','
		)
	
	else:
	
#		print('	No SGP4 file. Acquisition required.')
	
		#	generate SGP4 satellite object
		satellite = twoline2rv(tle_line1, tle_line2, wgs72)

		#	initilise position data matrix
		sat_teme_raw = gen_sgp4(sim_range, time_step, satellite)
		
		#	save sgp4 datafile
		np.savetxt(sgp4_path, sat_teme_raw, delimiter=',')
	
	###############################################################################################
	#	check for existing data files
	###############################################################################################
	
	print('\n[{}]	Checking for transform data. . .'.format(datetime.datetime.now()))
	if os.path.isfile(os.path.join(path_data, pos_dir, sat_orb_file)) \
		and os.path.isfile(os.path.join(path_data, pos_dir, sun_orb_file)):
	
#		print('	Orbital data exists, opening. . .')
		transform_req = False
	
		#	load azel sat data
		sat_orb = np.loadtxt(
			os.path.join(path_data, pos_dir, sat_orb_file),
			delimiter=','
		)
	
		#	load azel sun data
		sun_orb = np.loadtxt(
			os.path.join(path_data, pos_dir, sun_orb_file),
			delimiter=','
		)
	
		#	load itrf sat data
		site_orb = np.loadtxt(
			os.path.join(path_data, pos_dir, site_orb_file),
			delimiter=','
		)
	
		#	load itrf sat data
		earth_orb = np.loadtxt(
			os.path.join(path_data, pos_dir, earth_orb_file),
			delimiter=','
		)
	
	else:
		transform_req = True
	
	###################################################################################################
	#	transformation
	###################################################################################################
	
	if transform_req is True:
	
		###############################################################################################
		#	rotation matrix generation
		###############################################################################################
	
		#	tt time
		jdaytt_start = start_time.jday_tt
		jfractt_start = start_time.jfrac_tt
		jdtt_start = jdaytt_start + jfractt_start
		
		#	ut1 time
		jdayut1_start = start_time.jday_ut1
		jfracut1_start = start_time.jfrac_ut1
		jdut1_start = jdayut1_start + jfracut1_start
		
		#	generate rotation matrices
#		print('	Generating rotation matrices. . .')
	
		pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
			jdtt_start, jdut1_start, eop_data, time_step
		)
	
		###############################################################################################
		#	transformation
		###############################################################################################
	
#		print('\nStarting transformation')
			
		#	generate input arrays
		data_array = np.hstack([sat_teme_raw, sun_icrf_raw[:, 7:], earth_icrf_raw[:, 7:]])
		site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))
		time_data = np.array((jdaytt_start, jfractt_start, jdayut1_start, jfracut1_start))
		
		#	initilise output arrays
		out_data_itrf = np.zeros((len(data_array), 9))
		out_data_gcrf = np.zeros((len(data_array), 6))
		out_data_gcrf_sat = np.zeros((len(data_array), 6))
		out_data_scrf = np.zeros((len(data_array), 6))
		out_data_orb = np.zeros((len(data_array), 9))
		out_data_orb_sat = np.zeros((len(data_array), 6))
		out_data_azel = np.zeros((len(data_array), 6))

		#	run transformation module
		glim_trans(
			data_array,
			eop_data, dat_data, pn_array, pm_array, st_array, c2t_array,
			site_data, time_data,
			out_data_itrf,
			out_data_gcrf_sat, out_data_gcrf,
			out_data_scrf,
			out_data_orb, out_data_orb_sat,
			out_data_azel
		)
		
		#	prepare itrf data
		sat_itrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_itrf[:, 0:6]))
		sun_itrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_itrf[:, 6:9]))
		
		#	prepare gcrf data
		sat_gcrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_gcrf_sat[:, 0:6]))
		sun_gcrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_gcrf[:, 0:3]))
		site_gcrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_gcrf[:, 3:6]))
		
		#	prepare scrf data
		sun_scrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_scrf[:, 0:3]))
		site_scrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_scrf[:, 3:6]))
		
		#	prepare orb data
		sat_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb_sat[:, 0:6]))
		sun_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb[:, 0:3]))
		earth_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb[:, 3:6]))
		site_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb[:, 6:9]))
		
		#	prepare azimuth elevation data
		sat_azel = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_azel[:, 0:3]))
		sun_azel = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_azel[:, 3:6]))
		
		#	save itrf data
		np.savetxt(os.path.join(path_data, pos_dir, sat_itrf_file), sat_itrf, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, sun_itrf_file), sun_itrf, delimiter=',')
		
		#	save gcrf data
		np.savetxt(os.path.join(path_data, pos_dir, sat_gcrf_file), sat_gcrf, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, sun_gcrf_file), sun_gcrf, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, site_gcrf_file), site_gcrf, delimiter=',')
		
		#	save scrf data
		np.savetxt(os.path.join(path_data, pos_dir, sun_scrf_file), sun_scrf, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, site_scrf_file), site_scrf, delimiter=',')
		
		#	save orb data
		np.savetxt(os.path.join(path_data, pos_dir, sat_orb_file), sat_orb, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, sun_orb_file), sun_orb, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, earth_orb_file), earth_orb, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, site_orb_file), site_orb, delimiter=',')
		
		#	save azimuth elevation data
		np.savetxt(os.path.join(path_data, pos_dir, sat_azel_file), sat_azel, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, sun_azel_file), sun_azel, delimiter=',')
		
#		print('\nTransformation Complete.')
	
	###############################################################################################
	#	prepare for vector generation
	###############################################################################################

	print('\n[{}]	Initialising Object Geometry...'.format(datetime.datetime.now()))
	
	###############################################
	#	generate body mesh
	
	#	load static mesh
	body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
	
	#	define body axis vectors of object mesh
	x_body_unit = np.array((1.0, 0.0, 0.0))
	y_body_unit = np.array((0.0, 1.0, 0.0))
	z_body_unit = np.array((0.0, 0.0, 1.0))
	
	#	set vectors and areas
	body_vectors = body_mesh.vectors
	body_normals = body_mesh.normals
	body_areas = body_mesh.areas
	
	###############################################################################################
	#	generate light curve generation vector data
	###############################################################################################

	print('\n[{}]	Generating Vector Data...'.format(datetime.datetime.now()))

	#	generate orb2inertial velocity vector
	orb2inertial = np.array([0.0, 0.0, -orbital_mot])
	
	#	this is the initial offset between the orbital and body frames
	init_frame_offset = np.copy(initial_attitude)
	
	#	this is the angular velocity of the body frame wrt the orbital frame
	init_frame_vel = np.copy(attitude_motion)

	#	vector generation function
	vector_data_body, vector_data_orb, vector_data_inertial, attitude_out = gen_vectors(
		path_data, time_step, len_reccord,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_frame_offset, init_frame_vel,
		observation_vector, illumination_vector
	)

	###############################################################################################
	#	light curve generation
	###############################################################################################
	print('\n[{}]	Generating light curve...'.format(datetime.datetime.now()))
	
	###############################################################################################
	#	Prepare for propagation

	#	custom vector flag. uses predefined reflection vectors when true
	custom_vectors_flag = True

	if custom_vectors_flag is True:
		#	using: custom vectors
		sun_vec_data = vector_data_body[:, 22:25]
		site_vec_data = vector_data_body[:, 25:28]

	elif custom_vectors_flag is False:
		#	using: actual vectors
		sun_vec_data = vector_data_body[:, 1:4]
		site_vec_data = vector_data_body[:, 4:7]

	###############################################################################################
	#	generate light curve
	
	#	initilise lightcurve output
	light_curve_array = np.zeros((len(sim_time), 3))
	light_curve_array[:, 1] = sim_time

	#	standard distance flag. sets distance to 1000 km when true
	std_dist_flag = False

	#	run light curve generation function
	light_curve = glim_slc(sun_vec_data, site_vec_data, light_curve_array, \
			jd_start, body_vectors, body_areas, bdrf_data, std_dist_flag)

	#	append case number
	case_num_array = np.full([len(light_curve), 1], case_num)
	case_ident_array = np.full([len(light_curve), 1], case_ident)
	light_curve = np.hstack((case_ident_array, case_num_array, light_curve))


	print('\n[{}]	Run End: {}'.format(datetime.datetime.now(), case_ident))
	return light_curve

###################################################################################################
#	GLIMMA obs Function
###################################################################################################

def glimma_obs(input_str):

	#	set logging levels
	glimma_log_level = 0
	eop_log_level = 0
	horz_logging_level = 0
	trans_logging_level = 0

	graphical_output = True
	graphical_output = False

	###############################################################################################
	#	parse inputs
	###############################################################################################

	############################################
	#	parse input string
	root_path, archive_path, len_record, time_step, tle_line1, tle_line2, line_data = input_str

	#	path to data
	path_data = os.path.join(root_path, archive_path)
	geom_dir = 'geometry_dir/geom_dir'

	#	set data types
	len_record = int(len_record)
	time_step = float(time_step)

	###############################################################################################
	#	parsing case data

	#	split line data
	line_data = line_data.split(', ')
	
	#	case data
	case_num = int(line_data[0])
	case_ident = line_data[1]
	norad_id = int(line_data[2])

	############################################
	#	site data
	site_latgd = float(line_data[3])
	site_lon = float(line_data[4])
	site_alt = float(line_data[5])
	site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

	############################################
	#	time data
	jday_start = float(line_data[6])
	jfrac_start = float(line_data[7])
	jday_stop = float(line_data[8])
	jfrac_stop = float(line_data[9])

	# convert julian to utc date search strings
	year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)

	#	attitude
	initial_attitude_deg = np.array(line_data[10:13]).astype(float)
	initial_attitude = np.radians(np.array(line_data[10:13]).astype(float))

	rot_axis = np.array(line_data[13:16]).astype(float)
	rot_angle = np.array(line_data[16]).astype(float)

	#	angles data
	obs_angles = np.array(line_data[17:19]).astype(float)
	obs_azm_deg, obs_inc_deg = obs_angles
	obs_azm, obs_inc = np.radians(obs_angles)

	illum_angles = np.array(line_data[19:21]).astype(float)
	illum_azm_deg, illum_inc_deg = illum_angles
	illum_azm, illum_inc = np.radians(illum_angles)

	#	geometry
	geom_file = line_data[21]
	geom_case = geom_file[0:-4]
	nfaces = int(line_data[22])

	#	convert to rotation matrix
	attitude_matrix = axis2mat(np.radians(rot_angle), rot_axis)

	#	extract euler angles
	angles1 = np.array(mat2angles(attitude_matrix)[0:3])
	angles2 = np.array(mat2angles(attitude_matrix)[3:6])
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	initial_attitude_motion = angles1

	#	parse case ident string
	illum_string, \
	_, \
	obs_string, \
	_, \
	geom_case, \
	reflection_case, \
		= case_ident.split('_')

#	print('\nRESETTING OBS VEC TO [270, 40]!!!')
#
#	obs_azm = np.radians(270)
#	obs_inc = np.radians(40)

	############################################
	#	calculate rotation vector
	attitude_matrix = rot_mat(initial_attitude)
	x_vec = np.array([1.0, 0.0, 0.0])
	rot_vec = np.dot(x_vec, attitude_matrix)

	#	convert to plot frame
	rot_vec = rot_vec
	rot_vec[0] = -rot_vec[0]

	#	convert to polar coordinates
	rot_vec_polar = np.degrees([initial_attitude[0], initial_attitude[1]])

	############################################
	#	observation vector

	#	generate illumination vector
	#	apply inclination
	obs_vec_temp = rot2(x_vec, obs_inc)
	#	apply azimuth
	obs_vec = rot1(obs_vec_temp, obs_azm)
	obs_vec[0] = -obs_vec[0]

	############################################
	#	observation and illumination vectors

	#	generate illumination vector
	#	apply inclination
	illum_vec_temp = rot2(x_vec, illum_inc)
	#	apply azimuth
	illum_vec = rot1(illum_vec_temp, illum_azm)
	illum_vec[0] = -illum_vec[0]

	############################################
	#	geometry file

	#	bdrf data
	bdrf_data = line_data[23:]

	#	reshape bdrf data
	bdrf_elements = 5
	bdrf_data = np.reshape(np.array(bdrf_data), [nfaces, bdrf_elements]).astype(np.float)

	###############################################
	#	generate body mesh
	
	#	load static mesh
	geom_path = os.path.join(path_data, geom_dir, geom_file[0:-4])
	body_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))
	
	#	define body axis vectors of object mesh
	x_body_unit = np.array((1.0, 0.0, 0.0))
	y_body_unit = np.array((0.0, 1.0, 0.0))
	z_body_unit = np.array((0.0, 0.0, 1.0))
	
	#	set vectors and areas
	body_vectors = body_mesh.vectors
	body_normals = body_mesh.normals
	body_areas = body_mesh.areas

	###############################################################################################
	#	initilise
	###############################################################################################

	#	flag for outputing geom files for render
	render_flag = False

	#	find prop step in days
	prop_step = time_step / (24 * 60 * 60)
	
	#	add zero to len reccord
	len_record = len_record + 1
	
	#	extract mean motion from TLE
	mean_mot = float(tle_line2[52:64])
	orbital_mot = mean_mot * ((2 * np.pi) / (24 * 60 * 60))

	if glimma_log_level != 0:
		print('\n[{}]	Simulation initialised with inputs: \
			\n					Attitude 		= [{:1.3f}, {:1.3f}, {:1.3f}] \
			\n					Attitude Motion		= [{:1.3f}, {:1.3f}, {:1.3f}] \
			\n					Obs Vec Angles		= [{}, {}] \
			\n					Illum Vec Angles 	= [{:1.3f}, {:1.3f}] \
			\n					Geom Case 		= {} \
			\n					Reflection Case 	= {}'.format(
				datetime.datetime.now(),
				*initial_attitude,
				*initial_attitude_motion,
				obs_azm, obs_inc,
				illum_azm, illum_inc,
				geom_case,
				reflection_case)
		)
	###############################################################################################
	#	I/O

	############################################
	#	directories
	
	#	horizons data
	pos_dir = 'pos_dir'
	pos_path = os.path.join(path_data, pos_dir)

	horz_dir = pos_dir + '/horz_dir'
	sgp4_dir = pos_dir
	eop_dir = 'eop_files'
	
	##################################################
	#	data files
	
	#	icrf
	sun_icrf_file = 'sun_icrf.txt'; earth_icrf_file = 'earth_icrf.txt'; sgp4_file = 'sgp4_data.txt'
	eop_file = 'eop_parsed.csv'; dat_file = 'dat_parsed.csv'; sat_itrf_file = 'sat_itrf.txt'
	sun_itrf_file = 'sun_itrf.txt'
	site_scrf_file = 'site_scrf.txt'; sun_scrf_file = 'sun_scrf.txt'
	sat_azel_file = 'sat_azel.txt'; sun_azel_file = 'sun_azel.txt'; sat_gcrf_file = 'sat_gcrf.txt'
	sun_gcrf_file = 'sun_gcrf.txt'; site_gcrf_file = 'site_gcrf.txt'; sat_orb_file = 'sat_orb.txt'
	sun_orb_file = 'sun_orb.txt'; earth_orb_file = 'earth_orb.txt'; site_orb_file = 'site_orb.txt'
	
	###############################################################################################
	#	time data
	
	#	julian date
	jd_start = jday_start + jfrac_start
	mjd_start = jd_start - 2400000.5
	
	#	set end date as defined by len_record and prop_step
	jd_stop = jd_start + ((len_record - 1) * prop_step)
	mjd_stop = jd_stop - 2400000.5
	
	#	convert stop julian date to day and day fraction
	if jd_stop - np.floor(jd_stop) < 0.5:
		jday_stop = np.floor(jd_stop) - 0.5
		jfrac_stop = jd_stop - jday_stop
	
	elif jd_stop - np.floor(jd_stop) > 0.5:
		jday_stop = np.floor(jd_stop) + 0.5
		jfrac_stop = jd_stop - jday_stop
	
	#	convert julian date to date
	year2, mon2, day2, hour2, min2, sec2 = jd2date(jday_stop, jfrac_stop)
	
	#	set end date
	sim_end_date = [
		year2,
		mon2,
		day2,
		hour2,
		min2,
		round(sec2, 1)
	]
	
	#	simulation range in julian date
	sim_range = np.linspace(jd_start, jd_stop, len_record)
	
	#	simulation time in seconds
	duration = np.round((time_step * len_record) - time_step, 1)
	sim_time = np.linspace(0, duration, len_record)

	print_jd = sim_range
	
#	###############################################################################################
#	#	generate EOP data
#	###############################################################################################
#
#	if glimma_log_level != 0:
#		print('\n[{}]	Acquiring EOP data. . .'.format(datetime.datetime.now()))
#
#	#	assemble eop path
#	eop_path = os.path.join(root_path, archive_path, eop_dir)
#
#	#	generate interpolated eop_data
#	eop_data, dat_data = eop_interpolate(
#		eop_path, eop_file, dat_file,
#		sim_range, time_step,
#		eop_log_level
#	)
#
#	###############################################################################################
#	#	generate time objects
#	###############################################################################################
#	
#	if glimma_log_level != 0:
#		print('\n[{}]	Generating Time data. . .'.format(datetime.datetime.now()))
#
#	#	start dat value
#	dat = dat_data[0, -1]
#	
#	#	start eop values
#	eop_line = eop_data[0, :]
#	eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, \
#		_, lod, _, dX, _, dY, _ = eop_line
#	
#	#	generate start time object
#	start_time = get_times(year1, mon1, day1, hour1, min1, round(sec1, 1), ut_offset, dat)
#	
#	#	end dat value
#	dat = dat_data[-1, -1]
#	
#	#	end eop values
#	eop_line = eop_data[-1, :]
#	eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, \
#		_, lod, _, dX, _, dY, _ = eop_line
#	
#	stop_time = get_times(year2, mon2, day2, hour2, min2, sec2, ut_offset, dat)
#	
#	#	extract barycentric julian days and day fractions
#	jday_tdb_start = start_time.jday_tdb
#	jfrac_tdb_start = start_time.jfrac_tdb
#	jday_tdb_stop = stop_time.jday_tdb
#	jfrac_tdb_stop = stop_time.jfrac_tdb
#	
#	#	calculate juluan dates
#	start_tdb_jdate = (jday_tdb_start + jfrac_tdb_start)
#	stop_tdb_jdate = (jday_tdb_stop + jfrac_tdb_stop)
  
	#############################################################################################
	#	Horizons query
	#############################################################################################

	#	assemble path
	horz_path = os.path.join(path_data, horz_dir)

	if glimma_log_level != 0:
		print('\n[{}]	Acquiring HORIZONS data. . .'.format(datetime.datetime.now()))

	if os.path.isfile(os.path.join(horz_path, sun_icrf_file)):
#		print('	Horizons Data exists, opening. . .')
	
		#	load sun icrf data
		sun_icrf_raw = np.loadtxt(
			os.path.join(horz_path, sun_icrf_file),
			delimiter=','
		)
	
		#	load earth itrf data
		earth_icrf_raw = np.loadtxt(
			os.path.join(horz_path, earth_icrf_file),
			delimiter=','
		)
	
	else:

#		print('	No Horizons file. Acquisition required.')
		sun_icrf_raw, earth_icrf_raw = req_horizons(
			start_time,
			stop_time,
			pos_path,
			eop_path,
			sim_range,
			time_step,
			horz_logging_level
		)
	
		#	output file names
		output_files = [sun_icrf_file, earth_icrf_file]
	
		#	output data arrays
		output_data = [sun_icrf_raw, earth_icrf_raw]
	
		#	save files
		for (out_file, out_data) in zip(output_files, output_data):
			horz_output_path = os.path.join(horz_path, out_file)
		
			with open(horz_output_path, 'w') as output_file:
				for line in out_data:
					#	format time data
					time_str = '{:.10f}, {:d}, {:d}, {:d}, {:d}, {:d}, {:.1f}'.format(
						line[0],
						int(line[1]),
						int(line[2]),
						int(line[3]),
						int(line[4]),
						int(line[5]),
						line[6]
					)
		
					#	format position data
					data_str = '{:.10f}, {:.10f}, {:.10f}'.format(
						line[7],
						line[8],
						line[9],
					)
		
					#	assemble output string and write to file
					output_str = time_str + ', ' + data_str
					output_file.write(output_str + '\n')
		
			print('\nHorizons icrf file {} has been generated.'.format(out_file))
	
	#############################################################################################
	#	sgp4 generation
	#############################################################################################
	
	#	sgp4 path
	sgp4_path = os.path.join(path_data, sgp4_dir, sgp4_file)
	
	if glimma_log_level != 0:
		print('\n[{}]	Acquiring SGP4 data'.format(datetime.datetime.now()))

	if os.path.isfile(os.path.join(path_data, sgp4_dir, sgp4_file)):
#		print('	SGP4 Data exists, opening. . .')
	
		#	load sgp4 data
		sat_teme_raw = np.loadtxt(
			os.path.join(path_data, sgp4_dir, sgp4_file),
			delimiter=','
		)
	
	else:
	
#		print('	No SGP4 file. Acquisition required.')
	
		#	generate SGP4 satellite object
		satellite = twoline2rv(tle_line1, tle_line2, wgs72)

		#	initilise position data matrix
		sat_teme_raw = gen_sgp4(sim_range, time_step, satellite)
		
		#	save sgp4 datafile
		np.savetxt(sgp4_path, sat_teme_raw, delimiter=',')
	
	###############################################################################################
	#	check for existing data files
	###############################################################################################
	
	if glimma_log_level != 0:
		print('\n[{}]	Acquiring transform data. . .'.format(datetime.datetime.now()))

	if os.path.isfile(os.path.join(path_data, pos_dir, sat_orb_file)) \
		and os.path.isfile(os.path.join(path_data, pos_dir, sun_orb_file)):
	
#		print('	Orbital data exists, opening. . .')
		transform_req = False
	
		#	load azel sat data
		sat_orb = np.loadtxt(
			os.path.join(path_data, pos_dir, sat_orb_file),
			delimiter=','
		)
	
		#	load azel sun data
		sun_orb = np.loadtxt(
			os.path.join(path_data, pos_dir, sun_orb_file),
			delimiter=','
		)
	
		#	load itrf sat data
		site_orb = np.loadtxt(
			os.path.join(path_data, pos_dir, site_orb_file),
			delimiter=','
		)
	
		#	load itrf sat data
		earth_orb = np.loadtxt(
			os.path.join(path_data, pos_dir, earth_orb_file),
			delimiter=','
		)
	
	else:
		transform_req = True
	
	###############################################################################################
	#	transformation
	###############################################################################################
	
	if transform_req is True:
	
		###########################################################################################
		#	rotation matrix generation
		###########################################################################################
	
		#	tt time
		jdaytt_start = start_time.jday_tt
		jfractt_start = start_time.jfrac_tt
		jdtt_start = jdaytt_start + jfractt_start
		
		#	ut1 time
		jdayut1_start = start_time.jday_ut1
		jfracut1_start = start_time.jfrac_ut1
		jdut1_start = jdayut1_start + jfracut1_start
		
		#	generate rotation matrices
#		print('	Generating rotation matrices. . .')
	
		pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
			jdtt_start, jdut1_start, eop_data, time_step
		)
	
		###########################################################################################
		#	transformation
		###########################################################################################
	
#		print('\nStarting transformation')

		#	generate input arrays
		data_array = np.hstack([sat_teme_raw, sun_icrf_raw[:, 7:], earth_icrf_raw[:, 7:]])
		site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))
		time_data = np.array((jdaytt_start, jfractt_start, jdayut1_start, jfracut1_start))
		
		#	initilise output arrays
		out_data_itrf = np.zeros((len(data_array), 9))
		out_data_gcrf = np.zeros((len(data_array), 6))
		out_data_gcrf_sat = np.zeros((len(data_array), 6))
		out_data_scrf = np.zeros((len(data_array), 6))
		out_data_orb = np.zeros((len(data_array), 9))
		out_data_orb_sat = np.zeros((len(data_array), 6))
		out_data_azel = np.zeros((len(data_array), 6))

		simulation_data = np.array([trans_logging_level, time_step])

		#	run transformation module
		glim_trans(
			data_array,
			eop_data,
			dat_data,
			pn_array,
			pm_array,
			st_array,
			c2t_array,
			site_data,
			time_data,
			simulation_data,
			out_data_itrf,
			out_data_gcrf,
			out_data_scrf,
			out_data_orb,
			out_data_orb_sat,
			out_data_azel
		)
		
		#	prepare itrf data
		sat_itrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_itrf[:, 0:6]))
		sun_itrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_itrf[:, 6:9]))
		
		#	prepare gcrf data
		sat_gcrf = np.hstack((
			np.reshape(sim_range, [len(sim_range), 1]),
			out_data_gcrf_sat[:, 0:6]
		))

		sun_gcrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_gcrf[:, 0:3]))
		site_gcrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_gcrf[:, 3:6]))
		
		#	prepare scrf data
		sun_scrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_scrf[:, 0:3]))
		site_scrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_scrf[:, 3:6]))
		
		#	prepare orb data
		sat_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb_sat[:, 0:6]))
		sun_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb[:, 0:3]))
		earth_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb[:, 3:6]))
		site_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb[:, 6:9]))
		
		#	prepare azimuth elevation data
		sat_azel = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_azel[:, 0:3]))
		sun_azel = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_azel[:, 3:6]))
		
		#	save itrf data
		np.savetxt(os.path.join(path_data, pos_dir, sat_itrf_file), sat_itrf, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, sun_itrf_file), sun_itrf, delimiter=',')
		
		#	save gcrf data
		np.savetxt(os.path.join(path_data, pos_dir, sat_gcrf_file), sat_gcrf, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, sun_gcrf_file), sun_gcrf, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, site_gcrf_file), site_gcrf, delimiter=',')
		
		#	save scrf data
		np.savetxt(os.path.join(path_data, pos_dir, sun_scrf_file), sun_scrf, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, site_scrf_file), site_scrf, delimiter=',')
		
		#	save orb data
		np.savetxt(os.path.join(path_data, pos_dir, sat_orb_file), sat_orb, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, sun_orb_file), sun_orb, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, earth_orb_file), earth_orb, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, site_orb_file), site_orb, delimiter=',')
		
		#	save azimuth elevation data
		np.savetxt(os.path.join(path_data, pos_dir, sat_azel_file), sat_azel, delimiter=',')
		np.savetxt(os.path.join(path_data, pos_dir, sun_azel_file), sun_azel, delimiter=',')
		
#		print('\nTransformation Complete.')

	###############################################################################################
	#	generate light curve generation vector data
	###############################################################################################

	if glimma_log_level != 0:
		print('\n[{}]	Generating Vector Data...'.format(datetime.datetime.now()))
	
	#	generate orb2inertial velocity vector
	orb2inertial = np.array([0.0, 0.0, -orbital_mot])
	#	this is the initial offset between the orbital and body frames
	init_frame_offset = np.copy(initial_attitude)
	#	this is the angular velocity of the body frame wrt the orbital frame
	init_frame_vel = np.copy(initial_attitude_motion)

	#	vector generation function
	vector_data_body, vector_data_orb, vector_data_inertial, attitude_out = gen_vectors(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_frame_offset, init_frame_vel,
		obs_vec, illum_vec
	)

	###############################################################################################
	#	parse vector data

	#	orb data
	sun_orb_data = vector_data_orb[:, 1:4]
	site_orb_data = vector_data_orb[:, 4:7]
	earth_orb_data = vector_data_orb[:, 7:10]

	sat_orb_vel = vector_data_orb[:, 10:13]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])

	illum_vec_orb_data = vector_data_orb[:, 22:25]
	obs_vec_orb_data = vector_data_orb[:, 25:28]

	#	body data
	sun_body_data = vector_data_body[:, 1:4]
	site_body_data = vector_data_body[:, 4:7]
	earth_body_data = vector_data_body[:, 7:10]

	sat_body_vel = vector_data_body[:, 10:13]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])

	illum_vec_body_data = vector_data_body[:, 22:25]
	obs_vec_body_data = vector_data_body[:, 25:28]

#	print('\nIllum Vec Orb Data = \n', illum_vec_orb_data)

	###############################################################################################
	#	polar data generation
	###############################################################################################

	####################################################
	#	extract light curve polar data
	jd_time_data = sim_range
	lc_time_step = time_step

#	print('\nIllum Vec Orb Data = \n', illum_vec_orb_data)
#	print('\nEarth Vec Orb Data = \n', earth_orb_data)
#	print('\nObs Vec Orb Data = \n', obs_vec_orb_data)
#	print('\nSat Vec Orb Data = \n', sat_orb_data)

	sun_polar_data, site_polar_data, phase_angles, jd_time_data = extract_fun(
		jd_time_data, lc_time_step, time_step,
		illum_vec_orb_data, earth_orb_data, obs_vec_orb_data, sat_orb_data
	)

#	print('\nSun Polar Data = \n', np.degrees(sun_polar_data))
#	print('\nSite Polar Data = \n', np.degrees(site_polar_data))

	###############################################################################################
	#	light curve generation
	###############################################################################################

	if glimma_log_level != 0:
		print('\n[{}]	Generating light curve...'.format(datetime.datetime.now()))
	
	#	initilise lightcurve output
	light_curve_array = np.zeros((len_record, 3))
	light_curve_array[:, 1] = sim_time[0:len_record]

	#	standard distance flag. sets distance to 1000 km when true
	std_dist_flag = True

#	print('\nLC Input: Sun Body = \n', illum_vec_body_data)
#	print('LC Input: Site Body = \n', obs_vec_body_data)

	#	run light curve generation function
	light_curve = glim_slc(illum_vec_body_data, obs_vec_body_data, light_curve_array, \
			jd_start, body_vectors, body_areas, bdrf_data, std_dist_flag)

	#	generate additional data
	case_num_array = np.full([len(light_curve), 1], case_num)
	case_ident_array = np.full([len(light_curve), 1], case_ident)

	illum_azm_array = np.full([len(light_curve), 1], illum_azm_deg)
	illum_inc_array = np.full([len(light_curve), 1], illum_inc_deg)
	obs_azm_array = np.full([len(light_curve), 1], obs_azm_deg)
	obs_inc_array = np.full([len(light_curve), 1], obs_inc_deg)
	attitude_x_array = np.full([len(light_curve), 1], initial_attitude_deg[0])
	attitude_y_array = np.full([len(light_curve), 1], initial_attitude_deg[1])
	attitude_z_array = np.full([len(light_curve), 1], initial_attitude_deg[2])

	light_curve = np.hstack((
		case_ident_array,
		case_num_array,
		illum_azm_array,
		illum_inc_array,
		obs_azm_array,
		obs_inc_array,
		attitude_x_array,
		attitude_y_array,
		attitude_z_array,
		light_curve
	))

	if glimma_log_level != 0:
		print('\n[{}]	Run End: {}'.format(datetime.datetime.now(), case_ident))

 	###############################################################################################
 	#	graphical outputs
 	###############################################################################################
 
	if graphical_output is True:

		print('\n[{}]	Generating Graphical Outputs...'.format(datetime.datetime.now()))
	
		###########################################################################################
		#	Lightcurve plot
	
		####################################################
		#	initialise figure	
		fig1 = plt.figure()
		ax1 = plt.axes()
		ax1.set_title('Light Curve')
		ax1.set_xlabel('Simulation Time / Step')
		ax1.set_ylabel('Brightness / Standard Mangitude')
	
		####################################################
		#	extract light curve data
		light_curve_data = np.array(light_curve[:, -1]).astype(np.float)
	
		#	generate time data
		time_data = np.zeros([len(light_curve)])
	
		for i, time_line in enumerate(light_curve[:, 2]):
			time_data[i] = np.round(
				86400 * (np.float(light_curve[i, 2]) - np.float(light_curve[0, 2])), 1
			)
	
		####################################################
		#	plot lightcurve
		xdata = np.arange(0, len_record, 1)
		ax1.plot(xdata, light_curve_data)
		ax1.grid()

#		#	set bounds
#		ax1.set_ylim([7, 6])
	
		print('\n	Len LC data = ', len(light_curve_data))
	
		###########################################################################################
		#	Polar plot
	
		####################################################
		#	initialise polar figure
		fig2 = plt.figure(figsize=(10, 6))
		ax2 = plt.subplot(111, projection='polar')
		ax2.set_title('Polar plot')
		
		ax2.set_theta_zero_location("N")
		ax2.set_theta_direction(-1)
		
		ax2.set_rticks(np.arange(0, 100, 10))
		ax2.set_rlim([0, 100])
	
		####################################################
		#	calculate angles data
	
		#	rot vec to sun angle
		sun_angles = []
		site_angles = []
	
		for illum_orb_vec, obs_orb_vec in zip(illum_vec_orb_data, obs_vec_orb_data):
	
			sun_angles.append(np.arccos(np.divide(
				np.dot(illum_orb_vec, rot_vec),
				np.linalg.norm(illum_orb_vec) * np.linalg.norm(rot_vec)
			)))
	
			site_angles.append(np.arccos(np.divide(
				np.dot(obs_orb_vec, rot_vec),
				np.linalg.norm(obs_orb_vec) * np.linalg.norm(rot_vec)
			)))
	
		####################################################
		#	plot rotation vector
		marker_size = 10
		marker_edge = 2
		ax2.plot(np.radians(rot_vec_polar[0]), rot_vec_polar[1],
			marker='+',
			color='b',
			markersize=marker_size,
			markeredgewidth=marker_edge,
			clip_on=False,
			label='Rotation Vector'
		)
	
		####################################################
		#	plot illumination and observation positions
	
		marker_sun = [len(sun_polar_data) - 1]
		marker_site = [len(site_polar_data) - 1]
		
		#	plot sun positions
		ax2.plot(
			sun_polar_data[:, 0],
			np.degrees(sun_polar_data[:, 1]),
			linewidth=0.5, color='r', markevery=marker_sun, marker='>', label='Sun'
		)
		
		#	plot site positions
		ax2.plot(
			site_polar_data[:, 0],
			np.degrees(site_polar_data[:, 1]),
			linewidth=0.5, color='g', markevery=marker_site, marker='>', label='Site'
		)

		ax2.legend()
	
		###########################################################################################
		#	angles plot
	
		#	initialise phase angle plots
		fig3 = plt.figure(figsize=(10,6))
		ax3 = plt.axes()
		ax3.set_title('Phase Angles.')
	
		#	plot lightcurve
		ax3.plot(np.degrees(phase_angles), color='b', label='Phase Angle')
		ax3.plot(np.degrees(sun_angles), color='r', label='Sun Angle')
		ax3.plot(np.degrees(site_angles), color='g', label='Site Angle')
	
#		###########################################################################################
#		#	saving simple light curve
#
#		simple_lc_output_dir = '/home/ldjb/codebase/data_archive/GLIMMA_data/lightcurve_dir'
#		simple_lc_output_file = 'simple_lc.txt'
#		simple_lc_path = os.path.join(simple_lc_output_dir, simple_lc_output_file)
#
#		#	open output file
#		with open(simple_lc_path, mode='w', encoding='utf-8') as case_file:
#		
#			#	print data to output file
#			for line in light_curve:
#				output_line = ', '.join([*line]) + '\n'
#				case_file.write(output_line)
		
		plt.show()
		sys.exit()

	return light_curve
