###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	import from transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, rv2kep, \
	gen_vectors, rot1, rot2, rot3

#	import from general function list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, \
	get_times, interp

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

##################################################################################################
#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA synthetic observatory script initilised.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

###################################################################################################
#	light curve generation parameters
###################################################################################################

##################################################
#	simulation settings

#	start time
start_date = '2016-01-01'
start_time = '00-00-00.0'

#	length
days_record = 365
time_step = 60
len_record = int(86400 / time_step) * days_record + 1

#	output light curve data
lc_time_step = 1
max_steps = lc_time_step * 3

#	max length in seconds
max_length = 500
max_index = int(np.round(max_length / lc_time_step, 1)) + 1

##################################################
#	generate data file identifiers

#	site identifiers
latitude = 0
longitude = 0

#	object identifiers
inclination = 40

#	range of raan values
raan = 90

#	orbital height
orbital_height = 2000

#	generate strings
site_str = 'lat' + str(latitude) + 'lon' + str(longitude)
object_id = 'i' + str(inclination) + 'h' + str(orbital_height) + 'raan' + str(raan)
	
##################################################
#	geometry
geom_str = 'cylinder'
geom_dir = 'geometry_dir/geom_dir/' + geom_str
geom_file = geom_str + '.stl'

#	brdf file
brdf_file = geom_str + '.brdf'

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

#	load bdrf file
bdrf_data = np.loadtxt(
	os.path.join(path_data, geom_dir, brdf_file),
	delimiter=','
)

##################################################
#	illumination vector
illum_vec_azm = np.radians(133.4452)
illum_vec_inc = np.radians(76.1839)

illum_vec_azm = np.radians(137.0753)
illum_vec_inc = np.radians(66.1578)

illum_vec_azm = np.radians(142)
illum_vec_inc = np.radians(57)

##################################################
#	attitude

#	generate random attitude
initial_attitude = np.array([np.radians(142), np.radians(57), 0.0])

##################################################
#	attitude motion

#	1 rpm (radians per second)
one_rpm = (2.0 * np.pi) / 60.0

#	define rotation axis in body frame
axis_unit = np.array([1.0, 0.0, 0.0])

#	convert to rotation matrix
attitude_matrix = axis2mat(one_rpm, axis_unit)

#	extract euler angles
angles1 = np.array(mat2angles(attitude_matrix)[0:3])
angles2 = np.array(mat2angles(attitude_matrix)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

initial_attitude_motion = angles1

#	generate file name attitude string
attitude_str = '{:0.3f}_{:0.3f}_{:0.3f}_{:0.3f}_{:0.3f}_{:0.3f}'.format(
	*initial_attitude,
	*initial_attitude_motion
)

###################################################################################################
#	initialisation
###################################################################################################

print('\n#####################################################################################')
print('\n	Time parameters: ')
print('		Simulation time step:		{}'.format(time_step))
print('		Max Duration:			{}	[steps]'.format(max_steps))
print('						{}	[s]'.format(max_length))

print('\n	Site parameters: ')
print('		Site Str:			{}'.format(site_str))
print('		Site Latitude:			{}	deg'.format(latitude))
print('		Site Longitude:			{}	deg'.format(longitude))

print('\n	Object parameters: ')
print('\n		Geometry File: {}'.format(geom_file))
print('\n		Attitude:		{}'.format(initial_attitude))
print('		Attitude Motion: 	{}'.format(initial_attitude_motion))
print('\n		Inclination:		{}'.format(inclination))
print('		Orbital Height: 	{}'.format(orbital_height))
print('		Right Ascension:	{}'.format(raan))

###################################################################################################
#	load extracted light curves data file
###################################################################################################

#	set lightcurve output directory
lightcurve_data_path = os.path.join(path_data, 'lightcurve_dir', 'data_files')

#	generate file name
lightcurve_data_file = '_'.join(
	[site_str, str(object_id), start_date, start_time, str(len_record), str(time_step)]
)  + '.txt'

#	load light curve file
lightcurve_data_array = np.loadtxt(
	os.path.join(lightcurve_data_path, lightcurve_data_file),
	delimiter=','
)

#	parse time data
start_jdate = lightcurve_data_array[0, 1]
start_jday, start_jfrac = jday_jfrac(start_jdate)
time_step = np.round((lightcurve_data_array[1, 1] - start_jdate) * 24 * 60 * 60, 1)

#	parse norad id
site_str, norad_id, start_date_str, _, _, _ = lightcurve_data_file.split('_')

###################################################################################################
#	generate orbital motion from state vector
###################################################################################################

#	satellite posititon
sat_itrf_pos = lightcurve_data_array[0, 23:26]
sat_itrf_vel = lightcurve_data_array[0, 20:23]

#	convert to orbital elements
kep_ele = rv2kep(sat_itrf_pos, sat_itrf_vel)
raan, argp, true_anom, inc, ecc, a_axis = kep_ele 

#	define earth-sun mu in km3/s2
mu_earth = 3.986004418e14 * 1e-9

orbital_period = np.pi * 2 * np.sqrt(a_axis**3 / mu_earth)
orbital_mot = (2 * np.pi) / orbital_period

#	generate lightcurve output file name
lightcurve_output_file = '_'.join([site_str, object_id, geom_str, attitude_str]) + '.txt'

###################################################################################################
#	Unique light curves loop
###################################################################################################

#	unique light curve ids
unique_ids = np.unique(lightcurve_data_array[:, 0])
unique_ids = [232]

start_index = 0
index_step = max_length + 1
end_index = index_step

#	initialise output array
lightcurve_output_array = np.zeros([len(unique_ids) * end_index, 4])

#	set step attitude values
step_attit = np.copy(initial_attitude)
step_attit_mot = np.copy(initial_attitude_motion)
start_flag = True

#	reference vectors
x_vec = np.array([1.0, 0.0, 0.0])
negx_vec = np.array([-1.0, 0.0, 0.0])
zero_vec = np.array([0.0, 0.0, 0.0])

print('\n	Number of Light Curves:		', len(unique_ids))
# input('\nPress Enter to start light curve generation.')

for light_curve_id in unique_ids:
	print(
			'\nWorking on light curve ID: {:d} of {:d} #####################################'.format(
			int(light_curve_id),
			int(unique_ids[-1])
		)
	)
	
	###############################################################################################
	#	parse inputs
	###############################################################################################

	#	ids of light curve data
	data_ids = np.where(lightcurve_data_array[:, 0]==light_curve_id)
	
	#	extract data
	light_curve_data = lightcurve_data_array[data_ids]
	len_light_curve = len(light_curve_data)

	#	parse array
	jd_time_data = light_curve_data[:, 1]
	sun_r_data = light_curve_data[:, 2]
	sun_az_data = light_curve_data[:, 3]
	sun_el_data = light_curve_data[:, 4]
	sat_r_data = light_curve_data[:, 5]
	sat_az_data = light_curve_data[:, 6]
	sat_el_data = light_curve_data[:, 7]
	sun_orb_data = light_curve_data[:, 8:11]
	earth_orb_data = light_curve_data[:, 11:14]
	site_orb_data = light_curve_data[:, 14:17]
	sat_orb_data = light_curve_data[:, 17:23]
	sat_itrf_data = light_curve_data[:, 23:26]

	#	duration
	duration_sec = time_step * (len_light_curve - 1)
	
	#	time range in seconds
	light_curve_sim_time = np.linspace(0, duration_sec, len_light_curve)
	
	#	stop times
	start_jdate = jd_time_data[0]
	start_jday, start_jfrac = jday_jfrac(start_jdate)
	
	stop_jdate = jd_time_data[-1]
	stop_jday, stop_jfrac = jday_jfrac(stop_jdate)

	###############################################################################################
	#	interpolate if required
	###############################################################################################
	
	#	convert to light curve generation time step
	if lc_time_step != time_step:
	
		#	find nsteps
		prop_step = lc_time_step / (24 * 60 * 60)
		num_points = int(round((stop_jdate - start_jdate) / prop_step))
	
		#	new time range to interpolate to
		interp_range = np.linspace(start_jdate, stop_jdate, num_points + 1)
	
		#	current x spline
		spline_x = jd_time_data
	
		#	new spline x
		spline_x_new = interp_range
	
		#	generate data to interpolate
		interp_data_in = np.hstack((sat_orb_data, sun_orb_data, earth_orb_data, site_orb_data))
	
		#	initilise new y spline data
		interp_data_out = np.zeros((len(interp_range), len(interp_data_in[0])))
	
		#	perform interpolation	
		interp_data = interp(interp_data_in, interp_data_out, spline_x, spline_x_new)
	
		#	extract interpolated data
		jd_time_data = interp_range
		sat_orb_data = interp_data[:, 0:6]
		sun_orb_data = interp_data[:, 6:9]
		earth_orb_data = interp_data[:, 9:12]
		site_orb_data = interp_data[:, 12:15]
	
		#	reset data
		len_light_curve = len(jd_time_data)
		duration_sec = lc_time_step * (len_light_curve - 1)
		light_curve_sim_time = np.linspace(0, duration_sec, len_light_curve)

	#	add time columns to data
	time_col = np.reshape(jd_time_data, [len(jd_time_data), 1])
	sun_orb_data = np.hstack([time_col, sun_orb_data])
	earth_orb_data = np.hstack([time_col, earth_orb_data])
	site_orb_data = np.hstack([time_col, site_orb_data])
	sat_orb_data = np.hstack([time_col, sat_orb_data])

	###############################################################################################
	#	generating replacment data
	###############################################################################################

	#	time data
	time_col = np.reshape(jd_time_data, [len(jd_time_data), 1])

	##################################################
	#	observation vector

	#	generate site vectors
	angles_range = np.flip(np.subtract(np.linspace(0, 60, len(jd_time_data)), 10))

	#	initialise empty site orb array
	site_orb_data_simple = np.zeros([len(jd_time_data), 3])

	#	generate site vectors
	for data_index, rot_angle in enumerate(angles_range):
		site_orb_data_simple[data_index, :] = rot3(negx_vec, np.radians(rot_angle))

	#	add time column
	site_orb_data_simple = np.hstack([time_col, site_orb_data_simple])

	##################################################
	#	illumination vector

	#	apply inclination
	illum_vec_temp = rot2(x_vec, illum_vec_inc)

	#	apply azimuth
	illum_vec = rot1(illum_vec_temp, illum_vec_azm)

	#	convert from polar to orbital frame
	illum_vec[0] = -illum_vec[0]

	#	initialise empty sun orb array
	sun_orb_data_simple = np.zeros([len(jd_time_data), 3])

	#	generate sun vectors
	for tempi in range(len(sun_orb_data_simple)):
		sun_orb_data_simple[tempi, :] = illum_vec

	#	add time column
	sun_orb_data_simple = np.hstack([time_col, sun_orb_data_simple])

	###############################################################################################
	#	orb data residuals

	#	generate unit vectors
	sun_orb_units = np.zeros(sun_orb_data[:, 1:].shape)
	site_orb_units = np.zeros(site_orb_data[:, 1:].shape)
	for tempi, (sun_orb_vec, site_orb_vec) in enumerate(zip(sun_orb_data[:, 1:], site_orb_data[:, 1:])):
		sun_orb_units[tempi, :] = np.divide(sun_orb_vec, np.linalg.norm(sun_orb_vec))
		site_orb_units[tempi, :] = np.divide(site_orb_vec, np.linalg.norm(site_orb_vec))

	#	orbital residuals
	sun_orb_resid = np.subtract(sun_orb_units, sun_orb_data_simple[:, 1:])
	site_orb_resid = np.subtract(site_orb_units, site_orb_data_simple[:, 1:])

	fig_orb_resid, ax_orb_resid = plt.subplots()
	ax_orb_resid.set_title('Orbital Residuals')

	#	sun data
	ax_orb_resid.plot(sun_orb_resid[:, 0], label='Sun x-residuals', color='r')
	ax_orb_resid.plot(sun_orb_resid[:, 1], label='Sun y-residuals', color='r')
	ax_orb_resid.plot(sun_orb_resid[:, 2], label='Sun z-residuals', color='r')

	#	site data
	ax_orb_resid.plot(site_orb_resid[:, 0], label='Site x-residuals', color='g')
	ax_orb_resid.plot(site_orb_resid[:, 1], label='Site y-residuals', color='g')
	ax_orb_resid.plot(site_orb_resid[:, 2], label='Site z-residuals', color='g')
	ax_orb_resid.legend()

	###############################################################################################
	#	propagate attitude state
	###############################################################################################
	
	#	first light curve of set
	if start_flag is True:

		#	initialise prev_stop_jdate
		prev_stop_jdate = np.copy(stop_jdate)

		print('\n	First light curve of set:\n')
		print('		Light Curve NPTS: 		', len_light_curve)
		print('		Light Curve Duration:		', duration_sec)
		print('		Start Date:			{:4d}/{:02d}/{:02d} {:02d}:{:02d}:{:04.1f}'.format(
			*jd2date(start_jday, start_jfrac)
		))
		
		print('		Start Attitude:			{}'.format(np.round(step_attit, 2)))

		#	set start flag to false
		start_flag = False

	#	first data point of light curve
	elif start_flag is False:
	
		#	propagate attitude from previous light curve end
		light_curve_sep = start_jdate - prev_stop_jdate
		sep_sec = np.round(light_curve_sep * 24 * 60 * 60, 1)

		jday, jfrac = jday_jfrac(np.float(prev_stop_jdate))
	
		#	propagate attitude
		init_frame_offset = end_attitude + np.multiply(init_frame_vel, sep_sec)
	
		#	correct x
		if init_frame_offset[0] >= (2 * np.pi):
			init_frame_offset[0] = init_frame_offset[0] % (2 * np.pi) 
		elif init_frame_offset[0] < 0:
			init_frame_offset[0] = (2 * np.pi) - abs(init_frame_offset[0])
		#	correct y 
		if init_frame_offset[1] >= (2 * np.pi):
			init_frame_offset[1] = init_frame_offset[1] % (2 * np.pi) 
		elif init_frame_offset[1] < 0:
			init_frame_offset[1] = (2 * np.pi) - abs(init_frame_offset[1])
		#	correct z
		if init_frame_offset[2] >= (2 * np.pi):
			init_frame_offset[2] = init_frame_offset[2] % (2 * np.pi) 
		elif init_frame_offset[2] < 0:
			init_frame_offset[2] = (2 * np.pi) - abs(init_frame_offset[2])

		print('\n	Time since last light curve: 		{}h {}m {}s\n'.format(
			int(np.floor(sep_sec / 60 / 60)),
			int(np.floor(sep_sec / 60) - np.floor(sep_sec / 60 / 60) * 60),
			sep_sec - (np.floor(sep_sec / 60) * 60)
		))
	
		print('		Start Date:			{:4d}/{:02d}/{:02d} {:02d}:{:02d}:{:04.1f}'.format(
			*jd2date(start_jday, start_jfrac)
		))
		
		print('		New Start Attitude:		{}'.format(np.round(init_frame_offset, 2)))
	
	###############################################################################################
	#	generate light curve generation vector data
	###############################################################################################

	#	blank observation and illumination vectors
	observation_vector = np.array([1.0, 0.0, 0.0])
	illumination_vector = np.array([1.0, 0.0, 0.0])
	
	#	generate orb2inertial velocity vector
	orb2inertial = np.array([0.0, 0.0, -orbital_mot])
	#	this is the initial offset between the orbital and body frames
	init_frame_offset = np.copy(step_attit)
	#	this is the angular velocity of the body frame wrt the orbital frame
	init_frame_vel = np.copy(step_attit_mot)

	##################################################
	#	true data
#	print('\nTrue Sun Orb Data = \n', sun_orb_units)
#	print('\nTrue Site Orb Data = \n', site_orb_units)

	#	vector generation function
	vector_data_body, vector_data_orb, vector_data_inertial, attitude_out = gen_vectors(
		path_data, lc_time_step, len_light_curve,
		sat_orb_data, sun_orb_data, earth_orb_data, site_orb_data,
		orb2inertial, init_frame_offset, init_frame_vel,
		observation_vector, illumination_vector
	)

	##################################################
	#	simple data
#	print('\nSimple Sun Orb Data = \n', sun_orb_data_fs[:, 1:])
#	print('\nSimple Site Orb Data = \n', site_orb_data_fs[:, 1:])

	#	vector generation function
	vector_data_body_simple, vector_data_orb_simple, vector_data_inertial, attitude_out = gen_vectors(
		path_data, lc_time_step, len_light_curve,
		sat_orb_data, sun_orb_data_simple, earth_orb_data, site_orb_data_simple,
		orb2inertial, init_frame_offset, init_frame_vel,
		observation_vector, illumination_vector
	)

	###############################################################################################
	#	parse vector data
	###############################################################################################

	##################################################
	#	true data
	#	orb data
	sun_orb_data = vector_data_orb[:, 1:4]
	site_orb_data = vector_data_orb[:, 4:7]
	earth_orb_data = vector_data_orb[:, 7:10]
	sat_orb_vel = vector_data_orb[:, 10:13]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	illum_vec_orb_data = vector_data_orb[:, 22:25]
	obs_vec_orb_data = vector_data_orb[:, 25:28]

	#	body data
	sun_body_data = vector_data_body[:, 1:4]
	site_body_data = vector_data_body[:, 4:7]
	earth_body_data = vector_data_body[:, 7:10]
	sat_body_vel = vector_data_body[:, 10:13]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	illum_vec_body_data = vector_data_body[:, 22:25]
	obs_vec_body_data = vector_data_body[:, 25:28]

	#	set end attitude
	end_attitude = attitude_out[-1]

	##################################################
	#	simple data
	#	orb data
	sun_orb_data_simple = vector_data_orb_simple[:, 1:4]
	site_orb_data_simple = vector_data_orb_simple[:, 4:7]
	earth_orb_data_simple = vector_data_orb_simple[:, 7:10]
	sat_orb_vel_simple = vector_data_orb_simple[:, 10:13]
	sat_orb_pos_simple = np.zeros([len(sat_orb_vel_simple), 3])
	sat_orb_data_simple = np.hstack([sat_orb_pos_simple, sat_orb_vel_simple])
	illum_vec_orb_data_simple = vector_data_orb_simple[:, 22:25]
	obs_vec_orb_data_simple = vector_data_orb_simple[:, 25:28]

	#	body data
	sun_body_data_simple = vector_data_body_simple[:, 1:4]
	site_body_data_simple = vector_data_body_simple[:, 4:7]
	earth_body_data_simple = vector_data_body_simple[:, 7:10]
	sat_body_vel_simple = vector_data_body_simple[:, 10:13]
	sat_body_pos_simple = np.zeros([len(sat_body_vel_simple), 3])
	sat_body_data_simple = np.hstack([sat_body_pos_simple, sat_body_vel_simple])
	illum_vec_body_data_simple = vector_data_body_simple[:, 22:25]
	obs_vec_body_data_simple = vector_data_body_simple[:, 25:28]

	###############################################################################################
	#	body data residuals

	#	generate unit vectors
	sun_body_units = np.zeros(sun_body_data.shape)
	site_body_units = np.zeros(site_body_data.shape)

	for tempi, (sun_body_vec, site_body_vec) in enumerate(zip(sun_body_data, site_body_data)):
		sun_body_units[tempi, :] = np.divide(sun_body_vec, np.linalg.norm(sun_body_vec))
		site_body_units[tempi, :] = np.divide(site_body_vec, np.linalg.norm(site_body_vec))

	#	body residuals
	sun_body_resid = np.subtract(sun_body_units, sun_body_data_simple)
	site_body_resid = np.subtract(site_body_units, site_body_data_simple)

	fig_body_resid, ax_body_resid = plt.subplots()
	ax_body_resid.set_title('Body Residuals')

	#	sun
	ax_body_resid.plot(sun_body_resid[:, 0], label='Sun x-residuals', color='r')
	ax_body_resid.plot(sun_body_resid[:, 1], label='Sun y-residuals', color='r')
	ax_body_resid.plot(sun_body_resid[:, 2], label='Sun z-residuals', color='r')

	#	site
	ax_body_resid.plot(site_body_resid[:, 0], label='Site x-residuals', color='g')
	ax_body_resid.plot(site_body_resid[:, 1], label='Site y-residuals', color='g')
	ax_body_resid.plot(site_body_resid[:, 2], label='Site z-residuals', color='g')

	ax_body_resid.legend()

	###############################################################################################
	#	polar data generation
	###############################################################################################

	##################################################
	#	true data
	sun_polar_data, site_polar_data, phase_angles, jd_time_data = extract_fun(
		jd_time_data, lc_time_step, time_step,
		sun_orb_data, earth_orb_data, site_orb_data, sat_orb_data
	)
	print('\n	True Sun Polar Data = \n', np.degrees(sun_polar_data))
	print('\n	True Site Polar Data = \n', np.degrees(site_polar_data))

	print('\n	Avg. Sun Azm = ', np.degrees(sun_polar_data[:, 0].mean()))
	print('	Avg. Sun Inc = ', np.degrees(sun_polar_data[:, 1].mean()))

	##################################################
	#	simple data
	sun_polar_data_simple, site_polar_data_simple, phase_angles, jd_time_data = extract_fun(
		jd_time_data, lc_time_step, time_step,
		sun_orb_data_simple, earth_orb_data_simple, site_orb_data_simple, sat_orb_data_simple
	)
	print('\n	Simple Sun Polar Data = \n', np.degrees(sun_polar_data_simple))
	print('\n	Simple Site Polar Data = \n', np.degrees(site_polar_data_simple))

	###############################################################################################
	#	polar data residuals

	#	polar residuals
	sun_polar_resid = np.degrees(np.subtract(sun_polar_data, sun_polar_data_simple))
	site_polar_resid = np.degrees(np.subtract(site_polar_data, site_polar_data_simple))

	fig_polar_resid, ax_polar_resid = plt.subplots()
	ax_polar_resid.set_title('Polar Residuals')

	#	sun
	ax_polar_resid.plot(sun_polar_resid[:, 0], label='Sun azm-residuals', color='r')
	ax_polar_resid.plot(sun_polar_resid[:, 1], label='Sun inc-residuals', color='r')

	#	site
	ax_polar_resid.plot(site_polar_resid[:, 0], label='Site azm-residuals', color='g')
	ax_polar_resid.plot(site_polar_resid[:, 1], label='Site inc-residuals', color='g')

	ax_polar_resid.legend()

	###############################################################################################
	#	light curve generation
	###############################################################################################

	#	standard distance flag. sets distance to 1000 km when true
	std_dist_flag = True

	##################################################
	#	true data
	#	initilise lightcurve output
	final_light_curve = np.zeros((len_light_curve, 3))
	final_light_curve[:, 1] = light_curve_sim_time

	#	run light curve generation function
	light_curve = glim_slc(sun_body_data, site_body_data, final_light_curve, \
			start_jdate, body_vectors, body_areas, bdrf_data, std_dist_flag)

	#	extract light curve data
	light_curve_data = np.array(light_curve[:, -1]).astype(np.float)

	##################################################
	#	simple data
	#	initilise lightcurve output
	final_light_curve_simple = np.zeros((len_light_curve, 3))
	final_light_curve_simple[:, 1] = light_curve_sim_time

#	print('\nSimple Sun Body Data = \n', sun_body_data_simple)
#	print('\nSimple Site Body Data = \n', site_body_data_simple)

	#	run light curve generation function
	light_curve_simple = glim_slc(sun_body_data_simple, site_body_data_simple, \
		final_light_curve_simple, start_jdate, body_vectors, body_areas, bdrf_data, std_dist_flag)

	#	extract light curve data
	light_curve_data_simple = np.array(light_curve_simple[:, -1]).astype(np.float)

	##################################################
	#	simple illumination data
	#	initilise lightcurve output
	final_light_curve_simple_illum = np.zeros((len_light_curve, 3))
	final_light_curve_simple_illum[:, 1] = light_curve_sim_time

	#	run light curve generation function
	light_curve_simple_illum = glim_slc(sun_body_data_simple, site_body_data, \
		final_light_curve_simple_illum, start_jdate, body_vectors, body_areas, bdrf_data, std_dist_flag)

	#	extract light curve data
	light_curve_data_simple_illum = np.array(light_curve_simple_illum[:, -1]).astype(np.float)

	##################################################
	#	simple observation data
	#	initilise lightcurve output
	final_light_curve_simple_obs = np.zeros((len_light_curve, 3))
	final_light_curve_simple_obs[:, 1] = light_curve_sim_time

	#	run light curve generation function
	light_curve_simple_obs = glim_slc(sun_body_data, site_body_data_simple, \
		final_light_curve_simple_obs, start_jdate, body_vectors, body_areas, bdrf_data, std_dist_flag)

	#	extract light curve data
	light_curve_data_simple_obs = np.array(light_curve_simple_obs[:, -1]).astype(np.float)

	###############################################################################################
	#	lightcurve residuals data
	lc_resid = np.subtract(light_curve_data, light_curve_data_simple)
	lc_resid_simp_obs = np.subtract(light_curve_data, light_curve_data_simple_obs)
	lc_resid_simp_illum = np.subtract(light_curve_data, light_curve_data_simple_illum)

	fig_lc_resid, ax_lc_resid = plt.subplots()
	ax_lc_resid.set_title('Lightcurve Residuals')

	ax_lc_resid.plot(lc_resid, label='LC-residuals')
	ax_lc_resid.plot(lc_resid_simp_obs, label='Simple Obs Data / Real Illum Data')
	ax_lc_resid.plot(lc_resid_simp_illum, label='Simple Illum Data / Real Obs Data')

	ax_lc_resid.legend()

	###############################################################################################
	#	graphical output
	###############################################################################################

	###############################################################################################
	#	plot lightcurve
	fig_lc, ax_lc = plt.subplots()

	#	true light curve
	ax_lc.plot(light_curve[:, 1], light_curve_data, label='True')

	#	using simple obs data
	ax_lc.plot(
		light_curve_simple_obs[:, 1],
		light_curve_data_simple_obs,
		label='Simple Obs Data / Real Illum Data'
	)

	#	using simple illumination data
	ax_lc.plot(
		light_curve_simple_illum[:, 1],
		light_curve_data_simple_illum,
		label='Simple Illum Data / Real Obs Data'
	)

	#	full simple data
	ax_lc.plot(light_curve_simple[:, 1], light_curve_data_simple, label='Simple Data')

	#	finalise plot
	ax_lc.legend()
	ax_lc.grid()

	###################################################################################
	#	polar plot

	#	initialise polar figure
	fig1 = plt.figure(figsize=(10, 6))
	ax1 = plt.subplot(111, projection='polar')
	ax1.set_title('Polar plot')
	
	ax1.set_theta_zero_location("N")
	ax1.set_theta_direction(-1)
	
	ax1.set_rticks(np.arange(0, 100, 10))
	ax1.set_rlim([0, 100])

	marker_sun = [len(sun_polar_data) - 1]
	marker_site = [len(site_polar_data) - 1]
	
	#	plot sun positions
	ax1.plot(
		sun_polar_data[:, 0],
		np.degrees(sun_polar_data[:, 1]),
		linewidth=0.5, color='tab:blue', markevery=marker_sun, marker='>'
	)
	
	#	plot site positions
	ax1.plot(
		site_polar_data[:, 0],
		np.degrees(site_polar_data[:, 1]),
		linewidth=0.5, color='tab:orange', markevery=marker_site, marker='>'
	)


	#	plot sun positions
	ax1.plot(
		sun_polar_data_simple[:, 0],
		np.degrees(sun_polar_data_simple[:, 1]),
		linewidth=0.5, color='tab:green', markevery=marker_sun, marker='x'
	)
	
	#	plot site positions
	ax1.plot(
		site_polar_data_simple[:, 0],
		np.degrees(site_polar_data_simple[:, 1]),
		linewidth=0.5, color='tab:red', markevery=marker_site, marker='>'
	)



	plt.show()
	sys.exit()

	###############################################################################################
	#	RMS calculations
	###############################################################################################

	rms_simple = rms_calc(light_curve_data, light_curve_data_simple)
	rms_simple_obs = rms_calc(light_curve_data, light_curve_data_simple_obs)
	rms_simple_illum = rms_calc(light_curve_data, light_curve_data_simple_illum)

	print('\n	RMS_simple = ', rms_simple)
	print('\n	RMS_simple_obs = ', rms_simple_obs)
	print('\n	RMS_simple_illum = ', rms_simple_illum)

	###############################################################################################
	#	glimma rot vec outputs
	###############################################################################################
	
	#########################################################################################
	# set directories
	ident_string = 'x142y57-illumvec'
	case_desig = 'lightcurve_data_cylinder_1.0'
	
	#	file paths
	root_path = os.path.join(os.environ['HOME'], 'codebase')
	archive_path = 'data_archive/paper_data'
	path_data = os.path.join(root_path, archive_path)
	input_path = os.path.join(path_data, 'lightcurve_dir', case_desig)
	
	#	file paths
	input_path = os.path.join(path_data, 'lightcurve_dir', case_desig)
	
	#	light curve read path
	lightcurve_in = os.path.join(
		path_data,
		'lightcurve_dir',
		case_desig,
		'total_lightcurves',
		ident_string
	)
	
	lightcurve_dir = os.listdir(lightcurve_in)
	lightcurve_files = []
	
	#	sort files
	for file in lightcurve_dir:
		lcid = int(file.split('_', maxsplit=1)[0])
		lightcurve_files.append([lcid, file])
	
	lightcurve_files = sorted(lightcurve_files, key=lambda x: x[0])
	n_files = len(lightcurve_dir)
	small = 1e-8
	
	for index in range(n_files):
	
		#	get file name
		lightcurve_file = lightcurve_files[index][1]
		print('\n	Working from lightcurve file: ', lightcurve_file)
	
		#	parse file name data
		lc_id, attit_case_name, azm_case_name, elv_case_name, geom_str, reflection_case = \
			lightcurve_file[0:-11].split('_')
	
		#	observation vector azimuth / elevation
		azimuth = int(azm_case_name[3:])
		elevation = int(elv_case_name[3:])
	
		#	generate observation vector
		temp_obs_vec = rot2(x_vec, np.radians(elevation))
		output_obs_vec = rot1(temp_obs_vec, np.radians(azimuth))
		
		#	correct small values
		for elei, ele in enumerate(output_obs_vec):
			if -small < ele < small:
				output_obs_vec[elei] = 0.0
	
		#	generate name / label
		case_name = azm_case_name + '_' + elv_case_name
		case_label = case_name
	
		#	load light curve data
		lightcurve = np.genfromtxt(os.path.join(lightcurve_in, lightcurve_file), delimiter=',')
	
		#	extract plot data
		x_plot = lightcurve[:, 1]
		mag_plot = lightcurve[:, 2]
	
		ax_lc.plot(x_plot, mag_plot, linewidth=0.5, linestyle='--', label=case_label)
	
	
	




	plt.show()
	sys.exit()


















	###############################################################################################
	#	finalise outputs
	###############################################################################################

	##################################################
	#	truncate data and recalculate properties

#	#	truncate data to max light curve length
#	if max_index > 1:
#		jd_time_data = jd_time_data[0:max_index]
#		light_curve = light_curve[0:max_index]

	#	reset light curve length
	len_light_curve = len(light_curve)
	
	#	recalculate duration
	duration_sec = lc_time_step * (len_light_curve - 1)

	#	recalculate stop times
	stop_jdate = jd_time_data[-1]
	stop_jday, stop_jfrac = jday_jfrac(stop_jdate)

	#	set prev_stop_jdate
	prev_stop_jdate = np.copy(stop_jdate)

	##################################################
	#	generate output array

	#	stack light curve array with ID
	id_array = np.full([len_light_curve, 1], light_curve_id)
	light_curve = np.hstack([id_array, light_curve])

	#	add to output array
	lightcurve_output_array[start_index:end_index, :] = light_curve

	#	itterate indices
	start_index += index_step
	end_index += index_step

	print('\n		End Date:			{:4d}/{:02d}/{:02d} {:02d}:{:02d}:{:04.1f}'.format(
		*jd2date(stop_jday, stop_jfrac)
	))
	
	print('		End Attitude:			{}'.format(np.round(end_attitude, 2)))

	print('\n		Light curve duration:		{}m {}s'.format(
		np.floor(duration_sec / 60),
		duration_sec - (np.floor(duration_sec / 60) * 60)
	))

###################################################################################################
#	save file

#	calculate sim time
sim_end = time.time()
sim_time = np.round((sim_end - sim_start) / 60, 1)

print('\nSimulation Finished.')
print('\n\n		- Time taken = {} mins.'.format(sim_time))

#	generate lightcurve file name
lightcurve_output_file = '_'.join([site_str, norad_id, geom_str, attitude_str]) + '.txt'

print('\nSaving final file as: \n{}'.format(lightcurve_output_file))

#	set lightcurve output directory
lightcurve_file_path = os.path.join(
	path_data,
	'lightcurve_dir',
	'lightcurve_files',
	lightcurve_output_file
)

#	save light curve file
np.savetxt(lightcurve_file_path, lightcurve_output_array, delimiter=',')

plt.show()
print('\nEND')
