#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

from tools.functions.functions_transform import rot_mat, rot1, rot2, rot3, axis2mat, mat2angles

import numpy as np
import matplotlib.pyplot as plt
import scipy
from scipy import optimize

from stl import mesh

###################################################################################################
#	initilisation
###################################################################################################

##################################################################################################
#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

##################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front stop script initilised.')

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#########################################################################################
#   generate case designation and indentifier string
#########################################################################################

##################################################
#	simulation parameters

#	time step in seconds
time_step = 0.1

#	length of data record [number of points]
len_record = 1500

#	add zero
len_record += 1

##################################################
#	case parameters

#	reflection case
reflection_case = 'diffuse'

#	geometry case
geom_str = 'cylinder'

#	illumination vector polar angles
illum_azm_deg = 229.1
illum_inc_deg = 77.6

#	illumination vector polar angles
obs_azm_deg = 0.0
obs_inc_deg = 0.0

#	initial attitude
attitude_x = 140.0
attitude_y = 36.0
attitude_z = 0.0

#	initial attitude
initial_attitude = np.array([
	np.radians(attitude_x),
	np.radians(attitude_y),
	np.radians(attitude_z)
])

##################################################
#	attitude motion

#	rotational period
rot_period = 10.0

#	1 rpm (radians per second)
rot_angle_deg = 360.0 / rot_period

#	define rotation axis in body frame
rot_axis = np.array([1.0, 0.0, 0.0])

#	convert to rotation matrix
attitude_matrix = axis2mat(rot_angle_deg, rot_axis)

#	extract euler angles
angles1 = np.array(mat2angles(attitude_matrix)[0:3])
angles2 = np.array(mat2angles(attitude_matrix)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

initial_attitude_motion = angles1

##################################################
#	case indentifier
case_name = 'obsvec-y'
case_name = 'attit-y'

#	generate case vector
case_vector_deg = [illum_azm_deg, illum_inc_deg, obs_azm_deg, obs_inc_deg]

#	generate ident string
ident_string = 'illum_{}-{}_obs_{}-{}'.format(*case_vector_deg)

#	generate case designation
case_desig = geom_str + '_' + ident_string + '_' + case_name

print('ident_string = ', ident_string)
print('case desig = ', case_desig)

#########################################################################################
#	read and sort lightcurve files
#########################################################################################

#	light curve read path
lightcurve_in = os.path.join(
	path_data,
	'lightcurve_dir',
	case_desig,
	'total_lightcurves',
	ident_string
)

lightcurve_dir = os.listdir(lightcurve_in)

if case_name == 'obsvec-y':
	lightcurve_files_90 = []
	lightcurve_files_270 = []

elif case_name == 'attit-y':
	lightcurve_files = []

#	sort files
for file in lightcurve_dir:

	#	parse file name properties

	#	light curve id
	lcid = int(file.split('_')[0])
	geom_str = file.split('_')[1]
	illum_str = file.split('_')[2]
	illum_angles = file.split('_')[3]
	illum_azm, illum_inc = np.array(illum_angles.split('-')).astype(float)

	obs_str = file.split('_')[4]
	obs_angles = file.split('_')[5]
	obs_azm, obs_inc = np.array(obs_angles.split('-')).astype(float)

	attit = file.split('_')[6]
	attit_angles = file.split('_')[7]

	#	remove file extension
	ext_len = len(attit_angles.split('.')[-1]) + 1
	attit_angles = attit_angles[0:len(attit_angles) - ext_len]

	attit_x, attit_y, attit_z = np.array(attit_angles.split('-')).astype(float)

	if case_name == 'obsvec-y':
	
		if obs_azm == 90.0:
			lightcurve_files_90.append([lcid, obs_azm, obs_inc, file])
		elif obs_azm == 270.0:
			lightcurve_files_270.append([lcid, obs_azm, obs_inc, file])

	elif case_name == 'attit-y':
		lightcurve_files.append([lcid, attit_x, attit_y, file])

if case_name == 'obsvec-y':
	#	sort files by inclination
	lightcurve_files_90 = sorted(lightcurve_files_90, key=lambda x: x[2])
	lightcurve_files_270 = sorted(lightcurve_files_270, key=lambda x: x[2])
	
	lightcurve_files = lightcurve_files_90[::-1] + lightcurve_files_270

elif case_name == 'attit-y':
	lightcurve_files = sorted(lightcurve_files, key=lambda x: x[2])

#########################################################################################
#	generate vectors
#########################################################################################

############################################
#	calculate rotation vector
attitude_matrix = rot_mat(initial_attitude)
x_vec = np.array([1.0, 0.0, 0.0])
rot_vec = np.dot(x_vec, attitude_matrix)
rot_vec[0] = -rot_vec[0]

#	convert to polar coordinates
rot_vec_polar = np.degrees([initial_attitude[0], initial_attitude[1]])

############################################
#	generate illumination vector

#	apply inclination
illum_vec_temp = rot2(x_vec, illum_inc_deg)
#	apply azimuth
illum_vec = rot1(illum_vec_temp, illum_azm_deg)
illum_vec[0] = -illum_vec[0]

#########################################################################################
#	initialise plot

#	Plot lightcurves
fig_dimx = 8
fig_dimy = 6

n_files = len(lightcurve_dir)
small = 1e-8
phase_plot = []

#	initialise
fig, axes = plt.subplots(1, n_files, figsize=(50, 5), sharey=True)

#' fig, ax = plt.subplots()

#########################################################################################
#	plot loop

for index in range(n_files):

	#	get file name
	lightcurve_file = lightcurve_files[index][3]
	print('\n	Working from lightcurve file: ', lightcurve_file)

	#	parse file name data
	lc_id, geom_str, illum_str, illum_angles, obs_str, obs_angles, attit_str, attit_angles \
		= lightcurve_file[0:-11].split('_')

	rot_vec_x, rot_vec_y, rot_vec_z = np.array(attit_angles.split('-')).astype(float)

	#	observation vector azimuth / elevation
	illum_azm, illum_inc = np.array(illum_angles.split('-')).astype(float)
	obs_azm, obs_inc = np.array(obs_angles.split('-')).astype(float)

	#	generate observation vector
	temp_obs_vec = rot2(x_vec, np.radians(obs_inc))
	output_obs_vec = rot1(temp_obs_vec, np.radians(obs_azm))
	
	#	correct small values
	for elei, ele in enumerate(output_obs_vec):
		if -small < ele < small:
			output_obs_vec[elei] = 0.0

	if case_name == 'obsvec-y':
		case_label = obs_inc
	elif case_name == 'attit-y':
		case_label = rot_vec_y

	#	load light curve data
	lightcurve = np.genfromtxt(os.path.join(lightcurve_in, lightcurve_file), delimiter=',')

	#	extract plot data
	x_plot = lightcurve[:, 8]
	mag_plot = lightcurve[:, 9]

#	ax.plot(x_plot, mag_plot, label=case_label)
	axes[index].plot(x_plot, mag_plot, label=case_label)
	axes[index].title.set_text(case_label)



plt.show()
sys.exit()

