###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	requests
import requests

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	kep2rv, rv2kep, teme2itrf

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			TLE Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

norad_id = 44278

###############################################################################################
#	generate 'last5' of TLEs

print('\n					Acquiring TLEs...')

tle_file = '44278_TLE_Starlink67.txt'

#	assemble TLE path
tle_path = os.path.join(path_data, 'input_dir', 'tle_files')
tle_path = '/home/ldjb/Documents/NORSS/spacetrack_files/TLEs'

#	if exists, load
if os.path.isfile(os.path.join(tle_path, tle_file)):
	print('\n						Reading existing TLE file.')

	#	load file
	with open(os.path.join(tle_path, tle_file), 'r') as file:
		tle_data = file.readlines()

else:
	print('\n						Requesting TLE file...')

	#	request TLEs

	###################################################################################################
	#	generate spacetrack request
	
	print('Requesting TLE file from Spacetrack.com...')
	
#	year_start = int(start_date[0])
#	mon_start = int(start_date[1])
#	day_start = int(start_date[2])
#	
#	year_stop = int(stop_date[0])
#	mon_stop = int(stop_date[1])
#	day_stop = int(stop_date[2])
#	
#	#	create date string
#	req_datestart = '{:4}-{:02}-{:02}'.format(year_start, mon_start, day_start)
#	req_datestop = '{:4}-{:02}-{:02}'.format(year_stop, mon_stop, day_stop)
#	req_datestr = '--'.join((req_datestart, req_datestop))
	
	#	spacetrack login information
	login_url = 'https://www.space-track.org/ajaxauth/login'
	username = 'ldjblacketer@gmail.com'
	password = 'Chromodynamics1992'
	
	payload = {
		'identity': username,
		'password': password
	}
	
#	base_url = 'https://www.space-track.org/basicspacedata/query'
#	request_class = 'class/tle'
#	object_specifier = 'NORAD_CAT_ID/{}'.format(norad_id)
#	daterange_specifier = 'EPOCH/{}'.format(req_datestr)
#	format_specifier = 'format/{}'.format('3le')
	
	# spacetrack request url
#	req_url = '/'.join([
#		base_url,
#		request_class,
#		object_specifier,
#		daterange_specifier,
#		format_specifier,
#	])

	req_url = 'https://www.space-track.org/basicspacedata/query/class/tle_latest/ORDINAL/1--5/NORAD_CAT_ID/45429/orderby/TLE_LINE1 ASC/format/3le'
	
	#	create request session
	with requests.Session() as session:
		post = session.post(login_url, data=payload)
		output = session.get(req_url).text.split('\n')
	
	print('TLE Request Complete, pausing for 10s...')
	time.sleep(2)

	tle_data = output
	
	###################################################################################################
	#	save file
	with open(os.path.join(tle_path, tle_file), 'w') as file:
		for line in tle_data:
			file.write(line)

	print('							TLE file saved.')


###########################################################################################
#	calculating orbital height

print('\n					Calculating Height Data and On-Station Time...')

RE = 6371
small = 1e-10

#	initialise height data
height_data = []
time_data = []

difference_data = []
tle_datevec_array = []
difference_data_datevec_array = []

station_flag = False
time_found_flag = False
run_diff = 0
run_diff_data = []

spread_max = 1.5
diff_threshold = 1.5

kep_ele_data = []

tle_dot = []

for itt, tle_line in enumerate(range(int(len(tle_data) / 3))):

	#	generate tle line index
	line_index = (tle_line * 3)

	tle_line0 = tle_data[line_index]
	tle_line1 = tle_data[line_index + 1]
	tle_line2 = tle_data[line_index + 2]

	#	generate SGP4 satellite object
	satellite = twoline2rv(tle_line1, tle_line2, wgs72)

	#	extract TLE epoch
	tle_epoch = satellite.jdsatepoch
	time_data.append(tle_epoch)

	#	convert to state vector
	pos, vel = satellite.propagate(*jd2date(*jday_jfrac(tle_epoch)))



	#	convert to keplerian elements
	kep_ele = rv2kep(pos, vel)
	kep_ele_data.append(kep_ele)


	pos_teme = pos
	vel_teme = vel

	
	#	EOP data
	eop_dir = 'eop_files'
	eop_file = 'eop_parsed.csv'
	dat_file = 'dat_parsed.csv'
	
	#	assemble eop path
	eop_path = os.path.join(path_data, eop_dir)

	sim_range = [tle_epoch, tle_epoch + 1]
	time_step = 1
	
#	#	generate interpolated eop_data
#	eop_log_level = 0
#	eop_data, dat_data = eop_interpolate(
#		eop_path,
#		eop_file,
#		dat_file,
#		sim_range,
#		time_step,
#		eop_log_level
#	)

#	print(pos_teme)
#
#	pos_itrf, vel_itrf = teme2itrf(pos_teme)
#
#	sys.exit()





	#	calculate height
	sat_height = np.linalg.norm(pos) - RE

	#	convert jd to date
	year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(tle_epoch))]

	#	turn into strings
	datestr = str('-'.join(
		['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
	))

	timestr = ':'.join(
		['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
	)

	datevec  = ' '.join([datestr, timestr])


	#	convert to datetime object
	date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
	tle_datevec_array.append(date_time_obj)

	#	add to outputs
	height_data.append(sat_height)

	if itt > 0:
		dx = abs(sat_height - prev_height)
		dt = abs(tle_epoch - prev_jd)


		dxdt = dx / dt
		tle_dot.append(dxdt)

#		print(dx)
#		print(dt)
#		print(dxdt)
#
#		sys.exit()

	#	set previous data
	prev_height = sat_height
	prev_jd = tle_epoch

#	#	if not first line
#	if tle_line > 0:
#
#		#	continue condition for TLEs with same time
#		if (tle_epoch - prev_jd) < small:
#			continue
#
#		#	calculate difference
#		diff = prev_height - sat_height
#		difference_data.append(diff)
#		difference_data_datevec_array.append(date_time_obj)
#
#		#	continue condition once on station time is found
#		if time_found_flag is True:
#			continue
#
#		#	difference is below threshold
#		if abs(diff) <= diff_threshold:
#
#			#	if there is no ongoing check, initialise one
#			if station_flag is False:
#
#				#	set on station tle index to this line
#				on_station_index = line_index
#
#				#	itterate number of running differences
#				run_diff += 1
#				run_diff_data.append(sat_height)
#
#				#	set on station flag to true
#				station_flag = True
#
#			#	there is an ongoing run
#			elif station_flag is True:
#
#				#	itterate number of running differences
#				run_diff += 1
#				run_diff_data.append(sat_height)
#
#				#	calculate spread
#				spread_test_data = np.array(run_diff_data)
#				spread = spread_test_data.max() - spread_test_data.min()
#
#		#	difference is above threshold
#		elif abs(diff) > diff_threshold:
#
#			#	if there is an ongoing run, end and reset
#			if station_flag is True:
#
#				#	reset
#				run_diff = 0
#				run_diff_data = []
#				station_flag = False
#
#		#	end condition
#		if run_diff >= 5:
#
#			#	calculate spread
#			spread_test_data = np.array(run_diff_data)
#			spread = spread_test_data.max() - spread_test_data.min()
#
#			if spread > spread_max:
#
#				#	reset
#				run_diff = 0
#				run_diff_data = []
#				station_flag = False
#
#			else:
#
#				time_found_flag = True


tle_number = np.arange(1, 6, 1)

kep_ele_data = np.array(kep_ele_data)
print(kep_ele_data)


plt.figure()
plt.plot(tle_datevec_array, np.degrees(kep_ele_data[:, 0]), label='omega', marker='X')
plt.plot(tle_datevec_array, np.degrees(kep_ele_data[:, 1]), label='arg_p', marker='X')
plt.plot(tle_datevec_array, np.degrees(kep_ele_data[:, 2]), label='nu', marker='X')
plt.plot(tle_datevec_array, np.degrees(kep_ele_data[:, 3]), label='i', marker='X')
plt.plot(tle_datevec_array, np.degrees(kep_ele_data[:, 4]), label='e', marker='X')
#plt.plot(tle_datevec_array, kep_ele_data[:, 5], label='a')

plt.legend()




fig, ax = plt.subplots(2, 1, sharex=True)


# plt.figure()
ax[0].plot(tle_datevec_array, height_data, marker='+')
ax[0].set_ylabel('Orbital Height [km]')

# plt.figure()
ax[1].plot(tle_datevec_array[1:], tle_dot, marker='+')
ax[1].set_ylabel('Rate of Change of Height [km / day]')
ax[1].set_ylim([0, 200])


plt.show()
