###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Light curve rendering in the orbital frame
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime

#	numpy
import numpy as np
from numpy import random

#	numpy-stl
import stl
from stl import mesh

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation
from tools.functions.functions_transform import site_vec, rv2kep, gen_vectors2, \
	rot1, rot2, rot3, rot_mat, axis2mat, mat2angles

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d
from descartes import PolygonPatch

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
os.system('clear')
print('#####################################################################################')
print('			Brightness Validation')
print('#####################################################################################')

###################################################################################################
#	file paths

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

geom_path = '/home/ldjb/codebase/data_archive/GLIMMA_data/geometry_dir/geom_dir/rocket_body/'
geom_file = 'rocket_body_blunt.stl'

#	output
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)

###################################################################################################
#	define MMT observation site


#	random attitude
attit_x = (random.randint(100) / 100) * (np.pi * 2)
attit_y = ((random.randint(100) - 50) / 100) * (np.pi / 2)
attit_z = (random.randint(100) / 100) * (np.pi * 2)

init_attit = np.array([attit_x, attit_y, attit_z])
#init_attit = np.array([ 3.26725636, -0.73827427,  5.52920307])
init_attit = np.array([0.69115038, 0.43982297, 3.20442451])
# init_attit = np.array([0.0, 0.0, 0.0])
print('\n	Using State:	', init_attit)


rot_vec_body_unit = np.array([0.0, 0.0, 1.0])
rot_ang_per_sec = 0.252

rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

###############################################
#	import mesh
orb_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([0.0, 0.0, 90.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
orb_mesh.rotate_using_matrix(offset_rot_mat)

#	apply attitude
attit_rot_mat = rot_mat(init_attit)

#	transform mesh into body frame
orb_mesh.rotate_using_matrix(attit_rot_mat)

rot_vec_orb_unit = np.dot(np.array([0.0, 0.0, 1.0]), attit_rot_mat)
origin = np.array([0.0, 0.0, 0.0])
rot_vec_plot = np.vstack([origin, rot_vec_orb_unit * 10])


orb_vectors = orb_mesh.vectors
orb_normals = orb_mesh.normals

###############################################
#	import body mesh
body_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([0.0, 0.0, 90.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

m_brdf = 0.27
omega_brdf = 0.2
d_brdf = 0.3





alpha = 0.5
facecol_str = 'tab:blue'


###################################################################################################
#	define MMT observation site

site_str = 'maui'
site_latgd = np.radians(20.708)
site_lon = np.radians(156.257)
site_alt = 3075.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	initilise figure
 
#	set use of LaTeX font
plt.rc('text', usetex=True)


# figure3d = plt.figure(
# 	num=None,
# 	figsize=(6, 6),
# 	facecolor='w',
# 	edgecolor='k',
# 	dpi=300
# )
# axes3d = mplot3d.Axes3D(figure3d)
# 
# for face_index, orb_vector in enumerate(orb_vectors):
# 
# 	if face_index < 112:
# 		facecol_str = 'tab:red'
# 	elif face_index >= 144:
# 		facecol_str = 'tab:green'
# 
# 	else:
# 		facecol_str = 'tab:blue'
# 
# 	#	parse input vectors and calculate facet normal
# 	vect12 = orb_vector[1] - orb_vector[0]
# 	vect23 = orb_vector[2] - orb_vector[1]
# 	normal = np.cross(vect12, vect23)
# 	normal_unit = normal / np.linalg.norm(normal)
# 
# 	#	Add final vert to vector array
# 	orb_vector = np.vstack([orb_vector, orb_vector[0, :]])
# 
# 	#	extract x, y and z values
# 	xs = orb_vector[:,0]
# 	ys = orb_vector[:,1]
# 	zs = orb_vector[:,2]
# 	#	generate vertices list
# 	verts = [list(zip(xs, ys, zs))]
# 	#	add facet to plot
# 	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
# 	pc.set_alpha(alpha)
# 	pc.set_edgecolor([0.5, 0.5, 0.5])
# 	pc.set_facecolor(facecol_str)
# 	axes3d.add_collection3d(pc)
# 
# 
# axes3d.plot(
# 	rot_vec_plot[:, 0],
# 	rot_vec_plot[:, 1],
# 	rot_vec_plot[:, 2],
# 	marker='^',
# 	color='k',
# 	markevery=[-1]
# )
# 
# # a = Arrow3D(rot_vec_plot[:, 0], rot_vec_plot[:, 1], rot_vec_plot[:, 2],
# # 	mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
# # )
# # axes3d.add_artist(a)
# axes3d.text(rot_vec_plot[1, 0] + 0.5, rot_vec_plot[1, 1] + 0.5, rot_vec_plot[1, 2],
# 	r'$\mathrm{Rot.\ Vec}$', fontsize=14, fontweight='bold'
# )
# 
# axes3d.set_xlim([-7, 7])
# axes3d.set_ylim([-7, 7])
# axes3d.set_zlim([-7, 7])
# 
# axes3d.set_xlabel(r'$\mathrm{x\ Orbital\ Axis\ (m)}$')
# axes3d.set_ylabel(r'$\mathrm{y\ Orbital\ Axis\ (m)}$')
# axes3d.set_zlabel(r'$\mathrm{z\ Orbital\ Axis\ (m)}$')
# 
# #	figure name
# fig_name = 'lc_example_geometry.pdf'
# 
# out_path = os.path.join(fig_path, fig_name)
# 
# if os.path.exists(out_path):
# 	print('\nPath: ', out_path, ' exists.')
# 	ans = input('\nReplace File? [y/n] ')
# 
# 	if ans == 'y':
# 		saveflag = True
# 	else:
# 		saveflag = False
# 
# else:
# 	saveflag = True
# 
# if saveflag == True:
# 	figure3d.savefig(out_path, bbox_inches="tight", dpi=300)
# 	print('\n#######################################')
# 	print('Figure saved: ', fig_name)
# else:
# 	print('\nNot saving: ', fig_name)
# 
# 
# plt.show()
# sys.exit()

###############################################################################################
#	generate brdf

#	generate brdf
brdf_data = np.zeros([nfaces, 5])

for i in range(nfaces):
	
	if i < 112:
		omega_brdf = 0.6
		d_brdf = 0.8
	elif i >= 144:
		omega_brdf = 0.47
		d_brdf = 0.8
	else:
		omega_brdf = 0.25
		d_brdf = 0.3

	#	Refractive Index
	n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))
	
	#	Specular ratio
	s_brdf = 1.0 - d_brdf

	brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]


###############################################################################################
#	time inputs
###############################################################################################

#	start
start_jd = date2jd(2003, 3, 24, 14, 4, 0)
jdate_start = np.add(*start_jd)

stop_jd = date2jd(2003, 3, 24, 14, 6, 30)
time_step = 1.0

norad_id = 12069

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###############################################################################################
#	acquire TLE
###############################################################################################

#	set time
tle_start = time.time()

print('\n	[{}] Acquiring TLEs...'.format(datetime.datetime.now()))

#	TLE request dates, add day either side to increase tle find chance
tle_start_date = jd2date(*jday_jfrac(lc_start_jd - 1))
tle_stop_date = jd2date(*jday_jfrac(lc_stop_jd + 1)) 

#	generate TLE file name
tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)

#	assemble TLE path
tle_path = os.path.join(path_data, 'input_dir', 'tle_files')

#	if exists, load
if os.path.isfile(os.path.join(tle_path, tle_file)):

	print('\n		Reading existing TLE file.')

	#	load file
	with open(os.path.join(tle_path, tle_file), 'r') as file:
		tle_data = file.readlines()

else:
	print('\n		Requesting TLE file...')

	#	request TLEs
	tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)

	#	save file
	with open(os.path.join(tle_path, tle_file), 'w') as file:
		for line in tle_data:
			file.write(line)

#	grab first TLE pair
tle_line1 = tle_data[1]
tle_line2 = tle_data[2]

#	mean mot rad/sec
orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
orbital_mot = (2 * np.pi) / orbital_period

###############################################################################################
#	display TLE acquisition time
tle_end = time.time()
tle_time = np.round((tle_end - tle_start), 1)

print('\n		TLE Load Finished.')
print('\n			- Time taken = {} s.'.format(tle_time))

###############################################################################################
#	select time data to use
###############################################################################################

sim_range = lc_sim_range
len_record = lc_len_record
start_date = lc_start_date
stop_date = lc_stop_date
duration_sec = lc_duration_sec

###############################################################################################
#	generate eop data arrays
###############################################################################################

#	set time
eop_start = time.time()

print('\n	[{}] Generating EOP data array...'.format(datetime.datetime.now()))

#	EOP data
eop_dir = 'eop_files'
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	assemble eop path
eop_path = os.path.join(path_data, eop_dir)

#	generate interpolated eop_data
eop_log_level = 0
eop_data, dat_data = eop_interpolate(
	eop_path,
	eop_file,
	dat_file,
	sim_range,
	time_step,
	eop_log_level
)

###################################################################################################
#	display time
eop_end = time.time()
eop_time = np.round((eop_end - eop_start), 1)

print('\n		EOP data generated.')
print('\n			- Time taken = {} s.'.format(eop_time))

###################################################################################################
#	generate time objects
###################################################################################################

#	start dat value
dat = dat_data[0, -1]
	
#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate start time object
start_time_object = get_times(*start_date, ut_offset, dat)
	
#	end dat value
dat = dat_data[-1, -1]
	
#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate stop time object
stop_time_object = get_times(*stop_date, ut_offset, dat)

###################################################################################################
#	generate rotation matrices
###################################################################################################

#	set time
rot_mat_start = time.time()
print('\n	[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))

#	tt time
jdaytt_start = start_time_object.jday_tt
jfractt_start = start_time_object.jfrac_tt
jdtt_start = jdaytt_start + jfractt_start

#	ut1 time
jdayut1_start = start_time_object.jday_ut1
jfracut1_start = start_time_object.jfrac_ut1
jdut1_start = jdayut1_start + jfracut1_start

#	generate rotation matrices
rot_mat_log_level = 0
pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
	jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
)

###################################################################################################
#	display time
rot_mat_end = time.time()
rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)

print('\n		Rotation Matrices Generated.')
print('\n			- Time taken = {} s.'.format(rot_mat_time))

###################################################################################################
#	MODULE:	Horizons Positions
###################################################################################################

#	set time
horz_start = time.time()
print('\n	[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))

#	set horizons logging level
horz_logging_level = 0

sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
	path_data, horz_logging_level,
	start_time_object, stop_time_object,
	len_record, time_step,
	eop_path
)

#	find indices of light curve data
timecol = sun_icrf[:, 0]
lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]

###################################################################################################
#	display time
horz_end = time.time()
horz_time = np.round((horz_end - horz_start), 1)

print('\n		Horizons data generated.')
print('\n			- Time taken = {} s.'.format(horz_time))

###################################################################################################
#	MODULE:	Object Position
###################################################################################################

#	set time
sat_start = time.time()
print('\n	[{}]	Generating object position data...'.format(datetime.datetime.now()))

satpos_logging_level = 0

sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
	path_data, satpos_logging_level, len_record, time_step,
	start_time_object, stop_time_object,
	norad_id,
	eop_data, dat_data,
	pm_array, pn_array, st_array, c2t_array
)

###################################################################################################
#	display time
sat_end = time.time()
sat_time = np.round((sat_end - sat_start), 1)

print('\n		Object position data generated.')
print('\n			- Time taken = {} s.'.format(sat_time))

###################################################################################################
#	MODULE:	Transformation
###################################################################################################

#	set time
trans_start = time.time()
print('\n	[{}]	Performing transformations...'.format(datetime.datetime.now()))

identification_str = 'mmt-render'

trans_logging_level = 0
save_flag = False

sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
	= module_transformation(
		path_data, trans_logging_level, save_flag,
		start_time_object, stop_time_object,
		identification_str, site_str,
		len_record, time_step,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array,
		sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
	)

###################################################################################################
#	display time
trans_end = time.time()
trans_time = np.round((trans_end - trans_start), 1)

print('\n		Transformations complete.')
print('\n			- Time taken = {} s.'.format(trans_time))


















###################################################################################################
#	perform light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	extract orbital positions of custom input vectors
illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_body[:, 31:34]

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]

#	orb data
sun_orb_data = vector_data_orb[:, 28:31]
site_orb_data = vector_data_orb[:, 31:34]
earth_orb_data = vector_data_orb[:, 19:22]
sat_orb_vel = vector_data_orb[:, 22:25]
sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_orb[:, 31:34]

###############################################################################################
#	generate solar panel no shadowing light curve

#	generate sim time
sim_time = np.linspace(0, duration_sec, len_record)

#	initilise lightcurve output
light_curve_array = np.zeros((lc_len_record, 4))
light_curve_array[:, 1] = sim_time

#	run light curve generation function
light_curve_array = glim_slc_new(
	sun_body_data, site_body_data, light_curve_array,
	jdate_start, body_vectors, body_areas, brdf_data
)

lc_app = light_curve_array[:, 2]
lc_std = light_curve_array[:, 3]


print(lc_std)


fig, ax = plt.subplots(figsize=(6, 7), dpi=300)
ax.plot(lc_std, marker='o', markersize=3)
#ax.plot(lc_app, marker='o', markersize=3)

ax.set_xlim([0, 150])
ax.set_ylim([5, 0])
#ax.invert_yaxis()

ax.set_xlabel(r'$\mathrm{Time\ (s)}$')
ax.set_ylabel(r'$\mathrm{Standard\ Magnitude\ (1000\ m)}$')

ax.set_xticks([0, 50, 100, 150])
ax.set_yticks([5.0, 4.5, 4.0, 3.5, 3.0, 2.5, 2.0, 1.5, 1.0, 0.5, 0.0])

plt.pause(0.01)


#	figure name
fig_name = 'lc_example_comp.eps'

out_path = os.path.join(fig_path, fig_name)

if os.path.exists(out_path):
	print('\nPath: ', out_path, ' exists.')
	ans = input('\nReplace File? [y/n] ')

	if ans == 'y':
		saveflag = True
	else:
		saveflag = False

else:
	saveflag = True

if saveflag == True:
	fig.savefig(out_path, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)
else:
	print('\nNot saving: ', fig_name)


plt.show()
sys.exit()





plt.show()
sys.exit()






plt.show()
sys.exit()
