###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	geometry file plotter
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors
from matplotlib.patches import FancyArrowPatch

from tools.functions.functions_transform import site_vec, rot_mat, gen_vectors_render, rot1, rot2, rot3

###################################################################################################
#	define plotable arrow class
###################################################################################################

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################

#############################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Conferences',
	'IOC_2019',
	'technical_paper',
	'acta_figures',
	'Final'
)

fig_name = 'orbital_reference_frame.eps'
out_path = os.path.join(fig_path, fig_name)

#	figure save flag
savefile_flag = True
savefile_flag = False

###################################################################################################
#	geometry file
###################################################################################################

geom_dir = 'geometry_dir'
geom_dir = 'geometry_dir/geom_dir/' + 'CZ-2D'
geom_file = 'cz-2d_testing_double_pointed.stl'

#	brdf
top_faces_index = np.arange(0, 36)
side_faces_index = np.arange(36, 108)
bot_faces_index = np.arange(108, 144)


###################################################################################################
#	figure parameters
###################################################################################################

#	graphical output
inch2cm = 2.54

figure = plt.figure(num=None, figsize=(9.0/inch2cm, 9.0/inch2cm),
 	dpi=300, facecolor='w', edgecolor='k'
)

plt.rc('text', usetex=True)
plt.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
ax = plt.axes(projection='3d')

#	face plot parameters
alpha = 0.5
facecol = [0.1, 0.1, 0.1]
edgecol = [0.5, 0.5, 0.5]

font_sizex = 1
font_sizey = 1
font_pad = 0.5
marker_size = 4
marker_edge = 1

axis_font_size = 7

#	coordinate origin
origin = [0, 0, 0]

###################################################################################################
#	plot inertial axes

inertial_axis_len = 12

xaxis = [inertial_axis_len, 0, 0]
yaxis = [0, inertial_axis_len, 0]
zaxis = [0, 0, inertial_axis_len]

#	x axis
xaxis_plot = np.vstack([origin, xaxis])
c = Arrow3D(xaxis_plot[:, 0], xaxis_plot[:, 1], xaxis_plot[:, 2] - 5,
	mutation_scale=5,
	lw=0.25,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(xaxis_plot[1, 0], xaxis_plot[1, 1] - 5, xaxis_plot[1, 2] - 1 - 7,
	'$X_{axis} (Earth\ Zenith\ Vector)$',
	fontsize=axis_font_size,
	fontweight='bold'
)

#	y axis
yaxis_plot = np.vstack([origin, yaxis])
c = Arrow3D(yaxis_plot[:, 0], yaxis_plot[:, 1], yaxis_plot[:, 2] - 5,
	mutation_scale=5,
	lw=0.25,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(yaxis_plot[1, 0] + 3, yaxis_plot[1, 1] - 5, yaxis_plot[1, 2] - 7,
	'$Y_{axis}\ (Velocity\ Vector)$',
	fontsize=axis_font_size,
	fontweight='bold'
)

#	z axis
zaxis_plot = np.vstack([origin, zaxis])
c = Arrow3D(zaxis_plot[:, 0], zaxis_plot[:, 1], zaxis_plot[:, 2] - 5,
	mutation_scale=5,
	lw=0.25,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(zaxis_plot[1, 0], zaxis_plot[1, 1], zaxis_plot[1, 2] - 5,
	'$Z_{axis} (Orbit\ Plane\ Normal)$',
	fontsize=axis_font_size,
	fontweight='bold'
)

###################################################################################################
#	plot vectors
###################################################################################################

#	apply attitude
# attitude = np.array([np.pi / 4, 0.0, 0.0])
attitude = np.array([0.0, np.radians(26), np.radians(26)])
attitude_matrix = rot_mat(attitude)

###################################################################################################
#	observation and illumination vectors

#	vector length
vec_length = 12

#	Illumination Vector
illum_vec = [2, -1, 1]
illum_vec = np.divide(illum_vec, np.linalg.norm(illum_vec))
illum_vec = np.multiply(illum_vec, vec_length)
illum_vec_plot = np.vstack([origin, illum_vec])

#	Observation Vector
obs_vec = [1, 0, 0]
obs_vec = np.divide(obs_vec, np.linalg.norm(obs_vec))
obs_vec = np.multiply(obs_vec, vec_length)
obs_vec_plot = np.vstack([origin, obs_vec])

c = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2] - 5,
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(illum_vec_plot[1, 0] + 2, illum_vec_plot[1, 1], illum_vec_plot[1, 2] + 1 - 4,
	r'$\boldsymbol{R} (Rotation\ Vector)$',
	fontsize=7,
	fontweight='bold'
)

# c = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2] - 5,
# 	mutation_scale=5,
# 	lw=1,
# 	arrowstyle="-|>",
# 	color="k"
# )
# 
# ax.add_artist(c)
# ax.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2] + 1 - 5,
# 	r'$\boldsymbol{V}$',
# 	fontsize=5,
# 	fontweight='bold'
# )

###################################################################################################
#	define circle data

#	define radius
radius = 10

#	generate circle data
n = 360
circle_vals = []

for i in range(0, n + 1):
	x = np.cos(2 * np.pi / n * i) * radius
	y = np.sin(2 * np.pi / n * i) * radius
	z = 0

	outvec = np.array([x, y, z])
	circle_vals.append(outvec)

circle_vals = np.array(circle_vals)

###################################################################################################
#	generate PHI circle
circle1 = []

Lvec = np.array([0.0, illum_vec[1], illum_vec[2]])

#	angle of Lyz to yaxis
arccos_of = np.divide(
	np.dot(Lvec, yaxis),
	np.linalg.norm(Lvec) * np.linalg.norm(yaxis)
)

#	generate circle 1
angle1 = np.arccos(arccos_of)

#	angle of V to L
arccos_of = np.divide(
	np.dot(obs_vec, illum_vec),
	np.linalg.norm(obs_vec) * np.linalg.norm(illum_vec)
)

#	generate circle 1
angle2 = np.arccos(arccos_of)

#	rotate for N-Hvec circle
for vec in circle_vals:
	outvec = rot1(vec, -angle1)
	circle1.append(outvec)

circle1 = np.array(circle1)
circle1_lower = 0
circle1_upper = int(np.degrees(angle2))
circle1_plot = circle1[circle1_lower: circle1_upper]

#	plot circle
plt.plot(
	circle1_plot[:, 0],
	circle1_plot[:, 1],
	circle1_plot[:, 2] - 5,
	linewidth=0.5,
	color='k'
)

ax.text(
	circle1_plot[int(len(circle1_plot) / 2), 0] + 1.5,
	circle1_plot[int(len(circle1_plot) / 2), 1],
	circle1_plot[int(len(circle1_plot) / 2), 2] - 5,
	r'$\phi$',
	fontsize=12,
	fontweight='bold'
)

###################################################################################################
#	generate PSI circle
circle2 = []

circle2_offset = circle1[circle1_upper, 0]

dim1 = circle1[circle1_upper, 1]
dim2 = circle1[circle1_upper, 2]
dim3 = np.sqrt(dim1**2 + dim2**2)

#	define radius
radius = dim3

#	generate circle data
n = 360
circle_vals = []

for i in range(0, n + 1):
	x = np.cos(2 * np.pi / n * i) * radius
	y = np.sin(2 * np.pi / n * i) * radius
	z = 0

	outvec = np.array([x, y, z])
	circle_vals.append(outvec)

circle_vals = np.array(circle_vals)

angle1 = np.pi / 2
angle2 = np.pi / 2

Lvec = np.array([0.0, illum_vec[1], illum_vec[2]])

#	angle of Lyz to yaxis
arccos_of = np.divide(
	np.dot(Lvec, zaxis),
	np.linalg.norm(Lvec) * np.linalg.norm(zaxis)
)

#	generate circle 1
angle3 = np.arccos(arccos_of)

#	rotate for N-Hvec circle
for vec in circle_vals:
	outvec = rot2(vec, angle1)

	outvec = rot1(outvec, -angle3)

	circle2.append(outvec)

circle2 = np.array(circle2)
circle2_lower = 0
circle2_upper = int(np.degrees(angle3))

# circle2[:, 0] += circle2_offset
circle2_plot = circle2[circle2_lower: circle2_upper]

#	plot circle
plt.plot(
	circle2_plot[:, 0],
	circle2_plot[:, 1],
	circle2_plot[:, 2] - 5,
	linewidth=0.5,
	color='k'
)

ax.text(
	circle2_plot[int(len(circle2_plot) / 2), 0],
	circle2_plot[int(len(circle2_plot) / 2), 1],
	circle2_plot[int(len(circle2_plot) / 2), 2] + 1 - 5,
	r'$\psi$',
	fontsize=12,
	fontweight='bold'
)

#	plot dashed line 1
circle2_origin = np.array([0.0, circle2_plot[0, 1], circle2_plot[0, 2]])
dot_vec = np.array([circle2_offset, circle2_plot[0, 1], circle2_plot[0, 2]])
dot_vec_plot = np.vstack([circle2_origin, dot_vec])
plt.plot(dot_vec_plot[:, 0], dot_vec_plot[:, 1], dot_vec_plot[:, 2] - 5, color='k', lw=0.25, ls='--')

#	plot dashed line 2
circle2_origin = np.array([0.0, 0.0, 0.0])
dot_vec = np.array([0.0, circle2_plot[0, 1], circle2_plot[0, 2]])
dot_vec_plot = np.vstack([circle2_origin, dot_vec])
plt.plot(dot_vec_plot[:, 0], dot_vec_plot[:, 1], dot_vec_plot[:, 2] - 5, color='k', lw=0.25, ls='--')

###################################################################################################
#	plot geometry
###################################################################################################

# #	geometry file directory
# geom_dir = 'geometry_dir/geom_dir/cube'
# 
# #	set geometry input
# geom_file = '01_cube_cube.stl'

#	load mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
object_mesh.rotate_using_matrix(attitude_matrix)



scale_factor = 0.25

object_mesh.x *= scale_factor
object_mesh.y *= scale_factor
object_mesh.z *= scale_factor
object_mesh.z -= 2.5

vectors = 2 * object_mesh.vectors
normals = 2 * object_mesh.normals
nfaces = len(normals)

alpha = 0.75

#	facet loop
for i, (vector, normal) in enumerate(zip(vectors, normals)):

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_alpha(alpha)
	pc.set_edgecolor(edgecol)

	pc.set_linewidth(0.002)

#	pc.set_facecolor(facecol)

	if i in top_faces_index:
		pc.set_facecolor('tab:red')
	elif i in side_faces_index:
		pc.set_facecolor('tab:green')
	elif i in bot_faces_index:
		pc.set_facecolor('tab:blue')

	ax.add_collection3d(pc)

# ###################################################################################################
# #	plot body axes
# ###################################################################################################
# 
# #	generate body fixed axes
# x_vec = [1.0, 0.0, 0.0]
# y_vec = [0.0, 1.0, 0.0]
# z_vec = [0.0, 0.0, 1.0]
# 
# x_body = 5 * np.dot(x_vec, attitude_matrix)
# y_body = 5 * np.dot(x_vec, attitude_matrix)
# z_body = 5 * np.dot(x_vec, attitude_matrix)
# 
# #	x axis
# xaxis_plot = np.vstack([origin, x_body])
# c = Arrow3D(xaxis_plot[:, 0], xaxis_plot[:, 1], xaxis_plot[:, 2] - 5,
# 	mutation_scale=5,
# 	lw=0.25,
# 	arrowstyle="-|>",
# 	color="k"
# )
# 
# ax.add_artist(c)
# ax.text(xaxis_plot[1, 0], xaxis_plot[1, 1] - 5, xaxis_plot[1, 2] - 1 - 7,
# 	'$X_{axis}$',
# 	fontsize=4,
# 	fontweight='bold'
# )
# 
# #	y axis
# yaxis_plot = np.vstack([origin, y_body])
# c = Arrow3D(yaxis_plot[:, 0], yaxis_plot[:, 1], yaxis_plot[:, 2] - 5,
# 	mutation_scale=5,
# 	lw=0.25,
# 	arrowstyle="-|>",
# 	color="k"
# )
# 
# ax.add_artist(c)
# ax.text(yaxis_plot[1, 0] + 3, yaxis_plot[1, 1] - 5, yaxis_plot[1, 2] - 7,
# 	'$Y_{axis}$',
# 	fontsize=4,
# 	fontweight='bold'
# )
# 
# #	z axis
# zaxis_plot = np.vstack([origin, z_body])
# c = Arrow3D(zaxis_plot[:, 0], zaxis_plot[:, 1], zaxis_plot[:, 2] - 5,
# 	mutation_scale=5,
# 	lw=0.25,
# 	arrowstyle="-|>",
# 	color="k"
# )
# 
# ax.add_artist(c)
# ax.text(zaxis_plot[1, 0], zaxis_plot[1, 1], zaxis_plot[1, 2] - 5,
# 	'$Z_{axis}$',
# 	fontsize=4,
# 	fontweight='bold'
# )

###################################################################################################
#	finalise figure
###################################################################################################

ax.set_xlim([-6.5, 6.5])
ax.set_ylim([-6.5, 6.5])
ax.set_zlim([-6.5, 6.5])


ax.view_init(22, 50)
plt.axis('off')

if savefile_flag is True:
	plt.savefig(out_path, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
