###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front end
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

import tqdm

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	GLIMMA 
from GLIMMA_archive.function_GLIMMA import glimma

#	light curve generation
from tools.functions.glim_slc import glim_slc_new
from tools.functions.functions_transform import rot1, rot2, rot3, gen_vectors2, rot_mat

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import gridspec
from matplotlib.patches import FancyArrowPatch

#	parrellisation
from multiprocessing import Pool

#	progress bar
import progressbar

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front end script initilised.\n')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	inputs
###################################################################################################

plt.rc('text', usetex=True)
plt.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]

##################################################
#	figure output

#	set flag
savefile_flag = False
savefile_flag = True

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Journals',
	'ACTA_Astronautica',
	'acta_figures',
	'Final'
)

##################################################
#	constants

#	earth radius
earthR = 6371

#	astronomical unit
AU = 1.5e11

#	earth gravitational parameter
earthMu = 398600

#	inch to cm conversion
inch2cm = 2.54

##################################################
#	simulation time parameters

#	time step in seconds
time_step = 0.1

#	length of data reccord [number of points]
len_record = (60 * 4 * 10) + 1

#	generate time array
sim_range = np.linspace(0, len_record - 1, len_record)

##################################################
#	illumination / observation vector postions

#	sun constant position
sun_psi = np.radians(90.0)
sun_phi = np.radians(60.0)

# #	site initial position
# site_psi = np.radians(270.0)
# site_phi = np.radians(10.0)

#	site phi angle range
site_phi_start = 20.0
site_phi_end = -20.0
phi_range = np.linspace(site_phi_start, site_phi_end, len_record)

#	constant attitude
rot_vec_psi = np.radians(50.0)
#rot_vec_psi = np.radians(0.0)

rot_vec_phi = np.radians(0.0)
rot_vec_z = np.radians(50.0)

##################################################
#	object parameters

#	object position
sat_alt = 2000

#	object position
sat_a = sat_alt + earthR

#	actual
orb_period = 2 * np.pi * np.sqrt(sat_a**3 / earthMu)
orbital_mot =  (2 * np.pi) / (2 * np.pi * np.sqrt(sat_a**3 / earthMu))

#	using zero orbital motion
orbital_mot = 0.0

##################################################
#	load geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'cube'
geom_str = '01_cube_cube'

geom_file = geom_str + '.stl'

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

#	set attitude
init_attit = np.array([rot_vec_psi, rot_vec_phi, rot_vec_z])

##################################################
#	brdf parameters

d_brdf = 1.0
m_brdf = 0.25
omega_brdf = 0.5

#	generate brdf

#	Refractive Index
n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))

#	Specular ratio
s_brdf = 1.0 - d_brdf

#	generate brdf
brdf_data = np.zeros([nfaces, 5])

for i in range(nfaces):
	brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]

###################################################################################################
#	initialise light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

#	zero rotation vector motion
rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

##################################################
#	set constant vectors

#	generate sat orb data
sat_orb = np.zeros([len(sim_range), 7])
sat_orb[:, 0] = sim_range
sat_orb[:, 4] = 1

#	generate earth orb data
earth_orb = np.zeros([len(sim_range), 4])
earth_orb[:, 0] = sim_range
earth_orb[:, 1] = sat_a

##################################################
#	set variable vectors

#	generate sun orb data
sun_orb = np.zeros([len(sim_range), 4])

#	generate sun_orb_vector
sun_orb_vec = np.array([AU, 0.0, 0.0])
sun_orb_vec = rot2(sun_orb_vec, sun_phi)
sun_orb_vec = rot1(sun_orb_vec, sun_psi)
for i in range(len(sun_orb)):
	sun_orb[i, 0] = sim_range[i]
	sun_orb[i, 1:] = sun_orb_vec

#	generate site orb data
site_orb = np.zeros([len(sim_range), 4])


#	generate site_orb_vector
site_orb_vec_start = np.array([sat_alt, 0.0, 0.0])

#	generate site vec array
for i in range(len(sim_range)):

	#	set time
	site_orb[i, 0] = sim_range[i]

	#	generate site vector
	phi_ang = phi_range[i]

	if phi_ang >= 0:
		phi_ang_used = phi_ang
		psi_ang = 90.0

	elif phi_ang <= 0:
		phi_ang_used = abs(phi_ang)
		psi_ang = 270.0

	site_orb_vec = rot2(site_orb_vec_start, np.radians(phi_ang_used))
	site_orb_vec = rot1(site_orb_vec, np.radians(psi_ang))

	#	add to output
	site_orb[i, 1:] = site_orb_vec

##################################################
#	calculate phase angle start
phase_arccos_of = np.divide(
	np.dot(sun_orb[0, 1:], site_orb[0, 1:]),
	np.linalg.norm(sun_orb[0, 1:]) * np.linalg.norm(site_orb[0, 1:])
)

if phase_arccos_of > 1.0:
	if phase_arccos_of < 1.0 + 1e-6:
		phase_arccos_of = 1.0
elif phase_arccos_of < -1.0:
	if phase_arccos_of > -1.0 - 1e-6:
		phase_arccos_of = -1.0

phase_ang_start = np.arccos(phase_arccos_of)

##################################################
#	calculate phase angle end
phase_arccos_of = np.divide(
	np.dot(sun_orb[-1, 1:], site_orb[-1, 1:]),
	np.linalg.norm(sun_orb[-1, 1:]) * np.linalg.norm(site_orb[-1, 1:])
)

if phase_arccos_of > 1.0:
	if phase_arccos_of < 1.0 + 1e-6:
		phase_arccos_of = 1.0
elif phase_arccos_of < -1.0:
	if phase_arccos_of > -1.0 - 1e-6:
		phase_arccos_of = -1.0

phase_ang_end = np.arccos(phase_arccos_of)


###############################################################################################
#	generate light curve with no attitude motion
###############################################################################################

print('Performing No-Rotation case...')

#	set attitude motion
rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
rot_ang_per_sec = 0.0

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]

#	orb data
x_geom_orb = vector_data_orb[:, 10:13]
y_geom_orb = vector_data_orb[:, 13:16]
z_geom_orb = vector_data_orb[:, 16:19]

sun_orb_data = vector_data_orb[:, 28:31]
site_orb_data = vector_data_orb[:, 31:34]
earth_orb_data = vector_data_orb[:, 19:22]
sat_orb_vel = vector_data_orb[:, 22:25]
sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_orb[:, 31:34]

###############################################################################################
#	generate light curve

#	initilise lightcurve output
light_curve_array = np.zeros((len_record, 4))
jdate_start = 0

#	run light curve generation function
light_curve_array = glim_slc_new(
	sun_body_data, site_body_data, light_curve_array,
	jdate_start, body_vectors, body_areas, brdf_data
)

lc_seconds_no_rot = light_curve_array[:, 1]
lc_app_no_rot = light_curve_array[:, 2]
lc_std_no_rot = light_curve_array[:, 3]

###############################################################################################
#	regenerate light curve with attitude motion - x
###############################################################################################

print('Performing x-Rotation case...')

#	set attitude motion
rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
rot_ang_per_sec = 2 * np.pi / 60

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]

##################################################
#	generate light curve

#	initilise lightcurve output
light_curve_array = np.zeros((len_record, 4))
jdate_start = 0

#	run light curve generation function
light_curve_array = glim_slc_new(
	sun_body_data, site_body_data, light_curve_array,
	jdate_start, body_vectors, body_areas, brdf_data
)

lc_seconds_x_rot = light_curve_array[:, 1]
lc_app_x_rot = light_curve_array[:, 2]
lc_std_x_rot = light_curve_array[:, 3]

###############################################################################################
#	regenerate light curve with attitude motion - y
###############################################################################################

print('Performing y-Rotation case...')

#	set attitude motion
rot_vec_body_unit = np.array([0.0, 1.0, 0.0])
rot_ang_per_sec = 2 * np.pi / 60

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]

##################################################
#	generate light curve

#	initilise lightcurve output
light_curve_array = np.zeros((len_record, 4))
jdate_start = 0

#	run light curve generation function
light_curve_array = glim_slc_new(
	sun_body_data, site_body_data, light_curve_array,
	jdate_start, body_vectors, body_areas, brdf_data
)

lc_seconds_y_rot = light_curve_array[:, 1]
lc_app_y_rot = light_curve_array[:, 2]
lc_std_y_rot = light_curve_array[:, 3]

###############################################################################################
#	regenerate light curve with attitude motion - z
###############################################################################################

print('Performing z-Rotation case...')

#	set attitude motion
rot_vec_body_unit = np.array([0.0, 0.0, 1.0])
rot_ang_per_sec = 2 * np.pi / 60

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]

##################################################
#	generate light curve

#	initilise lightcurve output
light_curve_array = np.zeros((len_record, 4))
jdate_start = 0

#	run light curve generation function
light_curve_array = glim_slc_new(
	sun_body_data, site_body_data, light_curve_array,
	jdate_start, body_vectors, body_areas, brdf_data
)

lc_seconds_z_rot = light_curve_array[:, 1]
lc_app_z_rot = light_curve_array[:, 2]
lc_std_z_rot = light_curve_array[:, 3]

###################################################################################################
#	light curve figure generation
###################################################################################################

print('\n#####################################################################################')
print('Generating Final Plots...')

text_size = 8
tick_text_size = 6
legend_text_size = 6

fig = plt.figure(figsize=(6.0/inch2cm, 3.0/inch2cm), dpi=300)

ax = fig.add_subplot()

#	no rotation
ax.plot(sim_range, lc_app_no_rot, label=r'$no-rot$', linewidth=0.8, linestyle='-', color='k')

#	x rotation
ax.plot(sim_range, lc_app_x_rot, label=r'$x-rot$', linewidth=0.8, linestyle='--', color='tab:red')

#	y rotation
ax.plot(sim_range, lc_app_y_rot, label=r'$y-rot$', linewidth=0.8, linestyle=(0, (1, 1)), color='tab:green')

#	z rotation
ax.plot(sim_range, lc_app_z_rot, label=r'$z-rot$', linewidth=0.8, linestyle='-.', color='tab:blue')

print('\nPhase Angle (Start) = ', np.degrees(phase_ang_start))
print('\nPhase Angle (End) = ', np.degrees(phase_ang_end))

print('\nRMSE from baseline:')
print('x RMSE = ', np.round(rms_calc(lc_app_x_rot, lc_app_no_rot), 5))
print('y RMSE = ', np.round(rms_calc(lc_app_y_rot, lc_app_no_rot), 5))
print('z RMSE = ', np.round(rms_calc(lc_app_z_rot, lc_app_no_rot), 5))

print('\nRMSE between rotations:')
print('z-y RMSE = ', np.round(rms_calc(lc_app_z_rot, lc_app_y_rot), 5))
print('y-x RMSE = ', np.round(rms_calc(lc_app_y_rot, lc_app_x_rot), 5))
print('z-x RMSE = ', np.round(rms_calc(lc_app_z_rot, lc_app_x_rot), 5))


min1 = np.min(lc_app_no_rot)
max1 = np.max(lc_app_no_rot)

min2 = np.min(lc_app_x_rot)
max2 = np.max(lc_app_x_rot)

min3 = np.min(lc_app_y_rot)
max3 = np.max(lc_app_y_rot)

min4 = np.min(lc_app_z_rot)
max4 = np.max(lc_app_z_rot)

minall = np.min([min1, min2, min3, min4])
maxall = np.max([max1, max2, max3, max4])

#	finalise plot
ax.set_ylim([minall - 0.5, maxall + 0.5])
ax.invert_yaxis()
ax.set_xlabel(r'$Time\ (s)$', size=text_size)
ax.set_ylabel(r'$App\ Mag$', size=text_size)
ax.legend(frameon=False, prop={'size':legend_text_size}, loc='lower left')

ax.tick_params(axis='x', labelsize=tick_text_size)
ax.tick_params(axis='y', labelsize=tick_text_size)

####################################################
#	save figure

if rot_vec_psi == 0:
	fig_name = 'lightcurve_example_case1' + '.eps'

if rot_vec_psi == np.radians(50):
	fig_name = 'lightcurve_example_case2' + '.eps'

out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	plt.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

###################################################################################################
#	3d visualisation figure generation
###################################################################################################

axisLen = 1500

plot_offset = earthR * 0.2

origin = np.array([-plot_offset, 0.0, 0.0])

fig = plt.figure(dpi=300, figsize=(9.0/inch2cm, 4.0/inch2cm))
gs = gridspec.GridSpec(1, 2, width_ratios=[1, 1])

body_vec_scale = 1500

title_text_size = 10

###################################################################################################
###################################################################################################
#	initial conditions
###################################################################################################
###################################################################################################

ax2 = fig.add_subplot(gs[0], projection='3d')
ax2.set_title('$t = 0$', size=title_text_size)

##################################################
#	plot sun pos
sun_plot_vec = 1.6 * np.linalg.norm(site_orb[0, 1:]) * \
	np.divide(sun_orb[0, 1:], np.linalg.norm(sun_orb[0, 1:]))

sun_plot_vec[0] -= plot_offset
sun_orb_plot = np.vstack([origin, sun_plot_vec])

c = Arrow3D(sun_orb_plot[:, 0], sun_orb_plot[:, 1], sun_orb_plot[:, 2],
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="tab:red"
)

ax2.add_artist(c)
ax2.text(sun_orb_plot[1, 0], sun_orb_plot[1, 1], sun_orb_plot[1, 2],
	r'$\mathbf{L}$',
	fontsize=5,
)

##################################################
#	plot earth pos
earth_plot_vec = np.linalg.norm(sun_plot_vec) * \
		np.divide(earth_orb[0, 1:], np.linalg.norm(earth_orb[0, 1:]))

earth_plot_vec[0] -= plot_offset
earth_orb_plot = np.vstack([origin, earth_plot_vec])

c = Arrow3D(earth_orb_plot[:, 0], earth_orb_plot[:, 1], earth_orb_plot[:, 2],
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color='k',
)

ax2.add_artist(c)
ax2.text(
	1.1 * earth_orb_plot[1, 0],
	earth_orb_plot[1, 1] - (0.6 * earth_orb_plot[1, 0]),
	earth_orb_plot[1, 2],
	r'$X\ (Earth)$',
	fontsize=5,
)

##################################################
#	plot velocity
vel_plot_vec = np.linalg.norm(sun_plot_vec) * np.array([0.0, 1.0, 0.0])

vel_plot_vec[0] -= plot_offset
vel_orb_plot = np.vstack([origin, vel_plot_vec])

c = Arrow3D(vel_orb_plot[:, 0], vel_orb_plot[:, 1], vel_orb_plot[:, 2],
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color='k',
)

ax2.add_artist(c)
ax2.text(1.4 * vel_orb_plot[1, 0], 0.25 * vel_orb_plot[1, 1], vel_orb_plot[1, 2],
	r'$Y\ (Velocity)$',
	fontsize=5,
)

##################################################
#	plot initial site pos
site_plot_vec = site_orb[0, 1:]
site_plot_vec[0] -= plot_offset
site_orb_plot = np.vstack([origin, site_plot_vec])

c = Arrow3D(site_orb_plot[:, 0], site_orb_plot[:, 1], site_orb_plot[:, 2],
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="tab:blue"
)

ax2.add_artist(c)
ax2.text(site_orb_plot[1, 0], site_orb_plot[1, 1], site_orb_plot[1, 2],
	r'$\mathbf{V}$',
	fontsize=5,
)

##################################################
#	plot body x vector
body_x_vec = body_vec_scale * x_geom_orb[0, :]
body_x_vec[0] -= plot_offset
body_x_plot = np.vstack([origin, body_x_vec])

c = Arrow3D(body_x_plot[:, 0], body_x_plot[:, 1], body_x_plot[:, 2],
	mutation_scale=2.5,
	lw=0.5,
	arrowstyle="-|>",
	color="k"
)

ax2.add_artist(c)
ax2.text(0.2 * body_x_plot[1, 0], 1.1 * body_x_plot[1, 1], body_x_plot[1, 2],
	r'$\mathbf{x_b}$',
	fontsize=5,
)

##################################################
#	plot body y vector
body_y_vec = body_vec_scale * y_geom_orb[0, :]
body_y_vec[0] -= plot_offset
body_y_plot = np.vstack([origin, body_y_vec])

c = Arrow3D(body_y_plot[:, 0], body_y_plot[:, 1], body_y_plot[:, 2],
	mutation_scale=2.5,
	lw=0.5,
	arrowstyle="-|>",
	color="k"
)

ax2.add_artist(c)
ax2.text(body_y_plot[1, 0], body_y_plot[1, 1], body_y_plot[1, 2],
	r'$\mathbf{y_b}$',
	fontsize=5,
)

##################################################
#	plot body z vector
body_z_vec = body_vec_scale * z_geom_orb[0, :]
body_z_vec[0] -= plot_offset
body_z_plot = np.vstack([origin, body_z_vec])

c = Arrow3D(body_z_plot[:, 0], body_z_plot[:, 1], body_z_plot[:, 2],
	mutation_scale=2.5,
	lw=0.5,
	arrowstyle="-|>",
	color="k"
)

ax2.add_artist(c)
ax2.text(body_z_plot[1, 0], body_z_plot[1, 1], body_z_plot[1, 2],
	r'$\mathbf{z_b}$',
	fontsize=5,
)

##################################################
#	plot earth circle

theta = np.linspace(0, 2 * np.pi, 201)
x = np.subtract(np.add((earthR * np.cos(theta)), sat_a), plot_offset)
y = earthR * np.sin(theta)
z = 0 * np.sin(theta)

ax2.plot(x, y, z, color='k', linestyle='dashed')

##################################################
#	plot mesh

#	apply offset to rotate to body frame
init_offset = np.array([rot_vec_psi, rot_vec_phi, rot_vec_z])
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

facecol = [0.1, 0.1, 0.1]
edgecol = [0.5, 0.5, 0.5]

for face_index, body_vector in enumerate(body_vectors):

	#	Add final vert to vector array
	body_vector = np.vstack([body_vector, body_vector[0, :]])
	#	extract x, y and z values
	xs = (np.linalg.norm(site_orb_vec) / 5 * body_vector[:,0]) - plot_offset
	ys = np.linalg.norm(site_orb_vec) / 5 * body_vector[:,1]
	zs = np.linalg.norm(site_orb_vec) / 5 * body_vector[:,2]
	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]
	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_edgecolor(edgecol)
	pc.set_facecolor(facecol)
	ax2.add_collection3d(pc)

#	finalise plot
# ax2.set_xlim([-earthR, earthR])
# ax2.set_ylim([-earthR, earthR])
# ax2.set_zlim([-earthR, earthR])

ax2.set_xlim([-axisLen, axisLen])
ax2.set_ylim([-axisLen, axisLen])
ax2.set_zlim([-axisLen, axisLen])

# ax2.legend(frameon=False, loc='center left')

###############################################
#	set view direction
view_azimuth = 0.0
view_elevation = 90.0

ax2.view_init(view_elevation, view_azimuth)
plt.axis('off')

###################################################################################################
###################################################################################################
#	final conditions
###################################################################################################
###################################################################################################

ax3 = fig.add_subplot(gs[1], projection='3d')
ax3.set_title('$t = 240$', size=title_text_size)

##################################################
#	plot sun pos
c = Arrow3D(sun_orb_plot[:, 0], sun_orb_plot[:, 1], sun_orb_plot[:, 2],
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="tab:red"
)

ax3.add_artist(c)
ax3.text(sun_orb_plot[1, 0], sun_orb_plot[1, 1], sun_orb_plot[1, 2],
	r'$\mathbf{L}$',
	fontsize=5,
)

##################################################
#	plot earth pos
c = Arrow3D(earth_orb_plot[:, 0], earth_orb_plot[:, 1], earth_orb_plot[:, 2],
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color='k',
)

ax3.add_artist(c)
ax3.text(
	1.1 * earth_orb_plot[1, 0],
	earth_orb_plot[1, 1] - (0.6 * earth_orb_plot[1, 0]),
	earth_orb_plot[1, 2],
	r'$X\ (Earth)$',
	fontsize=5,
)

##################################################
#	plot velocity
c = Arrow3D(vel_orb_plot[:, 0], vel_orb_plot[:, 1], vel_orb_plot[:, 2],
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color='k',
)

ax3.add_artist(c)
ax3.text(1.4 * vel_orb_plot[1, 0], 0.25 * vel_orb_plot[1, 1], vel_orb_plot[1, 2],
	r'$Y\ (Velocity)$',
	fontsize=5,
)

##################################################
#	plot final site pos
site_plot_vec = site_orb[-1, 1:]
site_plot_vec[0] -= plot_offset
site_orb_plot = np.vstack([origin, site_plot_vec])

c = Arrow3D(site_orb_plot[:, 0], site_orb_plot[:, 1], site_orb_plot[:, 2],
	mutation_scale=5,
	lw=1.0,
	arrowstyle="-|>",
	color="tab:blue"
)

ax3.add_artist(c)
ax3.text(1.0 * site_orb_plot[1, 0], 1.5 * site_orb_plot[1, 1], site_orb_plot[1, 2],
	r'$\mathbf{V}$',
	fontsize=5,
)

##################################################
#	plot secondary vectors

##################################################
#	plot body x vector
c = Arrow3D(body_x_plot[:, 0], body_x_plot[:, 1], body_x_plot[:, 2],
	mutation_scale=2.5,
	lw=0.5,
	arrowstyle="-|>",
	color="k"
)

ax3.add_artist(c)
ax3.text(0.2 * body_x_plot[1, 0], 1.1 * body_x_plot[1, 1], body_x_plot[1, 2],
	r'$\mathbf{x_b}$',
	fontsize=5,
)

##################################################
#	plot body y vector
c = Arrow3D(body_y_plot[:, 0], body_y_plot[:, 1], body_y_plot[:, 2],
	mutation_scale=2.5,
	lw=0.5,
	arrowstyle="-|>",
	color="k"
)

ax3.add_artist(c)
ax3.text(body_y_plot[1, 0], body_y_plot[1, 1], body_y_plot[1, 2],
	r'$\mathbf{y_b}$',
	fontsize=5,
)

##################################################
#	plot body z vector
c = Arrow3D(body_z_plot[:, 0], body_z_plot[:, 1], body_z_plot[:, 2],
	mutation_scale=2.5,
	lw=0.5,
	arrowstyle="-|>",
	color="k"
)

ax3.add_artist(c)
ax3.text(body_z_plot[1, 0], body_z_plot[1, 1], body_z_plot[1, 2],
	r'$\mathbf{z_b}$',
	fontsize=5,
)

##################################################
#	plot earth circle
ax3.plot(x, y, z, color='k', linestyle='dashed')

##################################################
#	plot mesh

for face_index, body_vector in enumerate(body_vectors):

	#	Add final vert to vector array
	body_vector = np.vstack([body_vector, body_vector[0, :]])
	#	extract x, y and z values
	xs = (np.linalg.norm(site_orb_vec) / 5 * body_vector[:,0]) - plot_offset
	ys = np.linalg.norm(site_orb_vec) / 5 * body_vector[:,1]
	zs = np.linalg.norm(site_orb_vec) / 5 * body_vector[:,2]
	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]
	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_edgecolor(edgecol)
	pc.set_facecolor(facecol)
	ax3.add_collection3d(pc)

#	finalise plot
ax3.set_xlim([-axisLen, axisLen])
ax3.set_ylim([-axisLen, axisLen])
ax3.set_zlim([-axisLen, axisLen])

ax3.set_xlabel('X-Axis')
ax3.set_ylabel('Y-Axis')
ax3.set_zlabel('Z-Axis')

###############################################
#	set view direction
view_azimuth = 0.0
view_elevation = 90.0

ax3.view_init(view_elevation, view_azimuth)
plt.axis('off')

####################################################
#	save figure

if rot_vec_psi == 0:
	fig_name = 'lightcurve_example_case1b' + '.eps'

if rot_vec_psi == np.radians(50):
	fig_name = 'lightcurve_example_case2b' + '.eps'

out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	plt.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
