###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	geometry file plotter
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors
from matplotlib.patches import FancyArrowPatch

from tools.functions.functions_transform import rot1, rot2, rot3, gen_vectors2, rot_mat

###################################################################################################
#	define plotable arrow class
###################################################################################################

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################

#############################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Journals',
	'ACTA_Astronautica',
	'acta_figures',
	'Final'
)

fig_name = 'reference_frames.eps'
out_path = os.path.join(fig_path, fig_name)

#	figure save flag
savefile_flag = True
savefile_flag = False

###################################################################################################
#	geometry file
###################################################################################################

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'cube'
geom_str = '01_cube_cube'
geom_file = geom_str + '.stl'

attit_x = 0.0
attit_y = 0.0
attit_z = 0.0

#	apply attitude
# attitude = np.array([np.pi / 4, 0.0, 0.0])
attitude = np.array([np.radians(attit_x), np.radians(attit_y), np.radians(attit_z)])
attitude_matrix = rot_mat(attitude)

###################################################################################################
#	figure parameters
###################################################################################################

#	graphical output
inch2cm = 2.54

figure = plt.figure(num=None, figsize=(9.0/inch2cm, 9.0/inch2cm),
 	dpi=300, facecolor='w', edgecolor='k'
)

plt.rc('text', usetex=True)
plt.rcParams['text.latex.preamble']=[r"\usepackage{amsmath}"]
ax = plt.axes(projection='3d')

#	face plot parameters
alpha = 0.5
facecol = [0.1, 0.1, 0.1]
edgecol = [0.5, 0.5, 0.5]

axis_font_size = 7

#	coordinate origin
origin = [0, 0, 0]

###################################################################################################
#	plot geometry
###################################################################################################

#	load mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
object_mesh.rotate_using_matrix(attitude_matrix)

scale_factor = 0.20
geom_offset = 2.5
geom_offset = 0

object_mesh.x *= scale_factor
object_mesh.y *= scale_factor
object_mesh.z *= scale_factor
object_mesh.z -= geom_offset

vectors = 2 * object_mesh.vectors
normals = 2 * object_mesh.normals
nfaces = len(normals)

#	facet loop
for i, (vector, normal) in enumerate(zip(vectors, normals)):

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_edgecolor(edgecol)

	pc.set_linewidth(0.5)
	pc.set_facecolor(facecol)

	ax.add_collection3d(pc)

###################################################################################################
#	plot inertial axes

inertial_axis_len = 10

xaxis = [inertial_axis_len, 0, 0]
yaxis = [0, inertial_axis_len * 0.8, 0]
zaxis = [0, 0, inertial_axis_len * 0.7]

#	x axis
xaxis_plot = np.vstack([origin, xaxis])
c = Arrow3D(xaxis_plot[:, 0], xaxis_plot[:, 1], xaxis_plot[:, 2],
	mutation_scale=5,
	lw=0.25,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(xaxis_plot[1, 0], xaxis_plot[1, 1] - 0.5, xaxis_plot[1, 2],
#	'$X_{axis} (Earth\ Zenith\ Vector)$',
	'$X$',
	fontsize=axis_font_size,
	fontweight='bold'
)

#	y axis
yaxis_plot = np.vstack([origin, yaxis])
c = Arrow3D(yaxis_plot[:, 0], yaxis_plot[:, 1], yaxis_plot[:, 2],
	mutation_scale=5,
	lw=0.25,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(yaxis_plot[1, 0], yaxis_plot[1, 1], yaxis_plot[1, 2],
#	'$Y_{axis}\ (Velocity\ Vector)$',
	'$Y$',
	fontsize=axis_font_size,
	fontweight='bold'
)

#	z axis
zaxis_plot = np.vstack([origin, zaxis])
c = Arrow3D(zaxis_plot[:, 0], zaxis_plot[:, 1], zaxis_plot[:, 2],
	mutation_scale=5,
	lw=0.25,
	arrowstyle="-|>",
	color="k"
)

ax.add_artist(c)
ax.text(zaxis_plot[1, 0], zaxis_plot[1, 1], zaxis_plot[1, 2],
#	'$Z_{axis} (Orbit\ Plane\ Normal)$',
	'$Z$',
	fontsize=axis_font_size,
	fontweight='bold'
)

###################################################################################################
#	plot vectors
###################################################################################################

###################################################################################################
#	observation and illumination vectors

sun_psi = np.radians(60.0)
sun_phi = np.radians(50.0)

#	vector length
vec_length = 15

#	Illumination Vector
illum_vec = np.array([vec_length, 0.0, 0.0])

#	rotate vector
illum_vec = rot2(illum_vec, sun_phi)
illum_vec = rot1(illum_vec, sun_psi)

illum_vec_plot = np.vstack([origin, illum_vec])

#	Observation Vector
obs_vec = [vec_length, 0, 0]
obs_vec_plot = np.vstack([xaxis, obs_vec])

c = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2],
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="tab:blue"
)

ax.add_artist(c)
ax.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1] - 0.5, obs_vec_plot[1, 2],
	r'$\boldsymbol{V}$',
	fontsize=5,
	fontweight='bold'
)

c = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2],
	mutation_scale=5,
	lw=1,
	arrowstyle="-|>",
	color="tab:red"
)

ax.add_artist(c)
ax.text(illum_vec_plot[1, 0], illum_vec_plot[1, 1], illum_vec_plot[1, 2],
	r'$\boldsymbol{L}$',
	fontsize=5,
	fontweight='bold'
)

###################################################################################################
#	create dotted lines

temp1 = np.array([illum_vec[0], illum_vec[1], 0.0])
temp2 = np.array([illum_vec[0], 0.0, 0.0])
temp3 = np.array([0.0, illum_vec[1], 0.0])

dotted_multiplier = 0.75

dotted_1 = np.vstack([temp1 * dotted_multiplier, illum_vec * dotted_multiplier])
dotted_2 = np.vstack([temp2 * dotted_multiplier, temp1 * dotted_multiplier])
dotted_3 = np.vstack([temp3 * dotted_multiplier, temp1 * dotted_multiplier])

ax.plot(dotted_1[:, 0], dotted_1[:, 1], dotted_1[:, 2], linestyle='dotted', linewidth=0.5, color='k')
ax.plot(dotted_2[:, 0], dotted_2[:, 1], dotted_2[:, 2], linestyle='dotted', linewidth=0.5, color='k')
ax.plot(dotted_3[:, 0], dotted_3[:, 1], dotted_3[:, 2], linestyle='dotted', linewidth=0.5, color='k')

ax.text(
	dotted_1[1, 0],
	dotted_1[1, 1] + 0.1,
	dotted_1[1, 2] / 2 + 1,
	r'$z$',
	fontsize=5
)

ax.text(
	dotted_2[1, 0],
	dotted_2[1, 1] / 2,
	dotted_2[1, 2] / 2 - 0.4,
	r'$y$',
	fontsize=5
)

ax.text(
	dotted_3[1, 0] / 2,
	dotted_3[1, 1],
	dotted_3[1, 2] / 2 - 0.4,
	r'$x$',
	fontsize=5
)

###################################################################################################
#	define circle data

#	define radius
arc_radius = 8

#	generate circle data
n = 360
circle_vals = []

for i in range(0, n + 1):
	x = np.cos(2 * np.pi / n * i) * arc_radius
	y = np.sin(2 * np.pi / n * i) * arc_radius
	z = 0

	outvec = np.array([x, y, z])
	circle_vals.append(outvec)

circle_vals = np.array(circle_vals)

###################################################################################################
#	plot Z arc

circle_1_temp = circle_vals[0:90]
circle_1 = []

for circle_vec in circle_1_temp:

	circle_1.append(rot1(0.8 * circle_vec, -np.pi / 2))

circle_1 = np.array(circle_1)

plt.plot(
	circle_1[:, 0],
	circle_1[:, 1],
	circle_1[:, 2],
	color='tab:gray',
	linewidth=0.5
)

###################################################################################################
#	plot Y arc

circle_2 = 0.8 * circle_vals[0:90]

plt.plot(
	circle_2[:, 0],
	circle_2[:, 1],
	circle_2[:, 2],
	color='tab:gray',
	linewidth=0.5
)

###################################################################################################
#	plot Phi arc
circle_L_temp = circle_vals[0:50]
circle_L = []

for circle_vec in circle_L_temp:

	circle_L.append(rot1(0.8 * circle_vec, -((np.pi / 2) - sun_psi)))

circle_L = np.array(circle_L)

plt.plot(
	circle_L[:, 0],
	circle_L[:, 1],
	circle_L[:, 2],
	color='tab:red',
	linewidth=0.5
)

ax.text(
	circle_L[int(len(circle_L) / 2), 0],
	circle_L[int(len(circle_L) / 2), 1] + 1,
	circle_L[int(len(circle_L) / 2), 2] - 0.1,
	r'$\phi$',
	fontsize=6,
	fontweight='bold'
)




circle_L2_temp = circle_vals[5:35]
circle_L2 = []

for circle_vec in circle_L2_temp:

	circle_vec[1] += 0.2
	circle_vec[2] -= 0.2

	circle_L2.append(rot1(0.8 * circle_vec, -((np.pi / 2) - sun_psi)))

circle_L2 = np.array(circle_L2)

markevery1 = [-1]

plt.plot(
	circle_L2[:, 0],
	circle_L2[:, 1],
	circle_L2[:, 2],
	color='k',
	linewidth=0.5,
	markevery=markevery1,
	marker=(3, 0, 60),
	markersize=1
)

###################################################################################################
#	define circle data

#	define radius
radius = 2

#	generate circle data
n = 360
circle_vals = []

for i in range(0, n + 1):
	x = np.cos(2 * np.pi / n * i) * radius
	y = np.sin(2 * np.pi / n * i) * radius
	z = 0

	outvec = rot2(np.array([x, y, z]), np.pi/2)
	outvec[0] += 0.8 * arc_radius

	circle_vals.append(outvec)

circle_vals = np.array(circle_vals)[1:54]

plt.plot(
	circle_vals[:, 0],
	circle_vals[:, 1],
	circle_vals[:, 2],
	markevery=markevery1,
	marker=(3, 0, 80),
	markersize=1,
	color='k',
	linewidth=0.5
)

ax.text(
	circle_vals[int(len(circle_vals) / 2), 0],
	circle_vals[int(len(circle_vals) / 2), 1] - 0.4,
	circle_vals[int(len(circle_vals) / 2), 2] + 0.4,
	r'$\psi$',
	fontsize=6,
	fontweight='bold'
)

###################################################################################################
#	finalise figure
###################################################################################################

axis_len = 10

ax.set_xlim([0, axis_len])
ax.set_ylim([0, axis_len])
ax.set_zlim([0, axis_len])


ax.view_init(16, 14)
plt.axis('off')

if savefile_flag is True:
	plt.savefig(out_path, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
