###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	geometry file plotter
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time

#	numpy
import numpy as np

#	numpy-stl
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors
from matplotlib.patches import FancyArrowPatch

###################################################################################################
#	define plotable arrow class
###################################################################################################

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)


###################################################################################################
#	initilisation
###################################################################################################

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Journals',
	'ACTA_Astronautica',
	'acta_figures',
	'Final'
)

#	figure save flag
savefile_flag = True
savefile_flag = False

#############################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)

geom_dir = 'geometry_dir'

###################################################################################################
#	figure parameters
###################################################################################################

#	face plot parameters
alpha = 0.5
facecol = [0.1, 0.1, 0.1]
edgecol = [0.5, 0.5, 0.5]

#	graphical output
inch2cm = 2.54

font_sizex = 1
font_sizey = 1
font_pad = 0.5
marker_size = 4
marker_edge = 1

vec_len = 0.75
label_size = 10

figsize_x = 10
figsize_y = 3

#	initialise figure
figure3d = plt.figure(num=None, figsize=(figsize_x/inch2cm, figsize_y/inch2cm),
	dpi=300, facecolor='w', edgecolor='k'
)

plt.rc('text', usetex=True)
#plt.subplots_adjust(wspace=0.3)

###################################################################################################
#	cube
###################################################################################################

#	set geometry input
geom_file = '01_cube_cube.stl'

#	load mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
vectors = object_mesh.vectors
normals = object_mesh.normals
nfaces = len(normals)

#	initialse subplot axes
ax_cube = plt.subplot(131, projection="3d")

alpha = 1.0

#	facet loop
for vector, normal in zip(vectors, normals):

# 	vect12 = vector[1] - vector[0]
# 	vect23 = vector[2] - vector[1]
# 	normal = np.cross(vect12, vect23)
# 	normal_unit = normal / np.linalg.norm(normal)

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_alpha(alpha)
	pc.set_edgecolor(edgecol)
	pc.set_facecolor(facecol)

	ax_cube.add_collection3d(pc)

	#	find face centroid
	vec1 = vector[0]
	vec2 = vector[1]
	vec3 = vector[2]

	centroid_pos = np.array(
		[
			(1 / 3) * (vec1[0] + vec2[0] + vec3[0]),
			(1 / 3) * (vec1[1] + vec2[1] + vec3[1]),
			(1 / 3) * (vec1[2] + vec2[2] + vec3[2])
		]
	)

ax_cube.set_xlim([-5, 5])
ax_cube.set_ylim([-5, 5])
ax_cube.set_zlim([-5, 5])

#############################################
#	plot body axes

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

bodyx_vec_plot = 6 * np.vstack(([0, 0, 0], x_body_unit))
bodyy_vec_plot = 6 * np.vstack(([0, 0, 0], y_body_unit))
bodyz_vec_plot = 6 * np.vstack(([0, 0, 0], z_body_unit))

#	body x axis
c = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
	mutation_scale=5,
	lw=vec_len,
	arrowstyle="-|>",
	color="k"
)
ax_cube.add_artist(c)

#	body y axis
d = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
	mutation_scale=5,
	lw=vec_len,
	arrowstyle="-|>",
	color="k"
)
ax_cube.add_artist(d)

#	body z axis
e = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
	mutation_scale=5,
	lw=vec_len,
	arrowstyle="-|>",
	color="k"
)
ax_cube.add_artist(e)

#############################################
#	plot body axes text

bodyx_vec_plot /= 2 
bodyy_vec_plot /= 2 
bodyz_vec_plot /= 2 

ax_cube.text(bodyx_vec_plot[1, 0] + 0.2, bodyx_vec_plot[1, 1] - 0.1, bodyx_vec_plot[1, 2] + 0.4,
	r'$x_{b}$',
	fontsize=label_size,
	fontweight='bold'
)

ax_cube.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	r'$y_{b}$',
	fontsize=label_size,
	fontweight='bold'
)

ax_cube.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	r'$z_{b}$',
	fontsize=label_size,
	fontweight='bold'
)

#	set viewing direction
ax_cube.view_init(25, 45)

ax_cube.set_xlim([-1.5, 1.5])
ax_cube.set_ylim([-1.5, 1.5])
ax_cube.set_zlim([-1.5, 1.5])

ax_cube.set_xlabel('x axis')
ax_cube.set_ylabel('y axis')
ax_cube.set_zlabel('z axis')

ax_cube.axis('off')




###################################################################################################
#	div 1 sphere
###################################################################################################

#	set input geometry
geom_file = '01_div1_sphere.stl'

#	load mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
vectors = object_mesh.vectors
normals = object_mesh.normals
nfaces = len(normals)

#	initialise figure axis
ax_div1 = plt.subplot(132, projection="3d")
plt.axis('off')

#	facet loop
for vector, normal in zip(vectors, normals):

	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]
	normal = np.cross(vect12, vect23)
	normal_unit = normal / np.linalg.norm(normal)

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_alpha(alpha)
	pc.set_edgecolor(edgecol)
	pc.set_facecolor(facecol)

	ax_div1.add_collection3d(pc)

#############################################
#	plot body axes

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

bodyx_vec_plot = 4 * np.vstack(([0, 0, 0], x_body_unit))
bodyy_vec_plot = 4 * np.vstack(([0, 0, 0], y_body_unit))
bodyz_vec_plot = 4 * np.vstack(([0, 0, 0], z_body_unit))

#	body x axis
c = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
	mutation_scale=5, 
	lw=vec_len,
	arrowstyle="-|>",
	color="k"
)
ax_div1.add_artist(c)

#	body y axis
d = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
	mutation_scale=5, 
	lw=vec_len,
	arrowstyle="-|>",
	color="k"
)
ax_div1.add_artist(d)

#	body z axis
e = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
	mutation_scale=5,
	lw=vec_len,
	arrowstyle="-|>",
	color="k"
)
ax_div1.add_artist(e)
#############################################
#	plot body axes text

bodyx_vec_plot /= 2 
bodyy_vec_plot /= 2 
bodyz_vec_plot /= 2 

ax_div1.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1] - 0.1, bodyx_vec_plot[1, 2] + 0.2,
	r'$x_{b}$',
	fontsize=label_size,
	fontweight='bold'
)

ax_div1.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	r'$y_{b}$',
	fontsize=label_size,
	fontweight='bold'
)

ax_div1.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	r'$z_{b}$',
	fontsize=label_size,
	fontweight='bold'
)

#	set viewing direction
ax_div1.view_init(25, 45)

#	finalise subplot 
ax_div1.set_xlim([-1, 1])
ax_div1.set_ylim([-1, 1])
ax_div1.set_zlim([-1, 1])
ax_div1.set_xlabel('x axis')
ax_div1.set_ylabel('y axis')
ax_div1.set_zlabel('z axis')

###################################################################################################
#	div 5 sphere
###################################################################################################

#	set input geom file
geom_file = '05_div5_sphere.stl'

#	initialise axes
ax_div5 = plt.subplot(133, projection="3d")

#	load mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))
vectors = object_mesh.vectors
normals = object_mesh.normals
nfaces = len(normals)

#############################################
#	facet loop
for vector, normal in zip(vectors, normals):

	#	calculate face vectors and normals
	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]
	normal = np.cross(vect12, vect23)
	normal_unit = normal / np.linalg.norm(normal)

	#	generate verts
	vector = np.vstack([vector, vector[0, :]])
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]
	verts = [list(zip(xs, ys, zs))]

	#	plot face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_alpha(alpha)
	pc.set_edgecolor(edgecol)
	pc.set_facecolor(facecol)
	
	ax_div5.add_collection3d(pc)

#############################################
#	plot body axes

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

bodyx_vec_plot = 4 * np.vstack(([0, 0, 0], x_body_unit))
bodyy_vec_plot = 4 * np.vstack(([0, 0, 0], y_body_unit))
bodyz_vec_plot = 4 * np.vstack(([0, 0, 0], z_body_unit))

#	body x axis
c = Arrow3D(bodyx_vec_plot[:, 0] / 2, bodyx_vec_plot[:, 1] / 2, bodyx_vec_plot[:, 2] / 2,
	mutation_scale=5, 
	lw=vec_len,
	arrowstyle="-|>",
	color="k"
)

ax_div5.add_artist(c)

#	body y axis
d = Arrow3D(bodyy_vec_plot[:, 0] / 2, bodyy_vec_plot[:, 1] / 2, bodyy_vec_plot[:, 2] / 2,
	mutation_scale=5, 
	lw=vec_len,
	arrowstyle="-|>",
	color="k"
)

ax_div5.add_artist(d)

#	body z axis
e = Arrow3D(bodyz_vec_plot[:, 0] / 2, bodyz_vec_plot[:, 1] / 2, bodyz_vec_plot[:, 2] / 2,
	mutation_scale=5, 
	lw=vec_len,
	arrowstyle="-|>",
	color="k"
)

ax_div5.add_artist(e)

#############################################
#	plot body axes text
bodyx_vec_plot /= 2 
bodyy_vec_plot /= 2 
bodyz_vec_plot /= 2 

ax_div5.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1] - 0.1, bodyx_vec_plot[1, 2] + 0.2,
	r'$x_{b}$',
	fontsize=label_size,
	fontweight='bold'
)

ax_div5.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	r'$y_{b}$',
	fontsize=label_size,
	fontweight='bold'
)

ax_div5.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	r'$z_{b}$',
	fontsize=label_size,
	fontweight='bold'
)

#	set subplot veiw direction
ax_div5.view_init(25, 45)

#	finalise subplot
ax_div5.set_xlim([-1, 1])
ax_div5.set_ylim([-1, 1])
ax_div5.set_zlim([-1, 1])
ax_div5.set_xlabel('x axis')
ax_div5.set_ylabel('y axis')
ax_div5.set_zlabel('z axis')

plt.axis('off')

###################################################################################################
#	save output figure
###################################################################################################

#	saving figure
fig_name = 'example_geoms' + '.eps'
out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	plt.savefig(out_str, bbox_inches="tight", dpi=300)
	print('Figure saved: ', fig_name)

plt.show()
