###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('MMT testing script initilised.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

##################################################
#	simulation parameters
time_step = 0.1

###################################################################################################
#	light curve generation inputs
###################################################################################################

#	geometry file
geom_str = 'cz-2d_pointed'
geom_str = 'cz-2d'
geom_str = 'cz-2d_testing'
geom_str = 'cz-2d_testing_pointed'
geom_str = 'cz-2d_testing_double_pointed'

#	attitude motion
rot_period = 9.2
rot_period_index = int(np.ceil(rot_period) * 10)

#	brdf
top_faces_index = np.arange(0, 36)
side_faces_index = np.arange(36, 108)
bot_faces_index = np.arange(108, 144)

# ###################################################################################################
# #	EG1 Inputs
# ###################################################################################################
# 
# #	light curve example file used
# light_curve_example_file = 'lc_custom_eg1.txt'
# 
# #	attitude
# rot_vec_azm_deg = 267.0
# rot_vec_inc_deg = 65.0
# 
# top_omega_brdf = 0.79
# top_m_brdf = 0.34
# top_d_brdf = 0.91
# 
# bot_omega_brdf = 0.77
# bot_m_brdf = 0.55
# bot_d_brdf = 0.26
# 
# side_omega_brdf = 0.83
# side_m_brdf = 0.04
# side_d_brdf = 0.48

# ###################################################################################################
# #	EG2 Inputs
# ###################################################################################################
# 
# #	light curve example file used
# light_curve_example_file = 'lc_custom_eg2.txt'
# 
# #	attitude
# rot_vec_azm_deg = 114.0
# rot_vec_inc_deg = 15.0
# 
# top_omega_brdf = 0.66
# top_m_brdf = 0.2
# top_d_brdf = 0.11
# 
# bot_omega_brdf = 0.08
# bot_m_brdf = 0.63
# bot_d_brdf = 0.65
# 
# side_omega_brdf = 0.61
# side_m_brdf = 0.39
# side_d_brdf = 0.94

# ###################################################################################################
# #	EG3 Inputs
# ###################################################################################################
# 
# #	light curve example file used
# light_curve_example_file = 'lc_custom_eg3.txt'
# polar_data_output_file = light_curve_example_file[:-4] + '_polar_data.txt'
# 
# #	attitude
# rot_vec_azm_deg = 160.0
# rot_vec_inc_deg = 89.0
# 
# top_omega_brdf = 0.51
# top_m_brdf = 0.52
# top_d_brdf = 0.98
# 
# bot_omega_brdf = 0.73
# bot_m_brdf = 0.65
# bot_d_brdf = 0.84
# 
# side_omega_brdf = 0.78
# side_m_brdf = 0.82
# side_d_brdf = 0.62

###################################################################################################
#	EG4 Inputs
###################################################################################################

#	light curve example file used
light_curve_example_file = 'lc_custom_eg4.txt'
polar_data_output_file = light_curve_example_file[:-4] + '_polar_data.txt'

#	attitude
rot_vec_azm_deg = 324.0
rot_vec_inc_deg = 9.0

top_omega_brdf = 0.42
top_m_brdf = 0.61
top_d_brdf = 0.49

bot_omega_brdf = 0.63
bot_m_brdf = 0.91
bot_d_brdf = 0.09

side_omega_brdf = 0.35
side_m_brdf = 0.66
side_d_brdf = 0.27

###################################################################################################
#	load custom light curve data
###################################################################################################

print('\nLoading custom light curve data...')

#	load custom light curve
light_curve_custom = np.loadtxt(
	os.path.join(path_data, 'random_light_curves', light_curve_example_file),
	delimiter=','
)

base_signal = light_curve_custom[:, 2]

###################################################################################################
#	define MMT observation site		LOCKED
###################################################################################################

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	load MMT file					LOCKED
###################################################################################################

mmt_file = 'cz-2d_41858_11-02-17_03-02-59.txt'

name_str, norad_id, date_str, time_str = mmt_file.split('_')
norad_id = int(norad_id)

read_path = os.path.join(path_data, 'input_dir', 'mmt_files')

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file					LOCKED
###################################################################################################

#	header line
header_text = mmt_data[7]

#	strip header
mmt_data = mmt_data[8:]

###################################################################################################
#	extract start jd

mmt_line = mmt_data[0]

#	parse line elements
mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
mmt_date = np.array(mmt_date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = mmt_time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

mmt_time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([mmt_date, mmt_time])

#	calculate julian date
jd = date2jd(*datetime_array)
start_jd = jd

###################################################################################################
#	extract stop jd

mmt_line = mmt_data[-1]

#	parse line elements
mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
	penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

#	split date
mmt_date = np.array(mmt_date.split('-')).astype(np.int)

#	split time
raw_hour, raw_minute, raw_second = mmt_time.split(':')

#	set data types
hour = np.int(raw_hour)
minute = np.int(raw_minute)
second = np.round(np.float(raw_second), 1)

mmt_time = np.array([hour, minute, second])

#	create datetime array
datetime_array = np.hstack([mmt_date, mmt_time])

#	calculate julian date
jd = date2jd(*datetime_array)
stop_jd = jd

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(lc_duration_sec / time_step) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	extract MMT data

sim_index = 0
mmt_data_parsed = []

#	parse MMT data
for itt, mmt_line in enumerate(mmt_data):

	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')

	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)

	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')

	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)

	#	generate time array
	mmt_time = np.array([hour, minute, second])

	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])

	#	calculate julian date
	jd = date2jd(*datetime_array)
	step_jd = jd[0] + jd[1]

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	#	set light curve data datatypes
	std_mag = np.float(std_mag)
	app_mag = np.float(app_mag)
	distance = np.float(distance)
	phase = np.float(phase)

	#	generate light curve data array
	mmt_lc_data = [std_mag, app_mag, distance, phase]
		
	#	assemble line data
	line_data = np.hstack([jd, current_sec, mmt_lc_data])

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		mmt_data_parsed.append(line_data)
		sim_index += 1

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	set previous data values
		jd_prev = prev_line[0]
		jdfrac_prev = prev_line[1]
		prev_sec = prev_line[2]

		#	set previous light curve data
		std_mag_prev = prev_line[-4]
		app_mag_prev = prev_line[-3]
		distance_prev = prev_line[-2]
		phase_prev = prev_line[-1]

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		std_mag_last = std_mag
		app_mag_last = app_mag
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		std_mag_step = (std_mag_last - std_mag_prev) / npts
		app_mag_step = (app_mag_last - app_mag_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	fill first line
		interp_data = np.zeros([npts, len(line_data)])

		#	calculate interpolation data
		for n in range(1, len(interp_data) + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jd_prev + jdfrac_prev) + (n * (0.1 / 86400))
			jd = jday_jfrac(jdate)

			#	calculate interpolated light curve data
			std_mag = std_mag_prev + (n * std_mag_step)
			app_mag = app_mag_prev + (n * app_mag_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			mmt_lc_data = [std_mag, app_mag, distance, phase]
					
			#	assemble line data
			line_data = np.hstack([jd, interp_sec, mmt_lc_data])

			#	add to output
			interp_data[n - 1, :] = line_data

		#	find required lines in interpolated data
		for interp_line in interp_data:

			#	set current simulation second
			interp_line_sec = interp_line[2]

			if interp_line_sec == sim_range_sec[sim_index]:
				mmt_data_parsed.append(line_data)
				sim_index += 1

	#	save previous line
	if itt != 0:
		prev_line = line_data
		prev_time = mmt_time

#	ensure parsed MMT data matches simulation data
mmt_data_parsed = np.array(mmt_data_parsed)

if len(mmt_data_parsed) == len(lc_sim_range):
	print('\nSimulation and MMT data matches...')
else:
	print('\nError: Simulation and MMT data does not match...')
	sys.exit()

mmt_signal = mmt_data_parsed[:, 4]

###################################################################################################
#	generate datevec array			LOCKED
###################################################################################################

print('\nGenerating DateVec array...')

#	tick mark locators
years = mdates.YearLocator()		# every year
months = mdates.MonthLocator()		# every month
days = mdates.DayLocator()			# every day
hours = mdates.HourLocator()		# every hour
minutes = mdates.MinuteLocator()	# every minute
seconds = mdates.SecondLocator()	# every second

#	assemble jdate array
mmt_jdate_col = np.add(mmt_data_parsed[:, 0], mmt_data_parsed[:, 1])

#	generate date vectors
mmt_datevec_array = []

for line in mmt_jdate_col:
	#	convert jd to date
	year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(line))]
	#	turn into string
	datestr = str('-'.join(['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]))
	timestr = ':'.join(['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)])
	datevec  = ' '.join([datestr, timestr])
	#	convert to datetime object
	date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
	#	append  to output array
	mmt_datevec_array.append(date_time_obj)

###################################################################################################
#	simulation printout				LOCKED
###################################################################################################

print('\n#####################################################################################')
print('\n	MMT light curve parameters: ')
print('		Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_start_date))
print('		Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_stop_date))
print('		Simulation time step:		{}'.format(time_step))
print('		Duration:			{}		[steps]'.format(lc_len_record))
print('						{}		[s]'.format(np.round(lc_duration_sec, 1)))

###################################################################################################
#	acquire TLE					LOCKED
###################################################################################################

###################################################################################################
#	set TLE time data

print('\nAcquiring TLE...')

#	TLE request dates, add day either side to increase tle find chance
tle_start_date = jd2date(*jday_jfrac(lc_start_jd - 1))
tle_stop_date = jd2date(*jday_jfrac(lc_stop_jd + 1)) 

#	generate TLE file name
tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)

#	assemble TLE path
tle_path = os.path.join(path_data, 'input_dir', 'tle_files')

#	if exists, load
if os.path.isfile(os.path.join(tle_path, tle_file)):

	print('Reading existing TLE file.')

	#	load file
	with open(os.path.join(tle_path, tle_file), 'r') as file:
		tle_data = file.readlines()

else:
	#	request TLEs
	tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)

	#	save file
	with open(os.path.join(tle_path, tle_file), 'w') as file:
		for line in tle_data:
			file.write(line)

#	grab first TLE pair
tle_line1 = tle_data[1]
tle_line2 = tle_data[2]

#	mean mot rad/sec
orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
orbital_mot = (2 * np.pi) / orbital_period

###################################################################################################
#	set data time data
###################################################################################################

#	calculate full orbit start/stop times
time_sep = np.round((orbital_period - lc_duration_sec) / 2, 1)

#	start and stop times of single orbit containing the light curve
data_start_jd = lc_start_jd - (time_sep / 86400)
data_stop_jd = lc_stop_jd + (time_sep / 86400)

#	convert to date vectors
data_start_date = jd2date(*jday_jfrac(data_start_jd))
data_stop_date = jd2date(*jday_jfrac(data_stop_jd))

#	calculate data duration
data_duration_sec = np.round(86400 * (data_stop_jd - data_start_jd), 1)

#	set time step
data_len_record = 1 + int(data_duration_sec / time_step)

#	time range in julian days
data_sim_range = np.linspace(data_start_jd, data_stop_jd, data_len_record)





###################################################################################################
#	set time data to be used
###################################################################################################

start_jd = lc_start_jd
stop_jd = lc_stop_jd

start_date = lc_start_date
stop_date = lc_stop_date

duration_sec = lc_duration_sec
len_record = lc_len_record
sim_range = lc_sim_range






###################################################################################################
#	generate eop data array			LOCKED
###################################################################################################

#	EOP data
eop_dir = 'eop_files'
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	assemble eop path
eop_path = os.path.join(path_data, eop_dir)

#	generate interpolated eop_data
eop_log_level = 1
eop_data, dat_data = eop_interpolate(
	eop_path,
	eop_file,
	dat_file,
	sim_range,
	time_step,
	eop_log_level
)

###################################################################################################
#	generate time objects			LOCKED
###################################################################################################

#	start dat value
dat = dat_data[0, -1]
	
#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate start time object
start_time_object = get_times(*start_date, ut_offset, dat)
	
#	end dat value
dat = dat_data[-1, -1]
	
#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate stop time object
stop_time_object = get_times(*stop_date, ut_offset, dat)
	
###################################################################################################
#	generate rotation matrices		LOCKED
###################################################################################################

print('\n[{}]	Generating rotation matrices:\n'.format(datetime.datetime.now()))

#	tt time
jdaytt_start = start_time_object.jday_tt
jfractt_start = start_time_object.jfrac_tt
jdtt_start = jdaytt_start + jfractt_start

#	ut1 time
jdayut1_start = start_time_object.jday_ut1
jfracut1_start = start_time_object.jfrac_ut1
jdut1_start = jdayut1_start + jfracut1_start

#	generate rotation matrices
pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
	jdtt_start, jdut1_start, eop_data, time_step
)

###################################################################################################
#	MODULE:	Horizons Positions
###################################################################################################

print('\n[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))

#	set horizons logging level
horz_logging_level = 1

sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
	path_data, horz_logging_level,
	start_time_object, stop_time_object,
	len_record, time_step,
	eop_path
)

#	find indices of light curve data
timecol = sun_icrf[:, 0]

###################################################################################################
#	MODULE:	Object Position
###################################################################################################

satpos_logging_level = 1

sat_gcrf_sgp4, sat_itrf_sgp4 = module_sat_pos(
	path_data, satpos_logging_level, len_record, time_step,
	start_time_object, stop_time_object,
	norad_id,
	eop_data, dat_data,
	pm_array, pn_array, st_array, c2t_array
)

###################################################################################################
#	MODULE:	Transformation
###################################################################################################

identification_str = 'mmt-testing'

trans_logging_level = 1
save_flag = False

sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
	= module_transformation(
		path_data, trans_logging_level, save_flag,
		start_time_object, stop_time_object,
		identification_str, site_str,
		len_record, time_step,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array,
		sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
	)

###################################################################################################
#	generate polar angles
###################################################################################################

#	set light curve time step
lc_time_step = 0.1

#	extract light curve polar data
sun_polar_data, site_polar_data, phase_angles, jd_time_data = extract_fun(
	sim_range, lc_time_step, time_step,
	sun_orb, earth_orb, site_orb, sat_orb
)

###################################################################################################
#	initialise light curve generation	LOCKED
###################################################################################################

##################################################
#	geometry
geom_dir = 'geometry_dir/geom_dir/' + 'CZ-2D'
geom_file = geom_str + '.stl'

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

##################################################
#	attitude motion

# #	1 rpm (radians per second)
# rot_ang = 360.0 / 60.0
rot_ang_per_sec = (np.pi * 2) / rot_period

#	define rotation axis in body frame
rot_vec_body_unit = np.array([1.0, 0.0, 0.0])

#	convert to rotation matrix
attitude_matrix = axis2mat(rot_ang_per_sec, rot_vec_body_unit)

#	extract euler angles
angles1 = np.array(mat2angles(attitude_matrix)[0:3])
angles2 = np.array(mat2angles(attitude_matrix)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

initial_attitude_motion = angles1

##################################################
#	attitude

#	convert to radians
rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad = np.radians(rot_vec_inc_deg)

#	generate attitude
initial_attitude = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])

##################################################
#	orbital rot vec motion
rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

###############################################################################################
#	generate brdf						LOCKED
###############################################################################################

#	Refractive Index
top_n_brdf = (1 + np.sqrt(top_omega_brdf)) / (1 - np.sqrt(top_omega_brdf))
side_n_brdf = (1 + np.sqrt(side_omega_brdf)) / (1 - np.sqrt(side_omega_brdf))
bot_n_brdf = (1 + np.sqrt(bot_omega_brdf)) / (1 - np.sqrt(bot_omega_brdf))

#	Specular ratio
top_s_brdf = 1.0 - top_d_brdf
side_s_brdf = 1.0 - side_d_brdf
bot_s_brdf = 1.0 - bot_d_brdf

#	generate brdf
brdf_data = np.zeros([nfaces, 5])

for i in range(nfaces):

	#	working on a top face
	if i in top_faces_index:
		brdf_data[i, :] = [top_m_brdf, top_omega_brdf, top_d_brdf, top_s_brdf, top_n_brdf]
	
	#	working on a side face
	elif i in side_faces_index:
		brdf_data[i, :] = [side_m_brdf, side_omega_brdf, side_d_brdf, side_s_brdf, side_n_brdf]

	#	working on a bot face
	elif i in bot_faces_index:
		brdf_data[i, :] = [bot_m_brdf, bot_omega_brdf, bot_d_brdf, bot_s_brdf, bot_n_brdf]

	else:
		print('BRDF Generation Error, exiting.')
		sys.exit()

###################################################################################################
#	perform light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

init_attit = np.copy(initial_attitude)

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	extract orbital positions of custom input vectors
illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_body[:, 31:34]

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]

###############################################################################################
#	generate light curve
###############################################################################################

#	standard distance flag. sets distance to 1000 km when true
std_dist_flag = True

#	generate sim time
sim_time = np.linspace(0, duration_sec, len_record)

#	initilise lightcurve output
final_light_curve = np.zeros((lc_len_record, 3))
final_light_curve[:, 1] = sim_time

#	run light curve generation function
light_curve = glim_slc(sun_body_data, site_body_data, final_light_curve, \
		start_jd, body_vectors, body_areas, brdf_data, std_dist_flag)
	
light_curve_data = light_curve[:, 2]

###############################################################################################
#	plot single lightcurve

#	initialise light curve axes
fig_lightcurve, ax_lightcurve = plt.subplots(figsize=(10, 6))

#	true light curve
ax_lightcurve.plot(
	mmt_datevec_array, base_signal, label='Testing Target Light Curve'
)

#	simulated light curve
ax_lightcurve.plot(
	mmt_datevec_array, light_curve_data, label='Simulated'
)

#	finalise plot
ax_lightcurve.legend()
ax_lightcurve.grid()

####################################################
#	reverse y axis
ax_lightcurve.invert_yaxis()

plt.show()
sys.exit()
