###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_new

#	transformation functions
from tools.functions.functions_transform import gen_vectors2

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

def attitude_polar_function(input_line):
	
	#	parse input data
	rot_vec_inc_deg, rot_vec_azm_deg, _, path_data, orbital_mot, time_step, len_record, \
		sat_orb, sun_orb, earth_orb, site_orb, \
		rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb, \
		duration_sec, start_jd, stop_jd, \
		body_vectors, body_areas, brdf_data, rot_period_index, \
		base_signal, plot_max = input_line 

	#	break loop if above 90 deg inclination
	if rot_vec_inc_deg > 90:
#		print('\n	Inclination is greater than 90, setting RMS to 0.')
		rms = 0.0

	else:

		#	convert to radians
		rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
		rot_vec_inc_rad = np.radians(rot_vec_inc_deg)
		
		#	generate initial attitude
		initial_attitude = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
		
		##################################################
		#	regenerate light curve generation vectors
		
		#	blank observation and illumination vectors in orbital frame
		obs_vec_orb = np.array([1.0, 0.0, 0.0])
		illum_vec_orb = np.array([1.0, 0.0, 0.0])
		
		#	generate orb2inertial velocity vector
		orb2inertial = np.array([0.0, 0.0, -orbital_mot])
		
		init_attit = np.copy(initial_attitude)
		
		#	vector generation function
		vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
			path_data, time_step, len_record,
			sat_orb, sun_orb, earth_orb, site_orb,
			orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
			obs_vec_orb, illum_vec_orb
		)
		
		##################################################
		#	parse new vector data
		
		#	orbital reference vectors
		x_ref_orb_body = vector_data_body[:, 1:4]
		y_ref_orb_body = vector_data_body[:, 4:7]
		z_ref_orb_body = vector_data_body[:, 7:10]
		
		#	geometry vectors
		x_geom_body = vector_data_body[:, 10:13]
		y_geom_body = vector_data_body[:, 13:16]
		z_geom_body = vector_data_body[:, 16:19]
		
		#	earth, velocity and plane normal vectors
		earth_body_data = vector_data_body[:, 19:22]
		sat_body_vel = vector_data_body[:, 22:25]
		orb_norm_body_unit = vector_data_body[:, 25:28]
		
		#	sun, site and rotation vector
		sun_body_data = vector_data_body[:, 28:31]
		site_body_data = vector_data_body[:, 31:34]
		rot_vec_body = vector_data_body[:, 34:37]
		
		###############################################################################################
		#	light curve generation
		###############################################################################################
		
		#	standard distance flag. sets distance to 1000 km when true
		std_dist_flag = True
		
		#	generate sim time
		sim_time = np.linspace(0, duration_sec, len_record)
		
		#	initilise lightcurve output
		final_light_curve = np.zeros((len_record, 4))
		final_light_curve[:, 1] = sim_time
		
		##################################################
		#	generate light curve

		#	run light curve generation function
		light_curve = glim_slc_new(
			sun_body_data, site_body_data, final_light_curve,
			start_jd, body_vectors, body_areas, brdf_data
		)
		
		light_curve_sim_seconds = light_curve[:, 1]

		###############################################################################################
		#	RMS and output
		###############################################################################################

		case_signal = light_curve[:, 2]
		
		#	calculate RMS
		rms = rms_calc(case_signal[0:plot_max], base_signal[0:plot_max])

	#	generate output line
	data_line = [rot_vec_inc_deg, rot_vec_azm_deg, rms]

	return data_line

###################################################################################################
#
#
#
#
#
#
###################################################################################################

def brdf_function(input_line):
	
	#	parse input data
	bot_d_brdf, bot_omega_brdf, bot_m_brdf, \
		side_d_brdf, side_omega_brdf, side_m_brdf,\
		top_d_brdf, top_omega_brdf, top_m_brdf,\
		start_jd, duration_sec, len_record,\
		sun_body_data, site_body_data,\
		body_areas, body_vectors, \
		base_signal, nfaces, start_index, stop_index,\
		top_faces_index, bot_faces_index, side_faces_index = input_line 

	###############################################################################################
	#	regenerate brdf
	###############################################################################################
	
	#	Refractive Index
	top_n_brdf = (1 + np.sqrt(top_omega_brdf)) / (1 - np.sqrt(top_omega_brdf))
	bot_n_brdf = (1 + np.sqrt(bot_omega_brdf)) / (1 - np.sqrt(bot_omega_brdf))
	side_n_brdf = (1 + np.sqrt(side_omega_brdf)) / (1 - np.sqrt(side_omega_brdf))
	
	#	Specular ratio
	top_s_brdf = 1.0 - top_d_brdf
	bot_s_brdf = 1.0 - bot_d_brdf
	side_s_brdf = 1.0 - side_d_brdf
	
	#	generate brdf
	brdf_data = np.zeros([nfaces, 5])
	
	for i in range(nfaces):
	
		#	working on a top face
		if i in top_faces_index:
			brdf_data[i, :] = [top_m_brdf, top_omega_brdf, top_d_brdf, top_s_brdf, top_n_brdf]
		
		#	working on a side face
		elif i in side_faces_index:
			brdf_data[i, :] = [side_m_brdf, side_omega_brdf, side_d_brdf, side_s_brdf, side_n_brdf]
	
		#	working on a bot face
		elif i in bot_faces_index:
			brdf_data[i, :] = [bot_m_brdf, bot_omega_brdf, bot_d_brdf, bot_s_brdf, bot_n_brdf]
	
		else:
			print('BRDF Generation Error, exiting.')
			sys.exit()
	

	###############################################################################################
	#	light curve generation
	###############################################################################################
	
	#	standard distance flag. sets distance to 1000 km when true
	std_dist_flag = True
	
	#	generate sim time
	sim_time = np.linspace(0, duration_sec, len_record)
	
	#	initilise lightcurve output
	final_light_curve = np.zeros((len_record, 4))
	final_light_curve[:, 1] = sim_time
	
	##################################################
	#	generate light curve
	
	#	run light curve generation function
	light_curve = glim_slc_new(
		sun_body_data, site_body_data, final_light_curve,
		start_jd, body_vectors, body_areas, brdf_data
	)
	
	light_curve_sim_seconds = light_curve[:, 1]

	###############################################################################################
	#	RMS and output
	###############################################################################################

	case_signal = light_curve[:, 2]
	
	#	calculate distance
	rms = rms_calc(case_signal[start_index:stop_index], base_signal[start_index:stop_index])

	#	generate output line
	data_line = [
		np.round(bot_d_brdf, 1),
		np.round(bot_omega_brdf, 1),
		np.round(bot_m_brdf, 1),
		rms
	]

	return data_line

