###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Light curve rendering in the orbital frame
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

import random

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	scipy
from scipy import interpolate

#	progress bar
import progressbar

#	transformation
from tools.functions.functions_transform import site_vec, rv2kep, gen_vectors_render2, \
	rot1, rot2, rot3, rot_mat, axis2mat, mat2angles

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	case file generation
from tools.file_generation.case_generation import gen_case

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

##############################################
#	video command
#	ffmpeg -framerate 10 -pattern_type glob -i '*.png' -c:v libx264 -pix_fmt yuv420p ../orb.mp4

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################

#	clear terminal
call(['clear'])
print('Started render.')

#	set start time
sim_start = time.time()
init_start = time.time()

##################################################
#	file paths

root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

# local_input_path = os.path.join('input_dir', 'mmt_files', 'rocket_body')
local_input_path = os.path.join('input_dir', 'mmt_files', 'starlink', 'batch0')


read_path = os.path.join(path_data, local_input_path)

geom_auto_del = True
img_auto_del = True

render_auto_flag = True

###################################################################################################
#	light curve generation inputs
###################################################################################################

#	source mmt file
mmt_file = 'STARLINK-24_44238_MMT_6404.txt'

#	selected track
track_index = 3

#	parse filename
name_str = mmt_file.split('_')[0]
norad_id = int(mmt_file.split('-')[1].split('_')[1])

#	simulation time step
time_step = 0.1

window_start_date = [2010, 1, 1, 0, 0, 0]
window_stop_date = [2019, 11, 1, 0, 0, 0]

###################################################################################################
#	load geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir' + '/starlink'
# geom_dir = 'geometry_dir/geom_dir/' + 'CZ-2D'

geom_str = 'starlink_full_lowdrag'
geom_str = 'starlink_full'
# geom_str = 'cz-2d_testing_double_pointed'

geom_file = geom_str + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
# payload_pointing = np.array([-15.0, 0.0, 0.0])

#	attitude
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 0.0

rot_vec_azm_deg = random.randint(0, 180)
rot_vec_inc_deg = random.randint(0, 180)

# #	correct azm
# if rot_vec_azm_deg >= 180:
# 	rot_vec_azm_deg -= 180
# 
# elif rot_vec_azm_deg < 180:
# 	rot_vec_azm_deg += 180

print('\nAttitude azm angle = ', rot_vec_azm_deg)

#	attitude motion
rot_period = 10.0

#	define rotation axis in body frame as unit vector
rot_vec_body_unit = np.array([1.0, 0.0, 0.0])

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])


##################################################
#	render data

#	directory data
graphics_dir = 'graphics_dir'
render_dir = 'render_dir'
render_geom_dir = 'geom_files'
render_img_dir = 'image_files'

#	orb image data
render_image_dir = os.path.join(path_data, graphics_dir, render_dir, render_img_dir)
render_img_dir_list = os.listdir(render_image_dir)
nimg_files = len(render_img_dir_list)

#	orb image data
render_geom_dir = os.path.join(path_data, graphics_dir, render_dir, render_geom_dir)
render_geom_dir_list = os.listdir(render_geom_dir)
ngeom_files = len(render_geom_dir_list)

#	render outputs
render_data_orb = 'render_data_orb.txt'
render_data_body = 'render_data_body.txt'
render_data_inertial = 'render_data_inertial.txt'

#	render position data
sat_orb_file = 'sat_orb.txt'
sun_orb_file = 'sun_orb.txt'
earth_orb_file = 'earth_orb.txt'
site_orb_file = 'site_orb.txt'

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###############################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

###################################################################################################
#	load MMT file
###################################################################################################

#	set time
mmt_load_start = time.time()

print('\n[{}] Loading MMT data...'.format(datetime.datetime.now()))

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file

#	strip header
header_text = mmt_data[7]
raw_data = mmt_data[8:]

print('\n	Parsing MMT data...')

#	initialise output
n_rows = 14
parsed_data = np.zeros([len(raw_data), n_rows])

for i, line in enumerate(raw_data):

	#	split line
	date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
		distance, phase, channel, track_id = line.split(' ')

	#	parse data and set data types
	mag_std = float(mag_std)
	mag_app = float(mag_app)
	distance = float(distance)
	phase = float(phase)
	track_id = int(track_id)

	#	parse date
	year, month, day = date_str.split('-')
	year = int(year)
	month = int(month)
	day = int(day)

	#	parse time
	hour, minute, second = time_str.split(':')
	hour = int(hour)
	minute = int(minute)
	second = np.round(float(second), 1)

	#	calculate julian date
	jday, jfrac = date2jd(year, month, day, hour, minute, second)

	#	generate output line
	output_line = [
		track_id, norad_id,
		jday, jfrac,
		year, month, day,
		hour, minute, second,
		mag_std, mag_app,
		distance, phase
	]

	#	add to output array
	parsed_data[i, :] = output_line

print('		Total data length = ', len(parsed_data))
print('		Total number of objects = ', len(np.unique(parsed_data[:, 1])))
print('		Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

###################################################################################################
#	extracting data between specified windows

print('\n	Seaching for tracks inside time window:')

#	start window
window_start_jd = np.add(*date2jd(*window_start_date))
print('\n		Window start 	= ', window_start_date[0:3])

#	stop window
window_stop_jd = np.add(*date2jd(*window_stop_date))
print('		Window stop 	= ', window_stop_date[0:3])

#	extract track ids
track_id_list = np.unique(parsed_data[:, 0])
extracted_track_id_list = []

#	loop through tracks
ntracks = 0
first_lc_flag = True
for track_id in track_id_list:

	#	find track ids
	track_indices = np.where(parsed_data[:, 0]==track_id)[0]

	#	extract full data
	track_data = parsed_data[track_indices, :]

	#	extract light curve data
	jd_plot = np.add(track_data[:, 2], track_data[:, 3])
	std_mag_plot = track_data[:, 10]
	app_mag_plot = track_data[:, 11]
	distance_plot = track_data[:, 12]
	phase_plot = track_data[:, 13]
	start_line = track_data[0]
	stop_line = track_data[-1]
	start_jdate = np.add(start_line[2], start_line[3])
	stop_jdate = np.add(stop_line[2], stop_line[3])

	#	ensure between window start/stop
	if start_jdate > window_start_jd and stop_jdate < window_stop_jd:

		#	light curve is within window
		ntracks += 1

		#	if first light curve, start array
		if first_lc_flag is True:
			windowed_data = track_data
			first_lc_flag = False

		#	else append to list
		elif first_lc_flag is False:
			windowed_data = np.vstack([windowed_data, track_data])

		#	add track id to output
		extracted_track_id_list.append(track_id)

print('\n		{} track(s) extracted.'.format(ntracks))
extracted_track_id_list = np.array(extracted_track_id_list)

###############################################################################################
#	extracting requested single track

print('\n		Extracting requested track index {}'.format(track_index))

#	set track array index
track_array_index = track_index - 1
track_id = int(extracted_track_id_list[track_array_index])

track_indices = np.where(windowed_data[:, 0]==track_id)[0]
track_data = windowed_data[track_indices]

###############################################################################################
#	extract track time data

mmt_line = track_data[0]

#	parse line elements
track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = track_data[0]

start_jd = [jday, jfrac]

#	parse line elements
track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = track_data[-1]

stop_jd = [jday, jfrac]

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###############################################################################################
#	display initialisation time
mmt_load_end = time.time()
mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)

print('\n	MMT Load Finished.')
print('\n		- Time taken = {} s.'.format(mmt_load_time))

print('\n#####################################################################################')
print('Light curve parameters: ')
print('	Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_start_date))
print('	Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_stop_date))
print('	Simulation time step:		{}'.format(time_step))
print('	Duration:			{}		[steps]'.format(lc_len_record))
print('					{}		[s]'.format(np.round(lc_duration_sec, 1)))
print('\n#####################################################################################')

###################################################################################################
#	acquire TLE
###################################################################################################

#	set time
tle_start = time.time()

print('\n[{}] Acquiring TLEs...'.format(datetime.datetime.now()))

#	TLE request dates, add day either side to increase tle find chance
tle_start_date = jd2date(*jday_jfrac(lc_start_jd - 1))
tle_stop_date = jd2date(*jday_jfrac(lc_stop_jd + 1)) 

#	generate TLE file name
tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)

#	assemble TLE path
tle_path = os.path.join(path_data, 'input_dir', 'tle_files')

#	if exists, load
if os.path.isfile(os.path.join(tle_path, tle_file)):

	print('\n	Reading existing TLE file.')

	#	load file
	with open(os.path.join(tle_path, tle_file), 'r') as file:
		tle_data = file.readlines()

else:
	print('\n	Requesting TLE file...')

	#	request TLEs
	tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)

	#	save file
	with open(os.path.join(tle_path, tle_file), 'w') as file:
		for line in tle_data:
			file.write(line)

#	grab first TLE pair
tle_line1 = tle_data[1]
tle_line2 = tle_data[2]

#	mean mot rad/sec
orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
orbital_mot = (2 * np.pi) / orbital_period

###############################################################################################
#	display TLE acquisition time
tle_end = time.time()
tle_time = np.round((tle_end - tle_start), 1)

print('\n	TLE Load Finished.')
print('\n		- Time taken = {} s.'.format(tle_time))


###################################################################################################
#	select time data to use
###################################################################################################

sim_range = lc_sim_range
len_record = lc_len_record
start_date = lc_start_date
stop_date = lc_stop_date

###################################################################################################
#	generate eop data array
###################################################################################################

#	EOP data
eop_dir = 'eop_files'
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	assemble eop path
eop_path = os.path.join(path_data, eop_dir)

#	generate interpolated eop_data
eop_log_level = 1
eop_data, dat_data = eop_interpolate(
	eop_path,
	eop_file,
	dat_file,
	sim_range,
	time_step,
	eop_log_level
)

###################################################################################################
#	generate time objects
###################################################################################################

#	start dat value
dat = dat_data[0, -1]
	
#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate start time object
start_time_object = get_times(*start_date, ut_offset, dat)
	
#	end dat value
dat = dat_data[-1, -1]
	
#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate stop time object
stop_time_object = get_times(*stop_date, ut_offset, dat)
	
###################################################################################################
#	generate rotation matrices
###################################################################################################

print('\n[{}]	Generating rotation matrices:\n'.format(datetime.datetime.now()))

#	tt time
jdaytt_start = start_time_object.jday_tt
jfractt_start = start_time_object.jfrac_tt
jdtt_start = jdaytt_start + jfractt_start

#	ut1 time
jdayut1_start = start_time_object.jday_ut1
jfracut1_start = start_time_object.jfrac_ut1
jdut1_start = jdayut1_start + jfracut1_start

#	generate rotation matrices
rot_mat_log_level = 0
pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
	jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
)

###################################################################################################
#	MODULE:	Horizons Positions
###################################################################################################

print('\n[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))

#	set horizons logging level
horz_logging_level = 1

sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
	path_data, horz_logging_level,
	start_time_object, stop_time_object,
	len_record, time_step,
	eop_path
)

#	find indices of light curve data
timecol = sun_icrf[:, 0]
lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]

###################################################################################################
#	MODULE:	Object Position
###################################################################################################

satpos_logging_level = 1

sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
	path_data, satpos_logging_level, len_record, time_step,
	start_time_object, stop_time_object,
	norad_id,
	eop_data, dat_data,
	pm_array, pn_array, st_array, c2t_array
)

###################################################################################################
#	MODULE:	Transformation
###################################################################################################

identification_str = 'mmt-render'

trans_logging_level = 1
save_flag = False

sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
	= module_transformation(
		path_data, trans_logging_level, save_flag,
		start_time_object, stop_time_object,
		identification_str, site_str,
		len_record, time_step,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array,
		sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
	)

###################################################################################################
#	prepare for render file generation
###################################################################################################

###############################################
#	clear render image directory

print('\nChecking for existing Image files...')
if nimg_files != 0:

	if img_auto_del is True:
		for f in render_img_dir_list:
			if f.endswith('.png'):
				os.remove(os.path.join(render_image_dir, f))
		print('	Files deleted')

	else:

		#	ask user to delete files
		check = input('	{:d} image files exist. Delete? [y/n]: '.format(nimg_files))
	
		if check == 'y':
			for f in render_img_dir_list:
				if f.endswith('.png'):
					os.remove(os.path.join(render_image_dir, f))
			print('	Files deleted')
	
		elif check == 'n':
			print('	Files retained,')
			check = input('		Use existing image files? [y/n]: ')
			if check == 'y':
				print('		Continuing with existing image files...')
	
			elif check == 'n':
				print('		Exiting.')
				sys.exit()
			else:
				print('	error in provided answer')
				sys.exit()
		else:
			print('	error in provided answer')
			sys.exit()
else:
	print('\n	Image directory is clean.')

###############################################
#	clear render geometry directory

print('\nChecking for existing Geom files...')
if ngeom_files != 0:

	if geom_auto_del is True:
		for f in render_geom_dir_list:
			if f.endswith('.stl'):
				os.remove(os.path.join(render_geom_dir, f))
		print('	Files deleted')

	else:

		#	ask user to delete files
		check = input('	{:d} geom files exist. Delete? [y/n]: '.format(ngeom_files))
	
		if check == 'y':
			for f in render_geom_dir_list:
				if f.endswith('.stl'):
					os.remove(os.path.join(render_geom_dir, f))
			print('	Files deleted')
	
		elif check == 'n':
			print('	Files retained,')
			check = input('		Use existing geometry files? [y/n]: ')
			if check == 'y':
				print('		Continuing with existing geometry files...')
	
			elif check == 'n':
				print('		Exiting.')
				sys.exit()
			else:
				print('	error in provided answer')
				sys.exit()
		else:
			print('	error in provided answer')
			sys.exit()
else:
	print('\n	Geometry directory is clean.')

###################################################################################################
#	define initial attitude state
###################################################################################################

##################################################
#	attitude

#	convert to radians
rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad = np.radians(rot_vec_inc_deg)

#	generate initial t = 0 attitude
init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])

##################################################
#	attitude motion in body frame

#	generate rotation angle in radians
if rot_period != 0.0:
	rot_ang_per_sec = (np.pi * 2) / rot_period
else:
	rot_ang_per_sec = 0.0

#	generate attitude matrix in body frame
attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)

#	extract euler angles
angles1 = np.array(mat2angles(attit_mat_body)[0:3])
angles2 = np.array(mat2angles(attit_mat_body)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

#	extract attitude motion in the body frame
attit_mot_body = angles1

##################################################
#	generate initial attitude rotation matrix
init_attit_mat = rot_mat(init_attit)

##################################################
#	calculate rotation vector in orbital frame as unit vector
rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)

##################################################
#	define velocity of rotation vector in orbital frame

#	when equal to orbital motion, rot vec is stationary in inertial frame
rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

###################################################################################################
#	perform light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

validation_plot_flag = False
validation_plot_flag = True

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors_render2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb,
	body_mesh, render_geom_dir,
	validation_plot_flag
)

###################################################################################################
#	perform render
###################################################################################################

#	select vector data to use
vector_data = vector_data_body

#	define unit vectors
x_vec = np.array([1.0, 0.0, 0.0])
y_vec = np.array([0.0, 1.0, 0.0])
z_vec = np.array([0.0, 0.0, 1.0])

###############################################
#	initilise figure

# figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
figure3d = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
axes3d = mplot3d.Axes3D(figure3d)

###############################################
#	set title, axis labels and axis limits

axes3d.set_title('Body Frame')

axes3d.set_xlim([-15, 15])
axes3d.set_ylim([-15, 15])
axes3d.set_zlim([-15, 15])

axes3d.set_xlabel('x body axis')
axes3d.set_ylabel('y body axis')
axes3d.set_zlabel('z body axis')

rot_vec_body = np.array([1.0, 0.0, 0.0])

###############################################
#	main loop

print('\nPerforming Render...\n')

for i, vector_line in enumerate(
		progressbar.progressbar(vector_data)
	):

	###############################################################################################
	#	loop initialisation
	###############################################################################################

	###############################################
	#	parse vector render data

	#	julian date
	jdate = vector_data_orb[i, 0]

	#	orbital reference vectors
	x_ref_orb_body = vector_data_body[i, 1:4]
	y_ref_orb_body = vector_data_body[i, 4:7]
	z_ref_orb_body = vector_data_body[i, 7:10]

	#	geometry vectors
	x_geom_body = vector_data_body[i, 10:13]
	y_geom_body = vector_data_body[i, 13:16]
	z_geom_body = vector_data_body[i, 16:19]

	#	earth, velocity and plane normal vectors
	earth_body_vec = vector_data_body[i, 19:22]
	sat_body_vel = vector_data_body[i, 22:25]
	orb_norm_body_unit = vector_data_body[i, 25:28]

	#	sun, site and rotation vector
	sun_body_vec = vector_data_body[i, 28:31]
	site_body_vec = vector_data_body[i, 31:34]
	rot_vec_body_unit = vector_data_body[i, 34:37]

	###############################################
	#	plot figure

	#	generate image name
	if i < 10:
		image_id = '000' + str(i)
	elif 10 <= i < 100:
		image_id = '00' + str(i)
	elif 100 <= i < 1000:
		image_id = '0' + str(i)
	elif i > 1000:
		image_id = str(i)

	image_name = image_id + '_image.png'

	# Clear graphics output
	if i != 0:
		axes3d.cla()

		###############################################
		#	reset title, axis labels and axis limits
		
		axes3d.set_title('Body Frame')
		
		axes3d.set_xlim([-15, 15])
		axes3d.set_ylim([-15, 15])
		axes3d.set_zlim([-15, 15])
		
		axes3d.set_xlabel('x body axis')
		axes3d.set_ylabel('y body axis')
		axes3d.set_zlabel('z body axis')

	###############################################################################################
	#	generate plottable orbital frame axis vectors
	###############################################################################################

	#	orbital frame reference axes scale
	orb_ref_vec_scale = 15

	#	text font size
	font_size = 14

	###############################################
	#	earth vector

	#	generate plottable vector
	earth_vec_plot = orb_ref_vec_scale * np.divide(
		np.vstack((origin, earth_body_vec)), np.linalg.norm(earth_body_vec)
	)

	#	create plot arrow and add to plot
	a = Arrow3D(earth_vec_plot[:, 0], earth_vec_plot[:, 1], earth_vec_plot[:, 2],
		mutation_scale=20, lw=2, arrowstyle="-|>", color="g"
	)

	axes3d.add_artist(a)

	#	add text
	axes3d.text(earth_vec_plot[1, 0], earth_vec_plot[1, 1], earth_vec_plot[1, 2],
		'Earth Vec', fontsize=font_size, fontweight='bold'
	)

	###############################################
	#	velocity vector

	#	generate plottable vector
	vel_vec_plot = orb_ref_vec_scale * np.divide(
		np.vstack((origin, sat_body_vel)), np.linalg.norm(sat_body_vel)
	)

	#	create plot arrow and add to plot
	a = Arrow3D(vel_vec_plot[:, 0], vel_vec_plot[:, 1], vel_vec_plot[:, 2],
		mutation_scale=20, lw=2, arrowstyle="-|>", color="g"
	)

	axes3d.add_artist(a)

	#	add text
	axes3d.text(vel_vec_plot[1, 0], vel_vec_plot[1, 1], vel_vec_plot[1, 2],
		'Velocity Vec', fontsize=font_size, fontweight='bold'
	)

	###############################################
	#	orbital plane normal

	#	generate plottable vector
	orb_norm_unit_plot = orb_ref_vec_scale * np.divide(
		np.vstack((origin, orb_norm_body_unit)), np.linalg.norm(orb_norm_body_unit)
	)

	#	create plot arrow and add to plot
	a = Arrow3D(orb_norm_unit_plot[:, 0], orb_norm_unit_plot[:, 1], orb_norm_unit_plot[:, 2],
		mutation_scale=20, lw=2, arrowstyle="-|>", color="g"
	)

	axes3d.add_artist(a)

	#	add text
	axes3d.text(orb_norm_unit_plot[1, 0], orb_norm_unit_plot[1, 1], orb_norm_unit_plot[1, 2],
		'Orb Plane Normal', fontsize=font_size, fontweight='bold'
	)

	###############################################################################################
	#	generate plottable light curve generation vectors
	###############################################################################################

	lc_gen_vec_scale = 10

	###############################################
	#	illumination vector

	#	generate plottable vector
	illum_vec_plot = lc_gen_vec_scale * np.divide(
		np.vstack((origin, sun_body_vec)), np.linalg.norm(sun_body_vec)
	)

	#	create plot arrow and add to plot
	a = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2],
		mutation_scale=20, lw=2, arrowstyle="-|>", color="r"
	)

	axes3d.add_artist(a)

	#	add text
	axes3d.text(illum_vec_plot[1, 0], illum_vec_plot[1, 1], illum_vec_plot[1, 2],
		'Illum Vec', fontsize=font_size, fontweight='bold'
	)

	###############################################
	#	observation vector

	#	generate plottable vector
	obs_vec_plot = lc_gen_vec_scale * np.divide(
		np.vstack((origin, site_body_vec)), np.linalg.norm(site_body_vec)
	)

	#	create plot arrow and add to plot
	a = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2],
		mutation_scale=20, lw=2, arrowstyle="-|>", color="b"
	)

	axes3d.add_artist(a)

	#	add text
	axes3d.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2],
		'Obs Vec', fontsize=font_size, fontweight='bold'
	)

	###############################################
	#	rotation vector

	#	generate plottable vector
	rot_vec_plot = lc_gen_vec_scale * np.divide(
		np.vstack((origin, rot_vec_body_unit)), np.linalg.norm(rot_vec_body_unit)
	)

	#	create plot arrow and add to plot
	a = Arrow3D(rot_vec_plot[:, 0], rot_vec_plot[:, 1], rot_vec_plot[:, 2],
		mutation_scale=20, lw=2, arrowstyle="-|>", color="m"
	)

	axes3d.add_artist(a)

	#	add text
	axes3d.text(rot_vec_plot[1, 0], rot_vec_plot[1, 1], rot_vec_plot[1, 2],
		'Rot Vec', fontsize=font_size, fontweight='bold'
	)

	###############################################################################################
	#	generate plottable body frame axis vectors
	###############################################################################################

	body_axes_scale = 5

	###############################################
	#	add body x axis
	bodyx_vec_plot = body_axes_scale * np.vstack((origin, x_geom_body))

	a = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
	)

	axes3d.add_artist(a)

	axes3d.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1], bodyx_vec_plot[1, 2],
		'x_b', fontsize=font_size, fontweight='bold'
	)

	###############################################
	#	add body y axis
	bodyy_vec_plot = body_axes_scale * np.vstack((origin, y_geom_body))

	a = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
	)

	axes3d.add_artist(a)

	axes3d.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
		'y_b', fontsize=font_size, fontweight='bold'
	)

	###############################################
	#	add body z axis
	bodyz_vec_plot = body_axes_scale * np.vstack((origin, z_geom_body))

	a = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
	)

	axes3d.add_artist(a)

	axes3d.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
		'z_b', fontsize=font_size, fontweight='bold'
	)

	###############################################################################################
	#	plot mesh
	###############################################################################################

	#	import mesh
	vectors = body_mesh.vectors
	normals = body_mesh.normals

	#	set mesh alpha
	alpha = 0.5

	for vector, normal in zip(vectors, normals):
	
		#	parse input vectors and calculate facet normal
		vect12 = vector[1] - vector[0]
		vect23 = vector[2] - vector[1]
		normal = np.cross(vect12, vect23)
		normal_unit = normal / np.linalg.norm(normal)

		#	sun angle
		sun_ang = np.arccos(
			np.dot(normal, sun_body_vec)
			/ (np.linalg.norm(normal) *  np.linalg.norm(sun_body_vec))
		)

		#	site angle
		site_ang = np.arccos(
			np.dot(normal, site_body_vec)
			/ (np.linalg.norm(normal) * np.linalg.norm(site_body_vec))
		)

		###############################################
		#	rending

		#	Add final vert to vector array
		vector = np.vstack([vector, vector[0, :]])

		#	extract x, y and z values
		xs = vector[:,0]
		ys = vector[:,1]
		zs = vector[:,2]

		#	generate vertices list
		verts = [list(zip(xs, ys, zs))]

		#	ensure face is observable and illuminated
		if sun_ang < (0.5 * np.pi) and site_ang < (0.5 * np.pi):
			facecol = [np.absolute(np.cos(sun_ang)), np.absolute(np.cos(sun_ang)), \
					np.absolute(np.cos(sun_ang))]

		#	else set face colour dark
		else:
			facecol = [0.1, 0.1, 0.1]

		#	set facet alpha
		alpha = 0.75

		#	add facet to plot
		pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
		pc.set_alpha(alpha)
		pc.set_edgecolor([0.5, 0.5, 0.5])
		pc.set_facecolor(facecol)
		
		axes3d.add_collection3d(pc)

#	###############################################################################################
#	#	finalise plot
#	###############################################################################################
#
#	#	azimuth of observation vector
#	xy_comp = np.array([obs_vec_used[0], obs_vec_used[1], 0.0])
#
#	azimuth = np.arccos(
#		np.divide(
#			np.dot(xy_comp, x_vec),
#			np.linalg.norm(xy_comp) * np.linalg.norm(x_vec)
#		)
#	)
#
#	if obs_vec_used[1] < 0:
#		azimuth = -azimuth
#
#	#	elevation of observation vector
#	temp = rot3(obs_vec_used, azimuth)
#	xz_comp = np.array([obs_vec_used[0], 0.0, obs_vec_used[2]])
#
#	elevation = np.arccos(
#		np.divide(
#			np.dot(xz_comp, z_vec),
#			np.linalg.norm(xz_comp) * np.linalg.norm(z_vec)
#		)
#	)
#
#	elevation = (np.pi / 2) - elevation

	###############################################
	#	set view direction
	view_azimuth = np.radians(-45)
	view_elevation = np.radians(25)

	axes3d.view_init(np.degrees(view_elevation), np.degrees(view_azimuth))

#	plt.axis('off')
#	plt.show()
#	sys.exit()

	plt.pause(0.0001)

#	###############################################################################################
#	#	save output image
#	###############################################################################################
#
#	#	save single figure
#	figure3d.savefig(os.path.join(render_image_dir, image_name))

print('\nRender complete.')

#############################################################################################
#	delete geom files
for f in render_geom_dir_list:
	if f.endswith('.stl'):
		os.remove(os.path.join(render_geom_dir, f))
