###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	parrellisation
from multiprocessing import Pool
import tqdm
from mmt_parallel_functions import attitude_polar_function

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('MMT testing script initilised.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

##################################################
#	simulation parameters
time_step = 0.1





###################################################################################################
#	load MMT file					LOCKED
###################################################################################################

read_path = os.path.join(path_data, 'input_dir', 'mmt_files')

mmt_files = [
	'cz-2d_41858_11-02-17_03-02-59.txt',
	'cz-4_25942_23-09-2019_00-42-27.txt',
	'cz-6_40913_22-11-2015_15-04-54.txt',
	'cz-11_41847_20-11-2016_14:51:49.txt',
	'cz-11_44317_08-08-2019_00-36-44.txt',
	'DELTA-2_28476_22-09-2017_01-57-59.txt',
	'DELTA-2_32261_15-05-2019_19-42-42.txt',
	'DELTA-2_32712_07-07-2019_23-39-38.txt',
	'ELECTRON_43166_17-07-2018_23-47-40.txt',
	'N-1_10675_13-08-2019_00-56-18.txt',
	'UNHA-3_41333_31-03-2016_17-23-44.txt'
]

	
fig, ax = plt.subplots()
	
	
for mmt_file in mmt_files:

	ax.cla()

	print('\nShowing plot for: \n\n	', mmt_file)

	name_str, norad_id, date_str, time_str = mmt_file.split('_')
	norad_id = int(norad_id)
	
	with open(os.path.join(read_path, mmt_file)) as read_file:
		mmt_data = read_file.readlines()
	
	###################################################################################################
	#	parse MMT file					LOCKED
	###################################################################################################
	
	#	header line
	header_text = mmt_data[7]
	
	#	strip header
	mmt_data = mmt_data[8:]
	
	###################################################################################################
	#	extract start jd
	
	mmt_line = mmt_data[0]
	
	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')
	
	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)
	
	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')
	
	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)
	
	mmt_time = np.array([hour, minute, second])
	
	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])
	
	#	calculate julian date
	jd = date2jd(*datetime_array)
	start_jd = jd
	
	###################################################################################################
	#	extract stop jd
	
	mmt_line = mmt_data[-1]
	
	#	parse line elements
	mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
		penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')
	
	#	split date
	mmt_date = np.array(mmt_date.split('-')).astype(np.int)
	
	#	split time
	raw_hour, raw_minute, raw_second = mmt_time.split(':')
	
	#	set data types
	hour = np.int(raw_hour)
	minute = np.int(raw_minute)
	second = np.round(np.float(raw_second), 1)
	
	mmt_time = np.array([hour, minute, second])
	
	#	create datetime array
	datetime_array = np.hstack([mmt_date, mmt_time])
	
	#	calculate julian date
	jd = date2jd(*datetime_array)
	stop_jd = jd
	
	###################################################################################################
	#	generate simulation time data
	
	#	extract start and stop jds
	lc_start_jd = start_jd[0] + start_jd[1]
	lc_stop_jd = stop_jd[0] + stop_jd[1]
	
	#	convert to dates
	lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
	lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
	
	#	light curve duration
	lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
	
	#	set time step
	lc_len_record = int(lc_duration_sec / time_step) + 1
	
	#	time range in julian days
	lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
	sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)
	
	###################################################################################################
	#	extract MMT data
	
	sim_index = 0
	mmt_data_parsed = []
	
	#	parse MMT data
	for itt, mmt_line in enumerate(mmt_data):
	
		#	parse line elements
		mmt_date, mmt_time, std_mag, app_mag, opt_filter, \
			penumbra, distance, phase, channel = mmt_line.rstrip('\n').split(' ')
	
		#	split date
		mmt_date = np.array(mmt_date.split('-')).astype(np.int)
	
		#	split time
		raw_hour, raw_minute, raw_second = mmt_time.split(':')
	
		#	set data types
		hour = np.int(raw_hour)
		minute = np.int(raw_minute)
		second = np.round(np.float(raw_second), 1)
	
		#	generate time array
		mmt_time = np.array([hour, minute, second])
	
		#	create datetime array
		datetime_array = np.hstack([mmt_date, mmt_time])
	
		#	calculate julian date
		jd = date2jd(*datetime_array)
		step_jd = jd[0] + jd[1]
	
		#	calculate current seconds
		current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
	
		#	set light curve data datatypes
		std_mag = np.float(std_mag)
		app_mag = np.float(app_mag)
		distance = np.float(distance)
		phase = np.float(phase)
	
		#	generate light curve data array
		mmt_lc_data = [std_mag, app_mag, distance, phase]
			
		#	assemble line data
		line_data = np.hstack([jd, current_sec, mmt_lc_data])
	
		##################################################
		#	extracting data of time step
	
		#	data match found
		if current_sec == sim_range_sec[sim_index]:
			mmt_data_parsed.append(line_data)
			sim_index += 1
	
		#	data point is not required
		elif current_sec < sim_range_sec[sim_index]:
			pass
	
		#	interpolation required
		elif current_sec > sim_range_sec[sim_index]:
	
			#	set previous data values
			jd_prev = prev_line[0]
			jdfrac_prev = prev_line[1]
			prev_sec = prev_line[2]
	
			#	set previous light curve data
			std_mag_prev = prev_line[-4]
			app_mag_prev = prev_line[-3]
			distance_prev = prev_line[-2]
			phase_prev = prev_line[-1]
	
			#	calculate interpolation requirements
			gap = np.round(current_sec - prev_sec, 1)
			npts = int(gap / time_step)
	
			#	get current light curve data
			std_mag_last = std_mag
			app_mag_last = app_mag
			distance_last = distance
			phase_last = phase
	
			#	calculate linear interpolation step values
			std_mag_step = (std_mag_last - std_mag_prev) / npts
			app_mag_step = (app_mag_last - app_mag_prev) / npts
			distance_step = (distance_last - distance_prev) / npts
			phase_step = (phase_last - phase_prev) / npts
	
			#	fill first line
			interp_data = np.zeros([npts, len(line_data)])
	
			#	calculate interpolation data
			for n in range(1, len(interp_data) + 1):
	
				#	current simulation second
				interp_sec = np.round(prev_sec + (n * time_step), 1)
	
				#	time data
				jdate = (jd_prev + jdfrac_prev) + (n * (0.1 / 86400))
				jd = jday_jfrac(jdate)
	
				#	calculate interpolated light curve data
				std_mag = std_mag_prev + (n * std_mag_step)
				app_mag = app_mag_prev + (n * app_mag_step)
				distance = distance_prev + (n * distance_step)
				phase = phase_prev + (n * phase_step)
	
				mmt_lc_data = [std_mag, app_mag, distance, phase]
						
				#	assemble line data
				line_data = np.hstack([jd, interp_sec, mmt_lc_data])
	
				#	add to output
				interp_data[n - 1, :] = line_data
	
			#	find required lines in interpolated data
			for interp_line in interp_data:
	
				#	set current simulation second
				interp_line_sec = interp_line[2]
	
				if interp_line_sec == sim_range_sec[sim_index]:
					mmt_data_parsed.append(line_data)
					sim_index += 1
	
		#	save previous line
		if itt != 0:
			prev_line = line_data
			prev_time = mmt_time
	
	#	ensure parsed MMT data matches simulation data
	mmt_data_parsed = np.array(mmt_data_parsed)
	
	if len(mmt_data_parsed) == len(lc_sim_range):
		print('\nSimulation and MMT data matches...')
	else:
		print('\nError: Simulation and MMT data does not match...')
		sys.exit()
	
	mmt_time = mmt_data_parsed[:, 2]
	mmt_signal_std = mmt_data_parsed[:, 3]
	mmt_signal_app = mmt_data_parsed[:, 4]

	avg_val = np.average(mmt_signal_std)

	mmt_signal_plot = np.subtract(mmt_signal_std, avg_val)
	
	ax.plot(mmt_time, mmt_signal_plot, label=name_str + ' Std')

	plt.pause(5)

# 	plt.show()
# 	sys.exit()


ax.legend()
plt.show()
sys.exit()
