###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#
#	Generates access times for a satellite
#	Inputs:	NORAD_ID, Time_window
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	progress bar
import progressbar

#	parrellisation
from multiprocessing import Pool
import tqdm

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	import from transformation functions
from tools.functions.functions_transform import site_vec

#	import from general function list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, \
	get_times

#	import modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_custom_orbit, \
	module_transformation

from function_access import access_fun

###################################################################################################
#	inputs
###################################################################################################
#	clear terminal
call(['clear'])

print('\n[{}] GLIMMA access generation script initilised.'.format(datetime.datetime.now()))

#	get simulation start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set lightcurve output directory
lightcurve_path = os.path.join(path_data, 'lightcurve_dir', 'data_files')

###################################################################################################
#	simulation parameters
###################################################################################################

parral_flag = False

logging_level = 0
horz_logging_level = 0
	
ncores = 8

##################################################
#	time date

#	start time
start_date = np.array([2016, 1, 1, 0, 0, 0])

#	time step in seconds
time_step = 60

#	length of data record [number of points]
days_record = 365
len_record = (int(86400 / time_step) * days_record) + 1

##################################################
#	site data

site_latitude = 0.0
site_longitude = 0.0
site_altitude = 0.0

##################################################
#	object data

#	norad id
norad_id = 1575
norad_flag = False

#	define inclination
inclination_range = np.linspace(0, 90, 10)
# inclination_range = [50]

#	define right ascension
right_ascension_range = np.arange(0, 180, 20)
# right_ascension_range = [0]

#	constant parameters
argument_periapse = 0.0
true_anomaly = 0.0
eccentricity = 0.0

#	define orbital altitude
orbital_height = 1000

#	collate object inputs
object_data = [
	argument_periapse,
	true_anomaly,
	eccentricity,
	orbital_height,
	norad_id,
	norad_flag
]

###################################################################################################
#	parse simulation parameters
###################################################################################################

##################################################
#	simulation time parameters

#	start
start_jday, start_jdfrac = date2jd(*start_date)
start_jdate = start_jday + start_jdfrac

#	duration
duration_sec = time_step * (len_record - 1)
duration_days = duration_sec / 86400

#	stop
stop_jdate = start_jdate + duration_days
stop_date = jd2date(*jday_jfrac(stop_jdate))

#	time range in days
sim_range = np.linspace(start_jdate, stop_jdate, len_record)

#	time range in seconds
sim_time = np.linspace(0, duration_sec, len_record)

print('\n#####################################################################################')
print('\n	Time parameters: ')
print('		Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*start_date))
print('		Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*stop_date))
print('		Simulation time step:		{}'.format(time_step))
print('		Duration:			{}		[steps]'.format(len_record))
print('						{}	[s]'.format(np.round(duration_sec, 1)))
print('						{}	[m]'.format(np.round(duration_sec / 60, 1)))
print('						{}		[days]'.format(np.round(duration_sec / 60 / 60 / 24, 1)))

##################################################
#	simulation site parameters
#	site latitude / longitude
site_latgd = np.radians(site_latitude)
site_lon = np.radians(site_longitude)
site_alt = site_altitude

#	generate unique site indentifier
site_str = 'lat{}lon{}'.format(int(site_latitude), int(site_longitude))

#	site itrf vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

print('\n	Site parameters: ')
print('		Site Str:			{}'.format(site_str))
print('		Site Latitude:			{}	deg'.format(site_latitude))
print('		Site Longitude:			{}	deg'.format(site_longitude))
print('		Site Altitude:			{}	m'.format(site_altitude))
print('\n#####################################################################################')

###################################################################################################
#	initialisation
###################################################################################################

##################################################
#	generate eop data array

print('\n[{}]	Generating EOP data array...'.format(datetime.datetime.now()))

#	EOP data
eop_dir = 'eop_files'
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	assemble eop path
eop_path = os.path.join(path_data, eop_dir)

#	generate interpolated eop_data
eop_log_level = 1
eop_data, dat_data = eop_interpolate(eop_path, eop_file, dat_file, sim_range, time_step, eop_log_level)

##################################################
#	generate time objects

#	start dat value
dat = dat_data[0, -1]
	
#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate start time object
start_time_object = get_times(*start_date, ut_offset, dat)
	
#	end dat value
dat = dat_data[-1, -1]
	
#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate stop time object
stop_time_object = get_times(*jd2date(*jday_jfrac(stop_jdate)), ut_offset, dat)
	
#	generate unique file name identifiers
start_date_str = '{:4d}-{:02d}-{:02d}'.format(start_date[0], start_date[1], start_date[2])
start_time_str = '{:02d}-{:02d}-{:04.1f}'.format(start_date[3], start_date[4], start_date[5])
stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])
stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])

###################################################################################################
#	MODULE:	Horizons Positions
###################################################################################################

print('\n[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))

sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
	path_data, horz_logging_level,
	start_time_object, stop_time_object,
	len_record, time_step,
	eop_path
)

###################################################################################################
#	generate rotation matrices
###################################################################################################

print('\n[{}]	Generating rotation matrices:\n'.format(datetime.datetime.now()))

#	tt time
jdaytt_start = start_time_object.jday_tt
jfractt_start = start_time_object.jfrac_tt
jdtt_start = jdaytt_start + jfractt_start

#	ut1 time
jdayut1_start = start_time_object.jday_ut1
jfracut1_start = start_time_object.jfrac_ut1
jdut1_start = jdayut1_start + jfracut1_start

#	generate rotation matrices
pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
	jdtt_start, jdut1_start, eop_data, time_step
)
	
###################################################################################################
#	loop on inclination
###################################################################################################

print('\n[{}]	Starting parallelised loop:\n'.format(datetime.datetime.now()))

###################################################################################################
#	generate time data array

time_data = [
	time_step,
	len_record,
	start_date_str,
	start_time_str,
	start_date,
	start_jdate,
	stop_jdate
]

###################################################################################################
#	generate input data array
input_data = []

for right_ascension in right_ascension_range:
	for inclination in inclination_range:
	
		input_line = [
			path_data,
			inclination, right_ascension, object_data,
			site_str, site_data,
			time_data,
			eop_data, dat_data,
			pm_array, pn_array, st_array, c2t_array,
			sun_gcrf, earth_gcrf,
			logging_level
		]
	
		input_data.append(input_line)

###################################################################################################
#	parrallised loop

if parral_flag is True:
	
	#	initialise multiproccessing pool of workers
	p = Pool(ncores)
	
	#	map the total data array across the access function
	for light_curve_data in tqdm.tqdm(p.imap_unordered(access_fun, input_data), total=len(input_data)):

		#	parse outputs
		lightcurve_file, final_output_array = light_curve_data
	
		#	set output file and path
		np.savetxt(os.path.join(lightcurve_path, lightcurve_file), final_output_array, delimiter=',')

elif parral_flag is False:

	#	run inputs sequentially
	for input_line in input_data:

		lightcurve_file, final_output_array = access_fun(input_line)

		print('\n	Output lightcurve data file: ', lightcurve_file)
		print('\n	Output array:	', final_output_array)
		print('\n	Output array Shape:	', final_output_array.shape)
		print('\n	Output array Len:	', len(final_output_array))
		sys.exit()

###############################################################################################
#	writing lightcurve files
###############################################################################################

#	calculate sim time
sim_end = time.time()
sim_time = np.round((sim_end - sim_start) / 60, 1)

print('\n[{}] Simulation Finished.'.format(datetime.datetime.now()))
print('\n	- Time taken = {} mins.'.format(sim_time))
