###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	progress bar
import progressbar

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	import from transformation functions
from tools.functions.functions_transform import site_vec, rv2kep

#	import from general function list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, \
	get_times

##	import modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_custom_orbit, \
	module_transformation

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

###################################################################################################
#	inputs
###################################################################################################

def access_fun(input_str):

	##################################################
	#	parse input_string
	path_data, inclination, right_ascension, object_data, site_str, site_data, \
		time_data, eop_data, dat_data, pm_array, pn_array, st_array, c2t_array, \
		sun_gcrf, earth_gcrf, logging_level = input_str

	##################################################
	#	parse time data
	time_step, len_record, start_date_str, start_time_str, start_date, start_jdate, stop_jdate \
		= time_data

	##################################################
	#	generate time objects

	#	time range in days
	sim_range = np.linspace(start_jdate, stop_jdate, len_record)

	#	start dat value
	dat = dat_data[0, -1]
		
	#	start eop values
	eop_line = eop_data[0, :]
	eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
		
	#	generate start time object
	start_time_object = get_times(*start_date, ut_offset, dat)
		
	#	end dat value
	dat = dat_data[-1, -1]
		
	#	end eop values
	eop_line = eop_data[-1, :]
	eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
		
	#	generate stop time object
	stop_time_object = get_times(*jd2date(*jday_jfrac(stop_jdate)), ut_offset, dat)
		
	##################################################
	#	parse object data
	argument_periapse, true_anomaly, eccentricity, orbital_height, norad_id, norad_flag = object_data

	norad_flag = True

	if logging_level != 0:
		print('\n#####################################################################################')
		print('\n	Object orbit parameters: ')
		print('		RAAN =				{}	deg'.format(right_ascension))
		print('		ArgP =				{}	deg'.format(argument_periapse))
		print('		True Anom =			{}	deg'.format(true_anomaly))
		print('		Incl =				{}	deg'.format(inclination))
		print('		Ecce =				{}	deg'.format(eccentricity))
		print('		Orbital Altitude =		{}	km'.format(orbital_height))
		print('\n################################################################################')

	##################################################
	#	parse site data
	site_itrf = site_data[0:3]
	site_latgd, site_lon, site_alt = site_data[3:6]

	#	get loop start time
	loop_start = time.time()
	
	#	generate unique object identifier
	object_id = 'i{}h{}raan{}'.format(int(inclination), int(orbital_height), int(right_ascension))

	##################################################
	#	output light curve data file

	lightcurve_file = ''.join([
		site_str + '_',
		str(object_id) + '_',
		'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
		'.txt'
	])

	###############################################################################################
	#	MODULE:	Object Positions
	###############################################################################################
	
	if norad_flag is True:
	
		###########################################################################################
		#	using SGP4
		sat_gcrf_sgp4, sat_itrf_sgp4 = module_sat_pos(
			path_data, logging_level, len_record, time_step,
			start_time_object, stop_time_object,
			norad_id,
			eop_data, dat_data,
			pm_array, pn_array, st_array, c2t_array
		)

	elif norad_flag is False:
	
		##################################################
		#	simulation object parameters
		
		#	earth radius
		r_earth = 6371
		
		#	parse keplerian elements
		raan = np.radians(right_ascension)
		argp = np.radians(argument_periapse)
		true_anom = np.radians(true_anomaly)
		inc = np.radians(inclination)
		ecc = eccentricity
		semi_major_axis = r_earth + orbital_height

		###########################################################################################
		#	generating a custom orbit
		sat_gcrf, sat_itrf, orbital_elements = module_custom_orbit(
			path_data, logging_level, len_record, time_step,
			start_time_object, stop_time_object,
			object_id,
			eop_data, dat_data,
			pm_array, pn_array, st_array, c2t_array,
			raan, argp, true_anom, inc, ecc, semi_major_axis
		)

	print(sun_gcrf[:, 7:].shape)
	print(earth_gcrf[:, 7:].shape)
	print(sat_gcrf_sgp4.shape)
	
	sys.exit()

	###############################################################################################
	#	MODULE:	Transformation
	###############################################################################################
	sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
		= module_transformation(
			path_data, logging_level,
			start_time_object, stop_time_object,
			object_id, site_str,
			len_record, time_step,
			eop_data, dat_data,
			pm_array, pn_array, st_array, c2t_array,
			sat_gcrf, sun_gcrf, earth_gcrf, site_data
		)
	
	###############################################################################################
	#	extract viable light curves
	###############################################################################################
	
	if logging_level != 0:
		print('\n[{}]	Extracting viable light curve data points...'.format(datetime.datetime.now()))
	
	#	viability requirements
	min_sun_ele = np.radians(-10.0)
	min_sat_ele = np.radians(20.0)
	sun_site_phase_max = np.radians(90.0)
	min_lc_duration = time_step * 3
	
	#	prepare for loop
	first_record = True
	data_record = False
	data_end = False
	
	durations = []
	light_curve_index = 1

	#	set loop data
	loop_data = zip(sun_azel, sat_azel, sat_orb, sun_orb, earth_orb, site_orb, sat_itrf)
	
	for loop_line in loop_data:
	
		###########################################################################################
		#	parse inputs
	
		sun_azel_line, sat_azel_line, sat_orb_line, \
			sun_orb_line, earth_orb_line, site_orb_line, sat_itrf_line = loop_line
	
		#	azel data
		jd_time, sat_r, sat_az, sat_el = sat_azel_line
		_, sun_r, sun_az, sun_el = sun_azel_line
	
		#	orbital vectors
		sun_vec_orb = sun_orb_line[1:4]
		earth_vec_orb = earth_orb_line[1:4]
		site_vec_orb = site_orb_line[1:4]
	
		#	satellite data
		sat_vec_orb = sat_orb_line[1:4]
		sat_vel_orb = sat_orb_line[4:7]
	
		#	earth radial vector
		sat_r = np.linalg.norm(earth_vec_orb)
	
		#	sat itrf position
		sat_itrf_pos = sat_itrf_line[1:4]
		sat_itrf_vel = sat_itrf_line[4:7]
	
		#	generate output data
		temp = [
			light_curve_index, jd_time, \
			sun_r, sun_az, sun_el, \
			sat_r, sat_az, sat_el
		]
	
		#	assemble data line
		data_line = np.hstack([
			temp,
			sun_vec_orb,
			earth_vec_orb,
			site_vec_orb,
			sat_vec_orb,
			sat_vel_orb,
			sat_itrf_pos,
			sat_itrf_vel
		])
	
		###########################################################################################
		#	minimum possible phase angle such that the sun does not obscure the object
	
		#	illuminated condition
		sun_earth_phase_min = np.arcsin(r_earth / sat_r)
	
		#	sun-earth phase angle
		sun_earth_phase = np.arccos(
			np.divide(
				np.dot(sun_vec_orb, earth_vec_orb),
				np.linalg.norm(sun_vec_orb) * np.linalg.norm(earth_vec_orb)
			)
		)
	
		#	sun-site phase angle
		sun_site_phase = np.arccos(
			np.divide(
				np.dot(sun_vec_orb, site_vec_orb),
				np.linalg.norm(sun_vec_orb) * np.linalg.norm(site_vec_orb)
			)
		)
	
		###############################################################################################
		#	finding light curve start time
	
		#	no ongoing light curve
		if data_record is False:
	
			#	sun above horizon
			if sun_el >= min_sun_ele:
				pass
			else:
				#	sat below horizon
				if sat_el <= min_sat_ele:
					pass
		
				else:
					#	sat not illuminated
					if sun_earth_phase <= sun_earth_phase_min:
						pass
	
					#	below maximum phase angle
					elif sun_site_phase < sun_site_phase_max:
	
						###########################################################
						#	viable light curve data start point
	
						#	set start jd
						lc_start_jd = data_line[1]
	
						#	initialise light curve array
						light_curve_array = [data_line]
	
						#	set data recording flag to True
						data_record = True
	
						#	set jd of previous time step
						lc_prev_jd = data_line[1]
	
	
		###########################################################################################
		#	finding end of ongoing light curve, else add ongoing point
	
		#	ongoing light curve
		elif data_record is True:
	
			#	sun below horizon
			if sun_el >= min_sun_ele:
				data_end = True
	
			else:
				#	sat above horizon
				if sat_el <= min_sat_ele:
					data_end = True
	
				else:
					#	sat is illuminated
					if sun_earth_phase <= sun_earth_phase_min:
						data_end = True
	
					#	below maximum phase angle
					elif sun_site_phase < sun_site_phase_max:
	
						###########################################################
						#	viable light curve data point
	
						#	append to output array
						light_curve_array.append(data_line)
	
						#	set jd of previous time step
						lc_prev_jd = data_line[1]
	
	
		###########################################################################################
		#	finalise light curve
	
		if data_end is True:
	
			#	light curve duration
			lc_duration = np.round(86400 * (lc_prev_jd - lc_start_jd), 1)
	
			#	ensure required duration
			if lc_duration >= min_lc_duration:
	
				#	add to durations array
				durations.append(lc_duration)
	
				if first_record is True:
	
					#	initialise final output array
					light_curve_array = np.array(light_curve_array)
					final_output_array = light_curve_array
	
					#	set first flag to false
					first_record = False
				
				elif first_record is False:
	
					#	stack light curve array with output array
					final_output_array = np.vstack([final_output_array, light_curve_array])
	
#				print('\n				Light Curve Extracted. \
#					\n					Index = {}, \
#					\n					Duration = {} s / {} m'.format(
#					light_curve_index,
#					np.round(lc_duration, 1),
#					np.round(lc_duration / 60, 3)
#				))
	
				#	itterate light curve index
				light_curve_index += 1
	
			#	reset data recording flag to False
			data_record = False
			data_end = False

	num_lc = light_curve_index - 1

	###############################################################################################
	#	output
	###############################################################################################
	
	if num_lc == 0:
	
		#	calculate sim time
		loop_end = time.time()
		loop_time = np.round((loop_end - loop_start) / 60, 1)
	
		if logging_level != 0:
			print('\n[{}] Loop Finished.'.format(datetime.datetime.now()))
			print('\n	- Time taken = {} mins.'.format(loop_time))
			print('	- No Light Curves Found.')

		final_output_array = np.zeros([3, 3])

	else:
	
		#	calculate sim time
		loop_end = time.time()
		loop_time = np.round((loop_end - loop_start) / 60, 1)
		avg_duration = np.round(np.average(np.array(durations)), 1)
	
		if logging_level != 0:
			print('\n[{}] Loop Finished.'.format(datetime.datetime.now()))
			print('\n\n		- Time taken = {} mins.'.format(loop_time))
			print('			- Light Curves Generated = {}'.format(light_curve_index - 1))
			print('			- Average Duration = {} s / {} m'.format(
				avg_duration,
				np.round(avg_duration / 60, 3))
			)
		
	return [lightcurve_file, final_output_array]
