###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Light curve rendering in the orbital frame
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	scipy
from scipy import interpolate

#	progress bar
import progressbar

#	transformation
from tools.functions.functions_transform import site_vec, rv2kep, gen_vectors_render, \
	rot1, rot2, rot3, rot_mat

#	function list
from tools.functions.functions_list import object_gen, date2jd, jd2date, get_times, interp

#	case file generation
from tools.file_generation.case_generation import gen_case

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

##############################################
#	video command
#	ffmpeg -framerate 10 -pattern_type glob -i '*.png' -c:v libx264 -pix_fmt yuv420p ../orb.mp4

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################

#	clear terminal
call(['clear'])

print('Started render.')

##################################################
#	file paths

root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

##################################################
#	render data

#	directory data
graphics_dir = 'graphics_dir'
render_dir = 'render_dir'
render_geom_dir = 'geom_files'
render_img_dir = 'image_files'

#	orb image data
render_image_dir = os.path.join(path_data, graphics_dir, render_dir, render_img_dir)
render_img_dir_list = os.listdir(render_image_dir)
nimg_files = len(render_img_dir_list)

#	orb image data
render_geom_dir = os.path.join(path_data, graphics_dir, render_dir, render_geom_dir)
render_geom_dir_list = os.listdir(render_geom_dir)
ngeom_files = len(render_geom_dir_list)

#	render outputs
render_data_orb = 'render_data_orb.txt'
render_data_body = 'render_data_body.txt'
render_data_inertial = 'render_data_inertial.txt'

###################################################################################################
#	simulation parameters
###################################################################################################

##################################################
#	time date

#	start time
date_str = '2016-01-01'
time_str = '00:00:00.0'

#	length
days_record = 365
time_step = 60
len_record = int(86400 / time_step) * days_record + 1

#	duration
duration_sec = time_step * (len_record - 1)
duration_days = duration_sec / 86400

#	inclination
inclination = 40

#	raan
raan = 0

#	orbital height
orbital_height = 800

#	input identifiers
object_id = 'i' + str(inclination) + 'h' + str(orbital_height) + 'raan' + str(raan)

##################################################
#	simulation site parameters

#	site identifier
site_latitude = 0
site_longitude = 0
site_str = 'lat{}lon{}'.format(site_latitude, site_longitude)

##################################################
#	simulation object

#	geometry string
geom_str = 'cylinder'

#	initial attitude state
initial_attitude = np.array([4.045, 0.596, 1.677])
initial_attitude_motion = np.array([0.105, 0.000, 0.000])

attitude_state = np.hstack([initial_attitude, initial_attitude_motion])
attitude_state_str = '{:1.3f}_{:1.3f}_{:1.3f}_{:1.3f}_{:1.3f}_{:1.3f}'.format(*attitude_state)

#	generate rotation vector
rot_matrix = rot_mat(initial_attitude)
rot_vec_body = np.divide(initial_attitude_motion, np.linalg.norm(initial_attitude_motion))
rot_vec_orb = np.dot(rot_vec_body, rot_matrix)

##################################################
#	render parameters

#	rending time step
render_time_step = 1

print('\n#####################################################################################')
print('\n	Time parameters: ')
print('		Simulation time step:		{}'.format(time_step))
print('		Duration:			{}		[steps]'.format(len_record))
print('						{}	[s]'.format(np.round(duration_sec, 1)))
print('						{}	[m]'.format(np.round(duration_sec / 60, 1)))
print('						{}		[days]'.format(np.round(duration_sec / 60 / 60 / 24, 1)))
print('\n	Site parameters: ')
print('		Site Latitude:			{}		Degrees'.format(site_latitude))
print('		Site Longitude:			{}		Degrees'.format(site_longitude))
print('\n	Object parameters: ')
print('\n		Geometry Str: 		{}'.format(geom_str))
print('\n		Attitude:		{}, {}, {}	Radians'.format(*initial_attitude))
print('		Attitude Motion: 	{}, {}, {}		Radians/s'.format(*initial_attitude_motion))
print('\n		Inclination:		{}			Degrees'.format(inclination))
print('		Orbital Height: 	{}			Km'.format(orbital_height))
print('		Right Ascension:	{}			Degrees'.format(raan))
print('\n		Rotation Vector:	{:1.3f}, {:1.3f}, {:1.3f}	Radians'.format(*rot_vec_orb))
print('\n#####################################################################################')

###################################################################################################
#	load light curves files
###################################################################################################

#	set lightcurve output directory
lightcurve_path = os.path.join(path_data, 'lightcurve_dir', 'lightcurve_files')
lightcurve_data_path = os.path.join(path_data, 'lightcurve_dir', 'data_files')

#	generate file names
lightcurve_data_file = '_'.join(
	[site_str, str(object_id), date_str, time_str, str(len_record), str(time_step)]
)  + '.txt'

#	input files
lightcurve_file = '_'.join(
	[site_str, str(object_id), geom_str, attitude_state_str]
)  + '.txt'

print('\nLoading Lightcurve file:\n	', lightcurve_file)
#	load light curve file
lightcurve_data = np.loadtxt(
	os.path.join(lightcurve_path, lightcurve_file),
	delimiter=','
)

print('\nLoading Lightcurve data file:\n	', lightcurve_data_file)
#	load light curve data file
lightcurve_data_array = np.loadtxt(
	os.path.join(lightcurve_data_path, lightcurve_data_file),
	delimiter=','
)

###################################################################################################
#	define object
###################################################################################################

##################################################
#	geometry
geom_str = 'cylinder'
geom_dir = 'geometry_dir/geom_dir/' + geom_str
geom_file = geom_str + '.stl'

#	brdf file
brdf_file = geom_str + '.brdf'

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

#	load bdrf file
bdrf_data = np.loadtxt(
	os.path.join(path_data, geom_dir, brdf_file),
	delimiter=','
)

###################################################################################################
#	generate orbital motion from state vector
###################################################################################################

#	satellite posititon
sat_itrf_pos = lightcurve_data_array[0, 23:26]
sat_itrf_vel = lightcurve_data_array[0, 20:23]

#	convert to orbital elements
kep_ele = rv2kep(sat_itrf_pos, sat_itrf_vel)
raan, argp, true_anom, inc, ecc, a_axis = kep_ele 

#	define earth-sun mu in km3/s2
mu_earth = 3.986004418e14 * 1e-9

orbital_period = np.pi * 2 * np.sqrt(a_axis**3 / mu_earth)
orbital_mot = (2 * np.pi) / orbital_period

###################################################################################################
#	extract light curve data
###################################################################################################

#	light curve of interest index
light_curve_id = 1

#	ids of light curve data
data_ids = np.where(lightcurve_data_array[:, 0]==light_curve_id)
	
#	extract data
light_curve_data = lightcurve_data_array[data_ids]
len_light_curve = len(light_curve_data)

#	parse array
jd_time_data = light_curve_data[:, 1]
sun_r_data = light_curve_data[:, 2]
sun_az_data = light_curve_data[:, 3]
sun_el_data = light_curve_data[:, 4]
sat_r_data = light_curve_data[:, 5]
sat_az_data = light_curve_data[:, 6]
sat_el_data = light_curve_data[:, 7]
sun_orb_data = light_curve_data[:, 8:11]
earth_orb_data = light_curve_data[:, 11:14]
site_orb_data = light_curve_data[:, 14:17]
sat_orb_data = light_curve_data[:, 17:23]
sat_itrf_data = light_curve_data[:, 23:26]
sat_itrf_vel = light_curve_data[:, 26:30]

#	calculate lc properties
lc_time_step = np.round(86400 * (jd_time_data[1] - jd_time_data[0]), 1)
lc_duration_sec = time_step * (len_light_curve - 1)
lc_sim_time = np.linspace(0, lc_duration_sec, len_light_curve)

###############################################################################################
#	interpolate if required

#	convert to light curve generation time step
if render_time_step != lc_time_step:

	print('\n	Interpolating to required time step...')

	#	parse start and stop times
	start_jdate = jd_time_data[0]
	stop_jdate = jd_time_data[-1]

	#	find nsteps
	prop_step = render_time_step / (24 * 60 * 60)
	num_points = int(round((stop_jdate - start_jdate) / prop_step))

	#	new time range to interpolate to
	interp_range = np.linspace(start_jdate, stop_jdate, num_points + 1)

	#	current x spline
	spline_x = jd_time_data

	#	new spline x
	spline_x_new = interp_range

	#	generate data to interpolate
	interp_data_in = np.hstack((sat_orb_data, sun_orb_data, earth_orb_data, site_orb_data))

	#	initilise new y spline data
	interp_data_out = np.zeros((len(interp_range), len(interp_data_in[0])))

	#	perform interpolation	
	interp_data = interp(interp_data_in, interp_data_out, spline_x, spline_x_new)

	#	extract interpolated data
	jd_time_data = interp_range
	sat_orb_data = interp_data[:, 0:6]
	sun_orb_data = interp_data[:, 6:9]
	earth_orb_data = interp_data[:, 9:12]
	site_orb_data = interp_data[:, 12:15]

	#	recalculate lc properties
	len_light_curve = len(jd_time_data)
	lc_time_step = np.round(86400 * (jd_time_data[1] - jd_time_data[0]), 1)
	lc_duration_sec = render_time_step * (len_light_curve - 1)
	lc_sim_time = np.linspace(0, lc_duration_sec, len_light_curve)

###################################################################################################
#	prepare for render loop
###################################################################################################

###############################################
#	clear render image directory

print('\nChecking for existing Image files...')

if nimg_files != 0:
	#	ask user to delete files
	check = input('	{:d} image files exist. Delete? [y/n]: '.format(nimg_files))

	if check == 'y':
		for f in render_img_dir_list:
			if f.endswith('.png'):
				os.remove(os.path.join(render_image_dir, f))
		print('	Files deleted')

	elif check == 'n':
		print('	Files retained,')
		check = input('		Use existing image files? [y/n]: ')
		if check == 'y':
			print('		Continuing with existing image files...')

		elif check == 'n':
			print('		Exiting.')
			sys.exit()
		else:
			print('	error in provided answer')
			sys.exit()
	else:
		print('	error in provided answer')
		sys.exit()
else:
	print('\n	Image directory is clean.')

###############################################
#	clear render geometry directory

print('\nChecking for existing Geom files...')

if ngeom_files != 0:
	#	ask user to delete files
	check = input('	{:d} geom files exist. Delete? [y/n]: '.format(ngeom_files))

	if check == 'y':
		for f in render_geom_dir_list:
			if f.endswith('.stl'):
				os.remove(os.path.join(render_geom_dir, f))
		print('	Files deleted')

	elif check == 'n':
		print('	Files retained,')
		check = input('		Use existing geometry files? [y/n]: ')
		if check == 'y':
			print('		Continuing with existing geometry files...')

		elif check == 'n':
			print('		Exiting.')
			sys.exit()
		else:
			print('	error in provided answer')
			sys.exit()
	else:
		print('	error in provided answer')
		sys.exit()
else:
	print('\n	Geometry directory is clean.')


###############################################
#	vector and geometry file generation

#	generate observation and illumination vectors
observation_vector = np.array([0.0, 0.0, 0.0])
illumination_vector = np.array([0.0, 0.0, 0.0])

#	set initial frame offset and frame angular velocity

#	this is the initial offset between the orbital and body frames
init_frame_offset = np.copy(initial_attitude)

#	this is the angular velocity of the body frame
init_frame_vel = np.copy(initial_attitude_motion)

#	generate orb2inertial
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

#	generate vector data and render geometry files
vector_data_body, vector_data_orb, vector_data_inertial, attitude_out = gen_vectors_render(
	path_data, render_time_step, len_light_curve, jd_time_data,
	sat_orb_data, sun_orb_data, earth_orb_data, site_orb_data,
	orb2inertial, init_frame_offset, init_frame_vel,
	observation_vector, illumination_vector,
	body_mesh, render_geom_dir
)

###############################################
#	import geometry data
geom_file_names = []

#	get list of geometry file names
for geom_file in render_geom_dir_list:
	geom_file_names.append(geom_file)

#	generate file ids
geom_file_list = []
for i, geom_file in enumerate(geom_file_names):

	temp = geom_file.split('_')[2]
	geom_id = int(temp.split('.')[0])
	geom_file_list.append([geom_id, geom_file])

#	sort files
geom_files_sorted = sorted(geom_file_list, key=lambda x: int(x[0]))

###################################################################################################
#	perform render
###################################################################################################

###############################################
#	set vector data to use
vector_data = vector_data_orb

###############################################
#	initilise figure
#figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
figure3d = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
axes3d = mplot3d.Axes3D(figure3d)

if len(geom_files_sorted) == len(vector_data):
	input('\nRender from {:d} files?'.format(len(geom_files_sorted)))
else:
	print('Error in data input.')
	print('len(geom_files) = ', len(geom_files_sorted))
	print('len(vector_data) = ', len(vector_data))
	sys.exit()

###############################################
#	main loop
for i, (geom_line, vector_line) in enumerate(
		progressbar.progressbar(zip(geom_files_sorted, vector_data))
	):

	###############################################################################################
	#	loop initialisation
	###############################################################################################

	###############################################
	#	parse data
	mesh_file = geom_line[1]

	jdate = vector_line[0]
	sun_vec = vector_line[1:4]
	site_vec = vector_line[4:7]
	earth_vec = vector_line[7:10] 
	vel_vec = vector_line[10:13]
	body_vecx = vector_line[13:16]
	body_vecy = vector_line[16:19]
	body_vecz = vector_line[19:22]

	###############################################
	#	set which vectors to use
	illum_vec_used = sun_vec
	obs_vec_used = site_vec

	###############################################
	#	generate image name
	if i < 10:
		image_id = '000' + str(i)
	elif 10 <= i < 100:
		image_id = '00' + str(i)
	elif 100 <= i < 1000:
		image_id = '0' + str(i)
	elif i > 1000:
		image_id = str(i)

	image_name = image_id + '_image.png'

	# Clear graphics output
	if i != 0:
		axes3d.cla()

	#	import mesh
	object_mesh = mesh.Mesh.from_file(os.path.join(render_geom_dir, mesh_file))
	vectors = object_mesh.vectors
	normals = object_mesh.normals

	#	set mesh alpha
	alpha = 0.5

	###############################################################################################
	#	plot vector data
	###############################################################################################

	earth_vec_plot = 20 * np.divide(np.vstack(([0, 0, 0], -earth_vec)), np.linalg.norm(earth_vec))
	vel_vec_plot = 20 * np.divide(np.vstack(([0, 0, 0], vel_vec)), np.linalg.norm(vel_vec))
	z_vec_plot = 20 * np.divide(
		np.vstack(([0, 0, 0], np.array([0, 0, 1]))),
		np.linalg.norm(np.array([0, 0, 1]))
	)

	illum_vec_plot = 15 * np.divide(np.vstack(([0, 0, 0], illum_vec_used)), np.linalg.norm(illum_vec_used))
	obs_vec_plot = 15 * np.divide(np.vstack(([0, 0, 0], obs_vec_used)), np.linalg.norm(obs_vec_used))

	bodyx_vec_plot = 10 * np.vstack(([0, 0, 0], body_vecx))
	bodyy_vec_plot = 10 * np.vstack(([0, 0, 0], body_vecy))
	bodyz_vec_plot = 10 * np.vstack(([0, 0, 0], body_vecz))

	rot_vec_plot = 10 * np.vstack(([0, 0, 0], rot_vec_orb))

	###############################################################################################
	#	rotation vector

	a = Arrow3D(rot_vec_plot[:, 0], rot_vec_plot[:, 1], rot_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="g"
	)

	axes3d.add_artist(a)

	###############################################################################################
	#	orbital frame reference vectors

	###############################################
	#	velocity vector
	a = Arrow3D(vel_vec_plot[:, 0], vel_vec_plot[:, 1], vel_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="k"
	)

	axes3d.add_artist(a)

	###############################################
	#	earth vector
	a = Arrow3D(-earth_vec_plot[:, 0], -earth_vec_plot[:, 1], -earth_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="k"
	)

	axes3d.add_artist(a)

	###############################################
	#	azimuth reference vector
	a = Arrow3D(z_vec_plot[:, 0], z_vec_plot[:, 1], z_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="k"
	)

	axes3d.add_artist(a)

	###############################################################################################
	#	observation and illumination vectors

	###############################################
	#	illum vector
	a = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="r"
	)

	axes3d.add_artist(a)

	###############################################
	#	observation vector
	b = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="b"
	)
	axes3d.add_artist(b)

	###############################################################################################
	#	body axis vectors

	###############################################
	#	body x axis
	c = Arrow3D(bodyx_vec_plot[:, 0] / 2, bodyx_vec_plot[:, 1] / 2, bodyx_vec_plot[:, 2] / 2,
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="k"
	)
	axes3d.add_artist(c)

	###############################################
	#	body y axis
	d = Arrow3D(bodyy_vec_plot[:, 0] / 2, bodyy_vec_plot[:, 1] / 2, bodyy_vec_plot[:, 2] / 2,
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="k"
	)
	axes3d.add_artist(d)

	###############################################
	#	body z axis
	e = Arrow3D(bodyz_vec_plot[:, 0] / 2, bodyz_vec_plot[:, 1] / 2, bodyz_vec_plot[:, 2] / 2,
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="k"
	)
	axes3d.add_artist(e)

	###############################################################################################
	#	add text
	###############################################################################################

	font_size = 14

	#	illumination vector
	axes3d.text(illum_vec_plot[1, 0], illum_vec_plot[1, 1], illum_vec_plot[1, 2],
		'L',
		fontsize=font_size,
		fontweight='bold'
	)

	#	observation vector
	axes3d.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2] + 0.1,
		'V',
		fontsize=font_size,
		fontweight='bold'
	)

	bodyx_vec_plot /= 2 
	bodyy_vec_plot /= 2 
	bodyz_vec_plot /= 2 

	axes3d.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1] - 0.1, bodyx_vec_plot[1, 2] + 0.1,
		'x_b',
		fontsize=font_size,
		fontweight='bold'
	)

	axes3d.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
		'y_b',
		fontsize=font_size,
		fontweight='bold'
	)
	axes3d.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
		'z_b',
		fontsize=font_size,
		fontweight='bold'
	)


	###############################################################################################
	#	plot mesh
	###############################################################################################

	for vector, normal in zip(vectors, normals):
	
		#	parse input vectors and calculate facet normal
		vect12 = vector[1] - vector[0]
		vect23 = vector[2] - vector[1]
		normal = np.cross(vect12, vect23)
		normal_unit = normal / np.linalg.norm(normal)

		#	sun angle
		sun_ang = np.arccos(
			np.dot(normal, illum_vec_used)
			/ (np.linalg.norm(normal) *  np.linalg.norm(illum_vec_used))
		)

		#	site angle
		site_ang = np.arccos(
			np.dot(normal, obs_vec_used)
			/ (np.linalg.norm(normal) * np.linalg.norm(obs_vec_used))
		)

		###############################################
		# rending

		#	Add final vert to vector array
		vector = np.vstack([vector, vector[0, :]])

		#	extract x, y and z values
		xs = vector[:,0]
		ys = vector[:,1]
		zs = vector[:,2]

		#	generate vertices list
		verts = [list(zip(xs, ys, zs))]

		#	ensure face is observable and illuminated
		if sun_ang < (0.5 * np.pi) and site_ang < (0.5 * np.pi):
			facecol = [np.absolute(np.cos(sun_ang)), np.absolute(np.cos(sun_ang)), \
					np.absolute(np.cos(sun_ang))]

		#	else set face colour dark
		else:
			facecol = [0.1, 0.1, 0.1]

		#	set facet alpha
		alpha = 0.75

		#	add facet to plot
		pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
		pc.set_alpha(alpha)
		pc.set_edgecolor([0.5, 0.5, 0.5])
		pc.set_facecolor(facecol)
		
		axes3d.add_collection3d(pc)

	###############################################################################################
	#	finalise plot
	###############################################################################################

	###############################################
	#	plot labels and limits
	axes3d.set_xlim([-8, 8])
	axes3d.set_ylim([-8, 8])
	axes3d.set_zlim([-8, 8])
#	axes3d.set_aspect('equal')

	axes3d.set_xlabel('x axis')
	axes3d.set_ylabel('y axis')
	axes3d.set_zlabel('z axis')

	#	azimuth of observation vector
	xy_comp = np.array([obs_vec_used[0], obs_vec_used[1], 0.0])
	x_vec = np.array([1.0, 0.0, 0.0])

	azimuth = np.arccos(
		np.divide(
			np.dot(xy_comp, x_vec),
			np.linalg.norm(xy_comp) * np.linalg.norm(x_vec)
		)
	)

	if obs_vec_used[1] < 0:
		azimuth = -azimuth

	#	elevation of observation vector
	temp = rot3(obs_vec_used, azimuth)
	xz_comp = np.array([obs_vec_used[0], 0.0, obs_vec_used[2]])
	x_vec = np.array([0.0, 0.0, 1.0])

	elevation = np.arccos(
		np.divide(
			np.dot(xz_comp, x_vec),
			np.linalg.norm(xz_comp) * np.linalg.norm(x_vec)
		)
	)

	elevation = (np.pi / 2) - elevation

	###############################################
	#	set view direction
	elevation = 0
	azimuth = np.pi

	axes3d.view_init(np.degrees(elevation), np.degrees(azimuth))
	

	plt.pause(1)
#	plt.axis('off')
#	plt.show()
#	sys.exit()

	###############################################################################################
	#	save output image
	###############################################################################################

	#	save single figure
	figure3d.savefig(os.path.join(render_image_dir, image_name))

print('\nRender complete.')

#############################################################################################
#	delete geom files
for f in geom_dir_list:
	if f.endswith('.stl'):
		os.remove(os.path.join(geom_path, f))
