###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Light curve rendering in the orbital frame
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	scipy
from scipy import interpolate

#	progress bar
import progressbar

#	transformation
from tools.functions.functions_transform import site_vec, rv2kep, gen_vectors_render, \
	rot1, rot2, rot3, rot_mat

#	function list
from tools.functions.functions_list import object_gen, date2jd, jd2date, get_times, interp

#	case file generation
from tools.file_generation.case_generation import gen_case

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

##############################################
#	video command
#	ffmpeg -framerate 10 -pattern_type glob -i '*.png' -c:v libx264 -pix_fmt yuv420p ../orb.mp4

###################################################################################################
#	define plotable arrow class
###################################################################################################

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################

#	clear terminal
call(['clear'])

print('Started ORBITAL render.')

##################################################
#	file paths

root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)

##################################################
#	render data

#	directory data
graphics_dir = 'graphics_dir'
render_dir = 'render_dir'
render_geom_dir = 'geom_files'
render_img_dir = 'image_files'

#	orb image data
render_image_dir = os.path.join(path_data, graphics_dir, render_dir, render_img_dir)
render_img_dir_list = os.listdir(render_image_dir)
nimg_files = len(render_img_dir_list)

#	orb image data
render_geom_dir = os.path.join(path_data, graphics_dir, render_dir, render_geom_dir)
render_geom_dir_list = os.listdir(render_geom_dir)
ngeom_files = len(render_geom_dir_list)

#	render outputs
render_data_orb = 'render_data_orb.txt'
render_data_body = 'render_data_body.txt'
render_data_inertial = 'render_data_inertial.txt'

###################################################################################################
#	Inputs
###################################################################################################

##################################################
#	case definition

#	attitude case
attit_case = 'x-rot'

#	geometry case
geom_case = 'cylinder'

#	reflection case
reflection_case = 'diffuse'

#	azimuth and elevation
illum_inc = 75
illum_azm = 135

#	azimuth and elevation
obs_inc = 60
obs_azm = 90

##################################################
#	simulation parameters

#	time step in seconds
time_step = 1

#	length of data reccord [number of points]
len_record = 60 * 4

#	add zero to len reccord
len_record = len_record + 1

##################################################
#	BDRF parameters

if reflection_case == 'diffuse':
	#	Specular and Diffuse ratio
	d_bdrf = 1.0
	s_bdrf = 0.0


elif reflection_case == 'specular':
	#	Specular and Diffuse ratio
	d_bdrf = 0.0
	s_bdrf = 1.0

#	Diffuse albedo
omega_bdrf = 0.5

#	Refractive Index
n_bdrf = (1 + np.sqrt(omega_bdrf)) / (1 - np.sqrt(omega_bdrf))

#	Root-Mean-Square slope
m_bdrf = 0.25

###################################################################################################
#	initilisation
###################################################################################################

##################################################
#	file names

#	pos data
pos_dir = 'pos_dir'

#	orb
sat_orb_file = 'sat_orb.txt'
sun_orb_file = 'sun_orb.txt'
earth_orb_file = 'earth_orb.txt'
site_orb_file = 'site_orb.txt'

#	render outputs
render_data_orb = 'render_data_orb.txt'
render_data_body = 'render_data_body.txt'
render_data_inertial = 'render_data_inertial.txt'

##################################################
#	render case

#	bulk run directory
bulk_dir = 'bulk_run'

#	case file path
case_path = os.path.join(path_data, 'case_files', bulk_dir)

azm_inc_case = '_'.join(['azm' + str(illum_azm), 'inc' + str(illum_inc)])

#	generate case file name string
case_str = '_'.join([attit_case, azm_inc_case, geom_case, reflection_case])

print('\nCase to render:')
print('\n	Rotation case = ', attit_case)
print('	Geometry case = ', geom_case)
print('	Reflection case = ', reflection_case)
print('	Azm/Inc case = ', azm_inc_case)

###################################################################################################
#	parse inputs
###################################################################################################

small = 1e-6

x_vec = np.array([1.0, 0.0, 0.0])
z_vec = np.array([0.0, 0.0, 1.0])

##################################################
#	generate temperory case file
print('\nGenerating fake case file...')

#	fake id number
fake_id = '000001_'

#	case identifier
case_ident = fake_id + case_str

#	true lightcurve times
start_time_str = 'start_time = 2016, 12, 22, 02, 05, 27.0'
stop_time_str = 'stop_time = 2016, 12, 22, 02, 09, 58.0'

append_str = '_491_25885_2016-12-22.txt'

##################################################
#	set geometry data

#	geometry directory
geom_dir = 'geometry_dir'

#	geomtry input path
geom_path = os.path.join(path_data, geom_dir)

if geom_case == 'div1':
	geom_file = '01_div1_sphere.stl'

elif geom_case == 'div5':
	geom_file = '05_div5_sphere.stl'

elif geom_case == 'cube':
	geom_file = '01_cube_cube.stl'

elif geom_case == 'cylinder':
	geom_file = 'cylinder.stl'

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(geom_path, geom_file))

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_normals)

##################################################
#	set attitude motion
onerpm = 0.1047
initial_attitude = np.array([np.radians(illum_azm), np.radians(illum_inc), 0.0])

print('\n	Setting Initial Attitude to: ', np.degrees(initial_attitude))

if attit_case == '0-attit':
	initial_attitude_motion = np.array([0.0, 0.0, 0.0])

elif attit_case == 'x-rot':
	initial_attitude_motion = np.array([onerpm, 0.0, 0.0])

elif attit_case == 'y-rot':
	initial_attitude_motion = np.array([0.0, onerpm, 0.0])

elif attit_case == 'z-rot':
	initial_attitude_motion = np.array([0.0, 0.0, onerpm])

#	generate rotation vector
attit_rot_mat = rot_mat(initial_attitude)

#	define constant observation vector
rot_vec = np.dot(x_vec, attit_rot_mat)



#	calculate angles

#	angle to x-axis
temp_inc = np.arccos(
	np.dot(rot_vec, x_vec)
	/ (np.linalg.norm(rot_vec) *  np.linalg.norm(x_vec))
)


#	angle of yz comp to z-axis

yz_comp = np.array([0.0, rot_vec[1], rot_vec[2]])

temp_azm = np.arccos(
	np.dot(yz_comp, z_vec)
	/ (np.linalg.norm(yz_comp) *  np.linalg.norm(z_vec))
)


print('\n	Angle of rot vec to x-axis', np.degrees(temp_inc))
print('	Angle of rot vec to z-axis', np.degrees(temp_azm))

##################################################
#	generate illumination vector

print('\n	Using illumination offset angles: ', illum_inc, illum_azm)

#	apply inclination
temp_illum_vec = rot2(x_vec, np.radians(illum_inc))

#	apply azimuth
illum_vec = rot1(temp_illum_vec, np.radians(illum_azm))

#	correct small values
if -small < illum_vec[0] < small:
	illum_vec[0] = 0.0
if -small < illum_vec[1] < small:
	illum_vec[1] = 0.0
if -small < illum_vec[2] < small:
	illum_vec[2] = 0.0

##################################################
#	generate observation vector

#	apply elevation
temp_obs_vec = rot2(x_vec, np.radians(obs_inc))

#	apply azimuth
output_obs_vec = rot1(temp_obs_vec, np.radians(obs_azm))

#	correct small values
if -small < output_obs_vec[0] < small:
	output_obs_vec[0] = 0.0
if -small < output_obs_vec[1] < small:
	output_obs_vec[1] = 0.0
if -small < output_obs_vec[2] < small:
	output_obs_vec[2] = 0.0

#	append vector to outputs
obs_vec = output_obs_vec



print(rot_vec)
print(illum_vec)
sys.exit()

##################################################
#	file generation function
case_info = gen_case(
	case_ident,
	start_time_str,
	stop_time_str,
	geom_path,
	geom_file,
	initial_attitude_motion,
	initial_attitude,
	obs_vec,
	illum_vec,
	d_bdrf,
	s_bdrf,
	omega_bdrf,
	n_bdrf,
	m_bdrf
)

##################################################
#	Object 25885 TLE

#	end of 2016 tle
tle_line1 = '1 25885U 99043C   16347.16557405 -.00000088  00000-0 -43593-3 0  9992'
tle_line2 = '2 25885  52.0136 114.2293 0006645 165.6082 252.1930 11.39172050765052'

#	extract mean motion from TLE
mean_mot = float(tle_line2[52:64])
orbital_mot = mean_mot * ((2 * np.pi) / (24 * 60 * 60))

###################################################################################################
#	load orbital data
###################################################################################################

#	load azel sat data
sat_orb = np.loadtxt(
	os.path.join(path_data, pos_dir, sat_orb_file),
	delimiter=','
)

#	load azel sun data
sun_orb = np.loadtxt(
	os.path.join(path_data, pos_dir, sun_orb_file),
	delimiter=','
)

#	load itrf sat data
site_orb = np.loadtxt(
	os.path.join(path_data, pos_dir, site_orb_file),
	delimiter=','
)

#	load itrf sat data
earth_orb = np.loadtxt(
	os.path.join(path_data, pos_dir, earth_orb_file),
	delimiter=','
)

###################################################################################################
#	parse case file
###################################################################################################

#	generatate case data from case file
case_name = case_ident + append_str
object_data, bdrf_data = object_gen(case_name, case_info)

#	phase vectors
observation_vector = object_data.obs_vec
illumination_vector = object_data.illum_vec

#	attitude
initial_attitude = object_data.attitude
attitude_motion = object_data.attitude_motion

#	simulation time data
jday_start = object_data.jday_utc_start
jfrac_start = object_data.jfrac_utc_start
start_jdate = jday_start + jfrac_start

jday_stop = object_data.jday_utc_stop
jfrac_stop = object_data.jfrac_utc_stop
stop_jdate = jday_stop + jfrac_stop

#	time range in days
jd_time_data = np.linspace(start_jdate, stop_jdate, len_record)

#	observation site data
site_latgd = np.radians(object_data.site_latitudegd)
site_lon = np.radians(object_data.site_longitude)
site_alt = object_data.site_altitude
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

# convert julian to utc date search strings
year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)

#	create required subdirectories
norad_id = object_data.norad_ident

###################################################################################################
#	prepare for vector generation
###################################################################################################

###############################################
#	generate body mesh

#	geometry
geom_file = object_data.geometry_file

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

###################################################################################################
#	prepare for render loop
###################################################################################################

###############################################
#	clear render image directory

print('\nChecking for existing Image files...')

if nimg_files != 0:
	#	ask user to delete files
	check = input('	{:d} image files exist. Delete? [y/n]: '.format(nimg_files))

	if check == 'y':
		for f in render_img_dir_list:
			if f.endswith('.png'):
				os.remove(os.path.join(render_image_dir, f))
		print('	Files deleted')

	elif check == 'n':
		print('	Files retained,')
		check = input('		Use existing image files? [y/n]: ')
		if check == 'y':
			print('		Continuing with existing image files...')

		elif check == 'n':
			print('		Exiting.')
			sys.exit()
		else:
			print('	error in provided answer')
			sys.exit()
	else:
		print('	error in provided answer')
		sys.exit()
else:
	print('\n	Image directory is clean.')

###############################################
#	clear render geometry directory

print('\nChecking for existing Geom files...')

if ngeom_files != 0:
	#	ask user to delete files
	check = input('	{:d} geom files exist. Delete? [y/n]: '.format(ngeom_files))

	if check == 'y':
		for f in render_geom_dir_list:
			if f.endswith('.stl'):
				os.remove(os.path.join(render_geom_dir, f))
		print('	Files deleted')

	elif check == 'n':
		print('	Files retained,')
		check = input('		Use existing geometry files? [y/n]: ')
		if check == 'y':
			print('		Continuing with existing geometry files...')

		elif check == 'n':
			print('		Exiting.')
			sys.exit()
		else:
			print('	error in provided answer')
			sys.exit()
	else:
		print('	error in provided answer')
		sys.exit()
else:
	print('\n	Geometry directory is clean.')

###############################################
#	vector and geometry file generation

#	this is the initial offset between the orbital and body frames
init_frame_offset = np.copy(initial_attitude)

#	this is the angular velocity of the body frame
init_frame_vel = np.copy(initial_attitude_motion)

#	generate orb2inertial
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

#	vector generation
vector_data_body, vector_data_orb, vector_data_inertial, attitude_out = gen_vectors_render(
	path_data, time_step, len_record, jd_time_data,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_frame_offset, init_frame_vel,
	observation_vector, illumination_vector,
	body_mesh, render_geom_dir
)

###############################################
#	import geometry data
geom_file_names = []

#	get list of geometry file names
for geom_file in render_geom_dir_list:
	geom_file_names.append(geom_file)

#	generate file ids
geom_file_list = []
for i, geom_file in enumerate(geom_file_names):

	temp = geom_file.split('_')[2]
	geom_id = int(temp.split('.')[0])
	geom_file_list.append([geom_id, geom_file])

#	sort files
geom_files_sorted = sorted(geom_file_list, key=lambda x: int(x[0]))

###################################################################################################
#	perform render
###################################################################################################
#	select vector data to use
vector_data = vector_data_orb

###############################################
#	initilise figure
# figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
figure3d = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
axes3d = mplot3d.Axes3D(figure3d)

if len(geom_files_sorted) == len(vector_data):
	input('\nRender from {:d} files?'.format(len(geom_files_sorted)))
else:
	print('Error in data input.')
	print('len(geom_files) = ', len(geom_files_sorted))
	print('len(vector_data) = ', len(vector_data))
	sys.exit()

###############################################
#	main loop
for i, (geom_line, vector_line) in enumerate(
		progressbar.progressbar(zip(geom_files_sorted, vector_data))
	):

	###############################################################################################
	#	loop initialisation
	###############################################################################################

	###############################################
	#	parse data
	mesh_file = geom_line[1]

	jdate = vector_line[0]
	sun_vec = vector_line[1:4]
	site_vec = vector_line[4:7]
	earth_vec = vector_line[7:10] 
	vel_vec = vector_line[10:13]
	body_vecx = vector_line[13:16]
	body_vecy = vector_line[16:19]
	body_vecz = vector_line[19:22]

	illum_vec = vector_line[22:25]
	obs_vec = vector_line[25:28]

	###############################################
	#	set which vectors to use
	illum_vec_used = illum_vec
	obs_vec_used = obs_vec
#	illum_vec_used = sun_vec
#	obs_vec_used = site_vec

	###############################################
	#	plot figure

	#	generate image name
	if i < 10:
		image_id = '000' + str(i)
	elif 10 <= i < 100:
		image_id = '00' + str(i)
	elif 100 <= i < 1000:
		image_id = '0' + str(i)
	elif i > 1000:
		image_id = str(i)

	image_name = image_id + '_image.png'

	# Clear graphics output
	if i != 0:
		axes3d.cla()

	#	import mesh
	object_mesh = mesh.Mesh.from_file(os.path.join(render_geom_dir, mesh_file))
	vectors = object_mesh.vectors
	normals = object_mesh.normals

	#	set mesh alpha
	alpha = 0.5

	###############################################################################################
	#	plot vector data
	###############################################################################################

	earth_vec_plot = 20 * np.divide(np.vstack(([0, 0, 0], -earth_vec)), np.linalg.norm(earth_vec))
	vel_vec_plot = 20 * np.divide(np.vstack(([0, 0, 0], vel_vec)), np.linalg.norm(vel_vec))
	z_vec_plot = 20 * np.divide(
		np.vstack(([0, 0, 0], np.array([0, 0, 1]))),
		np.linalg.norm(np.array([0, 0, 1]))
	)

	illum_vec_plot = 15 * np.divide(np.vstack(([0, 0, 0], illum_vec_used)), np.linalg.norm(illum_vec_used))
	obs_vec_plot = 15 * np.divide(np.vstack(([0, 0, 0], obs_vec_used)), np.linalg.norm(obs_vec_used))

	rot_vec_plot = 10 * np.divide(np.vstack(([0, 0, 0], rot_vec)), np.linalg.norm(rot_vec))

	bodyx_vec_plot = 10 * np.vstack(([0, 0, 0], body_vecx))
	bodyy_vec_plot = 10 * np.vstack(([0, 0, 0], body_vecy))
	bodyz_vec_plot = 10 * np.vstack(([0, 0, 0], body_vecz))

	###############################################################################################
	#	orbital frame reference vectors

	###############################################
	#	velocity vector
	a = Arrow3D(vel_vec_plot[:, 0], vel_vec_plot[:, 1], vel_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="g"
	)

	axes3d.add_artist(a)

	###############################################
	#	earth vector
	a = Arrow3D(-earth_vec_plot[:, 0], -earth_vec_plot[:, 1], -earth_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="g"
	)

	axes3d.add_artist(a)

	###############################################
	#	azimuth reference vector
	a = Arrow3D(z_vec_plot[:, 0], z_vec_plot[:, 1], z_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="g"
	)

	axes3d.add_artist(a)

	###############################################################################################
	#	observation, illumination vector and rotation axis

	###############################################
	#	illum vector
	a = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="r"
	)

	axes3d.add_artist(a)

	###############################################
	#	observation vector
	b = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="b"
	)
	axes3d.add_artist(b)

	###############################################
	#	rotation vector
	b = Arrow3D(rot_vec_plot[:, 0], rot_vec_plot[:, 1], rot_vec_plot[:, 2],
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="m"
	)
	axes3d.add_artist(b)

	###############################################################################################
	#	body axis vectors

	###############################################
	#	body x axis
	c = Arrow3D(bodyx_vec_plot[:, 0] / 2, bodyx_vec_plot[:, 1] / 2, bodyx_vec_plot[:, 2] / 2,
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="k"
	)
	axes3d.add_artist(c)

	###############################################
	#	body y axis
	d = Arrow3D(bodyy_vec_plot[:, 0] / 2, bodyy_vec_plot[:, 1] / 2, bodyy_vec_plot[:, 2] / 2,
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="k"
	)
	axes3d.add_artist(d)

	###############################################
	#	body z axis
	e = Arrow3D(bodyz_vec_plot[:, 0] / 2, bodyz_vec_plot[:, 1] / 2, bodyz_vec_plot[:, 2] / 2,
		mutation_scale=20, 
		lw=2,
		arrowstyle="-|>",
		color="k"
	)
	axes3d.add_artist(e)

	###############################################################################################
	#	add text
	###############################################################################################

	font_size = 14

	#	illumination vector
	axes3d.text(illum_vec_plot[1, 0], illum_vec_plot[1, 1], illum_vec_plot[1, 2],
		'L',
		fontsize=font_size,
		fontweight='bold'
	)

	#	observation vector
	axes3d.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2] + 0.1,
		'V',
		fontsize=font_size,
		fontweight='bold'
	)

	bodyx_vec_plot /= 2 
	bodyy_vec_plot /= 2 
	bodyz_vec_plot /= 2 

	axes3d.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1] - 0.1, bodyx_vec_plot[1, 2] + 0.1,
		'x_b',
		fontsize=font_size,
		fontweight='bold'
	)

	axes3d.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
		'y_b',
		fontsize=font_size,
		fontweight='bold'
	)
	axes3d.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
		'z_b',
		fontsize=font_size,
		fontweight='bold'
	)

	###############################################################################################
	#	plot mesh
	###############################################################################################

	for vector, normal in zip(vectors, normals):
	
		#	parse input vectors and calculate facet normal
		vect12 = vector[1] - vector[0]
		vect23 = vector[2] - vector[1]
		normal = np.cross(vect12, vect23)
		normal_unit = normal / np.linalg.norm(normal)

		#	sun angle
		sun_ang = np.arccos(
			np.dot(normal, illum_vec_used)
			/ (np.linalg.norm(normal) *  np.linalg.norm(illum_vec_used))
		)

		#	site angle
		site_ang = np.arccos(
			np.dot(normal, obs_vec_used)
			/ (np.linalg.norm(normal) * np.linalg.norm(obs_vec_used))
		)

		###############################################
		# rending

		#	Add final vert to vector array
		vector = np.vstack([vector, vector[0, :]])

		#	extract x, y and z values
		xs = vector[:,0]
		ys = vector[:,1]
		zs = vector[:,2]

		#	generate vertices list
		verts = [list(zip(xs, ys, zs))]

		#	ensure face is observable and illuminated
		if sun_ang < (0.5 * np.pi) and site_ang < (0.5 * np.pi):
			facecol = [np.absolute(np.cos(sun_ang)), np.absolute(np.cos(sun_ang)), \
					np.absolute(np.cos(sun_ang))]

		#	else set face colour dark
		else:
			facecol = [0.1, 0.1, 0.1]

		#	set facet alpha
		alpha = 0.75

		#	add facet to plot
		pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
		pc.set_alpha(alpha)
		pc.set_edgecolor([0.5, 0.5, 0.5])
		pc.set_facecolor(facecol)
		
		axes3d.add_collection3d(pc)

	###############################################################################################
	#	finalise plot
	###############################################################################################

	###############################################
	#	plot labels and limits
	axes3d.set_xlim([-8, 8])
	axes3d.set_ylim([-8, 8])
	axes3d.set_zlim([-8, 8])
#	axes3d.set_aspect('equal')

	axes3d.set_xlabel('x axis')
	axes3d.set_ylabel('y axis')
	axes3d.set_zlabel('z axis')

	#	azimuth of observation vector
	xy_comp = np.array([obs_vec_used[0], obs_vec_used[1], 0.0])

	azimuth = np.arccos(
		np.divide(
			np.dot(xy_comp, x_vec),
			np.linalg.norm(xy_comp) * np.linalg.norm(x_vec)
		)
	)

	if obs_vec_used[1] < 0:
		azimuth = -azimuth

	#	elevation of observation vector
	temp = rot3(obs_vec_used, azimuth)
	xz_comp = np.array([obs_vec_used[0], 0.0, obs_vec_used[2]])

	elevation = np.arccos(
		np.divide(
			np.dot(xz_comp, z_vec),
			np.linalg.norm(xz_comp) * np.linalg.norm(z_vec)
		)
	)

	elevation = (np.pi / 2) - elevation

	###############################################
	#	set view direction
	view_elevation = 0
	view_azimuth = 0

	axes3d.view_init(np.degrees(view_elevation), np.degrees(view_azimuth))

#	plt.axis('off')
	plt.show()
	sys.exit()

	###############################################################################################
	#	save output image
	###############################################################################################

	#	save single figure
	figure3d.savefig(os.path.join(render_image_dir, image_name))

print('\nRender complete.')

#############################################################################################
#	delete geom files
for f in geom_dir_list:
	if f.endswith('.stl'):
		os.remove(os.path.join(geom_path, f))
