###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d
from descartes import PolygonPatch

#	shapely
import shapely.geometry as sg
# from shapely.geometry import Point
# from shapely.geometry import Polygon as sh
from shapely.ops import cascaded_union


#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	transformation
from tools.functions.functions_transform import site_vec, rv2kep, gen_vectors_render2, \
	rot1, rot2, rot3, rot_mat, axis2mat, mat2angles

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	Function: 		glim_slc_shadow
#	Description: 	generates light curve using provided sun and site positions and 
#					geometry properties. Includes self shadowing
#
###################################################################################################

def glim_slc_shadowing(
		sun_body_data, site_body_data, sun_orb_data, site_orb_data,
		light_curve,
		jd_start, render_time_step,
		body_mesh,
		init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		brdf_data
	):

#	print('\n	#################################################################################')
#	print('\n	Running glim_slc_shadowing script...')

	#	define unit vectors
	x_vec = np.array([1.0, 0.0, 0.0])
	y_vec = np.array([0.0, 1.0, 0.0])
	z_vec = np.array([0.0, 0.0, 1.0])

	###############################################################################################
	#	parse attit inputs
	###############################################################################################

#	print('\n		Parsing Attitude Inputs...')

	################################################
	#	initial attitude

	#	attitude matrix
	init_attit_mat = rot_mat(init_attit)

	#	transpose for inverse matrix
	init_attit_mat_inv = np.transpose(init_attit_mat)

	################################################
	#	attitude motion in orbital frame

	#	rotation_vector in orbital frame
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)

	#	generate attitude motion matrix in orbital frame
	attit_mot_mat_orb = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_orb_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_orb)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_orb)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_orb = angles1

	################################################
	#	attitude motion in body frame

	#	generate attitude motion matrix in body frame
	attit_mot_mat_body = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_body_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1

	###############################################################################################
	#	initialise step data

	total_rot_vec_vel_angle = np.array([0.0, 0.0, 0.0])

	#	generate orbital mesh from the body mesh
	orb_mesh_step = mesh.Mesh(body_mesh.data.copy())

	#	rotate into orbital frame
	orb_mesh_step.rotate_using_matrix(init_attit_mat)
	orb_vectors_step = orb_mesh_step.vectors
	orb_areas = orb_mesh_step.areas

	body_vectors_step = body_mesh.vectors

	#################################################
	#	initialise loop

	#	sensor detection limit [apparent magnitude]
	mag_limit = 18

	#	total number of facets
	tot_facets = len(orb_vectors_step)

	#	number of data points set to limit
	n_limits = 0
	tot_points = len(light_curve)

#	print('\n		Looping through time steps...')
	#	loop through and populate light curve array
	for time_step, row in enumerate(light_curve):

		###############################################################################################
		#	apply orbital attitude motion to geometry if i != 0
		if time_step != 0:
			orb_mesh_step.rotate_using_matrix(attit_mot_mat_orb)
			orb_vectors_step = orb_mesh_step.vectors

		###############################################################################################
		#	extract step data
		
#		print('\n			Extracting step data...')

		#	time data
		time_since = light_curve[time_step, 1]
		jdate = jd_start + (time_since / 86400)
		light_curve[time_step, 0] = jdate
	
		#	body position data
		sun_body_step = sun_body_data[time_step, :]
		site_body_step = site_body_data[time_step, :]

		#	orb position data
		sun_orb_step = sun_orb_data[time_step, :]
		site_orb_step = site_orb_data[time_step, :]

		#	using actual distance
		dist_vec = np.copy(site_body_step) * 1000
		r_obs_act = np.linalg.norm(dist_vec)

		#	using standard distance of 1000km
		dist_vec_std = np.array([1000.0, 0.0, 0.0]) * 1000
		r_obs_std = np.linalg.norm(dist_vec_std)

		#	unit vectors
		sun_orb_unit = sun_orb_step / np.linalg.norm(sun_orb_step)
		site_orb_unit = site_orb_step / np.linalg.norm(site_orb_step)
		sun_body_unit = sun_body_step / np.linalg.norm(sun_body_step)
		site_body_unit = site_body_step / np.linalg.norm(site_body_step)

		################################################################################################
		#	brightness calculation
		################################################################################################
	
#		print('\n			Brightness Calculation...')

		#	angular bisector
		H_vec = (sun_body_unit + site_body_unit) / len(sun_body_unit + site_body_unit) 

		#	observable facet array
		face_obs_app = []
		face_obs_std = []
		facet_num = 1

		################################################################################################
		#	generate data for shadowing

		obs_faces = []
		illum_faces = []
		illum_obs_faces = []

		faces_angles_data = np.zeros([len(orb_vectors_step), 2])

#		print('\n			Calculating Relevant Face Data...')

		for face_index, vector_step in enumerate(orb_vectors_step):
			
			#	parse input vectors and calculate facet normal
			vect12 = vector_step[1] - vector_step[0]
			vect23 = vector_step[2] - vector_step[1]
			normal = np.cross(vect12, vect23)
			normal_unit = normal / np.linalg.norm(normal)

			#	calculate ilumination angle
			sun_arccos_of = np.divide(
				np.dot(normal, sun_orb_unit),
				np.linalg.norm(normal) * np.linalg.norm(sun_orb_unit)
			)

			if sun_arccos_of > 1.0:
				if sun_arccos_of < 1.0 + 1e-6:
					sun_arccos_of = 1.0
			elif sun_arccos_of < -1.0:
				if sun_arccos_of > -1.0 - 1e-6:
					sun_arccos_of = -1.0

			sun_ang = np.arccos(sun_arccos_of)
	
			#	calculate observation angle
			site_arccos_of = np.divide(
				np.dot(normal, site_orb_unit),
				np.linalg.norm(normal) * np.linalg.norm(site_orb_unit)
			)

			if site_arccos_of > 1.0:
				if site_arccos_of < 1.0 + 1e-3:
					site_arccos_of = 1.0
			elif site_arccos_of < -1.0:
				if site_arccos_of > -1.0 - 1e-6:
					site_arccos_of = -1.0

			site_ang = np.arccos(site_arccos_of)

			#	add angles to output array
			faces_angles_data[face_index, :] = [sun_ang, site_ang]

			#	ensure face is observable and illuminated
			if site_ang < (0.5 * np.pi):
				obs_faces.append(face_index)
	
			if sun_ang < (0.5 * np.pi):
				illum_faces.append(face_index)

			if site_ang < (0.5 * np.pi) and sun_ang < (0.5 * np.pi):
				illum_obs_faces.append(face_index)


#		print('\n			Observable Faces:\n					', obs_faces)
#		print('\n			Observable and Illuminated Faces:\n					', illum_obs_faces)

		###############################################################################################
		#	calculate observation vector projection angles to reference vector

		#	reference vector = x_vec
	
		obs_xy_comp = np.array([site_orb_unit[0], site_orb_unit[1], 0.0])

		#	find azimuthal rotation angle
		obs_azm_ang = np.arccos(
			np.divide(
				np.dot(xy_comp, x_vec),
				np.linalg.norm(xy_comp) * np.linalg.norm(x_vec)
			)
		)
	
		if site_orb_unit[1] < 0:
			obs_azm_ang = -obs_azm_ang
	
		temp_attit = rot_mat(np.array([0.0, 0.0, obs_azm_ang]))
		temp_vec = np.dot(site_orb_unit, temp_attit)
	
		#	find inclination rotation angle
		obs_inc_ang = np.arccos(
			np.divide(
				np.dot(temp_vec, x_vec),
				np.linalg.norm(temp_vec) * np.linalg.norm(x_vec)
			)
		)
	
		#	set obs vector in x direction
		if site_orb_unit[2] > 0:
			obs_inc_ang = -obs_inc_ang
	
		#	attitude matrix
		obs_proj_attit = rot_mat(np.array([0.0, obs_inc_ang, obs_azm_ang]))
		obs_proj_vec = np.round(np.dot(site_orb_unit, proj_attit), 10)

		print('\n	obs_proj_vec = ', obs_proj_vec)
		print('\n	obs_angs = ', obs_inc_ang, obs_azm_ang)

		###############################################################################################
		#	calculate illumination vector projection angles to reference vector

		#	reference vector = x_vec
	
		illum_xy_comp = np.array([sun_orb_unit[0], sun_orb_unit[1], 0.0])

		#	find azimuthal rotation angle
		illum_azm_ang = np.arccos(
			np.divide(
				np.dot(xy_comp, x_vec),
				np.linalg.norm(xy_comp) * np.linalg.norm(x_vec)
			)
		)
	
		if sun_orb_unit[1] < 0:
			illum_azm_ang = -illum_azm_ang
	
		temp_attit = rot_mat(np.array([0.0, 0.0, illum_azm_ang]))
		temp_vec = np.dot(sun_orb_unit, temp_attit)
	
		#	find inclination rotation angle
		illum_inc_ang = np.arccos(
			np.divide(
				np.dot(temp_vec, x_vec),
				np.linalg.norm(temp_vec) * np.linalg.norm(x_vec)
			)
		)
	
		#	set obs vector in x direction
		if sun_orb_unit[2] > 0:
			illum_inc_ang = -illum_inc_ang
	
		#	attitude matrix
		illum_proj_attit = rot_mat(np.array([0.0, illum_inc_ang, illum_azm_ang]))
		illum_proj_vec = np.round(np.dot(sun_orb_unit, illum_proj_attit), 10)

		print('\n	illum_proj_vec = ', illum_proj_vec)
		print('\n	illum_angs = ', illum_inc_ang, illum_azm_ang)



		sys.exit()

		###############################################################################################
		#	perform orbital to projection frame conversion

		#	convert to projection frame
		obs_vec_proj = np.round(np.dot(site_orb_unit, proj_attit), 10)
		illum_vec_proj = np.round(np.dot(sun_orb_unit, proj_attit), 10)
	
		#	create copy of object mesh
		proj_mesh = mesh.Mesh(orb_mesh_step.data.copy())

		#	apply rotation
		proj_mesh.rotate_using_matrix(proj_attit)
	
		proj_vectors = proj_mesh.vectors
		proj_normals = proj_mesh.normals

		###############################################################################################
		#	calculating visibility fractions
		
		projected_visibility = np.zeros([len(illum_obs_faces), 2])

#		print('\n			Calculating Visibility Fractions...')
		for itt_num, i_illum in enumerate(illum_obs_faces):

			#	angles data
			sun_ang, site_ang = faces_angles_data[i_illum, :]
		
			#	get vector data
			proj_vector = proj_vectors[i_illum, :]
		
			#	calculate x-level
			obs_x_level = proj_vector.max()
		
			#	generate polygon
			step_poly = sg.Polygon([
				[proj_vector[0, 1], proj_vector[0, 2]],
				[proj_vector[1, 1], proj_vector[1, 2]],
				[proj_vector[2, 1], proj_vector[2, 2]]
			])
		
			###########################################################################################
			#	find shadowing facets
		
			total_polygons = []

			for i_obs in obs_faces:

				#	slice vector data
				vector_temp = proj_vectors[i_obs, :]

				#	calculate x-level
				x_level_temp = vector_temp[:, 0].max()
		
				#	ensure face is infront
				if x_level_temp > obs_x_level:
					if i_obs != i:

						#	generate polygon
						poly_temp = sg.Polygon([
							[vector_temp[0, 1], vector_temp[0, 2]],
							[vector_temp[1, 1], vector_temp[1, 2]],
							[vector_temp[2, 1], vector_temp[2, 2]]
						])

						#	add to outputs
						total_polygons.append(poly_temp)
		
			#	if no shadowing facets, set visibility to 100
			if len(total_polygons) == 0:
				visible_perc = 100.0
			else:
				#	combine
				cascaded_polygons = cascaded_union(total_polygons)
			
				#	find intersecting area
				intersection = step_poly.intersection(cascaded_polygons)

				#	calculate visible percentage
				visible_perc = 100 * (1.0 - (intersection.area / step_poly.area))
			
			#	add to output array
			projected_visibility[itt_num, :] = [i_illum, visible_perc]

		###############################################################################################
		#	facet brightness calculation

#		print('\n			Performing Facet Brightness Calculation Loop...')
		for itt_num, i_illum in enumerate(illum_obs_faces):

#			print('\n				Working on illum-obs facet: ', i_illum)

			#	visibility percentage
			vis_perc = projected_visibility[itt_num, 1]

			#	angles data
			sun_ang, site_ang = faces_angles_data[i_illum, :]
		
			#	get vector data
			body_vector = body_vectors_step[i_illum, :]
			proj_vector = proj_vectors[i_illum, :]
			area = orb_areas[i_illum]

			#	parse brdf data
			m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf = brdf_data[i_illum, :]
	
			#	calculate normal unit vector
			vect12 = body_vector[1] - body_vector[0]
			vect23 = body_vector[2] - body_vector[1]
			normal = np.cross(vect12, vect23)
			normal = normal / np.linalg.norm(normal)

			#	generate polygon (-y for flip)
			proj_poly = sg.Polygon([
				[proj_vector[0, 1], proj_vector[0, 2]],
				[proj_vector[1, 1], proj_vector[1, 2]],
				[proj_vector[2, 1], proj_vector[2, 2]]
			])

			#	extract projected area and modify by visibility percentage
			proj_area = proj_poly.area * (vis_perc / 100)

#			print('\n				BRDF Input Validation:')
#			print('\n					H_vec = ', H_vec)
#			print('					Norm = ', normal)
#			print('					site_body_unit = ', site_body_unit)
#			print('					sun_body_unit = ', sun_body_unit)
#			print('					area = ', area)
#			print('					proj_area = ', proj_area)

			#	apply cook-torrance BDRF
			facet_brightness = cook_torrance_new(m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf, \
				H_vec, normal, site_body_unit, sun_body_unit, proj_area)

			facet_brightness_app = facet_brightness / (4 * np.pi * r_obs_act**2)
			facet_brightness_std = facet_brightness / (4 * np.pi * r_obs_std**2)
	
			#	add facet brightness to output array
			face_obs_app.append(facet_brightness_app)
			face_obs_std.append(facet_brightness_std)

			#	update facet number
			facet_num += 1

#			print('\n				Brightnesses of Facets:')
#			print('					Apparent: ', facet_brightness_app)
#			print('					Standard: ', facet_brightness_std)



		#	convert to numpy array
		face_obs_app = np.array(face_obs_app)

		################################################################################################
		#	finalise data

		#	apparent data

		#	convert to numpy array
		face_obs_app = np.array(face_obs_app)

		if face_obs_app.size != 0:
			brightness_step_app = np.sum(face_obs_app)
			#	apparent magnitude calculation
			magnitude_app = -26.74 - 2.5 * np.log10(brightness_step_app)

		#	no observable facets
		elif face_obs_app.size == 0:

			#	set to limiting apparent magnitude
			magnitude_app = 18.0
			n_limits += 1 

		#	standard data

		#	convert to numpy array
		face_obs_std = np.array(face_obs_std)

		if face_obs_std.size != 0:
			brightness_step_std = np.sum(face_obs_std)
			#	standard magnitude calculation
			magnitude_std = -26.74 - 2.5 * np.log10(brightness_step_std)

		#	no observable facets
		elif face_obs_std.size == 0:

			#	set to limiting standard magnitude
			magnitude_std = 18.0
			n_limits += 1 

		################################################################################################
		#	populate return array
		light_curve[time_step, 2] = magnitude_app
		light_curve[time_step, 3] = magnitude_std

	if n_limits != 0:
		print('\n	{} of {} Data points set to limiting magnitude.'.format(
			int(n_limits),
			int(tot_points)
		))


#	print('\n	Execution of glim_slc_shadowing finished...')

	return light_curve


###################################################################################################
#	Function: 		glim_slc
#	Description: 	generates light curve using provided sun and site positions and 
#					geometry properties 
#
###################################################################################################

def glim_slc_new(sun_vec_data, site_vec_data, light_curve, \
	jd_start, vectors, areas, brdf_data):

	print('\n	#################################################################################')
	print('\n	Running glim_slc_new script...')

	#	sensor detection limit [apparent magnitude]
	mag_limit = 18

	#	total number of facets
	tot_facets = len(vectors)

	#	number of data points set to limit
	n_limits = 0
	tot_points = len(light_curve)

	print('\n		Looping through time steps...')
	#	loop through and populate light curve array
	for time_step, row in enumerate(light_curve):

		#	time data
		time_since = light_curve[time_step, 1]
		jdate = jd_start + (time_since / 86400)
		light_curve[time_step, 0] = jdate
	
		#	position data
		sun_vec_step = sun_vec_data[time_step, :]
		site_vec_step = site_vec_data[time_step, :]

		#	using actual distance
		dist_vec = np.copy(site_vec_step) * 1000
		r_obs_act = np.linalg.norm(dist_vec)

		#	using standard distance of 1000km
		dist_vec_std = np.array([1000.0, 0.0, 0.0]) * 1000
		r_obs_std = np.linalg.norm(dist_vec_std)

		#	unit vectors
		sun_vec_unit = sun_vec_step / np.linalg.norm(sun_vec_step)
		site_vec_unit = site_vec_step / np.linalg.norm(site_vec_step)

		#	sun-obs phase angle
		phase_arccos_of = np.divide(
			np.dot(sun_vec_unit, site_vec_unit),
			np.linalg.norm(sun_vec_unit) * np.linalg.norm(site_vec_unit)
		)

		if phase_arccos_of > 1.0:
			if phase_arccos_of < 1.0 + 1e-3:
				phase_arccos_of = 1.0
		elif phase_arccos_of < -1.0:
			if phase_arccos_of > -1.0 - 1e-6:
				phase_arccos_of = -1.0

		#	sun-obs phase angle
		phase_ang = np.arccos(phase_arccos_of)

		################################################################################################
		#	brightness calculation
	
		#	angular bisector
		H_vec = (sun_vec_unit + site_vec_unit) / len(sun_vec_unit + site_vec_unit) 

		#	observable facet array
		face_obs_app = []
		face_obs_std = []
		facet_num = 1

#		print('\n			Performing Facet Brightness Calculation Loop...')
		# loop through facets
		for itt, (vector, area, brdf_line) in enumerate(zip(vectors, areas, brdf_data)):

			#	parse brdf data
			m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf = brdf_line
	
			#	calculate normal unit vector
			vect12 = vector[1] - vector[0]
			vect23 = vector[2] - vector[1]
			normal = np.cross(vect12, vect23)
			normal = normal / np.linalg.norm(normal)
	
			#	calculate ilumination angle
			sun_arccos_of = np.divide(
				np.dot(normal, sun_vec_unit),
				np.linalg.norm(normal) * np.linalg.norm(sun_vec_unit)
			)

			if sun_arccos_of > 1.0:
				if sun_arccos_of < 1.0 + 1e-6:
					sun_arccos_of = 1.0
			elif sun_arccos_of < -1.0:
				if sun_arccos_of > -1.0 - 1e-6:
					sun_arccos_of = -1.0

			sun_ang = np.arccos(sun_arccos_of)
	
			#	calculate observation angle
			site_arccos_of = np.divide(
				np.dot(normal, site_vec_unit),
				np.linalg.norm(normal) * np.linalg.norm(site_vec_unit)
			)

			if site_arccos_of > 1.0:
				if site_arccos_of < 1.0 + 1e-3:
					site_arccos_of = 1.0
			elif site_arccos_of < -1.0:
				if site_arccos_of > -1.0 - 1e-6:
					site_arccos_of = -1.0

			site_ang = np.arccos(site_arccos_of)

			#	illuminated and observable condition
			if sun_ang < (0.5 * np.pi) and site_ang < (0.5 * np.pi):

#				print('\n				Working on illum-obs facet: ', itt)
	


#				print('\n				BRDF Input Validation:')
#				print('\n					H_vec = ', H_vec)
#				print('					Norm = ', normal)
#				print('					site_vec_unit = ', site_vec_unit)
#				print('					sun_vec_unit = ', sun_vec_unit)
#				print('					area = ', area)



				#	apply cook-torrance BDRF
				facet_brightness = cook_torrance(m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf, \
					H_vec, normal, site_vec_unit, sun_vec_unit, area)

				facet_brightness_app = facet_brightness / (4 * np.pi * r_obs_act**2)
				facet_brightness_std = facet_brightness / (4 * np.pi * r_obs_std**2)

				#	add facet brightness to output array
				face_obs_app.append(facet_brightness_app)
				face_obs_std.append(facet_brightness_std)

#				print('\n				Brightnesses of Facets:')
#				print('					Apparent: ', facet_brightness_app)
#				print('					Standard: ', facet_brightness_std)

#				break


			#	update facet number
			facet_num += 1

#		break
		################################################################################################
		#	finalise data

		#	apparent data

		#	convert to numpy array
		face_obs_app = np.array(face_obs_app)

		if face_obs_app.size != 0:
			brightness_step_app = np.sum(face_obs_app)
			#	apparent magnitude calculation
			magnitude_app = -26.74 - 2.5 * np.log10(brightness_step_app)

		#	no observable facets
		elif face_obs_app.size == 0:

			#	set to limiting apparent magnitude
			magnitude_app = 18.0
			n_limits += 1 

		#	standard data

		#	convert to numpy array
		face_obs_std = np.array(face_obs_std)

		if face_obs_std.size != 0:
			brightness_step_std = np.sum(face_obs_std)
			#	standard magnitude calculation
			magnitude_std = -26.74 - 2.5 * np.log10(brightness_step_std)

		#	no observable facets
		elif face_obs_std.size == 0:

			#	set to limiting standard magnitude
			magnitude_std = 18.0
			n_limits += 1 

		################################################################################################
		#	populate return array
		light_curve[time_step, 2] = magnitude_app
		light_curve[time_step, 3] = magnitude_std

	if n_limits != 0:
		print('\n	{} of {} Data points set to limiting magnitude.'.format(
			int(n_limits),
			int(tot_points)
		))


	print('\n	Execution of glim_slc_new finished...')

	return light_curve

###################################################################################################
#	Function: 		cook_torrance
#	Description: 	calculates an observed flux using the cook-torrance BDRF
#
###################################################################################################

def cook_torrance(m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf, \
		H_vec, normal, site_vec_unit, sun_vec_unit, area):

	#	dot product of H and N
	dot_prod = np.dot(H_vec, normal)

	#	product of magnitudes of H and N
	mag_prod = np.linalg.norm(H_vec) * np.linalg.norm(normal)

	#	arccos argument
	arccos_arg = dot_prod / mag_prod

	#	correct for arccos_args above 1.0
	small = 1e-6
	if 1.0 < arccos_arg < (1.0 + small):
		arccos_arg = 1.0

	#	alpha calculation
	alpha_ang = np.arccos(arccos_arg)
	
	#	G_brdf calculation
	temp1 = 2 * np.divide(
		np.multiply(np.dot(normal, H_vec), np.dot(normal, site_vec_unit)),
		np.dot(site_vec_unit, H_vec)
	)
	
	temp2 = 2 * np.divide(
		np.multiply(np.dot(normal, H_vec), np.dot(normal, sun_vec_unit)),
		np.dot(sun_vec_unit, H_vec)
	)

	if temp1 > 1 and temp2 > 1:
	    G_brdf = 1
	elif temp1 < 1 and temp2 > 1:
		G_brdf = temp1
	elif temp1 > 1 and temp2 < 1:
		G_brdf = temp2
	elif temp1 < 1 and temp2 < 1:
		temp3 = np.array((temp1, temp2))
		G_brdf = temp3.min()

	#	c_brdf calculation
	c_brdf = np.dot(site_vec_unit, H_vec)

	#	g_brdf calculation
	g_brdf = n_brdf**2 + c_brdf**2 - 1
	
	#	F_brdf calculation
	F_brdf = (
		((g_brdf - c_brdf)**2 / (2 * (g_brdf + g_brdf))) *
		((1 + (c_brdf * (g_brdf + c_brdf) - 1))**2 / ((c_brdf * (g_brdf - c_brdf) + 1)**2))
	)

	#	D_brdf_calculation
	D_brdf = (
		(1 / (m_brdf**2 * np.cos(alpha_ang)**4)) * np.exp(-((np.tan(alpha_ang) / m_brdf)**2))
	)

	#	specular reflectance calculation
	R_s = (
		(D_brdf * G_brdf * F_brdf) /
		(np.pi * np.dot(normal, sun_vec_unit) *
		np.dot(normal, site_vec_unit))
	)

	#	diffuse reflectance calculation
	R_d = omega_brdf / np.pi;
	
	#	brightness calculation
	facet_brightness = (
		(d_brdf * R_d + s_brdf * R_s) *
		area *
		np.pi *
		np.dot(normal, sun_vec_unit) *
		np.dot(normal, site_vec_unit)
	)

	return facet_brightness

###################################################################################################
#	Function: 		cook_torrance
#	Description: 	calculates an observed flux using the cook-torrance BDRF
#
###################################################################################################

def cook_torrance_new(m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf, \
		H_vec, normal, site_vec_unit, sun_vec_unit, proj_area):

	#	dot product of H and N
	dot_prod = np.dot(H_vec, normal)

	#	product of magnitudes of H and N
	mag_prod = np.linalg.norm(H_vec) * np.linalg.norm(normal)

	#	arccos argument
	arccos_arg = dot_prod / mag_prod

	#	correct for arccos_args above 1.0
	small = 1e-6
	if 1.0 < arccos_arg < (1.0 + small):
		arccos_arg = 1.0

	#	alpha calculation
	alpha_ang = np.arccos(arccos_arg)
	
	#	G_brdf calculation
	temp1 = 2 * np.divide(
		np.multiply(np.dot(normal, H_vec), np.dot(normal, site_vec_unit)),
		np.dot(site_vec_unit, H_vec)
	)
	
	temp2 = 2 * np.divide(
		np.multiply(np.dot(normal, H_vec), np.dot(normal, sun_vec_unit)),
		np.dot(sun_vec_unit, H_vec)
	)

	if temp1 > 1 and temp2 > 1:
	    G_brdf = 1
	elif temp1 < 1 and temp2 > 1:
		G_brdf = temp1
	elif temp1 > 1 and temp2 < 1:
		G_brdf = temp2
	elif temp1 < 1 and temp2 < 1:
		temp3 = np.array((temp1, temp2))
		G_brdf = temp3.min()

	#	c_brdf calculation
	c_brdf = np.dot(site_vec_unit, H_vec)

	#	g_brdf calculation
	g_brdf = n_brdf**2 + c_brdf**2 - 1
	
	#	F_brdf calculation
	F_brdf = (
		((g_brdf - c_brdf)**2 / (2 * (g_brdf + g_brdf))) *
		((1 + (c_brdf * (g_brdf + c_brdf) - 1))**2 / ((c_brdf * (g_brdf - c_brdf) + 1)**2))
	)

	#	D_brdf_calculation
	D_brdf = (
		(1 / (m_brdf**2 * np.cos(alpha_ang)**4)) * np.exp(-((np.tan(alpha_ang) / m_brdf)**2))
	)

	#	specular reflectance calculation
	R_s = (
		(D_brdf * G_brdf * F_brdf) /
		(np.pi * np.dot(normal, sun_vec_unit) *
		np.dot(normal, site_vec_unit))
	)

	#	diffuse reflectance calculation
	R_d = omega_brdf / np.pi;

	#	brightness calculation
	facet_brightness = (
		(d_brdf * R_d + s_brdf * R_s) *
		np.pi *
		proj_area *
		np.dot(normal, sun_vec_unit)
	)

	return facet_brightness
