###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	parallell functions
from starlink_para_funs import starlink_brdf_fit

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm

#	parrellisation
from multiprocessing import Pool
import tqdm

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('MMT testing script initilised.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

##################################################
#	simulation parameters
time_step = 0.1

###################################################################################################
#	light curve generation inputs

#	starlink MMT file to use
starlink_file = 'starlink-29_44243.txt'
starlink_id = int(starlink_file.split('-')[1].split('_')[0])

#	set track of interest index (starts at 0)
track_index = 2

#	output path for light curve files
out_path = os.path.join(
	path_data,
	'lightcurve_dir',
	'starlink_analysis',
	'starlink-{}'.format(starlink_id),
	'track-{}'.format(int(track_index + 1))
)

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str = 'starlink_payload'
geom_file = geom_str + '.stl'

#	attitude
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 90.0

#	attitude motion
rot_period = 9.2
rot_period_index = int(np.ceil(rot_period) * 10)

#	set initial BRDF values
d_brdf = 0.8
omega_brdf = 0.2
m_brdf = 0.95

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	load starlink file and extract data
###################################################################################################

###################################################################################################
#	load starlink file
print('Loading file...')

name_str, norad_id = starlink_file[:-4].split('_')
norad_id = int(norad_id)

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'starlink')

with open(os.path.join(read_path, starlink_file)) as read_file:
	starlink_data = read_file.readlines()

#	strip header
header_text = starlink_data[7]
starlink_raw_data = starlink_data[8:]

###################################################################################################
#	parse file into correct data types

#	initialise output
n_rows = 14
starlink_parsed_data = np.zeros([len(starlink_raw_data), n_rows])

for i, line in enumerate(starlink_raw_data):

	#	split line
	date_str, time_str, mag_std, mag_app, opt_filter, penumbral, distance, phase, channel, track_id \
		= line.split(' ')

	#	parse data and set data types
	mag_std = float(mag_std)
	mag_app = float(mag_app)
	distance = float(distance)
	phase = float(phase)
	track_id = int(track_id)

	#	parse date
	year, month, day = date_str.split('-')
	year = int(year)
	month = int(month)
	day = int(day)

	#	parse time
	hour, minute, second = time_str.split(':')
	hour = int(hour)
	minute = int(minute)
	second = np.round(float(second), 1)

	#	calculate julian date
	jday, jfrac = date2jd(year, month, day, hour, minute, second)

	#	generate output line
	output_line = [
		track_id, starlink_id,
		jday, jfrac,
		year, month, day,
		hour, minute, second,
		mag_std, mag_app,
		distance, phase
	]

	#	add to output array
	starlink_parsed_data[i, :] = output_line

print('\n	Done.')
print('\nFinished Starlink file parsing...')
print('	Total data length = ', len(starlink_parsed_data))
print('	Total number of objects = ', len(np.unique(starlink_parsed_data[:, 1])))
print('	Total number of light curves = ', len(np.unique(starlink_parsed_data[:, 0])))

###################################################################################################
#	extracting data between specified windows

print('\n#########################################################')
print('\nSeaching for tracks inside time window...')

#	start window
window_start_date = [2019, 5, 25, 0, 0, 0]
window_start_jd = np.add(*date2jd(*window_start_date))
print('\n	Window start 	= ', window_start_date[0:3])

#	stop window
window_stop_date = [2019, 6, 1, 0, 0, 0]
window_stop_jd = np.add(*date2jd(*window_stop_date))
print('	Window stop 	= ', window_stop_date[0:3])

#	extract track ids
track_id_list = np.unique(starlink_parsed_data[:, 0])

#	loop through tracks
ntracks = 0
first_lc_flag = True
for track_id in track_id_list:

	#	find track ids
	track_indices = np.where(starlink_parsed_data[:, 0]==track_id)[0]

	#	extract full data
	track_data = starlink_parsed_data[track_indices, :]

	#	extract light curve data
	jd_plot = np.add(track_data[:, 2], track_data[:, 3])
	std_mag_plot = track_data[:, 10]
	app_mag_plot = track_data[:, 11]
	distance_plot = track_data[:, 12]
	phase_plot = track_data[:, 13]
	start_line = track_data[0]
	stop_line = track_data[-1]
	start_jdate = np.add(start_line[2], start_line[3])
	stop_jdate = np.add(stop_line[2], stop_line[3])

	#	ensure between window start/stop
	if start_jdate > window_start_jd and stop_jdate < window_stop_jd:

		#	light curve is within window
		ntracks += 1

		#	if first light curve, start array
		if first_lc_flag is True:
			starlink_windowed_data = track_data
			first_lc_flag = False

		#	else append to list
		elif first_lc_flag is False:
			starlink_windowed_data = np.vstack([starlink_windowed_data, track_data])

print('\n{} track(s) extracted.'.format(ntracks))

###################################################################################################
#	extracting requested single track
###################################################################################################

#	extract first track
track_id = int(track_id_list[track_index])

print('\nUsing track index:	{}\nTrack ID:		{}'.format(track_index, track_id))

#	extract first track data
track_indices = np.where(starlink_windowed_data[:, 0]==track_id)[0]
starlink_track_data = starlink_windowed_data[track_indices]

###################################################################################################
#	setting simulation time data
###################################################################################################

###################################################################################################
#	extract track time data

mmt_line = starlink_track_data[0]

#	parse line elements
track_id, starlink_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = starlink_track_data[0]

start_jd = [jday, jfrac]

#	parse line elements
track_id, starlink_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = starlink_track_data[-1]

stop_jd = [jday, jfrac]

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	ensure time continuity
###################################################################################################

starlink_track_data_temp = []
sim_index = 0

for itt, starlink_line in enumerate(starlink_track_data):

	#	parse line data
	track_id, starlink_id, \
		jday, jfrac, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase = starlink_line
	
	step_jd = jday + jfrac

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	output_line = [
		jday, jfrac, current_sec, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase
	]

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		starlink_track_data_temp.append(output_line)
		sim_index += 1

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	parse previous line
		jday_prev, jfrac_prev, prev_sec, \
			year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
			mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		mag_std_last = mag_std
		mag_app_last = mag_app
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		mag_std_step = (mag_std_last - mag_std_prev) / npts
		mag_app_step = (mag_app_last - mag_app_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	fill first line
		interp_data = np.zeros([npts, len(output_line)])

		#	calculate interpolation data
		for n in range(1, len(interp_data) + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
			jday, jfrac = jday_jfrac(jdate)
			year, month, day, hour, minute, second = jd2date(jday, jfrac)

			#	calculate interpolated light curve data
			mag_std = mag_std_prev + (n * mag_std_step)
			mag_app = mag_app_prev + (n * mag_app_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			#	generate new output line
			output_line = [
				jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]

			#	add to output
			interp_data[n - 1, :] = output_line

		#	find required lines in interpolated data
		for interp_line in interp_data:

			#	set current simulation second
			interp_line_sec = interp_line[2]

			if interp_line_sec == sim_range_sec[sim_index]:
				starlink_track_data_temp.append(output_line)
				sim_index += 1

	#	save previous line
	if itt != 0:
		prev_line = output_line

#	ensure parsed MMT data matches simulation data
starlink_track_data = np.array(starlink_track_data_temp)

if len(starlink_track_data) == len(lc_sim_range):
	print('\nSimulation and MMT data matches...')
else:
	print('\nError: Simulation and MMT data does not match...')
	sys.exit()

starlink_signal = starlink_track_data[:, 10]

###################################################################################################
#	simulation printout				LOCKED
###################################################################################################

print('\n#####################################################################################')
print('\n	MMT light curve parameters: ')
print('		Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_start_date))
print('		Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_stop_date))
print('		Simulation time step:		{}'.format(time_step))
print('		Duration:			{}		[steps]'.format(lc_len_record))
print('						{}		[s]'.format(np.round(lc_duration_sec, 1)))

###################################################################################################
#	generating positioning data
###################################################################################################

###################################################################################################
#	set time data to be used

start_jd = lc_start_jd
stop_jd = lc_stop_jd

start_date = lc_start_date
stop_date = lc_stop_date

duration_sec = lc_duration_sec
len_record = lc_len_record
sim_range = lc_sim_range

###################################################################################################
#	generate eop data

#	EOP data
eop_dir = 'eop_files'
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	assemble eop path
eop_path = os.path.join(path_data, eop_dir)

#	generate interpolated eop_data
eop_log_level = 1
eop_data, dat_data = eop_interpolate(
	eop_path,
	eop_file,
	dat_file,
	sim_range,
	time_step,
	eop_log_level
)

###################################################################################################
#	use eop data to generate time objects

#	start dat value
dat = dat_data[0, -1]
	
#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate start time object
start_time_object = get_times(*start_date, ut_offset, dat)
	
#	end dat value
dat = dat_data[-1, -1]
	
#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate stop time object
stop_time_object = get_times(*stop_date, ut_offset, dat)

###################################################################################################
#	use top data to generate rotation matrices

print('\n[{}]	Generating rotation matrices:\n'.format(datetime.datetime.now()))

#	tt time
jdaytt_start = start_time_object.jday_tt
jfractt_start = start_time_object.jfrac_tt
jdtt_start = jdaytt_start + jfractt_start

#	ut1 time
jdayut1_start = start_time_object.jday_ut1
jfracut1_start = start_time_object.jfrac_ut1
jdut1_start = jdayut1_start + jfracut1_start

#	generate rotation matrices
pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
	jdtt_start, jdut1_start, eop_data, time_step
)

###################################################################################################
#	MODULE:	Horizons Positions

print('\n[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))

#	set horizons logging level
horz_logging_level = 1

sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
	path_data, horz_logging_level,
	start_time_object, stop_time_object,
	len_record, time_step,
	eop_path
)

#	find indices of light curve data
timecol = sun_icrf[:, 0]

###################################################################################################
#	MODULE:	Object Position

satpos_logging_level = 1

sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
	path_data, satpos_logging_level, len_record, time_step,
	start_time_object, stop_time_object,
	norad_id,
	eop_data, dat_data,
	pm_array, pn_array, st_array, c2t_array
)

###################################################################################################
#	MODULE:	Transformation

identification_str = 'mmt-testing'

trans_logging_level = 1
save_flag = False

sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
	= module_transformation(
		path_data, trans_logging_level, save_flag,
		start_time_object, stop_time_object,
		identification_str, site_str,
		len_record, time_step,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array,
		sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
	)

###################################################################################################
#	generate polar angles

#	set light curve time step
lc_time_step = time_step

#	extract light curve polar data
sun_polar_data, site_polar_data, phase_angles, jd_time_data = extract_fun(
	sim_range, lc_time_step, time_step,
	sun_orb, earth_orb, site_orb, sat_orb
)

###################################################################################################
#	initialise light curve generation inputs
###################################################################################################

###################################################################################################
#	geometry

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

###################################################################################################
#	attitude

#	convert to radians
rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad = np.radians(rot_vec_inc_deg)

#	generate attitude
initial_attitude = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])

###################################################################################################
#	attitude motion

#	rotation rate
rot_ang_per_sec = (np.pi * 2) / rot_period
rot_ang_per_sec = 0.0

#	define rotation axis in body frame
rot_vec_body_unit = np.array([1.0, 0.0, 0.0])

#	convert to rotation matrix
attitude_matrix = axis2mat(rot_ang_per_sec, rot_vec_body_unit)

#	extract euler angles
angles1 = np.array(mat2angles(attitude_matrix)[0:3])
angles2 = np.array(mat2angles(attitude_matrix)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

initial_attitude_motion = angles1

###############################################################################################
#	orbital rot vec motion
rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

###############################################################################################
#	generate brdf

#	Refractive Index
n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))

#	Specular ratio
s_brdf = 1.0 - d_brdf

#	generate brdf
brdf_data = np.zeros([nfaces, 5])

for i in range(nfaces):
	brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]

###################################################################################################
#	perform light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, initial_attitude, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	extract orbital positions of custom input vectors
illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_body[:, 31:34]

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]






# ###############################################################################################
# #	generate light curve
# ###############################################################################################
# 
# #	standard distance flag. sets distance to 1000 km when true
# std_dist_flag = True
# 
# #	generate sim time
# sim_time = np.linspace(0, duration_sec, len_record)
# 
# #	initilise lightcurve output
# final_light_curve = np.zeros((lc_len_record, 3))
# final_light_curve[:, 1] = sim_time
# 
# #	run light curve generation function
# light_curve = glim_slc(sun_body_data, site_body_data, final_light_curve, \
# 		start_jd, body_vectors, body_areas, brdf_data, std_dist_flag)
# 	
# light_curve_data = light_curve[:, 2]
# 
# light_curve_seconds = light_curve[:, 1]
# 
# ###############################################################################################
# #	plot single lightcurve
# 
# #	initialise light curve axes
# fig_lightcurve, ax_lightcurve = plt.subplots(figsize=(10, 6))
# 
# #	true light curve
# ax_lightcurve.plot(
# 	light_curve_seconds, starlink_signal, label='Real'
# )
# 
# #	simulated light curve
# ax_lightcurve.plot(
# 	light_curve_seconds, light_curve_data, label='Synthetic'
# )
# 
# #	finalise plot
# ax_lightcurve.legend()
# ax_lightcurve.grid()
# 
# #	reverse y axis
# #ax_lightcurve.set_ylim([0, 10])
# ax_lightcurve.invert_yaxis()
# 
# #	calculate RMS
# rms_single = rms_calc(starlink_signal, light_curve_data)
# print('\nRMS = ', rms_single)
# 
# plt.show()
# sys.exit()








###############################################################################################
#	brdf fitting
###############################################################################################

#	set d_brdf loop values
d_brdf_loop_range = [0.01, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 0.99]
# d_brdf_loop_range = [0.6, 0.7, 0.8, 0.9]

#	looping on d brdf values
for d_brdf in d_brdf_loop_range:

	print('\nWorking on slice:	d_brdf = ', d_brdf)

	#	generate subdirectory
	out_dir = 'd_brdf=0-{:02d}'.format(int(d_brdf * 100))
	
	if not os.path.exists(os.path.join(out_path, out_dir)):
		print('\nCreating Directory: ', os.path.join(out_path, out_dir))
		os.makedirs(os.path.join(out_path, out_dir))
	
	#	range for brdf parameters
	step = 0.01
	m_brdf_range = np.arange(0.01, 0.99 + step, step)
	omega_brdf_range = np.arange(0.01, 0.99 + step, step)
	
	#	generate an array of inputs
	inputs = []
	for omega_brdf in omega_brdf_range:
		for m_brdf in m_brdf_range:
	
			inputs.append([
				starlink_id,
				rot_vec_azm_deg,
				rot_vec_inc_deg,
				d_brdf,
				m_brdf,
				omega_brdf,
				nfaces,
				duration_sec,
				len_record,
				start_jd,
				sun_body_data,
				site_body_data,
				body_vectors,
				body_areas
			])
	
	inputs = np.array(inputs)
	
	parral_flag = True
	
	if parral_flag is True:
		
		#	initialise multiproccessing pool of workers
		ncores = 8
		p = Pool(ncores)
		
		#	map the total data array across the access function
		for light_curve_output in tqdm.tqdm(
			p.imap_unordered(starlink_brdf_fit, inputs), total=len(inputs)
			):
		
			#	parse outputs
			light_curve, out_file = light_curve_output
		
			###############################################################################################
			#	save light curve
			###############################################################################################
			#	save light curve to file
			np.savetxt(
				os.path.join(out_path, out_dir, out_file),
				light_curve,
				delimiter=','
			)
	
	elif parral_flag is False:
		#	run inputs sequentially
		for input_line in inputs:
			lightcurve_file, final_output_array = starlink_brdf_fit(input_line)
	
###############################################################################################
#	calculate run time
###############################################################################################
#	calculate sim time
sim_end = time.time()
sim_time = np.round((sim_end - sim_start) / 60, 1)

print('\n[{}] Simulation Finished.'.format(datetime.datetime.now()))
print('\n	- Time taken = {} mins.'.format(sim_time))
