###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front end
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	scipy
import scipy

#	custom functions
from tools.functions.functions_list import correct_nans
from tools.domain_splitting import cheb_app, cheb_app_required

#	shutil for recursive delete
import shutil

#	matplotlib
import matplotlib.pyplot as plt

##################################################################################################
#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

#	clear terminal
call(['clear'])
print('\n####	Script Start.')

###################################################################################################
#	initilisation
###################################################################################################

##################################################
#	inputs

#	define base file
base_attit = '0-attit'
base_file = 'azm0_elv0'

#	number of geom files per run
ngeom = 1

sim_step = 0.1

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)

# geom_cases
geom_case = 'cylinder'
geom_case = 'div1'
geom_case = 'cube'
geom_case = 'div5'

# reflection_cases
reflection_cases = ['diffuse', 'specular']

# attitude_cases
attit_cases = ['0-attit', 'x-rot', 'y-rot', 'z-rot']

#	generate case designation
case_desig = '_'.join([geom_case, '{}'.format(sim_step)])

#	generate output directories
# distance_dir = 'distance_data_{}'.format(case_desig)
rms_dir = 'rms_data_{}'.format(case_desig)

lightcurve_dir = 'lightcurve_data_{}'.format(case_desig)

# distance_path = os.path.join(path_data, 'distance_dir', distance_dir)
rms_path = os.path.join(path_data, 'rms_dir', rms_dir)

lightcurve_path = os.path.join(path_data, 'lightcurve_dir', lightcurve_dir, 'lightcurve_files')

##################################################
#	checking for output directory
if not os.path.exists(rms_path):
	print('Making Directory:\n', rms_path)
	os.mkdir(rms_path)

##################################################
#	generate list of case paths

case_paths = []
for reflection_case in reflection_cases:

	for attit_case in attit_cases:

		#	generate case name
		case_name = '_'.join([attit_case, geom_case])

		#	generate case path
		case_path = os.path.join(reflection_case, case_name)
		case_paths.append(case_path)

###################################################################################################
#	main loop
###################################################################################################

print('\nCase Path Loop...')
for case_path in case_paths:

	##################################################
	#	parse inputs
	reflection_case, current_case = case_path.split('/')
	current_attit, current_geom = current_case.split('_')

	print('\n##########################################################')

	print('\n	Acquiring Case Data: ')
	print('		Reflection Case = ', reflection_case)
	print('		Attitude Case = ', current_attit)
	print('		Geometry Case = ', current_geom)

	if os.path.exists(os.path.join(rms_path, case_path)):

		#	list directory contents
		dir_content = os.listdir(os.path.join(rms_path, case_path))
		
		if not dir_content == []:

			print(os.path.join(rms_path, case_path))
			user_in = input('\n		Delete current rms data for this casepath? [y/n]: ')
		
			if user_in == 'y':
				print('		Deleting current contents..')
				for item in dir_content:
					os.remove(os.path.join(rms_path, case_path, item))
		
			elif user_in != 'y':
				print('\n		Error, exiting.')
				sys.exit()

	###################################################################################################
	#	acquire base case data

	print('\n	Acquiring Base Data: ')

	#	set base geometry
	base_geom = current_geom

	#	generate base case name
	base_case_name = '_'.join([base_attit, current_geom])
	
	#	full case directory
	case_dir = os.listdir(os.path.join(lightcurve_path, reflection_case, base_case_name))
	
	#	read in files in case directory
	temp_case_data = []
	for case in case_dir:
		case_num, attit_str, azm_str, elv_str, _, _ = case.split('_')
		case_str = '_'.join([azm_str, elv_str])
		temp_case_data.append([int(case_num), attit_str, case_str, case])

	#	sort files
	temp_case_data = sorted(temp_case_data, key=lambda x: x[0])
	num_cases = len(temp_case_data)

	#	extract base case file
	for case_line in temp_case_data:
		if case_line[2] == base_file:
			base_case_num = case_line[0]
			base_case_attit = case_line[1]
			base_case_azmelv = case_line[2]
			base_case_file = case_line[3]
			break

	#	load base file
	base_case_path = os.path.join(lightcurve_path, reflection_case, base_case_name, base_case_file)
	base_case_data = np.genfromtxt(base_case_path, delimiter=',')

	#	set base signal
	base_signal = base_case_data[:, 2]
	
	#	convert infinite values
	if np.isinf(base_signal).any():
	 	for i, case_line in enumerate(base_signal):
	 		if np.isinf(case_line):
	 			base_signal[i] = 30
	
	#	set base domain
	base_domain = np.arange(0, len(base_signal), 1)
	
	print('		Base Reflection Case = ', reflection_case)
	print('		Base Attitude Case = ', base_case_attit)
	print('		Base Geometry Case = ', base_geom)

	###################################################################################################
	#	transform base signal into fourier space

	print('\n	Generating Hilbert Coefficients of Base Signal.')
			
	x_base = base_domain
	y_base = base_signal

#	##################################################
#	#	mimic signal in negative domain for sines only approximation
#	x_neg = -x_base[1:]
#	x_neg = np.flip(x_neg)
#	y_neg = y_base[1:]
#	y_neg = np.flip(y_neg)
#
#	x_base = np.append(x_neg, x_base)
#	y_base = np.append(y_neg, y_base)
#
#	##################################################
#	#	get starting coefficents
#
#	#	set P as domain length
#	P = len(x_base) - 1
#	
#	#	find a0 and b0
#	a_0 = (2 / P) * np.trapz(y_base, x_base)
#	b_0 = 0.0
#
#	#	starting number of terms
#	N = 1000
#
#	#	set range of n
#	nvals = np.arange(1, N, 1)
#		
#	#	initialise coefficients
#	base_coefs = np.zeros([len(nvals) + 1])
#	base_coefs[0] = a_0
#
#	#	find remaining coefficients
#	for i, n in enumerate(nvals):
#
#		#	find a_n coefficient
#		a_n = (2 / P) * np.trapz(y_base * np.cos(2 * np.pi * x_base * (n / P)), x_base)
#		if -1e-10 < a_n < 1e-10:
#			a_n = 0.0
#
#		#	find b_n coefficient
#		b_n = (2 / P) * np.trapz(y_base * np.sin(2 * np.pi * x_base * (n / P)), x_base)
#		if -1e-10 < b_n < 1e-10:
#			b_n = 0.0
#
#		#	add to output array
#		base_coefs[i + 1] = a_n
#
#		#	check for zero b coefficent
#		if b_n != 0.0:
#			print('Non-Zero b coefficient!')
#			sys.exit()
#	
#	##################################################
#	#	generate fourier series
#	approx_base_signal = a_0 / 2
#	
#	for i, n in enumerate(nvals):
#		approx_base_signal += base_coefs[i + 1] * np.cos(2 * np.pi * n * x_base / P)\
#	
#	#	calculate rms and norm
#	current_rms = rms_calc(y_base, approx_base_signal)
#	base_norm = np.linalg.norm(base_coefs)
	
	###################################################################################################
	#	acquire case data

	#	lightcurve input path
	input_path = os.path.join(lightcurve_path, case_path)
	input_dir = os.listdir(input_path)
	
	#	distance output path
#	output_path = os.path.join(distance_path, case_path)
	output_path = os.path.join(rms_path, case_path)
	
	#	read in files in case directory
	case_data = []

	for case in input_dir:
		num_str, attit_str, azm_str, elv_str, geom_str, _ = case.split('_')
		case_str = '_'.join([azm_str, elv_str])
		case_data.append([int(num_str), num_str, attit_str, geom_str, case_str, case])

	#	sort files
	case_data = sorted(case_data, key=lambda x: x[0])
	num_cases = len(case_data)

	#	prepare for case loop
	previous_case = 'empty'
	previous_case_file = 'empty'
	previous_case_name = 'empty'
	first_flag = 1

	###################################################################################################
	#	loop through illuminaion vector cases
	print('\n		Looping through Subcases...')
	for case in case_data:

		#	parse case data
		case_num = case[0]; num_str = case[1]; case_attit = case[2]
		case_geom = case[3]; case_name = case[4]; case_file = case[5]

		azimuth = int(case_name.split('_')[0][3:])
		elevation = int(case_name.split('_')[1][3:])

		#	read case data file
		raw_case_data = np.genfromtxt(
			os.path.join(lightcurve_path, case_path, case_file),
			delimiter=','
		)

		#	extract case signal
		case_signal = raw_case_data[:, 2]

		#	define space
		case_domain = np.arange(0, len(case_signal), 1)

		if case_name != previous_case_name:
			previous_case_name = case_name

		###############################################################################################
		#	NAN value correction
	
		if np.isnan(case_signal).any():
			print('\nFound nan values.')
			print('\nReflection case = ', reflection_case)
			print('current attit = ', current_attit)
			print('current geom = ', current_geom)
	
			#	number of nan occurances
			nnans = np.isnan(case_signal).sum()
			print('{:d} nans found.'.format(nnans))

			#	correct nans in case signal
			case_signal = correct_nans(case_signal)

		###############################################################################################
		#	generate output file name
		case_file_name = '_'.join([num_str, case_attit, case_geom, case_name, reflection_case]) + '.dist'
#		coef_file_name = '_'.join([num_str, case_attit, case_geom, case_name, reflection_case]) + '.coefs'

		##################################################
		#	start of first / new run
		if case_name != previous_case:

			#	set geometry number
			geom_num = 1
	
			#	if first run
			if first_flag == 1:
				#	initilize empty distance arrays
#				distances = np.zeros([ngeom])
				rmses = np.zeros([ngeom])

			#	else if new run
			else:
				#	save previous run
				np.savetxt(
					os.path.join(
#						distance_path,
						rms_path,
						reflection_case,
						'_'.join([case_attit, case_geom]),
						case_file_name,
					),
					rmses,
					delimiter=','
				)

				#	initilize empty distance arrays
#				distances = np.zeros([ngeom])
				rmses = np.zeros([ngeom])

		##################################################
		#	continuation of ongoing run
		elif case_name == previous_case:
			pass

		#	convert infinite values
		very_dim = 30
		if np.isinf(case_signal).any():
			print('Correcting infinite values.')
			for i, case_line in enumerate(case_signal):
				if np.isinf(case_line):
					case_signal[i] = very_dim

		#######################################################################################
		#	Fourier approximation

		x_case = case_domain
		y_case = case_signal

#		##################################################
#		#	mimic signal in negative domain for sines only approximation
#		x_neg = -x_case[1:]
#		x_neg = np.flip(x_neg)
#		y_neg = y_case[1:]
#		y_neg = np.flip(y_neg)
#
#		x_case = np.append(x_neg, x_case)
#		y_case = np.append(y_neg, y_case)
#
#		##################################################
#		#	get starting coefficents
#
#		#	set P as domain length
#		P = len(x_case) - 1
#			
#		#	find a0 and b0
#		a_0 = (2 / P) * np.trapz(y_case, x_case)
#		b_0 = 0.0
#
#		#	starting number of terms
#		N = 1000
#
#		first_flag = True
#
#		#	set range of n
#		nvals = np.arange(1, N, 1)
#				
#		#	initialise coefficients
#		case_coefs = np.zeros([len(nvals) + 1])
#		case_coefs[0] = a_0
#	
#		#	find remaining coefficients
#		for i, n in enumerate(nvals):
#	
#			#	find a_n coefficient
#			a_n = (2 / P) * np.trapz(y_case * np.cos(2 * np.pi * x_case * (n / P)), x_case)
#			if -1e-10 < a_n < 1e-10:
#				a_n = 0.0
#	
#			#	find b_n coefficient
#			b_n = (2 / P) * np.trapz(y_case * np.sin(2 * np.pi * x_case * (n / P)), x_case)
#			if -1e-10 < b_n < 1e-10:
#				b_n = 0.0
#	
#			#	add to output array
#			case_coefs[i + 1] = a_n
#	
#			#	check for zero b coefficent
#			if b_n != 0.0:
#				print('Non-Zero b coefficient!')
#				sys.exit()
#	
#		##################################################
#		#	generate fourier series
#		approx_case_signal = a_0 / 2
#	
#		for i, n in enumerate(nvals):
#			approx_case_signal += case_coefs[i + 1] * np.cos(2 * np.pi * n * x_case / P)\
#	
#		#	calculate rms and norm
#		current_rms = rms_calc(y_case, approx_case_signal)
#		case_norm = np.linalg.norm(case_coefs)
#	
#		##################################################
#		#	calculate distance
#		step_distance = np.linalg.norm(np.subtract(base_coefs, case_coefs))



		##################################################
		#	calculate distance
		step_rms = rms_calc(base_signal, case_signal)

		###############################################################################################
		#	finalise case loop

		#	add to distances array
#		distances[geom_num - 1] = step_distance
		rmses[geom_num - 1] = step_rms

		#	update external values
		previous_case = case_name
		previous_case_file = case_file_name

		#	itterate geometry number
		geom_num += 1

		#	reset first flag
		first_flag = 0

		#	generate reflection case directory if required
#		reflection_case_path = os.path.join(distance_path, reflection_case)
		reflection_case_path = os.path.join(rms_path, reflection_case)

		if not os.path.exists(reflection_case_path):
			os.mkdir(reflection_case_path)

		#	generate case directory if required
		case_name_dir = '_'.join([case_attit, case_geom])
		case_path_out = os.path.join(reflection_case_path, case_name_dir)
		if not os.path.exists(case_path_out):
			os.mkdir(case_path_out)

		#	save distances file
		np.savetxt(
			os.path.join(case_path_out, case_file_name),
#			distances,
			rmses,
			delimiter=','
		)

#		#	save coefficients file
#		np.savetxt(
#			os.path.join(case_path_out, coef_file_name),
##			distances,
#			rmses,
#			delimiter=','
#		)

#		print('	Saving distance file: ', case_file_name)
#		print('	Containing distances: ', distances)

	print('\n		- Hilbert Analysis of Subcases Complete.')

print('\nEND')
