###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Light curve rendering in the orbital frame
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	scipy
from scipy import interpolate

#	progress bar
import progressbar

#	transformation
from tools.functions.functions_transform import site_vec, rv2kep, gen_vectors_render2, \
	rot1, rot2, rot3, rot_mat, axis2mat, mat2angles

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	case file generation
from tools.file_generation.case_generation import gen_case

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d
from descartes import PolygonPatch

#	shapely
import shapely.geometry as sg
# from shapely.geometry import Point
# from shapely.geometry import Polygon as sh
from shapely.ops import cascaded_union

##############################################
#	video command
#	ffmpeg -framerate 10 -pattern_type glob -i '*.png' -c:v libx264 -pix_fmt yuv420p ../orb.mp4

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT Render Orb')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

###################################################################################################
#	light curve generation inputs

mmt_file = 'starlink-29_44243.txt'

track_index = 6

#	parse filename
name_str = mmt_file.split('-')[0]
object_id = int(mmt_file.split('-')[1].split('_')[0])
norad_id = int(mmt_file.split('-')[1].split('_')[1][:-4])

local_input_path = os.path.join('input_dir', 'mmt_files', 'starlink')
read_path = os.path.join(path_data, local_input_path)

geom_auto_del = True
render_auto_flag = True

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str = 'starlink_full'
geom_file = geom_str + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0
geom_offset_y = 270.0

#	attitude
rot_vec_azm_deg = 180.0
rot_vec_inc_deg = 65.0

#	attitude motion
rot_period = 0.0

#	simulation time step
time_step = 0.1

###################################################################################################
#	load geometry

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])
#solar_panel_norm_body = np.array([0.0, 0.0, 1.0])



###################################################################################################
#	generate plot data
###################################################################################################

##################################################
#	attitude

#	convert to radians
rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad = np.radians(rot_vec_inc_deg)

#	generate initial t = 0 attitude
init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])

################################################
#	initial attitude

#	attitude matrix
init_attit_mat = rot_mat(init_attit)

#	transpose for inverse matrix
init_attit_mat_inv = np.transpose(init_attit_mat)

#	generate orbital mesh from the body mesh
orb_mesh_step = mesh.Mesh(body_mesh.data.copy())

#	rotate into orbital frame
orb_mesh_step.rotate_using_matrix(init_attit_mat)

orb_vectors_step = orb_mesh_step.vectors
orb_areas = orb_mesh_step.areas
orb_vectors = orb_mesh_step.vectors
orb_normals = orb_mesh_step.normals


x_ref_orb = np.array([-1.0, 0.0, 0.0])
y_ref_orb = np.array([0.0, 1.0, 0.0])
z_ref_orb = np.array([0.0, 0.0, 1.0])

x_geom_orb = np.array([4.22618262e-01,  1.10990693e-16, -9.06307787e-01])
y_geom_orb = np.array([0.0000000e+00, -1.0000000e+00, -1.2246468e-16])
z_geom_orb = np.array([-9.06307787e-01,  5.17558102e-17, -4.22618262e-01])

earth_orb_vec = np.array([-6.81473430e+03, -5.64000586e-13,  7.13501192e-13])
sat_orb_vel = np.array([4.64354589e-03,  7.35184171e+00, -1.77635684e-15])
orb_norm_orb_unit = np.array([0., 0., 1.])

sun_orb_vec = np.array([-5.09076642e+07, -1.39520170e+07,  1.42182477e+08])
site_orb_vec = np.array([-461.68530059,  167.85062484,  433.34688502])
rot_vec_orb_unit = np.array([4.22618262e-01,  1.10990693e-16, -9.06307787e-01])

###################################################################################################
#	initilise figure

figure3d = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
axes3d = mplot3d.Axes3D(figure3d)

###############################################
#	set title, axis labels and axis limits

axes3d.set_title('Render Plot')

axes3d.set_xlim([-15, 15])
axes3d.set_ylim([-15, 15])
axes3d.set_zlim([-15, 15])

axes3d.set_xlabel('x orbital axis')
axes3d.set_ylabel('y orbital axis')
axes3d.set_zlabel('z orbital axis')

###################################################################################################
#	main render loop

print('\nPerforming Render...\n')

###############################################################################################
#	step initialisation

origin = np.array([0.0, 0.0, 0.0])

###############################################################################################
#	generate plottable orbital frame axis vectors

#	orbital frame reference axes scale
orb_ref_vec_scale = 15

#	text font size
font_size = 14

###############################################
#	earth vector
earth_vec_plot = orb_ref_vec_scale * np.divide(
	np.vstack((origin, earth_orb_vec)), np.linalg.norm(earth_orb_vec)
)
a = Arrow3D(earth_vec_plot[:, 0], earth_vec_plot[:, 1], earth_vec_plot[:, 2],
	mutation_scale=20, lw=2, arrowstyle="-|>", color="g"
)
axes3d.add_artist(a)
axes3d.text(earth_vec_plot[1, 0], earth_vec_plot[1, 1], earth_vec_plot[1, 2],
	'Earth Vec', fontsize=font_size, fontweight='bold'
)
###############################################
#	velocity vector
vel_vec_plot = orb_ref_vec_scale * np.divide(
	np.vstack((origin, sat_orb_vel)), np.linalg.norm(sat_orb_vel)
)
a = Arrow3D(vel_vec_plot[:, 0], vel_vec_plot[:, 1], vel_vec_plot[:, 2],
	mutation_scale=20, lw=2, arrowstyle="-|>", color="g"
)
axes3d.add_artist(a)
axes3d.text(vel_vec_plot[1, 0], vel_vec_plot[1, 1], vel_vec_plot[1, 2],
	'Velocity Vec', fontsize=font_size, fontweight='bold'
)
###############################################
#	orbital plane normal
orb_norm_orb_unit_plot = orb_ref_vec_scale * np.divide(
	np.vstack((origin, orb_norm_orb_unit)), np.linalg.norm(orb_norm_orb_unit)
)
a = Arrow3D(orb_norm_orb_unit_plot[:, 0], orb_norm_orb_unit_plot[:, 1], orb_norm_orb_unit_plot[:, 2],
	mutation_scale=20, lw=2, arrowstyle="-|>", color="g"
)
axes3d.add_artist(a)
axes3d.text(orb_norm_orb_unit_plot[1, 0], orb_norm_orb_unit_plot[1, 1], orb_norm_orb_unit_plot[1, 2],
	'Orb Plane Normal', fontsize=font_size, fontweight='bold'
)
###############################################################################################
#	generate plottable light curve generation vectors

lc_gen_vec_scale = 10

###############################################
#	illumination vector
illum_vec_plot = lc_gen_vec_scale * np.divide(
	np.vstack((origin, sun_orb_vec)), np.linalg.norm(sun_orb_vec)
)
a = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2],
	mutation_scale=20, lw=2, arrowstyle="-|>", color="r"
)
axes3d.add_artist(a)
axes3d.text(illum_vec_plot[1, 0], illum_vec_plot[1, 1], illum_vec_plot[1, 2],
	'Illum Vec', fontsize=font_size, fontweight='bold'
)
###############################################
#	observation vector
obs_vec_plot = lc_gen_vec_scale * np.divide(
	np.vstack((origin, site_orb_vec)), np.linalg.norm(site_orb_vec)
)
a = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2],
	mutation_scale=20, lw=2, arrowstyle="-|>", color="b"
)
axes3d.add_artist(a)
axes3d.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2],
	'Obs Vec', fontsize=font_size, fontweight='bold'
)
###############################################
#	rotation vector
rot_vec_plot = lc_gen_vec_scale * np.divide(
	np.vstack((origin, rot_vec_orb_unit)), np.linalg.norm(rot_vec_orb_unit)
)
a = Arrow3D(rot_vec_plot[:, 0], rot_vec_plot[:, 1], rot_vec_plot[:, 2],
	mutation_scale=20, lw=2, arrowstyle="-|>", color="m"
)
axes3d.add_artist(a)
axes3d.text(rot_vec_plot[1, 0], rot_vec_plot[1, 1], rot_vec_plot[1, 2],
	'Rot Vec', fontsize=font_size, fontweight='bold'
)
###############################################################################################
#	generate plottable body frame axis vectors

body_axes_scale = 5

###############################################
#	add body x axis
bodyx_vec_plot = body_axes_scale * np.vstack((origin, x_geom_orb))
a = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
	mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
)
axes3d.add_artist(a)
axes3d.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1], bodyx_vec_plot[1, 2],
	'x_b', fontsize=font_size, fontweight='bold'
)
###############################################
#	add body y axis
bodyy_vec_plot = body_axes_scale * np.vstack((origin, y_geom_orb))
a = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
	mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
)
axes3d.add_artist(a)
axes3d.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	'y_b', fontsize=font_size, fontweight='bold'
)
###############################################
#	add body z axis
bodyz_vec_plot = body_axes_scale * np.vstack((origin, z_geom_orb))
a = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
	mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
)
axes3d.add_artist(a)
axes3d.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	'z_b', fontsize=font_size, fontweight='bold'
)

###############################################################################################
#	plot mesh

#	set face alpha
alpha = 0.5

for face_index, orb_vector in enumerate(orb_vectors):

	#	Add final vert to vector array
	orb_vector = np.vstack([orb_vector, orb_vector[0, :]])
	#	extract x, y and z values
	xs = orb_vector[:,0]
	ys = orb_vector[:,1]
	zs = orb_vector[:,2]
	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]
	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_alpha(alpha)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	pc.set_facecolor([0.5, 0.5, 0.5])
	axes3d.add_collection3d(pc)


plt.show()
sys.exit()
