###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	Light curve rendering in the orbital frame
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	scipy
from scipy import interpolate

#	progress bar
import progressbar

#	transformation
from tools.functions.functions_transform import site_vec, rv2kep, gen_vectors_render2, \
	rot1, rot2, rot3, rot_mat, axis2mat, mat2angles

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	case file generation
from tools.file_generation.case_generation import gen_case

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d
from descartes import PolygonPatch

#	shapely
import shapely.geometry as sg
# from shapely.geometry import Point
# from shapely.geometry import Polygon as sh
from shapely.ops import cascaded_union

##############################################
#	video command
#	ffmpeg -framerate 10 -pattern_type glob -i '*.png' -c:v libx264 -pix_fmt yuv420p ../orb.mp4

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT Render Orb')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

###################################################################################################
#	light curve generation inputs

mmt_file = 'starlink-29_44243.txt'

track_index = 6

#	parse filename
name_str = mmt_file.split('-')[0]
object_id = int(mmt_file.split('-')[1].split('_')[0])
norad_id = int(mmt_file.split('-')[1].split('_')[1][:-4])

local_input_path = os.path.join('input_dir', 'mmt_files', 'starlink')
read_path = os.path.join(path_data, local_input_path)

geom_auto_del = True
render_auto_flag = True

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str = 'starlink_full'
geom_file = geom_str + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
rot_vec_azm_deg = 180.0
rot_vec_inc_deg = 75.0
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 0.0

#	attitude motion
rot_period = 0.0

#	simulation time step
time_step = 0.1

###################################################################################################
#	load geometry

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])
#solar_panel_norm_body = np.array([0.0, 0.0, 1.0])

###################################################################################################
#	set render data

#	directory data
graphics_dir = 'graphics_dir'
render_dir = 'render_dir'
render_geom_dir = 'geom_files'
render_img_dir = 'image_files'

#	orb image data
render_image_dir = os.path.join(path_data, graphics_dir, render_dir, render_img_dir)
render_img_dir_list = os.listdir(render_image_dir)
nimg_files = len(render_img_dir_list)

#	orb image data
render_geom_dir = os.path.join(path_data, graphics_dir, render_dir, render_geom_dir)
render_geom_dir_list = os.listdir(render_geom_dir)
ngeom_files = len(render_geom_dir_list)

#	render outputs
render_data_orb = 'render_data_orb.txt'
render_data_body = 'render_data_body.txt'
render_data_inertial = 'render_data_inertial.txt'

#	render position data
sat_orb_file = 'sat_orb.txt'
sun_orb_file = 'sun_orb.txt'
earth_orb_file = 'earth_orb.txt'
site_orb_file = 'site_orb.txt'

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])
	
print('\n#####################################################################################')
print('Simulation Inputs: ')
print('\n	Input File: ')
print('\n		Working from MMT data input file: 	{}'.format(mmt_file))
print('		Reading from input path: 		{}'.format(local_input_path))
print('\n			Name: 	{}'.format(name_str))
print('			Object ID: 	{}'.format(object_id))
print('			Norad ID: 	{}'.format(norad_id))
print('			Track ID: 	{}'.format(track_index))
print('\n	Light Curve Inputs: ')
print('\n			Geom File: 		{}'.format(geom_file))
print('			Rot Vec Azimuth: 	{}'.format(rot_vec_azm_deg))
print('			Rot Vec Inclination: 	{}'.format(rot_vec_inc_deg))
print('			Rotation Period: 	{}'.format(rot_period))
# print('			BRDF: 			m = {}, omega = {}, m = {}'.format(d_brdf, omega_brdf, m_brdf))
print('\n	Observation Site Inputs: ')
print('\n			Site String: 	{}'.format(site_str))
print('			Latitude: 	{}'.format(np.degrees(site_latgd)))
print('			Longitude: 	{}'.format(np.degrees(site_lon)))
print('			Altitude: 	{} m'.format(site_alt))


###############################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

###################################################################################################
#	load MMT file
###################################################################################################

#	set time
mmt_load_start = time.time()

print('\n[{}] Loading MMT data...'.format(datetime.datetime.now()))

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file

#	strip header
header_text = mmt_data[7]
raw_data = mmt_data[8:]

print('\n	Parsing MMT data...')

#	initialise output
n_rows = 14
parsed_data = np.zeros([len(raw_data), n_rows])

for i, line in enumerate(raw_data):

	#	split line
	date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
		distance, phase, channel, track_id = line.split(' ')

	#	parse data and set data types
	mag_std = float(mag_std)
	mag_app = float(mag_app)
	distance = float(distance)
	phase = float(phase)
	track_id = int(track_id)

	#	parse date
	year, month, day = date_str.split('-')
	year = int(year)
	month = int(month)
	day = int(day)

	#	parse time
	hour, minute, second = time_str.split(':')
	hour = int(hour)
	minute = int(minute)
	second = np.round(float(second), 1)

	#	calculate julian date
	jday, jfrac = date2jd(year, month, day, hour, minute, second)

	#	generate output line
	output_line = [
		track_id, object_id,
		jday, jfrac,
		year, month, day,
		hour, minute, second,
		mag_std, mag_app,
		distance, phase
	]

	#	add to output array
	parsed_data[i, :] = output_line

print('		Total data length = ', len(parsed_data))
print('		Total number of objects = ', len(np.unique(parsed_data[:, 1])))
print('		Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

###################################################################################################
#	extracting data between specified windows

print('\n	Seaching for tracks inside time window:')

#	start window
window_start_date = [2019, 5, 25, 0, 0, 0]
window_start_jd = np.add(*date2jd(*window_start_date))
print('\n		Window start 	= ', window_start_date[0:3])

#	stop window
window_stop_date = [2019, 6, 1, 0, 0, 0]
window_stop_jd = np.add(*date2jd(*window_stop_date))
print('		Window stop 	= ', window_stop_date[0:3])

#	extract track ids
track_id_list = np.unique(parsed_data[:, 0])

#	loop through tracks
ntracks = 0
first_lc_flag = True
for track_id in track_id_list:

	#	find track ids
	track_indices = np.where(parsed_data[:, 0]==track_id)[0]

	#	extract full data
	track_data = parsed_data[track_indices, :]

	#	extract light curve data
	jd_plot = np.add(track_data[:, 2], track_data[:, 3])
	std_mag_plot = track_data[:, 10]
	app_mag_plot = track_data[:, 11]
	distance_plot = track_data[:, 12]
	phase_plot = track_data[:, 13]
	start_line = track_data[0]
	stop_line = track_data[-1]
	start_jdate = np.add(start_line[2], start_line[3])
	stop_jdate = np.add(stop_line[2], stop_line[3])

	#	ensure between window start/stop
	if start_jdate > window_start_jd and stop_jdate < window_stop_jd:

		#	light curve is within window
		ntracks += 1

		#	if first light curve, start array
		if first_lc_flag is True:
			windowed_data = track_data
			first_lc_flag = False

		#	else append to list
		elif first_lc_flag is False:
			windowed_data = np.vstack([windowed_data, track_data])

print('\n		{} track(s) extracted.'.format(ntracks))

###############################################################################################
#	extracting requested single track

print('\n		Extracting requested track index {}'.format(track_index))

#	set track array index
track_array_index = track_index - 1
track_id = int(track_id_list[track_array_index])

track_indices = np.where(windowed_data[:, 0]==track_id)[0]
track_data = windowed_data[track_indices]

###############################################################################################
#	extract track time data

mmt_line = track_data[0]

#	parse line elements
track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = track_data[0]

start_jd = [jday, jfrac]

#	parse line elements
track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = track_data[-1]

stop_jd = [jday, jfrac]

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###############################################################################################
#	display initialisation time
mmt_load_end = time.time()
mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)

print('\n	MMT Load Finished.')
print('\n		- Time taken = {} s.'.format(mmt_load_time))

print('\n#####################################################################################')
print('Light curve parameters: ')
print('	Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_start_date))
print('	Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_stop_date))
print('	Simulation time step:		{}'.format(time_step))
print('	Duration:			{}		[steps]'.format(lc_len_record))
print('					{}		[s]'.format(np.round(lc_duration_sec, 1)))
print('\n#####################################################################################')

###################################################################################################
#	acquire TLE
###################################################################################################

#	set time
tle_start = time.time()

print('\n[{}] Acquiring TLEs...'.format(datetime.datetime.now()))

#	TLE request dates, add day either side to increase tle find chance
tle_start_date = jd2date(*jday_jfrac(lc_start_jd - 1))
tle_stop_date = jd2date(*jday_jfrac(lc_stop_jd + 1)) 

#	generate TLE file name
tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)

#	assemble TLE path
tle_path = os.path.join(path_data, 'input_dir', 'tle_files')

#	if exists, load
if os.path.isfile(os.path.join(tle_path, tle_file)):

	print('\n	Reading existing TLE file.')

	#	load file
	with open(os.path.join(tle_path, tle_file), 'r') as file:
		tle_data = file.readlines()

else:
	print('\n	Requesting TLE file...')

	#	request TLEs
	tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)

	#	save file
	with open(os.path.join(tle_path, tle_file), 'w') as file:
		for line in tle_data:
			file.write(line)

#	grab first TLE pair
tle_line1 = tle_data[1]
tle_line2 = tle_data[2]

#	mean mot rad/sec
orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
orbital_mot = (2 * np.pi) / orbital_period

###############################################################################################
#	display TLE acquisition time
tle_end = time.time()
tle_time = np.round((tle_end - tle_start), 1)

print('\n	TLE Load Finished.')
print('\n		- Time taken = {} s.'.format(tle_time))

###################################################################################################
#	select time data to use
###################################################################################################

sim_range = lc_sim_range
len_record = lc_len_record
start_date = lc_start_date
stop_date = lc_stop_date

###################################################################################################
#	generate eop data arrays
###################################################################################################

#	set time
eop_start = time.time()

print('\n[{}] Generating EOP data array...'.format(datetime.datetime.now()))

#	EOP data
eop_dir = 'eop_files'
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	assemble eop path
eop_path = os.path.join(path_data, eop_dir)

#	generate interpolated eop_data
eop_log_level = 1
eop_data, dat_data = eop_interpolate(
	eop_path,
	eop_file,
	dat_file,
	sim_range,
	time_step,
	eop_log_level
)

###################################################################################################
#	display time
eop_end = time.time()
eop_time = np.round((eop_end - eop_start), 1)

print('\n	EOP data generated.')
print('\n		- Time taken = {} s.'.format(eop_time))

###################################################################################################
#	generate time objects
###################################################################################################

#	start dat value
dat = dat_data[0, -1]
	
#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate start time object
start_time_object = get_times(*start_date, ut_offset, dat)
	
#	end dat value
dat = dat_data[-1, -1]
	
#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate stop time object
stop_time_object = get_times(*stop_date, ut_offset, dat)

###################################################################################################
#	generate rotation matrices
###################################################################################################

#	set time
rot_mat_start = time.time()
print('\n[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))

#	tt time
jdaytt_start = start_time_object.jday_tt
jfractt_start = start_time_object.jfrac_tt
jdtt_start = jdaytt_start + jfractt_start

#	ut1 time
jdayut1_start = start_time_object.jday_ut1
jfracut1_start = start_time_object.jfrac_ut1
jdut1_start = jdayut1_start + jfracut1_start

#	generate rotation matrices
rot_mat_log_level = 0
pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
	jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
)

###################################################################################################
#	display time
rot_mat_end = time.time()
rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)

print('\n	Rotation Matrices Generated.')
print('\n		- Time taken = {} s.'.format(rot_mat_time))

###################################################################################################
#	MODULE:	Horizons Positions
###################################################################################################

#	set time
horz_start = time.time()
print('\n[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))

#	set horizons logging level
horz_logging_level = 0

sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
	path_data, horz_logging_level,
	start_time_object, stop_time_object,
	len_record, time_step,
	eop_path
)

#	find indices of light curve data
timecol = sun_icrf[:, 0]
lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]

###################################################################################################
#	display time
horz_end = time.time()
horz_time = np.round((horz_end - horz_start), 1)

print('\n	Horizons data generated.')
print('\n		- Time taken = {} s.'.format(horz_time))

###################################################################################################
#	MODULE:	Object Position
###################################################################################################

#	set time
sat_start = time.time()
print('\n[{}]	Generating object position data...'.format(datetime.datetime.now()))

satpos_logging_level = 0

sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
	path_data, satpos_logging_level, len_record, time_step,
	start_time_object, stop_time_object,
	norad_id,
	eop_data, dat_data,
	pm_array, pn_array, st_array, c2t_array
)

###################################################################################################
#	display time
sat_end = time.time()
sat_time = np.round((sat_end - sat_start), 1)

print('\n	Object position data generated.')
print('\n		- Time taken = {} s.'.format(sat_time))

###################################################################################################
#	MODULE:	Transformation
###################################################################################################

#	set time
trans_start = time.time()
print('\n[{}]	Performing transformations...'.format(datetime.datetime.now()))

identification_str = 'mmt-render'

trans_logging_level = 0
save_flag = False

sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
	= module_transformation(
		path_data, trans_logging_level, save_flag,
		start_time_object, stop_time_object,
		identification_str, site_str,
		len_record, time_step,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array,
		sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
	)

###################################################################################################
#	display time
trans_end = time.time()
trans_time = np.round((trans_end - trans_start), 1)

print('\n	Transformations complete.')
print('\n		- Time taken = {} s.'.format(trans_time))

###################################################################################################
#	minimise sun angle
###################################################################################################

print('\nMinimising Solar Panel Sun Angle...')

#	convert to radians
rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad_temp = np.radians(rot_vec_inc_deg)

#	generate initial t = 0 attitude
init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])

#	generate initial attitude rotation matrix
init_attit_mat_temp = rot_mat(init_attit_temp)

#	rotate geometry axes into orbital frame
solar_panel_norm_orb = np.dot(solar_panel_norm_body, init_attit_mat_temp)

#	sun vector
sun_orb_vec_temp = sun_orb[0, 1:4]

attitude_angle_range = np.arange(0, 360, 1)
angle_data = np.zeros([len(attitude_angle_range)])

for i, rot_vec_azm_deg_temp in enumerate(attitude_angle_range):

	#	set new azimuth angle
	rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg_temp)
	
	#	generate new attitude state
	init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])

	#	generate new attitude matrix
	init_attit_mat_temp = rot_mat(init_attit_temp)

	#	generate new solar panel norm in orbital frame
	solar_panel_norm_orb_temp = np.dot(solar_panel_norm_body, init_attit_mat_temp)

	#	calculate angle to sun vector
	#	sun angle
	minimisation_angle = np.arccos(
		np.dot(solar_panel_norm_orb_temp, sun_orb_vec_temp)
		/ (np.linalg.norm(solar_panel_norm_orb_temp) *  np.linalg.norm(sun_orb_vec_temp))
	)

	angle_data[i] = minimisation_angle

#	find minimum angle
min_ang = angle_data.min()
min_index = np.where(angle_data==min_ang)[0][0]

azimuth_angle = attitude_angle_range[min_index]

#	set actual azimuth angle
rot_vec_azm_deg = azimuth_angle

print('\nSetting Azimuth Angle to: ', rot_vec_azm_deg)

###################################################################################################
#	prepare for render file generation
###################################################################################################

###############################################
#	clear render image directory

print('\nChecking for existing Image files...')
if nimg_files != 0:
	#	ask user to delete files
	check = input('	{:d} image files exist. Delete? [y/n]: '.format(nimg_files))

	if check == 'y':
		for f in render_img_dir_list:
			if f.endswith('.png'):
				os.remove(os.path.join(render_image_dir, f))
		print('	Files deleted')

	elif check == 'n':
		print('	Files retained,')
		check = input('		Use existing image files? [y/n]: ')
		if check == 'y':
			print('		Continuing with existing image files...')

		elif check == 'n':
			print('		Exiting.')
			sys.exit()
		else:
			print('	error in provided answer')
			sys.exit()
	else:
		print('	error in provided answer')
		sys.exit()
else:
	print('\n	Image directory is clean.')

###############################################
#	clear render geometry directory

print('\nChecking for existing Geom files...')
if ngeom_files != 0:

	if geom_auto_del is True:
		for f in render_geom_dir_list:
			if f.endswith('.stl'):
				os.remove(os.path.join(render_geom_dir, f))
		print('	Files deleted')

	else:

		#	ask user to delete files
		check = input('	{:d} geom files exist. Delete? [y/n]: '.format(ngeom_files))
	
		if check == 'y':
			for f in render_geom_dir_list:
				if f.endswith('.stl'):
					os.remove(os.path.join(render_geom_dir, f))
			print('	Files deleted')
	
		elif check == 'n':
			print('	Files retained,')
			check = input('		Use existing geometry files? [y/n]: ')
			if check == 'y':
				print('		Continuing with existing geometry files...')
	
			elif check == 'n':
				print('		Exiting.')
				sys.exit()
			else:
				print('	error in provided answer')
				sys.exit()
		else:
			print('	error in provided answer')
			sys.exit()
else:
	print('\n	Geometry directory is clean.')

###################################################################################################
#	define initial attitude state
###################################################################################################

##################################################
#	attitude

#	convert to radians
rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad = np.radians(rot_vec_inc_deg)

#	generate initial t = 0 attitude
init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])

##################################################
#	attitude motion in body frame

#	generate rotation angle in radians
if rot_period != 0.0:
	rot_ang_per_sec = (np.pi * 2) / rot_period
else:
	rot_ang_per_sec = 0.0

#	define rotation axis in body frame as unit vector
rot_vec_body_unit = np.array([1.0, 0.0, 0.0])

#	generate attitude matrix in body frame
attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)

#	extract euler angles
angles1 = np.array(mat2angles(attit_mat_body)[0:3])
angles2 = np.array(mat2angles(attit_mat_body)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

#	extract attitude motion in the body frame
attit_mot_body = angles1

##################################################
#	generate initial attitude rotation matrix
init_attit_mat = rot_mat(init_attit)

##################################################
#	calculate rotation vector in orbital frame as unit vector
rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)

##################################################
#	define velocity of rotation vector in orbital frame

#	when equal to orbital motion, rot vec is stationary in inertial frame
rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

###################################################################################################
#	perform light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

validation_plot_flag = False

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors_render2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb,
	body_mesh, render_geom_dir,
	validation_plot_flag
)

#	relist geom files 
render_geom_dir_list = os.listdir(render_geom_dir)
ngeom_files = len(render_geom_dir_list)

###################################################################################################
#	perform render
###################################################################################################

#	select vector data to use
vector_data = vector_data_orb

#	define unit vectors
x_vec = np.array([1.0, 0.0, 0.0])
y_vec = np.array([0.0, 1.0, 0.0])
z_vec = np.array([0.0, 0.0, 1.0])

###################################################################################################
#	import geometry data

geom_file_names = []

#	get list of geometry file names
for geom_file in render_geom_dir_list:
	geom_file_names.append(geom_file)

#	generate file ids
geom_file_list = []
for i, geom_file in enumerate(geom_file_names):

	temp = geom_file.split('_')[2]
	geom_id = int(temp.split('.')[0])
	geom_file_list.append([geom_id, geom_file])

#	sort files
geom_files_sorted = sorted(geom_file_list, key=lambda x: int(x[0]))

###################################################################################################
#	initilise figure

figure3d = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
axes3d = mplot3d.Axes3D(figure3d)

###############################################
#	set title, axis labels and axis limits

axes3d.set_title('Render Plot')

axes3d.set_xlim([-15, 15])
axes3d.set_ylim([-15, 15])
axes3d.set_zlim([-15, 15])

axes3d.set_xlabel('x orbital axis')
axes3d.set_ylabel('y orbital axis')
axes3d.set_zlabel('z orbital axis')

if render_auto_flag is False:
	if len(geom_files_sorted) == len(vector_data):
		input('\nRender from {:d} files?'.format(len(geom_files_sorted)))
	else:
		print('Error in data input.')
		print('len(geom_files) = ', len(geom_files_sorted))
		print('len(vector_data) = ', len(vector_data))
		sys.exit()
	
###################################################################################################
#	main render loop

cont_flag = True

print('\nPerforming Render...\n')

for i, (geom_line, vector_line) in enumerate(
		progressbar.progressbar(zip(geom_files_sorted, vector_data))
	):


#	if cont_flag is True:
	if i == 294:

		print('\n		Working on time step: ', i)
		# Clear graphics output
		if i != 0:
			axes3d.cla()
	
			###############################################
			#	reset title, axis labels and axis limits
			axes3d.set_title('Render Plot')
			axes3d.set_xlim([-15, 15])
			axes3d.set_ylim([-15, 15])
			axes3d.set_zlim([-15, 15])
			axes3d.set_xlabel('x orbital axis')
			axes3d.set_ylabel('y orbital axis')
			axes3d.set_zlabel('z orbital axis')
	
		###############################################################################################
		#	step initialisation
	
		###############################################
		#	parse vector render data
	
		mesh_file = geom_line[1]
	
		#	julian date
		jdate = vector_data_orb[i, 0]
	
		#	reference vectors
		x_ref_orb = vector_data_orb[i, 1:4]
		y_ref_orb = vector_data_orb[i, 4:7]
		z_ref_orb = vector_data_orb[i, 7:10]
	
		#	geometry vectors
		x_geom_orb = vector_data_orb[i, 10:13]
		y_geom_orb = vector_data_orb[i, 13:16]
		z_geom_orb = vector_data_orb[i, 16:19]
	
		#	earth, velocity and plane normal vectors
		earth_orb_vec = vector_data_orb[i, 19:22]
		sat_orb_vel = vector_data_orb[i, 22:25]
		orb_norm_orb_unit = vector_data_orb[i, 25:28]
	
		#	sun, site and rotation vector
		sun_orb_vec = vector_data_orb[i, 28:31]
		site_orb_vec = vector_data_orb[i, 31:34]
		rot_vec_orb_unit = vector_data_orb[i, 34:37]
	
		###############################################
		#	import mesh
		orb_mesh = mesh.Mesh.from_file(os.path.join(render_geom_dir, mesh_file))
		orb_vectors = orb_mesh.vectors
		orb_normals = orb_mesh.normals
	
		###############################################################################################
		#	generate plottable orbital frame axis vectors
	
		#	orbital frame reference axes scale
		orb_ref_vec_scale = 15
	
		#	text font size
		font_size = 14
	
		###############################################
		#	earth vector
		earth_vec_plot = orb_ref_vec_scale * np.divide(
			np.vstack((origin, earth_orb_vec)), np.linalg.norm(earth_orb_vec)
		)
		a = Arrow3D(earth_vec_plot[:, 0], earth_vec_plot[:, 1], earth_vec_plot[:, 2],
			mutation_scale=20, lw=2, arrowstyle="-|>", color="g"
		)
		axes3d.add_artist(a)
		axes3d.text(earth_vec_plot[1, 0], earth_vec_plot[1, 1], earth_vec_plot[1, 2],
			'Earth Vec', fontsize=font_size, fontweight='bold'
		)
		###############################################
		#	velocity vector
		vel_vec_plot = orb_ref_vec_scale * np.divide(
			np.vstack((origin, sat_orb_vel)), np.linalg.norm(sat_orb_vel)
		)
		a = Arrow3D(vel_vec_plot[:, 0], vel_vec_plot[:, 1], vel_vec_plot[:, 2],
			mutation_scale=20, lw=2, arrowstyle="-|>", color="g"
		)
		axes3d.add_artist(a)
		axes3d.text(vel_vec_plot[1, 0], vel_vec_plot[1, 1], vel_vec_plot[1, 2],
			'Velocity Vec', fontsize=font_size, fontweight='bold'
		)
		###############################################
		#	orbital plane normal
		orb_norm_orb_unit_plot = orb_ref_vec_scale * np.divide(
			np.vstack((origin, orb_norm_orb_unit)), np.linalg.norm(orb_norm_orb_unit)
		)
		a = Arrow3D(orb_norm_orb_unit_plot[:, 0], orb_norm_orb_unit_plot[:, 1], orb_norm_orb_unit_plot[:, 2],
			mutation_scale=20, lw=2, arrowstyle="-|>", color="g"
		)
		axes3d.add_artist(a)
		axes3d.text(orb_norm_orb_unit_plot[1, 0], orb_norm_orb_unit_plot[1, 1], orb_norm_orb_unit_plot[1, 2],
			'Orb Plane Normal', fontsize=font_size, fontweight='bold'
		)
		###############################################################################################
		#	generate plottable light curve generation vectors
	
		lc_gen_vec_scale = 10
	
		###############################################
		#	illumination vector
		illum_vec_plot = lc_gen_vec_scale * np.divide(
			np.vstack((origin, sun_orb_vec)), np.linalg.norm(sun_orb_vec)
		)
		a = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2],
			mutation_scale=20, lw=2, arrowstyle="-|>", color="r"
		)
		axes3d.add_artist(a)
		axes3d.text(illum_vec_plot[1, 0], illum_vec_plot[1, 1], illum_vec_plot[1, 2],
			'Illum Vec', fontsize=font_size, fontweight='bold'
		)
		###############################################
		#	observation vector
		obs_vec_plot = lc_gen_vec_scale * np.divide(
			np.vstack((origin, site_orb_vec)), np.linalg.norm(site_orb_vec)
		)
		a = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2],
			mutation_scale=20, lw=2, arrowstyle="-|>", color="b"
		)
		axes3d.add_artist(a)
		axes3d.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2],
			'Obs Vec', fontsize=font_size, fontweight='bold'
		)
		###############################################
		#	rotation vector
		rot_vec_plot = lc_gen_vec_scale * np.divide(
			np.vstack((origin, rot_vec_orb_unit)), np.linalg.norm(rot_vec_orb_unit)
		)
		a = Arrow3D(rot_vec_plot[:, 0], rot_vec_plot[:, 1], rot_vec_plot[:, 2],
			mutation_scale=20, lw=2, arrowstyle="-|>", color="m"
		)
		axes3d.add_artist(a)
		axes3d.text(rot_vec_plot[1, 0], rot_vec_plot[1, 1], rot_vec_plot[1, 2],
			'Rot Vec', fontsize=font_size, fontweight='bold'
		)
		###############################################################################################
		#	generate plottable body frame axis vectors
	
		body_axes_scale = 5
	
		###############################################
		#	add body x axis
		bodyx_vec_plot = body_axes_scale * np.vstack((origin, x_geom_orb))
		a = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
			mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
		)
		axes3d.add_artist(a)
		axes3d.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1], bodyx_vec_plot[1, 2],
			'x_b', fontsize=font_size, fontweight='bold'
		)
		###############################################
		#	add body y axis
		bodyy_vec_plot = body_axes_scale * np.vstack((origin, y_geom_orb))
		a = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
			mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
		)
		axes3d.add_artist(a)
		axes3d.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
			'y_b', fontsize=font_size, fontweight='bold'
		)
		###############################################
		#	add body z axis
		bodyz_vec_plot = body_axes_scale * np.vstack((origin, z_geom_orb))
		a = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
			mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
		)
		axes3d.add_artist(a)
		axes3d.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
			'z_b', fontsize=font_size, fontweight='bold'
		)
	
		###############################################################################################
		#	plot mesh
	
		#	set face alpha
		alpha = 0.5
	
		for face_index, orb_vector in enumerate(orb_vectors):
		
			#	Add final vert to vector array
			orb_vector = np.vstack([orb_vector, orb_vector[0, :]])
			#	extract x, y and z values
			xs = orb_vector[:,0]
			ys = orb_vector[:,1]
			zs = orb_vector[:,2]
			#	generate vertices list
			verts = [list(zip(xs, ys, zs))]
			#	add facet to plot
			pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
			pc.set_alpha(alpha)
			pc.set_edgecolor([0.5, 0.5, 0.5])
			pc.set_facecolor([0.5, 0.5, 0.5])
			axes3d.add_collection3d(pc)
	
	
	
	
	#	plt.show()
	#	sys.exit()
	
	
	
	
	
	
		###############################################################################################
		#	mesh properties calculation loop
		###############################################################################################
	
		#	initialise output arrays
		obs_faces = []
		illum_faces = []
		illum_obs_faces = []
		illum_or_obs_faces = []
		angles_data = np.zeros([len(orb_vectors), 2])
	
		for face_index, orb_vector in enumerate(orb_vectors):
		
			#	parse input vectors and calculate facet normal
			vect12 = orb_vector[1] - orb_vector[0]
			vect23 = orb_vector[2] - orb_vector[1]
			normal = np.cross(vect12, vect23)
			normal_unit = normal / np.linalg.norm(normal)
	
			#	sun angle
			sun_ang = np.arccos(
				np.dot(normal, sun_orb_vec)
				/ (np.linalg.norm(normal) *  np.linalg.norm(sun_orb_vec))
			)
	
			#	site angle
			site_ang = np.arccos(
				np.dot(normal, site_orb_vec)
				/ (np.linalg.norm(normal) * np.linalg.norm(site_orb_vec))
			)
	
			angles_data[face_index] = [sun_ang, site_ang]
	
			#	face is observable
			if site_ang < (0.5 * np.pi):
				obs_faces.append(face_index)
		
			#	face is illuminated
			if sun_ang < (0.5 * np.pi):
				illum_faces.append(face_index)
	
			#	face is observable and illuminated
			if site_ang < (0.5 * np.pi) and sun_ang < (0.5 * np.pi):
				illum_obs_faces.append(face_index)
	
			#	face is observable or illuminated
			if site_ang < (0.5 * np.pi) or sun_ang < (0.5 * np.pi):
				illum_or_obs_faces.append(face_index)
	
		print('\nobs_faces = ', obs_faces)
		print('\nillum_faces = ', illum_faces)
		print('\nillum_obs_faces = ', illum_obs_faces)
		print('\nillum_or_obs_faces = ', illum_or_obs_faces)
	
		###############################################################################################
		#	calculate observation vector projection angles to reference vector
		###############################################################################################
	
		#	reference vector = x_vec
		
		obs_xy_comp = np.array([site_orb_vec[0], site_orb_vec[1], 0.0])
	
		#	find azimuthal rotation angle
		obs_azm_ang = np.arccos(
			np.divide(
				np.dot(obs_xy_comp, x_vec),
				np.linalg.norm(obs_xy_comp) * np.linalg.norm(x_vec)
			)
		)
		
		if site_orb_vec[1] < 0:
			obs_azm_ang = -obs_azm_ang
		
		temp_attit = rot_mat(np.array([0.0, 0.0, obs_azm_ang]))
		temp_vec = np.dot(site_orb_vec, temp_attit)
		
		#	find inclination rotation angle
		obs_inc_ang = np.arccos(
			np.divide(
				np.dot(temp_vec, x_vec),
				np.linalg.norm(temp_vec) * np.linalg.norm(x_vec)
			)
		)
		
		#	set obs vector in x direction
		if site_orb_vec[2] > 0:
			obs_inc_ang = -obs_inc_ang
		
		#	attitude matrix
		obs_proj_attit = rot_mat(np.array([0.0, obs_inc_ang, obs_azm_ang]))
		obs_proj_vec = np.round(np.dot(site_orb_vec, obs_proj_attit), 10)
	
		###############################################################################################
		#	observation frame projection angles
	
		obs_proj_obs_vec = np.round(np.dot(site_orb_vec, obs_proj_attit), 10)
		obs_proj_illum_vec = np.round(np.dot(sun_orb_vec, obs_proj_attit), 10)
	
		#	apply obs vec rotation to data
	
		#	create copy of object mesh
		obs_proj_mesh = mesh.Mesh(orb_mesh.data.copy())
	
		#	apply rotation
		obs_proj_mesh.rotate_using_matrix(obs_proj_attit)
	
		obs_proj_vectors = obs_proj_mesh.vectors
		obs_proj_normals = obs_proj_mesh.normals
	
		#	apply inverse of obs vec position
		obs_proj_x_geom_orb = np.dot(x_geom_orb, obs_proj_attit)
		obs_proj_y_geom_orb = np.dot(y_geom_orb, obs_proj_attit)
		obs_proj_z_geom_orb = np.dot(z_geom_orb, obs_proj_attit)
	
		print('\n	obs_proj_vec = ', obs_proj_vec)
		print('\n	obs_angs = ', obs_inc_ang, obs_azm_ang)
	
		###############################################################################################
		#	calculate illumination vector projection angles to reference vector
		###############################################################################################
	
		#	reference vector = x_vec
		
		illum_xy_comp = np.array([sun_orb_vec[0], sun_orb_vec[1], 0.0])
	
		#	find azimuthal rotation angle
		illum_azm_ang = np.arccos(
			np.divide(
				np.dot(illum_xy_comp, x_vec),
				np.linalg.norm(illum_xy_comp) * np.linalg.norm(x_vec)
			)
		)
		
		if sun_orb_vec[1] < 0:
			illum_azm_ang = -illum_azm_ang
		
		temp_attit = rot_mat(np.array([0.0, 0.0, illum_azm_ang]))
		temp_vec = np.dot(sun_orb_vec, temp_attit)
		
		#	find inclination rotation angle
		illum_inc_ang = np.arccos(
			np.divide(
				np.dot(temp_vec, x_vec),
				np.linalg.norm(temp_vec) * np.linalg.norm(x_vec)
			)
		)
		
		#	set obs vector in x direction
		if sun_orb_vec[2] > 0:
			illum_inc_ang = -illum_inc_ang
		
		#	attitude matrix
		illum_proj_attit = rot_mat(np.array([0.0, illum_inc_ang, illum_azm_ang]))
		illum_proj_vec = np.round(np.dot(sun_orb_vec, illum_proj_attit), 10)
	
		###############################################################################################
		#	illumination frame projection angles
	
		illum_proj_obs_vec = np.round(np.dot(site_orb_vec, illum_proj_attit), 10)
		illum_proj_illum_vec = np.round(np.dot(sun_orb_vec, illum_proj_attit), 10)
	
		#	apply obs vec rotation to data
	
		#	create copy of object mesh
		illum_proj_mesh = mesh.Mesh(orb_mesh.data.copy())
	
		#	apply rotation
		illum_proj_mesh.rotate_using_matrix(illum_proj_attit)
	
		illum_proj_vectors = illum_proj_mesh.vectors
		illum_proj_normals = illum_proj_mesh.normals
	
	
		#	apply inverse of obs vec position
		illum_proj_x_geom_orb = np.dot(x_geom_orb, illum_proj_attit)
		illum_proj_y_geom_orb = np.dot(y_geom_orb, illum_proj_attit)
		illum_proj_z_geom_orb = np.dot(z_geom_orb, illum_proj_attit)
	
		print('\n	illum_proj_vec = ', illum_proj_vec)
		print('\n	illum_angs = ', illum_inc_ang, illum_azm_ang)
	
	#	###############################################
	#	#	set view direction
	#	view_azimuth = np.degrees(obs_azm_ang)
	#	view_elevation = np.degrees(-obs_inc_ang)
	#
	#	axes3d.view_init(view_elevation, view_azimuth)
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	# 
	# 	###############################################################################################
	# 	#	generate observation frame plots
	# 	###############################################################################################
	# 
	# 	axis_lims = 5
	# 	lc_gen_vec_scale = 10
	# 	orb_ref_vec_scale = 15
	# 	body_axes_scale = 5
	# 	
	# 	#	set mesh alpha
	# 	alpha = 0.5
	# 	
	# 	#	text font size
	# 	font_size = 14
	# 	
	# 	#	generate plottable vector
	# 	origin = np.array([0.0, 0.0, 0.0])
	# 	
	# 	###############################################
	# 	#	initilise 3d plot
	# 	
	# 	figure3d_obs = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
	# 	axes3d_obs = mplot3d.Axes3D(figure3d_obs)
	# 	
	# 	#	set title, axis labels and axis limits
	# 	axes3d_obs.set_title('Obs Projection 3D Plot')
	# 	axes3d_obs.set_xlim([-axis_lims, axis_lims])
	# 	axes3d_obs.set_ylim([-axis_lims, axis_lims])
	# 	axes3d_obs.set_zlim([-axis_lims, axis_lims])
	# 	axes3d_obs.set_xlabel('x orbital axis')
	# 	axes3d_obs.set_ylabel('y orbital axis')
	# 	axes3d_obs.set_zlabel('z orbital axis')
	# 	
	# 	###############################################################################################
	# 	#	generate and plot vector data
	# 	
	# 	###############################################
	# 	#	illumination vector
	# 	illum_vec_plot = lc_gen_vec_scale * np.divide(
	# 		np.vstack((origin, obs_proj_illum_vec)), np.linalg.norm(obs_proj_illum_vec)
	# 	)
	# 	a = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="r"
	# 	)
	# 	axes3d_obs.add_artist(a)
	# 	axes3d_obs.text(illum_vec_plot[1, 0], illum_vec_plot[1, 1], illum_vec_plot[1, 2],
	# 		'Illum Vec', fontsize=font_size, fontweight='bold'
	# 	)
	# 	###############################################
	# 	#	observation vector
	# 	obs_vec_plot = lc_gen_vec_scale * np.divide(
	# 		np.vstack((origin, obs_proj_obs_vec)), np.linalg.norm(obs_proj_obs_vec)
	# 	)
	# 	a = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="b"
	# 	)
	# 	axes3d_obs.add_artist(a)
	# 	axes3d_obs.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2],
	# 		'Obs Vec', fontsize=font_size, fontweight='bold'
	# 	)
	# 	###############################################
	# 	#	add body x axis
	# 	bodyx_vec_plot = body_axes_scale * np.vstack((origin, obs_proj_x_geom_orb))
	# 	a = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
	# 	)
	# 	axes3d_obs.add_artist(a)
	# 	axes3d_obs.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1], bodyx_vec_plot[1, 2],
	# 		'x_b', fontsize=font_size, fontweight='bold'
	# 	)
	# 	###############################################
	# 	#	add body y axis
	# 	bodyy_vec_plot = body_axes_scale * np.vstack((origin, obs_proj_y_geom_orb))
	# 	a = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
	# 	)
	# 	axes3d_obs.add_artist(a)
	# 	axes3d_obs.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	# 		'y_b', fontsize=font_size, fontweight='bold'
	# 	)
	# 	###############################################
	# 	#	add body z axis
	# 	bodyz_vec_plot = body_axes_scale * np.vstack((origin, obs_proj_z_geom_orb))
	# 	a = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
	# 	)
	# 	axes3d_obs.add_artist(a)
	# 	axes3d_obs.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	# 		'z_b', fontsize=font_size, fontweight='bold'
	# 	)
	# 
	# 	###############################################################################################
	# 	#	plot mesh
	# 
	# 	for face_index in illum_or_obs_faces:
	# #	for face_index, (vector_3d, angles_line) in enumerate(zip(obs_proj_vectors, angles_data)):
	# 
	# 		vector_3d = obs_proj_vectors[face_index, :]
	# 		angles_line = angles_data[face_index, :]
	# 
	# 		#	calculate normal unit vector
	# 		vect12 = vector_3d[1] - vector_3d[0]
	# 		vect23 = vector_3d[2] - vector_3d[1]
	# 		normal = np.cross(vect12, vect23)
	# 		normal_unit = 10 * (normal / np.linalg.norm(normal))
	# 	
	# 		origin_x = (vector_3d[0, 0] + vector_3d[1, 0] + vector_3d[2, 0]) / 3
	# 		origin_y = (vector_3d[0, 1] + vector_3d[1, 1] + vector_3d[2, 1]) / 3
	# 		origin_z = (vector_3d[0, 2] + vector_3d[1, 2] + vector_3d[2, 2]) / 3
	# 
	# 		normal_origin = np.array([origin_x, origin_y, origin_z])
	# 		normal_plot = np.vstack([normal_origin, normal_unit + normal_origin])
	# 	
	# 		axes3d_obs.plot(normal_plot[:, 0], normal_plot[:, 1], normal_plot[:, 2])
	# 
	# 		axes3d_obs.text(
	# 			*(normal_unit + normal_origin),
	# 			'Facet={}'.format(face_index)
	# 		)
	# 
	# 
	# 		#	parse angles line
	# 		sun_ang, site_ang = angles_line
	# 		
	# 		###############################################
	# 		#	generate plot data
	# 		#	Add final vert to vector array
	# 		vector_3d = np.vstack([vector_3d, vector_3d[0, :]])
	# 		#	extract x, y and z values
	# 		xs = vector_3d[:, 0]
	# 		ys = vector_3d[:, 1]
	# 		zs = vector_3d[:, 2]
	# 		#	generate vertices list
	# 		verts = [list(zip(xs, ys, zs))]
	# 
	# 		#	ensure face is observable
	# 		if site_ang < (0.5 * np.pi):
	# 			facecol='tab:green'
	# 		elif site_ang > (0.5 * np.pi):
	# 			facecol='tab:blue'
	# 
	# 		#	add facet to plot
	# 		pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	# 		pc.set_alpha(alpha)
	# 		pc.set_edgecolor([0.5, 0.5, 0.5])
	# 		pc.set_facecolor(facecol)
	# 		axes3d_obs.add_collection3d(pc)
	# 
	# 	#	finalise plot
	# 	axes3d_obs.view_init(np.degrees(0.0), np.degrees(0.0))
	# 
	# 	###############################################################################################
	# 	#	illumination projection plot
	# 	###############################################################################################
	# 
	# 	###############################################
	# 	#	initilise 3d plot
	# 	
	# 	figure3d_illum = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
	# 	axes3d_illum = mplot3d.Axes3D(figure3d_illum)
	# 	
	# 	#	set title, axis labels and axis limits
	# 	axes3d_illum.set_title('Illum Projection 3D Plot')
	# 	axes3d_illum.set_xlim([-axis_lims, axis_lims])
	# 	axes3d_illum.set_ylim([-axis_lims, axis_lims])
	# 	axes3d_illum.set_zlim([-axis_lims, axis_lims])
	# 	axes3d_illum.set_xlabel('x orbital axis')
	# 	axes3d_illum.set_ylabel('y orbital axis')
	# 	axes3d_illum.set_zlabel('z orbital axis')
	# 	
	# 	###############################################################################################
	# 	#	generate and plot vector data
	# 	
	# 	###############################################
	# 	#	illumination vector
	# 	illum_vec_plot = lc_gen_vec_scale * np.divide(
	# 		np.vstack((origin, illum_proj_illum_vec)), np.linalg.norm(illum_proj_illum_vec)
	# 	)
	# 	a = Arrow3D(illum_vec_plot[:, 0], illum_vec_plot[:, 1], illum_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="r"
	# 	)
	# 	axes3d_illum.add_artist(a)
	# 	axes3d_illum.text(illum_vec_plot[1, 0], illum_vec_plot[1, 1], illum_vec_plot[1, 2],
	# 		'Illum Vec', fontsize=font_size, fontweight='bold'
	# 	)
	# 	###############################################
	# 	#	observation vector
	# 	obs_vec_plot = lc_gen_vec_scale * np.divide(
	# 		np.vstack((origin, illum_proj_obs_vec)), np.linalg.norm(illum_proj_obs_vec)
	# 	)
	# 	a = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="b"
	# 	)
	# 	axes3d_illum.add_artist(a)
	# 	axes3d_illum.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2],
	# 		'Obs Vec', fontsize=font_size, fontweight='bold'
	# 	)
	# 	###############################################
	# 	#	add body x axis
	# 	bodyx_vec_plot = body_axes_scale * np.vstack((origin, illum_proj_x_geom_orb))
	# 	a = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
	# 	)
	# 	axes3d_illum.add_artist(a)
	# 	axes3d_illum.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1], bodyx_vec_plot[1, 2],
	# 		'x_b', fontsize=font_size, fontweight='bold'
	# 	)
	# 	###############################################
	# 	#	add body y axis
	# 	bodyy_vec_plot = body_axes_scale * np.vstack((origin, illum_proj_y_geom_orb))
	# 	a = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
	# 	)
	# 	axes3d_illum.add_artist(a)
	# 	axes3d_illum.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	# 		'y_b', fontsize=font_size, fontweight='bold'
	# 	)
	# 	###############################################
	# 	#	add body z axis
	# 	bodyz_vec_plot = body_axes_scale * np.vstack((origin, illum_proj_z_geom_orb))
	# 	a = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
	# 		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
	# 	)
	# 	axes3d_illum.add_artist(a)
	# 	axes3d_illum.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	# 		'z_b', fontsize=font_size, fontweight='bold'
	# 	)
	# 
	# 
	# 
	# 
	# 
	# 	###############################################################################################
	# 	#	illumination 3d plot mesh
	# 	###############################################################################################
	# 
	# 	for face_index in illum_or_obs_faces:
	# #	for face_index, (vector_3d, angles_line) in enumerate(zip(illum_proj_vectors, angles_data)):
	# 
	# 		vector_3d = illum_proj_vectors[face_index, :]
	# 		angles_line = angles_data[face_index, :]
	# 
	# 		#	calculate normal unit vector
	# 		vect12 = vector_3d[1] - vector_3d[0]
	# 		vect23 = vector_3d[2] - vector_3d[1]
	# 		normal = np.cross(vect12, vect23)
	# 		normal_unit = 10 * (normal / np.linalg.norm(normal))
	# 	
	# 		origin_x = (vector_3d[0, 0] + vector_3d[1, 0] + vector_3d[2, 0]) / 3
	# 		origin_y = (vector_3d[0, 1] + vector_3d[1, 1] + vector_3d[2, 1]) / 3
	# 		origin_z = (vector_3d[0, 2] + vector_3d[1, 2] + vector_3d[2, 2]) / 3
	# 
	# 		normal_origin = np.array([origin_x, origin_y, origin_z])
	# 		normal_plot = np.vstack([normal_origin, normal_unit + normal_origin])
	# 	
	# 		axes3d_illum.plot(normal_plot[:, 0], normal_plot[:, 1], normal_plot[:, 2])
	# 
	# 		axes3d_illum.text(
	# 			*(normal_unit + normal_origin),
	# 			'Facet={}'.format(face_index)
	# 		)
	# 
	# 		#	parse angles line
	# 		sun_ang, site_ang = angles_line
	# 	
	# 		###############################################
	# 		#	generate plot data
	# 		#	Add final vert to vector array
	# 		vector_3d = np.vstack([vector_3d, vector_3d[0, :]])
	# 		#	extract x, y and z values
	# 		xs = vector_3d[:, 0]
	# 		ys = vector_3d[:, 1]
	# 		zs = vector_3d[:, 2]
	# 		#	generate vertices list
	# 		verts = [list(zip(xs, ys, zs))]
	# 
	# 		#	ensure face is observable
	# 		if site_ang < (0.5 * np.pi):
	# 			facecol='tab:green'
	# 		elif site_ang > (0.5 * np.pi):
	# 			facecol='tab:blue'
	# 
	# 		#	add facet to plot
	# 		pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	# 		pc.set_alpha(alpha)
	# 		pc.set_edgecolor([0.5, 0.5, 0.5])
	# 		pc.set_facecolor(facecol)
	# 		axes3d_illum.add_collection3d(pc)
	# 
	# 	#	finalise plot
	# 	axes3d_illum.view_init(np.degrees(0.0), np.degrees(0.0))
	# 
	
	
	
	
	
	#	plt.show()
	#	sys.exit()
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
		###############################################################################################
		#	shadowing calculation loop
		###############################################################################################
	
		print('\n	#################################################################################')
		print('	Performing Shadowing Calculations...')
	
		###############################################################################################
		#	initilise obs proj figure
		figure2d_obs = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
		axes2d_obs = plt.axes()
		
		#	set title, axis labels and axis limits
		axes2d_obs.set_title('Obs Projection 2D Plot')
		axis_lims = 10
		axes2d_obs.set_xlim([-axis_lims, axis_lims])
		axes2d_obs.set_ylim([-axis_lims, axis_lims])
		axes2d_obs.set_xlabel('x orbital axis')
		axes2d_obs.set_ylabel('y orbital axis')
	
		###############################################################################################
		#	initilise illum proj figure
		figure2d_illum = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
		axes2d_illum = plt.axes()
		
		#	set title, axis labels and axis limits
		axes2d_illum.set_title('Illum Projection 2D Plot')
		axes2d_illum.set_xlim([-axis_lims, axis_lims])
		axes2d_illum.set_ylim([-axis_lims, axis_lims])
		axes2d_illum.set_xlabel('x orbital axis')
		axes2d_illum.set_ylabel('y orbital axis')
	
		obs_proj_vectors_2d = np.copy(obs_proj_vectors)
		illum_proj_vectors_2d = np.copy(illum_proj_vectors)
	
		###############################################################################################
		#	perform loop
	
		step_brightness_data = np.zeros([len(illum_or_obs_faces), 2])
	
		for face_index in illum_obs_faces:
	
			print('\n		Working on Illuminated Observable Facet: ', face_index)
	
			if face_index > obs_faces[0]:
	
				axes2d_obs.cla()
				#	set title, axis labels and axis limits
				axes2d_obs.set_title('Obs Projection 2D Plot')
				axes2d_obs.set_xlim([-axis_lims, axis_lims])
				axes2d_obs.set_ylim([-axis_lims, axis_lims])
				axes2d_obs.set_xlabel('x orbital axis')
				axes2d_obs.set_ylabel('y orbital axis')
	
				axes2d_illum.cla()
				#	set title, axis labels and axis limits
				axes2d_illum.set_title('Illum Projection 2D Plot')
				axes2d_illum.set_xlim([-axis_lims, axis_lims])
				axes2d_illum.set_ylim([-axis_lims, axis_lims])
				axes2d_illum.set_xlabel('x orbital axis')
				axes2d_illum.set_ylabel('y orbital axis')
	
			#	get vector data
			obs_proj_vector = obs_proj_vectors_2d[face_index, :]
			illum_proj_vector = illum_proj_vectors_2d[face_index, :]
		
			#	calculate max x-level
			obs_proj_x_level = obs_proj_vector[:, 0].max()
			illum_proj_x_level = illum_proj_vector[:, 0].max()
		
			#	generate polygons
			obs_proj_step_poly = sg.Polygon([
				[obs_proj_vector[0, 1], obs_proj_vector[0, 2]],
				[obs_proj_vector[1, 1], obs_proj_vector[1, 2]],
				[obs_proj_vector[2, 1], obs_proj_vector[2, 2]]
			])
	
			illum_proj_step_poly = sg.Polygon([
				[illum_proj_vector[0, 1], illum_proj_vector[0, 2]],
				[illum_proj_vector[1, 1], illum_proj_vector[1, 2]],
				[illum_proj_vector[2, 1], illum_proj_vector[2, 2]]
			])
	
			###########################################################################################
			#	shadowing facets
		
			print('\n			Finding Shadowing Facets...')
	
			obs_proj_shadowing_polygons = []
			illum_proj_shadowing_polygons = []
	
			obs_proj_back_polygons = []
			illum_proj_back_polygons = []
	
	#		if face_index == 12:
	#			input('\n\n\n\n\nStarting face index = 12 line:')
	#			print('\n\nGenerating combined array:')
	
			for temp_index in illum_or_obs_faces:
				if temp_index != face_index:
	
					if face_index == 13:
						print('\n###################################################################')
						print('shadowing facet: ', temp_index)
	
					obs_proj_vector_temp = obs_proj_vectors_2d[temp_index, :]
					illum_proj_vector_temp = illum_proj_vectors_2d[temp_index, :]
	
					angles_line = angles_data[temp_index, :]
					sun_ang_temp, site_ang_temp = angles_line
		
					###################################################################################
					#	observation projection
	
					#	generate polygon
					obs_proj_poly_temp = sg.Polygon([
						[obs_proj_vector_temp[0, 1], obs_proj_vector_temp[0, 2]],
						[obs_proj_vector_temp[1, 1], obs_proj_vector_temp[1, 2]],
						[obs_proj_vector_temp[2, 1], obs_proj_vector_temp[2, 2]]
					])
	
					#	calculate x level
					obs_proj_x_level_temp = obs_proj_vector_temp[:, 0].max()
	
					#	add to foreground or background polygon arrays
					if face_index == 12:
						print('\n', obs_proj_x_level_temp, obs_proj_x_level)
	
					if obs_proj_x_level_temp > obs_proj_x_level:
						if face_index == 13:
							print('\n	Greater:')
	
						#	combine
						before_test = cascaded_union(obs_proj_shadowing_polygons)
	
						obs_proj_shadowing_polygons.append(obs_proj_poly_temp)
	
						#	combine
						after_test = cascaded_union(obs_proj_shadowing_polygons)
	
						if face_index == 13:
							print('\n	old combined forground poly: \n', before_test)
							print('\n	appending: \n', obs_proj_poly_temp)
							print('\n	new combined forground poly: \n', after_test)
	
					elif obs_proj_x_level_temp <= obs_proj_x_level:
						if face_index == 13:
							print('\n	Less or Equal:')
						obs_proj_back_polygons.append(obs_proj_poly_temp)
		
					else:
						print('ERROR')
						sys.exit()
	
					###################################################################################
					#	illumination projection
	
					#	generate polygon
					illum_proj_poly_temp = sg.Polygon([
						[illum_proj_vector_temp[0, 1], illum_proj_vector_temp[0, 2]],
						[illum_proj_vector_temp[1, 1], illum_proj_vector_temp[1, 2]],
						[illum_proj_vector_temp[2, 1], illum_proj_vector_temp[2, 2]]
					])
	
					#	calculate x level
					illum_proj_x_level_temp = illum_proj_vector_temp[:, 0].max()
	
					#	add to foreground or background polygon arrays
					if illum_proj_x_level_temp > illum_proj_x_level:
						illum_proj_shadowing_polygons.append(illum_proj_poly_temp)
	
					elif illum_proj_x_level_temp <= illum_proj_x_level:
						illum_proj_back_polygons.append(illum_proj_poly_temp)
	
					else:
						print('ERROR')
						sys.exit()
	
			#######################################################################################
			#	plot background polygons
		
			#	observation projection
			if len(obs_proj_back_polygons) == 0:
	#			print('\n				Observation Projection: No background facets.')
				pass
		
			else:
	#			print('\n				Observation Projection: Plotting background facets.')
		
				obs_proj_cascaded_back_polygons = cascaded_union(obs_proj_back_polygons)
		
				obs_proj_cascaded_back_patch = PolygonPatch(
					obs_proj_cascaded_back_polygons, facecolor='tab:green', alpha=1.0, zorder=0
				)
			
				axes2d_obs.add_patch(obs_proj_cascaded_back_patch)
	
			#	illumination projection
			if len(illum_proj_back_polygons) == 0:
	#			print('\n				Illumination Projection: No background facets.')
				pass
		
			else:
	#			print('\n				Illumination Projection: Plotting background facets.')
				illum_proj_cascaded_back_polygons = cascaded_union(illum_proj_back_polygons)
				illum_proj_cascaded_back_patch = PolygonPatch(
					illum_proj_cascaded_back_polygons, facecolor='tab:green', alpha=1.0, zorder=0
				)
				axes2d_illum.add_patch(illum_proj_cascaded_back_patch)
	
			#######################################################################################
			#	plot faces of interest
	
			#	add to figure
			face_ofint_patch_obs = PolygonPatch(
				obs_proj_step_poly, facecolor='tab:blue', alpha=1.0, zorder=1
			)
	
			axes2d_obs.add_patch(face_ofint_patch_obs)
			axes2d_obs.text(obs_proj_vector[2, 1], obs_proj_vector[2, 2], 'Facet-{}'.format(face_index))
	
			#	add to figure
			face_ofint_patch_illum = PolygonPatch(
				illum_proj_step_poly, facecolor='tab:blue', alpha=1.0, zorder=1
			)
	
			axes2d_illum.add_patch(face_ofint_patch_illum)
			axes2d_illum.text(
				illum_proj_vector[2, 1], illum_proj_vector[2, 2],
				'Facet-{}'.format(face_index)
			)
	
			#######################################################################################
			#	plot foreground polygons
	
			#	observation projection 
			if len(obs_proj_shadowing_polygons) == 0:
	#			print('\n				Observation Projection: No foreground facets.')
				obs_visible_perc = 100.0
		
			else:
	
				#	combine
				obs_proj_cascaded_shadowing_polygons = cascaded_union(obs_proj_shadowing_polygons)
	
				if face_index == 13:
					print(
						'\n					Number of foreground facets: ',
						len(obs_proj_shadowing_polygons)
					)
	
					print('\n\n\n\n\n\n\n\n\n\n\n\n\n\n')
					print('\n', obs_proj_cascaded_shadowing_polygons)
					print('\n', obs_proj_shadowing_polygons)


				#	find intersecting area
				obs_proj_intersection = obs_proj_step_poly.intersection(
					obs_proj_cascaded_shadowing_polygons
				)
	
				if face_index == 13:
					print('\nSUCCESS')
					print('\nIntersection = ', obs_proj_intersection)
					sys.exit()

				obs_visible_perc = 100 * (
					1.0 - (obs_proj_intersection.area / obs_proj_step_poly.area)
				)
	
				obs_proj_cascaded_shadowing_patch = PolygonPatch(
					obs_proj_cascaded_shadowing_polygons, facecolor='tab:red', alpha=1.0, zorder=2
				)
				axes2d_obs.add_patch(obs_proj_cascaded_shadowing_patch)
	
	
			#	illumination projection 
			if len(illum_proj_shadowing_polygons) == 0:
	#			print('\n				Illumination Projection: No foreground facets.')
				illum_visible_perc = 100.0
			
			else:
	#			print('\n				Illumination Projection: Plotting foreground facets.')
		
				#	combine
				illum_proj_cascaded_shadowing_polygons = cascaded_union(illum_proj_shadowing_polygons)
				illum_proj_cascaded_shadowing_patch = PolygonPatch(
					illum_proj_cascaded_shadowing_polygons, facecolor='tab:red', alpha=1.0, zorder=2
				)
				axes2d_illum.add_patch(illum_proj_cascaded_shadowing_patch)
	
				#	find intersecting area
				illum_proj_intersection = illum_proj_step_poly.intersection(
					illum_proj_cascaded_shadowing_polygons
				)
	
				illum_visible_perc = 100 * (
					1.0 - (illum_proj_intersection.area / illum_proj_step_poly.area)
				)
	
			print('\n			Visibility Percentage = ', obs_visible_perc)
			print('			Illumination Percentage = ', illum_visible_perc)

#			if face_index == 13:
#				print('\nSUCCESS')
#				sys.exit()




		print('\nFINISHED TIME STEP')
		sys.exit()
	
		print('Finished.')
		plt.show()
		sys.exit()
	
		###############################################################################################
		#	save output image
		###############################################################################################
	
		#	save single figure
		figure3d.savefig(os.path.join(render_image_dir, image_name))

print('\nRender complete.')

#############################################################################################
#	delete geom files
for f in render_geom_dir_list:
	if f.endswith('.stl'):
		os.remove(os.path.join(render_geom_dir, f))

###############################################################################################
#	calculate run time
###############################################################################################
#	calculate sim time
sim_end = time.time()
sim_time = np.round((sim_end - sim_start) / 60, 1)

print('\n[{}] Simulation Finished.'.format(datetime.datetime.now()))
print('\n	- Time taken = {} mins.'.format(sim_time))
plt.show()
sys.exit()
