###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import BoundaryNorm
from matplotlib import gridspec

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Starlink File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()
init_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'rocket_body')

###################################################################################################
#	simulation inputs

#	set time step
time_step = 0.1

#	set time windows
window_start_date = [2010, 1, 1, 0, 0, 0]
window_stop_date = [2019, 11, 1, 0, 0, 0]

#	define specific mmt file
specific_file_flag = True
specific_file = 'CZ-2D_41858.txt'

#	define track id
specific_track_flag = True 
specific_track = 12047965

#	simulation flags
plot_all_minima_flag = False

#	rmse normalisation flag
rms_norm_flag = True
rms_norm_flag = False

savefile_flag = False

###################################################################################################
#	fitting inputs

#	geometry zone specification string
brdf_str = 'full'

#	set range of d_brdf values to plot
d_brdf_range = [0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8]

#	range for brdf parameters
m_brdf_min = 0.1
m_brdf_max = 0.9
m_brdf_step = 0.1
m_brdf_range = np.arange(m_brdf_min, m_brdf_max + m_brdf_step, m_brdf_step)

omega_brdf_min = 0.1
omega_brdf_max = 0.9
omega_brdf_step = 0.1
omega_brdf_range = np.arange(omega_brdf_min, omega_brdf_max + omega_brdf_step, omega_brdf_step)

#	range of attiude angles
rot_vec_azm_deg_range = np.arange(0, 360, 10)
rot_vec_inc_deg_range = np.arange(0, 100, 10)

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###############################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

###################################################################################################
#	figure generation paramters

#	inch to cm conversion
inch2cm = 2.54

#	set colour levels
n_cols = 100

###################################################################################################
#	loop through MMT files
###################################################################################################
print('\n	Looping Through Rocket Body Light Curve Files...')

#	set input directory
rocket_body_dir = os.listdir(read_path)

if specific_file_flag is True:
	rocket_body_dir = [specific_file]

#	loop through starlink IDs
for loop_itt, mmt_file in enumerate(rocket_body_dir):

	#	parse filename
	name_str = mmt_file.split('_')[0]
	norad_id = int(mmt_file.split('-')[1].split('_')[1][:-4])

	print('\n		#####################################################################')
	print('		Plotting data for: {}, {}'.format(name_str, norad_id))

	###############################################################################################
	#	load MMT file
	
	#	set time
	mmt_load_start = time.time()
	
	print('\n	[{}] Loading MMT data...'.format(datetime.datetime.now()))
	
	with open(os.path.join(read_path, mmt_file)) as read_file:
		mmt_data = read_file.readlines()

	###############################################################################################
	#	parse MMT file
	
	#	strip header
	header_text = mmt_data[7]
	raw_data = mmt_data[8:]
	
	print('\n		Parsing MMT data...')
	
	#	initialise output
	n_rows = 14
	parsed_data = np.zeros([len(raw_data), n_rows])
	
	for i, line in enumerate(raw_data):
	
		#	split line
		date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
			distance, phase, channel, track_id = line.split(' ')
	
		#	parse data and set data types
		mag_std = float(mag_std)
		mag_app = float(mag_app)
		distance = float(distance)
		phase = float(phase)
		track_id = int(track_id)
	
		#	parse date
		year, month, day = date_str.split('-')
		year = int(year)
		month = int(month)
		day = int(day)
	
		#	parse time
		hour, minute, second = time_str.split(':')
		hour = int(hour)
		minute = int(minute)
		second = np.round(float(second), 1)
	
		#	calculate julian date
		jday, jfrac = date2jd(year, month, day, hour, minute, second)
	
		#	generate output line
		output_line = [
			track_id, norad_id,
			jday, jfrac,
			year, month, day,
			hour, minute, second,
			mag_std, mag_app,
			distance, phase
		]
	
		#	add to output array
		parsed_data[i, :] = output_line
	
	print('			Total data length = ', len(parsed_data))
	print('			Total number of objects = ', len(np.unique(parsed_data[:, 1])))
	print('			Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

	###############################################################################################
	#	extracting data between specified windows
	
	print('\n		Seaching for tracks inside time window:')
	
	#	start window
	window_start_jd = np.add(*date2jd(*window_start_date))
	print('\n			Window start 	= ', window_start_date[0:3])
	
	#	stop window
	window_stop_jd = np.add(*date2jd(*window_stop_date))
	print('			Window stop 	= ', window_stop_date[0:3])
	
	#	extract track ids
	track_id_list = np.unique(parsed_data[:, 0])

	#	loop through tracks
	ntracks = 0
	first_lc_flag = True
	for track_id in track_id_list:

		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]
	
		#	extract light curve data
		jd_plot = np.add(track_data[:, 2], track_data[:, 3])
		std_mag_plot = track_data[:, 10]
		app_mag_plot = track_data[:, 11]
		distance_plot = track_data[:, 12]
		phase_plot = track_data[:, 13]
		start_line = track_data[0]
		stop_line = track_data[-1]
		start_jdate = np.add(start_line[2], start_line[3])
		stop_jdate = np.add(stop_line[2], stop_line[3])
	
		#	ensure between window start/stop
		if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
	
			#	light curve is within window
			ntracks += 1
	
			#	if first light curve, start array
			if first_lc_flag is True:
				windowed_data = track_data
				first_lc_flag = False
	
			#	else append to list
			elif first_lc_flag is False:
				windowed_data = np.vstack([windowed_data, track_data])
	
	print('\n			{} track(s) extracted.'.format(ntracks))

	#	reset track_id_list for windowed data
	track_id_list = np.unique(windowed_data[:, 0])

	###############################################################################################
	#	display initialisation time
	mmt_load_end = time.time()
	mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)
	
	print('\n	MMT Load Finished.')
	print('\n		- Time taken = {} s.'.format(mmt_load_time))

	###############################################################################################
	#	main track loop
	###############################################################################################


	sys.exit()

	#	loop initialisation
	first_track = True
	color_index = 0
	tracks_plotted = 0

	if specific_track_flag is True: 
		track_id_list = [specific_track]

	#	loop through tracks
	for track_itt, track_id in enumerate(track_id_list):

		#	convert to int
		track_id = int(track_id)
	
		print('\n			#############################################################')
		print('			Working on Track {}, ID: {}'.format(track_itt, track_id))

		###########################################################################################
		#	input directory

		#	create object directory
		object_dir = name_str + '_{}'.format(norad_id)
		input_path = os.path.join(path_data, 'lightcurve_dir', 'rocket_body_analysis', object_dir)

		#	create track directory
		track_dir = 'track-{}'.format(track_id)
		input_path = os.path.join(input_path, track_dir)

		###########################################################################################
		#	extracting requested single track
		
		print('\n		Extracting requested track index...')

		#	extract track data
		track_indices = np.where(windowed_data[:, 0]==track_id)[0]
		track_data = windowed_data[track_indices]
		
		###########################################################################################
		#	extract track time data
	
		mmt_line = track_data[0]
		
		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]

		###########################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
		sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)
		
		#######################################################################################
		#	ensure time continuity
		
		print('\n				Ensuring time continuity in extracted data...')
		
		track_data_temp = []
		sim_index = 0
	
		for itt, line in enumerate(track_data):
		
			#	parse line data
			line_track_id, line_norad_id,\
				jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = line
			
			step_jd = jday + jfrac
		
			#	calculate current seconds
			current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
		
			output_line = [
				jday, jfrac, current_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]
		
			##################################################
			#	extracting data of time step
		
			#	data match found
			if current_sec == sim_range_sec[sim_index]:
				track_data_temp.append(output_line)
		
				#	itterate number of output data points
				sim_index += 1
		
				#	save current line as previous line
				prev_line = output_line
		
			#	data point is not required
			elif current_sec < sim_range_sec[sim_index]:
				pass
		
			#	interpolation required
			elif current_sec > sim_range_sec[sim_index]:
		
				#	parse previous line
				jday_prev, jfrac_prev, prev_sec, \
					year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
					mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
		
				#	calculate interpolation requirements
				gap = np.round(current_sec - prev_sec, 1)
				npts = int(gap / time_step)
		
				#	get current light curve data
				mag_std_last = mag_std
				mag_app_last = mag_app
				distance_last = distance
				phase_last = phase
		
				#	calculate linear interpolation step values
				mag_std_step = (mag_std_last - mag_std_prev) / npts
				mag_app_step = (mag_app_last - mag_app_prev) / npts
				distance_step = (distance_last - distance_prev) / npts
				phase_step = (phase_last - phase_prev) / npts
		
				#	calculate interpolation data
				for n in range(1, npts + 1):
		
					#	current simulation second
					interp_sec = np.round(prev_sec + (n * time_step), 1)

					#	time data
					jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
					jday, jfrac = jday_jfrac(jdate)
					year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
					#	calculate interpolated light curve data
					mag_std = mag_std_prev + (n * mag_std_step)
					mag_app = mag_app_prev + (n * mag_app_step)
					distance = distance_prev + (n * distance_step)
					phase = phase_prev + (n * phase_step)
		
					#	generate new output line
					interp_output_line = [
						jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
		
					if interp_sec == sim_range_sec[sim_index]:
						track_data_temp.append(interp_output_line)
						sim_index += 1
		
				if interp_sec != current_sec:
					#	append final line
					track_data_temp.append(output_line)
					sim_index += 1
		
			else:
				print('ERROR')
				sys.exit()
		
			#	save previous line
			if itt != 0:
				prev_line = output_line
		
			#	array is complete condition
			if len(track_data_temp) == lc_len_record:
				break
		
		#	ensure parsed MMT data matches simulation data
		track_data = np.array(track_data_temp)
		
		if len(track_data) == len(lc_sim_range):
			print('					Simulation and MMT data matches.')
		else:
			print('					Error: Simulation and MMT data does not match...Exiting.')
			sys.exit()
		
		###########################################################################################
		#	extract mmt data

		jd_plot = np.add(track_data[:, 0], track_data[:, 1])
		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]

		###########################################################################################
		#	simulation printout
		###########################################################################################
		
		print('\n		#####################################################################')
		print('		Extracted MMT track parameters: ')
		print(
			'			Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
			.format(*lc_start_date)
		)
		print(
			'			Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
			.format(*lc_stop_date)
		)
		print('			Simulation time step:		{}'.format(time_step))
		print('			Duration:			{}		[steps]'.format(lc_len_record))
		print('							{}		[s]'.format(np.round(lc_duration_sec, 1)))

		###########################################################################################
		#	setting simulation time data
		###########################################################################################
		
		sim_range = lc_sim_range
		len_record = lc_len_record
		start_date = lc_start_date
		stop_date = lc_stop_date
		duration_sec = lc_duration_sec

		#	calculate sim time range in seconds
		sim_time = np.linspace(0, duration_sec, len_record)
		
		###########################################################################################
		#	extract plot data
		###########################################################################################

		print('\n	#############################################################################')
		print('	Beginning Extraction Loop...')

		for rot_vec_inc_deg in rot_vec_inc_deg_range:
			#	looping through attitude azm
			for rot_vec_azm_deg in rot_vec_azm_deg_range:

				print('\n			Running on Attitude Angles:')
				print('\n				Inclination = 	', rot_vec_inc_deg)
				print('				Azimuth = 	', rot_vec_azm_deg)

				#	generate output filename
				input_file = '{}-{}_track-{}_inc-{}_azm-{}.txt'.format(
					name_str,
					int(norad_id),
					track_id,
					rot_vec_inc_deg,
					rot_vec_azm_deg
				)

				#	generate the rms values output file name
				rms_values_file = input_file[:-4] + '.rms_values'
	

				print('\n			Looking for: {}'.format(os.path.join(input_path, rms_values_file)))
			
				if os.path.isfile(os.path.join(input_path, rms_values_file)):
			
					print('\n				Found: Reading .rms_values file...')
					file_rms_values = np.loadtxt(
						os.path.join(input_path, rms_values_file),
						delimiter=','
					)
			
				else:
		
					print('\n			Not Found: Generating .rms_values file...')
				#	track_rms_values = np.zeros(attit_azm_mesh.shape)
		
					#	total data array
					print('\n				Loading Light Curve Data File...\n')
					total_data = np.loadtxt(
						os.path.join(input_path, input_file), delimiter=','
					)
		
					#	extract light curve indices
					light_curve_indices = np.unique(total_data[:, 0])
					
					for light_curve_index in light_curve_indices:

						if light_curve_index % 10 == 0.0:
							print(
								'					Working on Light Curve ID: {}'.format(
									int(light_curve_index)
									),
								end='\r'
							)

						#	light curve data of index
						data_indices = np.where(total_data[:, 0]==light_curve_index)[0]
						light_curve_data = total_data[data_indices]
				
						#	parse data from light curve array
						light_curve_seconds = light_curve_data[:, 2]
						synth_signal_app = light_curve_data[:, 3]
						synth_signal_std = light_curve_data[:, 4]

						lc_index, jdate, sec_since, mag_app, mag_std, \
							object_id, rot_vec_azm_deg, rot_vec_inc_deg, \
							top_d_brdf, top_m_brdf, top_omega_brdf, \
							bot_d_brdf, bot_m_brdf, bot_omega_brdf, \
							side_d_brdf, side_m_brdf, side_omega_brdf, \
							= light_curve_data[0]

						#	calculate RMSE
						rms_std = rms_calc(synth_signal_std, mmt_signal_std)
						rms_app = rms_calc(synth_signal_app, mmt_signal_app)


						print('\nRMSE = ', rms_app)
						sys.exit()



						#	find occurance in omega/m grids
						inc_index = np.where(np.round(attit_inc_mesh[0, :], 2)==rot_vec_inc_deg)[0][0]
						azm_index = np.where(np.round(attit_azm_mesh[:, 0], 2)==rot_vec_azm_deg)[0][0]


						track_rms_values[azm_index, inc_index] = rms_app

					print('\n		Finished.')
		
					###################################################################################
					#	save rms_values array to avoid regeneration
				
					np.savetxt(
						os.path.join(lightcurve_path, in_dir, rms_values_file),
						track_rms_values,
						delimiter=','
					)
				
					print('\n	RMS Values file saved: ', rms_values_file)

		#	if normalisation flag is true, normalise
		if rms_norm_flag is True:
			print('\n				Normalising RMS...')
			track_rms_values = np.divide(track_rms_values, track_rms_values.max())

	###################################################################################################
	#	Generating Colormap Plots
	###################################################################################################
	
	print('\n#####################################################################################')
	print('Generating Colormap Plots...')
	print('#####################################################################################')
	
	###################################################################################################
	#	figure generation inputs
	
	#	set graphtype (cont/mesh)
	graphtype = 'cont'
	graphtype = 'mesh'
	
	#	set colour map
	cmap = plt.get_cmap('PiYG')
	
	#	set rms normalisation flag
	norm_flag = True
	
	#	colour levels
	levels_data = track_rms_values
	
	levels_min = levels_data.min()
	levels_max = levels_data.max()
	
	#	set color levels
	levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
	norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

	###################################################################################################
	#	figure generation paramters
	
	#	set colour levels
	n_cols = 100
	
	#	set subplot spacing values
	hspace_val = 0.35
	wspace_val = 0.35
	
	#	set font sizes
	title_fontsize = 6
	text_fontsize = 4
	cbar_fontsize = 4
	legend_fontsize = 4
	xlabel_fontsize = 6
	ylabel_fontsize = 6
	xticks_fontsize = 4
	yticks_fontsize = 4
	
	#	set text padding values
	title_pad = 2
	xticks_pad = 4
	yticks_pad = 0
	
	#	set marker parameters
	marker_size = 3
	
	#	set figure height/width
	fig_width = 6.0/inch2cm
	fig_height = 6.0/inch2cm

	fig = plt.figure()

	#	subplots figure
	ax = plt.subplot(projection='polar')
	
	ax.set_theta_zero_location("N")
	ax.set_theta_direction(-1)
	
	ax.set_rticks(np.arange(0, 100, 10))
	ax.set_rlim(0, 100)

	####################################################
	#	find azimuth offset to align with grid
	nazm_vals = len(attit_azm_mesh) - 1
	offset = (360 / nazm_vals) / 2

	###############################################################################################
	#	plotting
	
	#	round to reasonable decimal places
	attit_data = np.round(track_rms_values, 10)
	min_rms = attit_data.min()

	azm_step = rot_vec_azm_deg_range[1] - rot_vec_azm_deg_range[0]

	#	add addditional rows to plot data to complete circle
	attit_inc_mesh = np.vstack([attit_inc_mesh, attit_inc_mesh[-1, :]])
	attit_azm_mesh = np.vstack([attit_azm_mesh, np.add(attit_azm_mesh[-1, :], azm_step)])
	attit_data = np.vstack([attit_data, np.full(attit_data[-1, :].shape, 0)])
	
	#	add addditional cols to plot data to complete circle
	attit_inc_mesh = np.hstack([
		attit_inc_mesh,
		np.reshape(
			attit_inc_mesh[:, -1] + 10,
			[len(attit_inc_mesh[:, -1]), 1]
		)
	])
	
	attit_azm_mesh = np.hstack([
		attit_azm_mesh,
		np.reshape(
			attit_azm_mesh[:, -1],
			[len(attit_azm_mesh[:, -1]), 1]
		)
	])
	
	attit_data = np.hstack([
		attit_data,
		np.reshape(
			np.full(attit_data[:, -1].shape, 0),
			[len(attit_data[:, -1]), 1]
		)
	])

	print('\n			Plotting Color Data...')

	#	plot mesh
	plt.pcolor(
		np.radians(attit_azm_mesh - offset),
		attit_inc_mesh,
		attit_data,
		cmap=cmap,
		norm=norm
	)
	
	plt.colorbar()

	###########################################################################################
	#	finalise plot
	
	#	add title
	ax.set_title('Attitude Fitting', fontsize=title_fontsize, pad=title_pad)
	
	#	format x axis
	ax.set_xlabel('Azm', fontsize=xlabel_fontsize)
	ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
	
	#	format y axis
	ax.set_ylabel('Inc', fontsize=ylabel_fontsize)
	ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
	
	###############################################################################################
	#	save figures
	
	if savefile_flag is True:
	
		fig3_name = 'starlink_brdf_determination_plot.eps'
		out_path3 = os.path.join(fig_path, fig3_name)
		fig3.savefig(out_path3, bbox_inches="tight", dpi=300)
	
	#	fig_name = 'starlink_brdf_determination_stack.eps'
	#	out_path = os.path.join(fig_path, fig_name)
	#	fig.savefig(out_path, bbox_inches="tight", dpi=300)
	
		print('\n#######################################')



	print('Minimum RMS value = ', min_rms)

	min_rms_loc = np.where(attit_data==min_rms)

	print('\nMinimum RMS position = 	', min_rms_loc)
	print('\nMinimum RMS position = 	', attit_azm_mesh[min_rms_loc])
	print('Minimum RMS Inc = 	', attit_inc_mesh[min_rms_loc])





	plt.show()
	sys.exit()
