###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times, detect_local_minima

from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import BoundaryNorm
from matplotlib import gridspec

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Starlink File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()
init_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'rocket_body')

###################################################################################################
#	simulation inputs

#	set time step
time_step = 0.1

#	set time windows
window_start_date = [2010, 1, 1, 0, 0, 0]
window_stop_date = [2019, 11, 1, 0, 0, 0]

#	define specific mmt file
specific_file_flag = True
specific_file = 'CZ-2D_41858.txt'

#	define track id
specific_track_flag = True 
specific_track = 12047965
specific_track = 10833133

#	simulation flags
plot_all_minima_flag = False

#	rmse normalisation flag
rms_norm_flag = True
rms_norm_flag = False

#	set solution space constraints
#	m_brdf range
min_m_brdf = 0.1
max_m_brdf = 0.9

#	omega_brdf range
min_omega_brdf = 0.1
max_omega_brdf = 0.9

#	brdf determination plots output flag
brdf_det_flag = True
brdf_det_flag = False

#	flag used to show brdf colour map plots
brdf_map_flag = True
brdf_map_flag = False

#	flag used to regenerate and overwrite existing RMS data
rms_regen_flag = False

savefile_flag = False

###################################################################################################
#	fitting inputs

#	geometry zone specification string
brdf_str = 'full'

#	set range of d_brdf values to plot
d_brdf_range = [0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8]
d_brdf_range = [0.2, 0.3, 0.4, 0.5, 0.6, 0.7]

#	range for brdf parameters
m_brdf_min = 0.1
m_brdf_max = 0.9
m_brdf_step = 0.1
m_brdf_range = np.arange(m_brdf_min, m_brdf_max + m_brdf_step, m_brdf_step)

omega_brdf_min = 0.1
omega_brdf_max = 0.9
omega_brdf_step = 0.1
omega_brdf_range = np.arange(omega_brdf_min, omega_brdf_max + omega_brdf_step, omega_brdf_step)

#	range of attiude angles
rot_vec_azm_deg_range = np.arange(0, 360, 10)
rot_vec_inc_deg_range = np.arange(0, 100, 10)


# #	range of attiude angles
# rot_vec_azm_deg_range = [0, 10, 20, 320, 330, 340, 350]
# rot_vec_inc_deg_range = [30, 40, 50, 60, 70, 80]

# rot_vec_azm_deg_range = [190]
# rot_vec_inc_deg_range = [60]

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###############################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

###################################################################################################
#	figure generation paramters

#	inch to cm conversion
inch2cm = 2.54

#	set colour levels
n_cols = 100

#	set subplot spacing values
hspace_val = 0.35
wspace_val = 0.35

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4
xlabel_fontsize = 6
ylabel_fontsize = 6
xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set marker parameters
marker_size = 3

#	set figure line wdith
plot_linewidth = 1.0
 
#	set use of LaTeX font
plt.rc('text', usetex=True)

if brdf_det_flag is True:

	#	set figure height/width
	fig_width = 10.0/inch2cm
	fig_height = 4.0/inch2cm
	
	#	initialise figure axes
	fig2, ax2 = plt.subplots(1, 2, figsize=(fig_width, fig_height), dpi=300)
	
	#	adjust subplot spacing
	plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)
	plt.gcf().subplots_adjust(bottom=0.2)
	
#	#	initialise figure axes
#	fig3, ax3 = plt.subplots(1, 2, figsize=(fig_width, fig_height),	dpi=300)
#	
#	#	adjust subplot spacing
#	plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)
#	plt.gcf().subplots_adjust(bottom=0.2)

###################################################################################################
#	loop through MMT files
###################################################################################################
print('\n	Looping Through Rocket Body Light Curve Files...')

#	set input directory
rocket_body_dir = os.listdir(read_path)

if specific_file_flag is True:
	rocket_body_dir = [specific_file]

#	loop through rocket body IDs
for loop_itt, mmt_file in enumerate(rocket_body_dir):

	#	parse filename
	name_str = mmt_file.split('_')[0]
	norad_id = int(mmt_file.split('-')[1].split('_')[1][:-4])

	print('\n		#####################################################################')
	print('		Plotting data for: {}, {}'.format(name_str, norad_id))

	###############################################################################################
	#	load MMT file
	
	#	set time
	mmt_load_start = time.time()
	
	print('\n	[{}] Loading MMT data...'.format(datetime.datetime.now()))
	
	with open(os.path.join(read_path, mmt_file)) as read_file:
		mmt_data = read_file.readlines()

	###############################################################################################
	#	parse MMT file
	
	#	strip header
	header_text = mmt_data[7]
	raw_data = mmt_data[8:]
	
	print('\n		Parsing MMT data...')
	
	#	initialise output
	n_rows = 14
	parsed_data = np.zeros([len(raw_data), n_rows])
	
	for i, line in enumerate(raw_data):
	
		#	split line
		date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
			distance, phase, channel, track_id = line.split(' ')
	
		#	parse data and set data types
		mag_std = float(mag_std)
		mag_app = float(mag_app)
		distance = float(distance)
		phase = float(phase)
		track_id = int(track_id)
	
		#	parse date
		year, month, day = date_str.split('-')
		year = int(year)
		month = int(month)
		day = int(day)
	
		#	parse time
		hour, minute, second = time_str.split(':')
		hour = int(hour)
		minute = int(minute)
		second = np.round(float(second), 1)
	
		#	calculate julian date
		jday, jfrac = date2jd(year, month, day, hour, minute, second)
	
		#	generate output line
		output_line = [
			track_id, norad_id,
			jday, jfrac,
			year, month, day,
			hour, minute, second,
			mag_std, mag_app,
			distance, phase
		]
	
		#	add to output array
		parsed_data[i, :] = output_line
	
	print('			Total data length = ', len(parsed_data))
	print('			Total number of objects = ', len(np.unique(parsed_data[:, 1])))
	print('			Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

	###############################################################################################
	#	extracting data between specified windows
	
	print('\n		Seaching for tracks inside time window:')
	
	#	start window
	window_start_jd = np.add(*date2jd(*window_start_date))
	print('\n			Window start 	= ', window_start_date[0:3])
	
	#	stop window
	window_stop_jd = np.add(*date2jd(*window_stop_date))
	print('			Window stop 	= ', window_stop_date[0:3])
	
	#	extract track ids
	track_id_list = np.unique(parsed_data[:, 0])

	#	loop through tracks
	ntracks = 0
	first_lc_flag = True
	for track_id in track_id_list:

		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]
	
		#	extract light curve data
		jd_plot = np.add(track_data[:, 2], track_data[:, 3])
		std_mag_plot = track_data[:, 10]
		app_mag_plot = track_data[:, 11]
		distance_plot = track_data[:, 12]
		phase_plot = track_data[:, 13]
		start_line = track_data[0]
		stop_line = track_data[-1]
		start_jdate = np.add(start_line[2], start_line[3])
		stop_jdate = np.add(stop_line[2], stop_line[3])
	
		#	ensure between window start/stop
		if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
	
			#	light curve is within window
			ntracks += 1
	
			#	if first light curve, start array
			if first_lc_flag is True:
				windowed_data = track_data
				first_lc_flag = False
	
			#	else append to list
			elif first_lc_flag is False:
				windowed_data = np.vstack([windowed_data, track_data])
	
	print('\n			{} track(s) extracted.'.format(ntracks))

	#	reset track_id_list for windowed data
	track_id_list = np.unique(windowed_data[:, 0])

	###############################################################################################
	#	display initialisation time
	mmt_load_end = time.time()
	mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)
	
	print('\n	MMT Load Finished.')
	print('\n		- Time taken = {} s.'.format(mmt_load_time))

	###############################################################################################
	#	main track loop
	###############################################################################################

	#	loop initialisation
	first_track = True
	color_index = 0
	tracks_plotted = 0

	#	generate mesh grids
	omega_mesh, m_mesh = np.meshgrid(omega_brdf_range, m_brdf_range)

	#	calculate solution space thresholds
	#	min m index
	m_id_min = np.where(m_mesh[:, 0]==min_m_brdf)[0][0]
	m_id_max = np.where(m_mesh[:, 0]==max_m_brdf)[0][0] + 1
	#	min omega index
	omega_id_min = np.where(np.round(omega_mesh[0, :], 2)==min_omega_brdf)[0][0]
	omega_id_max = np.where(np.round(omega_mesh[0, :], 2)==max_omega_brdf)[0][0] + 1
	
	#	apply thresholds
	m_mesh_plot = m_mesh[omega_id_min:omega_id_max, m_id_min:m_id_max]
	omega_mesh_plot = omega_mesh[omega_id_min:omega_id_max, m_id_min:m_id_max]

	#	initialise total rms of d array
	total_rms_of_d = np.zeros(len(d_brdf_range))
	
	#	initialise total rms array
	total_rms_array = np.zeros([m_mesh_plot.shape[0], m_mesh_plot.shape[1], len(d_brdf_range)])

	if specific_track_flag is True: 
		track_id_list = [specific_track]

	#	initialise track rms array
	track_rms_data = np.zeros([len(track_id_list), 3])
	
	#	loop through tracks
	for track_itt, track_id in enumerate(track_id_list):

		#	convert to int
		track_id = int(track_id)
	
		print('\n			#############################################################')
		print('			Working on Track {}, ID: {}'.format(track_itt, track_id))

		###########################################################################################
		#	input directory

		#	create object directory
		object_dir = name_str + '_{}'.format(norad_id)
		input_path = os.path.join(path_data, 'lightcurve_dir', 'rocket_body_analysis', object_dir)

		#	create track directory
		track_dir = 'track-{}'.format(track_id)
		input_path = os.path.join(input_path, track_dir)

		###########################################################################################
		#	extracting requested single track
		
		print('\n		Extracting requested track index...')

		#	extract track data
		track_indices = np.where(windowed_data[:, 0]==track_id)[0]
		track_data = windowed_data[track_indices]
		
		###########################################################################################
		#	extract track time data
	
		mmt_line = track_data[0]
		
		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]

		###########################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
		sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)
		
		#######################################################################################
		#	ensure time continuity
		
		print('\n				Ensuring time continuity in extracted data...')
		
		track_data_temp = []
		sim_index = 0
	
		for itt, line in enumerate(track_data):
		
			#	parse line data
			line_track_id, line_norad_id,\
				jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = line
			
			step_jd = jday + jfrac
		
			#	calculate current seconds
			current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
		
			output_line = [
				jday, jfrac, current_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]
		
			##################################################
			#	extracting data of time step
		
			#	data match found
			if current_sec == sim_range_sec[sim_index]:
				track_data_temp.append(output_line)
		
				#	itterate number of output data points
				sim_index += 1
		
				#	save current line as previous line
				prev_line = output_line
		
			#	data point is not required
			elif current_sec < sim_range_sec[sim_index]:
				pass
		
			#	interpolation required
			elif current_sec > sim_range_sec[sim_index]:
		
				#	parse previous line
				jday_prev, jfrac_prev, prev_sec, \
					year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
					mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
		
				#	calculate interpolation requirements
				gap = np.round(current_sec - prev_sec, 1)
				npts = int(gap / time_step)
		
				#	get current light curve data
				mag_std_last = mag_std
				mag_app_last = mag_app
				distance_last = distance
				phase_last = phase
		
				#	calculate linear interpolation step values
				mag_std_step = (mag_std_last - mag_std_prev) / npts
				mag_app_step = (mag_app_last - mag_app_prev) / npts
				distance_step = (distance_last - distance_prev) / npts
				phase_step = (phase_last - phase_prev) / npts
		
				#	calculate interpolation data
				for n in range(1, npts + 1):
		
					#	current simulation second
					interp_sec = np.round(prev_sec + (n * time_step), 1)

					#	time data
					jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
					jday, jfrac = jday_jfrac(jdate)
					year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
					#	calculate interpolated light curve data
					mag_std = mag_std_prev + (n * mag_std_step)
					mag_app = mag_app_prev + (n * mag_app_step)
					distance = distance_prev + (n * distance_step)
					phase = phase_prev + (n * phase_step)
		
					#	generate new output line
					interp_output_line = [
						jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
		
					if interp_sec == sim_range_sec[sim_index]:
						track_data_temp.append(interp_output_line)
						sim_index += 1
		
				if interp_sec != current_sec:
					#	append final line
					track_data_temp.append(output_line)
					sim_index += 1
		
			else:
				print('ERROR')
				sys.exit()
		
			#	save previous line
			if itt != 0:
				prev_line = output_line
		
			#	array is complete condition
			if len(track_data_temp) == lc_len_record:
				break
		
		#	ensure parsed MMT data matches simulation data
		track_data = np.array(track_data_temp)
		
		if len(track_data) == len(lc_sim_range):
			print('					Simulation and MMT data matches.')
		else:
			print('					Error: Simulation and MMT data does not match...Exiting.')
			sys.exit()
		
		###########################################################################################
		#	extract mmt data

		jd_plot = np.add(track_data[:, 0], track_data[:, 1])
		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]

		###########################################################################################
		#	simulation printout
		###########################################################################################
		
		print('\n		#####################################################################')
		print('		Extracted MMT track parameters: ')
		print(
			'			Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
			.format(*lc_start_date)
		)
		print(
			'			Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
			.format(*lc_stop_date)
		)
		print('			Simulation time step:		{}'.format(time_step))
		print('			Duration:			{}		[steps]'.format(lc_len_record))
		print('							{}		[s]'.format(np.round(lc_duration_sec, 1)))

		###########################################################################################
		#	setting simulation time data
		###########################################################################################
		
		sim_range = lc_sim_range
		len_record = lc_len_record
		start_date = lc_start_date
		stop_date = lc_stop_date
		duration_sec = lc_duration_sec

		#	calculate sim time range in seconds
		sim_time = np.linspace(0, duration_sec, len_record)
		
		###########################################################################################
		#	extract plot data
		###########################################################################################

		print('\n	#############################################################################')
		print('	Beginning Extraction Loop...')

		#	generate mesh grids
		attit_inc_mesh, attit_azm_mesh = np.meshgrid(rot_vec_inc_deg_range, rot_vec_azm_deg_range)

		attit_min_rms_array = np.zeros(attit_azm_mesh.shape)
		attit_max_rms_array = np.zeros(attit_azm_mesh.shape)
		attit_avg_rms_array = np.zeros(attit_azm_mesh.shape)
		attit_std_rms_array = np.zeros(attit_azm_mesh.shape)

		#	looping through attitude inc
		for inc_index, rot_vec_inc_deg in enumerate(rot_vec_inc_deg_range):

			print('\n	#############################################################################')
			print('	Working on Inclination case: ', int(rot_vec_inc_deg))
			inc_dir = 'inc-{}'.format(int(rot_vec_inc_deg))
			inc_input_path = os.path.join(input_path, inc_dir)

			#	initialise rms of inclination array
			rms_of_inc = []

			#	looping through attitude azmimuth
			for rot_vec_azm_deg in rot_vec_azm_deg_range:

				print('\n		#####################################################################')
				print('		Working on Azimuth case: ', int(rot_vec_azm_deg))
				attit_dir = 'inc-{}_azm-{}'.format(int(rot_vec_inc_deg), int(rot_vec_azm_deg))
				attit_input_path = os.path.join(inc_input_path, attit_dir)

				#	generate output filename
				input_file = '{}-{}_track-{}_inc-{}_azm-{}.txt'.format(
					name_str,
					int(norad_id),
					track_id,
					int(rot_vec_inc_deg),
					int(rot_vec_azm_deg)
				)
	
				#	generate the rms values output file name
				rms_values_file = input_file[:-4] + '.rms_values'

				if os.path.isfile(os.path.join(attit_input_path, rms_values_file)) \
						and rms_regen_flag is False:

					print('\n			Found: Reading .rms_values file...')
					track_rms_array_read = np.loadtxt(
						os.path.join(attit_input_path, rms_values_file),
						delimiter=','
					)

					track_rms_array = np.reshape(track_rms_array_read, total_rms_array.shape)

				else:
			
					print('\n			Not Found: Generating .rms_values file...')

					#	generate track rms array
					track_rms_array = np.zeros([
						len(omega_mesh[:, 0]),
						len(omega_mesh[0, :]),
						len(d_brdf_range)
					])

					###############################################################################
					#	load total data array
					print('\n			Loading Light Curve Data File...')
					print('\n				File to load = ', input_file)
					pre_load_time = time.time()
		
					total_data = np.loadtxt(
						os.path.join(attit_input_path, input_file), delimiter=','
					)
		
					post_load_time = time.time()
		
					print('\n				- Time Taken to Load: ', post_load_time - pre_load_time)

					#	extract light curve indices
					light_curve_indices = np.unique(total_data[:, 0])

					print('\nExtracting RMS data...')

					for light_curve_index in light_curve_indices:
	
						if light_curve_index % 10 == 0.0:
							print(
								'					Working on Light Curve ID: {}'.format(
									int(light_curve_index)
									),
								end='\r'
							)
	
						#	light curve data of index
						data_indices = np.where(total_data[:, 0]==light_curve_index)[0]
						light_curve_data = total_data[data_indices]
					
						#	parse data from light curve array
						light_curve_seconds = light_curve_data[:, 2]
						synth_signal_app = light_curve_data[:, 3]
						synth_signal_std = light_curve_data[:, 4]
	
						lc_index, jdate, sec_since, mag_app, mag_std, \
							object_id, rot_vec_azm_deg, rot_vec_inc_deg, \
							top_d_brdf, top_m_brdf, top_omega_brdf, \
							bot_d_brdf, bot_m_brdf, bot_omega_brdf, \
							side_d_brdf, side_m_brdf, side_omega_brdf, \
							= light_curve_data[0]
	
						if brdf_str == 'top':
							plot_d_brdf = top_d_brdf
							plot_m_brdf = top_m_brdf
							plot_omega_brdf = top_omega_brdf
			
						elif brdf_str == 'bot':
							plot_d_brdf = bot_d_brdf
							plot_m_brdf = bot_m_brdf
							plot_omega_brdf = bot_omega_brdf
			
						elif brdf_str == 'side':
							plot_d_brdf = side_d_brdf
							plot_m_brdf = side_m_brdf
							plot_omega_brdf = side_omega_brdf
			
						else:
							plot_d_brdf = side_d_brdf
							plot_m_brdf = side_m_brdf
							plot_omega_brdf = side_omega_brdf
	
						#	find occurance in omega/m grids
						m_index = np.where(np.round(omega_mesh[0, :], 2)==plot_omega_brdf)[0]
						omega_index = np.where(np.round(m_mesh[:, 0], 2)==plot_m_brdf)[0]
						d_index = np.where(np.round(d_brdf_range, 2)==plot_d_brdf)[0]
	
						#	calculate RMSE
						rms_std = rms_calc(synth_signal_std, mmt_signal_std)
						rms_app = rms_calc(synth_signal_app, mmt_signal_app)

						#	add to output
						track_rms_array[m_index, omega_index, d_index] = rms_app

					print('\n		Finished')

					#	reshape array for saving
					reshape_array = [
						len(total_rms_array[:, 0, 0]) * len(total_rms_array[0, 0, :]),
						len(total_rms_array[0, :, 0])
					]

					track_rms_array_save = np.reshape(track_rms_array, reshape_array)

					###############################################################################
					#	save rms_values array to avoid regeneration
				
					np.savetxt(
						os.path.join(attit_input_path, rms_values_file),
						track_rms_array_save,
						delimiter=','
					)
		
					print('\n		RMS Values file saved: ', rms_values_file)
					print('\n		Data Shape = ', track_rms_array[:, :, :].shape)

				###################################################################################
				#	looping through d_brdf values
				###################################################################################
		
				#	initialise RMS of d array
				rms_of_d = np.zeros(len(d_brdf_range))
				coords_of_d = np.zeros([len(d_brdf_range), 2])
				
				max_minima = 1
				minima_plot_data = np.zeros([25, len(d_brdf_range), 2])

				print('\n			#############################################################')
				print('			Looping through d_brdf values...')

				for array_index in range(len(track_rms_array[0, 0, :])):
	
					#	get d_brdf value
					d_brdf = np.round(d_brdf_range[array_index], 2)

					print('\n				#####################################################')
					print('				Performing case d = {}...'.format(d_brdf))

					#	extract track rms values of d_brdf
					rms_array_of_d = track_rms_array[:, :, array_index]

					###############################################################################
					#	apply threshold
					rms_array_of_d = rms_array_of_d[
						omega_id_min:omega_id_max,
						m_id_min:m_id_max
					]
		
					print('\n				Maximum RMS = ', rms_array_of_d.max())
					print('				Minimum RMS = ', rms_array_of_d.min())
		
					#	if normalisation flag is true, normalise
					if rms_norm_flag is True:
						print('\n				Normalising RMS...')
						rms_array_of_d = np.divide(rms_array_of_d, rms_array_of_d.max())

					###############################################################################
					#	add track rms data to total rms data

#					total_rms_array[:, :, array_index] = np.add(
#						total_rms_array[:, :, array_index], rms_array_of_d
#					)
#	
#					print('\n				RMS data added to total array')
			
					###############################################################################
					#	generating properties plot data
	
					print('\n				Searching for minimum RMS location...')
					#	save minimum RMS value
					rms_of_d[array_index] = rms_array_of_d.min()
	
					#	round to reasonable decimal places
					rms_array_of_d = np.round(rms_array_of_d, 10)
					min_rms = np.round(rms_array_of_d.min(), 10)
	
					first_match_flag = True
	
					for row_index, line in enumerate(rms_array_of_d):
						for col_index, value in enumerate(line):
							
							if value == min_rms:
	
								#	apply threshold offset
								col_coord = col_index# + m_id_min
								row_coord = row_index# + omega_id_min
	
								if first_match_flag is True:
									min_rms_coord = [col_coord, row_coord]
									first_match_flag = False
								else:
									min_rms_coord = np.vstack([min_rms_coord, [col_coord, row_coord]])
	
					min_rms_coord = np.array(min_rms_coord)
					n_minima = len(min_rms_coord)
	
					print('\n					- Number of Minima: ', n_minima)
	
					#	set new max if increased
					if max_minima < n_minima:
						max_minima = n_minima
	
					#	if only a single coordinate
					if min_rms_coord.shape == (2,):
	
						#	find omega and m coordinates
						omega_coord = omega_mesh[min_rms_coord[0], min_rms_coord[1]]
						m_coord = m_mesh[min_rms_coord[0], min_rms_coord[1]]
	
					else:
						#	working through multiple minima
						for minima_index, min_coord in enumerate(min_rms_coord):
				
							#	find omega and m coordinates
							omega_coord = omega_mesh[min_coord[0], min_coord[1]]
							m_coord = m_mesh[min_coord[0], min_coord[1]]
	
							#	add to coordinate outputs
							minima_plot_data[minima_index, array_index, :] = [omega_coord, m_coord]
						
					#	save min RMS coordinate
					coords_of_d[array_index] = [omega_coord, m_coord]
	
				print('\n			#############################################################')
				print('			RMS Data Generation Loop Complete...')

				###################################################################################
				#	generate figures
				###################################################################################
			
				if brdf_det_flag is True:

					#	set color and label strings
					color_str = 'C' + str(color_index)
					label_str = r'$Track-{}$'.format(int(track_id))
				
					#	tracking minimum RMS value
					ax2[0].plot(
						d_brdf_range, rms_of_d, color=color_str, label=label_str, lw=plot_linewidth
					)
				
					#	tracking minimum RMS positon
					ax2[1].plot(
						coords_of_d[:, 0], coords_of_d[:, 1],
						color=color_str,
						lw=plot_linewidth,
						marker='x',
						ms=marker_size
					)
				
					#	add d_brdf labels
					for (coords, d_value) in zip(coords_of_d, d_brdf_range):
						ax2[1].text(*coords, '$' + str(d_value) + '$', fontsize=text_fontsize)
			
				###################################################################################
				#	generate equal minima data
			
				print('\n			#############################################################')
				print('			Calculating equal minima data...')
			
				#	initialise minima coordinate arrays
				minima_omega = np.zeros([len(d_brdf_range), max_minima])
				minima_m = np.zeros([len(d_brdf_range), max_minima])
				
				#	fill with with first data set
				max_minima_range = np.arange(max_minima)
				for minima_index in max_minima_range:
					minima_omega[:, minima_index] = coords_of_d[:, 0]
					minima_m[:, minima_index] = coords_of_d[:, 1]

				for plot_id, d_brdf in enumerate(d_brdf_range):
			
					print('\n				#####################################################')
					print('				Performing case d = {}...'.format(d_brdf))
			
					track_rms_values = track_rms_array[:, :, plot_id]

					#	apply threshold
					track_rms_values = track_rms_values[
						omega_id_min:omega_id_max,
						m_id_min:m_id_max
					]
			
					#	round to reasonable decimal places
					track_rms_values = np.round(track_rms_values, 10)
			
					#	extract min rms
					min_rms = np.round(track_rms_values.min(), 10)
			
					#	set first match flag to true
					first_match_flag = True
			
					print('\n					Extracting RMS minima data...')
			
					#	find all occurences of min rms in rms array
					for row_index, line in enumerate(track_rms_values):
						for col_index, value in enumerate(line):
							if value == min_rms:
			
								#	apply threshold offset
								col_coord = col_index + m_id_min
								row_coord = row_index + omega_id_min
			
								if first_match_flag is True:
									min_rms_coord = [col_coord, row_coord]
									first_match_flag = False
								else:
									min_rms_coord = np.vstack([
										min_rms_coord,
										[col_coord, row_coord]
									])
			
					#	convert to numpy array
					min_rms_coord = np.array(min_rms_coord)
			
					#	single minimum
					if min_rms_coord.shape == (2,):
			
						#	find omega and m coordinates
						omega_coord = omega_mesh[min_rms_coord[0], min_rms_coord[1]]
						m_coord = m_mesh[min_rms_coord[0], min_rms_coord[1]]
			
						minima_omega[plot_id, :] = [omega_coord]
						minima_m[plot_id, :] = [m_coord]
			
					#	multiple minima
					else:
						for coord_index, min_coord in enumerate(min_rms_coord):
				
							#	find omega and m coordinates
							omega_coord = omega_mesh[min_coord[0], min_coord[1]]
							m_coord = m_mesh[min_coord[0], min_coord[1]]
			
							minima_omega[plot_id, coord_index] = omega_coord
							minima_m[plot_id, coord_index] = m_coord
			
					print('\n						- Done.')

				print('\n			#############################################################')
				print('			RMS Minima Data Extracted...')

				###################################################################################
				#	plotting multiple minima possibilities
				
				print('\n			#############################################################')
				print('			Plotting minima data...')
			
				if plot_all_minima_flag is True:
					max_minima_range = np.arange(max_minima)
					for minima_index in max_minima_range:
					
						omega_col = minima_omega[:, minima_index]
						m_col = minima_m[:, minima_index]
				
#						minima_coords_of_d = np.transpose(np.vstack([omega_col, m_col]))
				
						if brdf_det_flag is True:

							#	tracking minimum RMS positon
							ax2[1].plot(
								omega_col, m_col,
								color=color_str,
								marker='x',
								linewidth=plot_linewidth,
								markersize=marker_size
							)
						
							#	add d_brdf labels
							for (coords, d_value) in zip(minima_coords_of_d, d_brdf_plot_range):
								ax2[1].text(*coords, '$' + str(d_value) + '$', fontsize=text_fontsize)

				#	itterate indices
				color_index += 1
				first_track = False
			
				#	add rms of d to total
				total_rms_of_d = np.add(total_rms_of_d, rms_of_d)
			
				if brdf_det_flag is True:

					#	finalise plot
#					ax2[1].set_xlim([min_omega_brdf, max_omega_brdf])
#					ax2[1].set_ylim([min_m_brdf, max_m_brdf])
					ax2[1].set_xlim([0.0, 1.0])
					ax2[1].set_ylim([0.0, 1.0])
					
					ax2[1].set_xlabel(r'$\omega$', fontsize=ylabel_fontsize)
					ax2[1].set_ylabel(r'$m$', fontsize=xlabel_fontsize)
					ax2[1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
					ax2[1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
					
					ax2[0].set_xlabel(r'$d$', fontsize=xlabel_fontsize)
					ax2[0].set_ylabel(r'$RMS\ Error$', fontsize=ylabel_fontsize)
					ax2[0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
					ax2[0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
					
#					fig2.legend(
#						loc='lower center',
#						fontsize=legend_fontsize,
#						ncol=4,
#						frameon=False
#					)

				print('\n				- Done.')

				###################################################################################
				#	Generating Colormap Plots
				###################################################################################
				
				print('\n			#############################################################')
				print('			Generating BRDF Colormap Plots...')
				print('			#############################################################')
				
				###################################################################################
				#	figure generation inputs
				
				#	set graphtype (cont/mesh)
				graphtype = 'cont'
				graphtype = 'mesh'
				
				#	set colour map
				cmap = plt.get_cmap('PiYG')
				
				#	set rms normalisation flag
				norm_flag = True
				
				#	colour levels
				levels_data = track_rms_array[
					omega_id_min:omega_id_max,
					m_id_min:m_id_max,
					:
				]
				
				levels_min = levels_data.min()
				levels_max = levels_data.max()
				
				#	set color levels
				levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
				norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)
			
				print('\n			Setting colourbar levels to:')
				print('				{} - {}'.format(levels_min, levels_max))
				 
				###################################################################################
				#	figure generation paramters
				
				#	set colour levels
				n_cols = 100
				
				#	set subplot spacing values
				hspace_val = 0.35
				wspace_val = 0.35
				
				#	set font sizes
				title_fontsize = 6
				text_fontsize = 4
				cbar_fontsize = 4
				legend_fontsize = 4
				xlabel_fontsize = 6
				ylabel_fontsize = 6
				xticks_fontsize = 4
				yticks_fontsize = 4
				
				#	set text padding values
				title_pad = 2
				xticks_pad = 4
				yticks_pad = 0
				
				#	set marker parameters
				marker_size = 3
				
				#	set figure height/width
				fig_width = 10.0/inch2cm
				fig_height = 12.0/inch2cm
			
				if brdf_map_flag is True:

					#	subplots figure
					fig, ax = plt.subplots(
						3, 2,
						figsize=(fig_width, fig_height),
						dpi=300,
					)
					
					plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)
				
				###################################################################################
				#	plotting

				#	define brdf filter range
				omega_set = 0.1
				omega_set2 = 0.9
				m_set = 0.1
				m_set2 = 0.9

				#	initialise RMS of d array
				total_coords_of_d = np.zeros([len(d_brdf_range), 2])
				
				#	initialise figure axis coords
				coords_array = np.array([
					[0, 0],
					[0, 1],
					[1, 0],
					[1, 1],
					[2, 0],
					[2, 1],
				])
				
				print('\n			#############################################################')
				print('			Looping through d_brdf values...')
				
				for plot_id, (d_brdf, plot_coords) in enumerate(
						zip(d_brdf_range, coords_array)
					):
				
					print('\n				#####################################################')
					print('				Performing case d = {}...'.format(d_brdf))
			
					#	extract plot coords
					plot_coord_x, plot_coord_y = plot_coords
			
					#	extract data
					track_rms_array_plot = track_rms_array[
						:,
						:,
						plot_id
					]

					#	round to reasonable decimal places
					track_rms_array_plot = np.round(track_rms_array_plot, 10)
					min_rms = track_rms_array_plot.min()

					#	save minimum RMS value
					rms_of_d[plot_id] = min_rms
					
					print('\n				Plotting Color Data...')

					#	apply filter
					omega_id = np.where(np.round(omega_mesh_plot[0, :], 2)==omega_set)[0][0]
					omega_id2 = np.where(np.round(omega_mesh_plot[0, :], 2)==omega_set2)[0][0] + 1
			
					m_id = np.where(np.round(m_mesh_plot[:, 0], 2)==m_set)[0][0]
					m_id2 = np.where(np.round(m_mesh_plot[:, 0], 2)==m_set2)[0][0] + 1

					#	apply
					omega_mesh_plot = omega_mesh_plot[m_id:m_id2, omega_id:omega_id2]
					m_mesh_plot = m_mesh_plot[m_id:m_id2, omega_id:omega_id2]
					track_rms_array_plot = track_rms_array_plot[m_id:m_id2, omega_id:omega_id2]

					#	separation
					m_step = m_brdf_range[1] - m_brdf_range[0]
					omega_step = omega_brdf_range[1] - omega_brdf_range[0]

					#	add addditional rows to plot data to complete circle
					omega_mesh_plot = np.vstack([omega_mesh_plot, omega_mesh_plot[-1, :]])
					m_mesh_plot = np.vstack([m_mesh_plot, np.add(m_mesh_plot[-1, :], m_step)])
					track_rms_array_plot = np.vstack([
						track_rms_array_plot,
						np.full(track_rms_array_plot[-1, :].shape, 0)
					])

					#	add addditional cols to plot data to complete circle
					omega_mesh_plot = np.hstack([
						omega_mesh_plot,
						np.reshape(
							omega_mesh_plot[:, -1] + omega_step,
							[len(omega_mesh_plot[:, -1]), 1]
						)
					])
					
					m_mesh_plot = np.hstack([
						m_mesh_plot,
						np.reshape(
							m_mesh_plot[:, -1],
							[len(m_mesh_plot[:, -1]), 1]
						)
					])
					
					track_rms_array_plot = np.hstack([
						track_rms_array_plot,
						np.reshape(
							np.full(track_rms_array_plot[:, -1].shape, 0),
							[len(track_rms_array_plot[:, -1]), 1]
						)
					])

					#	recolour levels
					levels_data = track_rms_array_plot
					
					levels_min = levels_data.min()
					levels_max = levels_data.max()
					
					#	set color levels
					levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
					norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

					###############################################################################
					#	mesh plot
			
					if brdf_map_flag is True:

						#	plot mesh
						cf = ax[plot_coord_x, plot_coord_y].pcolormesh(
							m_mesh_plot,
							omega_mesh_plot,
							track_rms_array_plot,
							cmap=cmap,
							norm=norm
						)
	
						###############################################################################
						#	finalise plot
						
						#	add title
						ax[plot_coord_x, plot_coord_y].set_title(
							r'$d = {}$'.format(d_brdf),
							fontsize=title_fontsize,
							pad=title_pad
						)
					
						#	add colorbar
						cbar = fig.colorbar(cf, ax=ax[plot_coord_x, plot_coord_y])
						cbar.ax.tick_params(labelsize=cbar_fontsize)
						
						#	format x axis
	#					ax[plot_coord_x, plot_coord_y].set_xlim([min_omega_brdf, max_omega_brdf])
						ax[plot_coord_x, plot_coord_y].set_xlim([0.0, 1.0])
				
						ax[plot_coord_x, plot_coord_y].set_xlabel(r'$\omega$', fontsize=xlabel_fontsize)
						ax[plot_coord_x, plot_coord_y].tick_params(
							axis='x', labelsize=xticks_fontsize, pad=xticks_pad
						)
	
						#	format y axis
	#					ax[plot_coord_x, plot_coord_y].set_ylim([min_m_brdf, max_m_brdf])
						ax[plot_coord_x, plot_coord_y].set_ylim([0.0, 1.0])
				
						ax[plot_coord_x, plot_coord_y].set_ylabel(r'$m$', fontsize=ylabel_fontsize)
						ax[plot_coord_x, plot_coord_y].tick_params(
							axis='y', labelsize=yticks_fontsize, pad=yticks_pad
						)
	
						#	add title
						fig.suptitle(
							'Attitude Case: Inc-{}, Azm-{}'.format(rot_vec_inc_deg, rot_vec_azm_deg)
						)
				
					###############################################################################
				 	#	finding position of Min RMS
				 
					print('\n				Searching for minimum RMS location...')
				
					first_match_flag = True

					print('\n				Finding minimum RMS: ', min_rms)

					#	find min rms coordinates
					for row_index, line in enumerate(track_rms_array_plot):
						for col_index, value in enumerate(line):
							if value == min_rms:

								print('					-Found at: ', row_index, col_index)
				
								#	apply threshold offset
								col_coord = col_index + m_id_min
								row_coord = row_index + omega_id_min
				
								if first_match_flag is True:
									min_rms_coord = [col_coord, row_coord]
									first_match_flag = False
								else:
									min_rms_coord = np.vstack(
										[min_rms_coord, [col_coord, row_coord]]
									)

				 	#	convert to numpy array
					min_rms_coord = np.array(min_rms_coord)

				 	###############################################################################
				 	#	plotting position of Min RMS

					#	calculate minima offsets to align with plot
					omega_offset = omega_brdf_step / 2
					m_offset = m_brdf_step / 2
					
					#	single minimum
					if min_rms_coord.shape == (2,):
					
						#	find omega and m coordinates
						omega_coord = omega_mesh[min_rms_coord[0], min_rms_coord[1]] + omega_offset
						m_coord = m_mesh[min_rms_coord[0], min_rms_coord[1]] + m_offset

						if brdf_map_flag is True:

							#	plot on figure
							ax[plot_coord_x, plot_coord_y].plot(
								omega_coord, m_coord,
								marker='+',
								markersize=marker_size,
								color='k',
							)
					
					#	multiple minima
					else:
						for min_coord in min_rms_coord:

							print(min_coord)
					
							#	find omega and m coordinates
							omega_coord = omega_mesh[min_coord[0], min_coord[1]]
							m_coord = m_mesh[min_coord[0], min_coord[1]]
				
							if brdf_map_flag is True:

								#	plot on figure
								ax[plot_coord_x, plot_coord_y].plot(
									omega_coord, m_coord,
									marker='+',
									markersize=marker_size,
									color='k',
								)

					#	save min RMS coordinate
					total_coords_of_d[plot_id] = [omega_coord, m_coord]

#				###################################################################################
#				#	plot total rms data
#
#				if brdf_det_flag is True:
#
#					#	tracking minimum RMS value
#					ax3[0].plot(
#						d_brdf_range, total_rms_of_d,
#						color='k',
#						label=r'$Total$',
#						linewidth=plot_linewidth,
#						marker='x',
#						ms=marker_size
#					)
#						
#					#	tracking minimum RMS positon
#					ax3[1].plot(
#						total_coords_of_d[:, 0], total_coords_of_d[:, 1],
#						color='k',
#						lw=plot_linewidth,
#						marker='x',
#						ms=marker_size
#					)
#					
#					#	add d_brdf labels
#					for (coords, d_value) in zip(total_coords_of_d, d_brdf_range):
#					
#						min_x_coord, min_y_coord = [*coords]
#						ax3[1].text(
#							min_x_coord, min_y_coord + 0.0005,
#							'$' + str(d_value) + '$',
#							fontsize=text_fontsize
#						)
#					
#					# #	finalise plot
#					# ax3[1].set_xlim([min_omega_brdf, max_omega_brdf])
#					# ax3[1].set_ylim([min_m_brdf, max_m_brdf])
#					
#					ax3[1].set_xlabel(r'$\omega$', fontsize=ylabel_fontsize)
#					ax3[1].set_ylabel(r'$m$', fontsize=xlabel_fontsize)
#					ax3[1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
#					ax3[1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
#					
#					ax3[0].set_xlabel(r'$d$', fontsize=xlabel_fontsize)
#					ax3[0].set_ylabel(r'$RMS\ Error$', fontsize=ylabel_fontsize)
#					ax3[0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
#					ax3[0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
#					
#					# fig3.legend(
#					# 	loc='lower center',
#					# 	ncol=7,
#					# 	fontsize=legend_fontsize,
#					# 	frameon=False
#					# )
#					
#					# plt.gcf().subplots_adjust(bottom=0.15)

				###################################################################################
				#	end of attitude state
				###################################################################################

				print('\n		#####################################################################')
				print('		Attitude State Finished:')
				print('			Inclination = 	{}'.format(int(rot_vec_inc_deg)))
				print('			Azimuth = 	{}'.format(int(rot_vec_azm_deg)))

				print('\n		Best Fitting Solutions:')
				
				for (d_brdf, coords, rms_val) in zip(
						d_brdf_range, total_coords_of_d, total_rms_of_d
					):
				
					omega_brdf, m_brdf = np.round(coords, 2)
				
					print('\n			D = {};	m = {};	omega = {};	RMS = {:1.3f}'.format(
						d_brdf, m_brdf, omega_brdf, rms_val
					))

				#	RMS properties
				min_rms = track_rms_array.min()
				max_rms = track_rms_array.max()
				avg_rms = np.average(track_rms_array)
				std_rms = np.std(track_rms_array)

				print('\n		RMS Min = ', min_rms)
				print('		RMS Max = ', max_rms)
				print('		RMS Avg = ', avg_rms)
				print('		RMS Std. Dev. = ', std_rms)

				#	add to rms output
				rms_of_inc.append([min_rms, max_rms, avg_rms, std_rms])

#			plt.show()
#			sys.exit()

			#######################################################################################
			#	end of inclination loop 
			#######################################################################################

			print('\n	#############################################################################')
			print('	Finished Inclination case: ', int(rot_vec_inc_deg))

			#	convert to numpy array
			rms_of_inc = np.array(rms_of_inc)

			#	add rms data to attitude output array
			attit_min_rms_array[:, inc_index] = rms_of_inc[:, 0]
			attit_max_rms_array[:, inc_index] = rms_of_inc[:, 1]
			attit_avg_rms_array[:, inc_index] = rms_of_inc[:, 2]
			attit_std_rms_array[:, inc_index] = rms_of_inc[:, 3]

#		plt.show()
#		sys.exit()

		#######################################################################################
		#	Generating Colormap Plots
		#######################################################################################
		
		print('\n#############################################################################')
		print('Generating Attitude Plots...')
		print('#############################################################################')
	
		#	define an attitude plot range
		azm_plot_min = 20
		azm_plot_max = 320
		inc_plot_min = 30
		inc_plot_max = 80
		
		azm_plot_min = 0
		azm_plot_max = 350
		inc_plot_min = 0
		inc_plot_max = 90

		azimuth_across_zero = False
		
		#######################################################################################
		#	generic figure initialisation

		#	set figure height/width
		fig_width = 6.0/inch2cm
		fig_height = 6.0/inch2cm

		marker_size = 10

		#	find azimuth offset to align with grid
		nazm_vals = len(attit_azm_mesh) - 1
		offset = (360 / nazm_vals) / 2
		azm_step = rot_vec_azm_deg_range[1] - rot_vec_azm_deg_range[0]
		inc_offset = (rot_vec_inc_deg_range[1] - rot_vec_inc_deg_range[0]) / 2
		
		#	add addditional rows to plot data to complete circle
		attit_inc_mesh = np.vstack([attit_inc_mesh, attit_inc_mesh[-1, :]])
		attit_azm_mesh = np.vstack([attit_azm_mesh, np.add(attit_azm_mesh[-1, :], azm_step)])

		#	add addditional cols to plot data to complete circle
		attit_inc_mesh = np.hstack([
			attit_inc_mesh,
			np.reshape(
				attit_inc_mesh[:, -1] + 10,
				[len(attit_inc_mesh[:, -1]), 1]
			)
		])
		
		attit_azm_mesh = np.hstack([
			attit_azm_mesh,
			np.reshape(
				attit_azm_mesh[:, -1],
				[len(attit_azm_mesh[:, -1]), 1]
			)
		])

		#	find azm mesh rows of defined azm range (+1 for index, +1 to plot)
		azm_min_row_index = np.where(attit_azm_mesh[:, 0]==azm_plot_min)[0][0]
		azm_max_row_index = np.where(attit_azm_mesh[:, 0]==azm_plot_max)[0][0] + 1 + 1

		#	find inc mesh cols of defined inc range
		inc_min_col_index = np.where(attit_inc_mesh[0, :]==inc_plot_min)[0][0]
		inc_max_col_index = np.where(attit_inc_mesh[0, :]==inc_plot_max)[0][0] + 1 + 1

		#	if range is across 0, split into two plot ranges
		if azimuth_across_zero is True:

			#	redefine two ranges
			azm_plot_min1 = 0
			azm_plot_max1 = azm_plot_min

			azm_plot_min2 = azm_plot_max
			azm_plot_max2 = 360

			#	find azm mesh rows of defined azm range (+1 for index, +1 to plot)
			azm_min_row_index1 = np.where(attit_azm_mesh[:, 0]==azm_plot_min1)[0][0]
			azm_max_row_index1 = np.where(attit_azm_mesh[:, 0]==azm_plot_max1)[0][0] + 1 + 1

			azm_min_row_index2 = np.where(attit_azm_mesh[:, 0]==azm_plot_min2)[0][0]
			azm_max_row_index2 = np.where(attit_azm_mesh[:, 0]==azm_plot_max2)[0][0] + 1 + 1

			#	apply filter range to azm mesh
			attit_azm_mesh1 = attit_azm_mesh[
				azm_min_row_index1:azm_max_row_index1,
				inc_min_col_index:inc_max_col_index
			]

			#	apply filter range to azm mesh
			attit_azm_mesh2 = attit_azm_mesh[
				azm_min_row_index2:azm_max_row_index2,
				inc_min_col_index:inc_max_col_index
			]

			#	apply filter range to inc mesh
			attit_inc_mesh1 = attit_inc_mesh[
				azm_min_row_index1:azm_max_row_index1,
				inc_min_col_index:inc_max_col_index
			]

			#	apply filter range to inc mesh
			attit_inc_mesh2 = attit_inc_mesh[
				azm_min_row_index2:azm_max_row_index2,
				inc_min_col_index:inc_max_col_index
			]

		else:

			#	apply filter range to azm mesh
			attit_azm_mesh = attit_azm_mesh[
				azm_min_row_index:azm_max_row_index,
				inc_min_col_index:inc_max_col_index
			]
	
			#	apply filter range to inc mesh
			attit_inc_mesh = attit_inc_mesh[
				azm_min_row_index:azm_max_row_index,
				inc_min_col_index:inc_max_col_index
			]

		#######################################################################################
		#	minimum rms plot
		#######################################################################################

		fig_rmsmin = plt.figure()
		
		#	subplots figure
		ax_rmsmin = plt.subplot(projection='polar')
		
		ax_rmsmin.set_theta_zero_location("N")
		ax_rmsmin.set_theta_direction(-1)
		
		ax_rmsmin.set_rticks(np.arange(0, 100, 10))
		ax_rmsmin.set_rlim(0, 100)
		
		####################################################
		#	plotting
		
		#	round to reasonable decimal places
		attit_min_rms_data = np.round(attit_min_rms_array, 10)
		attit_min_rms_data = np.vstack([attit_min_rms_data, np.full(attit_min_rms_data[-1, :].shape, 0)])
		
		attit_min_rms_data = np.hstack([
			attit_min_rms_data,
			np.reshape(
				np.full(attit_min_rms_data[:, -1].shape, 0),
				[len(attit_min_rms_data[:, -1]), 1]
			)
		])

		print('\n			Plotting Color Data...')

		if azimuth_across_zero is True:

			#	apply filter range to plot data
			attit_min_rms_data1 = attit_min_rms_data[
				azm_min_row_index1:azm_max_row_index1,
				inc_min_col_index:inc_max_col_index
			]

			#	apply filter range to plot data
			attit_min_rms_data2 = attit_min_rms_data[
				azm_min_row_index2:azm_max_row_index2,
				inc_min_col_index:inc_max_col_index
			]

			#	colour levels
			levels_data = np.vstack([attit_min_rms_data1, attit_min_rms_data2])
			levels_data = levels_data[np.nonzero(levels_data)]

			levels_min = levels_data.min()
			levels_max = levels_data.max()

			#	set color levels
			levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
			norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh1 - offset),
				attit_inc_mesh1,
				attit_min_rms_data1,
				cmap=cmap,
				norm=norm
			)
		
			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh2 - offset),
				attit_inc_mesh2,
				attit_min_rms_data2,
				cmap=cmap,
				norm=norm
			)

			#	plot position of minimum rms
			min1 = attit_min_rms_data1[np.nonzero(attit_min_rms_data1)].min()
			min2 = attit_min_rms_data2[np.nonzero(attit_min_rms_data2)].min()

			if min1 < min2:
				min_pos = np.where(attit_min_rms_data1==min1)

				inc_plot = attit_inc_mesh1[min_pos]
				azm_plot = attit_azm_mesh1[min_pos]

			else:
				min_pos = np.where(attit_min_rms_data2==min2)

				inc_plot = attit_inc_mesh2[min_pos]
				azm_plot = attit_azm_mesh2[min_pos]


			#	plot on figure
			ax_rmsmin.plot(
				np.radians(azm_plot),
				inc_plot + inc_offset, 
				marker='+',
				markersize=marker_size,
				color='k',
			)

			print('min_pos = ', min_pos)
			print('min_inc = ', inc_plot)
			print('min_azm = ', azm_plot)
		
		else:

			#	apply filter range to plot data
			attit_min_rms_data = attit_min_rms_data[
				azm_min_row_index:azm_max_row_index,
				inc_min_col_index:inc_max_col_index
			]

			#	colour levels
			levels_data = attit_min_rms_data
			levels_data = levels_data[np.nonzero(levels_data)]
			levels_min = levels_data.min()
			levels_max = levels_data.max()
			
			#	set color levels
			levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
			norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh - offset),
				attit_inc_mesh,
				attit_min_rms_data,
				cmap=cmap,
				norm=norm
			)

			#	plot position of minimum rms
			min1 = attit_min_rms_data[np.nonzero(attit_min_rms_data)].min()

			min_pos = np.where(attit_min_rms_data==min1)

			inc_plot = attit_inc_mesh[min_pos]
			azm_plot = attit_azm_mesh[min_pos]


			#	plot on figure
			ax_rmsmin.plot(
				np.radians(azm_plot),
				inc_plot + inc_offset, 
				marker='+',
				markersize=marker_size,
				color='k',
			)

			print('min_pos = ', min_pos)
			print('min_inc = ', inc_plot)
			print('min_azm = ', azm_plot)
		
		plt.colorbar()
		
		####################################################
		#	finalise plot
		
		#	add title
		ax_rmsmin.set_title('Minimum RMS Against Attitude', fontsize=title_fontsize, pad=title_pad)
		
		#	format x axis
		ax_rmsmin.set_xlabel('Azm', fontsize=xlabel_fontsize)
		ax_rmsmin.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
		
		#	format y axis
		ax_rmsmin.set_ylabel('Inc', fontsize=ylabel_fontsize)
		ax_rmsmin.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)


#		plt.show()
#		sys.exit()


		#######################################################################################
		#	maximum rms plot
		#######################################################################################

		fig_rmsmax = plt.figure()
		
		#	subplots figure
		ax_rmsmax = plt.subplot(projection='polar')
		
		ax_rmsmax.set_theta_zero_location("N")
		ax_rmsmax.set_theta_direction(-1)
		
		ax_rmsmax.set_rticks(np.arange(0, 100, 10))
		ax_rmsmax.set_rlim(0, 100)
		
		####################################################
		#	plotting
		
		#	round to reasonable decimal places
		attit_max_rms_data = np.round(attit_max_rms_array, 10)
		attit_max_rms_data = np.vstack([attit_max_rms_data, np.full(attit_max_rms_data[-1, :].shape, 0)])
		
		attit_max_rms_data = np.hstack([
			attit_max_rms_data,
			np.reshape(
				np.full(attit_max_rms_data[:, -1].shape, 0),
				[len(attit_max_rms_data[:, -1]), 1]
			)
		])
		
		print('\n			Plotting Color Data...')
		
		if azimuth_across_zero is True:

			#	apply filter range to plot data
			attit_max_rms_data1 = attit_max_rms_data[
				azm_min_row_index1:azm_max_row_index1,
				inc_min_col_index:inc_max_col_index
			]

			#	apply filter range to plot data
			attit_max_rms_data2 = attit_max_rms_data[
				azm_min_row_index2:azm_max_row_index2,
				inc_min_col_index:inc_max_col_index
			]

			#	colour levels
			levels_data = np.vstack([attit_max_rms_data1, attit_max_rms_data2])
			levels_data = levels_data[np.nonzero(levels_data)]
			levels_min = levels_data.min()
			levels_max = levels_data.max()
			
			#	set color levels
			levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
			norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh1 - offset),
				attit_inc_mesh1,
				attit_max_rms_data1,
				cmap=cmap,
				norm=norm
			)
		
			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh2 - offset),
				attit_inc_mesh2,
				attit_max_rms_data2,
				cmap=cmap,
				norm=norm
			)
		
		else:

			#	apply filter range to plot data
			attit_max_rms_data = attit_max_rms_data[
				azm_min_row_index:azm_max_row_index,
				inc_min_col_index:inc_max_col_index
			]
	
			#	colour levels
			levels_data = attit_max_rms_data
			levels_data = levels_data[np.nonzero(levels_data)]
			levels_min = levels_data.min()
			levels_max = levels_data.max()
			
			#	set color levels
			levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
			norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh - offset),
				attit_inc_mesh,
				attit_max_rms_data,
				cmap=cmap,
				norm=norm
			)
		
		plt.colorbar()
		
		####################################################
		#	finalise plot
		
		#	add title
		ax_rmsmax.set_title('Maximum RMS Against Attitude', fontsize=title_fontsize, pad=title_pad)
		
		#	format x axis
		ax_rmsmax.set_xlabel('Azm', fontsize=xlabel_fontsize)
		ax_rmsmax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
		
		#	format y axis
		ax_rmsmax.set_ylabel('Inc', fontsize=ylabel_fontsize)
		ax_rmsmax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

		#######################################################################################
		#	average rms plot
		#######################################################################################

		fig_rmsavg = plt.figure()
		
		#	subplots figure
		ax_rmsavg = plt.subplot(projection='polar')
		
		ax_rmsavg.set_theta_zero_location("N")
		ax_rmsavg.set_theta_direction(-1)
		
		ax_rmsavg.set_rticks(np.arange(0, 100, 10))
		ax_rmsavg.set_rlim(0, 100)
		
		####################################################
		#	plotting
		
		#	round to reasonable decimal places
		attit_avg_rms_data = np.round(attit_avg_rms_array, 10)
		attit_avg_rms_data = np.vstack([attit_avg_rms_data, np.full(attit_avg_rms_data[-1, :].shape, 0)])
		
		attit_avg_rms_data = np.hstack([
			attit_avg_rms_data,
			np.reshape(
				np.full(attit_avg_rms_data[:, -1].shape, 0),
				[len(attit_avg_rms_data[:, -1]), 1]
			)
		])
		
		print('\n			Plotting Color Data...')
			
		if azimuth_across_zero is True:

			#	apply filter range to plot data
			attit_avg_rms_data1 = attit_avg_rms_data[
				azm_min_row_index1:azm_max_row_index1,
				inc_min_col_index:inc_max_col_index
			]

			#	apply filter range to plot data
			attit_avg_rms_data2 = attit_avg_rms_data[
				azm_min_row_index2:azm_max_row_index2,
				inc_min_col_index:inc_max_col_index
			]

			#	colour levels
			levels_data = np.vstack([attit_avg_rms_data1, attit_avg_rms_data2])
			levels_data = levels_data[np.nonzero(levels_data)]
			levels_min = levels_data.min()
			levels_max = levels_data.max()
			
			#	set color levels
			levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
			norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh1 - offset),
				attit_inc_mesh1,
				attit_avg_rms_data1,
				cmap=cmap,
				norm=norm
			)
		
			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh2 - offset),
				attit_inc_mesh2,
				attit_avg_rms_data2,
				cmap=cmap,
				norm=norm
			)
		
		else:

			#	apply filter range to plot data
			attit_avg_rms_data = attit_avg_rms_data[
				azm_min_row_index:azm_max_row_index,
				inc_min_col_index:inc_max_col_index
			]
	
			#	colour levels
			levels_data = attit_avg_rms_data
			levels_data = levels_data[np.nonzero(levels_data)]
			levels_min = levels_data.min()
			levels_max = levels_data.max()
			
			#	set color levels
			levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
			norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh - offset),
				attit_inc_mesh,
				attit_avg_rms_data,
				cmap=cmap,
				norm=norm
			)
		
		plt.colorbar()
		
		####################################################
		#	finalise plot
		
		#	add title
		ax_rmsavg.set_title('Average RMS against Attitude', fontsize=title_fontsize, pad=title_pad)
		
		#	format x axis
		ax_rmsavg.set_xlabel('Azm', fontsize=xlabel_fontsize)
		ax_rmsavg.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
		
		#	format y axis
		ax_rmsavg.set_ylabel('Inc', fontsize=ylabel_fontsize)
		ax_rmsavg.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

		#######################################################################################
		#	std dev rms plot
		#######################################################################################

		fig_rmsstd = plt.figure()
		
		#	subplots figure
		ax_rmsstd = plt.subplot(projection='polar')
		
		ax_rmsstd.set_theta_zero_location("N")
		ax_rmsstd.set_theta_direction(-1)
		
		ax_rmsstd.set_rticks(np.arange(0, 100, 10))
		ax_rmsstd.set_rlim(0, 100)
		
		####################################################
		#	plotting
		
		#	round to reasonable decimal places
		attit_std_rms_data = np.round(attit_std_rms_array, 10)
		attit_std_rms_data = np.vstack([attit_std_rms_data, np.full(attit_std_rms_data[-1, :].shape, 0)])
		
		attit_std_rms_data = np.hstack([
			attit_std_rms_data,
			np.reshape(
				np.full(attit_std_rms_data[:, -1].shape, 0),
				[len(attit_std_rms_data[:, -1]), 1]
			)
		])
		
		print('\n			Plotting Color Data...')
			
		if azimuth_across_zero is True:

			#	apply filter range to plot data
			attit_std_rms_data1 = attit_std_rms_data[
				azm_min_row_index1:azm_max_row_index1,
				inc_min_col_index:inc_max_col_index
			]

			#	apply filter range to plot data
			attit_std_rms_data2 = attit_std_rms_data[
				azm_min_row_index2:azm_max_row_index2,
				inc_min_col_index:inc_max_col_index
			]

			#	colour levels
			levels_data = np.vstack([attit_std_rms_data1, attit_std_rms_data2])
			levels_data = levels_data[np.nonzero(levels_data)]
			levels_min = levels_data.min()
			levels_max = levels_data.max()
			
			#	set color levels
			levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
			norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh1 - offset),
				attit_inc_mesh1,
				attit_std_rms_data1,
				cmap=cmap,
				norm=norm
			)
		
			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh2 - offset),
				attit_inc_mesh2,
				attit_std_rms_data2,
				cmap=cmap,
				norm=norm
			)

		else:

			#	apply filter range to plot data
			attit_std_rms_data = attit_std_rms_data[
				azm_min_row_index:azm_max_row_index,
				inc_min_col_index:inc_max_col_index
			]
			
			#	colour levels
			levels_data = attit_std_rms_data
			levels_data = levels_data[np.nonzero(levels_data)]
			levels_min = levels_data.min()
			levels_max = levels_data.max()
			
			#	set color levels
			levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
			norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)

			#	plot mesh
			plt.pcolor(
				np.radians(attit_azm_mesh - offset),
				attit_inc_mesh,
				attit_std_rms_data,
				cmap=cmap,
				norm=norm
			)
		
		plt.colorbar()
		
		####################################################
		#	finalise plot
		
		#	add title
		ax_rmsstd.set_title(
			'RMS Standard Deviation against Attitude',
			fontsize=title_fontsize,
			pad=title_pad
		)
		
		#	format x axis
		ax_rmsstd.set_xlabel('Azm', fontsize=xlabel_fontsize)
		ax_rmsstd.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
		
		#	format y axis
		ax_rmsstd.set_ylabel('Inc', fontsize=ylabel_fontsize)
		ax_rmsstd.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)








		plt.show()
		sys.exit()

#		###################################################################################
#		#	finding minima
#
#		local_minima_locations = detect_local_minima(track_rms_array)
#		minima_array = np.array(local_minima_locations)
#
#		print(local_minima_locations)
#		print(minima_array)
#		
#		print('\n\n\n\nFinding:')
#		print(minima_array[:, 0])
#
#		#	looping through minima locations
#		for col_index in range(len(minima_array[:, 0])):
#
#			min_coord = minima_array[:, col_index]
#
#			print('Min Coords = ', min_coord)
#
#			d_brdf = d_brdf_range[min_coord[2]]
#
#			print('d_brdf value = ', d_brdf)
#
#			plot_coord_x, plot_coord_y = coords_array[min_coord[2]]
#
#			omega_coord = omega_brdf_range[min_coord[0]] + omega_offset
#			m_coord = m_brdf_range[min_coord[1]] + m_offset
#
#			#	plot on figure
#			ax[plot_coord_x, plot_coord_y].plot(
#				omega_coord, m_coord,
#				marker='+',
#				markersize=5,
#				color='r',
#			)
#
#		plt.show()
#		sys.exit()


#		###################################################################################
#		#	save figures
#		
#		if savefile_flag is True:
#		
#			fig3_name = 'starlink_brdf_determination_plot.eps'
#			out_path3 = os.path.join(fig_path, fig3_name)
#			fig3.savefig(out_path3, bbox_inches="tight", dpi=300)
#		
#		#	fig_name = 'starlink_brdf_determination_stack.eps'
#		#	out_path = os.path.join(fig_path, fig_name)
#		#	fig.savefig(out_path, bbox_inches="tight", dpi=300)
#		
#			print('\n#######################################')
#		
#		plt.show()
#		sys.exit()
