###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

from astropy.timeseries import LombScargle
from PyAstronomy.pyTiming import pyPDM
from PyAstronomy.pyasl import foldAt

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

from scipy import fftpack

#	parrellisation
from starlink_para_funs import rocket_body_combined_fitting
from multiprocessing import Pool
import tqdm

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Rocket Body Combined Attitude + BRDF Fitting.')
print('			CASE: int-10')
print('#####################################################################################')

#	set start time
sim_start = time.time()
init_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	MMT file read path
read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'rocket_body')

###################################################################################################
#	simulation inputs

#	set time step
time_step = 0.1

#	set time windows
window_start_date = [2010, 1, 1, 0, 0, 0]
window_stop_date = [2019, 11, 1, 0, 0, 0]

#	define specific mmt file
specific_file_flag = True
specific_file = 'CZ-2D_41858.txt'

#	define track id
specific_track_flag = True 
specific_track = 12047965

parral_flag = False
parral_flag = True

ncores = 16

###################################################################################################
#	fitting inputs

#	geometry zone specification string
brdf_str = 'full'

#	set range of d_brdf values to plot
d_brdf_range = [0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8]

#	range for brdf parameters
m_brdf_min = 0.1
m_brdf_max = 0.9
m_brdf_step = 0.1
m_brdf_range = np.arange(m_brdf_min, m_brdf_max + m_brdf_step, m_brdf_step)

omega_brdf_min = 0.1
omega_brdf_max = 0.9
omega_brdf_step = 0.1
omega_brdf_range = np.arange(omega_brdf_min, omega_brdf_max + omega_brdf_step, omega_brdf_step)

#	range of attiude angles
rot_vec_azm_deg_range = np.arange(0, 360, 10)
rot_vec_inc_deg_range = np.arange(0, 100, 10)

###################################################################################################
#	light curve generation inputs

#	set initial BRDF values
top_omega_brdf = 0.2
top_m_brdf = 0.7
top_d_brdf = 0.7

bot_omega_brdf = 0.2
bot_m_brdf = 0.7
bot_d_brdf = 0.7

side_omega_brdf = 0.2
side_m_brdf = 0.7
side_d_brdf = 0.7

#	geometry zone face indices
top_faces_index = np.arange(0, 36)
side_faces_index = np.arange(36, 108)
bot_faces_index = np.arange(108, 144)

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'CZ-2D'
geom_str = 'cz-2d_testing_double_pointed'
geom_file = geom_str + '.stl'

#	attitude
rot_vec_azm_deg = 350.0
rot_vec_inc_deg = 60.0

#	attitude motion
rot_period = 9.2

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	load geometry

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

###################################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\nInitialisation Finished.')
print('\n	- Time taken = {} s.'.format(init_time))

###################################################################################################
#	loop through MMT files
###################################################################################################
print('\nLooping Through Rocket Body Light Curve Files...')

#	set input directory
rocket_body_dir = os.listdir(read_path)

if specific_file_flag is True:
	rocket_body_dir = [specific_file]

#	loop through starlink IDs
for loop_itt, mmt_file in enumerate(rocket_body_dir):

	#	parse filename
	name_str = mmt_file.split('_')[0]
	norad_id = int(mmt_file.split('-')[1].split('_')[1][:-4])

	print('\n	#############################################################################')
	print('	Plotting data for: {}, {}'.format(name_str, norad_id))

	###############################################################################################
	#	load MMT file
	
	#	set time
	mmt_load_start = time.time()
	
	print('\n		[{}] Loading MMT data...'.format(datetime.datetime.now()))
	
	with open(os.path.join(read_path, mmt_file)) as read_file:
		mmt_data = read_file.readlines()

	###############################################################################################
	#	parse MMT file
	
	#	strip header
	header_text = mmt_data[7]
	raw_data = mmt_data[8:]
	
	print('\n		Parsing MMT data...')
	
	#	initialise output
	n_rows = 14
	parsed_data = np.zeros([len(raw_data), n_rows])
	
	for i, line in enumerate(raw_data):
	
		#	split line
		date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
			distance, phase, channel, track_id = line.split(' ')
	
		#	parse data and set data types
		mag_std = float(mag_std)
		mag_app = float(mag_app)
		distance = float(distance)
		phase = float(phase)
		track_id = int(track_id)
	
		#	parse date
		year, month, day = date_str.split('-')
		year = int(year)
		month = int(month)
		day = int(day)
	
		#	parse time
		hour, minute, second = time_str.split(':')
		hour = int(hour)
		minute = int(minute)
		second = np.round(float(second), 1)
	
		#	calculate julian date
		jday, jfrac = date2jd(year, month, day, hour, minute, second)
	
		#	generate output line
		output_line = [
			track_id, norad_id,
			jday, jfrac,
			year, month, day,
			hour, minute, second,
			mag_std, mag_app,
			distance, phase
		]
	
		#	add to output array
		parsed_data[i, :] = output_line
	
	print('			Total data length = ', len(parsed_data))
	print('			Total number of objects = ', len(np.unique(parsed_data[:, 1])))
	print('			Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

	###############################################################################################
	#	extracting data between specified windows
	
	print('\n		Seaching for tracks inside time window:')
	
	#	start window
	window_start_jd = np.add(*date2jd(*window_start_date))
	print('\n			Window start 	= ', window_start_date[0:3])
	
	#	stop window
	window_stop_jd = np.add(*date2jd(*window_stop_date))
	print('			Window stop 	= ', window_stop_date[0:3])
	
	#	extract track ids
	track_id_list = np.unique(parsed_data[:, 0])

	#	loop through tracks
	ntracks = 0
	first_lc_flag = True
	for track_id in track_id_list:
	
		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]
	
		#	extract light curve data
		jd_plot = np.add(track_data[:, 2], track_data[:, 3])
		std_mag_plot = track_data[:, 10]
		app_mag_plot = track_data[:, 11]
		distance_plot = track_data[:, 12]
		phase_plot = track_data[:, 13]
		start_line = track_data[0]
		stop_line = track_data[-1]
		start_jdate = np.add(start_line[2], start_line[3])
		stop_jdate = np.add(stop_line[2], stop_line[3])
	
		#	ensure between window start/stop
		if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
	
			#	light curve is within window
			ntracks += 1
	
			#	if first light curve, start array
			if first_lc_flag is True:
				windowed_data = track_data
				first_lc_flag = False
	
			#	else append to list
			elif first_lc_flag is False:
				windowed_data = np.vstack([windowed_data, track_data])
	
	print('\n			{} track(s) extracted.'.format(ntracks))

	#	reset track_id_list for windowed data
	track_id_list = np.unique(windowed_data[:, 0])

	###############################################################################################
	#	display initialisation time
	mmt_load_end = time.time()
	mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)
	
	print('\n		MMT Load Finished.')
	print('\n			- Time taken = {} s.'.format(mmt_load_time))

	###############################################################################################
	#	main track loop
	###############################################################################################

	#	loop initialisation
	first_track = True
	track_color_index = 0
	tracks_plotted = 0

	if specific_track_flag is True: 
		track_id_list = [specific_track]

	print('\n		Looping Through Tracks...')

	#	loop through tracks
	for track_itt, track_id in enumerate(track_id_list):

		#	convert to int
		track_id = int(track_id)
	
		print('\n			#############################################################')
		print('			Working on Track {}, ID: {}'.format(track_itt, track_id))

		###########################################################################################
		#	output directory

		#	create object directory
		object_dir = name_str + '_{}'.format(norad_id)
		output_path = os.path.join(path_data, 'lightcurve_dir', 'rocket_body_analysis', object_dir)

		if not os.path.exists(output_path):
			print('\n			Creating Directory: ', output_path)
			os.makedirs(output_path)

		#	create track directory
		track_dir = 'track-{}'.format(track_id)
		output_path = os.path.join(output_path, track_dir)

		#	create object directory
		if not os.path.exists(output_path):
			print('\n			Creating Directory: ', output_path)
			os.makedirs(output_path)

		###########################################################################################
		#	extracting requested single track
		
		print('\n				Extracting requested track index...')
		
		#	extract track data
		track_indices = np.where(windowed_data[:, 0]==track_id)[0]
		track_data = windowed_data[track_indices]

		#######################################################################################
		#	extract track time data
		
		mmt_line = track_data[0]

		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]
		
		###################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
		sim_range_sec = np.round(
			np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400),
			1
		)

		#######################################################################################
		#	ensure time continuity
		
		print('\n				Ensuring time continuity in extracted data...')
		
		track_data_temp = []
		sim_index = 0
	
		for itt, line in enumerate(track_data):
		
			#	parse line data
			line_track_id, line_norad_id,\
				jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = line
			
			step_jd = jday + jfrac
		
			#	calculate current seconds
			current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
		
			output_line = [
				jday, jfrac, current_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]
		
			##################################################
			#	extracting data of time step
		
			#	data match found
			if current_sec == sim_range_sec[sim_index]:
				track_data_temp.append(output_line)
		
				#	itterate number of output data points
				sim_index += 1
		
				#	save current line as previous line
				prev_line = output_line
		
			#	data point is not required
			elif current_sec < sim_range_sec[sim_index]:
				pass
		
			#	interpolation required
			elif current_sec > sim_range_sec[sim_index]:
		
				#	parse previous line
				jday_prev, jfrac_prev, prev_sec, \
					year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
					mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
		
				#	calculate interpolation requirements
				gap = np.round(current_sec - prev_sec, 1)
				npts = int(gap / time_step)
		
				#	get current light curve data
				mag_std_last = mag_std
				mag_app_last = mag_app
				distance_last = distance
				phase_last = phase
		
				#	calculate linear interpolation step values
				mag_std_step = (mag_std_last - mag_std_prev) / npts
				mag_app_step = (mag_app_last - mag_app_prev) / npts
				distance_step = (distance_last - distance_prev) / npts
				phase_step = (phase_last - phase_prev) / npts
		
				#	calculate interpolation data
				for n in range(1, npts + 1):
		
					#	current simulation second
					interp_sec = np.round(prev_sec + (n * time_step), 1)

					#	time data
					jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
					jday, jfrac = jday_jfrac(jdate)
					year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
					#	calculate interpolated light curve data
					mag_std = mag_std_prev + (n * mag_std_step)
					mag_app = mag_app_prev + (n * mag_app_step)
					distance = distance_prev + (n * distance_step)
					phase = phase_prev + (n * phase_step)
		
					#	generate new output line
					interp_output_line = [
						jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
		
					if interp_sec == sim_range_sec[sim_index]:
						track_data_temp.append(interp_output_line)
						sim_index += 1
		
				if interp_sec != current_sec:
					#	append final line
					track_data_temp.append(output_line)
					sim_index += 1
		
			else:
				print('ERROR')
				sys.exit()
		
			#	save previous line
			if itt != 0:
				prev_line = output_line
		
			#	array is complete condition
			if len(track_data_temp) == lc_len_record:
				break
		
		#	ensure parsed MMT data matches simulation data
		track_data = np.array(track_data_temp)
		
		if len(track_data) == len(lc_sim_range):
			print('					- Simulation and MMT data matches.')
		else:
			print('					- Error: Simulation and MMT data does not match...Exiting.')
			sys.exit()
		
		###########################################################################################
		#	extract mmt data

		jd_plot = np.add(track_data[:, 0], track_data[:, 1])
		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]

		###########################################################################################
		#	simulation printout
		###########################################################################################
		
		print('\n			#############################################################')
		print('			MMT light curve parameters: ')
		print('				Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(
			*lc_start_date
		))
		print('				Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(
			*lc_stop_date
		))
		print('				Simulation time step:		{}'.format(time_step))
		print('				Duration:			{}		[steps]'.format(lc_len_record))
		print('								{}		[s]'.format(np.round(lc_duration_sec, 1)))

		###########################################################################################
		#	setting simulation time data
		###########################################################################################
		
		sim_range = lc_sim_range
		len_record = lc_len_record
		start_date = lc_start_date
		stop_date = lc_stop_date
		duration_sec = lc_duration_sec

		#	calculate sim time range in seconds
		sim_time = np.linspace(0, duration_sec, len_record)

		###########################################################################################
		#	generate eop data arrays
		###########################################################################################
		
		#	set time
		eop_start = time.time()
		
		print('\n			[{}] Generating EOP data array...'.format(datetime.datetime.now()))
		
		#	EOP data
		eop_dir = 'eop_files'
		eop_file = 'eop_parsed.csv'
		dat_file = 'dat_parsed.csv'
		
		#	assemble eop path
		eop_path = os.path.join(path_data, eop_dir)
		
		#	generate interpolated eop_data
		eop_log_level = 0
		eop_data, dat_data = eop_interpolate(
			eop_path,
			eop_file,
			dat_file,
			sim_range,
			time_step,
			eop_log_level
		)
		
		###########################################################################################
		#	display time
		eop_end = time.time()
		eop_time = np.round((eop_end - eop_start), 1)
		
		print('\n					EOP data generated.')
		print('\n						- Time taken = {} s.'.format(eop_time))
		
		###########################################################################################
		#	generate time objects
		###########################################################################################
		
		#	start dat value
		dat = dat_data[0, -1]
			
		#	start eop values
		eop_line = eop_data[0, :]
		eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, \
			_, lod, _, dX, _, dY, _ = eop_line
			
		#	generate start time object
		start_time_object = get_times(*start_date, ut_offset, dat)
			
		#	end dat value
		dat = dat_data[-1, -1]
			
		#	end eop values
		eop_line = eop_data[-1, :]
		eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, \
			_, lod, _, dX, _, dY, _ = eop_line
			
		#	generate stop time object
		stop_time_object = get_times(*stop_date, ut_offset, dat)
		
		###########################################################################################
		#	generate rotation matrices
		###########################################################################################
		
		#	set time
		rot_mat_start = time.time()
		print('\n			[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))
		
		#	tt time
		jdaytt_start = start_time_object.jday_tt
		jfractt_start = start_time_object.jfrac_tt
		jdtt_start = jdaytt_start + jfractt_start
		
		#	ut1 time
		jdayut1_start = start_time_object.jday_ut1
		jfracut1_start = start_time_object.jfrac_ut1
		jdut1_start = jdayut1_start + jfracut1_start
		
		#	generate rotation matrices
		rot_mat_log_level = 0
		pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
			jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
		)
		
		###########################################################################################
		#	display time
		rot_mat_end = time.time()
		rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)
		
		print('\n				Rotation Matrices Generated.')
		print('\n					- Time taken = {} s.'.format(rot_mat_time))
		
		###########################################################################################
		#	MODULE:	Horizons Positions
		###########################################################################################
		
		#	set time
		horz_start = time.time()
		print('\n			[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))
		
		#	set horizons logging level
		horz_logging_level = 0
		
		sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
			path_data, horz_logging_level,
			start_time_object, stop_time_object,
			len_record, time_step,
			eop_path
		)
		
		#	find indices of light curve data
		timecol = sun_icrf[:, 0]
		lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]
		
		###########################################################################################
		#	display time
		horz_end = time.time()
		horz_time = np.round((horz_end - horz_start), 1)
		
		print('\n				Horizons data generated.')
		print('\n					- Time taken = {} s.'.format(horz_time))
		
		###########################################################################################
		#	MODULE:	Object Position
		###########################################################################################
		
		#	set time
		sat_start = time.time()
		print('\n			[{}]	Generating object position data...'.format(
			datetime.datetime.now()
		))
		
		satpos_logging_level = 0
		
		sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
			path_data, satpos_logging_level, len_record, time_step,
			start_time_object, stop_time_object,
			norad_id,
			eop_data, dat_data,
			pm_array, pn_array, st_array, c2t_array
		)
		
		##########################################################################################
		#	display time
		sat_end = time.time()
		sat_time = np.round((sat_end - sat_start), 1)
		
		print('\n				Object position data generated.')
		print('\n					- Time taken = {} s.'.format(sat_time))
		
		##########################################################################################
		#	MODULE:	Transformation
		##########################################################################################
		
		#	set time
		trans_start = time.time()
		print('\n			[{}]	Performing transformations...'.format(datetime.datetime.now()))
		
		identification_str = 'mmt-render'
		
		trans_logging_level = 0
		save_flag = False
		
		sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
			= module_transformation(
				path_data, trans_logging_level, save_flag,
				start_time_object, stop_time_object,
				identification_str, site_str,
				len_record, time_step,
				eop_data, dat_data,
				pm_array, pn_array, st_array, c2t_array,
				sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
			)
		
		##########################################################################################
		#	display time
		trans_end = time.time()
		trans_time = np.round((trans_end - trans_start), 1)
		
		print('\n				Transformations complete.')
		print('\n					- Time taken = {} s.'.format(trans_time))

		###########################################################################################
		#	perform fitting
		###########################################################################################
		
		print('\n			#############################################################')
		print('			Performing Fitting...')

		print('\n			Required number of simulations:')

		print(
			'				Number of Attitude Azimuth Runs = ',
			len(rot_vec_azm_deg_range)
		)

		print(
			'\n				Runs per Attitude Inclination = ',
			len(rot_vec_azm_deg_range) * \
			len(d_brdf_range) * \
			len(m_brdf_range) * \
			len(omega_brdf_range)
		)

		print(
			'				Runs per Attitude Azimuth = ',
			len(d_brdf_range) * \
			len(m_brdf_range) * \
			len(omega_brdf_range)
		)

		print(
			'\n					Total Runs = ',
			len(rot_vec_azm_deg_range) * \
			len(d_brdf_range) * \
			len(m_brdf_range) * \
			len(omega_brdf_range)
		)

#		print('\n				#####################################################')
#		print('				Working on Inclination = ', rot_vec_inc_deg)

		###################################################################################
		#	generate an array of inputs
		
		print('\n				Generating arary of inputs...')

		#	looping through attitude inclination 
		for rot_vec_inc_deg in rot_vec_inc_deg_range:

			#	looping through attitude azmimuth
			for rot_vec_azm_deg in rot_vec_azm_deg_range:

				print('\n				Working on Attitude State: [{}, {}]'.format(
					rot_vec_inc_deg, rot_vec_azm_deg
				))


				#	generate output filename
				out_file = '{}-{}_track-{}_inc-{}_azm-{}.txt'.format(
					name_str,
					int(norad_id),
					track_id,
					int(rot_vec_inc_deg),
					int(rot_vec_azm_deg)
				)

				#	generate attitude state directory
				attit_dir = 'inc-{}_azm-{}'.format(
					int(rot_vec_inc_deg),
					int(rot_vec_azm_deg)
				)

				full_output_path = os.path.join(output_path, attit_dir)

				if not os.path.exists(full_output_path):
					print('\n		Creating Attitude Directory: ', full_output_path)
					os.makedirs(full_output_path)

				inputs = []

				#	looping on d brdf values
				for d_brdf in d_brdf_range:
				
#					print('\n		#############################################################')
#					print('		Working on slice:	d_brdf = ', d_brdf)
#				
#					#	generate subdirectory
#					d_brdf_dir = 'd_brdf=0-{:02d}'.format(int(d_brdf * 100))
#					
#					temp_output_path = os.path.join(attit_output_path, d_brdf_dir)
#
#					if not os.path.exists(temp_output_path):
#						print('\n		Creating Directory: ', temp_output_path)
#						os.makedirs(temp_output_path)
#
#					#	generate an array of inputs
#					inputs = []
					#	looping through brdf values
					for omega_brdf in omega_brdf_range:
						for m_brdf in m_brdf_range:
	
							d_brdf = np.round(d_brdf, 2)
							m_brdf = np.round(m_brdf, 2)
							omega_brdf = np.round(omega_brdf, 2)
	
							if brdf_str == 'full':
								top_d_brdf = d_brdf
								top_m_brdf = m_brdf
								top_omega_brdf = omega_brdf
	
								bot_d_brdf = d_brdf
								bot_m_brdf = m_brdf
								bot_omega_brdf = omega_brdf
	
								side_d_brdf = d_brdf
								side_m_brdf = m_brdf
								side_omega_brdf = omega_brdf
	
							else:
	
								if brdf_str == 'top':
									top_d_brdf = d_brdf
									top_m_brdf = m_brdf
									top_omega_brdf = omega_brdf
		
								elif brdf_str == 'bot':
									bot_d_brdf = d_brdf
									bot_m_brdf = m_brdf
									bot_omega_brdf = omega_brdf
		
								elif brdf_str == 'side':
									side_d_brdf = d_brdf
									side_m_brdf = m_brdf
									side_omega_brdf = omega_brdf
	
							#	add line to inputs
							inputs.append([
								path_data, name_str, track_id,
								rot_vec_azm_deg, rot_vec_inc_deg,
								rot_period, orbital_mot,
								top_faces_index, bot_faces_index, side_faces_index,
								top_d_brdf, top_m_brdf, top_omega_brdf, 
								bot_d_brdf, bot_m_brdf, bot_omega_brdf,
								side_d_brdf, side_m_brdf, side_omega_brdf,
								duration_sec, time_step, len_record, start_jdate,
								sat_orb, sun_orb, earth_orb, site_orb,
								body_vectors, body_areas
							])
	
				#	convert to numpy array
				inputs = np.array(inputs)
		
				print('\n					- Number of Input Rows = ', len(inputs))

				###################################################################################
				#	run fitting function
			
				slice_start = time.time()
				full_time_before = time.time()
			
				if parral_flag is True:
					print('\n			Running Parrallised Function...')
					
					#	initialise multiproccessing pool of workers
					p = Pool(ncores)
				
					total_lightcurve_data = []
					
					#	map the total data array across the access function
					for light_curve_output in tqdm.tqdm(
						p.imap_unordered(rocket_body_combined_fitting, inputs), total=len(inputs)
						):
					
						#	parse outputs
						total_lightcurve_data.append(light_curve_output)
			
				elif parral_flag is False:
					print('\n				Running Sequentially...')
				
					#	run inputs sequentially
					for input_line in inputs:
			
						#	get time before entering fitting function
						time_before = time.time()
			
						#	combined fitting function
						light_curve_output = rocket_body_combined_fitting(input_line)
			
						#	extract apparent magnitude data
						lc_app = light_curve_output[:, 2]
			
						time_after = time.time()
						print('\n					- Function Time = ', time_after - time_before)
			
						run_times.append(time_after - time_before)
			
						#	extract light curve generation inputs
						rot_vec_inc_deg = light_curve_output[0, 6]
						rot_vec_azm_deg = light_curve_output[0, 5]
			
						top_d_brdf, top_m_brdf, top_omega_brdf = light_curve_output[0, 7:10]
						bot_d_brdf, bot_m_brdf, bot_omega_brdf = light_curve_output[0, 10:13]
						side_d_brdf, side_m_brdf, side_omega_brdf = light_curve_output[0, 13:16]
			
				print('\n			Data Generated...')
			
				full_time_after = time.time()
				full_time = full_time_after - full_time_before
			
				print('\n					- Function Time = ', full_time)
	
				###############################################################################
				#	save slice file
				
				#	open output file
				with open(os.path.join(full_output_path, out_file), 'a') as write_file:
					print('\n		Writing to: ', os.path.join(full_output_path, out_file))
				
					#	write each light curve to total light curve file
					for light_curve_index, light_curve_array in enumerate(total_lightcurve_data):
				
						#	generate index array
						index_array = np.full([len(light_curve_array[:, 0]), 1], light_curve_index)
				
						#	stack light curve index array with data
						output_array = np.hstack([index_array, light_curve_array])
				
						#	save light curve data
						np.savetxt(write_file, output_array, delimiter=',')
				
						#	add newline character
						write_file.write('\n')
				
					print('\n			File Saved...')
	
				#	calculate slice time
				print('\n			Slice Complete.')
				
				slice_end = time.time()
				slice_time = np.round((slice_end - slice_start) / 60, 1)
				
				print('\n			[{}] Slice Complete.'.format(datetime.datetime.now()))
				print('\n				- Time taken = {} mins.'.format(slice_time))

				if rot_vec_azm_deg > 20:
					sys.exit()

###############################################################################################
#	calculate run time
###############################################################################################
#	calculate sim time
sim_end = time.time()
sim_time = np.round((sim_end - sim_start) / 60, 1)

print('\n[{}] Simulation Finished.'.format(datetime.datetime.now()))
print('\n	- Time taken = {} mins.'.format(sim_time))

plt.show()
sys.exit()

