###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

from astropy.timeseries import LombScargle
from PyAstronomy.pyTiming import pyPDM
from PyAstronomy.pyasl import foldAt

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

from scipy import fftpack

# #	parrellisation
# from starlink_para_funs import rocket_body_brdf_fitting
# from multiprocessing import Pool
# import tqdm

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()
init_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'rocket_body')

#	set time windows
window_start_date = [2010, 11, 1, 0, 0, 0]

window_stop_date = [2020, 12, 1, 0, 0, 0]

#	set time step
time_step = 0.1

specific_file_flag = False
specific_file_flag = True

specific_file = 'N-1_10675.txt'
specific_file = 'DELTA-2_25637.txt'
specific_file = 'CZ-2D_41858.txt'

#	plot only tracks for which period data is available
period_tracks_flag = True

#	plot only a single specific track ID
specific_track_flag = False
specific_track_flag = True
specific_track = 10833133

#	calculate and plot a period
calc_period_flag = True
calc_period_flag = False

plot_period_flag = True
plot_period_flag = False

#	generate a synthetic light curve
gen_synth_lc = True

#	PDM parameters
min_freq = 4.0
max_freq = 10.0
delta_freq = 0.01

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

###################################################################################################
#	initialise figure
###################################################################################################

print('\n#####################################################################################')
print('Generating Final Plots...')

#	graphical output
inch2cm = 2.54

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4
xlabel_fontsize = 6
ylabel_fontsize = 6
xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set marker parameters
marker_size = 3

#	set figure line wdith
plot_linewidth = 0.5
 
#	set use of LaTeX font
plt.rc('text', usetex=True)

#	set figure height/width
fig_width = 9.0/inch2cm
fig_height = 18.0/inch2cm

#	initialise figure axes
fig, ax = plt.subplots(3, 1, figsize=(fig_width, fig_height), dpi=300)
plt.gcf().subplots_adjust(bottom=0.15)

ax[0].set_xlabel(r'$Time/s$', fontsize=ylabel_fontsize)
ax[0].set_ylabel(r'$App\ Mag$', fontsize=xlabel_fontsize)
ax[0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax[1].set_xlabel(r'$Date/jd$', fontsize=ylabel_fontsize)
ax[1].set_ylabel(r'$App\ Mag$', fontsize=xlabel_fontsize)
ax[1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

#	reverse magnitude axis
ax[1].invert_yaxis()
ax[1].invert_yaxis()
		
ax[2].set_xlabel(r'$Date/jd$', fontsize=ylabel_fontsize)
ax[2].set_ylabel(r'$Period/s$', fontsize=xlabel_fontsize)
ax[2].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[2].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

###################################################################################################
#	loop through MMT files
###################################################################################################
print('\n	Looping Through Rocket Body Light Curve Files...')

#	set input directory
rocket_body_dir = os.listdir(read_path)

if specific_file_flag is True:
	rocket_body_dir = [specific_file]

#	loop through starlink IDs
for loop_itt, mmt_file in enumerate(rocket_body_dir):

	print('\n	MMT file is: ', mmt_file)

	if '_periods' in mmt_file:
		continue

	#	parse filename
	name_str = mmt_file.split('_')[0]

	if '-' in mmt_file:
		norad_id = int(mmt_file.split('-')[-1].split('_')[-1][:-4])

	else:
		norad_id = int(mmt_file.split('_')[-1][:-4])

	print('\n		#####################################################################')
	print('		Plotting data for: {}, {}'.format(name_str, norad_id))

	###############################################################################################
	#	load MMT file
	
	#	set time
	mmt_load_start = time.time()
	
	print('\n	[{}] Loading MMT data...'.format(datetime.datetime.now()))
	
	with open(os.path.join(read_path, mmt_file)) as read_file:
		mmt_data = read_file.readlines()

	###############################################################################################
	#	load MMT periods file
	
	#	set time
	mmt_load_start = time.time()
	
	mmt_periods_file = mmt_file[:-4] + '_periods.txt'
	print('\n		Loading MMT periods datafile: ', mmt_periods_file)

	with open(os.path.join(read_path, mmt_periods_file)) as read_file:
		mmt_periods_data = read_file.readlines()

	mmt_period_data = []

	#	extract periodicity data
	for period_line in mmt_periods_data[7:]:

		#	parse line
		period_date, period_time, period_track_id, period_mjd, period = period_line.strip().split()

		#	parse date and time
		year, mon, day = [int(i) for i in period_date.split('-')]
		hour, minute, sec = period_time.split(':')
		hour = int(hour)
		minute = int(minute)
		sec = float(sec)

		#	set data types
		period_track_id = int(period_track_id)
		period_mjd = float(period_mjd)

		if period == 'Aperiodic':
			period = 0.0
		else:
			period = float(period)

		mmt_period_data.append(
			[year, mon, day, hour, minute, sec, period_track_id, period_mjd, period]
		)

	mmt_period_data = np.array(mmt_period_data)

	print('\n		Period Data:\n')
	for period_line in mmt_period_data:

		print('		ID:	{};		{}'.format(int(period_line[6]), period_line[8]))

	###############################################################################################
	#	parse MMT file
	
	#	strip header
	header_text = mmt_data[7]
	raw_data = mmt_data[8:]
	
	print('\n		Parsing MMT data...')
	
	#	initialise output
	n_rows = 14
	parsed_data = np.zeros([len(raw_data), n_rows])
	
	for i, line in enumerate(raw_data):
	
		#	split line
		date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
			distance, phase, channel, track_id = line.split(' ')
	
		#	parse data and set data types
		mag_std = float(mag_std)
		mag_app = float(mag_app)
		distance = float(distance)
		phase = float(phase)
		track_id = int(track_id)
	
		#	parse date
		year, month, day = date_str.split('-')
		year = int(year)
		month = int(month)
		day = int(day)
	
		#	parse time
		hour, minute, second = time_str.split(':')
		hour = int(hour)
		minute = int(minute)
		second = np.round(float(second), 1)
	
		#	calculate julian date
		jday, jfrac = date2jd(year, month, day, hour, minute, second)
	
		#	generate output line
		output_line = [
			track_id, norad_id,
			jday, jfrac,
			year, month, day,
			hour, minute, second,
			mag_std, mag_app,
			distance, phase
		]
	
		#	add to output array
		parsed_data[i, :] = output_line
	
	print('			Total data length = ', len(parsed_data))
	print('			Total number of objects = ', len(np.unique(parsed_data[:, 1])))
	print('			Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

	###############################################################################################
	#	extracting data between specified windows
	
	print('\n		Seaching for tracks inside time window:')
	
	#	start window
	window_start_jd = np.add(*date2jd(*window_start_date))
	print('\n			Window start 	= ', window_start_date[0:3])
	
	#	stop window
	window_stop_jd = np.add(*date2jd(*window_stop_date))
	print('			Window stop 	= ', window_stop_date[0:3])
	
	#	extract track ids
	track_id_list = np.unique(parsed_data[:, 0])

	#	loop through tracks
	ntracks = 0
	first_lc_flag = True
	for track_id in track_id_list:
	
		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]
	
		#	extract light curve data
		jd_plot = np.add(track_data[:, 2], track_data[:, 3])
		std_mag_plot = track_data[:, 10]
		app_mag_plot = track_data[:, 11]
		distance_plot = track_data[:, 12]
		phase_plot = track_data[:, 13]
		start_line = track_data[0]
		stop_line = track_data[-1]
		start_jdate = np.add(start_line[2], start_line[3])
		stop_jdate = np.add(stop_line[2], stop_line[3])
	
		#	ensure between window start/stop
		if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
	
			#	light curve is within window
			ntracks += 1
	
			#	if first light curve, start array
			if first_lc_flag is True:
				windowed_data = track_data
				first_lc_flag = False
	
			#	else append to list
			elif first_lc_flag is False:
				windowed_data = np.vstack([windowed_data, track_data])
	
	print('\n			{} track(s) extracted.'.format(ntracks))

	#	reset track_id_list for windowed data
	track_id_list = np.unique(windowed_data[:, 0])

	###############################################################################################
	#	main track loop
	###############################################################################################

	first_track = True
	track_color_index = 0
	tracks_plotted = 0
	period_data = []
	period_data_date = []
	mmt_period_plot_data = []

	if specific_track_flag is True: 
		track_id_list = [specific_track]

	if period_tracks_flag is True: 
		track_id_list = mmt_period_data[:, 6] 

	#	loop through tracks
	for track_itt, track_id in enumerate(track_id_list):

		track_id = int(track_id)
	
		print('\n			#############################################################')
		print('			Working on Track {}, ID: {}'.format(track_itt, track_id))

		###########################################################################################
		#	extracting requested single track

		#	find period from mmt_period_data
		period_index = np.where(mmt_period_data[:, 6]==track_id)[0]

		if len(period_index) == 0:
			print('\n				No period found...')
			break
		
		else:
			mmt_period = mmt_period_data[period_index[0], -1]
			print('\n				Extracted MMT Period: ', mmt_period)

		###########################################################################################
		#	extracting requested single track
		
		print('\n			Extracting requested track index...')
		
		#	extract track data
		track_indices = np.where(windowed_data[:, 0]==track_id)[0]
		track_data = windowed_data[track_indices]

		print(track_data.shape)
		#######################################################################################
		#	extract track time data
		
		mmt_line = track_data[0]

		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		print(year, month, day, hour, minute, second)
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		print(year, month, day, hour, minute, second)
		
		stop_jd = [jday, jfrac]
		
		###################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		print('lc_dur = ', lc_duration_sec)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		print('lc_len = ', lc_len_record)
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)

		sim_range_sec = np.round(
			np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400),
			1
		)

		#######################################################################################
		#	ensure time continuity
		
		print('\n				Ensuring time continuity in extracted data...')
		
		track_data_temp = []
		sim_index = 0

		for itt, line in enumerate(track_data):
		
			#	parse line data
			line_track_id, line_norad_id,\
				jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = line
	
			step_jd = jday + jfrac
		
			#	calculate current seconds
			current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
		
			output_line = [
				jday, jfrac, current_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]
		
			##################################################
			#	extracting data of time step
		
			#	data match found
			if current_sec == sim_range_sec[sim_index]:
				track_data_temp.append(output_line)
		
				#	itterate number of output data points
				sim_index += 1
		
				#	save current line as previous line
				prev_line = output_line
		
			#	data point is not required
			elif current_sec < sim_range_sec[sim_index]:
				pass
		
			#	interpolation required
			elif current_sec > sim_range_sec[sim_index]:
		
				#	parse previous line
				jday_prev, jfrac_prev, prev_sec, \
					year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
					mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
		
				#	calculate interpolation requirements
				gap = np.round(current_sec - prev_sec, 1)
				npts = int(gap / time_step)
		
				#	get current light curve data
				mag_std_last = mag_std
				mag_app_last = mag_app
				distance_last = distance
				phase_last = phase
		
				#	calculate linear interpolation step values
				mag_std_step = (mag_std_last - mag_std_prev) / npts
				mag_app_step = (mag_app_last - mag_app_prev) / npts
				distance_step = (distance_last - distance_prev) / npts
				phase_step = (phase_last - phase_prev) / npts
		
				#	calculate interpolation data
				for n in range(1, npts + 1):
		
					#	current simulation second
					interp_sec = np.round(prev_sec + (n * time_step), 1)

					#	time data
					jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
					jday, jfrac = jday_jfrac(jdate)
					year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
					#	calculate interpolated light curve data
					mag_std = mag_std_prev + (n * mag_std_step)
					mag_app = mag_app_prev + (n * mag_app_step)
					distance = distance_prev + (n * distance_step)
					phase = phase_prev + (n * phase_step)

					#	generate new output line
					interp_output_line = [
						jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
		
					if interp_sec == sim_range_sec[sim_index]:
						track_data_temp.append(interp_output_line)

						sim_index += 1
		
				if interp_sec != current_sec:
					#	append final line
					track_data_temp.append(output_line)
					sim_index += 1
		
			else:
				print('ERROR')
				sys.exit()
		
			#	save previous line
			if itt != 0:
				prev_line = output_line
		
			#	array is complete condition
			if len(track_data_temp) == lc_len_record:
				break
		
		#	ensure parsed MMT data matches simulation data
		track_data = np.array(track_data_temp)
		
		if len(track_data) == len(lc_sim_range):
			print('					Simulation and MMT data matches.')
		else:
			print('					Error: Simulation and MMT data does not match...Exiting.')
			sys.exit()
		
		###########################################################################################
		#	extract mmt data

		jd_plot = np.add(track_data[:, 0], track_data[:, 1])
		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]

		###########################################################################################
		#	generate date vectors
		datevec_array = []
	
		for line in jd_plot:

			#	convert jd to date
			year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(line))]
	
			#	turn into string
			datestr = str('-'.join(
				['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
			))
	
			timestr = ':'.join(
				['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
			)
	
			datevec  = ' '.join([datestr, timestr])
			#	convert to datetime object
			date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
			#	append  to output array
			datevec_array.append(date_time_obj)

		###########################################################################################
		#	select time data to use
		
		sim_range = lc_sim_range
		len_record = lc_len_record
		start_date = lc_start_date
		stop_date = lc_stop_date
		duration_sec = lc_duration_sec

		#	calculate sim time range in seconds
		sim_time = np.linspace(0, duration_sec, len_record)
		print('track_data shape: ', track_data.shape)
		print('sim_time: ', sim_time.shape)
		sys.exit()
		###########################################################################################
		#	simulation printout
		###########################################################################################
		
		print('\n	#############################################################################')
		print('\n	MMT light curve parameters: ')
		print('		Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(
			*lc_start_date
		))
		print('		Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(
			*lc_stop_date
		))
		print('		Simulation time step:		{}'.format(time_step))
		print('		Duration:			{}		[steps]'.format(lc_len_record))
		print('						{}		[s]'.format(np.round(lc_duration_sec, 1)))

		###########################################################################################
		#	plot data

#		#	stacked light curves
#		light_curve_plot1 = ax[0].plot(
#			sim_time, mmt_signal_app,
#			linewidth=plot_linewidth,
#			label='Track-{}'.format(track_id)
#		)
#
#		#	over time
#		light_curve_plot2 = ax[1].plot(
#			datevec_array, mmt_signal_app,
#			linewidth=plot_linewidth
#		)

		period_data_date.append(datevec_array[0])
		mmt_period_plot_data.append([mmt_period])

		if calc_period_flag == True:

			#######################################################################################
			#	find period using phase dispersion minimisation
			#######################################################################################
			
			print('\n		Performing Phase Dispersion Minimisation...')
	
			#	calculate frequency
			freq = 1 / mmt_period
	
			#	Get a ``scanner'', which defines the frequency interval to be checked.
			#	Alternatively, also periods could be used instead of frequency.
	
			S = pyPDM.Scanner(minVal=min_freq, maxVal=max_freq, dVal=delta_freq, mode="period")
			
			#	Carry out PDM analysis. Get frequency array
			#	(f, note that it is frequency, because the scanner's
			#	mode is ``frequency'') and associated Theta statistic (t).
			#	Use 10 phase bins and 3 covers (= phase-shifted set of bins).
			P = pyPDM.PyPDM(sim_time, mmt_signal_app)
			f1, t1 = P.pdmEquiBinCover(10, 3, S)
	
			#	For comparison, carry out PDM analysis using 10 bins equidistant
			#	bins (no covers).
			f2, t2 = P.pdmEquiBin(10, S)
	
			t1_data = np.transpose(np.vstack([f1, t1]))
			t2_data = np.transpose(np.vstack([f2, t2]))
	
			T1 = t1_data[np.argsort(t1_data[:, 1])][0, 0]
			T2 = t2_data[np.argsort(t2_data[:, 1])][0, 0]

			period = T1
	
			print('\n			Light Curve Period = {} s'.format(period))
	
			period_data.append([period])

			if plot_period_flag == True:

				###################################################################################
				#	perform period folding
		
				print('\n		Performing light curve folding...')
		
				T0_val = 0.0
		
				#	Obtain the phases with respect to some
				#	reference point (in this case T0=0)
				phases = foldAt(sim_time, T1, T0=T0_val)
		
				#	Sort with respect to phase
				#	First, get the order of indices ...
				sortIndi = np.argsort(phases)
				
				#	... and, second, rearrange the arrays.
				phases = phases[sortIndi]
				mmt_signal_app_folded = mmt_signal_app[sortIndi]
		
				###################################################################################
				#	period analysis plots
		
				#	plot results
				plt.figure(facecolor='white')
				plt.title("Result of PDM analysis")
				plt.xlabel("Period")
				plt.ylabel("Theta")
				plt.plot(t1_data[:, 0], t1_data[:, 1], 'bp-')
				plt.plot(t2_data[:, 0], t2_data[:, 1], 'gp-')
				plt.legend(["pdmEquiBinCover", "pdmEquiBin"])
		
				#	generate correct solution plot line
				MMT_ans = np.full(t1.shape, mmt_period)
				PDM_ans = np.full(t1.shape, T1)
		
				y_vals = np.linspace(t1.min(), t1.max(), len(t1))
		
				plt.plot(MMT_ans, y_vals, color='r')
				plt.plot(PDM_ans, y_vals, color='g')
				
				#	Plot the result
				plt.figure()
				plt.title("Folded Light Curve")
				plt.xlabel("Time")
				plt.ylabel("Apparent Magnitude")
				plt.plot(phases, mmt_signal_app_folded, 'bp')
	
				plt.show()
				sys.exit()

#		if track_itt > 4:
#			break

#		plt.show()
#		sys.exit()

	###############################################################################################

	#	period_data
	period_data = np.array(period_data)

	if calc_period_flag == True:
		calc_period_plot = ax[2].plot(
			period_data_date, period_data,
			linewidth=plot_linewidth, marker='x'
		)

	mmt_period_plot = ax[2].plot(
		period_data_date, mmt_period_plot_data,
		linewidth=plot_linewidth, marker='x'
	)

	#	finalise plot
	fig.legend(
		loc='lower center',
		ncol=7,
		fontsize=legend_fontsize,
		frameon=False
	)

	###############################################################################################
	#	calculate run time
	###############################################################################################
	#	calculate sim time
	sim_end = time.time()
	sim_time = np.round((sim_end - sim_start) / 60, 1)
	
	print('\n[{}] Simulation Finished.'.format(datetime.datetime.now()))
	print('\n	- Time taken = {} mins.'.format(sim_time))

	

	plt.pause(0.001)
	input('\nPress enter to continue to next object...')

	ax[2].cla()

	
#	if calc_period_flag == True:
#		calc_period_plot[0].remove()
#
#	mmt_period_plot[0].remove()


plt.show()
sys.exit()

