###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

from astropy.timeseries import LombScargle
from PyAstronomy.pyTiming import pyPDM
from PyAstronomy.pyasl import foldAt

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

from scipy import fftpack

import warnings

# #	parrellisation
# from starlink_para_funs import rocket_body_brdf_fitting
# from multiprocessing import Pool
# import tqdm

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()
init_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'rocket_body')

#	set time windows
window_start_date = [2010, 11, 1, 0, 0, 0]
window_stop_date = [2020, 12, 1, 0, 0, 0]

#	set time step
time_step = 0.1

specific_norad_flag = True
specific_norad_flag = False
specific_norad = 32712
specific_norad = 40698
specific_norad = 43161
specific_norad = 32712

#	plot only tracks for which period data is available
period_tracks_flag = True

#	plot only a single specific track ID
specific_track_flag = False
specific_track = 12047965

#	calculate and plot a period
calc_period_flag = True
calc_period_flag = False

plot_period_flag = True
plot_period_flag = False

normalise_flag = False
normalise_flag = True

#	generate a synthetic light curve
gen_synth_lc = True

#	PDM parameters
min_freq = 4.0
max_freq = 10.0
delta_freq = 0.01

degrees_data = np.array([
	[0, 0]
])

degrees_data = np.array([
	[41858,		1],
	[37151,		4],
	[37229,		8],
	[25942,		1],
	[10675,		1],
	[31702,		2],
	[27706,		7],
	[25637,		1],
	[41847,		1],
	[40698,		3],
	[28538,		1],
	[40879,		1],
	[43278,		1],
	[37365,		5],
	[32712,		7],
	[38253,		10],
	[43161,		3],
	[28476,		13],
	[34662,		2],
	[39475,		10]
])

fitting_output_data = []

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

###################################################################################################
#	initialise figure
###################################################################################################

print('\n#####################################################################################')
print('Generating Final Plots...')

#	graphical output
inch2cm = 2.54

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4
xlabel_fontsize = 6
ylabel_fontsize = 6
xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set marker parameters
marker_size = 8

#	set figure line wdith
plot_linewidth = 1.0
grid_linewidth = 0.5
 
#	set use of LaTeX font
plt.rc('text', usetex=True)

#	set figure height/width
fig_width = 13.0/inch2cm
fig_height = 8.0/inch2cm

#	initialise figure axes
fig, ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300, sharex=True)
ax.set_title('Fitting Plot')
plt.gcf().subplots_adjust(right=0.80)
ax.set_title('Fitting Plot')
ax.set_xlabel(r'$Date/jd$', fontsize=ylabel_fontsize)
ax.set_ylabel(r'$Period/s$', fontsize=xlabel_fontsize)
ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
ax.grid(linewidth=grid_linewidth)
ax.set_axisbelow(True)

#	initialise figure axes
fig_final, ax_final = plt.subplots(figsize=(fig_width, fig_height), dpi=300, sharex=True)
ax_final.set_title('Final Data Plot')
plt.gcf().subplots_adjust(right=0.80)
ax_final.set_xlabel(r'$Date/jd$', fontsize=ylabel_fontsize)
ax_final.set_ylabel(r'$Period/s$', fontsize=xlabel_fontsize)
ax_final.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax_final.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
ax_final.grid(linewidth=grid_linewidth)
ax_final.set_axisbelow(True)


degrees_range = np.arange(1, 21, 1)

rms_data_fig, rms_data_ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300)
plt.gcf().subplots_adjust(right=0.80)
rms_data_ax.set_title('RMS Data Plot')
rms_data_ax.set_xlabel(r'$Number\ of\ Degrees$', fontsize=xlabel_fontsize)
rms_data_ax.set_ylabel(r'$RMS\ to\ Data$', fontsize=ylabel_fontsize)
rms_data_ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
rms_data_ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

rms_data_ax.set_ylim([-0.01, 0.11])

rms_data_ax.set_xticks(degrees_range, minor=False)
rms_data_ax.grid(linewidth=grid_linewidth)
rms_data_ax.set_axisbelow(True)

###################################################################################################
#	loop through MMT files
###################################################################################################
print('\n	Looping Through Rocket Body Light Curve Files...')

#	set input directory
rocket_body_dir = os.listdir(read_path)

color_int = 0

nobj = 1

#	loop through starlink IDs
for loop_itt, mmt_file in enumerate(rocket_body_dir):

	if '_periods' not in mmt_file:
		continue

	#	parse filename
	name_str = mmt_file.split('_')[0]
	norad_id = int(mmt_file.split('_')[1])

	if specific_norad_flag is True:
		if norad_id != specific_norad:
			print('\n#### Not specifically requested NORAD ID ####')
			continue

	print('\n		#####################################################################')
	print('		Plotting data for: {}, {}'.format(name_str, norad_id))

#	rb_type_str = ['ATLAS']
#	rb_type_str = ['DELTA']
#	rb_type_str = ['CZ']
#
#	rb_type_str = ['ATLAS', 'DELTA', 'CZ']
#
#	if len(rb_type_str) == 1:
#
#		if name_str.split('-')[0] != rb_type_str[0]:
#			continue
#		else:
#			color_str = 'C' + str(loop_itt)
#	
#	else:
#
#		if name_str.split('-')[0] in rb_type_str:
#			continue
#		else:
#			color_str = 'C' + str(loop_itt)

	color_str = 'C' + str(color_int)
	print('\n		Setting color str: ', color_str)

	color_int += 1

	###############################################################################################
	#	load MMT periods file
	
	#	set time
	mmt_load_start = time.time()
	
	mmt_periods_file = mmt_file
	print('\n		Loading MMT periods datafile: ', mmt_periods_file)

	with open(os.path.join(read_path, mmt_periods_file)) as read_file:
		mmt_periods_data = read_file.readlines()

	mmt_period_data = []
	mmt_period_data_date = []

	if len(mmt_periods_data[7:]) == 1:
		print('\n			#### Only a Single Datapoint, Skipping. ####')
		continue

	#	extract periodicity data
	for period_line in mmt_periods_data[7:]:

		#	parse line
		period_date, period_time, period_track_id, period_mjd, period = period_line.strip().split()

		#	parse date and time
		year, mon, day = [int(i) for i in period_date.split('-')]
		hour, minute, sec = period_time.split(':')
		hour = int(hour)
		minute = int(minute)
		sec = float(sec)

		#	set data types
		period_track_id = int(period_track_id)
		period_mjd = float(period_mjd)

		if period_mjd == 12345.0:
			period_mjd = np.add(*date2jd(year, mon, day, hour, minute, sec)) - 2400000.5

		if period == 'Aperiodic':
			period = 0.0
			continue
		else:
			period = float(period)

		mmt_period_data.append(
			[year, mon, day, hour, minute, sec, period_track_id, period_mjd, period]
		)

		#	turn into string
		datestr = str('-'.join(
			['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
		))

		timestr = ':'.join(
			['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
		)

		datevec  = ' '.join([datestr, timestr])
		#	convert to datetime object
		date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')

#		#	append  to output array
#		mmt_period_data_date.append(date_time_obj)
		mmt_period_data_date.append(period_mjd)

	mmt_period_data = np.array(mmt_period_data)
	mmt_period_data_date = np.array(mmt_period_data_date)

# 	###############################################################################################
# 	#	calculate rates of change
# 
# 	rate_of_change = []
# 	rate_of_change_date = []
# 
# 	print('\n			Period Data:')
# 	for itt, period_line in enumerate(mmt_period_data):
# 
# #		print('		ID:	{};		{}'.format(int(period_line[6]), period_line[8]))
# 	
# 		year, mon, day, hour, minute, sec, period_track_id, period_mjd, period = period_line
# 
# 		#	parse date and time
# 		year = int(year)
# 		mon = int(mon)
# 		day = int(day) 
# 		hour = int(hour)
# 		minute = int(minute)
# 		sec = float(sec)
# 
# 		#	turn into string
# 		datestr = str('-'.join(
# 			['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
# 		))
# 
# 		timestr = ':'.join(
# 			['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
# 		)
# 
# 		datevec  = ' '.join([datestr, timestr])
# 		#	convert to datetime object
# 		date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
# 
# 		if itt != 0:
# 
# 			dx = period - previous_period
# 			dt = period_mjd - previous_mjd
# 
# 			rate = dx / dt
# 
# 			#	append  to output array
# 			rate_of_change.append(rate)
# 			rate_of_change_date.append(period_mjd)
# 
# 
# 		previous_period = period
# 		previous_mjd = period_mjd
# 
# 	print('\n			Plotted {} Datapoints.'.format(itt + 1))
# 
# 	rate_of_change = np.array(rate_of_change)
# 	rate_of_change_date = np.array(rate_of_change_date)

	###############################################################################################
	#	performing fit

#	#	extract degrees_data:
#	degrees = degrees_data[nobj - 1]
#	print('\n		Using Predefined Degrees = ', degrees)
#	nobj += 1

	# with warnings.catch_warnings():
	warnings.simplefilter('ignore', np.RankWarning)

	degrees = 1

	#	set x, y data
	x = np.subtract(mmt_period_data_date, mmt_period_data_date[0])

	if normalise_flag is True:
		y = mmt_period_data[:, 8] / mmt_period_data[0, 8]
	else:
		y = mmt_period_data[:, 8]

	#	generate new x range
	xp = np.arange(np.floor(x[0]), np.ceil(x[-1]) + 1, 1)
	
	#	fit an n-degree polynomal
	res_data = []
	rms_data = []

	print('\n		Generating polynomials...')

	plot_rest_flag = False

	for deg_itt, loop_degrees in enumerate(degrees_range):

		#	scatter raw data
		ax.scatter(
			x, y,
			linewidth=plot_linewidth, marker='x', s=marker_size, color=color_str,
			label=name_str
		)

#		print('\n	Using degrees: ', loop_degrees)

		z = np.polyfit(x, y, loop_degrees)
		p = np.poly1d(z)
	
		#	generate fit lines
		poly_fit_line = p(xp)
	
		poly_res_sum = np.sum((np.polyval(z, x) - y)**2)

		rms = rms_calc(y, p(x))

#		print('		RMS: ', rms)

		rms_data.append([loop_degrees, rms])

		#	plot polynomial fit line
		ax.plot(
			xp, poly_fit_line,
			linestyle=':',
			linewidth=plot_linewidth,
			color=color_str
		)

		if plot_rest_flag is False:

			if norad_id in degrees_data[:, 0]:

					temp_index2 = np.where(degrees_data[:, 0]==norad_id)[0][0]

					fitting_output = [norad_id, degrees_data[temp_index2, 1]]
					fitting_output_data.append(fitting_output)

					plot_rest_flag = True

			else:

				plt.pause(0.001)
				ans = input("\n	Satisfactory fit? ['y'es / 'n'o / too 'f'ar]: ")
		
				if ans == 'y':
		
					fitting_output = [norad_id, loop_degrees]
					fitting_output_data.append(fitting_output)
		
					plot_rest_flag = True
		
				elif ans[0] == 'f':
	
					decn = int(ans[1:])
	
					print('\n	Decreasing degrees by {}...'.format(decn))
	
					fitting_output = [norad_id, loop_degrees - decn]
					fitting_output_data.append(fitting_output)
	
					plot_rest_flag = True
	
				elif ans == 'n':
					print('\n	Increasing degrees by 1...')

		ax.cla()
		ax.set_title('Fitting Plot')
		ax.set_xlabel(r'$Date/jd$', fontsize=ylabel_fontsize)
		ax.set_ylabel(r'$Period/s$', fontsize=xlabel_fontsize)
		ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
		ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
		ax.grid(linewidth=grid_linewidth)
		ax.set_axisbelow(True)

	#	convert to numpy arrays
	rms_data = np.array(rms_data)

	print('\n			Fitting_output: ', fitting_output)

	temp_index = np.where(rms_data[:, 0]==fitting_output[1])[0][0]

	label_str = '{} [{}]'.format(name_str, str(norad_id))

	#	plot to figure
	rms_data_ax.plot(
		rms_data[:, 0],
		rms_data[:, 1],
		color=color_str,
		linewidth=plot_linewidth,
		label=label_str
	)

	rms_data_ax.scatter(
		rms_data[:, 0],
		rms_data[:, 1],
		color=color_str,
		marker='+',
		s=12
	)

	rms_data_ax.scatter(
		fitting_output[1],
		rms_data[temp_index, 1],
		color='k',
		marker='x'
	)

	#	plot final fit line

	#	scatter raw data
	mmt_period_plot = ax.scatter(
		x, y,
		linewidth=plot_linewidth,
		marker='x', s=marker_size,
		color=color_str,
		label=name_str
	)

	#	generate fit lines
	z = np.polyfit(x, y, fitting_output[1])
	p = np.poly1d(z)
	poly_fit_line = p(xp)

	#	plot polynomial fit line
	ax.plot(
		xp, poly_fit_line,
		linestyle=':',
		linewidth=plot_linewidth,
		color=color_str
	)

#	#	plot to final figure
#	x = np.add(x, mmt_period_data_date[0])
#	xp = np.add(xp, mmt_period_data_date[0])

	#	scatter raw data
	final_plot1 = ax_final.scatter(
		x, y,
		linewidth=plot_linewidth,
		marker='x', s=marker_size,
		color=color_str,
		label=label_str
	)

	#	plot polynomial fit line
	final_plot2 = ax_final.plot(
		xp, poly_fit_line,
		linestyle=':',
		linewidth=plot_linewidth,
		color=color_str
	)

	###############################################################################################
	#	calculate run time
	###############################################################################################
	#	calculate sim time
	sim_end = time.time()
	sim_time = np.round((sim_end - sim_start) / 60, 1)

#	#	residuals improvement plot
#	ax2.plot(res_data[:, 0], res_data[:, 1])
	
	ax_final.relim()
	ax_final.autoscale_view()
	plt.pause(0.001)

	ans = input('\nKeep? ["y" / "n"]: ')
	if ans == 'y':
		pass
	
	elif ans == 'n':
		final_plot1.remove()
		final_plot2[0].remove()

#	input('\nPress enter to continue to next object...')
	plt.show()
	sys.exit()

#	#	clear RMS plot
#	rms_data_ax.cla()
#	rms_data_ax.set_title('RMS Data Plot')
#	rms_data_ax.set_xlabel(r'$Number\ of\ Degrees$', fontsize=xlabel_fontsize)
#	rms_data_ax.set_ylabel(r'$RMS\ to\ Data$', fontsize=ylabel_fontsize)
#	rms_data_ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
#	rms_data_ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
#	rms_data_ax.set_xticks(degrees_range, minor=False)
#	rms_data_ax.grid(linewidth=grid_linewidth)
#	rms_data_ax.set_axisbelow(True)

ax_final.relim()
ax_final.autoscale_view()

#	finalise plot
fig_final.legend(
	loc='right',
	ncol=1,
	fontsize=legend_fontsize,
	frameon=False
)

#	finalise plot
rms_data_fig.legend(
	loc='right',
	ncol=1,
	fontsize=legend_fontsize,
	frameon=False
)

print('\nFinal output:\n')
print(np.array(fitting_output_data))

plt.show()
sys.exit()
