###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

from astropy.timeseries import LombScargle
from PyAstronomy.pyTiming import pyPDM
from PyAstronomy.pyasl import foldAt

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle, req_omm

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

from scipy import fftpack
from scipy.interpolate import make_interp_spline, BSpline
from scipy.ndimage.filters import gaussian_filter1d

import warnings

# #	parrellisation
# from starlink_para_funs import rocket_body_brdf_fitting
# from multiprocessing import Pool
# import tqdm

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()
init_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'rocket_body')

#	output
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)

#	figure names
final_fig_name = 'residuals_plot_orderly.eps'

final_fig_name = 'orderly_data_norm.eps'
final_fig_name = 'orderly_data_norm_fast_despin.eps'
final_fig_name = 'orderly_data_norm_slow_despin.eps'

final_fig_name = 'residuals_plot_chaotic.eps'

final_fig_name = 'chaotic_data_norm.eps'

theta_fig_name = 'orderly_data_slow_despin_theta.eps'
theta_fig_name = 'chaotic_data_theta.eps'


omm_fig_name = 'full_orb_data.eps'

#	set flag
final_savefile_flag = False
rms_savefile_flag = False

theta_savefile_flag = True
theta_savefile_flag = False

omm_savefile_flag = True
omm_savefile_flag = False

#	orderly fast despin examples1
pdm_fig_name = 'orderly_data_fast_despin_pdm_good.eps'
pdm_fig_name = 'orderly_data_fast_despin_pdm_poor.eps'

#	orderly fast despin examples2
pdm_fig_name = 'orderly_data_fast_despin_pdm_poor2.eps'
pdm_fig_name = 'orderly_data_fast_despin_pdm_good2_false.eps'


#	orderly slow despin examples
pdm_fig_name = 'orderly_data_slow_despin_pdm_good2.eps'
pdm_fig_name = 'orderly_data_slow_despin_pdm_poor2.eps'


#	chaotic examples
pdm_fig_name = 'chaotic_data_pdm_good1.eps'
pdm_fig_name = 'chaotic_data_pdm_poor1.eps'

pdm_fig_name = 'orderly_data_fast_despin_pdm_poor.eps'


pdm_savefile_flag = False
pdm_savefile_flag = True

###################################################################################################
#	run parameters

#	set time windows
window_start_date = [2010, 11, 1, 0, 0, 0]
window_stop_date = [2020, 12, 1, 0, 0, 0]

#	set time step
time_step = 0.1

#	define specific norad ids to use
specific_norad_flag = False
specific_norad_flag = True
specific_norad = [41858]


#	plot only tracks for which period data is available
period_tracks_flag = True

#	plot only a single specific track ID
specific_track_flag = False
specific_track_flag = True
specific_track = 10873144


#	define tracks to discard
discarded_tracks = [17813491, 1682730, 1860732]

#	calculate and plot a period
fold_lightcurve = False
fold_lightcurve = True

#	plot calculated period flag
plot_period_flag = False
plot_period_flag = True

#	normalise data
normalise_flag = False
normalise_flag = True

#	plot in days since first light curve
plot_in_days = False
plot_in_days = True

#	generate a synthetic light curve
gen_synth_lc = True

#	polynomial fitting degrees
degrees_data = np.array([
	[41858,		1],
	[37151,		4],
	[37229,		8],
	[25942,		1],
	[10675,		1],
	[31702,		2],
	[27706,		7],
	[25637,		1],
	[41847,		1],
	[40698,		3],
	[28538,		1],
	[40879,		1],
	[43278,		1],
	[37365,		5],
	[32712,		7],
	[38253,		10],
	[43161,		3],
	[28476,		13],
	[34662,		2],
	[39475,		10]
])

fitting_output_data = []

#	define the list of chaotic objects
chaotic = np.array([
	37151,
	37229,
	27706,
	40698,
	37365,
	28476,
#	34662,
#	39475,
	32712,
	38253,
])

orderly = np.array([
 	41858,
 	25637,
 	41847,
 	40879,
 	43278,
 	43161,
	39475, # slow despin
	25942, # slow despin
	10675, # slow despin
	31702, # slow despin
	28538, # slow despin
])

orderly_slow = np.array([
 	40698, # slow despin
	39475, # slow despin
 	25942, # slow despin
 	10675, # slow despin
 	31702, # slow despin
 	28538, # slow despin
])


# #	define the list of fast despinners
# fast_despin = np.array([
# 	41858,
# 	25637,
# 	41847,
# 	40879,
# 	43278,
# 	43161,
# 	40698,
# ])
# 
# #	define the list of slow despinners
# slow_despin = np.array([
# 	25942,
# 	10675,
# 	31702,
# 	28538,
# ])
# 
# overlap = np.array([
# 	38253,
# ])

#	plot only a specific group
group_used_flag = True
group_used_flag = False

group_used = 'overlap'
group_used = 'slow_despin'
group_used = 'fast_despin'
group_used = 'orderly'
group_used = 'chaotic'

#	set earth radius
RE = 6371
earth_mu = 398600.4418

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

###################################################################################################
#	initialise figures
###################################################################################################

print('\n#####################################################################################')
print('Generating Final Plots...')

color_by_group = True
color_by_group = False

#	graphical output
inch2cm = 2.54

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4
xlabel_fontsize = 6
ylabel_fontsize = 6
xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set marker parameters
marker_size = 8

#	set figure line wdith
plot_linewidth = 1.0
grid_linewidth = 0.5
 
#	set use of LaTeX font
plt.rc('text', usetex=True)

#	set figure height/width
fig_width = 13.0/inch2cm
fig_height = 8.0/inch2cm

# ###################################################################################################
# #	initialise figure axes
# fig_ele, ax_ele = plt.subplots(3, 1, figsize=(9/inch2cm, 12/inch2cm), dpi=300, sharex=True)
# plt.gcf().subplots_adjust(right=0.80)
# ax_ele[-1].set_xlabel(r'$\mathrm{Days\ since\ first\ light\ curve}$', fontsize=ylabel_fontsize)
# 
# # ax_ele[0].set_ylabel(r'$\mathrm{SMA/Km}$', fontsize=xlabel_fontsize)
# # ax_ele[0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# # ax_ele[0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# # ax_ele[0].grid(linewidth=grid_linewidth)
# # ax_ele[0].set_axisbelow(True)
# 
# ax_ele[0].set_ylabel(r'$\mathrm{Perigee/Km}$', fontsize=xlabel_fontsize)
# ax_ele[0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# ax_ele[0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# ax_ele[0].grid(linewidth=grid_linewidth)
# ax_ele[0].set_axisbelow(True)
# 
# ax_ele[1].set_ylabel(r'$\mathrm{Apogee/Km}$', fontsize=xlabel_fontsize)
# ax_ele[1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# ax_ele[1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# ax_ele[1].grid(linewidth=grid_linewidth)
# ax_ele[1].set_axisbelow(True)
# 
# ax_ele[2].set_ylabel(r'$\mathrm{Ecc}$', fontsize=xlabel_fontsize)
# ax_ele[2].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# ax_ele[2].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# ax_ele[2].grid(linewidth=grid_linewidth)
# ax_ele[2].set_axisbelow(True)
# 
# # ax_ele[4].set_ylabel(r'$\mathrm{Inc/Deg}$', fontsize=xlabel_fontsize)
# # ax_ele[4].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# # ax_ele[4].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# # ax_ele[4].grid(linewidth=grid_linewidth)
# # ax_ele[4].set_axisbelow(True)
# 
# # ###################################################################################################
# # #	initialise figure axes
# # fig_resid, ax_resid = plt.subplots(figsize=(fig_width, fig_height), dpi=300, sharex=True)
# # plt.gcf().subplots_adjust(right=0.80)
# # ax_resid.set_title(r'Residuals\ Plot')
# # ax_resid.set_xlabel(r'$Date/jd$', fontsize=ylabel_fontsize)
# # ax_resid.set_ylabel(r'$Period\ Residuals/s$', fontsize=xlabel_fontsize)
# # ax_resid.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# # ax_resid.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# # ax_resid.grid(linewidth=grid_linewidth)
# # ax_resid.set_axisbelow(True)
# 
# ###################################################################################################
# #	initialise figure axes
# fig, ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300, sharex=True)
# plt.gcf().subplots_adjust(right=0.80)
# ax.set_title(r'Fitting\ Plot')
# ax.set_xlabel(r'$Date/jd$', fontsize=ylabel_fontsize)
# ax.set_ylabel(r'$Period/s$', fontsize=xlabel_fontsize)
# ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# ax.grid(linewidth=grid_linewidth)
# ax.set_axisbelow(True)
# 
# ###################################################################################################
# #	initialise figure axes
# fig_final, ax_final = plt.subplots(figsize=(fig_width, fig_height), dpi=300, sharex=True)
# plt.gcf().subplots_adjust(right=0.80)
# 
# ax_final.set_xlabel(r'$\mathrm{Days\ Since\ First\ Light\ Curve}$', fontsize=ylabel_fontsize)
# ax_final.set_ylabel(r'$\mathrm{Normalised\ Period}$', fontsize=xlabel_fontsize)
# 
# # ax_final.set_xlabel(r'$\mathrm{Modified\ Julian\ Date}$', fontsize=ylabel_fontsize)
# # ax_final.set_ylabel(r'$\mathrm{Period\ /\ s}$', fontsize=xlabel_fontsize)
# 
# 
# ax_final.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# ax_final.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# ax_final.grid(linewidth=grid_linewidth)
# ax_final.set_axisbelow(True)
# 
# fig_final.canvas.set_window_title('Fig Final')
# 
# degrees_range = np.arange(1, 21, 1)
# 
# ###################################################################################################
# #	initialise figure axes
# rms_data_fig, rms_data_ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300)
# plt.gcf().subplots_adjust(right=0.80)
# #rms_data_ax.set_title('RMS Data Plot')
# rms_data_ax.set_xlabel(r'$\mathrm{Number\ of\ Degrees}$', fontsize=xlabel_fontsize)
# rms_data_ax.set_ylabel(r'$\mathrm{Least\ Squares\ Fit\ Residuals\ Sum}$', fontsize=ylabel_fontsize)
# rms_data_ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# rms_data_ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# 
# rms_data_ax.set_ylim([-0.02, 0.7])
# # rms_data_ax.set_ylim([-0.02, 0.2])
# 
# rms_data_ax.set_xticks(degrees_range, minor=False)
# rms_data_ax.grid(linewidth=grid_linewidth)
# rms_data_ax.set_axisbelow(True)
# 
# ###################################################################################################
# #	initialise figure axes
# theta_fig, theta_ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300)
# plt.gcf().subplots_adjust(right=0.80)
# #theta_ax.set_title('Minimum Theta Plot')
# theta_ax.set_xlabel(r'$\mathrm{Days\ since\ first\ light\ curve}$', fontsize=xlabel_fontsize)
# theta_ax.set_ylabel(r'$\mathrm{Minimum\ Dispersion}$', fontsize=ylabel_fontsize)
# theta_ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
# theta_ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
# theta_ax.grid(linewidth=grid_linewidth)
# theta_ax.set_axisbelow(True)
# 
# fig_ele, ax_ele = plt.subplots(3, 1, figsize=(9/inch2cm, 12/inch2cm), dpi=300, sharex=True)
# fig, ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300, sharex=True)
# fig_final, ax_final = plt.subplots(figsize=(fig_width, fig_height), dpi=300, sharex=True)
# rms_data_fig, rms_data_ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300)
# theta_fig, theta_ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300)

###################################################################################################
#	loop through MMT files
###################################################################################################
print('\n	Looping Through Rocket Body Light Curve Files...')

#	set input directory
rocket_body_dir = os.listdir(read_path)

marker_list = ['^', 's', 'o']
marker_id = 0

color_int = 0

nobj = 1

no_orbital_data = []
grad_data = []
fitting_residuals = []

plotted_tracks = []

first_fast_despin = True
first_slow_despin = True
first_chaotic = True
first_orderly = True

#	loop through starlink IDs
for loop_itt, mmt_file in enumerate(rocket_body_dir):

	if '_periods' not in mmt_file:
		continue

	###############################################################################################
	#	parse filename
	name_str = mmt_file.split('_')[0]
	norad_id = int(mmt_file.split('_')[1])

	###############################################################################################
	#	set colours
	if color_by_group is True:

		marker_used = 's'

		if norad_id in chaotic:
			print('\n		Setting color str or Chaotic')
			color_str = 'tab:red'
			label_str = 'Chaotic'

		elif norad_id in orderly:
			print('\n		Setting color str for Orderly')
			color_str = 'tab:green'
			label_str = 'Orderly'

		else:
			print('\n		Object not in either category')

		if norad_id == 40698:
			print('\n		Setting color str or AVUM')
			color_str = 'tab:blue'
			label_str = '{} [{}]'.format(name_str, str(norad_id))

#		if norad_id in fast_despin:
#			color_str = 'C0'
#
#		elif norad_id in slow_despin:
#			color_str = 'C1'
#
#		elif norad_id in chaotic:
#			color_str = 'C2'
#
#		elif norad_id in overlap:
#			color_str = 'C3'

	else:

		if color_int == 10:
			color_int = 0
			marker_id += 1

		color_str = 'C' + str(color_int)
		print('\n		Setting color str: ', color_str)
		color_int += 1

		marker_used = marker_list[marker_id]

		label_str = '{} [{}]'.format(name_str, str(norad_id))




#	if norad_id == 40698:
#		color_str = 'm'
#	elif norad_id == 34662:
#		color_str = 'c'
#	elif norad_id == 39475:
#		color_str = 'y'




#	label_str = '$\mathrm{' + '{} [{}]'.format(name_str, str(norad_id)) + '}$'

	if group_used_flag == True:

		if group_used == 'fast_despin':
			if norad_id not in fast_despin:
				print('\n#### Not Part of Reqested Group ####')
				continue
		elif group_used == 'slow_despin':
			if norad_id not in slow_despin:
				print('\n#### Not Part of Reqested Group ####')
				continue
		elif group_used == 'chaotic':
			if norad_id not in chaotic:
				print('\n#### Not Part of Reqested Group ####')
				continue
		elif group_used == 'overlap':
			if norad_id not in overlap:
				print('\n#### Not Part of Reqested Group ####')
				continue

		elif group_used == 'orderly':
			if norad_id not in orderly:
				print('\n#### Not Part of Reqested Group ####')
				continue

	if specific_norad_flag is True:
		if norad_id not in specific_norad:
			print('\n#### Not specifically requested NORAD ID ####')
			continue

#	print('\n')
#	if norad_id in fast_despin:
#		if first_fast_despin is True:
#			print('first fast spinner')
#			label_str = 'Fast De-Spin'
#			print('label = ', label_str)
#			plot_with_label = True
#			first_fast_despin = False
#		else:
#			plot_with_label = False
#
#	if norad_id in slow_despin:
#		if first_slow_despin is True:
#			print('first slow spinner')
#			label_str = 'Slow De-Spin'
#			print('label = ', label_str)
#			plot_with_label = True
#			first_slow_despin = False
#		else:
#			plot_with_label = False
#
#	plot_with_label = True


	print('\n		#####################################################################')
	print('		Plotting data for: {}, {}'.format(name_str, norad_id))

	###############################################################################################
	#	load MMT lightcurve file

	mmt_lightcurve_file = mmt_file[:-12] + '.txt'

	print('\n		Loading MMT lightcurve datafile: ', mmt_lightcurve_file)

	with open(os.path.join(read_path, mmt_lightcurve_file)) as read_file:
		mmt_lightcurve_data = read_file.readlines()

	###############################################################################################
	#	parse MMT file
	
	#	strip header
	header_text = mmt_lightcurve_data[7]
	raw_data = mmt_lightcurve_data[8:]
	
	print('\n				Parsing MMT data...')
	
	#	initialise output
	n_rows = 14
	parsed_data = np.zeros([len(raw_data), n_rows])
	
	for i, line in enumerate(raw_data):
	
		#	split line
		date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
			distance, phase, channel, track_id = line.split(' ')
	
		#	parse data and set data types
		mag_std = float(mag_std)
		mag_app = float(mag_app)
		distance = float(distance)
		phase = float(phase)
		track_id = int(track_id)
	
		#	parse date
		year, month, day = date_str.split('-')
		year = int(year)
		month = int(month)
		day = int(day)
	
		#	parse time
		hour, minute, second = time_str.split(':')
		hour = int(hour)
		minute = int(minute)
		second = np.round(float(second), 1)
	
		#	calculate julian date
		jday, jfrac = date2jd(year, month, day, hour, minute, second)
	
		#	generate output line
		output_line = [
			track_id, norad_id,
			jday, jfrac,
			year, month, day,
			hour, minute, second,
			mag_std, mag_app,
			distance, phase
		]
	
		#	add to output array
		parsed_data[i, :] = output_line
	
	print('				Total data length = ', len(parsed_data))
	print('				Total number of objects = ', len(np.unique(parsed_data[:, 1])))
	print('				Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

	###############################################################################################
	#	load MMT periods file
	
	#	set time
	mmt_load_start = time.time()
	
	mmt_periods_file = mmt_file
	print('\n		Loading MMT periods datafile: ', mmt_periods_file)

	with open(os.path.join(read_path, mmt_periods_file)) as read_file:
		mmt_periods_data = read_file.readlines()

	mmt_period_data = []

	#	extract periodicity data
	for period_line in mmt_periods_data[7:]:

		#	parse line
		period_date, period_time, period_track_id, period_mjd, mmt_period = period_line.strip().split()

		#	parse date and time
		year, mon, day = [int(i) for i in period_date.split('-')]
		hour, minute, sec = period_time.split(':')
		hour = int(hour)
		minute = int(minute)
		sec = float(sec)

		#	set data types
		period_track_id = int(period_track_id)

		if period_track_id in discarded_tracks:
			print('\n			#### Is a Discarded Track. ####')
			continue

		period_mjd = float(period_mjd)

#		print('\n			Period TrackID: ', period_track_id)

		if period_mjd == 12345.0:
			period_mjd = np.add(*date2jd(year, mon, day, hour, minute, sec)) - 2400000.5

		if mmt_period == 'Aperiodic':
			mmt_period = 0.0
			continue
		else:
			mmt_period = float(mmt_period)

#		print('			MMT Period = ', mmt_period)

		mmt_period_data.append(
			[year, mon, day, hour, minute, sec, period_track_id, period_mjd, mmt_period]
		)

	mmt_period_data = np.array(mmt_period_data)



	#	PDM parameters
	min_period = np.floor(mmt_period_data[:, 8].min()) - 1
	max_period = np.ceil(mmt_period_data[:, 8].max()) + 1
	delta_period = 0.01

	pdm_period_data = []
	period_residuals_data = []
	mmt_period_data_date = []

	if len(mmt_periods_data[7:]) == 1:
		print('\n			#### Only a Single Datapoint, Skipping. ####')
		continue

	#	extract periodicity data
	for period_itt, period_line in enumerate(mmt_period_data):

		year, mon, day, hour, minute, sec, period_track_id, period_mjd, mmt_period = period_line
		year, mon, day, hour, minute = [int(thing) for thing in [year, mon, day, hour, minute]]

		if period_track_id in discarded_tracks:
			print('\n			#### Is a Discarded Track. ####')
			continue

		if specific_track_flag is True:
			if period_track_id != specific_track:
				print('\n			#### Not Specifically Requested Track. ####')
				continue

		#	turn into string
		datestr = str('-'.join(
			['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
		))

		timestr = ':'.join(
			['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
		)
		datevec  = ' '.join([datestr, timestr])
		#	convert to datetime object
		date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')

		#	append  to output array
		mmt_period_data_date.append(period_mjd)

		###########################################################################################
		#	extract light curve of period_track_id

		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==period_track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]

		raw_mmt_app = track_data[:, 11]

		raw_mmt_time = 86400 * np.subtract(
			np.add(track_data[:, 2], track_data[:, 3]),
			np.add(track_data[0, 2], track_data[0, 3])
		)

#		print('\n			Average Range = ', np.average(track_data[:, 12]))

		#######################################################################################
		#	extract track time data
		
		mmt_line = track_data[0]

		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]

		start_jd = [jday, jfrac]
		
		#	parse line elements
		line_track_id, line_norad_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]
		
		###################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)

		sim_range_sec = np.round(
			np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400),
			1
		)

		#######################################################################################
		#	ensure time continuity
		
		track_data_temp = []
		sim_index = 0

		for itt, line in enumerate(track_data):
		
			#	parse line data
			line_track_id, line_norad_id,\
				jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = line
	
			step_jd = jday + jfrac
		
			#	calculate current seconds
			current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
		
			output_line = [
				jday, jfrac, current_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]
		
			##################################################
			#	extracting data of time step
		
			#	data match found
			if current_sec == sim_range_sec[sim_index]:
				track_data_temp.append(output_line)
		
				#	itterate number of output data points
				sim_index += 1
		
				#	save current line as previous line
				prev_line = output_line
		
			#	data point is not required
			elif current_sec < sim_range_sec[sim_index]:
				pass
		
			#	interpolation required
			elif current_sec > sim_range_sec[sim_index]:
		
				#	parse previous line
				jday_prev, jfrac_prev, prev_sec, \
					year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
					mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
		
				#	calculate interpolation requirements
				gap = np.round(current_sec - prev_sec, 1)
				npts = int(gap / time_step)
		
				#	get current light curve data
				mag_std_last = mag_std
				mag_app_last = mag_app
				distance_last = distance
				phase_last = phase
		
				#	calculate linear interpolation step values
				mag_std_step = (mag_std_last - mag_std_prev) / npts
				mag_app_step = (mag_app_last - mag_app_prev) / npts
				distance_step = (distance_last - distance_prev) / npts
				phase_step = (phase_last - phase_prev) / npts
		
				#	calculate interpolation data
				for n in range(1, npts + 1):
		
					#	current simulation second
					interp_sec = np.round(prev_sec + (n * time_step), 1)

					#	time data
					jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
					jday, jfrac = jday_jfrac(jdate)
					year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
					#	calculate interpolated light curve data
					mag_std = mag_std_prev + (n * mag_std_step)
					mag_app = mag_app_prev + (n * mag_app_step)
					distance = distance_prev + (n * distance_step)
					phase = phase_prev + (n * phase_step)

					#	generate new output line
					interp_output_line = [
						jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
		
					if interp_sec == sim_range_sec[sim_index]:
						track_data_temp.append(interp_output_line)

						sim_index += 1
		
				if interp_sec != current_sec:
					#	append final line
					track_data_temp.append(output_line)
					sim_index += 1
		
			else:
				print('ERROR')
				sys.exit()
		
			#	save previous line
			if itt != 0:
				prev_line = output_line
		
			#	array is complete condition
			if len(track_data_temp) == lc_len_record:
				break
		
		#	ensure parsed MMT data matches simulation data
		track_data = np.array(track_data_temp)
		
		if len(track_data) == len(lc_sim_range):
#			print('					Simulation and MMT data matches.')
			pass
		else:
			sys.exit()
		
		#	extract brightness data
		mmt_signal_app = track_data[:, 10]

		###########################################################################################
		#	select time data to use
		
		sim_range = lc_sim_range
		len_record = lc_len_record
		start_date = lc_start_date
		stop_date = lc_stop_date
		duration_sec = lc_duration_sec

		#	calculate sim time range in seconds
		sim_time = np.linspace(0, duration_sec, len_record)

		if fold_lightcurve is True:

			print('\n		Woring on Track: ', int(line_track_id))

			###########################################################################################
			#	find period using phase dispersion minimisation
			
			print('\n			Performing Phase Dispersion Minimisation...')
		
			min_period = mmt_period * 0.4
			max_period = mmt_period * 1.6

			min_period = mmt_period * 0.6
			max_period = mmt_period * 1.4

			#	Get a ``scanner'', which defines the frequency interval to be checked.
			#	Alternatively, also periods could be used instead of frequency.
			S = pyPDM.Scanner(minVal=min_period, maxVal=max_period, dVal=delta_period, mode="period")
	
			#	Carry out PDM analysis. Get frequency array
			#	(f, note that it is frequency, because the scanner's
			#	mode is ``frequency'') and associated Theta statistic (t).
			#	Use 10 phase bins and 3 covers (= phase-shifted set of bins).
			P = pyPDM.PyPDM(raw_mmt_time, raw_mmt_app)
			f1, t1 = P.pdmEquiBinCover(10, 3, S)
			t1_data = np.transpose(np.vstack([f1, t1]))
			T1 = t1_data[np.argsort(t1_data[:, 1])][0, 0]
	
			min_theta = t1.min()
			pdm_period = T1
		
			print('\n				MMT Period = {} s'.format(mmt_period))
			print('				PDM Period = {} s'.format(pdm_period))

			print('\n					Minimum Theta = {}'.format(min_theta))
			residual = abs(mmt_period - pdm_period)
			print('					Residual = {} s'.format(residual))

			pdm_period_data.append([pdm_period, min_theta, residual, int(line_track_id)])
	
			if plot_period_flag == True:
				#	if plot_period_flag == True and (min_theta > 0.714 or min_theta < 0.33):
				#	if plot_period_flag == True and min_theta < 0.15:
	
				###################################################################################
				#	perform period folding
			
				print('\n			Plotting Folded Light Curve...')
			
				T0_val = 0.0
			
				#	Obtain the phases with respect to some
				#	reference point (in this case T0=0)
				phases = foldAt(raw_mmt_time, T1, T0=T0_val)
			
				#	Sort with respect to phase
				#	First, get the order of indices ...
				sortIndi = np.argsort(phases)
				
				#	... and, second, rearrange the arrays.
				phases = phases[sortIndi]
				mmt_signal_app_folded = raw_mmt_app[sortIndi]
			
				###################################################################################
				#	period analysis plots
			
				#	set figure height/width
				pdm_fig_width = 20.0/inch2cm
				pdm_fig_height = 6.0/inch2cm

				pdm_fig, pdm_ax = plt.subplots(1, 3, figsize=(pdm_fig_width, pdm_fig_height), dpi=300)
				plt.subplots_adjust(wspace=0.3)
	
				pdm_ax0 = pdm_ax[0]
				pdm_ax1 = pdm_ax[1]
				pdm_ax2 = pdm_ax[2]
				pdm_marker = 0.6
	
				pdm_ax0.set_title(r"$\mathrm{Lightcurve}$", fontsize=xlabel_fontsize)
				pdm_ax0.set_xlabel(r"$\mathrm{Time/s}$", fontsize=xlabel_fontsize)
				pdm_ax0.set_ylabel(r"$\mathrm{Apparent\ Magnitude}$", fontsize=ylabel_fontsize)
				pdm_ax0.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
				pdm_ax0.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
				pdm_ax0.invert_yaxis()
				pdm_ax0.plot(
					raw_mmt_time, raw_mmt_app,
					linewidth=0.4,
					marker='o', markersize=pdm_marker,
					label=r'$\mathrm{' + str(norad_id) + '\ ' + str(int(period_track_id)) + '}$'
				)
				pdm_ax0.legend(frameon=False, fontsize=xlabel_fontsize, loc='lower right')
	
				#	plot results
				pdm_ax1.set_title(r"$\mathrm{Result\ of\ PDM\ analysis}$", fontsize=xlabel_fontsize)
				pdm_ax1.set_xlabel(r"$\mathrm{Period/s}$", fontsize=xlabel_fontsize)
				pdm_ax1.set_ylabel(r"$\mathrm{Dispersion}$", fontsize=ylabel_fontsize)
				pdm_ax1.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
				pdm_ax1.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
				pdm_ax1.plot(f1, t1, linewidth=0.4, marker='D', markersize=pdm_marker)
	
				#	generate correct solution plot line
				MMT_ans = np.full(t1.shape, mmt_period)
				PDM_ans = np.full(t1.shape, T1)
			
				y_vals = np.linspace(t1.min(), t1.max(), len(t1))
			
				pdm_ax1.plot(
					MMT_ans, y_vals,
					color='r',
					linestyle='-.',
					linewidth=0.8,
					label=r'$\mathrm{MMT\ Period}$'
				)
				pdm_ax1.plot(
					PDM_ans, y_vals,
					color='g',
					linestyle=':',
					linewidth=0.8,
					label=r'$\mathrm{PDM\ Period}$'
				)

				pdm_ax1.legend(frameon=False, fontsize=xlabel_fontsize, loc='lower right')
				
				#	Plot the result
				pdm_ax2.set_title(r"$\mathrm{Folded\ Light\ Curve}$", fontsize=xlabel_fontsize)
				pdm_ax2.set_xlabel(r"$\mathrm{Decimal\ Period}$", fontsize=xlabel_fontsize)
				pdm_ax2.set_ylabel(r"$\mathrm{Apparent\ Magnitude}$", fontsize=ylabel_fontsize)
				pdm_ax2.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
				pdm_ax2.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
				pdm_ax2.invert_yaxis()
				pdm_ax2.scatter(phases, mmt_signal_app_folded, marker='o', s=pdm_marker)



				pdm_ax2.text(0.0, 2.75, '1', color='r', fontsize=10)

				pdm_ax2.text(0.22, 1.5, '2', color='r', fontsize=10)

				pdm_ax2.text(0.38, 2.9, '3', color='r', fontsize=10)

				pdm_ax2.text(0.52, 1.9, '4', color='r', fontsize=10)

				pdm_ax2.text(0.65, 2.85, '5', color='r', fontsize=10)

				pdm_ax2.text(0.8, 1.5, '6', color='r', fontsize=10)



				if pdm_savefile_flag is True:
					plt.pause(0.01)
					out_path = os.path.join(fig_path, pdm_fig_name)
					
					if os.path.exists(out_path):
						print('\nPath: ', out_path, ' exists.')
						ans = input('\nReplace File? [y/n] ')
					
						if ans == 'y':
							saveflag = True
						else:
							saveflag = False
					
					else:
						saveflag = True
					
					if saveflag == True:
						pdm_fig.savefig(out_path, bbox_inches="tight", dpi=300)
						print('\n#######################################')
						print('Figure saved: ', pdm_fig_name)
					else:
						print('\nNot saving: ', pdm_fig_name)


				plt.pause(0.01)

#				print('Closing Figures')
#				plt.close(fig_ele)
#				plt.close(fig)
#				plt.close(fig_final)
#				plt.close(rms_data_fig)
#				plt.close(theta_fig)




				input('Press Enter to continue to next track...')

#				plt.show()
#				sys.exit()

	#	convert to numpy arrays
	pdm_period_data = np.array(pdm_period_data)
	mmt_period_data_date = np.array(mmt_period_data_date)
	
	###########################################################################################
	#	generate OMM data across range
	###########################################################################################
	
	print('\n		#####################################################################')
	print('		Acquiring OMMs...')
	
	#	TLE request dates, full range since launch
	tle_start_date = [int(s) for s in mmt_period_data[0, 0:5]] + [float(mmt_period_data[0, 5])]
	tle_stop_date = [int(s) for s in mmt_period_data[-1, 0:5]] + [float(mmt_period_data[-1, 5])]
	
	#	generate TLE file name
	tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
	tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
	
	omm_file = '{}_{}_{}.omm'.format(norad_id, tle_start_date_str, tle_stop_date_str)
	
	#	assemble TLE path
	omm_path = os.path.join(path_data, 'input_dir', 'omm_files')
	
	#	if exists, load
	if os.path.isfile(os.path.join(omm_path, omm_file)):
		print('\n			Reading existing OMM file: ', omm_file)
	
		#	load file
		with open(os.path.join(omm_path, omm_file), 'r') as file:
			omm_data = file.readlines()
	
	else:
		print('\n			Requesting OMM file: ', omm_file)
	
		#	request TLEs
		omm_data_raw = req_omm(tle_start_date, tle_stop_date, norad_id)
	
		omm_data = []
	
		#	save file
		with open(os.path.join(omm_path, omm_file), 'w') as file:
			for line_id, line in enumerate(omm_data_raw):
	
				if line != '':
	
					omm_data.append(line)
	
					if line_id != len(omm_data_raw) - 1:
						file.write(line + '\n')
					else:
						file.write(line)

		print('				OMM file saved.')
	
		print('\n			OMM Request Complete, pausing for 3s...')
		time.sleep(3)
	
	if omm_data == []:
		print('\n			####	NO OMM DATA EXISTS	####')
		no_orbital_data.append(norad_id)
		plot_omm = False
	else:
		plot_omm = True

	###########################################################################################
	#	calculating orbital height

	if plot_omm is True:
		
		print('\n			Parsing OMM Data...')
		
		#	initialise height data
		height_data = []
		difference_data = []
		tle_datevec_array = []
		difference_data_datevec_array = []
		
		on_station_flag = False
		time_found_flag = False
		run_diff = 0
		run_diff_data = []
		
		spread_max = 1.5
		diff_threshold = 1.5
		
		alt_data = []
		perc_error = 1
	
		elements_data = []
		elements_date_data = []
	
		for omm_line_index, omm_line in enumerate(omm_data[1:]):
		
			#	parse omm line
			omm_vers, comment, creation_date, originator, obj_name, obj_id, center_obj, \
				ref_frame, time_system, mean_ele_theory, epoch_str, mean_mot, ecc, inc, \
				raan, aop, mean_anom, ephem_type, class_type, norad_cat_id, ele_set_no, \
				rev_at_epoch, bstar, mean_mot_dot, mean_mot_ddot, tle_line0, tle_line1, \
				tle_line2, sma, period, r_apo, r_peri, obj_type, decayed_state \
				= omm_line.split(',')
		
			#	parse data types
			omm_vers = float(omm_vers.split('"')[1])
			comment = comment.split('"')[1]
			creation_date_str = creation_date.split('"')[1]
			originator = originator.split('"')[1]
			obj_name = obj_name.split('"')[1]
			obj_id = obj_id.split('"')[1]
			center_obj = center_obj.split('"')[1]
			ref_frame = ref_frame.split('"')[1]
			time_system = time_system.split('"')[1]
			mean_ele_theory = mean_ele_theory.split('"')[1]
			epoch_str = epoch_str.split('"')[1]
			mean_mot = float(mean_mot.split('"')[1])
			ecc = float(ecc.split('"')[1])
			inc = float(inc.split('"')[1])
			raan = float(raan.split('"')[1])
			aop = float(aop.split('"')[1])
			mean_anom = float(mean_anom.split('"')[1])
			ephem_type = int(ephem_type.split('"')[1])
			class_type = class_type.split('"')[1]
			norad_cat_id = int(norad_cat_id.split('"')[1])
			ele_set_no = int(ele_set_no.split('"')[1])
			rev_at_epoch = int(rev_at_epoch.split('"')[1])
			bstar = float(bstar.split('"')[1])
			mean_mot_dot = float(mean_mot_dot.split('"')[1])
			mean_mot_ddot = float(mean_mot_ddot.split('"')[1])
			tle_line0 = tle_line0.split('"')[1]
			tle_line1 = tle_line1.split('"')[1]
			tle_line2 = tle_line2.split('"')[1]
			sma = float(sma.split('"')[1])
			period = float(period.split('"')[1])
			r_apo = float(r_apo.split('"')[1])
			r_peri = float(r_peri.split('"')[1])
			obj_type = obj_type.split('"')[1]
			decayed_state = int(decayed_state.split('"')[1])
	
			date_str, time_str = epoch_str.split('T')
			year, month, day = [int(s) for s in date_str.split('-')]
			hour, minute = [int(s) for s in time_str.split(':')[0:2]]
			sec = float(time_str.split(':')[2])
	
			omm_jd = np.add(*date2jd(year, month, day, hour, minute, sec))
		
			#	convert to radians per second
			mean_mot_rps = mean_mot / 86400 * (2*np.pi)
		
			#	calculate orbital altitude
			sat_height = sma - RE
	
		#	print('Orbital Elements: ', sma, ecc, inc, raan, aop)
	
			#	append to outputs
			elements_vec = [inc, raan, ecc, aop, sma, sat_height, r_apo, r_peri]
			elements_data.append(elements_vec)
	
			elements_date_data.append(omm_jd)
	
		#	convert to numpy array
		elements_data = np.array(elements_data)
		elements_date_data = np.reshape(np.array(elements_date_data), [len(elements_data), 1])
	
		if plot_in_days is True:
			elements_date_data = elements_date_data - elements_date_data[0]
	
		elements_data = np.hstack([elements_data, elements_date_data])

#		#	plot to elements figure
#		if plot_with_label is True:
#			ax_ele[0].plot(
#				elements_date_data,
#				elements_data[:, 4],
#				linewidth=plot_linewidth,
#				color=color_str,
#				label=label_str
#			)
#		else:
#			ax_ele[0].plot(
#				elements_date_data,
#				elements_data[:, 4],
#				linewidth=plot_linewidth,
#				color=color_str,
#			)

		if norad_id in chaotic:
			if first_chaotic is True:
				print('first chaotic')
				print('label = ', label_str)
				plot_with_label = True

				total_elements_data_cha = elements_data

				first_chaotic = False
			else:
				total_elements_data_cha = np.vstack([total_elements_data_cha, elements_data])
				plot_with_label = False
	
		if norad_id in orderly:
			if first_orderly is True:
				print('first_orderly')
				print('label = ', label_str)
				plot_with_label = True

				total_elements_data_ord = elements_data

				first_orderly = False
			else:
				total_elements_data_ord = np.vstack([total_elements_data_ord, elements_data])
				plot_with_label = False








		if plot_with_label is True or norad_id == 40698:
			ax_ele[0].plot(
				elements_date_data,
				elements_data[:, 7],
				linewidth=plot_linewidth,
				color=color_str,
				label=label_str
			)

		else:
			ax_ele[0].plot(
				elements_date_data,
				elements_data[:, 7],
				linewidth=plot_linewidth,
				color=color_str,
			)
	
		ax_ele[1].plot(
			elements_date_data,
			elements_data[:, 6],
			linewidth=plot_linewidth,
			color=color_str,
		)
	
		ax_ele[2].plot(
			elements_date_data,
			elements_data[:, 2],
			linewidth=plot_linewidth,
			color=color_str,
		)
	
#		ax_ele[4].plot(
#			elements_date_data,
#			elements_data[:, 0],
#			linewidth=plot_linewidth,
#			color=color_str,
#		)

	###############################################################################################
	#	performing fit

	# with warnings.catch_warnings():
	warnings.simplefilter('ignore', np.RankWarning)

	#	degrees starting point
	degrees = 1
	max_resid = 0.025
	imp_margin = 10

	#	set x, y data
	if plot_in_days is True:
		x = np.subtract(mmt_period_data_date, mmt_period_data_date[0])
	else:
		x = mmt_period_data_date

	if normalise_flag is True:
		y = mmt_period_data[:, 8] / mmt_period_data[0, 8]
	else:
		y = mmt_period_data[:, 8]

	#	generate new x range
	xp = np.arange(np.floor(x[0]), np.ceil(x[-1]) + 1, 1)
	
	#	fit an n-degree polynomal
	resid_data = []
	imp_data = []

	print('\n		#####################################################################')
	print('		Generating polynomials...')

	plot_rest_flag = False

	for deg_itt, loop_degrees in enumerate(degrees_range):

#		#	scatter raw data
#		ax.scatter(
#			x, y,
#			linewidth=plot_linewidth,
#			marker='x', s=marker_size,
#			color=color_str,
#			label=name_str
#		)

		z = np.polyfit(x, y, loop_degrees, full=True)
		poly_coefs = z[0]

		if len(z[1]) != 0:
			residuals = z[1][0]
			resid_data.append([loop_degrees, residuals])
		else:
			break

		rank = z[2]
		singular_vals = z[3]
		rcond = z[4]

		if deg_itt != 0:
			resid_imp = ((prev_resid - residuals) / prev_resid) * 100
#			print('\n		Previous = ', prev_resid)
#			print('		New = ', residuals)
#			print('		Change = ', prev_resid - residuals)
			print('\n			Improvement = ', resid_imp)
			imp_data.append(resid_imp)
			prev_resid = residuals

		else:
			prev_resid = residuals



#		poly_res_sum = np.sum((np.polyval(z, x) - y)**2)
#		rms = rms_calc(y, p(x))
#		rms_data.append([loop_degrees, rms])
#
#
#
#
#		#	generate fit lines
#		p = np.poly1d(poly_coefs)
#		poly_fit_line = p(xp)
#
#		#	plot polynomial fit line
#		ax.plot(
#			xp, poly_fit_line,
#			linestyle=':',
#			linewidth=plot_linewidth,
#			color=color_str
#		)
#
#		if plot_rest_flag is False:
#
#			if norad_id in degrees_data[:, 0]:
#
#					temp_index2 = np.where(degrees_data[:, 0]==norad_id)[0][0]
#
#					fitting_output = [norad_id, degrees_data[temp_index2, 1]]
#					fitting_output_data.append(fitting_output)
#
#					plot_rest_flag = True
#
#			else:
#
#				plt.pause(0.001)
#				ans = input("\n	Satisfactory fit? ['y'es / 'n'o / too 'f'ar]: ")
#		
#				if ans == 'y':
#		
#					fitting_output = [norad_id, loop_degrees]
#					fitting_output_data.append(fitting_output)
#		
#					plot_rest_flag = True
#		
#				elif ans[0] == 'f':
#	
#					decn = int(ans[1:])
#	
#					print('\n	Decreasing degrees by {}...'.format(decn))
#	
#					fitting_output = [norad_id, loop_degrees - decn]
#					fitting_output_data.append(fitting_output)
#	
#					plot_rest_flag = True
#	
#				elif ans == 'n':
#					print('\n	Increasing degrees by 1...')
#
#		ax.cla()
#		ax.set_title('Fitting Plot')
#		ax.set_xlabel(r'$Date/jd$', fontsize=ylabel_fontsize)
#		ax.set_ylabel(r'$Period/s$', fontsize=xlabel_fontsize)
#		ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
#		ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
#		ax.grid(linewidth=grid_linewidth)
#		ax.set_axisbelow(True)

	#	convert to numpy arrays
	resid_data = np.array(resid_data)
	imp_data = np.array(imp_data)

	try:
		min_index1 = np.where(resid_data[:, 1]<max_resid)[0][0]
		print('\n			Index of Minimum = ', min_index1)

		nimp_lines = 0
		min_index2 = 100
		for imp_itt, imp_line in enumerate(imp_data):
			print('				checking imp line: ', imp_line)
			if imp_line < imp_margin:
				nimp_lines += 1
				print('				imp_lines so far: ', nimp_lines)
			else:
				nimp_lines = 0

			if nimp_lines == 2:
				print('				reached 2 no imp lines')
				min_index2 = imp_itt - 1
				print('				index from imp is: ', min_index2)
				break

		if min_index2 < min_index1:
			print('\n			Improved index from: ', min_index1, ' to ', min_index2)
			min_index = min_index2
		else:
			print('\n			Keeping index: ', min_index1)
			min_index = min_index1

	except:

		nimp_lines = 0
		min_index2 = 100
		for imp_itt, imp_line in enumerate(imp_data):
			print('				checking imp line: ', imp_line)
			if imp_line < imp_margin:
				nimp_lines += 1
				print('				imp_lines so far: ', nimp_lines)
			else:
				nimp_lines = 0

			if nimp_lines == 2:
				print('				reached 2 no imp lines')
				min_index = imp_itt - 1
				print('				index from imp is: ', min_index2)
				break


	fitting_output = [norad_id, *resid_data[min_index]]
	fitting_output_data.append(fitting_output)

	if norad_id in degrees_data[:, 0]:

		temp_index2 = np.where(degrees_data[:, 0]==norad_id)[0][0]
		expected = [norad_id, degrees_data[temp_index2, 1]]
		print('\n			Expected Result:	', expected)
#		rms_data_ax.scatter(
#			expected[1],
#			fitting_output[2],
#			color='r',
#			marker='+'
#		)

#		if norad_id in fast_despin:
#			print('			Expected Group:		Fast Despin')
#			if fitting_output[1] > 2:
#				print('GROUP CHANGE')
#				input()
#		if norad_id in slow_despin:
#			print('			Expected Group:		Slow Despin')
#			if fitting_output[1] > 2:
#				print('GROUP CHANGE')
#				input()
#		if norad_id in chaotic:
#			print('			Expected Group:		Chaotic')
#			if fitting_output[1] < 3:
#				print('GROUP CHANGE')
#				input()
#		if norad_id in overlap:
#			print('			Expected Group:		Overlap')

	print('\n			Fitting_output: 	', fitting_output[0:2])

	#	generate final fitting polynomial
	print('\n		Generating final fitting polynomial with degrees = ', fitting_output[1])
	z = np.polyfit(x, y, fitting_output[1], full=True)

	poly_coefs = z[0]
	residuals = z[1][0]
	rank = z[2]
	singular_vals = z[3]
	rcond = z[4]

	print('\n			Fitting Coefficents:	', poly_coefs)
	print('			residuals:		', residuals)
	print('			rank:			', rank)
	print('			singular_vals:		', singular_vals)
	print('			rcond:			', rcond)

	#	append fitting residuals
	fitting_residuals.append([norad_id, residuals])

	p = np.poly1d(poly_coefs)
	poly_fit_line = p(xp)

	if plot_omm is True:
		#	gradient data
		gradient = (poly_fit_line[-1] - poly_fit_line[0]) / (xp[-1] - xp[0])
		grad_data.append([
			np.average(elements_data[:, 4]),
			np.average(elements_data[:, 6]),
			np.average(elements_data[:, 7]),
			gradient
		])

		print('\n		Best fit gradient = ', gradient)




#	if plot_omm is True:
#
#		#	despin groups
#		if fitting_output[1] > 2.0 or gradient < 0.0:
#			continue



#	#	chaotic groups
#	if fitting_output[1] < 3.0:
#		continue









	#	plot to figure
	rms_data_ax.plot(
		resid_data[:, 0],
		resid_data[:, 1],
		color=color_str,
		linewidth=plot_linewidth/2,
	)

	rms_data_ax.scatter(
		resid_data[:, 0],
		resid_data[:, 1],
		color=color_str,
		marker=marker_used,
		s=6,
		label=label_str
	)

	rms_data_ax.scatter(
		fitting_output[1],
		fitting_output[2],
		color='k',
		marker='x'
	)

	#	plot final fit line

	#	scatter raw data
	mmt_period_plot = ax.scatter(
		x, y,
		linewidth=plot_linewidth,
		color=color_str,
		marker=marker_used, s=marker_size,
		label=name_str
	)



	#	plot polynomial fit line
	ax.plot(
		xp, poly_fit_line,
		linestyle=':',
		linewidth=plot_linewidth,
		color=color_str
	)

	if fold_lightcurve is True:

		if normalise_flag is True:
			pdm_plot_data = pdm_period_data[:, 0] / pdm_period_data[0, 0]
		else:
			pdm_plot_data = pdm_period_data[:, 0]

		#	scatter PDM period data
		ax.scatter(
			x, pdm_plot_data,
			linewidth=plot_linewidth,
			color=color_str,
			marker='*', s=marker_size,
		)

		#	scatter PDM period data
		final_plot1 = ax_final.scatter(
			x, pdm_plot_data,
			linewidth=plot_linewidth,
#			color='r',
			color=color_str,
			marker='*', s=marker_size,
#			label=label_str
		)

		print('\nPDM Period Data:')
		print(pdm_period_data[:, 0])




#	print('Colour string = ', color_str)
#	marker_size = 3
#	#	scatter raw data
#	ax_final.plot(
#		x, y,
#		linewidth=plot_linewidth,
#		marker=marker_used, markersize=marker_size,
#		color=color_str,
#		label=label_str
#	)

	print('\nMMT Period Data:')
	print(y)

	#	scatter raw data
	final_plot1 = ax_final.scatter(
		x, y,
		linewidth=plot_linewidth,
		color=color_str,
		marker=marker_used, s=marker_size,
		label=label_str
	)

	#	plot polynomial fit line
	final_plot2 = ax_final.plot(
		xp, poly_fit_line,
		linestyle=':',
		linewidth=plot_linewidth,
		color=color_str
	)

#	#	plot polynomial fit line
#	final_plot2 = ax_final.plot(
#		x, y,
#		linewidth=plot_linewidth,
#		color=color_str,
#		marker=marker_used, markersize=marker_size/2,
#		label=label_str
#	)



	if fold_lightcurve is True:
		#	scatter PDM period data
		theta_ax.plot(
			x, pdm_period_data[:, 1],
			linewidth=plot_linewidth,
			color=color_str,
		)

		#	scatter PDM period data
		theta_ax.scatter(
			x, pdm_period_data[:, 1],
			color=color_str,
			marker=marker_used, s=10,
			label=label_str
		)

		print('\n	Minimum Theta =	', pdm_period_data[:, 1].min())
		min_id = np.where(pdm_period_data[:, 1]==pdm_period_data[:, 1].min())[0][0]
		print('	Track ID =	', pdm_period_data[min_id, -1])

		print('\n	Maximum Theta =	', pdm_period_data[:, 1].max())
		max_id = np.where(pdm_period_data[:, 1]==pdm_period_data[:, 1].max())[0][0]
		print('	Track ID =	', pdm_period_data[max_id, -1])

#		#	scatter PDM period data
#		ax_resid.plot(
#			x, pdm_period_data[:, 2],
#			linewidth=plot_linewidth,
#			color=color_str,
#			marker=marker_used, markersize=5,
#			label=label_str
#		)

	###############################################################################################
	#	calculate run time
	###############################################################################################
	#	calculate sim time
	sim_end = time.time()
	sim_time = np.round((sim_end - sim_start) / 60, 1)

#	plt.pause(0.001)
#	input('\n	Enter to continue to next object...')

#	ax_final.relim()
#	ax_final.autoscale_view()

	#	plt.pause(0.001)
	#	ans = input('\nKeep? ["y" / "n"]: ')
	#	if ans == 'y':
	#		pass
	#	
	#	elif ans == 'n':
	#		final_plot1.remove()
	#		final_plot2[0].remove()
	
	#	input('\nPress enter to continue to next object...')
	
#	plt.pause(0.1)
#	input('\nEnter to Cont...')

#	#	clear RMS plot
#	rms_data_ax.cla()
#	rms_data_ax.set_title('RMS Data Plot')
#	rms_data_ax.set_xlabel(r'$Number\ of\ Degrees$', fontsize=xlabel_fontsize)
#	rms_data_ax.set_ylabel(r'$RMS\ to\ Data$', fontsize=ylabel_fontsize)
#	rms_data_ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
#	rms_data_ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
#	rms_data_ax.set_xticks(degrees_range, minor=False)
#	rms_data_ax.grid(linewidth=grid_linewidth)
#	rms_data_ax.set_axisbelow(True)

	ax.cla()
	ax.set_title('Fitting Plot')
	ax.set_xlabel(r'$Date/jd$', fontsize=ylabel_fontsize)
	ax.set_ylabel(r'$Period/s$', fontsize=xlabel_fontsize)
	ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
	ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
	ax.grid(linewidth=grid_linewidth)
	ax.set_axisbelow(True)




x_range = np.linspace(total_elements_data_ord[:, -1].min(), total_elements_data_ord[:, -1].max(), 1000)
elements_0_avg = np.full(x_range.shape, np.average(total_elements_data_ord[:, 7]))

ax_ele[0].plot(
	x_range,
	elements_0_avg,
	linewidth=plot_linewidth,
	label='Orderly Average',
	linestyle='dashed',
	color='k',
)

elements_1_avg = np.full(x_range.shape, np.average(total_elements_data_ord[:, 6]))

ax_ele[1].plot(
	x_range,
	elements_1_avg,
	linewidth=plot_linewidth,
	linestyle='dashed',
	color='k',
)

elements_2_avg = np.full(x_range.shape, np.average(total_elements_data_ord[:, 2]))

ax_ele[2].plot(
	x_range,
	elements_2_avg,
	linewidth=plot_linewidth,
	linestyle='dashed',
	color='k',
)


print(total_elements_data_cha)
print(total_elements_data_cha.shape)

elements_0_avg = np.full(x_range.shape, np.average(total_elements_data_cha[:, 7]))

ax_ele[0].plot(
	x_range,
	elements_0_avg,
	linewidth=plot_linewidth,
	label='Chaotic Average',
	linestyle='dotted',
	color='k',
)

elements_1_avg = np.full(x_range.shape, np.average(total_elements_data_cha[:, 6]))

ax_ele[1].plot(
	x_range,
	elements_1_avg,
	linewidth=plot_linewidth,
	linestyle='dotted',
	color='k',
)

elements_2_avg = np.full(x_range.shape, np.average(total_elements_data_cha[:, 2]))

ax_ele[2].plot(
	x_range,
	elements_2_avg,
	linewidth=plot_linewidth,
	linestyle='dotted',
	color='k',
)

















max_resid_plot = np.full(degrees_range.shape, max_resid)

#	plot to figure
rms_data_ax.plot(
	degrees_range,
	max_resid_plot,
	color='k',
	linewidth=plot_linewidth + 1,
)




if plot_omm is True:

	grad_data = np.array(grad_data)
	print('\nGradient Data:')
	print('	Minimum = ', grad_data[:, 3].min())
	print('	Maximum = ', grad_data[:, 3].max())
 
# grad_fig, grad_ax = plt.subplots()
# grad_ax.scatter(grad_data[:, 0], grad_data[:, 3], label='SMA')
# grad_ax.scatter(grad_data[:, 1], grad_data[:, 3], label='Apo')
# grad_ax.scatter(grad_data[:, 2], grad_data[:, 3], label='Peri')
# 
# 
# grad_ax.set_xlabel('Average Altitude')
# grad_ax.set_ylabel('Line Gradient')
# 
# grad_ax.legend()

no_orbital_data = np.array(no_orbital_data)

if len(no_orbital_data) != 0:
	print('\nNo Orbital Data for:\n')
	for line in no_orbital_data:
		print('	', line)

# ax_final.relim()
# ax_final.autoscale_view()

#	finalise plot
fig_ele.legend(
	loc='right',
	ncol=1,
	fontsize=legend_fontsize,
	frameon=False
)

#	finalise plot
fig_final.legend(
	loc='right',
	ncol=1,
	fontsize=legend_fontsize,
	frameon=False
)

#	finalise plot
theta_fig.legend(
	loc='right',
	ncol=1,
	fontsize=legend_fontsize,
	frameon=False
)

#	finalise plot
rms_data_fig.legend(
	loc='right',
	ncol=1,
	fontsize=legend_fontsize,
	frameon=False
)

fitting_residuals = np.array(fitting_residuals)
print('\nFitting output:\n')
for line in fitting_residuals:
	print(line[1])


# plt.figure()
# plt.plot(fitting_residuals[:, 1])

plt.pause(0.001)

###############################################################################################
#	Final Figure

if final_savefile_flag is True:

	out_path = os.path.join(fig_path, final_fig_name)

	if os.path.exists(out_path):
		print('\nPath: ', out_path, ' exists.')
		ans = input('\nReplace File? [y/n] ')
	
		if ans == 'y':
			saveflag = True
		else:
			saveflag = False
	
	else:
		saveflag = True
	
	if saveflag == True:
		fig_final.savefig(out_path, bbox_inches="tight", dpi=300)
		print('\n#######################################')
		print('Figure saved: ', final_fig_name)
	else:
		print('\nNot saving: ', final_fig_name)

###############################################################################################
#	Theta Figure

if theta_savefile_flag is True:

	out_path = os.path.join(fig_path, theta_fig_name)

	if os.path.exists(out_path):
		print('\nPath: ', out_path, ' exists.')
		ans = input('\nReplace File? [y/n] ')
	
		if ans == 'y':
			saveflag = True
		else:
			saveflag = False
	
	else:
		saveflag = True
	
	if saveflag == True:
		theta_fig.savefig(out_path, bbox_inches="tight", dpi=300)
		print('\n#######################################')
		print('Figure saved: ', theta_fig_name)
	else:
		print('\nNot saving: ', theta_fig_name)

###############################################################################################
#	OMM Figure

if omm_savefile_flag is True:

	out_path = os.path.join(fig_path, omm_fig_name)

	if os.path.exists(out_path):
		print('\nPath: ', out_path, ' exists.')
		ans = input('\nReplace File? [y/n] ')
	
		if ans == 'y':
			saveflag = True
		else:
			saveflag = False
	
	else:
		saveflag = True
	
	if saveflag == True:
		fig_ele.savefig(out_path, bbox_inches="tight", dpi=300)
		print('\n#######################################')
		print('Figure saved: ', omm_fig_name)
	else:
		print('\nNot saving: ', omm_fig_name)

plt.show()
sys.exit()
