###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import math
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	parallell functions
from starlink_para_funs import attit_brightness

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.colors as colors
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm

from mpl_toolkits.axes_grid1.inset_locator import inset_axes

#	parrellisation
from multiprocessing import Pool
import tqdm

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT Render Orb')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

local_input_path = os.path.join('input_dir', 'mmt_files', 'starlink', 'batch-0')
read_path = os.path.join(path_data, local_input_path)

###################################################################################################
#	script specific initialisation

#	set simulation flags
geom_auto_del = True
render_auto_flag = True

ncores = 8

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Articles',
	'starlink_paper',
	'Figures'
)

fig_name = 'starlink_attit.eps'

#	save flag
savefile_flag = True
savefile_flag = False

###################################################################################################
#	light curve generation inputs
###################################################################################################

#	set initial BRDF values
d_brdf = 0.4
omega_brdf = 0.5
m_brdf = 0.06

mmt_files = [
	'starlink-26_44240.txt',
	'starlink-27_44241.txt',
	'starlink-29_44243.txt',
	'starlink-39_44253.txt',
	'starlink-42_44256.txt'
]

track_index_ranges = np.array([
	[2, 3],
	[3, 4],
	[5, 6],
	[5, 6],
	[6, 7]
])

#	simulation time step
time_step = 0.1

#	set simulation date window
window_start_date = [2019, 5, 25, 0, 0, 0]
window_stop_date = [2020, 6, 1, 0, 0, 0]

####################################################
#	select colour map
cmap = plt.cm.get_cmap('cividis_r')
cmap.set_under('w')
		
###################################################################################################
#	load geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str = 'starlink_full'
geom_file = geom_str + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 0.0

#	attitude motion
rot_period = 0.0

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	figure generation inputs

#	graphical output
inch2cm = 2.54

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4

xlabel_fontsize = 4
ylabel_fontsize = 4

xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
thticks_pad = -6

cbar_pad = 2

xticks_pad = 0
yticks_pad = 0

#	set marker parameters
marker_size = 1
marker_edge = 1

#	set figure line wdith
plot_linewidth = 0.75
grid_linewidth = 0.3

#	set subfigure spacing
hspace_val = 0.0
wspace_val = 0.0
 
#	set use of latex font
plt.rc('text', usetex=True)

#	initialise figure
fig = plt.figure(figsize=(12.0/inch2cm, 24.0/inch2cm), dpi=300)

###############################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

###################################################################################################
#	starlink object loop
###################################################################################################

object_color_index = 0
total_fig_rms = 0

print('\n#####################################################################################')
print('Looping Through Starlink Satellites...')

for object_itt, (mmt_file, track_index_range) in enumerate(zip(mmt_files, track_index_ranges)):

	#	axes 1
	ax1 = plt.subplot(5, 2, (object_itt * 2) + 1, projection="polar")

	ax1.set_theta_zero_location("N")
	ax1.set_theta_direction(-1)
	ax1.tick_params(axis='x', labelsize=xticks_fontsize, pad=thticks_pad)
	ax1.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
	
	ax1.set_rticks(np.arange(0, 100, 20))
	ax1.set_rlim(0, 100)

	#	axes 1
	ax2 = plt.subplot(5, 2, (object_itt * 2) + 2, projection="polar")

	ax2.set_theta_zero_location("N")
	ax2.set_theta_direction(-1)
	ax2.tick_params(axis='x', labelsize=xticks_fontsize, pad=thticks_pad)
	ax2.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
	
	ax2.set_rticks(np.arange(0, 100, 20))
	ax2.set_rlim(0, 100)

	cbar_axes = inset_axes(
		ax1,
		width='5%',
		height='100%',
		bbox_to_anchor=(-1.1, 0.075, 1.0, 1.0),
		bbox_transform=ax1.transAxes
	)

	#	parse filename
	name_str = mmt_file.split('-')[0]
	object_id = int(mmt_file.split('-')[1].split('_')[0])
	norad_id = int(mmt_file.split('-')[1].split('_')[1][:-4])

	###############################################################################################
	#	find RMSE min/max
	###############################################################################################

	total_mag_data = []

	#	loop through tracks
	for track_itt, track_index in enumerate(track_index_range):

		#	output directory
		output_path = os.path.join(
			path_data,
			'lightcurve_dir',
			'starlink_analysis',
			'brightness_analysis'
		)
		
		#	generate output filename
		output_file = '{}-{}_track-{}_attit.txt'.format(
			name_str,
			int(object_id),
			track_index,
		)

		if os.path.isfile(os.path.join(output_path, output_file)):
			print('\n				Reading obs_vec_data file:')
			print('\n					', output_file)
		
			total_lightcurve_data = np.loadtxt(
				os.path.join(output_path, output_file),
				delimiter=','
			)

			total_mag_data.append([total_lightcurve_data[:, 2].min(), total_lightcurve_data[:, 2].max()])
		
		else:

			print('File doesnt exist...')
			sys.exit()

	cbar_mag_min = np.array(total_mag_data).min()
	cbar_mag_max = np.array(total_mag_data).max()

	print('\n	#############################################################################')
	print('	Plotting data for: Starlink-{}'.format(int(object_id)))

	###############################################################################################
	#	load MMT file
	###############################################################################################
	
	#	set time
	mmt_load_start = time.time()
	
	print('\n	[{}] Loading MMT data...'.format(datetime.datetime.now()))
	
	with open(os.path.join(read_path, mmt_file)) as read_file:
		mmt_data = read_file.readlines()
	
	###############################################################################################
	#	parse MMT file
	
	#	strip header
	header_text = mmt_data[7]
	raw_data = mmt_data[8:]
	
	print('\n		Parsing MMT data...')
	
	#	initialise output
	n_rows = 14
	parsed_data = np.zeros([len(raw_data), n_rows])
	
	for i, line in enumerate(raw_data):
	
		#	split line
		date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
			distance, phase, channel, track_id = line.split(' ')
	
		#	parse data and set data types
		mag_std = float(mag_std)
		mag_app = float(mag_app)
		distance = float(distance)
		phase = float(phase)
		track_id = int(track_id)
	
		#	parse date
		year, month, day = date_str.split('-')
		year = int(year)
		month = int(month)
		day = int(day)
	
		#	parse time
		hour, minute, second = time_str.split(':')
		hour = int(hour)
		minute = int(minute)
		second = np.round(float(second), 1)
	
		#	calculate julian date
		jday, jfrac = date2jd(year, month, day, hour, minute, second)
	
		#	generate output line
		output_line = [
			track_id, object_id,
			jday, jfrac,
			year, month, day,
			hour, minute, second,
			mag_std, mag_app,
			distance, phase
		]
	
		#	add to output array
		parsed_data[i, :] = output_line
	
	print('			Total data length = ', len(parsed_data))
	print('			Total number of objects = ', len(np.unique(parsed_data[:, 1])))
	print('			Total number of tracks = ', len(np.unique(parsed_data[:, 0])))
	
	###############################################################################################
	#	extracting data between specified windows
	
	print('\n		Seaching for tracks inside time window:')
	
	#	start window
	window_start_jd = np.add(*date2jd(*window_start_date))
	print('\n			Window start 	= ', window_start_date[0:3])
	
	#	stop window
	window_stop_jd = np.add(*date2jd(*window_stop_date))
	print('			Window stop 	= ', window_stop_date[0:3])
	
	#	extract track ids
	track_id_list = np.unique(parsed_data[:, 0])
	
	#	loop through tracks
	ntracks = 0
	first_lc_flag = True
	for track_id in track_id_list:
	
		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]
	
		#	extract light curve data
		jd_plot = np.add(track_data[:, 2], track_data[:, 3])
		std_mag_plot = track_data[:, 10]
		app_mag_plot = track_data[:, 11]
		distance_plot = track_data[:, 12]
		phase_plot = track_data[:, 13]
		start_line = track_data[0]
		stop_line = track_data[-1]
		start_jdate = np.add(start_line[2], start_line[3])
		stop_jdate = np.add(stop_line[2], stop_line[3])
	
		#	ensure between window start/stop
		if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
	
			#	light curve is within window
			ntracks += 1
	
			#	if first light curve, start array
			if first_lc_flag is True:
				windowed_data = track_data
				first_lc_flag = False
	
			#	else append to list
			elif first_lc_flag is False:
				windowed_data = np.vstack([windowed_data, track_data])
	
	print('\n			{} track(s) extracted.'.format(ntracks))
	
	###############################################################################################
	#	looping through tracks
	
	print('\n	#############################################################################')
	print('	Looping through Tracks...')

	first_plot_flag = True

	#	loop through tracks
	for track_itt, track_index in enumerate(track_index_range):
	
		print('\n	#####################################################################')
		print('	Working on Track = ', track_index)

		if first_plot_flag is True:
			plot_ax = ax1

		elif first_plot_flag is False:
			plot_ax = ax2
		
		###############################################################################################
		#	extracting requested single track
		
		print('\n		Extracting requested track...')
		
		#	set track array index
		track_array_index = track_index - 1
		track_id = int(track_id_list[track_array_index])
		
		track_indices = np.where(windowed_data[:, 0]==track_id)[0]
		track_data = windowed_data[track_indices]
		
		###############################################################################################
		#	extract track time data
		
		mmt_line = track_data[0]
		
		#	parse line elements
		track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]
		
		###############################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
		sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

		#	simulation time
		sim_time = np.linspace(0, lc_duration_sec, lc_len_record)

		###########################################################################################
		#	ensure time continuity
	
		print('\n			Ensuring time continuity in extracted data...')
		
		track_data_temp = []
		sim_index = 0
		
		for itt, line in enumerate(track_data):
		
			#	parse line data
			track_id, object_id, \
				jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = line
			
			step_jd = jday + jfrac
		
			#	calculate current seconds
			current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
		
			output_line = [
				jday, jfrac, current_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]
		
			##################################################
			#	extracting data of time step
		
			#	data match found
			if current_sec == sim_range_sec[sim_index]:
				track_data_temp.append(output_line)
		
				#	itterate number of output data points
				sim_index += 1
		
				#	save current line as previous line
				prev_line = output_line
		
			#	data point is not required
			elif current_sec < sim_range_sec[sim_index]:
				pass
		
			#	interpolation required
			elif current_sec > sim_range_sec[sim_index]:
		
				#	parse previous line
				jday_prev, jfrac_prev, prev_sec, \
					year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
					mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
		
				#	calculate interpolation requirements
				gap = np.round(current_sec - prev_sec, 1)
				npts = int(gap / time_step)
		
				#	get current light curve data
				mag_std_last = mag_std
				mag_app_last = mag_app
				distance_last = distance
				phase_last = phase
		
				#	calculate linear interpolation step values
				mag_std_step = (mag_std_last - mag_std_prev) / npts
				mag_app_step = (mag_app_last - mag_app_prev) / npts
				distance_step = (distance_last - distance_prev) / npts
				phase_step = (phase_last - phase_prev) / npts
		
				#	calculate interpolation data
				for n in range(1, npts + 1):
		
					#	current simulation second
					interp_sec = np.round(prev_sec + (n * time_step), 1)
	
					#	time data
					jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
					jday, jfrac = jday_jfrac(jdate)
					year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
					#	calculate interpolated light curve data
					mag_std = mag_std_prev + (n * mag_std_step)
					mag_app = mag_app_prev + (n * mag_app_step)
					distance = distance_prev + (n * distance_step)
					phase = phase_prev + (n * phase_step)
		
					#	generate new output line
					interp_output_line = [
						jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
		
					if interp_sec == sim_range_sec[sim_index]:
						track_data_temp.append(interp_output_line)
						sim_index += 1
		
				if interp_sec != current_sec:
					#	append final line
					track_data_temp.append(output_line)
					sim_index += 1
		
			else:
				print('ERROR')
				sys.exit()
		
			#	save previous line
			if itt != 0:
				prev_line = output_line
		
			#	array is complete condition
			if len(track_data_temp) == lc_len_record:
				break
		
		#	ensure parsed MMT data matches simulation data
		track_data = np.array(track_data_temp)
		
		if len(track_data) == len(lc_sim_range):
			print('					Simulation and MMT data matches.')
		else:
			print('					Error: Simulation and MMT data does not match...Exiting.')
			sys.exit()
	
		###########################################################################################
		#	extract mmt brightness data
	
		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]
		
		###############################################################################################
		#	display initialisation time
		mmt_load_end = time.time()
		mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)
		
		print('\n		MMT Load Finished.')
		print('\n			- Time taken = {} s.'.format(mmt_load_time))
		
		print('\n		#############################################################################')
		print('		Light curve parameters: ')
		print('			Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(
			*lc_start_date
		))
		print('			Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(
			*lc_stop_date
		))
		print('			Simulation time step:		{}'.format(time_step))
		print('			Duration:			{}		[steps]'.format(lc_len_record))
		print('							{}		[s]'.format(np.round(lc_duration_sec, 1)))
		print('\n		#############################################################################')

		###############################################################################################
		#	acquire TLE
		###############################################################################################
		
		#	set time
		tle_start = time.time()
		
		print('\n		[{}] Acquiring TLEs...'.format(datetime.datetime.now()))
		
		#	TLE request dates, add day either side to increase tle find chance
		tle_start_date = jd2date(*jday_jfrac(lc_start_jd - 1))
		tle_stop_date = jd2date(*jday_jfrac(lc_stop_jd + 1)) 
		
		#	generate TLE file name
		tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
		tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
		tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)
		
		#	assemble TLE path
		tle_path = os.path.join(path_data, 'input_dir', 'tle_files')
		
		#	if exists, load
		if os.path.isfile(os.path.join(tle_path, tle_file)):
		
			print('\n			Reading existing TLE file.')
		
			#	load file
			with open(os.path.join(tle_path, tle_file), 'r') as file:
				tle_data = file.readlines()
		
		else:
			print('\n			Requesting TLE file...')
		
			#	request TLEs
			tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)
		
			#	save file
			with open(os.path.join(tle_path, tle_file), 'w') as file:
				for line in tle_data:
					file.write(line)
		
		#	grab first TLE pair
		tle_line1 = tle_data[1]
		tle_line2 = tle_data[2]
		
		#	mean mot rad/sec
		orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
		orbital_mot = (2 * np.pi) / orbital_period
		
		###############################################################################################
		#	display TLE acquisition time
		tle_end = time.time()
		tle_time = np.round((tle_end - tle_start), 1)
		
		print('\n			TLE Load Finished.')
		print('\n				- Time taken = {} s.'.format(tle_time))
		
		###############################################################################################
		#	select time data to use
		###############################################################################################
		
		sim_range = lc_sim_range
		len_record = lc_len_record
		start_date = lc_start_date
		stop_date = lc_stop_date
		duration_sec = lc_duration_sec
		
		###############################################################################################
		#	generate eop data arrays
		###############################################################################################
		
		#	set time
		eop_start = time.time()
		
		print('\n		[{}]	Generating EOP data array...'.format(datetime.datetime.now()))
		
		#	EOP data
		eop_dir = 'eop_files'
		eop_file = 'eop_parsed.csv'
		dat_file = 'dat_parsed.csv'
		
		#	assemble eop path
		eop_path = os.path.join(path_data, eop_dir)
		
		#	generate interpolated eop_data
		eop_log_level = 0
		eop_data, dat_data = eop_interpolate(
			eop_path,
			eop_file,
			dat_file,
			sim_range,
			time_step,
			eop_log_level
		)
		
		###############################################################################################
		#	display time
		eop_end = time.time()
		eop_time = np.round((eop_end - eop_start), 1)
		
		print('\n			EOP data generated.')
		print('\n				- Time taken = {} s.'.format(eop_time))

		###############################################################################################
		#	generate time objects
		###############################################################################################
		
		#	start dat value
		dat = dat_data[0, -1]
			
		#	start eop values
		eop_line = eop_data[0, :]
		eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
			
		#	generate start time object
		start_time_object = get_times(*start_date, ut_offset, dat)
			
		#	end dat value
		dat = dat_data[-1, -1]
			
		#	end eop values
		eop_line = eop_data[-1, :]
		eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
			
		#	generate stop time object
		stop_time_object = get_times(*stop_date, ut_offset, dat)
		
		###############################################################################################
		#	generate rotation matrices
		###############################################################################################
		
		#	set time
		rot_mat_start = time.time()
		print('\n		[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))
		
		#	tt time
		jdaytt_start = start_time_object.jday_tt
		jfractt_start = start_time_object.jfrac_tt
		jdtt_start = jdaytt_start + jfractt_start
		
		#	ut1 time
		jdayut1_start = start_time_object.jday_ut1
		jfracut1_start = start_time_object.jfrac_ut1
		jdut1_start = jdayut1_start + jfracut1_start
		
		#	generate rotation matrices
		rot_mat_log_level = 0
		pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
			jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
		)
		
		###############################################################################################
		#	display time
		rot_mat_end = time.time()
		rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)
		
		print('\n			Rotation Matrices Generated.')
		print('\n				- Time taken = {} s.'.format(rot_mat_time))
		
		###############################################################################################
		#	MODULE:	Horizons Positions
		###############################################################################################
		
		#	set time
		horz_start = time.time()
		print('\n		[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))
		
		#	set horizons logging level
		horz_logging_level = 0
		
		sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
			path_data, horz_logging_level,
			start_time_object, stop_time_object,
			len_record, time_step,
			eop_path
		)
		
		#	find indices of light curve data
		timecol = sun_icrf[:, 0]
		lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]
		
		###############################################################################################
		#	display time
		horz_end = time.time()
		horz_time = np.round((horz_end - horz_start), 1)
		
		print('\n			Horizons data generated.')
		print('\n				- Time taken = {} s.'.format(horz_time))
		
		###############################################################################################
		#	MODULE:	Object Position
		###############################################################################################
		
		#	set time
		sat_start = time.time()
		print('\n		[{}]	Generating object position data...'.format(datetime.datetime.now()))
		
		satpos_logging_level = 0
		
		sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
			path_data, satpos_logging_level, len_record, time_step,
			start_time_object, stop_time_object,
			norad_id,
			eop_data, dat_data,
			pm_array, pn_array, st_array, c2t_array
		)
		
		###############################################################################################
		#	display time
		sat_end = time.time()
		sat_time = np.round((sat_end - sat_start), 1)
		
		print('\n			Object position data generated.')
		print('\n				- Time taken = {} s.'.format(sat_time))
		
		###############################################################################################
		#	MODULE:	Transformation
		###############################################################################################
		
		#	set time
		trans_start = time.time()
		print('\n		[{}]	Performing transformations...'.format(datetime.datetime.now()))
		
		identification_str = 'mmt-render'
		
		trans_logging_level = 0
		save_flag = False
		
		sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
			= module_transformation(
				path_data, trans_logging_level, save_flag,
				start_time_object, stop_time_object,
				identification_str, site_str,
				len_record, time_step,
				eop_data, dat_data,
				pm_array, pn_array, st_array, c2t_array,
				sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
			)
		
		###############################################################################################
		#	display time
		trans_end = time.time()
		trans_time = np.round((trans_end - trans_start), 1)
		
		print('\n			Transformations complete.')
		print('\n				- Time taken = {} s.'.format(trans_time))
			
		###########################################################################################
		#	generate brdf
		
		#	Refractive Index
		n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))
		
		#	Specular ratio
		s_brdf = 1.0 - d_brdf
		
		#	generate brdf
		brdf_data = np.zeros([nfaces, 5])
		
		for i in range(nfaces):
			brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
		
		###########################################################################################
		#	generate polar angles
		
		#	set light curve time step
		lc_time_step = time_step
		
		#	extract light curve polar data
		sun_polar_data, site_polar_data, phase_angles, jd_time_data = extract_fun(
			sim_range, lc_time_step, time_step,
			sun_orb, earth_orb, site_orb, sat_orb
		)





		###########################################################################################
		#	generate mesh grid ranges
		###########################################################################################
		
		#	set earth radius
		RE = 6371
		
		#	using actual distance
		earth_vec_temp = earth_orb[0, 1:]

		#	earth unit vector
		earth_vec_unit = np.round(np.divide(earth_vec_temp, np.linalg.norm(earth_vec_temp)), 4)

		#	satellite r vector
		sat_r = np.linalg.norm(earth_vec_temp)
		
		#	satellite orbital altitude
		sat_h = np.linalg.norm(earth_vec_temp) - RE
		
		#	calculate range along slant vector
		site_range = np.linalg.norm(site_orb[0, 1:])
		
		#	convert to vector
		site_range_vec = np.array([-site_range, 0.0, 0.0])
		sat_height_vec = np.array([-sat_h, 0.0, 0.0])
		
		#	calculate maximum inclination of earth surface
		horz_inc = np.arcsin(RE / sat_r)
		
		#	generate range of azimuth/inclination values
		attit_inc_vals = np.arange(0, 80 + 10, 10)
		attit_azm_vals = np.arange(0, 350 + 10, 10)
		
		#	generate polar mesh grid
		attit_inc_mesh, attit_azm_mesh = np.meshgrid(attit_inc_vals, attit_azm_vals)

		###############################################################################################
		#	parallelised function
		###############################################################################################
		
		parral_flag = False
		parral_flag = True
		
		#	output directory
		output_path = os.path.join(
			path_data,
			'lightcurve_dir',
			'starlink_analysis',
			'brightness_analysis'
		)
		
		#	generate output filename
		output_file = '{}-{}_track-{}_attit.txt'.format(
			name_str,
			int(object_id),
			track_index,
		)
		
		###############################################################################################
		#	load file if exists
		
		if os.path.isfile(os.path.join(output_path, output_file)):
			print('\n				Reading obs_vec_data file:')
			print('\n					', output_file)
		
			total_lightcurve_data = np.loadtxt(
				os.path.join(output_path, output_file),
				delimiter=','
			)
		
		else:
		
			###########################################################################################
			#	generate an array of inputs
			print('\n				Generating input data...')
		
			inputs = []
			for attit_inc_val in attit_inc_vals:
				for attit_azm_val in attit_azm_vals:
			
					inputs.append([
						path_data, start_jd, len_record, time_step, duration_sec,
						attit_inc_val, attit_azm_val, orbital_mot,
						sat_orb, sun_orb, earth_orb, site_orb,
						rot_period, body_mesh, brdf_data
					])
			
			inputs = np.array(inputs)
			print('\n					Done...')
		
			###########################################################################################
			#	parallelised function
			
			total_lightcurve_data = []
		
			print('\n				Running brightness function...')
		
			if parral_flag is True:
						
				fun_start = time.time()
		
				#	initialise multiproccessing pool of workers
				p = Pool(ncores)
				
				#	map the total data array across the access function
				for light_curve_output in tqdm.tqdm(
					p.imap_unordered(attit_brightness, inputs), total=len(inputs)
					):
				
					#	parse outputs
					total_lightcurve_data.append(light_curve_output)
				
				#	calculate run time
				fun_end = time.time()
				fun_time = np.round((fun_end - fun_start), 1)
				
				print('\n			- Time spent in parallelisation function = {} s.'.format(fun_time))
			
			elif parral_flag is False:
			
				#	run inputs sequentially
				for temp_itt, input_line in enumerate(inputs):
		
					light_curve_output = attit_brightness(input_line)
					temp_site_vec = light_curve_output[-1]
		
					#	parse outputs
					total_lightcurve_data.append(light_curve_output)

			total_lightcurve_data = np.array(total_lightcurve_data)

			#######################################################################################
			#	save array to avoid regeneration
			np.savetxt(
				os.path.join(output_path, output_file),
				total_lightcurve_data,
				delimiter=','
			)
			
			print('\n			File saved: ', output_file)

		###########################################################################################
		#	parse output data
	
		#	reinitialise output array
		attit_data = np.zeros(attit_inc_mesh.shape)
		
		for line_itt, line in enumerate(total_lightcurve_data):
		
			attit_inc_val, attit_azm_val, average_mag, min_mag, max_mag = line

			#	correct azm
			if attit_azm_val >= 180:
				attit_azm_val -= 180

			elif attit_azm_val < 180:
				attit_azm_val += 180

			#	find occurance in inc/azm grids
			attit_inc_index = np.where(np.round(attit_inc_mesh[0, :], 2)==attit_inc_val)[0][0]
			attit_azm_index = np.where(np.round(attit_azm_mesh[:, 0], 2)==attit_azm_val)[0][0]

			#	set which brightness metric to use
			brightness_val = min_mag
			brightness_val = average_mag

			if attit_inc_val == 0.0 and attit_azm_val == 0.0:
				#	set entire row
				attit_data[:, attit_inc_index] = brightness_val
			else:
				attit_data[attit_azm_index, attit_inc_index] = brightness_val

		print('\nazm_mesh = ', attit_azm_mesh.shape)
		print('\ninc mesh = ', attit_inc_mesh.shape)
	
		print('\nattit data = ', attit_data)
		print(attit_data.shape)
		sys.exit()

		###############################################################################################
		#	graphical outputs
		###############################################################################################
		
		print('\n		Generating Figure...')

		#	generate color string
		color_str = 'C' + str(track_itt)

		###############################################################################################
		#	plot polar plots
		
		#	find azimuth and inclination offsets to align with grid
		nazm_vals = len(attit_inc_mesh) - 1
		azm_offset = (360 / nazm_vals) / 2
		inc_offset = 10
		
		#	generate range of plotting azm values
		azm_step = 10
		plot_azm_vals = np.arange(0, 370, 10)

		#	generate Earth inclination limit plot data
		horz_inc_vals = np.full(attit_inc_vals.shape, horz_inc + np.radians(inc_offset))
		horz_inc_r, horz_inc_th = np.meshgrid(horz_inc_vals, plot_azm_vals)

		#	add addditional rows to plot data to complete circle
		attit_inc_mesh = np.vstack([attit_inc_mesh, attit_inc_mesh[-1, :]])
		attit_azm_mesh = np.vstack([attit_azm_mesh, np.add(attit_azm_mesh[-1, :], azm_step)])
		attit_data = np.vstack([attit_data, np.full(attit_data[-1, :].shape, 0)])
		
		#	add addditional cols to plot data to complete circle
		attit_inc_mesh = np.hstack([
			attit_inc_mesh,
			np.reshape(
				attit_inc_mesh[:, -1] + 10,
				[len(attit_inc_mesh[:, -1]), 1]
			)
		])
		
		attit_azm_mesh = np.hstack([
			attit_azm_mesh,
			np.reshape(
				attit_azm_mesh[:, -1],
				[len(attit_azm_mesh[:, -1]), 1]
			)
		])
		
		attit_data = np.hstack([
			attit_data,
			np.reshape(
				np.full(attit_data[:, -1].shape, 0),
				[len(attit_data[:, -1]), 1]
			)
		])

		####################################################
		#	colour the plot
		
		n_col = 100

#		cbar_mag_min = 0.0
		bounds = np.linspace(cbar_mag_min, cbar_mag_max, n_col)
		
		#	generate colour bar bounds
		norm = colors.BoundaryNorm(boundaries=bounds, ncolors=256)



		azm_set = 0
		azm_id = np.where(np.round(attit_azm_mesh[:, 0], 2)==azm_set)[0][0]

		azm_set2 = 360
		azm_id2 = np.where(np.round(attit_azm_mesh[:, 0], 2)==azm_set2)[0][0] + 2

		inc_set = 0
		inc_id = np.where(np.round(attit_inc_mesh[0, :], 2)==inc_set)[0][0]

		inc_set2 = 90
		inc_id2 = np.where(np.round(attit_inc_mesh[0, :], 2)==inc_set2)[0][0] + 2

		pcm = plot_ax.pcolor(
			np.radians(attit_azm_mesh - azm_offset)[azm_id:azm_id2, inc_id:inc_id2],
			attit_inc_mesh[azm_id:azm_id2, inc_id:inc_id2],
			attit_data[azm_id:azm_id2, inc_id:inc_id2],
			norm=norm,
			cmap=cmap
		)
		
		
		print('\nazm_mesh = ', attit_azm_mesh[azm_id:azm_id2, inc_id:inc_id2])
		print('\ninc mesh = ', attit_inc_mesh[azm_id:azm_id2, inc_id:inc_id2])

		print('\nattit data = ', attit_data[azm_id:azm_id2, inc_id:inc_id2])
		print(attit_data[azm_id:azm_id2, inc_id:inc_id2].shape)

		####################################################
		#	plot Earth outline
		plot_ax.plot(
			np.radians(plot_azm_vals), np.degrees(horz_inc_r),
			color='white',
			lw=plot_linewidth,
			linestyle='dashed'
		)
		
		####################################################
		#	plot the intial actual observatory position
		plot_ax.plot(site_polar_data[0, 0], np.add(np.degrees(site_polar_data[0, 1]), inc_offset),
			marker='x',
			color='white',
			markersize=marker_size,
			markeredgewidth=marker_edge,
			clip_on=False
		)

		#	plot rest of site locations
		marker_site = [len(site_polar_data) - 1]
		
		#	plot site positions
		plot_ax.plot(
			site_polar_data[:, 0],
			np.add(np.degrees(site_polar_data[:, 1]), inc_offset),
			linewidth=plot_linewidth, color='white', markevery=marker_site, marker='<',
			markersize=marker_size
		)

		####################################################
		#	plot sun position
		plot_ax.plot(sun_polar_data[0, 0], np.add(np.degrees(sun_polar_data[0, 1]), inc_offset),
			marker='x',
			color='r',
			markersize=marker_size,
			markeredgewidth=marker_edge,
			linewidth=plot_linewidth,
			clip_on=False
		)
		
		#	plot rest of site locations
		marker_site = [len(sun_polar_data) - 1]
		
		#	plot sun positions
		plot_ax.plot(
			sun_polar_data[:, 0],
			np.add(np.degrees(sun_polar_data[:, 1]), inc_offset),
			linewidth=plot_linewidth, color='r', markevery=marker_site, marker='<',
			markersize=marker_size
		)

		plot_ax.grid(which='major', axis='both', linewidth=grid_linewidth)

		###############################################################################################
		#	calculate run time
		###############################################################################################

		#	calculate sim time
		sim_end = time.time()
		sim_time = np.round((sim_end - sim_start) / 60, 1)
		
		print('\n		[{}] Simulation Finished.'.format(datetime.datetime.now()))
		print('\n			- Time taken = {} mins.'.format(sim_time))

		print('\n\nShow Plot Here...\n\n')
		first_plot_flag = False


		plt.show()
		sys.exit()

	####################################################
	#	add colour bar

	cbar = fig.colorbar(pcm, cax=cbar_axes)
	
	cbar_axes.tick_params(axis='y', labelsize=cbar_fontsize, pad=cbar_pad, length=2)
	cbar_axes.tick_params(axis='x', which='both', bottom=False, labelbottom=False)
	cbar_axes.yaxis.set_ticks_position('left')

###############################################################################################
#	save figure
###############################################################################################

if savefile_flag is True:

	out_path = os.path.join(fig_path, fig_name)
	fig.savefig(out_path, bbox_inches="tight", dpi=300)

	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
sys.exit()
