###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import BoundaryNorm
from matplotlib import gridspec

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			BRDF Determination')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

local_input_path = os.path.join('input_dir', 'mmt_files', 'starlink', 'batch-0')
read_path = os.path.join(path_data, local_input_path)

###################################################################################################
#	script specific initialisation

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Articles',
	'starlink_paper',
	'Figures'
)

#	save flag
savefile_flag = True
savefile_flag = False

#	simulation flags
plot_all_minima_flag = False
rms_norm_flag = True

#	set solution space constraints
#	m_brdf range
min_m_brdf = 0.05
max_m_brdf = 0.50

#	omega_brdf range
min_omega_brdf = 0.2
max_omega_brdf = 0.8

#	m_brdf range
min_m_brdf = 0.01
max_m_brdf = 0.99

#	omega_brdf range
min_omega_brdf = 0.01
max_omega_brdf = 0.99

###################################################################################################
#	light curve generation inputs
###################################################################################################

mmt_file = 'starlink-29_44243.txt'

#	parse filename
name_str = mmt_file.split('-')[0]
object_id = int(mmt_file.split('-')[1].split('_')[0])
norad_id = int(mmt_file.split('-')[1].split('_')[1][:-4])

#	simulation time step
time_step = 0.1

#	set simulation date window
window_start_date = [2019, 5, 25, 0, 0, 0]
window_stop_date = [2020, 6, 1, 0, 0, 0]

###################################################################################################
#	load geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str = 'starlink_payload'
geom_file = geom_str + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 0.0

#	attitude motion
rot_period = 0.0

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	simulation inputs

#	set range of d_brdf value
d_brdf_plot_range = [0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8]

d_brdf_plot_range = [0.2, 0.3, 0.4, 0.5, 0.6, 0.7]

#	generate range of track indices
track_index_range = [1, 2, 3, 4, 5]
track_index_range = [1, 2, 3, 4, 7, 8]

track_index_range = [1]

#	range for brdf parameters
step = 0.01
m_brdf_range = np.arange(0.01, 0.99 + step, step)
omega_brdf_range = np.arange(0.01, 0.99 + step, step)

###############################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

print('\n	#############################################################################')
print('	Simulation Inputs Set To: ')
print('\n	Input File: ')
print('\n		Working from MMT data input file: 	{}'.format(mmt_file))
print('		Reading from input path: 		{}'.format(local_input_path))
print('\n			Name:	 	{}'.format(name_str))
print('			Object ID: 	{}'.format(object_id))
print('			Norad ID: 	{}'.format(norad_id))
# print('			Track ID: 	{}'.format(track_index))
print('\n	Light Curve Inputs: ')
print('\n			Geom File: 		{}'.format(geom_file))
print('			Rot Vec Azimuth: 	{}'.format(rot_vec_azm_deg))
print('			Rot Vec Inclination: 	{}'.format(rot_vec_inc_deg))
print('			Rotation Period: 	{}'.format(rot_period))
# print('			BRDF: 			m = {}, omega = {}, m = {}'.format(d_brdf, omega_brdf, m_brdf))
print('\n	Observation Site Inputs: ')
print('\n			Site String: 	{}'.format(site_str))
print('			Latitude: 	{}'.format(np.degrees(site_latgd)))
print('			Longitude: 	{}'.format(np.degrees(site_lon)))
print('			Altitude: 	{} m'.format(site_alt))

###################################################################################################
#	load MMT file
###################################################################################################

#	set time
mmt_load_start = time.time()

print('\n[{}] Loading MMT data...'.format(datetime.datetime.now()))

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file

#	strip header
header_text = mmt_data[7]
raw_data = mmt_data[8:]

print('\n	Parsing MMT data...')

#	initialise output
n_rows = 14
parsed_data = np.zeros([len(raw_data), n_rows])

for i, line in enumerate(raw_data):

	#	split line
	date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
		distance, phase, channel, track_id = line.split(' ')

	#	parse data and set data types
	mag_std = float(mag_std)
	mag_app = float(mag_app)
	distance = float(distance)
	phase = float(phase)
	track_id = int(track_id)

	#	parse date
	year, month, day = date_str.split('-')
	year = int(year)
	month = int(month)
	day = int(day)

	#	parse time
	hour, minute, second = time_str.split(':')
	hour = int(hour)
	minute = int(minute)
	second = np.round(float(second), 1)

	#	calculate julian date
	jday, jfrac = date2jd(year, month, day, hour, minute, second)

	#	generate output line
	output_line = [
		track_id, object_id,
		jday, jfrac,
		year, month, day,
		hour, minute, second,
		mag_std, mag_app,
		distance, phase
	]

	#	add to output array
	parsed_data[i, :] = output_line

print('		Total data length = ', len(parsed_data))
print('		Total number of objects = ', len(np.unique(parsed_data[:, 1])))
print('		Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

###################################################################################################
#	extracting data between specified windows

print('\n	Seaching for tracks inside time window:')

#	start window
window_start_jd = np.add(*date2jd(*window_start_date))
print('\n		Window start 	= ', window_start_date[0:3])

#	stop window
window_stop_jd = np.add(*date2jd(*window_stop_date))
print('		Window stop 	= ', window_stop_date[0:3])

#	extract track ids
track_id_list = np.unique(parsed_data[:, 0])

#	loop through tracks
ntracks = 0
first_lc_flag = True
for track_id in track_id_list:

	#	find track ids
	track_indices = np.where(parsed_data[:, 0]==track_id)[0]

	#	extract full data
	track_data = parsed_data[track_indices, :]

	#	extract light curve data
	jd_plot = np.add(track_data[:, 2], track_data[:, 3])
	std_mag_plot = track_data[:, 10]
	app_mag_plot = track_data[:, 11]
	distance_plot = track_data[:, 12]
	phase_plot = track_data[:, 13]
	start_line = track_data[0]
	stop_line = track_data[-1]
	start_jdate = np.add(start_line[2], start_line[3])
	stop_jdate = np.add(stop_line[2], stop_line[3])

	#	ensure between window start/stop
	if start_jdate > window_start_jd and stop_jdate < window_stop_jd:

		#	light curve is within window
		ntracks += 1

		#	if first light curve, start array
		if first_lc_flag is True:
			windowed_data = track_data
			first_lc_flag = False

		#	else append to list
		elif first_lc_flag is False:
			windowed_data = np.vstack([windowed_data, track_data])

print('\n		{} track(s) extracted.'.format(ntracks))

###################################################################################################
#	display initialisation time
mmt_load_end = time.time()
mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)

print('\n	MMT Load Finished.')
print('\n		- Time taken = {} s.'.format(mmt_load_time))

###################################################################################################
#	loop through track indices
###################################################################################################

###################################################################################################
#	figure generation inputs

#	generate mesh grids
omega_mesh, m_mesh = np.meshgrid(omega_brdf_range, m_brdf_range)

#	calculate solution space thresholds
#	min m index
m_id_min = np.where(m_mesh[:, 0]==min_m_brdf)[0][0]
m_id_max = np.where(m_mesh[:, 0]==max_m_brdf)[0][0] + 1
#	min omega index
omega_id_min = np.where(np.round(omega_mesh[0, :], 2)==min_omega_brdf)[0][0]
omega_id_max = np.where(np.round(omega_mesh[0, :], 2)==max_omega_brdf)[0][0] + 1

#	apply thresholds
m_mesh_plot = m_mesh[omega_id_min:omega_id_max, m_id_min:m_id_max]
omega_mesh_plot = omega_mesh[omega_id_min:omega_id_max, m_id_min:m_id_max]

#	initialise total rms of d array
total_rms_of_d = np.zeros(len(d_brdf_plot_range))

#	initialise track rms array
track_rms_data = np.zeros([len(track_index_range), 3])

#	initialise total rms array
total_rms_values = np.zeros([m_mesh_plot.shape[0], m_mesh_plot.shape[1], len(d_brdf_plot_range)])

###################################################################################################
#	figure generation paramters

#	inch to cm conversion
inch2cm = 2.54

#	set colour levels
n_cols = 100

#	set subplot spacing values
hspace_val = 0.35
wspace_val = 0.35

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4
xlabel_fontsize = 6
ylabel_fontsize = 6
xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set marker parameters
marker_size = 3

#	set figure line wdith
plot_linewidth = 1.0
 
#	set use of LaTeX font
plt.rc('text', usetex=True)

#	set figure height/width
fig_width = 10.0/inch2cm
fig_height = 4.0/inch2cm

#	initialise figure axes
fig2, ax2 = plt.subplots(1, 2, figsize=(fig_width, fig_height), dpi=300)

#	adjust subplot spacing
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)
plt.gcf().subplots_adjust(bottom=0.2)

#	initialise figure axes
fig3, ax3 = plt.subplots(1, 2, figsize=(fig_width, fig_height),	dpi=300)

#	adjust subplot spacing
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)
plt.gcf().subplots_adjust(bottom=0.2)

print('\n#####################################################################################')
print('Looping Through Tracks...')

#	loop initialisation
light_curve_index = 1
color_index = 0
first_track = True

#	loop through tracks
for track_index in track_index_range:

	###############################################################################################
	#	initialise track
	###############################################################################################

	print('\n	#############################################################################')
	print('	Working on Track = ', track_index)

	#	output directory
	lightcurve_path = os.path.join(
		path_data,
		'lightcurve_dir',
		'starlink_analysis',
		name_str + '-{}'.format(int(object_id)),
		'track-{}'.format(int(track_index))
	)

	###############################################################################################
	#	extracting requested single track
	
	print('\n		Extracting requested track index...')
	
	#	set track array index
	track_array_index = track_index - 1
	track_id = int(track_id_list[track_array_index])

	track_indices = np.where(windowed_data[:, 0]==track_id)[0]
	track_data = windowed_data[track_indices]
		
	###############################################################################################
	#	extract track time data

	mmt_line = track_data[0]
	
	#	parse line elements
	track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase = track_data[0]
	
	start_jd = [jday, jfrac]
	
	#	parse line elements
	track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase = track_data[-1]
	
	stop_jd = [jday, jfrac]
	
	###################################################################################################
	#	generate simulation time data
	
	#	extract start and stop jds
	lc_start_jd = start_jd[0] + start_jd[1]
	lc_stop_jd = stop_jd[0] + stop_jd[1]
	
	#	convert to dates
	lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
	lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
	
	#	light curve duration
	lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
	
	#	set time step
	lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
	
	#	time range in julian days
	lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
	sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)
	
	###############################################################################################
	#	setting simulation time data
	###############################################################################################
	
	###############################################################################################
	#	extract track time data
	
	mmt_line = track_data[0]
	
	#	parse line elements
	track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase = track_data[0]
	
	start_jd = [jday, jfrac]
	
	#	parse line elements
	track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase = track_data[-1]
	
	stop_jd = [jday, jfrac]
	
	###############################################################################################
	#	generate simulation time data
	
	#	extract start and stop jds
	lc_start_jd = start_jd[0] + start_jd[1]
	lc_stop_jd = stop_jd[0] + stop_jd[1]
	
	#	convert to dates
	lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
	lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
	
	#	light curve duration
	lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
	
	#	set time step
	lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
	
	#	time range in julian days
	lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
	sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

	###################################################################################################
	#	ensure time continuity
	###################################################################################################

	print('\n		Ensuring time continuity in extracted data...')
	
	track_data_temp = []
	sim_index = 0
	
	for itt, line in enumerate(track_data):
	
		#	parse line data
		track_id, object_id, \
			jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = line
		
		step_jd = jday + jfrac
	
		#	calculate current seconds
		current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
	
		output_line = [
			jday, jfrac, current_sec, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase
		]
	
		##################################################
		#	extracting data of time step
	
		#	data match found
		if current_sec == sim_range_sec[sim_index]:
			track_data_temp.append(output_line)
	
			#	itterate number of output data points
			sim_index += 1
	
			#	save current line as previous line
			prev_line = output_line
	
		#	data point is not required
		elif current_sec < sim_range_sec[sim_index]:
			pass
	
		#	interpolation required
		elif current_sec > sim_range_sec[sim_index]:
	
			#	parse previous line
			jday_prev, jfrac_prev, prev_sec, \
				year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
				mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
	
			#	calculate interpolation requirements
			gap = np.round(current_sec - prev_sec, 1)
			npts = int(gap / time_step)
	
			#	get current light curve data
			mag_std_last = mag_std
			mag_app_last = mag_app
			distance_last = distance
			phase_last = phase
	
			#	calculate linear interpolation step values
			mag_std_step = (mag_std_last - mag_std_prev) / npts
			mag_app_step = (mag_app_last - mag_app_prev) / npts
			distance_step = (distance_last - distance_prev) / npts
			phase_step = (phase_last - phase_prev) / npts
	
			#	calculate interpolation data
			for n in range(1, npts + 1):
	
				#	current simulation second
				interp_sec = np.round(prev_sec + (n * time_step), 1)

				#	time data
				jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
				jday, jfrac = jday_jfrac(jdate)
				year, month, day, hour, minute, second = jd2date(jday, jfrac)
	
				#	calculate interpolated light curve data
				mag_std = mag_std_prev + (n * mag_std_step)
				mag_app = mag_app_prev + (n * mag_app_step)
				distance = distance_prev + (n * distance_step)
				phase = phase_prev + (n * phase_step)
	
				#	generate new output line
				interp_output_line = [
					jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
					mag_std, mag_app, distance, phase
				]
	
				if interp_sec == sim_range_sec[sim_index]:
					track_data_temp.append(interp_output_line)
					sim_index += 1
	
			if interp_sec != current_sec:
				#	append final line
				track_data_temp.append(output_line)
				sim_index += 1
	
		else:
			print('ERROR')
			sys.exit()
	
		#	save previous line
		if itt != 0:
			prev_line = output_line
	
		#	array is complete condition
		if len(track_data_temp) == lc_len_record:
			break
	
	#	ensure parsed MMT data matches simulation data
	track_data = np.array(track_data_temp)
	
	if len(track_data) == len(lc_sim_range):
		print('					Simulation and MMT data matches.')
	else:
		print('					Error: Simulation and MMT data does not match...Exiting.')
		sys.exit()
	
	###############################################################################################
	#	extract mmt brightness data

	mmt_signal_std = track_data[:, 9]
	mmt_signal_app = track_data[:, 10]
	
	print('\n		#####################################################################')
	print('		Extracted MMT track parameters: ')
	print(
		'			Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
		.format(*lc_start_date)
	)
	print(
		'			Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'\
		.format(*lc_stop_date)
	)
	print('			Simulation time step:		{}'.format(time_step))
	print('			Duration:			{}		[steps]'.format(lc_len_record))
	print('							{}		[s]'.format(np.round(lc_duration_sec, 1)))

	###############################################################################################
	#	extract plot data
	###############################################################################################
	
	#	initialise RMS of d array
	rms_of_d = np.zeros(len(d_brdf_plot_range))
	coords_of_d = np.zeros([len(d_brdf_plot_range), 2])
	
	print('\n		#####################################################################')
	print('		Looping through d_brdf values...')

	max_minima = 1
	minima_plot_data = np.zeros([25, len(d_brdf_plot_range), 2])

	for plot_id, d_brdf in enumerate(d_brdf_plot_range):
	
		print('\n			#############################################################')
		print('			Performing case d = {}...'.format(d_brdf))
	
		###########################################################################################
		#	read total fitting light curve file
	
		in_dir = 'd_brdf=0-{:02d}'.format(int(d_brdf * 100))
	
		#	assemble input file name
		if track_index in [7, 8]:
			read_file = '{}-{}_track-{}_az-{}_inc-{}_d-{}.txt'.format(
				name_str,
				int(object_id),
				track_index,
				np.round(rot_vec_azm_deg, 2),
				np.round(rot_vec_inc_deg, 2),
				np.round(d_brdf, 2),
			)

		else:
			read_file = '{}-{}_track-{}_az-{}_inc-{}_d-{}.txt'.format(
				name_str,
				int(object_id),
				track_index,
				np.round(rot_vec_azm_deg, 2),
				np.round(90.0, 2),
				np.round(d_brdf, 2),
			)
	
		#	generate the rms values output file name
		rms_values_file = read_file[:-4] + '.rms_values'

		print('\n			Looking for: {}'.format(os.path.join(in_dir, rms_values_file)))

		if os.path.isfile(os.path.join(lightcurve_path, in_dir, rms_values_file)):

			print('\n				Found: Reading .rms_values file...')
			track_rms_values = np.loadtxt(
				os.path.join(lightcurve_path, in_dir, rms_values_file),
				delimiter=','
			)

		else:
	
			print('\n			Not Found: Generating .rms_values file...')
			track_rms_values = np.zeros(omega_mesh.shape)
	
			#	total data array
			print('\n				Loading Light Curve Data File...\n')
			total_data = np.loadtxt(os.path.join(lightcurve_path, in_dir, read_file), delimiter=',')
		
			#	extract light curve indices
			light_curve_indices = np.unique(total_data[:, 0])
		

			print(len(total_data))
			print('\nNumber of Light Curve Indices = ', len(light_curve_indices))
			sys.exit()

			for light_curve_index in light_curve_indices:
		
				if light_curve_index % 10 == 0.0:
					print(
						'					Working on Light Curve ID: {}'.format(int(light_curve_index)),
						end='\r'
					)
	
				#	light curve data of index
				data_indices = np.where(total_data[:, 0]==light_curve_index)[0]
				light_curve_data = total_data[data_indices]
	
				#	parse data from light curve array
				light_curve_seconds = light_curve_data[:, 2]
				synth_signal_app = light_curve_data[:, 3]
				synth_signal_std = light_curve_data[:, 4]
	
				lc_index, jdate, sec_since, mag_app, mag_std, \
					object_id, rot_vec_azm_deg, rot_vec_inc_deg, \
					d_brdf, m_brdf, omega_brdf = light_curve_data[0]
	
				#	calculate RMSE
				rms_std = rms_calc(synth_signal_std, mmt_signal_std)
				rms_app = rms_calc(synth_signal_app, mmt_signal_app)
	
				omega_col = np.where(omega_mesh[0, :]==omega_brdf)[0]
				m_row = np.where(m_mesh[:, 0]==m_brdf)[0]
	
				#	find occurance in omega/m grids
				omega_index = np.where(np.round(omega_mesh[0, :], 2)==omega_brdf)[0]
				m_index = np.where(np.round(m_mesh[:, 0], 2)==m_brdf)[0]

				track_rms_values[omega_index, m_index] = rms_app

			print('\n		Finished.')

			###########################################################################################
			#	save rms_values array to avoid regeneration
		
			np.savetxt(
				os.path.join(lightcurve_path, in_dir, rms_values_file),
				track_rms_values,
				delimiter=','
			)
	
			print('\n	RMS Values file saved: ', rms_values_file)

		###########################################################################################
		#	apply threshold
		track_rms_values = track_rms_values[
			omega_id_min:omega_id_max,
			m_id_min:m_id_max
		]

		print('\n				Maximum RMS = ', track_rms_values.max())
		print('				Minimum RMS = ', track_rms_values.min())

		print(track_rms_values)
		print(track_rms_values.shape)
	
		#	if normalisation flag is true, normalise
		if rms_norm_flag is True:
			print('\n				Normalising RMS...')
			track_rms_values = np.divide(track_rms_values, track_rms_values.max())

		total_rms_values[:, :, plot_id] = np.add(total_rms_values[:, :, plot_id], track_rms_values)
		print('\n				RMS data added to total array')

		###############################################################################################
		#	generating properties plot data

		print('\n			Searching for minimum RMS location...')
		#	save minimum RMS value
		rms_of_d[plot_id] = track_rms_values.min()

		#	round to reasonable decimal places
		track_rms_values = np.round(track_rms_values, 10)
		min_rms = np.round(track_rms_values.min(), 10)

		first_match_flag = True

		for row_index, line in enumerate(track_rms_values):
			for col_index, value in enumerate(line):
				if value == min_rms:

					#	apply threshold offset
					col_coord = col_index# + m_id_min
					row_coord = row_index# + omega_id_min

					if first_match_flag is True:
						min_rms_coord = [col_coord, row_coord]
						first_match_flag = False
					else:
						min_rms_coord = np.vstack([min_rms_coord, [col_coord, row_coord]])

		min_rms_coord = np.array(min_rms_coord)
		n_minima = len(min_rms_coord)

		print('\n				Number of Minima: ', n_minima)

		#	set new max if increased
		if max_minima < n_minima:
			max_minima = n_minima

		#	if only a single coordinate
		if min_rms_coord.shape == (2,):

			#	find omega and m coordinates
			omega_coord = omega_mesh[min_rms_coord[0], min_rms_coord[1]]
			m_coord = m_mesh[min_rms_coord[0], min_rms_coord[1]]

		else:
			#	working through multiple minima
			for minima_index, min_coord in enumerate(min_rms_coord):
	
				#	find omega and m coordinates
				omega_coord = omega_mesh[min_coord[0], min_coord[1]]
				m_coord = m_mesh[min_coord[0], min_coord[1]]

				#	add to coordinate outputs
				minima_plot_data[minima_index, plot_id, :] = [omega_coord, m_coord]
			
		#	save min RMS coordinate
		coords_of_d[plot_id] = [omega_coord, m_coord]

	print('\n		#####################################################################')
	print('		RMS Data Generation Loop Complete...')

	###############################################################################################
	#	generate figures
	###############################################################################################

	print('\n		#####################################################################')
	print('		Generating Figures for Starlink-{}; Track-{}'.format(int(object_id), track_index))

	#	set color and label strings
	color_str = 'C' + str(color_index)
	label_str = r'$Track-{}$'.format(int(track_index))

	#	tracking minimum RMS value
	ax2[0].plot(d_brdf_plot_range, rms_of_d, color=color_str, label=label_str, lw=plot_linewidth)

	#	tracking minimum RMS positon
	ax2[1].plot(
		coords_of_d[:, 0], coords_of_d[:, 1],
		color=color_str,
		lw=plot_linewidth,
		marker='x',
		ms=marker_size
	)

	#	add d_brdf labels
	for (coords, d_value) in zip(coords_of_d, d_brdf_plot_range):
		ax2[1].text(*coords, '$' + str(d_value) + '$', fontsize=text_fontsize)

#	#	add to plot
#	ax3[0].plot(d_brdf_plot_range, rms_of_d, color=color_str, label=label_str, lw=plot_linewidth)
#
#	#	tracking minimum RMS positon
#	ax3[1].plot(
#		coords_of_d[:, 0], coords_of_d[:, 1],
#		color=color_str,
#		lw=plot_linewidth,
#		marker='x',
#		ms=marker_size
#	)
#
#	#	add d_brdf labels
#	for (coords, d_value) in zip(coords_of_d, d_brdf_plot_range):
#		ax3[1].text(*coords, '$' + str(d_value) + '$', fontsize=text_fontsize)

	###############################################################################################
	#	generate equal minima data

	print('\n			Calculating equal minima data...')

	#	initialise minima coordinate arrays
	minima_omega = np.zeros([len(d_brdf_plot_range), max_minima])
	minima_m = np.zeros([len(d_brdf_plot_range), max_minima])
	
	#	fill with with first data set
	max_minima_range = np.arange(max_minima)
	for minima_index in max_minima_range:
		minima_omega[:, minima_index] = coords_of_d[:, 0]
		minima_m[:, minima_index] = coords_of_d[:, 1]

	print('\n			#############################################################')
	print('			Looping through d_brdf values...')

	for plot_id, d_brdf in enumerate(d_brdf_plot_range):

		print('\n				#####################################################')
		print('				Performing case d = {}...'.format(d_brdf))

		###########################################################################################
		#	acquire rms file
	
		#	set input directory
		in_dir = 'd_brdf=0-{:02d}'.format(int(d_brdf * 100))
	
		#	assemble input file name
		if track_index in [7, 8]:
			read_file = '{}-{}_track-{}_az-{}_inc-{}_d-{}.txt'.format(
				name_str,
				int(object_id),
				track_index,
				np.round(rot_vec_azm_deg, 2),
				np.round(rot_vec_inc_deg, 2),
				np.round(d_brdf, 2),
			)

		else:
			read_file = '{}-{}_track-{}_az-{}_inc-{}_d-{}.txt'.format(
				name_str,
				int(object_id),
				track_index,
				np.round(rot_vec_azm_deg, 2),
				np.round(90.0, 2),
				np.round(d_brdf, 2),
			)
	
		#	generate the rms values output file name
		rms_values_file = read_file[:-4] + '.rms_values'
	
		print('\n					Loading .rms_values file...')
		#	load mmt file
		track_rms_values = np.loadtxt(
			os.path.join(lightcurve_path, in_dir, rms_values_file),
			delimiter=','
		)

		#	apply threshold
		track_rms_values = track_rms_values[
			omega_id_min:omega_id_max,
			m_id_min:m_id_max
		]

		#	round to reasonable decimal places
		track_rms_values = np.round(track_rms_values, 10)

		#	extract min rms
		min_rms = np.round(track_rms_values.min(), 10)

		#	set first match flag to true
		first_match_flag = True

		print('\n					Extracting RMS minima data...')

		#	find all occurences of min rms in rms array
		for row_index, line in enumerate(track_rms_values):
			for col_index, value in enumerate(line):
				if value == min_rms:

					#	apply threshold offset
					col_coord = col_index + m_id_min
					row_coord = row_index + omega_id_min

					if first_match_flag is True:
						min_rms_coord = [col_coord, row_coord]
						first_match_flag = False
					else:
						min_rms_coord = np.vstack([min_rms_coord, [col_coord, row_coord]])

		#	convert to numpy array
		min_rms_coord = np.array(min_rms_coord)

		#	single minimum
		if min_rms_coord.shape == (2,):

			#	find omega and m coordinates
			omega_coord = omega_mesh[min_rms_coord[0], min_rms_coord[1]]
			m_coord = m_mesh[min_rms_coord[0], min_rms_coord[1]]

			minima_omega[plot_id, :] = [omega_coord]
			minima_m[plot_id, :] = [m_coord]

		#	multiple minima
		else:
			for coord_index, min_coord in enumerate(min_rms_coord):
	
				#	find omega and m coordinates
				omega_coord = omega_mesh[min_coord[0], min_coord[1]]
				m_coord = m_mesh[min_coord[0], min_coord[1]]

				minima_omega[plot_id, coord_index] = omega_coord
				minima_m[plot_id, coord_index] = m_coord

	print('\n			#################################################################')
	print('			RMS Minima Data Extracted...')

	###############################################################################################
	#	plotting multiple minima possibilities
	
	print('\n			Plotting minima lines...')

	if plot_all_minima_flag is True:
		max_minima_range = np.arange(max_minima)
		for minima_index in max_minima_range:
		
			omega_col = minima_omega[:, minima_index]
			m_col = minima_m[:, minima_index]
	
			minima_coords_of_d = np.transpose(np.vstack([omega_col, m_col]))
	
			#	tracking minimum RMS positon
			ax2[1].plot(
				omega_col, m_col,
				color=color_str,
				marker='x',
				linewidth=plot_linewidth,
				markersize=marker_size
			)
		
			#	add d_brdf labels
			for (coords, d_value) in zip(minima_coords_of_d, d_brdf_plot_range):
				ax2[1].text(*coords, '$' + str(d_value) + '$', fontsize=text_fontsize)

	#	itterate indices
	color_index += 1
	light_curve_index += 1
	first_track = False

	#	add rms of d to total
	total_rms_of_d = np.add(total_rms_of_d, rms_of_d)


	print('\n			#################################################################')
	print('			Track Complete...')

print('\n#####################################################################################')
print('FINISHED TRACK LOOP')

#	finalise plot
ax2[1].set_xlim([min_omega_brdf, max_omega_brdf])
ax2[1].set_ylim([min_m_brdf, max_m_brdf])

ax2[1].set_xlabel(r'$\omega$', fontsize=ylabel_fontsize)
ax2[1].set_ylabel(r'$m$', fontsize=xlabel_fontsize)
ax2[1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax2[1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax2[0].set_xlabel(r'$d$', fontsize=xlabel_fontsize)
ax2[0].set_ylabel(r'$RMS\ Error$', fontsize=ylabel_fontsize)
ax2[0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax2[0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

fig2.legend(
	loc='lower center',
	fontsize=legend_fontsize,
	ncol=4,
	frameon=False
)

###################################################################################################
#	Generating Colormap Plots
###################################################################################################

print('\n#####################################################################################')
print('Generating Colormap Plots...')
print('#####################################################################################')

###################################################################################################
#	figure generation inputs

#	set graphtype (cont/mesh)
graphtype = 'cont'
graphtype = 'mesh'

#	set colour map
cmap = plt.get_cmap('PiYG')

#	set rms normalisation flag
norm_flag = True

#	colour levels
levels_data = total_rms_values[
	omega_id_min:omega_id_max,
	m_id_min:m_id_max,
	:
]

levels_min = levels_data.min()
levels_max = levels_data.max()

#	set color levels
levels = MaxNLocator(nbins=n_cols).tick_values(levels_min, levels_max)
norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)
 
###################################################################################################
#	figure generation paramters

#	set colour levels
n_cols = 100

#	set subplot spacing values
hspace_val = 0.35
wspace_val = 0.35

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4
xlabel_fontsize = 6
ylabel_fontsize = 6
xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set marker parameters
marker_size = 3

#	set figure height/width
fig_width = 9.0/inch2cm
fig_height = 12.0/inch2cm

#	subplots figure
fig, ax = plt.subplots(
	int(np.ceil(len(d_brdf_plot_range) / 2)), 2,
	figsize=(fig_width, fig_height),
	dpi=300,
	sharex = True,
	sharey = True
)

plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

###############################################################################################
#	plotting

#	initialise RMS of d array
total_coords_of_d = np.zeros([len(d_brdf_plot_range), 2])

#	initialise figure axis coords
coords_array = np.array([
	[0, 0],
	[0, 1],
	[1, 0],
	[1, 1],
	[2, 0],
	[2, 1],
])

print('\n	#############################################################################')
print('	Looping through d_brdf values...')

for plot_id, (d_brdf, plot_coords) in enumerate(
		zip(d_brdf_plot_range, coords_array)
	):

	print('\n		#####################################################################')
	print('		Performing case d = {}...'.format(d_brdf))

	#	extract plot coords
	plot_coord_x, plot_coord_y = plot_coords
	print('\n		Plot Coordinates = [{}, {}]'.format(plot_coord_x, plot_coord_y))
	
	#	extract data
	total_rms_values_plot = total_rms_values[
		:,
		:,
		plot_id
	]

	#	round to reasonable decimal places
	total_rms_values_plot = np.round(total_rms_values_plot, 10)
	min_rms = total_rms_values_plot.min()
	
	#	save minimum RMS value
	rms_of_d[plot_id] = min_rms
	
	print('\n			Plotting Color Data...')
	
	###########################################################################################
	#	contour plot
	if graphtype == 'cont':
	
		#	plot contours
		cf = ax[plot_coord_x, plot_coord_y].contourf(
			m_mesh_plot[:-1, :-1] + step/2.,
			omega_mesh_plot[:-1, :-1] + step/2.,
			rms_values[:-1, :-1],
			levels=levels,
			cmap=cmap
		)
	
	###########################################################################################
	#	mesh plot
	elif graphtype == 'mesh':
	
		#	plot mesh
		cf = ax[plot_coord_x, plot_coord_y].pcolormesh(
			m_mesh_plot - step/2., omega_mesh_plot - step/2., total_rms_values_plot,
			cmap=cmap,
			norm=norm
		)
	
	###########################################################################################
	#	finalise plot
	
	#	add title
	ax[plot_coord_x, plot_coord_y].set_title(
		r'$d = {}$'.format(d_brdf),
		fontsize=title_fontsize,
		pad=title_pad
	)

	#	add colorbar
	cbar = fig.colorbar(cf, ax=ax[plot_coord_x, plot_coord_y])
	cbar.ax.tick_params(labelsize=cbar_fontsize)
	
	#	format x axis
	ax[plot_coord_x, plot_coord_y].set_xlim([min_omega_brdf, max_omega_brdf])
	ax[plot_coord_x, plot_coord_y].set_xlabel(r'$\omega$', fontsize=xlabel_fontsize)
	ax[plot_coord_x, plot_coord_y].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)

	#	format y axis
	ax[plot_coord_x, plot_coord_y].set_ylim([min_m_brdf, max_m_brdf])
	ax[plot_coord_x, plot_coord_y].set_ylabel(r'$m$', fontsize=ylabel_fontsize)
	ax[plot_coord_x, plot_coord_y].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)
	
	###########################################################################################
 	#	finding position of Min RMS
 
	print('\n			Searching for minimum RMS location...')

	first_match_flag = True
	
	#	find min rms coordinates
	for row_index, line in enumerate(total_rms_values_plot):
		for col_index, value in enumerate(line):
			if value == min_rms:

				#	apply threshold offset
				col_coord = col_index + m_id_min
				row_coord = row_index + omega_id_min

				if first_match_flag is True:
					min_rms_coord = [col_coord, row_coord]
					first_match_flag = False
				else:
					min_rms_coord = np.vstack([min_rms_coord, [col_coord, row_coord]])

 	#	convert to numpy array
	min_rms_coord = np.array(min_rms_coord)

 	###############################################################################################
 	#	plotting position of Min RMS
 
	#	single minimum
	if min_rms_coord.shape == (2,):
	
		#	find omega and m coordinates
		omega_coord = omega_mesh[min_rms_coord[0], min_rms_coord[1]]
		m_coord = m_mesh[min_rms_coord[0], min_rms_coord[1]]

		#	plot on figure
		ax[plot_coord_x, plot_coord_y].plot(
			omega_coord, m_coord,
			marker='+',
			markersize=marker_size,
			color='k',
		)
	
	#	multiple minima
	else:
		for min_coord in min_rms_coord:
	
			#	find omega and m coordinates
			omega_coord = omega_mesh[min_coord[0], min_coord[1]]
			m_coord = m_mesh[min_coord[0], min_coord[1]]

			#	plot on figure
			ax[plot_coord_x, plot_coord_y].plot(
				omega_coord, m_coord,
				marker='+',
				markersize=marker_size,
				color='k',
			)
		
	#	save min RMS coordinate
	total_coords_of_d[plot_id] = [omega_coord, m_coord]

	#	update axis coordinates
	plot_coord_x += 1

	if plot_id == 2:

		plot_coord_x = 0
		plot_coord_y = 1

###############################################################################################
#	plot total rms data

#	tracking minimum RMS value
ax3[0].plot(
	d_brdf_plot_range, total_rms_of_d,
	color='k',
	label=r'$Total$',
	linewidth=plot_linewidth,
	marker='x',
	ms=marker_size
)
	
#	tracking minimum RMS positon
ax3[1].plot(
	total_coords_of_d[:, 0], total_coords_of_d[:, 1],
	color='k',
	lw=plot_linewidth,
	marker='x',
	ms=marker_size
)

#	add d_brdf labels
for (coords, d_value) in zip(total_coords_of_d, d_brdf_plot_range):

	min_x_coord, min_y_coord = [*coords]
	print('coords of txt = ', min_x_coord, min_y_coord)
	ax3[1].text(min_x_coord, min_y_coord + 0.0005, '$' + str(d_value) + '$', fontsize=text_fontsize)

# #	finalise plot
# ax3[1].set_xlim([min_omega_brdf, max_omega_brdf])
# ax3[1].set_ylim([min_m_brdf, max_m_brdf])

ax3[1].set_xlabel(r'$\omega$', fontsize=ylabel_fontsize)
ax3[1].set_ylabel(r'$m$', fontsize=xlabel_fontsize)
ax3[1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax3[1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax3[0].set_xlabel(r'$d$', fontsize=xlabel_fontsize)
ax3[0].set_ylabel(r'$RMS\ Error$', fontsize=ylabel_fontsize)
ax3[0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax3[0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

# fig3.legend(
# 	loc='lower center',
# 	ncol=7,
# 	fontsize=legend_fontsize,
# 	frameon=False
# )

# plt.gcf().subplots_adjust(bottom=0.15)

print('\n#################################################################################')
print('Plots Completed.')

print('\n#################################################################################')
print('Best Fitting Solutions:')

for (d_brdf, coords, rms_val) in zip(d_brdf_plot_range, total_coords_of_d, total_rms_of_d):

	omega_brdf, m_brdf = np.round(coords, 2)

	print('\n	D = {};	m = {};	omega = {};	RMS = {:1.3f}'.format(
		d_brdf, m_brdf, omega_brdf, rms_val
	))

###############################################################################################
#	save figures

if savefile_flag is True:

	fig3_name = 'starlink_brdf_determination_plot.eps'
	out_path3 = os.path.join(fig_path, fig3_name)
	fig3.savefig(out_path3, bbox_inches="tight", dpi=300)

#	fig_name = 'starlink_brdf_determination_stack.eps'
#	out_path = os.path.join(fig_path, fig_name)
#	fig.savefig(out_path, bbox_inches="tight", dpi=300)

	print('\n#######################################')

plt.show()
sys.exit()
