###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	parallell functions
from starlink_para_funs import brdf_fitting

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm

#	parrellisation
from multiprocessing import Pool
import tqdm

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Starlink BRDF Fitting')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

local_input_path = os.path.join('input_dir', 'mmt_files', 'starlink', 'batch-0')
read_path = os.path.join(path_data, local_input_path)

###################################################################################################
#	script specific initialisation

#	set parrallelisation settings
parral_flag = True
ncores = 8

###################################################################################################
#	light curve generation inputs
###################################################################################################

#	set source MMT file
mmt_file = 'starlink-22_44236.txt'
mmt_file = 'starlink-36_44250.txt'
mmt_file = 'starlink-26_44240.txt'
mmt_file = 'starlink-27_44241.txt'
mmt_file = 'starlink-29_44243.txt'

#	set track of interest index
track_indices = [1, 2, 3, 4, 5, 6]

#	parse filename
name_str = mmt_file.split('-')[0]
object_id = int(mmt_file.split('-')[1].split('_')[0])
norad_id = int(mmt_file.split('-')[1].split('_')[1][:-4])

#	simulation time step
time_step = 0.1

#	set simulation date window
window_start_date = [2010, 5, 25, 0, 0, 0]
window_stop_date = [2020, 6, 1, 0, 0, 0]

###################################################################################################
#	load geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str = 'starlink_payload'
geom_file = geom_str + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 0.0

#	attitude motion
rot_period = 0.0

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	simulation inputs

#	set range of d_brdf values to plot
d_brdf_plot_range = [0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8]

#	range for brdf parameters
step = 0.01
m_brdf_range = np.arange(0.01, 0.99 + step, step)
omega_brdf_range = np.arange(0.01, 0.99 + step, step)

d_brdf = 0.2
m_brdf = 0.2
omega_brdf = 0.5

###################################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

print('\n#####################################################################################')
print('Simulation Inputs: ')
print('\n	Input File: ')
print('\n		Working from MMT data input file: 	{}'.format(mmt_file))
print('		Reading from input path: 		{}'.format(local_input_path))
print('\n			Name: 	{}'.format(name_str))
print('			Object ID: 	{}'.format(object_id))
print('			Norad ID: 	{}'.format(norad_id))
# print('			Track ID: 	{}'.format(track_index))
print('\n	Light Curve Inputs: ')
print('\n			Geom File: 		{}'.format(geom_file))
print('			Rot Vec Azimuth: 	{}'.format(rot_vec_azm_deg))
print('			Rot Vec Inclination: 	{}'.format(rot_vec_inc_deg))
print('			Rotation Period: 	{}'.format(rot_period))
# print('			BRDF: 			m = {}, omega = {}, m = {}'.format(d_brdf, omega_brdf, m_brdf))
print('\n	Observation Site Inputs: ')
print('\n			Site String: 	{}'.format(site_str))
print('			Latitude: 	{}'.format(np.degrees(site_latgd)))
print('			Longitude: 	{}'.format(np.degrees(site_lon)))
print('			Altitude: 	{} m'.format(site_alt))

###################################################################################################
#	load MMT file
###################################################################################################

#	set time
mmt_load_start = time.time()

print('\n[{}] Loading MMT data...'.format(datetime.datetime.now()))

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file

#	strip header
header_text = mmt_data[7]
raw_data = mmt_data[8:]

print('\n	Parsing MMT data...')

#	initialise output
n_rows = 14
parsed_data = np.zeros([len(raw_data), n_rows])

for i, line in enumerate(raw_data):

	#	split line
	date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
		distance, phase, channel, track_id = line.split(' ')

	#	parse data and set data types
	mag_std = float(mag_std)
	mag_app = float(mag_app)
	distance = float(distance)
	phase = float(phase)
	track_id = int(track_id)

	#	parse date
	year, month, day = date_str.split('-')
	year = int(year)
	month = int(month)
	day = int(day)

	#	parse time
	hour, minute, second = time_str.split(':')
	hour = int(hour)
	minute = int(minute)
	second = np.round(float(second), 1)

	#	calculate julian date
	jday, jfrac = date2jd(year, month, day, hour, minute, second)

	#	generate output line
	output_line = [
		track_id, object_id,
		jday, jfrac,
		year, month, day,
		hour, minute, second,
		mag_std, mag_app,
		distance, phase
	]

	#	add to output array
	parsed_data[i, :] = output_line

print('		Total data length = ', len(parsed_data))
print('		Total number of objects = ', len(np.unique(parsed_data[:, 1])))
print('		Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

###################################################################################################
#	extracting data between specified windows

print('\n	Seaching for tracks inside time window:')

#	generate window start/stop JDs
window_start_jd = np.add(*date2jd(*window_start_date))
window_stop_jd = np.add(*date2jd(*window_stop_date))

print('\n	Window start 	= ', window_start_date[0:3])
print('	Window stop 	= ', window_stop_date[0:3])
print('\n')

#	extract track ids
track_id_list = np.unique(parsed_data[:, 0])

#	loop through tracks
ntracks = 0
first_lc_flag = True
for track_id in track_id_list:

	#	find track ids
	lc_track_indices = np.where(parsed_data[:, 0]==track_id)[0]

	#	extract full data
	track_data = parsed_data[lc_track_indices, :]

	#	extract light curve data
	jd_plot = np.add(track_data[:, 2], track_data[:, 3])
	std_mag_plot = track_data[:, 10]
	app_mag_plot = track_data[:, 11]
	distance_plot = track_data[:, 12]
	phase_plot = track_data[:, 13]
	start_line = track_data[0]
	stop_line = track_data[-1]
	start_jdate = np.add(start_line[2], start_line[3])
	stop_jdate = np.add(stop_line[2], stop_line[3])

	#	ensure between window start/stop
	if start_jdate > window_start_jd and stop_jdate < window_stop_jd:

		#	light curve is within window
		ntracks += 1

		#	if first light curve, start array
		if first_lc_flag is True:
			windowed_data = track_data
			first_lc_flag = False

		#	else append to list
		elif first_lc_flag is False:
			windowed_data = np.vstack([windowed_data, track_data])

print('\n		{} track(s) extracted.'.format(ntracks))

###############################################################################################
#	display initialisation time
mmt_load_end = time.time()
mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)

print('\n	MMT Load Finished.')
print('\n		- Time taken = {} s.'.format(mmt_load_time))

###################################################################################################
#	track loop
###################################################################################################

print('\n#####################################################################################')
print('Looping through Tracks...')

for track_itt, track_index in enumerate(track_indices):

	print('\n	#############################################################################')
	print('	Working on Track ID: {}'.format(int(track_index)))

	###############################################################################################
	#	output directory
	output_path = os.path.join(
		path_data,
		'lightcurve_dir',
		'starlink_analysis',
		name_str + '-{}'.format(object_id),
		'track-{}'.format(int(track_index))
	)
	
	if not os.path.exists(output_path):
		print('\n	Creating Directory: ', output_path)
		os.makedirs(output_path)
	
	###############################################################################################
	#	extracting requested single track
	
	print('\n	Extracting requested track index...')
	
	#	set track array index
	track_array_index = track_index - 1
	track_id = int(track_id_list[track_array_index])
	
	track_indices = np.where(windowed_data[:, 0]==track_id)[0]
	track_data = windowed_data[track_indices]
	
	###############################################################################################
	#	extract track time data

	mmt_line = track_data[0]
	
	#	parse line elements
	track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase = track_data[0]
	
	start_jd = [jday, jfrac]
	start_jdate = jday + jfrac
	
	#	parse line elements
	track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase = track_data[-1]
	
	stop_jd = [jday, jfrac]
	
	###############################################################################################
	#	generate simulation time data
	
	#	extract start and stop jds
	lc_start_jd = start_jd[0] + start_jd[1]
	lc_stop_jd = stop_jd[0] + stop_jd[1]
	
	#	convert to dates
	lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
	lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
	
	#	light curve duration
	lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
	
	#	set time step
	lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
	
	#	time range in julian days
	lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
	sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

	###############################################################################################
	#	ensure time continuity

	print('\n	Ensuring time continuity in extracted data...')

	track_data_temp = []
	sim_index = 0

	for itt, line in enumerate(track_data):

		#	parse line data
		track_id, object_id, \
			jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = line
		
		step_jdate = jday + jfrac
	
		#	calculate current seconds
		current_sec = np.round((step_jdate - lc_sim_range[0]) * 86400, 1)
	
		output_line = [
			jday, jfrac, current_sec, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase
		]

		##################################################
		#	extracting data of time step
	
		#	data match found
		if current_sec == sim_range_sec[sim_index]:
			track_data_temp.append(output_line)
			sim_index += 1

			#	save current line as previous line
			prev_line = output_line

		#	data point is not required
		elif current_sec < sim_range_sec[sim_index]:
			pass
	
		#	interpolation required
		elif current_sec > sim_range_sec[sim_index]:

			#	parse previous line
			jday_prev, jfrac_prev, prev_sec, \
				year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
				mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
	
			#	calculate interpolation requirements
			gap = np.round(current_sec - prev_sec, 1)
			npts = int(gap / time_step)
	
			#	get current light curve data
			mag_std_last = mag_std
			mag_app_last = mag_app
			distance_last = distance
			phase_last = phase
	
			#	calculate linear interpolation step values
			mag_std_step = (mag_std_last - mag_std_prev) / npts
			mag_app_step = (mag_app_last - mag_app_prev) / npts
			distance_step = (distance_last - distance_prev) / npts
			phase_step = (phase_last - phase_prev) / npts

			#	calculate interpolation data
			for n in range(1, npts + 1):
	
				#	current simulation second
				interp_sec = np.round(prev_sec + (n * time_step), 1)

				#	time data
				jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
				jday, jfrac = jday_jfrac(jdate)
				year, month, day, hour, minute, second = jd2date(jday, jfrac)
	
				#	calculate interpolated light curve data
				mag_std = mag_std_prev + (n * mag_std_step)
				mag_app = mag_app_prev + (n * mag_app_step)
				distance = distance_prev + (n * distance_step)
				phase = phase_prev + (n * phase_step)
	
				#	generate new output line
				interp_output_line = [
					jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
					mag_std, mag_app, distance, phase
				]
	
				if interp_sec == sim_range_sec[sim_index]:
					track_data_temp.append(interp_output_line)
					sim_index += 1

			if interp_sec != current_sec:
				#	append final line
				track_data_temp.append(output_line)
				sim_index += 1

		else:
			print('ERROR')
			sys.exit()

		#	save previous line
		if itt != 0:
			prev_line = output_line

		#	array is complete condition
		if len(track_data_temp) == lc_len_record:
			break

	#	ensure parsed MMT data matches simulation data
	track_data = np.array(track_data_temp)

	if len(track_data) == len(lc_sim_range):
		print('\n		Simulation and MMT data matches.')
	else:
		print('\n		Error: Simulation and MMT data does not match...Exiting.')
		sys.exit()

	###############################################################################################
	#	extract mmt brightness data

	mmt_signal_std = track_data[:, 9]
	mmt_signal_app = track_data[:, 10]
	
	print('\n	###############################################################################')
	print('	Light curve parameters: ')
	print('		Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_start_date))
	print('		Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_stop_date))
	print('		Simulation time step:		{}'.format(time_step))
	print('		Duration:			{}		[steps]'.format(lc_len_record))
	print('						{}		[s]'.format(np.round(lc_duration_sec, 1)))
	print('\n	###############################################################################')

	###############################################################################################
	#	acquire TLE
	###############################################################################################
	
	#	set time
	tle_start = time.time()
	
	print('\n	[{}] Acquiring TLEs...'.format(datetime.datetime.now()))
	
	#	TLE request dates, add day either side to increase tle find chance
	tle_start_date = jd2date(*jday_jfrac(lc_start_jd - 1))
	tle_stop_date = jd2date(*jday_jfrac(lc_stop_jd + 1)) 
	
	#	generate TLE file name
	tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
	tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
	tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)
	
	#	assemble TLE path
	tle_path = os.path.join(path_data, 'input_dir', 'tle_files')
	
	#	if exists, load
	if os.path.isfile(os.path.join(tle_path, tle_file)):
	
		print('\n		Reading existing TLE file.')
	
		#	load file
		with open(os.path.join(tle_path, tle_file), 'r') as file:
			tle_data = file.readlines()
	
	else:
		print('\n		Requesting TLE file...')
	
		#	request TLEs
		tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)
	
		#	save file
		with open(os.path.join(tle_path, tle_file), 'w') as file:
			for line in tle_data:
				file.write(line)
	
	#	grab first TLE pair
	tle_line1 = tle_data[1]
	tle_line2 = tle_data[2]
	
	#	mean mot rad/sec
	orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
	orbital_mot = (2 * np.pi) / orbital_period
	
	###############################################################################################
	#	display TLE acquisition time
	tle_end = time.time()
	tle_time = np.round((tle_end - tle_start), 1)
	
	print('\n		TLE Load Finished.')
	print('\n			- Time taken = {} s.'.format(tle_time))
	
	###############################################################################################
	#	select time data to use
	###############################################################################################
	
	sim_range = lc_sim_range
	len_record = lc_len_record
	start_date = lc_start_date
	stop_date = lc_stop_date
	duration_sec = lc_duration_sec
	
	###############################################################################################
	#	generate eop data arrays
	###############################################################################################
	
	#	set time
	eop_start = time.time()
	
	print('\n	[{}] Generating EOP data array...'.format(datetime.datetime.now()))
	
	#	EOP data
	eop_dir = 'eop_files'
	eop_file = 'eop_parsed.csv'
	dat_file = 'dat_parsed.csv'
	
	#	assemble eop path
	eop_path = os.path.join(path_data, eop_dir)
	
	#	generate interpolated eop_data
	eop_log_level = 0
	eop_data, dat_data = eop_interpolate(
		eop_path,
		eop_file,
		dat_file,
		sim_range,
		time_step,
		eop_log_level
	)
	
	###################################################################################################
	#	display time
	eop_end = time.time()
	eop_time = np.round((eop_end - eop_start), 1)
	
	print('\n		EOP data generated.')
	print('\n			- Time taken = {} s.'.format(eop_time))
	
	###################################################################################################
	#	generate time objects
	###################################################################################################
	
	#	start dat value
	dat = dat_data[0, -1]
		
	#	start eop values
	eop_line = eop_data[0, :]
	eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
		
	#	generate start time object
	start_time_object = get_times(*start_date, ut_offset, dat)
		
	#	end dat value
	dat = dat_data[-1, -1]
		
	#	end eop values
	eop_line = eop_data[-1, :]
	eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
		
	#	generate stop time object
	stop_time_object = get_times(*stop_date, ut_offset, dat)
	
	###################################################################################################
	#	generate rotation matrices
	###################################################################################################
	
	#	set time
	rot_mat_start = time.time()
	print('\n	[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))
	
	#	tt time
	jdaytt_start = start_time_object.jday_tt
	jfractt_start = start_time_object.jfrac_tt
	jdtt_start = jdaytt_start + jfractt_start
	
	#	ut1 time
	jdayut1_start = start_time_object.jday_ut1
	jfracut1_start = start_time_object.jfrac_ut1
	jdut1_start = jdayut1_start + jfracut1_start
	
	#	generate rotation matrices
	rot_mat_log_level = 0
	pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
		jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
	)
	
	###################################################################################################
	#	display time
	rot_mat_end = time.time()
	rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)
	
	print('\n		Rotation Matrices Generated.')
	print('\n			- Time taken = {} s.'.format(rot_mat_time))
	
	###################################################################################################
	#	MODULE:	Horizons Positions
	###################################################################################################
	
	#	set time
	horz_start = time.time()
	print('\n	[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))
	
	#	set horizons logging level
	horz_logging_level = 0
	
	sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
		path_data, horz_logging_level,
		start_time_object, stop_time_object,
		len_record, time_step,
		eop_path
	)
	
	#	find indices of light curve data
	timecol = sun_icrf[:, 0]
	lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]
	
	###################################################################################################
	#	display time
	horz_end = time.time()
	horz_time = np.round((horz_end - horz_start), 1)
	
	print('\n		Horizons data generated.')
	print('\n			- Time taken = {} s.'.format(horz_time))
	
	###################################################################################################
	#	MODULE:	Object Position
	###################################################################################################
	
	#	set time
	sat_start = time.time()
	print('\n	[{}]	Generating object position data...'.format(datetime.datetime.now()))
	
	satpos_logging_level = 0
	
	sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
		path_data, satpos_logging_level, len_record, time_step,
		start_time_object, stop_time_object,
		norad_id,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array
	)
	
	###################################################################################################
	#	display time
	sat_end = time.time()
	sat_time = np.round((sat_end - sat_start), 1)
	
	print('\n		Object position data generated.')
	print('\n			- Time taken = {} s.'.format(sat_time))
	
	###################################################################################################
	#	MODULE:	Transformation
	###################################################################################################
	
	#	set time
	trans_start = time.time()
	print('\n	[{}]	Performing transformations...'.format(datetime.datetime.now()))
	
	identification_str = 'mmt-render'
	
	trans_logging_level = 0
	save_flag = False
	
	sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
		= module_transformation(
			path_data, trans_logging_level, save_flag,
			start_time_object, stop_time_object,
			identification_str, site_str,
			len_record, time_step,
			eop_data, dat_data,
			pm_array, pn_array, st_array, c2t_array,
			sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
		)
	
	###################################################################################################
	#	display time
	trans_end = time.time()
	trans_time = np.round((trans_end - trans_start), 1)
	
	print('\n		Transformations complete.')
	print('\n			- Time taken = {} s.'.format(trans_time))
	
	###################################################################################################
	#	define initial attitude state
	###################################################################################################
	
	##################################################
	#	attitude
	
	#	convert to radians
	rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
	rot_vec_inc_rad = np.radians(rot_vec_inc_deg)
	
	#	generate initial t = 0 attitude
	init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
	
	##################################################
	#	attitude motion in body frame
	
	#	generate rotation angle in radians
	if rot_period != 0.0:
		rot_ang_per_sec = (np.pi * 2) / rot_period
	else:
		rot_ang_per_sec = 0.0
	
	#	define rotation axis in body frame as unit vector
	rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
	
	#	generate attitude matrix in body frame
	attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)
	
	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1
	
	##################################################
	#	generate initial attitude rotation matrix
	init_attit_mat = rot_mat(init_attit)
	
	##################################################
	#	calculate rotation vector in orbital frame as unit vector
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)
	
	##################################################
	#	define velocity of rotation vector in orbital frame
	
	#	when equal to orbital motion, rot vec is stationary in inertial frame
	rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
	rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])
	
	###############################################################################################
	#	generate brdf
	
	#	Refractive Index
	n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))
	
	#	Specular ratio
	s_brdf = 1.0 - d_brdf
	
	#	generate brdf
	brdf_data = np.zeros([nfaces, 5])
	
	for i in range(nfaces):
		brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
	
	###################################################################################################
	#	perform light curve vector generation
	###################################################################################################
	
	#	blank observation and illumination vectors in orbital frame
	obs_vec_orb = np.array([1.0, 0.0, 0.0])
	illum_vec_orb = np.array([1.0, 0.0, 0.0])
	
	#	generate orb2inertial velocity vector
	orb2inertial = np.array([0.0, 0.0, -orbital_mot])

	print(init_attit)
	print(rot_vec_body_unit)
	print(rot_ang_per_sec)
	print(rot_vec_vel_orb)
	sys.exit()
	
	#	vector generation function
	vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb
	)
	
	##################################################
	#	parse vector data
	
	#	extract orbital positions of custom input vectors
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_body[:, 31:34]
	
	#	body data
	sun_body_data = vector_data_body[:, 28:31]
	site_body_data = vector_data_body[:, 31:34]
	earth_body_data = vector_data_body[:, 19:22]
	sat_body_vel = vector_data_body[:, 22:25]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	illum_vec_body_data = vector_data_body[:, 28:31]
	obs_vec_body_data = vector_data_body[:, 31:34]
	
	#	orb data
	sun_orb_data = vector_data_orb[:, 28:31]
	site_orb_data = vector_data_orb[:, 31:34]
	earth_orb_data = vector_data_orb[:, 19:22]
	sat_orb_vel = vector_data_orb[:, 22:25]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_orb[:, 31:34]
	
	###############################################################################################
	#	brdf fitting
	###############################################################################################
	
	print('\n	#################################################################################')
	print('	Beginning Fitting Loop...')

	#	looping on d brdf values
	for d_brdf in d_brdf_loop_range:
	
		print('\n		#########################################################################')
		print('		Working on slice:	d_brdf = ', d_brdf)
	
		#	generate subdirectory
		out_dir = 'd_brdf=0-{:02d}'.format(int(d_brdf * 100))
		
		if not os.path.exists(os.path.join(output_path, out_dir)):
			print('\n		Creating Directory: ', os.path.join(output_path, out_dir))
			os.makedirs(os.path.join(output_path, out_dir))
	
		#	generate an array of inputs
		inputs = []
		for omega_brdf in omega_brdf_range:
			for m_brdf in m_brdf_range:
		
				d_brdf = np.round(d_brdf, 2)
				m_brdf = np.round(m_brdf, 2)
				omega_brdf = np.round(omega_brdf, 2)
	
				inputs.append([
					name_str,
					object_id,
					rot_vec_azm_deg,
					rot_vec_inc_deg,
					d_brdf,
					m_brdf,
					omega_brdf,
					nfaces,
					duration_sec,
					len_record,
					start_jdate,
					sun_body_data,
					site_body_data,
					body_vectors,
					body_areas
				])
		
		inputs = np.array(inputs)

		###############################################################################################
		if parral_flag is True:
			
			#	initialise multiproccessing pool of workers
			p = Pool(ncores)
	
			total_lightcurve_data = []
			
			#	map the total data array across the access function
			for light_curve_output in tqdm.tqdm(
				p.imap_unordered(brdf_fitting, inputs), total=len(inputs)
				):
			
				#	parse outputs
				total_lightcurve_data.append(light_curve_output)

		elif parral_flag is False:
	
			#	run inputs sequentially
			for input_line in inputs:
				light_curve_output = brdf_fitting(input_line)
	
		print('\n			Data Generated...')

		###############################################################################################
		#	save slice file
	
		#	generate output filename
		out_file = '{}-{}_track-{}_az-{}_inc-{}_d-{}.txt'.format(
			name_str,
			int(object_id),
			track_index,
			np.round(rot_vec_azm_deg, 2),
			np.round(rot_vec_inc_deg, 2),
			np.round(d_brdf, 2),
		)
	
		#	delete existing file if required
		if os.path.isfile(os.path.join(path_data, output_path, out_dir, out_file)):
			ans = input(
				'\n			Delete:	{}? [y/n]\n'.format(
					str(os.path.join(out_dir, out_file))
				)
			)

			if ans =='y':
				os.remove(os.path.join(path_data, output_path, out_dir, out_file))

			else:
				print('\n	Error in Response...')
				sys.exit()
	
		#	open output file
		with open(os.path.join(path_data, output_path, out_dir, out_file), 'a') as write_file:
			print('\n		Writing to: ', os.path.join(path_data, output_path, out_dir, out_file))
	
			#	write each light curve to total light curve file
			for light_curve_index, light_curve_array in enumerate(total_lightcurve_data):
	
				#	generate index array
				index_array = np.full([len(light_curve_array[:, 0]), 1], light_curve_index)
	
				#	stack light curve index array with data
				output_array = np.hstack([index_array, light_curve_array])
	
				#	save light curve data
				np.savetxt(write_file, output_array, delimiter=',')
	
				#	add newline character
				write_file.write('\n')

			print('\n			File Saved...')

		print('\n			Slice Complete.')

	###############################################################################################
	#	calculate run time
	###############################################################################################
	#	calculate sim time
	sim_end = time.time()
	sim_time = np.round((sim_end - sim_start) / 60, 1)
	
	print('\n[{}] Simulation Finished.'.format(datetime.datetime.now()))
	print('\n	- Time taken = {} mins.'.format(sim_time))
