###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, kep2rv

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle, req_omm

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Starlink File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set flag
savefile_flag = True
savefile_flag = False

###################################################################################################
#	inputs
###################################################################################################

#	set time windows
window_start_date = [2020, 1, 1, 0, 0, 0]
window_stop_date = [2021, 1, 1, 0, 0, 0]

#	set launch date
launch_date0 = [2019, 5, 24, 2, 30, 0.0]
launch_date1 = [2019, 11, 11, 14, 56, 0.0]
launch_date2 = [2020, 1, 7, 2, 19, 0.0]

simulation_end_date = [2017, 12, 30, 0, 0, 0.0]
simulation_end_date = [2021, 7, 30, 0, 0, 0.0]

#	set simulation flags
pause_flag = True
pause_flag = False

specific_object_flag = True
specific_object_flag = False

#	define list of specific objects
specific_objects = [1515]

#	convert to numpy array
specific_objects = np.array(specific_objects)

#	set time step
time_step = 0.1

#	set earth radius
RE = 6371
earth_mu = 398600.4418

###################################################################################################
#	filtering

#	altitude filter
operational_alt = 555
operational_band = 2

altitude_flag = False
altitude_flag = True

#	BATCH 1 define list of objects to exclude
exclusion_list = [
	1014, 1037, 1065,	#	batch 1
	1090, 1110	#	batch 2
]

exclusion_list_flag = False
exclusion_list_flag = True

#	minimum brightness filter
min_brightness = 3.0
min_brightness_filter = True
min_brightness_filter = False

max_brightness = 7.0
max_brightness_filter = True
max_brightness_filter = False

#	minimum number of tracks filter
minimum_tracks_filter = True
minimum_tracks_filter = False

#	define a small number
small = 1e-8

###################################################################################################
#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Articles',
	'starlink_paper',
	'Figures'
)

# fig_name = 'starlink_' + launch_string + '_full.eps'
# fig_name = 'starlink_' + launch_string + '_final.eps'
# 
# out_path = os.path.join(fig_path, fig_name)

###################################################################################################
#	initialise figure

#	font sizes
font_sizex = 6
font_sizey = 6
label_sizex = 5
label_sizey = 5
legend_fontsize = 5
font_pad = 1
title_size = 8
title_pad = 5

#	marker parameters
marker_flag = False
marker_symbol = 'o'
marker_size = 3
marker_edge = 0.5

#	line parameters
launch_linewidth = 1
plot_linewidth = 0.8
grid_width = 0.1

dpi_val = 300
inch2cm = 2.54

hspace_val = 0.2
wspace_val = 0.35

show_legend = True
show_legend = False

color_by_object = False
color_by_object = True

color_only_black = True
color_only_black = False

plot_on_station_date = True
plot_on_station_date = False

color_by_batch = True
color_by_batch = False


plt.rc('text', usetex=True)

fig, ax = plt.subplots(
	4, 1,
	figsize=(10.0/inch2cm, 14.0/inch2cm),
	sharex=True,
	dpi=dpi_val
)

plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)
plt.gcf().subplots_adjust(bottom=0.15)

#	configure axis
ax[0].set_ylim([450, 580])

ax[0].set_xlabel(r'$Date$', fontsize=label_sizex)
ax[0].set_ylabel(r'$Alt\ /\ km$', fontsize=label_sizey)

ax[0].tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax[0].tick_params(axis='y', labelsize=font_sizey, pad=0)

ax[0].set_xlim([datetime.date(*launch_date0[0:3]), datetime.date(*window_stop_date[0:3])])

#	configure axis
ax[1].set_ylim([52.95, 53.05])

ax[1].set_xlabel(r'$Date$', fontsize=label_sizex)
ax[1].set_ylabel(r'$Incl\ /\ Deg$', fontsize=label_sizey)

ax[1].tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax[1].tick_params(axis='y', labelsize=font_sizey, pad=0)

ax[1].set_xlim([datetime.date(*launch_date0[0:3]), datetime.date(*window_stop_date[0:3])])

#	configure axis
# ax[2].set_ylim([200, 600])

ax[2].set_xlabel(r'$Date$', fontsize=label_sizex)
ax[2].set_ylabel(r'$Eccen$', fontsize=label_sizey)

ax[2].tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax[2].tick_params(axis='y', labelsize=font_sizey, pad=0)

ax[2].set_xlim([datetime.date(*launch_date0[0:3]), datetime.date(*window_stop_date[0:3])])

#	configure axis
ax[3].set_ylim([450, 580])

ax[3].set_xlabel(r'$Date$', fontsize=label_sizex)
ax[3].set_ylabel(r'$Semi Major Axis$', fontsize=label_sizey)

ax[3].tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax[3].tick_params(axis='y', labelsize=font_sizey, pad=0)

ax[3].set_xlim([datetime.date(*launch_date0[0:3]), datetime.date(*window_stop_date[0:3])])

years = mdates.YearLocator()
months = mdates.MonthLocator()
days = mdates.DayLocator()

###################################################################################################
#	plot launch date

#	convert jd to date
year, mon, day, hour, minute, sec = launch_date0

#	turn into string
datestr = str('-'.join(['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]))
timestr = ':'.join(['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)])
datevec  = ' '.join([datestr, timestr])

#	convert to datetime object
date_time_obj0 = datetime.datetime.strptime(datevec, r'%Y-%m-%d %H:%M:%S.%f')

#	convert jd to date
year, mon, day, hour, minute, sec = launch_date1

#	turn into string
datestr = str('-'.join(['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]))
timestr = ':'.join(['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)])
datevec  = ' '.join([datestr, timestr])

#	convert to datetime object
date_time_obj1 = datetime.datetime.strptime(datevec, r'%Y-%m-%d %H:%M:%S.%f')

#	convert jd to date
year, mon, day, hour, minute, sec = launch_date2

#	turn into string
datestr = str('-'.join(['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]))
timestr = ':'.join(['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)])
datevec  = ' '.join([datestr, timestr])

#	convert to datetime object
date_time_obj2 = datetime.datetime.strptime(datevec, r'%Y-%m-%d %H:%M:%S.%f')


###################################################################################################
#	starlink batch file loop
###################################################################################################

launch_string_list = ['batch1', 'batch2', 'batch3', 'batch10']
launch_string_list = ['batch10']

total_obj_plotted = 0
total_tracks_plotted = 0

total_plotted_object_indices = []
total_plotted_light_curves = 0

first_batch = True

for launch_string in launch_string_list:

	#	set read path
	read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'starlink', launch_string)
	
	###############################################################################################
	#	read and sort starlink files
	###############################################################################################
	
	print('\n[{}] Generating Starlink file name list...'.format(datetime.datetime.now()))
	
	starlink_dir = os.listdir(read_path)
	starlink_files = []
	
	#	order by starlink id
	for file in starlink_dir:
		starlink_id = int(file.split('-')[1].split('_')[0])
		starlink_files.append([file, starlink_id])
	
	starlink_files = sorted(starlink_files, key=lambda x: x[1])

	###############################################################################################
	#	starlink file loop
	
	print('\n[{}] Extracting MMT file data...\n'.format(datetime.datetime.now()))
	first_flag = True
	
	norad_id_list = np.zeros([len(starlink_files), 2])
	
	for file_num, mmt_file in enumerate(starlink_files):
	
		#print('	Parsing MMT data file: {}'.format(mmt_file), end='\r')

		###########################################################################################
		#	load MMT file
	
		mmt_file = mmt_file[0]
	
		#	parse filename data
		name_str, norad_id, _, mmt_id = mmt_file[:-4].split('_')
	
		starlink_id = int(name_str.split('-')[1])
	
		name_str = ''.join(name_str.split('-'))
		norad_id = int(norad_id)
		
		#	open and read file
		with open(os.path.join(read_path, mmt_file)) as read_file:
			mmt_data = read_file.readlines()
		
		#	add to norad id list
		norad_id_list[file_num] = [starlink_id, norad_id]
	
		###########################################################################################
		#	parse mmt file
		
		#	header line
		header_text = mmt_data[7]
		
		#	strip header
		mmt_data = mmt_data[8:]
		
		n_rows = 14
		
		out_data = np.zeros([len(mmt_data), n_rows])
		
		for i, line in enumerate(mmt_data):
		
			#	split line
			date, mmt_time, mag_std, mag_app, opt_filter, penumbral, distance, \
				phase, channel, track_id = line.split(' ')
		
			#	parse data and set data types
			mag_std = float(mag_std)
			mag_app = float(mag_app)
			distance = float(distance)
			phase = float(phase)
			track_id = int(track_id)
		
			#	parse date
			year, month, day = date.split('-')
			year = int(year)
			month = int(month)
			day = int(day)
		
			#	parse time
			hour, minute, second = mmt_time.split(':')
			hour = int(hour)
			minute = int(minute)
			second = float(second)
		
			#	calculate julian date
			jday, jfrac = date2jd(year, month, day, hour, minute, second)
		
			#	generate output line
			output_line = [
				track_id, starlink_id,
				jday, jfrac,
				year, month, day,
				hour, minute, second,
				mag_std, mag_app,
				distance, phase
			]
		
			#	add to output array
			out_data[i, :] = output_line
	
		#	initialise output array
		if first_flag is True:
			total_data = out_data
	
			first_flag = False
	
		#	stack with existing output array
		elif first_flag is False:
			total_data = np.vstack([total_data, out_data])
	
	sys.stdout.write("\033[K")
	print('	Done.\n', end='\r')
	
	print('\nFinished MMT file parsing...')
	print('	Total data length = ', len(total_data))
	print('	Total number of objects = ', len(np.unique(total_data[:, 1])))
	print('	Total number of light curves = ', len(np.unique(total_data[:, 0])))
	
	###############################################################################################
	#	collect data in window of interest
	###############################################################################################
	
	print('\n[{}] Extracting light curve inside time window...'.format(datetime.datetime.now()))
	
	#	generate window start/stop JDs
	window_start_jd = np.add(*date2jd(*window_start_date))
	window_stop_jd = np.add(*date2jd(*window_stop_date))
	
	first_lc_flag = True
	
	#	extract starlink ids
	starlink_id_list = np.unique(total_data[:, 1])
	
	light_curve_index = 1
	
	plotted_light_curves = 0
	
	#	loop through starlink IDs
	for starlink_id in starlink_id_list:
	
		print('	Starlink-{}: '.format(int(starlink_id)), end='')
	
		#	find track ids
		starlink_indices = np.where(total_data[:, 1]==starlink_id)[0]
	
		#	extract data for this starlink object
		full_starlink_data = total_data[starlink_indices, :]
	
		#	extract track ids
		track_id_list = np.unique(full_starlink_data[:, 0])
	
		ntracks = 0
	
		#	loop through tracks
		for track_id in track_id_list:
		
			#	find track ids
			track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
	
			#	extract full data
			track_data = full_starlink_data[track_indices, :]
		
			#	extract light curve data
			distance_plot = track_data[:, 12]
			phase_plot = track_data[:, 13]
		
			start_line = track_data[0]
			stop_line = track_data[-1]
		
			start_jdate = np.add(start_line[2], start_line[3])
			stop_jdate = np.add(stop_line[2], stop_line[3])
		
			#	ensure between window start/stop
			if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
	
				#	light curve is within window
				ntracks += 1
		
				#	if first light curve, start array
				if first_lc_flag is True:
					final_data = track_data
					first_lc_flag = False
	
				#	else append to list
				elif first_lc_flag is False:
					final_data = np.vstack([final_data, track_data])
	
		print('{} track(s) extracted.'.format(ntracks))
	
	print('\nFinished window extraction...')
	print('	Total data length = ', len(final_data))
	print('	Total number of objects = ', len(np.unique(final_data[:, 1])))
	print('	Total number of light curves = ', len(np.unique(final_data[:, 0])))
	

	###############################################################################################
	#	plot light curve properties
	###############################################################################################
	
	print('\n#####################################################################################')
	print('[{}] Plotting Starlink Light Curve Properties...'.format(datetime.datetime.now()))
	
	###############################################################################################
	#	full final data
	
	#	extract starlink ids
	starlink_id_list = np.unique(final_data[:, 1])
	
	#	if specific object flag is true, use object list
	if specific_object_flag is True:
		starlink_id_list = specific_objects

	###############################################################################################
	#	starlink object loop
	
	print('\n	Looping Through Starlink Light Curve Files...')
	
	light_curve_index = 1
	color_index = 0
	total_duration = 0
	
	plotted_object_indices = []
	obj_plotted = 0
	tracks_plotted = 0
	
	save_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'starlink')

	#	loop through starlink IDs
	for loop_itt, starlink_id in enumerate(starlink_id_list):
	
		print('\n		#####################################################################')
		print('		Plotting data for: Starlink-{}'.format(int(starlink_id)))

		if exclusion_list_flag is True:
			if starlink_id in exclusion_list:
				print('\n			In exclusion list, skipping...')
				continue

		#	set norad id
		if starlink_id not in norad_id_list[:, 0]:
			continue

		norad_index = np.where(norad_id_list[:, 0]==starlink_id)[0][0]
		norad_id = int(norad_id_list[norad_index, 1])
	
		print('\n			NORAD ID: {}'.format(int(norad_id)))
	
		###########################################################################################
		#	extract required data
	
		#	find track ids
		starlink_indices = np.where(final_data[:, 1]==starlink_id)[0]
	
		#	extract data for this starlink object
		full_starlink_data = final_data[starlink_indices, :]
	
		#	extract track ids
		track_id_list = np.unique(full_starlink_data[:, 0])
	
		#	plot parameters
		height_color_str = 'C' + str(color_index)
		station_time_color_str = 'C' + str(color_index)
		light_curve_color_str = 'C' + str(color_index)
	
		label_str = r'$Starlink-{}$'.format(int(int(starlink_id)))

		print('\n			Total Available Tracks: {}'.format(int(len(track_id_list))))

		###########################################################################################
		#	generate full list of TLEs
		
		print('\n					Acquiring OMMs...')
		
		#	TLE request dates, full range since launch
		tle_start_date = launch_date0
		tle_stop_date = simulation_end_date
		
		#	generate TLE file name
		tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
		tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
	
		omm_file = '{}_{}_{}.omm'.format(norad_id, tle_start_date_str, tle_stop_date_str)
	
		#	assemble TLE path
		omm_path = os.path.join(path_data, 'input_dir', 'omm_files')
	
		#	if exists, load
		if os.path.isfile(os.path.join(omm_path, omm_file)):
			print('\n						Reading existing OMM file.')
		
			#	load file
			with open(os.path.join(omm_path, omm_file), 'r') as file:
				omm_data = file.readlines()
		
		else:
			print('\n						Requesting OMM file...')
	
			#	request TLEs
			omm_data_raw = req_omm(tle_start_date, tle_stop_date, norad_id)
	
			omm_data = []
	
			#	save file
			with open(os.path.join(omm_path, omm_file), 'w') as file:
				for line_id, line in enumerate(omm_data_raw):
	
					if line != '':
	
						omm_data.append(line)
		
						if line_id != len(omm_data_raw) - 1:
							file.write(line + '\n')
						else:
							file.write(line)

			print('							OMM file saved.')
	
			print('\n							OMM Request Complete, pausing for 8s...')
			time.sleep(3)
		
		###########################################################################################
		#	calculating orbital height
		
		print('\n					Calculating Height Data and On-Station Time...')
	
		#	initialise height data
		height_data = []
		difference_data = []
		tle_datevec_array = []
		difference_data_datevec_array = []
	
		on_station_flag = False
		time_found_flag = False
		run_diff = 0
		run_diff_data = []
	
		spread_max = 1.5
		diff_threshold = 1.5
		
		alt_data = []
		perc_error = 1

		elements_data = []

		for omm_line_index, omm_line in enumerate(omm_data[1:]):
	
			#	parse omm line
			omm_vers, comment, creation_date, originator, obj_name, obj_id, center_obj, \
				ref_frame, time_system, mean_ele_theory, epoch_str, mean_mot, ecc, inc, \
				raan, aop, mean_anom, ephem_type, class_type, norad_cat_id, ele_set_no, \
				rev_at_epoch, bstar, mean_mot_dot, mean_mot_ddot, tle_line0, tle_line1, \
				tle_line2, sma, period, r_apo, r_peri, obj_type, decayed_state \
				= omm_line.split(',')
	
			#	parse data types
			omm_vers = float(omm_vers.split('"')[1])
			comment = comment.split('"')[1]
			creation_date_str = creation_date.split('"')[1]
			originator = originator.split('"')[1]
			obj_name = obj_name.split('"')[1]
			obj_id = obj_id.split('"')[1]
			center_obj = center_obj.split('"')[1]
			ref_frame = ref_frame.split('"')[1]
			time_system = time_system.split('"')[1]
			mean_ele_theory = mean_ele_theory.split('"')[1]
			epoch_str = epoch_str.split('"')[1]
			mean_mot = float(mean_mot.split('"')[1])
			ecc = float(ecc.split('"')[1])
			inc = float(inc.split('"')[1])
			raan = float(raan.split('"')[1])
			aop = float(aop.split('"')[1])
			mean_anom = float(mean_anom.split('"')[1])
			ephem_type = int(ephem_type.split('"')[1])
			class_type = class_type.split('"')[1]
			norad_cat_id = int(norad_cat_id.split('"')[1])
			ele_set_no = int(ele_set_no.split('"')[1])
			rev_at_epoch = int(rev_at_epoch.split('"')[1])
			bstar = float(bstar.split('"')[1])
			mean_mot_dot = float(mean_mot_dot.split('"')[1])
			mean_mot_ddot = float(mean_mot_ddot.split('"')[1])
			tle_line0 = tle_line0.split('"')[1]
			tle_line1 = tle_line1.split('"')[1]
			tle_line2 = tle_line2.split('"')[1]
			sma = float(sma.split('"')[1])
			period = float(period.split('"')[1])
			r_apo = float(r_apo.split('"')[1])
			r_peri = float(r_peri.split('"')[1])
			obj_type = obj_type.split('"')[1]
			decayed_state = int(decayed_state.split('"')[1])
	
			#	convert to radians per second
			mean_mot_rps = mean_mot / 86400 * (2*np.pi)
	
			#	calculate orbital altitude
			sat_height = sma - RE
	
			#	append to list of satellite altitudes
			alt_data.append(sat_height)

			#########################################################
			#	check altitude is within margin of operational altitude
			if altitude_flag == True:
				if (operational_alt - operational_band) > sat_height:
					continue
	
				elif (operational_alt + operational_band) < sat_height:
					continue
	
				#####################################################
				#	check altitude is consitent
		
				if omm_line_index > 1:
		
					#	convert to numpy array
					temp_array = np.array(alt_data)
		
					#	extract only previous 3 points
					temp_array = temp_array[omm_line_index - 2: omm_line_index + 1]
		
					if abs(temp_array[2] - temp_array[1]) > 0.5:
						continue
					elif abs(temp_array[2] - temp_array[0]) > 0.5:
						continue
					elif abs(temp_array[1] - temp_array[0]) > 0.5:
						continue

			#	if this is the first TLE to pass checks, set on-station 
			if on_station_flag is False:
				on_station_index = omm_line_index
				on_station_flag = True
	
			#	generate SGP4 satellite object
			satellite = twoline2rv(tle_line1, tle_line2, wgs72)

			#	extract TLE epoch
			tle_epoch = satellite.jdsatepoch

			#	convert to state vector
			pos, vel = satellite.propagate(*jd2date(*jday_jfrac(tle_epoch)))
	
			#	calculate height
			sat_height_sgp4 = np.linalg.norm(pos) - RE
	
			#	break condition for TLEs beyond window stop date
			if tle_epoch > np.add(*date2jd(*window_stop_date)):
				break
	
			#	ensure between window start/stop
			if tle_epoch > np.add(*date2jd(*window_start_date)):

				#	convert jd to date
				year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(tle_epoch))]
		
				#	turn into strings
				datestr = str('-'.join(
					['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
				))
		
				timestr = ':'.join(
					['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
				)
		
				datevec  = ' '.join([datestr, timestr])
		
				#	convert to datetime object
				date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
				tle_datevec_array.append(date_time_obj)
		
#				#	USING TLE SMA: add to outputs
#				height_data.append(sat_height)

				#	USING SGP4: add to outputs
				height_data.append(sat_height_sgp4)

				#	append to outputs
				elements_vec = [inc, raan, ecc, aop, mean_anom, mean_mot, sma, sat_height]
				elements_data.append(elements_vec)

		#	set norad id
		if on_station_flag is False:
			print('\n		Object does not arrive on-station.')
			continue
	
		#	convert to numpy array
		elements_data = np.array(elements_data)

		###############################################################################################
		#	acquire on-station date

		omm_on_station_line = omm_data[on_station_index + 1]
	
		#	parse omm line
		omm_vers, comment, creation_date, originator, obj_name, obj_id, center_obj, ref_frame, \
				time_system, mean_ele_theory, epoch_str, mean_mot, ecc, inc, raan, aop, mean_anom, \
				ephem_type, class_type, norad_cat_id, ele_set_no, rev_at_epoch, bstar, mean_mot_dot, \
				mean_mot_ddot, tle_line0, tle_line1, tle_line2, sma, period, r_apo, r_peri, obj_type, \
				decayed_state = omm_on_station_line.split(',')

		#	extract TLE
		on_station_tle_line0 = tle_line0.split('"')[1]
		on_station_tle_line1 = tle_line1.split('"')[1]
		on_station_tle_line2 = tle_line2.split('"')[1]
	
		#	generate SGP4 satellite object
		on_station_satellite = twoline2rv(on_station_tle_line1, on_station_tle_line2, wgs72)
	
		#	extract TLE epoch
		on_station_tle_epoch = on_station_satellite.jdsatepoch

		###############################################################################################
		#	generate filtered track list
		###############################################################################################

		if altitude_flag is True:
			filtered_track_id_list = []
			print('\n				Filtering to only those lightcurves since on-station date...')
	
			for track_itt, track_id in enumerate(track_id_list):
				
				#	find track ids
				track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
			
				#	extract full data
				track_data = full_starlink_data[track_indices, :]
			
				#######################################################################################
				#	extract track time data
				
				mmt_line = track_data[0]
				
				#	parse line elements
				track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
					mag_std, mag_app, distance, phase = track_data[0]
				
				start_jd = [jday, jfrac]
				
				###################################################################################
				#	filter light curves when not on station
		
				if np.add(*start_jd) < on_station_tle_epoch:
					print('\n					Track: {}; Not yet on-station...'.format(int(track_id)))
				else:
					print('\n					Track: {}; On-station...'.format(int(track_id)))
					filtered_track_id_list.append(int(track_id))
	
	
			#	reset track id list to filtered list
			track_id_list = filtered_track_id_list

			print('\n			Total Available Tracks On-Station: {}'.format(int(len(track_id_list))))

		if track_id_list == []:
			print('\n		No available tracks, continuing to next object...')
			continue
		else:
			ntracks = len(track_id_list)

		###############################################################################################
		#	find maximum brightness
	
		if min_brightness_filter is True:

			#	generate maximum brightness data
			min_mag_data = []
		
			for track_itt, track_id in enumerate(track_id_list):
	
				#	find track ids
				track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
			
				#	extract full data
				track_data = full_starlink_data[track_indices, :]
			
				#	extract apparent signal
				mmt_signal_app = track_data[:, 11]
	
				#	append minimum magnitude to array
				min_mag_data.append(mmt_signal_app.min())

			app_mag_data_max = np.array(min_mag_data).min()

			if app_mag_data_max > min_brightness:
				print('\n				Brightness is below threshold, continuing...')
				brightness_flag = False
				continue
			
			else:
				print('\n				Brightness is above threshold...')
				brightness_flag = True

		if max_brightness_filter is True:

			#	generate maximum brightness data
			max_mag_data = []
		
			for track_itt, track_id in enumerate(track_id_list):
	
				#	find track ids
				track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
			
				#	extract full data
				track_data = full_starlink_data[track_indices, :]
			
				#	extract apparent signal
				mmt_signal_app = track_data[:, 11]
	
				#	append minimum magnitude to array
				max_mag_data.append(mmt_signal_app.max())

			app_mag_data_min = np.array(max_mag_data).max()

			if app_mag_data_min < max_brightness:
				print('\n				Brightness is above threshold, continuing...')
				brightness_flag = False
				continue
			
			else:
				print('\n				Brightness is below threshold...')
				brightness_flag = True


		###############################################################################################
		#	find number of tracks to plot
	
		if minimum_tracks_filter is True:
	
			print('\n					Finding Number of Tracks...')
	
			#	continue flag for less than 2 tracks
			if ntracks < 2:
				print('\n						Less than 2 tracks, continuing...')
				continue
	
		###############################################################################################
		#	main track loop
	
		first_track = True
		track_index = 1
		plotted_track_indices = []
	
		for track_itt, track_id in enumerate(track_id_list):
		
			print('\n			#############################################################')
			print('			Working on Track ID: {}'.format(int(track_itt + 1)))
		
			#	remove legend if not first track
			if first_track is False:
				label_str = '_no_legend_'
		
			#	find track ids
			track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
		
			#	extract full data
			track_data = full_starlink_data[track_indices, :]
		
			#######################################################################################
			#	extract track time data
			
			mmt_line = track_data[0]
			
			#	parse line elements
			track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = track_data[0]
			
			start_jd = [jday, jfrac]
			
			#	parse line elements
			track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = track_data[-1]
			
			stop_jd = [jday, jfrac]
	
			###################################################################################
			#	filter light curves when not on station
	
			if altitude_flag == True:
				if np.add(*start_jd) < on_station_tle_epoch:
					print('\n				Not yet on station...')
					continue
	
			###################################################################################
			#	generate simulation time data
			
			#	extract start and stop jds
			lc_start_jd = start_jd[0] + start_jd[1]
			lc_stop_jd = stop_jd[0] + stop_jd[1]
			
			#	convert to dates
			lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
			lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
			
			#	light curve duration
			lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
			
			#	set time step
			lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
			
			#	time range in julian days
			lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
	
			sim_range_sec = np.round(
				np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400),
				1
			)
		
			print('\n				Light curve parameters: ')
			print(
				'					Start Date: {}/{:02d}/{:02d}	- {:02d}:{:02d}:{:04.1f}'\
				.format(*lc_start_date)
			)
		
			print(
				'					Stop Date: {}/{:02d}/{:02d}	- {:02d}:{:02d}:{:04.1f}'\
				.format(*lc_stop_date)
			)
		
			print('					Simulation time step:	{}'.format(time_step))
			print('					Duration:		{}	[steps]'.format(lc_len_record))
			print('								{}	[s]'.format(np.round(lc_duration_sec, 1)))
		
			#######################################################################################
			#	ensure time continuity
			
			print('\n				Ensuring time continuity in extracted data...')
			
			track_data_temp = []
			sim_index = 0
		
			for itt, line in enumerate(track_data):
			
				#	parse line data
				track_id, object_id, \
					jday, jfrac, year, month, day, hour, minute, second, \
					mag_std, mag_app, distance, phase = line
				
				step_jd = jday + jfrac
			
				#	calculate current seconds
				current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
			
				output_line = [
					jday, jfrac, current_sec, year, month, day, hour, minute, second, \
					mag_std, mag_app, distance, phase
				]
			
				##################################################
				#	extracting data of time step
			
				#	data match found
				if current_sec == sim_range_sec[sim_index]:
					track_data_temp.append(output_line)
			
					#	itterate number of output data points
					sim_index += 1
			
					#	save current line as previous line
					prev_line = output_line
			
				#	data point is not required
				elif current_sec < sim_range_sec[sim_index]:
					pass
			
				#	interpolation required
				elif current_sec > sim_range_sec[sim_index]:
			
					#	parse previous line
					jday_prev, jfrac_prev, prev_sec, \
						year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
						mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
			
					#	calculate interpolation requirements
					gap = np.round(current_sec - prev_sec, 1)
					npts = int(gap / time_step)
			
					#	get current light curve data
					mag_std_last = mag_std
					mag_app_last = mag_app
					distance_last = distance
					phase_last = phase
			
					#	calculate linear interpolation step values
					mag_std_step = (mag_std_last - mag_std_prev) / npts
					mag_app_step = (mag_app_last - mag_app_prev) / npts
					distance_step = (distance_last - distance_prev) / npts
					phase_step = (phase_last - phase_prev) / npts
			
					#	calculate interpolation data
					for n in range(1, npts + 1):
			
						#	current simulation second
						interp_sec = np.round(prev_sec + (n * time_step), 1)
	
						#	time data
						jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
						jday, jfrac = jday_jfrac(jdate)
						year, month, day, hour, minute, second = jd2date(jday, jfrac)
			
						#	calculate interpolated light curve data
						mag_std = mag_std_prev + (n * mag_std_step)
						mag_app = mag_app_prev + (n * mag_app_step)
						distance = distance_prev + (n * distance_step)
						phase = phase_prev + (n * phase_step)
			
						#	generate new output line
						interp_output_line = [
							jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
							mag_std, mag_app, distance, phase
						]
			
						if interp_sec == sim_range_sec[sim_index]:
							track_data_temp.append(interp_output_line)
							sim_index += 1
			
					if interp_sec != current_sec:
						#	append final line
						track_data_temp.append(output_line)
						sim_index += 1
			
				else:
					print('ERROR')
					sys.exit()
			
				#	save previous line
				if itt != 0:
					prev_line = output_line
			
				#	array is complete condition
				if len(track_data_temp) == lc_len_record:
					break
			
			#	ensure parsed MMT data matches simulation data
			track_data = np.array(track_data_temp)
			
			if len(track_data) == len(lc_sim_range):
				print('					Simulation and MMT data matches.')
			else:
				print('					Error: Simulation and MMT data does not match...Exiting.')
				sys.exit()
			
			###########################################################################################
			#	extract mmt brightness data
			mmt_signal_std = track_data[:, 9]
			mmt_signal_app = track_data[:, 10]
			
			###########################################################################################
			#	select time data to use
			
			sim_range = lc_sim_range
			len_record = lc_len_record
			start_date = lc_start_date
			stop_date = lc_stop_date
			duration_sec = lc_duration_sec
		
			###########################################################################################
			#	generate date vectors
			datevec_array = []
		
			for line in sim_range:
				#	convert jd to date
				year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(line))]
		
				#	turn into string
				datestr = str('-'.join(
					['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
				))
		
				timestr = ':'.join(
					['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
				)
		
				datevec  = ' '.join([datestr, timestr])
				#	convert to datetime object
				date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
				#	append  to output array
				datevec_array.append(date_time_obj)
		
			###########################################################################################
			#	plot brightness data

			#	setting black only colours
			if color_only_black is True:
				light_curve_color_str = 'k'
				height_color_str = 'k'

			#	coloring by batch
			if color_by_batch == True:
				if launch_string == 'batch0':
					light_curve_color_str = 'tab:red'
					height_color_str = 'tab:red'
				elif launch_string == 'batch1':
					light_curve_color_str = 'tab:green'
					height_color_str = 'tab:green'
				elif launch_string == 'batch2':
					light_curve_color_str = 'tab:blue'
					height_color_str = 'tab:blue'

			light_curve_index += 1
			track_index += 1
			first_track = False
	
			plotted_light_curves += 1
			plotted_track_indices.append(int(track_itt + 1))

			if starlink_id not in plotted_object_indices:
				plotted_object_indices.append(starlink_id)

			tracks_plotted += 1
			total_tracks_plotted += 1

			print('\n				Track Complete.')
	
		if plotted_light_curves == 0:
			print('\n		No tracks plotted, continuing...')
			continue

		# find where onstation date occurs in height data
		if plot_on_station_date is True:

			#	find date index
			for date_index, date in enumerate(tle_datevec_array):
				if date == on_station_datevec_array[0]:
					break
			tle_datevec_array = tle_datevec_array[date_index:]
			height_data = height_data[date_index:]

		#	plot height data
		ax[0].scatter(
			tle_datevec_array, height_data,
			marker=marker_symbol,
			s=0.1,
			color=height_color_str,
		)
	
		#	plot inclination data
		ax[1].scatter(
			tle_datevec_array, elements_data[:, 0],
			marker=marker_symbol,
			s=0.1,
			color=height_color_str,
		)
	
		#	plot eccentricity data
		ax[2].scatter(
			tle_datevec_array, elements_data[:, 2],
			marker=marker_symbol,
			s=0.1,
			color=height_color_str,
		)
	
		#	plot orbital altitude data
		ax[3].scatter(
			tle_datevec_array, elements_data[:, -1],
			marker=marker_symbol,
			s=0.1,
			color=height_color_str,
		)

		obj_plotted += 1
		total_obj_plotted += 1

		print('\n		Starlink Object Complete.')
	
		if color_by_object is True:
			#	itterate color index
			color_index += 1
		
		###########################################################################################
		#	finalise
	
		if pause_flag is True:
			plt.pause(0.1)
			input('\n		Enter to Continue...')

		print('\n		Plotted_object_indices = ', plotted_object_indices)

	#	convert to numpy array
	plotted_object_indices = np.array(plotted_object_indices)

	#	set total plotted object indices list
	if first_batch is True:
		total_plotted_object_indices = plotted_object_indices

		first_batch = False

	else:
		total_plotted_object_indices = np.hstack([
			total_plotted_object_indices,
			plotted_object_indices
		])

	print('\n	Loop Complete...')
	print('\n	This Batch, {} Objects were Plotted...'.format(int(obj_plotted)))
	print('\n	This Batch, {} Light Curves were Plotted...'.format(int(tracks_plotted)))

	print('\n	This Batch, Plotted Objects:\n	', plotted_object_indices)
#	print('\n	This Batch, Plotted Track Indices:\n	', plotted_track_indices)

	###################################################################################################
	
	if show_legend is True:
	
		fig.legend(
			loc='lower center',
			fontsize=legend_fontsize,
			ncol=6,
			frameon=False
		)
	
print('\nLoop Complete...')
print('\nA Total of {} Objects were Plotted...'.format(int(total_obj_plotted)))
print('\nA Total of {} Light Curves were Plotted...'.format(int(total_tracks_plotted)))

print('\nIn Total, Plotted Objects:\n	', total_plotted_object_indices)
# print('\nIn Total, Plotted Track Indices:\n	', plotted_track_indices)

print('\nDone.')





if savefile_flag is True:
	fig_name = 'starlink_' + launch_string + '_full.eps'
	fig_name = 'starlink_' + launch_string + '_final.eps'

	out_path = os.path.join(fig_path, fig_name)

	plt.savefig(out_path, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)




plt.show()
sys.exit()

