###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Starlink File Plotter')
print('#####################################################################################')

print('\n\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

local_input_path = os.path.join('input_dir', 'mmt_files', 'starlink', 'batch-0')
read_path = os.path.join(path_data, local_input_path)

###################################################################################################
#	script specific initialisation

#	set earth radius
RE = 6371

#	define a small number
small = 1e-8

specific_object_flag = False
specific_object_flag = True

#	define list of specific objects
specific_objects = [1009, 1013]
specific_objects = [29]

#	convert to numpy array
specific_objects = np.array(specific_objects)

##################################################
#	filter flags

min_brightness = 3.0

min_brightness_filter = True
min_brightness_filter = False

on_station_filter = True
on_station_filter = False

minimum_tracks_filter = True
minimum_tracks_filter = False

color_by_object = False
color_by_object = True

label_by_object = False
label_by_object = True

plot_on_station_date = True
plot_on_station_date = False

###################################################################################################
#	light curve generation inputs
###################################################################################################

#	set time step
time_step = 0.1

#	set time windows
window_start_date = [2010, 1, 1, 0, 0, 0]
window_stop_date = [2022, 11, 1, 0, 0, 0]

#	set launch date
launch_date = [2019, 5, 24, 2, 30, 0.0]
simulation_end_date = [2019, 12, 30, 0, 0, 0.0]

###################################################################################################
#	read and sort starlink files
###################################################################################################

print('\n[{}] Generating Starlink file name list...'.format(datetime.datetime.now()))

starlink_dir = os.listdir(read_path)
starlink_files = []

#	order by starlink id
for file in starlink_dir:
	object_id = int(file.split('-')[1].split('_')[0])
	starlink_files.append([file, object_id])

starlink_files = sorted(starlink_files, key=lambda x: x[1])

###################################################################################################
#	starlink file loop
###################################################################################################

print('\n[{}] Extracting ALL MMT file data...\n'.format(datetime.datetime.now()))
first_flag = True

norad_id_list = np.zeros([len(starlink_files), 2])

for file_num, mmt_file in enumerate(starlink_files):

	print('	Parsing MMT data file: {}'.format(mmt_file), end='\r')

	###################################################################################################
	#	load MMT file
	###################################################################################################

	mmt_file = mmt_file[0]

	#	parse filename data
	name_str, norad_id = mmt_file[:-4].split('_')
	object_id = int(name_str.split('-')[1])

	name_str = ''.join(name_str.split('-'))
	norad_id = int(norad_id)
	object_id = int(object_id)
	
	#	open and read file
	with open(os.path.join(read_path, mmt_file)) as read_file:
		mmt_data = read_file.readlines()
	
	#	add to norad id list
	norad_id_list[file_num] = [object_id, norad_id]
	
	###################################################################################################
	#	parse mmt file
	###################################################################################################
	
	#	strip header
	header_text = mmt_data[7]
	raw_data = mmt_data[8:]
	
	#	initialise output
	n_rows = 14
	parsed_data = np.zeros([len(raw_data), n_rows])
	
	for i, line in enumerate(raw_data):

		#	split line
		date, time, mag_std, mag_app, opt_filter, penumbral, distance, phase, channel, track_id \
			= line.split(' ')
	
		#	parse data and set data types
		mag_std = float(mag_std)
		mag_app = float(mag_app)
		distance = float(distance)
		phase = float(phase)
		track_id = int(track_id)
	
		#	parse date
		year, month, day = date.split('-')
		year = int(year)
		month = int(month)
		day = int(day)
	
		#	parse time
		hour, minute, second = time.split(':')
		hour = int(hour)
		minute = int(minute)
		second = float(second)
	
		#	calculate julian date
		jday, jfrac = date2jd(year, month, day, hour, minute, second)
	
		#	generate output line
		output_line = [
			track_id, object_id,
			jday, jfrac,
			year, month, day,
			hour, minute, second,
			mag_std, mag_app,
			distance, phase
		]
	
		#	add to output array
		parsed_data[i, :] = output_line

	#	initialise output array
	if first_flag is True:
		total_data = parsed_data

		first_flag = False

	#	stack with existing output array
	elif first_flag is False:
		total_data = np.vstack([total_data, parsed_data])

sys.stdout.write("\033[K")
print('	Done.\n', end='\r')

print('\nFinished MMT file parsing...')
print('	Total data length = ', len(total_data))
print('	Total number of objects = ', len(np.unique(total_data[:, 1])))
print('	Total number of light curves = ', len(np.unique(total_data[:, 0])))

###################################################################################################
#	collect data in window of interest
###################################################################################################

print('\n[{}] Extracting light curve inside time window...'.format(datetime.datetime.now()))

#	generate window start/stop JDs
window_start_jd = np.add(*date2jd(*window_start_date))
window_stop_jd = np.add(*date2jd(*window_stop_date))

print('\n	Window start 	= ', window_start_date[0:3])
print('	Window stop 	= ', window_stop_date[0:3])
print('\n')

first_lc_flag = True

#	extract starlink ids
object_id_list = np.unique(total_data[:, 1])

light_curve_index = 1

#	loop through starlink IDs
for object_id in object_id_list:

	print('	Starlink-{}: '.format(int(object_id)), end='')

	#	find track ids
	starlink_indices = np.where(total_data[:, 1]==object_id)[0]

	#	extract data for this starlink object
	full_starlink_data = total_data[starlink_indices, :]

	#	extract track ids
	track_id_list = np.unique(full_starlink_data[:, 0])

	ntracks = 0

	#	loop through tracks
	for track_id in track_id_list:
	
		#	find track ids
		track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]

		#	extract full data
		track_data = full_starlink_data[track_indices, :]
	
		#	extract light curve data
		jd_plot = np.add(track_data[:, 2], track_data[:, 3])
		std_mag_plot = track_data[:, 10]
		app_mag_plot = track_data[:, 11]
		distance_plot = track_data[:, 12]
		phase_plot = track_data[:, 13]
	
		start_line = track_data[0]
		stop_line = track_data[-1]
	
		start_jdate = np.add(start_line[2], start_line[3])
		stop_jdate = np.add(stop_line[2], stop_line[3])
	
		#	ensure between window start/stop
		if start_jdate > window_start_jd and stop_jdate < window_stop_jd:

			#	light curve is within window
			ntracks += 1
	
			#	if first light curve, start array
			if first_lc_flag is True:
				final_data = track_data
				first_lc_flag = False

			#	else append to list
			elif first_lc_flag is False:
				final_data = np.vstack([final_data, track_data])

	print('{} track(s) extracted.'.format(ntracks))

print('\nFinished window extraction...')
print('	Total data length = ', len(final_data))
print('	Total number of objects = ', len(np.unique(final_data[:, 1])))
print('	Total number of light curves = ', len(np.unique(final_data[:, 0])))

###################################################################################################
#	plot light curve properties
###################################################################################################

print('\n#####################################################################################')
print('[{}] Plotting Starlink Light Curve Properties...'.format(datetime.datetime.now()))

#	inch to cm conversion
inch2cm = 2.54

#	set subplot spacing values
hspace_val = 0.35
wspace_val = 0.35

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4
xlabel_fontsize = 6
ylabel_fontsize = 6
xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set figure line wdith
plot_linewidth = 1.0
 
#	set figure height/width
fig_width = 9.0/inch2cm
fig_height = 12.0/inch2cm

#	set use of LaTeX font
plt.rc('text', usetex=True)

#	initialise figure
fig, ax = plt.subplots(3, 2, figsize=(fig_width, fig_height), dpi=300)

#	adjust subplot spacing
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

years = mdates.YearLocator()
months = mdates.MonthLocator()
days = mdates.DayLocator()

###################################################################################################
#	full final data

light_curve_index = 1
object_color_index = 0
total_duration = 0

#	extract starlink ids
starlink_id_list = np.unique(final_data[:, 1])

#	if specific object flag is true, use object list
if specific_object_flag is True:
	starlink_id_list = specific_objects

###################################################################################################
print('\n	Looping Through Starlink Light Curve Files...')

fig.suptitle('All Remaining MMT Starlink Light Curves')

#	loop through starlink IDs
for loop_itt, starlink_id in enumerate(starlink_id_list):

	#	set norad id
	norad_index = np.where(norad_id_list[:, 0]==starlink_id)[0][0]
	norad_id = int(norad_id_list[norad_index, 1])

	###############################################################################################
	print('\n		#####################################################################')
	print('		Plotting data for: Starlink-{}'.format(int(starlink_id)))

	#	find track ids
	starlink_indices = np.where(final_data[:, 1]==starlink_id)[0]

	#	extract data for this starlink object
	full_starlink_data = final_data[starlink_indices, :]

	#	extract track ids
	track_id_list = np.unique(full_starlink_data[:, 0])

	###############################################################################################
	#	main track loop

	first_track = True
	track_index = 1
	track_color_index = 0
	tracks_plotted = 0

	for track_itt, track_id in enumerate(track_id_list):
	
		print('\n			#############################################################')
		print('			Working on Track ID: {}'.format(int(track_itt + 1)))
	
		#	remove legend if not first track
		if first_track is False:
			label_str = '_no_legend_'
	
		#	find track ids
		track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = full_starlink_data[track_indices, :]

		#######################################################################################
		#	extract track time data
		
		mmt_line = track_data[0]
		
		#	parse line elements
		track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]
		
		###################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)

		sim_range_sec = np.round(
			np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400),
			1
		)

		#######################################################################################
		#	ensure time continuity
		
		print('\n				Ensuring time continuity in extracted data...')
		
		track_data_temp = []
		sim_index = 0
	
		for itt, line in enumerate(track_data):
		
			#	parse line data
			track_id, object_id, \
				jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = line
			
			step_jd = jday + jfrac
		
			#	calculate current seconds
			current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
		
			output_line = [
				jday, jfrac, current_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]
		
			##################################################
			#	extracting data of time step
		
			#	data match found
			if current_sec == sim_range_sec[sim_index]:
				track_data_temp.append(output_line)
		
				#	itterate number of output data points
				sim_index += 1
		
				#	save current line as previous line
				prev_line = output_line
		
			#	data point is not required
			elif current_sec < sim_range_sec[sim_index]:
				pass
		
			#	interpolation required
			elif current_sec > sim_range_sec[sim_index]:
		
				#	parse previous line
				jday_prev, jfrac_prev, prev_sec, \
					year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
					mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
		
				#	calculate interpolation requirements
				gap = np.round(current_sec - prev_sec, 1)
				npts = int(gap / time_step)
		
				#	get current light curve data
				mag_std_last = mag_std
				mag_app_last = mag_app
				distance_last = distance
				phase_last = phase
		
				#	calculate linear interpolation step values
				mag_std_step = (mag_std_last - mag_std_prev) / npts
				mag_app_step = (mag_app_last - mag_app_prev) / npts
				distance_step = (distance_last - distance_prev) / npts
				phase_step = (phase_last - phase_prev) / npts
		
				#	calculate interpolation data
				for n in range(1, npts + 1):
		
					#	current simulation second
					interp_sec = np.round(prev_sec + (n * time_step), 1)

					#	time data
					jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
					jday, jfrac = jday_jfrac(jdate)
					year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
					#	calculate interpolated light curve data
					mag_std = mag_std_prev + (n * mag_std_step)
					mag_app = mag_app_prev + (n * mag_app_step)
					distance = distance_prev + (n * distance_step)
					phase = phase_prev + (n * phase_step)
		
					#	generate new output line
					interp_output_line = [
						jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
		
					if interp_sec == sim_range_sec[sim_index]:
						track_data_temp.append(interp_output_line)
						sim_index += 1
		
				if interp_sec != current_sec:
					#	append final line
					track_data_temp.append(output_line)
					sim_index += 1
		
			else:
				print('ERROR')
				sys.exit()
		
			#	save previous line
			if itt != 0:
				prev_line = output_line
		
			#	array is complete condition
			if len(track_data_temp) == lc_len_record:
				break
		
		#	ensure parsed MMT data matches simulation data
		track_data = np.array(track_data_temp)
		
		if len(track_data) == len(lc_sim_range):
			print('					Simulation and MMT data matches.')
		else:
			print('					Error: Simulation and MMT data does not match...Exiting.')
			sys.exit()
		
		###########################################################################################
		#	extract mmt data

		jd_plot = np.add(track_data[:, 0], track_data[:, 1])
		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]
		distance_plot = track_data[:, 11]
		phase_plot = track_data[:, 12]

		#	calculate flux
		flux_plot = np.zeros(jd_plot.shape)
		for i, app_mag in enumerate(mmt_signal_app):
			flux_plot[i] = 1367 * 10**((app_mag - -26.74) / -2.5)
		
		#	seconds plot
		seconds_plot = np.multiply(np.subtract(jd_plot, jd_plot[0]), 86400)

		###########################################################################################
		#	select time data to use
		
		sim_range = lc_sim_range
		len_record = lc_len_record
		start_date = lc_start_date
		stop_date = lc_stop_date
		duration_sec = lc_duration_sec

		###########################################################################################
		#	generate date vectors
		datevec_array = []
		
		for line in sim_range:
			#	convert jd to date
			year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(line))]
		
			#	turn into string
			datestr = str('-'.join(
				['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
			))
		
			timestr = ':'.join(
				['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
			)
		
			datevec  = ' '.join([datestr, timestr])
			#	convert to datetime object
			date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
			#	append  to output array
			datevec_array.append(date_time_obj)
	
		###########################################################################################
		#	plot data

		if color_by_object is True:
			color_str = 'C' + str(object_color_index)

		elif color_by_object is False:
			color_str = 'C' + str(track_color_index)

		if label_by_object is True:
			label_str = 'Starlink-' + str(int(starlink_id))

		elif label_by_object is False:
			label_str = 'Track-' + str(int(track_index))

		#	stacked standard
		ax[0, 0].plot(seconds_plot, mmt_signal_std, color=color_str, lw=plot_linewidth)
		
		#	stacked apparant
		ax[0, 1].plot(seconds_plot, mmt_signal_app, color=color_str, lw=plot_linewidth)
		
		#	against distance
		ax[1, 0].plot(distance_plot, mmt_signal_app, color=color_str, lw=plot_linewidth)
		
		#	against phase
		ax[1, 1].plot(phase_plot, mmt_signal_app, color=color_str, lw=plot_linewidth)
		
		#	flux plot
		ax[2, 0].plot(seconds_plot, flux_plot, color=color_str, lw=plot_linewidth)
		
		#	against JD
		ax[2, 1].plot(
			datevec_array, mmt_signal_app,
			label=label_str, color=color_str,
			lw=plot_linewidth
		)
		
		total_duration += seconds_plot[-1] 
		
		light_curve_index += 1
		track_index += 1
		first_track = False
		tracks_plotted += 1
		track_color_index += 1
		
		print('\n				Track Complete.')

	print('\n		Plotted {} curves...'.format(int(tracks_plotted)))

	#	itterate color index
	object_color_index += 1
	
print('\nLoop Complete...')
print('\n	Total Duration = {} s'.format(np.round(total_duration, 1)))

###################################################################################################
#	finalise plots

print('\nFinalising Figure...')

#	set figure titles
ax[0, 0].set_title('Standard magnitude against Time',
	fontsize=title_fontsize, pad=title_pad
)

ax[0, 1].set_title('Apparent magnitude against Seconds since start',
	fontsize=title_fontsize, pad=title_pad
)

ax[1, 0].set_title('Apparent magnitude against Range',
	fontsize=title_fontsize, pad=title_pad
)

ax[1, 1].set_title('Apparent magnitude against Phase',
	fontsize=title_fontsize, pad=title_pad
)

ax[2, 0].set_title('Flux against Seconds since start',
	fontsize=title_fontsize, pad=title_pad
)

ax[2, 1].set_title('Apparent magnitude against Time',
	fontsize=title_fontsize, pad=title_pad
)

#	set axis range
ax[0, 0].set_ylim([10, 0])
ax[0, 0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[0, 0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax[0, 1].set_ylim([10, 0])
ax[0, 1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[0, 1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax[1, 0].set_ylim([10, 0])
ax[1, 0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[1, 0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax[1, 1].set_ylim([10, 0])
ax[1, 1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[1, 1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax[2, 0].set_ylim([0.0, 2.5e-8])
ax[2, 0].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[2, 0].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax[2, 1].set_ylim([10, 0])
ax[2, 1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[2, 1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax[1, 1].set_xlim([0.0, 180])
ax[1, 1].tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax[1, 1].tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

#	format time axis
formatter = mdates.DateFormatter('%m-%d')
ax[2, 1].xaxis.set_major_formatter(formatter)

print('\nDone. Showing Figure...')

plt.show()
sys.exit()
