###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Starlink File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'starlink', 'batch0')

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Articles',
	'starlink_paper',
	'Figures'
)

fig_name = 'starlink29_properties.eps'
out_path = os.path.join(fig_path, fig_name)

#	set flag
savefile_flag = True
savefile_flag = False

###################################################################################################
#	inputs
###################################################################################################

mmt_files = [
#	'starlink-26_44240.txt',
#	'starlink-27_44241.txt',
	'STARLINK-24_44238_MMT_6404.txt',
#	'starlink-39_44253.txt',
#	'starlink-42_44256.txt'
]

track_index_ranges = np.array([
	[2],
	[4],
	[6],
	[6],
	[5]
])

#	set time windows
window_start_date = [2010, 1, 1, 0, 0, 0]
window_stop_date = [2019, 11, 1, 0, 0, 0]

window_start_date = [2019, 11, 30, 0, 0, 0]
window_stop_date = [2020, 5, 21, 0, 0, 0]

#	set launch date
launch_date = [2019, 5, 24, 2, 30, 0.0]

#	set time step
time_step = 0.1

#	set earth radius
RE = 6371

#	define a small number
small = 1e-8

##################################################
#	filter flags
min_brightness = 3.0

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	light curve generation inputs

#	set initial BRDF values
d_brdf = 0.4
omega_brdf = 0.5
m_brdf = 0.06

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str_full = 'starlink_full'

geom_file_full = geom_str_full + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 0.0

#	attitude motion
rot_period = 0.0

###################################################################################################
#	load geometry

#	load static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file_full))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors = body_mesh.vectors
body_normals = body_mesh.normals
body_areas = body_mesh.areas

nfaces = len(body_areas)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])

###################################################################################################
#	plot light curve properties
###################################################################################################

print('\n#####################################################################################')
print('[{}] Plotting Starlink Light Curve Properties...'.format(datetime.datetime.now()))

###################################################################################################
#	initialise figure

color_by_object = True
label_by_object = True

#	figure parameters
font_sizex = 6
font_sizey = 6
legend_fontsize = 6

label_sizex = 6
label_sizey = 6
label_pad = 1

font_pad = 1
marker_size = 2
marker_edge = 1

hspace_val = 0.5
wspace_val = 0.35

grid_width = 0.1

launch_linewidth = 1
plot_lw = 0.8

title_size = 8
title_pad = 10

dpi_val = 300
inch2cm = 2.54

plt.rc('text', usetex=True)

fig, ax = plt.subplots(2, 2, figsize=(12.0/inch2cm, 6.0/inch2cm), dpi=300)

plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)
plt.gcf().subplots_adjust(bottom=0.25)

ax1 = ax[0, 0]
ax2 = ax[0, 1]
ax3 = ax[1, 0]
ax4 = ax[1, 1]

###################################################################################################
#	full final data

light_curve_index = 1
object_color_index = 0
total_duration = 0

###################################################################################################
print('\n	Looping Through Starlink Light Curve Files...')

#	loop through starlink IDs
# for loop_itt, starlink_id in enumerate(starlink_id_list):
for (mmt_file, track_index_range) in zip(mmt_files, track_index_ranges):

	#	parse filename
	name_str = mmt_file.split('-')[0]
	object_id = int(mmt_file.split('-')[1].split('_')[0])
	norad_id = int(mmt_file.split('-')[1].split('_')[1])

#	#	set norad id
#	norad_index = np.where(norad_id_list[:, 0]==starlink_id)[0][0]
#	norad_id = int(norad_id_list[norad_index, 1])

	print('\n		#####################################################################')
	print('		Plotting data for: Starlink-{}'.format(int(object_id)))

	###############################################################################################
	#	load MMT file
	
	#	set time
	mmt_load_start = time.time()
	
	print('\n	[{}] Loading MMT data...'.format(datetime.datetime.now()))
	
	with open(os.path.join(read_path, mmt_file)) as read_file:
		mmt_data = read_file.readlines()
	
	###############################################################################################
	#	parse MMT file
	
	#	strip header
	header_text = mmt_data[7]
	raw_data = mmt_data[8:]
	
	print('\n		Parsing MMT data...')
	
	#	initialise output
	n_rows = 14
	parsed_data = np.zeros([len(raw_data), n_rows])
	
	for i, line in enumerate(raw_data):
	
		#	split line
		date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
			distance, phase, channel, track_id = line.split(' ')
	
		#	parse data and set data types
		mag_std = float(mag_std)
		mag_app = float(mag_app)
		distance = float(distance)
		phase = float(phase)
		track_id = int(track_id)
	
		#	parse date
		year, month, day = date_str.split('-')
		year = int(year)
		month = int(month)
		day = int(day)
	
		#	parse time
		hour, minute, second = time_str.split(':')
		hour = int(hour)
		minute = int(minute)
		second = np.round(float(second), 1)
	
		#	calculate julian date
		jday, jfrac = date2jd(year, month, day, hour, minute, second)
	
		#	generate output line
		output_line = [
			track_id, object_id,
			jday, jfrac,
			year, month, day,
			hour, minute, second,
			mag_std, mag_app,
			distance, phase
		]
	
		#	add to output array
		parsed_data[i, :] = output_line
	
	print('			Total data length = ', len(parsed_data))
	print('			Total number of objects = ', len(np.unique(parsed_data[:, 1])))
	print('			Total number of tracks = ', len(np.unique(parsed_data[:, 0])))
	
	###############################################################################################
	#	extracting data between specified windows
	
	print('\n		Seaching for tracks inside time window:')
	
	#	start window
	window_start_jd = np.add(*date2jd(*window_start_date))
	print('\n			Window start 	= ', window_start_date[0:3])
	
	#	stop window
	window_stop_jd = np.add(*date2jd(*window_stop_date))
	print('			Window stop 	= ', window_stop_date[0:3])
	
	#	extract track ids
	track_id_list = np.unique(parsed_data[:, 0])
	extracted_track_id_list = []

	#	loop through tracks
	ntracks = 0
	first_lc_flag = True
	for track_id in track_id_list:
	
		#	find track ids
		track_indices = np.where(parsed_data[:, 0]==track_id)[0]
	
		#	extract full data
		track_data = parsed_data[track_indices, :]
	
		#	extract light curve data
		jd_plot = np.add(track_data[:, 2], track_data[:, 3])
		std_mag_plot = track_data[:, 10]
		app_mag_plot = track_data[:, 11]
		distance_plot = track_data[:, 12]
		phase_plot = track_data[:, 13]
		start_line = track_data[0]
		stop_line = track_data[-1]
		start_jdate = np.add(start_line[2], start_line[3])
		stop_jdate = np.add(stop_line[2], stop_line[3])
	
		#	ensure between window start/stop
		if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
	
			#	light curve is within window
			ntracks += 1
	
			#	if first light curve, start array
			if first_lc_flag is True:
				windowed_data = track_data
				first_lc_flag = False
	
			#	else append to list
			elif first_lc_flag is False:
				windowed_data = np.vstack([windowed_data, track_data])
	
			#	add track id to output
			extracted_track_id_list.append(track_id)

	print('\n			{} track(s) extracted.'.format(ntracks))
	extracted_track_id_list = np.array(extracted_track_id_list)

	###############################################################################################
	#	main track loop
	###############################################################################################

	first_track = True
	track_index = 1
	track_color_index = 0
	tracks_plotted = 0

	#	loop through tracks
	for track_itt, track_index in enumerate(track_index_range):
	
		print('\n			#############################################################')
		print('			Working on Track ID: {}'.format(int(track_itt + 1)))
	
		#	remove legend if not first track
		if first_track is False:
			l_str = '_no_legend_'
	
		###########################################################################################
		#	extracting requested single track
		
		print('\n			Extracting requested track index...')
		
		#	set track array index
		track_array_index = track_index - 1
		track_id = int(extracted_track_id_list[track_array_index])
		
		track_indices = np.where(windowed_data[:, 0]==track_id)[0]
		track_data = windowed_data[track_indices]

		#######################################################################################
		#	extract track time data
		
		mmt_line = track_data[0]
		
		#	parse line elements
		track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[0]
		
		start_jd = [jday, jfrac]
		
		#	parse line elements
		track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
			mag_std, mag_app, distance, phase = track_data[-1]
		
		stop_jd = [jday, jfrac]
		
		###################################################################################
		#	generate simulation time data
		
		#	extract start and stop jds
		lc_start_jd = start_jd[0] + start_jd[1]
		lc_stop_jd = stop_jd[0] + stop_jd[1]
		
		#	convert to dates
		lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
		lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
		
		#	light curve duration
		lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
		
		#	set time step
		lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
		
		#	time range in julian days
		lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)

		sim_range_sec = np.round(
			np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400),
			1
		)

		#######################################################################################
		#	ensure time continuity
		
		print('\n				Ensuring time continuity in extracted data...')
		
		track_data_temp = []
		sim_index = 0
	
		for itt, line in enumerate(track_data):
		
			#	parse line data
			track_id, object_id, \
				jday, jfrac, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase = line
			
			step_jd = jday + jfrac
		
			#	calculate current seconds
			current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
		
			output_line = [
				jday, jfrac, current_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]
		
			##################################################
			#	extracting data of time step
		
			#	data match found
			if current_sec == sim_range_sec[sim_index]:
				track_data_temp.append(output_line)
		
				#	itterate number of output data points
				sim_index += 1
		
				#	save current line as previous line
				prev_line = output_line
		
			#	data point is not required
			elif current_sec < sim_range_sec[sim_index]:
				pass
		
			#	interpolation required
			elif current_sec > sim_range_sec[sim_index]:
		
				#	parse previous line
				jday_prev, jfrac_prev, prev_sec, \
					year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
					mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
		
				#	calculate interpolation requirements
				gap = np.round(current_sec - prev_sec, 1)
				npts = int(gap / time_step)
		
				#	get current light curve data
				mag_std_last = mag_std
				mag_app_last = mag_app
				distance_last = distance
				phase_last = phase
		
				#	calculate linear interpolation step values
				mag_std_step = (mag_std_last - mag_std_prev) / npts
				mag_app_step = (mag_app_last - mag_app_prev) / npts
				distance_step = (distance_last - distance_prev) / npts
				phase_step = (phase_last - phase_prev) / npts
		
				#	calculate interpolation data
				for n in range(1, npts + 1):
		
					#	current simulation second
					interp_sec = np.round(prev_sec + (n * time_step), 1)

					#	time data
					jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
					jday, jfrac = jday_jfrac(jdate)
					year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
					#	calculate interpolated light curve data
					mag_std = mag_std_prev + (n * mag_std_step)
					mag_app = mag_app_prev + (n * mag_app_step)
					distance = distance_prev + (n * distance_step)
					phase = phase_prev + (n * phase_step)
		
					#	generate new output line
					interp_output_line = [
						jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
		
					if interp_sec == sim_range_sec[sim_index]:
						track_data_temp.append(interp_output_line)
						sim_index += 1
		
				if interp_sec != current_sec:
					#	append final line
					track_data_temp.append(output_line)
					sim_index += 1
		
			else:
				print('ERROR')
				sys.exit()
		
			#	save previous line
			if itt != 0:
				prev_line = output_line
		
			#	array is complete condition
			if len(track_data_temp) == lc_len_record:
				break
		
		#	ensure parsed MMT data matches simulation data
		track_data = np.array(track_data_temp)
		
		if len(track_data) == len(lc_sim_range):
			print('					Simulation and MMT data matches.')
		else:
			print('					Error: Simulation and MMT data does not match...Exiting.')
			sys.exit()
		
		###########################################################################################
		#	extract mmt data

		jd_plot = np.add(track_data[:, 0], track_data[:, 1])
		mmt_signal_std = track_data[:, 9]
		mmt_signal_app = track_data[:, 10]
		distance_plot = track_data[:, 11]
		phase_plot = track_data[:, 12]

		#	calculate flux
		flux_plot = np.zeros(jd_plot.shape)
		for i, app_mag in enumerate(mmt_signal_app):
			flux_plot[i] = 1367 * 10**((app_mag - -26.74) / -2.5)
		
		#	seconds plot
		seconds_plot = np.multiply(np.subtract(jd_plot, jd_plot[0]), 86400)

		###########################################################################################
		#	select time data to use
		
		sim_range = lc_sim_range
		len_record = lc_len_record
		start_date = lc_start_date
		stop_date = lc_stop_date
		duration_sec = lc_duration_sec

		###########################################################################################
		#	generate eop data arrays
		###########################################################################################
		
		#	set time
		eop_start = time.time()
		
		print('\n		[{}] Generating EOP data array...'.format(datetime.datetime.now()))
		
		#	EOP data
		eop_dir = 'eop_files'
		eop_file = 'eop_parsed.csv'
		dat_file = 'dat_parsed.csv'
		
		#	assemble eop path
		eop_path = os.path.join(path_data, eop_dir)
		
		#	generate interpolated eop_data
		eop_log_level = 0
		eop_data, dat_data = eop_interpolate(
			eop_path,
			eop_file,
			dat_file,
			sim_range,
			time_step,
			eop_log_level
		)
		
		###########################################################################################
		#	display time
		eop_end = time.time()
		eop_time = np.round((eop_end - eop_start), 1)
		
		print('\n				EOP data generated.')
		print('\n					- Time taken = {} s.'.format(eop_time))
		
		###########################################################################################
		#	generate time objects
		###########################################################################################
		
		#	start dat value
		dat = dat_data[0, -1]
			
		#	start eop values
		eop_line = eop_data[0, :]
		eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, \
			_, lod, _, dX, _, dY, _ = eop_line
			
		#	generate start time object
		start_time_object = get_times(*start_date, ut_offset, dat)
			
		#	end dat value
		dat = dat_data[-1, -1]
			
		#	end eop values
		eop_line = eop_data[-1, :]
		eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, \
			_, lod, _, dX, _, dY, _ = eop_line
			
		#	generate stop time object
		stop_time_object = get_times(*stop_date, ut_offset, dat)
		
		###########################################################################################
		#	generate rotation matrices
		###########################################################################################
		
		#	set time
		rot_mat_start = time.time()
		print('\n		[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))
		
		#	tt time
		jdaytt_start = start_time_object.jday_tt
		jfractt_start = start_time_object.jfrac_tt
		jdtt_start = jdaytt_start + jfractt_start
		
		#	ut1 time
		jdayut1_start = start_time_object.jday_ut1
		jfracut1_start = start_time_object.jfrac_ut1
		jdut1_start = jdayut1_start + jfracut1_start
		
		#	generate rotation matrices
		rot_mat_log_level = 0
		pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
			jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
		)
		
		###########################################################################################
		#	display time
		rot_mat_end = time.time()
		rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)
		
		print('\n			Rotation Matrices Generated.')
		print('\n				- Time taken = {} s.'.format(rot_mat_time))
		
		###########################################################################################
		#	MODULE:	Horizons Positions
		###########################################################################################
		
		#	set time
		horz_start = time.time()
		print('\n		[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))
		
		#	set horizons logging level
		horz_logging_level = 0
		
		sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
			path_data, horz_logging_level,
			start_time_object, stop_time_object,
			len_record, time_step,
			eop_path
		)
		
		#	find indices of light curve data
		timecol = sun_icrf[:, 0]
		lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]
		
		###########################################################################################
		#	display time
		horz_end = time.time()
		horz_time = np.round((horz_end - horz_start), 1)
		
		print('\n			Horizons data generated.')
		print('\n				- Time taken = {} s.'.format(horz_time))
		
		###########################################################################################
		#	MODULE:	Object Position
		###########################################################################################
		
		#	set time
		sat_start = time.time()
		print('\n		[{}]	Generating object position data...'.format(datetime.datetime.now()))
		
		satpos_logging_level = 0
		
		sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
			path_data, satpos_logging_level, len_record, time_step,
			start_time_object, stop_time_object,
			norad_id,
			eop_data, dat_data,
			pm_array, pn_array, st_array, c2t_array
		)
		
		##########################################################################################
		#	display time
		sat_end = time.time()
		sat_time = np.round((sat_end - sat_start), 1)
		
		print('\n			Object position data generated.')
		print('\n				- Time taken = {} s.'.format(sat_time))
		
		##########################################################################################
		#	MODULE:	Transformation
		##########################################################################################
		
		#	set time
		trans_start = time.time()
		print('\n		[{}]	Performing transformations...'.format(datetime.datetime.now()))
		
		identification_str = 'mmt-render'
		
		trans_logging_level = 0
		save_flag = False
		
		sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
			= module_transformation(
				path_data, trans_logging_level, save_flag,
				start_time_object, stop_time_object,
				identification_str, site_str,
				len_record, time_step,
				eop_data, dat_data,
				pm_array, pn_array, st_array, c2t_array,
				sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
			)
		
		##########################################################################################
		#	display time
		trans_end = time.time()
		trans_time = np.round((trans_end - trans_start), 1)
		
		print('\n			Transformations complete.')
		print('\n				- Time taken = {} s.'.format(trans_time))
		
		##########################################################################################
		#	minimise sun angle
		##########################################################################################
		
		print('\n		[{}]	Minimising Sun Angle...'.format(datetime.datetime.now()))
		
		#	convert to radians
		rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg)
		rot_vec_inc_rad_temp = np.radians(rot_vec_inc_deg)
		
		#	generate initial t = 0 attitude
		init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])
		
		#	generate initial attitude rotation matrix
		init_attit_mat_temp = rot_mat(init_attit_temp)
		
		#	rotate geometry axes into orbital frame
		solar_panel_norm_orb = np.dot(solar_panel_norm_body, init_attit_mat_temp)
		
		#	sun vector
		sun_orb_vec_temp = sun_orb[0, 1:4]
		
		attitude_angle_range = np.arange(0, 360, 1)
		angle_data = np.zeros([len(attitude_angle_range)])
		
		for i, rot_vec_azm_deg_temp in enumerate(attitude_angle_range):
		
			#	set new azimuth angle
			rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg_temp)
			
			#	generate new attitude state
			init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])
		
			#	generate new attitude matrix
			init_attit_mat_temp = rot_mat(init_attit_temp)
		
			#	generate new solar panel norm in orbital frame
			solar_panel_norm_orb_temp = np.dot(solar_panel_norm_body, init_attit_mat_temp)
		
			#	calculate angle to sun vector
			#	sun angle
			minimisation_angle = np.arccos(
				np.dot(solar_panel_norm_orb_temp, sun_orb_vec_temp)
				/ (np.linalg.norm(solar_panel_norm_orb_temp) *  np.linalg.norm(sun_orb_vec_temp))
			)
		
			angle_data[i] = minimisation_angle
		
		#	find minimum angle
		min_ang = angle_data.min()
		min_index = np.where(angle_data==min_ang)[0][0]
		
		azimuth_angle = attitude_angle_range[min_index]
		
		#	set actual azimuth angle
		rot_vec_azm_deg = azimuth_angle
		
		print('\n			Setting Azimuth Angle to: ', rot_vec_azm_deg)
		
		##########################################################################################
		#	define initial attitude state
		##########################################################################################
		
		##################################################
		#	attitude
		
		#	convert to radians
		rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
		rot_vec_inc_rad = np.radians(rot_vec_inc_deg)
		
		#	generate initial t = 0 attitude
		init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
		
		##################################################
		#	attitude motion in body frame
		
		#	generate rotation angle in radians
		if rot_period != 0.0:
			rot_ang_per_sec = (np.pi * 2) / rot_period
		else:
			rot_ang_per_sec = 0.0
		
		#	define rotation axis in body frame as unit vector
		rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
		
		#	generate attitude matrix in body frame
		attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)
		
		#	extract euler angles
		angles1 = np.array(mat2angles(attit_mat_body)[0:3])
		angles2 = np.array(mat2angles(attit_mat_body)[3:6])
		
		#	convert negative zeros
		for i, ele in enumerate(angles1):
			if str(ele)[0] == '-' and ele == 0:
				angles1[i] = 0.0
		
		#	extract attitude motion in the body frame
		attit_mot_body = angles1
		
		##################################################
		#	generate initial attitude rotation matrix
		init_attit_mat = rot_mat(init_attit)
		
		##################################################
		#	calculate rotation vector in orbital frame as unit vector
		rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)
		
		##################################################
		#	define velocity of rotation vector in orbital frame
		
		#	when equal to orbital motion, rot vec is stationary in inertial frame
		rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
		rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])
		
		######################################################################################
		#	generate brdf
		
		#	Refractive Index
		n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))
		
		#	Specular ratio
		s_brdf = 1.0 - d_brdf
		
		#	generate brdf
		brdf_data = np.zeros([nfaces, 5])
		
		for i in range(nfaces):
			brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
		
		##########################################################################################
		#	perform light curve vector generation
		##########################################################################################
		
		#	blank observation and illumination vectors in orbital frame
		obs_vec_orb = np.array([1.0, 0.0, 0.0])
		illum_vec_orb = np.array([1.0, 0.0, 0.0])
		
		#	generate orb2inertial velocity vector
		orb2inertial = np.array([0.0, 0.0, -orbital_mot])
		
		#	vector generation function
		vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
			path_data, time_step, len_record,
			sat_orb, sun_orb, earth_orb, site_orb,
			orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
			obs_vec_orb, illum_vec_orb
		)
		
		##################################################
		#	parse vector data
		
		#	extract orbital positions of custom input vectors
		illum_vec_orb_data = vector_data_orb[:, 28:31]
		obs_vec_orb_data = vector_data_body[:, 31:34]
		
		#	body data
		sun_body_data = vector_data_body[:, 28:31]
		site_body_data = vector_data_body[:, 31:34]
		earth_body_data = vector_data_body[:, 19:22]
		sat_body_vel = vector_data_body[:, 22:25]
		sat_body_pos = np.zeros([len(sat_body_vel), 3])
		sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
		illum_vec_body_data = vector_data_body[:, 28:31]
		obs_vec_body_data = vector_data_body[:, 31:34]
		
		#	orb data
		sun_orb_data = vector_data_orb[:, 28:31]
		site_orb_data = vector_data_orb[:, 31:34]
		earth_orb_data = vector_data_orb[:, 19:22]
		sat_orb_vel = vector_data_orb[:, 22:25]
		sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
		sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
		illum_vec_orb_data = vector_data_orb[:, 28:31]
		obs_vec_orb_data = vector_data_orb[:, 31:34]
	
		######################################################################################
		#	generate light curve
		######################################################################################
		
		print('\n		[{}]	Generating Light Curve...'.format(datetime.datetime.now()))
	
		#	generate sim time
		sim_time = np.linspace(0, duration_sec, len_record)
		
		#	initilise lightcurve output
		final_light_curve = np.zeros((lc_len_record, 4))
		final_light_curve[:, 1] = sim_time
		
		jdate_start = start_jd[0] + start_jd[1]
	
		slc_log_level = 0
		
		#	run light curve generation function
		light_curve = glim_slc_shadowing(
			sun_body_data, site_body_data, sun_orb_data, site_orb_data,
			final_light_curve,
			jdate_start, time_step,
			body_mesh,
			init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
			brdf_data,
			slc_log_level
		)
	
		light_curve_seconds = light_curve[:, 1]
		lc_app = light_curve[:, 2]
		lc_std = light_curve[:, 3]
	
		#	calculate flux
		lc_flux_plot = np.zeros(lc_app.shape)
		for i, app_mag in enumerate(lc_app):
			lc_flux_plot[i] = 1367 * 10**((app_mag - -26.74) / -2.5)
		
		###########################################################################################
		#	final figure generation steps
		###########################################################################################

		###########################################################################################
		#	generate date vectors
		datevec_array = []
		
		for line in sim_range:
			#	convert jd to date
			year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(line))]
		
			#	turn into string
			datestr = str('-'.join(
				['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
			))
		
			timestr = ':'.join(
				['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
			)
		
			datevec  = ' '.join([datestr, timestr])
			#	convert to datetime object
			date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
			#	append  to output array
			datevec_array.append(date_time_obj)
	
		###########################################################################################
		#	plot data

		if color_by_object is True:
			c_str = 'C' + str(object_color_index)

		elif color_by_object is False:
			c_str = 'C' + str(track_color_index)

		if label_by_object is True:
			l_str = 'Starlink-{}; Track-{}'.format(int(object_id), int(track_index))

		elif label_by_object is False:
			l_str = r'$Track-{}$'.format(int(track_index))

		#	stacked apparant
		ax1.plot(seconds_plot, mmt_signal_app, color=c_str, label=l_str, lw=plot_lw)
		ax1.plot(seconds_plot, lc_app, color=c_str, lw=plot_lw, ls='dashed')

		#	flux plot
		ax2.plot(seconds_plot, flux_plot, color=c_str, lw=plot_lw)
		ax2.plot(seconds_plot, lc_flux_plot, color=c_str, lw=plot_lw, ls='dashed')
		
		#	against distance
		ax3.plot(distance_plot, mmt_signal_app, color=c_str, lw=plot_lw)
		ax3.plot(distance_plot, lc_app, color=c_str, lw=plot_lw, ls='dashed')
		
		#	against phase
		ax4.plot(phase_plot, mmt_signal_app, color=c_str, lw=plot_lw)
		ax4.plot(phase_plot, lc_app, color=c_str, lw=plot_lw, ls='dashed')
		
		total_duration += seconds_plot[-1] 
		
		light_curve_index += 1
		track_index += 1
		first_track = False
		tracks_plotted += 1
		track_color_index += 1

		print('\n				Track Complete.')
		###########################################################################################

	print('\n		Plotted {} curves...'.format(int(tracks_plotted)))

	#	itterate color index
	object_color_index += 1
	print('\n		Done.')
	
print('\nLoop Complete...')

print('\nTotal Duration = {} s'.format(np.round(total_duration, 1)))
print('\nDone.')

###################################################################################################
#	finalise plots
		
#	set range
ax1.set_ylim([10, 0])
ax1.set_xlabel(r'$Time / s$', fontsize=label_sizex, labelpad=label_pad)
ax1.set_ylabel(r'$App Mag$', fontsize=label_sizey)
ax1.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax1.tick_params(axis='y', labelsize=font_sizey, pad=0)


ax2.set_ylim([0.0, 0.5e-8])
ax2.set_xlabel(r'$Time / s$', fontsize=label_sizex, labelpad=label_pad)
ax2.set_ylabel(r'$Flux / W m^{2}$', fontsize=label_sizey)
ax2.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax2.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax3.set_ylim([10, 0])
ax3.set_xlabel(r'$Range / km$', fontsize=label_sizex, labelpad=label_pad)
ax3.set_ylabel(r'$App Mag$', fontsize=label_sizey)
ax3.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax3.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax4.set_ylim([10, 0])
ax4.set_xlim([0.0, 180])
ax4.set_xlabel(r'$Phase / \circ$', fontsize=label_sizex, labelpad=label_pad)
ax4.set_ylabel(r'$App Mag$', fontsize=label_sizey)
ax4.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax4.tick_params(axis='y', labelsize=font_sizey, pad=0)

fig.legend(
	loc='lower center',
	fontsize=legend_fontsize,
	ncol=4,
	frameon=False
)

if savefile_flag is True:
	plt.savefig(out_path, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
sys.exit()
