###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	kep2rv, rot_mat, vec_ang

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	hilbert approximation
from tools.functions.functions_hilbert import hilbert_calc

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

###################################################################################################
#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	calculate line properties

def line_from_two_points(x1, y1, x2, y2):

    """
    This function calculates the slope and intercept of a line from two points.
    
    Inputs: two sets of (x,y) pixel coordinates
    Outputs: slope m, intercept b
    
    """
    if x2 == x1: 
        x2 += 1
    if y2 == y1:
        y2 += 1
    # Avoids NAN or 0 slope errors
    
    m = float(y2-y1)/float(x2-x1)
    b = (y2 - m*x2)
    return m, b

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			Starlink File Plotter')
print('#####################################################################################')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set flag
savefile_flag = True
savefile_flag = False

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures'
)

#	lightcurve output path
lightcurve_output_path = os.path.join(
	path_data,
	'lightcurve_dir/starlink_analysis/starlink_lightcurve_analysis'
)

###################################################################################################
#	inputs
###################################################################################################

#	set time windows
window_start_date = [2020, 1, 1, 0, 0, 0]
window_stop_date = [2021, 1, 1, 0, 0, 0]

#	set launch date
launch_date0 = [2019, 5, 24, 2, 30, 0.0]
launch_date1 = [2019, 11, 11, 14, 56, 0.0]
launch_date2 = [2020, 1, 7, 2, 19, 0.0]

simulation_end_date = [2017, 12, 30, 0, 0, 0.0]
simulation_end_date = [2021, 7, 30, 0, 0, 0.0]

#	set simulation flags
pause_flag = True
pause_flag = False

specific_object_flag = True
specific_object_flag = False

#	define list of specific objects
specific_objects = [1067]

#	convert to numpy array
specific_objects = np.array(specific_objects)

#	set time step
time_step = 0.1

#	set earth radius
RE = 6371
earth_mu = 398600.4418

###################################################################################################
#	filtering

#	altitude filter
operational_alt = 555
operational_band = 2

altitude_flag = False
altitude_flag = True

#	BATCH 1 define list of objects to exclude
exclusion_list = [
	1014, 1037, 1065,	#	batch 1
	1090, 1110			#	batch 2
]

exclusion_list_flag = False
exclusion_list_flag = True

#	minimum brightness filter
min_brightness = 4.0
min_brightness_filter = True
min_brightness_filter = False

max_brightness = 6.0
max_brightness_filter = True
max_brightness_filter = False

#	minimum number of tracks filter
minimum_tracks_filter = True
minimum_tracks_filter = False

#	define a small number
small = 1e-8

###################################################################################################
#	light curve generation inputs
###################################################################################################

###################################################################################################
#	set brdf parameters
d_brdf = 0.4
m_brdf = 0.1
omega_brdf = 0.45

###################################################################################################
#	load geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str_pl = 'starlink_payload'
geom_str_sp = 'starlink_full'

geom_file_pl = geom_str_pl + '.stl'
geom_file_sp = geom_str_sp + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
rot_vec_inc_deg = 40.0
rot_vec_azm_deg_input = 290.0

rot_vec_inc_deg = 0.0
rot_vec_azm_deg_input = 0.0

print('\nInput azm angle = ', rot_vec_azm_deg_input)

#	correct azm
if rot_vec_azm_deg_input >= 180:
	rot_vec_azm_deg = rot_vec_azm_deg_input - 180

elif rot_vec_azm_deg_input < 180:
	rot_vec_azm_deg = rot_vec_azm_deg_input + 180

print('\nAttitude azm angle = ', rot_vec_azm_deg)

#	attitude motion
rot_period = 0.0

#	load static mesh
body_mesh_pl = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file_pl))
body_mesh_sp = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file_sp))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh_pl.rotate_using_matrix(offset_rot_mat)
body_mesh_sp.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors_pl = body_mesh_pl.vectors
body_vectors_sp = body_mesh_sp.vectors

body_normals_pl = body_mesh_pl.normals
body_normals_sp = body_mesh_sp.normals

body_areas_pl = body_mesh_pl.areas
body_areas_sp = body_mesh_sp.areas

nfaces_pl = len(body_areas_pl)
nfaces_sp = len(body_areas_sp)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	initialise figure parameters
###################################################################################################

results_str = 'synth'
results_str = 'real'

#	inch to cm conversion
inch2cm = 2.54

#	dpi
dpi_val = 300

#	font sizes
font_sizex = 6
font_sizey = 6
label_sizex = 5
label_sizey = 5
tick_sizex = 5
tick_sizey = 5
legend_fontsize = 5

marker_str = '+'

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set use of LaTeX font
plt.rc('text', usetex=True)

#	set figure height/width
fig_width = 12.0/inch2cm
fig_height = 6.0/inch2cm

###################################################################################################
#	lightcurve output plot

lightcurve_plot_flag = False

if lightcurve_plot_flag is True:
	
	#	initialise figure axes
	fig, ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300)
	plt.gcf().subplots_adjust(bottom=0.15)
	
	#	invert magnitude scale
	ax.invert_yaxis()
	
	ax.set_ylim([10, 0])
	ax.set_xlabel(r'$Time/s$', fontsize=label_sizex)
	ax.set_ylabel(r'$App\ Mag$', fontsize=label_sizey)
	ax.tick_params(axis='x', labelsize=tick_sizex, pad=xticks_pad)
	ax.tick_params(axis='y', labelsize=tick_sizey, pad=yticks_pad)

###################################################################################################
#	distance and phase validation plots

phase_dist_validation_plot_flag = False

if phase_dist_validation_plot_flag is True:
	
	#	phase
	fig_phase, ax_phase = plt.subplots()
	ax_phase.set_title(r'Phase Validation')

	#	distance
	fig_dist, ax_dist = plt.subplots()
	ax_dist.set_title(r'Distance Validation')

###################################################################################################
#	solar panel involvment

sp_involvment_plot_flag = True
sp_involvment_plot_flag = False

if sp_involvment_plot_flag is True:

	#	average magnitude against solar panel involvment
	sp_data_fig_inv, sp_data_ax_inv = plt.subplots(
		1, 3,
		sharey=True,
		figsize=(24.0/inch2cm, 8.0/inch2cm),
		dpi=dpi_val
	)

	sp_data_ax1 = sp_data_ax_inv[1]
	sp_data_ax2 = sp_data_ax_inv[2]
	sp_data_ax3 = sp_data_ax_inv[0]

	sp_data_ax3.set_ylabel(r'$Solar\ Panel\ Involvement$')
	sp_data_ax1.invert_xaxis()
	sp_data_ax2.invert_xaxis()
	sp_data_ax3.invert_xaxis()

	if results_str == 'synth':
		sp_data_ax1.set_xlabel(r'$Average\ Synthetic\ Magnitude$')
		sp_data_ax2.set_xlabel(r'$Min\ Synthetic\ Magnitude$')
		sp_data_ax3.set_xlabel(r'$Max\ Synthetic\ Magnitude$')

	elif results_str == 'real':
		sp_data_ax1.set_xlabel(r'$Average\ MMT\ Magnitude$')
		sp_data_ax2.set_xlabel(r'$Min\ MMT\ Magnitude$')
		sp_data_ax3.set_xlabel(r'$Max\ MMT\ Magnitude$')


###################################################################################################
#	solar panel shadowing plots

sp_shadowing_plot_flag = False
sp_shadowing_plot_flag = True

if sp_shadowing_plot_flag is True:

	#	average magnitude against solar panel shadowing
	sp_data_fig_shad, sp_data_ax = plt.subplots(
		1, 3,
		sharey=True,
		figsize=(24.0/inch2cm, 8.0/inch2cm),
		dpi=dpi_val
	)

	sp_data_ax4 = sp_data_ax[1]
	sp_data_ax5 = sp_data_ax[2]
	sp_data_ax6 = sp_data_ax[0]

	sp_data_ax6.set_ylabel(r'$Solar\ Panel\ Shadowing$')
	sp_data_ax4.invert_xaxis()
	sp_data_ax5.invert_xaxis()
	sp_data_ax6.invert_xaxis()

	if results_str == 'synth':
		sp_data_ax4.set_xlabel(r'$Average\ Synthetic\ Magnitude$')
		sp_data_ax5.set_xlabel(r'$Min\ Synthetic\ Magnitude$')
		sp_data_ax6.set_xlabel(r'$Max\ Synthetic\ Magnitude$')

	elif results_str == 'real':
		sp_data_ax4.set_xlabel(r'$Average\ MMT\ Magnitude$')
		sp_data_ax5.set_xlabel(r'$Min\ MMT\ Magnitude$')
		sp_data_ax6.set_xlabel(r'$Max\ MMT\ Magnitude$')

###################################################################################################
#	phase and distance plots

phase_dist_plot_flag = True
phase_dist_plot_flag = False

if phase_dist_plot_flag is True:

	#	average magnitude against solar panel involvment
	sp_data_fig_dist, sp_data_ax_dist = plt.subplots(
		figsize=(12.0/inch2cm, 8.0/inch2cm),
		dpi=dpi_val
	)

	sp_data_ax_dist.set_xlabel(r'$Average\ Distance$')
#	sp_data_ax_dist.set_ylabel(r'$Average\ MMT\ Magnitude$')
	sp_data_ax_dist.set_ylabel(r'$Average\ Synthetic\ Magnitude$')
	sp_data_ax_dist.invert_yaxis()
	
	#	average magnitude against solar panel involvment
	sp_data_fig_phase, sp_data_ax_phase = plt.subplots(
		figsize=(12.0/inch2cm, 8.0/inch2cm),
		dpi=dpi_val
	)

	sp_data_ax_phase.set_xlabel(r'$Average\ Phase$')
	sp_data_ax_phase.set_ylabel(r'$Average\ MMT\ Magnitude$')
	sp_data_ax_phase.set_ylabel(r'$Average\ Synthetic\ Magnitude$')
	sp_data_ax_phase.invert_yaxis()

###################################################################################################
#	starlink batch file loop
###################################################################################################

launch_string_list = [
	'batch1', 'batch2', 'batch3', 'batch4', 'batch5', 'batch6', 'batch7', 'batch8', 'batch9'
]

# launch_string_list = ['batch1', 'batch2', 'batch3']

total_obj_plotted = 0
total_tracks_plotted = 0

total_plotted_object_indices = []
total_plotted_light_curves = 0

first_batch = True

#	initialise solar_panel_data
solar_panel_data = []

all_plotted_data1 = []
all_plotted_data2 = []
all_plotted_data3 = []

for batch_index, launch_string in enumerate(launch_string_list):

	print('\n#####################################################################################')
	print('Working on Launch: ', launch_string)
	print('#####################################################################################')
		
	batch_color_str = 'C' + str(batch_index)

	output_file = 'brightness_data_{}.txt'.format(launch_string)
	output_file = 'brightness_data_full_{}.txt'.format(launch_string)

	#	initialise solar_panel_data of batch
	batch_solar_panel_data = []

	#	if exists, load
	if os.path.isfile(os.path.join(lightcurve_output_path, output_file)):
		print('\n	Reading existing lightcurve file of batch...')
	
		#	load file
		batch_solar_panel_data = np.loadtxt(
			os.path.join(lightcurve_output_path, output_file), delimiter=', '
		)

		total_tracks_plotted += len(batch_solar_panel_data)

	else:
	
		print('\n	Generating data...')

		#	set read path
		read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'starlink', launch_string)
		
		###########################################################################################
		#	read and sort starlink files
		###########################################################################################
		
		print('\n[{}] Generating Starlink file name list...'.format(datetime.datetime.now()))
		
		starlink_dir = os.listdir(read_path)
		starlink_files = []
		
		#	order by starlink id
		for file in starlink_dir:
			starlink_id = int(file.split('-')[1].split('_')[0])
			starlink_files.append([file, starlink_id])
		
		starlink_files = sorted(starlink_files, key=lambda x: x[1])
		
		###########################################################################################
		#	starlink file loop
		
		print('\n[{}] Extracting MMT file data...\n'.format(datetime.datetime.now()))
		first_flag = True
		
		norad_id_list = np.zeros([len(starlink_files), 2])
		
		for file_num, mmt_file in enumerate(starlink_files):
		
			#print('	Parsing MMT data file: {}'.format(mmt_file), end='\r')
	
			#######################################################################################
			#	load MMT file
		
			mmt_file = mmt_file[0]
		
			#	parse filename data
			name_str, norad_id, _, mmt_id = mmt_file[:-4].split('_')
		
			starlink_id = int(name_str.split('-')[1])
		
			name_str = ''.join(name_str.split('-'))
			norad_id = int(norad_id)
			
			#	open and read file
			with open(os.path.join(read_path, mmt_file)) as read_file:
				mmt_data = read_file.readlines()
			
			#	add to norad id list
			norad_id_list[file_num] = [starlink_id, norad_id]
		
			#######################################################################################
			#	parse mmt file
			
			#	header line
			header_text = mmt_data[7]
			
			#	strip header
			mmt_data = mmt_data[8:]
			
			n_rows = 14
			
			out_data = np.zeros([len(mmt_data), n_rows])
			
			for i, line in enumerate(mmt_data):
			
				#	split line
				date, mmt_time, mag_std, mag_app, opt_filter, penumbral, distance, \
					phase, channel, track_id = line.split(' ')
			
				#	parse data and set data types
				mag_std = float(mag_std)
				mag_app = float(mag_app)
				distance = float(distance)
				phase = float(phase)
				track_id = int(track_id)
			
				#	parse date
				year, month, day = date.split('-')
				year = int(year)
				month = int(month)
				day = int(day)
			
				#	parse time
				hour, minute, second = mmt_time.split(':')
				hour = int(hour)
				minute = int(minute)
				second = float(second)
			
				#	calculate julian date
				jday, jfrac = date2jd(year, month, day, hour, minute, second)
			
				#	generate output line
				output_line = [
					track_id, starlink_id,
					jday, jfrac,
					year, month, day,
					hour, minute, second,
					mag_std, mag_app,
					distance, phase
				]
			
				#	add to output array
				out_data[i, :] = output_line
		
			#	initialise output array
			if first_flag is True:
				total_data = out_data
		
				first_flag = False
		
			#	stack with existing output array
			elif first_flag is False:
				total_data = np.vstack([total_data, out_data])
		
		sys.stdout.write("\033[K")
		print('	Done.\n', end='\r')
		
		print('\nFinished MMT file parsing...')
		print('	Total data length = ', len(total_data))
		print('	Total number of objects = ', len(np.unique(total_data[:, 1])))
		print('	Total number of light curves = ', len(np.unique(total_data[:, 0])))
		
		###########################################################################################
		#	collect data in window of interest
		###########################################################################################
		
		print('\n[{}] Extracting light curve inside time window...'.format(datetime.datetime.now()))
		
		#	generate window start/stop JDs
		window_start_jd = np.add(*date2jd(*window_start_date))
		window_stop_jd = np.add(*date2jd(*window_stop_date))
		
		first_lc_flag = True
		
		#	extract starlink ids
		starlink_id_list = np.unique(total_data[:, 1])
		
		light_curve_index = 1
		
		plotted_light_curves = 0
		
		#	loop through starlink IDs
		for starlink_id in starlink_id_list:
		
			print('	Starlink-{}: '.format(int(starlink_id)), end='')
		
			#	find track ids
			starlink_indices = np.where(total_data[:, 1]==starlink_id)[0]
		
			#	extract data for this starlink object
			full_starlink_data = total_data[starlink_indices, :]
		
			#	extract track ids
			track_id_list = np.unique(full_starlink_data[:, 0])
		
			ntracks = 0
		
			#	loop through tracks
			for track_id in track_id_list:
			
				#	find track ids
				track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
		
				#	extract full data
				track_data = full_starlink_data[track_indices, :]
			
				#	extract light curve data
				distance_plot = track_data[:, 12]
				phase_plot = track_data[:, 13]
			
				start_line = track_data[0]
				stop_line = track_data[-1]
			
				start_jdate = np.add(start_line[2], start_line[3])
				stop_jdate = np.add(stop_line[2], stop_line[3])
			
				#	ensure between window start/stop
				if start_jdate > window_start_jd and stop_jdate < window_stop_jd:
		
					#	light curve is within window
					ntracks += 1
			
					#	if first light curve, start array
					if first_lc_flag is True:
						final_data = track_data
						first_lc_flag = False
		
					#	else append to list
					elif first_lc_flag is False:
						final_data = np.vstack([final_data, track_data])
		
			print('{} track(s) extracted.'.format(ntracks))
		
		print('\nFinished window extraction...')
		print('	Total data length = ', len(final_data))
		print('	Total number of objects = ', len(np.unique(final_data[:, 1])))
		print('	Total number of light curves = ', len(np.unique(final_data[:, 0])))
		
		###########################################################################################
		#	plot light curve properties
		###########################################################################################
		
		print('\n#################################################################################')
		print('[{}] Plotting Starlink Light Curve Properties...'.format(datetime.datetime.now()))
		
		###########################################################################################
		#	full final data
		
		#	extract starlink ids
		starlink_id_list = np.unique(final_data[:, 1])
		
		#	if specific object flag is true, use object list
		if specific_object_flag is True:
			starlink_id_list = specific_objects
		
		###########################################################################################
		#	starlink object loop
		
		print('\n	Looping Through Starlink Light Curve Files...')
		
		light_curve_index = 1
		color_index = 0
		total_duration = 0
		
		plotted_object_indices = []
		obj_plotted = 0
		tracks_plotted = 0
	
		#	loop through starlink IDs
		for loop_itt, starlink_id in enumerate(starlink_id_list):
		
			print('\n		#####################################################################')
			print('		Plotting data for: Starlink-{}'.format(int(starlink_id)))
			print('			(Object {} of {})'.format(loop_itt + 1, len(starlink_id_list)))
	
			if exclusion_list_flag is True:
				if starlink_id in exclusion_list:
					print('\n			In exclusion list, skipping...')
					continue
	
			#	set norad id
			norad_index = np.where(norad_id_list[:, 0]==starlink_id)[0][0]
			norad_id = int(norad_id_list[norad_index, 1])
		
			print('\n			NORAD ID: {}'.format(int(norad_id)))
		
			#######################################################################################
			#	extract required data
		
			#	find track ids
			starlink_indices = np.where(final_data[:, 1]==starlink_id)[0]
		
			#	extract data for this starlink object
			full_starlink_data = final_data[starlink_indices, :]
		
			#	extract track ids
			track_id_list = np.unique(full_starlink_data[:, 0])
		
			#	plot parameters
			height_color_str = 'C' + str(color_index)
			station_time_color_str = 'C' + str(color_index)
			light_curve_color_str = 'C' + str(color_index)
		
			label_str = r'$Starlink-{}$'.format(int(int(starlink_id)))
	
			print('\n			Total Available Tracks: {}'.format(int(len(track_id_list))))
	
			#######################################################################################
			#	generate full list of TLEs
			
			print('\n					Acquiring TLEs...')
			
			#	TLE request dates, full range since launch
			tle_start_date = launch_date0
			tle_stop_date = simulation_end_date
			
			#	generate TLE file name
			tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
			tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
			tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)
		
			#	assemble TLE path
			tle_path = os.path.join(path_data, 'input_dir', 'tle_files')
		
			#	if exists, load
			if os.path.isfile(os.path.join(tle_path, tle_file)):
				print('\n						Reading existing TLE file.')
			
				#	load file
				with open(os.path.join(tle_path, tle_file), 'r') as file:
					tle_data = file.readlines()
			
			else:
				print('\n						Requesting TLE file...')
		
				#	request TLEs
				tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)
			
				#	save file
				with open(os.path.join(tle_path, tle_file), 'w') as file:
					for line in tle_data:
						file.write(line)
			
				print('							TLE file saved.')
			
			#######################################################################################
			#	calculating orbital height
			
			print('\n					Calculating Height Data and On-Station Time...')
		
			#	initialise height data
			height_data = []
			difference_data = []
			tle_datevec_array = []
			difference_data_datevec_array = []
		
			on_station_flag = False
			time_found_flag = False
			run_diff = 0
			run_diff_data = []
		
			spread_max = 1.5
			diff_threshold = 1.5
			
			alt_data = []
			perc_error = 1
	
			for tle_line_index, tle_line in enumerate(range(int(len(tle_data) / 3))):
	
				#	generate tle line index
				line_index = (tle_line * 3)
		
				tle_line0 = tle_data[line_index]
				tle_line1 = tle_data[line_index + 1]
				tle_line2 = tle_data[line_index + 2]
			
				#	extract mean motion and convert to SMA
				MM = float(tle_line2[52:63])
				#	calculate semi major axis
				mean_mot = MM / 86400 * (2*np.pi)
				sma = (earth_mu / mean_mot**2)**(1/3)
				sat_height = sma - RE
		
				#	append to list of satellite altitudes
				alt_data.append(sat_height)
	
				#########################################################
				#	check altitude is within margin of operational altitude
				if altitude_flag == True:
					if (operational_alt - operational_band) > sat_height:
						continue
		
					elif (operational_alt + operational_band) < sat_height:
						continue
		
					#####################################################
					#	check altitude is consitent
			
					if tle_line_index > 1:
			
						#	convert to numpy array
						temp_array = np.array(alt_data)
			
						#	extract only previous 3 points
						temp_array = temp_array[tle_line_index - 2: tle_line_index + 1]
			
						if abs(temp_array[2] - temp_array[1]) > 0.5:
							continue
						elif abs(temp_array[2] - temp_array[0]) > 0.5:
							continue
						elif abs(temp_array[1] - temp_array[0]) > 0.5:
							continue
	
				#	if this is the first TLE to pass checks, set on-station 
				if on_station_flag is False:
					on_station_index = line_index
					on_station_flag = True
	
				#	generate SGP4 satellite object
				satellite = twoline2rv(tle_line1, tle_line2, wgs72)
		
				#	extract TLE epoch
				tle_epoch = satellite.jdsatepoch
		
				#	convert to state vector
				pos, vel = satellite.propagate(*jd2date(*jday_jfrac(tle_epoch)))
		
				#	calculate height
				sat_height_sgp4 = np.linalg.norm(pos) - RE
		
				#	break condition for TLEs beyond window stop date
				if tle_epoch > np.add(*date2jd(*window_stop_date)):
					break
		
				#	ensure between window start/stop
				if tle_epoch > np.add(*date2jd(*window_start_date)):
	
					#	convert jd to date
					year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(tle_epoch))]
			
					#	turn into strings
					datestr = str('-'.join(
						['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)]
					))
			
					timestr = ':'.join(
						['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)]
					)
			
					datevec  = ' '.join([datestr, timestr])
			
					#	convert to datetime object
					date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
					tle_datevec_array.append(date_time_obj)
			
	
	
	#				#	USING TLE SMA: add to outputs
	#				height_data.append(sat_height)
	
					#	USING SGP4: add to outputs
					height_data.append(sat_height_sgp4)
	
			#######################################################################################
			#	acquire on-station date
	
			on_station_tle_line0 = tle_data[on_station_index]
			on_station_tle_line1 = tle_data[on_station_index + 1]
			on_station_tle_line2 = tle_data[on_station_index + 2]
		
			#	generate SGP4 satellite object
			on_station_satellite = twoline2rv(on_station_tle_line1, on_station_tle_line2, wgs72)
		
			#	extract TLE epoch
			on_station_tle_epoch = on_station_satellite.jdsatepoch
	
			#######################################################################################
			#	generate filtered track list
			#######################################################################################
	
			if altitude_flag is True:
				filtered_track_id_list = []
				print('\n				Filtering to only those lightcurves since on-station date...')
		
				for track_itt, track_id in enumerate(track_id_list):
					
					#	find track ids
					track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
				
					#	extract full data
					track_data = full_starlink_data[track_indices, :]
				
					###############################################################################
					#	extract track time data
					
					mmt_line = track_data[0]
					
					#	parse line elements
					track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase = track_data[0]
					
					start_jd = [jday, jfrac]
					
					###############################################################################
					#	filter light curves when not on station
			
					if np.add(*start_jd) < on_station_tle_epoch:
						print('\n					Track: {}; Not yet on station...'.format(
							int(track_id)
						))

					else:
						print('\n					Track: {}; On station...'.format(
							int(track_id)
						))

						filtered_track_id_list.append(int(track_id))
		
		
				#	reset track id list to filtered list
				track_id_list = filtered_track_id_list
	
				print('\n			Total Available Tracks On-Station: {}'.format(
					int(len(track_id_list))
				))
	
			if track_id_list == []:
				print('\n		No available tracks, continuing to next object...')
				continue
			else:
				ntracks = len(track_id_list)
	
			#######################################################################################
			#	find maximum brightness
		
			if min_brightness_filter is True:
	
				#	generate maximum brightness data
				min_mag_data = []
			
				for track_itt, track_id in enumerate(track_id_list):
		
					#	find track ids
					track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
				
					#	extract full data
					track_data = full_starlink_data[track_indices, :]
				
					#	extract apparent signal
					mmt_signal_app = track_data[:, 11]
		
					#	append minimum magnitude to array
					min_mag_data.append(mmt_signal_app.min())
	
				app_mag_data_max = np.array(min_mag_data).min()
	
				if app_mag_data_max > min_brightness:
					print('\n				Brightness is below threshold, continuing...')
					brightness_flag = False
					continue
				
				else:
					print('\n				Brightness is above threshold...')
					brightness_flag = True
	
			if max_brightness_filter is True:
	
				#	generate maximum brightness data
				max_mag_data = []
			
				for track_itt, track_id in enumerate(track_id_list):
		
					#	find track ids
					track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
				
					#	extract full data
					track_data = full_starlink_data[track_indices, :]
				
					#	extract apparent signal
					mmt_signal_app = track_data[:, 11]
		
					#	append minimum magnitude to array
					max_mag_data.append(mmt_signal_app.max())
	
				app_mag_data_min = np.array(max_mag_data).max()
	
				if app_mag_data_min < max_brightness:
					print('\n				Brightness is above threshold, continuing...')
					brightness_flag = False
					continue
				
				else:
					print('\n				Brightness is below threshold...')
					brightness_flag = True
	
	
			#######################################################################################
			#	find number of tracks to plot
		
			if minimum_tracks_filter is True:
		
				print('\n					Finding Number of Tracks...')
		
				#	continue flag for less than 2 tracks
				if ntracks < 2:
					print('\n						Less than 2 tracks, continuing...')
					continue
		
			#######################################################################################
			#	main track loop
		
			first_track = True
			track_index = 1
			colour_index = 0
			plotted_track_indices = []
	
			for track_itt, track_id in enumerate(filtered_track_id_list):
	
				print('\n			#############################################################')
				print('			Working on Track ID: {}'.format(int(track_itt + 1)))
	
				#	remove legend if not first track
				if first_track is False:
					label_str = '_no_legend_'
				
				#	find track ids
				track_indices = np.where(full_starlink_data[:, 0]==track_id)[0]
				
				#	extract full data
				track_data = full_starlink_data[track_indices, :]
				
				###################################################################################
				#	extract track time data
				
				mmt_line = track_data[0]
				
				#	parse line elements
				track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
					mag_std, mag_app, distance, phase = track_data[0]
				
				start_jd = [jday, jfrac]
				
				#	parse line elements
				track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
					mag_std, mag_app, distance, phase = track_data[-1]
				
				stop_jd = [jday, jfrac]
			
				###################################################################################
				#	filter light curves when not on station
			
				if altitude_flag == True:
					if np.add(*start_jd) < on_station_tle_epoch:
						print('\n				Not yet on station...')
						continue
			
				###################################################################################
				#	generate simulation time data
				
				#	extract start and stop jds
				lc_start_jd = start_jd[0] + start_jd[1]
				lc_stop_jd = stop_jd[0] + stop_jd[1]
				
				#	convert to dates
				lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
				lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 
				
				#	light curve duration
				lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)
				
				#	set time step
				lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1
				
				#	time range in julian days
				lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
			
				sim_range_sec = np.round(
					np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400),
					1
				)
				
				print('\n				Light curve parameters: ')
				print(
					'					Start Date: {}/{:02d}/{:02d}	- {:02d}:{:02d}:{:04.1f}'\
					.format(*lc_start_date)
				)
				
				print(
					'					Stop Date: {}/{:02d}/{:02d}	- {:02d}:{:02d}:{:04.1f}'\
					.format(*lc_stop_date)
				)
				
				print('					Simulation time step:	{}'.format(time_step))
				print('					Duration:		{}	[steps]'.format(lc_len_record))
				print('								{}	[s]'.format(np.round(lc_duration_sec, 1)))
				
				###################################################################################
				#	ensure time continuity
				
				print('\n				Ensuring time continuity in extracted data...')
				
				track_data_temp = []
				sim_index = 0
				
				for itt, line in enumerate(track_data):
				
					#	parse line data
					track_id, object_id, \
						jday, jfrac, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase = line
					
					step_jd = jday + jfrac
				
					#	calculate current seconds
					current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)
				
					output_line = [
						jday, jfrac, current_sec, year, month, day, hour, minute, second, \
						mag_std, mag_app, distance, phase
					]
				
					##################################################
					#	extracting data of time step
				
					#	data match found
					if current_sec == sim_range_sec[sim_index]:
						track_data_temp.append(output_line)
				
						#	itterate number of output data points
						sim_index += 1
				
						#	save current line as previous line
						prev_line = output_line
				
					#	data point is not required
					elif current_sec < sim_range_sec[sim_index]:
						pass
				
					#	interpolation required
					elif current_sec > sim_range_sec[sim_index]:
				
						#	parse previous line
						jday_prev, jfrac_prev, prev_sec, \
							year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
							mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line
				
						#	calculate interpolation requirements
						gap = np.round(current_sec - prev_sec, 1)
						npts = int(gap / time_step)
				
						#	get current light curve data
						mag_std_last = mag_std
						mag_app_last = mag_app
						distance_last = distance
						phase_last = phase
				
						#	calculate linear interpolation step values
						mag_std_step = (mag_std_last - mag_std_prev) / npts
						mag_app_step = (mag_app_last - mag_app_prev) / npts
						distance_step = (distance_last - distance_prev) / npts
						phase_step = (phase_last - phase_prev) / npts
				
						#	calculate interpolation data
						for n in range(1, npts + 1):
				
							#	current simulation second
							interp_sec = np.round(prev_sec + (n * time_step), 1)
			
							#	time data
							jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
							jday, jfrac = jday_jfrac(jdate)
							year, month, day, hour, minute, second = jd2date(jday, jfrac)
				
							#	calculate interpolated light curve data
							mag_std = mag_std_prev + (n * mag_std_step)
							mag_app = mag_app_prev + (n * mag_app_step)
							distance = distance_prev + (n * distance_step)
							phase = phase_prev + (n * phase_step)
				
							#	generate new output line
							interp_output_line = [
								jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
								mag_std, mag_app, distance, phase
							]
				
							if interp_sec == sim_range_sec[sim_index]:
								track_data_temp.append(interp_output_line)
								sim_index += 1
				
						if interp_sec != current_sec:
							#	append final line
							track_data_temp.append(output_line)
							sim_index += 1
				
					else:
						print('ERROR')
						sys.exit()
				
					#	save previous line
					if itt != 0:
						prev_line = output_line
				
					#	array is complete condition
					if len(track_data_temp) == lc_len_record:
						break
				
				#	ensure parsed MMT data matches simulation data
				track_data = np.array(track_data_temp)
				
				if len(track_data) == len(lc_sim_range):
					print('					Simulation and MMT data matches.')
				else:
					print('					Error: Simulation and MMT data does not match...Exiting.')
					sys.exit()
				
				###################################################################################
				#	extract mmt phase, distance and brightness data
				mmt_dist_data = track_data[:, 11]
				mmt_phase_data = track_data[:, 12]
				
				mmt_signal_std = track_data[:, 9]
				mmt_signal_app = track_data[:, 10]
	
				###################################################################################
				#	select time data to use
				
				sim_range = lc_sim_range
				len_record = lc_len_record
				start_date = lc_start_date
				stop_date = lc_stop_date
				duration_sec = lc_duration_sec
		
				###################################################################################
				#	generate eop data arrays
				###################################################################################
				
				#	set time
				eop_start = time.time()
				
				print('\n	[{}] Generating EOP data array...'.format(datetime.datetime.now()))
				
				#	EOP data
				eop_dir = 'eop_files'
				eop_file = 'eop_parsed.csv'
				dat_file = 'dat_parsed.csv'
				
				#	assemble eop path
				eop_path = os.path.join(path_data, eop_dir)
				
				#	generate interpolated eop_data
				eop_log_level = 0
				eop_data, dat_data = eop_interpolate(
					eop_path,
					eop_file,
					dat_file,
					sim_range,
					time_step,
					eop_log_level
				)
				
				###################################################################################
				#	display time
				eop_end = time.time()
				eop_time = np.round((eop_end - eop_start), 1)
				
				print('\n		EOP data generated.')
				print('\n			- Time taken = {} s.'.format(eop_time))
				
				###################################################################################
				#	generate time objects
				###################################################################################
				
				#	start dat value
				dat = dat_data[0, -1]
					
				#	start eop values
				eop_line = eop_data[0, :]
				eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX,\
					_, dY, _ = eop_line
					
				#	generate start time object
				start_time_object = get_times(*start_date, ut_offset, dat)
					
				#	end dat value
				dat = dat_data[-1, -1]
					
				#	end eop values
				eop_line = eop_data[-1, :]
				eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX,\
					_, dY, _ = eop_line
					
				#	generate stop time object
				stop_time_object = get_times(*stop_date, ut_offset, dat)
				
				###################################################################################
				#	generate rotation matrices
				###################################################################################
				
				#	set time
				rot_mat_start = time.time()
				print('\n	[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))
				
				#	tt time
				jdaytt_start = start_time_object.jday_tt
				jfractt_start = start_time_object.jfrac_tt
				jdtt_start = jdaytt_start + jfractt_start
				
				#	ut1 time
				jdayut1_start = start_time_object.jday_ut1
				jfracut1_start = start_time_object.jfrac_ut1
				jdut1_start = jdayut1_start + jfracut1_start
				
				#	generate rotation matrices
				rot_mat_log_level = 0
				pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
					jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
				)
				
				###################################################################################
				#	display time
				rot_mat_end = time.time()
				rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)
				
				print('\n		Rotation Matrices Generated.')
				print('\n			- Time taken = {} s.'.format(rot_mat_time))
				
				###################################################################################
				#	MODULE:	Horizons Positions
				###################################################################################
				
				#	set time
				horz_start = time.time()
				print('\n	[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))
				
				#	set horizons logging level
				horz_logging_level = 0
				
				sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
					path_data, horz_logging_level,
					start_time_object, stop_time_object,
					len_record, time_step,
					eop_path
				)
				
				#	find indices of light curve data
				timecol = sun_icrf[:, 0]
				lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]
				
				###################################################################################
				#	display time
				horz_end = time.time()
				horz_time = np.round((horz_end - horz_start), 1)
				
				print('\n		Horizons data generated.')
				print('\n			- Time taken = {} s.'.format(horz_time))
				
				###################################################################################
				#	MODULE:	Object Position
				###################################################################################
				
				#	set time
				sat_start = time.time()
				print('\n	[{}]	Generating object position data...'.format(datetime.datetime.now()))
				
				satpos_logging_level = 0
				
				sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
					path_data, satpos_logging_level, len_record, time_step,
					start_time_object, stop_time_object,
					norad_id,
					eop_data, dat_data,
					pm_array, pn_array, st_array, c2t_array
				)
				
				###################################################################################
				#	display time
				sat_end = time.time()
				sat_time = np.round((sat_end - sat_start), 1)
				
				print('\n		Object position data generated.')
				print('\n			- Time taken = {} s.'.format(sat_time))
				
				###################################################################################
				#	MODULE:	Transformation
				###################################################################################
				
				#	set time
				trans_start = time.time()
				print('\n	[{}]	Performing transformations...'.format(datetime.datetime.now()))
				
				identification_str = 'mmt-render'
				
				trans_logging_level = 0
				save_flag = False
				
				sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
					= module_transformation(
						path_data, trans_logging_level, save_flag,
						start_time_object, stop_time_object,
						identification_str, site_str,
						len_record, time_step,
						eop_data, dat_data,
						pm_array, pn_array, st_array, c2t_array,
						sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
					)
				
				###################################################################################
				#	display time
				trans_end = time.time()
				trans_time = np.round((trans_end - trans_start), 1)
				
				print('\n		Transformations complete.')
				print('\n			- Time taken = {} s.'.format(trans_time))
				
				###################################################################################
				#	minimise sun angle
				###################################################################################
				
				print('\n	Minimising Solar Panel Sun Angle...')
				
				#	convert to radians
				rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg)
				rot_vec_inc_rad_temp = np.radians(rot_vec_inc_deg)
				
				#	generate initial t = 0 attitude
				init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])
				
				#	generate initial attitude rotation matrix
				init_attit_mat_temp = rot_mat(init_attit_temp)
				
				#	rotate geometry axes into orbital frame
				solar_panel_norm_orb = np.dot(solar_panel_norm_body, init_attit_mat_temp)
				
				#	sun vector
				sun_orb_vec_temp = sun_orb[0, 1:4]
				
				attitude_angle_range = np.arange(0, 360, 1)
				angle_data = np.zeros([len(attitude_angle_range)])
				
				for i, rot_vec_azm_deg_temp in enumerate(attitude_angle_range):
				
					#	set new azimuth angle
					rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg_temp)
					
					#	generate new attitude state
					init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])
				
					#	generate new attitude matrix
					init_attit_mat_temp = rot_mat(init_attit_temp)
				
					#	generate new solar panel norm in orbital frame
					solar_panel_norm_orb_temp = np.dot(solar_panel_norm_body, init_attit_mat_temp)
				
					#	calculate angle to sun vector
					#	sun angle
					minimisation_angle = np.arccos(
						np.dot(solar_panel_norm_orb_temp, sun_orb_vec_temp)
						/ (np.linalg.norm(solar_panel_norm_orb_temp) *  np.linalg.norm(sun_orb_vec_temp))
					)
				
					angle_data[i] = minimisation_angle
				
				#	find minimum angle
				min_ang = angle_data.min()
				min_index = np.where(angle_data==min_ang)[0][0]
				
				azimuth_angle = attitude_angle_range[min_index]
				
				#	set actual azimuth angle
				rot_vec_azm_deg = azimuth_angle
				
				print('\n		Setting Azimuth Angle to: ', azimuth_angle)
					
				#	generate lightcurve filename
				lightcurve_output_file = '{}_{}_{}_{}_{}_{}_{}_starlink_analysis.txt'.format(
					norad_id,
					int(track_id),
					np.round(d_brdf, 2),
					np.round(omega_brdf, 2),
					np.round(m_brdf, 2),
					np.round(rot_vec_inc_deg, 2),
					np.round(rot_vec_azm_deg, 2)
				)
	
				#	if exists, load
				if os.path.isfile(os.path.join(lightcurve_output_path, lightcurve_output_file)):
					print('\n				Reading existing Light Curve file.')
				
					#	load file
					lightcurve_data = np.loadtxt(
						os.path.join(lightcurve_output_path, lightcurve_output_file), delimiter=', '
					)
	
					lc_jd = lightcurve_data[:, 0]
					lc_seconds = lightcurve_data[:, 1]
					lc_app_pl_noshad = lightcurve_data[:, 2]
					lc_std_pl_noshad = lightcurve_data[:, 3]
	
					lc_jd = lightcurve_data[:, 4]
					lc_seconds = lightcurve_data[:, 5]
					lc_app_sp_noshad = lightcurve_data[:, 6]
					lc_std_sp_noshad = lightcurve_data[:, 7]
	
					lc_jd = lightcurve_data[:, 8]
					lc_seconds = lightcurve_data[:, 9]
					lc_app_sp_shad = lightcurve_data[:, 10]
					lc_std_sp_shad = lightcurve_data[:, 11]
	
				else:
	
					###############################################################################
					#	define initial attitude state
					###############################################################################
					
					##################################################
					#	attitude
					
					#	convert to radians
					rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
					rot_vec_inc_rad = np.radians(rot_vec_inc_deg)
					
					#	generate initial t = 0 attitude
					init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
					
					##################################################
					#	attitude motion in body frame
					
					#	generate rotation angle in radians
					if rot_period != 0.0:
						rot_ang_per_sec = (np.pi * 2) / rot_period
					else:
						rot_ang_per_sec = 0.0
					
					#	define rotation axis in body frame as unit vector
					rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
					
					#	generate attitude matrix in body frame
					attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)
					
					#	extract euler angles
					angles1 = np.array(mat2angles(attit_mat_body)[0:3])
					angles2 = np.array(mat2angles(attit_mat_body)[3:6])
					
					#	convert negative zeros
					for i, ele in enumerate(angles1):
						if str(ele)[0] == '-' and ele == 0:
							angles1[i] = 0.0
					
					#	extract attitude motion in the body frame
					attit_mot_body = angles1
					
					##################################################
					#	generate initial attitude rotation matrix
					init_attit_mat = rot_mat(init_attit)
					
					##################################################
					#	calculate rotation vector in orbital frame as unit vector
					rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)
					
					##################################################
					#	define velocity of rotation vector in orbital frame
					
					#	when equal to orbital motion, rot vec is stationary in inertial frame
					rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
					rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])
					
					###############################################################################
					#	generate brdf
					
					#	Refractive Index
					n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))
					
					#	Specular ratio
					s_brdf = 1.0 - d_brdf
					
					#	generate brdf
					brdf_data_pl = np.zeros([nfaces_pl, 5])
					brdf_data_sp = np.zeros([nfaces_sp, 5])
					
					for i in range(nfaces_pl):
						brdf_data_pl[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
					
					for i in range(nfaces_sp):
						brdf_data_sp[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
					
					###############################################################################
					#	perform light curve vector generation
					###############################################################################
					
					#	blank observation and illumination vectors in orbital frame
					obs_vec_orb = np.array([1.0, 0.0, 0.0])
					illum_vec_orb = np.array([1.0, 0.0, 0.0])
					
					#	generate orb2inertial velocity vector
					orb2inertial = np.array([0.0, 0.0, -orbital_mot])
					
					#	vector generation function
					vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
						path_data, time_step, len_record,
						sat_orb, sun_orb, earth_orb, site_orb,
						orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
						obs_vec_orb, illum_vec_orb
					)
					
					##################################################
					#	parse vector data
					
					#	extract orbital positions of custom input vectors
					illum_vec_orb_data = vector_data_orb[:, 28:31]
					obs_vec_orb_data = vector_data_body[:, 31:34]
					
					#	body data
					sun_body_data = vector_data_body[:, 28:31]
					site_body_data = vector_data_body[:, 31:34]
					earth_body_data = vector_data_body[:, 19:22]
					sat_body_vel = vector_data_body[:, 22:25]
					sat_body_pos = np.zeros([len(sat_body_vel), 3])
					sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
					illum_vec_body_data = vector_data_body[:, 28:31]
					obs_vec_body_data = vector_data_body[:, 31:34]
					
					#	orb data
					sun_orb_data = vector_data_orb[:, 28:31]
					site_orb_data = vector_data_orb[:, 31:34]
					earth_orb_data = vector_data_orb[:, 19:22]
					sat_orb_vel = vector_data_orb[:, 22:25]
					sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
					sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
					illum_vec_orb_data = vector_data_orb[:, 28:31]
					obs_vec_orb_data = vector_data_orb[:, 31:34]
					
					###############################################################################
					#	validation
					###############################################################################
	
					print('\n	Validating Phase and Distance data...')
					
					#	initialise arrays
					sim_phase_data = []
					sim_dist_data = []
					
					#	calculate simulation phase and distance data
					for (sun_body_vec, site_body_vec) in zip(sun_body_data, site_body_data):
					
						sim_phase_ang = np.degrees(vec_ang(sun_body_vec, site_body_vec))
						sim_phase_data.append(sim_phase_ang)
					
						sim_dist_data.append(np.linalg.norm(site_body_vec))
					
					#	convert to numpy arrays
					sim_phase_data = np.array(sim_phase_data)
					sim_dist_data = np.array(sim_dist_data)
					
					if phase_dist_validation_plot_flag is True:
						#	plot phase data
						sim_phase_line, = ax_phase.plot(sim_phase_data)
						mmt_phase_line, = ax_phase.plot(mmt_phase_data)
			
						#	plot distance data
						sim_dist_line, = ax_dist.plot(sim_dist_data)
						mmt_dist_line, = ax_dist.plot(mmt_dist_data)
	
					#	initialise percentage difference array
					perc_diff_phase_array = []
					perc_diff_dist_array = []
	
					#	loop through data
					for sim_phase, mmt_phase, sim_dist, mmt_dist in zip(
							sim_phase_data, mmt_phase_data, sim_dist_data, mmt_dist_data
						):
	
						#	percentage difference in phase
						perc_diff_phase = (sim_phase / mmt_phase) * 100
	
						#	convert percentage
						if perc_diff_phase < 100:
							perc_diff_phase = 100 - perc_diff_phase
						elif perc_diff_phase > 100:
							perc_diff_phase = perc_diff_phase - 100
	
						#	append to output
						perc_diff_phase_array.append(perc_diff_phase)
	
						#	percentage difference in distance
						perc_diff_dist = (sim_dist / mmt_dist) * 100
	
						#	convert percentage
						if perc_diff_dist < 100:
							perc_diff_dist = 100 - perc_diff_dist
						elif perc_diff_dist > 100:
							perc_diff_dist = perc_diff_dist - 100
	
						#	append to output
						perc_diff_dist_array.append(perc_diff_dist)
	
					#	convert to numpy arrays
					perc_diff_phase_array = np.array(perc_diff_phase_array)
					perc_diff_dist_array = np.array(perc_diff_dist_array)
	
					avg_perc_diff_phase = np.average(np.array(perc_diff_phase_array))
					avg_perc_diff_dist = np.average(np.array(perc_diff_dist_array))
	
					print('\n	Average phase diff = ', avg_perc_diff_phase)
					print('	Average dist diff = ', avg_perc_diff_dist)
	
	# 				#	update plot
	# 				plt.pause(1)
	
					if avg_perc_diff_phase > 1.0:
						print('\n	Phase percentage difference is above threshold!')
						continue
	
					if avg_perc_diff_dist > 1.0:
						print('\n	Distance percentage difference is above threshold!')
						continue
	# 				
	# 				ans = input('\nEnter to continue...')
	# 				if ans != 'y':
	# 					continue
					
					###############################################################################
					#	generate light curves
					###############################################################################
					
					print('\n[{}]	Generating Light Curves...'.format(datetime.datetime.now()))
					
					###############################################################################
					#	initialise light curve generation
		
					#	generate sim time
					sim_time = np.linspace(0, duration_sec, len_record)
					
					#	initilise lightcurve output
					light_curve_array = np.zeros((lc_len_record, 4))
					light_curve_array[:, 1] = sim_time
					
					light_curve_array_pl_noshad = np.copy(light_curve_array)
					light_curve_array_sp_noshad = np.copy(light_curve_array)
					light_curve_array_sp_shad = np.copy(light_curve_array)
					
					jdate_start = start_jd[0] + start_jd[1]
					
					###############################################################################
					#	generate payload no shadowing light curve
					
					#	run light curve generation function
					light_curve_array_pl_noshad = glim_slc_new(
						sun_body_data, site_body_data, light_curve_array_pl_noshad,
						jdate_start, body_vectors_pl, body_areas_pl, brdf_data_pl
					)
					
					lc_seconds = light_curve_array_pl_noshad[:, 1]
					lc_app_pl_noshad = light_curve_array_pl_noshad[:, 2]
					lc_std_pl_noshad = light_curve_array_pl_noshad[:, 3]
					
					###############################################################################
					#	generate solar panel no shadowing light curve
					
					#	run light curve generation function
					light_curve_array_sp_noshad = glim_slc_new(
						sun_body_data, site_body_data, light_curve_array_sp_noshad,
						jdate_start, body_vectors_sp, body_areas_sp, brdf_data_sp
					)
					
					lc_app_sp_noshad = light_curve_array_sp_noshad[:, 2]
					lc_std_sp_noshad = light_curve_array_sp_noshad[:, 3]
					
					###############################################################################
					#	generate solar panel with shadowing light curve
					
					slc_log_level = 0
					
					#	run light curve generation function
					light_curve_array_sp_shad = glim_slc_shadowing(
						sun_body_data, site_body_data, sun_orb_data, site_orb_data,
						light_curve_array_sp_shad,
						jdate_start, time_step,
						body_mesh_sp,
						init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
						brdf_data_sp,
						slc_log_level
					)
					
					lc_app_sp_shad = light_curve_array_sp_shad[:, 2]
					lc_std_sp_shad = light_curve_array_sp_shad[:, 3]
	
					print('\nSaving to Lightcurve file: ', lightcurve_output_file)
	
	
					lightcurve_data = np.hstack([
						light_curve_array_pl_noshad,
						light_curve_array_sp_noshad,
						light_curve_array_sp_shad
					])
	
					#	save file
					np.savetxt(
						os.path.join(lightcurve_output_path, lightcurve_output_file),
						lightcurve_data,
						delimiter=', '
					)
	
#					#	remove data from phase/distance plot
#					sim_phase_line.remove()
#					mmt_phase_line.remove()
#					sim_dist_line.remove()
#					mmt_dist_line.remove()
	
				###################################################################################
				#	final figure generation
				###################################################################################
				
				print('\n	####################################################################')
				print('	Generating Final Plot Data...')
				
				###################################################################################
				#	plot lightcurve
				
				#	set colour string
				col_str = 'C' + str(colour_index)
				colour_index += 1
				
				if lightcurve_plot_flag is True:
					lc_line1, = ax.plot(
						lc_seconds, lc_app_pl_noshad,
						color=col_str, linestyle='dashdot', label='Synth; PL; No-Shadowing'
					)
	
					lc_line2, = ax.plot(
						lc_seconds, lc_app_sp_noshad,
						color=col_str, linestyle='dashed', label='Synth; SP; No-Shadowing'
					)
	
					lc_line3, = ax.plot(
						lc_seconds, lc_app_sp_shad,
						color=col_str, linestyle='dotted', label='Synth; SP; Shadowing'
					)
	
					lc_line4, = ax.plot(
						lc_seconds, mmt_signal_app,
						color=col_str, linestyle='solid', label='MMT'
					)
					
					ax.legend(
						loc='lower center',
						ncol=7,
						fontsize=legend_fontsize,
						frameon=False
					)
				
				print('\n		Average Mag. = ', np.average(lc_app_sp_shad))
	
				avg_mmt_signal_app = np.average(mmt_signal_app)
				max_mmt_signal_app = mmt_signal_app.max()
				min_mmt_signal_app = mmt_signal_app.min()
	
				panel_involvment = rms_calc(lc_app_sp_shad, lc_app_pl_noshad)
				print('\n		Panel Involvment = ', np.average(panel_involvment))

				panel_shadowing =  rms_calc(lc_app_sp_noshad, lc_app_sp_shad)
				print('\n		Panel Shadowing = ', np.average(panel_shadowing))

				###################################################################################
				#	finalise track loop
				###################################################################################
	
				light_curve_index += 1
				track_index += 1
				first_track = False
		
				plotted_light_curves += 1
				plotted_track_indices.append(int(track_itt + 1))
	
				if starlink_id not in plotted_object_indices:
					plotted_object_indices.append(starlink_id)
	
				tracks_plotted += 1
				total_tracks_plotted += 1
	
				print('\n				Track Complete.')
	
	
				batch_solar_panel_data.append([
					norad_id,
					int(starlink_id),
					lc_start_jd,
					lc_duration_sec,
					avg_mmt_signal_app,
					min_mmt_signal_app,
					max_mmt_signal_app,
					panel_involvment,
					panel_shadowing,
					np.average(mmt_dist_data),
					mmt_dist_data.min(),
					mmt_dist_data.max(),
					np.average(mmt_phase_data),
					mmt_phase_data.min(),
					mmt_phase_data.max(),

					np.average(lc_app_pl_noshad),
					lc_app_pl_noshad.min(),
					lc_app_pl_noshad.max(),

					np.average(lc_app_sp_noshad),
					lc_app_sp_noshad.min(),
					lc_app_sp_noshad.max(),

					np.average(lc_app_sp_shad),
					lc_app_sp_shad.min(),
					lc_app_sp_shad.max(),

				])

	#			#	show plot
	#			plt.pause(0.001)
	#			input('\nEnter to continue...')
	
				#	remove data
				lc_line1.remove()
				lc_line2.remove()
				lc_line3.remove()
				lc_line4.remove()
	
			#######################################################################################
			#	finalise object loop
			#######################################################################################
	
			if plotted_light_curves == 0:
				print('\n		No tracks plotted, continuing...')
				continue
	
			obj_plotted += 1
			total_obj_plotted += 1
	
			print('\n		Starlink Object Complete.')
			print('\n		Plotted_object_indices = ', plotted_object_indices)
		
		###########################################################################################
		#	finalise batch loop
		###########################################################################################
	
		#	convert to numpy array
		plotted_object_indices = np.array(plotted_object_indices)
	
		print('\n	Loop Complete...')
		print('\n	This Batch, {} Objects were Plotted...'.format(int(obj_plotted)))
		print('\n	This Batch, {} Light Curves were Plotted...'.format(int(tracks_plotted)))
	
		print('\n	This Batch, Plotted Objects:\n	', plotted_object_indices)
	
		###########################################################################################

		#	convert to numpy array
		batch_solar_panel_data = np.array(batch_solar_panel_data)

		print('\n	Saving Final Array of Batch:')

		#	save file
		np.savetxt(
			os.path.join(lightcurve_output_path, output_file),
			batch_solar_panel_data,
			delimiter=', '
		)

		print('\n	Done.')

	###############################################################################################
	#	data post processing
	###############################################################################################

	###############################################################################################
	#	parse data from array

	print('\n	Batch Data: ', type(batch_solar_panel_data))
	print('	Batch Data: ', batch_solar_panel_data.shape)

	norad_id_data = batch_solar_panel_data[:, 0]
	plotted_object_indices = batch_solar_panel_data[:, 1]
	lc_start_jd_data = batch_solar_panel_data[:, 2]
	lc_duration_sec_data = batch_solar_panel_data[:, 3]

	avg_mmt_signal_app = batch_solar_panel_data[:, 4]
	min_mmt_signal_app = batch_solar_panel_data[:, 5]
	max_mmt_signal_app = batch_solar_panel_data[:, 6]
	
	panel_involvment = batch_solar_panel_data[:, 7]
	panel_shadowing = batch_solar_panel_data[:, 8]
	
	avg_mmt_dist_data = batch_solar_panel_data[:, 9]
	min_mmt_dist_data = batch_solar_panel_data[:, 10]
	max_mmt_dist_data = batch_solar_panel_data[:, 11]
	
	avg_mmt_phase_data = batch_solar_panel_data[:, 12]
	min_mmt_phase_data = batch_solar_panel_data[:, 13]
	max_mmt_phase_data = batch_solar_panel_data[:, 14]
	
	avg_pl_noshad_app = batch_solar_panel_data[:, 15]
	min_pl_noshad_app = batch_solar_panel_data[:, 16]
	max_pl_noshad_app = batch_solar_panel_data[:, 17]
	
	avg_sp_noshad_app = batch_solar_panel_data[:, 18]
	min_sp_noshad_app = batch_solar_panel_data[:, 19]
	max_sp_noshad_app = batch_solar_panel_data[:, 20]
	
	avg_sp_shad_app = batch_solar_panel_data[:, 21]
	min_sp_shad_app = batch_solar_panel_data[:, 22]
	max_sp_shad_app = batch_solar_panel_data[:, 23]

	#	set total plotted object indices list
	if first_batch is True:
		total_plotted_object_indices = plotted_object_indices
	
		first_batch = False
	
	else:
		total_plotted_object_indices = np.hstack([
			total_plotted_object_indices,
			plotted_object_indices
		])

	###############################################################################################
	#	solar panel involvment plots
	
	label_str = '$B' + launch_string[1:-1] + ' ' + str(batch_index + 1) + '$'

	if sp_involvment_plot_flag is True:

		if results_str == 'synth':
			plot1 = avg_sp_shad_app
			plot2 = min_sp_shad_app
			plot3 = max_sp_shad_app

		elif results_str == 'real':
			plot1 = avg_mmt_signal_app
			plot2 = min_mmt_signal_app
			plot3 = max_mmt_signal_app


		#	average magnitude against solar panel involvment
		sp_data_ax1.scatter(
			plot1,
			panel_involvment,
			marker=marker_str,
			color=batch_color_str,
			label=label_str,
		)
		#	minimum magnitude against solar panel involvment
		sp_data_ax2.scatter(
			plot2,
			panel_involvment,
			marker=marker_str,
			color=batch_color_str,
#			label=label_str,
		)
		
		#	maximum magnitude against solar panel involvment
		sp_data_ax3.scatter(
			plot3,
			panel_involvment,
			marker=marker_str,
			color=batch_color_str,
#			label=label_str,
		)

		if batch_index == 0:
			all_plotted_data1 = np.hstack([
				np.reshape(plot1, [len(plot1), 1]),
				np.reshape(panel_involvment, [len(panel_involvment), 1])
			])

			all_plotted_data2 = np.hstack([
				np.reshape(plot2, [len(plot2), 1]),
				np.reshape(panel_involvment, [len(panel_involvment), 1])
			])

			all_plotted_data3 = np.hstack([
				np.reshape(plot3, [len(plot3), 1]),
				np.reshape(panel_involvment, [len(panel_involvment), 1])
			])
		else:

			all_plotted_data1_toadd = np.hstack([
				np.reshape(plot1, [len(plot1), 1]),
				np.reshape(panel_involvment, [len(panel_involvment), 1])
			])
			all_plotted_data1 = np.vstack([all_plotted_data1, all_plotted_data1_toadd])


			all_plotted_data2_toadd = np.hstack([
				np.reshape(plot2, [len(plot2), 1]),
				np.reshape(panel_involvment, [len(panel_involvment), 1])
			])
			all_plotted_data2 = np.vstack([all_plotted_data2, all_plotted_data2_toadd])


			all_plotted_data3_toadd = np.hstack([
				np.reshape(plot3, [len(plot3), 1]),
				np.reshape(panel_involvment, [len(panel_involvment), 1])
			])
			all_plotted_data3 = np.vstack([all_plotted_data3, all_plotted_data3_toadd])

	###############################################################################################
	#	solar panel shadowing plots
	
	if sp_shadowing_plot_flag is True:

		if results_str == 'synth':
			plot1 = avg_sp_shad_app
			plot2 = min_sp_shad_app
			plot3 = max_sp_shad_app

		elif results_str == 'real':
			plot1 = avg_mmt_signal_app
			plot2 = min_mmt_signal_app
			plot3 = max_mmt_signal_app

		#	average magnitude against solar panel involvment
		sp_data_ax4.scatter(
			plot1,
			panel_shadowing,
			marker=marker_str,
			color=batch_color_str,
			label=label_str,
		)
		
		#	minimum magnitude against solar panel involvment
		sp_data_ax5.scatter(
			plot2,
			panel_shadowing,
			marker=marker_str,
			color=batch_color_str,
		)
		
		#	maximum magnitude against solar panel involvment
		sp_data_ax6.scatter(
			plot3,
			panel_shadowing,
			marker=marker_str,
			color=batch_color_str,
		)

		if batch_index == 0:
			all_plotted_data1 = np.hstack([
				np.reshape(plot1, [len(plot1), 1]),
				np.reshape(panel_shadowing, [len(panel_shadowing), 1])
			])

			all_plotted_data2 = np.hstack([
				np.reshape(plot2, [len(plot2), 1]),
				np.reshape(panel_shadowing, [len(panel_shadowing), 1])
			])

			all_plotted_data3 = np.hstack([
				np.reshape(plot3, [len(plot3), 1]),
				np.reshape(panel_shadowing, [len(panel_shadowing), 1])
			])
		else:

			all_plotted_data1_toadd = np.hstack([
				np.reshape(plot1, [len(plot1), 1]),
				np.reshape(panel_shadowing, [len(panel_shadowing), 1])
			])
			all_plotted_data1 = np.vstack([all_plotted_data1, all_plotted_data1_toadd])


			all_plotted_data2_toadd = np.hstack([
				np.reshape(plot2, [len(plot2), 1]),
				np.reshape(panel_shadowing, [len(panel_shadowing), 1])
			])
			all_plotted_data2 = np.vstack([all_plotted_data2, all_plotted_data2_toadd])


			all_plotted_data3_toadd = np.hstack([
				np.reshape(plot3, [len(plot3), 1]),
				np.reshape(panel_shadowing, [len(panel_shadowing), 1])
			])
			all_plotted_data3 = np.vstack([all_plotted_data3, all_plotted_data3_toadd])

	###############################################################################################
	#	average magnitude against phase and distance

	if phase_dist_plot_flag is True:

		if results_str == 'synth':
			plot1 = avg_sp_shad_app

		elif results_str == 'real':
			plot1 = avg_mmt_signal_app

		sp_data_ax_dist.scatter(
			avg_mmt_dist_data,
			plot1,
			marker=marker_str,
			color=batch_color_str,
		)
		
		sp_data_ax_phase.scatter(
			avg_mmt_phase_data,
			plot1,
			marker=marker_str,
			color=batch_color_str,
		)

###################################################################################################
#	end of loop
###################################################################################################

if sp_involvment_plot_flag is True:

	minimum_thres = 1e-2
	minimum_thres = 0

	all_plotted_data1_temp = all_plotted_data1
	all_plotted_data1 = []
	for line in all_plotted_data1_temp:
		if line[1] > minimum_thres:
			all_plotted_data1.append(line)
	all_plotted_data1 = np.array(all_plotted_data1)
	
	all_plotted_data2_temp = all_plotted_data2
	all_plotted_data2 = []
	for line in all_plotted_data2_temp:
		if line[1] > minimum_thres:
			all_plotted_data2.append(line)
	all_plotted_data2 = np.array(all_plotted_data2)
	
	all_plotted_data3_temp = all_plotted_data3
	all_plotted_data3 = []
	for line in all_plotted_data3_temp:
		if line[1] > minimum_thres:
			all_plotted_data3.append(line)
	all_plotted_data3 = np.array(all_plotted_data3)
	
	
	x1 = all_plotted_data1[:, 0]
	y1 = all_plotted_data1[:, 1]
	z1 = np.polyfit(y1, x1, 1)
	p = np.poly1d(z1)
	trend1 = p(y1)
	trend1_temp = trend1

#	trend1 = []
#	xplot1 = []
#	for (line, x) in zip(trend1_temp, all_plotted_data1[:, 0]):
#		if line > 0:
#			trend1.append(line)
#			xplot1.append(x)
#	
#	trend1 = np.array(trend1)
#	xplot1 = np.array(xplot1)
	
	
	x2 = all_plotted_data2[:, 0]
	y2 = all_plotted_data2[:, 1]
	z2 = np.polyfit(y2, x2, 1)
	p = np.poly1d(z2)
	trend2 = p(y2)
	trend2_temp = trend2

#	trend2 = []
#	xplot2 = []
#	for (line, x) in zip(trend2_temp, all_plotted_data2[:, 0]):
#		if line > 0:
#			trend2.append(line)
#			xplot2.append(x)
#	
#	trend2 = np.array(trend2)
#	xplot2 = np.array(xplot2)
	
	
	x3 = all_plotted_data3[:, 0]
	y3 = all_plotted_data3[:, 1]
	z3 = np.polyfit(y3, x3, 1)
	p = np.poly1d(z3)
	trend3 = p(y3)
	trend3_temp = trend3

#	trend3 = []
#	xplot3 = []
#	for (line, x) in zip(trend3_temp, all_plotted_data3[:, 0]):
#		if line > 0:
#			trend3.append(line)
#			xplot3.append(x)
#	
#	trend3 = np.array(trend3)
#	xplot3 = np.array(xplot3)
	
	title_size = 10
	


	#	average magnitude against solar panel involvment
	sp_data_ax1.plot(
		trend1,
		y1,
		color='k',
	#	label=label_str,
	)

	plot_data = np.array([
		[trend1.max(), y1.min()],
		[trend1.min(), y1.max()]
	])

	grad, intercept = line_from_two_points(
		plot_data[0, 0], plot_data[0, 1],
		plot_data[1, 0], plot_data[1, 1]
	)

	sp_data_ax1.set_title('$y = {:1.3f}x + {:1.3f}$'.format(grad, intercept), fontsize=title_size)
	


	#	minimum magnitude against solar panel involvment
	sp_data_ax2.plot(
		trend2,
		y2,
		color='k',
	#	label=label_str,
	)

	plot_data = np.array([
		[trend2.max(), y2.min()],
		[trend2.min(), y2.max()]
	])

	grad, intercept = line_from_two_points(
		plot_data[0, 0], plot_data[0, 1],
		plot_data[1, 0], plot_data[1, 1]
	)

	sp_data_ax2.set_title('$y = {:1.3f}x + {:1.3f}$'.format(grad, intercept), fontsize=title_size)
	


	#	maximum magnitude against solar panel involvment
	sp_data_ax3.plot(
		trend3,
		y3,
		color='k',
	#	label=label_str,
	)

	plot_data = np.array([
		[trend3.max(), y3.min()],
		[trend3.min(), y3.max()]
	])

	grad, intercept = line_from_two_points(
		plot_data[0, 0], plot_data[0, 1],
		plot_data[1, 0], plot_data[1, 1]
	)

	sp_data_ax3.set_title('$y = {:1.3f}x + {:1.3f}$'.format(grad, intercept), fontsize=title_size)



elif sp_shadowing_plot_flag is True:

	minimum_thres = 1e-2
	minimum_thres = 0

	all_plotted_data1_temp = all_plotted_data1
	all_plotted_data1 = []
	for line in all_plotted_data1_temp:
		if line[1] > minimum_thres:
			all_plotted_data1.append(line)
	all_plotted_data1 = np.array(all_plotted_data1)
	
	all_plotted_data2_temp = all_plotted_data2
	all_plotted_data2 = []
	for line in all_plotted_data2_temp:
		if line[1] > minimum_thres:
			all_plotted_data2.append(line)
	all_plotted_data2 = np.array(all_plotted_data2)
	
	all_plotted_data3_temp = all_plotted_data3
	all_plotted_data3 = []
	for line in all_plotted_data3_temp:
		if line[1] > minimum_thres:
			all_plotted_data3.append(line)
	all_plotted_data3 = np.array(all_plotted_data3)
	
	
	x1 = all_plotted_data1[:, 0]
	y1 = all_plotted_data1[:, 1]
	z1 = np.polyfit(y1, x1, 1)
	p = np.poly1d(z1)
	trend1 = p(y1)
	trend1_temp = trend1

#	trend1 = []
#	xplot1 = []
#	for (line, x) in zip(trend1_temp, all_plotted_data1[:, 0]):
#		if line > 0:
#			trend1.append(line)
#			xplot1.append(x)
#	
#	trend1 = np.array(trend1)
#	xplot1 = np.array(xplot1)
	
	
	x2 = all_plotted_data2[:, 0]
	y2 = all_plotted_data2[:, 1]
	z2 = np.polyfit(y2, x2, 1)
	p = np.poly1d(z2)
	trend2 = p(y2)
	trend2_temp = trend2

#	trend2 = []
#	xplot2 = []
#	for (line, x) in zip(trend2_temp, all_plotted_data2[:, 0]):
#		if line > 0:
#			trend2.append(line)
#			xplot2.append(x)
#	
#	trend2 = np.array(trend2)
#	xplot2 = np.array(xplot2)
	
	
	x3 = all_plotted_data3[:, 0]
	y3 = all_plotted_data3[:, 1]
	z3 = np.polyfit(y3, x3, 1)
	p = np.poly1d(z3)
	trend3 = p(y3)
	trend3_temp = trend3

#	trend3 = []
#	xplot3 = []
#	for (line, x) in zip(trend3_temp, all_plotted_data3[:, 0]):
#		if line > 0:
#			trend3.append(line)
#			xplot3.append(x)
#	
#	trend3 = np.array(trend3)
#	xplot3 = np.array(xplot3)
	
	title_size = 10
	


	#	average magnitude against solar panel involvment
	sp_data_ax4.plot(
		trend1,
		y1,
		color='k',
	#	label=label_str,
	)

	plot_data = np.array([
		[trend1.max(), y1.min()],
		[trend1.min(), y1.max()]
	])

	grad, intercept = line_from_two_points(
		plot_data[0, 0], plot_data[0, 1],
		plot_data[1, 0], plot_data[1, 1]
	)

	sp_data_ax4.set_title('$y = {:1.3f}x + {:1.3f}$'.format(grad, intercept), fontsize=title_size)
	


	#	minimum magnitude against solar panel involvment
	sp_data_ax5.plot(
		trend2,
		y2,
		color='k',
	#	label=label_str,
	)

	plot_data = np.array([
		[trend2.max(), y2.min()],
		[trend2.min(), y2.max()]
	])

	grad, intercept = line_from_two_points(
		plot_data[0, 0], plot_data[0, 1],
		plot_data[1, 0], plot_data[1, 1]
	)

	sp_data_ax5.set_title('$y = {:1.3f}x + {:1.3f}$'.format(grad, intercept), fontsize=title_size)
	


	#	maximum magnitude against solar panel involvment
	sp_data_ax6.plot(
		trend3,
		y3,
		color='k',
	#	label=label_str,
	)

	plot_data = np.array([
		[trend3.max(), y3.min()],
		[trend3.min(), y3.max()]
	])

	grad, intercept = line_from_two_points(
		plot_data[0, 0], plot_data[0, 1],
		plot_data[1, 0], plot_data[1, 1]
	)

	sp_data_ax6.set_title('$y = {:1.3f}x + {:1.3f}$'.format(grad, intercept), fontsize=title_size)











print('\nLoop Complete...')
print('\nA Total of {} Objects were Plotted...'.format(int(total_obj_plotted)))
print('\nA Total of {} Light Curves were Plotted...'.format(int(total_tracks_plotted)))

print('\nIn Total, Plotted Objects:\n	', total_plotted_object_indices)
# print('\nIn Total, Plotted Track Indices:\n	', plotted_track_indices)

if sp_involvment_plot_flag is True:
	#	show legend
	sp_data_fig_inv.legend(
		loc='center right',
		fontsize=legend_fontsize,
		ncol=1,
		frameon=False
	)


if sp_shadowing_plot_flag is True:
	#	show legend
	sp_data_fig_shad.legend(
		loc='center right',
		fontsize=legend_fontsize,
		ncol=1,
		frameon=False
	)

plt.pause(0.01)

###################################################################################################
#	save plots
###################################################################################################

if savefile_flag is True:

	if sp_involvment_plot_flag is True:

		fig_name = 'starlink_sp_inv_' + results_str + '.eps'

		#	average magnitude against solar panel involvment
		sp_data_fig_inv.savefig(os.path.join(fig_path, fig_name), bbox_inches="tight", dpi=300)
		print('\nFigure saved: ', fig_name)
		
	###############################################################################################
	#	solar panel shadowing plots
	
	if sp_shadowing_plot_flag is True:

		fig_name = 'starlink_sp_shad_' + results_str + '.eps'
#		fig_name = 'starlink_sp_shad_' + results_str + '_test.png'

		#	average magnitude against solar panel involvment
		sp_data_fig_shad.savefig(os.path.join(fig_path, fig_name), bbox_inches="tight", dpi=300)
		print('\nFigure saved: ', fig_name)
		
	###############################################################################################
	#	average magnitude against phase and distance

	if phase_dist_plot_flag is True:

		fig_dist_name = 'starlink_' + launch_string + '_full.eps'
		fig_phase_name = 'starlink_' + launch_string + '_full.eps'

		#	distance figure
		sp_data_fig_dist.savefig(os.path.join(fig_path, fig_dist_name), bbox_inches="tight", dpi=300)
		
		#	phase figure
		sp_data_fig_phase.savefig(os.path.join(fig_path, fig_phase_name), bbox_inches="tight", dpi=300)


plt.show()
sys.exit()
