###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

#	transformation functions
from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
from matplotlib.ticker import MaxNLocator
from matplotlib.colors import BoundaryNorm

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('#####################################################################################')
print('			MMT BRDF Fitting Results Plotter')
print('#####################################################################################')

print('\n[{}] Initialising...'.format(datetime.datetime.now()))

#	set start time
sim_start = time.time()
init_start = time.time()

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

launch_string = 'batch1'

#	set read path
read_path = os.path.join(path_data, 'input_dir', 'mmt_files', 'starlink', launch_string)

###################################################################################################
#	script specific initialisation

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Articles',
	'starlink_paper',
	'Figures'
)

#	set flag
savefile_flag = True
savefile_flag = False

###################################################################################################
#	light curve generation inputs
###################################################################################################

#	set time windows
window_start_date = [2019, 11, 30, 0, 0, 0]
window_stop_date = [2020, 5, 21, 0, 0, 0]

#	source mmt file
mmt_file = 'STARLINK-24_44238_MMT_6404.txt'
track_index = 4

#	source mmt file
mmt_file = 'STARLINK-30_44244_MMT_6426.txt'
track_index = 7

#	starlink 35 case
mmt_file = 'STARLINK-35_44264_MMT_6422.txt'
track_index = 6

#	starlink 50 case
mmt_file = 'STARLINK-50_44285_MMT_6467.txt'
track_index = 7

#	starlink 63 case
mmt_file = 'STARLINK-63_44265_MMT_6435.txt'
track_index = 9

#	starlink 69 case
mmt_file = 'STARLINK-69_44267_MMT_6405.txt'
track_index = 7

#	starlink 70 case
mmt_file = 'STARLINK-70_44281_MMT_6411.txt'
track_index = 12


#######################################################
#	NEW

#	set time windows
window_start_date = [2010, 11, 30, 0, 0, 0]
window_stop_date = [2021, 5, 21, 0, 0, 0]

#	starlink 61 case
mmt_file = 'STARLINK-1017_44723_MMT_7071.txt'
track_index_range = [1, 2, 3]
track_index = 2

# #######################################################
# #	FIRST HALF
# #	set time windows
# window_start_date = [2019, 8, 1, 0, 0, 0]
# window_stop_date = [2019, 11, 20, 0, 0, 0]
# 
# #	starlink 21 case
# mmt_file = 'STARLINK-21_44245_MMT_6470.txt'
# track_index = 7
# 
# #	starlink 26 case
# mmt_file = 'STARLINK-26_44240_MMT_6640.txt'
# track_index = 4
# 
# #	starlink 27 case
# mmt_file = 'STARLINK-27_44241_MMT_6429.txt'
# track_index = 5
# 
# #	starlink 39 case
# mmt_file = 'STARLINK-39_44253_MMT_6439.txt'
# track_index = 5
# 
# #	starlink 42 case
# mmt_file = 'STARLINK-42_44256_MMT_6419.txt'
# track_index = 6
# 
# #	starlink 45 case
# mmt_file = 'STARLINK-45_44260_MMT_6425.txt'
# track_index = 4
# 
# #	starlink 61 case
# mmt_file = 'STARLINK-61_44249_MMT_6415.txt'
# track_index = 3


#	parse filename
name_str = mmt_file.split('-')[0]
object_id = int(mmt_file.split('-')[1].split('_')[0])
norad_id = int(mmt_file.split('-')[1].split('_')[1])

#	simulation time step
time_step = 0.1

#	set brdf parameters
d_brdf = 0.4
m_brdf = 0.5
omega_brdf = 0.06

###################################################################################################
#	load geometry

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str_pl = 'starlink_payload'

geom_str_sp = 'starlink_full'

# geom_str_sp = 'starlink_full_lowdrag'


geom_file_pl = geom_str_pl + '.stl'
geom_file_sp = geom_str_sp + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
rot_vec_inc_deg = 40.0
rot_vec_azm_deg_input = 290.0

rot_vec_inc_deg = 0.0
rot_vec_azm_deg_input = 0.0

print('\nInput azm angle = ', rot_vec_azm_deg_input)

#	correct azm
if rot_vec_azm_deg_input >= 180:
	rot_vec_azm_deg = rot_vec_azm_deg_input - 180

elif rot_vec_azm_deg_input < 180:
	rot_vec_azm_deg = rot_vec_azm_deg_input + 180

print('\nAttitude azm angle = ', rot_vec_azm_deg)

#	attitude motion
rot_period = 0.0

#	load static mesh
body_mesh_pl = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file_pl))
body_mesh_sp = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file_sp))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
body_mesh_pl.rotate_using_matrix(offset_rot_mat)
body_mesh_sp.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
body_vectors_pl = body_mesh_pl.vectors
body_vectors_sp = body_mesh_sp.vectors

body_normals_pl = body_mesh_pl.normals
body_normals_sp = body_mesh_sp.normals

body_areas_pl = body_mesh_pl.areas
body_areas_sp = body_mesh_sp.areas

nfaces_pl = len(body_areas_pl)
nfaces_sp = len(body_areas_sp)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])

###################################################################################################
#	define MMT observation site

site_str = 'mmt'
site_latgd = np.radians(43.64986)
site_lon = np.radians(41.43147)
site_alt = 2030.0

#	generate initial site vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

#	generate plottable vector
origin = np.array([0.0, 0.0, 0.0])

#	generate plottable vector
site_vec_plot = np.vstack([origin, site_itrf])

###################################################################################################
#	figure generation inputs

#	graphical output
inch2cm = 2.54

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4
xlabel_fontsize = 6
ylabel_fontsize = 6
xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set marker parameters
marker_size = 3

#	set figure line wdith
plot_linewidth = 1.0
 
###############################################################################################
#	display initialisation time
init_end = time.time()
init_time = np.round((init_end - init_start), 1)

print('\n	Initialisation Finished.')
print('\n		- Time taken = {} s.'.format(init_time))

print('\n	#############################################################################')
print('	Simulation Inputs Set To: ')
print('\n	Input File: ')
print('\n		Working from MMT data input file: 	{}'.format(mmt_file))
print('		Reading from input path: 		{}'.format(read_path))
print('\n			Name:	 	{}'.format(name_str))
print('			Object ID: 	{}'.format(object_id))
print('			Norad ID: 	{}'.format(norad_id))
print('			Track ID: 	{}'.format(track_index))
print('\n	Light Curve Inputs: ')
# print('\n			Geom File: 		{}'.format(geom_file))
print('			Rot Vec Azimuth: 	{}'.format(rot_vec_azm_deg))
print('			Rot Vec Inclination: 	{}'.format(rot_vec_inc_deg))
print('			Rotation Period: 	{}'.format(rot_period))
print('			BRDF: 			m = {}, omega = {}, m = {}'.format(d_brdf, omega_brdf, m_brdf))
print('\n	Observation Site Inputs: ')
print('\n			Site String: 	{}'.format(site_str))
print('			Latitude: 	{}'.format(np.degrees(site_latgd)))
print('			Longitude: 	{}'.format(np.degrees(site_lon)))
print('			Altitude: 	{} m'.format(site_alt))

###################################################################################################
#	load MMT file
###################################################################################################

#	set time
mmt_load_start = time.time()

print('\n[{}] Loading MMT data...'.format(datetime.datetime.now()))

with open(os.path.join(read_path, mmt_file)) as read_file:
	mmt_data = read_file.readlines()

###################################################################################################
#	parse MMT file

#	strip header
header_text = mmt_data[7]
raw_data = mmt_data[8:]

print('\n	Parsing MMT data...')

#	initialise output
n_rows = 14
parsed_data = np.zeros([len(raw_data), n_rows])

for i, line in enumerate(raw_data):

	#	split line
	date_str, time_str, mag_std, mag_app, opt_filter, penumbral, \
		distance, phase, channel, track_id = line.split(' ')

	#	parse data and set data types
	mag_std = float(mag_std)
	mag_app = float(mag_app)
	distance = float(distance)
	phase = float(phase)
	track_id = int(track_id)

	#	parse date
	year, month, day = date_str.split('-')
	year = int(year)
	month = int(month)
	day = int(day)

	#	parse time
	hour, minute, second = time_str.split(':')
	hour = int(hour)
	minute = int(minute)
	second = np.round(float(second), 1)

	#	calculate julian date
	jday, jfrac = date2jd(year, month, day, hour, minute, second)

	#	generate output line
	output_line = [
		track_id, object_id,
		jday, jfrac,
		year, month, day,
		hour, minute, second,
		mag_std, mag_app,
		distance, phase
	]

	#	add to output array
	parsed_data[i, :] = output_line

print('		Total data length = ', len(parsed_data))
print('		Total number of objects = ', len(np.unique(parsed_data[:, 1])))
print('		Total number of tracks = ', len(np.unique(parsed_data[:, 0])))

###################################################################################################
#	extracting data between specified windows

print('\n	Seaching for tracks inside time window:')

#	start window
window_start_jd = np.add(*date2jd(*window_start_date))
print('\n		Window start 	= ', window_start_date[0:3])

#	stop window
window_stop_jd = np.add(*date2jd(*window_stop_date))
print('		Window stop 	= ', window_stop_date[0:3])

#	extract track ids
track_id_list = np.unique(parsed_data[:, 0])
extracted_track_id_list = []

#	loop through tracks
ntracks = 0
first_lc_flag = True
for track_id in track_id_list:

	#	find track ids
	track_indices = np.where(parsed_data[:, 0]==track_id)[0]

	#	extract full data
	track_data = parsed_data[track_indices, :]

	#	extract light curve data
	jd_plot = np.add(track_data[:, 2], track_data[:, 3])
	std_mag_plot = track_data[:, 10]
	app_mag_plot = track_data[:, 11]
	distance_plot = track_data[:, 12]
	phase_plot = track_data[:, 13]
	start_line = track_data[0]
	stop_line = track_data[-1]
	start_jdate = np.add(start_line[2], start_line[3])
	stop_jdate = np.add(stop_line[2], stop_line[3])

	#	ensure between window start/stop
	if start_jdate > window_start_jd and stop_jdate < window_stop_jd:

		#	light curve is within window
		ntracks += 1

		#	if first light curve, start array
		if first_lc_flag is True:
			windowed_data = track_data
			first_lc_flag = False

		#	else append to list
		elif first_lc_flag is False:
			windowed_data = np.vstack([windowed_data, track_data])

		#	add track id to output
		extracted_track_id_list.append(track_id)

print('\n		{} track(s) extracted.'.format(ntracks))
extracted_track_id_list = np.array(extracted_track_id_list)

###############################################################################################
#	extracting requested single track

print('\n		Extracting requested track index {}'.format(track_index))

#	set track array index
track_array_index = track_index - 1
track_id = int(extracted_track_id_list[track_array_index])

track_indices = np.where(windowed_data[:, 0]==track_id)[0]
track_data = windowed_data[track_indices]

###############################################################################################
#	extract track time data

mmt_line = track_data[0]

#	parse line elements
track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = track_data[0]

start_jd = [jday, jfrac]

#	parse line elements
track_id, line_object_id, jday, jfrac, year, month, day, hour, minute, second, \
	mag_std, mag_app, distance, phase = track_data[-1]

stop_jd = [jday, jfrac]

###################################################################################################
#	generate simulation time data

#	extract start and stop jds
lc_start_jd = start_jd[0] + start_jd[1]
lc_stop_jd = stop_jd[0] + stop_jd[1]

#	convert to dates
lc_start_date = jd2date(*jday_jfrac(lc_start_jd))
lc_stop_date = jd2date(*jday_jfrac(lc_stop_jd)) 

#	light curve duration
lc_duration_sec = np.round(86400 * (lc_stop_jd - lc_start_jd), 1)

#	set time step
lc_len_record = int(np.round(lc_duration_sec / time_step, 1)) + 1

#	time range in julian days
lc_sim_range = np.linspace(lc_start_jd, lc_stop_jd, lc_len_record)
sim_range_sec = np.round(np.multiply(np.subtract(lc_sim_range, lc_sim_range[0]), 86400), 1)

###################################################################################################
#	ensure time continuity

print('\n	Ensuring time continuity in extracted data...')

track_data_temp = []
sim_index = 0

for itt, line in enumerate(track_data):

	#	parse line data
	track_id, object_id, \
		jday, jfrac, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase = line
	
	step_jd = jday + jfrac

	#	calculate current seconds
	current_sec = np.round((step_jd - lc_sim_range[0]) * 86400, 1)

	output_line = [
		jday, jfrac, current_sec, year, month, day, hour, minute, second, \
		mag_std, mag_app, distance, phase
	]

	##################################################
	#	extracting data of time step

	#	data match found
	if current_sec == sim_range_sec[sim_index]:
		track_data_temp.append(output_line)
		sim_index += 1

		#	save current line as previous line
		prev_line = output_line

	#	data point is not required
	elif current_sec < sim_range_sec[sim_index]:
		pass

	#	interpolation required
	elif current_sec > sim_range_sec[sim_index]:

		#	parse previous line
		jday_prev, jfrac_prev, prev_sec, \
			year_prev, month_prev, day_prev, hour_prev, minute_prev, second_prev, \
			mag_std_prev, mag_app_prev, distance_prev, phase_prev = prev_line

		#	calculate interpolation requirements
		gap = np.round(current_sec - prev_sec, 1)
		npts = int(gap / time_step)

		#	get current light curve data
		mag_std_last = mag_std
		mag_app_last = mag_app
		distance_last = distance
		phase_last = phase

		#	calculate linear interpolation step values
		mag_std_step = (mag_std_last - mag_std_prev) / npts
		mag_app_step = (mag_app_last - mag_app_prev) / npts
		distance_step = (distance_last - distance_prev) / npts
		phase_step = (phase_last - phase_prev) / npts

		#	calculate interpolation data
		for n in range(1, npts + 1):

			#	current simulation second
			interp_sec = np.round(prev_sec + (n * time_step), 1)

			#	time data
			jdate = (jday_prev + jfrac_prev) + (n * (time_step / 86400))
			jday, jfrac = jday_jfrac(jdate)
			year, month, day, hour, minute, second = jd2date(jday, jfrac)

			#	calculate interpolated light curve data
			mag_std = mag_std_prev + (n * mag_std_step)
			mag_app = mag_app_prev + (n * mag_app_step)
			distance = distance_prev + (n * distance_step)
			phase = phase_prev + (n * phase_step)

			#	generate new output line
			interp_output_line = [
				jday, jfrac, interp_sec, year, month, day, hour, minute, second, \
				mag_std, mag_app, distance, phase
			]

			if interp_sec == sim_range_sec[sim_index]:
				track_data_temp.append(interp_output_line)
				sim_index += 1

		if interp_sec != current_sec:
			#	append final line
			track_data_temp.append(output_line)
			sim_index += 1

	else:
		print('ERROR')
		sys.exit()

	#	save previous line
	if itt != 0:
		prev_line = output_line

	#	array is complete condition
	if len(track_data_temp) == lc_len_record:
		break

#	ensure parsed MMT data matches simulation data
track_data = np.array(track_data_temp)

if len(track_data) == len(lc_sim_range):
	print('\n		Simulation and MMT data matches.')
else:
	print('\n		Error: Simulation and MMT data does not match...Exiting.')
	sys.exit()

###############################################################################################
#	extract mmt brightness data

mmt_distance_data = track_data[:, 11]
mmt_phase_data = track_data[:, 12]

mmt_signal_std = track_data[:, 9]
mmt_signal_app = track_data[:, 10]

###############################################################################################
#	display initialisation time
mmt_load_end = time.time()
mmt_load_time = np.round((mmt_load_end - mmt_load_start), 1)

print('\n	MMT Load Finished.')
print('\n		- Time taken = {} s.'.format(mmt_load_time))

print('\n#####################################################################################')
print('Light curve parameters: ')
print('	Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_start_date))
print('	Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*lc_stop_date))
print('	Simulation time step:		{}'.format(time_step))
print('	Duration:			{}		[steps]'.format(lc_len_record))
print('					{}		[s]'.format(np.round(lc_duration_sec, 1)))
print('\n#####################################################################################')

###############################################################################################
#	acquire TLE
###############################################################################################

#	set time
tle_start = time.time()

print('\n	[{}] Acquiring TLEs...'.format(datetime.datetime.now()))

#	TLE request dates, add day either side to increase tle find chance
tle_start_date = jd2date(*jday_jfrac(lc_start_jd - 1))
tle_stop_date = jd2date(*jday_jfrac(lc_stop_jd + 1)) 

#	generate TLE file name
tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)

#	assemble TLE path
tle_path = os.path.join(path_data, 'input_dir', 'tle_files')

#	if exists, load
if os.path.isfile(os.path.join(tle_path, tle_file)):

	print('\n		Reading existing TLE file.')

	#	load file
	with open(os.path.join(tle_path, tle_file), 'r') as file:
		tle_data = file.readlines()

else:
	print('\n		Requesting TLE file...')

	#	request TLEs
	tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)

	#	save file
	with open(os.path.join(tle_path, tle_file), 'w') as file:
		for line in tle_data:
			file.write(line)

#	grab first TLE pair
tle_line1 = tle_data[1]
tle_line2 = tle_data[2]

#	mean mot rad/sec
orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
orbital_mot = (2 * np.pi) / orbital_period

###############################################################################################
#	display TLE acquisition time
tle_end = time.time()
tle_time = np.round((tle_end - tle_start), 1)

print('\n		TLE Load Finished.')
print('\n			- Time taken = {} s.'.format(tle_time))

###############################################################################################
#	select time data to use
###############################################################################################

sim_range = lc_sim_range
len_record = lc_len_record
start_date = lc_start_date
stop_date = lc_stop_date
duration_sec = lc_duration_sec

###############################################################################################
#	generate eop data arrays
###############################################################################################

#	set time
eop_start = time.time()

print('\n	[{}] Generating EOP data array...'.format(datetime.datetime.now()))

#	EOP data
eop_dir = 'eop_files'
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	assemble eop path
eop_path = os.path.join(path_data, eop_dir)

#	generate interpolated eop_data
eop_log_level = 0
eop_data, dat_data = eop_interpolate(
	eop_path,
	eop_file,
	dat_file,
	sim_range,
	time_step,
	eop_log_level
)

###################################################################################################
#	display time
eop_end = time.time()
eop_time = np.round((eop_end - eop_start), 1)

print('\n		EOP data generated.')
print('\n			- Time taken = {} s.'.format(eop_time))

###################################################################################################
#	generate time objects
###################################################################################################

#	start dat value
dat = dat_data[0, -1]
	
#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate start time object
start_time_object = get_times(*start_date, ut_offset, dat)
	
#	end dat value
dat = dat_data[-1, -1]
	
#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate stop time object
stop_time_object = get_times(*stop_date, ut_offset, dat)

###################################################################################################
#	generate rotation matrices
###################################################################################################

#	set time
rot_mat_start = time.time()
print('\n	[{}]	Generating rotation matrices:'.format(datetime.datetime.now()))

#	tt time
jdaytt_start = start_time_object.jday_tt
jfractt_start = start_time_object.jfrac_tt
jdtt_start = jdaytt_start + jfractt_start

#	ut1 time
jdayut1_start = start_time_object.jday_ut1
jfracut1_start = start_time_object.jfrac_ut1
jdut1_start = jdayut1_start + jfracut1_start

#	generate rotation matrices
rot_mat_log_level = 0
pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
	jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level
)

###################################################################################################
#	display time
rot_mat_end = time.time()
rot_mat_time = np.round((rot_mat_end - rot_mat_start), 1)

print('\n		Rotation Matrices Generated.')
print('\n			- Time taken = {} s.'.format(rot_mat_time))

###################################################################################################
#	MODULE:	Horizons Positions
###################################################################################################

#	set time
horz_start = time.time()
print('\n	[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))

#	set horizons logging level
horz_logging_level = 0

sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
	path_data, horz_logging_level,
	start_time_object, stop_time_object,
	len_record, time_step,
	eop_path
)

#	find indices of light curve data
timecol = sun_icrf[:, 0]
lc_index = np.where((timecol >= lc_start_jd) & (timecol <= lc_stop_jd))[0]

###################################################################################################
#	display time
horz_end = time.time()
horz_time = np.round((horz_end - horz_start), 1)

print('\n		Horizons data generated.')
print('\n			- Time taken = {} s.'.format(horz_time))

###################################################################################################
#	MODULE:	Object Position
###################################################################################################

#	set time
sat_start = time.time()
print('\n	[{}]	Generating object position data...'.format(datetime.datetime.now()))

satpos_logging_level = 0

sat_gcrf_sgp4, sat_itrf_sgp4, orbital_mot = module_sat_pos(
	path_data, satpos_logging_level, len_record, time_step,
	start_time_object, stop_time_object,
	norad_id,
	eop_data, dat_data,
	pm_array, pn_array, st_array, c2t_array
)

###################################################################################################
#	display time
sat_end = time.time()
sat_time = np.round((sat_end - sat_start), 1)

print('\n		Object position data generated.')
print('\n			- Time taken = {} s.'.format(sat_time))

###################################################################################################
#	MODULE:	Transformation
###################################################################################################

#	set time
trans_start = time.time()
print('\n	[{}]	Performing transformations...'.format(datetime.datetime.now()))

identification_str = 'mmt-render'

trans_logging_level = 0
save_flag = False

sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb \
	= module_transformation(
		path_data, trans_logging_level, save_flag,
		start_time_object, stop_time_object,
		identification_str, site_str,
		len_record, time_step,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array,
		sat_gcrf_sgp4, sun_gcrf, earth_gcrf, site_data
	)

###################################################################################################
#	display time
trans_end = time.time()
trans_time = np.round((trans_end - trans_start), 1)

print('\n		Transformations complete.')
print('\n			- Time taken = {} s.'.format(trans_time))

###################################################################################################
#	minimise sun angle
###################################################################################################

print('\nMinimising Solar Panel Sun Angle...')

#	convert to radians
rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad_temp = np.radians(rot_vec_inc_deg)

#	generate initial t = 0 attitude
init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])

#	generate initial attitude rotation matrix
init_attit_mat_temp = rot_mat(init_attit_temp)

#	rotate geometry axes into orbital frame
solar_panel_norm_orb = np.dot(solar_panel_norm_body, init_attit_mat_temp)

#	sun vector
sun_orb_vec_temp = sun_orb[0, 1:4]

attitude_angle_range = np.arange(0, 360, 1)
angle_data = np.zeros([len(attitude_angle_range)])

for i, rot_vec_azm_deg_temp in enumerate(attitude_angle_range):

	#	set new azimuth angle
	rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg_temp)
	
	#	generate new attitude state
	init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])

	#	generate new attitude matrix
	init_attit_mat_temp = rot_mat(init_attit_temp)

	#	generate new solar panel norm in orbital frame
	solar_panel_norm_orb_temp = np.dot(solar_panel_norm_body, init_attit_mat_temp)

	#	calculate angle to sun vector
	#	sun angle
	minimisation_angle = np.arccos(
		np.dot(solar_panel_norm_orb_temp, sun_orb_vec_temp)
		/ (np.linalg.norm(solar_panel_norm_orb_temp) *  np.linalg.norm(sun_orb_vec_temp))
	)

	angle_data[i] = minimisation_angle

#	find minimum angle
min_ang = angle_data.min()
min_index = np.where(angle_data==min_ang)[0][0]

azimuth_angle = attitude_angle_range[min_index]

#	set actual azimuth angle
rot_vec_azm_deg = azimuth_angle

print('\nSetting Azimuth Angle to: ', azimuth_angle)

###################################################################################################
#	define initial attitude state
###################################################################################################

##################################################
#	attitude

#	convert to radians
rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad = np.radians(rot_vec_inc_deg)

#	generate initial t = 0 attitude
init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])

##################################################
#	attitude motion in body frame

#	generate rotation angle in radians
if rot_period != 0.0:
	rot_ang_per_sec = (np.pi * 2) / rot_period
else:
	rot_ang_per_sec = 0.0

#	define rotation axis in body frame as unit vector
rot_vec_body_unit = np.array([1.0, 0.0, 0.0])

#	generate attitude matrix in body frame
attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)

#	extract euler angles
angles1 = np.array(mat2angles(attit_mat_body)[0:3])
angles2 = np.array(mat2angles(attit_mat_body)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

#	extract attitude motion in the body frame
attit_mot_body = angles1

##################################################
#	generate initial attitude rotation matrix
init_attit_mat = rot_mat(init_attit)

##################################################
#	calculate rotation vector in orbital frame as unit vector
rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)

##################################################
#	define velocity of rotation vector in orbital frame

#	when equal to orbital motion, rot vec is stationary in inertial frame
rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

###############################################################################################
#	generate brdf

#	Refractive Index
n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))

#	Specular ratio
s_brdf = 1.0 - d_brdf

#	generate brdf
brdf_data_pl = np.zeros([nfaces_pl, 5])
brdf_data_sp = np.zeros([nfaces_sp, 5])

for i in range(nfaces_pl):
	brdf_data_pl[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]

for i in range(nfaces_sp):
	brdf_data_sp[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]

###################################################################################################
#	perform light curve vector generation
###################################################################################################

#	blank observation and illumination vectors in orbital frame
obs_vec_orb = np.array([1.0, 0.0, 0.0])
illum_vec_orb = np.array([1.0, 0.0, 0.0])

#	generate orb2inertial velocity vector
orb2inertial = np.array([0.0, 0.0, -orbital_mot])

#	vector generation function
vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
	path_data, time_step, len_record,
	sat_orb, sun_orb, earth_orb, site_orb,
	orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	obs_vec_orb, illum_vec_orb
)

##################################################
#	parse vector data

#	extract orbital positions of custom input vectors
illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_body[:, 31:34]

#	body data
sun_body_data = vector_data_body[:, 28:31]
site_body_data = vector_data_body[:, 31:34]
earth_body_data = vector_data_body[:, 19:22]
sat_body_vel = vector_data_body[:, 22:25]
sat_body_pos = np.zeros([len(sat_body_vel), 3])
sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
illum_vec_body_data = vector_data_body[:, 28:31]
obs_vec_body_data = vector_data_body[:, 31:34]

#	orb data
sun_orb_data = vector_data_orb[:, 28:31]
site_orb_data = vector_data_orb[:, 31:34]
earth_orb_data = vector_data_orb[:, 19:22]
sat_orb_vel = vector_data_orb[:, 22:25]
sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
illum_vec_orb_data = vector_data_orb[:, 28:31]
obs_vec_orb_data = vector_data_orb[:, 31:34]

###############################################################################################
#	validation
###############################################################################################

sim_phase_data = []
sim_distance_data = []

for (sun_body_vec, site_body_vec) in zip(sun_body_data, site_body_data):

	sim_phase_ang = np.degrees(vec_ang(sun_body_vec, site_body_vec))
	sim_phase_data.append(sim_phase_ang)

	sim_distance_data.append(np.linalg.norm(site_body_vec))



sim_phase_data = np.array(sim_phase_data)

fig_phase, ax_phase = plt.subplots()
ax_phase.set_title('Phase Validation')
ax_phase.plot(sim_phase_data)
ax_phase.plot(mmt_phase_data)

fig_dist, ax_dist = plt.subplots()
ax_dist.set_title('Distance Validation')
ax_dist.plot(sim_distance_data)
ax_dist.plot(mmt_distance_data)

###############################################################################################
#	generate light curves
###############################################################################################

print('\n[{}]	Generating Light Curves...'.format(datetime.datetime.now()))

###############################################################################################
#	initialise light curve generation

#	generate sim time
sim_time = np.linspace(0, duration_sec, len_record)

#	initilise lightcurve output
light_curve_array = np.zeros((lc_len_record, 4))
light_curve_array[:, 1] = sim_time

light_curve_array_pl_noshad = np.copy(light_curve_array)
light_curve_array_sp_noshad = np.copy(light_curve_array)
light_curve_array_sp_shad = np.copy(light_curve_array)

jdate_start = start_jd[0] + start_jd[1]

###############################################################################################
#	generate payload no shadowing light curve

#	run light curve generation function
light_curve_array_pl_noshad = glim_slc_new(
	sun_body_data, site_body_data, light_curve_array_pl_noshad,
	jdate_start, body_vectors_pl, body_areas_pl, brdf_data_pl
)

lc_seconds = light_curve_array_pl_noshad[:, 1]
lc_app_pl_noshad = light_curve_array_pl_noshad[:, 2]
lc_std_pl_noshad = light_curve_array_pl_noshad[:, 3]

###############################################################################################
#	generate solar panel no shadowing light curve

#	run light curve generation function
light_curve_array_sp_noshad = glim_slc_new(
	sun_body_data, site_body_data, light_curve_array_sp_noshad,
	jdate_start, body_vectors_sp, body_areas_sp, brdf_data_sp
)

lc_app_sp_noshad = light_curve_array_sp_noshad[:, 2]
lc_std_sp_noshad = light_curve_array_sp_noshad[:, 3]

###############################################################################################
#	generate solar panel with shadowing light curve

slc_log_level = 0

#	run light curve generation function
light_curve_array_sp_shad = glim_slc_shadowing(
	sun_body_data, site_body_data, sun_orb_data, site_orb_data,
	light_curve_array_sp_shad,
	jdate_start, time_step,
	body_mesh_sp,
	init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
	brdf_data_sp,
	slc_log_level
)

lc_seconds = light_curve_array_sp_shad[:, 1]
lc_app_sp_shad = light_curve_array_sp_shad[:, 2]
lc_std_sp_shad = light_curve_array_sp_shad[:, 3]

###################################################################################################
#	final figure generation
###################################################################################################

print('\n#####################################################################################')
print('Generating Final Plots...')

#	set use of LaTeX font
plt.rc('text', usetex=True)

#	set figure height/width
fig_width = 12.0/inch2cm
fig_height = 6.0/inch2cm

###################################################################################################
#	plot lightcurve

#	initialise figure axes
fig, ax = plt.subplots(figsize=(fig_width, fig_height), dpi=300)
plt.gcf().subplots_adjust(bottom=0.15)

ax.plot(lc_seconds, lc_app_pl_noshad, label='Synthetic; Payload; No-Shadowing')
ax.plot(lc_seconds, lc_app_sp_noshad, label='Synthetic; Solar Panel; No-Shadowing')
ax.plot(lc_seconds, lc_app_sp_shad, label='Synthetic; Solar Panel; Shadowing')
ax.plot(lc_seconds, mmt_signal_app, label='Real MMT Data')

#	finalise plot
# ax.set_ylim([10.0, 0.0])
ax.invert_yaxis()

ax.set_xlabel(r'$Time/s$', fontsize=ylabel_fontsize)
ax.set_ylabel(r'$App\ Mag$', fontsize=xlabel_fontsize)
ax.tick_params(axis='x', labelsize=xticks_fontsize, pad=xticks_pad)
ax.tick_params(axis='y', labelsize=yticks_fontsize, pad=yticks_pad)

ax.legend(
	loc='lower center',
	ncol=7,
	fontsize=legend_fontsize,
	frameon=False
)

print('\nAverage Mag. = ', np.average(lc_app_sp_shad))

###################################################################################################
#	generate 3D plot







plt.show()
