###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	geometry file plotter
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time

#	numpy
import numpy as np

#	numpy-stl
from stl import mesh

from tools.functions.functions_transform import rot_mat, rot1

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors
from matplotlib.patches import FancyArrowPatch

###################################################################################################
#	define plotable arrow class
###################################################################################################

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Articles',
	'starlink_paper',
	'Figures'
)

fig_name = 'starlink_obs_vec_figure.eps'
out_path = os.path.join(fig_path, fig_name)

#	set flag
savefile_flag = False
savefile_flag = True

###################################################################################################
#	figure parameters

#	graphical output
inch2cm = 2.54

#	face plot parameters
alpha = 1.0
facecol = [0.1, 0.1, 0.1]
edgecol = [0.5, 0.5, 0.5]

#	set font sizes
title_fontsize = 6
text_fontsize = 4
cbar_fontsize = 4
legend_fontsize = 4

xlabel_fontsize = 6
ylabel_fontsize = 6

xticks_fontsize = 4
yticks_fontsize = 4

#	set text padding values
title_pad = 2
xticks_pad = 4
yticks_pad = 0

#	set marker parameters
marker_size = 3
marker_edge = 3

#	set figure line wdith
plot_linewidth = 1.0

#	set subfigure spacing
hspace_val = 0.35
wspace_val = 0.35

#	set axis lims
axis_limx = 8
axis_limy = 8
axis_limz = 8

#	set use of latex font
plt.rc('text', usetex=True)

#	initialise figure axis
fig, ax = plt.subplots(
	figsize=(9/inch2cm, 9/inch2cm),
	dpi=300,
	subplot_kw=dict(projection='3d'),
)

###################################################################################################
#	geometry file
###################################################################################################

#	geometry file
geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_str = 'starlink_full'
geom_file = geom_str + '.stl'

#	geom offset
geom_offset_x = 90.0
geom_offset_y = 90.0

#	attitude
rot_vec_azm_deg = 0.0
rot_vec_inc_deg = 0.0

#	attitude motion
rot_period = 0.0

#	load static mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

#	transform mesh into body frame
object_mesh.rotate_using_matrix(offset_rot_mat)

#	set vectors and areas
vectors = object_mesh.vectors
normals = object_mesh.normals
areas = object_mesh.areas

nfaces = len(areas)

solar_panel_norm_body = np.array([0.0, 0.0, -1.0])

###################################################################################################
#	output figure
###################################################################################################

#	sun vector
sun_azm = 320
sun_inc = 72

sun_vec = np.array([-1.0, 0.0, 0.0])

#	convert to rotation matrix
#	attitude matrix
obs_attit = rot_mat(np.radians(np.array([sun_azm, -sun_inc, 0.0])))

#	generate new observation unit vector
sun_vec_new = np.dot(sun_vec, obs_attit)

###################################################################################################
#	minimise sun angle

print('\n		Minimising Solar Panel Sun Angle...')

#	convert to radians
rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad_temp = np.radians(rot_vec_inc_deg)

#	generate initial t = 0 attitude
init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])

#	generate initial attitude rotation matrix
init_attit_mat_temp = rot_mat(init_attit_temp)

#	rotate geometry axes into orbital frame
solar_panel_norm_orb = np.dot(solar_panel_norm_body, init_attit_mat_temp)

#	sun vector
sun_orb_vec_temp = np.copy(sun_vec_new)

attitude_angle_range = np.arange(0, 360, 1)
angle_data = np.zeros([len(attitude_angle_range)])

for i, rot_vec_azm_deg_temp in enumerate(attitude_angle_range):

	#	set new azimuth angle
	rot_vec_azm_rad_temp = np.radians(rot_vec_azm_deg_temp)
	
	#	generate new attitude state
	init_attit_temp = np.array([rot_vec_azm_rad_temp, rot_vec_inc_rad_temp, 0.0])

	#	generate new attitude matrix
	init_attit_mat_temp = rot_mat(init_attit_temp)

	#	generate new solar panel norm in orbital frame
	solar_panel_norm_orb_temp = np.dot(solar_panel_norm_body, init_attit_mat_temp)

	#	calculate angle to sun vector
	#	sun angle
	minimisation_angle = np.arccos(
		np.dot(solar_panel_norm_orb_temp, sun_orb_vec_temp)
		/ (np.linalg.norm(solar_panel_norm_orb_temp) *  np.linalg.norm(sun_orb_vec_temp))
	)

	angle_data[i] = minimisation_angle

#	find minimum angle
min_ang = angle_data.min()
min_index = np.where(angle_data==min_ang)[0][0]

azimuth_angle = attitude_angle_range[min_index]

#	set actual azimuth angle
rot_vec_azm_deg = azimuth_angle
print('\n			Setting Azimuth Angle to: ', azimuth_angle)

###################################################################################################
#	generate orbital mesh

#	convert to radians
rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
rot_vec_inc_rad = np.radians(rot_vec_inc_deg)

#	generate initial t = 0 attitude
init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
init_attit_mat = rot_mat(init_attit)

#	generate orbital mesh from the body mesh
orb_mesh = mesh.Mesh(object_mesh.data.copy())

#	rotate into orbital frame
orb_mesh.rotate_using_matrix(init_attit_mat)

#	set vectors and areas
orb_vectors = orb_mesh.vectors
orb_normals = orb_mesh.normals
orb_areas = orb_mesh.areas

###################################################################################################
#	generate circle data

#	define radius
radius0 = 10

radius_data = [
	radius0,
	0.8 * radius0,
	0.6 * radius0,
	0.4 * radius0,
	0.2 * radius0
]

x_offset = radius0 / 2.747
ratio = radius0 / x_offset
half_ang = np.arctan(ratio)

circle_data = []

#	generate circle data
n = 360

for itt, radius in enumerate(radius_data):

	circle_data = []

	if itt == 0:
		col_str = 'k'
		line_style='-'
	else:
		line_style='-'
		col_str='tab:grey'

	for i in range(0, n + 1):
	
		x = 0
		y = np.sin(2 * np.pi / n * i)
		z = np.cos(2 * np.pi / n * i)

		circle_data.append(np.array([x * radius, y * radius, z * radius]))
	
	circle_data = np.array(circle_data)

	ax.plot(
		circle_data[:, 0] - x_offset,
		circle_data[:, 1],
		circle_data[:, 2],
		lw=0.5,
		color=col_str,
		zorder=0,
		linestyle=line_style
	)


radius = 69 * (radius0 / 70)


circle_data = []
for i in range(0, n + 1):

	x = 0
	y = np.sin(2 * np.pi / n * i)
	z = np.cos(2 * np.pi / n * i)

	circle_data.append(np.array([x * radius, y * radius, z * radius]))

circle_data = np.array(circle_data)

ax.plot(
	circle_data[:, 0] - x_offset,
	circle_data[:, 1],
	circle_data[:, 2],
	lw=1.0,
	color='g',
	zorder=0,
	linestyle='-.'
)




#	polar origin
new_origin = np.array([-x_offset, 0.0, 0.0])

vec1_start = np.array([-x_offset, -radius0, 0.0])
vec1_end = np.array([-x_offset, radius0, 0.0])
vec1_plot = np.vstack((vec1_start, vec1_end))
ax.plot(vec1_plot[:, 0], vec1_plot[:, 1], vec1_plot[:, 2], lw=0.5, color='tab:grey', zorder=0)

vec2_start = np.array([-x_offset, 0.0, -radius0])
vec2_end = np.array([-x_offset, 0.0, radius0])
vec2_plot = np.vstack((vec2_start, vec2_end))
ax.plot(vec2_plot[:, 0], vec2_plot[:, 1], vec2_plot[:, 2], lw=0.5, color='tab:grey', zorder=0)

vec3_start = np.array([-x_offset, -radius0, 0.0])
vec3_end = np.array([-x_offset, radius0, 0.0])
vec3_start = rot1(vec3_start, np.radians(45))
vec3_end = rot1(vec3_end, np.radians(45))
vec3_plot = np.vstack((vec3_start, vec3_end))
ax.plot(vec3_plot[:, 0], vec3_plot[:, 1], vec3_plot[:, 2], lw=0.5, color='tab:grey', zorder=0)

vec4_start = np.array([-x_offset, 0.0, -radius0])
vec4_end = np.array([-x_offset, 0.0, radius0])
vec4_start = rot1(vec4_start, np.radians(45))
vec4_end = rot1(vec4_end, np.radians(45))
vec4_plot = np.vstack((vec4_start, vec4_end))
ax.plot(vec4_plot[:, 0], vec4_plot[:, 1], vec4_plot[:, 2], lw=0.5, color='tab:grey', zorder=0)

###################################################################################################
#	plot polar plot angle text

#	define range of labels
angles = [0, 45, 90, 135, 180, 225, 270, 315]
offsets1 = [-2, -1, 0, 0, 0, 0, 0, 0]
offsets2 = [-0.2, 0, 0, 0, 0, 0, 0, 0]

vec_origin = np.array([-x_offset, 0.0, 0.0])
init_vec_pos = np.array([-x_offset, 0.0, radius0 * 1.1])
vec_plot = np.vstack((vec_origin, init_vec_pos))

for angle, offset1, offset2 in zip(angles, offsets1, offsets2):

	#	apply angle
	vec_pos = rot1(init_vec_pos, np.radians(angle))
	vec_plot = np.vstack((vec_origin, vec_pos))

	text = r'$' + str(angle) + '^{\circ}$'
	ax.text(vec_plot[1, 0], vec_plot[1, 1] - offset2, vec_plot[1, 2] - offset1,
		text,
		fontsize=text_fontsize,
		zorder=0
	)

###################################################################################################
#	plot geometry

#	facet loop
for i, (vector, normal) in enumerate(zip(orb_vectors, orb_normals)):

	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]
	normal = np.cross(vect12, vect23)
	normal_unit = normal / np.linalg.norm(normal)

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_alpha(alpha)

	pc.set_facecolor('tab:grey')
	pc.set_edgecolor('k')

	ax.add_collection3d(pc)

###################################################################################################
#	plot body axes

body_axis_scale = 15
linewidth_val = 1

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

bodyx_vec_plot = body_axis_scale * np.vstack(([0, 0, 0], x_body_unit))
bodyy_vec_plot = body_axis_scale * np.vstack(([0, 0, 0], y_body_unit))
bodyz_vec_plot = body_axis_scale * np.vstack(([0, 0, 0], z_body_unit))

#	body x axis
c = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
	mutation_scale=5, 
	lw=linewidth_val,
	arrowstyle="-|>",
	color="k"
)
ax.add_artist(c)

#	body y axis
d = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
	mutation_scale=5, 
	lw=linewidth_val,
	arrowstyle="-|>",
	color="k"
)
ax.add_artist(d)

#	body z axis
e = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
	mutation_scale=5,
	lw=linewidth_val,
	arrowstyle="-|>",
	color="k"
)
ax.add_artist(e)

###################################################################################################
#	plot body axes text

ax.text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1] - 0.1, bodyx_vec_plot[1, 2] + 1,
	r'$x_{b}$',
	fontsize=text_fontsize,
	fontweight='bold'
)

ax.text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	r'$y_{b}$',
	fontsize=text_fontsize,
	fontweight='bold'
)

ax.text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	r'$z_{b}$',
	fontsize=text_fontsize,
	fontweight='bold'
)

###################################################################################################
#	plot sun vector

vec_scale = 20

sun_vec_plot = vec_scale * np.vstack(([0, 0, 0], sun_vec_new))

#	body x axis
c = Arrow3D(sun_vec_plot[:, 0], sun_vec_plot[:, 1], sun_vec_plot[:, 2],
	mutation_scale=5, 
	lw=linewidth_val,
	arrowstyle="-|>",
	color="r"
)
ax.add_artist(c)

ax.text(sun_vec_plot[1, 0], sun_vec_plot[1, 1], sun_vec_plot[1, 2],
	r'$L$',
	fontsize=text_fontsize,
	fontweight='bold'
)

#############################################
#	finalise figure

#	finalise subplot 
# ax.set_xlim([-axis_limx, axis_limx])
ax.set_xlim([-4, 12])
ax.set_ylim([-axis_limy, axis_limy])
ax.set_zlim([-axis_limz, axis_limz])
ax.set_xlabel('x axis')
ax.set_ylabel('y axis')
ax.set_zlabel('z axis')

#	set viewing direction
ax.view_init(100, 180)

#	turn off axes
ax.axis('off')

###############################################################################################
#	save figure
###############################################################################################

if savefile_flag is True:

	fig_name = 'starlink_obsvec_geom.eps'
	out_path = os.path.join(fig_path, fig_name)
	fig.savefig(out_path, bbox_inches="tight", dpi=300)

	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
sys.exit()

