###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import random
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	light curve generation
from tools.functions.glim_slc import glim_slc_shadowing, glim_slc_new

#	functions list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, interp, \
	get_times

from tools.functions.functions_transform import site_vec, axis2mat, mat2angles, gen_vectors2, \
	rot_mat

#	spacetrack interface
from tools.functions.spacetrack_interface import req_tle

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	glimma modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_transformation

#	visualisation functions
from tools.functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm

import multiprocessing

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	Function: 		brdf fitting function
#	Description: 	
#					
#
###################################################################################################

def brdf_fitting(input_line):

	#	parse input line
	name_str, object_id, \
		rot_vec_azm_deg, rot_vec_inc_deg, \
		d_brdf, m_brdf, omega_brdf, \
		nfaces, duration_sec, len_record, start_jd,\
		sun_body_data, site_body_data, \
		body_vectors, body_areas = input_line

	nfaces = int(nfaces)
	len_record = int(len_record)

	###############################################################################################
	#	regenerate brdf
	
	#	Refractive Index
	n_brdf = (1 + np.sqrt(omega_brdf)) / (1 - np.sqrt(omega_brdf))
	
	#	Specular ratio
	s_brdf = 1.0 - d_brdf
	
	#	generate brdf
	brdf_data = np.zeros([nfaces, 5])
	
	for i in range(nfaces):
		brdf_data[i, :] = [m_brdf, omega_brdf, d_brdf, s_brdf, n_brdf]
	
	###############################################################################################
	#	generate light curve
	
	#	generate sim time
	sim_time = np.linspace(0, duration_sec, len_record)
	
	#	initilise lightcurve output
	final_light_curve = np.zeros((len_record, 4))
	final_light_curve[:, 1] = sim_time

	#	run light curve generation function
	light_curve = glim_slc_new(sun_body_data, site_body_data, final_light_curve, \
			start_jd, body_vectors, body_areas, brdf_data)
		
	light_curve_jd = light_curve[:, 0]
	light_curve_seconds = light_curve[:, 1]
	light_curve_data_app = light_curve[:, 2]
	light_curve_data_std = light_curve[:, 3]

	#	assemble total output array
	input_data = np.zeros([len(light_curve), 6])

	for i, line in enumerate(input_data):

		input_data[i] = [
			int(object_id),
			np.round(rot_vec_azm_deg, 2),
			np.round(rot_vec_inc_deg, 2),
			np.round(d_brdf, 2),
			np.round(m_brdf, 2),
			np.round(omega_brdf, 2)
		]

	light_curve_output = np.hstack([light_curve, input_data])

	return light_curve_output

###################################################################################################
#	Function: 		attitude fitting function
#	Description: 	
#					
#
###################################################################################################

def rocket_body_attit_fitting(input_line):

	#	parse input line
	path_data, name_str, object_id, \
		rot_vec_azm_deg, rot_vec_inc_deg, nfaces, \
		rot_period, orbital_mot, \
		top_faces_index, bot_faces_index, side_faces_index, \
		top_d_brdf, top_m_brdf, top_omega_brdf, \
		bot_d_brdf, bot_m_brdf, bot_omega_brdf, \
		side_d_brdf, side_m_brdf, side_omega_brdf, \
		duration_sec, time_step, len_record, start_jd, \
		sat_orb, sun_orb, earth_orb, site_orb, \
		body_vectors, body_areas = input_line

	nfaces = int(nfaces)
	len_record = int(len_record)

	##########################################################################################
	#	define initial attitude state
	##########################################################################################
	
	##################################################
	#	attitude
	
	#	convert to radians
	rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
	rot_vec_inc_rad = np.radians(rot_vec_inc_deg)
	
	#	generate initial t = 0 attitude
	init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
	
	##################################################
	#	attitude motion in body frame
	
	#	generate rotation angle in radians
	if rot_period != 0.0:
		rot_ang_per_sec = (np.pi * 2) / rot_period
	else:
		rot_ang_per_sec = 0.0
	
	#	define rotation axis in body frame as unit vector
	rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
	
	#	generate attitude matrix in body frame
	attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)
	
	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1
	
	##################################################
	#	generate initial attitude rotation matrix
	init_attit_mat = rot_mat(init_attit)
	
	##################################################
	#	calculate rotation vector in orbital frame as unit vector
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)
	
	##################################################
	#	define velocity of rotation vector in orbital frame
	
	#	when equal to orbital motion, rot vec is stationary in inertial frame
	rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
	rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

	######################################################################################
	#	generate brdf
	
	#	Refractive Index
	top_n_brdf = (1 + np.sqrt(top_omega_brdf)) / (1 - np.sqrt(top_omega_brdf))
	side_n_brdf = (1 + np.sqrt(side_omega_brdf)) / (1 - np.sqrt(side_omega_brdf))
	bot_n_brdf = (1 + np.sqrt(bot_omega_brdf)) / (1 - np.sqrt(bot_omega_brdf))
	
	#	Specular ratio
	top_s_brdf = 1.0 - top_d_brdf
	side_s_brdf = 1.0 - side_d_brdf
	bot_s_brdf = 1.0 - bot_d_brdf
	
	#	generate brdf
	brdf_data = np.zeros([nfaces, 5])
	
	for i in range(nfaces):
	
		#	working on a top face
		if i in top_faces_index:
			brdf_data[i, :] = [top_m_brdf, top_omega_brdf, top_d_brdf, top_s_brdf, top_n_brdf]
		
		#	working on a side face
		elif i in side_faces_index:
			brdf_data[i, :] = [side_m_brdf, side_omega_brdf, side_d_brdf, side_s_brdf, side_n_brdf]
	
		#	working on a bot face
		elif i in bot_faces_index:
			brdf_data[i, :] = [bot_m_brdf, bot_omega_brdf, bot_d_brdf, bot_s_brdf, bot_n_brdf]
	
		else:
			print('BRDF Generation Error, exiting.')
			sys.exit()
	
	##########################################################################################
	#	perform light curve vector generation
	##########################################################################################
	
	#	blank observation and illumination vectors in orbital frame
	obs_vec_orb = np.array([1.0, 0.0, 0.0])
	illum_vec_orb = np.array([1.0, 0.0, 0.0])
	
	#	generate orb2inertial velocity vector
	orb2inertial = np.array([0.0, 0.0, -orbital_mot])
	
	#	vector generation function
	vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb
	)
	
	##################################################
	#	parse vector data
	
	#	extract orbital positions of custom input vectors
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_body[:, 31:34]
	
	#	body data
	sun_body_data = vector_data_body[:, 28:31]
	site_body_data = vector_data_body[:, 31:34]
	earth_body_data = vector_data_body[:, 19:22]
	sat_body_vel = vector_data_body[:, 22:25]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	illum_vec_body_data = vector_data_body[:, 28:31]
	obs_vec_body_data = vector_data_body[:, 31:34]
	
	#	orb data
	sun_orb_data = vector_data_orb[:, 28:31]
	site_orb_data = vector_data_orb[:, 31:34]
	earth_orb_data = vector_data_orb[:, 19:22]
	sat_orb_vel = vector_data_orb[:, 22:25]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_orb[:, 31:34]
	
	###############################################################################################
	#	generate light curve
	
	#	generate sim time
	sim_time = np.linspace(0, duration_sec, len_record)
	
	#	initilise lightcurve output
	final_light_curve = np.zeros((len_record, 4))
	final_light_curve[:, 1] = sim_time

	#	run light curve generation function
	light_curve = glim_slc_new(sun_body_data, site_body_data, final_light_curve, \
			start_jd, body_vectors, body_areas, brdf_data)
		
	light_curve_jd = light_curve[:, 0]
	light_curve_seconds = light_curve[:, 1]
	light_curve_data_app = light_curve[:, 2]
	light_curve_data_std = light_curve[:, 3]

	#	assemble total output array
	input_data = np.zeros([len(light_curve), 12])

	for i, line in enumerate(input_data):

		input_data[i] = [
			int(object_id),
			np.round(rot_vec_azm_deg, 2),
			np.round(rot_vec_inc_deg, 2),
			np.round(top_d_brdf, 2), np.round(top_m_brdf, 2), np.round(top_omega_brdf, 2),
			np.round(bot_d_brdf, 2), np.round(bot_m_brdf, 2), np.round(bot_omega_brdf, 2),
			np.round(side_d_brdf, 2), np.round(side_m_brdf, 2), np.round(side_omega_brdf, 2)
		]

	light_curve_output = np.hstack([light_curve, input_data])

	return light_curve_output

###################################################################################################
#	Function: 		brdf fitting function
#	Description: 	
#					
#
###################################################################################################

def rocket_body_brdf_fitting(input_line):

	#	parse input line
	name_str, object_id, \
		rot_vec_azm_deg, rot_vec_inc_deg, nfaces, full_brdf_flag,\
		top_faces_index, bot_faces_index, side_faces_index, \
		top_d_brdf, top_m_brdf, top_omega_brdf, \
		bot_d_brdf, bot_m_brdf, bot_omega_brdf, \
		side_d_brdf, side_m_brdf, side_omega_brdf, \
		duration_sec, len_record, start_jd, \
		sun_body_data, site_body_data, \
		body_vectors, body_areas = input_line

	top_d_brdf, top_m_brdf, top_omega_brdf = [0.7, 0.7, 0.2]
	bot_d_brdf, bot_m_brdf, bot_omega_brdf = [0.7, 0.7, 0.2]
	side_d_brdf, side_m_brdf, side_omega_brdf = [0.7, 0.7, 0.2]

	nfaces = int(nfaces)
	len_record = int(len_record)

	###############################################################################################
	#	regenerate brdf

	#	Refractive Index
	top_n_brdf = (1 + np.sqrt(top_omega_brdf)) / (1 - np.sqrt(top_omega_brdf))
	side_n_brdf = (1 + np.sqrt(side_omega_brdf)) / (1 - np.sqrt(side_omega_brdf))
	bot_n_brdf = (1 + np.sqrt(bot_omega_brdf)) / (1 - np.sqrt(bot_omega_brdf))
	
	#	Specular ratio
	top_s_brdf = 1.0 - top_d_brdf
	side_s_brdf = 1.0 - side_d_brdf
	bot_s_brdf = 1.0 - bot_d_brdf

	#	generate brdf
	brdf_data = np.zeros([nfaces, 5])
	
	for i in range(nfaces):
	
		#	working on a top face
		if i in top_faces_index:
			brdf_data[i, :] = [top_m_brdf, top_omega_brdf, top_d_brdf, top_s_brdf, top_n_brdf]
		
		#	working on a side face
		elif i in side_faces_index:
			brdf_data[i, :] = [side_m_brdf, side_omega_brdf, side_d_brdf, side_s_brdf, side_n_brdf]
	
		#	working on a bot face
		elif i in bot_faces_index:
			brdf_data[i, :] = [bot_m_brdf, bot_omega_brdf, bot_d_brdf, bot_s_brdf, bot_n_brdf]
	
		else:
			print('BRDF Generation Error, exiting.')
			sys.exit()

	###############################################################################################
	#	generate light curve
	
	#	generate sim time
	sim_time = np.linspace(0, duration_sec, len_record)
	
	#	initilise lightcurve output
	final_light_curve = np.zeros((len_record, 4))
	final_light_curve[:, 1] = sim_time

	#	run light curve generation function
	light_curve = glim_slc_new(sun_body_data, site_body_data, final_light_curve, \
			start_jd, body_vectors, body_areas, brdf_data)
		
	light_curve_jd = light_curve[:, 0]
	light_curve_seconds = light_curve[:, 1]
	light_curve_data_app = light_curve[:, 2]
	light_curve_data_std = light_curve[:, 3]

	#	assemble total output array
	input_data = np.zeros([len(light_curve), 12])

	for i, line in enumerate(input_data):

		input_data[i] = [
			int(object_id),
			np.round(rot_vec_azm_deg, 2),
			np.round(rot_vec_inc_deg, 2),
			np.round(top_d_brdf, 2), np.round(top_m_brdf, 2), np.round(top_omega_brdf, 2),
			np.round(bot_d_brdf, 2), np.round(bot_m_brdf, 2), np.round(bot_omega_brdf, 2),
			np.round(side_d_brdf, 2), np.round(side_m_brdf, 2), np.round(side_omega_brdf, 2)
		]

	light_curve_output = np.hstack([light_curve, input_data])

	return light_curve_output

###################################################################################################
#	Function: 		rocket body combined fitting function
#	Description: 	performs both attitude and brdf fitting simultaneously
#					
#
###################################################################################################

def rocket_body_combined_fitting(input_line):

	#	parse input line
	path_data, name_str, track_id, \
		rot_vec_azm_deg, rot_vec_inc_deg, \
		rot_period, orbital_mot, \
		top_faces_index, bot_faces_index, side_faces_index, \
		top_d_brdf, top_m_brdf, top_omega_brdf, \
		bot_d_brdf, bot_m_brdf, bot_omega_brdf, \
		side_d_brdf, side_m_brdf, side_omega_brdf, \
		duration_sec, time_step, len_record, start_jd, \
		sat_orb, sun_orb, earth_orb, site_orb, \
		body_vectors, body_areas = input_line

	nfaces = int(len(body_areas))
	len_record = int(len_record)

	##########################################################################################
	#	define initial attitude state
	##########################################################################################
	
	##################################################
	#	attitude
	
	#	convert to radians
	rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
	rot_vec_inc_rad = np.radians(rot_vec_inc_deg)
	
	#	generate initial t = 0 attitude
	init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
	
	##################################################
	#	attitude motion in body frame
	
	#	generate rotation angle in radians
	if rot_period != 0.0:
		rot_ang_per_sec = (np.pi * 2) / rot_period
	else:
		rot_ang_per_sec = 0.0
	
	#	define rotation axis in body frame as unit vector
	rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
	
	#	generate attitude matrix in body frame
	attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)
	
	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1
	
	##################################################
	#	generate initial attitude rotation matrix
	init_attit_mat = rot_mat(init_attit)
	
	##################################################
	#	calculate rotation vector in orbital frame as unit vector
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)
	
	##################################################
	#	define velocity of rotation vector in orbital frame
	
	#	when equal to orbital motion, rot vec is stationary in inertial frame
	rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
	rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])

	###############################################################################################
	#	regenerate brdf

	#	Refractive Index
	top_n_brdf = (1 + np.sqrt(top_omega_brdf)) / (1 - np.sqrt(top_omega_brdf))
	side_n_brdf = (1 + np.sqrt(side_omega_brdf)) / (1 - np.sqrt(side_omega_brdf))
	bot_n_brdf = (1 + np.sqrt(bot_omega_brdf)) / (1 - np.sqrt(bot_omega_brdf))
	
	#	Specular ratio
	top_s_brdf = 1.0 - top_d_brdf
	side_s_brdf = 1.0 - side_d_brdf
	bot_s_brdf = 1.0 - bot_d_brdf

	#	generate brdf
	brdf_data = np.zeros([nfaces, 5])
	
	for i in range(nfaces):
	
		#	working on a top face
		if i in top_faces_index:
			brdf_data[i, :] = [top_m_brdf, top_omega_brdf, top_d_brdf, top_s_brdf, top_n_brdf]
		
		#	working on a side face
		elif i in side_faces_index:
			brdf_data[i, :] = [side_m_brdf, side_omega_brdf, side_d_brdf, side_s_brdf, side_n_brdf]
	
		#	working on a bot face
		elif i in bot_faces_index:
			brdf_data[i, :] = [bot_m_brdf, bot_omega_brdf, bot_d_brdf, bot_s_brdf, bot_n_brdf]
	
		else:
			print('BRDF Generation Error, exiting.')
			sys.exit()

	##########################################################################################
	#	perform light curve vector generation
	##########################################################################################
	
	#	blank observation and illumination vectors in orbital frame
	obs_vec_orb = np.array([1.0, 0.0, 0.0])
	illum_vec_orb = np.array([1.0, 0.0, 0.0])
	
	#	generate orb2inertial velocity vector
	orb2inertial = np.array([0.0, 0.0, -orbital_mot])
	
	#	vector generation function
	vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb
	)
	
	##################################################
	#	parse vector data
	
	#	extract orbital positions of custom input vectors
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_body[:, 31:34]
	
	#	body data
	sun_body_data = vector_data_body[:, 28:31]
	site_body_data = vector_data_body[:, 31:34]
	earth_body_data = vector_data_body[:, 19:22]
	sat_body_vel = vector_data_body[:, 22:25]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	illum_vec_body_data = vector_data_body[:, 28:31]
	obs_vec_body_data = vector_data_body[:, 31:34]
	
	#	orb data
	sun_orb_data = vector_data_orb[:, 28:31]
	site_orb_data = vector_data_orb[:, 31:34]
	earth_orb_data = vector_data_orb[:, 19:22]
	sat_orb_vel = vector_data_orb[:, 22:25]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_orb[:, 31:34]
	
	###############################################################################################
	#	generate light curve
	
	#	generate sim time
	sim_time = np.linspace(0, duration_sec, len_record)
	
	#	initilise lightcurve output
	final_light_curve = np.zeros((len_record, 4))
	final_light_curve[:, 1] = sim_time

	#	run light curve generation function
	light_curve = glim_slc_new(sun_body_data, site_body_data, final_light_curve, \
			start_jd, body_vectors, body_areas, brdf_data)
		
	light_curve_jd = light_curve[:, 0]
	light_curve_seconds = light_curve[:, 1]
	light_curve_data_app = light_curve[:, 2]
	light_curve_data_std = light_curve[:, 3]

	#	assemble total output array
	input_data = np.zeros([len(light_curve), 12])

	for i, line in enumerate(input_data):

		input_data[i] = [
			int(track_id),
			np.round(rot_vec_azm_deg, 2),
			np.round(rot_vec_inc_deg, 2),
			np.round(top_d_brdf, 2), np.round(top_m_brdf, 2), np.round(top_omega_brdf, 2),
			np.round(bot_d_brdf, 2), np.round(bot_m_brdf, 2), np.round(bot_omega_brdf, 2),
			np.round(side_d_brdf, 2), np.round(side_m_brdf, 2), np.round(side_omega_brdf, 2)
		]

	light_curve_output = np.hstack([light_curve, input_data])

#	lc_index, jdate, sec_since, mag_app, mag_std, \
#		object_id, rot_vec_azm_deg, rot_vec_inc_deg, \
#		top_d_brdf, top_m_brdf, top_omega_brdf, \
#		bot_d_brdf, bot_m_brdf, bot_omega_brdf, \
#		side_d_brdf, side_m_brdf, side_omega_brdf, \
#		= light_curve_data[0]

	return light_curve_output

###################################################################################################
#	Function: 		glim_slc_shadow
#	Description: 	generates light curve using provided sun and site positions and 
#					geometry properties. Includes self shadowing
#
###################################################################################################


def obs_vec_brightness(input_line):

	#	parse input line
	path_data, start_jd, len_record, time_step, duration_sec, \
		inc_val, azm_val, site_range_vec, orbital_mot, sat_orb, sun_orb, earth_orb, site_orb, \
		init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb, body_mesh, brdf_data \
			= input_line

	#	convert to rotation matrix
	#	attitude matrix
	obs_attit = rot_mat(np.radians(np.array([azm_val, -inc_val, 0.0])))

	#	generate new observation unit vector
	site_vec_new = np.dot(site_range_vec, obs_attit)

	###############################################################################################
	#	perform light curve vector generation
	###############################################################################################
	
	#	blank observation and illumination vectors in orbital frame
	obs_vec_orb = np.copy(site_vec_new)
	illum_vec_orb = np.array([1.0, 0.0, 0.0])
	
	#	generate orb2inertial velocity vector
	orb2inertial = np.array([0.0, 0.0, -orbital_mot])

	#	vector generation function
	vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb
	)

	##################################################
	#	parse vector data
	
	#	body data
	sun_body_data = vector_data_body[:, 28:31]
	site_body_data = vector_data_body[:, 31:34]
	earth_body_data = vector_data_body[:, 19:22]
	sat_body_vel = vector_data_body[:, 22:25]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	illum_vec_body_data = vector_data_body[:, 28:31]
	obs_vec_body_data = vector_data_body[:, 31:34]
	
	#	orb data
	sun_orb_data = vector_data_orb[:, 28:31]
	site_orb_data = vector_data_orb[:, 31:34]
	earth_orb_data = vector_data_orb[:, 19:22]
	sat_orb_vel = vector_data_orb[:, 22:25]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_orb[:, 31:34]
	
	#	extract orbital positions of custom input vectors
	illum_vec_orb_data = vector_data_orb[:, 37:40]
	obs_vec_orb_data = vector_data_orb[:, 40:43]
	
	illum_vec_body_data = vector_data_body[:, 37:40]
	obs_vec_body_data = vector_data_body[:, 40:43]

	###############################################################################################
	#	generate light curve
	###############################################################################################
	
	#	generate sim time
	sim_time = np.linspace(0, duration_sec, len_record)
	
	#	initilise lightcurve output
	final_light_curve = np.zeros((len_record, 4))
	final_light_curve[:, 1] = sim_time
	jdate_start = start_jd[0] + start_jd[1]
	
	#	run light curve generation function
	slc_log_level = 0

	light_curve = glim_slc_shadowing(
		sun_body_data, obs_vec_body_data, sun_orb_data, obs_vec_orb_data,
#		sun_body_data, site_body_data, sun_orb_data, site_orb_data,
		final_light_curve,
		jdate_start, time_step,
		body_mesh,
		init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		brdf_data,
		slc_log_level
	)

	light_curve_jd = light_curve[:, 0]
	light_curve_seconds = light_curve[:, 1]
	light_curve_data_app = light_curve[:, 2]
	light_curve_data_std = light_curve[:, 3]

	average_mag = np.average(light_curve_data_app)
	min_mag = light_curve_data_app.min()
	max_mag = light_curve_data_app.max()

	output = np.hstack([[inc_val, azm_val, average_mag, min_mag, max_mag], site_vec_new])

	return output

###################################################################################################
#	Function: 		glim_slc_shadow
#	Description: 	generates light curve using provided sun and site positions and 
#					geometry properties. Includes self shadowing
#
###################################################################################################

def attit_brightness(input_line):

	#	parse input line
	path_data, start_jd, len_record, time_step, duration_sec, \
		attit_inc_val, attit_azm_val, orbital_mot, \
		sat_orb, sun_orb, earth_orb, site_orb, \
		rot_period, body_mesh, brdf_data,\
		= input_line

#	attit_inc_val, attit_azm_val = [80, 10]
#	print('\ninc={} azm={}	'.format(attit_inc_val, attit_azm_val), multiprocessing.current_process())

	###############################################################################################
	#	define initial attitude state
	###############################################################################################
	
	##################################################
	#	attitude
	
	#	convert to radians
	rot_vec_azm_rad = np.radians(attit_azm_val)
	rot_vec_inc_rad = np.radians(attit_inc_val)
	
	#	generate initial t = 0 attitude
	init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
	
	##################################################
	#	attitude motion in body frame
	
	#	generate rotation angle in radians
	if rot_period != 0.0:
		rot_ang_per_sec = (np.pi * 2) / rot_period
	else:
		rot_ang_per_sec = 0.0
	
	#	define rotation axis in body frame as unit vector
	rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
	
	#	generate attitude matrix in body frame
	attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)
	
	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1
	
	##################################################
	#	generate initial attitude rotation matrix
	init_attit_mat = rot_mat(init_attit)
	
	##################################################
	#	calculate rotation vector in orbital frame as unit vector
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)
	
	##################################################
	#	define velocity of rotation vector in orbital frame
	
	#	when equal to orbital motion, rot vec is stationary in inertial frame
	rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
	rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])
	
	###############################################################################################
	#	perform light curve vector generation
	###############################################################################################
	
	#	blank observation and illumination vectors in orbital frame
	obs_vec_orb = np.array([1.0, 0.0, 0.0])
	illum_vec_orb = np.array([1.0, 0.0, 0.0])
	
	#	generate orb2inertial velocity vector
	orb2inertial = np.array([0.0, 0.0, -orbital_mot])

	#	vector generation function
	vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb
	)

	##################################################
	#	parse vector data
	
	#	body data
	sun_body_data = vector_data_body[:, 28:31]
	site_body_data = vector_data_body[:, 31:34]
	earth_body_data = vector_data_body[:, 19:22]
	sat_body_vel = vector_data_body[:, 22:25]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	illum_vec_body_data = vector_data_body[:, 28:31]
	obs_vec_body_data = vector_data_body[:, 31:34]
	
	#	orb data
	sun_orb_data = vector_data_orb[:, 28:31]
	site_orb_data = vector_data_orb[:, 31:34]
	earth_orb_data = vector_data_orb[:, 19:22]
	sat_orb_vel = vector_data_orb[:, 22:25]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_orb[:, 31:34]
	
	#	extract orbital positions of custom input vectors
	illum_vec_orb_data = vector_data_orb[:, 37:40]
	obs_vec_orb_data = vector_data_orb[:, 40:43]
	
	illum_vec_body_data = vector_data_body[:, 37:40]
	obs_vec_body_data = vector_data_body[:, 40:43]

	###############################################################################################
	#	generate light curve
	###############################################################################################
	
	#	generate sim time
	sim_time = np.linspace(0, duration_sec, len_record)
	
	#	initilise lightcurve output
	final_light_curve = np.zeros((len_record, 4))
	final_light_curve[:, 1] = sim_time
	jdate_start = start_jd[0] + start_jd[1]

	#	run light curve generation function
	slc_log_level = 0
	light_curve = glim_slc_shadowing(
		sun_body_data, site_body_data, sun_orb_data, site_orb_data,
		final_light_curve,
		jdate_start, time_step,
		body_mesh,
		init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		brdf_data,
		slc_log_level
	)
	
	light_curve_jd = light_curve[:, 0]
	light_curve_seconds = light_curve[:, 1]
	light_curve_data_app = light_curve[:, 2]
	light_curve_data_std = light_curve[:, 3]

	average_mag = np.average(light_curve_data_app)
	min_mag = light_curve_data_app.min()
	max_mag = light_curve_data_app.max()

#	print('\n\nAvg. Mag. = {}.	'.format(average_mag), multiprocessing.current_process())
#	sys.exit()

	return [attit_inc_val, attit_azm_val, average_mag, min_mag, max_mag]

###################################################################################################
#	Function: 		glim_slc_shadow
#	Description: 	generates light curve using provided sun and site positions and 
#					geometry properties. Includes self shadowing
#
###################################################################################################


def inc_azm_brightness(input_line):

	#	parse input line
	path_data, \
		start_jd, len_record, time_step, duration_sec, \
		orbital_mot, sat_orb, sun_orb, earth_orb, site_orb, \
		rot_vec_inc_deg, rot_vec_azm_deg, rot_period,\
		body_mesh, brdf_data \
			= input_line

	print('\ninc={} azm={}	'.format(rot_vec_inc_deg, rot_vec_azm_deg), multiprocessing.current_process())

	###############################################################################################
	#	define initial attitude state
	###############################################################################################
	
	##################################################
	#	attitude
	
	#	convert to radians
	rot_vec_azm_rad = np.radians(rot_vec_azm_deg)
	rot_vec_inc_rad = np.radians(rot_vec_inc_deg)
	
	#	generate initial t = 0 attitude
	init_attit = np.array([rot_vec_azm_rad, rot_vec_inc_rad, 0.0])
	
	##################################################
	#	attitude motion in body frame
	
	#	generate rotation angle in radians
	if rot_period != 0.0:
		rot_ang_per_sec = (np.pi * 2) / rot_period
	else:
		rot_ang_per_sec = 0.0
	
	#	define rotation axis in body frame as unit vector
	rot_vec_body_unit = np.array([1.0, 0.0, 0.0])
	
	#	generate attitude matrix in body frame
	attit_mat_body = axis2mat(np.radians(rot_ang_per_sec), rot_vec_body_unit)
	
	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1
	
	##################################################
	#	generate initial attitude rotation matrix
	init_attit_mat = rot_mat(init_attit)
	
	##################################################
	#	calculate rotation vector in orbital frame as unit vector
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)
	
	##################################################
	#	define velocity of rotation vector in orbital frame
	
	#	when equal to orbital motion, rot vec is stationary in inertial frame
	rot_vec_vel_orb = np.array([0.0, 0.0, orbital_mot])
	rot_vec_vel_orb = np.array([0.0, 0.0, 0.0])
	
	###############################################################################################
	#	perform light curve vector generation
	###############################################################################################
	
	#	blank observation and illumination vectors in orbital frame
	obs_vec_orb = np.array([1.0, 0.0, 0.0])
	illum_vec_orb = np.array([1.0, 0.0, 0.0])
	
	#	generate orb2inertial velocity vector
	orb2inertial = np.array([0.0, 0.0, -orbital_mot])

	#	vector generation function
	vector_data_body, vector_data_orb, attitude_out = gen_vectors2(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb
	)

	##################################################
	#	parse vector data
	
	#	body data
	sun_body_data = vector_data_body[:, 28:31]
	site_body_data = vector_data_body[:, 31:34]
	earth_body_data = vector_data_body[:, 19:22]
	sat_body_vel = vector_data_body[:, 22:25]
	sat_body_pos = np.zeros([len(sat_body_vel), 3])
	sat_body_data = np.hstack([sat_body_pos, sat_body_vel])
	illum_vec_body_data = vector_data_body[:, 28:31]
	obs_vec_body_data = vector_data_body[:, 31:34]
	
	#	orb data
	sun_orb_data = vector_data_orb[:, 28:31]
	site_orb_data = vector_data_orb[:, 31:34]
	earth_orb_data = vector_data_orb[:, 19:22]
	sat_orb_vel = vector_data_orb[:, 22:25]
	sat_orb_pos = np.zeros([len(sat_orb_vel), 3])
	sat_orb_data = np.hstack([sat_orb_pos, sat_orb_vel])
	illum_vec_orb_data = vector_data_orb[:, 28:31]
	obs_vec_orb_data = vector_data_orb[:, 31:34]
	
	#	extract orbital positions of custom input vectors
	illum_vec_orb_data = vector_data_orb[:, 37:40]
	obs_vec_orb_data = vector_data_orb[:, 40:43]
	
	illum_vec_body_data = vector_data_body[:, 37:40]
	obs_vec_body_data = vector_data_body[:, 40:43]

	###############################################################################################
	#	generate light curve
	###############################################################################################
	
	#	generate sim time
	sim_time = np.linspace(0, duration_sec, len_record)
	
	#	initilise lightcurve output
	final_light_curve = np.zeros((len_record, 4))
	final_light_curve[:, 1] = sim_time
	jdate_start = start_jd[0] + start_jd[1]

	#	run light curve generation function
	slc_log_level = 0
	light_curve = glim_slc_shadowing(
		sun_body_data, site_body_data, sun_orb_data, site_orb_data,
		final_light_curve,
		jdate_start, time_step,
		body_mesh,
		init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		brdf_data,
		slc_log_level
	)
	
	light_curve_jd = light_curve[:, 0]
	light_curve_seconds = light_curve[:, 1]
	light_curve_data_app = light_curve[:, 2]
	light_curve_data_std = light_curve[:, 3]

	average_mag = np.average(light_curve_data_app)
	min_mag = light_curve_data_app.min()
	max_mag = light_curve_data_app.max()

	return [rot_vec_inc_deg, rot_vec_azm_deg, average_mag, min_mag, max_mag]

