###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	combined elevation/azimuth polar plot code
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
from subprocess import call

#	numpy
import numpy as np

#	scipy
import scipy
from scipy import optimize

#	numpy-stl
from stl import mesh

#	matplotlib
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.colors as colors
from matplotlib.colors import BoundaryNorm
from matplotlib.colors import ListedColormap, LinearSegmentedColormap
from matplotlib.ticker import MaxNLocator
from mpl_toolkits.mplot3d import Axes3D

#	custom functions
from domain_splitting import cheb_app

###################################################################################################
#   initilisation
###################################################################################################

#	clear terminal
call(['clear'])
print('\nGenerating figures...')

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/paper_data'
path_data = os.path.join(root_path, archive_path)

##################################################
#	figure output

#	set flag
savefile_flag = False
savefile_flag = True

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Journals',
	'ACTA_Astronautica',
	'acta_figures',
	'Final'
)

##################################################
#	inputs

sim_step = 0.1

#	geometry case
geom_case = 'cylinder'
geom_case = 'cube'
geom_case = 'div1'
geom_case = 'div5'

#	reflection case
reflection_case = 'diffuse'
reflection_case = 'specular'

#	bulk run directory
if geom_case == 'cube':
	bulk_dir = 'distance_data_cube_{}'.format(str(sim_step))
	sub_dir = geom_case

elif geom_case == 'div1':
	bulk_dir = 'distance_data_div1_{}'.format(str(sim_step))
	sub_dir = 'sphere_' + geom_case

elif geom_case == 'div5':
	bulk_dir = 'distance_data_div5_{}'.format(str(sim_step))
	sub_dir = 'sphere_' + geom_case

elif geom_case == 'cylinder':
	bulk_dir = 'distance_data_cylinder_{}'.format(str(sim_step))
	sub_dir = geom_case

##################################################
#	figure output

#	output file path
#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Thesis',
	'Thesis_Main_Directory',
	'Figures',
)

###################################################################################################
#	acquiring 0-attit case data
###################################################################################################

#	set input case designation
case_desig_0attit = '_'.join(['0-attit', geom_case])

#	input data
distance_path = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_0attit)
distance_dir = os.listdir(distance_path)

print('\nGetting distances from: ', distance_path)

#########################################################################################
#	sort distance files
distance_files = []
coefs_files_0attit = []
case_num = 0

for distance_file in distance_dir:

	attit_case = distance_file.split('_')[1]

	if distance_file.split('.')[1] == 'dist':
		distance_files.append([int(case_num), distance_file])

	elif distance_file.split('.')[1] == 'coefs':
		coefs_files_0attit.append([int(case_num), distance_file])

	case_num += 1

distance_files = sorted(distance_files, key=lambda x: x[0])
coefs_files_0attit = sorted(coefs_files_0attit, key=lambda x: x[0])

#########################################################################################
#	generate data
data = []
for i, distance_file in enumerate(distance_files):
	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		distance_file[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])
	distance = np.genfromtxt(os.path.join(distance_path, distance_file[1]), delimiter=',')
	data_line = [elv, azm, distance]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	extract azimuth and elevation angles
elv_vals = np.unique(data[:, 0])
azm_vals = np.unique(data[:, 1])

#	reset to include 100 so that 90 is plotted
elv_vals = np.arange(0, 110, 10)

#	generate polar mesh grid
r, th = np.meshgrid(elv_vals, azm_vals)

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)

	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		distance_column = data_of_elv[:, 2]

		#	extract distance data into output array column
		data_array[:, column_index] = distance_column
		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)
		#	itterate data index
		column_index += 1
		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
distance_column = data_of_elv[:, 2]
data_array[:, column_index] = distance_column

#	set plot data
attit0_data = data_array

###################################################################################################
#	acquiring x-rot case data
#########################################################################################

#	change case designation and repeat
case_desig_xrot = '_'.join(['x-rot', geom_case])

#	input data
distance_path = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_xrot)
distance_dir = os.listdir(distance_path)
print('\nGetting distances from: ', distance_path)

#########################################################################################
#	sort distance files
distance_files = []
coefs_files_xrot = []

for distance_file in distance_dir:

	attit_case = distance_file.split('_')[1]

	if distance_file.split('.')[1] == 'dist':
		dist_case_num = int(distance_file.split('_')[0])
		distance_files.append([int(dist_case_num), distance_file])

	elif distance_file.split('.')[1] == 'coefs':
		coef_case_num = int(distance_file.split('_')[0])
		coefs_files_xrot.append([int(coef_case_num), distance_file])


distance_files = sorted(distance_files, key=lambda x: x[0])
coefs_files_xrot = sorted(coefs_files_xrot, key=lambda x: x[0])

#########################################################################################
#	generate data
data = []
for i, distance_file in enumerate(distance_files):
	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		distance_file[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])
	distance = np.genfromtxt(os.path.join(distance_path, distance_file[1]), delimiter=',')
	data_line = [elv, azm, distance]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)
	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		distance_column = data_of_elv[:, 2]

		#	extract distance data into output array column
		data_array[:, column_index] = distance_column
		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)
		#	itterate data index
		column_index += 1
		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
distance_column = data_of_elv[:, 2]
data_array[:, column_index] = distance_column

#	set plot data
xrot_data = data_array

###################################################################################################
#	acquiring y-rot case data
###################################################################################################

#	change case designation and repeat
case_desig_yrot = '_'.join(['y-rot', geom_case])

#	input data
distance_path = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_yrot)
distance_dir = os.listdir(distance_path)
print('\nGetting distances from: ', distance_path)

#########################################################################################
#	sort distance files
distance_files = []
coefs_files_yrot = []

for distance_file in distance_dir:

	attit_case = distance_file.split('_')[1]

	if distance_file.split('.')[1] == 'dist':
		dist_case_num = int(distance_file.split('_')[0])
		distance_files.append([int(dist_case_num), distance_file])

	elif distance_file.split('.')[1] == 'coefs':
		coef_case_num = int(distance_file.split('_')[0])
		coefs_files_yrot.append([int(coef_case_num), distance_file])

distance_files = sorted(distance_files, key=lambda x: x[0])
coefs_files_yrot = sorted(coefs_files_yrot, key=lambda x: x[0])

#########################################################################################
#	generate data
data = []
for i, distance_file in enumerate(distance_files):
	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		distance_file[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])
	distance = np.genfromtxt(os.path.join(distance_path, distance_file[1]), delimiter=',')
	data_line = [elv, azm, distance]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)
	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		distance_column = data_of_elv[:, 2]

		#	extract distance data into output array column
		data_array[:, column_index] = distance_column
		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)
		#	itterate data index
		column_index += 1
		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
distance_column = data_of_elv[:, 2]
data_array[:, column_index] = distance_column

#	set plot data
yrot_data = data_array

###################################################################################################
#	acquiring z-rot case data
#########################################################################################

#	change case designation and repeat
case_desig_zrot = '_'.join(['z-rot', geom_case])

#	input data
distance_path = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_zrot)
distance_dir = os.listdir(distance_path)
print('\nGetting distances from: ', distance_path)

#########################################################################################
#	sort distance files
distance_files = []
coefs_files_zrot = []

for distance_file in distance_dir:

	attit_case = distance_file.split('_')[1]

	if distance_file.split('.')[1] == 'dist':
		dist_case_num = int(distance_file.split('_')[0])
		distance_files.append([int(dist_case_num), distance_file])

	elif distance_file.split('.')[1] == 'coefs':
		coef_case_num = int(distance_file.split('_')[0])
		coefs_files_zrot.append([int(coef_case_num), distance_file])

distance_files = sorted(distance_files, key=lambda x: x[0])
coefs_files_zrot = sorted(coefs_files_zrot, key=lambda x: x[0])

#########################################################################################
#	generate data
data = []
for i, distance_file in enumerate(distance_files):
	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		distance_file[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])
	distance = np.genfromtxt(os.path.join(distance_path, distance_file[1]), delimiter=',')
	data_line = [elv, azm, distance]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)
	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		distance_column = data_of_elv[:, 2]

		#	extract distance data into output array column
		data_array[:, column_index] = distance_column
		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)
		#	itterate data index
		column_index += 1
		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
distance_column = data_of_elv[:, 2]
data_array[:, column_index] = distance_column

#	set plot data
zrot_data = data_array

###################################################################################################
###################################################################################################
#	generating subtraction data
###################################################################################################
###################################################################################################

#	set input directories
distance_path_xrot = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_xrot)
distance_path_yrot = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_yrot)
distance_path_zrot = os.path.join(path_data, 'distance_dir', bulk_dir, reflection_case, case_desig_zrot)

#########################################################################################
#	generating x-y data
#########################################################################################

data = []
for i, (coefs_file_xrot, coefs_file_yrot) in enumerate(zip(coefs_files_xrot, coefs_files_yrot)):

	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		coefs_file_xrot[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])

	#	load case1 coefs
	case1_coefs = np.genfromtxt(os.path.join(distance_path_xrot, coefs_file_xrot[1]), delimiter=',')

	#	load case2 coefs
	case2_coefs = np.genfromtxt(os.path.join(distance_path_yrot, coefs_file_yrot[1]), delimiter=',')

	#	calculate distance
	distance = np.linalg.norm(np.subtract(case1_coefs, case2_coefs))

	data_line = [elv, azm, distance]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	initiialise output array
data_array = np.zeros(r.shape)
#	current column index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)

	#	else if elevation value is new
	elif line_elv != current_elv:

		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		distance_column = data_of_elv[:, 2]

		#	extract distance data into output array column
		data_array[:, column_index] = distance_column

		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)

		#	itterate data index
		column_index += 1

		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
distance_column = data_of_elv[:, 2]
data_array[:, column_index] = distance_column

#	set plot data
y_x_data = data_array

#########################################################################################
#	generating x-z data
#########################################################################################

data = []
for i, (coefs_file_xrot, coefs_file_zrot) in enumerate(zip(coefs_files_xrot, coefs_files_zrot)):

	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		coefs_file_xrot[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])

	#	load case1 coefs
	case1_coefs = np.genfromtxt(os.path.join(distance_path_xrot, coefs_file_xrot[1]), delimiter=',')

	#	load case2 coefs
	case2_coefs = np.genfromtxt(os.path.join(distance_path_zrot, coefs_file_zrot[1]), delimiter=',')

	#	calculate distance
	distance = np.linalg.norm(np.subtract(case1_coefs, case2_coefs))

	data_line = [elv, azm, distance]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)
	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		distance_column = data_of_elv[:, 2]

		#	extract distance data into output array column
		data_array[:, column_index] = distance_column

		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)

		#	itterate data index
		column_index += 1

		#	reset current elevation
		current_elv = line_elv

#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
distance_column = data_of_elv[:, 2]
data_array[:, column_index] = distance_column

#	set plot data
z_x_data = data_array

#########################################################################################
#	generating z-y data
#########################################################################################

data = []
for i, (coefs_file_zrot, coefs_file_yrot) in enumerate(zip(coefs_files_zrot, coefs_files_yrot)):

	num_str, attit_case, geom_case, azm_str, elv_str, reflection_case = \
		coefs_file_zrot[1].split('.')[0].split('_')
	azm = int(azm_str[3:])
	elv = int(elv_str[3:])

	#	load case1 coefs
	case1_coefs = np.genfromtxt(os.path.join(distance_path_zrot, coefs_file_zrot[1]), delimiter=',')

	#	load case2 coefs
	case2_coefs = np.genfromtxt(os.path.join(distance_path_yrot, coefs_file_yrot[1]), delimiter=',')

	#	calculate distance
	distance = np.linalg.norm(np.subtract(case1_coefs, case2_coefs))

	data_line = [elv, azm, distance]
	data.append(data_line)

#	sort data by elevation
data = np.array(sorted(data, key=lambda x: x[0]))

#	initiialise output array
data_array = np.zeros(r.shape)
#	current line index
column_index = 0
#	current elevation
current_elv = data[0, 0]
#	initialise data array of elevation
data_of_elv = []

#	looping through data array
for i, line in enumerate(data):

	#	parse elevation of line
	line_elv = line[0]

	#	if elevation is same as current, append data
	if line_elv == current_elv:
		data_of_elv.append(line)
	#	else if elevation value is new
	elif line_elv != current_elv:
		#	sort current data of elevation by azimuth value
		data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
		distance_column = data_of_elv[:, 2]

		#	extract distance data into output array column
		data_array[:, column_index] = distance_column

		#	reinitilize data of elv on new value
		data_of_elv = []
		data_of_elv.append(line)

		#	itterate data index
		column_index += 1

		#	reset current elevation
		current_elv = line_elv


#	finalise elv = 90 column
data_of_elv = np.array(sorted(data_of_elv, key=lambda x: x[1]))
distance_column = data_of_elv[:, 2]
data_array[:, column_index] = distance_column

#	set plot data
z_y_data = data_array

###################################################################################################
###################################################################################################
#	graphical outputs
###################################################################################################
###################################################################################################

####################################################
#	set figure-wide parameters

dpi_val = 300
small = 1e-15
inch2cm = 2.54

####################################################
#	select colour map
cmap = plt.cm.get_cmap('cividis_r')
cmap.set_under('w')
 
####################################################
#	altitude rings
leo_alt = 400
meo_alt = 2000
geo_alt = 35786

RE = 6371

leo_elv = np.arctan(RE / (RE + leo_alt))
meo_elv = np.arctan(RE / (RE + meo_alt))
geo_elv = np.arctan(RE / (RE + geo_alt))

leo_elv_vals = np.full(elv_vals.shape, leo_elv)
meo_elv_vals = np.full(elv_vals.shape, meo_elv)
geo_elv_vals = np.full(elv_vals.shape, geo_elv)

# #	generate mesh grid for leo
leo_r, leo_th = np.meshgrid(leo_elv_vals, azm_vals)
meo_r, meo_th = np.meshgrid(meo_elv_vals, azm_vals)
geo_r, geo_th = np.meshgrid(geo_elv_vals, azm_vals)

####################################################
#	find azimuth offset to align with grid
nazm_vals = len(th) - 1
offset = (360 / nazm_vals) / 2

###################################################################################################
#	single axis graphs
###################################################################################################

####################################################
#	figure parameters
font_sizex = 6
font_sizey = 6
font_pad = -3
marker_size = 4
marker_edge = 1

hspace_val = 0.35
wspace_val = 0.35

grid_width = 0.1
alt_line_width = 0.1

title_size = 8
title_pad = 10

####################################################
#	set figure wide colour bounds

#	get max vals
attit0_data_raw = attit0_data.max()
xrot_data_raw = xrot_data.max()
yrot_data_raw = yrot_data.max()
zrot_data_raw = zrot_data.max()

#	specific case: remove outermost ring
if geom_case == 'cube' or geom_case == 'cylinder':
	attit0_data_raw = attit0_data[:, 0:-2].max()
	xrot_data_raw = xrot_data[:, 0:-2].max()
	yrot_data_raw = yrot_data[:, 0:-2].max()
	zrot_data_raw = zrot_data[:, 0:-2].max()

#	generate array of max vals
max_vals = np.array([
	[attit0_data_raw, 0],
	[xrot_data_raw, 1],
	[yrot_data_raw, 2],
	[zrot_data_raw, 3]
])

#	sort decending
max_vals = max_vals[max_vals[:, 0].argsort()][::-1]

#	take largest pixel
max_arg = max_vals[0, 1]
zmax = max_vals[0, 0]

# ####################################################
# ####################################################
# #	specific case: use second highest plot
# if geom_case == 'div1' and reflection_case == 'specular':
# 	#	take second largest to avoid specular glint
# 	max_arg = max_vals[1, 1]
# 	zmax = max_vals[1, 0]
# ####################################################
# ####################################################

#	extract data array containing largest pixel
if max_arg == 0:
	maxdata = attit0_data
elif max_arg == 1:
	maxdata = xrot_data
elif max_arg == 2:
	maxdata = yrot_data
elif max_arg == 3:
	maxdata = zrot_data

#	specific case: remove outermost ring
if geom_case == 'cube' or geom_case == 'cylinder':
	maxdata = maxdata[:, 0:-2]

#	set colour bounds
n_col = 100

if zmax <= 0.01:
	newmax = np.ceil(maxdata.max() * 10000) / 10000
	step = 0.0001

elif zmax <= 0.1:
	newmax = np.round(maxdata.max(), 3)
	step = 0.001

elif zmax <= 1:
	newmax = np.round(maxdata.max(), 2)
	step = 0.01

elif zmax <= 5:
	newmax = np.round(maxdata.max(), 1)
	step = 0.2

elif zmax <= 10:
	newmax = np.round(maxdata.max(), 1)
	step = 0.1

elif zmax <= 100:
	newmax = np.ceil(maxdata.max())
	step = 1

bounds = np.linspace(0, newmax + step, n_col)

#	generate colour bar bounds
bounds[0] = small
norm = colors.BoundaryNorm(boundaries=bounds, ncolors=256)

####################################################
#	define tick markers

####################################################
####################################################
#	specific case
if geom_case == 'div5' and reflection_case == 'specular':
	tick_marks = np.arange(0.0, newmax + (step * 2), step * 2)

#	specific case
elif geom_case == 'cube' and reflection_case == 'specular':
	tick_marks = np.arange(0.0, newmax + (step * 5), step * 5)
else:
	#	standard step tickmarks
	tick_marks = np.arange(0.0, newmax + (step), step)

####################################################
####################################################

####################################################
#	initialise figure
# fig = plt.figure(figsize=(9.0/inch2cm, 9.0/inch2cm), dpi=dpi_val)


if reflection_case == 'diffuse':
	if geom_case == 'div1' or  geom_case == 'div5':
		fig = plt.figure(figsize=(9.0/inch2cm, 4.5/inch2cm), dpi=dpi_val)
	else:
		fig = plt.figure(figsize=(9.0/inch2cm, 9.0/inch2cm), dpi=dpi_val)
	
else:
	fig = plt.figure(figsize=(9.0/inch2cm, 9.0/inch2cm), dpi=dpi_val)



plt.rc('text', usetex=True)
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

###################################################################################################
#	0-attit

####################################################
#	initialise axes
#ax0 = plt.subplot(221, projection="polar")

if reflection_case == 'diffuse':
	if geom_case == 'div1' or  geom_case == 'div5':
		ax0 = plt.subplot(121, projection="polar")
	else:
		ax0 = plt.subplot(221, projection="polar")
else:
	ax0 = plt.subplot(221, projection="polar")


ax0.set_theta_zero_location("N")
ax0.set_theta_direction(-1)

ax0.set_title(r'$no-rotation$', pad=title_pad, fontsize=title_size)

ax0.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax0.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax0.set_rticks(np.arange(0, 100, 20))
ax0.set_rlim(0, 100)

if geom_case == 'cube' or geom_case == 'cylinder':
	ax0.set_rlim(0, 90)

####################################################

#	colour the plot
plt.pcolor(np.radians(th - offset), r, attit0_data, norm=norm, cmap=cmap)

####################################################
#	plot altitude lines
ax0.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
ax0.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
ax0.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)

####################################################
#	plot measurement point
ax0.plot([0.0], [0.0],
	marker='x',
	color='g',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

####################################################
#	finalise figure
ax0.grid(which='major', axis='both')

if geom_case == 'cube' or geom_case == 'cylinder':
	plt.yticks(np.arange(0, 90, 20))

###################################################################################################
#	x-rot

####################################################
#	initialise axes
# ax1 = plt.subplot(222, projection="polar")

if reflection_case == 'diffuse':
	if geom_case == 'div1' or  geom_case == 'div5':
		ax1 = plt.subplot(122, projection="polar")
	else:
		ax1 = plt.subplot(222, projection="polar")

#	specular cases
else:
	ax1 = plt.subplot(222, projection="polar")

ax1.set_theta_zero_location("N")
ax1.set_theta_direction(-1)

ax1.set_title(r'$x-rotation$', pad=title_pad, fontsize=title_size)

ax1.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax1.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax1.set_rticks(np.arange(0, 100, 20))
ax1.set_rlim(0, 100)

if geom_case == 'cube' or geom_case == 'cylinder':
	ax1.set_rlim(0, 90)

####################################################
#	colour the plot
plt.pcolormesh(
	np.radians(th - offset),
	r,
	xrot_data,
	norm=norm,
	cmap=cmap
)

####################################################
#	plot altitude lines
ax1.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
ax1.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
ax1.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)

####################################################
#	plot rotation vector
ax1.plot([0.0], [0.0],
	marker='+',
	color='r',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

####################################################
#	finalise figure
ax1.grid(which='major', axis='both')

if geom_case == 'cube' or geom_case == 'cylinder':
	plt.yticks(np.arange(0, 90, 20))

###################################################################################################
#	y-rot

####################################################
#	initialise axes


plot_flag = False

if reflection_case == 'diffuse':
	if geom_case == 'div1' or  geom_case == 'div5':
		pass
	else:
		plot_flag = True
else:
	plot_flag = True


if plot_flag is True:

	ax2 = plt.subplot(223, projection="polar")
	
	ax2.set_theta_zero_location("N")
	ax2.set_theta_direction(-1)
	
	ax2.set_title(r'$y-rotation$', pad=title_pad, fontsize=title_size)
	
	ax2.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
	ax2.tick_params(axis='y', labelsize=font_sizey, pad=0)
	
	ax2.set_rticks(np.arange(0, 100, 20))
	ax2.set_rlim(0, 100)
	
	if geom_case == 'cube' or geom_case == 'cylinder':
		ax2.set_rlim(0, 90)
	
	####################################################
	#	colour the plot
	plt.pcolormesh(
		np.radians(th - offset),
		r,
		yrot_data,
		norm=norm,
		cmap=cmap
	)
	
	####################################################
	#	plot altitude lines
	ax2.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
	ax2.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
	ax2.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)
	
	####################################################
	#	plot rotation vector
	ax2.plot([np.pi/2], [90 + offset],
		marker='+',
		color='r',
		markersize=marker_size,
		markeredgewidth=marker_edge,
		clip_on=False
	)
	
	####################################################
	#	finalise figure
	ax2.grid(which='major', axis='both')
	
	if geom_case == 'cube' or geom_case == 'cylinder':
		plt.yticks(np.arange(0, 90, 20))
	
	###################################################################################################
	#	z-rot
	
	####################################################
	#	initialise figure
	ax3 = plt.subplot(224, projection="polar")
	ax3.set_theta_zero_location("N")
	ax3.set_theta_direction(-1)
	
	ax3.set_title(r'$z-rotation$', pad=title_pad, fontsize=title_size)
	
	ax3.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
	ax3.tick_params(axis='y', labelsize=font_sizey, pad=0)
	
	ax3.set_rticks(np.arange(0, 100, 20))
	ax3.set_rlim(0, 100)
	
	if geom_case == 'cube' or geom_case == 'cylinder':
		ax3.set_rlim(0, 90)
	
	####################################################
	#	colour the plot
	plt.pcolormesh(
		np.radians(th - offset),
		r,
		zrot_data,
		norm=norm,
		cmap=cmap
	)
	
	####################################################
	#	plot altitude lines
	ax3.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
	ax3.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
	ax3.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)
	
	####################################################
	#	plot rotation vector
	ax3.plot([0.0], [90 + offset],
		marker='+',
		color='r',
		markersize=marker_size,
		markeredgewidth=marker_edge,
		clip_on=False
	)
	
	####################################################
	#	finalise figure
	ax3.grid(which='major', axis='both')
	
	if geom_case == 'cube' or geom_case == 'cylinder':
		plt.yticks(np.arange(0, 90, 20))

####################################################
#	add colour bar
plt.subplots_adjust(bottom=0.1, right=0.8)
cax = plt.axes([0.87, 0.1, 0.03, 0.8])

#plt.colorbar(cax=cax, ticks=tick_marks)
plt.colorbar(cax=cax, label=r'$Hilbert\ Distance$')

cax.tick_params(axis='y', labelsize=font_sizey, length=3, direction='in')

####################################################
#	saving figure
fig_name = geom_case + '_' + reflection_case + '_apx.eps'
out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	plt.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

###################################################################################################
#	subtraction plots
###################################################################################################

####################################################
#	select colour map
cmap = plt.cm.get_cmap('viridis_r')
cmap = plt.cm.get_cmap('inferno_r')
cmap.set_under('w')

####################################################
#	set figure-specific parameters
font_size_cbar = 6
font_sizex = 6
font_sizey = 3


font_sizey = 6

font_pad = -3
marker_size = 4
marker_edge = 1

grid_width = 0.5
alt_line_width = 0.75

hspace_val = 0.35
wspace_val = 0.35

title_size = 8
title_pad = 10


grid_width = 0.1
alt_line_width = 0.1

####################################################
#	set figure wide colour bounds

#	get max values
z_y_data_raw = z_y_data.max()
y_x_data_raw = y_x_data.max()
z_x_data_raw = z_x_data.max()

#	specific case: remove outmost ring
if geom_case == 'cube' or geom_case == 'cylinder':
	z_y_data_raw = z_y_data[:, 0:-2].max()
	y_x_data_raw = y_x_data[:, 0:-2].max()
	z_x_data_raw = z_x_data[:, 0:-2].max()

#	find plot which contains maximum value
max_vals = np.array([
	[z_y_data_raw, 0],
	[y_x_data_raw, 1],
	[z_x_data_raw, 2]
])

#	sort decending
max_vals = max_vals[max_vals[:, 0].argsort()][::-1]

#	extract max and max data
max_arg = max_vals[0, 1]
zmax = max_vals[0, 0]

## ####################################################
## ####################################################
## #	specific case: use second highest plot
## if geom_case == 'div1' and reflection_case == 'specular':
## 	max_arg = max_vals[1, 1]
## 	zmax = max_vals[1, 0]
## ####################################################
## ####################################################

#	extract max data
if max_arg == 0:
	maxdata = z_y_data
elif max_arg == 1:
	maxdata = y_x_data
elif max_arg == 2:
	maxdata = z_x_data

#	specific case: remove outermost ring
if geom_case == 'cube' or geom_case == 'cylinder':
	maxdata = maxdata[:, 0:-2]

#	set colour bounds
n_col = 100

if zmax <= 0.01:
	newmax = np.ceil(maxdata.max() * 10000) / 10000
	step = 0.0001

elif zmax <= 0.1:
	newmax = np.round(maxdata.max(), 3)
	step = 0.001

elif zmax <= 1:
	newmax = np.round(maxdata.max(), 2)
	step = 0.01

elif zmax <= 10:
	newmax = np.round(maxdata.max(), 1)
	step = 0.1

elif zmax <= 100:
	newmax = np.ceil(maxdata.max())
	step = 1

bounds = np.linspace(0, newmax + step, n_col)

bounds[0] = small
norm = colors.BoundaryNorm(boundaries=bounds, ncolors=256)

####################################################
#	define tick markers
#if geom_case == 'div5':
#	if reflection_case == 'diffuse':
#		tick_marks = np.arange(0.0, newmax + (step * 2), step * 2)
#	elif reflection_case == 'specular':
#		tick_marks = np.arange(0.0, newmax + (step * 5), step * 5)
#
#elif geom_case == 'cube':
#	if reflection_case == 'diffuse':
#		tick_marks = np.arange(0.0, newmax + (step), step)
#	elif reflection_case == 'specular':
#		tick_marks = np.arange(0.0, newmax + (step * 5), step * 5)
#
#elif geom_case == 'div1' and reflection_case == 'specular':
#	tick_marks = np.arange(0.0, newmax + (step * 5), step * 5)
#
#else:
#	tick_marks = np.arange(0.0, newmax + (step * 10), step * 10)

####################################################
#	initialise figure
inch2cm = 2.54

#fig = plt.figure(figsize=(9.0/inch2cm, 4.5/inch2cm), dpi=dpi_val)
if geom_case == 'cube':
	if reflection_case == 'diffuse':
		fig = plt.figure(figsize=(9.0/inch2cm, 4.5/inch2cm), dpi=dpi_val)
	else:
		fig = plt.figure(figsize=(9.0/inch2cm, 4.5/inch2cm), dpi=dpi_val)
else:
	fig = plt.figure(figsize=(9.0/inch2cm, 4.5/inch2cm), dpi=dpi_val)

plt.rc('text', usetex=True)
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

###################################################################################################
#	z-rot - y-rot

####################################################
#	initialise axes
ax4 = plt.subplot(131, projection="polar")

if geom_case == 'cube':
	if reflection_case == 'diffuse':
		ax4 = plt.subplot(131, projection="polar")
	else:
		ax4 = plt.subplot(121, projection="polar")
else:
	ax4 = plt.subplot(121, projection="polar")


ax4.set_theta_zero_location("N")
ax4.set_theta_direction(-1)

ax4.set_title(r'$z - y$', pad=title_pad, fontsize=title_size)

ax4.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax4.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax4.set_rticks(np.arange(0, 100, 20))
ax4.set_rlim(0, 100)

if geom_case == 'cube' or geom_case == 'cylinder':
	ax4.set_rlim(0, 90)

####################################################
#	colour the plot
plt.pcolormesh(
	np.radians(th - offset),
	r,
	z_y_data,
	norm=norm,
	cmap=cmap
)
 
####################################################
#	plot altitude lines
ax4.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
ax4.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
ax4.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)

####################################################
#	plot rotation vectors
ax4.plot([np.pi/2], [90 + offset],
	marker='+',
	color='r',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

ax4.plot([0.0], [90 + offset],
	marker='+',
	color='r',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

####################################################
#	finalise figure
#ax4.grid(which='major', axis='both', linewidth=grid_width)
ax4.grid(which='major', axis='both')

if geom_case == 'cube' or geom_case == 'cylinder':
	plt.yticks(np.arange(0, 90, 20))

###################################################################################################
#	y-rot - x-rot

####################################################
#	initialise axes
if geom_case == 'cube':
	if reflection_case == 'diffuse':
		ax5 = plt.subplot(132, projection="polar")
	else:
		ax5 = plt.subplot(122, projection="polar")
else:
	ax5 = plt.subplot(122, projection="polar")

#ax5 = plt.subplot(132, projection="polar")
ax5.set_theta_zero_location("N")
ax5.set_theta_direction(-1)

ax5.set_title(r'$y - x$', pad=title_pad, fontsize=title_size)

ax5.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
ax5.tick_params(axis='y', labelsize=font_sizey, pad=0)

ax5.set_rticks(np.arange(0, 100, 20))
ax5.set_rlim(0, 100)

if geom_case == 'cube' or geom_case == 'cylinder':
	ax5.set_rlim(0, 90)

####################################################
#	colour the plot
plt.pcolormesh(
	np.radians(th - offset),
	r,
	y_x_data,
	norm=norm,
	cmap=cmap
)
 
####################################################
#	plot altitude lines
ax5.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
ax5.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
ax5.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)

####################################################
#	plot rotation vectors
ax5.plot([0.0], [0.0],
	marker='+',
	color='r',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

ax5.plot([np.pi/2], [90 + offset],
	marker='+',
	color='r',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

####################################################
#	finalise figure
# plt.grid(which='major', axis='both', linewidth=grid_width)
ax5.grid(which='major', axis='both')

if geom_case == 'cube' or geom_case == 'cylinder':
	plt.yticks(np.arange(0, 90, 20))

###################################################################################################
#	z-rot - x-rot

if geom_case == 'cube':
	if reflection_case == 'diffuse':

		####################################################
		#	initialise axes
		ax6 = plt.subplot(133, projection="polar")
		ax6.set_theta_zero_location("N")
		ax6.set_theta_direction(-1)
		
		ax6.set_title(r'$z - x$', pad=title_pad, fontsize=title_size)
		
		ax6.tick_params(axis='x', labelsize=font_sizex, pad=font_pad)
		ax6.tick_params(axis='y', labelsize=font_sizey, pad=0)
		
		ax6.set_rticks(np.arange(0, 100, 20))
		ax6.set_rlim(0, 100)
		
		if geom_case == 'cube' or geom_case == 'cylinder':
			ax6.set_rlim(0, 90)
		
		####################################################
		#	colour the plot
		plt.pcolormesh(
			np.radians(th - offset),
			r,
			z_x_data,
			norm=norm,
			cmap=cmap
		)
		 
		####################################################
		#	plot altitude lines
		ax6.plot(np.radians(azm_vals), np.degrees(leo_r), color='g', linewidth=0.5)
		ax6.plot(np.radians(azm_vals), np.degrees(meo_r), color='b', linewidth=0.5)
		ax6.plot(np.radians(azm_vals), np.degrees(geo_r), color='c', linewidth=0.5)
		
		####################################################
		#	plot rotation vectors
		ax6.plot([0.0], [0.0],
			marker='+',
			color='r',
			markersize=marker_size,
			markeredgewidth=marker_edge,
			clip_on=False
		)
		
		ax6.plot([0.0], [90 + offset],
			marker='+',
			color='r',
			markersize=marker_size,
			markeredgewidth=marker_edge,
			clip_on=False
		)
		
		####################################################
		#	finalise figure
		ax6.grid(which='major', axis='both', linewidth=grid_width)
		
		if geom_case == 'cube' or geom_case == 'cylinder':
			plt.yticks(np.arange(0, 90, 20))

####################################################
#	add colour bar
plt.subplots_adjust(bottom=0.1, right=0.8)
cax = plt.axes([0.87, 0.1, 0.03, 0.8])
# plt.subplots_adjust(bottom=0.25)
# cax = plt.axes([0.1, 0.2, 0.8, 0.05])

if geom_case == 'cube':
	if reflection_case == 'diffuse':
		tick_marks = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7]
	elif reflection_case == 'specular':
		tick_marks = [0, 0, 5, 10, 15]

elif geom_case == 'div1':
	if reflection_case == 'diffuse':
		tick_marks = [0.0, 0.01, 0.02, 0.03, 0.04, 0.05]
	if reflection_case == 'specular':
		tick_marks = [0.0, 0.5, 1.0, 1.5, 2.0, 2.5]

elif geom_case == 'div5':
	if reflection_case == 'diffuse':
		tick_marks = [0.0, 0.005, 0.001, 0.0015, 0.0020, 0.0025, 0.0030, 0.0035, 0.004]
	elif reflection_case == 'specular':
		tick_marks = np.linspace(0.0, 0.04, 9)


plt.colorbar(cax=cax, ticks=tick_marks, label=r'$Hilbert\ Distance$')
cax.tick_params(axis='y', labelsize=font_size_cbar, length=3, direction='in')
#plt.colorbar(cax=cax, orientation='horizontal', ticks=tick_marks)
#cax.tick_params(axis='x', labelsize=font_size_cbar, length=3, direction='in')

####################################################
#	save figure
fig_name = geom_case + '_' + reflection_case + '_subtraction' + '_apx.eps'
out_str = os.path.join(fig_path, fig_name)

if savefile_flag is True:
	plt.savefig(out_str, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
