###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	domain spliiting function for Hilbert distance calculation
#
#
#
#
#
#
###################################################################################################
#	standard packages
import os
import sys

#	numpy
import numpy as np

#	matplotlib
import matplotlib.pyplot as plt

#	scipy
import scipy
from scipy import signal

###################################################################################################
#	root mean square calculation function

def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	domain splitting function
###################################################################################################


def dom_split(case_domain, case_signal):

	#	chebyshev sampling limit
	samples = 1000

	# correct for odd domain size
	if len(case_domain) % 2 == 0:
		odd_correction = 0
	else:
		odd_correction = 1

	###########################################################################
	#	generate domain and signal arrays

	#	domain start value
	zero_point =  int(case_domain[0])

	#	signal start index
	index_point = np.int(np.ceil(len(case_signal) / 2))

	#	domain halfway point
	half_point = index_point + zero_point

	#	generate signal indexing arrays
	dom1_index = np.arange(0, index_point, 1)
	dom2_index = np.arange(index_point, len(case_domain), 1)

	domain1 = np.arange(zero_point, half_point, 1)
	domain2 = np.arange(half_point, half_point + len(domain1) - odd_correction, 1)

	#	slice signal
	dom1_signal = case_signal[dom1_index]
	dom2_signal = case_signal[dom2_index]

	###########################################################################
	#	generate Chebyshev approximation

	#	domain 1
	#	create fit
	dom1_fit_class = np.polynomial.chebyshev.Chebyshev.fit(domain1, dom1_signal, samples)
	#	extract chebyshev coefficients
	dom1_cheb_coef = dom1_fit_class.coef
	#	regenerate chebyshev class object
	dom1_cheb_class = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef, [domain1[0], domain1[-1]])
	#	extract time series
	dom1_cheb_signal = dom1_cheb_class(domain1)

	#	domain 2
	#	create fit
	dom2_fit_class = np.polynomial.chebyshev.Chebyshev.fit(domain2, dom2_signal, samples)
	#	extract chebyshev coefficients
	dom2_cheb_coef = dom2_fit_class.coef
	#	regenerate chebyshev class object
	dom2_cheb_class = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef, [domain2[0], domain2[-1]])
	#	extract time series
	dom2_cheb_signal = dom2_cheb_class(domain2)

	###########################################################################
	#	output

	#	calculate RMS values
	rms_1 = rms_calc(dom1_signal, dom1_cheb_signal)
	rms_2 = rms_calc(dom2_signal, dom2_cheb_signal)

	#	assemble outputs
	outputs = [
		domain1,
		dom1_signal,
		dom1_cheb_coef,
		rms_1,
		domain2,
		dom2_signal,
		dom2_cheb_coef,
		rms_2
	]

	return outputs

###################################################################################################
#	chebychev approximation function
###################################################################################################

def cheb_app(case_signal):

	print('\nPerforming Approximation')

	#	space
	case_domain = np.arange(0, len(case_signal), 1)
	
	#	Recursive approximation
	samples = 1000

	###################################################
	#	calculate chebyshev coefficients
	#	create fit
	fit_class = np.polynomial.chebyshev.Chebyshev.fit(case_domain, case_signal, samples)

	#	extract chebyshev coefficients
	cheb_coef = fit_class.coef
	
	#	regenerate chebyshev class object
	cheb_class = np.polynomial.chebyshev.Chebyshev(fit_class.coef, [0, case_domain[-1]])
	
	#	extract time series
	first_cheb_signal = cheb_class(case_domain)
	
	print('\nFirst approximation Chebyshev Coefficents = \n', cheb_coef)
	
	###################################################
	
	#	calculate rms
	rms_init = rms_calc(case_signal, first_cheb_signal)
	print('\nInitial RMS = ', rms_init)

	###############################################################################################
	#	split level 0

#	print('\nPerforming split level 0.')
	outputs = dom_split(case_domain, case_signal)

	#	parse outputs
	dom1_space, dom1_signal, dom1_cheb_coef, rms_1, \
		dom2_space, dom2_signal, dom2_cheb_coef, rms_2 = outputs

	#	generate chebyshev class object
	dom1_cheb_class = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef, [dom1_space[0], dom1_space[-1]])
	dom2_cheb_class = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef, [dom2_space[0], dom2_space[-1]])

	#	extract time series
	dom1_cheb_signal = dom1_cheb_class(dom1_space)
	dom2_cheb_signal = dom2_cheb_class(dom2_space)

#	print('\nSplit level 0 complete.')

#	###################################################
#	#	OLD OUTPUTS
#	cheb_old_domain = np.concatenate([
#		dom1_space,
#		dom2_space
#	])
#
#	cheb_old_signal = np.concatenate([
#		dom1_old_cheb_signal,
#		dom2_old_cheb_signal
#	])
	###################################################

	#	assemble output arrays
	cheb_domain_outputs = np.array([
		dom1_space,
		dom2_space
	])

	cheb_signal_outputs = np.array([
		dom1_signal,
		dom2_signal
	])

	cheb_coef_outputs = np.array([
		dom1_cheb_coef,
		dom2_cheb_coef
	])

	cheb_rms_outputs = np.array([
		rms_1,
		rms_2
	])

	print('\nDom 1 RMS = ', rms_1)
	print('Dom 2 RMS = ', rms_2)
	



	###############################################################################################
	#	split level 1

	print('\nPerforming first split run. . .')
#	#	initilise loop
#	rms_vals = [rms_1, rms_2]
#	dom_spaces = [dom1_space, dom2_space]
#	dom_signals = [dom1_signal, dom2_signal]


	for (domain, signal, coefs, rms) in zip(
			cheb_domain_outputs, 
			cheb_signal_outputs,
			cheb_coef_outputs,
			cheb_rms_outputs):
	
		#	generate chebyshev class object
		cheb_class = np.polynomial.chebyshev.Chebyshev(coefs, [domain[0], domain[-1]])

		#	extract time series
		cheb_signal = dom1_cheb_class(domain)
	
		if rms > 1e-6:
			print('\nSplit required..')

			outputs = dom_split(domain, signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, rms_2 = outputs

			#	generate chebyshev class object
			dom1_cheb_class = np.polynomial.chebyshev.Chebyshev(
				dom1_cheb_coef, [dom1_space[0], dom1_space[-1]]
			)

			dom2_cheb_class = np.polynomial.chebyshev.Chebyshev(
				dom2_cheb_coef, [dom2_space[0], dom2_space[-1]]
			)

			#	extract time series
			dom1_cheb_signal = dom1_cheb_class(dom1_space)
			dom2_cheb_signal = dom2_cheb_class(dom2_space)

			print('\nNew Dom 1 RMS = ', rms_1)
			print('New Dom 2 RMS = ', rms_2)
	
#	print('\nSplit level 0 complete.')

		else:
			print('Sufficient match.')


		plt.figure(1)
		plt.cla()
		plt.plot(domain, signal, label='actual signal')
		plt.plot(domain, cheb_signal, label='signal')

		plt.plot(dom1_space, dom1_cheb_signal, label='signal')
		plt.plot(dom2_space, dom2_cheb_signal, label='signal')

		plt.ylim([10, 7])
#		plt.xlim([-5, 260])
		plt.legend()
		plt.show()





		sys.exit()
	


	if any(i > 1e-6 for i in rms_vals):
#		print('\nPerforming split level 1. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces, dom_signals):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces1 = [dom1_space, dom2_space]
				dom_signals1 = [dom1_signal, dom2_signal]
				dom_cheb_signals1 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]
	
			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces1.append(dom1_space)
				dom_spaces1.append(dom2_space)
				dom_signals1.append(dom1_signal)
				dom_signals1.append(dom2_signal)
				dom_cheb_signals1.append(dom1_cheb_signal)
				dom_cheb_signals1.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces1)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces1[0]
		dom_spas = dom_spaces1[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals1[0]
		dom_sigs = dom_cheb_signals1[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################
	#	split level 2
	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 2. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces1, dom_signals1):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces2 = [dom1_space, dom2_space]
				dom_signals2 = [dom1_signal, dom2_signal]
				dom_cheb_signals2 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces2.append(dom1_space)
				dom_spaces2.append(dom2_space)
				dom_signals2.append(dom1_signal)
				dom_signals2.append(dom2_signal)
				dom_cheb_signals2.append(dom1_cheb_signal)
				dom_cheb_signals2.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces2)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces2[0]
		dom_spas = dom_spaces2[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals2[0]
		dom_sigs = dom_cheb_signals2[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal


	###############################################################################################
	#	split level 3

	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 3. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces2, dom_signals2):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces3 = [dom1_space, dom2_space]
				dom_signals3 = [dom1_signal, dom2_signal]
				dom_cheb_signals3 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces3.append(dom1_space)
				dom_spaces3.append(dom2_space)
				dom_signals3.append(dom1_signal)
				dom_signals3.append(dom2_signal)
				dom_cheb_signals3.append(dom1_cheb_signal)
				dom_cheb_signals3.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces3)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces3[0]
		dom_spas = dom_spaces3[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals3[0]
		dom_sigs = dom_cheb_signals3[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################
	#	split level 4

	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 4. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces3, dom_signals3):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces4 = [dom1_space, dom2_space]
				dom_signals4 = [dom1_signal, dom2_signal]
				dom_cheb_signals4 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces4.append(dom1_space)
				dom_spaces4.append(dom2_space)
				dom_signals4.append(dom1_signal)
				dom_signals4.append(dom2_signal)
				dom_cheb_signals4.append(dom1_cheb_signal)
				dom_cheb_signals4.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces4)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces4[0]
		dom_spas = dom_spaces4[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals4[0]
		dom_sigs = dom_cheb_signals4[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################

	#	split level 5
	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 5. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces4, dom_signals4):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces5 = [dom1_space, dom2_space]
				dom_signals5 = [dom1_signal, dom2_signal]
				dom_cheb_signals5 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces5.append(dom1_space)
				dom_spaces5.append(dom2_space)
				dom_signals5.append(dom1_signal)
				dom_signals5.append(dom2_signal)
				dom_cheb_signals5.append(dom1_cheb_signal)
				dom_cheb_signals5.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces5)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces5[0]
		dom_spas = dom_spaces5[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals5[0]
		dom_sigs = dom_cheb_signals5[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################
	#	split level 6

	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 6. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces5, dom_signals5):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces6 = [dom1_space, dom2_space]
				dom_signals6 = [dom1_signal, dom2_signal]
				dom_cheb_signals6 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces6.append(dom1_space)
				dom_spaces6.append(dom2_space)
				dom_signals6.append(dom1_signal)
				dom_signals6.append(dom2_signal)
				dom_cheb_signals6.append(dom1_cheb_signal)
				dom_cheb_signals6.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces6)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces6[0]
		dom_spas = dom_spaces6[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals6[0]
		dom_sigs = dom_cheb_signals6[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################
	#	split level 7
	if any(i > 1e-6 for i in rms_vals):
#		print('\nStarting split level 7. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces6, dom_signals6):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces7 = [dom1_space, dom2_space]
				dom_signals7 = [dom1_signal, dom2_signal]
				dom_cheb_signals7 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces7.append(dom1_space)
				dom_spaces7.append(dom2_space)
				dom_signals7.append(dom1_signal)
				dom_signals7.append(dom2_signal)
				dom_cheb_signals7.append(dom1_cheb_signal)
				dom_cheb_signals7.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces7)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces7[0]
		dom_spas = dom_spaces7[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals7[0]
		dom_sigs = dom_cheb_signals7[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal




###################################################################################################
#	chebychev approximation function
###################################################################################################

def cheb_app_required(case_signal, required_domains):
	
	#	Recursive approximation
	samples = 20
	
	#	space
	case_domain = np.arange(0, len(case_signal), 1)

	#	approximate signal
	#	find chebyshev coeficients
	cheb_coef, fit_info = np.polynomial.chebyshev.chebfit(
		case_domain,
		case_signal,
		samples,
		full=True
	)

	#	generate chebyshev series
	cheb_function = np.polynomial.chebyshev.Chebyshev(cheb_coef)

	#	evaluate chebyshev series at time points
	first_cheb_signal = cheb_function(case_domain)

	###################################################
	#	OLD TECHNIQUE
	p = np.polynomial.Chebyshev.fit(case_domain, case_signal, samples)
	old_first_cheb_signal = p(case_domain)

	###################################################
	
	#	calculate rms
	rms_init = rms_calc(case_signal, first_cheb_signal)
#	print('Initial RMS = ', rms_init)

	###############################################################################################
	#	split level 0

#	print('\nPerforming split level 0.')
	outputs = dom_split(case_domain, case_signal)

	#	parse outputs
	dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
		dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

	#	generate chebyshev series
	dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
	dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)

	#	evaluate chebyshev series at time points
	dom1_cheb_signal = dom1_cheb_function(dom1_space)
	dom2_cheb_signal = dom2_cheb_function(dom2_space)

#	print('\nSplit level 0 complete.')

	###################################################
	#	OLD OUTPUTS
	cheb_old_domain = np.concatenate([
		dom1_space,
		dom2_space
	])

	cheb_old_signal = np.concatenate([
		dom1_old_cheb_signal,
		dom2_old_cheb_signal
	])
	###################################################

	#	assemble output arrays
	cheb_domain_outputs = np.array([
		dom1_space,
		dom2_space
	])

	cheb_coef_outputs = np.array([
		dom1_cheb_coef,
		dom2_cheb_coef
	])
	
	###############################################################################################
	#	split level 1

	#	initilise loop
	rms_vals = [rms_1, rms_2]
	dom_spaces = [dom1_space, dom2_space]
	dom_signals = [dom1_signal, dom2_signal]

	if any(i > 1e-6 for i in rms_vals) or len(cheb_domain_outputs) < required_domains:
#		print('\nPerforming split level 1. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces, dom_signals):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces1 = [dom1_space, dom2_space]
				dom_signals1 = [dom1_signal, dom2_signal]
				dom_cheb_signals1 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]
	
			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces1.append(dom1_space)
				dom_spaces1.append(dom2_space)
				dom_signals1.append(dom1_signal)
				dom_signals1.append(dom2_signal)
				dom_cheb_signals1.append(dom1_cheb_signal)
				dom_cheb_signals1.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces1)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces1[0]
		dom_spas = dom_spaces1[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals1[0]
		dom_sigs = dom_cheb_signals1[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################
	#	split level 2
	if any(i > 1e-6 for i in rms_vals) or len(cheb_domain_outputs) < required_domains:
#		print('\nStarting split level 2. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces1, dom_signals1):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces2 = [dom1_space, dom2_space]
				dom_signals2 = [dom1_signal, dom2_signal]
				dom_cheb_signals2 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces2.append(dom1_space)
				dom_spaces2.append(dom2_space)
				dom_signals2.append(dom1_signal)
				dom_signals2.append(dom2_signal)
				dom_cheb_signals2.append(dom1_cheb_signal)
				dom_cheb_signals2.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces2)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces2[0]
		dom_spas = dom_spaces2[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals2[0]
		dom_sigs = dom_cheb_signals2[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################
	#	split level 3

	if any(i > 1e-6 for i in rms_vals) or len(cheb_domain_outputs) < required_domains:
#		print('\nStarting split level 3. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces2, dom_signals2):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces3 = [dom1_space, dom2_space]
				dom_signals3 = [dom1_signal, dom2_signal]
				dom_cheb_signals3 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces3.append(dom1_space)
				dom_spaces3.append(dom2_space)
				dom_signals3.append(dom1_signal)
				dom_signals3.append(dom2_signal)
				dom_cheb_signals3.append(dom1_cheb_signal)
				dom_cheb_signals3.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces3)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces3[0]
		dom_spas = dom_spaces3[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals3[0]
		dom_sigs = dom_cheb_signals3[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################
	#	split level 4

	if any(i > 1e-6 for i in rms_vals) or len(cheb_domain_outputs) < required_domains:
#		print('\nStarting split level 4. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces3, dom_signals3):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces4 = [dom1_space, dom2_space]
				dom_signals4 = [dom1_signal, dom2_signal]
				dom_cheb_signals4 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces4.append(dom1_space)
				dom_spaces4.append(dom2_space)
				dom_signals4.append(dom1_signal)
				dom_signals4.append(dom2_signal)
				dom_cheb_signals4.append(dom1_cheb_signal)
				dom_cheb_signals4.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces4)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces4[0]
		dom_spas = dom_spaces4[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals4[0]
		dom_sigs = dom_cheb_signals4[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################

	#	split level 5
	if any(i > 1e-6 for i in rms_vals) or len(cheb_domain_outputs) < required_domains:
#		print('\nStarting split level 5. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces4, dom_signals4):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces5 = [dom1_space, dom2_space]
				dom_signals5 = [dom1_signal, dom2_signal]
				dom_cheb_signals5 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces5.append(dom1_space)
				dom_spaces5.append(dom2_space)
				dom_signals5.append(dom1_signal)
				dom_signals5.append(dom2_signal)
				dom_cheb_signals5.append(dom1_cheb_signal)
				dom_cheb_signals5.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces5)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces5[0]
		dom_spas = dom_spaces5[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals5[0]
		dom_sigs = dom_cheb_signals5[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################
	#	split level 6

	if any(i > 1e-6 for i in rms_vals) or len(cheb_domain_outputs) < required_domains:
#		print('\nStarting split level 6. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces5, dom_signals5):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces6 = [dom1_space, dom2_space]
				dom_signals6 = [dom1_signal, dom2_signal]
				dom_cheb_signals6 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces6.append(dom1_space)
				dom_spaces6.append(dom2_space)
				dom_signals6.append(dom1_signal)
				dom_signals6.append(dom2_signal)
				dom_cheb_signals6.append(dom1_cheb_signal)
				dom_cheb_signals6.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces6)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces6[0]
		dom_spas = dom_spaces6[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals6[0]
		dom_sigs = dom_cheb_signals6[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	else:
#		print('No further splitting required.')
		return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal

	###############################################################################################
	#	split level 7
	if any(i > 1e-6 for i in rms_vals) or len(cheb_domain_outputs) < required_domains:
		print('\nStarting split level 7. . .')

		split_num = 1
		first_flag = 1

		#	loop though domains
		for dom_space, dom_signal in zip(dom_spaces6, dom_signals6):

			#	perform domain splitting
			outputs = dom_split(dom_space, dom_signal)

			#	parse outputs
			dom1_space, dom1_signal, dom1_cheb_coef, dom1_old_cheb_signal, rms_1, \
				dom2_space, dom2_signal, dom2_cheb_coef, dom2_old_cheb_signal, rms_2 = outputs

			#	generate chebyshev series
			dom1_cheb_function = np.polynomial.chebyshev.Chebyshev(dom1_cheb_coef)
			dom2_cheb_function = np.polynomial.chebyshev.Chebyshev(dom2_cheb_coef)
		
			#	evaluate chebyshev series at time points
			dom1_cheb_signal = dom1_cheb_function(dom1_space)
			dom2_cheb_signal = dom2_cheb_function(dom2_space)

			if first_flag is 1:
				rms_vals = [rms_1, rms_2]
				dom_spaces7 = [dom1_space, dom2_space]
				dom_signals7 = [dom1_signal, dom2_signal]
				dom_cheb_signals7 = [dom1_cheb_signal, dom2_cheb_signal]
				first_flag = 0

				cheb_coef_outputs = [dom1_cheb_coef, dom2_cheb_coef]

			else:
				rms_vals.append(rms_1)
				rms_vals.append(rms_2)
				dom_spaces7.append(dom1_space)
				dom_spaces7.append(dom2_space)
				dom_signals7.append(dom1_signal)
				dom_signals7.append(dom2_signal)
				dom_cheb_signals7.append(dom1_cheb_signal)
				dom_cheb_signals7.append(dom2_cheb_signal)

				cheb_coef_outputs.append(dom1_cheb_coef)
				cheb_coef_outputs.append(dom2_cheb_coef)

			split_num += 1

		#	assemble output array
		cheb_domain_outputs = np.array(dom_spaces7)
		cheb_coef_outputs = np.array(cheb_coef_outputs)

		#	assemble Chebeshev domain
		cheb_old_domain = dom_spaces7[0]
		dom_spas = dom_spaces7[1:]
		for dom_spa in dom_spas:
			cheb_old_domain = np.concatenate([cheb_old_domain, dom_spa])

		#	assemble Chebeshev signal
		cheb_old_signal = dom_cheb_signals7[0]
		dom_sigs = dom_cheb_signals7[1:]
		for dom_sig in dom_sigs:
			cheb_old_signal = np.concatenate([cheb_old_signal, dom_sig])
	
	return cheb_domain_outputs, cheb_coef_outputs, cheb_old_domain, cheb_old_signal
