###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys

#	numpy
import numpy as np

#	progress bar
import progressbar

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

#	root mean square calculation function
def rms_calc(predictions, targets):
	return np.sqrt(((predictions - targets)**2).mean())

###################################################################################################
#	Function: 		hilbert_calc
#	Description: 	performs hilbert approximation using fourier basis
#
###################################################################################################

def hilbert_calc(x_in, y_in, N_init, use_N_init, log_level):

	##################################################
	#	mimic signal in negative domain for sines only approximation
	x_neg = -x_in[1:]
	x_neg = np.flip(x_neg)

	y_neg = y_in[1:]
	y_neg = np.flip(y_neg)
	
	x = np.append(x_neg, x_in)
	y = np.append(y_neg, y_in)
	
	##################################################
	#	get starting coefficents
	
	#	set P as domain length
	P = len(x) - 1
	
	#	find a0 and b0
	a_0 = (2 / P) * np.trapz(y, x)
	b_0 = 0.0
	
	#	starting number of terms
	N = N_init

	#	initialise loop data
	n_stop = 0
	prev_rms = 100
	min_perc = 0.1
	
	norm_vals = []
	N_vals = []
	rms_vals = []
	
	#	initialise tracking figure
	if log_level > 0:
		rms_fig, rms_ax = plt.subplots()
		rms_ax.set_title('RMS')

		signal_fig, signal_ax = plt.subplots()
		signal_ax.set_title('Signal')

	#	continue until break condition is met
	while True:
	
		#	set range of n
		nvals = np.arange(1, N, 1)
			
		#	initialise coefficients
		coefs = np.zeros([len(nvals) + 1])
		coefs[0] = a_0
		
		#	find remaining coefficients
		for i, n in enumerate(nvals):
		
			#	find a_n coefficient
			a_n = (2 / P) * np.trapz(y * np.cos(2 * np.pi * x * (n / P)), x)
			if -1e-10 < a_n < 1e-10:
				a_n = 0.0
		
			#	find b_n coefficient
			b_n = (2 / P) * np.trapz(y * np.sin(2 * np.pi * x * (n / P)), x)
			if -1e-10 < b_n < 1e-10:
				b_n = 0.0
		
			#	add to output array
			coefs[i + 1] = a_n
		
			#	check for zero b coefficent
			if b_n != 0.0:
				print('Non-Zero b coefficient!')
				sys.exit()
		
		##################################################
		#	generate fourier series
		approx_signal = a_0 / 2
		
		for i, n in enumerate(nvals):
			approx_signal += coefs[i + 1] * np.cos(2 * np.pi * n * x / P)
		
		#	remove the data in the negative domain
		approx_signal = approx_signal[len(x_in) - 1:]

		#	RMS calculation
		rms = rms_calc(approx_signal, y_in)
	
		#	current hilbert norm
		hilbert_norm = np.linalg.norm(coefs)

		#	append data to output arrays
		norm_vals.append(hilbert_norm)
		N_vals.append(N)
		rms_vals.append(rms)
	
		##################################################
		#	break condition
	
		#	RMS percentage change
		rms_perc_change = 100 * abs(rms - prev_rms)
	
		if log_level > 0:
			print('\n	Current N-terms = ', N)
			print('	Current RMS = ', rms)
			print('\n		RMS percentage change = ', rms_perc_change)

		if use_N_init is True:
			break
		else:
			#	check for an rms percentage change less than min_perc
			if rms_perc_change < min_perc:

				#	find 5 consecutive low rms percentage changes
				if n_stop > 4:
					if log_level > 0:
						print('\nBreaking: 5 low change conditions found.')
					break
				else:
					n_stop += 1
					if log_level > 0:
						print('\n			Number of low RMS changes found: ', n_stop)
			elif n_stop > 0:
				n_stop = 0
				if log_level > 0:
					print('\n			Resetting to zero.')

		if log_level > 0:

			#	clear axes for this step
			rms_ax.cla()
			signal_ax.cla()

			#	compare approximation to input signal
			signal_ax.plot(x_in, y_in)
			signal_ax.plot(x_in, approx_signal)

			#	track RMS
			rms_ax.plot(rms_vals)

			#	pause plot
			plt.pause(0.5)
	
		##################################################
		#	reset for next loop
		prev_norm = hilbert_norm
		prev_rms = rms
		N += 100
	
	N_final = N
	rms_final = rms

	if log_level > 0:

		print('\nFinal N used = ', N_final)
		print('Final RMS = ', rms_final)
	
		#	showing final plot
		print('Showing final RMS plot')
		plt.show()
	
	return approx_signal, coefs, N_final, rms_final
