###################################################################################################
#	General function list
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################
###################################################################################################

#	standard packages
import os
import sys
import time

#	numpy
import numpy as np

#	numba
import numba

#	numpy-stl
from stl import mesh

#	object generation
import collections

#	interpolation
from scipy import interpolate

import scipy.ndimage.filters as filters
import scipy.ndimage.morphology as morphology

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	progressbar
import progressbar

###################################################################################################
#	Function: 		object_gen
#	Description: 	returns a case object containing relevant data
#
###################################################################################################

# Tuple case object
case_object = collections.namedtuple('case_object', [
	'case_ident',
	'norad_ident',
	'track_ident',
	'site_latitudegd',
	'site_longitude',
	'site_altitude',
	'jday_utc_start',
	'jfrac_utc_start',
	'jday_utc_stop',
	'jfrac_utc_stop',
	'attitude',
	'attitude_motion',
	'obs_vec',
	'illum_vec',
	'geometry_file'
])

# generate case object
def object_gen(case_ident, case_data):

	#	parse inputs
	case_id, \
	attit_case, \
	lat_case, \
	lon_case, \
	geom_case, \
	reflection_case, \
	mmt_id, \
	norad_id, \
	date_str \
		= case_ident.split('_')

	track_id = case_id

	# site data
	site_latgd = np.array(''.join(case_data[3]).split('=')[1]).astype(np.float)
	site_lon = np.array(''.join(case_data[4]).split('=')[1]).astype(np.float)
	site_alt = np.array(''.join(case_data[5]).split('=')[1]).astype(np.float)

	# time data
	start_time = (''.join(case_data[6]).split('=')[1]).strip().split(', ')
	stop_time = (''.join(case_data[7]).split('=')[1]).strip().split(', ')

	# attitude state
	attit = np.array(
		(''.join(case_data[8]).split('=')[1]).strip().split(', ')
	).astype(np.float)

	attit_mot = np.array(
		(''.join(case_data[9]).split('=')[1]).strip().split(', ')
	).astype(np.float)
	
	# observation vector
	observation_vec = np.array(
		(''.join(case_data[10]).split('=')[1]).strip().split(', ')
	).astype(np.float)

	#	illumination vector
	illumination_vec = np.array(
		(''.join(case_data[11]).split('=')[1]).strip().split(', ')
	).astype(np.float)

	# geometry file
	geom_file = ''.join(case_data[14].split('=')[1]).strip()

	#########################################################################################
	# generate time data

	# parse time information and change data type
	year1, mon1, day1, hour1, min1 = np.array(start_time[0:-1]).astype(np.int)
	sec1 = np.array(start_time[-1]).astype(np.float)

	year2, mon2, day2, hour2, min2 = np.array(stop_time[0:-1]).astype(np.int)
	sec2 = np.array(stop_time[-1]).astype(np.float)

	start_date = [year1, mon1, day1, hour1, min1, sec1]
	stop_date = [year2, mon2, day2, hour2, min2, sec2]

	#	convert to JD
	jday_start, jfrac_start = date2jd(*start_date)
	jday_stop, jfrac_stop = date2jd(*stop_date)
	duration = int(86400 * ((jday_stop + jfrac_stop) - (jday_start + jfrac_start)))

	#########################################################################################
	#	generate BDRF parameters

	raw_bdrf_data = case_data[18:len(case_data)]
	bdrf_data = []

	for bdrf_line in raw_bdrf_data:

		data_line = []
		for ele in bdrf_line.split(', '):
			data_line.append(float(ele))

		bdrf_data.append(data_line)

	#########################################################################################
	#	return object only if year is less than 2018

	if start_date[0] <= 2017:
		# Create case object
		object_data = case_object(
			case_ident=case_id,
			norad_ident=norad_id,
			track_ident=track_id,
			site_latitudegd=site_latgd,
			site_longitude=site_lon,
			site_altitude=site_alt,
			jday_utc_start=jday_start,
			jfrac_utc_start=jfrac_start,
			jday_utc_stop=jday_stop,
			jfrac_utc_stop=jfrac_stop,
			attitude=attit,
			attitude_motion=attit_mot,
			obs_vec=observation_vec,
			illum_vec=illumination_vec,
			geometry_file=geom_file
		)
	else:
		object_data = 0

	return object_data, bdrf_data

###################################################################################################
#	Function: 		object_gen_obs
#	Description: 	returns a case object containing relevant data
#
###################################################################################################

# Tuple case object
case_object_obs = collections.namedtuple('case_object_obs', [
	'case_ident',
	'norad_ident',
	'track_ident',
	'site_latitudegd',
	'site_longitude',
	'site_altitude',
	'jday_utc_start',
	'jfrac_utc_start',
	'jday_utc_stop',
	'jfrac_utc_stop',
	'attitude',
	'rotation_axis',
	'rotation_angle',
	'observation_angles',
	'illumination_angles',
	'geometry_file'
])

# generate case object
def object_gen_obs(case_ident, case_data):

	#	parse inputs
	case_id, \
	illum_string, \
	illum_angles, \
	obs_string, \
	obs_angles, \
	geom_str, \
	reflection_case, \
	mmt_id, \
	norad_id, \
	date_str \
		= case_ident.split('_')

	track_id = case_id

	# site data
	site_latgd = np.array(''.join(case_data[3]).split('=')[1]).astype(np.float)
	site_lon = np.array(''.join(case_data[4]).split('=')[1]).astype(np.float)
	site_alt = np.array(''.join(case_data[5]).split('=')[1]).astype(np.float)

	# time data
	start_time = (''.join(case_data[6]).split('=')[1]).strip().split(', ')
	stop_time = (''.join(case_data[7]).split('=')[1]).strip().split(', ')

	# attitude
	attit = np.array(
		(''.join(case_data[8]).split('=')[1]).strip().split(', ')
	).astype(np.float)

	# attitude motion
	rot_axis = np.array(
		(''.join(case_data[9]).split('=')[1]).strip().split(', ')
	).astype(np.float)
	
	rot_angle = np.array(
		(''.join(case_data[10]).split('=')[1]).strip().split(', ')
	).astype(np.float)

	# observation vector
	obs_angles = np.array(
		(''.join(case_data[11]).split('=')[1]).strip().split(', ')
	).astype(np.float)

	#	illumination vector
	illum_angles = np.array(
		(''.join(case_data[12]).split('=')[1]).strip().split(', ')
	).astype(np.float)

	# geometry file
	geom_file = ''.join(case_data[15].split('=')[1]).strip()

	#########################################################################################
	# generate time data

	# parse time information and change data type
	year1, mon1, day1, hour1, min1 = np.array(start_time[0:-1]).astype(np.int)
	sec1 = np.array(start_time[-1]).astype(np.float)

	year2, mon2, day2, hour2, min2 = np.array(stop_time[0:-1]).astype(np.int)
	sec2 = np.array(stop_time[-1]).astype(np.float)

	start_date = [year1, mon1, day1, hour1, min1, sec1]
	stop_date = [year2, mon2, day2, hour2, min2, sec2]

	#	convert to JD
	jday_start, jfrac_start = date2jd(*start_date)
	jday_stop, jfrac_stop = date2jd(*stop_date)
	duration = int(86400 * ((jday_stop + jfrac_stop) - (jday_start + jfrac_start)))

	#########################################################################################
	#	generate BDRF parameters

	raw_bdrf_data = case_data[19:len(case_data)]
	bdrf_data = []

	for bdrf_line in raw_bdrf_data:
		data_line = []
		for ele in bdrf_line.split(', '):
			data_line.append(float(ele))

		bdrf_data.append(data_line)

	#########################################################################################
	#	return object only if year is less than 2018

	if start_date[0] <= 2017:

		# Create case object
		object_data = case_object_obs(
			case_ident=case_id,
			norad_ident=norad_id,
			track_ident=track_id,
			site_latitudegd=site_latgd,
			site_longitude=site_lon,
			site_altitude=site_alt,
			jday_utc_start=jday_start,
			jfrac_utc_start=jfrac_start,
			jday_utc_stop=jday_stop,
			jfrac_utc_stop=jfrac_stop,
			attitude=attit,
			rotation_axis=rot_axis,
			rotation_angle=rot_angle,
			observation_angles=obs_angles,
			illumination_angles=illum_angles,
			geometry_file=geom_file
		)
	else:
		object_data = 0

	return object_data, bdrf_data

###################################################################################################
#	Function: 		jday_jfrac
#	Description: 	converts a julian date to julian day and day fraction
#
###################################################################################################

@numba.jit(nopython=True)
def jday_jfrac(jd):

	#	convert sim start to julian day and julian fraction
	if jd - np.floor(jd) < 0.5:
		jday = np.floor(jd) - 0.5
		jfrac = jd - jday

	elif jd - np.floor(jd) > 0.5:
		jday = np.floor(jd) + 0.5
		jfrac = jd - jday

	elif jd - np.floor(jd) == 0.5:
		jday = jd
		jfrac = 0.0

	return jday, jfrac

###################################################################################################
#	Function: 		date2jd
#	Description: 	converts a gregorian date to julian date
#
###################################################################################################

@numba.jit(nopython=True)
def date2jd(year, mon, day, hour, mins, sec):

	# days
	temp = np.floor((7.0 * (year + np.floor((mon + 9.0) / 12.0))) * 0.25)
	jday = 367.0 * year - temp + np.floor(275.0 * mon / 9.0) + day + 1721013.5

	# day fraction
	jfrac = (sec + mins * 60.0 + hour * 3600.0) / 86400.0
	if jfrac > 1.0:
		jday = jday + 1.0
		jfrac = jfrac - 1.0

	return jday, jfrac

###################################################################################################
#	Function: 		get_month
#	Description: 	converts month string to month number
#
###################################################################################################

def get_month(mon_str):

	if mon_str == 'Jan':
		temp_mon = int(1)
	
	elif mon_str == 'Feb':
		temp_mon = int(2)
	
	elif mon_str == 'Mar':
		temp_mon = int(3)
	
	elif mon_str == 'Apr':
		temp_mon = int(4)
	
	elif mon_str == 'May':
		temp_mon = int(5)
	
	elif mon_str == 'Jun':
		temp_mon = int(6)
	
	elif mon_str == 'Jul':
		temp_mon = int(7)
	
	elif mon_str == 'Aug':
		temp_mon = int(8)
	
	elif mon_str == 'Sep':
		temp_mon = int(9)
	
	elif mon_str == 'Oct':
		temp_mon = int(10)
	
	elif mon_str == 'Nov':
		temp_mon = int(11)
	
	elif mon_str == 'Dec':
		temp_mon = int(12)

	return temp_mon

###################################################################################################
#	Function: 		jd2date
#	Description: 	converts a julian date to gregorian date.
#					gregorian date is quantised to 0.1 sec
#
###################################################################################################

def jd2date(jday, jfrac):

		# correct for day fraction greater than 1
		if (abs(jfrac) >= 1):
			jday = jday + np.floor(jfrac)
			jfrac = jfrac - np.floor(jfrac)

		dt = jday - int(jday)
		if (abs(dt - 0.5) > 0.00000001):
			jfrac = jfrac + dt - 0.5

		# number of leap years
		temp = jday + jfrac - 2415019.5
		tu = temp / 365.25
		year = int(1900 + np.floor(tu))
		leap_years = np.floor((year - 1901) * 0.25)

		# elapsed days
		days = np.floor(temp - ((year - 1900) * 365 + leap_years))

		# correct for day numbers less than 0.5
		if (days < 1):
			year = year - 1
			leap_years = np.floor((year - 1901) * 0.25)
			days = np.floor(temp - ((year - 1900) * 365 + leap_years))

		days = days + jfrac

		# create days in month list
		length_mon = np.zeros((12), dtype=np.int)
		for i in range(12):
			length_mon[i] = 31
			if (i == 1):
				length_mon[i] = 28
			if (i == 3 or i == 5 or i == 8 or i == 10):
				length_mon[i] = 30

		# day of year
		doy = np.floor(days)
		if ((year - 1900) % 4 == 0):
			length_mon[1] = 29

		# month
		j = 0
		temp = 0
		while (doy > (temp + length_mon[j]) and j < 11):
			temp = temp + length_mon[j]
			j = j + 1

		month = j + 1
		day = int(doy - temp)
		temp = (days - doy) * 24
		hour = int(np.fix(temp))
		temp = (temp - hour) * 60
		minute = int(np.fix(temp))
		sec = (temp - minute) * 60.0

		#	perform rounding

		#	correct rounding issues
		if np.round(sec, 1) >= 60:
			minute += 1
			sec = 0.0
	
			if minute >= 60:
				hour += 1
				minute = 0
	
				if hour == 24:
					day += 1
					hour = 0
	
		else:
			sec = np.round(sec, 1)

		return year, month, day, hour, minute, sec

###################################################################################################
#	Function: 		jd2date_exact
#	Description: 	converts a julian date to gregorian date.
#
###################################################################################################

def jd2date_exact(jday, jfrac):

		# correct for day fraction greater than 1
		if (abs(jfrac) >= 1):
			jday = jday + np.floor(jfrac)
			jfrac = jfrac - np.floor(jfrac)

		dt = jday - int(jday)
		if (abs(dt - 0.5) > 0.00000001):
			jfrac = jfrac + dt - 0.5

		# number of leap years
		temp = jday + jfrac - 2415019.5
		tu = temp / 365.25
		year = int(1900 + np.floor(tu))
		leap_years = np.floor((year - 1901) * 0.25)

		# elapsed days
		days = np.floor(temp - ((year - 1900) * 365 + leap_years))

		# correct for day numbers less than 0.5
		if (days < 1):
			year = year - 1
			leap_years = np.floor((year - 1901) * 0.25)
			days = np.floor(temp - ((year - 1900) * 365 + leap_years))

		days = days + jfrac

		# create days in month list
		length_mon = np.zeros((12), dtype=np.int)
		for i in range(12):
			length_mon[i] = 31
			if (i == 1):
				length_mon[i] = 28
			if (i == 3 or i == 5 or i == 8 or i == 10):
				length_mon[i] = 30

		# day of year
		doy = np.floor(days)
		if ((year - 1900) % 4 == 0):
			length_mon[1] = 29

		# month
		j = 0
		temp = 0
		while (doy > (temp + length_mon[j]) and j < 11):
			temp = temp + length_mon[j]
			j = j + 1

		month = j + 1
		day = int(doy - temp)
		temp = (days - doy) * 24
		hour = int(np.fix(temp))
		temp = (temp - hour) * 60
		minute = int(np.fix(temp))
		sec = (temp - minute) * 60.0

		return year, month, day, hour, minute, sec

###################################################################################################
#	Function: 		get_times
#	Description: 	converts UTC time to verious different time standards
#
###################################################################################################

def get_times(year, mon, day, hour, minute, sec, dut1, dat):

	# time object output
	time_obj = collections.namedtuple('time_object', [
		'jday_utc',
		'jfrac_utc',
		'tutc',
		'jday_ut1',
		'jfrac_ut1',
		'tut1',
		'jday_tai',
		'jfrac_tai',
		'ttai',
		'jday_tt',
		'jfrac_tt',
		'ttt',
		'jday_tdb',
		'jfrac_tdb',
		'ttdb',
		'jday_tcg',
		'jfrac_tcg',
		'ttcg',
		'jday_tcb',
		'jfrac_tcb',
		'ttcb',
	])

	# UTC
	utc = (hour * 3600) + (minute * 60) + sec
	jday_utc, jfrac_utc = date2jd(year, mon, day, hour, minute, sec)
	jdutc = jday_utc + jfrac_utc
	tutc = (jdutc - 2451545) / 36525

	# UT1
	ut1 = utc + dut1
	ut1_hour = int(ut1 / 3600)
	ut1_minute = int(ut1 / 60 - ut1_hour * 3600)
	ut1_sec = ut1 - ut1_minute * 60 - ut1_hour * 3600
	jday_ut1, jfrac_ut1 = date2jd(year, mon, day, ut1_hour, ut1_minute, ut1_sec)
	jdut1 = jday_ut1 + jfrac_ut1
	tut1 = (jdut1 - 2451545) / 36525

	# TAI
	tai = utc + dat
	tai_hour = int(tai / 3600)
	tai_minute = int(tai / 60 - tai_hour * 60)
	tai_sec = tai - tai_minute * 60 - tai_hour * 3600
	jday_tai, jfrac_tai = date2jd(year, mon, day, tai_hour, tai_minute, tai_sec)
	jdtai = jday_tai + jfrac_tai
	ttai = (jdtai - 2451545) / 36525

	# TT
	tt = tai + 32.184
	tt_hour = int(tt / 3600)
	tt_minute = int(tt / 60 - tt_hour * 60)
	tt_sec = tt - tt_minute * 60 - tt_hour * 3600
	jday_tt, jfrac_tt = date2jd(year, mon, day, tt_hour, tt_minute, tt_sec)
	jdtt = jday_tt + jfrac_tt
	ttt = (jdtt - 2451545) / 36525

	# TDB
	tdb = (
		tt + 0.001657 * np.sin(628.3076 * ttt + 6.2401) + 0.000022 * np.sin(575.3385 * ttt + 4.2970) +
		0.000014 * np.sin(1256.6152 * ttt + 6.1969) + 0.000005 * np.sin(606.9777 * ttt + 4.0212) +
		0.000005 * np.sin(52.9691 * ttt + 0.4444) + 0.000002 * np.sin(21.3299 * ttt + 5.5431) +
		0.000010 * ttt * np.sin(628.3076 * ttt + 4.2490)
	)

	tdb_hour = int(tdb / 3600)
	tdb_minute = int(tdb / 60 - tdb_hour * 60)
	tdb_sec = tdb - tdb_minute * 60 - tdb_hour * 3600
	jday_tdb, jfrac_tdb = date2jd(year, mon, day, tdb_hour, tdb_minute, tdb_sec)
	jdtdb = jday_tdb + jfrac_tdb
	ttdb = (jdtdb - 2451545) / 36525

	# TCG
	tcg = tt + 6.969290134e-10 * (jdtai - 2443144.5) * 86400.0
	tcg_hour = int(tcg / 3600)
	tcg_minute = int(tcg / 60 - tcg_hour * 60)
	tcg_sec = tcg - tcg_minute * 60 - tcg_hour * 3600
	jday_tcg, jfrac_tcg = date2jd(year, mon, day, tcg_hour, tcg_minute, tcg_sec)
	jdtcg = jday_tcg + jfrac_tcg
	ttcg = (jdtcg - 2451545) / 36525

	# TCB
	tcb = tdb + 1.55051976772e-8 * (jdtai - 2443144.5) * 86400
	tcb_hour = int(tcb / 3600)
	tcb_minute = int(tcb / 60 - tcb_hour * 60)
	tcb_sec = tcb - tcb_minute * 60 - tcb_hour * 3600
	jday_tcb, jfrac_tcb = date2jd(year, mon, day, tcb_hour, tcb_minute, tcb_sec)
	jdtcb = jday_tcb + jfrac_tcb
	ttcb = (jdtcb - 2451545) / 36525

	time_object = time_obj(
		jday_utc=jday_utc,
		jfrac_utc=jfrac_utc,
		tutc=tutc,
		jday_ut1=jday_ut1,
		jfrac_ut1=jfrac_ut1,
		tut1=tut1,
		jday_tai=jday_tai,
		jfrac_tai=jfrac_tai,
		ttai=ttai,
		jday_tt=jday_tt,
		jfrac_tt=jfrac_tt,
		ttt=ttt,
		jday_tdb=jday_tdb,
		jfrac_tdb=jfrac_tdb,
		ttdb=ttdb,
		jday_tcg=jday_tcg,
		jfrac_tcg=jfrac_tcg,
		ttcg=ttcg,
		jday_tcb=jday_tcb,
		jfrac_tcb=jfrac_tcb,
		ttcb=ttcb,
	)

	return time_object

###################################################################################################
#	Function: 		interp
#	Description: 	performs a standard spline interpolation
#
###################################################################################################

def interp(data, interp_data, spline_x, spline_x_new):

	#	for each provided spline in data
	for spline in range(len(data[0])):

		#	extract spline to interpolate
		spline_y = data[:, spline]

		#	perform interpolation
		tck = interpolate.splrep(spline_x, spline_y, s=0)

		spline_y_new = interpolate.splev(spline_x_new, tck, der=0)

		#	populate return array
		interp_data[:, spline] = spline_y_new

	return interp_data

###################################################################################################
#	Function: 		eop_interpolate
#	Description: 	return eop data interpolated to the required sim range
#
###################################################################################################

def eop_interpolate(eop_path, eop_parsed, dat_parsed, sim_range, time_step, log_level):

	##################################################
	#	parse time data
	start_jd = sim_range[0]
	start_mjd = start_jd - 2400000.5
	stop_mjd = sim_range[-1] - 2400000.5

	load_start = time.time()

	###############################################################################################
	#	load in EOP data files
	if log_level != 0:
		print('\n				Loading EOP data files...')

	dat_file_data = np.loadtxt(os.path.join(eop_path, dat_parsed), delimiter=',')
	eop_file_data = np.loadtxt(os.path.join(eop_path, eop_parsed), delimiter=',')

	#	eop start and stop indices
	if np.floor(start_mjd) in eop_file_data[:, 0]:

		eop_start = np.where(eop_file_data[:, 0]==np.floor(start_mjd))[0][0]
		eop_stop = np.where(eop_file_data[:, 0]==np.floor(stop_mjd))[0][0]

	else:

		print('Requested EOP Start MJD is beyond end of existing EOP file. Exiting.')
		sys.exit()

	#	add/subtract 1 to cover the data
	eop_start -= 1
	eop_stop += 1

	#	number of eop points
	eop_npts = eop_stop - eop_start

	#	increase number of eop data points for interpolations below 1 day
	while eop_npts <= 3:

		#	add/subtract 1 to cover the data
		eop_start -= 1
		eop_stop += 1

		#	number of eop points
		eop_npts = eop_stop - eop_start

	##################################################
	#	truncate eop data
	eop_data_trunc = eop_file_data[eop_start: eop_stop, :]
	
	##################################################
	#	display time
	load_end = time.time()
	
	if log_level != 0:
		load_time = np.round((load_end - load_start), 1)
		print('\n					EOP data files loaded.')
		print('\n						- time taken = {} s.'.format(load_time))

	###############################################################################################
	#	EOP Spline Interpolation
	interp_start = time.time()
	if log_level != 0:
		print('\n				Interpolating EOP data...')

	eop_data_interp = np.hstack((eop_data_trunc[:, 0:1], eop_data_trunc[:, 4:]))

	#	current x spline
	spline_x = eop_data_trunc[:, 0]

	#	create new spline
	eop_start_mjd = eop_data_trunc[0, 0]
	eop_stop_mjd = eop_data_trunc[-1, 0]

	#	range of eop data in days
	data_range = int(eop_stop_mjd - eop_start_mjd)
	
	#	npts in 1 day
	day_npts = int(86400 / time_step)

	#	total npts
	interp_npts = data_range * day_npts

	#	new spline x
	spline_x_new = np.linspace(eop_start_mjd, eop_stop_mjd, interp_npts + 1)

	#	initilise new y splines
	interp_data = np.zeros((interp_npts + 1, len(eop_data_trunc[0, :])))

	#	perform interpolation
	eop_interp = interp(eop_data_interp, interp_data, spline_x, spline_x_new)

	##################################################
	#	display time
	interp_end = time.time()
	
	if log_level != 0:
		interp_time = np.round((interp_end - interp_start), 1)
		print('\n					EOP data interpolated.')
		print('\n						- time taken = {} s.'.format(interp_time))

	###############################################################################################
	#	extract data
	extract_start = time.time()
	if log_level != 0:
		print('\n				Extracting EOP data...')

	sim_start = sim_range[0] - 2400000.5
	sim_stop = sim_range[-1] - 2400000.5

	#	simulation start date vector
	start_date_vec = jd2date(*jday_jfrac(sim_range[0]))
	start_day = start_date_vec[2]
	data_start_day = jd2date(*jday_jfrac(eop_interp[0, 0] + 2400000.5))[2]

	##################################################
	#	display time
	extract_end = time.time()
	if log_level != 0:
		extract_time = np.round((extract_end - extract_start), 1)
		print('\n					EOP data extracted.')
		print('\n						- time taken = {} s.'.format(extract_time))

	###############################################################################################
	#	truncating data
	trunc_start = time.time()
	if log_level != 0:
		print('\n				Truncating Data...')

	while data_start_day < start_day:

		#	remove first day
		eop_interp = eop_interp[day_npts:, :]

		#	update data start day
		data_start_day = jd2date(*jday_jfrac(eop_interp[0, 0] + 2400000.5))[2]

	#	find line index of sim start
	test_index = np.where(np.round(eop_interp[:, 0], 7)==np.round(sim_start, 7))[0]

	#	if exact date match not found
	if not test_index:

		#	calculate upper and lower indices
		lower_index = np.where(np.round(eop_interp[:, 0], 7)<np.round(sim_start, 7))[0][-1]
		upper_index = np.where(np.round(eop_interp[:, 0], 7)>np.round(sim_start, 7))[0][0]

		#	generate temp index range
		temp_index_range = np.arange(lower_index - 1, upper_index + 1, 1)

		print('\n		Temp Index Range = ', temp_index_range)

		#	search for matching date
		for temp_index in temp_index_range:
	
			#	extract eop data line
			eop_line = eop_interp[temp_index]

			#	calculate datevec
			eop_date_vec = jd2date(*jday_jfrac(eop_line[0] + 2400000.5))

			if eop_date_vec == start_date_vec:
				start_index = temp_index
				break
	
	else:
		start_index = test_index[0]

#	#	find matching date vector
#	start_index = np.where(np.round(eop_interp[:, 0], 7)==np.round(sim_start, 7))[0][0]

	#	get stop index
	stop_index = start_index + (len(sim_range) - 1)

	#	extract eop data
	eop_data = eop_interp[start_index:stop_index + 1]

	##################################################
	#	display time
	trunc_end = time.time()
	
	if log_level != 0:
		trunc_time = np.round((trunc_end - trunc_start), 1)
		print('\n					EOP data truncated.')
		print('\n						- time taken = {} s.'.format(trunc_time))

	###############################################################################################
	#	DAT
	#	Number of leap-seconds, no interpolation required
	dat_ex_start = time.time()
	if log_level != 0:
		print('\n				Extracting DAT data...')

	#	initilise dat array
	dat_all = np.zeros((len(sim_range), 2))

	#	find all lines of dat data which are greater than start date
	dat_start = np.where(dat_file_data[:, 0] > start_jd)[0]

	#	check shape for dates after 2017
	dat_shape = dat_start.shape[0]

	#	if shape is zero, all data is greater than final dat value
	if dat_shape == 0:
		#	extract final line into truncated data
		dat_data_trunc = dat_file_data[-1, :]
		dat_val = dat_data_trunc[-1]

		#	generate dat all array
		dat_array = np.full([len(sim_range),], dat_val)

		#	add to output array
		dat_all[:, 0] = sim_range
		dat_all[:, 1] = dat_array

	else:

		#	extract the lines greater than the start date, in addition to previous line
		dat_data_trunc = dat_file_data[dat_start[0] - 1:, :]

		dat_index = 0

		data_start_date = sim_range[0]
		data_stop_date = sim_range[-1]

		#	set initial dates
		current_date = dat_data_trunc[dat_index, 0]
		next_date = dat_data_trunc[dat_index + 1, 0]
		current_dat = dat_data_trunc[dat_index, -1]

		#	all data lies between truncated data lines
		if data_start_date >= current_date and data_stop_date < next_date:

			#	generate dat all array
			dat_array = np.full([len(sim_range),], current_dat)

			#	add to output array
			dat_all[:, 0] = sim_range
			dat_all[:, 1] = dat_array

		#	final data line is greater than current date
		elif data_start_date >= current_date and data_stop_date > next_date:

			#	run though sim times and generate dat 
			for i, sim_time_step in enumerate(sim_range):
	
				#	date is less than the next dat date
				if sim_time_step < next_date:
	
					#	add to output array
					dat_all[i] = [sim_time_step, current_dat]

				elif sim_time_step >= next_date:
	
					#	itterate data
					dat_index += 1
	
					#	reset dates
					current_date = dat_data_trunc[dat_index, 0]
					next_date = dat_data_trunc[dat_index + 1, 0]
				
					current_dat = dat_data_trunc[dat_index, -1]
	
					#	add to output array
					dat_all[i] = [sim_time_step, current_dat]
	
	dat_data = np.copy(dat_all)

	##################################################
	#	display time
	dat_end = time.time()
	
	if log_level != 0:
		dat_time = np.round((dat_end - dat_ex_start), 1)
		print('\n					DAT data extracted.')
		print('\n						- time taken = {} s.'.format(dat_time))

	return eop_data, dat_data

# ###################################################################################################
# #	Function: 		gen_sgp4
# #	Description: 	use SGP4 to generate object positions and velocities at the required time
# #					range
# #
# ###################################################################################################
# 
# def gen_sgp4(sim_range, time_step, satellite):
# 
# 	#	initilise position data
# 	sat_teme_raw = np.zeros([len(sim_range), 7])
# 
# 	if time_step <= 60:
# 
# 		#	perform SGP4 propagation
# 		for i, jdate in enumerate(progressbar.progressbar(sim_range)):
# 		
# 			#	convert step jdate to fraction
# 			jday, jfrac = jday_jfrac(jdate)
# 		
# 			#	modified step julian date
# 			mjd = jday - 2400000.5
# 		
# 			#	convert to step date
# 			year, month, day, hour, minute, second = jd2date(jday, jfrac)
# 		
# 			#	evaluate SGP4 at step date
# 			pos, vel = satellite.propagate(year, month, day, hour, minute, second)
# 		
# 			#	add data to output
# 			sat_teme_raw[i, :] = np.hstack((jday + jfrac, pos, vel))
# 
# 	elif time_step > 60:
# 		print('					ERROR: Sat position unsufficient sampling.')
# 		sys.exit()
# 
# 	return sat_teme_raw
# 
###################################################################################################
#	Function: 		gen_sgp4
#	Description: 	use SGP4 to generate object positions and velocities at the required time
#					range
#
###################################################################################################

def gen_sgp4(sim_range, time_step, satellite):

	#	initilise position data
	sat_teme_raw = np.zeros([len(sim_range), 7])

	if time_step <= 60:

		#	perform SGP4 propagation
		for i, jdate in enumerate(progressbar.progressbar(sim_range)):
		
			#	convert step jdate to fraction
			jday, jfrac = jday_jfrac(jdate)
		
			#	modified step julian date
			mjd = jday - 2400000.5
		
			#	convert to step date
			year, month, day, hour, minute, second = jd2date(jday, jfrac)
		
			#	evaluate SGP4 at step date
			pos, vel = satellite.propagate(year, month, day, hour, minute, second)
		
			#	add data to output
			sat_teme_raw[i, :] = np.hstack((jday + jfrac, pos, vel))

	elif time_step > 60:
		print('					ERROR: Sat position unsufficient sampling.')
		sys.exit()

	return sat_teme_raw

###################################################################################################
#	Function: 		correct_nans
#	Description: 	corrects for occurances of NaN values in light curve data
#
#
###################################################################################################

def correct_nans(case_signal):

	#	ensure nan occurances are sequential
	nan_indices = np.argwhere(np.isnan(case_signal))
	nan_current = nan_indices[0]
	nan_others = nan_indices[1:]
	sequent = True

	for nan_index in nan_others:
		if nan_index != (nan_current + 1):
			sequent = False
			break

		nan_current = nan_index

	#	correction
	if sequent is False:
		print('NAN values are not sequential! Exiting..')

	elif sequent is True:
		print('nans are sequential, correcting..')

		#	indices of surrounding non-NAN values
		preval = case_signal[nan_indices[0] - 1]
		postval = case_signal[nan_indices[-1] + 1]

		#	define correction array
		correction_array = np.zeros((nnans + 2))
		correction_array[0] = preval
		correction_array[-1] = postval

		#	values to interpolate
		x = np.arange(1, len(correction_array) - 1, 1)

		#	known values
		xp = np.array([0, len(correction_array) - 1])
		fp = np.array([preval, postval])[:, 0]

		#	perform interpolation
		interp_data = np.interp(x, xp, fp)

		print('Interpolated data = ', interp_data)
		print('Interpolated data shape = ', interp_data.shape)

		#	correct case_signal array
		start_index = nan_indices[0][0]
		end_index = nan_indices[-1][0]

		#	reshape interp_data
		interp_data = np.reshape(interp_data, [len(interp_data), 1])

		#	replace nans
		case_signal[nan_indices] = interp_data

		print('nans replaced.')

	#	confirm nan values are removed
	if np.isnan(case_signal).any():
		print('ERROR: nan values still found')
	else:
		print('\nCONFIRMED: nans corrected')

	return case_signal

###################################################################################################
#	Function: 		gen_trans_files
#	Description: 	generates transformation file names
#
#
###################################################################################################

def gen_trans_files(start_date_str, start_time_str, len_record, time_step, norad_id, site_str):

	file_names_out = []

	##################################################
	#	satellite
	object_str = 'sat_'
	frame_list = ['orb_', 'azel_']

	for frame_str in frame_list:

		#	generate unique data file
		file_name = ''.join([
			object_str,
			frame_str,
			str(norad_id) + '_',
			'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
			'.txt'
		])

		file_names_out.append(file_name)

	##################################################
	#	sun
	object_str = 'sun_'
	frame_list = ['scrf_', 'orb_', 'azel_']

	for frame_str in frame_list:

		#	generate unique data file
		file_name = ''.join([
			object_str,
			frame_str,
			str(norad_id) + '_',
			'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
			'.txt'
		])

		file_names_out.append(file_name)

	##################################################
	#	site
	object_str = 'site_'
	frame_list = ['gcrf_', 'scrf_', 'orb_']

	for frame_str in frame_list:

		#	generate unique data file
		file_name = ''.join([
			object_str,
			frame_str,
			str(site_str) + '_',
			str(norad_id) + '_',
			'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
			'.txt'
		])

		file_names_out.append(file_name)

	##################################################
	#	earth
	object_str = 'earth_'
	frame_str = 'orb_'

	#	generate unique data file
	file_name = ''.join([
		object_str,
		frame_str,
		str(norad_id) + '_',
		'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
		'.txt'
	])

	file_names_out.append(file_name)

	return file_names_out

###################################################################################################

def detect_local_minima(arr):
    # https://stackoverflow.com/questions/3684484/peak-detection-in-a-2d-array/3689710#3689710
    """
    Takes an array and detects the troughs using the local maximum filter.
    Returns a boolean mask of the troughs (i.e. 1 when
    the pixel's value is the neighborhood maximum, 0 otherwise)
    """
    # define an connected neighborhood
    # http://www.scipy.org/doc/api_docs/SciPy.ndimage.morphology.html#generate_binary_structure
    neighborhood = morphology.generate_binary_structure(len(arr.shape),2)
    # apply the local minimum filter; all locations of minimum value 
    # in their neighborhood are set to 1
    # http://www.scipy.org/doc/api_docs/SciPy.ndimage.filters.html#minimum_filter
    local_min = (filters.minimum_filter(arr, footprint=neighborhood)==arr)
    # local_min is a mask that contains the peaks we are 
    # looking for, but also the background.
    # In order to isolate the peaks we must remove the background from the mask.
    # 
    # we create the mask of the background
    background = (arr==0)
    # 
    # a little technicality: we must erode the background in order to 
    # successfully subtract it from local_min, otherwise a line will 
    # appear along the background border (artifact of the local minimum filter)
    # http://www.scipy.org/doc/api_docs/SciPy.ndimage.morphology.html#binary_erosion
    eroded_background = morphology.binary_erosion(
        background, structure=neighborhood, border_value=1)
    # 
    # we obtain the final mask, containing only peaks, 
    # by removing the background from the local_min mask
    detected_minima = local_min ^ eroded_background

    return np.where(detected_minima)   

###################################################################################################
