###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	numba
import numba

#	progress bar
import progressbar

###################################################################################################
#	Function: 		gen_vectors_render2
#	Description: 	generate required data for render
#
###################################################################################################

def gen_vectors_render2(
		path_data, render_time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_body, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb,
		body_mesh, render_geom_dir
	):

	print('\nGenerating render vector data...')

	###############################################################################################
	#	initialisation
	###############################################################################################

	###############################################################################################
	#	parse time inputs

	#	extract jd time data
	jd_time_data = sun_orb[:, 0]

	#	set simulation start julian date
	start_jdate = jd_time_data[0]
	stop_jdate = jd_time_data[-1]

	#	duration
	duration_sec = render_time_step * (len_record - 1)

	###############################################################################################
	#	parse attit inputs

	#	set step attitude to initial attitude
	step_attit = np.copy(init_attit)

	#	generate attitude motion in the body frame
	attitude_matrix = axis2mat(np.radians(rot_ang_body), rot_vec_body_unit)
	
	#	extract euler angles
	angles1 = np.array(mat2angles(attitude_matrix)[0:3])
	angles2 = np.array(mat2angles(attitude_matrix)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	attit_mot_body = angles1

	###############################################################################################
	#	initialise variables

	#	initialise output arrays
	attitude_out = np.zeros((len_record, 3))
	vector_data_body = np.zeros((len_record, 31))
	vector_data_orb = np.zeros((len_record, 31))
	vector_data_inertial = np.zeros((len_record, 31))

	#	orbit normal in orbital frame
	orb_norm_orb_unit = np.array([0.0, 0.0, 1.0])

	###############################################################################################
	#	initialise geometry

	#	define body axis vectors of object mesh
	x_body_unit = np.array((1.0, 0.0, 0.0))
	y_body_unit = np.array((0.0, 1.0, 0.0))
	z_body_unit = np.array((0.0, 0.0, 1.0))

	#	apply initial frame offset
	rot_mat_step = rot_mat(step_attit)

	#	take inverse for orb2body rotation matrix
	rot_mat_step_inv = np.linalg.inv(rot_mat_step)

	#	generate a temporary orb mesh from the body mesh
	temp_orb_mesh = mesh.Mesh(body_mesh.data.copy())
	#	rotate into orbital frame
	temp_orb_mesh.rotate_using_matrix(rot_mat_step)

	#	apply initial frame offset to axis units
	x_orb_unit = np.dot(x_body_unit, rot_mat_step)
	y_orb_unit = np.dot(y_body_unit, rot_mat_step)
	z_orb_unit = np.dot(z_body_unit, rot_mat_step)

	#	calculate rot axis in orbital frame
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, rot_mat_step)

	#	generate orbital rot vec of step
	rot_vec_orb_unit_step = np.copy(rot_vec_orb_unit)

	#	calculate orbit plane normal in body frame
	orb_norm_body_unit = np.dot(orb_norm_orb_unit, rot_mat_step_inv)

	#	calculate rotation matrix for single time step motion in the orbital frame
	rot_vec_vel_orb_angle_step = np.multiply(rot_vec_vel_orb, render_time_step)
	rot_vec_vel_orb_step = axis2mat(rot_vec_vel_orb_angle_step, orb_norm_orb_unit)

	#	parse input vectors
	observation_vec_orb = obs_vec_orb
	illumination_vec_orb = illum_vec_orb

	###############################################################################################
	#	loop through ephemeris data
	###############################################################################################

	#	set the initial frame offset due to rot_vec_vel_orb
	orbital_frame_offset = np.array((0.0, 0.0, 0.0))

	#	running total of rot vec orbital motion to apply in body frame
	rot_vec_vel_angle_offset = 0.0
	
	for i, (jdate, sat_orb_line, sun_orb_line, earth_orb_line, site_orb_line) in enumerate(
			zip(jd_time_data, sat_orb, sun_orb, earth_orb, site_orb)
		):
	
		###########################################################################################
		#	propagate attitude state in orbital frame

		#	if i is not 0, propergate step attitude
		if i != 0:
			step_attit += np.multiply(attit_mot_body, render_time_step)

#		print('\nstep change in attitude = ', np.multiply(attit_mot_body, render_time_step))

		#	generate body2orb frame offset rotation matrix for new attitude
		rot_mat_step = rot_mat(step_attit)

		#	take inverse for orb2body rotation matrix
		rot_mat_step_inv = np.linalg.inv(rot_mat_step)

		###########################################################################################
		#	propagate rotation vector velocity in body and orbital frames

		#	if i is not 0, propergate step attitude
		if i != 0:
			rot_vec_vel_angle_offset += np.multiply(rot_vec_vel_orb, render_time_step)

#		print('\nstep change in rot vec position = ', np.multiply(rot_vec_vel_orb, render_time_step))

		#	generate rotation matrix for total rotation vector motion in body frame
		rot_vec_vel_body = axis2mat(rot_vec_vel_angle_offset, orb_norm_body_unit)

		#	take inverse for application to vectors
		rot_vec_mot_body_inv = np.linalg.inv(rot_vec_vel_body)

		###########################################################################################
		#	parse orbital data

		#	object positions and velocities in m/s
		sat_orb_pos = sat_orb_line[0:3]
		sat_orb_vel = sat_orb_line[3:6]

		#	orbital data
		sun_orb_vec = sun_orb_line[0:3]
		earth_orb_vec = earth_orb_line[0:3]
		site_orb_vec = site_orb_line[0:3]

		#	orbital unit vectors
		sun_orb_unit = np.divide(sun_orb_vec, np.linalg.norm(sun_orb_vec))
		earth_orb_unit = np.divide(earth_orb_vec, np.linalg.norm(earth_orb_vec))
		site_orb_unit = np.divide(site_orb_vec, np.linalg.norm(site_orb_vec))
	
		###########################################################################################
		#	propagate model in the BODY FRAME
		###########################################################################################

		####	body object mesh remains unchanged

		####	inverse of attitude motion is applied to vector data

		#	calculate body frame vectors (apply frame offset to orbital position vectors)
		sat_body_pos = np.dot(sat_orb_pos, rot_mat_step_inv)
		sat_body_vel = np.dot(sat_orb_vel, rot_mat_step_inv)

		sun_body_vec = np.dot(sun_orb_vec, rot_mat_step_inv) 
		earth_body_vec = np.dot(earth_orb_vec, rot_mat_step_inv)
		site_body_vec = np.dot(site_orb_vec, rot_mat_step_inv)

		####	inverse of orbital motion is applied to vector data

		#	apply motion of rot vec in orbital frame
		sat_body_pos = np.dot(sat_orb_pos, rot_vec_mot_body_inv)
		sat_body_vel = np.dot(sat_orb_vel, rot_vec_mot_body_inv)

		sun_body_vec = np.dot(sun_orb_vec, rot_vec_mot_body_inv) 
		earth_body_vec = np.dot(earth_orb_vec, rot_vec_mot_body_inv)
		site_body_vec = np.dot(site_orb_vec, rot_vec_mot_body_inv)

		#	calculate body unit vectors
		sun_body_unit = np.divide(sun_body_vec, np.linalg.norm(sun_body_vec))
		earth_body_unit = np.divide(earth_body_vec, np.linalg.norm(earth_body_vec))
		site_body_unit = np.divide(site_body_vec, np.linalg.norm(site_body_vec))

		#	custom vectors
		illumination_vec_body = np.dot(illumination_vec_orb, rot_mat_step_inv)
		observation_vec_body = np.dot(observation_vec_orb, rot_mat_step_inv)

		###########################################################################################
		#	evolve object geometry in orbital frame
		###########################################################################################

		####	orbital frame vectors remain unchanged
		####	attitude motion is applied orbital object mesh

		#	calculate the angular offset applied this step due to attitude motion
		if i == 0:
			rot_angx = 0
			rot_angy = 0
			rot_angz = 0
		else:
			rot_angx = attit_mot_body[0] * render_time_step
			rot_angy = attit_mot_body[1] * render_time_step
			rot_angz = attit_mot_body[2] * render_time_step

		###########################################################################################
		#	apply x-rotation in the body x-axis to the orbital object mesh
		att_mat = axis2mat(rot_angx, x_orb_unit)
		temp_orb_mesh.rotate_using_matrix(att_mat)

		#	apply x-rotation in the body frame to the unit axes
		x_orb_unit = np.dot(x_orb_unit, att_mat)
		y_orb_unit = np.dot(y_orb_unit, att_mat)
		z_orb_unit = np.dot(z_orb_unit, att_mat)

		###########################################################################################
		#	apply y-rotation in the NEW body y-axis frame to the NEW orbital object mesh
		att_mat = axis2mat(rot_angy, y_orb_unit)
		temp_orb_mesh.rotate_using_matrix(att_mat)

		#	apply y-rotation in the body frame to the NEW unit axes
		x_orb_unit = np.dot(x_orb_unit, att_mat)
		y_orb_unit = np.dot(y_orb_unit, att_mat)
		z_orb_unit = np.dot(z_orb_unit, att_mat)

		###########################################################################################
		#	apply z-rotation in the NEW body z-axis frame to the NEW orbital object mesh
		att_mat = axis2mat(rot_angz, z_orb_unit)
		temp_orb_mesh.rotate_using_matrix(att_mat)

		#	apply z-rotation in the body frame to the NEW unit axes
		x_orb_unit = np.dot(x_orb_unit, att_mat)
		y_orb_unit = np.dot(y_orb_unit, att_mat)
		z_orb_unit = np.dot(z_orb_unit, att_mat)

		###########################################################################################
		#	apply rot vec orbital motion
		temp_orb_mesh.rotate_using_matrix(rot_vec_vel_orb_step)

		x_orb_unit = np.dot(x_orb_unit, rot_vec_vel_orb_step)
		y_orb_unit = np.dot(y_orb_unit, rot_vec_vel_orb_step)
		z_orb_unit = np.dot(z_orb_unit, rot_vec_vel_orb_step)

		#	generate new position of rotation vector in orbital frame
		rot_vec_orb_unit_step = np.dot(rot_vec_orb_unit_step, rot_vec_vel_orb_step)

		###########################################################################################
		#	generate inertial data
		###########################################################################################

		####	orbital angular velocity is applied to orbital vector data
		####	orbital angular velocity is applied to orbital object mesh

		###########################################################################################
		#	apply total orbital motion frame offset

		#	calculate total orbital motion frame offset of time
		if i != 0:
			orbital_frame_offset = orbital_frame_offset + np.multiply(orb2inertial, render_time_step)
	
		#	generate rotation matrix of step ang
		rot_mat_step_offset = rot_mat(orbital_frame_offset)
	
		#	apply to vectors
		sat_inertial_vel = np.dot(sat_orb_vel, rot_mat_step_offset)
		sun_inertial_vec = np.dot(sun_orb_vec, rot_mat_step_offset)
		earth_inertial_vec = np.dot(earth_orb_vec, rot_mat_step_offset)
		site_inertial_vec = np.dot(site_orb_vec, rot_mat_step_offset)

		#	inertial unit vectors
		sun_inertial_unit = np.divide(sun_inertial_vec, np.linalg.norm(sun_inertial_vec))
		earth_inertial_unit = np.divide(earth_inertial_vec, np.linalg.norm(earth_inertial_vec))
		site_inertial_unit = np.divide(site_inertial_vec, np.linalg.norm(site_inertial_vec))

		#	apply to geometry

		#	generate a temporary inertial mesh from the orbital mesh
		temp_inertial_mesh = mesh.Mesh(temp_orb_mesh.data.copy())

		#	rotate by frame offset
		temp_inertial_mesh.rotate_using_matrix(rot_mat_step_offset)

		#	generate inertial unit vectors from orbital unit vectors
		x_inertial_unit = np.dot(x_orb_unit, rot_mat_step_offset)
		y_inertial_unit = np.dot(y_orb_unit, rot_mat_step_offset)
		z_inertial_unit = np.dot(z_orb_unit, rot_mat_step_offset)

		###########################################################################################
		#	Data Output
		###########################################################################################
	
		#	set image index
		index = i

		###########################################################################################
		#	populate output attitude array
		attitude_out[index, :] = step_attit
	
		###########################################################################################
		#	orbital frame vector data
	
		#	julian date
		vector_data_orb[index, 0] = jdate
		vector_data_orb[index, 1:4] = sun_orb_vec
		vector_data_orb[index, 4:7] = site_orb_vec
		vector_data_orb[index, 7:10] = earth_orb_vec
		vector_data_orb[index, 10:13] = sat_orb_vel
		vector_data_orb[index, 13:16] = x_orb_unit
		vector_data_orb[index, 16:19] = y_orb_unit
		vector_data_orb[index, 19:22] = z_orb_unit
		vector_data_orb[index, 22:25] = illumination_vec_orb
		vector_data_orb[index, 25:28] = observation_vec_orb
		vector_data_orb[index, 28:31] = rot_vec_orb_unit_step 
	
		###########################################################################################
		#	body frame vector data

		#	julian date
		vector_data_body[index, 0] = jdate
		vector_data_body[index, 1:4] = sun_body_vec
		vector_data_body[index, 4:7] = site_body_vec
		vector_data_body[index, 7:10] = earth_body_vec
		vector_data_body[index, 10:13] = sat_body_vel
		vector_data_body[index, 13:16] = x_body_unit
		vector_data_body[index, 16:19] = y_body_unit
		vector_data_body[index, 19:22] = z_body_unit
		vector_data_body[index, 22:25] = illumination_vec_body
		vector_data_body[index, 25:28] = observation_vec_body
		vector_data_body[index, 28:31] = rot_vec_body_unit
	
		###########################################################################################
		#	inertial frame vector data
	
		#	julian date
		vector_data_inertial[index, 0] = jdate
		vector_data_inertial[index, 1:4] = sun_inertial_vec
		vector_data_inertial[index, 4:7] = site_inertial_vec
		vector_data_inertial[index, 7:10] = earth_inertial_vec
		vector_data_inertial[index, 10:13] = sat_inertial_vel
		vector_data_inertial[index, 13:16] = x_inertial_unit
		vector_data_inertial[index, 16:19] = y_inertial_unit
		vector_data_inertial[index, 19:22] = z_inertial_unit
		vector_data_inertial[index, 22:25] = illumination_vec_body
		vector_data_inertial[index, 25:28] = observation_vec_body
		vector_data_inertial[index, 28:31] = rot_vec_body_unit

		###########################################################################################
		#	rendering code
		###########################################################################################

		#	render output files
		render_data_orb = 'render_data_orb.txt'

		#	generate image ID
		if index < 10:
			geom_id = '000' + str(index)
		elif 10 <= index < 100:
			geom_id = '00' + str(index)
		elif 100 <= index < 1000:
			geom_id = '0' + str(index)
		elif 1000 <= index < 10000:
			geom_id = str(index)

		#	saving orbital mesh file
		mesh_file = 'orbital_mesh_{}.stl'.format(geom_id)
		temp_orb_mesh.save(os.path.join(render_geom_dir, mesh_file), mode=stl.Mode.ASCII)

		#	save and end
		if i >= len_record - 1:
			break

#		input('\nTime step complete. Enter to continue.')

	return vector_data_body, vector_data_orb, vector_data_inertial, attitude_out

