#############################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
#############################################################################################
#
#
#
#
#
#
#
#############################################################################################

#	imports
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

def accel_grav(radial_vec, moi, theta, phi):

	#	Earth gravitational parameter [m3/kg]
	earth_mu = 3.986e14

	#	gravitational coefficient
	coef = (3 * earth_mu) / (2 * radial_vec**3)

	#	principle moments of inertia
	I_xx = moi[0, 0]
	I_yy = moi[1, 1]
	I_zz = moi[2, 2]
	
	Ix = I_yy - I_zz
	Iy = I_xx - I_zz
	Iz = I_xx - I_yy

	xang = np.sin(2 * theta) * np.sin(phi)
	yang = np.sin(2 * phi) * np.cos(theta)**2
	zang = np.sin(2 * theta) * np.cos(phi)

	#	torque components
	torque_grav_x = coef * Ix * xang
	torque_grav_y = coef * Iy * yang
	torque_grav_z = coef * Iz * zang

	#	sign correction for upside down
	if phi > (np.pi / 2):
		torque_grav_z = -torque_grav_z
	elif phi < -(np.pi / 2):
		torque_grav_z = -torque_grav_z

	torque_grav = [torque_grav_x, torque_grav_y, torque_grav_z]

	#	multiply by inertial matrix for acceleration of step
	ang_accel_grav = np.dot(torque_grav, np.linalg.inv(moi))

#	print('Gravity gradient Torque = ', torque_grav)
#	print('Gravity gradient Accel = ', ang_accel_grav)

	return ang_accel_grav

def plot_graphs(
		geom_orb_units, vector_orb_units, sat_orb_vel, temp_orb_vectors,
		geom_body_units, vector_body_units, sat_body_vel, body_vectors
	):

	###########################################################################################
	#	ORBITAL FRAME
	###########################################################################################

	###########################################################################################
	#	setup plot data

	x_orb_unit_step, y_orb_unit_step, z_orb_unit_step = geom_orb_units
	sun_orb_unit, site_orb_unit, earth_orb_unit = vector_orb_units

	#	body unit vectors
	x_orb_plot = 4 * (np.vstack(([0, 0, 0], x_orb_unit_step)))
	y_orb_plot = 4 * (np.vstack(([0, 0, 0], y_orb_unit_step)))
	z_orb_plot = 4 * (np.vstack(([0, 0, 0], z_orb_unit_step)))

	#	sun/site body vectors
	sun_vec_plot = 4 * (np.vstack(([0, 0, 0], sun_orb_unit)))
	site_vec_plot = 4 * (np.vstack(([0, 0, 0], site_orb_unit)))
	earth_vec_plot = 4 * (np.vstack(([0, 0, 0], earth_orb_unit)))

	#	radial vector in the body frame
	rad_orb_vec = 4 * (np.vstack(([0, 0, 0], -earth_orb_unit)))

	#	velocity vector in the body frame
	sat_orb_vel_unit = np.divide(sat_orb_vel, np.linalg.norm(sat_orb_vel))
	vel_orb_vec = 4 * (np.vstack(([0, 0, 0], sat_orb_vel_unit)))

	###########################################################################################
	#	plot graph

	#	generate figure
	fig = plt.figure(num=2, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
	ax1 = mplot3d.Axes3D(fig)
	
	#	geometry plot
	ax1.add_collection3d(
		mplot3d.art3d.Poly3DCollection(temp_orb_vectors, facecolor=[0.5, 0.5, 0.5])
	)

	#	plot body direction vectors
	ax1.plot(x_orb_plot[:, 0], x_orb_plot[:, 1], x_orb_plot[:, 2], color='black')
	ax1.plot(y_orb_plot[:, 0], y_orb_plot[:, 1], y_orb_plot[:, 2], color='black')
	ax1.plot(z_orb_plot[:, 0], z_orb_plot[:, 1], z_orb_plot[:, 2], color='black')
	ax1.text(*x_orb_plot[1, :], 'x_axis_orb')
	ax1.text(*y_orb_plot[1, :], 'y_axis_orb')
	ax1.text(*z_orb_plot[1, :], 'z_axis_orb')

	#	plot observation and illumination vectors
	ax1.plot(sun_vec_plot[:, 0], sun_vec_plot[:, 1], sun_vec_plot[:, 2], color='r')
	ax1.plot(site_vec_plot[:, 0], site_vec_plot[:, 1], site_vec_plot[:, 2], color='b')
	ax1.text(*sun_vec_plot[1, :], 'sun_vec_orb')
	ax1.text(*site_vec_plot[1, :], 'site_vec_orb')

	ax1.plot(rad_orb_vec[:, 0], rad_orb_vec[:, 1], rad_orb_vec[:, 2], color='g')
	ax1.text(*rad_orb_vec[1, :], 'radial_vec_orb')

	ax1.plot(vel_orb_vec[:, 0], vel_orb_vec[:, 1], vel_orb_vec[:, 2], color='m')
	ax1.text(*vel_orb_vec[1, :], 'velocity_vec_orb')

	ax1.set_xlim([-4, 4])
	ax1.set_ylim([-4, 4])
	ax1.set_zlim([-4, 4])
	ax1.set_aspect('equal')
	ax1.set_xlabel('x axis')
	ax1.set_ylabel('y axis')
	ax1.set_zlabel('z axis')
	ax1.set_title('Orbital Frame')

	###########################################################################################
	#	BODY FRAME
	###########################################################################################

	###########################################################################################
	#	setup plot data

	x_body_unit, y_body_unit, z_body_unit = geom_body_units
	sun_body_unit, site_body_unit, earth_body_unit = vector_body_units

	#	body unit vectors
	x_body_plot = 4 * (np.vstack(([0, 0, 0], x_body_unit)))
	y_body_plot = 4 * (np.vstack(([0, 0, 0], y_body_unit)))
	z_body_plot = 4 * (np.vstack(([0, 0, 0], z_body_unit)))

	#	sun/site body vectors
	sun_vec_plot = 4 * (np.vstack(([0, 0, 0], sun_body_unit)))
	site_vec_plot = 4 * (np.vstack(([0, 0, 0], site_body_unit)))
	earth_vec_plot = 4 * (np.vstack(([0, 0, 0], earth_body_unit)))

	#	radial vector in the body frame
	rad_body_vec = 4 * (np.vstack(([0, 0, 0], -earth_body_unit)))

	#	velocity vector in the body frame
	sat_body_vel_unit = np.divide(sat_body_vel, np.linalg.norm(sat_body_vel))
	vel_body_vec = 4 * (np.vstack(([0, 0, 0], sat_body_vel_unit)))

	###########################################################################################
	#	plot graph

	#	generate figure
	fig = plt.figure(num=3, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
	ax1 = mplot3d.Axes3D(fig)
	
	#	geometry plot
	ax1.add_collection3d(
		mplot3d.art3d.Poly3DCollection(body_vectors, facecolor=[0.5, 0.5, 0.5])
	)

	#	plot body direction vectors
	ax1.plot(x_body_plot[:, 0], x_body_plot[:, 1], x_body_plot[:, 2], color='black')
	ax1.plot(y_body_plot[:, 0], y_body_plot[:, 1], y_body_plot[:, 2], color='black')
	ax1.plot(z_body_plot[:, 0], z_body_plot[:, 1], z_body_plot[:, 2], color='black')
	ax1.text(*x_body_plot[1, :], 'x_axis_body')
	ax1.text(*y_body_plot[1, :], 'y_axis_body')
	ax1.text(*z_body_plot[1, :], 'z_axis_body')

	#	plot observation and illumination vectors
	ax1.plot(sun_vec_plot[:, 0], sun_vec_plot[:, 1], sun_vec_plot[:, 2], color='r')
	ax1.plot(site_vec_plot[:, 0], site_vec_plot[:, 1], site_vec_plot[:, 2], color='b')
	ax1.text(*sun_vec_plot[1, :], 'sun_vec_body')
	ax1.text(*site_vec_plot[1, :], 'site_vec_body')

	ax1.plot(rad_body_vec[:, 0], rad_body_vec[:, 1], rad_body_vec[:, 2], color='g')
	ax1.text(*rad_body_vec[1, :], 'radial_vec_body')

	ax1.plot(vel_body_vec[:, 0], vel_body_vec[:, 1], vel_body_vec[:, 2], color='m')
	ax1.text(*vel_body_vec[1, :], 'velocity_vec_body')

	ax1.set_xlim([-4, 4])
	ax1.set_ylim([-4, 4])
	ax1.set_zlim([-4, 4])
	ax1.set_aspect('equal')
	ax1.set_xlabel('x axis')
	ax1.set_ylabel('y axis')
	ax1.set_zlabel('z axis')
	ax1.set_title('Body Frame')

	plt.show()
