###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	numba
import numba

#	progress bar
import progressbar

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

###################################################################################################
#	define plotable arrow class

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)


###################################################################################################
#	Function: 		jday_jfrac
#	Description: 	converts a julian date to julian day and day fraction
#
###################################################################################################

@numba.jit(nopython=True)
def jday_jfrac(jd):

	#	convert sim start to julian day and julian fraction
	if jd - np.floor(jd) < 0.5:
		jday = np.floor(jd) - 0.5
		jfrac = jd - jday

	elif jd - np.floor(jd) > 0.5:
		jday = np.floor(jd) + 0.5
		jfrac = jd - jday

	elif jd - np.floor(jd) == 0.5:
		jday = jd
		jfrac = 0.0

	return jday, jfrac


###################################################################################################
#	Function: 		rot1
#	Description: 	rotates a provided vector by angle around x-axis
#
###################################################################################################

@numba.jit(nopython=True)
def rot1(vector, rotation_angle):

	cosang = np.cos(rotation_angle)
	sinang = np.sin(rotation_angle)

	rot1_mat = np.array([
		[1.0, 0.0, 0.0],
		[0.0, cosang, -sinang],
		[0.0, sinang, cosang],
	])

	outvec = vector.dot(rot1_mat)

	return outvec

###################################################################################################
#	Function: 		rot2
#	Description: 	rotates a provided vector by angle around y-axis
#
###################################################################################################

@numba.jit(nopython=True)
def rot2(vector, rotation_angle):

	cosang = np.cos(rotation_angle)
	sinang = np.sin(rotation_angle)

	rot2_mat = np.array([
		[cosang, 0.0, sinang],
		[0.0, 1.0, 0.0],
		[-sinang, 0.0, cosang]
	])

	outvec = vector.dot(rot2_mat)

	return outvec

###################################################################################################
#	Function: 		rot3
#	Description: 	rotates a provided vector by angle around z-axis
#
###################################################################################################

@numba.jit(nopython=True)
def rot3(vector, rotation_angle):

	cosang = np.cos(rotation_angle)
	sinang = np.sin(rotation_angle)

	rot3_mat = np.array([
		[cosang, -sinang, 0.0],
		[sinang, cosang, 0.0],
		[0.0, 0.0, 1.0]
	])

	outvec = vector.dot(rot3_mat)

	return outvec

###################################################################################################
#	Function: 		calc_cross
#	Description: 	calculates the cross product of two vectors
#
###################################################################################################

@numba.jit(nopython=True)
def calc_cross(a, b):
    c = [a[1]*b[2] - a[2]*b[1],
         a[2]*b[0] - a[0]*b[2],
         a[0]*b[1] - a[1]*b[0]]

    return c


###################################################################################################
#	Function: 		vec_ang
#	Description: 	calculates angle between two vectors
#
###################################################################################################

#	angle between two vectors
def vec_ang(vector_1, vector_2):

	#	convert to unit vectors
	unit_vector_1 = vector_1 / np.linalg.norm(vector_1)
	unit_vector_2 = vector_2 / np.linalg.norm(vector_2)
	
	#	calculate dot product
	dot_product = np.dot(unit_vector_1, unit_vector_2)
	
	#	calculate angle
	angle = np.arccos(dot_product)

	return angle

###################################################################################################
#	Function: 		rot_mat
#	Description: 	generates the 3-axis Euler rotation matrix
#
###################################################################################################

@numba.jit(nopython=True)
def rot_mat(attit_mot):

	rot_mat = np.array([
		[
			np.cos(attit_mot[1]) * np.cos(attit_mot[2]),
			-np.cos(attit_mot[0]) * np.sin(attit_mot[2])
			+ np.sin(attit_mot[0]) * np.sin(attit_mot[1]) * np.cos(attit_mot[2]),
			np.sin(attit_mot[0]) * np.sin(attit_mot[2])
			+ np.cos(attit_mot[0]) * np.sin(attit_mot[1]) * np.cos(attit_mot[2])
		],
		[
			np.cos(attit_mot[1]) * np.sin(attit_mot[2]),
			np.cos(attit_mot[0]) * np.cos(attit_mot[2])
			+ np.sin(attit_mot[0]) * np.sin(attit_mot[1]) * np.sin(attit_mot[2]),
			-np.sin(attit_mot[0]) * np.cos(attit_mot[2])
			+ np.cos(attit_mot[0]) * np.sin(attit_mot[1]) * np.sin(attit_mot[2])
		],
		[
			-np.sin(attit_mot[1]),
			np.sin(attit_mot[0]) * np.cos(attit_mot[1]),
			np.cos(attit_mot[0]) * np.cos(attit_mot[1])
		]
	])

	return rot_mat

###################################################################################################
#	Function: 		mat2axis
#	Description: 	converts a rotation matrix to axis angle representation
#
###################################################################################################

@numba.jit(nopython=True)
def mat2axis(att_mat):

	A11 = att_mat[0, 0]
	A12 = att_mat[0, 1]
	A13 = att_mat[0, 2]
	A21 = att_mat[1, 0]
	A22 = att_mat[1, 1]
	A23 = att_mat[1, 2]
	A31 = att_mat[2, 0]
	A32 = att_mat[2, 1]
	A33 = att_mat[2, 2]

	theta = np.arccos((A11 + A22 + A33 - 1) / 2)

	e1 = (A32 - A23) / (2 * np.sin(theta))
	e2 = (A13 - A31) / (2 * np.sin(theta))
	e3 = (A21 - A12) / (2 * np.sin(theta))

	return theta, [e1, e2, e3]

###################################################################################################
#	Function: 		axis2mat
#	Description: 	converts axis-angle to rotation matrix representation
#
###################################################################################################

@numba.jit(nopython=True)
def axis2mat(angle, axis):

	ux = axis[0]
	uy = axis[1]
	uz = axis[2]

	att_mat = np.zeros((3, 3))

	att_mat[0, 0] = np.cos(angle) + (ux**2 * (1 - np.cos(angle)))
	att_mat[0, 1] = ux * uy * (1 - np.cos(angle)) - (uz * np.sin(angle))
	att_mat[0, 2] = ux * uz * (1 - np.cos(angle)) + (uy * np.sin(angle))

	att_mat[1, 0] = uy * ux * (1 - np.cos(angle)) + (uz * np.sin(angle))
	att_mat[1, 1] = np.cos(angle) + (uy**2 * (1 - np.cos(angle)))
	att_mat[1, 2] = uy * uz * (1 - np.cos(angle)) - (ux * np.sin(angle))

	att_mat[2, 0] = uz * ux * (1 - np.cos(angle)) - (uy * np.sin(angle))
	att_mat[2, 1] = uz * uy * (1 - np.cos(angle)) + (ux * np.sin(angle))
	att_mat[2, 2] = np.cos(angle) + (uz**2 * (1 - np.cos(angle)))

	return att_mat

###################################################################################################
#	Function: 		mat2angles
#	Description: 	extracts Euler angles from a rotation matrix
#
###################################################################################################

@numba.jit(nopython=True)
def mat2angles(att_mat):

	A11 = att_mat[0, 0]
	A12 = att_mat[0, 1]
	A13 = att_mat[0, 2]
	A21 = att_mat[1, 0]
	A22 = att_mat[1, 1]
	A23 = att_mat[1, 2]
	A31 = att_mat[2, 0]
	A32 = att_mat[2, 1]
	A33 = att_mat[2, 2]

	theta1 = -np.arcsin(A31)
	theta2 = np.pi - theta1

	psi1 = np.arctan2(A32 / np.cos(theta1), A33 / np.cos(theta1))
	psi2 = np.arctan2(A32 / np.cos(theta2), A33 / np.cos(theta2))

	phi1 = np.arctan2(A21 / np.cos(theta1), A11 / np.cos(theta1))
	phi2 = np.arctan2(A21 / np.cos(theta2), A11 / np.cos(theta2))

	return psi1, theta1, phi1, psi2, theta2, phi2

###################################################################################################
#	Function: 		site_vec
#	Description: 	converts site latitude, longitude and altitude to ITRF vector
#
###################################################################################################

@numba.jit(nopython=True)
def site_vec(latgd, lon, alt):

	# const
	earth_r = 6378137.0
	flatness = 1.0 / 298.257223563
	earth_e = np.sqrt(2.0 * flatness - flatness**2)
	earth_e2 = earth_e**2

	sinlat = np.sin(latgd)
	coslat = np.cos(latgd)
	
	sinlon = np.sin(lon)
	coslon = np.cos(lon)
	
	cearth = earth_r / np.sqrt(1 - (earth_e2 * sinlat**2))
	rdel = (cearth + alt ) * coslat
	rk = ((1 - earth_e2) * cearth + alt ) * sinlat
	
	temp1 = rdel * coslon
	temp2 = rdel * sinlon
	
	r_site = [temp1, temp2, rk]

	return r_site

###################################################################################################
#	Function: 		gtime
#	Description: 	caluclates sidereal angle of date
#
###################################################################################################

@numba.jit(nopython=True)
def gtime(jdayut1, jfracut1):

	jdut1 = jdayut1 + jfracut1
	tut1 = (jdut1 - 2451545.0) / 36525.0

	gmst_total = np.radians(67310.54841
		+ (876600.0 * 3600.0 + 8640184.812866) * tut1
		+ 0.093104 * tut1 * tut1
		- 6.2E-6 * tut1 * tut1 * tut1
	)

	gmst = np.remainder(gmst_total * (360.0/86400.0), 2.0 * np.pi)

	return gmst

###################################################################################################
#	Function: 		teme2itrf
#	Description: 	convert TEME frame to ITRF
#
###################################################################################################

@numba.jit(nopython=True)
def teme2itrf(r_teme, v_teme, jdayut1, jfracut1, lod, x_p, y_p):

	# greenwich mean sidreal time
	gmst = gtime(jdayut1, jfracut1)

	# sidreal motion rotation
	st = np.zeros((3,3))

	st[0,0] =  np.cos(gmst)
	st[0,1] = -np.sin(gmst)
	st[0,2] = 0.0
	st[1,0] =  np.sin(gmst)
	st[1,1] =  np.cos(gmst)
	st[1,2] = 0.0
	st[2,0] = 0.0
	st[2,1] = 0.0
	st[2,2] = 1.0

	inv_st = np.transpose(st)

	# polar motion rotation
	pm = np.zeros((3,3))

	cosxp = np.cos(x_p)
	sinxp = np.sin(x_p)
	cosyp = np.cos(y_p)
	sinyp = np.sin(y_p)

	pm[0,0] =  cosxp
	pm[0,1] =  0.0
	pm[0,2] = -sinxp
	pm[1,0] =  sinxp * sinyp
	pm[1,1] =  cosyp
	pm[1,2] =  cosxp * sinyp
	pm[2,0] =  sinxp * cosyp
	pm[2,1] = -sinyp
	pm[2,2] =  cosxp * cosyp

	inv_pm = np.transpose(pm)

	#	position transformation
	r_pef = np.dot(np.transpose(st), r_teme)
	r_itrf = np.dot(np.transpose(pm), r_pef)

	#	velocity transformation
	thetasa = 7.29211514670698e-05 * (1.0  - lod / 86400.0 )
	omega_earth = np.array((0.0, 0.0, thetasa))

	temp = np.array(calc_cross(omega_earth, r_pef))
	temp2 = inv_st.dot(v_teme)

	v_pef = np.subtract(temp2, temp)
	v_itrf = inv_pm.dot(v_pef)

	return r_itrf, v_itrf

###################################################################################################
#	Function: 		itrf2gcrf
#	Description: 	convert ITRF frame to GCRF
#
###################################################################################################

@numba.jit(nopython=True)
def itrf2gcrf(sat_itrf_pos, sat_itrf_vel, inv_pm, inv_st, inv_pn, lod):

	#	earth rotation angle
	thetasa= 7.29211514670698e-05 * (1.0  - lod / 86400.0 )
	omegaearth = np.array((0, 0, thetasa))
	
	#	object GCRF position / velocity
	sat_pef_pos = inv_pm.dot(sat_itrf_pos)
	temp = inv_st.dot(sat_pef_pos)
	sat_gcrf_pos = inv_pn.dot(temp)
	
	sat_pef_vel = inv_pm.dot(sat_itrf_vel)
	temp2 = np.array(calc_cross(omegaearth, sat_pef_pos))
	temp3 = np.add(sat_pef_vel, temp2)

	temp4 = inv_st.dot(temp3)
	sat_gcrf_vel = inv_pn.dot(temp4)

	return sat_gcrf_pos, sat_gcrf_vel

###################################################################################################
#	Function: 		gcrf2itrf
#	Description: 	convert GCRF frame to ITRF
#
###################################################################################################

@numba.jit(nopython=True)
def gcrf2itrf(sat_gcrf_pos, sat_gcrf_vel, pm, st, pn, lod):

	#	earth rotation angle
	thetasa= 7.29211514670698e-05 * (1.0  - lod / 86400.0 )
	omegaearth = np.array((0, 0, thetasa))
	
	temp = pn.dot(sat_gcrf_pos)
	sat_pef_pos = st.dot(temp)
	sat_itrf_pos = pm.dot(sat_pef_pos)

	temp = pn.dot(sat_gcrf_vel)
	temp2 = st.dot(temp)

	temp3 = np.array(calc_cross(omegaearth, sat_pef_pos))

	sat_pef_vel = np.subtract(temp2, temp3)
	sat_itrf_vel = pm.dot(sat_pef_vel)

	return sat_itrf_pos, sat_itrf_vel

###################################################################################################
#	Function: 		itrf2razel
#	Description: 	convert ITRF frame to RAZEL
#
###################################################################################################

@numba.jit(nopython=True)
def itrf2razel(site_latgd, site_lon, site_alt, sat_itrf):

	small = 1e-8

	#	calculate site vector in km
	site_vect = np.array(site_vec(site_latgd, site_lon, site_alt))
	site_vect = np.divide(site_vect, 1000.0)

	#	object slant vector
	slant_vect = np.subtract(sat_itrf, site_vect)

	#	convert to intermediary SEZ frame
	slant_temp = rot3(slant_vect, site_lon)
	slant_sez = rot2(slant_temp, (0.5 * np.pi) - site_latgd)

	#	extent in x-y plane
	temp = np.sqrt(slant_sez[0] * slant_sez[0] + slant_sez[1] * slant_sez[1])

	#	elevation

	if temp < small:
		el = np.sin(slant_sez[2]) * np.pi / 2

	else:
		sez_mag = np.linalg.norm(slant_sez)
		el = np.arcsin(slant_sez[2] / sez_mag)

	#	azimuth
	if temp < small:
		az = np.arctan2(slant_sez[1], -slant_sez[0])

	else:
		az = np.arctan2(slant_sez[1] / temp, -slant_sez[0] / temp)

	#	sat range placeholder
	sat_range = sez_mag

	razel = np.array((sat_range, az, el))

	return razel

###################################################################################################
#	Function: 		scrf2orb
#	Description: 	convert SCRF to ORBITAL frame
#
###################################################################################################

@numba.jit(nopython=True)
def scrf2orb(sun_scrf, earth_scrf, site_scrf, sat_scrf_vel):

	#	x unit vector
	x_unit = np.array([1.0, 0.0, 0.0])

	#	z unit vector
	z_unit = np.array([0.0, 0.0, 1.0])

	#	get plane normal vector
	plane_norm = np.array(calc_cross(-earth_scrf, sat_scrf_vel))

	###############################################################################################
	#	find z rotation angle for earth vector in negative x direction
	earth_xy = np.array([earth_scrf[0], earth_scrf[1], 0.0])

	#	angle to z
	rot_z = np.arccos(np.divide(
		earth_xy.dot(-x_unit),
		(np.multiply(np.linalg.norm(earth_xy), np.linalg.norm(-x_unit)))
	))

	#	rot_z negative check
	if earth_xy[1] > 0:
		rot_z = -rot_z

	#	rotate earth by z angle
	earth_rotz = rot3(earth_scrf, rot_z)
	sun_rotz = rot3(sun_scrf, rot_z)
	site_rotz = rot3(site_scrf, rot_z)
	vel_rotz = rot3(sat_scrf_vel, rot_z)
	plane_norm_rotz = rot3(plane_norm, rot_z)

	###############################################################################################
	#	find required y angle

	#	new earth vector xz components
	earth_xz = np.array([earth_rotz[0], 0.0, earth_rotz[2]])

	#	angle of earth vector to the negative-x direction, in the y-axis
	rot_y = np.arccos(np.divide(
		earth_xz.dot(-x_unit),
		(np.multiply(np.linalg.norm(earth_xz), np.linalg.norm(-x_unit)))
	))

	#	rot_y negative check
	if earth_xz[2] < 0:
		rot_y = -rot_y

	#	apply

	earth_roty = rot2(earth_rotz, rot_y)
	sun_roty = rot2(sun_rotz, rot_y)
	site_roty = rot2(site_rotz, rot_y)
	vel_roty = rot2(vel_rotz, rot_y)
	plane_norm_roty = rot2(plane_norm_rotz, rot_y)

	###############################################################################################
	#	x-angle

	normal_yz = np.array([0.0, plane_norm_roty[1], plane_norm_roty[2]])

	rot_x = np.arccos(np.divide(
		normal_yz.dot(z_unit),
		(np.multiply(np.linalg.norm(normal_yz), np.linalg.norm(z_unit)))
	))

	#	rot_x negative check
	if normal_yz[1] > 0:
		rot_x = -rot_x

	earth_orb = rot1(earth_roty, rot_x)
	sun_orb = rot1(sun_roty, rot_x)
	site_orb = rot1(site_roty, rot_x)
	vel_orb = rot1(vel_roty, rot_x)
	plane_norm_orb = rot1(plane_norm_roty, rot_x)


	return sun_orb, earth_orb, site_orb, vel_orb

###################################################################################################
#	Function: 		body2orb_mat
#	Description: 	generate Euler rotation matrix to convert from body to orbital frame
#
###################################################################################################

@numba.jit(nopython=True)
def body2orb_mat(psi, phi, theta):

	mat = np.zeros((3, 3))

	mat[0, 0] = np.cos(theta) * np.cos(phi)
	mat[0, 1] = -np.sin(theta)
	mat[0, 2] = np.sin(phi) * np.cos(theta)

	mat[1, 0] = np.cos(phi) * np.sin(theta) * np.cos(psi) + np.sin(phi) * np.sin(psi)
	mat[1, 1] = np.cos(phi) * np.cos(theta)
	mat[1, 2] = np.sin(phi) * np.sin(theta) * np.cos(psi) - np.cos(phi) * np.sin(psi)

	mat[2, 0] = np.cos(phi) * np.sin(theta) * np.sin(psi) - np.sin(phi) * np.cos(psi)
	mat[2, 1] = np.sin(psi) * np.cos(theta)
	mat[2, 2] = np.sin(phi) * np.sin(theta) * np.sin(psi) + np.cos(phi) * np.cos(psi)

	return mat

###################################################################################################
#	Function: 		orb2body_vel
#	Description: 	convert velocity from orbital to body frame
#
###################################################################################################

@numba.jit(nopython=True)
def orb2body_vel(psi, phi, theta, psi_dot, phi_dot, theta_dot):

	ib = ((psi_dot * np.cos(theta) * np.cos(phi)) - (theta_dot * np.sin(phi)))

	jb = phi_dot - (psi_dot * np.sin(theta))

	kb = (psi_dot * np.sin(phi) * np.cos(theta)) + (theta_dot * np.cos(phi))

	return np.array([ib, jb, kb])

###################################################################################################
#	Function: 		orb2inertial_vel
#	Description: 	convert velocity from orbital to inertial frame
#
###################################################################################################

# @numba.jit(nopython=True)
def orb2inertial_vel(psi, phi, theta, psi_dot, phi_dot, theta_dot, orbital_mot):

	ib = (
		(
			(psi_dot * np.cos(theta) * np.cos(phi)) - \
			(theta_dot * np.sin(phi))
		) - \
		orbital_mot * (
			(np.cos(phi) * np.sin(theta) * np.sin(psi)) - \
			(np.sin(phi) * np.cos(psi))
		)
	)

	jb = (phi_dot - (psi_dot * np.sin(theta)) - orbital_mot * (np.sin(phi) * np.cos(theta)))

	kb = (psi_dot * np.sin(phi) * np.cos(theta)) + (theta_dot * np.cos(phi)) - \
		orbital_mot * (
			(np.sin(phi) * np.sin(theta) * np.sin(psi)) + \
			(np.cos(phi) * np.cos(psi))
		)

	return np.array([-ib, -jb, -kb])

###################################################################################################
#	Function: 		gen_vectors
#	Description: 	generate the required vectors for light curve generation
#
###################################################################################################

def gen_vectors(
		path_data, time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_frame_offset, init_frame_vel,
		observation_vector, illumination_vector
	):

	###############################################################################################
	#	initialisation
	###############################################################################################

#	print('\n###############################################################')
#	print('INPUTS')
#	print('###############################################################')
#	print('\nOrb data going into gen vectors:')
#	print('\nsun orb = \n', sun_orb)
#	print('\ninit_frame_vel = ', init_frame_vel.astype(str))

	###############################################################################################
	#	parse inputs

	#	set simulation start julian date
	jdate_start = sun_orb[0, 0]

	#	parse initial attitude and attitude motion
	frame_offset = np.copy(init_frame_offset)	#	offset between orbital and body frames
	frame_velocity = np.copy(init_frame_vel)	#	angular velocity of body frame

	###############################################################################################
	#	initialise variables

	#	initialise output arrays
	attitude_out = np.zeros((len_record, 3))
	vector_data_body = np.zeros((len_record, 28))
	vector_data_orb = np.zeros((len_record, 28))
	vector_data_inertial = np.zeros((len_record, 28))

	#	define body axis vectors of object mesh
	x_body_unit = np.array((1.0, 0.0, 0.0))
	y_body_unit = np.array((0.0, 1.0, 0.0))
	z_body_unit = np.array((0.0, 0.0, 1.0))

	#	apply initial frame offset
	rot_mat_step = rot_mat(init_frame_offset)

	#	apply initial frame offset to axis units
	x_orb_unit = np.dot(x_body_unit, rot_mat_step)
	y_orb_unit = np.dot(y_body_unit, rot_mat_step)
	z_orb_unit = np.dot(z_body_unit, rot_mat_step)

	#	frame offset due to orbital motion
	orbital_frame_offset = np.array((0.0, 0.0, 0.0))

	#	parse input vectors
	obs_vec_orb = observation_vector
	illum_vec_orb = illumination_vector

	#	angle output arrays
	obs_orb_plot_angles = []
	obs_body_plot_angles = []
	illum_orb_plot_angles = []
	illum_body_plot_angles = []
	rot_orb_plot_angles = []
	rot_body_plot_angles = []
	geomz_orb_plot_angles = []
	geomz_body_plot_angles = []

	###############################################################################################
	#	loop through ephemeris data
	###############################################################################################

	for i, (sat_orb_line, sun_orb_line, earth_orb_line,	site_orb_line) in enumerate(
			zip(sat_orb, sun_orb, earth_orb, site_orb)
		):

		###########################################################################################
		#	time
		 	
		#	set date and previous date
		jdate = sun_orb_line[0]
		 
		###########################################################################################
		#	propagate velocities

		#	if i is not 0, calculate new total frame offset
		if i != 0:
			frame_offset = frame_offset + np.multiply(frame_velocity, time_step)

			#	correct x
			if frame_offset[0] >= (2 * np.pi):
				frame_offset[0] = frame_offset[0] % (2 * np.pi) 

			elif frame_offset[0] < 0:
				frame_offset[0] = (2 * np.pi) - abs(frame_offset[0])

			#	correct y 
			if frame_offset[1] >= (2 * np.pi):
				frame_offset[1] = frame_offset[1] % (2 * np.pi) 

			elif frame_offset[1] < 0:
				frame_offset[1] = (2 * np.pi) - abs(frame_offset[1])

			#	correct z
			if frame_offset[2] >= (2 * np.pi):
				frame_offset[2] = frame_offset[2] % (2 * np.pi) 

			elif frame_offset[2] < 0:
				frame_offset[2] = (2 * np.pi) - abs(frame_offset[2])

		#	generate body2orb frame offset rotation matrix
		rot_mat_step = rot_mat(frame_offset)
		
		#	take inverse for orb2body rotation matrix
		rot_mat_step_inv = np.linalg.inv(rot_mat_step)

		###########################################################################################
		#	parse orbital data

		#	object positions and velocities in m/s
		sat_orb_pos = sat_orb_line[1:4]
		sat_orb_vel = sat_orb_line[4:7]

		#	orbital data
		sun_orb_vec = sun_orb_line[1:4]
		earth_orb_vec = earth_orb_line[1:4]
		site_orb_vec = site_orb_line[1:4]

		#	orbital unit vectors
		sun_orb_unit = np.divide(sun_orb_vec, np.linalg.norm(sun_orb_vec))
		earth_orb_unit = np.divide(earth_orb_vec, np.linalg.norm(earth_orb_vec))
		site_orb_unit = np.divide(site_orb_vec, np.linalg.norm(site_orb_vec))


		##	TEST
		rot_vec_body = np.array([1.0, 0.0, 0.0])

		orb_norm_orb = np.array([0.0, 0.0, 1.0])

		rot_vec_orb = np.dot(rot_vec_body, rot_mat_step)
		orb_norm_orb_body = np.dot(orb_norm_orb, rot_mat_step_inv)

	
		###########################################################################################
		#	propagate model in the BODY FRAME
		###########################################################################################

		####	body object mesh remains unchanged
		####	attitude motion is applied to vector data

		#	calculate body frame vectors (apply frame offset to orbital position vectors)
		sat_body_pos = np.dot(sat_orb_pos, rot_mat_step_inv)
		sat_body_vel = np.dot(sat_orb_vel, rot_mat_step_inv)
		sun_body_vec = np.dot(sun_orb_vec, rot_mat_step_inv) 
		earth_body_vec = np.dot(earth_orb_vec, rot_mat_step_inv)
		site_body_vec = np.dot(site_orb_vec, rot_mat_step_inv)

		#	calculate body unit vectors
		sun_body_unit = np.divide(sun_body_vec, np.linalg.norm(sun_body_vec))
		earth_body_unit = np.divide(earth_body_vec, np.linalg.norm(earth_body_vec))
		site_body_unit = np.divide(site_body_vec, np.linalg.norm(site_body_vec))

		#	custom vectors
		illum_vec_body = np.dot(illum_vec_orb, rot_mat_step_inv)
		obs_vec_body = np.dot(obs_vec_orb, rot_mat_step_inv)

		###########################################################################################
		#	evolve object geometry in orbital frame
		###########################################################################################

		####	orbital frame vectors remain unchanged
		####	attitude motion is applied orbital object mesh

		#	calculate the total angular offset applied this step to each body axis
		if i == 0:
			rot_angx = 0
			rot_angy = 0
			rot_angz = 0
		else:
			rot_angx = init_frame_vel[0] * time_step
			rot_angy = init_frame_vel[1] * time_step
			rot_angz = init_frame_vel[2] * time_step

		###########################################################################################
		#	apply x-rotation in the body x-axis to the orbital object mesh
		att_mat = axis2mat(rot_angx, x_orb_unit)

		#	apply x-rotation in the body frame to the unit axes
		x_orb_unit = np.dot(x_orb_unit, att_mat)
		y_orb_unit = np.dot(y_orb_unit, att_mat)
		z_orb_unit = np.dot(z_orb_unit, att_mat)

		###########################################################################################
		#	apply y-rotation in the NEW body y-axis frame to the NEW orbital object mesh
		att_mat = axis2mat(rot_angy, y_orb_unit)

		#	apply y-rotation in the body frame to the NEW unit axes
		x_orb_unit = np.dot(x_orb_unit, att_mat)
		y_orb_unit = np.dot(y_orb_unit, att_mat)
		z_orb_unit = np.dot(z_orb_unit, att_mat)

		###########################################################################################
		#	apply z-rotation in the NEW body z-axis frame to the NEW orbital object mesh
		att_mat = axis2mat(rot_angz, z_orb_unit)

		#	apply z-rotation in the body frame to the NEW unit axes
		x_orb_unit = np.dot(x_orb_unit, att_mat)
		y_orb_unit = np.dot(y_orb_unit, att_mat)
		z_orb_unit = np.dot(z_orb_unit, att_mat)

		###########################################################################################
		#	generate inertial data
		###########################################################################################

		####	orbital angular velocity is applied to orbital vector data
		####	orbital angular velocity is applied to orbital object mesh

		###########################################################################################
		#	apply total orbital motion frame offset

		#	calculate total orbital motion frame offset of time
		if i != 0:
			orbital_frame_offset = orbital_frame_offset + np.multiply(orb2inertial, time_step)
	
		#	generate rotation matrix of step ang
		rot_mat_step_offset = rot_mat(orbital_frame_offset)
	
		#	apply to vectors
		sat_inertial_vel = np.dot(sat_orb_vel, rot_mat_step_offset)
		sun_inertial_vec = np.dot(sun_orb_vec, rot_mat_step_offset)
		earth_inertial_vec = np.dot(earth_orb_vec, rot_mat_step_offset)
		site_inertial_vec = np.dot(site_orb_vec, rot_mat_step_offset)

		#	inertial unit vectors
		sun_inertial_unit = np.divide(sun_inertial_vec, np.linalg.norm(sun_inertial_vec))
		earth_inertial_unit = np.divide(earth_inertial_vec, np.linalg.norm(earth_inertial_vec))
		site_inertial_unit = np.divide(site_inertial_vec, np.linalg.norm(site_inertial_vec))

		#	generate inertial unit vectors from orbital unit vectors
		x_inertial_unit = np.dot(x_orb_unit, rot_mat_step_offset)
		y_inertial_unit = np.dot(y_orb_unit, rot_mat_step_offset)
		z_inertial_unit = np.dot(z_orb_unit, rot_mat_step_offset)

		###########################################################################################
		#	Data Output
		###########################################################################################
	
		#	set image index
		index = i

		###########################################################################################
		#	populate output attitude array
		attitude_out[index, :] = frame_offset
	
		###########################################################################################
		#	orbital frame vector data
	
		#	julian date
		vector_data_orb[index, 0] = jdate
		vector_data_orb[index, 1:4] = sun_orb_vec
		vector_data_orb[index, 4:7] = site_orb_vec
		vector_data_orb[index, 7:10] = earth_orb_vec
		vector_data_orb[index, 10:13] = sat_orb_vel
		vector_data_orb[index, 13:16] = x_orb_unit
		vector_data_orb[index, 16:19] = y_orb_unit
		vector_data_orb[index, 19:22] = z_orb_unit
		vector_data_orb[index, 22:25] = illum_vec_orb
		vector_data_orb[index, 25:28] = obs_vec_orb

		###########################################################################################
		#	body frame vector data

		#	julian date
		vector_data_body[index, 0] = jdate
		vector_data_body[index, 1:4] = sun_body_vec
		vector_data_body[index, 4:7] = site_body_vec
		vector_data_body[index, 7:10] = earth_body_vec
		vector_data_body[index, 10:13] = sat_body_vel
		vector_data_body[index, 13:16] = x_body_unit
		vector_data_body[index, 16:19] = y_body_unit
		vector_data_body[index, 19:22] = z_body_unit
		vector_data_body[index, 22:25] = illum_vec_body
		vector_data_body[index, 25:28] = obs_vec_body
	
		###########################################################################################
		#	inertial frame vector data
	
		#	julian date
		vector_data_inertial[index, 0] = jdate
		vector_data_inertial[index, 1:4] = sun_inertial_vec
		vector_data_inertial[index, 4:7] = site_inertial_vec
		vector_data_inertial[index, 7:10] = earth_inertial_vec
		vector_data_inertial[index, 10:13] = sat_inertial_vel
		vector_data_inertial[index, 13:16] = x_inertial_unit
		vector_data_inertial[index, 16:19] = y_inertial_unit
		vector_data_inertial[index, 19:22] = z_inertial_unit
		vector_data_inertial[index, 22:25] = illum_vec_body
		vector_data_inertial[index, 25:28] = obs_vec_body












		x_vec = np.array((1.0, 0.0, 0.0))
		y_vec = np.array((0.0, 1.0, 0.0))
		z_vec = np.array((0.0, 0.0, 1.0))


		rot_vec_body = np.copy(x_vec)

		x_ref_orb = np.copy(-earth_orb_vec)
		y_ref_orb = np.copy(sat_orb_vel)
		z_ref_orb = np.copy(z_vec)
	
		x_ref_orb_body = np.copy(-earth_body_vec)
		y_ref_orb_body = np.copy(-earth_orb_vec)
		z_ref_orb_body = np.copy(-earth_orb_vec)
		
		z_geom_orb = np.copy(z_orb_unit)
		z_geom_body = np.copy(z_vec)
		
		
		
		###############################################################################################
		#	observation vector validation
		###############################################################################################
		print('\n######################################################')
		print('Observation Vector Validation')
		print('######################################################')
		###########################################################################################
		#	orbital data
		
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(site_orb_vec, x_ref_orb),
				np.linalg.norm(site_orb_vec) * np.linalg.norm(x_ref_orb)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(site_orb_vec, y_ref_orb),
				np.linalg.norm(site_orb_vec) * np.linalg.norm(y_ref_orb)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(site_orb_vec, z_ref_orb),
				np.linalg.norm(site_orb_vec) * np.linalg.norm(z_ref_orb)
			)
		)
		
		print('\n####	Orb	############################################')
		print('\nsite_orb_vec_step = {}'.format(site_orb_vec))
		print('\nAngle to Earth Vec_step = ', np.degrees(x_angle_orb_step))
		print('Angle to Velocity Vec_step = ', np.degrees(y_angle_orb_step))
		print('Angle to Plane Normal_step = ', np.degrees(z_angle_orb_step))
		
		###########################################################################################
		#	body data
		
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(site_body_vec, x_ref_orb_body),
				np.linalg.norm(site_body_vec) * np.linalg.norm(x_ref_orb_body)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(site_body_vec, y_ref_orb_body),
				np.linalg.norm(site_body_vec) * np.linalg.norm(y_ref_orb_body)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(site_body_vec, z_ref_orb_body),
				np.linalg.norm(site_body_vec) * np.linalg.norm(z_ref_orb_body)
			)
		)
		
		print('\n####	Body	############################################')
		print('\nsite_body_vec_step = {}'.format(site_body_vec))
		print('\nAngle to Earth Vec = ', np.degrees(x_angle_body_step))
		print('Angle to Velocity Vec = ', np.degrees(y_angle_body_step))
		print('Angle to Plane Normal = ', np.degrees(z_angle_body_step))
		
		#	append angles to output array
		obs_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		obs_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])
		
		###############################################################################################
		#	illumination vector validation
		###############################################################################################
		print('\n######################################################')
		print('Illumination Vector Validation')
		print('######################################################')
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(sun_orb_vec, x_ref_orb),
				np.linalg.norm(sun_orb_vec) * np.linalg.norm(x_ref_orb)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(sun_orb_vec, y_ref_orb),
				np.linalg.norm(sun_orb_vec) * np.linalg.norm(y_ref_orb)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(sun_orb_vec, z_ref_orb),
				np.linalg.norm(sun_orb_vec) * np.linalg.norm(z_ref_orb)
			)
		)
	
		print('\n####	Orb	############################################')
		print('\nsun_orb_vec_step = {}'.format(sun_orb_vec))
		print('\nAngle to Earth Vec_step = ', np.degrees(x_angle_orb_step))
		print('Angle to Velocity Vec_step = ', np.degrees(y_angle_orb_step))
		print('Angle to Plane Normal_step = ', np.degrees(z_angle_orb_step))
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(sun_body_vec, x_ref_orb_body),
				np.linalg.norm(sun_body_vec) * np.linalg.norm(x_ref_orb_body)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(sun_body_vec, y_ref_orb_body),
				np.linalg.norm(sun_body_vec) * np.linalg.norm(y_ref_orb_body)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(sun_body_vec, z_ref_orb_body),
				np.linalg.norm(sun_body_vec) * np.linalg.norm(z_ref_orb_body)
			)
		)
		
		print('\n####	Body	############################################')
		print('\nsun_body_vec_step = {}'.format(sun_body_vec))
		print('\nAngle to Earth Vec = ', np.degrees(x_angle_body_step))
		print('Angle to Velocity Vec = ', np.degrees(y_angle_body_step))
		print('Angle to Plane Normal = ', np.degrees(z_angle_body_step))
   
		#	append angles to output array
		illum_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		illum_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###############################################################################################
		#	rotation vector validation
		###############################################################################################
		print('\n######################################################')
		print('Rotation Vector Validation')
		print('######################################################')
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(rot_vec_orb, x_ref_orb),
				np.linalg.norm(rot_vec_orb) * np.linalg.norm(x_ref_orb)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(rot_vec_orb, y_ref_orb),
				np.linalg.norm(rot_vec_orb) * np.linalg.norm(y_ref_orb)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(rot_vec_orb, z_ref_orb),
				np.linalg.norm(rot_vec_orb) * np.linalg.norm(z_ref_orb)
			)
		)
	
		print('\n####	Orb	############################################')
		print('\nrot_vec_orb_unit_step = {}'.format(rot_vec_orb))
		print('\nAngle to Earth Vec_step = ', np.degrees(x_angle_orb_step))
		print('Angle to Velocity Vec_step = ', np.degrees(y_angle_orb_step))
		print('Angle to Plane Normal_step = ', np.degrees(z_angle_orb_step))
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(rot_vec_body, x_ref_orb_body),
				np.linalg.norm(rot_vec_body) * np.linalg.norm(x_ref_orb_body)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(rot_vec_body, y_ref_orb_body),
				np.linalg.norm(rot_vec_body) * np.linalg.norm(y_ref_orb_body)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(rot_vec_body, z_ref_orb_body),
				np.linalg.norm(rot_vec_body) * np.linalg.norm(z_ref_orb_body)
			)
		)
		
#		print('\n####	Body	############################################')
#		print('\nrot_vec_body_unit = {}'.format(rot_vec_body_unit))
#		print('\nAngle to Earth Vec = ', np.degrees(x_angle_body_step))
#		print('Angle to Velocity Vec = ', np.degrees(y_angle_body_step))
#		print('Angle to Plane Normal = ', np.degrees(z_angle_body_step))

		#	append angles to output array
		rot_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		rot_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###############################################################################################
		#	geom z-axis validation
		###############################################################################################
#		print('\n######################################################')
#		print('Geom z-Axis Validation')
#		print('######################################################')
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(z_geom_orb, x_ref_orb),
				np.linalg.norm(z_geom_orb) * np.linalg.norm(x_ref_orb)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(z_geom_orb, y_ref_orb),
				np.linalg.norm(z_geom_orb) * np.linalg.norm(y_ref_orb)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(z_geom_orb, z_ref_orb),
				np.linalg.norm(z_geom_orb) * np.linalg.norm(z_ref_orb)
			)
		)
	
#		print('\n####	Orb	############################################')
#		print('\nz_geom_orb_step = {}'.format(z_geom_orb_step))
#		print('\nAngle to Earth Vec_step = ', np.degrees(x_angle_orb_step))
#		print('Angle to Velocity Vec_step = ', np.degrees(y_angle_orb_step))
#		print('Angle to Plane Normal_step = ', np.degrees(z_angle_orb_step))
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(z_geom_body, x_ref_orb_body),
				np.linalg.norm(z_geom_body) * np.linalg.norm(x_ref_orb_body)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(z_geom_body, y_ref_orb_body),
				np.linalg.norm(z_geom_body) * np.linalg.norm(y_ref_orb_body)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(z_geom_body, z_ref_orb_body),
				np.linalg.norm(z_geom_body) * np.linalg.norm(z_ref_orb_body)
			)
		)
		
#		print('\n####	Body	############################################')
#		print('\nz_geom_body_step = {}'.format(z_geom_body_step))
#		print('\nAngle to Earth Vec = ', np.degrees(x_angle_body_step))
#		print('Angle to Velocity Vec = ', np.degrees(y_angle_body_step))
#		print('Angle to Plane Normal = ', np.degrees(z_angle_body_step))

		#	append angles to output array
		geomz_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		geomz_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])
		#	save and end
		if i >= len_record - 1:
			break

	###########################################################################################
	#	convert angle outputs to numpy arrays

	obs_orb_plot_angles = np.array(obs_orb_plot_angles)
	obs_body_plot_angles = np.array(obs_body_plot_angles)

	illum_orb_plot_angles = np.array(illum_orb_plot_angles)
	illum_body_plot_angles = np.array(illum_body_plot_angles)

	rot_orb_plot_angles = np.array(rot_orb_plot_angles)
	rot_body_plot_angles = np.array(rot_body_plot_angles)

	geomz_orb_plot_angles = np.array(geomz_orb_plot_angles)
	geomz_body_plot_angles = np.array(geomz_body_plot_angles)

	###########################################################################################
	#	initialise figure
	fig, ax = plt.subplots(1, 4, figsize=(20, 6))

	###########################################################################################
	ax[0].title.set_text('Observation Vector Validation')
	ax[0].plot(obs_orb_plot_angles[:, 0], label='Angle from Earth_Vec in Orb Frame',  color='r')
	ax[0].plot(obs_orb_plot_angles[:, 1], label='Angle from Vel_Vec in Orb Frame',    color='g')
	ax[0].plot(obs_orb_plot_angles[:, 2], label='Angle from Plane_Norm in Orb Frame', color='b')

	ax[0].plot(obs_body_plot_angles[:, 0], label='Angle from Earth_Vec in Body Frame', color='tab:red')
	ax[0].plot(obs_body_plot_angles[:, 1], label='Angle from Vel_Vec in Body Frame',   color='tab:green')
	ax[0].plot(obs_body_plot_angles[:, 2], label='Angle from Plane_Vec in Body Frame', color='tab:blue')
	ax[0].legend()

	###########################################################################################
	ax[1].title.set_text('Illumination Vector Validation')
	ax[1].plot(illum_orb_plot_angles[:, 0], label='Angle from Earth_Vec in Orb Frame', color='r')
	ax[1].plot(illum_orb_plot_angles[:, 1], label='Angle from Vel_Vec in Orb Frame', color='g')
	ax[1].plot(illum_orb_plot_angles[:, 2], label='Angle from Plane_Norm in Orb Frame', color='b')

	ax[1].plot(illum_body_plot_angles[:, 0], label='Angle from Earth_Vec in Body Frame', color='tab:red')
	ax[1].plot(illum_body_plot_angles[:, 1], label='Angle from Vel_Vec in Body Frame', color='tab:green')
	ax[1].plot(illum_body_plot_angles[:, 2], label='Angle from Plane_Vec in Body Frame', color='tab:blue')
	ax[1].legend()

	###########################################################################################
	ax[2].title.set_text('Rotation Vector Validation')
	ax[2].plot(rot_orb_plot_angles[:, 0], label='Angle from Earth_Vec in Orb Frame', color='r')
	ax[2].plot(rot_orb_plot_angles[:, 1], label='Angle from Vel_Vec in Orb Frame', color='g')
	ax[2].plot(rot_orb_plot_angles[:, 2], label='Angle from Plane_Norm in Orb Frame', color='b')

	ax[2].plot(rot_body_plot_angles[:, 0], label='Angle from Earth_Vec in Body Frame', color='tab:red')
	ax[2].plot(rot_body_plot_angles[:, 1], label='Angle from Vel_Vec in Body Frame', color='tab:green')
	ax[2].plot(rot_body_plot_angles[:, 2], label='Angle from Plane_Vec in Body Frame', color='tab:blue')
	ax[2].legend()

	###########################################################################################
	ax[3].title.set_text('Geometry Z Axis Validation')
	ax[3].plot(geomz_orb_plot_angles[:, 0], label='Angle from Earth_Vec in Orb Frame', color='r')
	ax[3].plot(geomz_orb_plot_angles[:, 1], label='Angle from Vel_Vec in Orb Frame', color='g')
	ax[3].plot(geomz_orb_plot_angles[:, 2], label='Angle from Plane_Norm in Orb Frame', color='b')

	ax[3].plot(geomz_body_plot_angles[:, 0], label='Angle from Earth_Vec in Body Frame', color='tab:red')
	ax[3].plot(geomz_body_plot_angles[:, 1], label='Angle from Vel_Vec in Body Frame', color='tab:green')
	ax[3].plot(geomz_body_plot_angles[:, 2], label='Angle from Plane_Vec in Body Frame', color='tab:blue')
	ax[3].legend()


	print('\nShowing validation plots.')
	plt.show()
	
	return vector_data_body, vector_data_orb, vector_data_inertial, attitude_out

###################################################################################################
#	Function: 		gen_vectors_render
#	Description: 	generate required data for render
#
###################################################################################################

def gen_vectors_render(
		path_data, render_time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_frame_offset, init_frame_vel,
		observation_vector, illumination_vector,
		body_mesh, render_geom_dir
	):

	print('\nGenerating render vector data...')

	###############################################################################################
	#	initialisation
	###############################################################################################

	###############################################################################################
	#	parse inputs

	#	set step attitude to initial attitude
	step_attitude = np.copy(init_frame_offset)

	#	extract jd time data
	jd_time_data = sun_orb[:, 0]

	#	set simulation start julian date
	start_jdate = jd_time_data[0]
	stop_jdate = jd_time_data[-1]

	#	duration
	duration_sec = render_time_step * (len_record - 1)

	#	parse initial attitude and attitude motion
	frame_offset = np.copy(init_frame_offset)
	frame_velocity = np.copy(init_frame_vel)

	###############################################################################################
	#	initialise variables

	#	initialise output arrays
	attitude_out = np.zeros((len_record, 3))
	vector_data_body = np.zeros((len_record, 28))
	vector_data_orb = np.zeros((len_record, 28))
	vector_data_inertial = np.zeros((len_record, 28))

	#	define body axis vectors of object mesh
	x_body_unit = np.array((1.0, 0.0, 0.0))
	y_body_unit = np.array((0.0, 1.0, 0.0))
	z_body_unit = np.array((0.0, 0.0, 1.0))

	#	generate a temporary orb mesh from the body mesh
	temp_orb_mesh = mesh.Mesh(body_mesh.data.copy())

	#	apply initial frame offset
	rot_mat_step = rot_mat(init_frame_offset)
	temp_orb_mesh.rotate_using_matrix(rot_mat_step)

	#	apply initial frame offset to axis units
	x_orb_unit = np.dot(x_body_unit, rot_mat_step)
	y_orb_unit = np.dot(y_body_unit, rot_mat_step)
	z_orb_unit = np.dot(z_body_unit, rot_mat_step)

#	rot_axis_orb = np.dot(rot_axis_body, rot_mat_step)

	#	frame offset due to orbital motion
	orbital_frame_offset = np.array((0.0, 0.0, 0.0))

	#	parse input vectors
	observation_vec_orb = observation_vector
	illumination_vec_orb = illumination_vector

	###############################################################################################
	#	loop through ephemeris data
	###############################################################################################

	for i, (jdate, sat_orb_line, sun_orb_line, earth_orb_line, site_orb_line) in enumerate(
			zip(jd_time_data, sat_orb, sun_orb, earth_orb, site_orb)
		):
	
		###########################################################################################
		#	propagate velocities

		#	if i is not 0, calculate new total frame offset
		if i != 0:
			frame_offset = frame_offset + np.multiply(frame_velocity, render_time_step)

		#	generate body2orb frame offset rotation matrix
		rot_mat_step = rot_mat(frame_offset)
   	
		#	take inverse for orb2body rotation matrix
		rot_mat_step_inv = np.linalg.inv(rot_mat_step)

		###########################################################################################
		#	parse orbital data

		#	object positions and velocities in m/s
		sat_orb_pos = sat_orb_line[0:3]
		sat_orb_vel = sat_orb_line[3:6]

		#	orbital data
		sun_orb_vec = sun_orb_line[0:3]
		earth_orb_vec = earth_orb_line[0:3]
		site_orb_vec = site_orb_line[0:3]

		#	orbital unit vectors
		sun_orb_unit = np.divide(sun_orb_vec, np.linalg.norm(sun_orb_vec))
		earth_orb_unit = np.divide(earth_orb_vec, np.linalg.norm(earth_orb_vec))
		site_orb_unit = np.divide(site_orb_vec, np.linalg.norm(site_orb_vec))
	
		###########################################################################################
		#	propagate model in the BODY FRAME
		###########################################################################################

		####	body object mesh remains unchanged
		####	attitude motion is applied to vector data

		#	calculate body frame vectors (apply frame offset to orbital position vectors)
		sat_body_pos = np.dot(sat_orb_pos, rot_mat_step_inv)
		sat_body_vel = np.dot(sat_orb_vel, rot_mat_step_inv)
		sun_body_vec = np.dot(sun_orb_vec, rot_mat_step_inv) 
		earth_body_vec = np.dot(earth_orb_vec, rot_mat_step_inv)
		site_body_vec = np.dot(site_orb_vec, rot_mat_step_inv)

		#	calculate body unit vectors
		sun_body_unit = np.divide(sun_body_vec, np.linalg.norm(sun_body_vec))
		earth_body_unit = np.divide(earth_body_vec, np.linalg.norm(earth_body_vec))
		site_body_unit = np.divide(site_body_vec, np.linalg.norm(site_body_vec))
	
		###########################################################################################
		#	evolve object geometry in orbital frame
		###########################################################################################

		####	orbital frame vectors remain unchanged
		####	attitude motion is applied orbital object mesh

		#	calculate the total angular offset applied this step to each body axis
		if i == 0:
			rot_angx = 0
			rot_angy = 0
			rot_angz = 0
		else:
			rot_angx = init_frame_vel[0] * render_time_step
			rot_angy = init_frame_vel[1] * render_time_step
			rot_angz = init_frame_vel[2] * render_time_step

		###########################################################################################
		#	apply x-rotation in the body x-axis to the orbital object mesh
		att_mat = axis2mat(rot_angx, x_orb_unit)
		temp_orb_mesh.rotate_using_matrix(att_mat)

		#	apply x-rotation in the body frame to the unit axes
		x_orb_unit = np.dot(x_orb_unit, att_mat)
		y_orb_unit = np.dot(y_orb_unit, att_mat)
		z_orb_unit = np.dot(z_orb_unit, att_mat)

		###########################################################################################
		#	apply y-rotation in the NEW body y-axis frame to the NEW orbital object mesh
		att_mat = axis2mat(rot_angy, y_orb_unit)
		temp_orb_mesh.rotate_using_matrix(att_mat)

		#	apply y-rotation in the body frame to the NEW unit axes
		x_orb_unit = np.dot(x_orb_unit, att_mat)
		y_orb_unit = np.dot(y_orb_unit, att_mat)
		z_orb_unit = np.dot(z_orb_unit, att_mat)

		###########################################################################################
		#	apply z-rotation in the NEW body z-axis frame to the NEW orbital object mesh
		att_mat = axis2mat(rot_angz, z_orb_unit)
		temp_orb_mesh.rotate_using_matrix(att_mat)

		#	apply z-rotation in the body frame to the NEW unit axes
		x_orb_unit = np.dot(x_orb_unit, att_mat)
		y_orb_unit = np.dot(y_orb_unit, att_mat)
		z_orb_unit = np.dot(z_orb_unit, att_mat)

		#	custom vectors
		illumination_vec_body = np.dot(illumination_vec_orb, rot_mat_step_inv)
		observation_vec_body = np.dot(observation_vec_orb, rot_mat_step_inv)

		###########################################################################################
		#	generate inertial data
		###########################################################################################

		####	orbital angular velocity is applied to orbital vector data
		####	orbital angular velocity is applied to orbital object mesh

		###########################################################################################
		#	apply total orbital motion frame offset

		#	calculate total orbital motion frame offset of time
		if i != 0:
			orbital_frame_offset = orbital_frame_offset + np.multiply(orb2inertial, render_time_step)
	
		#	generate rotation matrix of step ang
		rot_mat_step_offset = rot_mat(orbital_frame_offset)
	
		#	apply to vectors
		sat_inertial_vel = np.dot(sat_orb_vel, rot_mat_step_offset)
		sun_inertial_vec = np.dot(sun_orb_vec, rot_mat_step_offset)
		earth_inertial_vec = np.dot(earth_orb_vec, rot_mat_step_offset)
		site_inertial_vec = np.dot(site_orb_vec, rot_mat_step_offset)

		#	inertial unit vectors
		sun_inertial_unit = np.divide(sun_inertial_vec, np.linalg.norm(sun_inertial_vec))
		earth_inertial_unit = np.divide(earth_inertial_vec, np.linalg.norm(earth_inertial_vec))
		site_inertial_unit = np.divide(site_inertial_vec, np.linalg.norm(site_inertial_vec))

		#	apply to geometry

		#	generate a temporary inertial mesh from the orbital mesh
		temp_inertial_mesh = mesh.Mesh(temp_orb_mesh.data.copy())

		#	rotate by frame offset
		temp_inertial_mesh.rotate_using_matrix(rot_mat_step_offset)

		#	generate inertial unit vectors from orbital unit vectors
		x_inertial_unit = np.dot(x_orb_unit, rot_mat_step_offset)
		y_inertial_unit = np.dot(y_orb_unit, rot_mat_step_offset)
		z_inertial_unit = np.dot(z_orb_unit, rot_mat_step_offset)

		###########################################################################################
		#	Data Output
		###########################################################################################
	
		#	set image index
		index = i

		###########################################################################################
		#	populate output attitude array
		attitude_out[index, :] = frame_offset
	
		###########################################################################################
		#	orbital frame vector data
	
		#	julian date
		vector_data_orb[index, 0] = jdate
		vector_data_orb[index, 1:4] = sun_orb_vec
		vector_data_orb[index, 4:7] = site_orb_vec
		vector_data_orb[index, 7:10] = earth_orb_vec
		vector_data_orb[index, 10:13] = sat_orb_vel
		vector_data_orb[index, 13:16] = x_orb_unit
		vector_data_orb[index, 16:19] = y_orb_unit
		vector_data_orb[index, 19:22] = z_orb_unit
		vector_data_orb[index, 22:25] = illumination_vec_orb
		vector_data_orb[index, 25:28] = observation_vec_orb
	
		###########################################################################################
		#	body frame vector data

		#	julian date
		vector_data_body[index, 0] = jdate
		vector_data_body[index, 1:4] = sun_body_vec
		vector_data_body[index, 4:7] = site_body_vec
		vector_data_body[index, 7:10] = earth_body_vec
		vector_data_body[index, 10:13] = sat_body_vel
		vector_data_body[index, 13:16] = x_body_unit
		vector_data_body[index, 16:19] = y_body_unit
		vector_data_body[index, 19:22] = z_body_unit
		vector_data_body[index, 22:25] = illumination_vec_body
		vector_data_body[index, 25:28] = observation_vec_body
	
		###########################################################################################
		#	inertial frame vector data
	
		#	julian date
		vector_data_inertial[index, 0] = jdate
		vector_data_inertial[index, 1:4] = sun_inertial_vec
		vector_data_inertial[index, 4:7] = site_inertial_vec
		vector_data_inertial[index, 7:10] = earth_inertial_vec
		vector_data_inertial[index, 10:13] = sat_inertial_vel
		vector_data_inertial[index, 13:16] = x_inertial_unit
		vector_data_inertial[index, 16:19] = y_inertial_unit
		vector_data_inertial[index, 19:22] = z_inertial_unit
		vector_data_inertial[index, 22:25] = illumination_vec_body
		vector_data_inertial[index, 25:28] = observation_vec_body

		###########################################################################################
		#	rendering code
		###########################################################################################

		#	render output files
		render_data_orb = 'render_data_orb.txt'

		#	generate image ID
		if index < 10:
			geom_id = '000' + str(index)
		elif 10 <= index < 100:
			geom_id = '00' + str(index)
		elif 100 <= index < 1000:
			geom_id = '0' + str(index)
		elif 1000 <= index < 10000:
			geom_id = str(index)

		#	saving orbital mesh file
		mesh_file = 'orbital_mesh_{}.stl'.format(geom_id)
		temp_orb_mesh.save(os.path.join(render_geom_dir, mesh_file), mode=stl.Mode.ASCII)

		#	save and end
		if i >= len_record - 1:
			break

	return vector_data_body, vector_data_orb, vector_data_inertial, attitude_out

###################################################################################################
#	Function: 		gen_vectors2
#	Description: 	generate vectors for light curve generation
#
###################################################################################################

def gen_vectors2(
		path_data, render_time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb,
	):

	#	define unit vectors
	x_vec = np.array((1.0, 0.0, 0.0))
	y_vec = np.array((0.0, 1.0, 0.0))
	z_vec = np.array((0.0, 0.0, 1.0))

	###############################################################################################
	#	parse time inputs
	###############################################################################################
	
	#	extract jd time data
	jd_time_data = sun_orb[:, 0]
	
	#	set simulation start julian date
	start_jdate = jd_time_data[0]
	stop_jdate = jd_time_data[-1]
	
	#	duration
	duration_sec = render_time_step * (len_record - 1)

	###############################################################################################
	#	parse attit inputs
	###############################################################################################

	################################################
	#	initial attitude

	#	attitude matrix
	init_attit_mat = rot_mat(init_attit)

	#	transpose for inverse matrix
	init_attit_mat_inv = np.transpose(init_attit_mat)

	################################################
	#	attitude motion in orbital frame

	#	rotation_vector in orbital frame
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)

	#	generate attitude motion matrix in orbital frame
	attit_mot_mat_orb = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_orb_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_orb)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_orb)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_orb = angles1

	################################################
	#	attitude motion in body frame

	#	generate attitude motion matrix in body frame
	attit_mot_mat_body = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_body_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1

	###############################################################################################
	#	initialise geometry axes
	###############################################################################################

	#	body

	#	define geometry axes in body frame
	x_geom_body = np.copy(x_vec)
	y_geom_body = np.copy(y_vec)
	z_geom_body = np.copy(z_vec)

	#	rotate geometry axes into orbital frame
	x_geom_orb = np.dot(x_geom_body, init_attit_mat)
	y_geom_orb = np.dot(y_geom_body, init_attit_mat)
	z_geom_orb = np.dot(z_geom_body, init_attit_mat)

	###############################################################################################
	#	initialise reference axes
	###############################################################################################

	#	extract orbital vector data
	earth_orb_vec = earth_orb[0, 1:4]

	#	velcocity vector in orbital frame
	sat_orb_vel = sat_orb[0, 4:7]

	#	set orbital plane normal in orbital frame
	orb_norm_orb_unit = np.array([0.0, 0.0, 1.0])

	#	orbital reference frame vectors
	x_ref_orb = np.divide(earth_orb_vec, np.linalg.norm(earth_orb_vec))
	y_ref_orb = np.divide(sat_orb_vel, np.linalg.norm(sat_orb_vel))
	z_ref_orb = np.divide(orb_norm_orb_unit, np.linalg.norm(orb_norm_orb_unit))

	#	orbital reference frame vectors in body frame
	x_ref_orb_body = np.dot(x_ref_orb, init_attit_mat_inv)
	y_ref_orb_body = np.dot(y_ref_orb, init_attit_mat_inv)
	z_ref_orb_body = np.dot(z_ref_orb, init_attit_mat_inv)

	###############################################################################################
	#	loop through ephemeris data
	###############################################################################################

	###############################################################################################
	#	initialise step data

	total_rot_vec_vel_angle = np.array([0.0, 0.0, 0.0])

	#	rotate geometry axes into orbital frame
	x_geom_orb_step = np.copy(x_geom_orb)
	y_geom_orb_step = np.copy(y_geom_orb)
	z_geom_orb_step = np.copy(z_geom_orb)

	#	generate rotation matrices for initial attitude

	init_attit_step = np.copy(init_attit)

	#	attitude matrix
	init_attit_mat_step = rot_mat(init_attit_step)

	#	transpose for inverse matrix
	init_attit_mat_inv_step = np.transpose(init_attit_mat_step)

	###############################################################################################
	#	initialise output arrays

	#	vector output arrays
	attitude_out = np.zeros((len_record, 3))
	vector_data_orb = np.zeros((len_record, 43))
	vector_data_body = np.zeros((len_record, 43))

	for i, (jdate, sat_orb_line, sun_orb_line, earth_orb_line, site_orb_line) in enumerate(
			zip(jd_time_data, sat_orb, sun_orb, earth_orb, site_orb)
		):

		###########################################################################################
		#	parse input orbital data
		###########################################################################################

		#	extract orbital vectors of step
		sat_orb_pos_step = sat_orb_line[1:4]
		sat_orb_vel_step = sat_orb_line[4:7]

		#	extract orbital vector data
		sun_orb_vec_step = sun_orb_line[1:4]
		earth_orb_vec_step = earth_orb_line[1:4]
		site_orb_vec_step = site_orb_line[1:4]

		#	unit vectors
		sun_orb_unit_step = np.divide(sun_orb_vec_step, np.linalg.norm(sun_orb_vec_step))
		earth_orb_unit_step = np.divide(earth_orb_vec_step, np.linalg.norm(earth_orb_vec_step))
		site_orb_unit_step = np.divide(site_orb_vec_step, np.linalg.norm(site_orb_vec_step))

		###########################################################################################
		#	orbital frame data
		###########################################################################################

		###############################################################################################
		#	orbital reference frame vectors
		x_ref_orb_step = np.divide(earth_orb_vec_step, np.linalg.norm(earth_orb_vec_step))
		y_ref_orb_step = np.divide(sat_orb_vel_step, np.linalg.norm(sat_orb_vel_step))
		z_ref_orb_step = np.divide(orb_norm_orb_unit, np.linalg.norm(orb_norm_orb_unit))
	
		###############################################################################################
		#	update geometry axes in orbital frame
		x_geom_orb_step = np.dot(x_geom_orb_step, attit_mot_mat_orb)
		y_geom_orb_step = np.dot(y_geom_orb_step, attit_mot_mat_orb)
		z_geom_orb_step = np.dot(z_geom_orb_step, attit_mot_mat_orb)

		###########################################################################################
		#	vector positions in orbital frame
	
		#	apply initial attitude to rotation vector in orbital frame
		rot_vec_orb_unit_step = np.dot(rot_vec_body_unit, init_attit_mat_step)

		#	set orbital plane normal in orbital frame
		orb_norm_orb_unit_step = np.copy(orb_norm_orb_unit)

		###########################################################################################
		#	body frame vectors
		###########################################################################################

		#	total attitude motion in body frame since simulation start
		total_attit_mot_body_step = np.multiply(attit_mot_body, i)

		#	total attitude rotation matrix
		total_attit_mot_body_mat_step = rot_mat(total_attit_mot_body_step)

		#	inverse total attitude rotation matrix
		total_attit_mot_body_mat_inv_step = np.transpose(total_attit_mot_body_mat_step)

		###############################################################################################
		#	orbital ref vectors in body frame

		#	apply initial attitude
		x_ref_orb_body_step = np.dot(x_ref_orb_step, init_attit_mat_inv_step)
		y_ref_orb_body_step = np.dot(y_ref_orb_step, init_attit_mat_inv_step)
		z_ref_orb_body_step = np.dot(z_ref_orb_step, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		x_ref_orb_body_step = np.dot(x_ref_orb_body_step, total_attit_mot_body_mat_inv_step)
		y_ref_orb_body_step = np.dot(y_ref_orb_body_step, total_attit_mot_body_mat_inv_step)
		z_ref_orb_body_step = np.dot(z_ref_orb_body_step, total_attit_mot_body_mat_inv_step)

		###############################################################################################
		#	define geometry axes in body frame
		x_geom_body_step = np.copy(x_vec)
		y_geom_body_step = np.copy(y_vec)
		z_geom_body_step = np.copy(z_vec)

		###########################################################################################
		#	vector positions in body frame
	
		################################################
		#	rotation vector in body frame
		rot_vec_body_unit_step = np.copy(rot_vec_body_unit)

		################################################
		#	orbital plane normal vector

		#	apply attitude
		orb_norm_body_unit_step = np.dot(orb_norm_orb_unit, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		orb_norm_body_unit_step = np.dot(orb_norm_body_unit_step, total_attit_mot_body_mat_inv_step)

		###########################################################################################
		#	convert input orbital data to body frame
		###########################################################################################

		################################################
		#	velocity vector

		#	apply initial attitude
		sat_body_vel_step = np.dot(sat_orb_vel_step, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		sat_body_vel_step = np.dot(sat_body_vel_step, total_attit_mot_body_mat_inv_step)

		################################################
		#	sun/site/earth vector

		#	apply initial attitude
		sun_body_vec_step = np.dot(sun_orb_vec_step, init_attit_mat_inv_step)
		earth_body_vec_step = np.dot(earth_orb_vec_step, init_attit_mat_inv_step)
		site_body_vec_step = np.dot(site_orb_vec_step, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		sun_body_vec_step = np.dot(sun_body_vec_step, total_attit_mot_body_mat_inv_step)
		earth_body_vec_step = np.dot(earth_body_vec_step, total_attit_mot_body_mat_inv_step)
		site_body_vec_step = np.dot(site_body_vec_step, total_attit_mot_body_mat_inv_step)

		################################################
		#	custom stationary observation/illumination vectors

		#	apply initial attitude
		illum_vec_body_step = np.dot(illum_vec_orb, init_attit_mat_inv_step)
		obs_vec_body_step = np.dot(obs_vec_orb, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		illum_vec_body_step = np.dot(illum_vec_body_step, total_attit_mot_body_mat_inv_step)
		obs_vec_body_step = np.dot(obs_vec_body_step, total_attit_mot_body_mat_inv_step)

#		###########################################################################################
#		#	propagate rotation vector velocity in body and orbital frames
#
#		#	calculate total motion of the rotation vector since since of simulation
#		if i != 0:
#			total_rot_vec_vel_angle += np.multiply(rot_vec_vel_orb, render_time_step)

#		###########################################################################################
#		#	generate inertial data
#		###########################################################################################
#
#		####	orbital angular velocity is applied to orbital vector data
#		####	orbital angular velocity is applied to orbital object mesh
#
#		###########################################################################################
#		#	apply total orbital motion frame offset
#
#		#	calculate total orbital motion frame offset of time
#		if i != 0:
#			orbital_frame_offset = orbital_frame_offset + np.multiply(orb2inertial, render_time_step)
#	
#		#	generate rotation matrix of step ang
#		rot_mat_step_offset = rot_mat(orbital_frame_offset)
#	
#		#	apply to vectors
#		sat_inertial_vel = np.dot(sat_orb_vel, rot_mat_step_offset)
#		sun_inertial_vec = np.dot(sun_orb_vec, rot_mat_step_offset)
#		earth_inertial_vec = np.dot(earth_orb_vec, rot_mat_step_offset)
#		site_inertial_vec = np.dot(site_orb_vec, rot_mat_step_offset)
#
#		#	inertial unit vectors
#		sun_inertial_unit = np.divide(sun_inertial_vec, np.linalg.norm(sun_inertial_vec))
#		earth_inertial_unit = np.divide(earth_inertial_vec, np.linalg.norm(earth_inertial_vec))
#		site_inertial_unit = np.divide(site_inertial_vec, np.linalg.norm(site_inertial_vec))
#
#		#	apply to geometry
#
#		#	generate a temporary inertial mesh from the orbital mesh
#		temp_inertial_mesh = mesh.Mesh(temp_orb_mesh.data.copy())
#
#		#	rotate by frame offset
#		temp_inertial_mesh.rotate_using_matrix(rot_mat_step_offset)
#
#		#	generate inertial unit vectors from orbital unit vectors
#		x_inertial_unit = np.dot(x_orb_unit, rot_mat_step_offset)
#		y_inertial_unit = np.dot(y_orb_unit, rot_mat_step_offset)
#		z_inertial_unit = np.dot(z_orb_unit, rot_mat_step_offset)

		###########################################################################################
		#	Data Output
		###########################################################################################
	
		#	set image index
		index = i

		###########################################################################################
		#	populate output attitude array
#		attitude_out[index, :] = attit_step
		attitude_out[index, :] = np.array([0.0, 0.0, 0.0]) 
	
		###########################################################################################
		#	orbital frame vector data

		#	julian date
		vector_data_orb[index, 0] = jdate

		#	reference vectors
		vector_data_orb[index, 1:4] = x_ref_orb_step
		vector_data_orb[index, 4:7] = y_ref_orb_step
		vector_data_orb[index, 7:10] = z_ref_orb_step

		#	geometry vectors
		vector_data_orb[index, 10:13] = x_geom_orb_step
		vector_data_orb[index, 13:16] = y_geom_orb_step
		vector_data_orb[index, 16:19] = z_geom_orb_step

		#	earth, velocity and plane normal vectors
		vector_data_orb[index, 19:22] = earth_orb_vec_step
		vector_data_orb[index, 22:25] = sat_orb_vel_step
		vector_data_orb[index, 25:28] = orb_norm_orb_unit_step

		#	sun, site and rotation vector
		vector_data_orb[index, 28:31] = sun_orb_vec_step
		vector_data_orb[index, 31:34] = site_orb_vec_step
		vector_data_orb[index, 34:37] = rot_vec_orb_unit_step

		vector_data_orb[index, 37:40] = illum_vec_orb
		vector_data_orb[index, 40:43] = obs_vec_orb

		###########################################################################################
		#	body frame vector data

		#	julian date
		vector_data_body[index, 0] = jdate

		#	reference vectors
		vector_data_body[index, 1:4] = x_ref_orb_body_step
		vector_data_body[index, 4:7] = y_ref_orb_body_step
		vector_data_body[index, 7:10] = z_ref_orb_body_step

		#	geometry vectors
		vector_data_body[index, 10:13] = x_geom_body_step
		vector_data_body[index, 13:16] = y_geom_body_step
		vector_data_body[index, 16:19] = z_geom_body_step

		#	earth, velocity and plane normal vectors
		vector_data_body[index, 19:22] = earth_body_vec_step
		vector_data_body[index, 22:25] = sat_body_vel_step
		vector_data_body[index, 25:28] = orb_norm_body_unit_step

		#	sun, site and rotation vector
		vector_data_body[index, 28:31] = sun_body_vec_step
		vector_data_body[index, 31:34] = site_body_vec_step
		vector_data_body[index, 34:37] = rot_vec_body_unit_step

		vector_data_body[index, 37:40] = illum_vec_body_step
		vector_data_body[index, 40:43] = obs_vec_body_step

		###########################################################################################
		#	inertial frame vector data
	
		#	save and end
		if i >= len_record - 1:
			break

	return vector_data_body, vector_data_orb, attitude_out

###################################################################################################
#	Function: 		gen_vectors_render2
#	Description: 	generate required data for render
#
###################################################################################################

def gen_vectors_render2(
		path_data, render_time_step, len_record,
		sat_orb, sun_orb, earth_orb, site_orb,
		orb2inertial, init_attit, rot_vec_body_unit, rot_ang_per_sec, rot_vec_vel_orb,
		obs_vec_orb, illum_vec_orb,
		body_mesh, render_geom_dir,
		validation_plot_flag
	):

	print('\nGenerating render vector data...')

	#	define unit vectors
	x_vec = np.array((1.0, 0.0, 0.0))
	y_vec = np.array((0.0, 1.0, 0.0))
	z_vec = np.array((0.0, 0.0, 1.0))

	###############################################################################################
	#	parse time inputs
	###############################################################################################
	
	#	extract jd time data
	jd_time_data = sun_orb[:, 0]
	
	#	set simulation start julian date
	start_jdate = jd_time_data[0]
	stop_jdate = jd_time_data[-1]
	
	#	duration
	duration_sec = render_time_step * (len_record - 1)

	###############################################################################################
	#	parse attit inputs
	###############################################################################################

	################################################
	#	initial attitude

	#	attitude matrix
	init_attit_mat = rot_mat(init_attit)

	#	transpose for inverse matrix
	init_attit_mat_inv = np.transpose(init_attit_mat)

	################################################
	#	attitude motion in orbital frame

	#	rotation_vector in orbital frame
	rot_vec_orb_unit = np.dot(rot_vec_body_unit, init_attit_mat)

	#	generate attitude motion matrix in orbital frame
	attit_mot_mat_orb = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_orb_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_orb)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_orb)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_orb = angles1

	################################################
	#	attitude motion in body frame

	#	generate attitude motion matrix in body frame
	attit_mot_mat_body = axis2mat(rot_ang_per_sec * render_time_step, rot_vec_body_unit)

	#	extract euler angles
	angles1 = np.array(mat2angles(attit_mot_mat_body)[0:3])
	angles2 = np.array(mat2angles(attit_mot_mat_body)[3:6])
	
	#	convert negative zeros
	for i, ele in enumerate(angles1):
		if str(ele)[0] == '-' and ele == 0:
			angles1[i] = 0.0
	
	#	extract attitude motion in the body frame
	attit_mot_body = angles1

	print('\nAttit Mot Body = {}'.format(attit_mot_body))

	###############################################################################################
	#	initialise geometry axes
	###############################################################################################

	#	body

	#	define geometry axes in body frame
	x_geom_body = np.copy(x_vec)
	y_geom_body = np.copy(y_vec)
	z_geom_body = np.copy(z_vec)

	#	rotate geometry axes into orbital frame
	x_geom_orb = np.dot(x_geom_body, init_attit_mat)
	y_geom_orb = np.dot(y_geom_body, init_attit_mat)
	z_geom_orb = np.dot(z_geom_body, init_attit_mat)

	###############################################################################################
	#	initialise reference axes
	###############################################################################################

	#	extract orbital vector data
	earth_orb_vec = earth_orb[0, 1:4]

	#	velcocity vector in orbital frame
	sat_orb_vel = sat_orb[0, 4:7]

	#	set orbital plane normal in orbital frame
	orb_norm_orb_unit = np.array([0.0, 0.0, 1.0])

	#	orbital reference frame vectors
	x_ref_orb = np.divide(earth_orb_vec, np.linalg.norm(earth_orb_vec))
	y_ref_orb = np.divide(sat_orb_vel, np.linalg.norm(sat_orb_vel))
	z_ref_orb = np.divide(orb_norm_orb_unit, np.linalg.norm(orb_norm_orb_unit))

	#	orbital reference frame vectors in body frame
	x_ref_orb_body = np.dot(x_ref_orb, init_attit_mat_inv)
	y_ref_orb_body = np.dot(y_ref_orb, init_attit_mat_inv)
	z_ref_orb_body = np.dot(z_ref_orb, init_attit_mat_inv)

	###############################################################################################
	#	loop through ephemeris data
	###############################################################################################

	###############################################################################################
	#	initialise step data

	total_rot_vec_vel_angle = np.array([0.0, 0.0, 0.0])

	#	generate orbital mesh from the body mesh
	orb_mesh_step = mesh.Mesh(body_mesh.data.copy())

	#	rotate into orbital frame
	orb_mesh_step.rotate_using_matrix(init_attit_mat)












#	###############################################
#	figure3d = plt.figure(num=None, figsize=(9, 9), facecolor='w', edgecolor='k')
#	axes3d = mplot3d.Axes3D(figure3d)
#	
#	###############################################
#	#	set title, axis labels and axis limits
#	
#	axis_lims = 10
#	
#	axes3d.set_title('Orbital Frame')
#	
#	axes3d.set_xlim([-axis_lims, axis_lims])
#	axes3d.set_ylim([-axis_lims, axis_lims])
#	axes3d.set_zlim([-axis_lims, axis_lims])
#	
#	axes3d.set_xlabel('x orbital axis')
#	axes3d.set_ylabel('y orbital axis')
#	axes3d.set_zlabel('z orbital axis')
#	
#	###############################################
#	#	observation vector
#	
#	origin = np.array([0.0, 0.0, 0.0])
#	font_size = 14
#	
#	x_vec = np.array([-1000, 0.0, 0.0])
#	y_vec = np.array([0.0, 1000, 0.0])
#	z_vec = np.array([0.0, 0.0, 1000])
#	
#	#	generate plottable vector
#	x_vec_plot = 20 * np.divide(np.vstack((origin, x_vec)), np.linalg.norm(x_vec))
#	y_vec_plot = 20 * np.divide(np.vstack((origin, y_vec)), np.linalg.norm(y_vec))
#	z_vec_plot = 20 * np.divide(np.vstack((origin, z_vec)), np.linalg.norm(z_vec))
#	
#	#	create plot arrow and add to plot
#	a = Arrow3D(x_vec_plot[:, 0], x_vec_plot[:, 1], x_vec_plot[:, 2],
#		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
#	)
#	b = Arrow3D(y_vec_plot[:, 0], y_vec_plot[:, 1], y_vec_plot[:, 2],
#		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
#	)
#	c = Arrow3D(z_vec_plot[:, 0], z_vec_plot[:, 1], z_vec_plot[:, 2],
#		mutation_scale=20, lw=2, arrowstyle="-|>", color="k"
#	)
#	
#	axes3d.add_artist(a)
#	axes3d.add_artist(b)
#	axes3d.add_artist(c)
#	
#	#	add text
#	axes3d.text(x_vec_plot[1, 0], x_vec_plot[1, 1], x_vec_plot[1, 2],
#		'X Vec', fontsize=font_size, fontweight='bold'
#	)
#	axes3d.text(y_vec_plot[1, 0], y_vec_plot[1, 1], y_vec_plot[1, 2],
#		'X Vec', fontsize=font_size, fontweight='bold'
#	)
#	axes3d.text(z_vec_plot[1, 0], z_vec_plot[1, 1], z_vec_plot[1, 2],
#		'X Vec', fontsize=font_size, fontweight='bold'
#	)
#	
#	###############################################
#	#	observation vector
#	
#	real_site_vec = np.array([-461.68530059, 167.85062484, 433.34688502])
#	
#	origin = np.array([0.0, 0.0, 0.0])
#	font_size = 14
#	
#	#	generate plottable vector
#	obs_vec_plot = 10 * np.divide(
#		np.vstack((origin, real_site_vec)), np.linalg.norm(real_site_vec)
#	)
#	
#	#	create plot arrow and add to plot
#	a = Arrow3D(obs_vec_plot[:, 0], obs_vec_plot[:, 1], obs_vec_plot[:, 2],
#		mutation_scale=20, lw=2, arrowstyle="-|>", color="r"
#	)
#	
#	axes3d.add_artist(a)
#	
#	#	add text
#	axes3d.text(obs_vec_plot[1, 0], obs_vec_plot[1, 1], obs_vec_plot[1, 2],
#		'Obs Vec', fontsize=font_size, fontweight='bold'
#	)
#	
#	#	set vectors and areas
#	orb_vectors = orb_mesh_step.vectors
#	orb_normals = orb_mesh_step.normals
#	orb_areas = orb_mesh_step.areas
#	
#	nfaces = len(orb_areas)
#	
#	for i_face, vector in enumerate(orb_vectors):
#	
#	
#		#	rending
#	
#		#	Add final vert to vector array
#		vector = np.vstack([vector, vector[0, :]])
#	
#		#	extract x, y and z values
#		xs = vector[:,0]
#		ys = vector[:,1]
#		zs = vector[:,2]
#	
#		#	generate vertices list
#		verts = [list(zip(xs, ys, zs))]
#	
#	
#		#	set facet alpha
#		alpha = 0.75
#	
#		#	add facet to plot
#		pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
#		pc.set_alpha(alpha)
#		pc.set_edgecolor([0.5, 0.5, 0.5])
#	#	pc.set_facecolor(facecol)
#		
#		axes3d.add_collection3d(pc)
#	
#	
#	plt.show()
#	sys.exit()
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	



















	#	rotate geometry axes into orbital frame
	x_geom_orb_step = np.copy(x_geom_orb)
	y_geom_orb_step = np.copy(y_geom_orb)
	z_geom_orb_step = np.copy(z_geom_orb)

	#	generate rotation matrices for initial attitude

	init_attit_step = np.copy(init_attit)

	#	attitude matrix
	init_attit_mat_step = rot_mat(init_attit_step)

	#	transpose for inverse matrix
	init_attit_mat_inv_step = np.transpose(init_attit_mat_step)

	###############################################################################################
	#	initialise output arrays

	#	vector output arrays
	attitude_out = np.zeros((len_record, 3))
	vector_data_body = np.zeros((len_record, 37))
	vector_data_orb = np.zeros((len_record, 37))

	#	angle output arrays
	obs_orb_plot_angles = []
	obs_body_plot_angles = []
	illum_orb_plot_angles = []
	illum_body_plot_angles = []
	rot_orb_plot_angles = []
	rot_body_plot_angles = []
	geomz_orb_plot_angles = []
	geomz_body_plot_angles = []

	for i, (jdate, sat_orb_line, sun_orb_line, earth_orb_line, site_orb_line) in enumerate(
			zip(jd_time_data, sat_orb, sun_orb, earth_orb, site_orb)
		):

#		print('\n#####################################################################################')

		###########################################################################################
		#	calcualte new attitude state

#		#	propergate attitude
#		if i != 0:
#
#			print('\nNew Attiude = {} + {}'.format(attit_step, np.multiply(attit_mot, render_time_step)))
#			attit_step += np.multiply(attit_mot, render_time_step)
#			print('Attit Step = ', attit_step)

		###########################################################################################
		#	parse input orbital data
		###########################################################################################

		#	extract orbital vectors of step
		sat_orb_pos_step = sat_orb_line[1:4]
		sat_orb_vel_step = sat_orb_line[4:7]

		#	extract orbital vector data
		sun_orb_vec_step = sun_orb_line[1:4]
		earth_orb_vec_step = earth_orb_line[1:4]
		site_orb_vec_step = site_orb_line[1:4]

		#	unit vectors
		sun_orb_unit_step = np.divide(sun_orb_vec_step, np.linalg.norm(sun_orb_vec_step))
		earth_orb_unit_step = np.divide(earth_orb_vec_step, np.linalg.norm(earth_orb_vec_step))
		site_orb_unit_step = np.divide(site_orb_vec_step, np.linalg.norm(site_orb_vec_step))

		###########################################################################################
		#	orbital frame data
		###########################################################################################

		###############################################################################################
		#	orbital reference frame vectors
		x_ref_orb_step = np.divide(earth_orb_vec_step, np.linalg.norm(earth_orb_vec_step))
		y_ref_orb_step = np.divide(sat_orb_vel_step, np.linalg.norm(sat_orb_vel_step))
		z_ref_orb_step = np.divide(orb_norm_orb_unit, np.linalg.norm(orb_norm_orb_unit))
	
		###############################################################################################
		#	apply orbital attitude motion if i != 0

		if i != 0:
			x_geom_orb_step = np.dot(x_geom_orb_step, attit_mot_mat_orb)
			y_geom_orb_step = np.dot(y_geom_orb_step, attit_mot_mat_orb)
			z_geom_orb_step = np.dot(z_geom_orb_step, attit_mot_mat_orb)

		###########################################################################################
		#	vector positions in orbital frame
	
		#	apply initial attitude to rotation vector in orbital frame
		rot_vec_orb_unit_step = np.dot(rot_vec_body_unit, init_attit_mat_step)

		#	set orbital plane normal in orbital frame
		orb_norm_orb_unit_step = np.copy(orb_norm_orb_unit)

		###############################################################################################
		#	apply orbital attitude motion to geometry if i != 0
		if i != 0:
			orb_mesh_step.rotate_using_matrix(attit_mot_mat_orb)

		###########################################################################################
		#	body frame vectors
		###########################################################################################

		#	total attitude motion in body frame since simulation start
		total_attit_mot_body_step = np.multiply(attit_mot_body, i)

		#	total attitude rotation matrix
		total_attit_mot_body_mat_step = rot_mat(total_attit_mot_body_step)

		#	inverse total attitude rotation matrix
		total_attit_mot_body_mat_inv_step = np.transpose(total_attit_mot_body_mat_step)

		###############################################################################################
		#	orbital ref vectors in body frame

		#	apply initial attitude
		x_ref_orb_body_step = np.dot(x_ref_orb_step, init_attit_mat_inv_step)
		y_ref_orb_body_step = np.dot(y_ref_orb_step, init_attit_mat_inv_step)
		z_ref_orb_body_step = np.dot(z_ref_orb_step, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		x_ref_orb_body_step = np.dot(x_ref_orb_body_step, total_attit_mot_body_mat_inv_step)
		y_ref_orb_body_step = np.dot(y_ref_orb_body_step, total_attit_mot_body_mat_inv_step)
		z_ref_orb_body_step = np.dot(z_ref_orb_body_step, total_attit_mot_body_mat_inv_step)

		###############################################################################################
		#	define geometry axes in body frame
		x_geom_body_step = np.copy(x_vec)
		y_geom_body_step = np.copy(y_vec)
		z_geom_body_step = np.copy(z_vec)

		###########################################################################################
		#	vector positions in body frame
	
		################################################
		#	rotation vector in orbital frame
		rot_vec_body_unit_step = np.copy(rot_vec_body_unit)

		################################################
		#	orbital plane normal vector

		#	apply attitude
		orb_norm_body_unit_step = np.dot(orb_norm_orb_unit, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		orb_norm_body_unit_step = np.dot(orb_norm_body_unit_step, total_attit_mot_body_mat_inv_step)

		###########################################################################################
		#	convert input orbital data to body frame
		###########################################################################################

		################################################
		#	velocity vector

		#	apply initial attitude
		sat_body_vel_step = np.dot(sat_orb_vel_step, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		sat_body_vel_step = np.dot(sat_body_vel_step, total_attit_mot_body_mat_inv_step)

		################################################
		#	sun/illumination vector

		#	apply initial attitude
		sun_body_vec_step = np.dot(sun_orb_vec_step, init_attit_mat_inv_step)
		earth_body_vec_step = np.dot(earth_orb_vec_step, init_attit_mat_inv_step)
		site_body_vec_step = np.dot(site_orb_vec_step, init_attit_mat_inv_step)

		#	apply total attitude motion since simulation start
		sun_body_vec_step = np.dot(sun_body_vec_step, total_attit_mot_body_mat_inv_step)
		earth_body_vec_step = np.dot(earth_body_vec_step, total_attit_mot_body_mat_inv_step)
		site_body_vec_step = np.dot(site_body_vec_step, total_attit_mot_body_mat_inv_step)

		###########################################################################################
		#	observation vector validation
		###########################################################################################
#		print('\n######################################################')
#		print('Observation Vector Validation')
#		print('######################################################')
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(site_orb_vec_step, x_ref_orb_step),
				np.linalg.norm(site_orb_vec_step) * np.linalg.norm(x_ref_orb_step)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(site_orb_vec_step, y_ref_orb_step),
				np.linalg.norm(site_orb_vec_step) * np.linalg.norm(y_ref_orb_step)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(site_orb_vec_step, z_ref_orb_step),
				np.linalg.norm(site_orb_vec_step) * np.linalg.norm(z_ref_orb_step)
			)
		)
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(site_body_vec_step, x_ref_orb_body_step),
				np.linalg.norm(site_body_vec_step) * np.linalg.norm(x_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(site_body_vec_step, y_ref_orb_body_step),
				np.linalg.norm(site_body_vec_step) * np.linalg.norm(y_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(site_body_vec_step, z_ref_orb_body_step),
				np.linalg.norm(site_body_vec_step) * np.linalg.norm(z_ref_orb_body_step)
			)
		)
		
		#	append angles to output array
		obs_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		obs_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###############################################################################################
		#	illumination vector validation
		###############################################################################################
#		print('\n######################################################')
#		print('Illumination Vector Validation')
#		print('######################################################')
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(sun_orb_vec_step, x_ref_orb_step),
				np.linalg.norm(sun_orb_vec_step) * np.linalg.norm(x_ref_orb_step)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(sun_orb_vec_step, y_ref_orb_step),
				np.linalg.norm(sun_orb_vec_step) * np.linalg.norm(y_ref_orb_step)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(sun_orb_vec_step, z_ref_orb_step),
				np.linalg.norm(sun_orb_vec_step) * np.linalg.norm(z_ref_orb_step)
			)
		)
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(sun_body_vec_step, x_ref_orb_body_step),
				np.linalg.norm(sun_body_vec_step) * np.linalg.norm(x_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(sun_body_vec_step, y_ref_orb_body_step),
				np.linalg.norm(sun_body_vec_step) * np.linalg.norm(y_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(sun_body_vec_step, z_ref_orb_body_step),
				np.linalg.norm(sun_body_vec_step) * np.linalg.norm(z_ref_orb_body_step)
			)
		)
		
		#	append angles to output array
		illum_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		illum_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###############################################################################################
		#	rotation vector validation
		###############################################################################################
#		print('\n######################################################')
#		print('Rotation Vector Validation')
#		print('######################################################')
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(rot_vec_orb_unit_step, x_ref_orb_step),
				np.linalg.norm(rot_vec_orb_unit_step) * np.linalg.norm(x_ref_orb_step)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(rot_vec_orb_unit_step, y_ref_orb_step),
				np.linalg.norm(rot_vec_orb_unit_step) * np.linalg.norm(y_ref_orb_step)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(rot_vec_orb_unit_step, z_ref_orb_step),
				np.linalg.norm(rot_vec_orb_unit_step) * np.linalg.norm(z_ref_orb_step)
			)
		)
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(rot_vec_body_unit, x_ref_orb_body_step),
				np.linalg.norm(rot_vec_body_unit) * np.linalg.norm(x_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(rot_vec_body_unit, y_ref_orb_body_step),
				np.linalg.norm(rot_vec_body_unit) * np.linalg.norm(y_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(rot_vec_body_unit, z_ref_orb_body_step),
				np.linalg.norm(rot_vec_body_unit) * np.linalg.norm(z_ref_orb_body_step)
			)
		)
		
		#	append angles to output array
		rot_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		rot_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###############################################################################################
		#	geom z-axis validation
		###############################################################################################
#		print('\n######################################################')
#		print('Geom z-Axis Validation')
#		print('######################################################')
		###########################################################################################
		#	orbital data
	
		#	angle to orbital x-axis
		x_angle_orb_step = np.arccos(
			np.divide(
				np.dot(z_geom_orb_step, x_ref_orb_step),
				np.linalg.norm(z_geom_orb_step) * np.linalg.norm(x_ref_orb_step)
			)
		)
		
		#	angle to orbital y-axis
		y_angle_orb_step = np.arccos(
			np.divide(
				np.dot(z_geom_orb_step, y_ref_orb_step),
				np.linalg.norm(z_geom_orb_step) * np.linalg.norm(y_ref_orb_step)
			)
		)
		
		#	angle to orbital z-axis
		z_angle_orb_step = np.arccos(
			np.divide(
				np.dot(z_geom_orb_step, z_ref_orb_step),
				np.linalg.norm(z_geom_orb_step) * np.linalg.norm(z_ref_orb_step)
			)
		)
	
		###########################################################################################
		#	body data
	
		#	angle to body x-axis
		x_angle_body_step = np.arccos(
			np.divide(
				np.dot(z_geom_body_step, x_ref_orb_body_step),
				np.linalg.norm(z_geom_body_step) * np.linalg.norm(x_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		y_angle_body_step = np.arccos(
			np.divide(
				np.dot(z_geom_body_step, y_ref_orb_body_step),
				np.linalg.norm(z_geom_body_step) * np.linalg.norm(y_ref_orb_body_step)
			)
		)
		
		#	angle to body y-axis
		z_angle_body_step = np.arccos(
			np.divide(
				np.dot(z_geom_body_step, z_ref_orb_body_step),
				np.linalg.norm(z_geom_body_step) * np.linalg.norm(z_ref_orb_body_step)
			)
		)
		
		#	append angles to output array
		geomz_orb_plot_angles.append([x_angle_orb_step, y_angle_orb_step, z_angle_orb_step])
		geomz_body_plot_angles.append([x_angle_body_step, y_angle_body_step, z_angle_body_step])

		###########################################################################################
		#	Data Output
		###########################################################################################
	
		#	set image index
		index = i

		###########################################################################################
		#	populate output attitude array
#		attitude_out[index, :] = attit_step
		attitude_out[index, :] = np.array([0.0, 0.0, 0.0]) 
	
		###########################################################################################
		#	orbital frame vector data

		#	julian date
		vector_data_orb[index, 0] = jdate

		#	reference vectors
		vector_data_orb[index, 1:4] = x_ref_orb_step
		vector_data_orb[index, 4:7] = y_ref_orb_step
		vector_data_orb[index, 7:10] = z_ref_orb_step

		#	geometry vectors
		vector_data_orb[index, 10:13] = x_geom_orb_step
		vector_data_orb[index, 13:16] = y_geom_orb_step
		vector_data_orb[index, 16:19] = z_geom_orb_step

		#	earth, velocity and plane normal vectors
		vector_data_orb[index, 19:22] = earth_orb_vec_step
		vector_data_orb[index, 22:25] = sat_orb_vel_step
		vector_data_orb[index, 25:28] = orb_norm_orb_unit_step

		#	sun, site and rotation vector
		vector_data_orb[index, 28:31] = sun_orb_vec_step
		vector_data_orb[index, 31:34] = site_orb_vec_step
		vector_data_orb[index, 34:37] = rot_vec_orb_unit_step

		###########################################################################################
		#	body frame vector data

		#	julian date
		vector_data_orb[index, 0] = jdate

		#	reference vectors
		vector_data_body[index, 1:4] = x_ref_orb_body_step
		vector_data_body[index, 4:7] = y_ref_orb_body_step
		vector_data_body[index, 7:10] = z_ref_orb_body_step

		#	geometry vectors
		vector_data_body[index, 10:13] = x_geom_body_step
		vector_data_body[index, 13:16] = y_geom_body_step
		vector_data_body[index, 16:19] = z_geom_body_step

		#	earth, velocity and plane normal vectors
		vector_data_body[index, 19:22] = earth_body_vec_step
		vector_data_body[index, 22:25] = sat_body_vel_step
		vector_data_body[index, 25:28] = orb_norm_body_unit_step

		#	sun, site and rotation vector
		vector_data_body[index, 28:31] = sun_body_vec_step
		vector_data_body[index, 31:34] = site_body_vec_step
		vector_data_body[index, 34:37] = rot_vec_body_unit_step

		###########################################################################################
		#	inertial frame vector data
	
		###########################################################################################
		#	orbital mesh files
		###########################################################################################

		#	generate image ID
		if index < 10:
			geom_id = '000' + str(index)
		elif 10 <= index < 100:
			geom_id = '00' + str(index)
		elif 100 <= index < 1000:
			geom_id = '0' + str(index)
		elif 1000 <= index < 10000:
			geom_id = str(index)

		#	saving orbital mesh file
		mesh_file = 'orbital_mesh_{}.stl'.format(geom_id)
		orb_mesh_step.save(os.path.join(render_geom_dir, mesh_file), mode=stl.Mode.ASCII)

		#	save and end
		if i >= len_record - 1:
			break

	###########################################################################################
	#	convert angle outputs to numpy arrays

	obs_orb_plot_angles = np.array(obs_orb_plot_angles)
	obs_body_plot_angles = np.array(obs_body_plot_angles)

	illum_orb_plot_angles = np.array(illum_orb_plot_angles)
	illum_body_plot_angles = np.array(illum_body_plot_angles)

	rot_orb_plot_angles = np.array(rot_orb_plot_angles)
	rot_body_plot_angles = np.array(rot_body_plot_angles)

	geomz_orb_plot_angles = np.array(geomz_orb_plot_angles)
	geomz_body_plot_angles = np.array(geomz_body_plot_angles)

	if validation_plot_flag is True:
		###########################################################################################
		#	initialise figure
		fig, ax = plt.subplots(1, 4, figsize=(20, 6))
	
		###########################################################################################
		ax[0].title.set_text('Observation Vector Validation')
		ax[0].plot(obs_orb_plot_angles[:, 0], label='Angle from Earth_Vec in Orb Frame',  color='r')
		ax[0].plot(obs_orb_plot_angles[:, 1], label='Angle from Vel_Vec in Orb Frame',    color='g')
		ax[0].plot(obs_orb_plot_angles[:, 2], label='Angle from Plane_Norm in Orb Frame', color='b')
	
		ax[0].plot(
			obs_body_plot_angles[:, 0], label='Angle from Earth_Vec in Body Frame',
			color='tab:red'
		)

		ax[0].plot(
			obs_body_plot_angles[:, 1], label='Angle from Vel_Vec in Body Frame',
			color='tab:green'
		)

		ax[0].plot(
			obs_body_plot_angles[:, 2], label='Angle from Plane_Vec in Body Frame',
			color='tab:blue'
		)

		ax[0].legend()
	
		###########################################################################################
		ax[1].title.set_text('Illumination Vector Validation')
		ax[1].plot(illum_orb_plot_angles[:, 0], label='Angle from Earth_Vec in Orb Frame', color='r')
		ax[1].plot(illum_orb_plot_angles[:, 1], label='Angle from Vel_Vec in Orb Frame', color='g')
		ax[1].plot(illum_orb_plot_angles[:, 2], label='Angle from Plane_Norm in Orb Frame', color='b')
	
		ax[1].plot(
			illum_body_plot_angles[:, 0], label='Angle from Earth_Vec in Body Frame',
			color='tab:red'
		)

		ax[1].plot(
			illum_body_plot_angles[:, 1], label='Angle from Vel_Vec in Body Frame',
			color='tab:green'
		)

		ax[1].plot(
			illum_body_plot_angles[:, 2], label='Angle from Plane_Vec in Body Frame',
			color='tab:blue'
		)

		ax[1].legend()
	
		###########################################################################################
		ax[2].title.set_text('Rotation Vector Validation')
		ax[2].plot(rot_orb_plot_angles[:, 0], label='Angle from Earth_Vec in Orb Frame', color='r')
		ax[2].plot(rot_orb_plot_angles[:, 1], label='Angle from Vel_Vec in Orb Frame', color='g')
		ax[2].plot(rot_orb_plot_angles[:, 2], label='Angle from Plane_Norm in Orb Frame', color='b')
	
		ax[2].plot(
			rot_body_plot_angles[:, 0], label='Angle from Earth_Vec in Body Frame',
			color='tab:red'
		)

		ax[2].plot(
			rot_body_plot_angles[:, 1], label='Angle from Vel_Vec in Body Frame',
			color='tab:green'
		)

		ax[2].plot(
			rot_body_plot_angles[:, 2], label='Angle from Plane_Vec in Body Frame',
			color='tab:blue'
		)

		ax[2].legend()
	
		###########################################################################################
		ax[3].title.set_text('Geometry Z Axis Validation')
		ax[3].plot(geomz_orb_plot_angles[:, 0], label='Angle from Earth_Vec in Orb Frame', color='r')
		ax[3].plot(geomz_orb_plot_angles[:, 1], label='Angle from Vel_Vec in Orb Frame', color='g')
		ax[3].plot(geomz_orb_plot_angles[:, 2], label='Angle from Plane_Norm in Orb Frame', color='b')
	
		ax[3].plot(
			geomz_body_plot_angles[:, 0], label='Angle from Earth_Vec in Body Frame',
			color='tab:red'
		)

		ax[3].plot(
			geomz_body_plot_angles[:, 1], label='Angle from Vel_Vec in Body Frame',
			color='tab:green'
		)

		ax[3].plot(
			geomz_body_plot_angles[:, 2], label='Angle from Plane_Vec in Body Frame',
			color='tab:blue'
		)

		ax[3].legend()
	
		print('\nShowing validation plots.')
		#plt.show()

	return vector_data_body, vector_data_orb, attitude_out

###################################################################################################
#	Function: 		kep2rv
#	Description: 	returns state vector
#
###################################################################################################

def kep2rv(raan, argp, nu, a, i, e):

	#	define earth-sun mu in km3/s2
	mu_earth = 3.986004418e14 * 1e-9

	#	calculate p
	p = a * (1 - e**2)

	#	generate state vector
	cosnu = np.cos(nu)
	sinnu = np.sin(nu)
	
	temp = p / (1.0  + e * cosnu)

	rpqw = np.array([
		temp * cosnu,
		temp * sinnu,
		0.0
	])

	if abs(p) < 0.0001:
	    p = 0.0001

	vpqw = np.array([
		-sinnu * np.sqrt(mu_earth)  / np.sqrt(p),
		(e + cosnu) * np.sqrt(mu_earth) / np.sqrt(p),
		0.0
	])
	
	tempvec = rot3(rpqw, -argp)
	tempvec = rot1(tempvec, -i)
	r_orb = rot3(tempvec, -raan)

	tempvec = rot3(vpqw, -argp)
	tempvec = rot1(tempvec, -i)
	v_orb = rot3(tempvec, -raan)
	
	r_orb = np.transpose(r_orb)
	v_orb = np.transpose(v_orb)
	
	return(r_orb, v_orb)

###################################################################################################
#	Function: 		rv2kep
#	Description: 	returns keplerian orbital elements
#
###################################################################################################

def rv2kep(r_vec, v_vec):

	#	define earth-sun mu in km3/s2
	mu_earth = 3.986004418e14 * 1e-9
	
	r = np.linalg.norm(r_vec)
	v = np.linalg.norm(v_vec)
	
	##################################################
	#	calculate angular momentum vector
	h_vec = np.cross(r_vec, v_vec)
	h = np.linalg.norm(h_vec)
	
	##################################################
	#	caculate line of nodes vector
	n_vec = np.array([-h_vec[1], h_vec[0], 0.0])
	n = np.linalg.norm(n_vec)
	
	##################################################
	#	calculate eccentricity vector
	c1 = v**2 - (mu_earth / r)
	rdotv = np.dot(r_vec, v_vec)
	
	e_vec = np.array([
		(c1 * r_vec[0] - rdotv * v_vec[0]) / mu_earth,
		(c1 * r_vec[1] - rdotv * v_vec[1]) / mu_earth,
		(c1 * r_vec[2] - rdotv * v_vec[2]) / mu_earth
	])
	
	e = np.linalg.norm(e_vec)
	
	##################################################
	#	semi major axis
	sme = (0.5 * v**2) - (mu_earth / r)
	a = -mu_earth / (2 * sme)
	
	##################################################
	#	semi-latus rectum
	p = h**2 / mu_earth
	
	##################################################
	#	inclination
	hk = h_vec[2] / h
	i = np.arccos(hk)
	
	##################################################
	#	right ascension of ascending node
	
	temp = n_vec[0] / n
	
	if abs(temp) > 1.0:
		temp = 1.0
	
	omega = np.arccos(temp)
	
	if n_vec[1] < 0.0:
		omega = (np.pi * 2) - omega
	
	##################################################
	#	argument of perigee
	
	temp = np.dot(n_vec, e_vec) / np.dot(n, e)
	
	if abs(temp) > 1.0:
		temp = 1.0
	
	arg_p = np.arccos(temp)
	
	if e_vec[2] < 0.0:
		arg_p = (2 * np.pi) - arg_p
	
	##################################################
	#	true anomaly
	
	temp = np.dot(e_vec, r_vec) / np.dot(e, r)
	
	if abs(temp) > 1.0:
		temp = 1.0
	
	nu = np.arccos(temp)
	
	if rdotv < 0.0:
		nu = (2 * np.pi) - nu

	return [omega, arg_p, nu, i, e, a]

###################################################################################################
#	Function: 		r2polar
#	Description: 	returns polar coordinate angles
#
###################################################################################################

def r2polar(inc_ref_vec, azm_ref_vec, r):

	#	inclination: angle between vector and inclination reference vector
	inclination = np.arccos(
		np.divide(
			np.dot(r, inc_ref_vec),
			np.linalg.norm(r) * np.linalg.norm(inc_ref_vec)
		)
	)
	
	#	define yz component of rotation vector
	yz_comp = np.array([0.0, r[1], r[2]])
	
	#	azimuth: angle to azimuthal reference vector in the yz plane
	azimuth = np.arccos(
		np.divide(
			np.dot(yz_comp, azm_ref_vec),
			np.linalg.norm(yz_comp) * np.linalg.norm(azm_ref_vec)
		)
	)
	
	#	corrected for -x
	if r[1] < 0:
		azimuth = np.pi + (np.pi - azimuth)
	
	#	correct inclination for inveiw axis
	if inclination > (np.pi / 2):
	
		azimuth =+ np.pi 
		inclination = abs(inclination - np.pi)
	
	return inclination, azimuth
