###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#
#
#
#
#
#
#
###################################################################################################

print('Script started...')

###################################################################################################
#	imports

#	standard packages
import os
import sys
import time
import numpy as np

from scipy.integrate import odeint
from scipy.integrate import simps

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

#	stl
import stl
from stl import mesh

#	acceleration
from functions.function_torque import plot_graphs, accel_grav

#	transformation
from functions.function_transform import teme2itrf, \
											site_vec, \
											rot_mat, \
											mat2axis, \
											axis2mat, \
											axis2mat2, \
											mat2angles, \
											mat2angles2, \
											rot3, \
											rot2, \
											rot1, \
											body2orb_mat, \
											body2orb_vel

#	function list
from functions.function_list import date2jd, \
									jd2date, \
									get_times, \
									interp, \
									eop_interpolate, \
									req_horizons, \
									gen_sgp4

###################################################################################################
#	Initialisation
###################################################################################################


###################################################################################################
#	simulation parameters

#	output graphs at each timestep
graphical_output = 1

#	reference frame selection
orb_frame_out = True
body_frame_out = False

#	simulation length
len_reccord = 500

#	set no data reccord
if orb_frame_out is False and body_frame_out is False:
	start_point = -1
else:
	start_point = 0

###################################################################################################
#	object parameters

#	object mass [kg]
object_mass = 1435

#	initial attitude in orbital frame
initial_attitude = np.array((0.0, np.pi * 0.05, np.pi * 0.4))

#	initial attitude motion in body frame
attitude_motion = np.array((0.0, 0.0, 0.0))

#############################################################################################
#	data input/output
#############################################################################################

#	working directory
cwd = os.getcwd()

#	path to data directory
path_data = '../../../data_archive/GLIMMA_data'

#	data path
file_path = os.path.join(cwd, path_data)

#############################################################################################
#	inputs

#	positions
pos_dir = 'pos_dir'
sat_orb_file = 'sat_orb.txt'
sun_orb_file = 'sun_orb.txt'
earth_orb_file = 'earth_orb.txt'
site_orb_file = 'site_orb.txt'

#	geometry
geom_dir = 'geom_dir/SL-8'
geom_file = 'sl8_grav.stl'

#############################################################################################
#	outputs

render_dir = 'render_dir'
render_geom_dir = 'geom_files'

render_geom_dir_orb  = os.path.join(render_geom_dir, 'orb_frame')
render_geom_dir_body = os.path.join(render_geom_dir, 'body_frame')

render_data_orb = 'render_data_orb.txt'
render_data_body = 'render_data_body.txt'


if start_point != -1:
	print('Recording data. . .')

	if orb_frame_out is True:

		print('orbital frame data.')

		#	remove vector data file
		if os.path.isfile(os.path.join(render_dir, render_data_orb)):
			os.remove(os.path.join(render_dir, render_data_orb))

		#	check geometry directory
		geom_file_dir = os.path.join(file_path, render_dir, render_geom_dir_orb)
		geom_dir_list = os.listdir(geom_file_dir)
		ngeom_files = len(geom_dir_list)

		#	if directory is not clean
		if ngeom_files != 0:

			#	ask user to delete files
			check = input('{:d} orbital geometry files exist. Delete? [y/n]'.format(ngeom_files))

			#	yes
			if check == 'y':
				for f in geom_dir_list:
					if f.endswith('.stl'):
						os.remove(os.path.join(geom_file_dir, f))
				print('files deleted')

			#	no
			elif check == 'n':
				print('files retained')

			#	error
			else:
				print('error in provided answer')
				sys.exit()

		else:
			print('Geom directory is clean.')

	if body_frame_out is True:

		print('body frame data.')

		#	remove vector data file
		if os.path.isfile(os.path.join(render_dir, render_data_body)):
			os.remove(os.path.join(render_dir, render_data_body))

#############################################################################################
#	load data files 
#############################################################################################

#	time step in seconds (property of the data)
time_step = 60

#	load orb sat data
sat_orb_raw = np.loadtxt(
	os.path.join(file_path, pos_dir, sat_orb_file),
	delimiter=','
)

#	load orb sun data
sun_orb_raw = np.loadtxt(
	os.path.join(file_path, pos_dir, sun_orb_file),
	delimiter=','
)

#	load orb sun data
earth_orb_raw = np.loadtxt(
	os.path.join(file_path, pos_dir, earth_orb_file),
	delimiter=','
)

#	load orb site data
site_orb_raw = np.loadtxt(
	os.path.join(file_path, pos_dir, site_orb_file),
	delimiter=','
)

#############################################################################################
#	load geometry
#############################################################################################

#	static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(cwd, path_data, geom_dir, geom_file))

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

#############################################################################################
#	convert to orbital frame

#	orb mesh
orb_mesh = mesh.Mesh(body_mesh.data.copy())

#	get mesh properties for CoM
volume, com, inertia = body_mesh.get_mass_properties()

#	translate by CoM
orb_mesh.translate(-com)
body_mesh.translate(-com)

#	update mesh properties
volume, cog, inertia = body_mesh.get_mass_properties()

#	scale inertia by mass
moi_body = np.multiply(object_mass, inertia)

#	set areas and vectors
areas = body_mesh.areas
body_vectors = body_mesh.vectors
orb_vectors = orb_mesh.vectors

#	define known vectors
x_unit = np.array((1.0, 0.0, 0.0))
y_unit = np.array((0.0, 1.0, 0.0))
z_unit = np.array((0.0, 0.0, 1.0))

###################################################################################################
#	attitude profile
###################################################################################################

#	orbital motion in radians per second
orbital_mot = 9.141e-4

#	define orbital to inertial angular velocity
ang_vel_orb2inertial = np.array((0.0, 0.0, -orbital_mot))

###################################################################################################
#	BODY

ang_vel_body = np.copy(attitude_motion)

body_attitude = initial_attitude

###################################################################################################
#	ORBITAL

#	apply initial attitude to mesh
att_rot_mat = rot_mat(initial_attitude)
orb_mesh.rotate_using_matrix(att_rot_mat)

#	apply inital attitude to orbital axis units
x_orb_unit = np.dot(x_body_unit, att_rot_mat)
y_orb_unit = np.dot(y_body_unit, att_rot_mat)
z_orb_unit = np.dot(z_body_unit, att_rot_mat)

#	update vectors
orb_vectors = orb_mesh.vectors

#	convert orbital frame
body2orb = body2orb_vel(
	body_attitude[0],
	ang_vel_body[0],
	body_attitude[1],
	ang_vel_body[1],
	body_attitude[2],
	ang_vel_body[2]
)

#	set orbital angular velocity
ang_vel_orb = np.copy(body2orb)

#	create temporary mesh loop update
temp_orb_mesh = mesh.Mesh(orb_mesh.data.copy())

###################################################################################################
#	time loop
###################################################################################################

#	initilise vector data
vector_data_inertial = np.zeros((len_reccord, 19))
vector_data_body = np.zeros((len_reccord, 19))
vector_data_orb = np.zeros((len_reccord, 19))

#	initialise plot data
ang_vel_plot = []
ang_plot = []
ang_vel_plot2 = []
ang_vel_plot3 = []

#	simulation start date
jdate_start = sun_orb_raw[0, 0]

#	set step angular velocities
ang_vel_body_step = np.copy(ang_vel_body)
ang_vel_orb_step = np.copy(ang_vel_orb)

#	set step attitude
step_attitude = np.copy(initial_attitude)

#	set step angle offsets
ang_step_orb = np.array((0.0, 0.0, 0.0))
ang_step_body = np.array((0.0, 0.0, 0.0))

#	apply to orbital axis units
x_orb_unit_step = np.copy(x_orb_unit)
y_orb_unit_step = np.copy(y_orb_unit)
z_orb_unit_step = np.copy(z_orb_unit)

for i, (sat_orb_line, sun_orb_line, earth_orb_line,	site_orb_line) in enumerate(
		zip(sat_orb_raw, sun_orb_raw, earth_orb_raw, site_orb_raw)
	):

	###############################################################################################
	#	parse data
	###############################################################################################

	###############################################################################################
	#	time

	#	set date and previous date
	jdate = sun_orb_line[0]

	if i == 0:
		jdate_prev = np.copy(jdate)

		step_factor = 24 * 60 * 60 / time_step

		time_since = jdate_prev - jdate_start

	else:
		jdate_prev = jdate
		jdate = sun_orb_line[0]

	#	time since start

	if i % 10 == 0:
		step_factor = 24 * 60 * 60 / time_step
		time_since = jdate_prev - jdate_start
		print('Time since start = {:d} [Mins]\n'.format(
			int(round(step_factor * time_since)))
		)

	###############################################################################################
	#	orbital frame

	#	satellite position and velocity in m/s
	sat_orb_pos = sat_orb_line[1:4]
	sat_orb_vel = sat_orb_line[4:7]
	sun_orb_vec = sun_orb_line[1:4]
	earth_orb_vec = earth_orb_line[1:4]
	site_orb_vec = site_orb_line[1:4]

	#	orbital unit vectors
	sun_orb_unit = np.divide(sun_orb_vec, np.linalg.norm(sun_orb_vec))
	earth_orb_unit = np.divide(earth_orb_vec, np.linalg.norm(earth_orb_vec))
	site_orb_unit = np.divide(site_orb_vec, np.linalg.norm(site_orb_vec))

	###############################################################################################
	#	apply attitude motion
	###############################################################################################

	###########################################################################################
	#	body frame

	#	apply initial attitude
	ang_step = initial_attitude

	#	generate rotation matrix of step
	rot_mat_step = rot_mat(ang_step)

	#	take inverse for application to vectors
	rot_mat_inv = np.linalg.inv(rot_mat_step)

	#	apply to orbital position vectors
	sat_body_pos = np.dot(sat_orb_pos, rot_mat_inv)
	sat_body_vel = np.dot(sat_orb_vel, rot_mat_inv)
	sun_body_vec = np.dot(sun_orb_vec, rot_mat_inv) 
	earth_body_vec = np.dot(earth_orb_vec, rot_mat_inv)
	site_body_vec = np.dot(site_orb_vec, rot_mat_inv)

	#	apply total attitude motion of step
	if i != 0:
		ang_step_body = ang_step_body + np.multiply(ang_vel_body_step, time_step)

	#	generate rotation matrix of step
	rot_mat_step = rot_mat(ang_step_body)

	#	apply to inertial position vectors
	sat_body_pos = np.dot(sat_body_pos, rot_mat_step)
	sat_body_vel = np.dot(sat_body_vel, rot_mat_step)
	sun_body_vec = np.dot(sun_body_vec, rot_mat_step) 
	earth_body_vec = np.dot(earth_body_vec, rot_mat_step)
	site_body_vec = np.dot(site_body_vec, rot_mat_step)

	#	body unit vectors
	sun_body_unit = np.divide(sun_body_vec, np.linalg.norm(sun_body_vec))
	earth_body_unit = np.divide(earth_body_vec, np.linalg.norm(earth_body_vec))
	site_body_unit = np.divide(site_body_vec, np.linalg.norm(site_body_vec))

	###########################################################################################
	#	orbital frame

	ang_step_orb = np.multiply(ang_vel_orb_step, time_step)

	if i != 0:
		print(ang_step_orb)
		sys.exit()

	#	generate rotation matrix of step ang
	rot_mat_step = rot_mat(ang_step_orb)

	#	apply to temp orbital mesh
	temp_orb_mesh.rotate_using_matrix(rot_mat_step)

	#	update vectors
	temp_orb_vectors = temp_orb_mesh.vectors
	
	#	apply to orbital axis units
	x_orb_unit_step = np.dot(x_orb_unit_step, rot_mat_step)
	y_orb_unit_step = np.dot(y_orb_unit_step, rot_mat_step)
	z_orb_unit_step = np.dot(z_orb_unit_step, rot_mat_step)

	###############################################################################################
	#	TORQUE ANALYSES
	###############################################################################################

	###############################################################################################
	#	gravity grad torque
 
	print('\nAttitude [body] = ', body_attitude)

	#	set local vertical offset angles
	theta = body_attitude[2]
	phi = body_attitude[1]

	# radial vector converted to meters
	radial_vect = np.linalg.norm(np.multiply(earth_body_vec, 1000.0))

	#	calculate applied acceleration
	ang_accel_grav = accel_grav(radial_vect, moi_body, theta, phi)

	#	change in angular velocity in step
	ang_vel_grav = ang_accel_grav * time_step

	for i2, el in enumerate(ang_vel_grav):
		if -1e-10 < el < 1e-10:
			ang_vel_grav[i2] = 0.0

	print('Change in attitude motion [body] = ', ang_vel_grav)

	###############################################################################################
	#	atmospheric torque
 

	###############################################################################################
	#	SRP torque
 

	###############################################################################################
	#	Graphical Output
	###############################################################################################


	###############################################################################################

	if graphical_output == 1:

		geom_orb_units = np.vstack((x_orb_unit_step, y_orb_unit_step, z_orb_unit_step))
		vector_orb_units = np.vstack((sun_orb_unit, site_orb_unit, earth_orb_unit))
		geom_body_units = np.vstack((x_unit, y_unit, z_unit))
		vector_body_units = np.vstack((sun_body_unit, site_body_unit, earth_body_unit))

		plot_graphs(
			geom_orb_units, vector_orb_units, sat_orb_vel, temp_orb_vectors,
			geom_body_units, vector_body_units, sat_body_vel, body_vectors
		)

	###############################################################################################
	#	update for next time step
	###############################################################################################

	###############################################################################################
	#	record data for final graphs

	#	orb
	ang_vel_plot.append(ang_vel_orb_step)
	ang_plot.append(body_attitude)

	ang_vel_plot2.append(ang_vel_body_step)
	ang_vel_plot3.append(ang_vel_grav)

	###############################################################################################
	#	update values

	#	change to angular velocity of time step in body frame
	delta_ang_vel = ang_vel_grav


	#	update body angluar velocity for gravity gradient
	ang_vel_body_step = ang_vel_body_step + delta_ang_vel

	print('New Attitude motion [body] = ', ang_vel_body_step)

	#	find new orbital angluar velocity
	body2orb = body2orb_vel(
		step_attitude[0],
		ang_vel_body_step[0],
		step_attitude[1],
		ang_vel_body_step[1],
		step_attitude[2],
		ang_vel_body_step[2]
	)

	ang_vel_orb_step = np.copy(body2orb)

	#	update attitude
	body_attitude = body_attitude + np.multiply(ang_vel_body_step, time_step)

	if body_attitude[0] > np.pi:
		body_attitude[0] = body_attitude[0] - (2 * np.pi)
	elif body_attitude[0] < -np.pi:
		body_attitude[0] = body_attitude[0] + (2 * np.pi)

	if body_attitude[1] > np.pi:
		body_attitude[1] = body_attitude[1] - (2 * np.pi)
	elif body_attitude[1] < -np.pi:
		body_attitude[1] = body_attitude[1] + (2 * np.pi)

	if body_attitude[2] > np.pi:
		body_attitude[2] = body_attitude[2] - (2 * np.pi)
	elif body_attitude[2] < -np.pi:
		body_attitude[2] = body_attitude[2] + (2 * np.pi)

	###############################################################################################
	#	Data Output
	###############################################################################################

	#	if not reccording data, then break
	if start_point == -1:
		if i >= len_reccord:
			break

	#	else if reccording data and within reccord range
	elif start_point != -1 and i >= start_point:

		#	set image index
		index = i - start_point

		if orb_frame_out is True:

			#	orbital frame vector data
			vector_data_orb[index, 0] = jdate
			vector_data_orb[index, 1:4] = sun_orb_vec
			vector_data_orb[index, 4:7] = site_orb_vec
			vector_data_orb[index, 7:10] = earth_orb_vec
			vector_data_orb[index, 10:13] = x_orb_unit_step
			vector_data_orb[index, 13:16] = sat_orb_vel
	
			#	generate image ID
			if index < 10:
				geom_id = '000' + str(index)
			elif 10 <= index < 100:
				geom_id = '00' + str(index)
			elif 100 <= index < 1000:
				geom_id = '0' + str(index)
			elif 1000 <= index < 10000:
				geom_id = str(index)
	
			#	saving orbital mesh file
			mesh_file = 'orbital_mesh_{}.stl'.format(geom_id)
			mesh_path = os.path.join(file_path, render_dir, render_geom_dir, 'orb_frame')
			temp_orb_mesh.save(os.path.join(mesh_path, mesh_file), mode=stl.Mode.ASCII)

		if body_frame_out is True:

			#	body frame vector data
			vector_data_body[index, 0] = jdate
			vector_data_body[index, 1:4] = sun_body_vec
			vector_data_body[index, 4:7] = site_body_vec
			vector_data_body[index, 7:10] = earth_body_vec
			vector_data_body[index, 10:13] = x_body_unit
			vector_data_body[index, 13:16] = sat_body_vel

		#	save and end
		if i >= len_reccord - 1:

			print('Finished. Saving...')

			if orb_frame_out is True:

				#	orbital
				np.savetxt(
					os.path.join(cwd, path_data, render_dir, render_data_orb),
					vector_data_orb,
					delimiter=','
				)

			if body_frame_out is True:

				#	body
				np.savetxt(
					os.path.join(cwd, path_data, render_dir, render_data_body),
					vector_data_body,
					delimiter=','
				)

			break

###################################################################################################
#	output graphs

ang_plot = np.array(ang_plot)

for i, line in enumerate(ang_plot):

	if line[0] > np.pi:
		ang_plot[i, 0] = -(2 * np.pi - line[0])

	if line[1] > np.pi:
		ang_plot[i, 1] = -(2 * np.pi - line[1])
	
	if line[2] > np.pi:
		ang_plot[i, 2] = -(2 * np.pi - line[2])


#	attitude
plot_data1 = np.array(ang_plot)
temp1 = np.arange(0, len(plot_data1), 1)

#	angular velocity body
plot_data3 = np.array(ang_vel_plot2)
temp3 = np.arange(0, len(plot_data3), 1)

#	angular velocity grav
plot_data4 = np.array(ang_vel_plot3)
temp4 = np.arange(0, len(plot_data4), 1)

#	attitude
plt.figure(1)
plt.title('Attitude [body]')
plt.plot(temp1, plot_data1[:, 0], label='Attitude-x')
plt.plot(temp1, plot_data1[:, 1], label='Attitude-y')
plt.plot(temp1, plot_data1[:, 2], label='Attitude-z')
plt.legend()

#	total angular velocity
plt.figure(3)
plt.title('Total Angular Velocity [body]')
plt.plot(temp3, plot_data3[:, 0], label='Ang vel - x')
plt.plot(temp3, plot_data3[:, 1], label='Ang vel - y')
plt.plot(temp3, plot_data3[:, 2], label='Ang vel - z')
plt.legend()

#	angular velocity - grav
plt.figure(4)
plt.title('Change in Angular Velocity [grav]')
plt.plot(temp4, plot_data4[:, 0], label='Ang vel - x')
plt.plot(temp4, plot_data4[:, 1], label='Ang vel - y')
plt.plot(temp4, plot_data4[:, 2], label='Ang vel - z')
plt.legend()

plt.show()
