#############################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
#############################################################################################
#
#
#
#
#
#
#
#############################################################################################

#	standard packages
import os
import sys

#	numpy
import numpy as np

#	numba
import numba

#	progress bar
import progressbar

#	custom transformation functions
from tools.functions.functions_transform import teme2itrf, gcrf2itrf, scrf2orb, itrf2razel, \
		site_vec, rot1, rot2, rot3

#	custom general functions
from tools.functions.functions_list import date2jd, \
									jd2date, \
									jday_jfrac, \
									get_times

###################################################################################################
#	Function: 		glim_trans
#	Description: 	performs all required rotations
#
###################################################################################################

#	a = 13, b = 16, c = 2, d = 9, e = 6, f = 4
#	[i, 13], [i, 16], [i, 2], [i, 9], [i, 9], [i, 9], [i, 9], [6], [4]
#	->
#	[i, 6], [i, 6], [i,6], [i, 9], [i, 6], [i, 6]

# @numba.guvectorize(
# 	[(
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:],
# 		numba.float64[:],
# 		numba.float64[:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:],
# 		numba.float64[:,:]
# 	)],
# 	'(i,a),(i,b),(i,c),(i,d),(i,d),(i,d),(i,d),(e),(f),(g)->(i,e),(i,e),(i,e),(i,d),(i,e),(i,e)'
# )
def glim_trans(
		data_array,
		eop_data,
		dat_data,
		pn_array,
		pm_array,
		st_array,
		c2t_array,
		site_data,
		time_data,
		simulation_data,
		out_data_itrf,
		out_data_gcrf,
		out_data_scrf,
		out_data_orb, out_data_orb_sat,
		out_data_azel
	):

	###############################################################################################
	#	initilise
	###############################################################################################

	#	simulation data
	logging_level, time_step = simulation_data

	#	time data
	step_size = data_array[1, 0] - data_array[0, 0]
	jdaytt_start, jfractt_start, jdayut1_start, jfracut1_start = time_data
	jdtt_start = jdaytt_start + jfractt_start
	jdut1_start = jdayut1_start + jfracut1_start

	#	parse site data
	site_itrf = site_data[0:3]
	site_latgd = site_data[3]
	site_lon = site_data[4]
	site_alt = site_data[5]

	###############################################################################################
	#	time loop
	###############################################################################################

	loop_data = zip(data_array, eop_data, dat_data, pn_array, pm_array, st_array, c2t_array)

	#	set enumeration value
	if logging_level == 0:
		enumeration_value = loop_data
	else:
		enumeration_value = progressbar.progressbar(loop_data, max_value=len(data_array))

	for step, loop_line in enumerate(enumeration_value):

		###########################################################################################
		#	parse loop line
		data_line, eop_line, dat_line, pn_line, pm_line, st_line, c2t_line = loop_line

		###########################################################################################
		#	generate time data
		time_since = step * step_size
	
		jdtt = jdtt_start + time_since
		jdaytt_epoch, jfractt_epoch = jday_jfrac(jdtt)

		jdut1 = jdut1_start + time_since
		jdayut1_epoch, jfracut1_epoch = jday_jfrac(jdut1)

		###########################################################################################
		#	parse data inputs

		#	julian date
		jdate = data_line[0]

		#	GCRF data
		sat_gcrf_line = data_line[1:7]
		sat_gcrf_pos = sat_gcrf_line[0:3]
		sat_gcrf_vel = sat_gcrf_line[3:6]

		sun_gcrf_line = data_line[7:10]
		earth_gcrf_line = data_line[10:13]

		###########################################################################################
		#	parse eop parameters
		eop_mjd, eop_year, eop_mon, eop_day, \
		x_p, _, y_p, _, \
		ut_offset, _, \
		lod, _, \
		dX, _, dY, _ = eop_line

		dat = dat_line[1]

		###########################################################################################
		#	parse rotation matrices

		#	convert c2t array
		pn = np.array([
			[pn_line[0], pn_line[1], pn_line[2]],
			[pn_line[3], pn_line[4], pn_line[5]],
			[pn_line[6], pn_line[7], pn_line[8]]
		])

		#	inverse c2t array
		inv_pn = np.transpose(pn)

		#	convert c2t array
		pm = np.array([
			[pm_line[0], pm_line[1], pm_line[2]],
			[pm_line[3], pm_line[4], pm_line[5]],
			[pm_line[6], pm_line[7], pm_line[8]]
		])

		#	inverse c2t array
		inv_pm = np.transpose(pm)

		#	convert c2t array
		st = np.array([
			[st_line[0], st_line[1], st_line[2]],
			[st_line[3], st_line[4], st_line[5]],
			[st_line[6], st_line[7], st_line[8]]
		])

		#	inverse c2t array
		inv_st = np.transpose(st)

		#	convert c2t array
		c2t = np.array([
			[c2t_line[0], c2t_line[1], c2t_line[2]],
			[c2t_line[3], c2t_line[4], c2t_line[5]],
			[c2t_line[6], c2t_line[7], c2t_line[8]]
		])

		#	inverse c2t array
		inv_c2t = np.transpose(c2t)

		###########################################################################################
		#	convert to GCRF from ITRF
		###########################################################################################

		#	site
		site_gcrf_line = np.dot(inv_c2t, site_itrf)

		###########################################################################################
		#	convert GCRF frames to ITRF
		###########################################################################################
	
		#	earth rotation angle
		thetasa= 7.29211514670698e-05 * (1.0  - lod / 86400.0 )
		omegaearth = np.array((0, 0, thetasa))

		#	sun and earth
		sun_itrf_line = np.dot(c2t, sun_gcrf_line)
		earth_itrf_line = np.dot(c2t, earth_gcrf_line)

		#	satellite
		sat_itrf_pos, sat_itrf_vel = gcrf2itrf(sat_gcrf_pos, sat_gcrf_vel, pm, st, pn, lod)

		###########################################################################################
		#	convert to SCRF frame from ITRF
		###########################################################################################

		sat_scrf_pos = sat_itrf_pos - sat_itrf_pos
		sat_scrf_vel = sat_itrf_vel

		sun_scrf_line = sun_itrf_line - sat_itrf_pos
		earth_scrf_line = earth_itrf_line - sat_itrf_pos
		site_scrf_line = site_itrf - sat_itrf_pos

		###########################################################################################
		#	convert to ORB from SCRF
		###########################################################################################

		sat_orb_pos = sat_scrf_pos

		sun_orb_line, earth_orb_line, site_orb_line, sat_orb_vel = scrf2orb(
			sun_scrf_line, earth_scrf_line, site_scrf_line, sat_scrf_vel
		)

		###########################################################################################
		#	convert to AZEL from ITRF
		###########################################################################################

		#	azel
		sat_azel_line = itrf2razel(site_latgd, site_lon, site_alt, sat_itrf_pos)
		sun_azel_line = itrf2razel(site_latgd, site_lon, site_alt, sun_itrf_line)

		###########################################################################################
		#	populate return arrays
		###########################################################################################

		#	ITRF data (excludes earth and site)
		out_data_itrf[step, 0:3] = sun_itrf_line

		#	GCRF data (excludes earth and sat)
		out_data_gcrf[step, 0:3] = site_gcrf_line

		#	SCRF data (excludes earth and sat)
		out_data_scrf[step, 0:3] = sun_scrf_line
		out_data_scrf[step, 3:6] = site_scrf_line

		#	Orb data (all objects)
		out_data_orb_sat[step, 0:3] = sat_orb_pos
		out_data_orb_sat[step, 3:6] = sat_orb_vel

		out_data_orb[step, 0:3] = sun_orb_line
		out_data_orb[step, 3:6] = earth_orb_line
		out_data_orb[step, 6:9] = site_orb_line

		#	AZEL data (excludes earth and site)
		out_data_azel[step, 0:3] = sat_azel_line
		out_data_azel[step, 3:6] = sun_azel_line
