###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	parrellisation
from multiprocessing import Pool

#	progress bar
import progressbar

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	import from general function list
from tools.functions.functions_list import object_gen, \
 									date2jd, \
 									jd2date, \
 									jday_jfrac, \
 									eop_interpolate, \
 									get_times, \
 									gen_sgp4, \
									interp

###################################################################################################
#	extraction_function
###################################################################################################
 
def extract_fun(
		jd_time_data, lc_time_step, time_step,
		sun_orb_data, earth_orb_data, site_orb_data, sat_orb_data
	):

#	print('\n###############################################################')
#	print('INPUTS')
#	print('#################################################################')
#	print('\nsun orb = \n', sun_orb_data)
#	print('\nsite orb = \n', site_orb_data)

	#	remove time column
	if len(sun_orb_data[0]) == 4:
		sun_orb_data = sun_orb_data[:, 1:]
	if len(earth_orb_data[0]) == 4:
		earth_orb_data = earth_orb_data[:, 1:]
	if len(site_orb_data[0]) == 4:
		site_orb_data = site_orb_data[:, 1:]
	if len(sat_orb_data[0]) == 7:
		sat_orb_data = sat_orb_data[:, 1:]

	#	duration
	len_light_curve = len(sun_orb_data)
	duration_sec = time_step * (len_light_curve - 1)
	
	#	time range in seconds
	light_curve_sim_time = np.linspace(0, duration_sec, len_light_curve)
	
	#	stop times
	start_jdate = jd_time_data[0]
	start_jday, start_jfrac = jday_jfrac(start_jdate)
	
	stop_jdate = jd_time_data[-1]
	stop_jday, stop_jfrac = jday_jfrac(stop_jdate)

	###############################################################################################
	#	interpolate if required
	
	#	convert to light curve generation time step
	if lc_time_step != time_step:

		#	find nsteps
		prop_step = lc_time_step / (24 * 60 * 60)
		num_points = int(round((stop_jdate - start_jdate) / prop_step))
	
		#	new time range to interpolate to
		interp_range = np.linspace(start_jdate, stop_jdate, num_points + 1)
	
		#	current x spline
		spline_x = jd_time_data
	
		#	new spline x
		spline_x_new = interp_range
	
		#	generate data to interpolate
		interp_data_in = np.hstack((sat_orb_data, sun_orb_data, earth_orb_data, site_orb_data))
	
		#	initilise new y spline data
		interp_data_out = np.zeros((len(interp_range), len(interp_data_in[0])))
	
		#	perform interpolation	
		interp_data = interp(interp_data_in, interp_data_out, spline_x, spline_x_new)
	
		#	extract interpolated data
		jd_time_data = interp_range
		sat_orb_data = interp_data[:, 0:6]
		sun_orb_data = interp_data[:, 6:9]
		earth_orb_data = interp_data[:, 9:12]
		site_orb_data = interp_data[:, 12:15]
	
		#	reset data
		len_light_curve = len(jd_time_data)
		duration_sec = lc_time_step * (len_light_curve - 1)
		light_curve_sim_time = np.linspace(0, duration_sec, len_light_curve)
	
	###############################################################################################
	#	Generate Plot Data
	###############################################################################################

	#	initialise output arrays
	sun_polar_data = np.zeros([len(sun_orb_data), 2])
	site_polar_data = np.zeros([len(site_orb_data), 2])
	polar_index = 0

	phase_angles = []
	
	#	define z and x-axes in required frame
	z_vec = np.array([0.0, 0.0, 1.0])
	x_vec = np.array([-1.0, 0.0, 0.0])
	
	for (sun_vec_line, site_vec_line) in zip(sun_orb_data, site_orb_data):
	
		#	calculate polar positions of illumination and observation vectors
		sun_vec_unit = np.divide(sun_vec_line, np.linalg.norm(sun_vec_line))
		site_vec_unit = np.divide(site_vec_line, np.linalg.norm(site_vec_line))

		####################################################
		#	calculate phase angle
		phase_angle = np.arccos(
			np.divide(
				np.dot(sun_vec_unit, site_vec_unit),
				np.linalg.norm(sun_vec_unit) * np.linalg.norm(site_vec_unit)
			)
		)

		phase_angles.append(phase_angle)
	
		####################################################
		#	SUN
		el_sun = np.arccos(
			np.divide(
				np.dot(sun_vec_line, x_vec),
				np.linalg.norm(sun_vec_line) * np.linalg.norm(x_vec)
			)
		)
	
		#	remove x component
		yz_comp = np.array([0.0, sun_vec_line[1], sun_vec_line[2]])
	
		if np.array_equal(yz_comp, np.array([0.0, 0.0, 0.0])):
			az_sun = 0.0

		else:
			#	angle to z axis
			#	negative because of -x axis
			az_sun = np.arccos(
				np.divide(
					np.dot(yz_comp, z_vec),
					np.linalg.norm(yz_comp) * np.linalg.norm(z_vec)
				)
			)
		
			#	corrected for -x
			if sun_vec_line[1] < 0:
				az_sun = np.pi + (np.pi - az_sun)
	
		####################################################
		#	SITE
		el_site = np.arccos(
			np.divide(
				np.dot(site_vec_line, x_vec),
				np.linalg.norm(site_vec_line) * np.linalg.norm(x_vec)
			)
		)

		#	remove x component
		yz_comp = np.array([0.0, site_vec_line[1], site_vec_line[2]])

		if np.array_equal(yz_comp, np.array([0.0, 0.0, 0.0])):
			az_site = 0.0

		else:
			#	angle to z axis
			#	negative because of -x axis
			az_site = np.arccos(
				np.divide(
					np.dot(yz_comp, z_vec),
					np.linalg.norm(yz_comp) * np.linalg.norm(z_vec)
				)
			)
		
			#	corrected for -x
			if site_vec_line[1] < 0:
				az_site = np.pi + (np.pi - az_site)
	
		#	collect outputs
		sun_polar_data[polar_index, :] = [az_sun, el_sun]
		site_polar_data[polar_index, :] = [az_site, el_site]

		polar_index += 1

	#	convert to numpy array
	phase_angles = np.array(phase_angles)

	return sun_polar_data, site_polar_data, phase_angles, jd_time_data

###################################################################################################
#	allowd band generation
###################################################################################################

def gen_bands(az_vals):

	print('\n			Finding bands...')

	#	minumum gap for band end
	min_band_gap = 20

	#	extract sun azimuth bands
	band_start = az_vals[0]

	###############################################################################################
	#	initialise bands
	if band_start < 90:
		quad1_band = [band_start]
		quad2_band = []
		quad3_band = []
		quad4_band = []

	elif band_start < 180:
		quad1_band = [0, 0]
		quad2_band = [band_start]
		quad3_band = []
		quad4_band = []

	elif band_start < 270:
		quad1_band = [0, 0]
		quad2_band = [90, 90]
		quad3_band = [band_start]
		quad4_band = []

	elif band_start < 360:
		quad1_band = [0, 0]
		quad2_band = [90, 90]
		quad3_band = [180, 180]
		quad4_band = [band_start, sun_az_vals[-1]]

	#	loop through sun azimuth values
	for itt, az_val in enumerate(az_vals):
		#	if not first value
		if itt != 0:

			#	calculate angular gap since last point
			angular_change =  az_val - az_vals[itt - 1]

			#	if gap is greater than 10deg, then is band end
			if angular_change > min_band_gap:

				#	new band start
				new_band_start = az_val

				#	set the previous point as the band end
				band_end = az_vals[itt - 1]

				###################################################################################
				#	Quandrent 1

				if band_end < 90:

					##################################################
					#	close previous bands

					#	this is end of a quad1 band
					quad1_band = [quad1_band[0], band_end]

					##################################################
					#	initialise subsequent bands

					if new_band_start < 90:
						print('Error: Multiple bands start in quadrent 1.')
						sys.exit()

					elif new_band_start < 180:
						#	initialise quad2 band
						quad2_band = [new_band_start]

					elif new_band_start < 270:
						#	initialise quad2 and quad3 bands
						quad2_band = [90, 180]
						quad3_band = [new_band_start]

					elif new_band_start < 360:
						#	initialise quad2 and quad3 bands
						quad2_band = [90, 180]
						quad3_band = [180, 270]
						quad4_band = [new_band_start]

				###################################################################################
				#	Quandrent 2

				elif band_end < 180:

					##################################################
					#	close previous bands

					if len(quad1_band) == 1:
						#	adding end value
						quad1_band = [quad1_band[0], 90]

					#	quad2 band
					if len(quad2_band) == 1:
						#	adding end value
						quad2_band = [quad2_band[0], band_end]

					elif len(quad2_band) == 0:
						#	this is end of a quad2 band
						quad2_band = [90, band_end]

					##################################################
					#	initialise subsequent bands

					if new_band_start < 180:
						print('Error: Multiple bands start in quadrent 2.')
						sys.exit()
	
					elif new_band_start < 270:
						#	initialise quad3
						quad3_band = [new_band_start]
	
					elif new_band_start < 360:
						#	initialise quad3 and quad4 bands
						quad3_band = [180, 270]
						quad4_band = [new_band_start]

				###################################################################################
				#	Quandrent 3

				elif band_end < 270:

					##################################################
					#	close previous bands

					if len(quad2_band) == 1:
						#	adding end value
						quad2_band = [quad2_band[0], 180]

					elif len(quad2_band) == 0:
						#	this is end of a quad2 band
						quad2_band = [90, 180]

					if len(quad3_band) == 1:
						#	adding end value
						quad3_band = [quad3_band[0], band_end]

					elif len(quad3_band) == 0:
						#	this is end of a quad3 band
						quad3_band = [180, band_end]

					##################################################
					#	initialise subsequent bands

					if new_band_start < 270:
						print('Error: Multiple bands start in quadrent 3.')
						sys.exit()

					elif new_band_start < 360:
						#	initialise quad4 band
						quad4_band = [new_band_start]

				###################################################################################
				#	Quandrent 4
				elif band_end < 360:

					if len(quad1_band) == 1:
						#	adding end value
						quad1_band = [quad1_band[0], 90]

					elif len(quad1_band) == 0:
						#	this is end of a quad1 band
						quad1_band = [0, 90]

					if len(quad2_band) == 1:
						#	adding end value
						quad2_band = [quad2_band[0], 180]

					elif len(quad2_band) == 0:
						#	this is end of a quad2 band
						quad2_band = [90, 180]

					if len(quad3_band) == 1:
						#	adding end value
						quad3_band = [quad3_band[0], 270]

					elif len(quad3_band) == 0:
						#	this is end of a quad3 band
						quad3_band = [180, 270]

					if len(quad4_band) == 1:
						#	adding end value
						quad4_band = [quad4_band[0], band_end]

	#######################################################################################
	#	finalise bands if required
	print('\n			Finalising Bands...')

	if len(quad1_band) == 1:
		#	adding end value
		quad1_band = [quad1_band[0], 90]
		quad2_band = [90, 180]
		quad3_band = [180, 270]
		quad4_band = [270, az_vals[-1]]

	elif len(quad2_band) == 1:
		#	adding end value
		quad2_band = [quad2_band[0], 180]
		quad3_band = [180, 270]
		quad4_band = [270, az_vals[-1]]

	elif len(quad3_band) == 1:
		#	adding end value
		quad3_band = [quad3_band[0], 270]
		quad4_band = [270, az_vals[-1]]

	elif len(quad4_band) == 1:
		#	adding end value
		quad4_band = [quad4_band[0], az_vals[-1]]

	return quad1_band, quad2_band, quad3_band, quad4_band
