#############################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
#############################################################################################
#
#
#
#
#
#
#
#############################################################################################

#	standard packages
import os
import sys

#	numpy
import numpy as np

#	url requests
import requests

#	custom general functions
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, get_month, \
	get_times, interp

###################################################################################################
#	Function: 		req_horizons
#	Description: 	returns required horizons data.
#					data is interpolated to required range.
#					if source file doesnt exist, is generated
#
###################################################################################################

def req_horizons(
		start_time_object,
		stop_time_object,
		pos_path,
		eop_path,
		sim_range,
		time_step,
		logging_level
	):

	if logging_level != 0:
		print('					Acquiring Horizons Data...')

	#	horizons data directory
	horz_dir = 'horz_dir'

	#	assemble horizons paths
	horz_path = os.path.join(pos_path, horz_dir)
	horz_source_path = os.path.join(horz_path, 'source_files')

	#	target bodies (10 = sun, 399 = earth)
	target_ids = [10, 399]

	###############################################################################################
	#	parse start times
	
	prop_step = time_step / (24 * 60 * 60)

	#	utc start time
	start_jd = sim_range[0]

	#	convert start julian date to date
	year_utc, mon_utc, day_utc, hour_utc, min_utc, sec_utc = jd2date(*jday_jfrac(start_jd))
	start_utc_time_vec = [year_utc, mon_utc, day_utc, hour_utc, min_utc, sec_utc]
	
	#	extract barycentric julian days and day fractions
	jday_tdb_start = start_time_object.jday_tdb
	jfrac_tdb_start = start_time_object.jfrac_tdb

	#	calculate julian dates
	start_tdb_jdate = (jday_tdb_start + jfrac_tdb_start)
	
	#	convert start julian date to date
	year_tdb, mon_tdb, day_tdb, hour_tdb, min_tdb, sec_tdb = jd2date(
		jday_tdb_start,
		jfrac_tdb_start
	)

	#	generate tdb start time vector
	start_tdb_time_vec = [year_tdb, mon_tdb, day_tdb, hour_tdb, min_tdb, sec_tdb]

	###############################################################################################
	#	parse stop times
	
	stop_jd = sim_range[-1]

	#	convert start julian date to date
	year_utc, mon_utc, day_utc, hour_utc, min_utc, sec_utc = jd2date(*jday_jfrac(stop_jd))
	stop_utc_time_vec = [year_utc, mon_utc, day_utc, hour_utc, min_utc, sec_utc]

	#	extract barycentric julian days and day fractions
	jday_tdb_stop = stop_time_object.jday_tdb
	jfrac_tdb_stop = stop_time_object.jfrac_tdb

	#	calculate juluan dates
	stop_tdb_jdate = (jday_tdb_stop + jfrac_tdb_stop)

	#	convert stop julian date to date
	year_tdb, mon_tdb, day_tdb, hour_tdb, min_tdb, sec_tdb = jd2date(
		jday_tdb_stop,
		jfrac_tdb_stop
	)
	
	#	generate tdb stop time vector
	stop_tdb_time_vec = [year_tdb, mon_tdb, day_tdb, hour_tdb, min_tdb, sec_tdb]

	#	simulation duration
	duration_days = stop_jd - start_jd

	###############################################################################################
	#	generate horizons data file names

	#	set year of interest
	if start_utc_time_vec[0] == stop_utc_time_vec[0]:
		year_of_interest = stop_utc_time_vec[0]
	
	elif start_utc_time_vec[0] != stop_utc_time_vec[0]:
		print('		ERROR: Data spans year end.')
		sys.exit()

	#	source file names
	sun_source_file = 'sun_horz_source_' + str(year_of_interest) + '.txt'
	earth_source_file = 'earth_horz_source_' + str(year_of_interest) + '.txt'
	sun_source_file_interp = 'sun_horz_source_interp_' + str(year_of_interest) + '.txt'
	earth_source_file_interp = 'earth_horz_source_interp_' + str(year_of_interest) + '.txt'

	###############################################################################################
	#	object loop

	for target_id in target_ids:

		#	set source file names based from target id
		if target_id == 10:
			target_str = 'Sun'
			source_file = sun_source_file
			source_file_interp = sun_source_file_interp

		elif target_id == 399:
			target_str = 'Earth'
			source_file = earth_source_file
			source_file_interp = earth_source_file_interp

		###########################################################################################
		#	generate source files as required

		if os.path.isfile(os.path.join(horz_source_path, source_file)) and \
				os.path.isfile(os.path.join(horz_source_path, source_file_interp)):

			if logging_level != 0:
				print('					{}: Loading source file...'.format(target_str))
			#	load the interpolated horizons source file
			icrf_raw = np.loadtxt(
				os.path.join(horz_source_path, source_file_interp),
				delimiter=','
			)

		else:

			if logging_level != 0:
				print('					{}: Generating source file for {}...'.format(
					target_str, year_of_interest
				))

			#	generate horizons source file on year of interest
			gen_horz_source(eop_path, horz_source_path, year_of_interest, \
				source_file, source_file_interp, target_id \
			)

			#	load the interpolated horizons source file
			icrf_raw = np.loadtxt(
				os.path.join(horz_source_path, source_file_interp),
				delimiter=','
			)

		###########################################################################################
		#	extract data

		############################################
		#	1 minute time step

		if time_step == 60:
			if logging_level != 0:
				print('					{}: Extracting final exact data...'.format(target_str))
			start_index, end_index = get_exact_data(start_utc_time_vec, stop_utc_time_vec, icrf_raw)
			icrf_final = icrf_raw[start_index:end_index + 1, :]

		############################################
		#	greater than 1 minute time step

		elif time_step > 60:
			if logging_level != 0:
				print('					{}: Extracting final exact data...'.format(target_str))

			#	initialise output array
			icrf_final = np.zeros([len(sim_range), len(icrf_raw[0])])
			data_index = 0

			#	step date vector
			step_date_vec = [*jd2date(*jday_jfrac(sim_range[data_index]))]

			#	extract ICRF data
			for line in icrf_raw:

				#	generate horizons data date vector
				date_line = [
					int(line[1]),
					int(line[2]),
					int(line[3]),
					int(line[4]),
					int(line[5]),
					line[6]
				]

				#	check for match with output dates
				if step_date_vec == date_line:

					#	add to output array
					icrf_final[data_index, :] = line

					#	itterate index
					data_index += 1

					#	break condition
					if data_index >= len(icrf_final):
						break
					else:
						#	update step date vector
						step_date_vec = [*jd2date(*jday_jfrac(sim_range[data_index]))]

		############################################
		#	less than 1 minute time step
		elif time_step < 60:

			if logging_level != 0:
				print('					{}: Extracting cover data...'.format(target_str))

			duration_mins = round(duration_days * (24 * 60))
			duration_seconds = round(duration_days * (24 * 60 * 60))

			start_index, end_index = get_cover_data(start_jd, stop_jd, icrf_raw)
			npts = end_index - start_index

			############################################
			#	check for sufficent points for interpolation
			while npts <= 3:
		
				#	special case for start of data file
				if start_index == 0:
					end_index += 1
					npts = end_index - start_index
		
				elif end_index == 0:
					start_index -= 1
					npts = end_index - start_index
		
				else:
					start_index -= 1
					end_index += 1
					npts = end_index - start_index
					#	add/subtract 1 to ensure cover
					start_index -= 1
					end_index += 1

			#	extract icrf data
			icrf_out = icrf_raw[start_index:end_index, :]

			############################################
			#	interpolate date to required time step
	
			#	uninterpolated data
			sim_time = icrf_out[:, 0]
	
			#	start and stop jdates
			start_jdate_icrf = icrf_out[0, 0]
			stop_jdate_icrf = icrf_out[-1, 0]

			#	find nsteps
			num_points = int(round((stop_jdate_icrf - start_jdate_icrf) / prop_step))

			#	new time range to interpolate to
			interp_range = np.linspace(start_jdate_icrf, stop_jdate_icrf, num_points + 1)

			############################################
			#	generate exact time data for output file
			sim_time_data = np.zeros((len(interp_range), 7))
	
			for itt, jdate1 in enumerate(interp_range):
	
				#	sim time data
				year3, mon3, day3, hour3, min3, sec3 = jd2date(*jday_jfrac(jdate1))
	
				#	assemble date vector
				vec = [
					float(jdate1),
					int(year3),
					int(mon3),
					int(day3),
					int(hour3),
					int(min3),
					float(sec3)
				]
	
				#	populate output
				sim_time_data[itt, :] = vec

			############################################
			#	interpolate to required step
			if logging_level != 0:
				print('					{}: Performing Interpolation...'.format(target_str))

			#	current x spline
			spline_x = sim_time

			#	new spline x
			spline_x_new = interp_range

			#	generate data to interpolate
			interp_data_in = np.hstack((icrf_out[:, 0:1], icrf_out[:, 7:10]))

			#	initilise new y spline data
			interp_data_out = np.zeros((len(interp_range), len(interp_data_in[0])))

			#	perform interpolation	
			interp_data = interp(interp_data_in, interp_data_out, spline_x, spline_x_new)

			#	add full time data
			interp_data = np.hstack((sim_time_data, interp_data[:, 1:]))

			############################################
			#	extract the required data
			if logging_level != 0:
				print('					{}: Extracting final exact data...'.format(target_str))

			start_index, end_index = get_exact_data(start_utc_time_vec, stop_utc_time_vec, interp_data)
			icrf_final = interp_data[start_index:end_index + 1, :]

		############################################
		#	return data file

		#	set output file names based from target id
		if target_id == 10:
			#	output data
			sun_icrf_out = icrf_final

		elif target_id == 399:
			#	output_data
			earth_icrf_out = icrf_final

	return sun_icrf_out, earth_icrf_out

###################################################################################################
#	Function: 		gen_horz_source
#	Description: 	generates a horizons source file for the year of interest.
#
###################################################################################################

def gen_horz_source(eop_path,
	horz_source_path,
	year_of_interest,
	source_file,
	source_file_interp,
	target_id
	):

	print('Acquiring source file {}'.format(source_file))

	#	central body (0 = barycenter)
	center = '0'

	###############################################################################################
	#	generate UTC time data
	
	#	set start time utc vector to 1st Jan 00:00:00
	start_time_vec_utc = [year_of_interest, 1, 1, 0, 0, 0.0]

	#	convert start utc times to julian date
	jday_start_utc, jfrac_start_utc = date2jd(*start_time_vec_utc)
	jd_start_utc = jday_start_utc + jfrac_start_utc

	#	set stop time utc vector to 1st Jan 00:00:00 of next year
	stop_time_vec_utc = [year_of_interest + 1, 1, 1, 0, 0, 0.0]

	#	convert stop utc times to julian date
	jday_stop_utc, jfrac_stop_utc = date2jd(*stop_time_vec_utc)
	jd_stop_utc = jday_stop_utc + jfrac_stop_utc

	###############################################################################################
	#	generate eop data

	#	load eop files
	eop_file = 'eop_parsed.csv'
	dat_file = 'dat_parsed.csv'

	eop_file_data = np.loadtxt(os.path.join(eop_path, eop_file), delimiter=',')
	dat_file_data = np.loadtxt(os.path.join(eop_path, dat_file), delimiter=',')

	#	find relevant eop data for start of year
	eop_start_index = np.where(eop_file_data[:, 1] == year_of_interest)[0][0]
	eop_start_line = eop_file_data[eop_start_index, :]

	#	parse eop_line
	eop_mjd, eop_year, eop_mon, eop_day, \
		x_p, _, y_p, _, ut_offset, _, lod, _, \
		dX, _, dY, _ = eop_start_line

	dat_start_index = np.where(dat_file_data[:, 0] > jd_start_utc)[0]
	dat_shape = dat_start_index.shape[0]

	#	if none found, use last
	if dat_shape == 0:
		dat_start_index = [-1]


	if dat_start_index[0] == -1:
		#	find relevant dat data for start of year
		dat_start_line = dat_file_data[dat_start_index[0], :]

	else:
		#	find relevant dat data for start of year
		dat_start_line = dat_file_data[dat_start_index[0] - 1, :]

	#	extract starting DAT value
	dat_start = dat_start_line[-1]

	#	length of data in days
	len_data_days = int(jd_stop_utc - jd_start_utc)

	###############################################################################################
	#	generate TDB time data

	#	use eop data to generate start time object
	start_time = get_times(
		*start_time_vec_utc,
		ut_offset,
		dat_start
	)

	#	extract barycentric julian days and day fractions
	jday_tdb_start = start_time.jday_tdb
	jfrac_tdb_start = start_time.jfrac_tdb
	jd_start_tdb = jday_tdb_start + jfrac_tdb_start

	#	convert start julian date to date
	year_tdb, mon_tdb, day_tdb, hour_tdb, min_tdb, sec_tdb = jd2date(
		jday_tdb_start,
		jfrac_tdb_start
	)

	#	generate tdb start time vector
	start_time_vec_tdb = [year_tdb, mon_tdb, day_tdb, hour_tdb, min_tdb, sec_tdb]

	#	generate start time string for horizons query
	start_str = '{:4}-{:02}-{:02}%20{:02}:{:02}:{:1.1f}'.format(*start_time_vec_tdb)

	#	calculate stop time
	jd_stop_tdb = jd_start_tdb + len_data_days

	#	convert stop julian date to day and day fraction
	jday_stop_tdb, jfrac_stop_tdb = jday_jfrac(jd_stop_tdb)

	#	convert stop julian date to date
	year_tdb, mon_tdb, day_tdb, hour_tdb, min_tdb, sec_tdb = jd2date(
		jday_stop_tdb,
		jfrac_stop_tdb
	)

	#	generate tdb start time vector
	stop_time_vec_tdb = [year_tdb, mon_tdb, day_tdb, hour_tdb, min_tdb, sec_tdb]

	#	generate stop time string for horizons query
	stop_str = '{:4}-{:02}-{:02}%20{:02}:{:02}:{:1.1f}'.format(*stop_time_vec_tdb)

	###############################################################################################
	#	performing horizons query

	#	set the number of horizons data_points to be number of hours in a year
	horz_npts = len_data_days * 24

	#	set horizons query string
	query = [
		"ssd.jpl.nasa.gov/"
		"horizons_batch.cgi?batch=1"
		"&COMMAND='" + str(target_id) + "'",
		"&CENTER='@" + center + "'",
		"&MAKE_EPHEM='YES'",
		"&TABLE_TYPE='VECTORS'",
		"&START_TIME='" + start_str + "'",
		"&STOP_TIME='" + stop_str + "'",
		"&STEP_SIZE='" + str(horz_npts) + "'",
		"&OUT_UNITS='KM-s'",
		"&REF_PLANE='FRAME'",
		"&REF_SYSTEM='J2000'",
		"&VECT_CORR='NONE'",
		"&VEC_LABLES='YES'",
		"&VEC_DELTA_T='NO'",
		"&CSV_FORMAT='NO'",
		"&OBJ_DATA='YES'",
		"&VEC_TABLE='2'"
	]

	#	assemble string
	query_str = "".join(query)
	
	# horz url
	url = ''.join(('https://', "".join(query)))

	print('\n' + url + '\n')

	temp_data = []
	temp_flag = 0

	#	request data and parse
	with requests.get(url) as response:

		for line in response.text.split('\n'):
			if line[0:5] == '$$EOE':
				temp_flag = 0
				break
			elif temp_flag == 1:
				temp_data.append(line)
			elif line[0:5] == '$$SOE':
				temp_flag = 1

		#	parse data
		ephem_data = np.array(temp_data)

		len_data = len(ephem_data)
		data_points = int(len(ephem_data) / 3)

		# body date line
		date_line = ephem_data[np.arange(0, len_data - 2, 3)]
		# body position line
		pos_line = ephem_data[np.arange(1, len_data - 1, 3)]
		# body velocity line
		vel_line = ephem_data[np.arange(2, len_data, 3)]
		
		# combine data
		string_data = np.transpose(
			np.vstack([date_line, pos_line, vel_line])
		)

		# initilize outputs
		pos_data = np.zeros((data_points, 10))
		time_since = np.zeros((data_points, 1))

		#	parse data
		for sim_step, row in enumerate(string_data):

			#	current tdb jdate
			jdate_tdb = np.float(row[0].split(' ', maxsplit=1)[0])
			
			#	extract date and time strings
			date_str, time_str, _ = row[0].split('A.D. ')[1].split(' ', maxsplit=2)
			
			#	parse date string
			year_str, mon_str, day_str = date_str.split('-')
			
			temp_year = int(year_str)
			temp_mon = get_month(mon_str)
			temp_day = int(day_str)
			
			#	parse time string
			hour_str, min_str, sec_str = time_str.split(':')
			
			temp_hour = int(hour_str)
			temp_min = int(min_str)
			temp_sec = float(sec_str)
			
			#	assemble date vector
			date_vector = [
				jdate_tdb, temp_year, temp_mon, temp_day, temp_hour, temp_min, temp_sec
			]

			#	parse X, Y, Z positions
			temp = row[1].split('Z')
			temp2 = temp[0].split('Y')
			temp3 = temp2[0].split('=')

			#	populate output array
			pos_data[sim_step, 0:7] = date_vector
			pos_data[sim_step, 7] = np.float(temp3[1])
			pos_data[sim_step, 8] = np.float(temp2[1].split('=')[1])
			pos_data[sim_step, 9] = np.float(temp[1].split('=')[1])

		print('Data acquired.')

		###########################################################################################
		#	save intermediary file

		#	set output file and path
		horz_output_path = os.path.join(horz_source_path, source_file)

		with open(horz_output_path, 'w') as output_file:
			for line in pos_data:

				#	format time data
				time_str = '{:.10f}, {:d}, {:d}, {:d}, {:d}, {:d}, {:.1f}'.format(
					line[0],
					int(line[1]),
					int(line[2]),
					int(line[3]),
					int(line[4]),
					int(line[5]),
					line[6]
				)

				#	format position data
				data_str = '{:.10f}, {:.10f}, {:.10f}'.format(
					line[7],
					line[8],
					line[9],
				)

				#	assemble output string and write to file
				output_str = time_str + ', ' + data_str
				output_file.write(output_str + '\n')

		print('\nHorizons source file {} has been generated.'.format(source_file))

		###########################################################################################
		#	interpolate data to minute time step
		print('\nInterpolating to minute time step. . .')

		#	generate interpolated time range

		#	leap year
		if len_data_days == 366:
			num_points = (len_data_days * 24 * 60) + 1

		#	normal year
		elif len_data_days == 365:
			num_points = (len_data_days * 24 * 60) + 1

		#	interpolated time range
		sim_range_interp_utc = np.linspace(
			jd_start_utc,
			jd_stop_utc,
			num_points
		)

		#	time step in minutes

		#	interpolated time range
		sim_range_interp_tdb = np.linspace(
			jd_start_tdb,
			jd_stop_tdb,
			num_points
		)

		#	generate exact time data for output file
		utc_time_data = np.zeros((len(sim_range_interp_utc), 7))

		print('\nGenerating interpolated time data. . .')
		for itt, jdate1 in enumerate(sim_range_interp_utc):

			#	convert minutes to days
			jd = jdate1

			#	sim time data
			year3, mon3, day3, hour3, min3, sec3 = jd2date(*jday_jfrac(jd))

			#	correct rounding issues
			if round(sec3) == 60:
				min3 += 1
				sec3 = 0.0

				if min3 == 60:
					hour3 += 1
					min3 = 0

					if hour3 == 24:
						day3 += 1
						hour3 = 0
			
			elif round(sec3) == 0:
				sec3 = 0.0

			#	assemble date vector
			vec = [
				float(jdate1),
				int(year3),
				int(mon3),
				int(day3),
				int(hour3),
				int(min3),
				float(sec3)
			]

			#	populate output
			utc_time_data[itt, :] = vec

		print('Interpolating. . .')

		#	uninterpolated tdb time data
		sim_time = pos_data[:, 0]

		#	current x spline
		spline_x = sim_time

		#	new spline x
		spline_x_new = sim_range_interp_tdb

		#	generate data to interpolate
		interp_data_in = np.hstack((pos_data[:, 0:1], pos_data[:, 7:10]))

		#	initilise new y spline data
		interp_data_out = np.zeros((len(sim_range_interp_tdb), len(pos_data[0, :])))

		#	perform interpolation	
		pos_interp = interp(interp_data_in, interp_data_out, spline_x, spline_x_new)

		#	replace tdb time column with utc time column
		source_data = np.hstack((utc_time_data, interp_data_out[:, 1:]))

		#	set output file and path
		horz_output_path = os.path.join(horz_source_path, source_file_interp)

		#	save horizons source file
		with open(horz_output_path, 'w') as output_file:
			for line in source_data:

				#	genrate formatted time string
				time_str = '{:.10f}, {:d}, {:d}, {:d}, {:d}, {:d}, {:.1f}'.format(
					line[0],
					int(line[1]),
					int(line[2]),
					int(line[3]),
					int(line[4]),
					int(line[5]),
					line[6]
				)

				#	generate formated data string
				data_str = '{:.10f}, {:.10f}, {:.10f}'.format(
					line[7],
					line[8],
					line[9],
				)

				#	assemble output string and write to file
				output_str = time_str + ', ' + data_str
				output_file.write(output_str + '\n')

		print('\nHorizons source file {} has been generated.'.format(source_file_interp))

###################################################################################################
#	Function: 		get_cover data
#	Description: 	extracts data for interpolation, covering required range
#
###################################################################################################

def get_cover_data(start_jd, end_jd, data_in):

	data_reccord = False

	#	search through interpolated data and extract data of interest
	for index, line in enumerate(data_in):

		#	parse current date vec
		date_vec = list(line[1:7])
		jd = line[0]

		if jd >= start_jd and data_reccord is False:
			data_reccord = True

			if index == 0:
				start_index = index
			else:
				start_index = index - 1

		if  jd >= end_jd:
			end_index = index + 1
			break

	return start_index, end_index

###################################################################################################
#	Function: 		get_cover data
#	Description: 	extracts data for interpolation, exactly between two values
#
###################################################################################################

def get_exact_data(start_vec, end_vec, data_in):

	#	search through interpolated data and extract data of interest
	for index, line in enumerate(data_in):

		date_vec = [
			int(line[1]),
			int(line[2]),
			int(line[3]),
			int(line[4]),
			int(line[5]),
			float(line[6])
		]

		if date_vec == start_vec:
			start_index = index

		if  date_vec == end_vec:
			end_index = index
			break

	return start_index, end_index
