# Python SOFA Implementation
#
# Requires compiled sofa object.
# SOFA library is compiled using:	f2py -c *.for -m sofa
#
# This code accesess the wrapper around the 04/01/2018 SOFA routines for the
# IAU 2000/2006 resolutions.
#
# Relevant resolutions:
#
# 2000
#
# IAU 2000 B1.3:
# The systems of space-time coordinates for the solar system and
# the Earth, within the framework of General Relativity, are now named the
# Barycentric Celestial Reference System (BCRS) and the Geocentric Celestial
# Reference System (GCRS) respectively. 
#
# IAU 2000 Resolution B1.6:
# The IAU 1976 precession model and the IAU 1980 theory
# of nutation be replaced by the precession-nutation model IAU2000A, for those
# who need a model at the 0.2 mas level, or its shorter version IAU 2000B for 
# those who need a model only at the 1 mas level, together with their associated
# celestial pole offsets. In addition to that model are frame bias values between
# the mean pole and equinox at J2000.0 and the GCRS.
#
# IAU 2000 B1.7:
# The Celestial Intermediate Pole (CIP) be implemented in place 
# of the Celestial Ephemeris Pole (CEP).
#
# IAU 2000 B1.8:
# The transformation between the ITRS and GCRS be specified by
# the position of the CIP in the GCRS, the position of the CIP in the ITRS,
# and the Earth Rotation Angle. 
#
# 2006
#
# IAU 2006 Resolution B1:
# The precession component of the IAU 2000A precession-nutation model be replaced
# by the P03 precession theory, refered to as  “IAU 2006 precession”.
#
# IAU 2006 Resolution B2:
# Naming conventions; harmonizing “intermediate” in the names of the pole and the
# origin (i.e. celestial and terrestrial intermediate origins, CIO and TIO instead
# of CEO and TEO, respectively) and defining the celestial and terrestrial
# “intermediate” systems; and fixing the default orientation of the BCRS and GCRS,
# which are, unless otherwise stated, assumed to be oriented according to the ICRS
# axes.
#
#
#
# The transformation to be used to relate the International Terrestrial Reference
# System (ITRS) to the Geocentric Celestial Reference System (GCRS) at the date
# (t) of the observation can be written as:
#	[GCRS] = Q(t)R(t)W(t) [ITRS]
# where: Q(t), R(t) and W(t)
# are the transformation matrices arising from the motion of the celestial pole
# in the celestial reference system, from the rotation of the Earth around the 
# axis associated with the pole,  and from polar motion respectively.

####################################################################################################

#	standard packages
import sys

#	numpy
import numpy as np

#	custom sofa module import
import sofa

#	import progressbar
import progressbar

#	import custom functions
from tools.functions.functions_list import jday_jfrac

####################################################################################################

##	IMPORTANT NOTE: THE ARRAYS ARE PRE-TRANSPOSED SO THAT THEY CONVERT GCRF -> ITRF

# This function implements the IAU 2000/2006 routines

def gc2it(jdaytt, jfractt, jdayut1, jfracut1, dX, dY, x_p, y_p):
	
	# Calculate Polar Motion matrix W(t)
	# Inputs:
	#			- x_p = polar coordinate
	#			- y_p = polar coordinate
	#			- s'  = TIO locator

	# calculate s_prime
	sp00 = sofa.iau_sp00(jdaytt, jfractt)

	# assemble polar motion matrix
	pm = sofa.iau_pom00(x_p, y_p, sp00)

	# Calculate Intermediate Transformation matrix Q(t)
	# Inputs:
	#			- d_X = correction
	#			- d_Y = correction
	#			- s'  = CIO locator
	
	# calculate X and Y according to the polynomial series of Capitaine and Wallace, 2006.
	X, Y = sofa.iau_xy06(jdaytt, jfractt)
	
	# calculate CIO locator using 2006 compatible theory
	s_06 = sofa.iau_s06(jdaytt, jfractt, X, Y)

	# apply corrections
	X = X + dX
	Y = Y + dY
	
	# assemble celestial to intermediate matrix
	pn = sofa.iau_c2ixys(X, Y, s_06)
	
	# Calculate Sidereal Rotation R(t) and Final C2T Matrix
	# Inputs:
	#			- pm  = polar motion matrix
	#			- pn  = celestial to intermediate matrix
	#			- era = Earth rotation angle

	# Earth rotation is calculated using 2000 resolutions
	era00 = sofa.iau_era00(jdayut1, jfracut1)
	
	# create final celestial to terrestrial matrix
	c2t = sofa.iau_c2tcio(pn, era00, pm)

	#	generate st matrix
	temp = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]])

	st = sofa.iau_rz(era00, temp)


	return pm, pn, st, c2t

###################################################################################################
#	Function: 		rot_mat_gen
#	Description: 	generate required transformation matrices using eop data
#
###################################################################################################

def rot_mat_gen(jdtt_start, jdut1_start, eop_data, time_step, rot_mat_log_level):

	pm_list = []
	pn_list = []
	st_list = []
	c2t_list = []

	if rot_mat_log_level == 0:
		loop_str = eop_data

	else:
		loop_str = progressbar.progressbar(eop_data)

	#	matrix generation loop
	for step, eop_line in enumerate(loop_str):
	
		#	parse eops
		eop_mjd, eop_year, eop_mon, eop_day,\
			x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
		#	generate time data
		time_since = step * time_step

		#	time step in days
		prop_step = time_step / (24 * 60 * 60)
		
		jdtt = jdtt_start + (prop_step * step)
		jdaytt_epoch, jfractt_epoch = jday_jfrac(jdtt)

		jdut1 = jdut1_start + (prop_step * step)
		jdayut1_epoch, jfracut1_epoch = jday_jfrac(jdut1)
		
		#	generate GCRF to ITRF transformation matrix
		pm, pn, st, c2t = gc2it(
			jdaytt_epoch, jfractt_epoch,
			jdayut1_epoch, jfracut1_epoch,
			dX, dY,
			x_p, y_p
		)

		#	add to list
		pm_list.append(pm)
		pn_list.append(pn)
		st_list.append(st)
		c2t_list.append(c2t)
	
	#	convert to 2d numpy array
	pm_array = np.zeros((len(pm_list), 9))
	pn_array = np.zeros((len(pn_list), 9))
	st_array = np.zeros((len(st_list), 9))
	c2t_array = np.zeros((len(c2t_list), 9))

	for i, mat in enumerate(pm_list):
		pm_array[i, :] = *mat[0], *mat[1][:], *mat[2][:]

	for i, mat in enumerate(pn_list):
		pn_array[i, :] = *mat[0], *mat[1][:], *mat[2][:]

	for i, mat in enumerate(st_list):
		st_array[i, :] = *mat[0], *mat[1][:], *mat[2][:]

	for i, mat in enumerate(c2t_list):
		c2t_array[i, :] = *mat[0], *mat[1][:], *mat[2][:]

	return pm_array, pn_array, st_array, c2t_array


