###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	
#	This is the satellite position generation module
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import datetime

#	progress bar
import progressbar

#	numpy
import numpy as np

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	custom general functions
from tools.functions.functions_list import jd2date, date2jd, jday_jfrac, gen_sgp4, gen_trans_files

#	custom transformation functions
from tools.functions.functions_transform import teme2itrf, itrf2gcrf, gcrf2itrf, kep2rv
 
#	import GLIMMA transformation function
from tools.functions.glim_transform import glim_trans

#	import horizons interface
from tools.functions.horizons_interface import req_horizons

#	import spacetrack interface
from tools.functions.spacetrack_interface import req_tle

###################################################################################################
#	Module: 		horizons_pos
#	Description: 	generate Sun and Earth ICRF and GCRF data
#
###################################################################################################

def module_horizons_pos(
		path_data, logging_level,
		start_time_object, stop_time_object,
		len_record, time_step,
		eop_path
	):

	if logging_level != 0:
		print('\n[{}]	Acquiring Horizons ICRF data...'.format(datetime.datetime.now()))

	###############################################################################################
	#	parse time data

	#	utc start
	jdayutc_start = start_time_object.jday_utc
	jfracutc_start = start_time_object.jfrac_utc
	jdutc_start = jdayutc_start + jfracutc_start

	#	utc stop
	jdayutc_stop = stop_time_object.jday_utc
	jfracutc_stop = stop_time_object.jfrac_utc
	jdutc_stop = jdayutc_stop + jfracutc_stop

	#	tt start
	jdaytt_start = start_time_object.jday_tt
	jfractt_start = start_time_object.jfrac_tt
	jdtt_start = jdaytt_start + jfractt_start
	
	#	ut1 start
	jdayut1_start = start_time_object.jday_ut1
	jfracut1_start = start_time_object.jfrac_ut1
	jdut1_start = jdayut1_start + jfracut1_start

	#	generate start and stop datetimes
	start_date = jd2date(jdayutc_start, jfracutc_start)
	stop_date = jd2date(jdayutc_stop, jfracutc_stop)

	#	generate sun/earth unique data files
	start_date_str = '{:4d}-{:02d}-{:02d}'.format(start_date[0], start_date[1], start_date[2])
	start_time_str = '{:02d}:{:02d}:{:04.1f}'.format(start_date[3], start_date[4], start_date[5])
	stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])
	stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])

	#	time range in days
	sim_range = np.linspace(jdutc_start, jdutc_stop, len_record)

	###############################################################################################
	#	define file paths

	#	positional data directory
	pos_dir = 'pos_dir'

	###############################################################################################
	#	generate file names

	#	sun
	sun_icrf_file = ''.join([
		'sun_icrf_',
		'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
		'.txt'
	])

	sun_gcrf_file = ''.join([
		'sun_gcrf_',
		'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
		'.txt'
	])

	#	earth
	earth_icrf_file = ''.join([
		'earth_icrf_',
		'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
		'.txt'
	])

	earth_gcrf_file = ''.join([
		'earth_gcrf_',
		'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
		'.txt'
	])

	###############################################################################################
	#	generate output paths

	#	assemble path
	pos_path = os.path.join(path_data, pos_dir)

	###############################################################################################
	#	search for existing data files

	if os.path.isfile(os.path.join(pos_path, sun_icrf_file)):
		if logging_level != 0:
			print('					Reading Horizons ICRF and GCRF data files...')
	
		#	load sun icrf data
		sun_icrf = np.loadtxt(
			os.path.join(pos_path, sun_icrf_file),
			delimiter=','
		)

		#	load sun gcrf data
		sun_gcrf = np.loadtxt(
			os.path.join(pos_path, sun_gcrf_file),
			delimiter=','
		)
		
		#	load earth itrf data
		earth_icrf = np.loadtxt(
			os.path.join(pos_path, earth_icrf_file),
			delimiter=','
		)

		#	load earth gcrf data
		earth_gcrf = np.loadtxt(
			os.path.join(pos_path, earth_gcrf_file),
			delimiter=','
		)
		
	else:

		###########################################################################################
		#	generate horizons data file
	
		if logging_level != 0:
			print('					Generating Horizons ICRF data files...')

		sun_icrf, earth_icrf = req_horizons(
			start_time_object,
			stop_time_object,
			pos_path,
			eop_path,
			sim_range,
			time_step,
			logging_level
		)


		###########################################################################################
		#	convert icrf frames to gcrf

		if logging_level != 0:
			print('					Converting to GCRF frame...')
		
		#	initialise outputs
		sun_gcrf = np.zeros(sun_icrf.shape)
		earth_gcrf = np.zeros(earth_icrf.shape)

		for i, (sun_line, earth_line) in enumerate(zip(sun_icrf, earth_icrf)):

			#	parse time line
			time_line = sun_line[0:7]

			#	parse icrf data
			sun_icrf_line = sun_line[7:10]
			earth_icrf_line = earth_line[7:10]

			#	convert to gcrf
			sun_gcrf_line = sun_icrf_line - earth_icrf_line
			earth_gcrf_line = earth_icrf_line - earth_icrf_line

			#	add to outputs
			sun_gcrf[i, 0:7] = time_line
			sun_gcrf[i, 7:10] = sun_gcrf_line
			earth_gcrf[i, 0:7] = time_line
			earth_gcrf[i, 7:10] = earth_gcrf_line

		#	output file names
		output_files = [sun_icrf_file, sun_gcrf_file, earth_icrf_file, earth_gcrf_file]
	
		#	output data arrays
		output_data = [sun_icrf, sun_gcrf, earth_icrf, earth_gcrf]

		#	save files
		for (out_file, out_data) in zip(output_files, output_data):
			horz_output_path = os.path.join(pos_path, out_file)
		
			with open(horz_output_path, 'w') as output_file:
				for line in out_data:
					#	format time data
					time_str = '{:.10f}, {:d}, {:d}, {:d}, {:d}, {:d}, {:.1f}'.format(
						line[0],
						int(line[1]),
						int(line[2]),
						int(line[3]),
						int(line[4]),
						int(line[5]),
						line[6]
					)
		
					#	format position data
					data_str = '{:.10f}, {:.10f}, {:.10f}'.format(
						line[7],
						line[8],
						line[9],
					)
		
					#	assemble output string and write to file
					output_str = time_str + ', ' + data_str
					output_file.write(output_str + '\n')

	return sun_icrf, sun_gcrf, earth_icrf, earth_gcrf

###################################################################################################
#	Module: 		sat_pos
#	Description: 	generate TEME, ITRF and GCRF satellite positions from TLE data
#
###################################################################################################

def module_sat_pos(
		path_data, logging_level, len_record, time_step,
		start_time_object, stop_time_object,
		norad_id,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array
	):

	if logging_level != 0:
		print('\n[{}]	Acquiring Satellite SGP4 Data...'.format(datetime.datetime.now()))

	###############################################################################################
	#	parse time data

	#	utc start
	jdayutc_start = start_time_object.jday_utc
	jfracutc_start = start_time_object.jfrac_utc
	jdutc_start = jdayutc_start + jfracutc_start

	#	utc stop
	jdayutc_stop = stop_time_object.jday_utc
	jfracutc_stop = stop_time_object.jfrac_utc
	jdutc_stop = jdayutc_stop + jfracutc_stop

	#	tt start
	jdaytt_start = start_time_object.jday_tt
	jfractt_start = start_time_object.jfrac_tt
	jdtt_start = jdaytt_start + jfractt_start
	
	#	ut1 start
	jdayut1_start = start_time_object.jday_ut1
	jfracut1_start = start_time_object.jfrac_ut1
	jdut1_start = jdayut1_start + jfracut1_start

	#	generate start and stop datetimes
	start_date = jd2date(jdayutc_start, jfracutc_start)
	stop_date = jd2date(jdayutc_stop, jfracutc_stop)

	#	generate sun/earth unique data files
	start_date_str = '{:4d}-{:02d}-{:02d}'.format(start_date[0], start_date[1], start_date[2])
	start_time_str = '{:02d}:{:02d}:{:04.1f}'.format(start_date[3], start_date[4], start_date[5])
	stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])
	stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])

	#	time range in days
	sim_range = np.linspace(jdutc_start, jdutc_stop, len_record)

	###############################################################################################
	#	define file paths

	#	input dir
	input_dir = 'input_dir'
	
	#	tle subdirectory
	tle_dir = 'tle_files'

	#	pos dir
	pos_dir = 'pos_dir'

	###############################################################################################
	#	generate file names
	
	#	generate unique TLE data file

	#	TLE request dates, add day either side to increase tle find chance
	tle_start_date = jd2date(*jday_jfrac(jdayutc_start + jfracutc_start - 1))
	tle_stop_date = jd2date(*jday_jfrac(jdayutc_stop + jfracutc_stop + 1)) 
	
	#	generate TLE file name
	tle_start_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_start_date[0:3])
	tle_stop_date_str = '{:4d}-{:02d}-{:02d}'.format(*tle_stop_date[0:3])
	tle_file = '{}_{}_{}.tle'.format(norad_id, tle_start_date_str, tle_stop_date_str)

	#	generate unique sgp4 TEME data file
	sat_teme_file = ''.join([
		'sat_teme' + '_',
		str(norad_id) + '_',
		'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
		'.txt'
	])

	#	generate unique sgp4 TEME data file
	sat_itrf_file = ''.join([
		'sat_itrf' + '_',
		str(norad_id) + '_',
		'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
		'.txt'
	])
	
	sat_gcrf_file = ''.join([
		'sat_gcrf' + '_',
		str(norad_id) + '_',
		'_'.join([start_date_str, start_time_str, str(len_record), str(time_step)]),
		'.txt'
	])

	###############################################################################################
	#	generate output paths

	#	tle path
	tle_path = os.path.join(path_data, input_dir, tle_dir, tle_file)

	#	output paths
	teme_path = os.path.join(path_data, pos_dir, sat_teme_file)
	itrf_path = os.path.join(path_data, pos_dir, sat_itrf_file)
	gcrf_path = os.path.join(path_data, pos_dir, sat_gcrf_file)
	
	###############################################################################################
	#	acquiring TLE
	
	#	check for TLE file
	if os.path.isfile(tle_path):
		if logging_level != 0:
			print('						Reading TLE file...')
	
		with open(tle_path, 'r') as open_file:
			tle_data = open_file.readlines()
	
	else:
		if logging_level != 0:
			print('						Requesting TLE file...')

		time_sep = jdutc_stop - jdutc_start

		if time_sep < 2:
			tle_start_date = jd2date(*jday_jfrac(jdutc_start - 1))
			tle_stop_date = jd2date(*jday_jfrac(jdutc_stop + 1))
		else:
			tle_start_date = jdutc_start
			tle_stop_date = jdutc_stop

		#	initialise request success flag
		request_success = False

		while request_success is False:

			#	request TLE file
			tle_data = req_tle(tle_start_date, tle_stop_date, norad_id)
	
			#	check for empty result
			if tle_data[0] == 'NO RESULTS RETURNED':
				print('\n	{}: No resuts returned from Spacetrack'.format(norad_id))
				print('\n	Broadening Search Range...')

				#	generate new dates
				tle_start_date = jd2date(*jday_jfrac(np.add(*date2jd(*tle_start_date)) - 1))
				tle_stop_date = jd2date(*jday_jfrac(np.add(*date2jd(*tle_stop_date)) + 1))
	
			else:

				#	set successful request
				request_success = True

				#	save file
				np.savetxt(tle_path, tle_data, fmt='%s', delimiter=',')

	#	extract first tle set
	tle_line1 = tle_data[1]
	tle_line2 = tle_data[2]
	
	#	mean mot rad/sec
	orbital_period = np.round(86400 / np.float(tle_line2[52:63]), 1)
	orbital_mot = (2 * np.pi) / orbital_period

	###############################################################################################
	#	generate TEME data
	
	#	check for TEME data
	if os.path.isfile(teme_path):
		if logging_level != 0:
			print('					Reading TEME data file...')
	
		#	load sgp4 data
		sat_teme = np.loadtxt(os.path.join(teme_path), delimiter=',')

	else:
		if logging_level != 0:
			print('					Generating TEME data file...')
			print('						Performing SGP4 Propagation:\n')

		#	generate SGP4 satellite object
		satellite = twoline2rv(tle_line1, tle_line2, wgs72)
	
		#	initilise position data matrix
		sat_teme = gen_sgp4(sim_range, time_step, satellite)
		
		#	save sgp4 TEME data file
		np.savetxt(teme_path, sat_teme, delimiter=',')

	###############################################################################################
	#	generate ITRF and GCRF data

	if os.path.isfile(gcrf_path) and os.path.isfile(itrf_path):
		if logging_level != 0:
			print('\n					Reading Satellite GCRF and ITRF data files...')
	
		#	load gcrf data
		sat_gcrf = np.loadtxt(os.path.join(gcrf_path), delimiter=',')

		#	load itrf data
		sat_itrf = np.loadtxt(os.path.join(itrf_path), delimiter=',')

	else:
		if logging_level != 0:
			print('\n					Generating Satellite GCRF and ITRF data files...')

		###########################################################################################
		#	perform transformation

		if logging_level != 0:
			print('\n						Transforming to GCRF frame:\n')

		#	initialise output
		sat_gcrf = np.zeros(sat_teme.shape)
		sat_itrf = np.zeros(sat_teme.shape)
		
		#	generate loop data
		loop_data = zip(sat_teme, eop_data, dat_data, pn_array, pm_array, st_array, c2t_array)

		#	convert TEME to GCRF
		#	set enumeration value
		if logging_level != 0:
			enumeration_value = progressbar.progressbar(loop_data, max_value=len(sat_teme))
		elif logging_level is 0:
			enumeration_value = loop_data

		for step, loop_line in enumerate(enumeration_value):

			#	parse loop data
			sat_teme_line, eop_line, dat_line, pn_line, pm_line, st_line, c2t_line = loop_line

			#######################################################################################
			#	parse eop parameters
			eop_mjd, eop_year, eop_mon, eop_day, \
			x_p, _, y_p, _, \
			ut_offset, _, \
			lod, _, \
			dX, _, dY, _ = eop_line
		
			dat = dat_line[1]
			
			#######################################################################################
			#	parse time data

			#	generate time data and convert to days
			time_since_sec = step * time_step
			time_since_days = time_since_sec / 86400
			
			jdtt = jdtt_start + time_since_days
			jdaytt_epoch, jfractt_epoch = jday_jfrac(jdtt)
		
			jdut1 = jdut1_start + time_since_days
			jdayut1_epoch, jfracut1_epoch = jday_jfrac(jdut1)
			
			#######################################################################################
			#	parse object data
		
			#	object TEME position / velocity
			jdate = sat_teme_line[0]
			sat_teme_pos = sat_teme_line[1:4]
			sat_teme_vel = sat_teme_line[4:7]
		
			#######################################################################################
			#	parse rotation matrices

			#	convert c2t array
			pn = np.array([
				[pn_line[0], pn_line[1], pn_line[2]],
				[pn_line[3], pn_line[4], pn_line[5]],
				[pn_line[6], pn_line[7], pn_line[8]]
			])
		
			#	inverse c2t array
			inv_pn = np.transpose(pn)
		
			#	convert c2t array
			pm = np.array([
				[pm_line[0], pm_line[1], pm_line[2]],
				[pm_line[3], pm_line[4], pm_line[5]],
				[pm_line[6], pm_line[7], pm_line[8]]
			])
		
			#	inverse c2t array
			inv_pm = np.transpose(pm)
		
			#	convert c2t array
			st = np.array([
				[st_line[0], st_line[1], st_line[2]],
				[st_line[3], st_line[4], st_line[5]],
				[st_line[6], st_line[7], st_line[8]]
			])
		
			#	inverse c2t array
			inv_st = np.transpose(st)
		
			#	convert c2t array
			c2t = np.array([
				[c2t_line[0], c2t_line[1], c2t_line[2]],
				[c2t_line[3], c2t_line[4], c2t_line[5]],
				[c2t_line[6], c2t_line[7], c2t_line[8]]
			])
		
			#	inverse c2t array
			inv_c2t = np.transpose(c2t)
	
			#######################################################################################
			#	convert to ITRF position / velocity
			sat_itrf_pos, sat_itrf_vel = teme2itrf(
				sat_teme_pos,
				sat_teme_vel,
				jdayut1_epoch,
				jfracut1_epoch,
				lod,
				x_p,
				y_p
			)
			#######################################################################################
			#	convert to GCRF position / velocity
			sat_gcrf_pos, sat_gcrf_vel = itrf2gcrf(
				sat_itrf_pos,
				sat_itrf_vel,
				inv_pm,
				inv_st,
				inv_pn,
				lod
			)

			#######################################################################################
			#	add to output arrays
			sat_gcrf[step, :] = [jdate, *sat_gcrf_pos, *sat_gcrf_vel]
			sat_itrf[step, :] = [jdate, *sat_itrf_pos, *sat_itrf_vel]

		###########################################################################################
		#	save data files
		np.savetxt(itrf_path, sat_itrf, delimiter=',')
		np.savetxt(gcrf_path, sat_gcrf, delimiter=',')

	###############################################################################################
	#	return output arrys
	return sat_gcrf, sat_itrf, orbital_mot

###################################################################################################
#	Module: 		custom orbit
#	Description: 	generate a custom orbit
#					Inputs:	Angles in Radians!
#
###################################################################################################

def module_custom_orbit(
		path_data, logging_level, len_record, time_step,
		start_time_object, stop_time_object,
		object_id,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array,
		raan, argp, true_anom, i, e, a
	):

	if logging_level != 0:
		print('\n[{}]	Acquiring Custom Satellite Position Data...'.format(datetime.datetime.now()))

	###############################################################################################
	#	parse time data

	#	utc start
	jdayutc_start = start_time_object.jday_utc
	jfracutc_start = start_time_object.jfrac_utc
	jdutc_start = jdayutc_start + jfracutc_start

	#	utc stop
	jdayutc_stop = stop_time_object.jday_utc
	jfracutc_stop = stop_time_object.jfrac_utc
	jdutc_stop = jdayutc_stop + jfracutc_stop

	#	tt start
	jdaytt_start = start_time_object.jday_tt
	jfractt_start = start_time_object.jfrac_tt
	jdtt_start = jdaytt_start + jfractt_start
	
	#	ut1 start
	jdayut1_start = start_time_object.jday_ut1
	jfracut1_start = start_time_object.jfrac_ut1
	jdut1_start = jdayut1_start + jfracut1_start

	#	generate start and stop datetimes
	start_date = jd2date(jdayutc_start, jfracutc_start)
	stop_date = jd2date(jdayutc_stop, jfracutc_stop)

	#	time range in days
	sim_range = np.linspace(jdutc_start, jdutc_stop, len_record)

	#	generate sun/earth unique data files
	start_date_str = '{:4d}-{:02d}-{:02d}'.format(start_date[0], start_date[1], start_date[2])
	start_time_str = '{:02d}:{:02d}:{:04.1f}'.format(start_date[3], start_date[4], start_date[5])
	stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])
	stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])

	###############################################################################################
	#	calculate orbit properties
	
	#	define earth-sun mu in km3/s2
	mu_earth = 3.986004418e14 * 1e-9
	
	#	calculate p
	p = a * (1 - e**2)
	
	#	calculate T
	T = np.pi * 2 * np.sqrt(a**3 / mu_earth)

	#	calculate n
	n = (2 * np.pi) / T
	
	#	convert to revolutions per day
	rev_day = (n / (2 * np.pi)) * 86400
	
	###############################################################################################
	#	calculate perturbations in the GCRF frame

	#	third body perturbation to RAAN [degrees / day]
	thirdb_raan_dot_moon = -0.00338 * np.cos(i) / rev_day
	thirdb_raan_dot_sun = -0.00154 * np.cos(i) / rev_day
	
	#	convert to radians per second
	thirdb_raan_dot_moon = np.radians(thirdb_raan_dot_moon) / 86400
	thirdb_raan_dot_sun = np.radians(thirdb_raan_dot_sun) / 86400
	
	#	third body perturbation of ARGP [degrees / day]
	thirdb_argp_dot_moon = 0.00169 * (4 - (5 * np.sin(i)**2)) / rev_day
	thirdb_argp_dot_sun = 0.00077 * (4 - (5 * np.sin(i)**2)) / rev_day
	
	#	convert to radians per second
	thirdb_argp_dot_moon = np.radians(thirdb_argp_dot_moon) / 86400
	thirdb_argp_dot_sun = np.radians(thirdb_argp_dot_sun) / 86400
	
	#	non-sphericity J2 perturbation
	j2_raan_dot = -2.06474e14 * a**-(7/2) * np.cos(i) * (1 - e**2)**-2
	j2_argp_dot = 1.03237e14 * a**-(7/2) * (4 - (5 * np.sin(i)**2)) * (1 - e**2)**-2
	
	#	convert to radians per second
	j2_raan_dot = np.radians(j2_raan_dot) / 86400
	j2_argp_dot = np.radians(j2_argp_dot) / 86400
	
	###############################################################################################
	#	generate ideal orbit data
	
	#	define jdate time range
	jd_time_data = sim_range
	
	#	define raan rate of change [rad / s]
	raan_dot = thirdb_raan_dot_moon + thirdb_raan_dot_sun + j2_raan_dot
	
	#	define argp rate of change [rad / s]
	argp_dot = thirdb_argp_dot_moon + thirdb_argp_dot_sun + j2_argp_dot

	#	initialise output
	sat_gcrf = np.zeros([len(sim_range), 7])
	sat_itrf = np.zeros([len(sim_range), 7])
	orbital_elements_gcrf = np.zeros([len(jd_time_data), 7])

	if logging_level != 0:
		print('\n					Generating Position Data:\n')

	#	generate loop data
	loop_data = zip(jd_time_data, eop_data, dat_data, pn_array, pm_array, st_array)

	#	set enumeration value
	if logging_level == 0:
		enumeration_value = loop_data
	else:
		enumeration_value = progressbar.progressbar(loop_data, max_value=len(eop_data))

	for itt, loop_line in enumerate(enumeration_value):

		#######################################################################################
		#	parse loop line data

		jdate, eop_line, dat_line, pn_line, pm_line, st_line = loop_line

		#######################################################################################
		#	parse eop parameters
	
		eop_mjd, eop_year, eop_mon, eop_day, \
		x_p, _, y_p, _, \
		ut_offset, _, \
		lod, _, \
		dX, _, dY, _ = eop_line
		
		dat = dat_line[1]
	
		###########################################################################################
		#	parse time data
	
		#	generate time data and convert to days
		time_since_sec = itt * time_step
		time_since_days = time_since_sec / 86400

		jdtt = jdtt_start + time_since_days
		jdaytt_epoch, jfractt_epoch = jday_jfrac(jdtt)
		
		jdut1 = jdut1_start + time_since_days
		jdayut1_epoch, jfracut1_epoch = jday_jfrac(jdut1)
		
		###########################################################################################
		#	parse rotation matrices
	
		#	convert c2t array
		pn = np.array([
			[pn_line[0], pn_line[1], pn_line[2]],
			[pn_line[3], pn_line[4], pn_line[5]],
			[pn_line[6], pn_line[7], pn_line[8]]
		])
		
		#	inverse c2t array
		inv_pn = np.transpose(pn)
		
		#	convert c2t array
		pm = np.array([
			[pm_line[0], pm_line[1], pm_line[2]],
			[pm_line[3], pm_line[4], pm_line[5]],
			[pm_line[6], pm_line[7], pm_line[8]]
		])
		
		#	inverse c2t array
		inv_pm = np.transpose(pm)
		
		#	convert c2t array
		st = np.array([
			[st_line[0], st_line[1], st_line[2]],
			[st_line[3], st_line[4], st_line[5]],
			[st_line[6], st_line[7], st_line[8]]
		])
		
		#	inverse c2t array
		inv_st = np.transpose(st)
		
		#######################################################################################
		#	evolve GCRF kelperian elements
	
		#	raan
		raan_step = raan + (raan_dot * time_since_sec)

		if abs(raan_step) > (2 * np.pi):
			raan_step = raan_step % (2 * np.pi)

		if raan_step < 0.0:
			raan_step += (2 * np.pi)
		
		#	argp
		argp_step = argp + (argp_dot * time_since_sec)
	
		if abs(argp_step) > (2 * np.pi):
			argp_step = argp_step % (2 * np.pi)

		if argp_step < 0.0:
			argp_step += (2 * np.pi)

		#	true anom
		true_anom_step = true_anom + (n * time_since_sec)
	
		if abs(true_anom_step) > (2 * np.pi):
			true_anom_step = true_anom_step % (2 * np.pi)

		if true_anom_step < 0.0:
			true_anom_step += (2 * np.pi)

		#	add orbital elements to output array
		orbital_elements_gcrf[itt, :] = [jdate, raan_step, argp_step, true_anom_step, i, e, a]

		#######################################################################################
		#	generate state vector
		sat_gcrf_pos, sat_gcrf_vel = kep2rv(raan_step, argp_step, true_anom_step, a, i, e)

		#	add position and velocity to output array
		sat_gcrf[itt, 0] = jdate
		sat_gcrf[itt, 1:4] = sat_gcrf_pos
		sat_gcrf[itt, 4:7] = sat_gcrf_vel
	
		#######################################################################################
		#	perform GCRF to ITRF conversion
		sat_itrf_pos, sat_itrf_vel = gcrf2itrf(
			sat_gcrf_pos,
			sat_gcrf_vel,
			pm,
			st,
			pn,
			lod
		)
	
		#	add position and velocity to output array
		sat_itrf[itt, 0] = jdate
		sat_itrf[itt, 1:4] = sat_itrf_pos
		sat_itrf[itt, 4:7] = sat_itrf_vel

	return sat_gcrf, sat_itrf, orbital_elements_gcrf

###################################################################################################
#	Module: 		transformation
#	Description: 	generate Sun and Earth ICRF and GCRF data
#
###################################################################################################

def module_transformation(
		path_data, logging_level, save_flag,
		start_time_object, stop_time_object,
		object_id, site_str,
		len_record, time_step,
		eop_data, dat_data,
		pm_array, pn_array, st_array, c2t_array,
		sat_gcrf, sun_gcrf, earth_gcrf, site_data
	):

	if logging_level != 0:
		print('\n[{}]	Acquiring Transformation Data Files.'.format(datetime.datetime.now()))

	###############################################################################################
	#	parse time data

	#	utc start
	jdayutc_start = start_time_object.jday_utc
	jfracutc_start = start_time_object.jfrac_utc
	jdutc_start = jdayutc_start + jfracutc_start

	#	utc stop
	jdayutc_stop = stop_time_object.jday_utc
	jfracutc_stop = stop_time_object.jfrac_utc
	jdutc_stop = jdayutc_stop + jfracutc_stop

	#	tt start
	jdaytt_start = start_time_object.jday_tt
	jfractt_start = start_time_object.jfrac_tt
	jdtt_start = jdaytt_start + jfractt_start
	
	#	ut1 start
	jdayut1_start = start_time_object.jday_ut1
	jfracut1_start = start_time_object.jfrac_ut1
	jdut1_start = jdayut1_start + jfracut1_start

	#	generate start and stop datetimes
	start_date = jd2date(jdayutc_start, jfracutc_start)
	stop_date = jd2date(jdayutc_stop, jfracutc_stop)

	#	time range in days
	sim_range = np.linspace(jdutc_start, jdutc_stop, len_record)

	#	generate sun/earth unique data files
	start_date_str = '{:4d}-{:02d}-{:02d}'.format(start_date[0], start_date[1], start_date[2])
	start_time_str = '{:02d}:{:02d}:{:04.1f}'.format(start_date[3], start_date[4], start_date[5])
	stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])
	stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])

	###############################################################################################
	#	define file paths

	#	input dir
	input_dir = 'input_dir'
	
	#	tle subdirectory
	tle_dir = 'tle_files'

	#	pos dir
	pos_dir = 'pos_dir'

	###############################################################################################
	#	generate file names

	#	generate transformation data files
	file_names = gen_trans_files(
		start_date_str, start_time_str, len_record, time_step, object_id, site_str
	)

	###############################################################################################
	#	generate output paths

	#	position data
	pos_path = os.path.join(path_data, pos_dir)

	#	TLE files
	tle_path = os.path.join(path_data, pos_dir)

	###############################################################################################
	#	perform transformation

	#	set transformation flag to false
	transformation_flag = False
	
	#	check all files exist
	for file_name in file_names:
		flag = os.path.isfile(os.path.join(pos_path, file_name))
	
		if flag is False:
			transformation_flag = True
			break

	if transformation_flag is False:
	
		if logging_level != 0:
			print('					Reading transformation data...')
	
		#	load transformation data files
		sat_orb = np.loadtxt(os.path.join(pos_path, file_names[0]), delimiter=',')
		sat_azel = np.loadtxt(os.path.join(pos_path, file_names[1]), delimiter=',')
		sun_scrf = np.loadtxt(os.path.join(pos_path, file_names[2]), delimiter=',')
		sun_orb = np.loadtxt(os.path.join(pos_path, file_names[3]), delimiter=',')
		sun_azel = np.loadtxt(os.path.join(pos_path, file_names[4]), delimiter=',')
		site_gcrf = np.loadtxt(os.path.join(pos_path, file_names[5]), delimiter=',')
		site_scrf = np.loadtxt(os.path.join(pos_path, file_names[6]), delimiter=',')
		site_orb = np.loadtxt(os.path.join(pos_path, file_names[7]), delimiter=',')
		earth_orb = np.loadtxt(os.path.join(pos_path, file_names[8]), delimiter=',')

	else:
	
		if logging_level != 0:
			print('\n					Generating transformation data...')
	
		###########################################################################################
		#	run transformation module

		if logging_level != 0:
			print('\n						Performing Transformations:\n')

		#	generate input arrays
		data_array = np.hstack([sat_gcrf, sun_gcrf[:, 7:], earth_gcrf[:, 7:]])
		time_data = np.array((jdaytt_start, jfractt_start, jdayut1_start, jfracut1_start))
				
		#	initilise output arrays
		out_data_itrf = np.zeros((len(data_array), 6))
		out_data_gcrf = np.zeros((len(data_array), 6))
		out_data_scrf = np.zeros((len(data_array), 6))
		out_data_orb_sat = np.zeros((len(data_array), 6))
		out_data_orb = np.zeros((len(data_array), 9))
		out_data_azel = np.zeros((len(data_array), 6))

		simulation_data = [logging_level, time_step]

		#	run transformation function
		glim_trans(
			data_array,
			eop_data, dat_data, pn_array, pm_array, st_array, c2t_array,
			site_data, time_data,
			simulation_data,
			out_data_itrf,
			out_data_gcrf,
			out_data_scrf,
			out_data_orb, out_data_orb_sat,
			out_data_azel
		)

		###########################################################################################
		#	prepare output arrays
		jd_column = np.reshape(sim_range, [len(sim_range), 1])
	
		#	satellite
		sat_orb = np.hstack((jd_column, out_data_orb_sat[:, 0:6]))
		sat_azel = np.hstack((jd_column, out_data_azel[:, 0:3]))
	
		#	sun
		sun_scrf = np.hstack((jd_column, out_data_scrf[:, 0:3]))
		sun_orb = np.hstack((jd_column, out_data_orb[:, 0:3]))
		sun_azel = np.hstack((jd_column, out_data_azel[:, 3:6]))
	
		#	site
		site_gcrf = np.hstack((jd_column, out_data_gcrf[:, 0:3]))
		site_scrf = np.hstack((jd_column, out_data_scrf[:, 3:6]))
		site_orb = np.hstack((jd_column, out_data_orb[:, 6:9]))
	
		#	earth
		earth_orb = np.hstack((jd_column, out_data_orb[:, 3:6]))
   	
		###########################################################################################
		#	save output arrays

		if save_flag == True:

			if logging_level != 0:
				print('\n						Saving Transformation Data Files...')
	
			for file_name in file_names:
				#	generate data identifier
				identifier = '_'.join([*file_name.split('_')[0:2]])
	
				#	save file
				exec("np.savetxt(os.path.join(pos_path, file_name), {}, delimiter=',')".format(
					identifier
				))
	
	return sat_orb, sat_azel, sun_orb, sun_azel, site_orb, earth_orb
