###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	import from general function list
from functions.functions_list import jday_jfrac

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.ticker as ticker

from matplotlib import cm
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D


###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front stop script initilised.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

###################################################################################################
#	light curve generation parameters
###################################################################################################

##################################################
#	simulation settings

#	start time
start_date = '2016-01-01'
start_time = '00-00-00.0'

#	length
days_record = 365
time_step = 30
len_record = int(86400 / time_step) * days_record + 1

#	output light curve data
lc_time_step = 1

##################################################
#	site data

#	site identifiers
site_latitude_range = np.arange(0, 90, 10)
site_longitude = 0

##################################################
#	object data

#	object identifiers
inclination_range = np.arange(0, 90, 10)
orbital_height = 1000

#	range of raan values
raan_range = [0, 20, 40, 60, 80, 100, 120, 140, 160]

#	specific light curve id to simulate
specific_id = 0

#	set lightcurve output directory
lightcurve_data_path = os.path.join(path_data, 'lightcurve_dir', 'data_files')

###################################################################################################
#	Initialise Figure
###################################################################################################

fig, ax = plt.subplots()

viridis = cm.get_cmap('viridis', 256)
cmap = viridis

data = np.random.randn(len(inclination_range) + 1, len(site_latitude_range) + 1)

site_latitude_range2 = np.arange(0, 100, 10)
inclination_range2 = np.arange(0, 100, 10)

x_vals = np.zeros(data.shape)
for i in range(len(x_vals)):
	x_vals[i] = inclination_range2

y_vals = np.zeros(data.shape)
for i in range(len(y_vals)):
	y_vals[:, i] = site_latitude_range2

###################################################################################################
#	looping through inclination files
###################################################################################################

max_nlc = 0

for lat_itt, site_latitude in enumerate(site_latitude_range):
	print('\n	Working on Site Latitude = ', site_latitude)

	for inc_itt, inclination in enumerate(inclination_range):
	
		print('		Working on Inclination = ', inclination)
	
		#	initialise data files list
		lightcurve_data_files = []
	
		tot_lc = 0
		
		print('			Looping through RAAN files...')
		#	generate file names
		for raan in raan_range:
	
			#	generate strings
			site_str = 'lat' + str(site_latitude) + 'lon0'
			object_id = 'i' + str(inclination) + 'h' + str(orbital_height) + 'raan' + str(raan)
			
			lightcurve_data_file = '_'.join(
				[site_str, str(object_id), start_date, start_time, str(len_record), str(time_step)]
			)  + '.txt'
		
			###############################################################################################
			#	load extracted light curves data file
		
			#	load light curve file
			lightcurve_data_array = np.loadtxt(
				os.path.join(lightcurve_data_path, lightcurve_data_file),
				delimiter=','
			)
		
			#	unique light curve ids
			unique_ids = np.unique(lightcurve_data_array[:, 0])
	
			n_lc = len(unique_ids)
	
			###############################################################################################
			#	unique light curves loop
			
			if lightcurve_data_array.shape == (3,3):
				#	no light curves
				pass
			else:
				tot_lc += n_lc
	
		#	set data value
		data[lat_itt, inc_itt] = tot_lc

		#	compare with max_nlc
		if max_nlc < tot_lc:
			max_nlc = tot_lc

#	color plot
colmap = ax.pcolormesh(x_vals, y_vals, data, cmap=cmap, rasterized=True, vmin=0, vmax=max_nlc)

ax.set_xlabel('Spacecraft Inclination')
ax.set_ylabel('Observation Site Latitude')

#	set x ticks
minor_ticks = np.arange(0, 90, 10)
minor_tick_locs = np.arange(5, 95, 10)

ax.xaxis.set_minor_locator(ticker.FixedLocator(minor_tick_locs))
ax.xaxis.set_minor_formatter(ticker.FixedFormatter(minor_ticks))

ax.xaxis.set_major_locator(ticker.NullLocator())
ax.xaxis.set_major_formatter(ticker.NullFormatter())

#	set y ticks
major_ticks = np.arange(0, 90, 10)
major_tick_locs = np.arange(5, 95, 10)

ax.yaxis.set_minor_locator(ticker.FixedLocator(minor_tick_locs))
ax.yaxis.set_minor_formatter(ticker.FixedFormatter(minor_ticks))

ax.yaxis.set_major_locator(ticker.NullLocator())
ax.yaxis.set_major_formatter(ticker.NullFormatter())

fig.colorbar(colmap, ax=ax)

plt.show()
sys.exit()
