###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	parrellisation
from multiprocessing import Pool

#	progress bar
import progressbar

#	import from general function list
from functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, get_times, \
	interp

#	import from general function list
from functions.functions_transform import site_vec, rot_mat, r2polar, rot1, rot2, rot3, axis2mat, \
	mat2angles

#	visualisation functions
from functions.glim_visualisation import extract_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front stop script initilised.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

###################################################################################################
#	simulation parameters
###################################################################################################

#	number of light curves to visualise
num_lc = 1

##################################################
#	simulation time parameters

#	start time
date_str = '2016-01-01'
time_str = '00-00-00.0'

#	length
days_record = 365
time_step = 60
len_record = int(86400 / time_step) * days_record + 1

#	duration
duration_sec = time_step * (len_record - 1)
duration_days = duration_sec / 86400

##################################################
#	simulation site parameters

#	site identifier
site_latitude = 0
site_longitude = 0
site_str = 'lat{}lon{}'.format(site_latitude, site_longitude)

##################################################
#	simulation object parameters

#	inclination
inclination = 40

#	raan
raan = 90

#	orbital height
orbital_height = 2000

#	input identifiers
object_id = 'i' + str(inclination) + 'h' + str(orbital_height) + 'raan' + str(raan)

##################################################
#	object geometry and attitude state

#	geometry string
geom_str = 'cylinder'

##################################################
#	attitude

#	generate random attitude
initial_attitude = np.array([np.radians(142), np.radians(57), 0.0])

##################################################
#	attitude motion

#	1 rpm (radians per second)
one_rpm = (2.0 * np.pi) / 60.0

#	define rotation axis in body frame
axis_unit = np.array([1.0, 0.0, 0.0])

#	convert to rotation matrix
attitude_matrix = axis2mat(one_rpm, axis_unit)

#	extract euler angles
angles1 = np.array(mat2angles(attitude_matrix)[0:3])
angles2 = np.array(mat2angles(attitude_matrix)[3:6])

#	convert negative zeros
for i, ele in enumerate(angles1):
	if str(ele)[0] == '-' and ele == 0:
		angles1[i] = 0.0

initial_attitude_motion = angles1

#	generate file name attitude string
attitude_state_str = '{:0.3f}_{:0.3f}_{:0.3f}_{:0.3f}_{:0.3f}_{:0.3f}'.format(
	*initial_attitude,
	*initial_attitude_motion
)

##################################################
#	generate rotation vector

#	rotation matrix
rot_matrix = rot_mat(initial_attitude)

#	make unit vector
rotation_vector = np.divide(initial_attitude_motion, np.linalg.norm(initial_attitude_motion))

#	generate rot vec
rot_vec = np.dot(rotation_vector, rot_matrix)
rot_vec[0] = -rot_vec[0]

#	convert to polar coordinates
rot_vec_polar = np.degrees([initial_attitude[0], initial_attitude[1]])

print('\n#####################################################################################')
print('\n	Time parameters: ')
print('		Simulation time step:		{}'.format(time_step))
print('		Duration:			{}		[steps]'.format(len_record))
print('						{}	[s]'.format(np.round(duration_sec, 1)))
print('						{}	[m]'.format(np.round(duration_sec / 60, 1)))
print('						{}		[days]'.format(np.round(duration_sec / 60 / 60 / 24, 1)))
print('\n	Site parameters: ')
print('		Site Latitude:			{}		Degrees'.format(site_latitude))
print('		Site Longitude:			{}		Degrees'.format(site_longitude))
print('\n	Object parameters: ')
print('\n		Geometry Str: 		{}'.format(geom_str))
print('\n		Attitude:		{}, {}, {}	Radians'.format(*initial_attitude))
print('		Attitude Motion: 	{}, {}, {}		Radians/s'.format(*initial_attitude_motion))
print('\n		Inclination:		{}			Degrees'.format(inclination))
print('		Orbital Height: 	{}			Km'.format(orbital_height))
print('		Right Ascension:	{}			Degrees'.format(raan))
print('\n		Rotation Vector:	{:1.3f}, {:1.3f}, {:1.3f}	Radians'.format(*rot_vec))
print('		Polar Coordinates:	{:02.1f}, {:02.1f}		Degrees'.format(*rot_vec_polar))
print('\n#####################################################################################')

###################################################################################################
#	load light curves files
###################################################################################################

#	set lightcurve output directory
lightcurve_path = os.path.join(path_data, 'lightcurve_dir', 'lightcurve_files')
lightcurve_data_path = os.path.join(path_data, 'lightcurve_dir', 'data_files')

#	generate file names
lightcurve_data_file = '_'.join(
	[site_str, str(object_id), date_str, time_str, str(len_record), str(time_step)]
)  + '.txt'

#	input files
# lightcurve_file = 'lat0lon0_i80h800raan0_cylinder_1.063_4.157_3.145_0.105_0.000_0.000.txt'
lightcurve_file = '_'.join(
	[site_str, str(object_id), geom_str, attitude_state_str]
)  + '.txt'

lc_time_step = 1

print('\nLoading Lightcurve file:\n	', lightcurve_file)
#	load light curve file
lightcurve_data = np.loadtxt(
	os.path.join(lightcurve_path, lightcurve_file),
	delimiter=','
)

print('\nLoading Lightcurve data file:\n	', lightcurve_data_file)
#	load light curve data file
lightcurve_data_array = np.loadtxt(
	os.path.join(lightcurve_data_path, lightcurve_data_file),
	delimiter=','
)

###################################################################################################
#	parsing light curves file data
###################################################################################################

#	time step
time_step = int(np.round(86400 * (lightcurve_data_array[1, 1] - lightcurve_data_array[0, 1]), 1))

###################################################################################################
#	Unique light curves
###################################################################################################

#	unique light curve ids
unique_ids = np.unique(lightcurve_data[:, 0])

#	initialise all light curves figure	
fig1 = plt.figure(figsize=(10,6))
ax1 = plt.axes()
ax1.set_title('Light Curves')

#	initialise stacked light curves figure	
#fig1c = plt.figure(figsize=(10,6))
#ax1c = plt.axes()
#ax1c.set_title('Stacked Light Curves')

#	initialise polar figure
fig2 = plt.figure(figsize=(10, 6))
ax2 = plt.subplot(111, projection='polar')
ax2.set_title('Polar plot')

ax2.set_theta_zero_location("N")
ax2.set_theta_direction(-1)

ax2.set_rticks(np.arange(0, 100, 10))
ax2.set_rlim([0, 100])

#	initialise phase angle plots
fig3 = plt.figure(figsize=(10,6))
ax3 = plt.axes()
ax3.set_title('Phase Angles.')

#	initialise phase angle plots
fig3b = plt.figure(figsize=(10,6))
ax3b = plt.axes()
ax3b.set_title('Obs Angles.')

####################################################
#	plot rotation vector
marker_size = 10
marker_edge = 2
ax2.plot(np.radians(rot_vec_polar[0]), rot_vec_polar[1],
	marker='+',
	color='g',
	markersize=marker_size,
	markeredgewidth=marker_edge,
	clip_on=False
)

unique_ids = [232]

print('\nLooping through unqiue light curves...')
for light_curve_id in unique_ids:

	print('	\nUsing Light Curve ID: ', light_curve_id)

	###############################################################################################
	#	parse input data

	#	ids of light curve data
	data_ids1 = np.where(lightcurve_data[:, 0]==light_curve_id)
	data_ids2 = np.where(lightcurve_data_array[:, 0]==light_curve_id)

	#	extract data
	lightcurve = lightcurve_data[data_ids1]
	light_curve_data = lightcurve_data_array[data_ids2]
	len_light_curve = len(light_curve_data)
	
	#	parse array
	lc_jd_time_data = lightcurve[:, 1]

	###############################################################################################
	#	parse light curve data array
	jd_time_data = light_curve_data[:, 1]
	sun_r_data = light_curve_data[:, 2]
	sun_az_data = light_curve_data[:, 3]
	sun_el_data = light_curve_data[:, 4]
	sat_r_data = light_curve_data[:, 5]
	sat_az_data = light_curve_data[:, 6]
	sat_el_data = light_curve_data[:, 7]
	sun_orb_data = light_curve_data[:, 8:11]
	earth_orb_data = light_curve_data[:, 11:14]
	site_orb_data = light_curve_data[:, 14:17]
	sat_orb_data = light_curve_data[:, 17:23]
	sat_itrf_data = light_curve_data[:, 23:26]

	###############################################################################################
	#	interpolate if required

	start_jdate = jd_time_data[0]
	stop_jdate = jd_time_data[-1]
	
	#	convert to light curve generation time step
	if lc_time_step != time_step:
	
		#	find nsteps
		prop_step = lc_time_step / (24 * 60 * 60)
		num_points = int(round((stop_jdate - start_jdate) / prop_step))
	
		#	new time range to interpolate to
		interp_range = np.linspace(start_jdate, stop_jdate, num_points + 1)
	
		#	current x spline
		spline_x = jd_time_data
	
		#	new spline x
		spline_x_new = interp_range
	
		#	generate data to interpolate
		interp_data_in = np.hstack((sat_orb_data, sun_orb_data, earth_orb_data, site_orb_data))
	
		#	initilise new y spline data
		interp_data_out = np.zeros((len(interp_range), len(interp_data_in[0])))
	
		#	perform interpolation	
		interp_data = interp(interp_data_in, interp_data_out, spline_x, spline_x_new)
	
		#	extract interpolated data
		jd_time_data = interp_range
		sat_orb_data = interp_data[:, 0:6]
		sun_orb_data = interp_data[:, 6:9]
		earth_orb_data = interp_data[:, 9:12]
		site_orb_data = interp_data[:, 12:15]
	
		#	reset data
		len_light_curve = len(jd_time_data)
		duration_sec = lc_time_step * (len_light_curve - 1)
		light_curve_sim_time = np.linspace(0, duration_sec, len_light_curve)

	###############################################################################################
	#	extract light curve polar data

	sun_polar_data, site_polar_data, phase_angles, jd_time_data = extract_fun(
		jd_time_data, lc_time_step, time_step,
		sun_orb_data, earth_orb_data, site_orb_data, sat_orb_data
	)

	#	rot vec to sun angle
	sun_angles = []
	site_angles = []

	for sun_orb_vec, site_orb_vec in zip(sun_orb_data, site_orb_data):

		sun_angles.append(np.arccos(np.divide(
			np.dot(sun_orb_vec, rot_vec),
			np.linalg.norm(sun_orb_vec) * np.linalg.norm(rot_vec)
		)))

		site_angles.append(np.arccos(np.divide(
			np.dot(site_orb_vec, rot_vec),
			np.linalg.norm(site_orb_vec) * np.linalg.norm(rot_vec)
		)))

	###############################################################################################
	#	graphical outputs
	###############################################################################################
	
	#	generate date vectors
	datevec_array = []
	
	for line in lc_jd_time_data:
		#	convert jd to date
		year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(line))]
	
		#	turn into string
		datestr = '-'.join(['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)])
		timestr = ':'.join(['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)])
		datevec  = ' '.join([datestr, timestr])

		print(type(datestr))
		print(len(datestr))
		print(type(timestr))
		print(len(timestr))
		print(type(datevec))
		print(len(datevec))
		sys.exit()



		#	convert to datetime object
		date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
	
		print(datevec)
		print(datevec.type)
		print(date_time_obj)
		sys.exit()
	
		#	append  to output array
		datevec_array.append(date_time_obj)
	
	#	tick mark locators
	years = mdates.YearLocator()   # every year
	months = mdates.MonthLocator()  # every month
	days = mdates.DayLocator()  # every day
	hours = mdates.HourLocator()  # every hour
	minutes = mdates.MinuteLocator()  # every minute
	seconds = mdates.SecondLocator()  # every second
	
	###############################################################################################
	#	light curve plot

	#	extract light curve magnitude data
	mag_plot = np.array(lightcurve[:, -1]).astype(np.float)
	time_plot = np.linspace(0, len(mag_plot) - 1, len(mag_plot))
	
	color_str = 'r'
	#	plot lightcurve
	ax1.plot(time_plot, mag_plot, color=color_str)

	#	format plot
#	ax1.set_ylim([20, 0])
	ax1.grid()

	#	format major ticks
#	ax1.xaxis.set_major_locator(hours)
#	ax1.xaxis.set_major_formatter(mdates.DateFormatter('%H:%M'))
#	ax1.xaxis.set_tick_params(which='major', pad=15)
	
#	turn off minor ticks
#	ax1.tick_params(axis='x', which='minor', bottom=False)
#	ax1.xaxis.set_minor_locator(seconds)
#	ax1.xaxis.set_minor_formatter(mdates.DateFormatter('%S'))
	
	###############################################################################################
	#	stacked light curve plot
	
	#	plot lightcurve
#	ax1c.plot(time_plot, mag_plot, color=color_str)
	
	#	turn off minor ticks
#	ax1c.tick_params(axis='x', which='minor', bottom=False)
#	#	format plot
#	ax1c.set_ylim([20, 0])
#	ax1c.grid()

	###########################################################################################
	#	polar plot
	marker_sun = [len(sun_polar_data) - 1]
	marker_site = [len(site_polar_data) - 1]
	
	#	plot sun positions
	ax2.plot(
		sun_polar_data[:, 0],
		np.degrees(sun_polar_data[:, 1]),
		linewidth=0.5, color='r', markevery=marker_sun, marker='>'
	)
	
	#	plot site positions
	ax2.plot(
		site_polar_data[:, 0],
		np.degrees(site_polar_data[:, 1]),
		linewidth=0.5, color='g', markevery=marker_site, marker='>'
	)

	###############################################################################################
	#	phase angle plots
	
	time_plot = np.linspace(0, len(mag_plot) - 1, len(mag_plot))
	
	#	plot lightcurve
	ax3.plot(np.degrees(phase_angles), color='b', label='Phase Angle')
	ax3.plot(np.degrees(sun_angles), color='r', label='Sun Angle')
	ax3.plot(np.degrees(site_angles), color='g', label='Site Angle')
	
	#	format major ticks
	# ax1b.xaxis.set_major_locator(hours)
	# ax1b.xaxis.set_major_formatter(mdates.DateFormatter('%H:%M'))
	# ax1b.xaxis.set_tick_params(which='major', pad=15)
	
	#	turn off minor ticks
	#ax1b.xaxis.set_minor_locator(seconds)
	#ax1b.xaxis.set_minor_formatter(mdates.DateFormatter('%S'))
	
	#	format plot
#	ax3.set_ylim([0, 90])
	ax3.grid()
	ax3.legend()

	#	plot lightcurve
	ax3b.plot(np.degrees(site_polar_data[:, 0]), color='g', label='Site Azm Angle')
	ax3b.plot(np.degrees(site_polar_data[:, 1]), color='g', label='Site Inc Angle')
	ax3b.grid()
	ax3b.legend()
	
	
print('\nEND')
plt.show()
sys.exit()
