###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	
#	Parse outputs of a bulk run into separate directories
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

# shutil for file copy and recursive delete
import shutil

###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front end script initilised.')

#	set start time
sim_start = time.time()

###################################################################################################
#	inputs
###################################################################################################

##################################################
#	simulation parameters

#	time step in seconds
time_step = 0.1

#	length of data record [number of points]
len_record = 1500

#	add zero
len_record += 1

##################################################
#	case parameters

#	reflection case
reflection_case = 'diffuse'

#	geometry case
geom_str = 'cylinder'

#	illumination vector polar angles
illum_azm_deg = 229.1
illum_inc_deg = 77.6

#	illumination vector polar angles
obs_azm_deg = 270.0
obs_inc_deg = 40.0

#	attitude state
attit_x_deg = 140.0
attit_y_deg = 36.0
attit_z_deg = 0.0

attit_x_rad = np.radians(attit_x_deg)
attit_y_rad = np.radians(attit_y_deg)
attit_z_rad = np.radians(attit_z_deg)

attitude = np.array([attit_x_rad, attit_y_rad, attit_z_rad])

##################################################
#	case indentifier
case_name = 'obsvec-y'
case_name = 'attit-y'
case_name = 'attit-x'

#	generate case vector
case_vector_deg = [illum_azm_deg, illum_inc_deg, obs_azm_deg, obs_inc_deg]

#	generate ident string
ident_string = 'illum_{}-{}_obs_{}-{}'.format(*case_vector_deg)

#	generate case designation
case_desig = geom_str + '_' + ident_string + '_' + case_name

#	input file
input_file = geom_str + '_' \
		+ ident_string + '_' \
		+ case_name + '_lightcurves_{}-{}.txt'.format(len_record, time_step)

###################################################################################################
#	initilisation
###################################################################################################

#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	light curve read path
lightcurve_in = os.path.join(path_data, 'lightcurve_dir', case_desig, 'total_lightcurves')

#	parsed lightcurve file temp path
lightcurve_out = os.path.join(lightcurve_in, ident_string)

#	generate lightcurve output directory
if not os.path.exists(lightcurve_out):
	os.mkdir(lightcurve_out)

###################################################################################################
#	parse lightcurves
###################################################################################################

##################################################
#	total light curve data
#	read in run file
print('\nReading bulk lightcurve file...')
bulk_file_data = []

with open(os.path.join(lightcurve_in, input_file), mode='r', encoding='utf-8') as case_file:

	for line in case_file:
		line_data = line.strip('\n').split(', ')

		#	total data
		bulk_file_data.append(line_data)

##################################################
#	separating files

first_flag = True
for line in bulk_file_data:

	##################################################
	#	parse bulk file line
	case_ident, case_num,\
		illum_azm_step, illum_inc_step, \
		obs_azm_step, obs_inc_step, \
		attit_x_step, attit_y_step, attit_z_step, \
		jdate, sec_since, magnitude = line

	case_num = int(case_num)

	#	create rounded numbers for output file name
	illum_azm_step = np.round(float(illum_azm_step), 1)
	illum_inc_step = np.round(float(illum_inc_step), 1)

	obs_azm_step = np.round(float(obs_azm_step), 1)
	obs_inc_step = np.round(float(obs_inc_step), 1)

	attit_x_step = np.round(float(attit_x_step), 1)
	attit_y_step = np.round(float(attit_y_step), 1)
	attit_z_step = np.round(float(attit_z_step), 1)

	##################################################
	#	generate case num string
	if case_num < 10:
		case_num_str = '00000' + str(case_num)
	elif case_num < 100:
		case_num_str = '0000' + str(case_num)
	elif case_num < 1000:
		case_num_str = '000' + str(case_num)
	elif case_num < 10000:
		case_num_str = '00' + str(case_num)
	elif case_num < 100000:
		case_num_str = '0' + str(case_num)
	elif case_num > 100000:
		case_num_str = str(case_num)

	##################################################
	#	generate output file name
	case_vector_step = [
		illum_azm_step, illum_inc_step,
		obs_azm_step, obs_inc_step,
		attit_x_step, attit_y_step, attit_z_step
	]

	ident_string = 'illum_{}-{}_obs_{}-{}_attit_{}-{}-{}'.format(*case_vector_step)
	filename = case_num_str + '_' + geom_str + '_' + ident_string + '.lightcurve'

	##################################################
	#	extract line data
	data_line = np.array(line[2:]).astype(np.float)

	##################################################
	#	start of a first light curve
	if first_flag is True:
	
		#	initilise data
		output_data = data_line
		current_id = case_num
		first_flag = False

	##################################################
	#	not a first run
	elif first_flag is False:

		##################################################
		#	continued run
		if case_num == current_id:

			#	append to output data
			output_data = np.vstack((output_data, data_line))

		##################################################
		#	start of new run
		elif case_num != current_id:

			#	save previous data
			np.savetxt(os.path.join(lightcurve_out, prev_filename), output_data, delimiter=',')

			#	initilise new data
			output_data = data_line
			current_id = case_num

	#	set previous file name
	prev_filename = filename

#	save ongoing case at loop end
np.savetxt(os.path.join(lightcurve_out, filename), output_data, delimiter=',')

