###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	progress bar
import progressbar

#	parrellisation
from multiprocessing import Pool
import tqdm

#	import SOFA interface
from tools.functions.sofa_interface import rot_mat_gen

#	import from transformation functions
from tools.functions.functions_transform import site_vec, rv2kep

#	import from general function list
from tools.functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, \
	get_times

#	import modules
from tools.glim_modules import module_horizons_pos, module_sat_pos, module_custom_orbit, \
	module_transformation

#	custom access function for parrallisation
from function_access import access_fun

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

###################################################################################################
#	inputs
###################################################################################################
#	clear terminal
call(['clear'])

print('\n[{}] GLIMMA access generation script initilised.'.format(datetime.datetime.now()))

#	get simulation start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set lightcurve output directory
lightcurve_path = os.path.join(path_data, 'lightcurve_dir', 'data_files')

###################################################################################################
#	simulation parameters
###################################################################################################

#	parrellisation settings
parral_flag = False
ncores = 8

#	logging settings
logging_level = 1
horz_logging_level = 1

##################################################
#	time date

#	start time
start_date = np.array([2016, 1, 1, 0, 0, 0])

#	time step in seconds
time_step = 60

#	length of data record [number of points]
days_record = 1
len_record = (int(86400 / time_step) * days_record) + 1

##################################################
#	site data
site_latitude = 0.0
site_longitude = 0.0
site_altitude = 0.0

##################################################
#	object data

#	norad id
norad_id = 1575
norad_flag = False

###################################################################################################
#	parse simulation parameters
###################################################################################################

##################################################
#	simulation time parameters

#	start
start_jday, start_jdfrac = date2jd(*start_date)
start_jdate = start_jday + start_jdfrac

#	duration
duration_sec = time_step * (len_record - 1)
duration_days = duration_sec / 86400

#	stop
stop_jdate = start_jdate + duration_days
stop_date = jd2date(*jday_jfrac(stop_jdate))

#	time range in days
sim_range = np.linspace(start_jdate, stop_jdate, len_record)

#	time range in seconds
sim_time = np.linspace(0, duration_sec, len_record)

print('\n#####################################################################################')
print('\n	Time parameters: ')
print('		Start Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*start_date))
print('		Stop Date:			{}/{:02d}/{:02d} - {:02d}:{:02d}:{:04.1f}'.format(*stop_date))
print('		Simulation time step:		{}'.format(time_step))
print('		Duration:			{}		[steps]'.format(len_record))
print('						{}	[s]'.format(np.round(duration_sec, 1)))
print('						{}	[m]'.format(np.round(duration_sec / 60, 1)))
print('						{}		[days]'.format(np.round(duration_sec / 60 / 60 / 24, 1)))

##################################################
#	simulation site parameters

#	site latitude / longitude
site_latgd = np.radians(site_latitude)
site_lon = np.radians(site_longitude)
site_alt = site_altitude

#	generate unique site indentifier
site_str = 'lat{}lon{}'.format(int(site_latitude), int(site_longitude))

#	site itrf vector
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000)

#	site data array
site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))

print('\n	Site parameters: ')
print('		Site Str:			{}'.format(site_str))
print('		Site Latitude:			{}	deg'.format(site_latitude))
print('		Site Longitude:			{}	deg'.format(site_longitude))
print('		Site Altitude:			{}	m'.format(site_altitude))
print('\n#####################################################################################')

###################################################################################################
#	initialisation
###################################################################################################

##################################################
#	generate eop data array

print('\n[{}]	Generating EOP data array...'.format(datetime.datetime.now()))

#	EOP data
eop_dir = 'eop_files'
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	assemble eop path
eop_path = os.path.join(path_data, eop_dir)

#	generate interpolated eop_data
eop_log_level = 1
eop_data, dat_data = eop_interpolate(eop_path, eop_file, dat_file, sim_range, time_step, eop_log_level)

##################################################
#	generate time objects

#	start dat value
dat = dat_data[0, -1]
	
#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate start time object
start_time_object = get_times(*start_date, ut_offset, dat)
	
#	end dat value
dat = dat_data[-1, -1]
	
#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
#	generate stop time object
stop_time_object = get_times(*jd2date(*jday_jfrac(stop_jdate)), ut_offset, dat)
	
#	generate unique file name identifiers
start_date_str = '{:4d}-{:02d}-{:02d}'.format(start_date[0], start_date[1], start_date[2])
start_time_str = '{:02d}:{:02d}:{:04.1f}'.format(start_date[3], start_date[4], start_date[5])
stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])
stop_date_str = '{:4d}-{:02d}-{:02d}'.format(stop_date[0], stop_date[1], stop_date[2])

###################################################################################################
#	MODULE:	Horizons Positions
###################################################################################################
print('\n[{}]	Acquiring Horizons Data...'.format(datetime.datetime.now()))

sun_icrf, sun_gcrf, earth_icrf, earth_gcrf = module_horizons_pos(
	path_data, horz_logging_level,
	start_time_object, stop_time_object,
	len_record, time_step,
	eop_path
)

###################################################################################################
#	generate rotation matrices
###################################################################################################
print('\n[{}]	Generating rotation matrices:\n'.format(datetime.datetime.now()))

#	tt time
jdaytt_start = start_time_object.jday_tt
jfractt_start = start_time_object.jfrac_tt
jdtt_start = jdaytt_start + jfractt_start

#	ut1 time
jdayut1_start = start_time_object.jday_ut1
jfracut1_start = start_time_object.jfrac_ut1
jdut1_start = jdayut1_start + jfracut1_start

#	generate rotation matrices
pm_array, pn_array, st_array, c2t_array = rot_mat_gen(
	jdtt_start, jdut1_start, eop_data, time_step
)
	
###################################################################################################
#	generating position data
###################################################################################################
print('\n[{}]	Generating position data:\n'.format(datetime.datetime.now()))

###################################################################################################
#	using SGP4
sat_gcrf_sgp4, sat_itrf_sgp4 = module_sat_pos(
	path_data, logging_level, len_record, time_step,
	start_time_object, stop_time_object,
	norad_id,
	eop_data, dat_data,
	pm_array, pn_array, st_array, c2t_array
)

###################################################################################################
#	converting to orbital elements

orbital_elements_gcrf_sgp4 = []
orbital_elements_itrf_sgp4 = []

for (sat_gcrf_data, sat_itrf_data) in zip(sat_gcrf_sgp4, sat_itrf_sgp4):

	#	parse inputs
	jdate = sat_gcrf_data[0]
	sat_gcrf_pos = sat_gcrf_data[1:4]
	sat_gcrf_vel = sat_gcrf_data[4:7]

	sat_itrf_pos = sat_itrf_data[1:4]
	sat_itrf_vel = sat_itrf_data[4:7]

	#	calculate initial GCRF orbital elements
	raan_gcrf_sgp4, argp_gcrf_sgp4, trueanom_gcrf_sgp4, \
		inc_gcrf_sgp4, ecc_gcrf_sgp4, semimajor_gcrf_sgp4 = rv2kep(sat_gcrf_pos, sat_gcrf_vel)

	#	add to GCRF orbital elements output
	orbital_elements_gcrf_sgp4.append([
		jdate,
		raan_gcrf_sgp4,
		argp_gcrf_sgp4,
		trueanom_gcrf_sgp4,
		inc_gcrf_sgp4,
		ecc_gcrf_sgp4,
		semimajor_gcrf_sgp4
	])

	#	calculate initial ITRF orbital elements
	raan_itrf_sgp4, argp_itrf_sgp4, trueanom_itrf_sgp4, \
		inc_itrf_sgp4, ecc_itrf_sgp4, semimajor_itrf_sgp4 = rv2kep(sat_itrf_pos, sat_itrf_vel)

	#	add to ITRF orbital elements output
	orbital_elements_itrf_sgp4.append([
		jdate,
		raan_itrf_sgp4,
		argp_itrf_sgp4,
		trueanom_itrf_sgp4,
		inc_itrf_sgp4,
		ecc_itrf_sgp4,
		semimajor_itrf_sgp4
	])

#	finalise outputs
orbital_elements_gcrf_sgp4 = np.array(orbital_elements_gcrf_sgp4)
orbital_elements_itrf_sgp4 = np.array(orbital_elements_itrf_sgp4)

###################################################################################################
#	custom orbit

jd = sat_gcrf_sgp4[0, 0]
sat_gcrf_pos = sat_gcrf_sgp4[0, 1:4]
sat_gcrf_vel = sat_gcrf_sgp4[0, 4:7]

#	calculate initial GCRF orbital elements
jdate, raan_gcrf_sgp4, argp_gcrf_sgp4, trueanom_gcrf_sgp4, \
	inc_gcrf_sgp4, ecc_gcrf_sgp4, semimajor_gcrf_sgp4 = orbital_elements_gcrf_sgp4[0, :]

#	earth radius
r_earth = 6371

#	set orbital elements from sgp4 outputs
raan = raan_gcrf_sgp4
argp = argp_gcrf_sgp4
true_anom = trueanom_gcrf_sgp4
inc = inc_gcrf_sgp4
ecc = ecc_gcrf_sgp4
semi_major_axis = semimajor_gcrf_sgp4

#	generate unique object identifier
orbital_height = semi_major_axis - r_earth
object_id = 'i{}h{}raan{}'.format(int(inc), int(orbital_height), int(raan))

print('\n#####################################################################################')
print('\n	Object orbit parameters: ')
print('		JD =				{}	days'.format(jd))
print('		RAAN =				{}	deg'.format(np.degrees(raan_gcrf_sgp4)))
print('		ArgP =				{}	deg'.format(np.degrees(argp_gcrf_sgp4)))
print('		True Anom =			{}	deg'.format(np.degrees(trueanom_gcrf_sgp4)))
print('		Incl =				{}	deg'.format(np.degrees(inc_gcrf_sgp4)))
print('		Ecce =				{}	deg'.format(ecc_gcrf_sgp4))
print('		Semi-Major Axis =		{}	km'.format(semimajor_gcrf_sgp4))
print('\n#####################################################################################')

#	generating a custom orbit
sat_gcrf, sat_itrf, orbital_elements = module_custom_orbit(
	path_data, logging_level, len_record, time_step,
	start_time_object, stop_time_object,
	object_id,
	eop_data, dat_data,
	pm_array, pn_array, st_array, c2t_array,
	raan, argp, true_anom, inc, ecc, semi_major_axis
)

###################################################################################################
#	3d visualisations
###################################################################################################

# ###################################################################################################
# #	gcrf plot
# 
# fig_gcrf, ax_gcrf = plt.subplots(subplot_kw=dict(projection='3d'))
# ax_gcrf.set_title('GCRF Data')
# 
# #	SGP4 GCRF data
# ax_gcrf.plot(
# 	sat_gcrf[:, 1], sat_gcrf[:, 2], sat_gcrf[:, 3],
# 	label='Custom', linestyle='-.', color='tab:blue'
# )
# 
# #	custom GCRF data
# ax_gcrf.plot(
# 	sat_gcrf_sgp4[:, 1], sat_gcrf_sgp4[:, 2], sat_gcrf_sgp4[:, 3],
# 	label='SGP4', linestyle='--', color='tab:red'
# )
# 
# #	finalise plot
# ax_gcrf.set_xlim([-1e4, 1e4])
# ax_gcrf.set_ylim([-1e4, 1e4])
# ax_gcrf.set_zlim([-1e4, 1e4])
# ax_gcrf.legend()
# 
# ###################################################################################################
# #	itrf plot
# 
# fig_itrf, ax_itrf = plt.subplots(subplot_kw=dict(projection='3d'))
# ax_itrf.set_title('ITRF Data')
# 
# #	SGP4 ITRF data
# ax_itrf.plot(
# 	sat_itrf[:, 1], sat_itrf[:, 2], sat_itrf[:, 3],
# 	label='Custom', linestyle='-.', color='tab:blue'
# )
# 
# #	custom ITRF data
# ax_itrf.plot(
# 	sat_itrf_sgp4[:, 1], sat_itrf_sgp4[:, 2], sat_itrf_sgp4[:, 3],
# 	label='SGP4', linestyle='--', color='tab:red'
# )
# 
# #	finalise plot
# ax_itrf.set_xlim([-1e4, 1e4])
# ax_itrf.set_ylim([-1e4, 1e4])
# ax_itrf.set_zlim([-1e4, 1e4])
# ax_itrf.legend()

###################################################################################################
#	generate date vectors
###################################################################################################

datevec_array = []

for line in sim_range:

	#	convert jd to date
	year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(line))]

	#	turn into string
	datestr = '-'.join(['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)])
	timestr = ':'.join(['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)])
	datevec  = ' '.join([datestr, timestr])

	#	convert to datetime object
	date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')

	#	append  to output array
	datevec_array.append(date_time_obj)

#	tick mark locators
years = mdates.YearLocator()   # every year
months = mdates.MonthLocator()  # every month
days = mdates.DayLocator()  # every day
hours = mdates.HourLocator()  # every hour
minutes = mdates.MinuteLocator()  # every minute
seconds = mdates.SecondLocator()  # every second

###################################################################################################
#	orbital element visualisation
###################################################################################################

###################################################################################################
#	GCRF Elements
fig_elements_gcrf, ((ax1_gcrf, ax2_gcrf), (ax3_gcrf, ax4_gcrf), (ax5_gcrf, ax6_gcrf)) \
	= plt.subplots(3, 2, figsize=(15, 10))

fig_elements_gcrf.suptitle('SGP4 Orbital Elements')

##################################################
ax1_gcrf.set_title('Right Ascension of Acending Node')

ax1_gcrf.plot(
	datevec_array, orbital_elements_gcrf_sgp4[:, 1],
	label='SGP4', linestyle='--', color='tab:red'
)

ax1_gcrf.plot(
	datevec_array, orbital_elements[:, 1],
	label='Custom', linestyle='-.', color='tab:blue'
)

# ax1_gcrf.set_ylim([0, 2 * np.pi])
ax1_gcrf.xaxis.set_major_locator(months)
ax1_gcrf.xaxis.set_major_formatter(mdates.DateFormatter('%b'))

##################################################
ax2_gcrf.set_title('Argument of Periapsis')

ax2_gcrf.plot(
	datevec_array, orbital_elements_gcrf_sgp4[:, 2],
	label='SGP4', linestyle='--', color='tab:red'
)

ax2_gcrf.plot(
	datevec_array, orbital_elements[:, 2],
	label='Custom', linestyle='-.', color='tab:blue'
)

ax2_gcrf.set_ylim([0, 2 * np.pi])
ax2_gcrf.xaxis.set_major_locator(months)
ax2_gcrf.xaxis.set_major_formatter(mdates.DateFormatter('%b'))

##################################################
ax3_gcrf.set_title('True Anomaly')

ax3_gcrf.plot(
	datevec_array, orbital_elements_gcrf_sgp4[:, 3],
	label='SGP4', linestyle='--', color='tab:red'
)

ax3_gcrf.plot(
	datevec_array, orbital_elements[:, 3],
	label='Custom', linestyle='-.', color='tab:blue'
)

#ax3_gcrf.set_ylim([0, 2 * np.pi])
ax3_gcrf.xaxis.set_major_locator(months)
ax3_gcrf.xaxis.set_major_formatter(mdates.DateFormatter('%b'))

##################################################
ax4_gcrf.set_title('Inclination')

ax4_gcrf.plot(
	datevec_array, np.degrees(orbital_elements_gcrf_sgp4[:, 4]),
	label='SGP4', linestyle='--', color='tab:red'
)

ax4_gcrf.plot(
	datevec_array, np.degrees(orbital_elements[:, 4]),
	label='Custom', linestyle='-.', color='tab:blue'
)

ax4_gcrf.set_ylim([0, np.degrees(np.pi / 2)])
ax4_gcrf.xaxis.set_major_locator(months)
ax4_gcrf.xaxis.set_major_formatter(mdates.DateFormatter('%b'))

##################################################
ax5_gcrf.set_title('Eccentricity')

ax5_gcrf.plot(
	datevec_array, orbital_elements_gcrf_sgp4[:, 5],
	label='SGP4', linestyle='--', color='tab:red'
)

ax5_gcrf.plot(
	datevec_array, orbital_elements[:, 5],
	label='Custom', linestyle='-.', color='tab:blue'
)

ax5_gcrf.set_ylim([0, 1])
ax5_gcrf.xaxis.set_major_locator(months)
ax5_gcrf.xaxis.set_major_formatter(mdates.DateFormatter('%b'))

##################################################
ax6_gcrf.set_title('Semi-Major Axis')

ax6_gcrf.plot(
	datevec_array, orbital_elements_gcrf_sgp4[:, 6],
	label='SGP4', linestyle='--', color='tab:red'
)

ax6_gcrf.plot(
	datevec_array, orbital_elements[:, 6],
	label='Custom', linestyle='-.', color='tab:blue'
)

ax6_gcrf.set_ylim([0, 10000])
ax6_gcrf.xaxis.set_major_locator(months)
ax6_gcrf.xaxis.set_major_formatter(mdates.DateFormatter('%b'))

fig_elements_gcrf.legend()

plt.show()

