###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	This is the GLIMMA front stop
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

#	numpy
import numpy as np

#	numpy-stl
import stl
from stl import mesh

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	parrellisation
from multiprocessing import Pool

#	progress bar
import progressbar

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D

#	import from general function list
from tools.functions.functions_list import object_gen, \
 									date2jd, \
 									jd2date, \
 									jday_jfrac, \
 									eop_interpolate, \
 									get_times, \
 									gen_sgp4, \
									interp

#	visualisation functions
from functions.glim_visualisation import extract_fun

###################################################################################################
#	initilisation
###################################################################################################
#	clear terminal
call(['clear'])
print('\nGLIMMA front stop script initilised.')

#	set start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

###################################################################################################
#	light curve generation parameters
###################################################################################################

##################################################
#	simulation settings

#	start time
start_date = '2016-01-01'
start_time = '00-00-00.0'

#	length
days_record = 365
time_step = 30
len_record = int(86400 / time_step) * days_record + 1

#	output light curve data
lc_time_step = 1
max_length = 300
max_index = int(np.round(max_length / lc_time_step, 1)) + 1

##################################################
#	generate data file names

#	specific light curve id to simulate
specific_id = 0

#	site identifiers
latitude = 0
longitude = 0

#	object identifiers
inclination = 60

orbital_height = 500

#	range of raan values
raan_vals = np.arange(0, 180, 20)

#	initialise data files list
lightcurve_data_files = []

#	generate file names
for raan in raan_vals:

	#	generate strings
	site_str = 'lat' + str(latitude) + 'lon0'
	object_id = 'i' + str(inclination) + 'h' + str(orbital_height) + 'raan' + str(raan)
	
	lightcurve_data_file = '_'.join(
		[site_str, str(object_id), start_date, start_time, str(len_record), str(time_step)]
	)  + '.txt'

	lightcurve_data_files.append(lightcurve_data_file)

#	set lightcurve output directory
lightcurve_data_path = os.path.join(path_data, 'lightcurve_dir', 'data_files')

###################################################################################################
#	Initialise Figure
###################################################################################################

#	initialise polar figure
fig1 = plt.figure(figsize=(10, 6))
ax1 = plt.subplot(111, projection='polar')
ax1.set_title('Polar plot')

ax1.set_theta_zero_location("N")
ax1.set_theta_direction(-1)

ax1.set_rticks(np.arange(0, 100, 10))
ax1.set_rlim([0, 100])

# #	initialise phase figure
# fig2 = plt.figure(figsize=(10, 6))
# ax2 = plt.subplot(111)
# ax2.set_title('Phase Angles')

#	tick mark locators
years = mdates.YearLocator()   # every year
months = mdates.MonthLocator()  # every month
days = mdates.DayLocator()  # every day
hours = mdates.HourLocator()  # every hour
minutes = mdates.MinuteLocator()  # every minute
seconds = mdates.SecondLocator()  # every second

###################################################
#	plot coordinate axes

###################################################
#	x_axis
x_axis_inc = np.full([10], 0)
x_axis_azm = np.full(x_axis_inc.shape, 0)
marker_x = [len(x_axis_inc) - 1]

#	plot site positions
ax1.plot(
	np.radians(x_axis_azm),
	x_axis_inc,
	linewidth=1, color='k', markevery=marker_x, marker='x'
)

ax1.text(
	np.radians(225),
	20,
	'X-axis'
)

###################################################
#	y_axis
y_axis_inc = np.linspace(0, 90, 10)
y_axis_azm = np.full(x_axis_inc.shape, 90)

ax1.plot(
	np.radians(y_axis_azm),
	y_axis_inc,
	linewidth=1, color='k', markevery=marker_x, marker='>'
)

ax1.text(
	np.radians(100),
	90,
	'Y-axis'
)

###################################################
#	z_axis
z_axis_inc = np.linspace(0, 90, 10)
z_axis_azm = np.full(x_axis_inc.shape, 0)
ax1.plot(
	np.radians(z_axis_azm),
	z_axis_inc,
	linewidth=1, color='k', markevery=marker_x, marker='^'
)

ax1.text(
	np.radians(5),
	90,
	'Z-axis'
)

###################################################################################################
#	looping through provided data files
###################################################################################################

plotted_data = []

for lightcurve_data_file in lightcurve_data_files:

	###############################################################################################
	#	parse case parameters

	site_str, object_id, _, _, _, _ = \
		lightcurve_data_file.split('_')

	#	parse indentifier strings
	longitude = int(site_str.split('lon')[1])
	latitude = int(site_str.split('lon')[0][3:])

	inclination = int(object_id.split('h')[0][1:])
	orbital_height = int(object_id.split('raan')[0].split('h')[1])
	raan = int(object_id.split('raan')[1])

	print('\n#################################################################################')
	print('\n	Working on case: ')
	print('\n		Site:', site_str)
	print('			Latitude =		', latitude)
	print('			Longitude =		', longitude)

	print('\n		Object ID:', object_id)
	print('			Inclination = 		', inclination)
	print('			Orbital Height = 	', orbital_height)
	print('			Right Ascension =	', raan)

	###############################################################################################
	#	load extracted light curves data file

	#	load light curve file
	lightcurve_data_array = np.loadtxt(
		os.path.join(lightcurve_data_path, lightcurve_data_file),
		delimiter=','
	)

	#	parse generic time data
	start_jdate = lightcurve_data_array[0, 1]
	start_jday, start_jfrac = jday_jfrac(start_jdate)
	
	###############################################################################################
	#	Unique light curves 1
	
	#	unique light curve ids
	unique_ids = np.unique(lightcurve_data_array[:, 0])
	
	start_index = 0
	index_step = max_length + 1
	end_index = index_step
	
	#	initialise output array
	lightcurve_output_array = np.zeros([len(unique_ids) * end_index, 4])
	
	start_flag = True

	if lightcurve_data_array.shape == (3,3):
		print('\n	NO LIGHTCURVES!')

	else:

		#	set specific light curve ID if requested
		if specific_id != 0:
			unique_ids = [specific_id]

		print('\n		Looping through {} unique light curve IDs...'.format(len(unique_ids)))
		
		for light_curve_id in unique_ids:
			
			#######################################################################################
			#	parse input data
		
			#	ids of light curve data
			data_ids = np.where(lightcurve_data_array[:, 0]==light_curve_id)
			
			#	extract data
			light_curve_data = lightcurve_data_array[data_ids]

			#	generating input arrays
			jd_time_data = light_curve_data[:, 1]
			start_jdate = jd_time_data[0]
			stop_jdate = jd_time_data[-1]

			#	parse array
			sun_orb_data = light_curve_data[:, 8:11]
			earth_orb_data = light_curve_data[:, 11:14]
			site_orb_data = light_curve_data[:, 14:17]
			sat_orb_data = light_curve_data[:, 17:23]

			#	convert to light curve generation time step
			if lc_time_step != time_step:
			
				#	find nsteps
				prop_step = lc_time_step / (24 * 60 * 60)
				num_points = int(round((stop_jdate - start_jdate) / prop_step))
			
				#	new time range to interpolate to
				interp_range = np.linspace(start_jdate, stop_jdate, num_points + 1)

				#	current x spline
				spline_x = jd_time_data
			
				#	new spline x
				spline_x_new = interp_range
			
				#	generate data to interpolate
				interp_data_in = np.hstack((sat_orb_data, sun_orb_data, earth_orb_data, site_orb_data))
			
				#	initilise new y spline data
				interp_data_out = np.zeros((len(interp_range), len(interp_data_in[0])))
			
				#	perform interpolation	
				interp_data = interp(interp_data_in, interp_data_out, spline_x, spline_x_new)
			
				#	extract interpolated data
				jd_time_data = interp_range
				sat_orb_data = interp_data[:, 0:6]
				sun_orb_data = interp_data[:, 6:9]
				earth_orb_data = interp_data[:, 9:12]
				site_orb_data = interp_data[:, 12:15]
			
				#	reset data
				len_light_curve = len(jd_time_data)
				duration_sec = lc_time_step * (len_light_curve - 1)
				light_curve_sim_time = np.linspace(0, duration_sec, len_light_curve)

			#	extract light curve polar data
			sun_polar_data, site_polar_data, phase_angles, jd_time_data = extract_fun(
				jd_time_data, lc_time_step, time_step,
				sun_orb_data, earth_orb_data, site_orb_data, sat_orb_data
			)

			###########################################################################################
			#	generate date vectors
			datevec_array = []
			
			for line in jd_time_data:
				#	convert jd to date
				year, mon, day, hour, minute, sec = [*jd2date(*jday_jfrac(line))]
			
				#	turn into string
				datestr = '-'.join(['{:4d}'.format(year), '{:02d}'.format(mon), '{:02d}'.format(day)])
				timestr = ':'.join(['{:02d}'.format(hour), '{:02d}'.format(minute), '{:04.1f}'.format(sec)])
				datevec  = ' '.join([datestr, timestr])
			
				#	convert to datetime object
				date_time_obj = datetime.datetime.strptime(datevec, '%Y-%m-%d %H:%M:%S.%f')
			
				#	append  to output array
				datevec_array.append(date_time_obj)
			
			#	extract site inclination
			site_inc_data = site_polar_data[:, 1]
			min_site_inc = np.min(np.degrees(site_inc_data))

			if min_site_inc < 1:

				plotted_data.append([light_curve_id, raan])
				
				###################################################################################
				#	Graphical Outputs
				###################################################################################
				
				###################################################################################
				#	polar plot
				marker_sun = [len(sun_polar_data) - 1]
				marker_site = [len(site_polar_data) - 1]
				
				#	plot sun positions
				ax1.plot(
					sun_polar_data[:, 0],
					np.degrees(sun_polar_data[:, 1]),
					linewidth=0.5, color='r', markevery=marker_sun, marker='>'
				)
				
				#	plot site positions
				ax1.plot(
					site_polar_data[:, 0],
					np.degrees(site_polar_data[:, 1]),
					linewidth=0.5, color='g', markevery=marker_site, marker='<'
				)
	


print('\nPlotted Light Curves: ')
for line in plotted_data:
	print('\n	Right Ascension 	= ', int(line[1]))
	print('	Lightcurve ID 		= ', int(line[0]))

print('\nEND')
plt.show()
