###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	
#	Plot 3D visualisation of orbit/site geometry
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time
import datetime
from subprocess import call

#	numpy
import numpy as np

#	import from general function list
from functions.functions_list import date2jd, jd2date, jday_jfrac, eop_interpolate, get_times

#	import from general function list
from functions.functions_transform import site_vec

#	visualisation functions
from functions.glim_visualisation import extract_fun, gen_bands

#	matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.colors as colors

from mpl_toolkits import mplot3d
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.ticker import MultipleLocator
from matplotlib.ticker import AutoMinorLocator

###################################################################################################
#	inputs
###################################################################################################
#	clear terminal
call(['clear'])

print('#########################################################################################')
print('\n[{}] GLIMMA access generation script initilised.'.format(datetime.datetime.now()))

#	get simulation start time
sim_start = time.time()

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	set lightcurve output directory
lightcurve_path = os.path.join(path_data, 'lightcurve_dir', 'data_files')
		
###################################################################################################
#	simulation parameters
###################################################################################################

#	set output time step
lc_time_step = 1
		
##################################################
#	time date

#	start time
date_str = '2016-01-01'
time_str = '00-00-00.0'

#	length
days_record = 365
time_step = 60
len_record = int(86400 / time_step) * days_record + 1

#	duration
duration_sec = time_step * (len_record - 1)
duration_days = duration_sec / 86400

#	inclination values
inclination_flag = False

#	define inclination
inclination_range = np.arange(0, 90, 10)
inclination_val = [30]

#	define right ascension
raan_range = np.arange(0, 180, 20)

#	orbital altitude
orbital_height = 1400

##################################################
#	simulation site parameters

#	site identifier
site_latitude = 0
site_longitude = 0
site_str = 'lat{}lon{}'.format(site_latitude, site_longitude)

print('\n#####################################################################################')
print('\n	Time parameters: ')
print('		Simulation time step:		{}'.format(time_step))
print('		Duration:			{}		[steps]'.format(len_record))
print('						{}	[s]'.format(np.round(duration_sec, 1)))
print('						{}	[m]'.format(np.round(duration_sec / 60, 1)))
print('						{}		[days]'.format(np.round(duration_sec / 60 / 60 / 24, 1)))
print('\n	Site parameters: ')
print('		Site Str:			{}'.format(site_str))
print('		Site Latitude:			{}	deg'.format(site_latitude))
print('		Site Longitude:			{}	deg'.format(site_longitude))
print('\n#####################################################################################')

# ##################################################
# #	reset to include 100 so that 90 is plotted
# azm_vals = np.arange(0, 361, 1)
# elv_vals = np.arange(0, 91, 1)
# 
# #	generate polar mesh grid
# r, th = np.meshgrid(elv_vals, azm_vals)
# 
# ####################################################
# #	colour bounds
# cmap = colors.ListedColormap(['white', 'red', 'green', 'blue', 'cyan', 'magenta', 'yellow'])
# bounds = [0, 1, 2, 3, 4, 5, 6, 7]
# 	
# #	generate colour bar bounds
# norm = colors.BoundaryNorm(boundaries=bounds, ncolors=cmap.N)
# 	
# #	find azimuth offset to align with grid
# nazm_vals = len(th) - 1
# offset = (360 / nazm_vals) / 2
# 
# ####################################################
# #	initialise figure
# fig0 = plt.figure(figsize=(9.0, 9.0))
# 	
# ax0 = plt.subplot(111, projection="polar")
# ax0.set_theta_zero_location("N", offset=0)
# ax0.set_theta_direction(-1)
# 	
# ax0.set_title('Possible Vector Positions')
# 
# ax0.set_yticks(np.arange(0, 100, 10))
# ax0.set_ylim(0, 100)
# 
# ax0.set_xticks(np.arange(0, np.radians(360), np.radians(10)))
# ax0.set_xlim(0, np.pi * 2)

###################################################################################################
#	satellite inclination loop
###################################################################################################

print('\n[{}] Determining light curve file properties...'.format(datetime.datetime.now()))

##################################################
#	initialise of inclination data

#	sun azimuths
q1_band_az_sun_inclination = np.zeros([2, len(inclination_range)])
q2_band_az_sun_inclination = np.zeros([2, len(inclination_range)])
q3_band_az_sun_inclination = np.zeros([2, len(inclination_range)])
q4_band_az_sun_inclination = np.zeros([2, len(inclination_range)])

#	sun inclinations
band_inc_sun_inclination = np.zeros([2, len(inclination_range)])

#	site azimuth
band_az_site_inclination = np.zeros([2, len(inclination_range)])

#	site inclination
band_inc_site_inclination = np.zeros([2, len(inclination_range)])

#	maximum phase
avg_max_phase_inclination = np.zeros([len(inclination_range)])
std_max_phase_inclination = np.zeros([len(inclination_range)])

#	minimum phase
avg_min_phase_inclination = np.zeros([len(inclination_range)])
std_min_phase_inclination = np.zeros([len(inclination_range)])

#	light curve duration
avg_duration_inclination = np.zeros([len(inclination_range)])
std_duration_inclination = np.zeros([len(inclination_range)])

if inclination_flag is True:
	inclination_loop = inclination_val

elif inclination_flag is False:
	inclination_loop = inclination_range

for inclination_index, inclination in enumerate(inclination_loop):
	print('\n	Inclination: ', inclination)

#	#	initialise inclination step polar plot array
#	data_array = np.zeros([len(r[:, 0]) - 1, len(r[0, :]) - 1])
		
	###############################################################################################
	#	satellite RAAN loop
	###############################################################################################

	#	initialise sun azimuth data
	sun_az_min_raan = np.zeros([len(raan_range)])
	sun_az_max_raan = np.zeros([len(raan_range)])
	sun_az_med_raan = np.zeros([len(raan_range)])
	
	#	initialise sun inclination data
	sun_inc_min_raan = np.zeros([len(raan_range)])
	sun_inc_max_raan = np.zeros([len(raan_range)])
	sun_inc_med_raan = np.zeros([len(raan_range)])

	#	initialise site azimuth data
	site_az_min_raan = np.zeros([len(raan_range)])
	site_az_max_raan = np.zeros([len(raan_range)])
	site_az_med_raan = np.zeros([len(raan_range)])
	
	#	initialise site inclination data
	site_inc_min_raan = np.zeros([len(raan_range)])
	site_inc_max_raan = np.zeros([len(raan_range)])
	site_inc_med_raan = np.zeros([len(raan_range)])

	#	initialise of raan data
	avg_max_phase_raan = np.zeros([len(raan_range)])
	std_max_phase_raan = np.zeros([len(raan_range)])

	avg_min_phase_raan = np.zeros([len(raan_range)])
	std_min_phase_raan = np.zeros([len(raan_range)])

	avg_duration_raan = np.zeros([len(raan_range)])

	first_raan = True
	raan_id = 0

	for raan_index, raan in enumerate(raan_range):
		print('\n		RAAN: ', raan)
	
		#	input identifiers
		object_id = 'i' + str(inclination) + 'h' + str(orbital_height) + 'raan' + str(raan)
		
		#	generate file name
		lightcurve_data_file = '_'.join(
			[site_str, str(object_id), date_str, time_str, str(len_record), str(time_step)]
		)  + '.txt'
		
		##################################################
		#	load extracted light curves data file
		
		#	load light curve file
		lightcurve_data_array = np.loadtxt(
			os.path.join(lightcurve_path, lightcurve_data_file),
			delimiter=','
		)
		
		#	parse generic time data
		start_jdate = lightcurve_data_array[0, 1]
		start_jday, start_jfrac = jday_jfrac(start_jdate)
		
		###########################################################################################
		#	determine file properties
		
		#	extract unique light curve ids
		unique_ids = np.unique(lightcurve_data_array[:, 0])
		
		#	number of light curves
		n_lc = len(unique_ids)
		
		###########################################################################################
		#	initialise of light curve data

		#	initialise sun azimuth data
		sun_az_min_lc = np.zeros([n_lc])
		sun_az_max_lc = np.zeros([n_lc])
		sun_az_med_lc = np.zeros([n_lc])

		#	initialise sun inclination data
		sun_inc_min_lc = np.zeros([n_lc])
		sun_inc_max_lc = np.zeros([n_lc])
		sun_inc_med_lc = np.zeros([n_lc])
	
		#	initialise site azimuth data
		site_az_min_lc = np.zeros([n_lc])
		site_az_max_lc = np.zeros([n_lc])
		site_az_med_lc = np.zeros([n_lc])

		#	initialise site inclination data
		site_inc_min_lc = np.zeros([n_lc])
		site_inc_max_lc = np.zeros([n_lc])
		site_inc_med_lc = np.zeros([n_lc])
	
		#	initialise general data
		max_phase_lc = np.zeros([n_lc])
		min_phase_lc = np.zeros([n_lc])
		duration_lc = np.zeros([n_lc])

		#	set starting index
		output_index = 0
		
		###########################################################################################
		#	unique light curve id loop
		print('			Looping through {} unique light curve IDs...'.format(n_lc))
		first_lc = True

		for lc_index, lc_id in enumerate(unique_ids):
		
			#	ids of light curve data
			data_ids = np.where(lightcurve_data_array[:, 0]==lc_id)
			
			#	extract data
			light_curve_data = lightcurve_data_array[data_ids]
		
			##################################################
			#	parse light curve array
			jd_time_data = light_curve_data[:, 1]
			sun_r_data = light_curve_data[:, 2]
			sun_az_data = light_curve_data[:, 3]
			sun_el_data = light_curve_data[:, 4]
			sat_r_data = light_curve_data[:, 5]
			sat_az_data = light_curve_data[:, 6]
			sat_el_data = light_curve_data[:, 7]
			sun_orb_data = light_curve_data[:, 8:11]
			earth_orb_data = light_curve_data[:, 11:14]
			site_orb_data = light_curve_data[:, 14:17]
			sat_orb_data = light_curve_data[:, 17:23]
			sat_itrf_data = light_curve_data[:, 23:29]
		
			##################################################
			#	extract light curve polar data
			sun_polar_data, site_polar_data, phase_angles, jd_time_data = extract_fun(
				jd_time_data, lc_time_step, time_step,
				sun_orb_data, earth_orb_data, site_orb_data, sat_orb_data
			)
		
			##################################################
			#	extract angle data
			site_azimuth_data = np.degrees(site_polar_data[:, 0])
			site_inclination_data = np.degrees(site_polar_data[:, 1])

			sun_azimuth_data = np.degrees(sun_polar_data[:, 0])
			sun_inclination_data = np.degrees(sun_polar_data[:, 1])
		
			if first_lc is True:
				#	generate output arrays
				sun_azimuth_data_lc = sun_azimuth_data
				sun_inclination_data_lc = sun_inclination_data

				first_lc = False

			elif first_lc is False:

				#	stack output arrays
				sun_azimuth_data_lc = np.hstack([sun_azimuth_data_lc, sun_azimuth_data])
				sun_inclination_data_lc = np.hstack([sun_inclination_data_lc, sun_inclination_data])

			##################################################
			#	light  curve duration
			duration = np.round(86400 * (jd_time_data[-1] - jd_time_data[0]), 1)

			#	add to duration of lc data
			duration_lc[lc_index] = duration
		
			##################################################
			#	light curve phase data
			min_phase_lc[lc_index] = np.degrees(phase_angles).min()
			max_phase_lc[lc_index] = np.degrees(phase_angles).max()

			##################################################
			#	calculate sun vector properties
		
			#	azimuth
			sun_az_min_lc[lc_index] = sun_azimuth_data.min()
			sun_az_max_lc[lc_index] = sun_azimuth_data.max()
			sun_az_med_lc[lc_index] = np.median(sun_azimuth_data)

			#	inclination
			sun_inc_min_lc[lc_index] = sun_inclination_data.min()
			sun_inc_max_lc[lc_index] = sun_inclination_data.max()
			sun_inc_med_lc[lc_index] = np.median(sun_inclination_data)

			##################################################
			#	calculate site vector properties

			#	azimuth
			site_az_min_lc[lc_index] = site_azimuth_data.min()
			site_az_max_lc[lc_index] = site_azimuth_data.max()
			site_az_med_lc[lc_index] = np.median(site_azimuth_data)

			#	inclination
			site_inc_min_lc[lc_index] = site_inclination_data.min()
			site_inc_max_lc[lc_index] = site_inclination_data.max()
			site_inc_med_lc[lc_index] = np.median(site_inclination_data)

#		###############################################################################################
#		#	of LC plot
#		###############################################################################################
#	
#		fig_phase_lc, ax_phase_lc = plt.subplots()
#		ax_phase_lc.set_title('Light curve properties')
#	
#		ax_phase_lc.plot(unique_ids, site_az_max_lc, label='Maximum', color='tab:red', linestyle='--')
#		ax_phase_lc.plot(unique_ids, site_az_min_lc, label='Minimum', color='tab:blue', linestyle='--')
#	
#		#	finalise
#		ax_phase_lc.set_xlabel('Light Curve ID')
#		ax_phase_lc.set_ylabel('Phase Angle / degrees')
#		
#		plt.show()
#		sys.exit()

		###########################################################################################
		#	add to of raan data

		if first_raan is True:
			#	generate output arrays
			sun_azimuth_data_raan = sun_azimuth_data_lc
			sun_inclination_data_raan = sun_inclination_data_lc

			first_raan = False

		elif first_raan is False:
			#	stack output arrays
			sun_azimuth_data_raan = np.hstack([sun_azimuth_data_raan, sun_azimuth_data_lc])
			sun_inclination_data_raan = np.hstack([sun_inclination_data_raan, sun_inclination_data_lc])

		#	initialise sun azimuth data
		sun_az_min_raan[raan_id] = sun_az_min_lc.min()
		sun_az_max_raan[raan_id] = sun_az_max_lc.max()
		
		#	initialise sun inclination data
		sun_inc_min_raan[raan_id] = sun_inc_min_lc.min()
		sun_inc_max_raan[raan_id] = sun_inc_max_lc.max()

		#	initialise site azimuth data
		site_az_min_raan[raan_id] = site_az_min_lc.min()
		site_az_max_raan[raan_id] = site_az_max_lc.max()
		
		#	initialise site inclination data
		site_inc_min_raan[raan_id] = site_inc_min_lc.min()
		site_inc_max_raan[raan_id] = site_inc_max_lc.max()

		#	phase data
		avg_max_phase_raan[raan_id] = np.average(max_phase_lc)
		std_max_phase_raan[raan_id] = np.std(max_phase_lc)

		avg_min_phase_raan[raan_id] = np.average(min_phase_lc)
		std_min_phase_raan[raan_id] = np.std(min_phase_lc)

		#	duration
		avg_duration_raan[raan_id] = np.average(duration_lc)

		raan_id += 1

#	###############################################################################################
#	#	of RAAN plot
#	###############################################################################################
#
#	fig_phase_raan, ax_phase_raan = plt.subplots()
#	ax_phase_raan.set_title('Maximum Phase Angle')
#
#	#	max data
#	ax_phase_raan.plot(raan_range, sun_az_max_raan, label='Average Maximum', color='k', linestyle='--')
#	ax_phase_raan.plot(raan_range, sun_az_min_raan, label='Minimum', color='k', linestyle='--')
#
#	ax_phase_raan.fill_between(
#		raan_range,
#		np.subtract(avg_max_phase_raan, std_max_phase_raan),
#		np.add(avg_max_phase_raan, std_max_phase_raan)
#	)
#
#	ax_phase_raan.fill_between(
#		raan_range,
#		np.subtract(avg_min_phase_raan, std_min_phase_raan),
#		np.add(avg_min_phase_raan, std_min_phase_raan)
#	)
#
#	#	finalise
#	ax_phase_raan.set_xlabel('Right Ascension / deg')
#	ax_phase_raan.set_ylabel('Phase Angle / degrees')
#	ax_phase_raan.set_ylim([0, 90])

	###############################################################################################
	#	generating allowed bands of inclination
	###############################################################################################

	##################################################
	#	sun azimuths

	#	stack and sort output data
	sun_az_vals_inclination = np.sort(sun_azimuth_data_raan)

	#	generate bands
	quad1_band, quad2_band, quad3_band, quad4_band = gen_bands(sun_az_vals_inclination)

	#	add quadrent lines to output data arrays
	q1_band_az_sun_inclination[:, inclination_index] = quad1_band
	q2_band_az_sun_inclination[:, inclination_index] = quad2_band 
	q3_band_az_sun_inclination[:, inclination_index] = quad3_band 
	q4_band_az_sun_inclination[:, inclination_index] = quad4_band 

	##################################################
	#	sun inclinations

	#	stack and sort output data
	sun_inc_vals_inclination = np.sort(np.hstack([sun_inc_min_raan, sun_inc_max_raan]))

	#	generate sun inclination band
	band_inc_sun_inclination[:, inclination_index] = np.array([
		sun_inc_vals_inclination[0],
		sun_inc_vals_inclination[-1]]
	)

	##################################################
	#	site azimuths
	min_site_az_inclination = site_az_min_raan.min()
	max_site_az_inclination = site_az_max_raan.max()

	#	generate site azimuth bands
	band_az_site_inclination[:, inclination_index] = np.array([
		min_site_az_inclination,
		max_site_az_inclination
	])

	##################################################
	#	site inclinations

	#	stack and sort output data
	site_inc_vals_inclination = np.sort(np.hstack([site_inc_min_raan, site_inc_max_raan]))

	#	generate site inclination bands
	band_inc_site_inclination[:, inclination_index] = np.array([
		site_inc_vals_inclination[0],
		site_inc_vals_inclination[-1]
	])

	##################################################
	#	phase data
	avg_max_phase_inclination[inclination_index] = np.average(avg_max_phase_raan)
	std_max_phase_inclination[inclination_index] = np.std(avg_max_phase_raan)

	avg_min_phase_inclination[inclination_index] = np.average(avg_min_phase_raan)
	std_min_phase_inclination[inclination_index] = np.std(avg_min_phase_raan)

	#	duration
	avg_duration_inclination[inclination_index] = np.average(avg_duration_raan)
	std_duration_inclination[inclination_index] = np.std(avg_duration_raan)





	###############################################################################################
#	#	polar plot
#
#	#	colour the plot
#	ax0.pcolor(np.radians(th), r, data_array, norm=norm, cmap=cmap)
#	ax0.grid()














# ###############################################################################################
# #	Inclination plot
# fig_phase_inclination, ax_phase_inclination = plt.subplots()
# ax_phase_inclination.set_title('Maximum Phase Angle')
# 
# #	max data
# ax_phase_inclination.plot(
# 	inclination_range,
# 	avg_max_phase_inclination,
# 	label='Average Maximum',
# 	color='k', linestyle='--'
# )
# 
# ax_phase_inclination.fill_between(
# 	inclination_range,
# 	np.subtract(avg_max_phase_inclination, std_max_phase_inclination),
# 	np.add(avg_max_phase_inclination, std_max_phase_inclination)
# )
# 
# #	min data
# ax_phase_inclination.plot(
# 	inclination_range,
# 	avg_min_phase_inclination,
# 	label='Average Minimum',
# 	color='k', linestyle='--'
# )
# 
# ax_phase_inclination.fill_between(
# 	inclination_range,
# 	np.subtract(avg_min_phase_inclination, std_min_phase_inclination),
# 	np.add(avg_min_phase_inclination, std_min_phase_inclination)
# )
# 
# #	finalise
# ax_phase_inclination.set_xlabel('Right Ascension / deg')
# ax_phase_inclination.set_ylabel('Phase Angle / degrees')
# ax_phase_inclination.set_ylim([0, 90])





###################################################################################################
#	output figure
###################################################################################################

print('\nPlotting output Figure...')

#	RAAN plot
fig1, ((ax1, ax2), (ax3, ax4), (ax5, ax6)) = plt.subplots(3, 2, figsize=(15, 10))

hspace_val = 0.35
wspace_val = 0.35
plt.subplots_adjust(hspace=hspace_val, wspace=wspace_val)

fig1.suptitle('Properties of a year of Light Curves at Site Latitude = {}'.format(site_latitude))

###################################################################################################
#	duration data
ax1.set_title('Average Light Curve Duration')

#	max data
ax1.plot(
	inclination_range, avg_duration_inclination,
	label='Average Maximum', color='k', linestyle='--'
)
#	max std
ax1.fill_between(
	inclination_range,
	np.subtract(avg_duration_inclination, std_duration_inclination),
	np.add(avg_duration_inclination, std_duration_inclination)
)

ax1.set_xlabel('Satellite Orbit Inclination / deg')
ax1.set_ylabel('Duration / s')
ax1.xaxis.set_minor_locator(MultipleLocator(1))
ax1.grid(which='both')

###################################################################################################
#	phase data
ax2.set_title('Maximum Phase Angle')

#	max data
ax2.plot(
	inclination_range, avg_max_phase_inclination,
	label='Average Maximum', color='k', linestyle='--'
)
#	max std
ax2.fill_between(
	inclination_range,
	np.subtract(avg_max_phase_inclination, std_max_phase_inclination),
	np.add(avg_max_phase_inclination, std_max_phase_inclination)
)
#	min data
ax2.plot(
	inclination_range, avg_min_phase_inclination,
	label='Average Minimum', color='k', linestyle='--'
)
#	min std
ax2.fill_between(
	inclination_range,
	np.subtract(avg_min_phase_inclination, std_min_phase_inclination),
	np.add(avg_min_phase_inclination, std_min_phase_inclination)
)

ax2.set_xlabel('Satellite Orbit Inclination / deg')
ax2.set_ylabel('Phase Angle / degrees')
ax2.set_ylim([0, 90])

ax2.xaxis.set_minor_locator(MultipleLocator(1))
ax2.grid(which='both')
ax2.legend()

###################################################################################################
#	sun azimuth data
ax3.set_title('Possible Sun Azimuths')
ax3.fill_between(
	inclination_range,
	q1_band_az_sun_inclination[0, :],
	q1_band_az_sun_inclination[1, :],
	color='r'
)

ax3.fill_between(
	inclination_range,
	q2_band_az_sun_inclination[0, :],
	q2_band_az_sun_inclination[1, :],
	color='r'
)

ax3.fill_between(
	inclination_range,
	q3_band_az_sun_inclination[0, :],
	q3_band_az_sun_inclination[1, :],
	color='r'
)

ax3.fill_between(
	inclination_range,
	q4_band_az_sun_inclination[0, :],
	q4_band_az_sun_inclination[1, :],
	color='r'
)

ax3.set_xlabel('Satellite Orbit Inclination')
ax3.set_ylabel('Azimuth Angle / degrees')
ax3.set_ylim([0, 360])

ax3.xaxis.set_minor_locator(MultipleLocator(1))
ax3.grid(which='both')
#ax3.legend()

###################################################################################################
#	sun inclination data
ax4.set_title('Possible Sun Inclinations')
ax4.fill_between(
	inclination_range,
	band_inc_sun_inclination[0, :],
	band_inc_sun_inclination[1, :],
	color='r'
)

ax4.set_xlabel('Satellite Orbit Inclination')
ax4.set_ylabel('Inclination Angle / degrees')
ax4.set_ylim([0, 90])

ax4.xaxis.set_minor_locator(MultipleLocator(1))
ax4.grid(which='both')
#ax4.legend()

###################################################################################################
#	site azimuth data
ax5.set_title('Possible Site Azimuths')
ax5.fill_between(
	inclination_range,
	band_az_site_inclination[0, :],
	band_az_site_inclination[1, :],
	color='g'
)

ax5.set_xlabel('Satellite Orbit Inclination')
ax5.set_ylabel('Azimuth Angle / degrees')
ax5.set_ylim([0, 360])

ax5.xaxis.set_minor_locator(MultipleLocator(1))
ax5.grid(which='both')
#ax5.legend()

###################################################################################################
#	site inclination data
ax6.set_title('Possible Site Inclinations')
ax6.fill_between(
	inclination_range,
	band_inc_site_inclination[0, :],
	band_inc_site_inclination[1, :],
	color='g'
)

ax6.set_xlabel('Satellite Orbit Inclination')
ax6.set_ylabel('Inclination Angle / degrees')
ax6.set_ylim([0, 90])

ax6.xaxis.set_minor_locator(MultipleLocator(1))
ax6.grid(which='both')
#ax6.legend()

###################################################################################################

#	calculate sim time
sim_end = time.time()
sim_time = np.round((sim_end - sim_start) / 60, 1)

print('\n[{}] Simulation Finished.'.format(datetime.datetime.now()))
print('\n	- Time taken = {} mins.'.format(sim_time))

plt.show()
sys.exit()


#			##################################################
#			#	add positions to polar ourput array
#			for sun_polar_line, site_polar_line in zip(sun_polar_data, site_polar_data):
#		
#				##################################################
#				#	sun
#				#	create rounded values
#				azm_rounded = np.array([int(np.floor(np.degrees(sun_polar_line[0])))])
#				inc_rounded = np.array([int(np.floor(np.degrees(sun_polar_line[1])))])
#		
#				#	find coordinates
#				row_coord = np.where(th[:, 0]==azm_rounded)[0]
#				col_coord = np.where(r[0, :]==inc_rounded)[0]
#		
#				#	set corrosponding pixel to 1
#				data_array[row_coord, col_coord] = 1
#		
#				##################################################
#				#	site
#				#	create rounded values
#				azm_rounded = np.array([int(np.floor(np.degrees(site_polar_line[0])))])
#				inc_rounded = np.array([int(np.floor(np.degrees(site_polar_line[1])))])
#		
#				#	find coordinates
#				row_coord = np.where(th[:, 0]==azm_rounded)[0]
#				col_coord = np.where(r[0, :]==inc_rounded)[0]
#		
#				#	set corrosponding pixel to 2
#				data_array[row_coord, col_coord] = 2





plt.show()
sys.exit()

