###################################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
###################################################################################################
#	geometry file plotter
#
#
#
#
#
#
###################################################################################################

#	standard packages
import os
import sys
import time

#	numpy
import numpy as np

#	numpy-stl
from stl import mesh

from functions.functions_transform import rot_mat

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits import mplot3d

from matplotlib.collections import PolyCollection
from matplotlib import colors as mcolors
from matplotlib.patches import FancyArrowPatch

###################################################################################################
#	define plotable arrow class
###################################################################################################

class Arrow3D(FancyArrowPatch):
    def __init__(self, xs, ys, zs, *args, **kwargs):
        FancyArrowPatch.__init__(self, (0,0), (0,0), *args, **kwargs)
        self._verts3d = xs, ys, zs

    def draw(self, renderer):
        xs3d, ys3d, zs3d = self._verts3d
        xs, ys, zs = mplot3d.proj3d.proj_transform(xs3d, ys3d, zs3d, renderer.M)
        self.set_positions((xs[0],ys[0]),(xs[1],ys[1]))
        FancyArrowPatch.draw(self, renderer)

###################################################################################################
#	initilisation
###################################################################################################

#############################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output file path
fig_path = os.path.join(
	os.environ['HOME'],
	'PhD',
	'Documents',
	'Articles',
	'starlink_paper',
	'Figures'
)

fig_name = 'starlink_geom.eps'
out_path = os.path.join(fig_path, fig_name)

#	set flag
savefile_flag = True
savefile_flag = False

###################################################################################################
#	geometry file
###################################################################################################

geom_dir = 'geometry_dir'
geom_dir = 'geometry_dir/geom_dir/' + 'CZ-2D'
geom_file = 'cz-2d_testing_double_pointed.stl'

geom_dir = 'geometry_dir/geom_dir/' + 'starlink'
geom_file = 'starlink_full.stl'

#	brdf
top_faces_index = np.arange(0, 36)
side_faces_index = np.arange(36, 108)
bot_faces_index = np.arange(108, 144)

###################################################################################################
#	figure parameters
###################################################################################################

#	face plot parameters
alpha = 1.0

facecol = [0.1, 0.1, 0.1]
edgecol = [0.5, 0.5, 0.5]

#	graphical output
inch2cm = 2.54

font_sizex = 1
font_sizey = 1
font_pad = 0.5
marker_size = 4
marker_edge = 1

axis_lim0 = 8
axis_lim1 = 3

text_fsize = 10

plt.rc('text', usetex=True)

#	initialise figure axis
fig, ax = plt.subplots(
	1, 2,
	figsize=(18/inch2cm, 9/inch2cm),
	dpi=300,
	subplot_kw=dict(projection='3d'),
)

# plt.subplots_adjust(wspace=0.3)

###################################################################################################
#	plot geometry
###################################################################################################

#	load mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
geom_offset_x = 0.0
geom_offset_y = 180.0
init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

object_mesh.rotate_using_matrix(offset_rot_mat)

vectors = object_mesh.vectors
normals = object_mesh.normals
nfaces = len(normals)

#	facet loop
for i, (vector, normal) in enumerate(zip(vectors, normals)):

	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]
	normal = np.cross(vect12, vect23)
	normal_unit = normal / np.linalg.norm(normal)

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_alpha(alpha)

#	pc.set_edgecolor(edgecol)
#	pc.set_facecolor('tab:blue')

	pc.set_facecolor('tab:grey')
	pc.set_edgecolor('k')

#	if i in top_faces_index:
#		pc.set_facecolor('tab:red')
#	elif i in side_faces_index:
#		pc.set_facecolor('tab:green')
#	elif i in bot_faces_index:
#		pc.set_facecolor('tab:blue')

	ax[0].add_collection3d(pc)

#############################################
#	plot body axes

body_axis_scale = 15
linewidth_val = 1

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

bodyx_vec_plot = body_axis_scale * np.vstack(([0, 0, 0], x_body_unit))
bodyy_vec_plot = body_axis_scale * np.vstack(([0, 0, 0], y_body_unit))
bodyz_vec_plot = body_axis_scale * np.vstack(([0, 0, 0], z_body_unit))

#	body x axis
c = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
	mutation_scale=5, 
	lw=linewidth_val,
	arrowstyle="-|>",
	color="k"
)
ax[0].add_artist(c)

#	body y axis
d = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
	mutation_scale=5, 
	lw=linewidth_val,
	arrowstyle="-|>",
	color="k"
)
ax[0].add_artist(d)

#	body z axis
e = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
	mutation_scale=5,
	lw=linewidth_val,
	arrowstyle="-|>",
	color="k"
)
ax[0].add_artist(e)

#############################################
#	plot body axes text

# bodyx_vec_plot /= 2 
# bodyy_vec_plot /= 2 
# bodyz_vec_plot /= 2 

ax[0].text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1] - 0.1, bodyx_vec_plot[1, 2] + 1,
	r'$x_{b}$',
	fontsize=text_fsize,
	fontweight='bold'
)

ax[0].text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	r'$y_{b}$',
	fontsize=text_fsize,
	fontweight='bold'
)

ax[0].text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	r'$z_{b}$',
	fontsize=text_fsize,
	fontweight='bold'
)

#	set viewing direction
ax[0].view_init(22, 45)

#	finalise subplot 
ax[0].set_xlim([-axis_lim0, axis_lim0])
ax[0].set_ylim([-axis_lim0, axis_lim0])
ax[0].set_zlim([-axis_lim0, axis_lim0])
ax[0].set_xlabel('x axis')
ax[0].set_ylabel('y axis')
ax[0].set_zlabel('z axis')


#	turn off axes
ax[0].axis('off')


###################################################################################################
#	plot geometry
###################################################################################################

geom_file = 'starlink_payload.stl'

#	load mesh
object_mesh = mesh.Mesh.from_file(os.path.join(path_data, geom_dir, geom_file))

#	apply offset to rotate to body frame
geom_offset_x = 0.0
geom_offset_y = 180.0

init_offset = np.array(np.radians([geom_offset_x, geom_offset_y, 0.0]))
offset_rot_mat = rot_mat(init_offset)

object_mesh.rotate_using_matrix(offset_rot_mat)

vectors = object_mesh.vectors
normals = object_mesh.normals
nfaces = len(normals)

#	facet loop
for i, (vector, normal) in enumerate(zip(vectors, normals)):

	vect12 = vector[1] - vector[0]
	vect23 = vector[2] - vector[1]
	normal = np.cross(vect12, vect23)
	normal_unit = normal / np.linalg.norm(normal)

	#	Add final vert
	vector = np.vstack([vector, vector[0, :]])

	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	verts = [list(zip(xs, ys, zs))]

	#	Graphics Output

	#	face
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=0.5)
	pc.set_alpha(alpha)
#	pc.set_edgecolor(edgecol)

	pc.set_facecolor('tab:grey')
	pc.set_edgecolor('k')

#	if i in top_faces_index:
#		pc.set_facecolor('tab:red')
#	elif i in side_faces_index:
#		pc.set_facecolor('tab:green')
#	elif i in bot_faces_index:
#		pc.set_facecolor('tab:blue')

	ax[1].add_collection3d(pc)

#############################################
#	plot body axes

body_axis_scale = 5
linewidth_val = 1

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

bodyx_vec_plot = body_axis_scale * np.vstack(([0, 0, 0], x_body_unit))
bodyy_vec_plot = body_axis_scale * np.vstack(([0, 0, 0], y_body_unit))
bodyz_vec_plot = body_axis_scale * np.vstack(([0, 0, 0], z_body_unit))

#	body x axis
c = Arrow3D(bodyx_vec_plot[:, 0], bodyx_vec_plot[:, 1], bodyx_vec_plot[:, 2],
	mutation_scale=5, 
	lw=linewidth_val,
	arrowstyle="-|>",
	color="k"
)
ax[1].add_artist(c)

#	body y axis
d = Arrow3D(bodyy_vec_plot[:, 0], bodyy_vec_plot[:, 1], bodyy_vec_plot[:, 2],
	mutation_scale=5, 
	lw=linewidth_val,
	arrowstyle="-|>",
	color="k"
)
ax[1].add_artist(d)

#	body z axis
e = Arrow3D(bodyz_vec_plot[:, 0], bodyz_vec_plot[:, 1], bodyz_vec_plot[:, 2],
	mutation_scale=5,
	lw=linewidth_val,
	arrowstyle="-|>",
	color="k"
)
ax[1].add_artist(e)

#############################################
#	plot body axes text

# bodyx_vec_plot /= 2 
# bodyy_vec_plot /= 2 
# bodyz_vec_plot /= 2 

ax[1].text(bodyx_vec_plot[1, 0], bodyx_vec_plot[1, 1] - 0.1, bodyx_vec_plot[1, 2] + 1,
	r'$x_{b}$',
	fontsize=text_fsize,
	fontweight='bold'
)

ax[1].text(bodyy_vec_plot[1, 0], bodyy_vec_plot[1, 1], bodyy_vec_plot[1, 2],
	r'$y_{b}$',
	fontsize=text_fsize,
	fontweight='bold'
)

ax[1].text(bodyz_vec_plot[1, 0], bodyz_vec_plot[1, 1], bodyz_vec_plot[1, 2],
	r'$z_{b}$',
	fontsize=text_fsize,
	fontweight='bold'
)

#	set viewing direction
ax[1].view_init(22, 45)

#	finalise subplot 
ax[1].set_xlim([-axis_lim1, axis_lim1])
ax[1].set_ylim([-axis_lim1, axis_lim1])
ax[1].set_zlim([-axis_lim1, axis_lim1])
ax[1].set_xlabel('x axis')
ax[1].set_ylabel('y axis')
ax[1].set_zlabel('z axis')



#	turn off axes
ax[1].axis('off')

if savefile_flag is True:
	plt.savefig(out_path, bbox_inches="tight", dpi=300)
	print('\n#######################################')
	print('Figure saved: ', fig_name)

plt.show()
