#############################################################################################
#  - Acquire Horizons data files
#  - Interpolate for required sample rate
#############################################################################################
import os
import csv
import sys
import numpy as np
import urllib.request
import shutil
import time

from scipy import interpolate
from function_list import date2jd, jd2date, get_iers, get_times
#############################################################################################


# request_horz

def request_horz(object_data, horz_fpath, horz_spath):

	# unpack object data
	norad_id = object_data.norad_ident
	jday_start = object_data.jday_utc_start
	jfrac_start = object_data.jfrac_utc_start
	jday_stop = object_data.jday_utc_stop
	jfrac_stop = object_data.jfrac_utc_stop

	# convert julian to utc date
	year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)
	date_vec1 = [year1, mon1, day1, hour1, min1, sec1]

	# create date string
	date_str = '{:4}-{:02}-{:02}_{:02}-{:02}-{:1.4f}'.format(*date_vec1)

	# propagation duration
	duration = int(86400 * ((jday_stop + jfrac_stop) - (jday_start + jfrac_start)))
	horz_step = duration

	# get simulation end time
	jd_stop = jday_start + jfrac_start + (duration / 86400)
	jfrac_horzstop = jd_stop - np.floor(jd_stop)

	if jfrac_horzstop >= 0.5:
		jday_horzstop = np.floor(jd_stop) + 0.5
		jfrac_horzstop = jfrac_horzstop - 0.5
	else:
		jday_horzstop = np.floor(jd_stop) - 0.5
		jfrac_horzstop = jfrac_horzstop + 0.5

	year2, mon2, day2, hour2, min2, sec2 = jd2date(jday_horzstop, jfrac_horzstop)
	date_vec2 = [year2, mon2, day2, hour2, min2, sec2]

	# Acquire IERS EOPs
	dut, dutps, _, _, _, _, _, _, _, _, _, utc, dat = get_iers(jday_start, jfrac_start)

	# Perform dut interpolation
	dut = dut + (utc * dutps)

	# Calculate TCB times for Horizons querey
	# create time object
	time_object = get_times(*date_vec1, dut, dat)

	# Start time
	jday_tdb_start = time_object.jday_tdb
	jfrac_tdb_start = time_object.jfrac_tdb

	# Convert to date
	year1_tdb, mon1_tdb, day1_tdb, hour1_tdb, min1_tdb, sec1_tdb = jd2date(
		jday_tdb_start,
		jfrac_tdb_start
	)

	# create time object
	time_object = get_times(*date_vec2, dut, dat)

	# Stop time
	jday_tdb = time_object.jday_tdb
	jfrac_tdb = time_object.jfrac_tdb

	# Convert to date
	year2_tdb, mon2_tdb, day2_tdb, hour2_tdb, min2_tdb, sec2_tdb = jd2date(
		jday_tdb,
		jfrac_tdb
	)

	tdb_vec1 = [year1_tdb, mon1_tdb, day1_tdb, hour1_tdb, min1_tdb, sec1_tdb]
	tdb_vec2 = [year2_tdb, mon2_tdb, day2_tdb, hour2_tdb, min2_tdb, sec2_tdb]

	# Request input strings
	start_str = '{:4}-{:02}-{:02}%20{:02}:{:02}:{:1.4f}'.format(*tdb_vec1)
	stop_str = '{:4}-{:02}-{:02}%20{:02}:{:02}:{:1.4f}'.format(*tdb_vec2)

	# central body (0 = barycenter)
	center = 0
	# target bodies (10 = sun, 399 = earth)
	target_ids = np.array([10, 399])

	filenames = []
	for target_id in target_ids:

		# horz source file name
		horz_source_file = '{:d}_{:d}@{:d}_{:s}.horiz'.format(
			norad_id,
			target_id,
			center,
			date_str
		)

		# check for file existance
		if os.path.isfile(os.path.join(horz_spath, horz_source_file)):

			print('	Horizons source file exists')
			filenames.append(horz_source_file)

		else:

			print('	Requesting source file: {:s}'.format(horz_source_file))

			query = [
				"ssd.jpl.nasa.gov/"
				"horizons_batch.cgi?batch=1"
				"&COMMAND='" + str(target_id) + "'",
				"&CENTER='@" + str(center) + "'",
				"&MAKE_EPHEM='YES'",
				"&TABLE_TYPE='VECTORS'",
				"&START_TIME='" + start_str + "'",
				"&STOP_TIME='" + stop_str + "'",
				"&STEP_SIZE='" + str(horz_step) + "'",
				"&OUT_UNITS='KM-s'",
				"&REF_PLANE='FRAME'",
				"&REF_SYSTEM='J2000'",
				"&VECT_CORR='NONE'",
				"&VEC_LABLES='YES'",
				"&VEC_DELTA_T='NO'",
				"&CSV_FORMAT='NO'",
				"&OBJ_DATA='YES'",
				"&VEC_TABLE='2'"
			]
			query_str = "".join(query)

			# horz url
			url = ''.join(('https://', "".join(query)))

			# acquire file
			open_path = os.path.join(horz_spath, horz_source_file)

			with urllib.request.urlopen(url) as response, open(open_path, 'wb') as out_file:
				shutil.copyfileobj(response, out_file)

			filenames.append(horz_source_file)

			print('	File Aquired.')
			print('== > waiting 5 seconds ...')
			time.sleep(5)

	# return state date in tdb for make_horz
	jdate_tdb_start = jday_tdb_start + jfrac_tdb_start

	return filenames, jdate_tdb_start

#############################################################################################


def make_horz(filenames, fname_horz,  horz_fpath, horz_spath, jdate_tdb_start):

	# initilise
	ephem_data = []
	temp_flag = 0
	intere = 0

	# open files and extract ephemeris
	for file in filenames:
		with open(os.path.join(horz_spath, file)) as source_file:
			# get ephemeris data
			temp_data = []
			for line in source_file:
				if line[0:5] == '$$EOE':
					temp_flag = 0
					break
				elif temp_flag == 1:
					temp_data.append(line)
				elif line[0:5] == '$$SOE':
					temp_flag = 1
			ephem_data.append(temp_data)

	# extract data elements
	ephem_data = np.array(ephem_data)
	len_data = len(ephem_data[0])
	data_points = len(ephem_data[0]) / 3

	# target1 body date line
	t1_line1 = ephem_data[0, np.arange(0, len_data - 2, 3)]
	# target1 body position line
	t1_line2 = ephem_data[0, np.arange(1, len_data - 1, 3)]
	# target1 body velocity line
	t1_line3 = ephem_data[0, np.arange(2, len_data, 3)]

	# target2 body date line
	t2_line1 = ephem_data[1, np.arange(0, len_data - 2, 3)]
	# target2 body position line
	t2_line2 = ephem_data[1, np.arange(1, len_data - 1, 3)]
	# target2 body velocity line
	t2_line3 = ephem_data[1, np.arange(2, len_data, 3)]

	# combine data
	string_data = np.transpose(
		np.vstack([t1_line1, t1_line2, t1_line3, t2_line2, t2_line3])
	)

	# initilize outputs
	t1_pos = np.zeros((int(data_points), 3))
	t2_pos = np.zeros((int(data_points), 3))
	temp_sec_since = np.zeros((int(data_points), 1))

	for sim_seconds, row in enumerate(string_data):
		jdate_tdb = np.float(row[0].split(' ', maxsplit=1)[0])

		temp_sec_since[sim_seconds] = np.round(
			86400 * ((jdate_tdb - jdate_tdb_start))
		)

		temp = row[1].split('Z')
		temp2 = temp[0].split('Y')
		temp3 = temp2[0].split('=')

		t1_pos[sim_seconds, 0] = np.float(temp3[1])
		t1_pos[sim_seconds, 1] = np.float(temp2[1].split('=')[1])
		t1_pos[sim_seconds, 2] = np.float(temp[1].split('=')[1])

		temp = row[3].split('Z')
		temp2 = temp[0].split('Y')
		temp3 = temp2[0].split('=')

		t2_pos[sim_seconds, 0] = np.float(temp3[1])
		t2_pos[sim_seconds, 1] = np.float(temp2[1].split('=')[1])
		t2_pos[sim_seconds, 2] = np.float(temp[1].split('=')[1])


	# uninterpolated data
	celestial_data = np.hstack([temp_sec_since, t1_pos, t2_pos])

	# interpolated celestial data
	sec_since = np.linspace(0, sim_seconds, sim_seconds + 1)
	sim_time = np.linspace(0, sim_seconds, 10 * (sim_seconds) + 1)

	# Spline Interpolation
	# current x spline
	spline_x = sec_since
	# new spline x
	spline_x_new = sim_time

	celestial_interp = np.zeros(((10 * sim_seconds + 1), len(celestial_data[0])))

	for spline in range(len(celestial_data[0])):

		spline_y = celestial_data[:, spline]
		tck = interpolate.splrep(spline_x, spline_y, s=0)
		spline_y_new = interpolate.splev(spline_x_new, tck, der=0)
		celestial_interp[:, spline] = spline_y_new

	# save interpolated celestial data using track file name
	file_string = os.path.join(horz_fpath, fname_horz)
	np.savetxt(file_string, celestial_interp, delimiter=',')

	print('	Track File Generated: {:s}'.format(fname_horz))

	return 1

#############################################################################################
