
#############################################################################################
import os
import csv
import sys
import numpy as np
import time

import urllib.request
import shutil
import requests

# sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

# function_list
from function_list import date2jd, jd2date, get_iers, get_times

#############################################################################################

# Get TLE Time


def get_tletime(tle_line1, tle_line2):

	year_raw = int(tle_line1[18:20])

	if year_raw < 57:
		year = 2000 + year_raw
	else:
		year = 1900 + year_raw

	# epoch day
	day = float(tle_line1[20:32])
	if year < 2000:
		jday_tle = 2451545.0 - ((2000 - year) * 365.25)
		jdtle = jday_tle + day
	else:
		jday_tle = 2451545.0 + ((year - 2000) * 365.25)
		jdtle = jday_tle + day
	jfrac_tle = jdtle - np.floor(jdtle)

	# break into jday and jfrac
	if jfrac_tle >= 0.5:
		jday_tle = np.floor(jdtle) + 0.5
		jfrac_tle = jfrac_tle - 0.5
	else:
		jday_tle = np.floor(jdtle) - 0.5
		jfrac_tle = jfrac_tle + 0.5

	return jday_tle, jfrac_tle

#############################################################################################

# Request TLE Data


def req_tle(object_data, root_path, archive_path):

	# unpack case data
	norad_id = object_data.norad_ident
	jday_start = object_data.jday_utc_start
	jfrac_start = object_data.jfrac_utc_start
	jday_stop = object_data.jday_utc_stop
	jfrac_stop = object_data.jfrac_utc_stop

	# convert julian to utc date search strings
	year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start - 10, jfrac_start)
	year2, mon2, day2, hour2, min2, sec2 = jd2date(jday_start + 10, jfrac_start)

	# create date string
	req_datestart = '{:4}-{:02}-{:02}'.format(year1, mon1, day1)
	req_datestop = '{:4}-{:02}-{:02}'.format(year2, mon2, day2)
	req_datestr = '--'.join((req_datestart, req_datestop))

	# perform request
	time_start = '{:02}:{:02}:{:02.4}'.format(hour1, min1, sec1)
	tle_source_file = '{:d}_{:s}_{:s}.tle'.format(norad_id, req_datestart, time_start)

	# set working directories
	cwd = os.getcwd()
	tle_path = 'Outputs/TLE'
	object_dir = '{:d}'.format(norad_id)
	tle_source_path = os.path.join(root_path, archive_path, tle_path, 'tle_source', object_dir)

	if os.path.isfile(os.path.join(tle_source_path, tle_source_file)):
		print('	TLE source file exists')

		# open file and acquire the last 2 TLEs
		with open(os.path.join(tle_source_path, tle_source_file), 'r') as f:
			tle_line1 = ''
			tle_line2 = ''
			for line in reversed(list(f)):
				if not tle_line1 and not tle_line2:
					tle_line2 = line.rstrip()
				elif tle_line2:
					tle_line1 = line.rstrip()
					break

		jday_tle, jfrac_tle = get_tletime(tle_line1, tle_line2)

	else:

		# spacetrack login information
		login_url = 'https://www.space-track.org/ajaxauth/login'
		username = 'ldjblacketer@gmail.com'
		password = 'Chromodynamics1992'

		payload = {
			'identity': username,
			'password': password
		}

		# spacetrack request url
		req_url = os.path.join(
			'https://www.space-track.org/basicspacedata',
			'query/class/tle',
			'NORAD_CAT_ID',
			str(norad_id),
			'EPOCH',
			req_datestr,
			'format/tle'
		)

		# request TLE data
		with requests.Session() as session:
			post = session.post(login_url, data=payload)
			r = session.get(req_url).text.split('\n')

		# remove last row if empty
		if not r[-1]:
			r.pop()

		with open(os.path.join(tle_source_path, tle_source_file), 'w') as out_file:

			# search data for closest TLE
			for i in range(int(0.5 * len(r))):
				tle_line1 = r[2 * i]
				tle_line2 = r[(2 * i) + 1]

				# get tle julian times
				jday_tle, jfrac_tle = get_tletime(tle_line1, tle_line2)

				# closest TLE
				if (jday_tle + jfrac_tle) > (jday_start + jfrac_start):
					tle_line1 = tle_line1_prev
					tle_line2 = tle_line2_prev
					jday_tle = jday_tle_prev
					jfrac_tle = jfrac_tle_prev
					break
				else:
					out_file.write('\n{:s}\n{:s}'.format(tle_line1, tle_line2))
					# reset previous jd
					tle_line1_prev = tle_line1
					tle_line2_prev = tle_line2
					jday_tle_prev = jday_tle
					jfrac_tle_prev = jfrac_tle

	return tle_line1, tle_line2, jday_tle, jfrac_tle

#############################################################################################

# get satpos


def make_satpos(object_data, fname_tle, tle_fpath, tle_line1, tle_line2, jday_tle, jfrac_tle):

	# unpack object data
	norad_id = object_data.norad_ident
	track_id = object_data.track_ident
	jday_start = object_data.jday_utc_start
	jfrac_start = object_data.jfrac_utc_start
	jday_stop = object_data.jday_utc_stop
	jfrac_stop = object_data.jfrac_utc_stop

	# SGP4
	# tle separation
	jday_sep = jday_start - jday_tle
	jfrac_sep = jfrac_start - jfrac_tle
	separation = jday_sep + jfrac_sep

	# simulation parameters
	sim_start = jday_start + jfrac_start
	sim_seconds = int(86400 * ((jday_stop + jfrac_stop) - (jday_start + jfrac_start)))
	sim_time = np.linspace(0, sim_seconds, 10 * (sim_seconds) + 1)

	sat_pos = np.zeros(((sim_seconds * 10) + 1, 3))
	jdate = np.zeros(((sim_seconds * 10) + 1, 1))

	print('	Performing SGP4 Propagation:')
	print('		TLE Separation (days) = {:1.4f}'.format(separation))

	# propagation
	satellite = twoline2rv(tle_line1, tle_line2, wgs72)
	for time_point, sim_point in enumerate(sim_time):

		elapsed_time = sim_point / 86400
		jd_epoch = sim_start + elapsed_time

		# break into jday and jfrac
		jfrac = jd_epoch - np.floor(jd_epoch)
		jday = np.floor(jd_epoch)
		if jfrac >= 0.5:
			jday = np.floor(jd_epoch) + 0.5
			jfrac = jfrac - 0.5
		else:
			jday = np.floor(jd_epoch) - 0.5
			jfrac = jfrac + 0.5

		jdate[time_point] = (jday + jfrac) 
		year3, mon3, day3, hour3, min3, sec3 = jd2date(jday, jfrac)
		position, velocity = satellite.propagate(year3, mon3, day3, hour3, min3, sec3)
		sat_pos[time_point, :] = position

	# convert to meters
	sat_pos = np.multiply(sat_pos, 1000)
	sat_data = np.hstack([jdate, sat_pos])

	# sgp4 output
	sat_err_msg = satellite.error_message
	sat_err = satellite.error
	print('		SGP4 Finished: Error = {:d}'.format(sat_err))

	# save file
	file_string = os.path.join(tle_fpath, fname_tle)
	np.savetxt(file_string, sat_data, delimiter=',')

	print('	TLE track file generated: {:s}'.format(fname_tle))

#############################################################################################
