import os
import sys
import csv
import numpy as np
from progressbar import ProgressBar
from function_list import jd2date, date2jd

#############################################################################################

def parse_raw_files(file, path_raw, path_index, path_data):

	mmt_id = file.split('_')[2]

	# open file
	with open(os.path.join(path_raw, file), 'r') as mmt_file:
		# read data
		full_mmt_data = mmt_file.read().split('\n')

   	# name string
	name_str = full_mmt_data[0].split(' ', 2)[2]

	dataname = '{:s}_{:s}.csv'.format(mmt_id, name_str.split(' ')[0])
	indexname = '{:s}_{:s}.index'.format(mmt_id, name_str.split(' ')[0])

	if os.path.isfile(os.path.join(path_data, dataname)):
	
		print('\n{:s} file exists, loading...'.format(dataname))

		mmt_data = np.genfromtxt(os.path.join(path_data, dataname), delimiter=',')

	else:

		print('\nGenerating data file from {:s}'.format(mmt_id))

		# strip header
		mmt_data_str = full_mmt_data[7:-1]
		mmt_data = []
	
		progress = ProgressBar()
		for i, line in enumerate(progress(mmt_data_str)):
	
			# parse the line data
			line_split = line.split(' ')
			date = np.array(line_split[0].split('-')).astype(np.int)
			time = np.array(line_split[1].split(':')).astype(np.float)
			std_mag = np.array(line_split[2]).astype(np.float)
			mag = np.array(line_split[3]).astype(np.float)
			track_id = np.array(line_split[-1]).astype(np.int)
	
			# generate times
			datetime = np.concatenate((date, time))
			jday, jfrac = date2jd(*datetime)
			jdate = jday + jfrac

			# mag data
			mag_data = np.append(std_mag, mag)
			light_curve = np.append(jdate, np.append(datetime, mag_data))
	
			# assemble line
			mmt_data_line = np.append(light_curve, track_id)
			mmt_data.append(mmt_data_line)

		# save full data file
		mmt_data = np.array(mmt_data)
		np.savetxt(os.path.join(path_data, dataname), mmt_data, delimiter=',')

		print('Complete')

	print('\nGenerating index file from {:s}...'.format(file))

	# split track data
	track_ids_all = mmt_data[:, -1]
	track_ids, indices = np.unique(track_ids_all, return_index=True)
	tracks  = np.split(mmt_data, indices[1:])
	mmt_index = []

	for track in tracks:

		track_id = track[0, -1]
		NPTS = len(track)
		start = track[0, 0]
		datestart = [
			track[0, 1],
			track[0, 2],
			track[0, 3],
			track[0, 4],
			track[0, 5],
			track[0, 6]
		]

		stop = track[-1, 0]
		datestop = [
			track[-1, 1],
			track[-1, 2],
			track[-1, 3],
			track[-1, 4],
			track[-1, 5],
			track[-1, 6]
		]

		duration = 86400 * (track[-1, 0] - track[0, 0])
		MMT_sample_rate = 86400 * (track[-1, 0] - track[0, 0]) / len(track)

		index_line = np.append(np.append(start, datestart), np.append(stop, datestop))
		mmt_index.append(np.append(track_id, index_line))

	mmt_index = np.array(mmt_index)

	# save index file
	np.savetxt(os.path.join(path_index, indexname), mmt_index, delimiter=',')

	print('Complete')
	print('{:d} tracks found\n'.format(len(tracks)))

	return dataname, indexname

#############################################################################################

def gen_light_curves(dataname, path_data, path_light_curves):

	print('\nGenerating light curve files from: {:s}...'.format(dataname))
	norad_id = np.int(dataname.split('_')[1][:-4])
	path_lc = os.path.join(path_light_curves, str(norad_id))

	# create subdirectory if required
	if not os.path.exists(path_lc):
		os.makedirs(path_lc)

	mmt_data = np.genfromtxt(os.path.join(path_data, dataname), delimiter=',')

	# split track data
	track_ids_all = mmt_data[:, -1]
	track_ids, indices = np.unique(track_ids_all, return_index=True)
	tracks  = np.split(mmt_data, indices[1:])

	light_curves = 0

	for track in tracks:
		
		track_id = np.int(track[0, -1])
		light_curve_data = []

		for dataline in track:
			jdtime = dataline[0]
			year = dataline[1]
			mon = dataline[2]
			day = dataline[3]
			hour = dataline[4]
			mins = dataline[5]
			sec = dataline[6]
			std_mag = dataline[7]
			mag = dataline[8]
			light_curve = [jdtime, year, mon, day, hour, mins, sec, std_mag, mag, track_id]
			light_curve_data.append(light_curve)

		light_curve_data = np.array(light_curve_data)

		# save track file
		trackname = '{:d}_lightcurve_{:s}'.format(track_id, dataname)
		np.savetxt(os.path.join(path_lc, trackname), light_curve_data, delimiter=',')

		light_curves += 1

	print('{:d} Light Curve files generated.\n'.format(light_curves))


#############################################################################################


def gen_case_files(indexname, path_index, path_case_files, attit, attit_mot):

	print('\nGenerating case files from: {:s}...'.format(indexname))

	# case file format strings
	geometry_file = 'globalstar.stl'
	site_latgd = 43.64986
	site_lon = 41.43147
	site_alt = 2030.0
	# norad id
	norad_id = np.int(indexname.split('_')[1][:-6])

	# output path
	path_cf = os.path.join(path_case_files, str(norad_id))

	# create subdirectory if required
	if not os.path.exists(path_cf):
		os.makedirs(path_cf)

	# get index data
	index_data = np.genfromtxt(os.path.join(path_index, indexname), delimiter=',')

	cases = 0

	# make case file for each:
	for data_line in index_data:

		# create casefile format
		casedata = []
		track_id = data_line[0]

		# header
		casedata.append('# object config file\n')
		casedata.append('\n')

		# norad id
		casedata.append('norad_id = {:d}\n'.format(norad_id))

		# site data
		casedata.append('site_latgd = {:1.5f}\n'.format(site_latgd))
		casedata.append('site_lon = {:1.5f}\n'.format(site_lon))
		casedata.append('site_alt = {:1.5f}\n'.format(site_alt))

		# start strings
		year = data_line[2]
		mon = data_line[3]
		day = data_line[4]
		hour = data_line[5]
		mins = data_line[6]
		sec = data_line[7]

		start_str = '{:4.0f}, {:02.0f}, {:02.0f}, {:02.0f}, {:02.0f}, {:02.6f}'.format(
			year,
			mon,
			day,
			hour,
			mins,
			sec
		)

		start_name_str = '{:4.0f}-{:02.0f}-{:02.0f}_{:02.0f}:{:02.0f}:{:02.6f}'.format(
			year,
			mon,
			day,
			hour,
			mins,
			sec
		)

		# stop string
		year = data_line[9]
		mon = data_line[10]
		day = data_line[11]
		hour = data_line[12]
		mins = data_line[13]
		sec = data_line[14]

		stop_str = '{:4.0f}, {:02.0f}, {:02.0f}, {:02.0f}, {:02.0f}, {:02.6f}'.format(
			year,
			mon,
			day,
			hour,
			mins,
			sec
		)

		# time data
		casedata.append('start_time = {:s}\n'.format(start_str))
		casedata.append('stop_time = {:s}\n'.format(stop_str))

		# attitude state
		casedata.append('attit = {:1.4f}, {:1.4f}, {:1.4f}\n'.format(*attit))
		casedata.append('attit_mot = {:1.4f}, {:1.4f}, {:1.4f}\n'.format(*attit_mot))

		# geometry file
		casedata.append('\n')
		casedata.append('# Geometry\n')
		casedata.append('geometry_file={:s}\n'.format(geometry_file))

		# BDRF parameters
		casedata.append('\n')
		casedata.append('# BDRF\n')
		casedata.append('m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf\n')

		# generate globalstar bdrf data
		body_faces = np.arange(0,14,1)
		body_faces = np.append(body_faces, np.arange(39,41,1))
		body_faces = np.append(body_faces, np.arange(50,54,1))
		body_faces = np.append(body_faces, np.arange(58,70,1))
		
		for face in range(84):
			m = 0.17
			omega = 0.5

			n = (1 + np.sqrt(omega)) / (1 - np.sqrt(omega))

			if face in body_faces:
				d = 0.3
				s = 0.7
			else:
				d= 0.7
				s = 0.3

			face_data = [m, omega, d, s, n]
			casedata.append(
				'{:1.4f}, {:1.4f}, {:1.4f}, {:1.4f}, {:1.4f}\n'.format(*face_data)
			)

		# case file name
		casename = '{:.0f}_casefile_{:s}_{:s}.txt'.format(
			track_id,
			indexname[:-6],
			start_name_str
		)

		with open(os.path.join(path_cf, casename), mode='wt', encoding='utf-8') as casefile:
			for caseline in casedata:
				casefile.write(caseline)

		cases += 1

	print('Complete.')
	print('{:d} Case files generated.\n'.format(cases))


#############################################################################################
