#############################################################################################
#
#   SETUP
#
#############################################################################################
import csv
import numpy as np
import os
import sys
import collections
import time

from function_list import date2jd, jd2date
from get_horz import request_horz, make_horz
from get_satpos import req_tle, make_satpos
#############################################################################################

# Tuple case object
case_object = collections.namedtuple('case_object', [
	'case_ident',
	'norad_ident',
	'track_ident',
	'site_latitudegd',
	'site_longitude',
	'site_altitude',
	'jday_utc_start',
	'jfrac_utc_start',
	'jday_utc_stop',
	'jfrac_utc_stop',
	'attitude',
	'attitude_motion',
	'geometry_file'
])

#############################################################################################

# generate case object

def object_gen(case_file, case_path, lightcurve_path, root_path, archive_path, prop_step):

	# parse case file name
	case_str, case_name, mmt_id, norad_str, _ = case_file.split('_', maxsplit=4)

	track_str = case_str
	case_id = int(case_str)
	norad_id = int(norad_str)
	track_id = case_id

	#########################################################################################

	# create required subdirectories
	cwd = os.getcwd()
	object_dir = '{:d}'.format(norad_id)

	# horizons
	horz_path = 'Outputs/Horizons'
	horz_fpath = os.path.join(root_path, archive_path, horz_path, object_dir)
	horz_spath = os.path.join(root_path, archive_path, horz_path, 'horizons_source', object_dir)

	# tle
	tle_path = 'Outputs/TLE'
	tle_fpath = os.path.join(root_path, archive_path, tle_path, object_dir)
	tle_spath = os.path.join(root_path, archive_path, tle_path, 'tle_source', object_dir)

	# transforms
	transform_path = 'Outputs/Transformation'
	transform_fpath = os.path.join(root_path, archive_path, transform_path, object_dir)

	# light curves
	lightcurve_fpath = os.path.join(root_path, lightcurve_path, object_dir)

	if not os.path.exists(horz_fpath):
		os.makedirs(horz_fpath)
	if not os.path.exists(horz_spath):
		os.makedirs(horz_spath)

	if not os.path.exists(tle_fpath):
		os.makedirs(tle_fpath)
	if not os.path.exists(tle_spath):
		os.makedirs(tle_spath)

	if not os.path.exists(transform_fpath):
		os.makedirs(transform_fpath)

	if not os.path.exists(lightcurve_fpath):
		os.makedirs(lightcurve_fpath)

	#########################################################################################

	# parse case data

	# Read case file for time information
	case_data = []
	with open(os.path.join(case_path, case_file)) as csvfile:
		readCSV = csv.reader(csvfile, delimiter=',')
		for i, row in enumerate(readCSV):
			case_data.append(row)

	# site data
	site_latgd = np.array(''.join(case_data[3]).split('=')[1]).astype(np.float)
	site_lon = np.array(''.join(case_data[4]).split('=')[1]).astype(np.float)
	site_alt = np.array(''.join(case_data[5]).split('=')[1]).astype(np.float)

	# time data
	start_time = (''.join(case_data[6]).split('=')[1]).strip().split(' ')
	stop_time = (''.join(case_data[7]).split('=')[1]).strip().split(' ')

	# attitude state
	attit = np.array(
		(''.join(case_data[8]).split('=')[1]).strip().split(' ')
	).astype(np.float)

	attit_mot = np.array(
		(''.join(case_data[9]).split('=')[1]).strip().split(' ')
	).astype(np.float)
	
	# geometry file
	geom_file = ''.join(case_data[12][0].split('=')[1]).strip()

	# parse time information and change data type
	year1, mon1, day1, hour1, min1 = np.array(start_time[0:-1]).astype(np.int)
	sec1 = np.array(start_time[-1]).astype(np.float)

	year2, mon2, day2, hour2, min2 = np.array(stop_time[0:-1]).astype(np.int)
	sec2 = np.array(stop_time[-1]).astype(np.float)

	start_date = [year1, mon1, day1, hour1, min1, sec1]
	stop_date = [year2, mon2, day2, hour2, min2, sec2]

	#	convert to JD
	jday_start, jfrac_start = date2jd(*start_date)
	jday_stop, jfrac_stop = date2jd(*stop_date)
	duration = int(86400 * ((jday_stop + jfrac_stop) - (jday_start + jfrac_start)))

	#	BDRF parameters

	raw_bdrf_data = case_data[16:len(case_data)]
	bdrf_data = []

	for bdrf_line in raw_bdrf_data:
		data_line = []
		for ele in bdrf_line:
			data_line.append(np.float(ele))

		bdrf_data.append(data_line)

	#########################################################################################

	# Assemble filename strings
	date_str = '{:4}-{:02}-{:02}_{:02}-{:02}-{:1.4f}'.format(*start_date)

	fname_horz = 'horz_{:1.1}_{:02}_{:s}.csv'.format(
		prop_step,
		norad_id,
		date_str
	)

	fname_tle = 'tle_{:1.1}_{:02}_{:s}.csv'.format(
		prop_step,
		norad_id,
		date_str
	)

	fname_lc = '{:d}_{:s}_{:d}_lightcurve.csv'.format(track_id, case_name, norad_id)

	#########################################################################################

	if start_date[0] <= 2017:
		# Create case object
		object_data = case_object(
			case_ident=case_id,
			norad_ident=norad_id,
			track_ident=track_id,
			site_latitudegd=site_latgd,
			site_longitude=site_lon,
			site_altitude=site_alt,
			jday_utc_start=jday_start,
			jfrac_utc_start=jfrac_start,
			jday_utc_stop=jday_stop,
			jfrac_utc_stop=jfrac_stop,
			attitude=attit,
			attitude_motion=attit_mot,
			geometry_file=geom_file
		)
	else:
		object_data = 0

	return object_data, fname_horz, fname_tle, fname_lc, bdrf_data

#############################################################################################

# initilise files 

def fred_setup(inputs):

	horz_files_gen = 0
	tle_files_gen = 0

	object_data, fname_horz, fname_tle, root_path, archive_path = inputs

	# Initilization

	# unpack object data
	norad_id = object_data.norad_ident
	track_id = object_data.track_ident
	jday_start = object_data.jday_utc_start
	jfrac_start = object_data.jfrac_utc_start

	year, mon, day, hour, mins, sec = jd2date(jday_start, jfrac_start)

#	print(
#		'\nNorad: {:d} \n'
#		'Track: {:d} \n'
#		'Date: {:4.0f}-{:02.0f}-{:02.0f}'
#		'{:02.0f}:{:02.0f}:{:02.2f}\n'.format(
#		norad_id, track_id,
#		year, mon, day, hour, mins, sec
#	))

	# set directories
	cwd = os.getcwd()
	object_dir = '{:d}'.format(norad_id)
	horz_path = 'Outputs/Horizons'
	horz_fpath = os.path.join(root_path, archive_path, horz_path, object_dir)
	horz_spath = os.path.join(root_path, archive_path, horz_path, 'horizons_source', object_dir)

	# set working directories
	cwd = os.getcwd()
	file_str = 'Outputs/TLE/{:d}'.format(norad_id)
	source_str = 'Outputs/TLE/tle_source/{:d}'.format(norad_id)
	tle_fpath = os.path.join(root_path, archive_path, file_str)
	tle_source_path = os.path.join(root_path, archive_path, source_str)

	# acquire and process horz ephemeris data
	if os.path.isfile(os.path.join(horz_fpath, fname_horz)):
		tempval = 1
#		print('Horizons Track File Exists')
	else:
#		print('No Track File Found - Running Horizons acquisition...')
		tle_files_gen += 1

		# Perform Requests
		filenames, jdate_tdb_start = request_horz(object_data, horz_fpath, horz_spath)

		# Track File Generation
		temp = make_horz(filenames, fname_horz, horz_fpath, horz_spath, jdate_tdb_start)

	# acquire and process object ephemeris data
	if os.path.isfile(os.path.join(tle_fpath, fname_tle)):
		tempval = 1
#		print('TLE Track File Exists')
	else:
#		print('No TLE File Found - Acquiring . . .')
		horz_files_gen += 1

		# get tle data
		tle_line1, tle_line2, jday_tle, jfrac_tle = req_tle(object_data, root_path, archive_path)

		# Track File Generation
		make_satpos(object_data, fname_tle, tle_fpath, tle_line1, tle_line2, jday_tle, jfrac_tle)

	return horz_files_gen, tle_files_gen
