####################################################################################################
#
#	SYNTHETIC LIGHT CURVE GENERATION
#
#	INPUTS:		OBJECT DATA, LIGHT CURVE FILE NAME, BDRF PARAMETERS
#
#	OUTPUTS:	SYNTHETIC LIGHT CURVE
#
####################################################################################################

import os
import sys
import csv
import numpy as np

from stl import mesh

from function_list import jd2date, date2jd, get_iers, get_times
from trans_func import rot_mat, site_vec, teme2itrf, it2rsw

####################################################################################################

#	import sat and horiz data

def slc(object_data, root_path, archive_path, file_name_lc, bdrf_data):

	#	simulation data

	#	unpack object data
	norad_id = object_data.norad_ident
	jday_start = object_data.jday_utc_start
	jfrac_start = object_data.jfrac_utc_start
	jday_stop = object_data.jday_utc_stop
	jfrac_stop = object_data.jfrac_utc_stop
	site_latgd = np.radians(object_data.site_latitudegd)
	site_lon = np.radians(object_data.site_longitude)
	site_alt = object_data.site_altitude
	attit = object_data.attitude
	attit_mot = object_data.attitude_motion
	geom_file = object_data.geometry_file

	#	convert julian to utc date
	year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)
	date_vec1 = [year1, mon1, day1, hour1, min1, sec1] 

	#	simulation parameters
	sim_start = jday_start + jfrac_start
	sim_seconds = int(86400 * ((jday_stop + jfrac_stop) - (jday_start + jfrac_start)))
	sim_time = np.linspace(0, sim_seconds, 10 * (sim_seconds) + 1)

	################################################################################################

	#	data files

	#	directories
	cwd = os.getcwd()

	transform_path = 'Outputs/Transformation/{:d}'.format(norad_id)
	geom_path = 'sensitivity_study_newdata/Geometry/new_geoms'

	#	create file name
	date_str = '{:4}-{:02}-{:02}_{:02}-{:02}-{:1.4f}'.format(*date_vec1)
	file_name_rsw =  '{:d}_rsw_{:s}.csv'.format(norad_id, date_str)
	data_pos = []

	#	read positional data
	data_pos = np.loadtxt(os.path.join(
		root_path, archive_path, transform_path, file_name_rsw),
		delimiter=','
	)

	#	parse data
	sun_pos = data_pos[:, 0:3]
	site_pos = data_pos[:, 6:9]

	################################################################################################

	#	geometry

	#	import
	static_mesh = mesh.Mesh.from_file(os.path.join(root_path, geom_path, geom_file))
	object_mesh = mesh.Mesh.from_file(os.path.join(root_path, geom_path, geom_file))

	#	rotate for xaxis orientation
#	object_mesh.rotate([0,0,0.5], (np.pi / 2))
	# translate by CoM
#	volume, cog, inertia = object_mesh.get_mass_properties()
#	object_mesh.translate(-cog)

	#	convert units
#	areas = object_mesh.areas / 1000**2
#	vectors = object_mesh.vectors / 1000

	areas = object_mesh.areas
	vectors = object_mesh.vectors

	################################################################################################

	#	Prepare for propagation
	
	#	initial attitude rotation matrix
	att_mat = rot_mat(attit)

	#	allocate memory
	sun_vec_data = np.zeros((sun_pos.shape))
	site_vec_data = np.zeros((site_pos.shape))
	light_curve = np.zeros((len(sim_time), 3))
	light_curve[:,1] = sim_time

	#	generate illumination and observation vectors
	for time_step, (sun_vec, site_vec) in enumerate(zip(sun_pos, site_pos)):

		time_since = light_curve[time_step, 1]

		# apply initial attitude rotations
		sun_vec_step = np.dot(sun_vec, att_mat)
		site_vec_step = np.dot(site_vec, att_mat)

		# time step rotation matrix
		total_attit_rot = np.multiply(attit_mot, time_since)

		# reform attitude motion rotation matrix
		rot_mat_step = rot_mat(total_attit_rot)

		# apply attitude motion rotations
		sun_vect = np.dot(sun_vec_step, rot_mat_step)
		sun_vec_data[time_step, :] = sun_vect
		site_vect = np.dot(site_vec_step, rot_mat_step)
		site_vec_data[time_step, :] = site_vect


	####################################################################################################
	#	Initilization complete
	####################################################################################################

	print('Started Light Curve Generation...')
	print('	Duration = ', sim_seconds)
	print('	NPTS = ', len(sim_time))


	#	time loop
	for time_step, row in enumerate(light_curve):

		################################################################################################

		#	initilise

		#	time data
		time_since = light_curve[time_step, 1]
		jdate = sim_start + (time_since / 86400)
		light_curve[time_step, 0] = jdate

		#	position data
		sun_vec_step = sun_vec_data[time_step, :]
		site_vec_step = site_vec_data[time_step, :]
		sun_vec_unit = sun_vec_step / np.linalg.norm(sun_vec_step)
		site_vec_unit = site_vec_step / np.linalg.norm(site_vec_step)

		#	set obervation distance in meters
		r_obs = np.linalg.norm(site_vec_step)

		################################################################################################

		#	brightness calculation

		#	angular bisector
		H_vec = (sun_vec_unit + site_vec_unit) / len(sun_vec_unit + site_vec_unit) 

		#	observable facet array
		face_obs = []

		print('\nnum faces = \n', len(vectors))
		# facet loop
		for vector, area, bdrf_line in zip(vectors, areas, bdrf_data):

			#	parse bdrf data
			m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf = bdrf_line

			#	calculate normal unit vector
			vect12 = vector[1] - vector[0]
			vect23 = vector[2] - vector[1]
			normal = np.cross(vect12, vect23)
			normal = normal / np.linalg.norm(normal)

			#	calculate angles
			sun_ang = np.arccos(
				np.divide(
					np.dot(normal, sun_vec_step),
					np.linalg.norm(normal) * np.linalg.norm(sun_vec_step)
				)
			)

			site_ang = np.arccos(
				np.divide(
					np.dot(normal, site_vec_step),
					np.linalg.norm(normal) * np.linalg.norm(site_vec_step)
				)
			)

			#	illuminated and observable condition
			if sun_ang < (0.5 * np.pi) and site_ang < (0.5 * np.pi):

				#	alpha calculation
				alpha_ang = np.arccos(
					np.divide(
						np.dot(H_vec, normal),
						np.linalg.norm(H_vec) * np.linalg.norm(normal)
					)
				)

				#	G_bdrf calculation
				temp1 = 2 * np.divide(
					np.multiply(
						np.dot(normal, H_vec),
						np.dot(normal, site_vec_unit)
					), 
					np.dot(site_vec_unit, H_vec)
				)
				
				temp2 = 2 * np.divide(
					np.multiply(
						np.dot(normal, H_vec),
						np.dot(normal, sun_vec_unit)
					), 
					np.dot(sun_vec_unit, H_vec)
				)
	
				if temp1 > 1 and temp2 > 1:
				    G_bdrf = 1
				elif temp1 < 1 and temp2 > 1:
					G_bdrf = temp1
				elif temp1 > 1 and temp2 < 1:
					G_bdrf = temp2
				elif temp1 < 1 and temp2 < 1:
					G_bdrf = np.min((temp1, temp2))

				#	c_bdrf calculation
				c_bdrf = np.dot(site_vec_unit, H_vec)

				#	g_bdrf calculation
				g_bdrf = n_bdrf**2 + c_bdrf**2 - 1
	
				#	F_bdrf calculation
				F_bdrf = (
					((g_bdrf - c_bdrf)**2 / (2 * (g_bdrf + g_bdrf))) * 
					((1 + (c_bdrf * (g_bdrf + c_bdrf) - 1))**2 / ((c_bdrf * 
					(g_bdrf - c_bdrf) + 1)**2))
				)

				#	D_bdrf_calculation
				D_bdrf = (
					(1 / (m_bdrf**2 * np.cos(alpha_ang)**4)) * 
					np.exp(-((np.tan(alpha_ang) / m_bdrf)**2))
				)

				#	specular reflectance calculation
				R_s = (
					(D_bdrf * G_bdrf * F_bdrf) / (np.pi * np.dot(normal, sun_vec_unit) * 
					np.dot(normal, site_vec_unit))
				)

				#	diffuse reflectance calculation
				R_d = omega_bdrf / np.pi;
	
				#	brightness calculation
				facet_brightness = (
					((d_bdrf * R_d) + (s_bdrf * R_s)) * \
					area * \
					np.pi * \
					np.dot(normal, sun_vec_unit) * \
					np.dot(normal, site_vec_unit)
				)

				face_obs.append(facet_brightness / (4 * np.pi * r_obs**2))

				print('observation distance', r_obs)
				print('\nfacet brightness = ', facet_brightness / (4 * np.pi * r_obs**2))

		#	apparent magnitude calculation
		brightness_step = np.sum(face_obs)
		magnitude = -26.74 - 2.5 * np.log10(brightness_step)
		light_curve[time_step, 2] = magnitude

		print(magnitude)
		sys.exit()

	print('Lightcurve Generated.')
	return light_curve
