#############################################################################################
#	Generation of Light Measurments for Machine Learning Applications [GLIMMA]
#############################################################################################
#
#
#
#
#
#
#
#############################################################################################

print('Script started...')

#############################################################################################

#	standard packages
import os
import sys
import time
from subprocess import call

#	numpy
import numpy as np

#	matplotlib
import matplotlib.pyplot as plt

#	numpy-stl
import stl
from stl import mesh

#	progress bar
import progressbar

#	sgp4
from sgp4.earth_gravity import wgs72
from sgp4.io import twoline2rv

#	custom functions
from setup import object_gen
from functions.function_list import jd2date, date2jd, get_iers, get_times
from trans_func import rot_mat, site_vec, teme2itrf, it2rsw

#	transformation
from functions.function_transform import teme2itrf, \
											site_vec, \
											rot_mat, \
											mat2axis, \
											axis2mat, \
											axis2mat2, \
											mat2angles, \
											mat2angles2, \
											rot3, \
											rot2, \
											rot1, \
											body2orb_mat, \
											body2orb_vel

#	function list
from functions.function_list import date2jd, \
									jd2date, \
									get_times, \
									interp, \
									eop_interpolate, \
									req_horizons, \
									gen_sgp4
#	sofa routines
from functions.sofa_interface import gc2it

from glim_transform import glim_trans

###################################################################################################
#	initilisation
###################################################################################################

#	clear terminal
call(['clear'])
print('Started.')


##################################################
#	old case_file

track_str = '0001_'
case_id = 'test-geom_'
track_id = '999_'
norad_id = '25885_testing'

case_file = track_str + case_id + track_id + norad_id + '.txt'

##################################################
#	Object 25885 TLE closest to start of 2017

tle_line1 = '1 25885U 99043C   16347.16557405 -.00000088  00000-0 -43593-3 0  9992'
tle_line2 = '2 25885  52.0136 114.2293 0006645 165.6082 252.1930 11.39172050765052'

#	satellite object
satellite = twoline2rv(tle_line1, tle_line2, wgs72)

##################################################
#	simulation parameters

#	propagation step in seconds
prop_step_sec = 1
prop_step = prop_step_sec / (24 * 60 * 60)

#	time step in seconds
time_step = 1

#	length of data reccord [number of points]
len_reccord = 120

#	add zero
len_reccord = len_reccord + 1

#	reference frame selection
start_point = 0
orb_frame_out = True
body_frame_out = True









###################################################################################################
#	file in/out
###################################################################################################

##################################################
#	file paths
root_path = '/home/ldjb/codebase'
archive_path = 'data_archive/paper_data'

#	path to data
path_data = os.path.join(root_path, archive_path)

##################################################
#	directories

#	case file
case_dir = 'input_dir'

#	horizons data
horz_dir = 'pos_dir/horz_dir'

#	sgp4 data
sgp4_dir = 'sgp4_dir'

#	pos data
pos_dir = 'pos_dir'

#	EOP data
eop_dir = 'eop_files'

#	lightcurve data
lightcurve_dir = 'lightcurve_dir'

#	render data
render_dir = 'render_dir'
render_geom_dir = 'geom_files'

##################################################
#	data files

#	icrf
sun_icrf_file = 'sun_icrf.txt'
earth_icrf_file = 'earth_icrf.txt'

#	sgp4
sgp4_file = 'sgp4_data.txt'

#	EOP
eop_file = 'eop_parsed.csv'
dat_file = 'dat_parsed.csv'

#	itrf
sat_itrf_file = 'sat_itrf.txt'
sun_itrf_file = 'sun_itrf.txt'

#	azel
sat_azel_file = 'sat_azel.txt'
sun_azel_file = 'sun_azel.txt'

#	gcrf
sat_gcrf_file = 'sat_gcrf.txt'
sun_gcrf_file = 'sun_gcrf.txt'
site_gcrf_file = 'site_gcrf.txt'

#	orb
sat_orb_file = 'sat_orb.txt'
sun_orb_file = 'sun_orb.txt'
earth_orb_file = 'earth_orb.txt'
site_orb_file = 'site_orb.txt'

#	render outputs
render_data_orb = 'render_data_orb.txt'
render_data_body = 'render_data_body.txt'







###################################################################################################
#	parse input data
###################################################################################################

#	assemble file paths
case_path = os.path.join(root_path, archive_path, case_dir)
lightcurve_path = os.path.join(root_path, archive_path, lightcurve_dir)

#	generatate case data from case file
object_data, fname_horz, fname_tle, fname_lc, bdrf_data =  object_gen(
	case_file,
	case_path,
	lightcurve_path,
	root_path,
	archive_path,
	prop_step
)

#	attitude
initial_attitude = object_data.attitude
attitude_motion = object_data.attitude_motion

#	simulation time data
jday_start = object_data.jday_utc_start
jfrac_start = object_data.jfrac_utc_start
jday_stop = object_data.jday_utc_stop
jfrac_stop = object_data.jfrac_utc_stop

#	observation site data
site_latgd = np.radians(object_data.site_latitudegd)
site_lon = np.radians(object_data.site_longitude)
site_alt = object_data.site_altitude
site_itrf = np.divide(site_vec(site_latgd, site_lon, site_alt), 1000.0)

# convert julian to utc date search strings
year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)
year2, mon2, day2, hour2, min2, sec2 = jd2date(jday_stop, jfrac_stop)

#	create required subdirectories
norad_id = object_data.norad_ident

###################################################################################################
#	initilise time data

#	julian date
jd_start = jday_start + jfrac_start
mjd_start = jd_start - 2400000.5

#	sim end jdate
sim_end_date = [
	year2,
	mon2,
	day2,
	hour2,
	min2,
	round(sec1, 1)
]

#	end date
jd_stop = jday_stop + jfrac_stop
mjd_stop = jd_stop - 2400000.5

#	simulation range in julian date
sim_range = np.arange(jd_start, jd_stop, prop_step)

#	simulation time in seconds
duration = time_step * len_reccord
sim_time = np.arange(0, duration, time_step)







#############################################################################################
#	generate EOP data
###################################################################################################

#	assemble eop path
eop_path = os.path.join(root_path, archive_path, eop_dir)

#	generate interpolated eop_data
eop_data, dat_data = eop_interpolate(eop_path, eop_file, dat_file, mjd_start, sim_range)








#############################################################################################
#	generate time objects
###################################################################################################

#	start dat value
dat = dat_data[0, -1]

#	start eop values
eop_line = eop_data[0, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line

#	generate start time object
start_time = get_times(
	year1,
	mon1,
	day1,
	hour1,
	min1,
	round(sec1, 1),
	ut_offset,
	dat
)

#	end dat value
dat = dat_data[-1, -1]

#	end eop values
eop_line = eop_data[-1, :]
eop_mjd, eop_year, eop_mon, eop_day, x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line

#	generate stop time object
stop_time = get_times(
	*sim_end_date,
	ut_offset,
	dat
)









#############################################################################################
#	Horizons query
#############################################################################################

#	assemble path
horz_path = os.path.join(path_data, horz_dir)

print('Checking for HORIZONS data. . .')
if os.path.isfile(os.path.join(horz_path, sun_icrf_file)):
	print('	Horizons Data exists, opening. . .')

	#	load sun icrf data
	sun_icrf_raw = np.loadtxt(
		os.path.join(horz_path, sun_icrf_file),
		delimiter=','
	)

	#	load earth itrf data
	earth_icrf_raw = np.loadtxt(
		os.path.join(horz_path, earth_icrf_file),
		delimiter=','
	)

else:

	print('	No Horizons file. Acquisition required.')
	sun_icrf_raw, earth_icrf_raw = req_horizons(
		start_time,
		stop_time,
		jd_start,
		horz_path,
		sim_range,
		sun_icrf_file,
		earth_icrf_file
	)









#############################################################################################
#	sgp4 generation
#############################################################################################

#	sgp4 path
sgp4_path = os.path.join(path_data, sgp4_dir, sgp4_file)

print('Checking for SGP4 data')
if os.path.isfile(os.path.join(path_data, sgp4_dir, sgp4_file)):
	print('	SGP4 Data exists, opening. . .')

	#	load sgp4 data
	sat_teme_raw = np.loadtxt(
		os.path.join(path_data, sgp4_dir, sgp4_file),
		delimiter=','
	)

else:

	print('	No SGP4 file. Acquisition required.')

	#	initilise position data matrix
	sat_teme_raw = gen_sgp4(sim_range, sgp4_path, satellite)










###############################################################################################
#	orbit propagation, transformation
###############################################################################################

print('Checking for transform data. . .')
if os.path.isfile(os.path.join(path_data, pos_dir, sat_azel_file)) \
	and os.path.isfile(os.path.join(path_data, pos_dir, site_orb_file)):

	print('	Azel and RSW data exists, opening. . .')

	#	load azel sat data
	sat_azel = np.loadtxt(
		os.path.join(path_data, pos_dir, sat_azel_file),
		delimiter=','
	)

	#	load azel sun data
	sun_azel = np.loadtxt(
		os.path.join(path_data, pos_dir, sun_azel_file),
		delimiter=','
	)

	#	load itrf sat data
	sat_itrf = np.loadtxt(
		os.path.join(path_data, pos_dir, sat_itrf_file),
		delimiter=','
	)

	#	load itrf sun data
	sun_itrf = np.loadtxt(
		os.path.join(path_data, pos_dir, sun_itrf_file),
		delimiter=','
	)

	#	load azel sat data
	sat_orb = np.loadtxt(
		os.path.join(path_data, pos_dir, sat_orb_file),
		delimiter=','
	)

	#	load azel sun data
	sun_orb = np.loadtxt(
		os.path.join(path_data, pos_dir, sun_orb_file),
		delimiter=','
	)

	#	load itrf sat data
	site_orb = np.loadtxt(
		os.path.join(path_data, pos_dir, site_orb_file),
		delimiter=','
	)

	#	load itrf sat data
	earth_orb = np.loadtxt(
		os.path.join(path_data, pos_dir, earth_orb_file),
		delimiter=','
	)

else:
	
	print('	Transformation required.')

	#	tt time
	jdaytt_start = start_time.jday_tt
	jfractt_start = start_time.jfrac_tt
	jdtt_start = jdaytt_start + jfractt_start

	
	#	ut1 time
	jdayut1_start = start_time.jday_ut1
	jfracut1_start = start_time.jfrac_ut1
	jdut1_start = jdayut1_start + jfracut1_start
	
	#	generate rotation matrices
	print('Generating rotation matrices. . .')
	
	pm_list = []
	pn_list = []
	st_list = []
	c2t_list = []

	###############################################################################################
	#	rotation matrix generation
	###############################################################################################

	#	matrix generation loop
	for step, eop_line in enumerate(progressbar.progressbar(eop_data)):
	
		#	parse eops
		eop_mjd, eop_year, eop_mon, eop_day,\
			x_p, _, y_p, _, ut_offset, _, lod, _, dX, _, dY, _ = eop_line
	
		#	generate time data
		time_since = step * time_step
		
		jdtt = jdtt_start + time_since
		jdut1 = jdut1_start + time_since
		
		#	as fraction
		if jdtt - np.floor(jdtt) < 0.5:
			jdaytt_epoch = np.floor(jdtt) - 0.5
			jfractt_epoch = jdtt - jdaytt_epoch
	
		elif jdtt - np.floor(jdtt) > 0.5:
			jdaytt_epoch = np.floor(jdtt) + 0.5
			jfractt_epoch = jdtt - jdaytt_epoch
	
		if jdut1 - np.floor(jdut1) < 0.5:
			jdayut1_epoch = np.floor(jdut1) - 0.5
			jfracut1_epoch = jdut1 - jdayut1_epoch
	
		elif jdut1 - np.floor(jdut1) > 0.5:
			jdayut1_epoch = np.floor(jdut1) + 0.5
			jfracut1_epoch = jdut1 - jdayut1_epoch

		#	generate GCRF to ITRF transformation matrix
		pm, pn, st, c2t = gc2it(
			jdaytt_epoch,
			jfractt_epoch,
			jdayut1_epoch,
			jfracut1_epoch,
			dX,
			dY,
			x_p,
			y_p
		)

		#	add to list
		pm_list.append(pm)
		pn_list.append(pn)
		st_list.append(st)
		c2t_list.append(c2t)
	
	#	convert to 2d numpy array
	print('Converting to array...')
	
	pm_array = np.zeros((len(pm_list), 9))
	pn_array = np.zeros((len(pn_list), 9))
	st_array = np.zeros((len(st_list), 9))
	c2t_array = np.zeros((len(c2t_list), 9))

	
	for i, mat in enumerate(pm_list):
		pm_array[i, :] = *mat[0], *mat[1][:], *mat[2][:]

	for i, mat in enumerate(pn_list):
		pn_array[i, :] = *mat[0], *mat[1][:], *mat[2][:]

	for i, mat in enumerate(st_list):
		st_array[i, :] = *mat[0], *mat[1][:], *mat[2][:]

	for i, mat in enumerate(c2t_list):
		c2t_array[i, :] = *mat[0], *mat[1][:], *mat[2][:]

	print('Done.\n')

	###############################################################################################
	#	transformation
	###############################################################################################

	print('\nStarting transformation')
		
	#	site and time arrays
	data_array = np.hstack([sat_teme_raw, sun_icrf_raw[:, 1:], earth_icrf_raw[:, 1:]])
	site_data = np.hstack((site_itrf, site_latgd, site_lon, site_alt))
	time_data = np.array((jdaytt_start, jfractt_start, jdayut1_start, jfracut1_start))
	
	#	initilise output arrays
	out_data_itrf = np.zeros((len(data_array), 9))
	out_data_gcrf_sat = np.zeros((len(data_array), 6))
	out_data_gcrf = np.zeros((len(data_array), 6))
	out_data_orb = np.zeros((len(data_array), 9))
	out_data_orb_sat = np.zeros((len(data_array), 6))
	out_data_azel = np.zeros((len(data_array), 6))
	
	#	run transformation module
	glim_trans(
		data_array,
		eop_data,
		dat_data,
		pn_array,
		pm_array,
		st_array,
		c2t_array,
		site_data,
		time_data,
		out_data_itrf,
		out_data_gcrf_sat,
		out_data_gcrf,
		out_data_orb,
		out_data_orb_sat,
		out_data_azel
	)
	
	#	itrf data
	sat_itrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_itrf[:, 0:6]))
	sun_itrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_itrf[:, 6:9]))
	
	#	gcrf data
	sat_gcrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_gcrf_sat[:, 0:6]))
	sun_gcrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_gcrf[:, 0:3]))
	site_gcrf = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_gcrf[:, 3:6]))
	
	#	orb data
	sat_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb_sat[:, 0:6]))
	sun_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb[:, 0:3]))
	earth_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb[:, 3:6]))
	site_orb = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_orb[:, 6:9]))
	
	#	azimuth elevation data
	sat_azel = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_azel[:, 0:3]))
	sun_azel = np.hstack((np.reshape(sim_range, [len(sim_range), 1]), out_data_azel[:, 3:6]))
	
	#	save files
	
	#	itrf
	np.savetxt(os.path.join(path_data, pos_dir, sat_itrf_file), sat_itrf, delimiter=',')
	np.savetxt(os.path.join(path_data, pos_dir, sun_itrf_file), sun_itrf, delimiter=',')
	
	#	gcrf
	np.savetxt(os.path.join(path_data, pos_dir, sat_gcrf_file), sat_gcrf, delimiter=',')
	np.savetxt(os.path.join(path_data, pos_dir, sun_gcrf_file), sun_gcrf, delimiter=',')
	np.savetxt(os.path.join(path_data, pos_dir, site_gcrf_file), site_gcrf, delimiter=',')
	
	#	orb
	np.savetxt(os.path.join(path_data, pos_dir, sat_orb_file), sat_orb, delimiter=',')
	np.savetxt(os.path.join(path_data, pos_dir, sun_orb_file), sun_orb, delimiter=',')
	np.savetxt(os.path.join(path_data, pos_dir, earth_orb_file), earth_orb, delimiter=',')
	np.savetxt(os.path.join(path_data, pos_dir, site_orb_file), site_orb, delimiter=',')
	
	#	azel
	np.savetxt(os.path.join(path_data, pos_dir, sat_azel_file), sat_azel, delimiter=',')
	np.savetxt(os.path.join(path_data, pos_dir, sun_azel_file), sun_azel, delimiter=',')
	
	print('\nTransformation Complete.')









#############################################################################################
#	prepare vector generation
#############################################################################################

#	define directories
render_geom_dir_orb  = os.path.join(render_geom_dir, 'orb_frame')
render_geom_dir_body = os.path.join(render_geom_dir, 'body_frame')

#	geometry
geom_file = object_data.geometry_file
geom_path = 'sensitivity_study_newdata/Geometry/new_geoms'

#	static mesh
body_mesh = mesh.Mesh.from_file(os.path.join(root_path, geom_path, geom_file))

#	define body axis vectors of object mesh
x_body_unit = np.array((1.0, 0.0, 0.0))
y_body_unit = np.array((0.0, 1.0, 0.0))
z_body_unit = np.array((0.0, 0.0, 1.0))

###############################################
#	convert to orbital frame

#	orb mesh
orb_mesh = mesh.Mesh(body_mesh.data.copy())

#	get mesh properties for CoM
volume, com, inertia = body_mesh.get_mass_properties()

#	translate by CoM
orb_mesh.translate(-com)
body_mesh.translate(-com)

#	update mesh properties
volume, cog, inertia = body_mesh.get_mass_properties()

#	set areas and vectors
areas = body_mesh.areas
body_vectors = body_mesh.vectors
orb_vectors = orb_mesh.vectors

#	define known vectors
x_unit = np.array((1.0, 0.0, 0.0))
y_unit = np.array((0.0, 1.0, 0.0))
z_unit = np.array((0.0, 0.0, 1.0))

###################################################################################################
#	attitude profile
###################################################################################################

#	orbital motion in radians per second
orbital_mot = 9.141e-4

#	define orbital to inertial angular velocity
ang_vel_orb2inertial = np.array((0.0, 0.0, -orbital_mot))

###################################################################################################
#	BODY

ang_vel_body = np.copy(attitude_motion)
body_attitude = initial_attitude

###################################################################################################
#	ORBITAL

#	apply initial attitude to mesh
att_rot_mat = rot_mat(initial_attitude)
orb_mesh.rotate_using_matrix(att_rot_mat)

#	apply inital attitude to orbital axis units
x_orb_unit = np.dot(x_body_unit, att_rot_mat)
y_orb_unit = np.dot(y_body_unit, att_rot_mat)
z_orb_unit = np.dot(z_body_unit, att_rot_mat)

#	update vectors
orb_vectors = orb_mesh.vectors
orb_areas = orb_mesh.areas

#	convert orbital frame
body2orb = body2orb_vel(
	body_attitude[0],
	ang_vel_body[0],
	body_attitude[1],
	ang_vel_body[1],
	body_attitude[2],
	ang_vel_body[2]
)

#	set orbital angular velocity
ang_vel_orb = np.copy(body2orb)

#	create temporary mesh loop update
temp_orb_mesh = mesh.Mesh(orb_mesh.data.copy())

#	initilise output vector data
vector_data_inertial = np.zeros((len_reccord, 28))
vector_data_body = np.zeros((len_reccord, 28))
vector_data_orb = np.zeros((len_reccord, 28))

sun_orb_raw = np.copy(sun_orb)
sat_orb_raw = np.copy(sat_orb)
earth_orb_raw = np.copy(earth_orb)
site_orb_raw = np.copy(site_orb)

#	simulation start date
jdate_start = sun_orb_raw[0, 0]

#	set step angular velocities
ang_vel_body_step = np.copy(ang_vel_body)
ang_vel_orb_step = np.copy(ang_vel_orb)

#	set step attitude
step_attitude = np.copy(initial_attitude)

#	set step angle offsets
ang_step_orb = np.array((0.0, 0.0, 0.0))
ang_step_body = np.array((0.0, 0.0, 0.0))

#	apply to orbital axis units
x_orb_unit_step = np.copy(x_orb_unit)
y_orb_unit_step = np.copy(y_orb_unit)
z_orb_unit_step = np.copy(z_orb_unit)







###################################################################################################
#	generate vector data
###################################################################################################

print('Starting time loop.')

for i, (sat_orb_line, sun_orb_line, earth_orb_line,	site_orb_line) in enumerate(
		zip(sat_orb_raw, sun_orb_raw, earth_orb_raw, site_orb_raw)
	):

	###############################################################################################
	#	parse data
	###############################################################################################

	###############################################################################################
	#	time

	#	set date and previous date
	jdate = sun_orb_line[0]

	if i == 0:
		jdate_prev = np.copy(jdate)
		step_factor = 24 * 60 * 60 / time_step
		time_since = jdate_prev - jdate_start

	else:
		jdate_prev = jdate
		jdate = sun_orb_line[0]

	#	time since start

	if i % 10 == 0:
		step_factor = 24 * 60 * 60 / time_step
		time_since = jdate_prev - jdate_start
		print('Time since start = {:d} [Mins]\n'.format(
			int(round(step_factor * time_since)))
		)

	###############################################################################################
	#	orbital frame

	#	positions and velocities in m/s
	sat_orb_pos = sat_orb_line[1:4]
	sat_orb_vel = sat_orb_line[4:7]
	sun_orb_vec = sun_orb_line[1:4]
	earth_orb_vec = earth_orb_line[1:4]
	site_orb_vec = site_orb_line[1:4]

	#	orbital unit vectors
	sun_orb_unit = np.divide(sun_orb_vec, np.linalg.norm(sun_orb_vec))
	earth_orb_unit = np.divide(earth_orb_vec, np.linalg.norm(earth_orb_vec))
	site_orb_unit = np.divide(site_orb_vec, np.linalg.norm(site_orb_vec))

	###############################################################################################
	#	apply attitude motion
	###############################################################################################

	###########################################################################################
	#	body frame

	#	APPLY INITIAL ATTITUDE TO VECTORS

	#	apply initial attitude
	ang_step = initial_attitude

	#	generate rotation matrix of step
	rot_mat_step = rot_mat(ang_step)

	#	take inverse for application to vectors
	rot_mat_inv = np.linalg.inv(rot_mat_step)

	#	apply to orbital position vectors
	sat_body_pos = np.dot(sat_orb_pos, rot_mat_inv)
	sat_body_vel = np.dot(sat_orb_vel, rot_mat_inv)
	sun_body_vec = np.dot(sun_orb_vec, rot_mat_inv) 
	earth_body_vec = np.dot(earth_orb_vec, rot_mat_inv)
	site_body_vec = np.dot(site_orb_vec, rot_mat_inv)

	#	APPLY TOTAL ATTITUDE MOTION TO VECTORS

	#	apply total attitude motion of step
	if i != 0:
		ang_step_body = ang_step_body + np.multiply(ang_vel_body_step, time_step)

	#	generate rotation matrix of step
	rot_mat_step = rot_mat(ang_step_body)

	#	apply to inertial position vectors
	sat_body_pos = np.dot(sat_body_pos, rot_mat_step)
	sat_body_vel = np.dot(sat_body_vel, rot_mat_step)
	sun_body_vec = np.dot(sun_body_vec, rot_mat_step) 
	earth_body_vec = np.dot(earth_body_vec, rot_mat_step)
	site_body_vec = np.dot(site_body_vec, rot_mat_step)

	#	body unit vectors
	sun_body_unit = np.divide(sun_body_vec, np.linalg.norm(sun_body_vec))
	earth_body_unit = np.divide(earth_body_vec, np.linalg.norm(earth_body_vec))
	site_body_unit = np.divide(site_body_vec, np.linalg.norm(site_body_vec))

	###############################################################################################
	#	Data Output
	###############################################################################################

	#	if not reccording data, then break
	if start_point == -1:
		if i >= len_reccord:
			break

	#	else if reccording data and within reccord range
	elif start_point != -1 and i >= start_point:

		#	set image index
		index = i - start_point

		if orb_frame_out is True:

			#	orbital frame vector data

			#	julian date
			vector_data_orb[index, 0] = jdate

			#	sun vector
			vector_data_orb[index, 1:4] = sun_orb_vec

			#	site vector
			vector_data_orb[index, 4:7] = site_orb_vec

			#	earth vector
			vector_data_orb[index, 7:10] = earth_orb_vec

			#	velocity vector
			vector_data_orb[index, 10:13] = sat_orb_vel

			#	geometry x-axis
			vector_data_orb[index, 13:16] = x_orb_unit_step
	
			#	geometry y-axis
			vector_data_orb[index, 16:19] = y_orb_unit_step

			#	geometry z-axis
			vector_data_orb[index, 19:22] = z_orb_unit_step

			#	custom illumination vector
			vector_data_orb[index, 22:25] = np.array([-1, 1, 0])

			#	custom observation vector
			vector_data_orb[index, 25:28] = np.array([-1, 0, 0])

			#	generate image ID
			if index < 10:
				geom_id = '000' + str(index)
			elif 10 <= index < 100:
				geom_id = '00' + str(index)
			elif 100 <= index < 1000:
				geom_id = '0' + str(index)
			elif 1000 <= index < 10000:
				geom_id = str(index)
	
			#	saving orbital mesh file
			mesh_file = 'orbital_mesh_{}.stl'.format(geom_id)
			mesh_path = os.path.join(path_data, render_dir, render_geom_dir, 'orb_frame')
			temp_orb_mesh.save(os.path.join(mesh_path, mesh_file), mode=stl.Mode.ASCII)

		if body_frame_out is True:

			#	body frame vector data
			#	julian date
			vector_data_body[index, 0] = jdate

			#	sun vector
			vector_data_body[index, 1:4] = sun_body_vec

			#	site vector
			vector_data_body[index, 4:7] = site_body_vec

			#	earth vector
			vector_data_body[index, 7:10] = earth_body_vec

			#	velocity vector
			vector_data_body[index, 10:13] = sat_body_vel

			#	geometry x-axis
			vector_data_body[index, 13:16] = x_body_unit
	
			#	geometry y-axis
			vector_data_body[index, 16:19] = y_body_unit

			#	geometry z-axis
			vector_data_body[index, 19:22] = z_body_unit

			#	custom illumination vector
			vector_data_body[index, 22:25] = np.array([-1, 1, 0])

			#	custom observation vector
			vector_data_body[index, 25:28] = np.array([-1, 0, 0])

		#	save and end
		if i >= len_reccord - 1:

			print('Finished. Saving...')

			if orb_frame_out is True:

				#	orbital
				np.savetxt(
					os.path.join(path_data, render_dir, render_data_orb),
					vector_data_orb,
					delimiter=','
				)

			if body_frame_out is True:

				#	body
				np.savetxt(
					os.path.join(path_data, render_dir, render_data_body),
					vector_data_body,
					delimiter=','
				)

			break



len1 = np.round(24 * 60 * 60 * (vector_data_body[-1, 0] - vector_data_body[0, 0]), 1)
print('data spans {} seconds. [{} mins]'.format(len1, len1 / 60))






###################################################################################################
#	light curve generation
###################################################################################################
print('Preparing for light curve generation.')

###################################################################################################

#	Prepare for propagation
sun_vec_data = vector_data_body[:, 22:25]
site_vec_data = vector_data_body[:, 25:28]

light_curve = np.zeros((len(sim_time), 3))
light_curve[:, 1] = sim_time

################################################################################################
#	generate light curve
print('Generating light curve.')

for time_step, row in enumerate(light_curve):

	#	initilise

	#	time data
	time_since = light_curve[time_step, 1]
	jdate = jd_start + (time_since / 86400)
	light_curve[time_step, 0] = jdate

	#	position data
	sun_vec_step = sun_vec_data[time_step, :]
	site_vec_step = site_vec_data[time_step, :]

	#	set obervation distance in meters
	dist_vec = np.copy(site_vec_step) * 1000
	r_obs = np.linalg.norm(dist_vec)

	#	normal vectors
	sun_vec_unit = sun_vec_step / np.linalg.norm(sun_vec_step)
	site_vec_unit = site_vec_step / np.linalg.norm(site_vec_step)

	################################################################################################

	#	brightness calculation

	#	angular bisector
	H_vec = (sun_vec_unit + site_vec_unit) / len(sun_vec_unit + site_vec_unit) 

	#	observable facet array
	face_obs = []

	# facet loop
	for vector, area, bdrf_line in zip(orb_vectors, orb_areas, bdrf_data):

		#	parse bdrf data
		m_bdrf, omega_bdrf, d_bdrf, s_bdrf, n_bdrf = bdrf_line

		#	calculate normal unit vector
		vect12 = vector[1] - vector[0]
		vect23 = vector[2] - vector[1]
		normal = np.cross(vect12, vect23)
		normal = normal / np.linalg.norm(normal)

		#	calculate angles
		sun_ang = np.arccos(
			np.divide(
				np.dot(normal, sun_vec_step),
				np.linalg.norm(normal) * np.linalg.norm(sun_vec_step)
			)
		)

		site_ang = np.arccos(
			np.divide(
				np.dot(normal, site_vec_step),
				np.linalg.norm(normal) * np.linalg.norm(site_vec_step)
			)
		)

		#	illuminated and observable condition
		if sun_ang < (0.5 * np.pi) and site_ang < (0.5 * np.pi):

			#	alpha calculation
			alpha_ang = np.arccos(
				np.divide(
					np.dot(H_vec, normal),
					np.linalg.norm(H_vec) * np.linalg.norm(normal)
				)
			)

			#	G_bdrf calculation
			temp1 = 2 * np.divide(
				np.multiply(
					np.dot(normal, H_vec),
					np.dot(normal, site_vec_unit)
				), 
				np.dot(site_vec_unit, H_vec)
			)
			
			temp2 = 2 * np.divide(
				np.multiply(
					np.dot(normal, H_vec),
					np.dot(normal, sun_vec_unit)
				), 
				np.dot(sun_vec_unit, H_vec)
			)

			if temp1 > 1 and temp2 > 1:
			    G_bdrf = 1
			elif temp1 < 1 and temp2 > 1:
				G_bdrf = temp1
			elif temp1 > 1 and temp2 < 1:
				G_bdrf = temp2
			elif temp1 < 1 and temp2 < 1:
				G_bdrf = np.min((temp1, temp2))

			#	c_bdrf calculation
			c_bdrf = np.dot(site_vec_unit, H_vec)

			#	g_bdrf calculation
			g_bdrf = n_bdrf**2 + c_bdrf**2 - 1

			#	F_bdrf calculation
			F_bdrf = (
				((g_bdrf - c_bdrf)**2 / (2 * (g_bdrf + g_bdrf))) * 
				((1 + (c_bdrf * (g_bdrf + c_bdrf) - 1))**2 / ((c_bdrf * 
				(g_bdrf - c_bdrf) + 1)**2))
			)

			#	D_bdrf_calculation
			D_bdrf = (
				(1 / (m_bdrf**2 * np.cos(alpha_ang)**4)) * 
				np.exp(-((np.tan(alpha_ang) / m_bdrf)**2))
			)

			#	specular reflectance calculation
			R_s = (
				(D_bdrf * G_bdrf * F_bdrf) / (np.pi * np.dot(normal, sun_vec_unit) * 
				np.dot(normal, site_vec_unit))
			)

			#	diffuse reflectance calculation
			R_d = omega_bdrf / np.pi;

			#	brightness calculation
			facet_brightness = (
				(d_bdrf * R_d + s_bdrf * R_s) * \
				area * \
				np.pi * \
				np.dot(normal, sun_vec_unit) * \
				np.dot(normal, site_vec_unit)
			)

			face_obs.append(facet_brightness / (4 * np.pi * r_obs**2))

	#	apparent magnitude calculation
	brightness_step = np.sum(face_obs)
	magnitude = -26.74 - 2.5 * np.log10(brightness_step)
	light_curve[time_step, 2] = magnitude

print('Lightcurve Generated.')

time_data = np.zeros([len(light_curve)])
for i, time_line in enumerate(light_curve[:, 0]):
	time_data[i] = 86400 * (light_curve[i, 0] - light_curve[0, 0])

plt.plot(time_data, light_curve[:,2], marker='x')
plt.grid()
plt.show()
