####################################################################################################
#
#	COORDINATE TRANSFORMATION
#
#	INPUTS:		OBJECT DATA, TLE FILE NAME, HORIZONS FILE NAME
#
#	OUTPUTS:	POSITION DATA IN ITRF AND RSW
#
####################################################################################################

import os
import sys
import csv
import numpy as np

from function_list import jd2date, date2jd, get_iers, get_times
from trans_func import site_vec, teme2itrf, it2rsw
from functions.sofa_interface import gc2it

####################################################################################################

def transform(object_data, root_path, archive_path, file_name_tle, file_name_horizons):

	#	unpack object data
	norad_id = object_data.norad_ident
	jday_start = object_data.jday_utc_start
	jfrac_start = object_data.jfrac_utc_start
	jday_stop = object_data.jday_utc_stop
	jfrac_stop = object_data.jfrac_utc_stop
	site_latgd = np.radians(object_data.site_latitudegd)
	site_lon = np.radians(object_data.site_longitude)
	site_alt = object_data.site_altitude

	#	initilize file paths
	cwd = os.getcwd()

	transform_path = 'Outputs/Transformation/{:d}'.format(norad_id)
	tle_path = 'Outputs/TLE/{:d}'.format(norad_id)
	horiz_path = 'Outputs/Horizons/{:d}'.format(norad_id)

	#	read sat data	
	sat_pos = np.loadtxt(
		os.path.join(root_path, archive_path, tle_path, file_name_tle),
		delimiter=','
	)

	#	read horiz data
	horiz_pos = np.loadtxt(
		os.path.join(root_path, archive_path, horiz_path, file_name_horizons),
		delimiter=','
	)

	#	set reference frames
	data_teme = sat_pos[:, 1:4]

	#	convert to meters
	horiz_pos = np.multiply(1000, horiz_pos)
	data_icrf = horiz_pos[:, 1:7]

	#	create site vector
	site_itrf = site_vec(site_latgd, site_lon, site_alt)

	#	convert julian to utc date
	year1, mon1, day1, hour1, min1, sec1 = jd2date(jday_start, jfrac_start)
	date_vec1 = [year1, mon1, day1, hour1, min1, sec1] 

	#	create file names
	date_str = '{:4}-{:02}-{:02}_{:02}-{:02}-{:1.4f}'.format(*date_vec1)
	file_name_itrf = '{:d}_itrf_{:s}.csv'.format(norad_id, date_str)
	file_name_rsw =  '{:d}_rsw_{:s}.csv'.format(norad_id, date_str)

	#	allocate memory
	data_gcrf  = np.zeros((len(data_icrf), 6))
	data_itrf  = np.zeros((len(data_icrf), 12))
	data_rsw  = np.zeros((len(data_icrf), 12))
	
    ###############################################################################################

	if os.path.isfile(os.path.join(root_path, archive_path, transform_path, file_name_itrf)):
		print('ICRF Transformation File Exists')
	else:
		print('No Transformation File Found - Generating...')

		# Acquire IERS EOPs
		dut, dutps, lod, x_pole, x_poleps, y_pole, y_poleps, dX, dXps, dY, dYps, utc, dat = get_iers(jday_start, jfrac_start)

		# Perform interpolations
		dut = dut + utc * dutps
		x_p = x_pole + utc * x_poleps
		y_p = y_pole + utc * y_poleps
		dX = dX + utc * dXps
		dY = dY + utc * dYps

		# create time object
		time_object = get_times(*date_vec1, dut, dat)
		jdayut1 = time_object.jday_ut1
		jfracut1 = time_object.jfrac_ut1
		jdaytt = time_object.jday_tt
		jfractt = time_object.jfrac_tt

        ############################################################################################

		# Perform ICRF to GCRF translation
		data_gcrf[:,0:3] = data_icrf[:,0:3] - data_icrf[:,3:6]

        ############################################################################################

		# Perform GCRF to ITRF transformation
		c2t = gc2it(jdaytt, jfractt, jdayut1, jfracut1, dX, dY, x_p, y_p)

		for i, row in enumerate(data_gcrf):
			data_itrf[i, 0:3] = np.dot(c2t, row[0:3])
			data_itrf[i, 6:9] = site_itrf
		
        ############################################################################################

		# Perform TEME to ITRF translation
		te2it = teme2itrf(jdayut1, jfracut1, lod, x_p, y_p)

		for i, row in enumerate(data_teme):
			data_itrf[i, 9:12] = np.dot(te2it, row[0:3])

        ############################################################################################

		# Perform ITRF to RSW translation

		central_object = data_itrf[0, 9:12]
		transform_object = data_itrf[0, 0:3]

		testout = it2rsw(central_object, transform_object)

		for i, row in enumerate(data_itrf):
			central_body = data_itrf[i, 9:12]

			# Sun
			transform_body = data_itrf[i, 0:3]
			data_rsw[i, 0:3] = it2rsw(central_body, transform_body)

			# Earth
			transform_body = data_itrf[i, 3:6]
			data_rsw[i, 3:6] = it2rsw(central_body, transform_body)

			# Site
			transform_body = data_itrf[i, 6:9]
			data_rsw[i, 6:9] = it2rsw(central_body, transform_body)

			# Sat
			transform_body = data_itrf[i, 9:12]
			data_rsw[i, 9:12] = it2rsw(central_body, transform_body)

        ############################################################################################

		# Save data to file
		np.savetxt(
			os.path.join(root_path, archive_path, transform_path, file_name_itrf),
			data_itrf,
			delimiter=','
		)

		np.savetxt(
			os.path.join(root_path, archive_path, transform_path, file_name_rsw),
			data_rsw,
			delimiter=','
		)

		print('Transformation Files Generated: {:s} - {:s}'.format(file_name_itrf, file_name_rsw))

###################################################################################################
