import os
import sys

import numpy as np
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

#
from functions_geom_gen import top_faces_gen
from functions_geom_gen import bottom_faces_gen
from functions_geom_gen import side_faces_gen
from functions_geom_gen import connecting_faces_gen

#############################################################################################
#	initialise
#############################################################################################

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output
geom_out_path = os.path.join(path_data, 'geometry_dir/geom_dir/CZ-2D')
geom_file_name = 'cz-2d_pointed.stl'

geom_file_name = 'cz-2d_testing_pointed.stl'

geom_file_name = 'cz-2d_testing_double_pointed.stl'

#############################################################################################
#	range of x values
#############################################################################################

#	generate angular segments
ang_sep = np.linspace(0, 2 * np.pi, 37)

#	initialise vert arrays
pos_data = np.zeros([37, 2])

#	bottom
i = 0
x_vec = np.array([1.0, 0.0])

for ang in ang_sep:
	#	rotation matrix
	R = np.array([
		[np.cos(ang), -np.sin(ang)],
		[np.sin(ang), np.cos(ang)]
	])

	vec = np.dot(x_vec, R)
	pos_data[i, :] = vec
	i += 1

verts_array = []
faces_array = []

#############################################################################################
#	Part 1
#############################################################################################

assembly_level = 0

#	main tank
top_width = 3.35 / 2
bottom_width = 3.35 / 2
length = 9.5

#	rounded top height
round_top_height = 1.5
round_bottom_height = 1.5

pos_data_top = np.multiply(pos_data, top_width)
pos_data_bottom = np.multiply(pos_data, bottom_width)

#	z-position columns
top_col = np.full([len(pos_data_top), 1], length)
bottom_col = np.full([len(pos_data_top), 1], assembly_level)

#	generate rings
bottom_ring = np.hstack([pos_data_bottom, bottom_col])[0:-1]
top_ring = np.hstack([pos_data_top, top_col])[0:-1]

bottom_center = np.array([0, 0, 0 - round_bottom_height])
top_center = np.array([0, 0, length + round_top_height])

#############################################################################################
#	add top faces to verts/faces array
verts_array, faces_array = top_faces_gen(top_ring, top_center, verts_array, faces_array)

#	find indices of top array
temp_verts = np.array(verts_array)
temp_faces = np.array(faces_array)

#	properties
top_start_index = 0
top_stop_index = len(temp_faces) - 1
top_nfaces = len(temp_faces) - 1

print('\nNumber of Top faces = ', top_nfaces)
print('\n	Start Index = ', top_start_index)
print('	Stop Index = ', top_stop_index)

#	set next start index
side_start_index = len(temp_faces)

#############################################################################################
#	add side faces to verts/faces array
verts_array, faces_array = side_faces_gen(top_ring, bottom_ring, verts_array, faces_array)

#	find indices of top array
temp_verts = np.array(verts_array)
temp_faces = np.array(faces_array)

side_stop_index = len(temp_faces) - 1
side_nfaces = (len(temp_faces) - 1) - top_nfaces

print('\nNumber of Side faces = ', side_nfaces)
print('\n	Start Index = ', side_start_index)
print('	Stop Index = ', side_stop_index)

#	set next start index
bot_start_index = len(temp_faces)

#############################################################################################
#	add bottom faces to verts/faces array
verts_array, faces_array = bottom_faces_gen(bottom_ring, bottom_center, verts_array, faces_array)

#	find indices of top array
temp_verts = np.array(verts_array)
temp_faces = np.array(faces_array)

bot_stop_index = len(temp_faces) - 1
bot_nfaces = (len(temp_faces) - 1) - top_nfaces - side_nfaces

print('\nNumber of bottom faces = ', bot_nfaces)
print('\n	Start Index = ', bot_start_index)
print('	Stop Index = ', bot_stop_index)

#############################################################################################
#	generate mesh
#############################################################################################

#	convert to numpy arrays
verts_array = np.array(verts_array)
faces_array = np.array(faces_array)

print('\ntotal nverts = ', verts_array.shape[0])
print('total nfaces = ', faces_array.shape[0])

#	initialise output mesh
output_mesh = mesh.Mesh(np.zeros(faces_array.shape[0], dtype=mesh.Mesh.dtype))

#	generate output mesh
for i, f in enumerate(faces_array):
    for j in range(3):
        output_mesh.vectors[i][j] = verts_array[f[j],:]

#	update mesh properties
output_mesh.update_areas()
output_mesh.update_normals()
output_mesh.update_units()

#	translate by com
vol, com, inertia = output_mesh.get_mass_properties()
output_mesh.z -= com[2]

#	update mesh properties
output_mesh.update_areas()
output_mesh.update_normals()
output_mesh.update_units()

vol, com, inertia = output_mesh.get_mass_properties()

#############################################################################################
#	plot mesh
#############################################################################################

vectors = output_mesh.vectors
normals = output_mesh.normals

###############################################
#	initilise figure
figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
ax3d = mplot3d.Axes3D(figure3d)

i = 0

for vector, normal in zip(vectors, normals):

	###############################################
	# rending face

	#	Add final vert to vector array
	vector = np.vstack([vector, vector[0, :]])

	#	extract x, y and z values
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]

	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	ax3d.add_collection3d(pc)

#	###############################################
#	# plotting normal
#
#	#	find face centroid
#	vec1 = vector[0]
#	vec2 = vector[1]
#	vec3 = vector[2]
#
#	centroid_pos = np.array(
#		[
#			(1 / 3) * (vec1[0] + vec2[0] + vec3[0]),
#			(1 / 3) * (vec1[1] + vec2[1] + vec3[1]),
#			(1 / 3) * (vec1[2] + vec2[2] + vec3[2])
#		]
#	)
#
#	#	normal
#	normal_plot = np.vstack((centroid_pos, 10 * normal + centroid_pos))
#
#	ax3d.plot(
#		normal_plot[:, 0],
#		normal_plot[:, 1],
#		normal_plot[:, 2],
#		color='r'
#	)

ax3d.set_xlim([-5, 5])
ax3d.set_ylim([-5, 5])
ax3d.set_zlim([-5, 5])


plt.show()

###############################################################################################
#	save output mesh

user_input = input('\nSave file as {}? [y/n] '.format(geom_file_name))

if user_input == 'y':
	#	save stl mesh file
	output_mesh.save(os.path.join(geom_out_path, geom_file_name))
	print('File saved to: ', geom_out_path)

elif user_input == 'no':
	print('Not saved.')
	
print('\nEND')
