import os
import sys

#	add root directory to sys.path
sys.path.append(os.path.join(os.environ['HOME'], 'codebase/git_dir/python/GLIMMA'))

import numpy as np
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

#	import from transformation functions
from tools.functions.functions_transform import site_vec, rot1, rot2, rot3, \
										rot_mat, \
										axis2mat, \
										mat2angles, \
										gen_vectors

#############################################################################################
#	initialise
#############################################################################################

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output
geom_out_path = os.path.join(path_data, 'geometry_dir/geom_dir/box_wing')
geom_file_name = 'box_wing.stl'

#############################################################################################
#	define parts

#	spacecraft main bus
bus_depth = 0.71724
bus_height = 2.866
bus_front_width = 1.70923
bus_back_width = 0.66345

# solar panel boom
boom_depth = 0.05
boom_height = 0.05
boom_width = 1.0

# solar panel
panel_height = 2.29687
panel_width = 1.06656
panel_depth = 0.15
panel_angle = np.pi / 8.0

#############################################################################################
#	assemble mesh
#############################################################################################

#	find bus slope angle
angle = np.arctan(((bus_front_width / 2) - (bus_back_width / 2)) / bus_depth)

#	use slope angle to find y_0
y_0 = (bus_back_width / 2) + (bus_depth / 2.0) * np.tan(angle) 
y_delta = (boom_depth / 2) * np.tan(angle)

#############################################################################################
#	Spacecraft Bus
#############################################################################################

bus_verts_array = np.array([

	#	front face verts
	#	i = 0:7
	[bus_depth / 2, bus_front_width / 2, bus_height / 2],
	[bus_depth / 2, bus_front_width / 2, 0 + boom_height / 2],
	[bus_depth / 2, bus_front_width / 2, 0 - boom_height / 2],
	[bus_depth / 2, bus_front_width / 2, -bus_height / 2],
	[bus_depth / 2, -bus_front_width / 2, bus_height / 2],
	[bus_depth / 2, -bus_front_width / 2, 0 + boom_height / 2],
	[bus_depth / 2, -bus_front_width / 2, 0 - boom_height / 2],
	[bus_depth / 2, -bus_front_width / 2, -bus_height / 2],

	#	back face verts
	#	i = 8:15
	[-bus_depth / 2, bus_back_width / 2, bus_height / 2],
	[-bus_depth / 2, bus_back_width / 2, 0 + boom_height / 2],
	[-bus_depth / 2, bus_back_width / 2, 0 - boom_height / 2],
	[-bus_depth / 2, bus_back_width / 2, -bus_height / 2],
	[-bus_depth / 2, -bus_back_width / 2, bus_height / 2],
	[-bus_depth / 2, -bus_back_width / 2, 0 + boom_height / 2],
	[-bus_depth / 2, -bus_back_width / 2, 0 - boom_height / 2],
	[-bus_depth / 2, -bus_back_width / 2, -bus_height / 2],

	#	boom start verts
	#	i = 16:19
	[boom_depth / 2, y_0 + y_delta, boom_height / 2],
	[boom_depth / 2, y_0 + y_delta, -boom_height / 2],
	[-boom_depth / 2, y_0 - y_delta, -boom_height / 2],
	[-boom_depth / 2, y_0 - y_delta, boom_height / 2],

	#	top and bottom centers
	#	i = 20:21
	[0.0, y_0, bus_height / 2],
	[0.0, y_0, -bus_height / 2],

	#	boom start verts
	#	i = 22:25
	[boom_depth / 2, -y_0 - y_delta, boom_height / 2],
	[boom_depth / 2, -y_0 - y_delta, -boom_height / 2],
	[-boom_depth / 2, -y_0 + y_delta, -boom_height / 2],
	[-boom_depth / 2, -y_0 + y_delta, boom_height / 2],

	#	top and bottom centers
	#	i = 26:27
	[0.0, -y_0, bus_height / 2],
	[0.0, -y_0, -bus_height / 2],

])

bus_faces_array = np.array([

	#	front faces
	[4, 1, 0],
	[4, 5, 1],
	[6, 2, 1],
	[1, 5, 6],
	[7, 3, 2],
	[2, 6, 7],

	#	back faces
	[8, 9, 12],
	[13, 12, 9],
	[9, 10, 13],
	[10, 14, 13],
	[10, 11, 15],
	[15, 14, 10],

	#############################################################
	#	SIDE 1

	#	top side faces
 	[0, 1, 16],
 	[16, 20, 0],
 	[16, 19, 20],
 	[8, 20, 19],
 	[9, 8, 19],

	#	middle side faces
 	[1, 2, 16],
 	[2, 17, 16],
 	[9, 19, 18],
 	[10, 9, 18],

	#	top side faces
 	[2, 3, 17],
 	[21, 17, 3],
 	[21, 18, 17],
 	[11, 10, 21],
 	[21, 10, 18],

	#############################################################
	#	SIDE 2

	#	top side faces
	[22, 5, 4],
	[4, 26, 22],
 	[26, 25, 22],
	[25, 26, 12],
	[25, 12, 13],

	#	middle side faces
	[22, 6, 5],
	[22, 23, 6],
	[24, 25, 13],
	[24, 13, 14],

	#	bottom side faces
	[23, 7, 6],
	[7, 23, 27],
	[23, 24, 27],
	[27, 14, 15],
	[24, 14, 27],

	#############################################################
	#	top and bottom

	#	close top and bottom
	[8, 4, 0],
	[12, 4, 8],
	[3, 7, 11],
	[7, 15, 11],

])

#############################################################################################
#	generate output mesh

#	initialise output mesh
bus_mesh = mesh.Mesh(np.zeros(bus_faces_array.shape[0], dtype=mesh.Mesh.dtype))

#	generate output mesh
for i, f in enumerate(bus_faces_array):
    for j in range(3):
        bus_mesh.vectors[i][j] = bus_verts_array[f[j],:]

#	update mesh properties
bus_mesh.update_areas()
bus_mesh.update_normals()
bus_mesh.update_units()

#############################################################################################
#	Solar panel boom
#############################################################################################

boom_verts_array = np.array([

	#	boom start verts
	#	i = 16:19
	[boom_depth / 2, y_0 + y_delta, boom_height / 2],
	[boom_depth / 2, y_0 + y_delta, -boom_height / 2],
	[-boom_depth / 2, y_0 - y_delta, -boom_height / 2],
	[-boom_depth / 2, y_0 - y_delta, boom_height / 2],

	#	boom end verts
	#	i = 20:23
	[boom_depth / 2, y_0 + boom_width, boom_height / 2],
	[boom_depth / 2, y_0 + boom_width, -boom_height / 2],
	[-boom_depth / 2, y_0 + boom_width, -boom_height / 2],
	[-boom_depth / 2, y_0 + boom_width, boom_height / 2],

	#	boom start verts
	#	i = 26:29
	[boom_depth / 2, -y_0 - y_delta, boom_height / 2],
	[boom_depth / 2, -y_0 - y_delta, -boom_height / 2],
	[-boom_depth / 2, -y_0 + y_delta, -boom_height / 2],
	[-boom_depth / 2, -y_0 + y_delta, boom_height / 2],

	#	boom end verts
	#	i = 30:33
	[boom_depth / 2, -y_0 - boom_width, boom_height / 2],
	[boom_depth / 2, -y_0 - boom_width, -boom_height / 2],
	[-boom_depth / 2, -y_0 - boom_width, -boom_height / 2],
	[-boom_depth / 2, -y_0 - boom_width, boom_height / 2],

])

boom_faces_array = np.array([
	#	boom front faces
 	[0, 1, 4],
 	[1, 5, 4],

	#	boom back faces
 	[2, 3, 6],
 	[3, 7, 6],

	#	boom top faces
 	[7, 3, 0],
 	[7, 0, 4],

	#	boom bottom faces
 	[1, 2, 5],
 	[2, 6, 5],

	#	boom front faces
	[12, 9, 8],
	[12, 13, 9],
 
	#	boom back faces
	[14, 11, 10],
	[14, 15, 11],

	#	boom top faces
	[8, 11, 15],
	[12, 8, 15],

	#	boom bottom faces
	[13, 10, 9],
	[13, 14, 10],

])

#############################################################################################
#	generate output mesh

#	initialise output mesh
boom_mesh = mesh.Mesh(np.zeros(boom_faces_array.shape[0], dtype=mesh.Mesh.dtype))

#	generate output mesh
for i, f in enumerate(boom_faces_array):
    for j in range(3):
        boom_mesh.vectors[i][j] = boom_verts_array[f[j],:]

#	update mesh properties
boom_mesh.update_areas()
boom_mesh.update_normals()
boom_mesh.update_units()

#############################################################################################
#	Solar panels
#############################################################################################

solar_verts_array = np.array([

	#############################################################
	#	Side 1 Solar Panel

	#	i = 0:3
	#	solar panel verts
	[panel_depth / 2, y_0 + boom_width, panel_depth / 2],
	[panel_depth / 2, y_0 + boom_width, -panel_depth / 2],
	[-panel_depth / 2, y_0 + boom_width, -panel_depth / 2],
	[-panel_depth / 2, y_0 + boom_width, panel_depth / 2],

	#	i = 4:7
	#	solar panel inner verts
	[panel_depth / 2, y_0 + boom_width, panel_height / 2],
	[-panel_depth / 2, y_0 + boom_width, panel_height / 2],
	[panel_depth / 2, y_0 + boom_width, -panel_height / 2],
	[-panel_depth / 2, y_0 + boom_width, -panel_height / 2],

	# i = 8:11
	#	solar panel outer verts
	[panel_depth / 2, y_0 + boom_width + panel_width, panel_height / 2],
	[-panel_depth / 2, y_0 + boom_width + panel_width, panel_height / 2],
	[panel_depth / 2, y_0 + boom_width + panel_width, -panel_height / 2],
	[-panel_depth / 2, y_0 + boom_width + panel_width, -panel_height / 2],

	#############################################################
	#	Side 2 Solar Panel

	#	i = 12:15
	#	solar panel verts
	[panel_depth / 2, -y_0 - boom_width, panel_depth / 2],
	[panel_depth / 2, -y_0 - boom_width, -panel_depth / 2],
	[-panel_depth / 2, -y_0 - boom_width, -panel_depth / 2],
	[-panel_depth / 2, -y_0 - boom_width, panel_depth / 2],

	#	i = 16:19
	#	solar panel inner verts
	[panel_depth / 2, -y_0 - boom_width, panel_height / 2],
	[-panel_depth / 2, -y_0 - boom_width, panel_height / 2],
	[panel_depth / 2, -y_0 - boom_width, -panel_height / 2],
	[-panel_depth / 2, -y_0 - boom_width, -panel_height / 2],

	#	solar panel outer verts
	# i = 20:23
	[panel_depth / 2, -y_0 - boom_width - panel_width, panel_height / 2],
	[-panel_depth / 2, -y_0 - boom_width - panel_width, panel_height / 2],
	[panel_depth / 2, -y_0 - boom_width - panel_width, -panel_height / 2],
	[-panel_depth / 2, -y_0 - boom_width - panel_width, -panel_height / 2],

])

##################################################
#	rotate verts

#	rotation axis
axis = np.array([0.0, 1.0, 0.0])

#	convert to rotation matrix
rot_mat = axis2mat(angle, axis)

#	rotate panel verts
for i, vert in enumerate(solar_verts_array):
	solar_verts_array[i] = np.dot(rot_mat, vert)

solar_faces_array = np.array([

	#############################################################
	#	solar panel 1

	#	solar panel insides
	[0, 4, 3],
	[4, 5, 3],
	[1, 2, 7],
	[7, 6, 1],

	#	solar panel front
	[8, 4, 0],
	[0, 10, 8],
	[0, 1, 10],
	[1, 6, 10],

	#	solar panel back
	[3, 5, 9],
	[3, 9, 11],
	[2, 3, 11],
	[11, 7, 2],

	#	solar panel top/bottom
	[8, 5, 4],
	[8, 9, 5],
	[6, 7, 10],
	[7, 11, 10],

	#	solar panel outsides
	[11, 9, 8],
	[8, 10, 11],

	#############################################################
	#	solar_panel 2
	
	#	solar panel insides
	[15, 17, 12],
	[17, 16, 12],
	[14, 13, 18],
	[18, 19, 14],
	
	#	solar panel front
	[21, 17, 15],
	[15, 23, 21],
	[15, 14, 23],
	[14, 19, 23],
	
	#	solar panel back
	[12, 16, 20],
	[12, 20, 22],
	[13, 12, 22],
	[22, 18, 13],
	
	#	solar panel top/bottom
	[21, 16, 17],
	[21, 20, 16],
	[19, 18, 23],
	[18, 22, 23],

	#	solar panel outsides
	[22, 20, 21],
	[21, 23, 22]
])

#############################################################################################
#	generate output mesh

#	initialise output mesh
solar_mesh = mesh.Mesh(np.zeros(solar_faces_array.shape[0], dtype=mesh.Mesh.dtype))

#	generate output mesh
for i, f in enumerate(solar_faces_array):
    for j in range(3):
        solar_mesh.vectors[i][j] = solar_verts_array[f[j],:]

#	update mesh properties
solar_mesh.update_areas()
solar_mesh.update_normals()
solar_mesh.update_units()

y_axis = np.array([0.0, 1.0, 0.0])
solar_mesh.rotate(y_axis, panel_angle)

#	update mesh properties
solar_mesh.update_areas()
solar_mesh.update_normals()
solar_mesh.update_units()

solar_verts_array = solar_mesh.vectors

#############################################################################################
#	Solar panel connectors
#############################################################################################


print(solar_verts_array[0:4])

connector_verts_array = np.array([

	#############################################################
	#	Side 1 Solar Panel

	#	boom end verts
	#	i = 0:3
	[boom_depth / 2, y_0 + boom_width, boom_height / 2],
	[boom_depth / 2, y_0 + boom_width, -boom_height / 2],
	[-boom_depth / 2, y_0 + boom_width, -boom_height / 2],
	[-boom_depth / 2, y_0 + boom_width, boom_height / 2],

	#	solar panel verts
	#	i = 4:7
	[panel_depth / 2, y_0 + boom_width, panel_depth / 2],
	[panel_depth / 2, y_0 + boom_width, -panel_depth / 2],
	[-panel_depth / 2, y_0 + boom_width, -panel_depth / 2],
	[-panel_depth / 2, y_0 + boom_width, panel_depth / 2],


	#############################################################
	#	Side 2 Solar Panel

	#	boom end verts
	#	i = 8:11
	[boom_depth / 2, -y_0 - boom_width, boom_height / 2],
	[boom_depth / 2, -y_0 - boom_width, -boom_height / 2],
	[-boom_depth / 2, -y_0 - boom_width, -boom_height / 2],
	[-boom_depth / 2, -y_0 - boom_width, boom_height / 2],

	# i = 12:15
	#	solar panel verts
	[panel_depth / 2, -y_0 - boom_width, panel_depth / 2],
	[panel_depth / 2, -y_0 - boom_width, -panel_depth / 2],
	[-panel_depth / 2, -y_0 - boom_width, -panel_depth / 2],
	[-panel_depth / 2, -y_0 - boom_width, panel_depth / 2],
])


connector_faces_array = np.array([

	#############################################################
	#	solar panel 1

	#	solar panel connector
	[0, 1, 4],
	[5, 4, 1],
	[1, 2, 5],
	[6, 5, 2],
	[2, 3, 7],
	[7, 6, 2],
	[7, 3, 0],
	[0, 4, 7],

	#############################################################
	#	solar_panel 2
	
	#	solar panel connector
	[12, 9, 8],
	[9, 12, 13],
	[13, 10, 9],
	[10, 13, 14],
	[15, 11, 10],
	[10, 14, 15],
	[8, 11, 15],
	[15, 12, 8],
])
	



print('\nboom 1 end verts: \n', connector_verts_array[0:4])

print('\nsolar panel 1 verts: \n', connector_verts_array[4:8])




#############################################################################################
#	Graphical Outputs
#############################################################################################

###############################################
#	initilise figure
figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
ax3d = mplot3d.Axes3D(figure3d)

#############################################################################################
#	plot main bus

vectors = bus_mesh.vectors
normals = bus_mesh.normals

i = 0
for vector, normal in zip(vectors, normals):

	###############################################
	# rending face

	#	Add final vert to vector array
	vector = np.vstack([vector, vector[0, :]])

	#	extract x, y and z values
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]

	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	ax3d.add_collection3d(pc)

#############################################################################################
#	plot solar panel booms

vectors = boom_mesh.vectors
normals = boom_mesh.normals

i = 0
for vector, normal in zip(vectors, normals):

	###############################################
	# rending face

	#	Add final vert to vector array
	vector = np.vstack([vector, vector[0, :]])

	#	extract x, y and z values
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]

	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	ax3d.add_collection3d(pc)

#############################################################################################
#	plot solar panels

vectors = solar_mesh.vectors
normals = solar_mesh.normals

i = 0
for vector, normal in zip(vectors, normals):

	###############################################
	# rending face

	#	Add final vert to vector array
	vector = np.vstack([vector, vector[0, :]])

	#	extract x, y and z values
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]

	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	ax3d.add_collection3d(pc)

#	###############################################
#	# plotting normal
#	
#	#	find face centroid
#	vec1 = vector[0]
#	vec2 = vector[1]
#	vec3 = vector[2]
#	
#	centroid_pos = np.array(
#		[
#			(1 / 3) * (vec1[0] + vec2[0] + vec3[0]),
#			(1 / 3) * (vec1[1] + vec2[1] + vec3[1]),
#			(1 / 3) * (vec1[2] + vec2[2] + vec3[2])
#		]
#	)
#	
#	#	normal
#	normal_plot = np.vstack((centroid_pos, 1000 * normal + centroid_pos))
#	
#	ax3d.plot(
#		normal_plot[:, 0],
#		normal_plot[:, 1],
#		normal_plot[:, 2],
#		color='k'
#	)

ax3d.set_xlim([-5, 5])
ax3d.set_ylim([-5, 5])
ax3d.set_zlim([-5, 5])

ax3d.set_xlabel('x - axis')
ax3d.set_ylabel('y - axis')
ax3d.set_zlabel('z - axis')

plt.show()

# ###############################################################################################
# #	save output mesh
# 
# user_input = input('\nSave file as {}? [y/n] '.format(geom_file_name))
# 
# if user_input == 'y':
# 	#	save stl mesh file
# 	output_mesh.save(os.path.join(geom_out_path, geom_file_name))
# 	print('File saved to: ', geom_out_path)
# 
# elif user_input == 'no':
# 	print('Not saved.')
# 	
# print('\nEND')
