import os
import sys

import numpy as np
from stl import mesh

#	matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.patches import FancyArrowPatch
from mpl_toolkits import mplot3d

#
from functions_geom_gen import top_faces_gen
from functions_geom_gen import bottom_faces_gen
from functions_geom_gen import side_faces_gen
from functions_geom_gen import connecting_faces_gen

#############################################################################################
#	initialise
#############################################################################################

##################################################
#	file paths
root_path = os.path.join(os.environ['HOME'], 'codebase')
archive_path = 'data_archive/GLIMMA_data'
path_data = os.path.join(root_path, archive_path)

#	output
geom_out_path = os.path.join(path_data, 'geometry_dir/geom_dir/SL-8')
geom_file_name = 'sl8.stl'

#############################################################################################
#	range of x values
#############################################################################################

#	generate angular segments
ang_sep = np.linspace(0, 2 * np.pi, 37)

#	initialise vert arrays
pos_data = np.zeros([37, 2])

#	bottom
i = 0
x_vec = np.array([1.0, 0.0])

for ang in ang_sep:
	#	rotation matrix
	R = np.array([
		[np.cos(ang), -np.sin(ang)],
		[np.sin(ang), np.cos(ang)]
	])

	vec = np.dot(x_vec, R)
	pos_data[i, :] = vec
	i += 1

verts_array = []
faces_array = []

#############################################################################################
#	Part 1
#############################################################################################

assembly_level = 0

#	main tank
p1_top_width = 2.4 / 2
p1_bottom_width = 2.4 / 2
p1_length = 3.9

p1_pos_data_top = np.multiply(pos_data, p1_top_width)
p1_pos_data_bottom = np.multiply(pos_data, p1_bottom_width)

#	z-position columns
top_col = np.full([len(p1_pos_data_top), 1], p1_length)
bottom_col = np.full([len(p1_pos_data_top), 1], assembly_level)

#	generate rings
p1_bottom_ring = np.hstack([p1_pos_data_bottom, bottom_col])[0:-1]
p1_top_ring = np.hstack([p1_pos_data_top, top_col])[0:-1]

p1_bottom_center = np.array([0, 0, 0])
p1_top_center = np.array([0, 0, p1_length])

#############################################################################################
#	Top faces
verts_array, faces_array = top_faces_gen(p1_top_ring, p1_top_center, verts_array, faces_array)

#############################################################################################
#	side faces
verts_array, faces_array = side_faces_gen(p1_top_ring, p1_bottom_ring, verts_array, faces_array)

#############################################################################################
#	Part 2
#############################################################################################

#	tank_connector
p2_top_width = 1.68 / 2
p2_bottom_width = 1.02 / 2
p2_length = 0.84

p2_pos_data_top = np.multiply(pos_data, p2_top_width)
p2_pos_data_bottom = np.multiply(pos_data, p2_bottom_width)

#	z-position columns
top_col = np.full([len(p2_pos_data_top), 1], assembly_level)

assembly_level -= p2_length

bottom_col = np.full([len(p2_pos_data_top), 1], assembly_level)

#	generate rings
p2_top_ring = np.hstack([p2_pos_data_top, top_col])[0:-1]
p2_bottom_ring = np.hstack([p2_pos_data_bottom, bottom_col])[0:-1]

p2_bottom_center = np.array([0, 0, 0])
p2_top_center = np.array([0, 0, p2_length])

#############################################################################################
#	Part1 to Part2 connecting faces
verts_array, faces_array = connecting_faces_gen(p2_top_ring, p1_bottom_ring, verts_array, faces_array)

#############################################################################################
#	side faces
verts_array, faces_array = side_faces_gen(p2_top_ring, p2_bottom_ring, verts_array, faces_array)

#############################################################################################
#	Part 3
#############################################################################################

#	nozzel
p3_top_width = 0.6 / 2
p3_bottom_width = 1.2 / 2
p3_length = 0.84

p3_pos_data_top = np.multiply(pos_data, p3_top_width)
p3_pos_data_bottom = np.multiply(pos_data, p3_bottom_width)

#	z-position columns
top_col = np.full([len(p3_pos_data_top), 1], assembly_level)

assembly_level -= p3_length

bottom_col = np.full([len(p3_pos_data_top), 1], assembly_level)

#	generate rings
p3_top_ring = np.hstack([p3_pos_data_top, top_col])[0:-1]
p3_bottom_ring = np.hstack([p3_pos_data_bottom, bottom_col])[0:-1]

p3_bottom_center = np.array([0, 0, assembly_level])
p3_top_center = np.array([0, 0, p3_length])

#############################################################################################
#	Part2 to Part3 connecting faces
verts_array, faces_array = connecting_faces_gen(p3_top_ring, p2_bottom_ring, verts_array, faces_array)

#############################################################################################
#	side faces
verts_array, faces_array = side_faces_gen(p3_top_ring, p3_bottom_ring, verts_array, faces_array)

#############################################################################################
#	Bottom faces
verts_array, faces_array = bottom_faces_gen(p3_bottom_ring, p3_bottom_center, verts_array, faces_array)

#############################################################################################
#	finalise

#	convert to numpy arrays
verts_array = np.array(verts_array)
faces_array = np.array(faces_array)

print('nverts = ', verts_array.shape[0])
print('nfaces = ', faces_array.shape[0])

#	initialise output mesh
output_mesh = mesh.Mesh(np.zeros(faces_array.shape[0], dtype=mesh.Mesh.dtype))

#	generate output mesh
for i, f in enumerate(faces_array):
    for j in range(3):
        output_mesh.vectors[i][j] = verts_array[f[j],:]

#	update mesh properties
output_mesh.update_areas()
output_mesh.update_normals()
output_mesh.update_units()

#	translate by com
vol, com, inertia = output_mesh.get_mass_properties()
output_mesh.z -= com[2]

#	update mesh properties
output_mesh.update_areas()
output_mesh.update_normals()
output_mesh.update_units()

vol, com, inertia = output_mesh.get_mass_properties()

#############################################################################################
#	plot mesh
#############################################################################################

vectors = output_mesh.vectors
normals = output_mesh.normals

###############################################
#	initilise figure
figure3d = plt.figure(num=None, figsize=(16, 9), dpi=80, facecolor='w', edgecolor='k')
ax3d = mplot3d.Axes3D(figure3d)

i = 0

for vector, normal in zip(vectors, normals):

	###############################################
	# rending face

	#	Add final vert to vector array
	vector = np.vstack([vector, vector[0, :]])

	#	extract x, y and z values
	xs = vector[:,0]
	ys = vector[:,1]
	zs = vector[:,2]

	#	generate vertices list
	verts = [list(zip(xs, ys, zs))]

	#	add facet to plot
	pc = mplot3d.art3d.Poly3DCollection(verts, linewidth=1)
	pc.set_edgecolor([0.5, 0.5, 0.5])
	ax3d.add_collection3d(pc)

# 	###############################################
# 	# plotting normal
# 
# 	#	find face centroid
# 	vec1 = vector[0]
# 	vec2 = vector[1]
# 	vec3 = vector[2]
# 
# 	centroid_pos = np.array(
# 		[
# 			(1 / 3) * (vec1[0] + vec2[0] + vec3[0]),
# 			(1 / 3) * (vec1[1] + vec2[1] + vec3[1]),
# 			(1 / 3) * (vec1[2] + vec2[2] + vec3[2])
# 		]
# 	)
# 
# 	#	normal
# 	normal_plot = np.vstack((centroid_pos, 10 * normal + centroid_pos))
# 
# 	ax3d.plot(
# 		normal_plot[:, 0],
# 		normal_plot[:, 1],
# 		normal_plot[:, 2],
# 		color='r'
# 	)

ax3d.set_xlim([-5, 5])
ax3d.set_ylim([-5, 5])
ax3d.set_zlim([-5, 5])

plt.show()
sys.exit()

###############################################################################################
#	save output mesh

user_input = input('\nSave file as {}? [y/n] '.format(geom_file_name))

if user_input == 'y':
	#	save stl mesh file
	output_mesh.save(os.path.join(geom_out_path, geom_file_name))
	print('File saved to: ', geom_out_path)

elif user_input == 'no':
	print('Not saved.')
	
print('\nEND')
